#include <iostream>
#include <string>
#include <tuple>

#include <boost/algorithm/string/replace.hpp>

#include "baldr/location.h"
#include "midgard/pointll.h"
#include "midgard/util.h"
#include "proto/navigator.pb.h"
#include "proto/route.pb.h"
#include "tyr/navigator.h"

#include "test.h"

using namespace std;
using namespace valhalla;
using namespace valhalla::baldr;
using namespace valhalla::tyr;

namespace {
const std::string kLanguageTag = "<LANGUAGE>";
const std::string kUnitsTag = "<UNITS>";
const std::string kStatusMessageTag = "<STATUS_MESSAGE>";
const std::string kStatusTag = "<STATUS>";
const std::string kIdTag = "<ID>";
const std::string kSummaryLengthTag = "<SUMMARY_LENGTH>";
const std::string kSummaryTimeTag = "<SUMMARY_TIME>";
const std::string kSummaryMinLatTag = "<SUMMARY_MIN_LAT>";
const std::string kSummaryMinLonTag = "<SUMMARY_MIN_LON>";
const std::string kSummaryMaxLatTag = "<SUMMARY_MAX_LAT>";
const std::string kSummaryMaxLonTag = "<SUMMARY_MAX_LON>";
const std::string kLocationOrigLatTag = "<LOCATION_ORIG_LAT>";
const std::string kLocationOrigLonTag = "<LOCATION_ORIG_LON>";
const std::string kLocationOrigTypeTag = "<LOCATION_ORIG_TYPE>";
const std::string kLocationOrigHeadingTag = "<LOCATION_ORIG_HEADING>";
const std::string kLocationOrigNameTag = "<LOCATION_ORIG_NAME>";
const std::string kLocationOrigStreetTag = "<LOCATION_ORIG_STREET>";
const std::string kLocationOrigCityTag = "<LOCATION_ORIG_CITY>";
const std::string kLocationOrigStateTag = "<LOCATION_ORIG_STATE>";
const std::string kLocationOrigPostalCodeTag = "<LOCATION_ORIG_POSTAL_CODE>";
const std::string kLocationOrigCountryTag = "<LOCATION_ORIG_COUNTRY>";
const std::string kLocationOrigDateTimeTag = "<LOCATION_ORIG_DATE_TIME>";
const std::string kLocationOrigSideOfStreetTag = "<LOCATION_ORIG_SIDE_OF_STREET>";
const std::string kLocationOrigOriginalIndexTag = "<LOCATION_ORIG_ORIGINAL_INDEX>";
const std::string kLocationDestLatTag = "<LOCATION_DEST_LAT>";
const std::string kLocationDestLonTag = "<LOCATION_DEST_LON>";
const std::string kLocationDestTypeTag = "<LOCATION_DEST_TYPE>";
const std::string kLocationDestHeadingTag = "<LOCATION_DEST_HEADING>";
const std::string kLocationDestNameTag = "<LOCATION_DEST_NAME>";
const std::string kLocationDestStreetTag = "<LOCATION_DEST_STREET>";
const std::string kLocationDestCityTag = "<LOCATION_DEST_CITY>";
const std::string kLocationDestStateTag = "<LOCATION_DEST_STATE>";
const std::string kLocationDestPostalCodeTag = "<LOCATION_DEST_POSTAL_CODE>";
const std::string kLocationDestCountryTag = "<LOCATION_DEST_COUNTRY>";
const std::string kLocationDestDateTimeTag = "<LOCATION_DEST_DATE_TIME>";
const std::string kLocationDestSideOfStreetTag = "<LOCATION_DEST_SIDE_OF_STREET>";
const std::string kLocationDestOriginalIndexTag = "<LOCATION_DEST_ORIGINAL_INDEX>";
const std::string kLeg1ShapeTag = "<LEG1_SHAPE_TAG>";

// Sub class to test protected methods
class NavigatorTest : public Navigator {
public:
  NavigatorTest() : Navigator() {
  }

  NavigationStatus SetRoute(const std::string& route_json_str) {
    return Navigator::SetRoute(route_json_str);
  }

  NavigationStatus OnLocationChanged(const FixLocation& fix_location) {
    return Navigator::OnLocationChanged(fix_location);
  }

  const Route& route() const {
    return Navigator::route_;
  }

  const std::vector<std::pair<float, uint32_t>>& remaining_leg_values() const {
    return Navigator::remaining_leg_values_;
  }

  const size_t leg_index() const {
    return Navigator::leg_index_;
  }

  bool HasKilometerUnits() const {
    return Navigator::HasKilometerUnits();
  }

  bool IsDestinationShapeIndex(size_t idx) const {
    return Navigator::IsDestinationShapeIndex(idx);
  }

  bool IsStartManeuverIndex(size_t idx) const {
    return Navigator::IsStartManeuverIndex(idx);
  }

  bool IsDestinationManeuverIndex(size_t idx) const {
    return Navigator::IsDestinationManeuverIndex(idx);
  }

  NavigationStatus SnapToRoute(const FixLocation& fix_location) {
    return Navigator::SnapToRoute(fix_location);
  }

  bool OnRouteLocationCloseToOrigin(const NavigationStatus& nav_status) const {
    return Navigator::OnRouteLocationCloseToOrigin(nav_status);
  }

  float UnitsToMeters(float units) const {
    return Navigator::UnitsToMeters(units);
  }

  size_t GetWordCount(const std::string& instruction) const {
    return Navigator::GetWordCount(instruction);
  }

  uint32_t GetPreTransitionThreshold(size_t instruction_index) const {
    return Navigator::GetPreTransitionThreshold(instruction_index);
  }

  bool IsTimeWithinBounds(uint32_t time, uint32_t lower_bound, uint32_t upper_bound) const {
    return Navigator::IsTimeWithinBounds(time, lower_bound, upper_bound);
  }

  float GetInitialLongTransitionAlertLength() const {
    return Navigator::GetInitialLongTransitionAlertLength();
  }

  float GetInitialLongTransitionAlertLowerLength() const {
    return Navigator::GetInitialLongTransitionAlertLowerLength();
  }

  float GetInitialLongTransitionAlertUpperLength() const {
    return Navigator::GetInitialLongTransitionAlertUpperLength();
  }

  float GetInitialLongTransitionAlertMinManeuverLength() const {
    return Navigator::GetInitialLongTransitionAlertMinManeuverLength();
  }

  float GetInitialShortTransitionAlertLength() const {
    return Navigator::GetInitialShortTransitionAlertLength();
  }

  float GetInitialShortTransitionAlertLowerLength() const {
    return Navigator::GetInitialShortTransitionAlertLowerLength();
  }

  float GetInitialShortTransitionAlertUpperLength() const {
    return Navigator::GetInitialShortTransitionAlertUpperLength();
  }

  float GetInitialShortTransitionAlertMinManeuverLength() const {
    return Navigator::GetInitialShortTransitionAlertMinManeuverLength();
  }

  float GetFinalLongTransitionAlertLength() const {
    return Navigator::GetFinalLongTransitionAlertLength();
  }

  float GetFinalLongTransitionAlertLowerLength() const {
    return Navigator::GetFinalLongTransitionAlertLowerLength();
  }

  float GetFinalLongTransitionAlertUpperLength() const {
    return Navigator::GetFinalLongTransitionAlertUpperLength();
  }

  float GetFinalLongTransitionAlertMinManeuverLength() const {
    return Navigator::GetFinalLongTransitionAlertMinManeuverLength();
  }

  float GetFinalMediumTransitionAlertLength() const {
    return Navigator::GetFinalMediumTransitionAlertLength();
  }

  float GetFinalMediumTransitionAlertLowerLength() const {
    return Navigator::GetFinalMediumTransitionAlertLowerLength();
  }

  float GetFinalMediumTransitionAlertUpperLength() const {
    return Navigator::GetFinalMediumTransitionAlertUpperLength();
  }

  float GetFinalMediumTransitionAlertMinManeuverLength() const {
    return Navigator::GetFinalMediumTransitionAlertMinManeuverLength();
  }

  float GetFinalShortTransitionAlertLength() const {
    return Navigator::GetFinalShortTransitionAlertLength();
  }

  float GetFinalShortTransitionAlertLowerLength() const {
    return Navigator::GetFinalShortTransitionAlertLowerLength();
  }

  float GetFinalShortTransitionAlertUpperLength() const {
    return Navigator::GetFinalShortTransitionAlertUpperLength();
  }

  std::vector<std::tuple<bool, bool, bool, bool>>& used_instructions() {
    return Navigator::used_instructions_;
  }

  NavigationStatus_RouteState GetRouteState() {
    return route_state_;
  }
};

void TryTopLevelSetRoute(const std::string& route_json_str,
                         const std::string& expected_language,
                         const std::string& expected_units,
                         const std::string& expected_status_message,
                         uint32_t expected_status,
                         const std::string& expected_id) {
  NavigatorTest nt;
  nt.SetRoute(route_json_str);

  if (nt.route().trip().language() != expected_language)
    throw std::runtime_error("Incorrect language - found: " + nt.route().trip().language() +
                             " | expected: " + expected_language);

  if (nt.route().trip().units() != expected_units)
    throw std::runtime_error("Incorrect units - found: " + nt.route().trip().units() +
                             " | expected: " + expected_units);

  if (nt.route().trip().status_message() != expected_status_message)
    throw std::runtime_error(
        "Incorrect status_message - found: " + nt.route().trip().status_message() +
        " | expected: " + expected_status_message);

  if (nt.route().trip().status() != expected_status)
    throw std::runtime_error(
        "Incorrect status - found: " + std::to_string(nt.route().trip().status()) +
        " | expected: " + std::to_string(expected_status));

  if (nt.route().trip().id() != expected_id)
    throw std::runtime_error("Incorrect id - found: " + nt.route().trip().id() +
                             " | expected: " + expected_id);
}

void TryTopLevelSummarySetRoute(const std::string& route_json_str,
                                float expected_summary_length,
                                uint32_t expected_summary_time,
                                float expected_summary_min_lat,
                                float expected_summary_min_lon,
                                float expected_summary_max_lat,
                                float expected_summary_max_lon) {
  NavigatorTest nt;
  nt.SetRoute(route_json_str);

  if (nt.route().trip().summary().length() != expected_summary_length)
    throw std::runtime_error(
        "Incorrect status - found: " + std::to_string(nt.route().trip().summary().length()) +
        " | expected: " + std::to_string(expected_summary_length));

  if (nt.route().trip().summary().time() != expected_summary_time)
    throw std::runtime_error(
        "Incorrect status - found: " + std::to_string(nt.route().trip().summary().time()) +
        " | expected: " + std::to_string(expected_summary_time));

  if (nt.route().trip().summary().min_lat() != expected_summary_min_lat)
    throw std::runtime_error(
        "Incorrect status - found: " + std::to_string(nt.route().trip().summary().min_lat()) +
        " | expected: " + std::to_string(expected_summary_min_lat));

  if (nt.route().trip().summary().min_lon() != expected_summary_min_lon)
    throw std::runtime_error(
        "Incorrect status - found: " + std::to_string(nt.route().trip().summary().min_lon()) +
        " | expected: " + std::to_string(expected_summary_min_lon));

  if (nt.route().trip().summary().max_lat() != expected_summary_max_lat)
    throw std::runtime_error(
        "Incorrect status - found: " + std::to_string(nt.route().trip().summary().max_lat()) +
        " | expected: " + std::to_string(expected_summary_max_lat));

  if (nt.route().trip().summary().max_lon() != expected_summary_max_lon)
    throw std::runtime_error(
        "Incorrect status - found: " + std::to_string(nt.route().trip().summary().max_lon()) +
        " | expected: " + std::to_string(expected_summary_max_lon));
}

void TryTopLevelLocationSetRoute(const std::string& route_json_str,
                                 size_t expected_location_count,
                                 const std::string& expected_orig_street,
                                 float expected_orig_lat,
                                 float expected_orig_lon,
                                 const std::string& expected_orig_type,
                                 const std::string& expected_dest_street,
                                 float expected_dest_lat,
                                 float expected_dest_lon,
                                 const std::string& expected_dest_type,
                                 const std::string& expected_dest_side_of_street) {
  NavigatorTest nt;
  nt.SetRoute(route_json_str);

  int location_count = nt.route().trip().locations_size();
  if (location_count != expected_location_count)
    throw std::runtime_error("Incorrect location count - found: " + std::to_string(location_count) +
                             " | expected: " + std::to_string(expected_location_count));

  auto& orig = nt.route().trip().locations(0);
  if (orig.street() != expected_orig_street)
    throw std::runtime_error("Incorrect orig street - found: " + orig.street() +
                             " | expected: " + expected_orig_street);

  if (orig.lat() != expected_orig_lat)
    throw std::runtime_error("Incorrect orig lat - found: " + std::to_string(orig.lat()) +
                             " | expected: " + std::to_string(expected_orig_lat));

  if (orig.lon() != expected_orig_lon)
    throw std::runtime_error("Incorrect orig lon - found: " + std::to_string(orig.lon()) +
                             " | expected: " + std::to_string(expected_orig_lon));

  if (orig.type() != expected_orig_type)
    throw std::runtime_error("Incorrect orig type - found: " + orig.type() +
                             " | expected: " + expected_orig_type);

  auto& dest = nt.route().trip().locations(1);
  if (dest.street() != expected_dest_street)
    throw std::runtime_error("Incorrect dest street - found: " + dest.street() +
                             " | expected: " + expected_dest_street);

  if (dest.lat() != expected_dest_lat)
    throw std::runtime_error("Incorrect dest lat - found: " + std::to_string(dest.lat()) +
                             " | expected: " + std::to_string(expected_dest_lat));

  if (dest.lon() != expected_dest_lon)
    throw std::runtime_error("Incorrect dest lon - found: " + std::to_string(dest.lon()) +
                             " | expected: " + std::to_string(expected_dest_lon));

  if (dest.type() != expected_dest_type)
    throw std::runtime_error("Incorrect dest type - found: " + dest.type() +
                             " | expected: " + expected_dest_type);

  if (dest.side_of_street() != expected_dest_side_of_street)
    throw std::runtime_error("Incorrect dest side_of_street - found: " + dest.side_of_street() +
                             " | expected: " + expected_dest_side_of_street);
}

void TryTopLevelLegSetRoute(const std::string& route_json_str,
                            size_t expected_leg_maneuver_count,
                            const std::string& expected_leg_shape) {
  NavigatorTest nt;
  nt.SetRoute(route_json_str);

  auto& leg = nt.route().trip().legs(0);

  int maneuver_count = leg.maneuvers_size();
  if (maneuver_count != expected_leg_maneuver_count)
    throw std::runtime_error("Incorrect maneuver count - found: " + std::to_string(maneuver_count) +
                             " | expected: " + std::to_string(expected_leg_maneuver_count));

  if (leg.shape() != expected_leg_shape)
    throw std::runtime_error("Incorrect leg shape - found: " + leg.shape() +
                             " | expected: " + expected_leg_shape);
}

void TestTopLevelLegSetRoute() {
  // Tagged route json string
  std::string route_json_str =
      "{\"trip\":{\"language\":\"" + kLanguageTag + "\",\"units\":\"" + kUnitsTag +
      "\",\"status_message\":\"" + kStatusMessageTag + "\",\"status\":" + kStatusTag + ",\"id\":\"" +
      kIdTag + "\",\"summary\":{\"max_lon\":" + kSummaryMaxLonTag +
      ",\"max_lat\":" + kSummaryMaxLatTag + ",\"time\":" + kSummaryTimeTag +
      ",\"length\":" + kSummaryLengthTag + ",\"min_lat\":" + kSummaryMinLatTag +
      ",\"min_lon\":" + kSummaryMinLonTag + "}" + ",\"locations\":[{\"street\":\"" +
      kLocationOrigStreetTag + "\",\"lon\":" + kLocationOrigLonTag +
      ",\"lat\":" + kLocationOrigLatTag + ",\"type\":\"" + kLocationOrigTypeTag + "\"},{\"type\":\"" +
      kLocationDestTypeTag + "\",\"side_of_street\":\"" + kLocationDestSideOfStreetTag +
      "\",\"lat\":" + kLocationDestLatTag + ",\"lon\":" + kLocationDestLonTag + ",\"street\":\"" +
      kLocationDestStreetTag + "\"}]" + ",\"legs\":[{\"shape\":\"" + kLeg1ShapeTag +
      "\",\"summary\":{\"max_lon\":-76.276863,\"max_lat\":40.063713,\"time\":287,\"length\":2.168,"
      "\"min_lat\":40.042500,\"min_lon\":-76.299324}" +
      ",\"maneuvers\":[{\"travel_mode\":\"drive\",\"begin_shape_index\":0,\"length\":0.081,"
      "\"time\":16,\"type\":1,\"end_shape_index\":2,\"instruction\":\"Drive east on East Fulton "
      "Street.\",\"verbal_pre_transition_instruction\":\"Drive east on East Fulton Street for 400 "
      "feet.\",\"travel_type\":\"car\",\"street_names\":[\"East Fulton "
      "Street\"]},{\"travel_type\":\"car\",\"travel_mode\":\"drive\",\"verbal_pre_transition_"
      "instruction\":\"Turn left onto North Plum "
      "Street.\",\"verbal_transition_alert_instruction\":\"Turn left onto North Plum "
      "Street.\",\"length\":0.176,\"instruction\":\"Turn left onto North Plum "
      "Street.\",\"end_shape_index\":5,\"type\":15,\"time\":58,\"verbal_post_transition_"
      "instruction\":\"Continue for 2 tenths of a mile.\",\"street_names\":[\"North Plum "
      "Street\"],\"begin_shape_index\":2},{\"travel_type\":\"car\",\"travel_mode\":\"drive\","
      "\"verbal_pre_transition_instruction\":\"Bear right onto New Holland "
      "Avenue.\",\"verbal_transition_alert_instruction\":\"Bear right onto New Holland "
      "Avenue.\",\"length\":1.276,\"instruction\":\"Bear right onto New Holland "
      "Avenue.\",\"end_shape_index\":29,\"type\":9,\"time\":124,\"verbal_post_transition_"
      "instruction\":\"Continue for 1.3 miles.\",\"street_names\":[\"New Holland "
      "Avenue\"],\"begin_shape_index\":5},{\"travel_type\":\"car\",\"verbal_pre_transition_"
      "instruction\":\"Continue on New Holland Pike for 3 tenths of a "
      "mile.\",\"verbal_transition_alert_instruction\":\"Continue on New Holland "
      "Pike.\",\"length\":0.284,\"instruction\":\"Continue on New Holland "
      "Pike.\",\"end_shape_index\":39,\"type\":8,\"time\":35,\"street_names\":[\"New Holland "
      "Pike\"],\"begin_shape_index\":29,\"travel_mode\":\"drive\"},{\"travel_type\":\"car\","
      "\"verbal_pre_transition_instruction\":\"Turn left to take the U.S. 30 West ramp toward "
      "York, Harrisburg.\",\"verbal_transition_alert_instruction\":\"Turn left to take the U.S. 30 "
      "West ramp.\",\"instruction\":\"Turn left to take the US 30 West ramp toward "
      "York/"
      "Harrisburg.\",\"end_shape_index\":53,\"type\":19,\"time\":48,\"begin_shape_index\":39,"
      "\"length\":0.327,\"sign\":{\"exit_toward_elements\":[{\"text\":\"York\"},{\"text\":"
      "\"Harrisburg\"}],\"exit_branch_elements\":[{\"consecutive_count\":1,\"text\":\"US 30 "
      "West\"}]},\"travel_mode\":\"drive\"},{\"travel_type\":\"car\",\"travel_mode\":\"drive\","
      "\"verbal_pre_transition_instruction\":\"Merge onto U.S. 30 West. Then U.S. 30 West will be "
      "on the right.\",\"verbal_post_transition_instruction\":\"Continue for 100 "
      "feet.\",\"instruction\":\"Merge onto US 30 "
      "West.\",\"end_shape_index\":54,\"type\":25,\"time\":6,\"verbal_multi_cue\":true,\"street_"
      "names\":[\"US 30 "
      "West\"],\"length\":0.023,\"begin_shape_index\":53},{\"travel_type\":\"car\",\"travel_mode\":"
      "\"drive\",\"begin_shape_index\":54,\"time\":0,\"type\":5,\"end_shape_index\":54,"
      "\"instruction\":\"US 30 West is on the "
      "right.\",\"length\":0.000,\"verbal_transition_alert_instruction\":\"U.S. 30 West will be on "
      "the right.\",\"verbal_pre_transition_instruction\":\"U.S. 30 West is on the right.\"}]" +
      "}]" + "}}";

  ////////////////////////////////////////////////////////////////////////////
  // Expected top level values
  std::string expected_language = "en-US";
  std::string expected_units = "miles";
  std::string expected_status_message = "Found route between points";
  uint32_t expected_status = 0;
  std::string expected_id = "SIMPLE_TEST";

  // Expected top level summary values
  float expected_summary_length = 2.168;
  uint32_t expected_summary_time = 287;
  float expected_summary_min_lat = 40.042500;
  float expected_summary_min_lon = -76.299324;
  float expected_summary_max_lat = 40.063713;
  float expected_summary_max_lon = -76.276863;

  // Expected top level location values
  size_t expected_location_count = 2;
  std::string expected_orig_street = "East Fulton Street";
  float expected_orig_lat = 40.042538;
  float expected_orig_lon = -76.299332;
  std::string expected_orig_type = "break";
  std::string expected_dest_street = "US 30 West";
  float expected_dest_lat = 40.063747;
  float expected_dest_lon = -76.282028;
  std::string expected_dest_type = "break";
  std::string expected_dest_side_of_street = "right";

  // Expected top level leg values
  size_t expected_leg_maneuver_count = 7;
  std::string leg_shape = "e__kkAxb}opCcBq\\\\sFk`Aqp@`HqRxBexAfNwHk@y`AgwA{KePed@qp@iCwDwM_S{z@"
                          "snAemA}gByoAekBqGoJgJwMiGmJiv@}hAil@q{@eU_]s_@{j@gYwc@mi@}}@qMiW{F}"
                          "JcPwX}Ymi@}Zmi@yoAqxBiHwNqk@gbA_Xqf@aDkKyVee@}OiWqBwCwI{Ks[a\\\\iByB}D`"
                          "HkGjKoHfNmEnIuE|IsGfOkUhl@{KvWeK~SaBvDaNhVwRp]c[li@eUrd@sL`S";
  std::string expected_leg_shape =
      "e__kkAxb}opCcBq\\sFk`Aqp@`HqRxBexAfNwHk@y`AgwA{KePed@qp@iCwDwM_S{z@snAemA}"
      "gByoAekBqGoJgJwMiGmJiv@}hAil@q{@eU_]s_@{j@gYwc@mi@}}@qMiW{F}JcPwX}Ymi@}Zmi@yoAqxBiHwNqk@gbA_"
      "Xqf@aDkKyVee@}OiWqBwCwI{Ks[a\\iByB}D`HkGjKoHfNmEnIuE|IsGfOkUhl@{KvWeK~SaBvDaNhVwRp]c[li@"
      "eUrd@sL`S";

  ////////////////////////////////////////////////////////////////////////////
  // Replace tags with values
  boost::replace_all(route_json_str, kLanguageTag, expected_language);
  boost::replace_all(route_json_str, kUnitsTag, expected_units);
  boost::replace_all(route_json_str, kStatusMessageTag, expected_status_message);
  boost::replace_all(route_json_str, kStatusTag, std::to_string(expected_status));
  boost::replace_all(route_json_str, kIdTag, expected_id);
  boost::replace_all(route_json_str, kSummaryLengthTag, std::to_string(expected_summary_length));
  boost::replace_all(route_json_str, kSummaryTimeTag, std::to_string(expected_summary_time));
  boost::replace_all(route_json_str, kSummaryMinLatTag, std::to_string(expected_summary_min_lat));
  boost::replace_all(route_json_str, kSummaryMinLonTag, std::to_string(expected_summary_min_lon));
  boost::replace_all(route_json_str, kSummaryMaxLatTag, std::to_string(expected_summary_max_lat));
  boost::replace_all(route_json_str, kSummaryMaxLonTag, std::to_string(expected_summary_max_lon));
  boost::replace_all(route_json_str, kLocationOrigStreetTag, expected_orig_street);
  boost::replace_all(route_json_str, kLocationOrigLatTag, std::to_string(expected_orig_lat));
  boost::replace_all(route_json_str, kLocationOrigLonTag, std::to_string(expected_orig_lon));
  boost::replace_all(route_json_str, kLocationOrigTypeTag, expected_orig_type);
  boost::replace_all(route_json_str, kLocationDestStreetTag, expected_dest_street);
  boost::replace_all(route_json_str, kLocationDestLatTag, std::to_string(expected_dest_lat));
  boost::replace_all(route_json_str, kLocationDestLonTag, std::to_string(expected_dest_lon));
  boost::replace_all(route_json_str, kLocationDestTypeTag, expected_dest_type);
  boost::replace_all(route_json_str, kLocationDestSideOfStreetTag, expected_dest_side_of_street);
  boost::replace_all(route_json_str, kLeg1ShapeTag, leg_shape);

  TryTopLevelSetRoute(route_json_str, expected_language, expected_units, expected_status_message,
                      expected_status, expected_id);

  TryTopLevelSummarySetRoute(route_json_str, expected_summary_length, expected_summary_time,
                             expected_summary_min_lat, expected_summary_min_lon,
                             expected_summary_max_lat, expected_summary_max_lon);

  TryTopLevelLocationSetRoute(route_json_str, expected_location_count, expected_orig_street,
                              expected_orig_lat, expected_orig_lon, expected_orig_type,
                              expected_dest_street, expected_dest_lat, expected_dest_lon,
                              expected_dest_type, expected_dest_side_of_street);

  TryTopLevelLegSetRoute(route_json_str, expected_leg_maneuver_count, expected_leg_shape);

  // Test runtime error

  std::string runtimeErrorTest = R"({trip:20})";
  NavigatorTest nt;
  NavigationStatus status = nt.SetRoute(runtimeErrorTest);
  if (status.route_state() != NavigationStatus_RouteState_kInvalid) {
    throw std::runtime_error("Invalid route did not return an invalid state.");
  }

  if (nt.GetRouteState() != NavigationStatus_RouteState_kInvalid) {
    throw std::runtime_error("Invalid route did not cause an internal invalid state.");
  }
}

void TryRouteLegCount(NavigatorTest& nav, int expected_leg_count) {

  int leg_count = nav.route().trip().legs_size();

  if (leg_count != expected_leg_count)
    throw std::runtime_error("Incorrect leg count for route - found: " + std::to_string(leg_count) +
                             " | expected: " + std::to_string(expected_leg_count));
}

void TryRouteManeuverCount(NavigatorTest& nav, uint32_t leg_index, int expected_leg_maneuver_count) {

  int maneuver_count = nav.route().trip().legs(leg_index).maneuvers_size();

  if (maneuver_count != expected_leg_maneuver_count)
    throw std::runtime_error(
        "Incorrect maneuver count for route - found: " + std::to_string(maneuver_count) +
        " | expected: " + std::to_string(expected_leg_maneuver_count));
}

void TryRouteLanguage(NavigatorTest& nav, std::string expected_language) {

  std::string language = nav.route().trip().language();

  if (language != expected_language)
    throw std::runtime_error("Incorrect language for route - found: " + language +
                             " | expected: " + expected_language);
}

void TryRouteUnits(NavigatorTest& nav, std::string expected_units, bool expected_units_boolean) {

  std::string units = nav.route().trip().units();
  bool units_boolean = nav.HasKilometerUnits();

  if (units != expected_units)
    throw std::runtime_error("Incorrect units for route - found: " + units +
                             " | expected: " + expected_units);

  if (units_boolean != expected_units_boolean)
    throw std::runtime_error("Incorrect units boolean for route - found: " +
                             std::string(units_boolean ? "true" : "false") + " | expected: " +
                             std::string(expected_units_boolean ? "true" : "false"));
}

void TryUnitsToMeters(float found_meters, float expected_meters) {

  float epsilon = 0.005f;
  if (expected_meters >= 100.0f)
    epsilon *= 10.0f;
  if (expected_meters >= 1000.0f)
    epsilon *= 10.0f;
  if (expected_meters >= 10000.0f)
    epsilon *= 10.0f;
  if (!valhalla::midgard::equal<float>(found_meters, expected_meters, epsilon))
    throw std::runtime_error("Incorrect units to meters - found: " + std::to_string(found_meters) +
                             " | expected: " + std::to_string(expected_meters));
}

void TryRemainingLegValues(NavigatorTest& nav,
                           uint32_t index,
                           float expected_remaining_leg_length,
                           uint32_t expected_remaining_leg_time) {

  float remaining_leg_length = nav.remaining_leg_values().at(index).first;
  uint32_t remaining_leg_time = nav.remaining_leg_values().at(index).second;

  if (!valhalla::midgard::equal<float>(remaining_leg_length, expected_remaining_leg_length, 0.005f))
    throw std::runtime_error(
        "Incorrect remaining leg length - found: " + std::to_string(remaining_leg_length) +
        " | expected: " + std::to_string(expected_remaining_leg_length));

  if (!valhalla::midgard::equal<float>(remaining_leg_time, expected_remaining_leg_time,
                                       (expected_remaining_leg_time * 0.0066f)))
    throw std::runtime_error(
        "Incorrect remaining leg time - found: " + std::to_string(remaining_leg_time) +
        " | expected: " + std::to_string(expected_remaining_leg_time));
}

void TryLength(std::string method, float found_length, float expected_length) {
  float epsilon = 0.005f;
  if (!valhalla::midgard::equal<float>(found_length, expected_length, epsilon))
    throw std::runtime_error("Incorrect " + method + " - found: " + std::to_string(found_length) +
                             " | expected: " + std::to_string(expected_length));
}

FixLocation GetFixLocation(float lon, float lat, uint64_t time) {
  FixLocation fix;
  fix.set_lon(lon);
  fix.set_lat(lat);
  fix.set_time(time);
  return fix;
}

FixLocation GetFixLocation(float lon, float lat, uint64_t time, float speed) {
  FixLocation fix;
  fix.set_lon(lon);
  fix.set_lat(lat);
  fix.set_time(time);
  fix.set_speed(speed);
  return fix;
}

FixLocation GetFixLocation(float lon,
                           float lat,
                           uint64_t time,
                           float speed,
                           float bearing,
                           float altitude,
                           float accuracy,
                           std::string provider) {
  FixLocation fix;
  fix.set_lon(lon);
  fix.set_lat(lat);
  fix.set_time(time);
  fix.set_speed(speed);
  fix.set_bearing(bearing);
  fix.set_altitude(altitude);
  fix.set_accuracy(accuracy);
  fix.set_provider(provider);
  return fix;
}

NavigationStatus GetNavigationStatus(NavigationStatus_RouteState route_state) {
  NavigationStatus nav_status;
  nav_status.set_route_state(route_state);
  return nav_status;
}

NavigationStatus GetNavigationStatus(NavigationStatus_RouteState route_state,
                                     float lon,
                                     float lat,
                                     uint32_t leg_index,
                                     float remaining_leg_length,
                                     uint32_t remaining_leg_time,
                                     uint32_t maneuver_index,
                                     float remaining_maneuver_length,
                                     uint32_t remaining_maneuver_time) {
  NavigationStatus nav_status;
  nav_status.set_route_state(route_state);
  nav_status.set_lon(lon);
  nav_status.set_lat(lat);
  nav_status.set_leg_index(leg_index);
  nav_status.set_remaining_leg_length(remaining_leg_length);
  nav_status.set_remaining_leg_time(remaining_leg_time);
  nav_status.set_maneuver_index(maneuver_index);
  nav_status.set_remaining_maneuver_length(remaining_maneuver_length);
  nav_status.set_remaining_maneuver_time(remaining_maneuver_time);
  return nav_status;
}

NavigationStatus GetNavigationStatus(NavigationStatus_RouteState route_state,
                                     float lon,
                                     float lat,
                                     uint32_t leg_index,
                                     float remaining_leg_length,
                                     uint32_t remaining_leg_time,
                                     uint32_t maneuver_index,
                                     float remaining_maneuver_length,
                                     uint32_t remaining_maneuver_time,
                                     uint32_t instruction_maneuver_index) {
  NavigationStatus nav_status;
  nav_status.set_route_state(route_state);
  nav_status.set_lon(lon);
  nav_status.set_lat(lat);
  nav_status.set_leg_index(leg_index);
  nav_status.set_remaining_leg_length(remaining_leg_length);
  nav_status.set_remaining_leg_time(remaining_leg_time);
  nav_status.set_maneuver_index(maneuver_index);
  nav_status.set_remaining_maneuver_length(remaining_maneuver_length);
  nav_status.set_remaining_maneuver_time(remaining_maneuver_time);
  nav_status.set_instruction_maneuver_index(instruction_maneuver_index);
  return nav_status;
}

NavigationStatus GetNavigationStatus(NavigationStatus_RouteState route_state,
                                     float lon,
                                     float lat,
                                     uint32_t leg_index,
                                     float remaining_leg_length,
                                     uint32_t remaining_leg_time,
                                     uint32_t maneuver_index,
                                     float remaining_maneuver_length,
                                     uint32_t remaining_maneuver_time,
                                     uint32_t instruction_maneuver_index,
                                     float alert_length) {
  NavigationStatus nav_status;
  nav_status.set_route_state(route_state);
  nav_status.set_lon(lon);
  nav_status.set_lat(lat);
  nav_status.set_leg_index(leg_index);
  nav_status.set_remaining_leg_length(remaining_leg_length);
  nav_status.set_remaining_leg_time(remaining_leg_time);
  nav_status.set_maneuver_index(maneuver_index);
  nav_status.set_remaining_maneuver_length(remaining_maneuver_length);
  nav_status.set_remaining_maneuver_time(remaining_maneuver_time);
  nav_status.set_instruction_maneuver_index(instruction_maneuver_index);
  nav_status.set_transition_alert_length(alert_length);
  return nav_status;
}

void ValidateNavigationStatus(const NavigationStatus& nav_status,
                              const NavigationStatus& expected_nav_status) {

  if (expected_nav_status.has_route_state() &&
      (nav_status.route_state() != expected_nav_status.route_state()))
    throw std::runtime_error(
        "Incorrect route state for NavigationStatus - found: " +
        NavigationStatus_RouteState_Name(nav_status.route_state()) +
        " | expected: " + NavigationStatus_RouteState_Name(expected_nav_status.route_state()));

  if (expected_nav_status.has_lat() &&
      (!valhalla::midgard::equal<float>(nav_status.lat(), expected_nav_status.lat(), 0.00002f)))
    throw std::runtime_error(
        "Incorrect lat for NavigationStatus - found: " + std::to_string(nav_status.lat()) +
        " | expected: " + std::to_string(expected_nav_status.lat()));

  if (expected_nav_status.has_lon() &&
      (!valhalla::midgard::equal<float>(nav_status.lon(), expected_nav_status.lon(), 0.00002f)))
    throw std::runtime_error(
        "Incorrect lon for NavigationStatus - found: " + std::to_string(nav_status.lon()) +
        " | expected: " + std::to_string(expected_nav_status.lon()));

  if (expected_nav_status.has_leg_index() &&
      (nav_status.leg_index() != expected_nav_status.leg_index()))
    throw std::runtime_error("Incorrect leg_index for NavigationStatus - found: " +
                             std::to_string(nav_status.leg_index()) +
                             " | expected: " + std::to_string(expected_nav_status.leg_index()));

  if (expected_nav_status.has_remaining_leg_length() &&
      (!valhalla::midgard::equal<float>(nav_status.remaining_leg_length(),
                                        expected_nav_status.remaining_leg_length(), 0.005f)))
    throw std::runtime_error("Incorrect remaining_leg_length for NavigationStatus - found: " +
                             std::to_string(nav_status.remaining_leg_length()) + " | expected: " +
                             std::to_string(expected_nav_status.remaining_leg_length()));

  if (expected_nav_status.has_remaining_leg_time() &&
      (!valhalla::midgard::equal<float>(nav_status.remaining_leg_time(),
                                        expected_nav_status.remaining_leg_time(),
                                        (expected_nav_status.remaining_leg_time() * 0.01f))))
    throw std::runtime_error("Incorrect remaining_leg_time for NavigationStatus - found: " +
                             std::to_string(nav_status.remaining_leg_time()) + " | expected: " +
                             std::to_string(expected_nav_status.remaining_leg_time()));

  if (expected_nav_status.has_maneuver_index() &&
      (nav_status.maneuver_index() != expected_nav_status.maneuver_index()))
    throw std::runtime_error("Incorrect maneuver_index for NavigationStatus - found: " +
                             std::to_string(nav_status.maneuver_index()) +
                             " | expected: " + std::to_string(expected_nav_status.maneuver_index()));

  if (expected_nav_status.has_remaining_maneuver_length() &&
      (!valhalla::midgard::equal<float>(nav_status.remaining_maneuver_length(),
                                        expected_nav_status.remaining_maneuver_length(), 0.005f)))
    throw std::runtime_error(
        "Incorrect remaining_maneuver_length for NavigationStatus - found: " +
        std::to_string(nav_status.remaining_maneuver_length()) +
        " | expected: " + std::to_string(expected_nav_status.remaining_maneuver_length()));

  if (expected_nav_status.has_remaining_maneuver_time() &&
      (!valhalla::midgard::equal<float>(nav_status.remaining_maneuver_time(),
                                        expected_nav_status.remaining_maneuver_time(),
                                        (expected_nav_status.remaining_maneuver_time() * 0.01f))))
    throw std::runtime_error("Incorrect remaining_maneuver_time for NavigationStatus - found: " +
                             std::to_string(nav_status.remaining_maneuver_time()) + " | expected: " +
                             std::to_string(expected_nav_status.remaining_maneuver_time()));

  if (expected_nav_status.has_instruction_maneuver_index() &&
      (nav_status.instruction_maneuver_index() != expected_nav_status.instruction_maneuver_index()))
    throw std::runtime_error(
        "Incorrect instruction maneuver_index for NavigationStatus - found: " +
        std::to_string(nav_status.instruction_maneuver_index()) +
        " | expected: " + std::to_string(expected_nav_status.instruction_maneuver_index()));

  if (expected_nav_status.has_transition_alert_length() &&
      (!valhalla::midgard::equal<float>(nav_status.transition_alert_length(),
                                        expected_nav_status.transition_alert_length(), 0.005f)))
    throw std::runtime_error("Incorrect transition_alert_length for NavigationStatus - found: " +
                             std::to_string(nav_status.transition_alert_length()) + " | expected: " +
                             std::to_string(expected_nav_status.transition_alert_length()));
}

void ValidateOnRouteLocationCloseToOrigin(bool found_value,
                                          bool expected_on_route_location_close_to_origin) {

  if (found_value != expected_on_route_location_close_to_origin)
    throw std::runtime_error(
        "Incorrect OnRouteLocationCloseToOrigin boolean - found: " +
        std::string(found_value ? "true" : "false") +
        " | expected: " + std::string(expected_on_route_location_close_to_origin ? "true" : "false"));
}

void TrySnapToRoute(NavigatorTest& nav,
                    const FixLocation& fix_location,
                    NavigationStatus expected_nav_status) {
  NavigationStatus nav_status = nav.SnapToRoute(fix_location);
  ValidateNavigationStatus(nav_status, expected_nav_status);
}

void TryRouteOnLocationChanged(NavigatorTest& nav,
                               const FixLocation& fix,
                               const NavigationStatus& expected_nav_status,
                               bool expected_on_route_location_close_to_origin = false) {
  NavigationStatus nav_status = nav.OnLocationChanged(fix);
  ValidateNavigationStatus(nav_status, expected_nav_status);
  ValidateOnRouteLocationCloseToOrigin(nav.OnRouteLocationCloseToOrigin(nav_status),
                                       expected_on_route_location_close_to_origin);
}

void TryUsedInstructions(bool found_value, bool expected_value) {

  if (found_value != expected_value)
    throw std::runtime_error(
        "Incorrect instruction used value - found: " + std::string(found_value ? "true" : "false") +
        " | expected: " + std::string(expected_value ? "true" : "false"));
}

void TryGetWordCount(const NavigatorTest& nav,
                     const std::string& instruction,
                     size_t expected_word_count) {

  size_t found_word_count = nav.GetWordCount(instruction);

  if (found_word_count != expected_word_count)
    throw std::runtime_error("Incorrect word count for the instruction=\"" + instruction +
                             "\" - found: " + std::to_string(found_word_count) +
                             " | expected: " + std::to_string(expected_word_count));
}

void TestGetWordCount(const NavigatorTest& nav) {
  std::string instruction;

  instruction = " . ; Test with,crazy:weird   stuff. ;ok  .";
  TryGetWordCount(nav, instruction, 6);

  /////////////////////////////////////////////////////////////////////////////
  // English

  // Turn right with no street names
  instruction = "Turn right.";
  TryGetWordCount(nav, instruction, 2);

  // Simple turn right
  instruction = "Turn right onto Middletown Road.";
  TryGetWordCount(nav, instruction, 5);

  // Turn right with multiple street names
  instruction = "Turn right onto Maryland 43 West, White Marsh Boulevard.";
  TryGetWordCount(nav, instruction, 9);

  // Turn right with multiple street names and multi-cue
  instruction = "Turn right onto Riverside Parkway, Maryland 5 43. Then Take the Interstate 95 "
                "North ramp on the right.";
  TryGetWordCount(nav, instruction, 18);

  // Destination
  instruction = "1 hundred South Charles Street will be on the left.";
  TryGetWordCount(nav, instruction, 10);

  // Bear left
  instruction = "Bear left onto South Lime Street.";
  TryGetWordCount(nav, instruction, 6);

  // Bear right with multi-cue
  instruction =
      "Bear right onto Lindle Road, Pennsylvania 4 41. Then Turn right onto Eisenhower Boulevard.";
  TryGetWordCount(nav, instruction, 14);

  // Continue
  instruction = "Continue for 500 feet.";
  TryGetWordCount(nav, instruction, 4);

  // Continue with decimal distance
  instruction = "Continue for 30.7 miles.";
  TryGetWordCount(nav, instruction, 5);

  // Continue with multi-cue
  instruction = "Continue on Virginia 7 Business, Harry Byrd Highway for 1 tenth of a mile. Then "
                "Take the ramp on the right.";
  TryGetWordCount(nav, instruction, 21);

  // Origin
  instruction = "Drive east on East Fulton Street for 400 feet.";
  TryGetWordCount(nav, instruction, 9);

  // Origin with multi-cue
  instruction = "Drive southeast on Interstate 2 83 South for 1 tenth of a mile. Then Take exit 2 "
                "on the right.";
  TryGetWordCount(nav, instruction, 20);

  // Enter roundabout
  instruction = "Enter the roundabout and take the 2nd exit.";
  TryGetWordCount(nav, instruction, 8);

  // Exit roundabout
  instruction = "Exit the roundabout onto Philadelphia Road, Maryland 7.";
  TryGetWordCount(nav, instruction, 8);

  // Keep left with multi-cue
  instruction = "Keep left to take Pennsylvania 7 43 South toward Grantville, Hershey. Then Turn "
                "left onto Ridge Road.";
  TryGetWordCount(nav, instruction, 17);

  // Keep right with multi-cue
  instruction = "Keep right to take New Holland Avenue toward Downtown Lancaster. Then Bear right "
                "onto New Holland Pike.";
  TryGetWordCount(nav, instruction, 17);

  // Keep straight with multi-cue
  instruction = "Keep straight to take Pennsylvania 4 41 South toward Middletown. Then Turn left "
                "onto Lindle Road.";
  TryGetWordCount(nav, instruction, 16);

  // Merge
  instruction = "Merge onto Interstate 90 East, Massachusetts Turnpike.";
  TryGetWordCount(nav, instruction, 7);

  // Take exit
  instruction = "Take exit 2 42 on the right onto Interstate 83 toward York, Baltimore.";
  TryGetWordCount(nav, instruction, 13);

  /////////////////////////////////////////////////////////////////////////////
  // German
  // "Take the A 9 ramp on the right toward Stuttgart, Nürnberg."
  instruction = "Über die Auffahrt leicht nach rechts auf A 9 abbiegen Richtung Stuttgart, Nürnberg.";
  TryGetWordCount(nav, instruction, 13);

  /////////////////////////////////////////////////////////////////////////////
  // Russian
  // "Continue on Триумфальная площадь, Triumfalnaya Square for 90 meters. Then Continue on 1-я
  // Тверская-Ямская улица."
  instruction = "Проедьте 90 метров по Триумфальная площадь, Triumfalnaya Square. Затем "
                "Продолжайте по 1-я Тверская-Ямская улица.";
  TryGetWordCount(nav, instruction, 16);

  /////////////////////////////////////////////////////////////////////////////
  // Hindi
  // "Continue on Mahatma Gandhi Road for 1.1 kilometers."
  instruction = "Mahatma Gandhi Road पर 1.1 किलोमीटर के लिए जारी रहें.";
  TryGetWordCount(nav, instruction, 11);
}

void TryGetPreTransitionThreshold(const NavigatorTest& nav,
                                  uint32_t leg_index,
                                  size_t instruction_index,
                                  uint32_t expected_time) {

  const auto& instruction = nav.route()
                                .trip()
                                .legs(leg_index)
                                .maneuvers(instruction_index)
                                .verbal_pre_transition_instruction();
  uint32_t found_time = nav.GetPreTransitionThreshold(instruction_index);

  if (found_time != expected_time)
    throw std::runtime_error("Incorrect PreTransitionThreshold time for the instruction=\"" +
                             instruction + "\" - found: " + std::to_string(found_time) +
                             " | expected: " + std::to_string(expected_time));
}

/* AutoLancasterToHershey geojson to visualize test route
{"type":"FeatureCollection","features":[
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299179,40.042572]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":0}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299057,40.042595]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298897,40.042610]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298691,40.042622]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":3}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298584,40.042633]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":4}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298477,40.042645]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":5}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298363,40.042652]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":6}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298256,40.042667]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":7}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298088,40.042683]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":8}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297966,40.042698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":9}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297844,40.042709]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":10}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297791,40.042622]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":11}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297768,40.042503]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":12}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297745,40.042370]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":13}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297729,40.042263]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":14}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297752,40.042183]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":15}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297714,40.042095]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":16}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297585,40.042072]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":17}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297409,40.042095]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":18}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297173,40.042126]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":19}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297020,40.042133]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":20}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296860,40.042149]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":21}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296707,40.042156]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":22}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296547,40.042171]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":23}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296379,40.042183]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":24}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296219,40.042191]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":25}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296036,40.042210]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":26}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295860,40.042233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":27}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295685,40.042252]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":28}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295502,40.042267]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":29}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295326,40.042286]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":30}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295143,40.042305]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":31}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294975,40.042320]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":32}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294800,40.042336]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":33}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294624,40.042355]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":34}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294456,40.042374]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":35}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294289,40.042393]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":36}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294128,40.042404]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":37}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293953,40.042412]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":38}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293777,40.042427]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":39}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293610,40.042439]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":40}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293442,40.042454]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":41}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293266,40.042477]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":42}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293106,40.042496]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":43}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292946,40.042522]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":44}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292793,40.042542]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":45}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292648,40.042561]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":46}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292534,40.042572]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":47}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292419,40.042587]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":48}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292236,40.042606]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":49}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292099,40.042625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":50}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291992,40.042648]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":51}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291855,40.042671]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":52}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291687,40.042683]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":53}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291573,40.042690]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":54}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291451,40.042698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":55}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291328,40.042709]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":56}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291199,40.042725]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":57}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291061,40.042740]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":58}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290916,40.042767]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":59}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290756,40.042789]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":60}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290588,40.042812]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":61}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290421,40.042831]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":62}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290245,40.042847]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":63}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290054,40.042858]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":64}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289879,40.042877]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":65}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289703,40.042885]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":66}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289513,40.042896]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":67}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289330,40.042919]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":68}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289162,40.042919]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":69}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288986,40.042927]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":70}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288811,40.042942]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":71}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288643,40.042942]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":72}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288498,40.042965]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":73}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288368,40.042976]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":74}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288246,40.042999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":75}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288094,40.043018]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":76}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287971,40.043041]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":77}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287804,40.043053]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":78}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287689,40.043060]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":79}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287582,40.043079]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":80}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287437,40.043106]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":81}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287300,40.043125]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":82}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287132,40.043144]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":83}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286972,40.043152]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":84}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286858,40.043198]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":85}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286713,40.043247]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":86}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286575,40.043289]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":87}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286469,40.043369]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":88}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286377,40.043465]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":89}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286263,40.043530]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":90}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286171,40.043652]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":91}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286049,40.043751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":92}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285942,40.043869]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":93}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285843,40.043999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":94}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285736,40.044117]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":95}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285622,40.044243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":96}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285515,40.044357]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":97}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285385,40.044476]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":98}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285240,40.044582]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":99}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285088,40.044678]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":100}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284920,40.044777]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":101}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284737,40.044811]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284348,40.044895]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":103}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284149,40.044918]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":104}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283936,40.044945]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":105}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283714,40.044964]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":106}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283501,40.044991]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283272,40.045017]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283035,40.045055]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282822,40.045097]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":110}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282600,40.045158]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":111}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282372,40.045235]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":112}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282150,40.045319]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281937,40.045422]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":114}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281738,40.045521]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":115}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281548,40.045647]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":116}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281357,40.045776]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281158,40.045910]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":118}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280968,40.046051]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":119}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280777,40.046200]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":120}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280609,40.046352]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":121}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280441,40.046509]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280281,40.046665]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280121,40.046829]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279968,40.047005]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279816,40.047176]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":126}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279686,40.047379]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":127}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279549,40.047562]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279449,40.047771]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279366,40.047974]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":130}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279259,40.048183]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":131}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279152,40.048409]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":132}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279053,40.048611]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":133}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278938,40.048809]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":134}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278839,40.049030]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":135}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278748,40.049236]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":136}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278648,40.049446]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":137}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278557,40.049648]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":138}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278458,40.049854]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":139}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278366,40.050049]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":140}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278252,40.050247]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":141}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278152,40.050434]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":142}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278053,40.050625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":143}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277954,40.050804]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":144}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277863,40.050987]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":145}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277779,40.051167]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":146}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277687,40.051342]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":147}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277580,40.051506]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":148}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277458,40.051674]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":149}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277313,40.051830]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":150}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277145,40.051971]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":151}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276955,40.052097]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":152}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276749,40.052212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":153}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276535,40.052315]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":154}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276299,40.052391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":155}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276054,40.052448]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":156}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275810,40.052498]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":157}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275551,40.052547]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":158}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275291,40.052593]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":159}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275024,40.052643]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":160}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274765,40.052704]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":161}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274498,40.052761]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":162}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274239,40.052837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":163}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273972,40.052914]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":164}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273705,40.052998]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":165}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273445,40.053085]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":166}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273178,40.053169]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":167}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272911,40.053265]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":168}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272636,40.053352]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":169}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272369,40.053448]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":170}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272095,40.053539]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":171}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271820,40.053631]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":172}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271553,40.053722]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":173}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271271,40.053814]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":174}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270996,40.053909]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":175}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270721,40.054001]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":176}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270447,40.054096]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":177}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270172,40.054192]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":178}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269897,40.054283]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":179}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269630,40.054375]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":180}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269363,40.054462]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":181}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269104,40.054546]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":182}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268837,40.054634]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268593,40.054722]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":184}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268349,40.054802]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":185}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268127,40.054886]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267899,40.054970]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267670,40.055050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":188}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267448,40.055126]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":189}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267250,40.055202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":190}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267136,40.055286]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":191}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266953,40.055340]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":192}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266800,40.055397]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266640,40.055450]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266510,40.055492]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":195}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266411,40.055523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":196}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266327,40.055576]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":197}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266243,40.055698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":198}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266304,40.055779]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266342,40.055862]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":200}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266388,40.055958]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":201}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266426,40.056053]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":202}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266472,40.056168]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":203}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266579,40.056293]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":204}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266632,40.056404]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266731,40.056530]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266861,40.056641]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266991,40.056747]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267166,40.056835]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267326,40.056892]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":210}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267502,40.056950]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":211}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267723,40.056896]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":212}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267929,40.056896]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":213}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268097,40.056889]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268311,40.056744]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":215}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268486,40.056648]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":216}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268661,40.056557]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":217}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268852,40.056469]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269066,40.056408]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269272,40.056358]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":220}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269485,40.056335]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269699,40.056339]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269928,40.056362]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270149,40.056408]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":224}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270355,40.056477]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270569,40.056568]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":226}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270775,40.056679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":227}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270988,40.056789]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":228}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271202,40.056908]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":229}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271423,40.057030]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":230}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271652,40.057152]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":231}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271889,40.057278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":232}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272118,40.057407]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":233}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272362,40.057537]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":234}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272614,40.057671]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272858,40.057804]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":236}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273117,40.057945]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":237}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273369,40.058079]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":238}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273628,40.058212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":239}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273895,40.058353]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":240}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274162,40.058491]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":241}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274437,40.058640]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":242}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274696,40.058792]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":243}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274956,40.058945]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275215,40.059097]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275467,40.059258]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":246}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275711,40.059422]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":247}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275955,40.059578]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":248}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276199,40.059742]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":249}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276443,40.059910]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":250}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276695,40.060078]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":251}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276947,40.060246]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":252}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277191,40.060417]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":253}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277435,40.060585]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":254}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277687,40.060745]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":255}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277946,40.060913]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":256}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278198,40.061077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":257}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278442,40.061241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":258}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278687,40.061413]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278931,40.061573]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":260}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279175,40.061741]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279427,40.061905]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":262}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279671,40.062073]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":263}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279922,40.062244]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":264}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280174,40.062412]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280418,40.062584]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":266}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280663,40.062744]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280899,40.062912]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":268}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281128,40.063068]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":269}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281364,40.063229]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":270}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281601,40.063389]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281837,40.063541]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282074,40.063705]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":273}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282318,40.063866]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":274}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282555,40.064034]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":275}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282799,40.064198]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":276}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283035,40.064365]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":277}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283287,40.064529]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":278}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283531,40.064697]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":279}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283775,40.064873]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":280}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284027,40.065044]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":281}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284279,40.065212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":282}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284531,40.065388]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":283}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284782,40.065563]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":284}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285034,40.065735]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":285}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285294,40.065907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":286}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285545,40.066074]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":287}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285805,40.066235]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286072,40.066391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286346,40.066544]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":290}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286621,40.066685]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":291}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286896,40.066818]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287170,40.066952]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":293}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287453,40.067074]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":294}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287743,40.067184]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":295}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288033,40.067287]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":296}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288330,40.067383]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":297}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288635,40.067467]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288933,40.067554]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289253,40.067642]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":300}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289566,40.067722]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":301}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289879,40.067787]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":302}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290192,40.067848]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":303}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290512,40.067902]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":304}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290825,40.067947]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":305}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291130,40.067986]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":306}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291435,40.068020]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":307}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291748,40.068047]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292061,40.068066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":309}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292374,40.068081]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292686,40.068089]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":311}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292999,40.068092]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":312}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293327,40.068085]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":313}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293648,40.068073]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":314}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293968,40.068066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":315}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294304,40.068062]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":316}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294624,40.068054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":317}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294968,40.068058]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":318}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295303,40.068050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":319}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295647,40.068043]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":320}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295982,40.068031]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":321}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296326,40.068024]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":322}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296677,40.068024]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":323}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297028,40.068016]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":324}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297371,40.068008]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":325}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297722,40.068005]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":326}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298073,40.068001]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":327}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298424,40.068008]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":328}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298782,40.068020]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":329}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299133,40.068031]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":330}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299492,40.068054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":331}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299850,40.068081]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":332}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300209,40.068108]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":333}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300560,40.068146]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":334}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300919,40.068188]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":335}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301277,40.068230]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":336}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301628,40.068275]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":337}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301979,40.068325]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":338}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302338,40.068371]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":339}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302689,40.068417]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":340}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303032,40.068462]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":341}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303375,40.068512]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":342}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303711,40.068558]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":343}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304047,40.068600]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":344}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304367,40.068642]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":345}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304695,40.068687]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":346}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305016,40.068729]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":347}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305344,40.068779]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":348}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305672,40.068825]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":349}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306007,40.068871]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":350}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306343,40.068916]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":351}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306671,40.068962]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":352}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306999,40.069008]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":353}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307327,40.069050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307655,40.069096]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307991,40.069145]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.308319,40.069195]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":357}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.308655,40.069241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":358}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.308983,40.069286]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.309319,40.069332]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.309662,40.069378]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":361}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.309998,40.069427]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.310341,40.069477]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":363}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.310684,40.069523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":364}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.311028,40.069572]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":365}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.311378,40.069622]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.311722,40.069672]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":367}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.312080,40.069717]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":368}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.312431,40.069767]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":369}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.312782,40.069817]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":370}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.313141,40.069862]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":371}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.313492,40.069912]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":372}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.313850,40.069965]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":373}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.314209,40.070015]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.314568,40.070065]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":375}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.314919,40.070114]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":376}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.315277,40.070164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":377}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.315628,40.070213]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":378}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.315979,40.070267]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":379}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.316322,40.070312]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":380}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.316666,40.070358]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":381}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.317017,40.070404]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":382}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.317368,40.070450]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":383}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.317711,40.070496]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":384}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.318054,40.070541]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":385}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.318405,40.070583]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":386}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.318748,40.070625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":387}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.319099,40.070667]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.319450,40.070702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":389}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.319809,40.070736]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":390}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.320160,40.070766]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":391}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.320511,40.070793]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":392}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.320869,40.070827]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":393}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.321213,40.070854]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":394}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.321564,40.070885]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":395}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.321915,40.070911]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":396}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.322266,40.070942]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":397}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.322609,40.070976]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":398}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.322960,40.071018]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":399}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.323303,40.071072]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":400}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.323654,40.071136]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":401}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.324005,40.071209]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":402}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.324341,40.071285]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":403}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.324677,40.071369]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":404}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.325020,40.071442]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":405}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.325378,40.071526]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":406}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.325729,40.071609]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":407}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.326073,40.071690]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":408}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.326424,40.071770]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":409}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.326775,40.071850]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":410}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.327103,40.071930]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":411}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.327431,40.072018]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":412}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.327751,40.072117]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":413}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.328072,40.072212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":414}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.328400,40.072315]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":415}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.328728,40.072418]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":416}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.329048,40.072533]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":417}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.329376,40.072647]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":418}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.329689,40.072769]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":419}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.330002,40.072895]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.330315,40.073017]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.330620,40.073147]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.330933,40.073280]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":423}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.331238,40.073418]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":424}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.331543,40.073559]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":425}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.331848,40.073704]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":426}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.332161,40.073849]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":427}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.332474,40.073990]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":428}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.332787,40.074139]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":429}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.333099,40.074280]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":430}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.333420,40.074425]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":431}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.333733,40.074570]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":432}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.334053,40.074718]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":433}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.334373,40.074863]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.334686,40.075012]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.334999,40.075157]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.335304,40.075298]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":437}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.335609,40.075439]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":438}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.335907,40.075577]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":439}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.336205,40.075718]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":440}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.336502,40.075851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.336807,40.075989]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":442}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.337105,40.076126]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":443}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.337402,40.076267]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":444}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.337708,40.076405]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":445}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.338013,40.076542]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":446}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.338310,40.076683]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":447}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.338623,40.076820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":448}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.338928,40.076965]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":449}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.339233,40.077110]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":450}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.339546,40.077251]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.339859,40.077396]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":452}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.340172,40.077541]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":453}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.340492,40.077686]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":454}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.340805,40.077835]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":455}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.341125,40.077980]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":456}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.341454,40.078129]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":457}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.341774,40.078278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":458}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.342102,40.078426]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":459}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.342422,40.078571]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":460}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.342751,40.078724]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.343079,40.078876]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":462}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.343407,40.079029]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":463}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.343750,40.079178]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":464}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.344078,40.079327]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":465}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.344421,40.079464]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":466}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.344757,40.079597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":467}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.345100,40.079731]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":468}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.345444,40.079853]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":469}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.345787,40.079971]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":470}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.346130,40.080086]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.346474,40.080193]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.346825,40.080296]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.347176,40.080391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":474}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.347527,40.080482]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":475}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.347885,40.080574]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.348251,40.080658]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.348618,40.080738]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":478}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.348991,40.080814]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":479}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.349365,40.080887]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":480}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.349739,40.080952]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":481}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.350121,40.081013]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":482}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.350494,40.081066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.350868,40.081127]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.351242,40.081184]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.351616,40.081245]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":486}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.351990,40.081306]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":487}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.352364,40.081360]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":488}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.352737,40.081421]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":489}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.353111,40.081474]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":490}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.353485,40.081532]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":491}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.353859,40.081593]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":492}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.354233,40.081650]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":493}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.354599,40.081707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":494}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.354965,40.081760]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":495}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.355339,40.081821]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":496}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.355713,40.081882]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.356087,40.081936]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":498}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.356468,40.081993]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":499}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.356850,40.082050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":500}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.357231,40.082115]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":501}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.357620,40.082172]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":502}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.358009,40.082233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":503}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.358398,40.082294]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":504}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.358780,40.082359]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":505}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.359169,40.082417]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":506}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.359550,40.082478]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":507}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.359932,40.082539]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":508}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.360321,40.082596]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":509}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.360703,40.082657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":510}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.361092,40.082718]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":511}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.361473,40.082775]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":512}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.361855,40.082836]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":513}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.362236,40.082893]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":514}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.362617,40.082954]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":515}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.362999,40.083019]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":516}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.363380,40.083076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":517}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.363762,40.083138]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":518}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.364151,40.083199]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":519}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.364532,40.083256]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":520}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.364922,40.083313]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.365303,40.083374]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":522}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.365685,40.083435]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":523}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.366074,40.083492]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":524}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.366455,40.083557]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":525}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.366844,40.083614]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":526}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.367226,40.083675]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":527}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.367615,40.083744]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.368004,40.083813]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":529}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.368385,40.083893]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.368767,40.083981]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.369148,40.084072]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":532}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.369522,40.084167]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":533}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.369888,40.084267]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":534}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.370247,40.084366]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":535}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.370605,40.084473]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":536}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.370949,40.084583]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":537}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.371292,40.084698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":538}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.371628,40.084812]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.371956,40.084934]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.372284,40.085064]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.372620,40.085197]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":542}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.372948,40.085335]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.373276,40.085476]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":544}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.373596,40.085625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":545}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.373917,40.085770]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":546}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.374229,40.085918]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":547}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.374557,40.086071]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":548}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.374878,40.086224]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.375198,40.086369]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":550}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.375526,40.086517]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":551}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.375839,40.086666]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":552}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.376160,40.086815]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":553}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.376472,40.086964]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":554}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.376785,40.087109]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.377098,40.087257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":556}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.377411,40.087402]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":557}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.377716,40.087547]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":558}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.378021,40.087696]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":559}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.378326,40.087837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":560}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.378632,40.087978]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":561}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.378937,40.088123]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.379242,40.088272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":563}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.379555,40.088421]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.379868,40.088566]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":565}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.380180,40.088711]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":566}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.380501,40.088863]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.380814,40.089012]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":568}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.381134,40.089165]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":569}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.381454,40.089314]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":570}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.381775,40.089466]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":571}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.382103,40.089619]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":572}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.382439,40.089767]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":573}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.382767,40.089920]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":574}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.383095,40.090076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":575}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.383430,40.090233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":576}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.383766,40.090389]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.384102,40.090546]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":578}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.384438,40.090706]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":579}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.384773,40.090862]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":580}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.385109,40.091019]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":581}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.385445,40.091175]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":582}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.385773,40.091331]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":583}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.386108,40.091488]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":584}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.386436,40.091640]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":585}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.386772,40.091801]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":586}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.387115,40.091957]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":587}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.387451,40.092117]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":588}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.387787,40.092278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":589}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.388123,40.092434]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":590}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.388458,40.092590]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":591}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.388802,40.092747]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":592}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.389137,40.092907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":593}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.389473,40.093063]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":594}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.389816,40.093227]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.390160,40.093388]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.390503,40.093544]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":597}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.390846,40.093704]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":598}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.391182,40.093868]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":599}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.391525,40.094028]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":600}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.391869,40.094185]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":601}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.392212,40.094345]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":602}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.392548,40.094505]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":603}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.392883,40.094662]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":604}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.393227,40.094822]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":605}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.393570,40.094978]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":606}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.393906,40.095131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":607}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.394249,40.095280]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":608}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.394592,40.095421]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":609}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.394943,40.095554]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.395294,40.095684]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.395653,40.095814]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.396011,40.095928]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.396370,40.096046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.396729,40.096153]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.397095,40.096256]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.397446,40.096355]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":617}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.397797,40.096443]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":618}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.398148,40.096523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":619}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.398499,40.096596]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":620}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.398842,40.096668]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":621}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.399178,40.096729]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":622}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.399521,40.096790]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":623}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.399872,40.096851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":624}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.400215,40.096905]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":625}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.400551,40.096954]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":626}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.400887,40.097000]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":627}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.401222,40.097050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":628}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.401566,40.097095]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.401909,40.097145]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.402245,40.097198]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":631}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.402557,40.097240]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":632}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.402855,40.097282]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":633}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.403137,40.097321]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":634}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.403404,40.097359]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.403679,40.097397]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":636}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.403954,40.097439]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":637}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.404228,40.097481]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":638}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.404503,40.097523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":639}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.404785,40.097565]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.405075,40.097607]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":641}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.405373,40.097649]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":642}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.405678,40.097694]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.405991,40.097744]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.406311,40.097790]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.406631,40.097836]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.406960,40.097881]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":647}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.407288,40.097923]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":648}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.407608,40.097973]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.407944,40.098019]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":650}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.408272,40.098076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":651}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.408607,40.098129]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":652}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.408936,40.098186]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":653}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.409271,40.098255]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":654}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.409599,40.098328]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":655}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.409920,40.098404]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":656}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.410240,40.098480]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":657}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.410568,40.098560]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":658}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.410889,40.098640]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":659}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.411201,40.098732]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.411514,40.098827]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":661}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.411835,40.098927]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":662}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.412148,40.099033]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":663}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.412468,40.099140]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":664}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.412781,40.099255]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":665}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.413101,40.099373]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":666}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.413429,40.099491]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":667}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.413742,40.099621]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":668}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.414070,40.099751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":669}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.414391,40.099884]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":670}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.414719,40.100018]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":671}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.415047,40.100151]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":672}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.415375,40.100285]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":673}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.415703,40.100410]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":674}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.416031,40.100540]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":675}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.416359,40.100674]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":676}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.416687,40.100803]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":677}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.417007,40.100933]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":678}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.417336,40.101063]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":679}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.417664,40.101196]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":680}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.417992,40.101334]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":681}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.418312,40.101463]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":682}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.418640,40.101601]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":683}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.418961,40.101734]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.419281,40.101871]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":685}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.419601,40.102005]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":686}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.419922,40.102142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.420242,40.102272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":688}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.420555,40.102402]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":689}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.420883,40.102531]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":690}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.421204,40.102657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":691}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.421524,40.102787]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":692}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.421844,40.102917]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":693}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.422165,40.103046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":694}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.422493,40.103176]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":695}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.422821,40.103310]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":696}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.423149,40.103443]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":697}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.423477,40.103580]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":698}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.423798,40.103710]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":699}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.424126,40.103844]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.424454,40.103970]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":701}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.424782,40.104099]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":702}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.425102,40.104229]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":703}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.425430,40.104359]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":704}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.425751,40.104488]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":705}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.426079,40.104618]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":706}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.426399,40.104748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":707}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.426727,40.104877]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":708}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.427048,40.105007]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":709}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.427376,40.105145]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.427711,40.105278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":711}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.428047,40.105412]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":712}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.428383,40.105545]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":713}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.428719,40.105679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":714}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.429047,40.105816]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":715}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.429382,40.105953]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":716}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.429718,40.106091]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":717}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.430054,40.106228]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":718}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.430397,40.106361]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":719}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.430733,40.106503]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":720}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.431068,40.106636]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":721}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.431404,40.106770]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":722}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.431740,40.106907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":723}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.432083,40.107040]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":724}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.432419,40.107178]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":725}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.432755,40.107315]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":726}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.433098,40.107449]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":727}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.433434,40.107590]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":728}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.433769,40.107723]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.434113,40.107861]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":730}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.434456,40.107998]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.434799,40.108139]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":732}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.435143,40.108276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":733}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.435486,40.108418]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":734}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.435829,40.108551]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":735}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.436165,40.108688]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":736}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.436508,40.108826]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":737}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.436844,40.108959]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":738}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.437187,40.109097]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":739}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.437531,40.109226]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":740}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.437874,40.109356]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":741}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.438225,40.109489]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":742}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.438576,40.109631]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.438919,40.109772]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":744}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.439270,40.109913]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":745}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.439613,40.110054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":746}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.439957,40.110199]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":747}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.440292,40.110344]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":748}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.440620,40.110489]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":749}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.440948,40.110634]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":750}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.441277,40.110779]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":751}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.441597,40.110931]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":752}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.441917,40.111080]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":753}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.442238,40.111233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":754}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.442551,40.111385]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":755}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.442863,40.111542]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":756}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.443176,40.111694]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":757}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.443489,40.111851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":758}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.443802,40.112007]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":759}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.444107,40.112160]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":760}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.444420,40.112316]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":761}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.444733,40.112473]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":762}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.445038,40.112629]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":763}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.445343,40.112782]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":764}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.445656,40.112934]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":765}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.445961,40.113087]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":766}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.446274,40.113239]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":767}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.446587,40.113392]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":768}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.446892,40.113552]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.447205,40.113705]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.447517,40.113865]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.447830,40.114021]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":772}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.448143,40.114178]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":773}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.448456,40.114338]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.448776,40.114494]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":775}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.449089,40.114658]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":776}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.449417,40.114819]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":777}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.449738,40.114975]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":778}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.450058,40.115131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.450386,40.115288]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":780}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.450699,40.115444]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":781}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.451019,40.115604]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":782}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.451347,40.115765]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":783}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.451668,40.115925]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":784}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.451981,40.116089]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":785}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.452301,40.116249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":786}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.452621,40.116413]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":787}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.452934,40.116570]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":788}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.453255,40.116730]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":789}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.453575,40.116886]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":790}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.453896,40.117046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":791}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.454224,40.117207]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":792}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.454544,40.117363]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":793}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.454857,40.117527]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":794}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.455185,40.117687]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":795}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.455505,40.117851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.455826,40.118011]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":797}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.456146,40.118176]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":798}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.456467,40.118336]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":799}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.456787,40.118492]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":800}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.457108,40.118649]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":801}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.457428,40.118809]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":802}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.457748,40.118973]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":803}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.458069,40.119137]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":804}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.458389,40.119293]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.458710,40.119450]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.459030,40.119610]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.459343,40.119766]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.459663,40.119923]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":809}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.459984,40.120083]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":810}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.460304,40.120243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":811}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.460625,40.120399]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.460945,40.120560]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.461266,40.120720]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":814}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.461578,40.120880]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":815}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.461899,40.121037]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":816}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.462212,40.121189]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":817}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.462517,40.121346]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":818}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.462822,40.121498]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":819}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.463127,40.121647]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":820}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.463425,40.121799]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":821}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.463722,40.121948]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":822}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.464020,40.122097]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":823}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.464310,40.122242]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":824}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.464607,40.122391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":825}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.464905,40.122540]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":826}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.465195,40.122688]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":827}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.465485,40.122837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":828}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.465775,40.122982]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":829}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.466072,40.123127]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":830}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.466362,40.123272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.466652,40.123417]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":832}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.466934,40.123562]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":833}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.467216,40.123699]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":834}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.467499,40.123833]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":835}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.467789,40.123974]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":836}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.468071,40.124111]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":837}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.468353,40.124256]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":838}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.468636,40.124397]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.468925,40.124546]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":840}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.469208,40.124691]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.469498,40.124832]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":842}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.469795,40.124973]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":843}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.470093,40.125118]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.470390,40.125263]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":845}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.470688,40.125408]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":846}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.470993,40.125553]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":847}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.471283,40.125702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":848}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.471588,40.125851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":849}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.471893,40.125996]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.472206,40.126144]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.472511,40.126297]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":852}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.472816,40.126450]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":853}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.473129,40.126602]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":854}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.473434,40.126759]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":855}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.473755,40.126919]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.474068,40.127079]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":857}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.474388,40.127239]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":858}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.474709,40.127399]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.475029,40.127560]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.475349,40.127720]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":861}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.475677,40.127880]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.475998,40.128044]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.476318,40.128204]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":864}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.476639,40.128361]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":865}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.476959,40.128517]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":866}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.477287,40.128666]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":867}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.477615,40.128811]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":868}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.477951,40.128956]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.478294,40.129093]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":870}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.478630,40.129227]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.478973,40.129356]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":872}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.479324,40.129478]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":873}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.479668,40.129585]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":874}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.480019,40.129696]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":875}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.480362,40.129799]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":876}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.480721,40.129902]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":877}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.481071,40.129997]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":878}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.481430,40.130089]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":879}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.481789,40.130177]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":880}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.482147,40.130257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":881}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.482506,40.130329]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":882}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.482864,40.130402]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":883}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.483238,40.130466]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":884}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.483604,40.130524]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":885}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.483978,40.130581]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":886}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.484352,40.130630]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.484726,40.130672]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.485100,40.130711]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.485481,40.130749]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.485863,40.130779]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":891}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.486244,40.130802]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":892}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.486626,40.130821]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":893}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.487015,40.130829]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":894}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.487396,40.130836]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.487778,40.130836]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":896}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.488167,40.130833]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":897}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.488548,40.130825]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":898}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.488937,40.130814]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":899}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.489326,40.130798]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":900}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.489700,40.130772]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":901}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.490082,40.130741]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":902}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.490456,40.130699]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":903}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.490837,40.130657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":904}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.491203,40.130611]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":905}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.491577,40.130566]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":906}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.491943,40.130520]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":907}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.492317,40.130474]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":908}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.492683,40.130424]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":909}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.493042,40.130379]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":910}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.493416,40.130329]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":911}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.493790,40.130283]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":912}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.494164,40.130238]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":913}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.494537,40.130188]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":914}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.494911,40.130142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.495285,40.130093]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.495659,40.130047]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.496033,40.130001]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":918}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.496407,40.129955]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":919}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.496788,40.129910]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":920}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.497162,40.129860]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":921}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.497543,40.129814]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":922}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.497917,40.129765]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":923}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.498291,40.129719]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":924}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.498672,40.129673]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":925}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.499046,40.129623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":926}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.499428,40.129578]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":927}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.499802,40.129528]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":928}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.500183,40.129478]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":929}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.500557,40.129429]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":930}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.500931,40.129379]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.501305,40.129333]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":932}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.501686,40.129288]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.502060,40.129238]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":934}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.502441,40.129192]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":935}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.502815,40.129143]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":936}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.503189,40.129101]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":937}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.503571,40.129055]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":938}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.503944,40.129005]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":939}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.504318,40.128956]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.504700,40.128906]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":941}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.505074,40.128857]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":942}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.505447,40.128807]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":943}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.505829,40.128757]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":944}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.506210,40.128708]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":945}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.506592,40.128666]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.506966,40.128616]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.507347,40.128567]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":948}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.507721,40.128521]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":949}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.508102,40.128471]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":950}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.508476,40.128426]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":951}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.508858,40.128372]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.509254,40.128345]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":953}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.509628,40.128292]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":954}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.510002,40.128239]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":955}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.510376,40.128189]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":956}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.510750,40.128139]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":957}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.511131,40.128094]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":958}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.511505,40.128048]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":959}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.511879,40.128014]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":960}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.512260,40.127987]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":961}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.512642,40.127968]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":962}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.513023,40.127953]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":963}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.513412,40.127937]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":964}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.513786,40.127934]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":965}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.514175,40.127934]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":966}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.514549,40.127937]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":967}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.514931,40.127949]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":968}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.515312,40.127964]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":969}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.515694,40.127983]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":970}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.516075,40.128014]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":971}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.516449,40.128048]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":972}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.516823,40.128082]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":973}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.517204,40.128124]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":974}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.517586,40.128178]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":975}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.517960,40.128231]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.518333,40.128288]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.518707,40.128353]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.519081,40.128422]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":979}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.519447,40.128494]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":980}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.519814,40.128567]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":981}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.520180,40.128639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":982}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.520554,40.128716]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":983}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.520920,40.128784]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":984}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.521294,40.128860]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":985}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.521660,40.128937]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":986}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.522034,40.129009]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":987}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.522400,40.129082]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":988}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.522766,40.129158]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":989}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.523132,40.129234]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":990}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.523506,40.129311]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":991}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.523880,40.129383]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":992}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.524246,40.129456]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.524612,40.129528]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":994}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.524979,40.129601]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":995}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.525352,40.129673]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":996}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.525719,40.129749]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":997}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.526093,40.129826]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":998}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.526459,40.129902]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":999}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.526833,40.129974]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.527206,40.130054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1001}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.527573,40.130131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1002}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.527939,40.130215]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.528305,40.130295]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1004}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.528679,40.130371]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1005}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.529045,40.130447]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1006}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.529411,40.130524]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1007}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.529785,40.130596]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1008}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.530151,40.130672]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1009}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.530525,40.130749]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1010}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.530891,40.130821]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1011}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.531265,40.130894]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1012}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.531631,40.130970]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1013}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.532005,40.131046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1014}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.532372,40.131123]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1015}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.532738,40.131191]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1016}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.533112,40.131268]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1017}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.533478,40.131344]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1018}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.533852,40.131416]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1019}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.534218,40.131489]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1020}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.534592,40.131561]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1021}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.534958,40.131641]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.535332,40.131714]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.535698,40.131786]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1024}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.536072,40.131863]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1025}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.536438,40.131943]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1026}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.536804,40.132027]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1027}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.537170,40.132111]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1028}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.537537,40.132202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1029}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.537895,40.132301]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1030}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.538254,40.132401]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1031}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.538612,40.132511]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.538963,40.132629]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1033}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.539307,40.132748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1034}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.539658,40.132870]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1035}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.540009,40.133003]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1036}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.540352,40.133137]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1037}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.540688,40.133274]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1038}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.541023,40.133419]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1039}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.541351,40.133568]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1040}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.541679,40.133717]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1041}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.542000,40.133869]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1042}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.542313,40.134026]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1043}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.542618,40.134186]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1044}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.542915,40.134346]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1045}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.543205,40.134518]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1046}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.543495,40.134686]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1047}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.543785,40.134857]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1048}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.544083,40.135033]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1049}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.544365,40.135212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1050}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.544647,40.135395]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1051}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.544914,40.135586]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1052}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.545189,40.135780]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1053}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.545448,40.135979]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1054}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.545715,40.136181]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1055}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.545982,40.136379]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1056}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.546249,40.136578]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1057}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.546516,40.136776]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1058}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.546768,40.136971]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1059}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.547028,40.137165]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1060}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.547287,40.137356]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.547539,40.137543]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.547791,40.137730]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1063}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.548042,40.137917]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.548294,40.138103]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.548546,40.138294]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1066}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.548798,40.138481]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.549057,40.138676]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1068}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.549309,40.138863]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1069}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.549561,40.139053]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1070}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.549820,40.139244]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1071}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.550079,40.139435]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1072}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.550339,40.139626]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1073}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.550598,40.139820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1074}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.550865,40.140011]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1075}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.551125,40.140209]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1076}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.551384,40.140411]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1077}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.551651,40.140610]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.551918,40.140812]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1079}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.552193,40.141018]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1080}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.552460,40.141220]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1081}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.552727,40.141426]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1082}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.552994,40.141632]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1083}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.553261,40.141834]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1084}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.553535,40.142036]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1085}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.553802,40.142239]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1086}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.554077,40.142445]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1087}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.554344,40.142647]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1088}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.554619,40.142853]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1089}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.554893,40.143059]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1090}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.555161,40.143261]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1091}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.555428,40.143459]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1092}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.555695,40.143658]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1093}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.555962,40.143848]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1094}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.556236,40.144043]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1095}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.556503,40.144241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1096}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.556770,40.144440]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1097}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.557037,40.144638]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1098}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.557304,40.144833]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1099}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.557571,40.145035]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1100}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.557838,40.145237]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1101}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.558113,40.145439]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.558388,40.145641]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1103}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.558655,40.145844]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1104}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.558929,40.146042]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1105}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.559196,40.146240]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1106}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.559456,40.146435]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.559715,40.146626]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.559975,40.146820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.560234,40.147011]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1110}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.560486,40.147198]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1111}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.560745,40.147381]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1112}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.561005,40.147572]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.561256,40.147762]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1114}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.561516,40.147953]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1115}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.561775,40.148144]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1116}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.562035,40.148331]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.562294,40.148521]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1118}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.562553,40.148712]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1119}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.562813,40.148907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1120}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.563065,40.149101]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1121}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.563324,40.149292]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.563583,40.149490]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.563835,40.149681]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.564087,40.149872]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.564346,40.150063]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1126}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.564590,40.150249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1127}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.564835,40.150444]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.565086,40.150639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.565331,40.150833]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1130}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.565582,40.151028]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1131}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.565826,40.151222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1132}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566078,40.151417]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1133}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566330,40.151611]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1134}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566582,40.151806]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1135}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566833,40.152000]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1136}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567093,40.152195]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1137}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567345,40.152390]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1138}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567604,40.152584]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1139}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567863,40.152779]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1140}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568115,40.152966]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1141}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568367,40.153164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1142}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568626,40.153362]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1143}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568878,40.153557]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1144}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569130,40.153751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1145}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569389,40.153950]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1146}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569641,40.154148]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1147}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569901,40.154346]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1148}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570152,40.154545]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1149}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570412,40.154743]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1150}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570671,40.154942]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1151}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570923,40.155136]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1152}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571182,40.155334]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1153}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571434,40.155537]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1154}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571701,40.155739]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1155}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571960,40.155941]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1156}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572220,40.156147]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1157}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572495,40.156353]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1158}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572762,40.156551]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1159}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573021,40.156754]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1160}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573288,40.156956]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1161}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573547,40.157158]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1162}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573807,40.157356]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1163}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574074,40.157558]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1164}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574341,40.157761]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1165}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574600,40.157963]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1166}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574867,40.158161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1167}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575119,40.158363]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1168}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575386,40.158562]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1169}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575638,40.158760]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1170}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575897,40.158958]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1171}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576149,40.159157]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1172}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576393,40.159351]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1173}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576645,40.159546]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1174}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576897,40.159737]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1175}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577141,40.159927]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1176}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577393,40.160122]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1177}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577644,40.160316]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1178}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577888,40.160511]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1179}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578140,40.160702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1180}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578384,40.160892]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1181}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578636,40.161079]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1182}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578888,40.161274]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579140,40.161469]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1184}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579391,40.161663]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1185}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579643,40.161858]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579887,40.162052]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580132,40.162243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1188}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580383,40.162434]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1189}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580627,40.162624]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1190}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580872,40.162815]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1191}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581108,40.163010]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1192}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581360,40.163197]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581596,40.163387]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581841,40.163574]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1195}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582077,40.163757]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1196}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582329,40.163940]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1197}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582565,40.164131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1198}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582817,40.164322]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583069,40.164513]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1200}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583313,40.164703]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1201}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583557,40.164890]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1202}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583794,40.165070]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1203}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584030,40.165249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1204}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584267,40.165432]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584496,40.165611]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584732,40.165794]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584961,40.165977]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585197,40.166164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585434,40.166348]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1210}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585670,40.166527]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1211}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585907,40.166706]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1212}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586143,40.166893]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1213}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586388,40.167080]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586624,40.167267]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1215}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586868,40.167450]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1216}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587105,40.167633]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1217}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587334,40.167809]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587563,40.167988]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587799,40.168167]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1220}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588028,40.168346]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588257,40.168526]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588493,40.168701]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588730,40.168877]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1224}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588959,40.169052]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589188,40.169228]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1226}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589417,40.169407]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1227}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589645,40.169579]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1228}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589874,40.169758]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1229}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590096,40.169930]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1230}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590317,40.170101]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1231}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590546,40.170273]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1232}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590767,40.170441]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1233}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590981,40.170612]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1234}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591202,40.170784]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591423,40.170952]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1236}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591637,40.171116]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1237}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591858,40.171288]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1238}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592072,40.171452]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1239}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592285,40.171616]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1240}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592506,40.171780]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1241}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592720,40.171944]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1242}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592934,40.172112]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1243}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593147,40.172276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593361,40.172440]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593575,40.172607]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1246}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593796,40.172775]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1247}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594017,40.172947]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1248}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594238,40.173119]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1249}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594460,40.173286]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1250}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594681,40.173466]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1251}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594902,40.173641]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1252}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595123,40.173820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1253}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595345,40.174000]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1254}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595573,40.174179]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1255}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595795,40.174355]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1256}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596031,40.174538]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1257}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596252,40.174709]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1258}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596474,40.174877]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596687,40.175049]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1260}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596909,40.175213]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597115,40.175385]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1262}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597321,40.175545]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1263}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597527,40.175713]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1264}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597717,40.175877]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597908,40.176044]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1266}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598099,40.176212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598274,40.176380]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1268}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598457,40.176544]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1269}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598633,40.176716]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1270}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598801,40.176876]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598961,40.177032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599113,40.177200]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1273}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599258,40.177364]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1274}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599396,40.177528]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1275}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599518,40.177685]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1276}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599632,40.177834]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1277}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599739,40.177963]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1278}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599823,40.178085]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1279}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599907,40.178196]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1280}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599976,40.178295]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1281}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600052,40.178391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1282}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600113,40.178478]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1283}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600182,40.178562]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1284}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600243,40.178646]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1285}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600304,40.178730]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1286}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600357,40.178810]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1287}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600441,40.178944]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600479,40.179054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600479,40.179153]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1290}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600464,40.179287]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1291}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600449,40.179371]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600426,40.179462]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1293}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600403,40.179562]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1294}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600380,40.179676]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1295}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600349,40.179806]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1296}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600311,40.179935]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1297}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600281,40.180088]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600250,40.180252]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600220,40.180420]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1300}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600189,40.180607]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1301}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600174,40.180794]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1302}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600166,40.180973]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1303}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600159,40.181160]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1304}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600166,40.181339]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1305}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600166,40.181526]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1306}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600166,40.181709]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1307}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600174,40.181889]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600189,40.182068]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1309}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600212,40.182243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600235,40.182426]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1311}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600266,40.182602]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1312}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600304,40.182781]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1313}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600365,40.182953]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1314}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600410,40.183128]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1315}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600464,40.183304]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1316}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600517,40.183483]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1317}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600578,40.183662]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1318}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600655,40.183849]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1319}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600723,40.184032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1320}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600815,40.184196]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1321}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600899,40.184372]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1322}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600990,40.184544]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1323}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601082,40.184715]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1324}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601181,40.184879]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1325}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601288,40.185040]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1326}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601387,40.185207]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1327}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601479,40.185371]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1328}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601578,40.185532]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1329}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601677,40.185696]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1330}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601776,40.185856]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1331}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601868,40.186012]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1332}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601967,40.186169]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1333}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602058,40.186325]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1334}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602150,40.186478]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1335}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602242,40.186630]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1336}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602333,40.186783]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1337}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602425,40.186935]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1338}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602524,40.187092]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1339}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602615,40.187244]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1340}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602715,40.187405]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1341}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602814,40.187565]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1342}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602913,40.187733]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1343}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603012,40.187901]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1344}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603119,40.188072]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1345}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603226,40.188240]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1346}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603333,40.188404]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1347}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603432,40.188568]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1348}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603531,40.188732]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1349}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603622,40.188896]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1350}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603714,40.189053]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1351}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603806,40.189205]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1352}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603897,40.189350]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1353}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603989,40.189499]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604073,40.189640]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604164,40.189785]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604256,40.189930]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1357}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604340,40.190075]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1358}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604424,40.190224]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604523,40.190372]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604614,40.190529]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1361}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604706,40.190689]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604805,40.190853]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1363}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604904,40.191017]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1364}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605003,40.191189]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1365}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605110,40.191360]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605217,40.191532]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1367}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605324,40.191704]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1368}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605431,40.191879]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1369}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605537,40.192059]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1370}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605644,40.192245]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1371}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605759,40.192429]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1372}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605881,40.192623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1373}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605995,40.192818]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606117,40.193016]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1375}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606239,40.193214]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1376}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606354,40.193413]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1377}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606468,40.193607]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1378}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606590,40.193802]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1379}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606705,40.194000]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1380}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606827,40.194195]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1381}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606934,40.194382]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1382}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607048,40.194569]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1383}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607147,40.194756]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1384}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607254,40.194935]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1385}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607361,40.195110]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1386}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607468,40.195286]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1387}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607574,40.195461]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607681,40.195637]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1389}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607788,40.195812]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1390}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607903,40.195995]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1391}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608009,40.196171]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1392}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608116,40.196346]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1393}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608223,40.196522]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1394}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608330,40.196705]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1395}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608437,40.196880]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1396}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608543,40.197063]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1397}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608650,40.197250]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1398}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608765,40.197433]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1399}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608879,40.197617]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1400}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608986,40.197792]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1401}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609100,40.197968]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1402}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609207,40.198147]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1403}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609322,40.198326]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1404}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609421,40.198509]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1405}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609528,40.198696]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1406}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609627,40.198887]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1407}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609741,40.199066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1408}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609856,40.199253]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1409}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609970,40.199436]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1410}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610085,40.199623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1411}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610191,40.199814]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1412}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610298,40.200005]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1413}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610405,40.200195]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1414}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610504,40.200386]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1415}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610603,40.200581]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1416}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610695,40.200775]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1417}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610771,40.200970]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1418}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610855,40.201164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1419}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610939,40.201366]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611015,40.201572]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611092,40.201782]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611168,40.201996]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1423}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611244,40.202202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1424}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611313,40.202412]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1425}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611389,40.202618]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1426}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611458,40.202816]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1427}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611519,40.203011]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1428}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611588,40.203197]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1429}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611656,40.203381]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1430}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611710,40.203560]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1431}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611771,40.203735]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1432}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611832,40.203903]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1433}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611893,40.204075]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611954,40.204243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612000,40.204403]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612061,40.204563]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1437}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612114,40.204716]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1438}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612167,40.204876]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1439}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612228,40.205036]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1440}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612289,40.205208]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612343,40.205383]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1442}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612404,40.205555]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1443}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612457,40.205727]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1444}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612526,40.205906]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1445}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612587,40.206085]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1446}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612640,40.206268]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1447}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612701,40.206448]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1448}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612770,40.206623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1449}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612831,40.206802]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1450}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612900,40.206982]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612961,40.207161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1452}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613022,40.207344]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1453}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613091,40.207523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1454}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613152,40.207706]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1455}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613220,40.207890]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1456}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613281,40.208073]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1457}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613350,40.208252]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1458}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613411,40.208435]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1459}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613480,40.208618]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1460}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613548,40.208801]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613617,40.208988]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1462}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613670,40.209183]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1463}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613724,40.209370]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1464}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613785,40.209564]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1465}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613846,40.209751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1466}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613914,40.209938]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1467}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613976,40.210117]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1468}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614044,40.210300]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1469}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614105,40.210484]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1470}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614166,40.210663]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614235,40.210838]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614304,40.211010]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614380,40.211185]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1474}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614449,40.211361]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1475}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614525,40.211536]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614609,40.211704]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614693,40.211864]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1478}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614777,40.212029]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1479}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614876,40.212185]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1480}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614967,40.212345]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1481}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615059,40.212505]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1482}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615166,40.212669]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615273,40.212833]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615379,40.212997]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615494,40.213165]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1486}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615623,40.213329]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1487}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615746,40.213490]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1488}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615883,40.213642]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1489}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616013,40.213795]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1490}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616142,40.213943]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1491}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616264,40.214088]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1492}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616394,40.214241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1493}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616531,40.214386]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1494}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616661,40.214539]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1495}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616798,40.214691]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1496}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616936,40.214851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617073,40.215008]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1498}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617210,40.215164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1499}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617348,40.215328]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1500}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617485,40.215488]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1501}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617615,40.215656]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1502}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617744,40.215824]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1503}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617859,40.215996]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1504}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617973,40.216164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1505}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618073,40.216335]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1506}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618179,40.216515]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1507}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618271,40.216690]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1508}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618355,40.216873]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1509}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618439,40.217056]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1510}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618507,40.217243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1511}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618576,40.217430]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1512}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618637,40.217628]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1513}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618698,40.217819]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1514}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618752,40.218014]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1515}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618790,40.218216]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1516}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618835,40.218410]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1517}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618874,40.218609]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1518}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618896,40.218807]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1519}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618904,40.219013]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1520}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618912,40.219219]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618904,40.219437]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1522}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618889,40.219643]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1523}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618866,40.219864]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1524}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618835,40.220066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1525}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618797,40.220276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1526}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618752,40.220482]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1527}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618713,40.220695]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618683,40.220909]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1529}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618652,40.221115]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618622,40.221321]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618607,40.221523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1532}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618591,40.221722]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1533}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618591,40.221912]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1534}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618599,40.222099]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1535}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618614,40.222282]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1536}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618637,40.222458]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1537}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618668,40.222633]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1538}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618706,40.222797]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618752,40.222950]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618790,40.223083]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618828,40.223209]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1542}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618866,40.223335]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618904,40.223446]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1544}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618950,40.223549]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1545}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618988,40.223644]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1546}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619019,40.223728]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1547}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619064,40.223866]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1548}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619102,40.223949]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619141,40.224041]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1550}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619179,40.224140]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1551}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619232,40.224281]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1552}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619263,40.224365]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1553}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619293,40.224449]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1554}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619331,40.224541]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619377,40.224636]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1556}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619415,40.224739]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1557}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619461,40.224850]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1558}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619507,40.224964]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1559}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619553,40.225082]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1560}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619598,40.225204]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1561}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619652,40.225330]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619713,40.225460]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1563}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619766,40.225597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619820,40.225735]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1565}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619873,40.225872]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1566}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619926,40.226009]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619987,40.226151]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1568}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620041,40.226303]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1569}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620102,40.226460]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1570}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620171,40.226620]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1571}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620232,40.226784]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1572}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620300,40.226952]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1573}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620369,40.227123]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1574}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620438,40.227299]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1575}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620506,40.227478]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1576}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620575,40.227657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620644,40.227840]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1578}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620712,40.228024]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1579}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620781,40.228203]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1580}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620842,40.228378]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1581}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620911,40.228558]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1582}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620979,40.228748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1583}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621048,40.228935]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1584}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621117,40.229122]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1585}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621193,40.229313]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1586}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621269,40.229511]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1587}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621346,40.229713]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1588}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621422,40.229912]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1589}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621498,40.230110]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1590}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621574,40.230309]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1591}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621651,40.230503]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1592}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621719,40.230690]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1593}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621788,40.230869]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1594}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621857,40.231045]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621925,40.231216]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621986,40.231380]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1597}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622040,40.231529]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1598}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622101,40.231667]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1599}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622147,40.231789]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1600}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622185,40.231907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1601}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622231,40.232018]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1602}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622276,40.232128]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1603}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622314,40.232246]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1604}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622368,40.232372]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1605}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622414,40.232510]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1606}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622467,40.232651]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1607}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622528,40.232800]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1608}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622581,40.232956]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1609}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622643,40.233116]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622711,40.233276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622772,40.233444]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622841,40.233616]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622910,40.233784]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622971,40.233955]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623039,40.234119]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623100,40.234295]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1617}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623161,40.234467]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1618}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623230,40.234638]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1619}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623299,40.234814]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1620}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623367,40.234989]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1621}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623428,40.235168]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1622}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623505,40.235352]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1623}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623573,40.235535]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1624}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623642,40.235718]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1625}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623718,40.235905]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1626}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623795,40.236092]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1627}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623856,40.236282]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1628}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623932,40.236473]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624008,40.236668]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624077,40.236858]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1631}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624153,40.237053]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1632}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624222,40.237247]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1633}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624290,40.237438]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1634}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624359,40.237637]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624428,40.237831]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1636}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624496,40.238029]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1637}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624573,40.238232]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1638}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624641,40.238430]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1639}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624718,40.238632]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624794,40.238838]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1641}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624870,40.239040]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1642}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624947,40.239243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625023,40.239449]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625107,40.239658]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625183,40.239861]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625259,40.240070]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1647}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625320,40.240292]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1648}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625381,40.240517]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625443,40.240746]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1650}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625481,40.240971]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1651}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625511,40.241192]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1652}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625534,40.241409]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1653}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625542,40.241631]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1654}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625549,40.241856]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1655}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625557,40.242085]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1656}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625557,40.242313]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1657}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625565,40.242550]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1658}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625572,40.242783]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1659}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625572,40.243015]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625580,40.243244]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1661}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625595,40.243477]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1662}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625595,40.243706]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1663}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625603,40.243931]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1664}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625610,40.244148]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1665}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625626,40.244377]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1666}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625633,40.244602]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1667}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625641,40.244831]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1668}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625641,40.245064]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1669}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625648,40.245293]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1670}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625656,40.245522]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1671}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625664,40.245743]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1672}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625671,40.245964]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1673}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625679,40.246185]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1674}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625687,40.246403]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1675}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625687,40.246616]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1676}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625694,40.246826]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1677}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625702,40.247032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1678}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625710,40.247242]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1679}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625717,40.247448]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1680}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625725,40.247654]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1681}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625740,40.247868]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1682}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625748,40.248070]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1683}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625763,40.248280]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625786,40.248493]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1685}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625832,40.248707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1686}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625893,40.248924]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625961,40.249142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1688}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626038,40.249352]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1689}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626122,40.249561]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1690}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626228,40.249771]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1691}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626343,40.249977]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1692}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626465,40.250187]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1693}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626587,40.250389]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1694}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626701,40.250591]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1695}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626823,40.250797]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1696}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626945,40.250999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1697}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627060,40.251202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1698}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627174,40.251396]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1699}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627296,40.251595]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627411,40.251797]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1701}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627525,40.251995]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1702}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627640,40.252190]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1703}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627762,40.252380]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1704}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627876,40.252567]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1705}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627991,40.252754]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1706}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628105,40.252937]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1707}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628227,40.253113]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1708}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628342,40.253288]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1709}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628471,40.253456]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628616,40.253628]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1711}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628769,40.253792]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1712}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628929,40.253948]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1713}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629097,40.254105]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1714}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629272,40.254257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1715}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629456,40.254406]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1716}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629654,40.254543]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1717}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629852,40.254677]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1718}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630066,40.254807]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1719}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630280,40.254929]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1720}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630493,40.255047]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1721}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630707,40.255154]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1722}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630928,40.255253]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1723}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631134,40.255348]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1724}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631340,40.255440]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1725}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631538,40.255527]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1726}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631729,40.255615]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1727}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631912,40.255695]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1728}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632095,40.255772]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632271,40.255852]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1730}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632446,40.255936]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632614,40.256020]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1732}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632782,40.256092]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1733}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632942,40.256165]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1734}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633110,40.256229]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1735}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633286,40.256298]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1736}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633469,40.256367]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1737}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633636,40.256443]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1738}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633804,40.256523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1739}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633965,40.256615]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1740}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634117,40.256718]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1741}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634270,40.256828]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1742}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634407,40.256950]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634544,40.257080]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1744}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634666,40.257214]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1745}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634796,40.257351]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1746}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634918,40.257481]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1747}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635033,40.257599]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1748}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635162,40.257710]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1749}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635300,40.257809]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1750}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635445,40.257912]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1751}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635574,40.258007]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1752}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635712,40.258106]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1753}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635849,40.258205]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1754}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635986,40.258308]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1755}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636131,40.258411]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1756}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636276,40.258518]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1757}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636429,40.258625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1758}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636574,40.258732]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1759}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636726,40.258835]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1760}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636871,40.258942]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1761}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637009,40.259041]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1762}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637154,40.259136]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1763}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637291,40.259232]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1764}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637428,40.259331]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1765}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637566,40.259422]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1766}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637688,40.259510]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1767}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637802,40.259598]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1768}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637917,40.259682]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638031,40.259762]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638145,40.259834]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638260,40.259922]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1772}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638390,40.260014]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1773}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638527,40.260109]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638672,40.260216]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1775}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638832,40.260326]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1776}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638985,40.260441]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1777}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639137,40.260551]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1778}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639297,40.260666]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639450,40.260784]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1780}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639595,40.260902]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1781}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639732,40.261024]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1782}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639862,40.261150]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1783}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639992,40.261284]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1784}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640121,40.261421]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1785}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640259,40.261562]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1786}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640396,40.261707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1787}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640533,40.261852]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1788}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640678,40.261997]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1789}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640816,40.262135]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1790}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640945,40.262280]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1791}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641083,40.262421]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1792}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641220,40.262566]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1793}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641357,40.262714]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1794}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641495,40.262859]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1795}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641624,40.263004]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641762,40.263145]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1797}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641891,40.263287]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1798}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642021,40.263428]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1799}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642151,40.263565]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1800}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642281,40.263702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1801}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642410,40.263836]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1802}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642540,40.263973]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1803}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642670,40.264103]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1804}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642799,40.264236]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642921,40.264366]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643044,40.264496]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643166,40.264614]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643280,40.264736]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1809}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643394,40.264854]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1810}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643509,40.264965]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1811}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643639,40.265076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643776,40.265190]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643929,40.265289]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1814}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644089,40.265392]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1815}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644249,40.265491]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1816}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644424,40.265587]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1817}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644585,40.265678]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1818}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644745,40.265770]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1819}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644905,40.265862]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1820}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645073,40.265953]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1821}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645241,40.266045]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1822}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645416,40.266140]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1823}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645592,40.266243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1824}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645767,40.266342]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1825}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645943,40.266441]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1826}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646126,40.266544]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1827}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646286,40.266636]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1828}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646446,40.266724]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1829}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646584,40.266796]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1830}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646721,40.266865]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646851,40.266933]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1832}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646965,40.267006]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1833}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647095,40.267078]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1834}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647224,40.267155]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1835}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647377,40.267242]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1836}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647522,40.267330]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1837}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647682,40.267426]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1838}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647850,40.267525]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648018,40.267620]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1840}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648170,40.267715]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648331,40.267803]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1842}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648499,40.267891]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1843}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648651,40.267979]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648811,40.268070]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1845}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648964,40.268150]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1846}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649109,40.268227]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1847}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649246,40.268299]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1848}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649384,40.268372]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1849}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649513,40.268444]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649651,40.268517]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649803,40.268593]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1852}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649933,40.268669]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1853}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650063,40.268749]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1854}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650200,40.268826]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1855}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650337,40.268902]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650467,40.268978]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1857}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650604,40.269058]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1858}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650742,40.269138]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650887,40.269215]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651024,40.269295]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1861}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651169,40.269375]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651314,40.269455]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651459,40.269539]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1864}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651604,40.269623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1865}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651756,40.269711]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1866}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651901,40.269787]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1867}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652061,40.269875]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1868}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652214,40.269962]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652367,40.270046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1870}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652519,40.270130]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652664,40.270218]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1872}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652817,40.270302]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1873}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652954,40.270378]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1874}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653099,40.270458]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1875}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653244,40.270542]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1876}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653397,40.270634]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1877}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653542,40.270725]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1878}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653694,40.270802]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1879}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653847,40.270882]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1880}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653992,40.270966]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1881}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654144,40.271046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1882}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654289,40.271122]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1883}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654427,40.271202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1884}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654556,40.271278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1885}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654694,40.271355]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1886}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654823,40.271431]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654930,40.271515]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655037,40.271599]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655121,40.271675]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655228,40.271786]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1891}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655296,40.271858]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1892}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655327,40.271942]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1893}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655365,40.272030]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1894}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655418,40.272148]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655457,40.272228]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1896}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655495,40.272308]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1897}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655533,40.272392]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1898}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655563,40.272472]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1899}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655640,40.272614]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1900}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655693,40.272720]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1901}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655739,40.272800]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1902}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655777,40.272877]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1903}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655830,40.272964]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1904}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655907,40.273067]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1905}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656059,40.273167]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1906}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656143,40.273220]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1907}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656242,40.273273]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1908}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656349,40.273342]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1909}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656464,40.273415]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1910}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656586,40.273491]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1911}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656723,40.273571]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1912}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656860,40.273659]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1913}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657005,40.273743]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1914}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657150,40.273827]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657295,40.273914]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657433,40.274010]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657562,40.274094]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1918}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657692,40.274181]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1919}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657829,40.274273]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1920}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657967,40.274361]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1921}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658112,40.274448]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1922}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658249,40.274532]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1923}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658379,40.274616]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1924}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658516,40.274708]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1925}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658653,40.274799]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1926}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658798,40.274891]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1927}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658951,40.274979]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1928}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659088,40.275066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1929}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659225,40.275146]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1930}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659370,40.275246]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659523,40.275341]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1932}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659683,40.275436]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659843,40.275536]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1934}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660004,40.275642]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1935}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660156,40.275745]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1936}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660309,40.275841]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1937}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660469,40.275951]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1938}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660614,40.276039]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1939}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660759,40.276131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660904,40.276222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1941}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661049,40.276295]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1942}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661186,40.276382]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1943}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661324,40.276463]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1944}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661461,40.276546]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1945}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661606,40.276634]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661743,40.276733]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661888,40.276836]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1948}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662033,40.276924]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1949}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662193,40.277027]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1950}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662346,40.277119]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1951}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662498,40.277214]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662643,40.277302]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1953}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662781,40.277390]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1954}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662918,40.277470]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1955}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663055,40.277523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1956}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663193,40.277592]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1957}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663322,40.277676]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1958}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663445,40.277752]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1959}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663567,40.277821]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1960}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663673,40.277885]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1961}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663765,40.277939]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1962}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663834,40.278008]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1963}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663910,40.278065]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1964}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663986,40.278122]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1965}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664139,40.278214]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1966}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664230,40.278305]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1967}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664185,40.278431]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1968}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664131,40.278503]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1969}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664078,40.278587]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1970}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664009,40.278683]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1971}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663933,40.278778]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1972}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663841,40.278889]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1973}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663750,40.278999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1974}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663651,40.279106]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1975}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663559,40.279217]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663467,40.279327]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663368,40.279446]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663261,40.279560]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1979}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663162,40.279682]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1980}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663055,40.279804]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1981}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662949,40.279938]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1982}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662834,40.280064]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1983}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662720,40.280186]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1984}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662605,40.280312]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1985}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662491,40.280441]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1986}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662376,40.280563]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1987}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662262,40.280689]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1988}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662148,40.280807]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1989}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662018,40.280918]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1990}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661880,40.281025]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1991}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661743,40.281124]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1992}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661598,40.281219]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661453,40.281303]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1994}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661301,40.281384]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1995}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661148,40.281456]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1996}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660980,40.281528]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1997}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660812,40.281593]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1998}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660645,40.281662]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1999}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660469,40.281731]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660286,40.281796]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2001}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660118,40.281860]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2002}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659927,40.281940]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659760,40.282001]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2004}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659607,40.282070]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2005}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659439,40.282127]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2006}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659271,40.282188]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2007}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659096,40.282249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2008}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658920,40.282314]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2009}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658745,40.282379]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2010}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658562,40.282440]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2011}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658386,40.282513]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2012}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658203,40.282574]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2013}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658028,40.282639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2014}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657852,40.282707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2015}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657661,40.282772]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2016}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657486,40.282837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2017}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657310,40.282909]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2018}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657143,40.282974]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2019}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656975,40.283028]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2020}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656822,40.283089]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2021}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656662,40.283142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656517,40.283203]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656380,40.283264]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2024}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656235,40.283314]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2025}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656090,40.283371]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2026}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655945,40.283424]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2027}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655800,40.283474]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2028}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655663,40.283535]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2029}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655525,40.283585]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2030}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655388,40.283630]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2031}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655251,40.283676]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655128,40.283718]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2033}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655014,40.283768]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2034}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654846,40.283829]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2035}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654724,40.283878]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2036}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654625,40.283924]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2037}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654625,40.283924]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2038}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299171,40.042519]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":0,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.726,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299049,40.042530]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.830,"distance_from_trace_point":7.199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298889,40.042545]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.967,"distance_from_trace_point":7.199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298683,40.042568]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":3,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.160,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298576,40.042580]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":4,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.262,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298470,40.042595]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":5,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.365,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298355,40.042606]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":6,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.474,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298248,40.042622]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":7,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.577,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298080,40.042641]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":8,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.738,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297958,40.042652]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":9,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.854,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297836,40.042667]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":10,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.971,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297798,40.042622]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":11,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.075,"distance_from_trace_point":0.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297775,40.042503]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":12,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.256,"distance_from_trace_point":0.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297752,40.042370]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":13,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.459,"distance_from_trace_point":0.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297737,40.042263]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":14,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.622,"distance_from_trace_point":0.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297722,40.042187]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":15,"matched_point_type":"interpolated","edge_index":3,"distance_along_edge":0.738,"distance_from_trace_point":2.617}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297707,40.042095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":16,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.878,"distance_from_trace_point":0.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297577,40.042027]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":17,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.134,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297401,40.042042]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":18,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.339,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297165,40.042068]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":19,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.616,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297012,40.042084]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":20,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.795,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296852,40.042099]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":21,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.982,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296700,40.042114]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":22,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.115,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296539,40.042130]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":23,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.250,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296371,40.042149]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":24,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.391,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296211,40.042164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":25,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.525,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296028,40.042183]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":26,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.679,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295853,40.042202]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":27,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.827,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295677,40.042221]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":28,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.974,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295494,40.042244]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":29,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.082,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295319,40.042263]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":30,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.176,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295135,40.042282]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":31,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.274,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294975,40.042301]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":32,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.361,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294800,40.042324]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":33,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.455,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294624,40.042343]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":34,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.549,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294456,40.042362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":35,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.639,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294289,40.042381]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":36,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.730,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294128,40.042400]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":37,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.816,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293953,40.042419]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":38,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.910,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293777,40.042439]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":39,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.004,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293610,40.042454]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":40,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.090,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293442,40.042469]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":41,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.177,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293266,40.042488]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":42,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.268,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293106,40.042503]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":43,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.350,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292946,40.042519]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":44,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.433,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292793,40.042534]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":45,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.512,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292648,40.042545]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":46,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.587,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292534,40.042557]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":47,"matched_point_type":"interpolated","edge_index":7,"distance_along_edge":0.646,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292419,40.042568]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":48,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.705,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292236,40.042583]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":49,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.799,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292091,40.042599]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":50,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.874,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291985,40.042610]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":51,"matched_point_type":"interpolated","edge_index":7,"distance_along_edge":0.929,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291847,40.042622]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":52,"matched_point_type":"matched","edge_index":7,"distance_along_edge":1.000,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291679,40.042641]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":53,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.087,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291565,40.042652]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":54,"matched_point_type":"interpolated","edge_index":8,"distance_along_edge":0.146,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291443,40.042664]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":55,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.209,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291321,40.042675]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":56,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.272,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291191,40.042690]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":57,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.339,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291054,40.042706]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":58,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.410,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290909,40.042721]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":59,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.484,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290749,40.042736]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":60,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.567,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290581,40.042755]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":61,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.654,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290413,40.042770]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":62,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.740,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290237,40.042789]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":63,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.831,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290047,40.042809]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":64,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.929,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289871,40.042828]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":65,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.019,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289696,40.042847]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":66,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.105,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289505,40.042866]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":67,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.198,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289322,40.042885]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":68,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.288,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289162,40.042904]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":69,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.367,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288986,40.042923]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":70,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.453,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288811,40.042938]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":71,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.539,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288643,40.042957]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":72,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.622,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288498,40.042973]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":73,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.693,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288368,40.042980]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":74,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.756,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288246,40.042995]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":75,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.816,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288094,40.043011]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":76,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.891,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287971,40.043026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":77,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.951,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287804,40.043045]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":78,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.061,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287689,40.043056]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":79,"matched_point_type":"interpolated","edge_index":10,"distance_along_edge":0.162,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287582,40.043068]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":80,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.257,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287437,40.043079]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":81,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.385,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287292,40.043091]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":82,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.514,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287117,40.043110]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":83,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.670,"distance_from_trace_point":4.010}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286964,40.043137]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":84,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.807,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286842,40.043167]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":85,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.920,"distance_from_trace_point":3.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286697,40.043224]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":86,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.066,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286583,40.043293]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":87,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.200,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286484,40.043377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":88,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.335,"distance_from_trace_point":1.543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286385,40.043468]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":89,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.477,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286301,40.043552]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":90,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.602,"distance_from_trace_point":4.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286186,40.043659]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":91,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.767,"distance_from_trace_point":1.543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286072,40.043766]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":92,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.931,"distance_from_trace_point":2.569}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285957,40.043877]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":93,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.282,"distance_from_trace_point":1.543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285843,40.043999]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":94,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.037,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285736,40.044117]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":95,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.105,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285622,40.044243]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":96,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.178,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285507,40.044353]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":97,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.245,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285385,40.044472]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":98,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.317,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285240,40.044582]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":99,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.391,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285088,40.044674]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":100,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.462,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284904,40.044754]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":101,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.539,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284737,40.044819]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":102,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.608,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284348,40.044899]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":103,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.755,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284149,40.044922]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":104,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.828,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283936,40.044949]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":105,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.907,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283714,40.044979]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":106,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.989,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283501,40.044998]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":107,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.017,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283272,40.045025]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":108,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.038,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283035,40.045063]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":109,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.060,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282829,40.045109]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":110,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.079,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282608,40.045166]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":111,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.101,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282379,40.045242]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":112,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.123,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282158,40.045326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":113,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.146,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281937,40.045418]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":114,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.169,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281723,40.045502]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":115,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.191,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281525,40.045624]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":116,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.214,"distance_from_trace_point":3.187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281326,40.045750]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":117,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.237,"distance_from_trace_point":3.923}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281128,40.045887]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":118,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.262,"distance_from_trace_point":3.616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280937,40.046028]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":119,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.286,"distance_from_trace_point":3.616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280754,40.046185]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":120,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.311,"distance_from_trace_point":2.569}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280571,40.046329]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":121,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.335,"distance_from_trace_point":4.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280418,40.046497]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":122,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.359,"distance_from_trace_point":2.314}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280281,40.046665]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":123,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.383,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280128,40.046833]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":124,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.407,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279976,40.047009]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":125,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.432,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279823,40.047180]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":126,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.457,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279663,40.047367]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":127,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.483,"distance_from_trace_point":2.314}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279526,40.047554]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":128,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.509,"distance_from_trace_point":2.113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279411,40.047756]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":129,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.535,"distance_from_trace_point":3.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279305,40.047955]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":130,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.561,"distance_from_trace_point":5.580}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279198,40.048168]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":131,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.588,"distance_from_trace_point":5.434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279091,40.048393]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":132,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.616,"distance_from_trace_point":5.434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278999,40.048595]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":133,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.642,"distance_from_trace_point":4.825}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278900,40.048798]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":134,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.668,"distance_from_trace_point":3.468}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278793,40.049015]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":135,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.695,"distance_from_trace_point":4.226}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278694,40.049221]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":136,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.721,"distance_from_trace_point":4.825}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278603,40.049435]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":137,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.748,"distance_from_trace_point":4.076}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278511,40.049637]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":138,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.773,"distance_from_trace_point":4.076}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278412,40.049843]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":139,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.800,"distance_from_trace_point":4.076}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278328,40.050037]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":140,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.824,"distance_from_trace_point":3.468}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278221,40.050240]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":141,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.850,"distance_from_trace_point":2.717}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278130,40.050426]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":142,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.874,"distance_from_trace_point":2.113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278023,40.050617]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":143,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.898,"distance_from_trace_point":2.717}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277931,40.050797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":144,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.921,"distance_from_trace_point":2.113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277847,40.050983]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":145,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.945,"distance_from_trace_point":1.359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277763,40.051163]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":146,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.967,"distance_from_trace_point":1.359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277672,40.051334]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":147,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.989,"distance_from_trace_point":1.543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277557,40.051498]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":148,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.026,"distance_from_trace_point":2.113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277428,40.051659]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":149,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.076,"distance_from_trace_point":3.085}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277275,40.051807]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":150,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.126,"distance_from_trace_point":4.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277115,40.051949]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":151,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.176,"distance_from_trace_point":3.616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276932,40.052074]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":152,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.226,"distance_from_trace_point":3.187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276726,40.052185]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":153,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.277,"distance_from_trace_point":3.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276512,40.052277]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":154,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.327,"distance_from_trace_point":4.641}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276283,40.052357]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":155,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.378,"distance_from_trace_point":4.010}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276039,40.052418]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":156,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.430,"distance_from_trace_point":3.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275803,40.052467]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":157,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.480,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275543,40.052517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":158,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.534,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275284,40.052567]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":159,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.589,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275017,40.052620]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":160,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.645,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274757,40.052681]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":161,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.700,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274490,40.052742]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":162,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.757,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274223,40.052811]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":163,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.814,"distance_from_trace_point":3.223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273956,40.052887]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":164,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.872,"distance_from_trace_point":3.223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273689,40.052975]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":165,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.931,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273430,40.053059]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":166,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.988,"distance_from_trace_point":3.223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273163,40.053150]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":167,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.078,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272896,40.053242]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":168,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.175,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272629,40.053333]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":169,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.273,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272354,40.053429]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":170,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.373,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272087,40.053520]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":171,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.471,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271812,40.053616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":172,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.571,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271545,40.053707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":173,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.669,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271263,40.053802]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":174,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.772,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270988,40.053894]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":175,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.872,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270714,40.053989]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":176,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.972,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270439,40.054081]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":177,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.138,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270164,40.054173]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":178,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.330,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269890,40.054264]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":179,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.522,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269615,40.054352]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":180,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.712,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269348,40.054440]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":181,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.898,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269096,40.054527]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":182,"matched_point_type":"matched","edge_index":19,"distance_along_edge":0.186,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268829,40.054619]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":183,"matched_point_type":"matched","edge_index":19,"distance_along_edge":0.646,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268578,40.054703]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":184,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.035,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268341,40.054783]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":185,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.218,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268112,40.054859]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":186,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.394,"distance_from_trace_point":3.223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267883,40.054939]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":187,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.571,"distance_from_trace_point":3.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267647,40.055016]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":188,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.752,"distance_from_trace_point":4.262}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267426,40.055092]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":189,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.923,"distance_from_trace_point":4.262}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267227,40.055161]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":190,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.090,"distance_from_trace_point":5.028}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267090,40.055210]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":191,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.216,"distance_from_trace_point":9.283}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266914,40.055271]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":192,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.375,"distance_from_trace_point":8.250}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266754,40.055325]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":193,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.520,"distance_from_trace_point":8.901}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266602,40.055382]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":194,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.660,"distance_from_trace_point":8.250}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266472,40.055428]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":195,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.779,"distance_from_trace_point":7.864}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266373,40.055462]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":196,"matched_point_type":"interpolated","edge_index":21,"distance_along_edge":0.869,"distance_from_trace_point":7.481}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266281,40.055492]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":197,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.951,"distance_from_trace_point":10.055}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266296,40.055683]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":198,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.013,"distance_from_trace_point":4.824}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266335,40.055767]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":199,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.030,"distance_from_trace_point":2.876}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266380,40.055851]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":200,"matched_point_type":"interpolated","edge_index":23,"distance_along_edge":0.047,"distance_from_trace_point":3.467}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266434,40.055946]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":201,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.067,"distance_from_trace_point":4.075}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266479,40.056038]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":202,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.085,"distance_from_trace_point":4.824}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266533,40.056149]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":203,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.107,"distance_from_trace_point":5.579}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266602,40.056286]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":204,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.135,"distance_from_trace_point":2.113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266663,40.056393]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":205,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.157,"distance_from_trace_point":2.876}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266754,40.056519]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":206,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.184,"distance_from_trace_point":2.314}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.266876,40.056629]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":207,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.211,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267014,40.056725]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":208,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.238,"distance_from_trace_point":3.187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267181,40.056805]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":209,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.267,"distance_from_trace_point":3.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267357,40.056850]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":210,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.293,"distance_from_trace_point":5.310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267517,40.056873]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":211,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.317,"distance_from_trace_point":8.534}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267723,40.056881]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":212,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.347,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.267914,40.056858]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":213,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.374,"distance_from_trace_point":4.411}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268066,40.056816]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":214,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.398,"distance_from_trace_point":8.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268288,40.056721]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":215,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.434,"distance_from_trace_point":3.187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268471,40.056629]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":216,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.466,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268646,40.056541]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":217,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.496,"distance_from_trace_point":2.125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.268845,40.056461]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":218,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.528,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269066,40.056404]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":219,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.562,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269272,40.056362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":220,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.593,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269485,40.056347]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":221,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.624,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269699,40.056347]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":222,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.655,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.269928,40.056362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":223,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.688,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270149,40.056412]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":224,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.721,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270355,40.056477]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":225,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.753,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270569,40.056572]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":226,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.789,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270775,40.056683]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":227,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.825,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.270981,40.056793]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":228,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.861,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271202,40.056911]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":229,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.900,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271423,40.057026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":230,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.939,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271667,40.057137]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":231,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.979,"distance_from_trace_point":2.125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.271904,40.057259]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":232,"matched_point_type":"matched","edge_index":24,"distance_along_edge":0.070,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272141,40.057384]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":233,"matched_point_type":"matched","edge_index":24,"distance_along_edge":0.212,"distance_from_trace_point":3.187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272377,40.057518]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":234,"matched_point_type":"matched","edge_index":24,"distance_along_edge":0.356,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272629,40.057652]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":235,"matched_point_type":"matched","edge_index":24,"distance_along_edge":0.507,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.272873,40.057785]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":236,"matched_point_type":"matched","edge_index":24,"distance_along_edge":0.654,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273132,40.057926]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":237,"matched_point_type":"matched","edge_index":24,"distance_along_edge":0.811,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273384,40.058064]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":238,"matched_point_type":"matched","edge_index":24,"distance_along_edge":0.963,"distance_from_trace_point":2.125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273643,40.058193]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":239,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.021,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.273918,40.058334]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":240,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.052,"distance_from_trace_point":2.863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274178,40.058479]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":241,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.081,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274445,40.058628]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":242,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.112,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274712,40.058781]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":243,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.142,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.274963,40.058933]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":244,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.172,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275223,40.059090]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":245,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.202,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275475,40.059246]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":246,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.232,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275726,40.059402]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":247,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.261,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.275970,40.059563]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":248,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.291,"distance_from_trace_point":2.125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276215,40.059727]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":249,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.321,"distance_from_trace_point":2.125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276466,40.059891]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":250,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.351,"distance_from_trace_point":2.863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276718,40.060059]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":251,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.382,"distance_from_trace_point":2.863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.276962,40.060230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":252,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.412,"distance_from_trace_point":2.125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277214,40.060398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":253,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.443,"distance_from_trace_point":2.863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277458,40.060566]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":254,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.473,"distance_from_trace_point":2.863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277702,40.060734]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":255,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.503,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.277954,40.060905]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":256,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.534,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278198,40.061073]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":257,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.564,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278442,40.061241]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":258,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.594,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278694,40.061409]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":259,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.625,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.278931,40.061573]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":260,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.654,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279175,40.061737]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":261,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.684,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279427,40.061905]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":262,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.714,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279671,40.062073]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":263,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.744,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.279922,40.062241]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":264,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.775,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280174,40.062412]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":265,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.806,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280426,40.062580]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":266,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.837,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280663,40.062740]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":267,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.866,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.280907,40.062904]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":268,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.895,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281136,40.063061]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":269,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.924,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281372,40.063225]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":270,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.953,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281601,40.063389]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":271,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.982,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.281830,40.063549]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":272,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.040,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282066,40.063713]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":273,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.158,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282310,40.063877]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":274,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.279,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282547,40.064037]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":275,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.395,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.282791,40.064205]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":276,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.517,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283035,40.064369]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":277,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.637,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283279,40.064537]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":278,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.758,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283524,40.064705]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":279,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.880,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.283775,40.064873]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":280,"matched_point_type":"matched","edge_index":27,"distance_along_edge":0.017,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284027,40.065048]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":281,"matched_point_type":"matched","edge_index":27,"distance_along_edge":0.653,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284279,40.065212]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":282,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.014,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284546,40.065376]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":283,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.047,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.284805,40.065544]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":284,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.079,"distance_from_trace_point":2.863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285057,40.065712]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":285,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.111,"distance_from_trace_point":3.187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285324,40.065880]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":286,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.144,"distance_from_trace_point":3.922}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285576,40.066044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":287,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.176,"distance_from_trace_point":4.249}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.285835,40.066204]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":288,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.208,"distance_from_trace_point":4.249}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286102,40.066360]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":289,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.240,"distance_from_trace_point":4.249}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286377,40.066513]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":290,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.272,"distance_from_trace_point":4.249}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286644,40.066662]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":291,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.304,"distance_from_trace_point":3.187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.286911,40.066795]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":292,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.334,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287193,40.066929]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":293,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.366,"distance_from_trace_point":3.187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287468,40.067055]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":294,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.397,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.287750,40.067169]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":295,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.427,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288048,40.067268]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":296,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.458,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288338,40.067368]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":297,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.488,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288635,40.067471]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":298,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.520,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.288933,40.067558]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":299,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.550,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289253,40.067642]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":300,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.583,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289566,40.067722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":301,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.614,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.289879,40.067791]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":302,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.645,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290192,40.067852]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":303,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.676,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290512,40.067905]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":304,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.707,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.290825,40.067955]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":305,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.738,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291130,40.067993]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":306,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.767,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291435,40.068024]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":307,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.797,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291748,40.068050]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":308,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.827,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292061,40.068069]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":309,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.857,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292374,40.068081]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":310,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.887,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292686,40.068089]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":311,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.917,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292999,40.068085]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":312,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.947,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293327,40.068081]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":313,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.978,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293648,40.068081]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":314,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.075,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293968,40.068073]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":315,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.338,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294304,40.068069]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":316,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.612,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294624,40.068066]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":317,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.875,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294968,40.068058]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":318,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.024,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295303,40.068054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":319,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.067,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295647,40.068047]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":320,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.111,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295982,40.068043]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":321,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.154,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296326,40.068043]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":322,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.198,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296677,40.068043]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":323,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.243,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297028,40.068043]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":324,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.287,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297371,40.068047]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":325,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.331,"distance_from_trace_point":4.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297722,40.068047]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":326,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.376,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298073,40.068047]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":327,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.421,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298424,40.068054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":328,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.466,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298782,40.068066]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":329,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.512,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299133,40.068077]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":330,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.556,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299484,40.068092]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":331,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.601,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299843,40.068119]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":332,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.647,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300201,40.068146]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":333,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.693,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300560,40.068169]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":334,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.739,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300911,40.068207]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":335,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.784,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301277,40.068249]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":336,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.832,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301628,40.068287]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":337,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.877,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301979,40.068336]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":338,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.923,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302330,40.068390]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":339,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.968,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302681,40.068439]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":340,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.142,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303024,40.068485]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":341,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.596,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303368,40.068535]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":342,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.010,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303703,40.068581]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":343,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.097,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304039,40.068630]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":344,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.185,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304359,40.068680]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":345,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.268,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304688,40.068726]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":346,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.354,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305008,40.068775]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":347,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.437,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305336,40.068821]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":348,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.523,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305664,40.068871]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":349,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.608,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305992,40.068920]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":350,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.694,"distance_from_trace_point":5.633}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306328,40.068970]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":351,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.781,"distance_from_trace_point":6.044}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306656,40.069016]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":352,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.867,"distance_from_trace_point":6.044}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306984,40.069065]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":353,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.952,"distance_from_trace_point":6.457}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307312,40.069111]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":354,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.120,"distance_from_trace_point":6.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307640,40.069153]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":355,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.392,"distance_from_trace_point":6.457}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307976,40.069199]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":356,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.671,"distance_from_trace_point":6.044}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.308311,40.069241]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":357,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.949,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.308647,40.069283]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":358,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.047,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.308975,40.069324]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":359,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.103,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.309311,40.069370]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":360,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.160,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.309654,40.069416]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":361,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.218,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.309990,40.069466]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":362,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.276,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.310333,40.069515]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":363,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.335,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.310677,40.069565]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":364,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.393,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.311020,40.069611]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":365,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.452,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.311371,40.069664]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":366,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.512,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.311714,40.069710]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":367,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.570,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.312073,40.069759]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":368,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.632,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.312424,40.069805]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":369,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.691,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.312775,40.069855]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":370,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.751,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.313133,40.069904]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":371,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.813,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.313484,40.069950]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":372,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.872,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.313843,40.070000]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":373,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.934,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.314201,40.070049]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":374,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.995,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.314560,40.070103]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":375,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.137,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.314903,40.070160]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":376,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.281,"distance_from_trace_point":5.223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.315262,40.070213]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":377,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.430,"distance_from_trace_point":5.633}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.315613,40.070267]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":378,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.577,"distance_from_trace_point":6.044}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.315964,40.070324]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":379,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.724,"distance_from_trace_point":6.457}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.316307,40.070374]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":380,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.867,"distance_from_trace_point":6.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.316650,40.070423]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":381,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.012,"distance_from_trace_point":7.286}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.317001,40.070469]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":382,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.199,"distance_from_trace_point":7.286}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.317352,40.070515]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":383,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.386,"distance_from_trace_point":7.286}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.317696,40.070557]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":384,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.569,"distance_from_trace_point":6.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.318039,40.070602]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":385,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.752,"distance_from_trace_point":6.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.318390,40.070648]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":386,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.939,"distance_from_trace_point":7.286}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.318741,40.070679]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":387,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.028,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.319092,40.070698]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":388,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.070,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.319443,40.070728]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":389,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.112,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.319809,40.070755]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":390,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.156,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.320160,40.070778]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":391,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.197,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.320511,40.070805]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":392,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.239,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.320869,40.070839]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":393,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.282,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.321213,40.070869]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":394,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.323,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.321564,40.070904]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":395,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.365,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.321915,40.070934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":396,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.407,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.322266,40.070965]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":397,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.449,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.322609,40.070995]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":398,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.490,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.322960,40.071026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":399,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.532,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.323303,40.071072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":400,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.574,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.323654,40.071136]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":401,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.617,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.323997,40.071220]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":402,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.659,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.324333,40.071301]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":403,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.701,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.324677,40.071369]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":404,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.743,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.325020,40.071445]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":405,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.786,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.325378,40.071533]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":406,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.831,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.325722,40.071625]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":407,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.874,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.326065,40.071712]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":408,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.917,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.326408,40.071800]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":409,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.960,"distance_from_trace_point":3.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.326759,40.071892]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":410,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.003,"distance_from_trace_point":4.816}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.327080,40.071972]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":411,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.040,"distance_from_trace_point":5.028}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.327408,40.072063]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":412,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.078,"distance_from_trace_point":5.419}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.327728,40.072159]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":413,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.115,"distance_from_trace_point":5.028}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.328049,40.072250]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":414,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.152,"distance_from_trace_point":4.641}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.328369,40.072361]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":415,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.190,"distance_from_trace_point":5.682}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.328697,40.072472]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":416,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.228,"distance_from_trace_point":6.445}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.329018,40.072582]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":417,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.266,"distance_from_trace_point":6.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.329346,40.072701]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":418,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.305,"distance_from_trace_point":6.445}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.329659,40.072815]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":419,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.343,"distance_from_trace_point":5.682}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.329971,40.072937]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":420,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.381,"distance_from_trace_point":5.310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.330284,40.073063]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":421,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.419,"distance_from_trace_point":5.682}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.330582,40.073193]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":422,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.456,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.330894,40.073334]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":423,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.495,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.331192,40.073471]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":424,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.533,"distance_from_trace_point":7.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.331505,40.073612]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":425,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.572,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.331810,40.073750]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":426,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.610,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.332123,40.073895]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":427,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.650,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.332436,40.074039]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":428,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.689,"distance_from_trace_point":6.363}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.332748,40.074184]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":429,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.729,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.333061,40.074326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":430,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.768,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.333382,40.074474]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":431,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.808,"distance_from_trace_point":6.363}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.333694,40.074619]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":432,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.848,"distance_from_trace_point":6.363}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.334015,40.074768]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":433,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.888,"distance_from_trace_point":6.363}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.334328,40.074917]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":434,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.928,"distance_from_trace_point":7.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.334641,40.075066]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":435,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.968,"distance_from_trace_point":7.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.334953,40.075214]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":436,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.031,"distance_from_trace_point":7.418}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.335258,40.075356]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":437,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.185,"distance_from_trace_point":7.418}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.335571,40.075493]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":438,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.342,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.335876,40.075619]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":439,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.493,"distance_from_trace_point":5.310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.336174,40.075752]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":440,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.643,"distance_from_trace_point":4.591}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.336472,40.075893]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":441,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.795,"distance_from_trace_point":5.310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.336769,40.076031]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":442,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.946,"distance_from_trace_point":5.652}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.337067,40.076172]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":443,"matched_point_type":"matched","edge_index":41,"distance_along_edge":0.030,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.337364,40.076309]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":444,"matched_point_type":"matched","edge_index":41,"distance_along_edge":0.594,"distance_from_trace_point":5.652}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.337669,40.076450]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":445,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.010,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.337975,40.076591]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":446,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.042,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.338272,40.076729]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":447,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.073,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.338585,40.076874]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":448,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.106,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.338890,40.077015]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":449,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.138,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.339195,40.077156]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":450,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.170,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.339508,40.077301]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":451,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.202,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.339821,40.077442]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":452,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.235,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.340134,40.077587]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":453,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.268,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.340454,40.077736]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":454,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.301,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.340767,40.077881]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":455,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.334,"distance_from_trace_point":6.003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.341087,40.078030]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":456,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.368,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.341415,40.078178]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":457,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.402,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.341736,40.078327]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":458,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.436,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.342064,40.078476]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":459,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.470,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.342384,40.078625]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":460,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.504,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.342712,40.078777]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":461,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.538,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.343040,40.078926]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":462,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.572,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.343369,40.079079]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":463,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.607,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.343712,40.079231]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":464,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.642,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.344040,40.079376]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":465,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.676,"distance_from_trace_point":6.362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.344383,40.079517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":466,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.711,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.344719,40.079651]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":467,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.745,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.345070,40.079781]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":468,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.780,"distance_from_trace_point":6.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.345406,40.079906]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":469,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.814,"distance_from_trace_point":6.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.345757,40.080029]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":470,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.848,"distance_from_trace_point":6.833}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.346100,40.080147]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":471,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.882,"distance_from_trace_point":7.225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.346443,40.080254]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":472,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.915,"distance_from_trace_point":7.225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.346794,40.080360]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":473,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.949,"distance_from_trace_point":7.621}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.347145,40.080460]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":474,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.983,"distance_from_trace_point":8.019}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.347504,40.080544]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":475,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.060,"distance_from_trace_point":7.021}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.347862,40.080624]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":476,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.183,"distance_from_trace_point":5.815}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.348228,40.080708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":477,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.309,"distance_from_trace_point":5.815}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.348602,40.080788]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":478,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.436,"distance_from_trace_point":5.633}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.348976,40.080860]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":479,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.563,"distance_from_trace_point":5.223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.349350,40.080929]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":480,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.690,"distance_from_trace_point":4.816}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.349731,40.080990]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":481,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.818,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.350105,40.081055]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":482,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.944,"distance_from_trace_point":4.816}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.350487,40.081112]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":483,"matched_point_type":"matched","edge_index":44,"distance_along_edge":0.215,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.350861,40.081165]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":484,"matched_point_type":"matched","edge_index":44,"distance_along_edge":0.592,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.351234,40.081226]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":485,"matched_point_type":"matched","edge_index":44,"distance_along_edge":0.969,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.351608,40.081284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":486,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.150,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.351982,40.081341]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":487,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.312,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.352356,40.081398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":488,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.475,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.352730,40.081455]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":489,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.638,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.353104,40.081512]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":490,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.800,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.353477,40.081570]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":491,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.963,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.353851,40.081627]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":492,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.014,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.354225,40.081684]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":493,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.115,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.354591,40.081741]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":494,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.213,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.354958,40.081795]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":495,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.312,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.355331,40.081852]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":496,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.412,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.355705,40.081905]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":497,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.512,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.356079,40.081959]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":498,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.613,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.356461,40.082016]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":499,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.715,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.356842,40.082073]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":500,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.818,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.357231,40.082130]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":501,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.922,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.357613,40.082188]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":502,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.087,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.358002,40.082256]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":503,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.455,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.358391,40.082314]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":504,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.821,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.358780,40.082371]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":505,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.010,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.359161,40.082432]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":506,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.028,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.359543,40.082493]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":507,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.047,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.359932,40.082554]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":508,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.066,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.360313,40.082615]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":509,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.084,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.360695,40.082676]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":510,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.103,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.361084,40.082737]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":511,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.122,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.361465,40.082794]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":512,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.140,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.361847,40.082855]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":513,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.159,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.362228,40.082916]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":514,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.178,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.362610,40.082973]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":515,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.196,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.362999,40.083031]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":516,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.215,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.363380,40.083088]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":517,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.233,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.363762,40.083145]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":518,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.252,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.364151,40.083206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":519,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.271,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.364532,40.083267]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":520,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.289,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.364914,40.083328]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":521,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.308,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.365303,40.083385]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":522,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.327,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.365685,40.083443]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":523,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.345,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.366074,40.083500]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":524,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.364,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.366455,40.083557]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":525,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.383,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.366844,40.083618]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":526,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.402,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.367226,40.083679]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":527,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.420,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.367615,40.083748]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":528,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.439,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.368004,40.083820]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":529,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.459,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.368385,40.083900]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":530,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.477,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.368767,40.083984]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":531,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.496,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.369148,40.084072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":532,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.515,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.369522,40.084167]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":533,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.534,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.369888,40.084270]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":534,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.553,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.370247,40.084370]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":535,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.571,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.370598,40.084480]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":536,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.589,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.370941,40.084591]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":537,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.607,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.371284,40.084709]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":538,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.625,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.371620,40.084827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":539,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.642,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.371948,40.084949]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":540,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.660,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.372276,40.085075]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":541,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.677,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.372612,40.085213]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":542,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.695,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.372940,40.085350]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":543,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.713,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.373268,40.085491]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":544,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.731,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.373581,40.085640]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":545,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.749,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.373901,40.085789]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":546,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.766,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.374214,40.085934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":547,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.784,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.374542,40.086090]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":548,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.802,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.374863,40.086239]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":549,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.820,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.375183,40.086388]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":550,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.838,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.375504,40.086540]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":551,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.856,"distance_from_trace_point":3.186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.375824,40.086689]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":552,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.874,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.376137,40.086838]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":553,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.891,"distance_from_trace_point":3.186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.376442,40.086994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":554,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.909,"distance_from_trace_point":4.248}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.376755,40.087143]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":555,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.927,"distance_from_trace_point":4.591}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.377075,40.087292]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":556,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.944,"distance_from_trace_point":4.261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.377388,40.087437]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":557,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.962,"distance_from_trace_point":4.261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.377686,40.087582]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":558,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.979,"distance_from_trace_point":4.591}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.378006,40.087734]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":559,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.997,"distance_from_trace_point":4.411}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.378304,40.087864]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":560,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.037,"distance_from_trace_point":3.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.378616,40.088001]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":561,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.085,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.378922,40.088142]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":562,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.133,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.379227,40.088291]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":563,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.182,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.379539,40.088436]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":564,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.231,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.379852,40.088585]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":565,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.281,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.380165,40.088730]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":566,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.330,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.380486,40.088879]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":567,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.380,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.380806,40.089024]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":568,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.430,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.381126,40.089172]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":569,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.481,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.381447,40.089321]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":570,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.531,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.381767,40.089474]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":571,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.582,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.382095,40.089630]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":572,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.634,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.382423,40.089787]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":573,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.686,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.382751,40.089939]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":574,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.738,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.383080,40.090096]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":575,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.790,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.383415,40.090248]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":576,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.842,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.383751,40.090405]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":577,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.895,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.384087,40.090565]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":578,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.948,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.384422,40.090729]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":579,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.000,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.384758,40.090881]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":580,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.009,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.385094,40.091034]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":581,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.018,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.385429,40.091194]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":582,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.027,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.385757,40.091351]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":583,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.035,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.386093,40.091507]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":584,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.044,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.386421,40.091660]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":585,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.053,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.386757,40.091816]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":586,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.062,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.387100,40.091976]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":587,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.071,"distance_from_trace_point":2.473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.387436,40.092133]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":588,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.079,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.387779,40.092289]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":589,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.088,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.388115,40.092445]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":590,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.097,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.388451,40.092602]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":591,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.106,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.388786,40.092762]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":592,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.115,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.389130,40.092922]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":593,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.124,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.389458,40.093079]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":594,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.133,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.389809,40.093239]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":595,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.142,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.390152,40.093403]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":596,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.151,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.390488,40.093559]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":597,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.160,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.390831,40.093723]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":598,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.169,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.391174,40.093880]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":599,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.178,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.391518,40.094044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":600,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.187,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.391853,40.094200]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":601,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.196,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.392197,40.094364]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":602,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.205,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.392532,40.094521]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":603,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.213,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.392868,40.094677]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":604,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.222,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.393211,40.094837]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":605,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.231,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.393562,40.094994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":606,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.240,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.393898,40.095142]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":607,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.249,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.394241,40.095291]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":608,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.258,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.394585,40.095432]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":609,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.267,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.394936,40.095562]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":610,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.275,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.395287,40.095692]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":611,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.284,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.395653,40.095818]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":612,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.293,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.396004,40.095940]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":613,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.302,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.396370,40.096050]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":614,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.311,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.396721,40.096161]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":615,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.319,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.397095,40.096264]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":616,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.328,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.397446,40.096359]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":617,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.336,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.397797,40.096451]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":618,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.345,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.398148,40.096527]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":619,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.353,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.398499,40.096603]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":620,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.361,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.398842,40.096676]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":621,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.369,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.399170,40.096745]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":622,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.377,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.399513,40.096806]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":623,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.385,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.399872,40.096863]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":624,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.393,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.400215,40.096912]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":625,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.401,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.400551,40.096962]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":626,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.408,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.400887,40.097012]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":627,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.416,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.401222,40.097061]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":628,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.424,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.401566,40.097107]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":629,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.432,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.401909,40.097157]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":630,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.439,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.402245,40.097206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":631,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.447,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.402557,40.097252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":632,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.454,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.402855,40.097294]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":633,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.461,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.403130,40.097336]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":634,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.467,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.403397,40.097374]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":635,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.473,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.403671,40.097416]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":636,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.480,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.403946,40.097454]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":637,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.486,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.404228,40.097492]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":638,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.492,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.404503,40.097534]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":639,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.499,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.404785,40.097572]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":640,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.505,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.405075,40.097614]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":641,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.512,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.405373,40.097656]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":642,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.518,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.405678,40.097702]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":643,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.525,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.405991,40.097748]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":644,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.533,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.406311,40.097797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":645,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.540,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.406631,40.097843]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":646,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.547,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.406960,40.097893]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":647,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.555,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.407280,40.097942]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":648,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.562,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.407600,40.097988]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":649,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.569,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.407936,40.098038]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":650,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.577,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.408264,40.098091]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":651,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.584,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.408600,40.098148]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":652,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.592,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.408928,40.098206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":653,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.600,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.409264,40.098267]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":654,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.607,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.409599,40.098335]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":655,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.615,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.409920,40.098412]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":656,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.623,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.410233,40.098488]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":657,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.630,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.410561,40.098576]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":658,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.638,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.410881,40.098663]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":659,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.646,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.411194,40.098751]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":660,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.653,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.411507,40.098843]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":661,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.661,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.411827,40.098942]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":662,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.668,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.412140,40.099041]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":663,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.676,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.412460,40.099152]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":664,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.684,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.412773,40.099262]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":665,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.692,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.413094,40.099384]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":666,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.700,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.413414,40.099510]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":667,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.708,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.413734,40.099636]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":668,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.716,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.414055,40.099766]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":669,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.724,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.414383,40.099899]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":670,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.732,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.414711,40.100033]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":671,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.740,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.415039,40.100166]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":672,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.749,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.415359,40.100300]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":673,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.757,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.415680,40.100437]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":674,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.765,"distance_from_trace_point":3.530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.416008,40.100571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":675,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.774,"distance_from_trace_point":3.890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.416336,40.100704]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":676,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.782,"distance_from_trace_point":3.890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.416664,40.100834]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":677,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.790,"distance_from_trace_point":3.890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.416992,40.100960]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":678,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.798,"distance_from_trace_point":3.222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.417320,40.101082]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":679,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.807,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.417648,40.101215]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":680,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.815,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.417984,40.101349]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":681,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.823,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.418304,40.101479]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":682,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.832,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.418633,40.101612]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":683,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.840,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.418953,40.101742]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":684,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.848,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.419281,40.101871]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":685,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.856,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.419601,40.102001]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":686,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.864,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.419930,40.102131]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":687,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.873,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.420250,40.102261]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":688,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.881,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.420563,40.102390]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":689,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.889,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.420891,40.102520]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":690,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.897,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.421211,40.102650]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":691,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.905,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.421532,40.102776]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":692,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.913,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.421852,40.102905]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":693,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.921,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.422173,40.103035]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":694,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.930,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.422501,40.103165]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":695,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.938,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.422829,40.103302]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":696,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.946,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.423149,40.103439]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":697,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.954,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.423477,40.103577]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":698,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.963,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.423798,40.103710]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":699,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.971,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.424126,40.103836]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":700,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.979,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.424454,40.103966]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":701,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.988,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.424782,40.104095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":702,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.996,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.425110,40.104214]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":703,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.021,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.425438,40.104343]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":704,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.065,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.425758,40.104473]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":705,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.108,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.426086,40.104607]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":706,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.153,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.426407,40.104736]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":707,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.196,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.426735,40.104866]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":708,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.240,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.427055,40.104996]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":709,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.284,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.427383,40.105129]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":710,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.328,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.427719,40.105267]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":711,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.374,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.428055,40.105400]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":712,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.419,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.428391,40.105537]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":713,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.464,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.428726,40.105671]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":714,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.510,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.429054,40.105804]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":715,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.554,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.429390,40.105938]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":716,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.599,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.429733,40.106071]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":717,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.645,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.430069,40.106209]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":718,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.691,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.430412,40.106346]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":719,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.737,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.430748,40.106480]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":720,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.782,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.431084,40.106613]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":721,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.828,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.431419,40.106747]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":722,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.873,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.431755,40.106884]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":723,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.918,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.432091,40.107029]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":724,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.964,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.432426,40.107170]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":725,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.013,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.432762,40.107307]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":726,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.072,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.433105,40.107441]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":727,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.131,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.433449,40.107571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":728,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.189,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.433784,40.107700]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":729,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.247,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.434128,40.107838]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":730,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.306,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.434471,40.107975]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":731,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.366,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.434814,40.108112]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":732,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.425,"distance_from_trace_point":3.222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.435158,40.108250]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":733,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.484,"distance_from_trace_point":3.222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.435509,40.108391]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":734,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.545,"distance_from_trace_point":3.530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.435844,40.108528]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":735,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.603,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.436180,40.108662]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":736,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.661,"distance_from_trace_point":3.222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.436523,40.108803]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":737,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.721,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.436859,40.108936]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":738,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.779,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.437202,40.109074]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":739,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.839,"distance_from_trace_point":2.841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.437538,40.109211]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":740,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.897,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.437881,40.109348]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":741,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.956,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.438225,40.109489]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":742,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.167,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.438576,40.109631]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":743,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.804,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.438919,40.109772]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":744,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.131,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.439270,40.109913]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":745,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.326,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.439613,40.110050]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":746,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.515,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.439964,40.110191]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":747,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.710,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.440300,40.110329]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":748,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.897,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.440628,40.110474]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":749,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.030,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.440956,40.110619]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":750,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.098,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.441284,40.110764]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":751,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.167,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.441612,40.110912]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":752,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.235,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.441933,40.111065]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":753,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.303,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.442245,40.111221]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":754,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.371,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.442558,40.111374]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":755,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.437,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.442871,40.111530]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":756,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.505,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.443184,40.111683]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":757,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.571,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.443497,40.111843]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":758,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.639,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.443810,40.112003]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":759,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.707,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.444107,40.112156]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":760,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.772,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.444420,40.112316]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":761,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.839,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.444733,40.112476]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":762,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.907,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.445038,40.112633]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":763,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.973,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.445335,40.112789]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":764,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.345,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.445648,40.112946]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":765,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.951,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.445953,40.113094]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":766,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.034,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.446266,40.113243]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":767,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.072,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.446587,40.113392]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":768,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.110,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.446899,40.113544]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":769,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.148,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.447205,40.113701]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":770,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.185,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.447525,40.113857]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":771,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.224,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.447838,40.114014]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":772,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.262,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.448151,40.114170]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":773,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.300,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.448463,40.114326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":774,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.339,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.448784,40.114487]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":775,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.378,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.449104,40.114643]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":776,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.416,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.449425,40.114807]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":777,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.456,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.449745,40.114964]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":778,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.495,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.450066,40.115124]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":779,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.534,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.450386,40.115284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":780,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.573,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.450706,40.115440]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":781,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.612,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.451027,40.115601]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":782,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.651,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.451347,40.115761]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":783,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.690,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.451668,40.115921]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":784,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.729,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.451988,40.116081]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":785,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.768,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.452309,40.116238]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":786,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.807,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.452629,40.116402]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":787,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.846,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.452942,40.116558]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":788,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.884,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.453262,40.116722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":789,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.924,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.453575,40.116886]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":790,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.962,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.453896,40.117050]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":791,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.001,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.454216,40.117214]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":792,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.030,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.454536,40.117371]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":793,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.058,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.454857,40.117527]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":794,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.087,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.455185,40.117687]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":795,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.116,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.455505,40.117847]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":796,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.144,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.455826,40.118008]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":797,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.172,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.456154,40.118164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":798,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.201,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.456482,40.118320]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":799,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.230,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.456795,40.118477]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":800,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.258,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.457115,40.118637]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":801,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.286,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.457436,40.118797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":802,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.315,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.457756,40.118958]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":803,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.343,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.458084,40.119122]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":804,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.372,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.458397,40.119278]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":805,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.400,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.458717,40.119438]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":806,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.429,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.459038,40.119598]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":807,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.457,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.459351,40.119759]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":808,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.485,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.459671,40.119915]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":809,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.513,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.459991,40.120079]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":810,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.542,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.460312,40.120235]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":811,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.570,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.460632,40.120388]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":812,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.598,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.460953,40.120548]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":813,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.627,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.461273,40.120708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":814,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.655,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.461586,40.120865]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":815,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.683,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.461906,40.121025]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":816,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.712,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.462219,40.121181]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":817,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.739,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.462524,40.121334]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":818,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.766,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.462830,40.121487]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":819,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.794,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.463135,40.121639]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":820,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.821,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.463432,40.121788]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":821,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.847,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.463730,40.121937]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":822,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.874,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.464027,40.122086]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":823,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.900,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.464317,40.122231]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":824,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.926,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.464615,40.122379]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":825,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.952,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.464912,40.122532]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":826,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.979,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.465202,40.122681]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":827,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.007,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.465492,40.122826]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":828,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.048,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.465790,40.122967]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":829,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.089,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.466080,40.123112]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":830,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.129,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.466377,40.123257]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":831,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.170,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.466667,40.123402]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":832,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.211,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.466949,40.123547]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":833,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.251,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.467224,40.123684]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":834,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.289,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.467506,40.123825]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":835,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.328,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.467796,40.123970]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":836,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.369,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.468071,40.124107]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":837,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.407,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.468361,40.124252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":838,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.448,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.468643,40.124393]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":839,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.487,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.468933,40.124538]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":840,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.527,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.469215,40.124680]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":841,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.567,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.469505,40.124825]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":842,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.607,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.469795,40.124969]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":843,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.648,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.470093,40.125114]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":844,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.689,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.470390,40.125263]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":845,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.730,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.470688,40.125408]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":846,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.772,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.470985,40.125557]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":847,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.813,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.471275,40.125706]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":848,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.854,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.471581,40.125858]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":849,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.896,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.471878,40.126011]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":850,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.938,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.472183,40.126171]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":851,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.982,"distance_from_trace_point":3.530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.472488,40.126328]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":852,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.009,"distance_from_trace_point":3.890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.472794,40.126476]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":853,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.026,"distance_from_trace_point":3.530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.473106,40.126629]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":854,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.042,"distance_from_trace_point":3.530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.473412,40.126781]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":855,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.058,"distance_from_trace_point":3.186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.473740,40.126938]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":856,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.076,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.474060,40.127090]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":857,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.092,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.474380,40.127251]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":858,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.110,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.474701,40.127407]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":859,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.126,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.475021,40.127567]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":860,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.144,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.475342,40.127728]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":861,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.161,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.475670,40.127892]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":862,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.178,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.475990,40.128052]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":863,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.195,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.476311,40.128212]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":864,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.212,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.476631,40.128368]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":865,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.229,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.476952,40.128525]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":866,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.246,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.477280,40.128674]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":867,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.263,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.477608,40.128819]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":868,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.280,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.477943,40.128963]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":869,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.297,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.478287,40.129101]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":870,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.315,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.478630,40.129230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":871,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.332,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.478973,40.129356]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":872,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.349,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.479324,40.129482]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":873,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.366,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.479660,40.129593]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":874,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.382,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.480011,40.129707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":875,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.399,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.480354,40.129810]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":876,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.416,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.480721,40.129910]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":877,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.433,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.481071,40.130005]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":878,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.450,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.481422,40.130100]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":879,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.466,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.481789,40.130180]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":880,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.483,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.482147,40.130257]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":881,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.500,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.482506,40.130333]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":882,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.517,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.482864,40.130405]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":883,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.533,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.483238,40.130470]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":884,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.550,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.483604,40.130535]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":885,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.567,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.483978,40.130589]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":886,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.584,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.484352,40.130638]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":887,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.601,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.484726,40.130684]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":888,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.618,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.485100,40.130722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":889,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.635,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.485481,40.130756]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":890,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.652,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.485863,40.130779]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":891,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.669,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.486244,40.130802]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":892,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.686,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.486626,40.130817]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":893,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.703,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.487015,40.130833]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":894,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.720,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.487396,40.130840]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":895,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.738,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.487778,40.130833]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":896,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.755,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.488167,40.130836]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":897,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.772,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.488548,40.130829]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":898,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.789,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.488937,40.130810]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":899,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.806,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.489326,40.130791]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":900,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.824,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.489700,40.130760]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":901,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.841,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.490082,40.130733]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":902,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.858,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.490456,40.130695]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":903,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.874,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.490837,40.130653]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":904,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.892,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.491203,40.130608]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":905,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.908,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.491577,40.130558]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":906,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.925,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.491943,40.130512]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":907,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.942,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.492317,40.130466]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":908,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.959,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.492683,40.130421]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":909,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.975,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.493042,40.130375]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":910,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.991,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.493416,40.130329]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":911,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.429,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.493790,40.130280]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":912,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.019,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.494164,40.130230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":913,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.072,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.494537,40.130184]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":914,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.126,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.494911,40.130138]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":915,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.179,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.495285,40.130093]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":916,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.233,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.495659,40.130043]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":917,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.286,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.496033,40.129997]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":918,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.340,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.496407,40.129951]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":919,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.393,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.496788,40.129906]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":920,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.448,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.497162,40.129856]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":921,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.501,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.497543,40.129803]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":922,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.556,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.497917,40.129753]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":923,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.609,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.498291,40.129707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":924,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.663,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.498672,40.129662]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":925,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.717,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.499046,40.129616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":926,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.771,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.499428,40.129570]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":927,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.825,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.499802,40.129524]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":928,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.879,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.500183,40.129475]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":929,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.933,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.500557,40.129429]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":930,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.987,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.500931,40.129379]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":931,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.501,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.501305,40.129333]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":932,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.004,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.501686,40.129284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":933,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.021,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.502060,40.129234]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":934,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.038,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.502441,40.129185]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":935,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.055,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.502815,40.129139]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":936,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.072,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.503189,40.129093]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":937,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.089,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.503571,40.129044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":938,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.106,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.503944,40.128998]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":939,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.123,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.504318,40.128952]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":940,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.140,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.504700,40.128902]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":941,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.157,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.505074,40.128857]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":942,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.173,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.505447,40.128807]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":943,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.190,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.505829,40.128761]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":944,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.207,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.506210,40.128712]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":945,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.225,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.506592,40.128666]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":946,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.242,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.506966,40.128616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":947,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.258,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.507347,40.128571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":948,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.276,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.507721,40.128521]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":949,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.292,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.508102,40.128475]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":950,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.309,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.508476,40.128426]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":951,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.326,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.508858,40.128380]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":952,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.343,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.509254,40.128330]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":953,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.361,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.509628,40.128281]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":954,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.378,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.510002,40.128235]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":955,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.395,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.510376,40.128185]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":956,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.412,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.510750,40.128139]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":957,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.428,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.511131,40.128094]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":958,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.446,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.511505,40.128056]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":959,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.462,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.511879,40.128021]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":960,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.479,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.512260,40.127995]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":961,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.496,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.512642,40.127972]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":962,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.513,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.513023,40.127953]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":963,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.530,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.513412,40.127941]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":964,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.547,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.513786,40.127937]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":965,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.564,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.514175,40.127934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":966,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.581,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.514549,40.127934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":967,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.597,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.514931,40.127953]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":968,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.614,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.515312,40.127972]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":969,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.631,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.515694,40.127991]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":970,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.648,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.516075,40.128014]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":971,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.665,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.516449,40.128048]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":972,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.682,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.516823,40.128082]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":973,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.699,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.517204,40.128128]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":974,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.716,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.517586,40.128174]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":975,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.733,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.517960,40.128231]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":976,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.750,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.518333,40.128292]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":977,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.767,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.518707,40.128353]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":978,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.784,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.519081,40.128422]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":979,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.801,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.519447,40.128494]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":980,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.817,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.519814,40.128571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":981,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.834,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.520180,40.128643]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":982,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.851,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.520554,40.128719]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":983,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.868,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.520920,40.128792]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":984,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.885,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.521294,40.128868]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":985,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.902,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.521660,40.128941]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":986,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.919,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.522034,40.129013]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":987,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.936,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.522400,40.129089]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":988,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.953,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.522766,40.129162]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":989,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.970,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.523132,40.129234]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":990,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.986,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.523506,40.129311]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":991,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.003,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.523880,40.129387]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":992,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.015,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.524246,40.129459]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":993,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.027,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.524612,40.129532]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":994,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.040,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.524979,40.129604]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":995,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.052,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.525352,40.129681]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":996,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.065,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.525719,40.129753]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":997,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.077,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.526093,40.129829]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":998,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.090,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.526459,40.129902]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":999,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.102,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.526833,40.129974]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1000,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.115,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.527206,40.130051]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1001,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.127,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.527573,40.130123]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1002,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.139,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.527946,40.130199]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1003,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.152,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.528313,40.130276]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1004,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.164,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.528687,40.130352]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1005,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.177,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.529053,40.130428]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1006,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.189,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.529419,40.130505]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1007,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.202,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.529793,40.130577]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1008,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.214,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.530159,40.130650]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1009,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.227,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.530533,40.130722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1010,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.239,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.530899,40.130795]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1011,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.252,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.531273,40.130871]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1012,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.264,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.531639,40.130943]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1013,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.276,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.532013,40.131016]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1014,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.289,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.532379,40.131088]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1015,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.301,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.532745,40.131161]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1016,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.314,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.533119,40.131237]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1017,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.326,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.533493,40.131310]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1018,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.339,"distance_from_trace_point":4.009}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.533859,40.131386]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1019,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.351,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.534225,40.131462]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1020,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.364,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.534599,40.131538]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1021,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.376,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.534966,40.131611]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1022,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.388,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.535339,40.131687]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1023,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.401,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.535706,40.131760]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1024,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.413,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.536079,40.131832]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1025,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.426,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.536453,40.131912]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1026,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.439,"distance_from_trace_point":3.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.536819,40.131992]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1027,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.451,"distance_from_trace_point":4.009}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.537186,40.132080]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1028,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.463,"distance_from_trace_point":3.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.537552,40.132175]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1029,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.476,"distance_from_trace_point":3.222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.537910,40.132275]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1030,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.489,"distance_from_trace_point":3.222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.538261,40.132378]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1031,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.501,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.538620,40.132488]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1032,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.513,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.538979,40.132603]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1033,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.526,"distance_from_trace_point":3.222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.539322,40.132721]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1034,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.538,"distance_from_trace_point":3.222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.539673,40.132851]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1035,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.551,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.540024,40.132980]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1036,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.564,"distance_from_trace_point":2.840}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.540367,40.133118]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1037,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.576,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.540703,40.133255]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1038,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.589,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.541039,40.133400]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1039,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.601,"distance_from_trace_point":2.472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.541367,40.133553]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1040,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.614,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.541687,40.133705]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1041,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.626,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.542007,40.133858]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1042,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.638,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.542320,40.134018]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1043,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.651,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.542625,40.134178]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1044,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.663,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.542915,40.134342]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1045,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.674,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.543213,40.134510]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1046,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.686,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.543503,40.134682]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1047,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.698,"distance_from_trace_point":0.771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.543785,40.134861]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1048,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.710,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.544067,40.135044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1049,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.722,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.544350,40.135227]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1050,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.734,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.544624,40.135418]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1051,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.747,"distance_from_trace_point":3.186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.544891,40.135609]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1052,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.758,"distance_from_trace_point":3.186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.545158,40.135803]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1053,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.770,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.545418,40.136002]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1054,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.782,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.545692,40.136200]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1055,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.795,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.545959,40.136398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1056,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.807,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.546219,40.136600]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1057,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.819,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.546486,40.136799]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1058,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.831,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.546745,40.136990]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1059,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.843,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.547005,40.137184]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1060,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.855,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.547264,40.137375]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1061,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.866,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.547516,40.137562]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1062,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.878,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.547768,40.137749]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1063,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.889,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.548019,40.137936]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1064,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.901,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.548264,40.138126]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1065,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.912,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.548523,40.138313]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1066,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.924,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.548775,40.138500]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1067,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.935,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.549034,40.138695]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1068,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.947,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.549286,40.138882]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1069,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.958,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.549538,40.139072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1070,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.970,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.549797,40.139263]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1071,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.981,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.550049,40.139458]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1072,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.993,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.550308,40.139648]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1073,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.343,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.550568,40.139843]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1074,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.032,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.550835,40.140038]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1075,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.185,"distance_from_trace_point":3.921}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.551094,40.140232]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1076,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.336,"distance_from_trace_point":3.614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.551361,40.140430]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1077,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.490,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.551628,40.140625]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1078,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.643,"distance_from_trace_point":2.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.551903,40.140827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1079,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.801,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.552177,40.141029]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1080,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.959,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.552452,40.141228]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1081,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.059,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.552727,40.141426]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1082,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.138,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.553001,40.141624]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1083,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.217,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.553268,40.141827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1084,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.295,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.553551,40.142025]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1085,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.375,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.553818,40.142227]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1086,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.454,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.554092,40.142433]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1087,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.535,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.554359,40.142632]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1088,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.612,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.554634,40.142838]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1089,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.693,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.554916,40.143044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1090,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.775,"distance_from_trace_point":2.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.555183,40.143242]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1091,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.852,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.555450,40.143444]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1092,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.931,"distance_from_trace_point":2.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.555717,40.143642]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1093,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.090,"distance_from_trace_point":2.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.555977,40.143833]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1094,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.846,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.556252,40.144032]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1095,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.058,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.556519,40.144230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1096,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.129,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.556786,40.144428]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1097,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.200,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.557053,40.144627]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1098,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.270,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.557320,40.144821]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1099,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.340,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.557587,40.145023]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1100,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.412,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.557854,40.145222]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1101,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.483,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.558128,40.145424]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1102,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.555,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.558403,40.145630]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1103,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.628,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.558670,40.145828]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1104,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.699,"distance_from_trace_point":2.124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.558945,40.146030]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1105,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.772,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.559212,40.146233]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1106,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.843,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.559471,40.146427]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1107,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.912,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.559723,40.146618]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1108,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.980,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.559982,40.146812]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1109,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.052,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.560249,40.147003]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1110,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.126,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.560501,40.147186]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1111,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.196,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.560753,40.147373]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1112,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.267,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.561012,40.147564]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1113,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.340,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.561272,40.147751]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1114,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.412,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.561531,40.147942]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1115,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.485,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.561790,40.148132]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1116,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.558,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.562042,40.148323]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1117,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.630,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.562302,40.148518]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1118,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.703,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.562561,40.148708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1119,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.776,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.562820,40.148899]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1120,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.849,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.563080,40.149090]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1121,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.922,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.563339,40.149281]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1122,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.995,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.563599,40.149479]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1123,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.008,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.563850,40.149670]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1124,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.016,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.564102,40.149860]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1125,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.024,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.564354,40.150055]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1126,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.032,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.564598,40.150242]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1127,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.040,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.564850,40.150433]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1128,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.048,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.565102,40.150627]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1129,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.056,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.565353,40.150818]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1130,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.064,"distance_from_trace_point":2.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.565605,40.151012]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1131,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.072,"distance_from_trace_point":2.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.565849,40.151203]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1132,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.080,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566101,40.151398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1133,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.088,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566353,40.151592]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1134,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.096,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566605,40.151787]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1135,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.104,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566856,40.151981]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1136,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.112,"distance_from_trace_point":2.862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567116,40.152180]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1137,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.120,"distance_from_trace_point":2.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567368,40.152374]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1138,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.128,"distance_from_trace_point":2.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567619,40.152569]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1139,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.136,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567879,40.152767]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1140,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.144,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568123,40.152958]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1141,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.152,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568382,40.153156]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1142,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.161,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568634,40.153355]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1143,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.169,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568886,40.153549]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1144,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.177,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569138,40.153744]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1145,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.185,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569397,40.153942]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1146,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.193,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569649,40.154140]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1147,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.201,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569908,40.154339]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1148,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.210,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570168,40.154537]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1149,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.218,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570419,40.154736]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1150,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.226,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570679,40.154934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1151,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.234,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570930,40.155128]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1152,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.242,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571190,40.155327]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1153,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.251,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571449,40.155525]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1154,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.259,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571709,40.155731]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1155,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.267,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571968,40.155933]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1156,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.276,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572235,40.156136]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1157,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.284,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572502,40.156345]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1158,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.293,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572769,40.156548]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1159,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.301,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573029,40.156750]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1160,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.310,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573288,40.156952]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1161,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.318,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573555,40.157154]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1162,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.326,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573807,40.157352]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1163,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.335,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574074,40.157558]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1164,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.343,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574341,40.157761]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1165,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.352,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574600,40.157963]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1166,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.360,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574860,40.158165]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1167,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.368,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575119,40.158363]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1168,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.377,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575386,40.158566]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1169,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.385,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575638,40.158760]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1170,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.393,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575897,40.158958]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1171,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.401,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576149,40.159157]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1172,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.410,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576401,40.159348]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1173,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.418,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576653,40.159542]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1174,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.426,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576897,40.159737]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1175,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.434,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577141,40.159927]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1176,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.441,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577393,40.160118]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1177,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.449,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577644,40.160313]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1178,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.457,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577896,40.160503]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1179,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.465,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578148,40.160698]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1180,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.474,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578392,40.160889]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1181,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.481,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578636,40.161079]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1182,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.489,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578896,40.161270]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1183,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.497,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579147,40.161465]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1184,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.505,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579399,40.161655]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1185,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.513,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579651,40.161850]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1186,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.521,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579903,40.162041]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1187,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.529,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580147,40.162228]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1188,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.537,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580399,40.162422]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1189,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.545,"distance_from_trace_point":1.807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580643,40.162613]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1190,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.553,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580887,40.162807]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1191,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.561,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581123,40.162998]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1192,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.569,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581367,40.163189]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1193,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.577,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581612,40.163376]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1194,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.585,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581856,40.163567]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1195,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.592,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582092,40.163746]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1196,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.600,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582336,40.163937]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1197,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.608,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582581,40.164124]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1198,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.616,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582825,40.164314]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1199,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.623,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583076,40.164509]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1200,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.632,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583321,40.164696]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1201,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.639,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583565,40.164886]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1202,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.647,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583801,40.165066]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1203,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.655,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584030,40.165249]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1204,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.662,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584267,40.165432]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1205,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.670,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584496,40.165607]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1206,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.677,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584732,40.165791]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1207,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.685,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584969,40.165974]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1208,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.692,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585205,40.166157]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1209,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.700,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585449,40.166336]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1210,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.707,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585686,40.166515]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1211,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.715,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585922,40.166698]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1212,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.723,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586159,40.166882]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1213,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.730,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586403,40.167068]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1214,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.738,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586639,40.167252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1215,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.745,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586884,40.167439]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1216,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.753,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587120,40.167622]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1217,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.761,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587349,40.167797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1218,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.768,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587578,40.167976]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1219,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.776,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587814,40.168159]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1220,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.783,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588043,40.168335]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1221,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.790,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588272,40.168514]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1222,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.798,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588509,40.168690]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1223,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.805,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588745,40.168869]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1224,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.813,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588974,40.169044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1225,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.820,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589203,40.169220]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1226,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.827,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589432,40.169395]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1227,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.835,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589661,40.169571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1228,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.842,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589890,40.169746]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1229,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.849,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590111,40.169918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1230,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.856,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590332,40.170090]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1231,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.864,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590561,40.170261]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1232,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.871,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590782,40.170429]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1233,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.878,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590996,40.170597]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1234,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.885,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591225,40.170769]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1235,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.892,"distance_from_trace_point":2.566}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591438,40.170940]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1236,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.899,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591652,40.171104]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1237,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.906,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591873,40.171276]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1238,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.913,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592087,40.171444]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1239,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.920,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592293,40.171608]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1240,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.926,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592506,40.171776]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1241,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.933,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592720,40.171944]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1242,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.940,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592941,40.172108]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1243,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.947,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593155,40.172272]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1244,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.954,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593369,40.172432]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1245,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.961,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593590,40.172600]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1246,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.968,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593811,40.172768]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1247,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.975,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594032,40.172935]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1248,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.982,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594254,40.173107]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1249,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.989,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594475,40.173275]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1250,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.996,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594688,40.173462]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1251,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.017,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594902,40.173641]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1252,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.052,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595123,40.173820]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1253,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.089,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595352,40.173996]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1254,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.125,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595573,40.174175]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1255,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.162,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595795,40.174351]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1256,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.198,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596031,40.174538]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1257,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.236,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596252,40.174709]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1258,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.272,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596466,40.174881]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1259,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.307,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596687,40.175053]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1260,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.342,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596901,40.175220]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1261,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.377,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597107,40.175388]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1262,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.411,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597313,40.175549]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1263,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.444,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597527,40.175713]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1264,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.478,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597717,40.175877]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1265,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.511,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597900,40.176048]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1266,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.543,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598091,40.176216]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1267,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.576,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598282,40.176376]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1268,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.608,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598465,40.176540]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1269,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.640,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598648,40.176708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1270,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.672,"distance_from_trace_point":1.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598808,40.176868]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1271,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.702,"distance_from_trace_point":1.062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598961,40.177032]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1272,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.732,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599121,40.177197]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1273,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.762,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599266,40.177361]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1274,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.791,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599403,40.177525]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1275,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.820,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599525,40.177681]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1276,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.847,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599640,40.177834]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1277,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.873,"distance_from_trace_point":0.645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599739,40.177963]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1278,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.895,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599823,40.178085]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1279,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.915,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599907,40.178196]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1280,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.934,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599983,40.178291]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1281,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.951,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600052,40.178391]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1282,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.967,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600121,40.178474]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1283,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.982,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600182,40.178562]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1284,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.997,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600243,40.178646]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1285,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.076,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600311,40.178730]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1286,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.177,"distance_from_trace_point":0.645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600365,40.178806]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1287,"matched_point_type":"interpolated","edge_index":75,"distance_along_edge":0.266,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600441,40.178944]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1288,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.420,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600456,40.179054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1289,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.535,"distance_from_trace_point":1.934}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600464,40.179153]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1290,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.637,"distance_from_trace_point":1.289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600441,40.179287]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1291,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.775,"distance_from_trace_point":1.934}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600433,40.179371]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1292,"matched_point_type":"interpolated","edge_index":75,"distance_along_edge":0.862,"distance_from_trace_point":1.289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600418,40.179462]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1293,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.956,"distance_from_trace_point":0.645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600395,40.179562]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1294,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.036,"distance_from_trace_point":0.645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600365,40.179672]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1295,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.105,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600334,40.179802]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1296,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.185,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600296,40.179932]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1297,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.266,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600250,40.180084]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1298,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.362,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600220,40.180252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1299,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.466,"distance_from_trace_point":2.578}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600204,40.180420]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1300,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.568,"distance_from_trace_point":1.289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600189,40.180607]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1301,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.683,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600174,40.180794]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1302,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.797,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600174,40.180973]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1303,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.907,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600182,40.181160]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1304,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.010,"distance_from_trace_point":1.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600182,40.181339]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1305,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.065,"distance_from_trace_point":1.289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600182,40.181526]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1306,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.122,"distance_from_trace_point":1.289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600189,40.181709]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1307,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.178,"distance_from_trace_point":1.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600197,40.181889]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1308,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.233,"distance_from_trace_point":1.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600212,40.182068]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1309,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.288,"distance_from_trace_point":1.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600235,40.182243]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1310,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.342,"distance_from_trace_point":1.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600258,40.182426]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1311,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.398,"distance_from_trace_point":1.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600288,40.182602]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1312,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.452,"distance_from_trace_point":1.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600311,40.182781]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1313,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.507,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600349,40.182953]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1314,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.561,"distance_from_trace_point":1.289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600403,40.183128]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1315,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.616,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600456,40.183304]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1316,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.671,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600517,40.183483]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1317,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.727,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600586,40.183662]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1318,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.784,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600655,40.183849]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1319,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.844,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600731,40.184029]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1320,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.901,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600815,40.184196]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1321,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.956,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600906,40.184368]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1322,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.016,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601006,40.184540]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1323,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.086,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601105,40.184708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1324,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.155,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601196,40.184875]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1325,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.223,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601295,40.185040]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1326,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.291,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601395,40.185207]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1327,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.360,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601486,40.185368]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1328,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.425,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601578,40.185532]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1329,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.492,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601677,40.185696]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1330,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.560,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601761,40.185860]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1331,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.626,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601837,40.186020]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1332,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.690,"distance_from_trace_point":2.712}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601913,40.186184]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1333,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.755,"distance_from_trace_point":4.816}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601997,40.186344]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1334,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.820,"distance_from_trace_point":5.570}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602074,40.186501]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1335,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.882,"distance_from_trace_point":6.923}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602150,40.186657]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1336,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.945,"distance_from_trace_point":8.278}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602234,40.186821]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1337,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.010,"distance_from_trace_point":9.379}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602333,40.186970]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1338,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.064,"distance_from_trace_point":8.615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602432,40.187126]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1339,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.121,"distance_from_trace_point":8.615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602531,40.187279]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1340,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.177,"distance_from_trace_point":8.041}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602631,40.187435]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1341,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.234,"distance_from_trace_point":7.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602730,40.187595]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1342,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.293,"distance_from_trace_point":7.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602837,40.187759]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1343,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.353,"distance_from_trace_point":7.088}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602943,40.187927]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1344,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.414,"distance_from_trace_point":6.508}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603058,40.188095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1345,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.476,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603165,40.188263]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1346,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.537,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603264,40.188427]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1347,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.597,"distance_from_trace_point":6.328}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603371,40.188591]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1348,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.657,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603477,40.188751]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1349,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.716,"distance_from_trace_point":4.980}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603577,40.188911]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1350,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.774,"distance_from_trace_point":4.219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603676,40.189068]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1351,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.831,"distance_from_trace_point":3.637}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603775,40.189217]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1352,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.886,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603867,40.189362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1353,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.939,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603958,40.189510]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1354,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.993,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604050,40.189648]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1355,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.601,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604134,40.189796]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1356,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.010,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604225,40.189941]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1357,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.033,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604309,40.190086]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1358,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.055,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604401,40.190231]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1359,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.078,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604492,40.190384]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1360,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.101,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604584,40.190540]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1361,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.125,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604683,40.190697]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1362,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.149,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604782,40.190861]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1363,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.175,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604881,40.191025]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1364,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.200,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604980,40.191196]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1365,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.226,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605087,40.191368]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1366,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.253,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605194,40.191540]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1367,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.279,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605293,40.191715]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1368,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.306,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605400,40.191891]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1369,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.333,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605507,40.192070]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1370,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.361,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605621,40.192253]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1371,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.389,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605736,40.192436]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1372,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.417,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605850,40.192635]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1373,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.448,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605965,40.192829]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1374,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.477,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606087,40.193027]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1375,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.508,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606209,40.193226]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1376,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.539,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606323,40.193420]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1377,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.569,"distance_from_trace_point":2.712}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606445,40.193615]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1378,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.599,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606560,40.193813]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1379,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.629,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606682,40.194008]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1380,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.659,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606796,40.194206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1381,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.690,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606911,40.194389]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1382,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.718,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607025,40.194576]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1383,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.747,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607132,40.194759]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1384,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.775,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607246,40.194939]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1385,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.803,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607346,40.195114]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1386,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.829,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607452,40.195290]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1387,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.857,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607559,40.195465]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1388,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.884,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607666,40.195641]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1389,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.911,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607773,40.195816]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1390,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.938,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607887,40.196003]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1391,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.967,"distance_from_trace_point":1.540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607994,40.196178]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1392,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.994,"distance_from_trace_point":1.540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608101,40.196354]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1393,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.073,"distance_from_trace_point":1.540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608208,40.196529]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1394,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.168,"distance_from_trace_point":1.540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608315,40.196709]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1395,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.266,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608429,40.196884]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1396,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.362,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608536,40.197067]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1397,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.461,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608650,40.197250]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1398,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.561,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608765,40.197433]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1399,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.661,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608879,40.197617]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1400,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.761,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608986,40.197792]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1401,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.857,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609093,40.197971]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1402,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.954,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609200,40.198147]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1403,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.037,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609314,40.198330]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1404,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.112,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609421,40.198509]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1405,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.185,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609535,40.198692]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1406,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.260,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609650,40.198879]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1407,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.337,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609756,40.199059]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1408,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.410,"distance_from_trace_point":1.540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609871,40.199245]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1409,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.486,"distance_from_trace_point":1.540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609985,40.199432]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1410,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.563,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610092,40.199623]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1411,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.640,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610199,40.199814]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1412,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.716,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610306,40.200001]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1413,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.792,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610413,40.200195]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1414,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.870,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610512,40.200382]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1415,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.945,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610611,40.200577]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1416,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.004,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610695,40.200775]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1417,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.020,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610771,40.200970]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1418,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.035,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610855,40.201164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1419,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.051,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610931,40.201366]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1420,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.067,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611015,40.201572]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1421,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.083,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611092,40.201782]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1422,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.099,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611168,40.201996]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1423,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.116,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611237,40.202202]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1424,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.132,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611305,40.202412]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1425,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.148,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611374,40.202621]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1426,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.164,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611443,40.202820]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1427,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.180,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611504,40.203014]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1428,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.195,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611565,40.203201]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1429,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.209,"distance_from_trace_point":1.978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611626,40.203388]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1430,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.224,"distance_from_trace_point":2.712}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611679,40.203564]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1431,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.237,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611740,40.203743]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1432,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.251,"distance_from_trace_point":2.712}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611794,40.203911]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1433,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.264,"distance_from_trace_point":3.330}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611855,40.204082]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1434,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.278,"distance_from_trace_point":3.330}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611908,40.204250]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1435,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.291,"distance_from_trace_point":3.957}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611961,40.204411]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1436,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.303,"distance_from_trace_point":3.330}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612015,40.204571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1437,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.316,"distance_from_trace_point":3.957}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612061,40.204727]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1438,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.328,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612114,40.204887]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1439,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.340,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612167,40.205048]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1440,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.352,"distance_from_trace_point":5.307}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612221,40.205219]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1441,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.366,"distance_from_trace_point":5.935}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612282,40.205395]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1442,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.379,"distance_from_trace_point":5.307}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612335,40.205566]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1443,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.393,"distance_from_trace_point":5.935}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612389,40.205738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1444,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.406,"distance_from_trace_point":5.935}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612450,40.205921]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1445,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.420,"distance_from_trace_point":6.660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612511,40.206100]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1446,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.434,"distance_from_trace_point":6.660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612572,40.206284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1447,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.448,"distance_from_trace_point":6.039}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612625,40.206463]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1448,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.462,"distance_from_trace_point":6.660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612686,40.206638]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1449,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.476,"distance_from_trace_point":7.285}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612740,40.206818]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1450,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.490,"distance_from_trace_point":7.913}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612801,40.207001]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1451,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.504,"distance_from_trace_point":8.637}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612862,40.207180]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1452,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.518,"distance_from_trace_point":8.637}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612923,40.207363]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1453,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.532,"distance_from_trace_point":8.637}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612999,40.207550]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1454,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.547,"distance_from_trace_point":8.275}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613091,40.207726]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1455,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.561,"distance_from_trace_point":5.569}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613174,40.207901]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1456,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.575,"distance_from_trace_point":4.067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613258,40.208076]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1457,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.589,"distance_from_trace_point":1.978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613335,40.208256]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1458,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.603,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613396,40.208439]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1459,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.617,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613464,40.208622]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1460,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.632,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613533,40.208805]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1461,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.646,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613602,40.208992]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1462,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.661,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613663,40.209183]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1463,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.675,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613724,40.209370]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1464,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.690,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613785,40.209564]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1465,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.705,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613846,40.209751]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1466,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.719,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613914,40.209938]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1467,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.734,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613983,40.210114]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1468,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.748,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614059,40.210297]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1469,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.762,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614136,40.210476]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1470,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.776,"distance_from_trace_point":2.711}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614204,40.210651]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1471,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.790,"distance_from_trace_point":3.461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614281,40.210827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1472,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.804,"distance_from_trace_point":4.067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614349,40.210999]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1473,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.817,"distance_from_trace_point":4.067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614426,40.211174]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1474,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.831,"distance_from_trace_point":4.067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614494,40.211349]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1475,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.845,"distance_from_trace_point":4.067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614571,40.211525]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1476,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.859,"distance_from_trace_point":4.067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614639,40.211697]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1477,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.873,"distance_from_trace_point":2.711}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614723,40.211853]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1478,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.885,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614815,40.212013]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1479,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.898,"distance_from_trace_point":3.636}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614914,40.212173]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1480,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.912,"distance_from_trace_point":3.461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615005,40.212334]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1481,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.925,"distance_from_trace_point":3.461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615097,40.212490]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1482,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.938,"distance_from_trace_point":3.636}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615196,40.212658]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1483,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.952,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615295,40.212826]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1484,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.965,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615395,40.212994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1485,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.979,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615494,40.213165]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1486,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.993,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615616,40.213333]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1487,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.009,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615753,40.213486]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1488,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.025,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615891,40.213638]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1489,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.040,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616028,40.213787]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1490,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.056,"distance_from_trace_point":1.540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616165,40.213932]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1491,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.071,"distance_from_trace_point":2.310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616295,40.214073]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1492,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.086,"distance_from_trace_point":3.080}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616432,40.214218]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1493,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.101,"distance_from_trace_point":4.096}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616570,40.214363]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1494,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.116,"distance_from_trace_point":4.096}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616707,40.214512]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1495,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.132,"distance_from_trace_point":4.864}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616852,40.214664]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1496,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.148,"distance_from_trace_point":5.390}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616997,40.214821]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1497,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.164,"distance_from_trace_point":6.160}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617134,40.214985]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1498,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.180,"distance_from_trace_point":5.741}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617241,40.215153]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1499,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.196,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617348,40.215328]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1500,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.213,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617447,40.215500]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1501,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.229,"distance_from_trace_point":3.460}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617561,40.215675]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1502,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.245,"distance_from_trace_point":4.978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617668,40.215851]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1503,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.262,"distance_from_trace_point":7.086}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617775,40.216026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1504,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.278,"distance_from_trace_point":7.848}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617882,40.216198]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1505,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.294,"distance_from_trace_point":8.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617989,40.216366]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1506,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.310,"distance_from_trace_point":7.848}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618103,40.216545]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1507,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.327,"distance_from_trace_point":7.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618210,40.216713]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1508,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.343,"distance_from_trace_point":5.741}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618317,40.216888]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1509,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.359,"distance_from_trace_point":3.636}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618423,40.217060]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1510,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.376,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618530,40.217236]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1511,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.392,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618591,40.217426]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1512,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.409,"distance_from_trace_point":1.356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618629,40.217628]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1513,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.426,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618668,40.217823]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1514,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.443,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618706,40.218018]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1515,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.460,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618706,40.218216]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1516,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.477,"distance_from_trace_point":7.085}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618698,40.218407]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1517,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.493,"distance_from_trace_point":11.602}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618690,40.218605]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1518,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.510,"distance_from_trace_point":15.465}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618690,40.218803]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1519,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.527,"distance_from_trace_point":17.397}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618683,40.219009]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1520,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.544,"distance_from_trace_point":18.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618675,40.219215]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1521,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.562,"distance_from_trace_point":19.972}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618668,40.219433]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1522,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.580,"distance_from_trace_point":19.972}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618660,40.219639]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1523,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.598,"distance_from_trace_point":19.328}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618652,40.219860]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1524,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.616,"distance_from_trace_point":18.040}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618645,40.220062]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1525,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.634,"distance_from_trace_point":16.108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618637,40.220272]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1526,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.651,"distance_from_trace_point":13.533}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618629,40.220478]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1527,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.669,"distance_from_trace_point":10.314}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618622,40.220695]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1528,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.687,"distance_from_trace_point":7.729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618614,40.220909]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1529,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.706,"distance_from_trace_point":5.797}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618607,40.221115]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1530,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.723,"distance_from_trace_point":3.865}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618599,40.221321]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1531,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.741,"distance_from_trace_point":1.932}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618614,40.221523]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1532,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.758,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618637,40.221718]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1533,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.774,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618660,40.221909]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1534,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.791,"distance_from_trace_point":5.812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618683,40.222095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1535,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.807,"distance_from_trace_point":7.098}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618706,40.222275]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1536,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.822,"distance_from_trace_point":7.775}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618721,40.222443]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1537,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.836,"distance_from_trace_point":7.283}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618774,40.222618]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1538,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.851,"distance_from_trace_point":9.174}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618820,40.222778]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1539,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.865,"distance_from_trace_point":9.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618858,40.222931]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1540,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.879,"distance_from_trace_point":9.261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618896,40.223064]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1541,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.890,"distance_from_trace_point":9.261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618935,40.223190]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1542,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.901,"distance_from_trace_point":9.261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618973,40.223320]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1543,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.912,"distance_from_trace_point":9.174}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619003,40.223431]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1544,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.922,"distance_from_trace_point":8.541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619034,40.223534]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1545,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.931,"distance_from_trace_point":7.283}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619057,40.223633]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1546,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.940,"distance_from_trace_point":5.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619080,40.223717]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1547,"matched_point_type":"interpolated","edge_index":86,"distance_along_edge":0.947,"distance_from_trace_point":5.306}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619125,40.223858]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1548,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.959,"distance_from_trace_point":5.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619148,40.223942]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1549,"matched_point_type":"interpolated","edge_index":86,"distance_along_edge":0.967,"distance_from_trace_point":3.955}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619171,40.224037]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1550,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.975,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619202,40.224136]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1551,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.983,"distance_from_trace_point":1.978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619240,40.224281]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1552,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.996,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619270,40.224365]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1553,"matched_point_type":"interpolated","edge_index":87,"distance_along_edge":0.012,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619301,40.224445]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1554,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.037,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619339,40.224541]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1555,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.067,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619377,40.224636]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1556,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.097,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619415,40.224739]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1557,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.129,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619461,40.224850]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1558,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.163,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619507,40.224964]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1559,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.199,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619553,40.225082]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1560,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.236,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619598,40.225204]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1561,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.274,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619644,40.225330]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1562,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.313,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619698,40.225464]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1563,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.355,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619751,40.225601]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1564,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.398,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619804,40.225739]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1565,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.440,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619858,40.225876]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1566,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.483,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619911,40.226013]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1567,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.526,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619972,40.226154]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1568,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.570,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620026,40.226307]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1569,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.618,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620087,40.226463]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1570,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.667,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620155,40.226624]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1571,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.717,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620216,40.226788]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1572,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.768,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620285,40.226955]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1573,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.820,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620346,40.227127]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1574,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.874,"distance_from_trace_point":1.978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620415,40.227303]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1575,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.928,"distance_from_trace_point":1.978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620483,40.227482]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1576,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.984,"distance_from_trace_point":1.978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620560,40.227661]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1577,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.010,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620628,40.227844]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1578,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.023,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620697,40.228027]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1579,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.036,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620766,40.228207]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1580,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.049,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620834,40.228378]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1581,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.062,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620903,40.228558]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1582,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.075,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620979,40.228748]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1583,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.089,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621048,40.228935]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1584,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.103,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621117,40.229122]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1585,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.116,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621193,40.229313]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1586,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.130,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621269,40.229511]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1587,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.145,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621346,40.229713]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1588,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.160,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621422,40.229912]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1589,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.174,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621490,40.230110]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1590,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.189,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621567,40.230309]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1591,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.203,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621643,40.230507]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1592,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.218,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621712,40.230690]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1593,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.231,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621780,40.230869]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1594,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.244,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621849,40.231049]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1595,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.257,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621910,40.231220]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1596,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.270,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621971,40.231384]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1597,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.282,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622032,40.231533]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1598,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.293,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622086,40.231670]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1599,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.303,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622131,40.231792]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1600,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.312,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622177,40.231911]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1601,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.320,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622223,40.232021]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1602,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.329,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622269,40.232132]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1603,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.337,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622314,40.232246]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1604,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.345,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622368,40.232372]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1605,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.354,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622414,40.232510]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1606,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.364,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622467,40.232651]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1607,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.375,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622528,40.232800]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1608,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.386,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622581,40.232956]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1609,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.397,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622643,40.233116]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1610,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.409,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622704,40.233276]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1611,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.420,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622772,40.233444]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1612,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.433,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622841,40.233616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1613,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.445,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622910,40.233784]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1614,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.458,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622971,40.233955]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1615,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.470,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623032,40.234119]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1616,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.482,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623085,40.234299]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1617,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.495,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623154,40.234470]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1618,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.508,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623222,40.234642]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1619,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.520,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623283,40.234818]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1620,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.533,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623352,40.234993]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1621,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.546,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623421,40.235168]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1622,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.559,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623489,40.235355]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1623,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.572,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623558,40.235538]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1624,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.586,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623627,40.235722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1625,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.599,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623695,40.235909]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1626,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.613,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623772,40.236095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1627,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.627,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623840,40.236286]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1628,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.641,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623917,40.236477]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1629,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.655,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623993,40.236671]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1630,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.669,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624062,40.236862]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1631,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.683,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624138,40.237057]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1632,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.697,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624214,40.237247]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1633,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.711,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624283,40.237438]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1634,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.725,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624359,40.237637]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1635,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.739,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624435,40.237827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1636,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.753,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624512,40.238026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1637,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.768,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624596,40.238224]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1638,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.783,"distance_from_trace_point":2.108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624672,40.238422]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1639,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.797,"distance_from_trace_point":2.710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624748,40.238625]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1640,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.812,"distance_from_trace_point":2.710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624825,40.238831]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1641,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.827,"distance_from_trace_point":2.710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624893,40.239037]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1642,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.842,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624969,40.239239]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1643,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.857,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625046,40.239445]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1644,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.872,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625122,40.239655]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1645,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.887,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625191,40.239861]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1646,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.902,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625259,40.240070]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1647,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.917,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625328,40.240292]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1648,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.933,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625381,40.240517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1649,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.949,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625435,40.240746]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1650,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.965,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625473,40.240971]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1651,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.981,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625504,40.241192]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1652,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.997,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625519,40.241409]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1653,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.047,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625534,40.241631]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1654,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.107,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625549,40.241856]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1655,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.168,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625549,40.242085]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1656,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.230,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625557,40.242313]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1657,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.292,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625557,40.242550]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1658,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.355,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625565,40.242783]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1659,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.418,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625565,40.243015]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1660,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.481,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625572,40.243244]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1661,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.543,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625587,40.243477]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1662,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.606,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625595,40.243706]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1663,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.667,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625595,40.243931]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1664,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.728,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625603,40.244148]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1665,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.787,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625610,40.244377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1666,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.849,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625618,40.244602]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1667,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.909,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625633,40.244831]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1668,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.971,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625641,40.245064]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1669,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.081,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625648,40.245293]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1670,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.228,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625648,40.245522]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1671,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.376,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625664,40.245743]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1672,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.518,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625671,40.245964]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1673,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.661,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625687,40.246185]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1674,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.804,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625694,40.246403]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1675,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.944,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625694,40.246616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1676,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.031,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625694,40.246826]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1677,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.082,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625694,40.247032]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1678,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.132,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625702,40.247242]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1679,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.183,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625710,40.247448]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1680,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.234,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625717,40.247654]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1681,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.284,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625725,40.247868]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1682,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.336,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625732,40.248070]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1683,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.386,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625740,40.248280]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1684,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.437,"distance_from_trace_point":1.932}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625771,40.248497]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1685,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.490,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625824,40.248707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1686,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.543,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625870,40.248928]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1687,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.597,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625954,40.249142]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1688,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.652,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626038,40.249352]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1689,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.705,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626122,40.249561]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1690,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.759,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626221,40.249775]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1691,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.814,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626335,40.249981]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1692,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.869,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626472,40.250183]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1693,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.924,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626617,40.250378]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1694,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.979,"distance_from_trace_point":2.869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626740,40.250576]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1695,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.059,"distance_from_trace_point":3.634}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626862,40.250786]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1696,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.160,"distance_from_trace_point":3.459}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626976,40.250988]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1697,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.258,"distance_from_trace_point":2.869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627090,40.251190]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1698,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.355,"distance_from_trace_point":2.869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627205,40.251385]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1699,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.449,"distance_from_trace_point":2.869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627319,40.251587]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1700,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.547,"distance_from_trace_point":2.108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627434,40.251789]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1701,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.644,"distance_from_trace_point":2.108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627548,40.251987]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1702,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.740,"distance_from_trace_point":2.108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627663,40.252182]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1703,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.834,"distance_from_trace_point":2.108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627777,40.252377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1704,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.928,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627884,40.252567]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1705,"matched_point_type":"matched","edge_index":93,"distance_along_edge":0.091,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627975,40.252758]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1706,"matched_point_type":"matched","edge_index":93,"distance_along_edge":0.496,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628082,40.252945]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1707,"matched_point_type":"matched","edge_index":93,"distance_along_edge":0.902,"distance_from_trace_point":2.108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628212,40.253120]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1708,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.014,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628342,40.253288]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1709,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.032,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628471,40.253456]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1710,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.050,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628609,40.253628]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1711,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.068,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628777,40.253784]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1712,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.086,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628944,40.253937]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1713,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.105,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629112,40.254093]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1714,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.123,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629288,40.254246]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1715,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.142,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629471,40.254395]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1716,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.160,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629662,40.254536]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1717,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.179,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629860,40.254665]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1718,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.197,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630074,40.254795]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1719,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.216,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630287,40.254917]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1720,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.235,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630501,40.255035]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1721,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.253,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630722,40.255131]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1722,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.271,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630943,40.255230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1723,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.289,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631157,40.255322]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1724,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.306,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631363,40.255413]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1725,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.322,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631561,40.255501]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1726,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.338,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631752,40.255585]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1727,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.354,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631935,40.255665]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1728,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.368,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632118,40.255745]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1729,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.383,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632294,40.255825]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1730,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.397,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632469,40.255901]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1731,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.411,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632645,40.255978]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1732,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.426,"distance_from_trace_point":5.306}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632812,40.256046]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1733,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.439,"distance_from_trace_point":5.679}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632980,40.256119]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1734,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.452,"distance_from_trace_point":5.998}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633141,40.256187]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1735,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.465,"distance_from_trace_point":5.306}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633316,40.256260]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1736,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.479,"distance_from_trace_point":4.942}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633492,40.256336]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1737,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.493,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633659,40.256416]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1738,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.507,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633820,40.256504]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1739,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.521,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633987,40.256592]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1740,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.535,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634140,40.256699]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1741,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.549,"distance_from_trace_point":2.860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634293,40.256813]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1742,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.564,"distance_from_trace_point":2.564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634430,40.256931]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1743,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.579,"distance_from_trace_point":2.860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634567,40.257069]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1744,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.594,"distance_from_trace_point":2.309}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634689,40.257202]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1745,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.609,"distance_from_trace_point":2.309}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634819,40.257339]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1746,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.625,"distance_from_trace_point":2.309}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634941,40.257465]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1747,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.639,"distance_from_trace_point":2.564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635063,40.257576]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1748,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.652,"distance_from_trace_point":3.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635193,40.257690]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1749,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.666,"distance_from_trace_point":3.328}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635323,40.257793]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1750,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.679,"distance_from_trace_point":2.564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635460,40.257900]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1751,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.693,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635590,40.257996]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1752,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.705,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635719,40.258099]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1753,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.718,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635857,40.258198]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1754,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.732,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635994,40.258305]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1755,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.745,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636131,40.258411]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1756,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.759,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636276,40.258518]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1757,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.773,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636429,40.258625]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1758,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.787,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636581,40.258728]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1759,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.802,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636726,40.258835]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1760,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.816,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636879,40.258938]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1761,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.830,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637016,40.259033]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1762,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.843,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637154,40.259132]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1763,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.856,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637291,40.259228]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1764,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.869,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637436,40.259327]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1765,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.883,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637566,40.259418]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1766,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.895,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637695,40.259506]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1767,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.907,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637810,40.259590]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1768,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.918,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637932,40.259670]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1769,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.929,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638046,40.259750]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1770,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.940,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638153,40.259827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1771,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.950,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638275,40.259911]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1772,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.962,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638405,40.260002]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1773,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.974,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638535,40.260101]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1774,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.987,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638680,40.260208]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1775,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.009,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638840,40.260323]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1776,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.167,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638992,40.260433]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1777,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.318,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639145,40.260544]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1778,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.469,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639305,40.260658]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1779,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.627,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639465,40.260777]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1780,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.787,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639603,40.260895]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1781,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.936,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639740,40.261021]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1782,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.084,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639870,40.261147]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1783,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.224,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639999,40.261280]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1784,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.370,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640137,40.261414]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1785,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.519,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640274,40.261555]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1786,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.673,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640411,40.261700]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1787,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.829,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640541,40.261848]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1788,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.986,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640686,40.261990]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1789,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.101,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640831,40.262127]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1790,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.210,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640968,40.262268]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1791,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.319,"distance_from_trace_point":2.309}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641098,40.262413]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1792,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.428,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641228,40.262558]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1793,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.537,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641365,40.262711]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1794,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.652,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641502,40.262856]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1795,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.763,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641632,40.263000]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1796,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.871,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641762,40.263145]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1797,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.980,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641891,40.263287]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1798,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.151,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642021,40.263428]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1799,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.336,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642143,40.263569]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1800,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.517,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642273,40.263706]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1801,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.699,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642395,40.263844]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1802,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.877,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642525,40.263985]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1803,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.051,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642654,40.264111]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1804,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.192,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642784,40.264244]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1805,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.339,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642914,40.264370]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1806,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.480,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643044,40.264496]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1807,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.621,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643166,40.264614]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1808,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.754,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643288,40.264732]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1809,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.887,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643402,40.264851]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1810,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.024,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643524,40.264954]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1811,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.196,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643661,40.265060]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1812,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.381,"distance_from_trace_point":2.564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643799,40.265175]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1813,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.573,"distance_from_trace_point":2.564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643944,40.265270]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1814,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.756,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644112,40.265373]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1815,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.960,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644272,40.265472]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1816,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.125,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644440,40.265575]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1817,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.290,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644592,40.265671]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1818,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.441,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644753,40.265766]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1819,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.596,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644905,40.265862]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1820,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.747,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645065,40.265957]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1821,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.903,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645233,40.266056]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1822,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.027,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645409,40.266151]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1823,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.094,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645584,40.266251]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1824,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.162,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645760,40.266350]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1825,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.230,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645935,40.266449]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1826,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.298,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646118,40.266552]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1827,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.369,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646278,40.266640]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1828,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.431,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646439,40.266731]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1829,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.493,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646576,40.266804]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1830,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.545,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646706,40.266880]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1831,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.597,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646835,40.266949]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1832,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.646,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646957,40.267017]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1833,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.693,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647087,40.267090]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1834,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.743,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647217,40.267162]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1835,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.794,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647369,40.267250]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1836,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.853,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647522,40.267330]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1837,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.911,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647682,40.267422]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1838,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.973,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647858,40.267521]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1839,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.126,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648018,40.267616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1840,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.319,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648178,40.267708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1841,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.509,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648331,40.267799]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1842,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.693,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648499,40.267895]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1843,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.891,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648651,40.267979]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1844,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.147,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648819,40.268063]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1845,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.552,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648972,40.268143]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1846,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.926,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649117,40.268219]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1847,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.019,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649246,40.268295]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1848,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.041,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649384,40.268372]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1849,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.063,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649513,40.268444]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1850,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.085,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649651,40.268520]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1851,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.108,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649796,40.268600]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1852,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.132,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649925,40.268673]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1853,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.153,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650063,40.268749]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1854,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.176,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650200,40.268826]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1855,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.199,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650337,40.268902]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1856,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.222,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650467,40.268974]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1857,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.243,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650612,40.269054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1858,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.267,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650749,40.269131]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1859,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.290,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650887,40.269211]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1860,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.313,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651031,40.269287]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1861,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.337,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651176,40.269371]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1862,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.361,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651321,40.269451]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1863,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.385,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651466,40.269531]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1864,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.409,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651611,40.269615]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1865,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.434,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651764,40.269699]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1866,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.459,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651909,40.269779]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1867,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.483,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652069,40.269867]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1868,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.509,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652222,40.269955]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1869,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.535,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652374,40.270039]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1870,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.560,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652527,40.270123]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1871,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.586,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652672,40.270206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1872,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.610,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652824,40.270290]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1873,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.635,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652962,40.270367]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1874,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.658,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653107,40.270447]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1875,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.682,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653252,40.270531]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1876,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.706,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653412,40.270618]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1877,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.733,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653564,40.270702]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1878,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.758,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653709,40.270786]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1879,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.783,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653862,40.270866]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1880,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.807,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654007,40.270950]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1881,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.832,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654160,40.271034]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1882,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.857,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654297,40.271111]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1883,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.880,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654434,40.271191]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1884,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.903,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654572,40.271267]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1885,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.926,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654701,40.271343]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1886,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.948,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654831,40.271427]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1887,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.971,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654938,40.271507]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1888,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.991,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655045,40.271591]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1889,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.086,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655136,40.271667]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1890,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.225,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655235,40.271782]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1891,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.410,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655281,40.271862]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1892,"matched_point_type":"interpolated","edge_index":106,"distance_along_edge":0.527,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655327,40.271942]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1893,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.644,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655373,40.272030]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1894,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.771,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655426,40.272144]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1895,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.934,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655472,40.272224]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1896,"matched_point_type":"interpolated","edge_index":107,"distance_along_edge":0.040,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655510,40.272305]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1897,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.130,"distance_from_trace_point":1.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655556,40.272385]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1898,"matched_point_type":"interpolated","edge_index":107,"distance_along_edge":0.221,"distance_from_trace_point":2.107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655602,40.272461]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1899,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.309,"distance_from_trace_point":3.458}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655678,40.272602]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1900,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.469,"distance_from_trace_point":3.458}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655739,40.272705]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1901,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.588,"distance_from_trace_point":4.214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655785,40.272785]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1902,"matched_point_type":"interpolated","edge_index":107,"distance_along_edge":0.680,"distance_from_trace_point":4.214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655823,40.272861]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1903,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.765,"distance_from_trace_point":4.214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655869,40.272949]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1904,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.864,"distance_from_trace_point":3.633}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655930,40.273060]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1905,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.990,"distance_from_trace_point":2.107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656075,40.273151]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1906,"matched_point_type":"matched","edge_index":108,"distance_along_edge":0.178,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656158,40.273205]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1907,"matched_point_type":"interpolated","edge_index":108,"distance_along_edge":0.289,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656258,40.273262]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1908,"matched_point_type":"matched","edge_index":108,"distance_along_edge":0.415,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656364,40.273327]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1909,"matched_point_type":"matched","edge_index":108,"distance_along_edge":0.554,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656479,40.273399]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1910,"matched_point_type":"matched","edge_index":108,"distance_along_edge":0.705,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656601,40.273472]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1911,"matched_point_type":"matched","edge_index":108,"distance_along_edge":0.862,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656738,40.273556]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1912,"matched_point_type":"matched","edge_index":109,"distance_along_edge":0.105,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656876,40.273643]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1913,"matched_point_type":"matched","edge_index":109,"distance_along_edge":0.581,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657021,40.273731]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1914,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.007,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657158,40.273819]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1915,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.050,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657303,40.273911]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1916,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.096,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657440,40.273998]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1917,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.140,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657578,40.274082]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1918,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.183,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657707,40.274166]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1919,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.224,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657852,40.274254]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1920,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.270,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657990,40.274342]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1921,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.313,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658134,40.274429]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1922,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.358,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658264,40.274517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1923,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.401,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658401,40.274597]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1924,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.443,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658539,40.274685]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1925,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.487,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658684,40.274773]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1926,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.532,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658829,40.274864]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1927,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.578,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658974,40.274956]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1928,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.624,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659111,40.275043]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1929,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.667,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659248,40.275127]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1930,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.710,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659393,40.275223]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1931,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.757,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659546,40.275318]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1932,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.805,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659706,40.275417]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1933,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.856,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659866,40.275517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1934,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.906,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660027,40.275620]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1935,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.957,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660187,40.275719]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1936,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.012,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660339,40.275814]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1937,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.087,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660500,40.275921]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1938,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.169,"distance_from_trace_point":4.244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660645,40.276012]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1939,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.242,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660789,40.276104]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1940,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.314,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660934,40.276196]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1941,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.386,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661064,40.276279]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1942,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.452,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661201,40.276367]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1943,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.520,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661331,40.276455]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1944,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.587,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661469,40.276539]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1945,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.655,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661606,40.276630]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1946,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.725,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661751,40.276722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1947,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.797,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661903,40.276821]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1948,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.874,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662048,40.276913]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1949,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.946,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662209,40.277016]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1950,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.081,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662361,40.277107]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1951,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.307,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662506,40.277203]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1952,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.530,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662651,40.277294]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1953,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.749,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662788,40.277378]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1954,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.954,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662926,40.277458]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1955,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.113,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663048,40.277531]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1956,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.244,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663177,40.277603]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1957,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.379,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663315,40.277683]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1958,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.526,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663445,40.277756]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1959,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.662,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663567,40.277824]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1960,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.790,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663673,40.277885]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1961,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.903,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663765,40.277939]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1962,"matched_point_type":"interpolated","edge_index":113,"distance_along_edge":1.000,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663849,40.277988]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1963,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.143,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663940,40.278038]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1964,"matched_point_type":"interpolated","edge_index":114,"distance_along_edge":0.294,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664024,40.278084]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1965,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.433,"distance_from_trace_point":5.305}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664177,40.278175]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1966,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.694,"distance_from_trace_point":5.305}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664307,40.278343]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1967,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.052,"distance_from_trace_point":7.695}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664215,40.278446]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1968,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.138,"distance_from_trace_point":3.078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664154,40.278515]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1969,"matched_point_type":"interpolated","edge_index":116,"distance_along_edge":0.194,"distance_from_trace_point":2.308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664085,40.278591]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1970,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.258,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664001,40.278679]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1971,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.332,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663918,40.278770]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1972,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.408,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663826,40.278881]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1973,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.498,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663727,40.278988]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1974,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.587,"distance_from_trace_point":2.308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663635,40.279099]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1975,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.677,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663536,40.279205]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1976,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.766,"distance_from_trace_point":2.308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663437,40.279312]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1977,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.856,"distance_from_trace_point":3.078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663338,40.279430]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1978,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.952,"distance_from_trace_point":3.078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663231,40.279545]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1979,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.053,"distance_from_trace_point":3.078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663132,40.279667]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1980,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.161,"distance_from_trace_point":3.078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663025,40.279789]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1981,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.272,"distance_from_trace_point":3.078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662918,40.279922]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1982,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.389,"distance_from_trace_point":3.078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662804,40.280048]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1983,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.505,"distance_from_trace_point":3.078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662697,40.280174]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1984,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.618,"distance_from_trace_point":2.308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662582,40.280300]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1985,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.733,"distance_from_trace_point":2.308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662468,40.280430]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1986,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.851,"distance_from_trace_point":2.308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662361,40.280556]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1987,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.964,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662231,40.280670]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1988,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.046,"distance_from_trace_point":3.328}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662109,40.280785]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1989,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.112,"distance_from_trace_point":4.094}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661980,40.280895]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1990,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.178,"distance_from_trace_point":4.094}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661850,40.281002]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1991,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.243,"distance_from_trace_point":3.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661720,40.281105]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1992,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.307,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661575,40.281197]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1993,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.371,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661430,40.281281]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1994,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.433,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661285,40.281364]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1995,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.495,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661133,40.281433]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1996,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.556,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660965,40.281509]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1997,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.622,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660797,40.281574]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1998,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.686,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660629,40.281639]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1999,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.750,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660454,40.281708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2000,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.817,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660278,40.281788]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2001,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.887,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660126,40.281868]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2002,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.950,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659943,40.281963]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2003,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.045,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659775,40.282024]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2004,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.155,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659615,40.282085]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2005,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.260,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659454,40.282146]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2006,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.366,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659286,40.282207]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2007,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.475,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659111,40.282272]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2008,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.590,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658936,40.282337]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2009,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.705,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658760,40.282402]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2010,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.820,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658577,40.282471]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2011,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.940,"distance_from_trace_point":3.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658401,40.282536]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2012,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.118,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658226,40.282604]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2013,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.367,"distance_from_trace_point":3.887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658051,40.282669]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2014,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.613,"distance_from_trace_point":3.887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657867,40.282738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2015,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.871,"distance_from_trace_point":3.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657684,40.282806]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2016,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.068,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657509,40.282871]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2017,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.198,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657326,40.282936]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2018,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.332,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657158,40.283001]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2019,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.458,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656998,40.283062]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2020,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.577,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656837,40.283119]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2021,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.695,"distance_from_trace_point":3.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656685,40.283176]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2022,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.808,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656532,40.283230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2023,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.920,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656395,40.283283]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2024,"matched_point_type":"matched","edge_index":122,"distance_along_edge":0.071,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656250,40.283340]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2025,"matched_point_type":"matched","edge_index":122,"distance_along_edge":0.412,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656105,40.283394]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2026,"matched_point_type":"matched","edge_index":122,"distance_along_edge":0.749,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655960,40.283451]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2027,"matched_point_type":"matched","edge_index":123,"distance_along_edge":0.068,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655815,40.283501]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2028,"matched_point_type":"matched","edge_index":123,"distance_along_edge":0.318,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655678,40.283554]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2029,"matched_point_type":"matched","edge_index":123,"distance_along_edge":0.561,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655540,40.283604]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2030,"matched_point_type":"matched","edge_index":123,"distance_along_edge":0.800,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655403,40.283653]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2031,"matched_point_type":"matched","edge_index":124,"distance_along_edge":0.047,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655266,40.283707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2032,"matched_point_type":"matched","edge_index":124,"distance_along_edge":0.339,"distance_from_trace_point":3.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655151,40.283749]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2033,"matched_point_type":"matched","edge_index":124,"distance_along_edge":0.579,"distance_from_trace_point":3.887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655029,40.283794]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2034,"matched_point_type":"matched","edge_index":124,"distance_along_edge":0.837,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654861,40.283859]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2035,"matched_point_type":"matched","edge_index":125,"distance_along_edge":0.143,"distance_from_trace_point":3.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654739,40.283905]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2036,"matched_point_type":"matched","edge_index":125,"distance_along_edge":0.334,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654633,40.283943]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":2037,"matched_point_type":"interpolated","edge_index":125,"distance_along_edge":0.500,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654633,40.283943]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2038,"matched_point_type":"matched","edge_index":125,"distance_along_edge":0.500,"distance_from_trace_point":2.205}}
]}
 */
void TestAutoLancasterToHershey() {
  std::string route_json_str =
      R"({"trip":{"language":"en-US","summary":{"max_lon":-76.266228,"max_lat":40.283943,"time":2438,"length":31.322,"min_lat":40.042015,"min_lon":-76.664360},"locations":[{"lon":-76.299179,"lat":40.042572,"type":"break"},{"lon":-76.654625,"lat":40.283924,"type":"break"}],"units":"miles","legs":[{"shape":"k`_kkAfy|opC}@_SsFk`A~g@cFeDmt@{FkiAkLisBmJcxBsKcxBaHytAWaIqBm^u@wNsAo]yAyMsAqGyBsFiCeEeZs[iBiBsA{AgDyC{QsPaMiMcFaH}@yA{@{AgDoIgDyLU}@eAcFcAmJm@oIe@_JiBwXOaGm@mKu@{K{@oHcBkLgCeOiC}JoDyLaCaIuDgMeF}J}IuOaMqRcLwNgDcFiHcGoNkLke@y`@uDyCu@MeEgDuOqHsKuEuEiBcu@gX_EkB{~@a\\}JeEc[yM}IgDoNcFsFyBcGgDqGeEiHcG{FcGcGoHmEqHsEmJoD}JqC{KaCiLyAkLeKo|@aCsPaCuOoCuO_DePgI_^uy@ejDaGiWoTi`AkKgc@mZapAaWucAmEzAcB\\iBl@qLdFqXzKkFvCuEtDmDtEyCrFoCpH{A|Iu@lJEnJj@zJbBlJ`C`HhHrQtD|JxBnIbBhLt@pGd@~IDpHElJe@zJsA|JaCzKqCnIsZ~r@{KrZow@nfB}JhW{Vzi@cVbf@sZjk@s`@`q@{iArlB}c@ju@iHxLoHhMg}@`{AiWx`@_r@jjAya@~q@ePxWcGxLwCdF_r@bnAkLzUkLhWkFjLgSli@aHjVkQzt@wCdO{Fb\\cF~]iCjUeFxk@{@dPkAhWu@tZe@`\\Nz_Ad@fkAd@liAGxgCkAxkAoDhjA{F`fA{F|s@mEdn@gb@znFyHhjAaGl}@qRjhCuUjeDaRpyByBdZgNhtB{@z_@cBfc@cBdn@eEvbA{FdwA{@nScBtOgCnTaDbQs@rF_DhWcGtc@aXh_BwC`RcB|JoM`q@yHb[kFlU}Jj`@aG|TiHzVaHzTiHpSwNx`@}IxW}bAhrCqW`r@mPbe@cK`]gIlT_Oha@gD~IgNz_@mx@n{Bi\\z~@yRli@i`AbmC}EtNoHlUqMvb@}Jl_@qGzUkB~HeEbQgI|_@wHx`@{AlKeOpeAcBvNaH`p@iCtZqB~SgDdc@aCvXcUpnCeAvMiGju@gYjqDaCzUmEjj@mc@ffFoIpeAgNx_B_JjiAoH~|@aHpq@oDfYuJ|r@kFb[yHja@uIxa@kGfXuTrz@uJp[cQ|i@erAjpDsGtOsEjL_Yfw@_Oz_@s@zB{Ll^s@hCoSzi@gEjLiGdPwm@pdB{d@rnAaSjj@oNj`@oMb\\iRzi@{Lb\\{JfXakApbDyWls@ib@hjAed@dnAgg@huA_Tvm@qMfb@wNli@iLde@m@fCgNrp@eJ`g@}EtYoIro@aHhk@G|@_Dp\\oDbe@}Dre@yBb\\oSdkCcL~zAeOtlB_J|hA_Idx@iCzUwH~h@iCtOoNvw@aMvl@qGfXwI`]oHxWmFpQ]jA_Srp@uUhv@iL~]mZbdAwIr[u^dmAgS~q@ke@|}A__@`oAeOrf@aWvw@sKz_@_JdZgDxLgCzK{nAldEwqA|mEiRhl@{K~]_Ozi@mc@pzAenA~bEeEfN{F`R{e@x_B{EvN}Y~{@gOz`@uc@jiAe~@~yBmO~^s`@~eAeKzViQdd@cRbe@aa@tcAscBdiEkLdYyVxl@_IpR{Qre@y[by@sVbp@y{@rxB}m@l|AmTjk@sxBdrFqQdc@eKxWs`@dcA{KhXmOz_@w|@nzBei@jtAya@rcAkBdE{EzK}_@xaA}Ovb@erAjfDcKvWyH|TqQ|i@gItYcQ`q@wI|^cGfYuJ|h@aG`]_J`q@{Ftd@sFli@aBnSaC`\\wCrf@m@vMu@`Se@nScAz_@u@li@OlUN~]G~\\NbQbApg@hCby@pB|_@vRx|CjBtYlEdo@pGbeAnDli@hCvb@vHzhAtJj~AxG`fApChb@bArPrArPj[naFzj@rzInCjj@|@|TbAl_@l@l_@TlTNfc@?z`@e@tYcBz~@kAtYaBn^oDfm@iCdZiHxu@gw@jwGor@n`G_TpdB{o@~uFuTjiBsPtwAiGrd@{F~^oIbe@oD`RwIha@eJl`@_N~f@oIvYyMxa@yL~\\sPvc@wNn]{P|^{Pn]wX`g@yQtZ{QdYqoFjaIqMpRaoAfkBoXvb@cRvXaGlJ{hB`mCwNnTs_@hk@wqArnB{z@rnAopApnBe_@|i@a]~g@_StYaMrQusA`nB_vApoBckIrlLcyAhtB_Yj`@g]`f@md@to@gYx`@gpAxiBqaAlsAqG|Iom@p{@ioB|oC{JvNqHjKwCtE}s@tcA{j@vw@m_@|h@uYhb@evBlzCucAvvAqzArwByGbGanCdtD}IlJ_Y`]mTxVgI~HoT`SoNhLgN|J}s@pf@yGtEgIdFwCz@cBN{ELiCMqR{AyGkAmJ{AeKyB{K{AoMm@yWm@uON{P?wSNyRl@oXhBmThBeKzAiGjA_c@lKsV~HwMrFaNpGmn@tZ{UhLucAz`@}iDtlBkKrFytK`tFuqBreAitA|r@wm@fY}T|Jsj@`Q{[~Ice@jLm_IjgB}i@~Sqp@rQuy@pQuuBbp@m}Ahv@u^r[}hAbdAqq@~]w_Bp{@_n@`HyoEeFy`AtFauBj`@ggEdmA{cAtY}mA`]ePdE}kBnh@kGhBmIvCm~@zVk[lJk[nI}IhBeo@rQ_IxBmU`GyGjBeJhCkxBtm@i\\|J}wAl_@}JxBmJxBmJhBuJzAeJjAmJjAcVjBsf@jAw\\Lud@N}d@l@iu@l@e_@z@ca@Nun@jAgNNcf@?{o@z@_]^ib@pGob@xLsQpGcP`IcVbPo`CliAiLrEwNrGyf@l^_d@`g@kPlUqM|TcQ|^sZ|}@{Vzt@yQzj@oNfc@kKzUaIhMaLvNqMhLyMxLsPpRiq@haAe}A|eC}IvMyQvXaM`SsLdOog@pg@uOfNmUzUsz@zt@{n@jj@aw@vw@uDtE_JjLkF`HwC`HgE`Hye@z~@cyAr`Dq]ns@aLvXs{D`qIkKlToIxM}JhLqLbGaNbFyu@p]i\\bo@mJbQ_{BzdE{nAn{B_Ylh@y`@bz@gTfd@qjAgbAy\\iXif@ya@cLwM_ImJoIyLmE_JkFkKaI}TsK}_@}EwM{EmJqb@k~AuOkk@e^ksAgIuYcLwb@eJq\\mFaS","summary":{"max_lon":-76.266228,"max_lat":40.283943,"time":2438,"length":31.322,"min_lat":40.042015,"min_lon":-76.664360},"maneuvers":[{"travel_mode":"drive","begin_shape_index":0,"length":0.073,"time":14,"type":1,"end_shape_index":2,"instruction":"Drive east on East Fulton Street.","verbal_pre_transition_instruction":"Drive east on East Fulton Street for 400 feet.","travel_type":"car","street_names":["East Fulton Street"]},{"travel_type":"car","travel_mode":"drive","verbal_multi_cue":true,"verbal_pre_transition_instruction":"Turn right onto North Plum Street. Then Turn left onto East Chestnut Street.","verbal_transition_alert_instruction":"Turn right onto North Plum Street.","length":0.046,"instruction":"Turn right onto North Plum Street.","end_shape_index":3,"type":10,"time":34,"verbal_post_transition_instruction":"Continue for 200 feet.","street_names":["North Plum Street"],"begin_shape_index":2},{"travel_type":"car","travel_mode":"drive","end_shape_index":89,"verbal_pre_transition_instruction":"Turn left onto East Chestnut Street, Pennsylvania 23 East.","begin_street_names":["East Chestnut Street","PA 23 East"],"verbal_transition_alert_instruction":"Turn left onto East Chestnut Street.","length":2.054,"instruction":"Turn left onto East Chestnut Street\/PA 23 East. Continue on PA 23 East.","type":15,"time":213,"verbal_post_transition_instruction":"Continue on Pennsylvania 23 East for 2.1 miles.","street_names":["PA 23 East"],"begin_shape_index":3},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn left to take the U.S. 30 West ramp toward New Holland, Harrisburg.","verbal_transition_alert_instruction":"Turn left to take the U.S. 30 West ramp.","instruction":"Turn left to take the US 30 West ramp toward New Holland\/Harrisburg.","end_shape_index":119,"type":19,"time":45,"street_names":["PA 23 East"],"begin_shape_index":89,"length":0.375,"sign":{"exit_toward_elements":[{"text":"New Holland"},{"text":"Harrisburg"}],"exit_branch_elements":[{"consecutive_count":1,"text":"US 30 West"},{"text":"PA 23 East"}]}},{"travel_type":"car","verbal_pre_transition_instruction":"Merge onto U.S. 30 West.","verbal_post_transition_instruction":"Continue for 2.7 miles.","instruction":"Merge onto US 30 West.","end_shape_index":169,"type":25,"time":164,"street_names":["US 30 West"],"length":2.746,"begin_shape_index":119,"travel_mode":"drive"},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Keep left to take Pennsylvania 2 83 West toward Harrisburg.","verbal_transition_alert_instruction":"Keep left to take Pennsylvania 2 83 West.","sign":{"exit_toward_elements":[{"text":"Harrisburg"}],"exit_branch_elements":[{"text":"PA 283 West"}]},"length":16.845,"instruction":"Keep left to take PA 283 West toward Harrisburg.","end_shape_index":475,"type":24,"time":956,"verbal_post_transition_instruction":"Continue for 16.8 miles.","street_names":["PA 283 West"],"begin_shape_index":169},{"travel_type":"car","verbal_pre_transition_instruction":"Take the Pennsylvania 7 43 exit on the right toward Hershey.","verbal_transition_alert_instruction":"Take the Pennsylvania 7 43 exit on the right.","instruction":"Take the PA 743 exit on the right toward Hershey.","end_shape_index":485,"type":20,"time":31,"begin_shape_index":475,"length":0.469,"sign":{"exit_toward_elements":[{"consecutive_count":1,"text":"Hershey"},{"text":"Elizabethtown"}],"exit_branch_elements":[{"text":"PA 743"}]},"travel_mode":"drive"},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Keep right to take Pennsylvania 7 43 North toward Hershey. Then Continue on Pennsylvania 7 43 North.","verbal_transition_alert_instruction":"Keep right to take Pennsylvania 7 43 North.","instruction":"Keep right to take PA 743 North toward Hershey.","end_shape_index":492,"type":23,"time":4,"verbal_multi_cue":true,"begin_shape_index":485,"length":0.067,"sign":{"exit_toward_elements":[{"consecutive_count":1,"text":"Hershey"}],"exit_branch_elements":[{"consecutive_count":1,"text":"PA 743 North"}]}},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Continue on Pennsylvania 7 43 North for 4.9 miles.","begin_street_names":["Hershey Road","PA 743 North","PA 341 Truck"],"verbal_transition_alert_instruction":"Continue on Pennsylvania 7 43 North.","length":4.885,"instruction":"Continue on PA 743 North.","end_shape_index":572,"type":8,"time":522,"street_names":["PA 743 North"],"begin_shape_index":492},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Continue on Fishburn Road for 2.5 miles.","begin_street_names":["Fishburn Road","PA 743"],"verbal_transition_alert_instruction":"Continue on Fishburn Road.","length":2.526,"instruction":"Continue on Fishburn Road.","end_shape_index":626,"type":8,"time":261,"street_names":["Fishburn Road"],"begin_shape_index":572},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Bear left onto Hockersville Road.","verbal_transition_alert_instruction":"Bear left onto Hockersville Road.","length":0.572,"instruction":"Bear left onto Hockersville Road.","end_shape_index":633,"type":16,"time":92,"verbal_post_transition_instruction":"Continue for 6 tenths of a mile.","street_names":["Hockersville Road"],"begin_shape_index":626},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn right onto U.S. 4 22, West Chocolate Avenue.","verbal_transition_alert_instruction":"Turn right onto U.S. 4 22.","length":0.664,"instruction":"Turn right onto US 422\/West Chocolate Avenue.","end_shape_index":652,"type":10,"time":102,"verbal_post_transition_instruction":"Continue for 7 tenths of a mile.","street_names":["US 422","West Chocolate Avenue"],"begin_shape_index":633},{"travel_type":"car","travel_mode":"drive","begin_shape_index":652,"time":0,"type":4,"end_shape_index":652,"instruction":"You have arrived at your destination.","length":0.000,"verbal_transition_alert_instruction":"You will arrive at your destination.","verbal_pre_transition_instruction":"You have arrived at your destination."}]}],"status_message":"Found route between points","status":0}})";
  NavigatorTest nav;
  nav.SetRoute(route_json_str);
  uint32_t leg_index = 0;

  TryRouteLegCount(nav, 1);
  TryRouteManeuverCount(nav, leg_index, 13);
  TryRouteLanguage(nav, "en-US");
  TryRouteUnits(nav, "miles", false);

  // Test miles to meters
  TryUnitsToMeters(nav.UnitsToMeters(1000.0f), 1609344.0f);
  TryUnitsToMeters(nav.UnitsToMeters(142.3f), 229009.7f);
  TryUnitsToMeters(nav.UnitsToMeters(37.2f), 59867.6f);
  TryUnitsToMeters(nav.UnitsToMeters(1.0f), 1609.34f);
  TryUnitsToMeters(nav.UnitsToMeters(0.5f), 804.672f);
  TryUnitsToMeters(nav.UnitsToMeters(0.25f), 402.336f);
  TryUnitsToMeters(nav.UnitsToMeters(0.062137f), 100.0f);
  TryUnitsToMeters(nav.UnitsToMeters(0.012427f), 20.0f);
  TryUnitsToMeters(nav.UnitsToMeters(0.003106f), 5.0f);

  // Test remaining leg values
  TryRemainingLegValues(nav, 652, 0.0f, 0);       // 0
  TryRemainingLegValues(nav, 633, 0.664f, 102);   // 102
  TryRemainingLegValues(nav, 626, 1.236f, 194);   // 193
  TryRemainingLegValues(nav, 572, 3.762f, 455);   // 452
  TryRemainingLegValues(nav, 492, 8.647f, 977);   // 973
  TryRemainingLegValues(nav, 485, 8.714f, 981);   // 976
  TryRemainingLegValues(nav, 475, 9.183f, 1012);  // 1007
  TryRemainingLegValues(nav, 169, 26.028f, 1968); // 1963
  TryRemainingLegValues(nav, 119, 28.774f, 2132); // 2124
  TryRemainingLegValues(nav, 89, 29.149f, 2177);  // 2164
  TryRemainingLegValues(nav, 3, 31.203f, 2390);   // 2375
  TryRemainingLegValues(nav, 2, 31.249f, 2424);   // 2409
  TryRemainingLegValues(nav, 0, 31.322f, 2438);   // 2423

  // Test the initial long transition alert methods for imperial values
  TryLength("GetInitialLongTransitionAlertLength", nav.GetInitialLongTransitionAlertLength(),
            kInitialLongTransitionAlertImperialLength);
  TryLength("GetInitialLongTransitionAlertLowerLength",
            nav.GetInitialLongTransitionAlertLowerLength(),
            kInitialLongTransitionAlertLowerImperialLength);
  TryLength("GetInitialLongTransitionAlertUpperLength",
            nav.GetInitialLongTransitionAlertUpperLength(),
            kInitialLongTransitionAlertUpperImperialLength);
  TryLength("GetInitialLongTransitionAlertMinManeuverLength",
            nav.GetInitialLongTransitionAlertMinManeuverLength(),
            kInitialLongTransitionAlertMinManeuverImperialLength);

  // Test the initial short transition alert methods for imperial values
  TryLength("GetInitialShortTransitionAlertLength", nav.GetInitialShortTransitionAlertLength(),
            kInitialShortTransitionAlertImperialLength);
  TryLength("GetInitialShortTransitionAlertLowerLength",
            nav.GetInitialShortTransitionAlertLowerLength(),
            kInitialShortTransitionAlertLowerImperialLength);
  TryLength("GetInitialShortTransitionAlertUpperLength",
            nav.GetInitialShortTransitionAlertUpperLength(),
            kInitialShortTransitionAlertUpperImperialLength);
  TryLength("GetInitialShortTransitionAlertMinManeuverLength",
            nav.GetInitialShortTransitionAlertMinManeuverLength(),
            kInitialShortTransitionAlertMinManeuverImperialLength);

  //  TryGetRemainingManeuverTime()

  ////////////////////////////////////////////////////////////////////////////
  uint32_t maneuver_index = 0;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[0] | segment index 0 | begin of maneuver index 0
  TrySnapToRoute(nav, GetFixLocation(-76.299179f, 40.042572f, 0),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.299171f, 40.042519f,
                                     leg_index, 31.322f, 2438, maneuver_index, 0.073f, 14));

  // off route | segment index 1 | partial maneuver 0
  TrySnapToRoute(nav, GetFixLocation(-76.29875f, 40.04316f, 0),
                 GetNavigationStatus(NavigationStatus_RouteState_kInvalid));

  // trace_pt[6] | segment index 1 | partial maneuver 0
  TrySnapToRoute(nav, GetFixLocation(-76.298363f, 40.042652f, 0),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.298355f, 40.042606f,
                                     leg_index, 31.278385f, 2415, maneuver_index, 0.029385f, 6));

  // trace_pt[9] | segment index 1 | near end of maneuver index 0
  TrySnapToRoute(nav, GetFixLocation(-76.297966f, 40.042698f, 12),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.297958f, 40.042652f,
                                     leg_index, 31.257146f, 2411, maneuver_index, 0.008146f, 2));

  // trace_pt[10] | segment index 1 | near end of maneuver index 0
  TrySnapToRoute(nav, GetFixLocation(-76.297844f, 40.042709f, 13),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.297836f, 40.042667f,
                                     leg_index, 31.250611f, 2409, maneuver_index, 0.001611f, 0));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 1;
  ////////////////////////////////////////////////////////////////////////////

  // snap to shape_index[2] | segment index 1/2 | begin maneuver index 1
  TrySnapToRoute(nav, GetFixLocation(-76.297820f, 40.042671f, 14),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.297806f, 40.042671f,
                                     leg_index, 31.249f, 2409, maneuver_index, 0.046f, 34));

  // near shape_index[2] | segment index 1/2 | begin maneuver index 1
  TrySnapToRoute(nav, GetFixLocation(-76.297810f, 40.042671f, 14),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.297806f, 40.042671f,
                                     leg_index, 31.249f, 2409, maneuver_index, 0.046f, 34));

  // shape_index[2] | segment index 1/2 | begin maneuver index 1
  TrySnapToRoute(nav, GetFixLocation(-76.297806f, 40.042671f, 14),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.297806f, 40.042671f,
                                     leg_index, 31.249f, 2409, maneuver_index, 0.046f, 34));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 2;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[132] | segment index 15 | middle of maneuver index 2
  TrySnapToRoute(nav, GetFixLocation(-76.279152f, 40.048409f, 154),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.279091f, 40.048393f,
                                     leg_index, 30.046f, 2255, maneuver_index, 0.8978f, 91));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 3;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[211] | segment index 23 | middle of maneuver index 3
  TrySnapToRoute(nav, GetFixLocation(-76.267502f, 40.056950f, 283),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.267517f, 40.056873f,
                                     leg_index, 29.024f, 2150, maneuver_index, 0.251f, 26));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 4;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[315] | segment index 29 | middle of maneuver index 4
  TrySnapToRoute(nav, GetFixLocation(-76.293968f, 40.068066f, 388),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.293968f, 40.068073f,
                                     leg_index, 27.341f, 2042, maneuver_index, 1.3148f, 79));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 5;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[850] | segment index 60 | middle of maneuver index 5
  TrySnapToRoute(nav, GetFixLocation(-76.471893f, 40.125996f, 948),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.471878f, 40.126011f,
                                     leg_index, 16.9535f, 1450, maneuver_index, 7.771f, 443));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 6;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[1270] | segment index 74 | middle of maneuver index 6
  TrySnapToRoute(nav, GetFixLocation(-76.598633f, 40.176716f, 1441),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.598648f, 40.176708f,
                                     leg_index, 8.86744f, 985, maneuver_index, 0.1534f, 9));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 7;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[1288] | segment index 75 | middle of maneuver index 7
  TrySnapToRoute(nav, GetFixLocation(-76.600441f, 40.178944f, 1459),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.600441f, 40.178944f,
                                     leg_index, 8.6843f, 974, maneuver_index, 0.03765f, 1));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 8;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[1522] | segment index 86 | middle of maneuver index 8
  TrySnapToRoute(nav, GetFixLocation(-76.618904f, 40.219437f, 1722),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.618668f, 40.219433f,
                                     leg_index, 5.6829f, 654, maneuver_index, 1.9216f, 202));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 9;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[1870] | segment index 105 | middle of maneuver index 9
  TrySnapToRoute(nav, GetFixLocation(-76.652367f, 40.270046f, 2113),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.652374f, 40.270039f,
                                     leg_index, 1.5266f, 224, maneuver_index, 0.2904f, 31));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 10;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[1936] | segment index 111 | middle of maneuver index 10
  TrySnapToRoute(nav, GetFixLocation(-76.660156f, 40.275745f, 2290),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.660187f, 40.275719f,
                                     leg_index, 0.9464f, 147, maneuver_index, 0.2826f, 45));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 11;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[2003] | segment index 119 | middle of maneuver index 11
  TrySnapToRoute(nav, GetFixLocation(-76.659927f, 40.281940f, 2386),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.659943f, 40.281963f,
                                     leg_index, 0.3119f, 47, maneuver_index, 0.3119f, 47));

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 12;
  ////////////////////////////////////////////////////////////////////////////

  // trace_pt[2038] | segment index 125 | destination maneuver index 12
  TrySnapToRoute(nav, GetFixLocation(-76.654625f, 40.283924f, 2438),
                 GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.654633f, 40.283943f,
                                     leg_index, 0.0f, 0, maneuver_index, 0.0f, 0));

  //==========================================================================
  //==========================================================================

  // reset the route
  nav.SetRoute(route_json_str);

  // Test GetWordCount
  TestGetWordCount(nav);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 0;
  uint32_t instruction_index = maneuver_index;

  ////////////////////////////////////////////////////////////////////////////
  bool expected_on_route_location_close_to_origin = true;

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // Start maneuver at beginning of route
  // trace_pt[0] | segment index 0 | begin of maneuver index 0
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.29931f, 40.04240f, 0),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.299171f, 40.042519f, leg_index, 31.322f, 2438,
                                                maneuver_index, 0.073f, 14, instruction_index),
                            expected_on_route_location_close_to_origin);

  // kPreTransition instruction should be used after OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  // reset the route
  nav.SetRoute(route_json_str);

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // Start maneuver near beginning of route
  // trace_pt[1] | segment index 0 | near begin of maneuver index 0
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.299057f, 40.042595f, 1),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.299049f, 40.042530f, leg_index, 31.3163f, 2422,
                                                maneuver_index, 0.066578f, 13, instruction_index),
                            expected_on_route_location_close_to_origin);

  // kPreTransition instruction should be used after OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  // reset the route
  nav.SetRoute(route_json_str);

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Turn right onto North Plum Street. Then Turn left onto East Chestnut Street."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 8);

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // No kPreTransition because not within time threshold for pre transition
  // trace_pt[2] | segment index 0 | near begin of maneuver index 0 | maneuver speed ~ 18.7714 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.298897f, 40.042610f, 3),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.298889f,
                                                40.042545f, leg_index, 31.3077f, 2420, maneuver_index,
                                                0.058027f, 11));

  // kPreTransition instruction should be not used after OnLocationChanged
  // because not within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // trace_pt[5] | segment index 1 | near middle of maneuver index 0 | fix speed ~ 20 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.298477f, 40.042645f, 6, 8.94f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.298470f, 40.042595f, leg_index, 31.2852f, 2416,
                                                maneuver_index, 0.035543f, 7, instruction_index));

  // kPreTransition instruction should be used after OnLocationChanged
  // because it was within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 1;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Turn left onto East Chestnut Street, Pennsylvania 23 East."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 8);

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // trace_pt[13] | segment index 2 | near middle of maneuver index 1 | fix speed >~ 10 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.297745f, 40.042370f, 20, 5.0f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.297752f, 40.042370f, leg_index, 31.2287f, 2393,
                                                maneuver_index, 0.0247517f, 18, instruction_index));

  // kPreTransition instruction should be used after OnLocationChanged
  // because it was within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 2;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // kPostTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPostTransition>(nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[17] | segment index 4 | near begin of maneuver index 2 |  maneuver speed ~ 34.7 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.297585f, 40.042072f, 49),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2975769f,
                                                40.0420265f, leg_index, 31.1978016f, 2374,
                                                maneuver_index, 2.04964066f, 210));

  // kPostTransition instruction should not be used after OnLocationChanged
  // because it was less than the time threshold for post-transition
  TryUsedInstructions(std::get<kPostTransition>(nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[19] | segment index 4 | near begin of maneuver index 2 | fix speed ~ 30 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.297173f, 40.042126f, 51, 13.41f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.2971649f, 40.0420685f, leg_index, 31.1757927f,
                                                2372, maneuver_index, 2.02763176f, 208,
                                                instruction_index));

  // kPostTransition instruction should be used after OnLocationChanged
  // because it was outside of the time threshold for post-transition
  TryUsedInstructions(std::get<kPostTransition>(nav.used_instructions().at(instruction_index)), true);

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Turn left to take the U.S. 30 West ramp toward New Holland, Harrisburg."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 10);

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // trace_pt[186] | segment index 87 | near end of maneuver index 2 | fix speed ~ 45 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.268127f, 40.054886f, 262, 20.12f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.268112f, 40.054859f, leg_index, 29.2577f, 2176,
                                                maneuver_index, 0.109503f, 12, instruction_index));

  // kPreTransition instruction should be used after OnLocationChanged
  // because it was within time threshold for pre-transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 3;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Merge onto U.S. 30 West."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 6);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 4;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // kInitialTransitionAlert instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kInitialTransitionAlert>(
                          nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[332] | segment index 30 | 1 mile before end of maneuver index 4 | maneuver speed
  // ~ 60.3 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.299850f, 40.068081f, 401),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.2998428f, 40.068119f, leg_index, 27.0300159f,
                                                2022, maneuver_index, 1.00362206f, 59,
                                                instruction_index, 1.0f));

  // kInitialTransitionAlert instruction should be used after OnLocationChanged
  // because it was within length bounds for initial transition alert
  TryUsedInstructions(std::get<kInitialTransitionAlert>(
                          nav.used_instructions().at(instruction_index)),
                      true);

  // kFinalTransitionAlert instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[373] | segment index 34 | 1/4 mile before end of maneuver index 4 | maneuver speed
  // ~ 60.3 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.313850f, 40.069965f, 446),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.3138428f, 40.0699997f, leg_index, 26.2774086f,
                                                1977, maneuver_index, 0.251014709f, 14,
                                                instruction_index, 0.25f));

  // kFinalTransitionAlert instruction should be used after OnLocationChanged
  // because it was within length bounds for final transition alert
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      true);

  // Test GetPreTransitionThreshold
  // "Keep left to take Pennsylvania 2 83 West toward Harrisburg."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 8);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 5;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // kInitialTransitionAlert instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kInitialTransitionAlert>(
                          nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[1139] | segment index 73 | 2 miles before end of maneuver index 5 | fix speed ~ 72 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.567604f, 40.152584f, 1301, 32.19f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5676193f, 40.1525688f, leg_index, 11.2084436f,
                                                1122, maneuver_index, 2.02614307f, 115,
                                                instruction_index, 2.0f));

  // kInitialTransitionAlert instruction should be used after OnLocationChanged
  // because it was within length bounds for initial transition alert
  TryUsedInstructions(std::get<kInitialTransitionAlert>(
                          nav.used_instructions().at(instruction_index)),
                      true);

  // kFinalTransitionAlert instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[1220] | segment index 73 | 1/2 mile before end of maneuver index 5 | fix speed ~ 72
  // MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.587799f, 40.168167f, 1387, 32.19f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5878143f, 40.1681595f, leg_index, 9.69095802f,
                                                1036, maneuver_index, 0.508657455f, 29,
                                                instruction_index, 0.5f));

  // kFinalTransitionAlert instruction should be used after OnLocationChanged
  // because it was within length bounds for final transition alert
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      true);

  // Test GetPreTransitionThreshold
  // "Take the Pennsylvania 7 43 exit on the right toward Hershey."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 8);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 6;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Keep right to take Pennsylvania 7 43 North toward Hershey. Then Continue on Pennsylvania 7 43
  // North."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 9);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 7;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Continue on Pennsylvania 7 43 North for 4.9 miles."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 8);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 8;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Continue on Fishburn Road for 2.5 miles."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 7);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 9;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Bear left onto Hockersville Road."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 6);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 10;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "Turn right onto U.S. 4 22, West Chocolate Avenue."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 8);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 11;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // Test GetPreTransitionThreshold
  // "You have arrived at your destination."
  TryGetPreTransitionThreshold(nav, leg_index, instruction_index, 6);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 12;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  //----------------------------------------------------------------
  // reset the route
  nav.SetRoute(route_json_str);

  //----------------------------------------------------------------
  // trace point = 0 | Pre | Drive east on East Fulton Street for 400 feet.
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2991791f, 40.042572f, 1491514643, 0),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2991714f, 40.0425186f, leg_index, 31.3227901f,
                                                2423, maneuver_index, 0.0730895996f, 14,
                                                instruction_index),
                            true);
  //----------------------------------------------------------------
  // trace point = 1
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.299057f, 40.0425949f, 1491514645, 5.25819588),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2990494f,
                                                40.0425301f, leg_index, 31.3162785f, 2422,
                                                maneuver_index, 0.0665779114f, 13),
                            true);
  //----------------------------------------------------------------
  // trace point = 2
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2988968f, 40.0426102f, 1491514647, 7.07390213),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2988892f,
                                                40.0425453f, leg_index, 31.3077278f, 2420,
                                                maneuver_index, 0.0580272675f, 11));
  //----------------------------------------------------------------
  // trace point = 3
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2986908f, 40.0426216f, 1491514649, 8.55290508),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2986832f,
                                                40.0425682f, leg_index, 31.2967052f, 2418,
                                                maneuver_index, 0.0470046997f, 9));
  //----------------------------------------------------------------
  // trace point = 4 | Pre | Turn right onto North Plum Street. Then Turn left onto East Chestnut
  // Street.
  maneuver_index = 0;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.298584f, 40.0426331f, 1491514650, 9.53861332),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2985764f, 40.0425797f, leg_index, 31.2909946f,
                                                2417, maneuver_index, 0.0412940979f, 8,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 5
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2984772f, 40.0426445f, 1491514651, 9.0889616),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2984695f,
                                                40.0425949f, leg_index, 31.285244f, 2416,
                                                maneuver_index, 0.0355434418f, 7));
  //----------------------------------------------------------------
  // trace point = 6
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2983627f, 40.0426521f, 1491514652, 9.70766926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2983551f,
                                                40.0426064f, leg_index, 31.2791328f, 2415,
                                                maneuver_index, 0.0294322968f, 6));
  //----------------------------------------------------------------
  // trace point = 7
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2982559f, 40.0426674f, 1491514653, 9.56973743),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2982483f,
                                                40.0426216f, leg_index, 31.2733822f, 2414,
                                                maneuver_index, 0.0236816406f, 5));
  //----------------------------------------------------------------
  // trace point = 8
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2980881f, 40.0426826f, 1491514655, 7.12929583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2980804f,
                                                40.0426407f, leg_index, 31.2643967f, 2412,
                                                maneuver_index, 0.0146961212f, 3));
  //----------------------------------------------------------------
  // trace point = 9
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.297966f, 40.0426979f, 1491514658, 3.54419708),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2979584f,
                                                40.0426521f, leg_index, 31.2578869f, 2411,
                                                maneuver_index, 0.00818634033f, 2));
  //----------------------------------------------------------------
  // trace point = 10
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2978439f, 40.0427094f, 1491514662, 2.52941489),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2978363f,
                                                40.0426674f, leg_index, 31.251339f, 2409,
                                                maneuver_index, 0.00163841248f, 0));
  //----------------------------------------------------------------
  // trace point = 11
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2977905f, 40.0426216f, 1491514664, 5.37971592),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2977982f,
                                                40.0426216f, leg_index, 31.2462463f, 2406,
                                                maneuver_index, 0.0423336029f, 31));
  //----------------------------------------------------------------
  // trace point = 12 | Pre | Turn left onto East Chestnut Street, Pennsylvania 23 East.
  maneuver_index = 1;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2977676f, 40.0425034f, 1491514666, 6.66063595),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2977753f, 40.0425034f, leg_index, 31.237978f,
                                                2400, maneuver_index, 0.0340652466f, 25,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 13
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2977448f, 40.0423698f, 1491514668, 7.50693798),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2977524f,
                                                40.0423698f, leg_index, 31.2286644f, 2393,
                                                maneuver_index, 0.0247516632f, 18));
  //----------------------------------------------------------------
  // trace point = 14
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2977295f, 40.042263f, 1491514670, 5.98129892),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2977371f,
                                                40.042263f, leg_index, 31.2212353f, 2388,
                                                maneuver_index, 0.0173225403f, 13));
  //----------------------------------------------------------------
  // trace point = 15
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2977524f, 40.0421829f, 1491514679, 0.997165978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2977219f,
                                                40.0421867f, leg_index, 31.2158966f, 2384,
                                                maneuver_index, 0.0119838715f, 9));
  //----------------------------------------------------------------
  // trace point = 16
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2977142f, 40.0420952f, 1491514708, 0.356036991),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2977066f,
                                                40.0420952f, leg_index, 31.2095127f, 2379,
                                                maneuver_index, 0.00559997559f, 4));
  //----------------------------------------------------------------
  // trace point = 17
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2975845f, 40.0420723f, 1491514710, 5.49139786),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2975769f,
                                                40.0420265f, leg_index, 31.1978016f, 2374,
                                                maneuver_index, 2.04964066f, 210));
  //----------------------------------------------------------------
  // trace point = 18 | Post | Continue on Pennsylvania 23 East for 2.1 miles.
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2974091f, 40.0420952f, 1491514712, 7.50597715),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.2974014f, 40.0420418f, leg_index, 31.1884518f,
                                                2373, maneuver_index, 2.04029083f, 209,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 19
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2971725f, 40.0421257f, 1491514714, 10.3292894),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2971649f,
                                                40.0420685f, leg_index, 31.1757927f, 2372,
                                                maneuver_index, 2.02763176f, 208));
  //----------------------------------------------------------------
  // trace point = 20
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.29702f, 40.0421333f, 1491514715, 13.0775595),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2970123f,
                                                40.0420837f, leg_index, 31.1676445f, 2371,
                                                maneuver_index, 2.01948357f, 207));
  //----------------------------------------------------------------
  // trace point = 21
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2968597f, 40.0421486f, 1491514716, 13.6275177),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2968521f,
                                                40.042099f, leg_index, 31.1582928f, 2370,
                                                maneuver_index, 2.01013184f, 206));
  //----------------------------------------------------------------
  // trace point = 22
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2967072f, 40.0421562f, 1491514717, 13.3324862),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2966995f,
                                                40.0421143f, leg_index, 31.1509438f, 2369,
                                                maneuver_index, 2.00278282f, 205));
  //----------------------------------------------------------------
  // trace point = 23
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2965469f, 40.0421715f, 1491514718, 13.3858213),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2965393f,
                                                40.0421295f, leg_index, 31.142395f, 2368,
                                                maneuver_index, 1.99423409f, 204));
  //----------------------------------------------------------------
  // trace point = 24
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2963791f, 40.0421829f, 1491514719, 14.6562252),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2963715f,
                                                40.0421486f, leg_index, 31.1334095f, 2367,
                                                maneuver_index, 1.98524857f, 203));
  //----------------------------------------------------------------
  // trace point = 25
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2962189f, 40.0421906f, 1491514720, 13.7430067),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2962112f,
                                                40.0421638f, leg_index, 31.1248608f, 2366,
                                                maneuver_index, 1.97669983f, 202));
  //----------------------------------------------------------------
  // trace point = 26
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2960358f, 40.0422096f, 1491514721, 15.3164253),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2960281f,
                                                40.0421829f, leg_index, 31.1150761f, 2365,
                                                maneuver_index, 1.96691513f, 201));
  //----------------------------------------------------------------
  // trace point = 27
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2958603f, 40.0422325f, 1491514722, 15.2161894),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2958527f,
                                                40.042202f, leg_index, 31.10569f, 2364,
                                                maneuver_index, 1.95752907f, 200));
  //----------------------------------------------------------------
  // trace point = 28
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2956848f, 40.0422516f, 1491514723, 15.3859339),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2956772f,
                                                40.0422211f, leg_index, 31.0963058f, 2363,
                                                maneuver_index, 1.94814491f, 199));
  //----------------------------------------------------------------
  // trace point = 29
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2955017f, 40.0422668f, 1491514724, 15.4431725),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2954941f,
                                                40.042244f, leg_index, 31.0864811f, 2362,
                                                maneuver_index, 1.93832016f, 198));
  //----------------------------------------------------------------
  // trace point = 30
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2953262f, 40.0422859f, 1491514725, 15.3164082),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2953186f,
                                                40.042263f, leg_index, 31.077095f, 2362,
                                                maneuver_index, 1.9289341f, 198));
  //----------------------------------------------------------------
  // trace point = 31
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2951431f, 40.042305f, 1491514726, 15.5008316),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2951355f,
                                                40.0422821f, leg_index, 31.0673103f, 2361,
                                                maneuver_index, 1.9191494f, 197));
  //----------------------------------------------------------------
  // trace point = 32
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2949753f, 40.0423203f, 1491514727, 14.8378019),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2949753f,
                                                40.0423012f, leg_index, 31.0587234f, 2360,
                                                maneuver_index, 1.91056252f, 196));
  //----------------------------------------------------------------
  // trace point = 33
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2947998f, 40.0423355f, 1491514728, 15.0199671),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2947998f,
                                                40.0423241f, leg_index, 31.0492992f, 2359,
                                                maneuver_index, 1.90113831f, 195));
  //----------------------------------------------------------------
  // trace point = 34
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2946243f, 40.0423546f, 1491514729, 15.0632153),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2946243f,
                                                40.0423431f, leg_index, 31.0399151f, 2358,
                                                maneuver_index, 1.89175415f, 194));
  //----------------------------------------------------------------
  // trace point = 35
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2944565f, 40.0423737f, 1491514730, 14.3725615),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2944565f,
                                                40.0423622f, leg_index, 31.0309277f, 2357,
                                                maneuver_index, 1.88276672f, 193));
  //----------------------------------------------------------------
  // trace point = 36
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2942886f, 40.0423927f, 1491514731, 14.3884983),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2942886f,
                                                40.0423813f, leg_index, 31.0219421f, 2356,
                                                maneuver_index, 1.8737812f, 192));
  //----------------------------------------------------------------
  // trace point = 37
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2941284f, 40.0424042f, 1491514732, 13.9664774),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2941284f,
                                                40.0424004f, leg_index, 31.0133572f, 2355,
                                                maneuver_index, 1.86519623f, 191));
  //----------------------------------------------------------------
  // trace point = 38
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2939529f, 40.0424118f, 1491514733, 14.9243536),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2939529f,
                                                40.0424194f, leg_index, 31.0039711f, 2354,
                                                maneuver_index, 1.85581017f, 190));
  //----------------------------------------------------------------
  // trace point = 39
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2937775f, 40.0424271f, 1491514734, 14.6684294),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2937775f,
                                                40.0424385f, leg_index, 30.9945831f, 2353,
                                                maneuver_index, 1.8464222f, 189));
  //----------------------------------------------------------------
  // trace point = 40
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2936096f, 40.0424385f, 1491514735, 14.7190399),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2936096f,
                                                40.0424538f, leg_index, 30.9856339f, 2352,
                                                maneuver_index, 1.83747292f, 188));
  //----------------------------------------------------------------
  // trace point = 41
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2934418f, 40.0424538f, 1491514736, 14.5969229),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2934418f,
                                                40.042469f, leg_index, 30.9766827f, 2351,
                                                maneuver_index, 1.82852173f, 187));
  //----------------------------------------------------------------
  // trace point = 42
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2932663f, 40.0424767f, 1491514737, 14.8614683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2932663f,
                                                40.0424881f, leg_index, 30.9672985f, 2350,
                                                maneuver_index, 1.81913757f, 186));
  //----------------------------------------------------------------
  // trace point = 43
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2931061f, 40.0424957f, 1491514738, 14.0866117),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2931061f,
                                                40.0425034f, leg_index, 30.9587479f, 2349,
                                                maneuver_index, 1.81058693f, 185));
  //----------------------------------------------------------------
  // trace point = 44
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2929459f, 40.0425224f, 1491514739, 13.6440716),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2929459f,
                                                40.0425186f, leg_index, 30.9501991f, 2348,
                                                maneuver_index, 1.80203819f, 184));
  //----------------------------------------------------------------
  // trace point = 45
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2927933f, 40.0425415f, 1491514740, 13.2277584),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2927933f,
                                                40.0425339f, leg_index, 30.9420509f, 2347,
                                                maneuver_index, 1.79389f, 183));
  //----------------------------------------------------------------
  // trace point = 46
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2926483f, 40.0425606f, 1491514741, 12.4529734),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2926483f,
                                                40.0425453f, leg_index, 30.9343357f, 2346,
                                                maneuver_index, 1.78617477f, 182));
  //----------------------------------------------------------------
  // trace point = 47
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2925339f, 40.042572f, 1491514742, 10.1305323),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2925339f,
                                                40.0425568f, leg_index, 30.9282227f, 2346,
                                                maneuver_index, 1.78006172f, 182));
  //----------------------------------------------------------------
  // trace point = 48
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2924194f, 40.0425873f, 1491514743, 9.50290394),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2924194f,
                                                40.0425682f, leg_index, 30.9221115f, 2345,
                                                maneuver_index, 1.77395058f, 181));
  //----------------------------------------------------------------
  // trace point = 49
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2922363f, 40.0426064f, 1491514745, 7.86195183),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2922363f,
                                                40.0425835f, leg_index, 30.9123592f, 2344,
                                                maneuver_index, 1.7641983f, 180));
  //----------------------------------------------------------------
  // trace point = 50
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.292099f, 40.0426254f, 1491514747, 6.06854391),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2920914f,
                                                40.0425987f, leg_index, 30.9046116f, 2343,
                                                maneuver_index, 1.75645065f, 179));
  //----------------------------------------------------------------
  // trace point = 51
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2919922f, 40.0426483f, 1491514750, 3.16541791),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2919846f,
                                                40.0426102f, leg_index, 30.898901f, 2343,
                                                maneuver_index, 1.75074005f, 179));
  //----------------------------------------------------------------
  // trace point = 52
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2918549f, 40.0426712f, 1491514791, 0.285445988),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2918472f,
                                                40.0426216f, leg_index, 30.8915863f, 2342,
                                                maneuver_index, 1.74342537f, 178));
  //----------------------------------------------------------------
  // trace point = 53
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.291687f, 40.0426826f, 1491514793, 7.32235003),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2916794f,
                                                40.0426407f, leg_index, 30.8826027f, 2341,
                                                maneuver_index, 1.73444176f, 177));
  //----------------------------------------------------------------
  // trace point = 54
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2915726f, 40.0426903f, 1491514794, 9.40678787),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2915649f,
                                                40.0426521f, leg_index, 30.8764896f, 2340,
                                                maneuver_index, 1.7283287f, 176));
  //----------------------------------------------------------------
  // trace point = 55
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2914505f, 40.0426979f, 1491514795, 10.4185562),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2914429f,
                                                40.0426636f, leg_index, 30.869978f, 2339,
                                                maneuver_index, 1.72181702f, 175));
  //----------------------------------------------------------------
  // trace point = 56
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2913284f, 40.0427094f, 1491514796, 10.9195547),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2913208f,
                                                40.042675f, leg_index, 30.8634663f, 2339,
                                                maneuver_index, 1.71530533f, 175));
  //----------------------------------------------------------------
  // trace point = 57
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2911987f, 40.0427246f, 1491514797, 11.3567858),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2911911f,
                                                40.0426903f, leg_index, 30.8565197f, 2338,
                                                maneuver_index, 1.70835876f, 174));
  //----------------------------------------------------------------
  // trace point = 58
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2910614f, 40.0427399f, 1491514798, 11.8278275),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2910538f,
                                                40.0427055f, leg_index, 30.8491707f, 2337,
                                                maneuver_index, 1.70100975f, 173));
  //----------------------------------------------------------------
  // trace point = 59
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2909164f, 40.0427666f, 1491514799, 12.6916542),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2909088f,
                                                40.0427208f, leg_index, 30.841423f, 2337,
                                                maneuver_index, 1.6932621f, 173));
  //----------------------------------------------------------------
  // trace point = 60
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2907562f, 40.0427895f, 1491514800, 13.5825748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2907486f,
                                                40.0427361f, leg_index, 30.8328724f, 2336,
                                                maneuver_index, 1.68471146f, 172));
  //----------------------------------------------------------------
  // trace point = 61
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2905884f, 40.0428123f, 1491514801, 14.5444603),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2905807f,
                                                40.0427551f, leg_index, 30.8238888f, 2335,
                                                maneuver_index, 1.67572784f, 171));
  //----------------------------------------------------------------
  // trace point = 62
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2904205f, 40.0428314f, 1491514802, 14.3884087),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2904129f,
                                                40.0427704f, leg_index, 30.8149395f, 2334,
                                                maneuver_index, 1.66677856f, 170));
  //----------------------------------------------------------------
  // trace point = 63
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2902451f, 40.0428467f, 1491514803, 15.0478811),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2902374f,
                                                40.0427895f, leg_index, 30.8055534f, 2333,
                                                maneuver_index, 1.6573925f, 169));
  //----------------------------------------------------------------
  // trace point = 64
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2900543f, 40.0428581f, 1491514804, 16.1622829),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2900467f,
                                                40.0428085f, leg_index, 30.7953682f, 2332,
                                                maneuver_index, 1.64720726f, 168));
  //----------------------------------------------------------------
  // trace point = 65
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2898788f, 40.0428772f, 1491514805, 15.2168188),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2898712f,
                                                40.0428276f, leg_index, 30.7859821f, 2330,
                                                maneuver_index, 1.6378212f, 166));
  //----------------------------------------------------------------
  // trace point = 66
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2897034f, 40.0428848f, 1491514806, 15.1842308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2896957f,
                                                40.0428467f, leg_index, 30.7765961f, 2329,
                                                maneuver_index, 1.62843513f, 165));
  //----------------------------------------------------------------
  // trace point = 67
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2895126f, 40.0428963f, 1491514807, 16.2472057),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.289505f,
                                                40.0428658f, leg_index, 30.7664108f, 2328,
                                                maneuver_index, 1.61824989f, 164));
  //----------------------------------------------------------------
  // trace point = 68
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2893295f, 40.0429192f, 1491514808, 15.63624),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2893219f,
                                                40.0428848f, leg_index, 30.7566261f, 2327,
                                                maneuver_index, 1.60846519f, 163));
  //----------------------------------------------------------------
  // trace point = 69
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2891617f, 40.0429192f, 1491514809, 14.6598825),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2891617f,
                                                40.0429039f, leg_index, 30.7480412f, 2326,
                                                maneuver_index, 1.59988022f, 162));
  //----------------------------------------------------------------
  // trace point = 70
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2889862f, 40.0429268f, 1491514810, 14.9242411),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2889862f,
                                                40.042923f, leg_index, 30.7386551f, 2325,
                                                maneuver_index, 1.59049416f, 161));
  //----------------------------------------------------------------
  // trace point = 71
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2888107f, 40.042942f, 1491514811, 15.1044598),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2888107f,
                                                40.0429382f, leg_index, 30.7293053f, 2325,
                                                maneuver_index, 1.58114433f, 161));
  //----------------------------------------------------------------
  // trace point = 72
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2886429f, 40.042942f, 1491514812, 13.8064175),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2886429f,
                                                40.0429573f, leg_index, 30.7203217f, 2324,
                                                maneuver_index, 1.57216072f, 160));
  //----------------------------------------------------------------
  // trace point = 73
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2884979f, 40.0429649f, 1491514813, 12.5975361),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2884979f,
                                                40.0429726f, leg_index, 30.7125721f, 2323,
                                                maneuver_index, 1.56441116f, 159));
  //----------------------------------------------------------------
  // trace point = 74
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2883682f, 40.0429764f, 1491514814, 11.6801872),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2883682f,
                                                40.0429802f, leg_index, 30.7056885f, 2322,
                                                maneuver_index, 1.55752754f, 158));
  //----------------------------------------------------------------
  // trace point = 75
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2882462f, 40.0429993f, 1491514815, 10.2413445),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2882462f,
                                                40.0429955f, leg_index, 30.6991386f, 2321,
                                                maneuver_index, 1.55097771f, 157));
  //----------------------------------------------------------------
  // trace point = 76
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2880936f, 40.0430183f, 1491514817, 6.83324385),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2880936f,
                                                40.0430107f, leg_index, 30.6909924f, 2320,
                                                maneuver_index, 1.54283142f, 156));
  //----------------------------------------------------------------
  // trace point = 77
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2879715f, 40.0430412f, 1491514819, 5.10595894),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2879715f,
                                                40.043026f, leg_index, 30.6844425f, 2320,
                                                maneuver_index, 1.53628159f, 156));
  //----------------------------------------------------------------
  // trace point = 78
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2878036f, 40.0430527f, 1491514821, 7.35018682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2878036f,
                                                40.043045f, leg_index, 30.675457f, 2319,
                                                maneuver_index, 1.52729607f, 155));
  //----------------------------------------------------------------
  // trace point = 79
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2876892f, 40.0430603f, 1491514822, 9.74649429),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2876892f,
                                                40.0430565f, leg_index, 30.6693439f, 2318,
                                                maneuver_index, 1.52118301f, 154));
  //----------------------------------------------------------------
  // trace point = 80
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2875824f, 40.0430794f, 1491514823, 9.57882118),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2875824f,
                                                40.0430679f, leg_index, 30.6636333f, 2317,
                                                maneuver_index, 1.51547241f, 153));
  //----------------------------------------------------------------
  // trace point = 81
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2874374f, 40.0431061f, 1491514824, 12.4761381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2874374f,
                                                40.0430794f, leg_index, 30.6559162f, 2317,
                                                maneuver_index, 1.50775528f, 153));
  //----------------------------------------------------------------
  // trace point = 82
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2873001f, 40.0431252f, 1491514825, 11.6139393),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2872925f,
                                                40.0430908f, leg_index, 30.6481991f, 2316,
                                                maneuver_index, 1.50003815f, 152));
  //----------------------------------------------------------------
  // trace point = 83
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2871323f, 40.0431442f, 1491514826, 14.894309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.287117f,
                                                40.0431099f, leg_index, 30.6388016f, 2315,
                                                maneuver_index, 1.49064064f, 151));
  //----------------------------------------------------------------
  // trace point = 84
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.286972f, 40.0431519f, 1491514827, 13.4172754),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2869644f,
                                                40.0431366f, leg_index, 30.630518f, 2314,
                                                maneuver_index, 1.48235703f, 150));
  //----------------------------------------------------------------
  // trace point = 85
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2868576f, 40.0431976f, 1491514828, 11.2089796),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2868423f,
                                                40.0431671f, leg_index, 30.6237144f, 2313,
                                                maneuver_index, 1.47555351f, 149));
  //----------------------------------------------------------------
  // trace point = 86
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2867126f, 40.0432472f, 1491514829, 13.0729036),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2866974f,
                                                40.0432243f, leg_index, 30.6150417f, 2312,
                                                maneuver_index, 1.4668808f, 148));
  //----------------------------------------------------------------
  // trace point = 87
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2865753f, 40.0432892f, 1491514830, 12.7763824),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2865829f,
                                                40.043293f, leg_index, 30.6073132f, 2311,
                                                maneuver_index, 1.45915222f, 147));
  //----------------------------------------------------------------
  // trace point = 88
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2864685f, 40.0433693f, 1491514831, 12.9920301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2864838f,
                                                40.0433769f, leg_index, 30.5994492f, 2311,
                                                maneuver_index, 1.45128822f, 147));
  //----------------------------------------------------------------
  // trace point = 89
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.286377f, 40.0434647f, 1491514832, 13.1956196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2863846f,
                                                40.0434685f, leg_index, 30.5912228f, 2310,
                                                maneuver_index, 1.44306183f, 146));
  //----------------------------------------------------------------
  // trace point = 90
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2862625f, 40.0435295f, 1491514833, 11.9774504),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2863007f,
                                                40.0435524f, leg_index, 30.5839157f, 2309,
                                                maneuver_index, 1.43575478f, 145));
  //----------------------------------------------------------------
  // trace point = 91
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.286171f, 40.0436516f, 1491514834, 15.6392136),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2861862f,
                                                40.0436592f, leg_index, 30.5743599f, 2308,
                                                maneuver_index, 1.42619896f, 144));
  //----------------------------------------------------------------
  // trace point = 92
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2860489f, 40.0437508f, 1491514835, 15.188261),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2860718f,
                                                40.043766f, leg_index, 30.5648041f, 2307,
                                                maneuver_index, 1.41664314f, 143));
  //----------------------------------------------------------------
  // trace point = 93
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2859421f, 40.043869f, 1491514836, 16.0883083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2859573f,
                                                40.0438766f, leg_index, 30.5550423f, 2307,
                                                maneuver_index, 1.40688133f, 143));
  //----------------------------------------------------------------
  // trace point = 94
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2858429f, 40.0439987f, 1491514837, 16.3463516),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2858429f,
                                                40.0439987f, leg_index, 30.5446472f, 2305,
                                                maneuver_index, 1.39648628f, 141));
  //----------------------------------------------------------------
  // trace point = 95
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2857361f, 40.044117f, 1491514838, 16.3138638),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2857361f,
                                                40.044117f, leg_index, 30.5347023f, 2304,
                                                maneuver_index, 1.38654137f, 140));
  //----------------------------------------------------------------
  // trace point = 96
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2856216f, 40.0442429f, 1491514839, 16.928875),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2856216f,
                                                40.0442429f, leg_index, 30.5241013f, 2303,
                                                maneuver_index, 1.37594032f, 139));
  //----------------------------------------------------------------
  // trace point = 97
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2855148f, 40.0443573f, 1491514840, 16.0970745),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2855072f,
                                                40.0443535f, leg_index, 30.5143414f, 2302,
                                                maneuver_index, 1.36618042f, 138));
  //----------------------------------------------------------------
  // trace point = 98
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2853851f, 40.0444756f, 1491514841, 17.0988407),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2853851f,
                                                40.0444717f, leg_index, 30.5039196f, 2301,
                                                maneuver_index, 1.35575867f, 137));
  //----------------------------------------------------------------
  // trace point = 99
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2852402f, 40.0445824f, 1491514842, 16.7803745),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2852402f,
                                                40.0445824f, leg_index, 30.4930763f, 2300,
                                                maneuver_index, 1.34491539f, 136));
  //----------------------------------------------------------------
  // trace point = 100
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2850876f, 40.0446777f, 1491514843, 16.9184685),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2850876f,
                                                40.0446739f, leg_index, 30.4827824f, 2300,
                                                maneuver_index, 1.33462143f, 136));
  //----------------------------------------------------------------
  // trace point = 101
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2849197f, 40.0447769f, 1491514844, 17.8708191),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2849045f,
                                                40.044754f, leg_index, 30.47159f, 2299,
                                                maneuver_index, 1.32342911f, 135));
  //----------------------------------------------------------------
  // trace point = 102
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2847366f, 40.0448112f, 1491514845, 16.5201416),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2847366f,
                                                40.0448189f, leg_index, 30.4616356f, 2298,
                                                maneuver_index, 1.31347466f, 134));
  //----------------------------------------------------------------
  // trace point = 103
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2843475f, 40.0448952f, 1491514847, 17.0636864),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2843475f,
                                                40.044899f, leg_index, 30.4402332f, 2295,
                                                maneuver_index, 1.2920723f, 131));
  //----------------------------------------------------------------
  // trace point = 104
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2841492f, 40.0449181f, 1491514848, 17.4568062),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2841492f,
                                                40.0449219f, leg_index, 30.4296055f, 2294,
                                                maneuver_index, 1.28144455f, 130));
  //----------------------------------------------------------------
  // trace point = 105
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2839355f, 40.0449448f, 1491514849, 18.363863),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2839355f,
                                                40.0449486f, leg_index, 30.4181442f, 2293,
                                                maneuver_index, 1.26998329f, 129));
  //----------------------------------------------------------------
  // trace point = 106
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2837143f, 40.0449638f, 1491514850, 19.049284),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2837143f,
                                                40.0449791f, leg_index, 30.4062405f, 2292,
                                                maneuver_index, 1.25807953f, 128));
  //----------------------------------------------------------------
  // trace point = 107
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2835007f, 40.0449905f, 1491514851, 18.3989067),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2835007f,
                                                40.0449982f, leg_index, 30.3948402f, 2291,
                                                maneuver_index, 1.24667931f, 127));
  //----------------------------------------------------------------
  // trace point = 108
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2832718f, 40.0450172f, 1491514852, 19.9309273),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2832718f,
                                                40.0450249f, leg_index, 30.3825798f, 2290,
                                                maneuver_index, 1.23441887f, 126));
  //----------------------------------------------------------------
  // trace point = 109
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2830353f, 40.0450554f, 1491514853, 20.2419567),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2830353f,
                                                40.045063f, leg_index, 30.3697681f, 2288,
                                                maneuver_index, 1.22160721f, 124));
  //----------------------------------------------------------------
  // trace point = 110
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2828217f, 40.0450974f, 1491514854, 18.826725),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2828293f,
                                                40.0451088f, leg_index, 30.3584099f, 2287,
                                                maneuver_index, 1.21024895f, 123));
  //----------------------------------------------------------------
  // trace point = 111
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2826004f, 40.0451584f, 1491514855, 20.1874027),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.282608f,
                                                40.045166f, leg_index, 30.3460426f, 2285,
                                                maneuver_index, 1.1978817f, 121));
  //----------------------------------------------------------------
  // trace point = 112
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2823715f, 40.0452347f, 1491514856, 21.2701511),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2823792f,
                                                40.0452423f, leg_index, 30.3328075f, 2284,
                                                maneuver_index, 1.18464661f, 120));
  //----------------------------------------------------------------
  // trace point = 113
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2821503f, 40.0453186f, 1491514857, 20.7727394),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2821579f,
                                                40.0453262f, leg_index, 30.3197289f, 2283,
                                                maneuver_index, 1.17156792f, 119));
  //----------------------------------------------------------------
  // trace point = 114
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2819366f, 40.0454216f, 1491514858, 21.4104843),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2819366f,
                                                40.0454178f, leg_index, 30.3064137f, 2282,
                                                maneuver_index, 1.15825272f, 118));
  //----------------------------------------------------------------
  // trace point = 115
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2817383f, 40.0455208f, 1491514859, 20.5715961),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.281723f,
                                                40.0455017f, leg_index, 30.2936993f, 2281,
                                                maneuver_index, 1.14553833f, 117));
  //----------------------------------------------------------------
  // trace point = 116
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2815475f, 40.0456467f, 1491514860, 21.2294331),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2815247f,
                                                40.0456238f, leg_index, 30.2802162f, 2280,
                                                maneuver_index, 1.13205528f, 116));
  //----------------------------------------------------------------
  // trace point = 117
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2813568f, 40.0457764f, 1491514861, 22.0272694),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2813263f,
                                                40.0457497f, leg_index, 30.2665749f, 2278,
                                                maneuver_index, 1.11841393f, 114));
  //----------------------------------------------------------------
  // trace point = 118
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2811584f, 40.0459099f, 1491514862, 22.2464027),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2811279f,
                                                40.045887f, leg_index, 30.2524166f, 2277,
                                                maneuver_index, 1.10425568f, 113));
  //----------------------------------------------------------------
  // trace point = 119
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2809677f, 40.046051f, 1491514863, 22.7843056),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2809372f,
                                                40.0460281f, leg_index, 30.2383709f, 2276,
                                                maneuver_index, 1.09020996f, 112));
  //----------------------------------------------------------------
  // trace point = 120
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.280777f, 40.0461998f, 1491514864, 23.0605202),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2807541f,
                                                40.0461845f, leg_index, 30.2238445f, 2274,
                                                maneuver_index, 1.07568359f, 110));
  //----------------------------------------------------------------
  // trace point = 121
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2806091f, 40.0463524f, 1491514865, 22.246376),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.280571f,
                                                40.0463295f, leg_index, 30.209856f, 2273,
                                                maneuver_index, 1.0616951f, 109));
  //----------------------------------------------------------------
  // trace point = 122
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2804413f, 40.0465088f, 1491514866, 22.5713902),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2804184f,
                                                40.0464973f, leg_index, 30.1957169f, 2272,
                                                maneuver_index, 1.04755592f, 108));
  //----------------------------------------------------------------
  // trace point = 123
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2802811f, 40.0466652f, 1491514867, 21.9741077),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2802811f,
                                                40.0466652f, leg_index, 30.1820202f, 2270,
                                                maneuver_index, 1.03385925f, 106));
  //----------------------------------------------------------------
  // trace point = 124 | Alert | In 1 mile Turn left to take the U.S. 30 West ramp.
  maneuver_index = 2;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2801208f, 40.0468292f, 1491514868, 22.9399128),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.2801285f, 40.046833f, leg_index, 30.1678772f,
                                                2269, maneuver_index, 1.01971626f, 105,
                                                instruction_index,
                                                kInitialShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 125
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2799683f, 40.0470047f, 1491514869, 23.4889374),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2799759f,
                                                40.0470085f, leg_index, 30.1532955f, 2268,
                                                maneuver_index, 1.00513458f, 104));
  //----------------------------------------------------------------
  // trace point = 126
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2798157f, 40.0471764f, 1491514870, 22.8818054),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2798233f,
                                                40.0471802f, leg_index, 30.1389351f, 2266,
                                                maneuver_index, 0.990774155f, 102));
  //----------------------------------------------------------------
  // trace point = 127
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.279686f, 40.0473785f, 1491514871, 25.319109),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2796631f,
                                                40.0473671f, leg_index, 30.1234703f, 2264,
                                                maneuver_index, 0.975309372f, 100));
  //----------------------------------------------------------------
  // trace point = 128
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2795486f, 40.0475616f, 1491514872, 23.468689),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2795258f,
                                                40.047554f, leg_index, 30.1085224f, 2263,
                                                maneuver_index, 0.960361481f, 99));
  //----------------------------------------------------------------
  // trace point = 129
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2794495f, 40.0477715f, 1491514873, 24.9004345),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2794113f,
                                                40.0477562f, leg_index, 30.0932827f, 2261,
                                                maneuver_index, 0.945121765f, 97));
  //----------------------------------------------------------------
  // trace point = 130
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2793655f, 40.0479736f, 1491514874, 23.4923229),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2793045f,
                                                40.0479546f, leg_index, 30.0784378f, 2259,
                                                maneuver_index, 0.930276871f, 95));
  //----------------------------------------------------------------
  // trace point = 131
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2792587f, 40.0481834f, 1491514875, 25.2999325),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2791977f,
                                                40.0481682f, leg_index, 30.0626068f, 2257,
                                                maneuver_index, 0.914445877f, 93));
  //----------------------------------------------------------------
  // trace point = 132
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2791519f, 40.0484085f, 1491514876, 26.3875523),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2790909f,
                                                40.0483932f, leg_index, 30.0460434f, 2255,
                                                maneuver_index, 0.897882462f, 91));
  //----------------------------------------------------------------
  // trace point = 133
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2790527f, 40.0486107f, 1491514877, 24.3157654),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2789993f,
                                                40.0485954f, leg_index, 30.0312424f, 2254,
                                                maneuver_index, 0.883081436f, 90));
  //----------------------------------------------------------------
  // trace point = 134
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2789383f, 40.0488091f, 1491514878, 23.9851379),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2789001f,
                                                40.0487976f, leg_index, 30.0163059f, 2252,
                                                maneuver_index, 0.868144989f, 88));
  //----------------------------------------------------------------
  // trace point = 135
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2788391f, 40.0490303f, 1491514879, 25.7421398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2787933f,
                                                40.049015f, leg_index, 30.000227f, 2250,
                                                maneuver_index, 0.85206604f, 86));
  //----------------------------------------------------------------
  // trace point = 136
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2787476f, 40.0492363f, 1491514880, 24.5570869),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2786942f,
                                                40.049221f, leg_index, 29.9850426f, 2249,
                                                maneuver_index, 0.836881638f, 85));
  //----------------------------------------------------------------
  // trace point = 137
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2786484f, 40.0494461f, 1491514881, 24.5855961),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2786026f,
                                                40.0494347f, leg_index, 29.9694939f, 2247,
                                                maneuver_index, 0.821332932f, 83));
  //----------------------------------------------------------------
  // trace point = 138
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2785568f, 40.0496483f, 1491514882, 23.8421497),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.278511f,
                                                40.0496368f, leg_index, 29.9546928f, 2246,
                                                maneuver_index, 0.806531906f, 82));
  //----------------------------------------------------------------
  // trace point = 139
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2784576f, 40.0498543f, 1491514883, 24.4237518),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2784119f,
                                                40.0498428f, leg_index, 29.9395084f, 2244,
                                                maneuver_index, 0.791347504f, 80));
  //----------------------------------------------------------------
  // trace point = 140
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2783661f, 40.0500488f, 1491514884, 23.1543503),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2783279f,
                                                40.0500374f, leg_index, 29.9253407f, 2242,
                                                maneuver_index, 0.777179718f, 78));
  //----------------------------------------------------------------
  // trace point = 141
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2782516f, 40.0502472f, 1491514885, 23.9515839),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2782211f,
                                                40.0502396f, leg_index, 29.9102573f, 2241,
                                                maneuver_index, 0.762096405f, 77));
  //----------------------------------------------------------------
  // trace point = 142
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2781525f, 40.0504341f, 1491514886, 22.5643768),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2781296f,
                                                40.0504265f, leg_index, 29.896452f, 2240,
                                                maneuver_index, 0.748291016f, 76));
  //----------------------------------------------------------------
  // trace point = 143
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2780533f, 40.0506248f, 1491514887, 22.6047802),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2780228f,
                                                40.0506172f, leg_index, 29.8821011f, 2239,
                                                maneuver_index, 0.733940125f, 75));
  //----------------------------------------------------------------
  // trace point = 144
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2779541f, 40.0508041f, 1491514888, 21.7741394),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2779312f,
                                                40.0507965f, leg_index, 29.8687897f, 2238,
                                                maneuver_index, 0.720628738f, 74));
  //----------------------------------------------------------------
  // trace point = 145
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2778625f, 40.0509872f, 1491514889, 22.1701565),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2778473f,
                                                40.0509834f, leg_index, 29.8551178f, 2236,
                                                maneuver_index, 0.706956863f, 72));
  //----------------------------------------------------------------
  // trace point = 146
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2777786f, 40.0511665f, 1491514890, 21.0221958),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2777634f,
                                                40.0511627f, leg_index, 29.8419437f, 2235,
                                                maneuver_index, 0.693782806f, 71));
  //----------------------------------------------------------------
  // trace point = 147
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2776871f, 40.051342f, 1491514891, 20.9674606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2776718f,
                                                40.0513344f, leg_index, 29.829113f, 2234,
                                                maneuver_index, 0.680952072f, 70));
  //----------------------------------------------------------------
  // trace point = 148
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2775803f, 40.051506f, 1491514892, 20.3309708),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2775574f,
                                                40.0514984f, leg_index, 29.816246f, 2232,
                                                maneuver_index, 0.668085098f, 68));
  //----------------------------------------------------------------
  // trace point = 149
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2774582f, 40.0516739f, 1491514893, 21.5917034),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2774277f,
                                                40.0516586f, leg_index, 29.8032017f, 2231,
                                                maneuver_index, 0.655040741f, 67));
  //----------------------------------------------------------------
  // trace point = 150
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2773132f, 40.0518303f, 1491514894, 21.4528503),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2772751f,
                                                40.0518074f, leg_index, 29.7885761f, 2230,
                                                maneuver_index, 0.640415192f, 66));
  //----------------------------------------------------------------
  // trace point = 151
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2771454f, 40.0519714f, 1491514895, 21.1147671),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2771149f,
                                                40.0519485f, leg_index, 29.7765026f, 2229,
                                                maneuver_index, 0.628341675f, 65));
  //----------------------------------------------------------------
  // trace point = 152
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2769547f, 40.0520973f, 1491514896, 21.4396534),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2769318f,
                                                40.0520744f, leg_index, 29.7641277f, 2228,
                                                maneuver_index, 0.615966797f, 64));
  //----------------------------------------------------------------
  // trace point = 153
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2767487f, 40.0522118f, 1491514897, 21.5223961),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2767258f,
                                                40.0521851f, leg_index, 29.7507858f, 2227,
                                                maneuver_index, 0.602624893f, 63));
  //----------------------------------------------------------------
  // trace point = 154
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.276535f, 40.0523148f, 1491514898, 21.6405926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2765121f,
                                                40.0522766f, leg_index, 29.7377949f, 2226,
                                                maneuver_index, 0.589633942f, 62));
  //----------------------------------------------------------------
  // trace point = 155
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2762985f, 40.0523911f, 1491514899, 21.7477551),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2762833f,
                                                40.0523567f, leg_index, 29.724474f, 2225,
                                                maneuver_index, 0.576313019f, 61));
  //----------------------------------------------------------------
  // trace point = 156
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2760544f, 40.0524483f, 1491514900, 21.7057343),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2760391f,
                                                40.0524178f, leg_index, 29.7108669f, 2224,
                                                maneuver_index, 0.562705994f, 60));
  //----------------------------------------------------------------
  // trace point = 157
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2758102f, 40.0524979f, 1491514901, 22.0176697),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2758026f,
                                                40.0524673f, leg_index, 29.6978836f, 2222,
                                                maneuver_index, 0.549722672f, 58));
  //----------------------------------------------------------------
  // trace point = 158
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2755508f, 40.0525475f, 1491514902, 22.4027615),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2755432f,
                                                40.0525169f, leg_index, 29.6837273f, 2220,
                                                maneuver_index, 0.53556633f, 56));
  //----------------------------------------------------------------
  // trace point = 159
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2752914f, 40.0525932f, 1491514903, 23.1213303),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2752838f,
                                                40.0525665f, leg_index, 29.6695728f, 2219,
                                                maneuver_index, 0.521411896f, 55));
  //----------------------------------------------------------------
  // trace point = 160
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2750244f, 40.0526428f, 1491514904, 22.8430634),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2750168f,
                                                40.0526199f, leg_index, 29.6549587f, 2217,
                                                maneuver_index, 0.506797791f, 53));
  //----------------------------------------------------------------
  // trace point = 161
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.274765f, 40.0527039f, 1491514905, 23.4345856),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2747574f,
                                                40.052681f, leg_index, 29.6405945f, 2215,
                                                maneuver_index, 0.492433548f, 51));
  //----------------------------------------------------------------
  // trace point = 162
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.274498f, 40.0527611f, 1491514906, 23.4345665),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2744904f,
                                                40.052742f, leg_index, 29.6258354f, 2213,
                                                maneuver_index, 0.477674484f, 49));
  //----------------------------------------------------------------
  // trace point = 163
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2742386f, 40.0528374f, 1491514907, 23.9562111),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2742233f,
                                                40.0528107f, leg_index, 29.6109238f, 2211,
                                                maneuver_index, 0.462762833f, 47));
  //----------------------------------------------------------------
  // trace point = 164
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2739716f, 40.0529137f, 1491514908, 24.2719917),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2739563f,
                                                40.052887f, leg_index, 29.5958347f, 2209,
                                                maneuver_index, 0.447673798f, 45));
  //----------------------------------------------------------------
  // trace point = 165
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2737045f, 40.0529976f, 1491514909, 24.1978645),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2736893f,
                                                40.0529747f, leg_index, 29.5804482f, 2208,
                                                maneuver_index, 0.432287216f, 44));
  //----------------------------------------------------------------
  // trace point = 166
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2734451f, 40.0530853f, 1491514910, 24.4576206),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2734299f,
                                                40.0530586f, leg_index, 29.5655384f, 2206,
                                                maneuver_index, 0.417377472f, 42));
  //----------------------------------------------------------------
  // trace point = 167
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2731781f, 40.0531693f, 1491514911, 24.6480808),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2731628f,
                                                40.0531502f, leg_index, 29.5500469f, 2205,
                                                maneuver_index, 0.401885986f, 41));
  //----------------------------------------------------------------
  // trace point = 168
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2729111f, 40.0532646f, 1491514912, 25.2284908),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2728958f,
                                                40.0532417f, leg_index, 29.5345554f, 2204,
                                                maneuver_index, 0.386394501f, 40));
  //----------------------------------------------------------------
  // trace point = 169
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2726364f, 40.0533524f, 1491514913, 25.0151672),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2726288f,
                                                40.0533333f, leg_index, 29.519062f, 2202,
                                                maneuver_index, 0.370901108f, 38));
  //----------------------------------------------------------------
  // trace point = 170
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2723694f, 40.0534477f, 1491514914, 25.2608261),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2723541f,
                                                40.0534286f, leg_index, 29.5030937f, 2200,
                                                maneuver_index, 0.354932785f, 36));
  //----------------------------------------------------------------
  // trace point = 171
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2720947f, 40.0535393f, 1491514915, 25.3061333),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2720871f,
                                                40.0535202f, leg_index, 29.4876022f, 2199,
                                                maneuver_index, 0.339441299f, 35));
  //----------------------------------------------------------------
  // trace point = 172
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2718201f, 40.0536308f, 1491514916, 25.4946537),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2718124f,
                                                40.0536156f, leg_index, 29.4716339f, 2197,
                                                maneuver_index, 0.323472977f, 33));
  //----------------------------------------------------------------
  // trace point = 173
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.271553f, 40.0537224f, 1491514917, 25.3386555),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2715454f,
                                                40.0537071f, leg_index, 29.4561424f, 2195,
                                                maneuver_index, 0.307981491f, 31));
  //----------------------------------------------------------------
  // trace point = 174
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2712708f, 40.0538139f, 1491514918, 25.7288952),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2712631f,
                                                40.0538025f, leg_index, 29.439806f, 2194,
                                                maneuver_index, 0.29164505f, 30));
  //----------------------------------------------------------------
  // trace point = 175 | Alert | In a quarter mile Turn left to take the U.S. 30 West ramp.
  maneuver_index = 2;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2709961f, 40.0539093f, 1491514919, 25.754179),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.2709885f, 40.053894f, leg_index, 29.4239464f,
                                                2192, maneuver_index, 0.275785446f, 28,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 176
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2707214f, 40.0540009f, 1491514920, 25.876524),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2707138f,
                                                40.0539894f, leg_index, 29.4079781f, 2190,
                                                maneuver_index, 0.259817123f, 26));
  //----------------------------------------------------------------
  // trace point = 177
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2704468f, 40.0540962f, 1491514921, 25.3517056),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2704391f,
                                                40.054081f, leg_index, 29.3921146f, 2189,
                                                maneuver_index, 0.243953705f, 25));
  //----------------------------------------------------------------
  // trace point = 178
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2701721f, 40.0541916f, 1491514922, 25.5991364),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2701645f,
                                                40.0541725f, leg_index, 29.3762531f, 2188,
                                                maneuver_index, 0.228092194f, 24));
  //----------------------------------------------------------------
  // trace point = 179
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2698975f, 40.0542831f, 1491514923, 25.8413277),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2698898f,
                                                40.0542641f, leg_index, 29.3603935f, 2186,
                                                maneuver_index, 0.21223259f, 22));
  //----------------------------------------------------------------
  // trace point = 180
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2696304f, 40.0543747f, 1491514924, 24.9038696),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2696152f,
                                                40.0543518f, leg_index, 29.3446369f, 2184,
                                                maneuver_index, 0.196475983f, 20));
  //----------------------------------------------------------------
  // trace point = 181
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2693634f, 40.0544624f, 1491514925, 24.5472298),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2693481f,
                                                40.0544395f, leg_index, 29.3292503f, 2183,
                                                maneuver_index, 0.181089401f, 19));
  //----------------------------------------------------------------
  // trace point = 182
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.269104f, 40.0545464f, 1491514926, 24.4396152),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2690964f,
                                                40.0545273f, leg_index, 29.3146057f, 2182,
                                                maneuver_index, 0.166444778f, 18));
  //----------------------------------------------------------------
  // trace point = 183 | Pre | Turn left to take the U.S. 30 West ramp toward New Holland,
  // Harrisburg.
  maneuver_index = 2;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.268837f, 40.0546341f, 1491514927, 24.6915874),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2688293f, 40.0546188f, leg_index, 29.2991142f,
                                                2180, maneuver_index, 0.150953293f, 16,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 184
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2685928f, 40.0547218f, 1491514928, 22.5982094),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2685776f,
                                                40.0547028f, leg_index, 29.2845745f, 2179,
                                                maneuver_index, 0.136413574f, 15));
  //----------------------------------------------------------------
  // trace point = 185
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2683487f, 40.0548019f, 1491514929, 22.6882324),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2683411f,
                                                40.0547829f, leg_index, 29.2708817f, 2177,
                                                maneuver_index, 0.122720718f, 13));
  //----------------------------------------------------------------
  // trace point = 186
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2681274f, 40.0548859f, 1491514930, 21.3586903),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2681122f,
                                                40.0548592f, leg_index, 29.2576637f, 2176,
                                                maneuver_index, 0.109502792f, 12));
  //----------------------------------------------------------------
  // trace point = 187
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2678986f, 40.0549698f, 1491514931, 21.3304443),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2678833f,
                                                40.0549393f, leg_index, 29.2443409f, 2174,
                                                maneuver_index, 0.0961799622f, 10));
  //----------------------------------------------------------------
  // trace point = 188
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2676697f, 40.0550499f, 1491514932, 21.5893955),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2676468f,
                                                40.0550156f, leg_index, 29.2307549f, 2173,
                                                maneuver_index, 0.0825939178f, 9));
  //----------------------------------------------------------------
  // trace point = 189
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2674484f, 40.0551262f, 1491514933, 20.5348244),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2674255f,
                                                40.0550919f, leg_index, 29.217907f, 2172,
                                                maneuver_index, 0.0697460175f, 8));
  //----------------------------------------------------------------
  // trace point = 190
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2672501f, 40.0552025f, 1491514934, 19.283287),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2672272f,
                                                40.0551605f, leg_index, 29.2063789f, 2170,
                                                maneuver_index, 0.0582180023f, 6));
  //----------------------------------------------------------------
  // trace point = 191
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2671356f, 40.0552864f, 1491514935, 13.2219992),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2670898f,
                                                40.0552101f, leg_index, 29.1983414f, 2169,
                                                maneuver_index, 0.0501804352f, 5));
  //----------------------------------------------------------------
  // trace point = 192
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2669525f, 40.0553398f, 1491514936, 16.4328423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2669144f,
                                                40.0552711f, leg_index, 29.1881351f, 2168,
                                                maneuver_index, 0.0399742126f, 4));
  //----------------------------------------------------------------
  // trace point = 193
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2667999f, 40.055397f, 1491514937, 14.8830099),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2667542f,
                                                40.0553246f, leg_index, 29.1788864f, 2167,
                                                maneuver_index, 0.0307254791f, 3));
  //----------------------------------------------------------------
  // trace point = 194
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2666397f, 40.0554504f, 1491514938, 15.0114679),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2666016f,
                                                40.0553818f, leg_index, 29.1698914f, 2166,
                                                maneuver_index, 0.021730423f, 2));
  //----------------------------------------------------------------
  // trace point = 195
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.26651f, 40.0554924f, 1491514939, 11.7230711),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2664719f,
                                                40.0554276f, leg_index, 29.1623306f, 2165,
                                                maneuver_index, 0.014169693f, 1));
  //----------------------------------------------------------------
  // trace point = 196
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2664108f, 40.0555229f, 1491514940, 9.19292641),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2663727f,
                                                40.0554619f, leg_index, 29.1565685f, 2165,
                                                maneuver_index, 0.00840759277f, 1));
  //----------------------------------------------------------------
  // trace point = 197
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2663269f, 40.0555763f, 1491514941, 9.20518112),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2662659f,
                                                40.0555916f, leg_index, 29.1422691f, 2163,
                                                maneuver_index, 0.369197845f, 39));
  //----------------------------------------------------------------
  // trace point = 198
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.266243f, 40.0556984f, 1491514943, 7.68738079),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2662964f,
                                                40.0556831f, leg_index, 29.1357327f, 2163,
                                                maneuver_index, 0.362661362f, 39));
  //----------------------------------------------------------------
  // trace point = 199
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.266304f, 40.0557785f, 1491514944, 10.4872665),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2663345f,
                                                40.0557671f, leg_index, 29.129591f, 2163,
                                                maneuver_index, 0.356519699f, 39));
  //----------------------------------------------------------------
  // trace point = 200
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2663422f, 40.0558624f, 1491514945, 10.0528841),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2663803f,
                                                40.055851f, leg_index, 29.1233006f, 2162,
                                                maneuver_index, 0.350229263f, 38));
  //----------------------------------------------------------------
  // trace point = 201
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2663879f, 40.0559578f, 1491514946, 11.1312971),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2664337f,
                                                40.0559464f, leg_index, 29.1161251f, 2161,
                                                maneuver_index, 0.343053818f, 37));
  //----------------------------------------------------------------
  // trace point = 202
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2664261f, 40.0560532f, 1491514947, 11.429307),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2664795f,
                                                40.0560379f, leg_index, 29.1093445f, 2160,
                                                maneuver_index, 0.336273193f, 36));
  //----------------------------------------------------------------
  // trace point = 203
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2664719f, 40.0561676f, 1491514948, 13.0192528),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2665329f,
                                                40.0561485f, leg_index, 29.1011868f, 2159,
                                                maneuver_index, 0.328115463f, 35));
  //----------------------------------------------------------------
  // trace point = 204
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2665787f, 40.0562935f, 1491514949, 16.6400719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2666016f,
                                                40.0562859f, leg_index, 29.0910187f, 2158,
                                                maneuver_index, 0.317947388f, 34));
  //----------------------------------------------------------------
  // trace point = 205
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2666321f, 40.0564041f, 1491514950, 13.4114885),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2666626f,
                                                40.0563927f, leg_index, 29.0829525f, 2157,
                                                maneuver_index, 0.30988121f, 33));
  //----------------------------------------------------------------
  // trace point = 206
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2667313f, 40.05653f, 1491514951, 16.0018921),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2667542f,
                                                40.0565186f, leg_index, 29.0729828f, 2156,
                                                maneuver_index, 0.299911499f, 32));
  //----------------------------------------------------------------
  // trace point = 207
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.266861f, 40.0566406f, 1491514952, 16.7911243),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2668762f,
                                                40.0566292f, leg_index, 29.0629425f, 2154,
                                                maneuver_index, 0.289871216f, 30));
  //----------------------------------------------------------------
  // trace point = 208
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2669907f, 40.0567474f, 1491514953, 16.5215111),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2670135f,
                                                40.0567245f, leg_index, 29.0521431f, 2153,
                                                maneuver_index, 0.279071808f, 29));
  //----------------------------------------------------------------
  // trace point = 209
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2671661f, 40.0568352f, 1491514954, 17.6597023),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2671814f,
                                                40.0568047f, leg_index, 29.042635f, 2152,
                                                maneuver_index, 0.269563675f, 28));
  //----------------------------------------------------------------
  // trace point = 210
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2673264f, 40.0568924f, 1491514955, 14.9024878),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2673569f,
                                                40.0568504f, leg_index, 29.0328197f, 2151,
                                                maneuver_index, 0.259748459f, 27));
  //----------------------------------------------------------------
  // trace point = 211
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2675018f, 40.0569496f, 1491514956, 16.0400772),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2675171f,
                                                40.0568733f, leg_index, 29.024189f, 2150,
                                                maneuver_index, 0.251117706f, 26));
  //----------------------------------------------------------------
  // trace point = 212
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2677231f, 40.0568962f, 1491514957, 19.8808594),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2677231f,
                                                40.056881f, leg_index, 29.0132542f, 2149,
                                                maneuver_index, 0.240182877f, 25));
  //----------------------------------------------------------------
  // trace point = 213
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2679291f, 40.0568962f, 1491514958, 17.7239513),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2679138f,
                                                40.0568581f, leg_index, 29.0030327f, 2148,
                                                maneuver_index, 0.229961395f, 24));
  //----------------------------------------------------------------
  // trace point = 214
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2680969f, 40.0568886f, 1491514959, 14.3493881),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2680664f,
                                                40.0568161f, leg_index, 28.9944496f, 2147,
                                                maneuver_index, 0.221378326f, 23));
  //----------------------------------------------------------------
  // trace point = 215
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2683105f, 40.0567436f, 1491514960, 23.8966675),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2682877f,
                                                40.0567207f, leg_index, 28.9809666f, 2146,
                                                maneuver_index, 0.207895279f, 22));
  //----------------------------------------------------------------
  // trace point = 216
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.268486f, 40.0566483f, 1491514961, 18.6294823),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2684708f,
                                                40.0566292f, leg_index, 28.9693871f, 2145,
                                                maneuver_index, 0.196315765f, 21));
  //----------------------------------------------------------------
  // trace point = 217
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2686615f, 40.0565567f, 1491514962, 18.159523),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2686462f,
                                                40.0565414f, leg_index, 28.9582939f, 2144,
                                                maneuver_index, 0.185222626f, 20));
  //----------------------------------------------------------------
  // trace point = 218
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2688522f, 40.056469f, 1491514963, 18.9221134),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2688446f,
                                                40.0564613f, leg_index, 28.9464054f, 2142,
                                                maneuver_index, 0.173334122f, 18));
  //----------------------------------------------------------------
  // trace point = 219
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2690659f, 40.0564079f, 1491514964, 19.5371552),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2690659f,
                                                40.0564041f, leg_index, 28.9340172f, 2141,
                                                maneuver_index, 0.160945892f, 17));
  //----------------------------------------------------------------
  // trace point = 220
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2692719f, 40.0563583f, 1491514965, 18.3154678),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2692719f,
                                                40.0563622f, leg_index, 28.9227314f, 2140,
                                                maneuver_index, 0.14966011f, 16));
  //----------------------------------------------------------------
  // trace point = 221
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2694855f, 40.0563354f, 1491514966, 18.2600231),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2694855f,
                                                40.0563469f, leg_index, 28.9113541f, 2139,
                                                maneuver_index, 0.138282776f, 15));
  //----------------------------------------------------------------
  // trace point = 222
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2696991f, 40.0563393f, 1491514967, 18.2352886),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2696991f,
                                                40.0563469f, leg_index, 28.900032f, 2137,
                                                maneuver_index, 0.126960754f, 13));
  //----------------------------------------------------------------
  // trace point = 223
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.269928f, 40.0563622f, 1491514968, 19.4408417),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.269928f,
                                                40.0563622f, leg_index, 28.8878593f, 2136,
                                                maneuver_index, 0.114788055f, 12));
  //----------------------------------------------------------------
  // trace point = 224
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2701492f, 40.0564079f, 1491514969, 19.5141926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2701492f,
                                                40.0564117f, leg_index, 28.8756351f, 2135,
                                                maneuver_index, 0.102563858f, 11));
  //----------------------------------------------------------------
  // trace point = 225
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2703552f, 40.0564766f, 1491514970, 19.5503769),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2703552f,
                                                40.0564766f, leg_index, 28.8638363f, 2134,
                                                maneuver_index, 0.0907649994f, 10));
  //----------------------------------------------------------------
  // trace point = 226 | Pre | Merge onto U.S. 30 West.
  maneuver_index = 3;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2705688f, 40.0565681f, 1491514971, 20.6166859),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2705688f, 40.056572f, leg_index, 28.8507385f,
                                                2133, maneuver_index, 0.0776672363f, 9,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 227
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2707748f, 40.0566788f, 1491514972, 21.1345158),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2707748f,
                                                40.0566826f, leg_index, 28.8374157f, 2131,
                                                maneuver_index, 0.0643444061f, 7));
  //----------------------------------------------------------------
  // trace point = 228
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2709885f, 40.0567894f, 1491514973, 22.0888653),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2709808f,
                                                40.0567932f, leg_index, 28.8240929f, 2130,
                                                maneuver_index, 0.0510215759f, 6));
  //----------------------------------------------------------------
  // trace point = 229
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2712021f, 40.0569077f, 1491514974, 22.4077148),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2712021f,
                                                40.0569115f, leg_index, 28.8098049f, 2128,
                                                maneuver_index, 0.0367336273f, 4));
  //----------------------------------------------------------------
  // trace point = 230
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2714233f, 40.0570297f, 1491514975, 23.6234512),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2714233f,
                                                40.0570259f, leg_index, 28.7956696f, 2127,
                                                maneuver_index, 0.0225982666f, 3));
  //----------------------------------------------------------------
  // trace point = 231
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2716522f, 40.0571518f, 1491514976, 23.7796974),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2716675f,
                                                40.0571365f, leg_index, 28.7806492f, 2125,
                                                maneuver_index, 0.00757789612f, 1));
  //----------------------------------------------------------------
  // trace point = 232
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2718887f, 40.0572777f, 1491514977, 24.0346909),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.271904f,
                                                40.0572586f, leg_index, 28.7655334f, 2124,
                                                maneuver_index, 2.73913956f, 161));
  //----------------------------------------------------------------
  // trace point = 233
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2721176f, 40.0574074f, 1491514978, 24.6361294),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2721405f,
                                                40.0573845f, leg_index, 28.7502823f, 2123,
                                                maneuver_index, 2.7238884f, 160));
  //----------------------------------------------------------------
  // trace point = 234
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2723618f, 40.0575371f, 1491514979, 25.3194714),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.272377f,
                                                40.057518f, leg_index, 28.734726f, 2122,
                                                maneuver_index, 2.70833206f, 159));
  //----------------------------------------------------------------
  // trace point = 235 | Post | Continue for 2.7 miles.
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2726135f, 40.0576706f, 1491514980, 25.9348812),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.2726288f, 40.0576515f, leg_index, 28.7185116f,
                                                2121, maneuver_index, 2.69211769f, 158,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 236
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2728577f, 40.0578041f, 1491514981, 26.0045929),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2728729f,
                                                40.057785f, leg_index, 28.7026253f, 2120,
                                                maneuver_index, 2.67623138f, 157));
  //----------------------------------------------------------------
  // trace point = 237
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2731171f, 40.0579453f, 1491514982, 26.6042767),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2731323f,
                                                40.0579262f, leg_index, 28.6857758f, 2119,
                                                maneuver_index, 2.65938187f, 156));
  //----------------------------------------------------------------
  // trace point = 238
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2733688f, 40.0580788f, 1491514983, 26.2844162),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2733841f,
                                                40.0580635f, leg_index, 28.6694069f, 2118,
                                                maneuver_index, 2.643013f, 155));
  //----------------------------------------------------------------
  // trace point = 239
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2736282f, 40.0582123f, 1491514984, 26.9192753),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2736435f,
                                                40.0581932f, leg_index, 28.6529961f, 2118,
                                                maneuver_index, 2.62660217f, 155));
  //----------------------------------------------------------------
  // trace point = 240
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2738953f, 40.0583534f, 1491514985, 27.3017635),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2739182f,
                                                40.0583344f, leg_index, 28.6354771f, 2116,
                                                maneuver_index, 2.60908318f, 153));
  //----------------------------------------------------------------
  // trace point = 241
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2741623f, 40.0584908f, 1491514986, 27.6537647),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2741776f,
                                                40.0584793f, leg_index, 28.6184731f, 2115,
                                                maneuver_index, 2.59207916f, 152));
  //----------------------------------------------------------------
  // trace point = 242
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.274437f, 40.0586395f, 1491514987, 28.226532),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2744446f,
                                                40.0586281f, leg_index, 28.6009884f, 2114,
                                                maneuver_index, 2.5745945f, 151));
  //----------------------------------------------------------------
  // trace point = 243
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2746964f, 40.0587921f, 1491514988, 28.079052),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2747116f,
                                                40.0587807f, leg_index, 28.5833454f, 2114,
                                                maneuver_index, 2.55695152f, 151));
  //----------------------------------------------------------------
  // trace point = 244
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2749557f, 40.0589447f, 1491514989, 28.0784492),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2749634f,
                                                40.0589333f, leg_index, 28.5663433f, 2113,
                                                maneuver_index, 2.53994942f, 150));
  //----------------------------------------------------------------
  // trace point = 245
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2752151f, 40.0590973f, 1491514990, 27.8781719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2752228f,
                                                40.0590897f, leg_index, 28.5488567f, 2112,
                                                maneuver_index, 2.52246284f, 149));
  //----------------------------------------------------------------
  // trace point = 246
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2754669f, 40.0592575f, 1491514991, 27.8258801),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2754745f,
                                                40.0592461f, leg_index, 28.5316887f, 2111,
                                                maneuver_index, 2.5052948f, 148));
  //----------------------------------------------------------------
  // trace point = 247
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2757111f, 40.0594215f, 1491514992, 27.5210114),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2757263f,
                                                40.0594025f, leg_index, 28.5145226f, 2110,
                                                maneuver_index, 2.48812866f, 147));
  //----------------------------------------------------------------
  // trace point = 248
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2759552f, 40.0595779f, 1491514993, 27.3033066),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2759705f,
                                                40.0595627f, leg_index, 28.4974899f, 2109,
                                                maneuver_index, 2.47109604f, 146));
  //----------------------------------------------------------------
  // trace point = 249
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2761993f, 40.059742f, 1491514994, 27.5853539),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2762146f,
                                                40.0597267f, leg_index, 28.4802914f, 2108,
                                                maneuver_index, 2.45389748f, 145));
  //----------------------------------------------------------------
  // trace point = 250
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2764435f, 40.0599098f, 1491514995, 28.1755733),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2764664f,
                                                40.0598907f, leg_index, 28.4627895f, 2107,
                                                maneuver_index, 2.43639565f, 144));
  //----------------------------------------------------------------
  // trace point = 251
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2766953f, 40.0600777f, 1491514996, 28.3451958),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2767181f,
                                                40.0600586f, leg_index, 28.4451141f, 2106,
                                                maneuver_index, 2.41872025f, 143));
  //----------------------------------------------------------------
  // trace point = 252
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.276947f, 40.0602455f, 1491514997, 28.2173309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2769623f,
                                                40.0602303f, leg_index, 28.4275646f, 2105,
                                                maneuver_index, 2.40117073f, 142));
  //----------------------------------------------------------------
  // trace point = 253
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2771912f, 40.0604172f, 1491514998, 28.3653412),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2772141f,
                                                40.0603981f, leg_index, 28.4098892f, 2104,
                                                maneuver_index, 2.38349533f, 141));
  //----------------------------------------------------------------
  // trace point = 254
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2774353f, 40.060585f, 1491514999, 28.100771),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2774582f,
                                                40.0605659f, leg_index, 28.3925152f, 2103,
                                                maneuver_index, 2.36612129f, 140));
  //----------------------------------------------------------------
  // trace point = 255
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2776871f, 40.0607452f, 1491515000, 27.8964615),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2777023f,
                                                40.0607338f, leg_index, 28.3751411f, 2102,
                                                maneuver_index, 2.34874725f, 139));
  //----------------------------------------------------------------
  // trace point = 256
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2779465f, 40.0609131f, 1491515001, 28.7311211),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2779541f,
                                                40.0609055f, leg_index, 28.3572903f, 2101,
                                                maneuver_index, 2.33089638f, 138));
  //----------------------------------------------------------------
  // trace point = 257
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2781982f, 40.0610771f, 1491515002, 27.9087906),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2781982f,
                                                40.0610733f, leg_index, 28.3399162f, 2100,
                                                maneuver_index, 2.31352234f, 137));
  //----------------------------------------------------------------
  // trace point = 258
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2784424f, 40.0612411f, 1491515003, 27.9419956),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2784424f,
                                                40.0612411f, leg_index, 28.3225422f, 2099,
                                                maneuver_index, 2.2961483f, 136));
  //----------------------------------------------------------------
  // trace point = 259
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2786865f, 40.0614128f, 1491515004, 28.0373535),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2786942f,
                                                40.061409f, leg_index, 28.3048668f, 2097,
                                                maneuver_index, 2.2784729f, 134));
  //----------------------------------------------------------------
  // trace point = 260
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2789307f, 40.061573f, 1491515005, 27.5849304),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2789307f,
                                                40.061573f, leg_index, 28.2879715f, 2096,
                                                maneuver_index, 2.26157761f, 133));
  //----------------------------------------------------------------
  // trace point = 261
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2791748f, 40.0617409f, 1491515006, 28.0699196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2791748f,
                                                40.0617371f, leg_index, 28.2707729f, 2095,
                                                maneuver_index, 2.24437904f, 132));
  //----------------------------------------------------------------
  // trace point = 262
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2794266f, 40.0619049f, 1491515007, 28.0698814),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2794266f,
                                                40.0619049f, leg_index, 28.2530994f, 2094,
                                                maneuver_index, 2.22670555f, 131));
  //----------------------------------------------------------------
  // trace point = 263
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2796707f, 40.0620728f, 1491515008, 28.1637783),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2796707f,
                                                40.0620728f, leg_index, 28.2357273f, 2093,
                                                maneuver_index, 2.20933342f, 130));
  //----------------------------------------------------------------
  // trace point = 264
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2799225f, 40.0622444f, 1491515009, 28.492075),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2799225f,
                                                40.0622406f, leg_index, 28.2180481f, 2092,
                                                maneuver_index, 2.19165421f, 129));
  //----------------------------------------------------------------
  // trace point = 265
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2801743f, 40.0624123f, 1491515010, 28.2806816),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2801743f,
                                                40.0624123f, leg_index, 28.2001972f, 2091,
                                                maneuver_index, 2.17380333f, 128));
  //----------------------------------------------------------------
  // trace point = 266
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2804184f, 40.0625839f, 1491515011, 28.3759918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.280426f,
                                                40.0625801f, leg_index, 28.1825218f, 2090,
                                                maneuver_index, 2.15612793f, 127));
  //----------------------------------------------------------------
  // trace point = 267
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2806625f, 40.0627441f, 1491515012, 27.4651775),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2806625f,
                                                40.0627403f, leg_index, 28.1658001f, 2089,
                                                maneuver_index, 2.1394062f, 126));
  //----------------------------------------------------------------
  // trace point = 268
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.280899f, 40.062912f, 1491515013, 27.296133),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2809067f,
                                                40.0629044f, leg_index, 28.1486034f, 2088,
                                                maneuver_index, 2.12220955f, 125));
  //----------------------------------------------------------------
  // trace point = 269
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2811279f, 40.0630684f, 1491515014, 26.5857525),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2811356f,
                                                40.0630608f, leg_index, 28.1323605f, 2087,
                                                maneuver_index, 2.10596657f, 124));
  //----------------------------------------------------------------
  // trace point = 270
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2813644f, 40.0632286f, 1491515015, 26.6496639),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2813721f,
                                                40.0632248f, leg_index, 28.1154633f, 2087,
                                                maneuver_index, 2.08906937f, 124));
  //----------------------------------------------------------------
  // trace point = 271
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.281601f, 40.0633888f, 1491515016, 26.8712101),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.281601f,
                                                40.0633888f, leg_index, 28.0988636f, 2086,
                                                maneuver_index, 2.07246971f, 123));
  //----------------------------------------------------------------
  // trace point = 272
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2818375f, 40.0635414f, 1491515017, 26.6894493),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2818298f,
                                                40.063549f, leg_index, 28.0824356f, 2085,
                                                maneuver_index, 2.05604172f, 122));
  //----------------------------------------------------------------
  // trace point = 273
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.282074f, 40.0637054f, 1491515018, 26.9455166),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2820663f,
                                                40.0637131f, leg_index, 28.0655403f, 2084,
                                                maneuver_index, 2.03914642f, 121));
  //----------------------------------------------------------------
  // trace point = 274
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2823181f, 40.0638657f, 1491515019, 27.263483),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2823105f,
                                                40.0638771f, leg_index, 28.0483418f, 2083,
                                                maneuver_index, 2.02194786f, 120));
  //----------------------------------------------------------------
  // trace point = 275
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2825546f, 40.0640335f, 1491515020, 27.4854889),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.282547f,
                                                40.0640373f, leg_index, 28.0316219f, 2082,
                                                maneuver_index, 2.00522804f, 119));
  //----------------------------------------------------------------
  // trace point = 276
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2827988f, 40.0641975f, 1491515021, 27.6232338),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2827911f,
                                                40.0642052f, leg_index, 28.0142479f, 2081,
                                                maneuver_index, 1.987854f, 118));
  //----------------------------------------------------------------
  // trace point = 277
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2830353f, 40.0643654f, 1491515022, 27.7610016),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2830353f,
                                                40.0643692f, leg_index, 27.9970512f, 2080,
                                                maneuver_index, 1.97065735f, 117));
  //----------------------------------------------------------------
  // trace point = 278
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.283287f, 40.0645294f, 1491515023, 27.8505459),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2832794f,
                                                40.064537f, leg_index, 27.9796772f, 2079,
                                                maneuver_index, 1.95328331f, 116));
  //----------------------------------------------------------------
  // trace point = 279
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2835312f, 40.0646973f, 1491515024, 28.2493782),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2835236f,
                                                40.0647049f, leg_index, 27.9623051f, 2078,
                                                maneuver_index, 1.93591118f, 115));
  //----------------------------------------------------------------
  // trace point = 280
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2837753f, 40.0648727f, 1491515025, 28.5133381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2837753f,
                                                40.0648727f, leg_index, 27.9446297f, 2077,
                                                maneuver_index, 1.91823578f, 114));
  //----------------------------------------------------------------
  // trace point = 281
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2840271f, 40.0650444f, 1491515026, 28.6005363),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2840271f,
                                                40.0650482f, leg_index, 27.9266033f, 2076,
                                                maneuver_index, 1.90020943f, 113));
  //----------------------------------------------------------------
  // trace point = 282
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2842789f, 40.0652122f, 1491515027, 28.4175625),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2842789f,
                                                40.0652122f, leg_index, 27.90909f, 2074,
                                                maneuver_index, 1.88269615f, 111));
  //----------------------------------------------------------------
  // trace point = 283
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2845306f, 40.0653877f, 1491515028, 28.9043293),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2845459f,
                                                40.0653763f, leg_index, 27.8909569f, 2074,
                                                maneuver_index, 1.86456299f, 111));
  //----------------------------------------------------------------
  // trace point = 284
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2847824f, 40.0655632f, 1491515029, 29.1163616),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2848053f,
                                                40.0655441f, leg_index, 27.8729744f, 2073,
                                                maneuver_index, 1.84658051f, 110));
  //----------------------------------------------------------------
  // trace point = 285
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2850342f, 40.0657349f, 1491515030, 29.0419579),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2850571f,
                                                40.065712f, leg_index, 27.8553009f, 2072,
                                                maneuver_index, 1.82890701f, 109));
  //----------------------------------------------------------------
  // trace point = 286
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2852936f, 40.0659065f, 1491515031, 28.7563457),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2853241f,
                                                40.0658798f, leg_index, 27.8370094f, 2071,
                                                maneuver_index, 1.81061554f, 108));
  //----------------------------------------------------------------
  // trace point = 287
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2855453f, 40.0660744f, 1491515032, 28.6575279),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2855759f,
                                                40.0660439f, leg_index, 27.8195076f, 2070,
                                                maneuver_index, 1.79311371f, 107));
  //----------------------------------------------------------------
  // trace point = 288
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2858047f, 40.0662346f, 1491515033, 28.4199715),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2858353f,
                                                40.0662041f, leg_index, 27.8018551f, 2069,
                                                maneuver_index, 1.7754612f, 106));
  //----------------------------------------------------------------
  // trace point = 289
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2860718f, 40.066391f, 1491515034, 28.8888931),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2861023f,
                                                40.0663605f, leg_index, 27.7840557f, 2068,
                                                maneuver_index, 1.75766182f, 105));
  //----------------------------------------------------------------
  // trace point = 290
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2863464f, 40.0665436f, 1491515035, 28.4206085),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.286377f,
                                                40.0665131f, leg_index, 27.7660847f, 2067,
                                                maneuver_index, 1.73969078f, 104));
  //----------------------------------------------------------------
  // trace point = 291
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2866211f, 40.0666847f, 1491515036, 28.2800922),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.286644f,
                                                40.0666618f, leg_index, 27.7486019f, 2065,
                                                maneuver_index, 1.72220802f, 102));
  //----------------------------------------------------------------
  // trace point = 292
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2868958f, 40.0668182f, 1491515037, 27.8938713),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.286911f,
                                                40.0667953f, leg_index, 27.7317104f, 2064,
                                                maneuver_index, 1.70531654f, 101));
  //----------------------------------------------------------------
  // trace point = 293
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2871704f, 40.0669518f, 1491515038, 27.8130016),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2871933f,
                                                40.0669289f, leg_index, 27.7141438f, 2063,
                                                maneuver_index, 1.68774986f, 100));
  //----------------------------------------------------------------
  // trace point = 294
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2874527f, 40.0670738f, 1491515039, 27.6335354),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.287468f,
                                                40.0670547f, leg_index, 27.6971798f, 2062,
                                                maneuver_index, 1.6707859f, 99));
  //----------------------------------------------------------------
  // trace point = 295
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2877426f, 40.0671844f, 1491515040, 27.4710732),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2877502f,
                                                40.0671692f, leg_index, 27.6802807f, 2061,
                                                maneuver_index, 1.6538868f, 98));
  //----------------------------------------------------------------
  // trace point = 296
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2880325f, 40.0672874f, 1491515041, 27.3139038),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2880478f,
                                                40.0672684f, leg_index, 27.6630993f, 2060,
                                                maneuver_index, 1.6367054f, 97));
  //----------------------------------------------------------------
  // trace point = 297
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2883301f, 40.0673828f, 1491515042, 27.2591305),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2883377f,
                                                40.0673676f, leg_index, 27.6462879f, 2059,
                                                maneuver_index, 1.61989403f, 96));
  //----------------------------------------------------------------
  // trace point = 298
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2886353f, 40.0674667f, 1491515043, 27.6498013),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2886353f,
                                                40.0674706f, leg_index, 27.6281567f, 2058,
                                                maneuver_index, 1.60176277f, 95));
  //----------------------------------------------------------------
  // trace point = 299
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2889328f, 40.0675545f, 1491515044, 27.4512253),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2889328f,
                                                40.0675583f, leg_index, 27.6121273f, 2056,
                                                maneuver_index, 1.58573341f, 93));
  //----------------------------------------------------------------
  // trace point = 300
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2892532f, 40.0676422f, 1491515045, 28.6896877),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2892532f,
                                                40.0676422f, leg_index, 27.5942001f, 2055,
                                                maneuver_index, 1.56780624f, 92));
  //----------------------------------------------------------------
  // trace point = 301
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.289566f, 40.0677223f, 1491515046, 28.1126518),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.289566f,
                                                40.0677223f, leg_index, 27.5767365f, 2054,
                                                maneuver_index, 1.55034256f, 91));
  //----------------------------------------------------------------
  // trace point = 302
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2898788f, 40.0677872f, 1491515047, 27.7184315),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2898788f,
                                                40.067791f, leg_index, 27.5595055f, 2053,
                                                maneuver_index, 1.53311157f, 90));
  //----------------------------------------------------------------
  // trace point = 303
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2901917f, 40.0678482f, 1491515048, 27.544445),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2901917f,
                                                40.067852f, leg_index, 27.5424175f, 2052,
                                                maneuver_index, 1.51602364f, 89));
  //----------------------------------------------------------------
  // trace point = 304
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2905121f, 40.0679016f, 1491515049, 27.7855473),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2905121f,
                                                40.0679054f, leg_index, 27.5250549f, 2051,
                                                maneuver_index, 1.49866104f, 88));
  //----------------------------------------------------------------
  // trace point = 305
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2908249f, 40.0679474f, 1491515050, 26.9665756),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2908249f,
                                                40.067955f, leg_index, 27.5081463f, 2050,
                                                maneuver_index, 1.4817524f, 87));
  //----------------------------------------------------------------
  // trace point = 306
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2911301f, 40.0679855f, 1491515051, 26.4758968),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2911301f,
                                                40.0679932f, leg_index, 27.4917698f, 2049,
                                                maneuver_index, 1.4653759f, 86));
  //----------------------------------------------------------------
  // trace point = 307
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2914352f, 40.0680199f, 1491515052, 26.4946423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2914352f,
                                                40.0680237f, leg_index, 27.4754753f, 2049,
                                                maneuver_index, 1.44908142f, 86));
  //----------------------------------------------------------------
  // trace point = 308
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.291748f, 40.0680466f, 1491515053, 26.6902466),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.291748f,
                                                40.0680504f, leg_index, 27.4588051f, 2048,
                                                maneuver_index, 1.43241119f, 85));
  //----------------------------------------------------------------
  // trace point = 309
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2920609f, 40.0680656f, 1491515054, 26.8247986),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2920609f,
                                                40.0680695f, leg_index, 27.4421921f, 2047,
                                                maneuver_index, 1.41579819f, 84));
  //----------------------------------------------------------------
  // trace point = 310
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2923737f, 40.0680809f, 1491515055, 26.4487076),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2923737f,
                                                40.0680809f, leg_index, 27.4256172f, 2046,
                                                maneuver_index, 1.39922333f, 83));
  //----------------------------------------------------------------
  // trace point = 311
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2926865f, 40.0680885f, 1491515056, 26.6878815),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2926865f,
                                                40.0680885f, leg_index, 27.4090405f, 2046,
                                                maneuver_index, 1.38264656f, 83));
  //----------------------------------------------------------------
  // trace point = 312
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2929993f, 40.0680923f, 1491515057, 27.0074787),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2929993f,
                                                40.0680847f, leg_index, 27.3924809f, 2045,
                                                maneuver_index, 1.36608696f, 82));
  //----------------------------------------------------------------
  // trace point = 313
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2933273f, 40.0680847f, 1491515058, 27.5347919),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2933273f,
                                                40.0680809f, leg_index, 27.3751125f, 2044,
                                                maneuver_index, 1.34871864f, 81));
  //----------------------------------------------------------------
  // trace point = 314
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2936478f, 40.0680733f, 1491515059, 27.3606777),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2936478f,
                                                40.0680809f, leg_index, 27.3581505f, 2043,
                                                maneuver_index, 1.33175659f, 80));
  //----------------------------------------------------------------
  // trace point = 315
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2939682f, 40.0680656f, 1491515060, 27.7012653),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2939682f,
                                                40.0680733f, leg_index, 27.3411808f, 2042,
                                                maneuver_index, 1.31478691f, 79));
  //----------------------------------------------------------------
  // trace point = 316
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2943039f, 40.0680618f, 1491515061, 28.1237011),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2943039f,
                                                40.0680695f, leg_index, 27.32341f, 2040,
                                                maneuver_index, 1.29701614f, 77));
  //----------------------------------------------------------------
  // trace point = 317
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2946243f, 40.0680542f, 1491515062, 27.6905327),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2946243f,
                                                40.0680656f, leg_index, 27.3064461f, 2039,
                                                maneuver_index, 1.28005219f, 76));
  //----------------------------------------------------------------
  // trace point = 318
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2949677f, 40.068058f, 1491515063, 28.9650269),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2949677f,
                                                40.068058f, leg_index, 27.2882652f, 2038,
                                                maneuver_index, 1.26187134f, 75));
  //----------------------------------------------------------------
  // trace point = 319
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2953033f, 40.0680504f, 1491515064, 28.7198143),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2953033f,
                                                40.0680542f, leg_index, 27.2704926f, 2037,
                                                maneuver_index, 1.24409866f, 74));
  //----------------------------------------------------------------
  // trace point = 320
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2956467f, 40.0680428f, 1491515065, 28.9785042),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2956467f,
                                                40.0680466f, leg_index, 27.2523117f, 2036,
                                                maneuver_index, 1.22591782f, 73));
  //----------------------------------------------------------------
  // trace point = 321
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2959824f, 40.0680313f, 1491515066, 29.0758076),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2959824f,
                                                40.0680428f, leg_index, 27.234539f, 2035,
                                                maneuver_index, 1.20814514f, 72));
  //----------------------------------------------------------------
  // trace point = 322
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2963257f, 40.0680237f, 1491515067, 29.2280293),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2963257f,
                                                40.0680428f, leg_index, 27.2163658f, 2034,
                                                maneuver_index, 1.18997192f, 71));
  //----------------------------------------------------------------
  // trace point = 323
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2966766f, 40.0680237f, 1491515068, 29.4778614),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2966766f,
                                                40.0680428f, leg_index, 27.1977882f, 2033,
                                                maneuver_index, 1.17139435f, 70));
  //----------------------------------------------------------------
  // trace point = 324
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2970276f, 40.0680161f, 1491515069, 29.9945526),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2970276f,
                                                40.0680428f, leg_index, 27.1792107f, 2032,
                                                maneuver_index, 1.15281677f, 69));
  //----------------------------------------------------------------
  // trace point = 325
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2973709f, 40.0680084f, 1491515070, 29.571434),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2973709f,
                                                40.0680466f, leg_index, 27.1610355f, 2030,
                                                maneuver_index, 1.13464165f, 67));
  //----------------------------------------------------------------
  // trace point = 326
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2977219f, 40.0680046f, 1491515071, 29.6482391),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2977219f,
                                                40.0680466f, leg_index, 27.142458f, 2029,
                                                maneuver_index, 1.11606407f, 66));
  //----------------------------------------------------------------
  // trace point = 327
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2980728f, 40.0680008f, 1491515072, 29.9052467),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2980728f,
                                                40.0680466f, leg_index, 27.1238804f, 2028,
                                                maneuver_index, 1.0974865f, 65));
  //----------------------------------------------------------------
  // trace point = 328
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2984238f, 40.0680084f, 1491515073, 30.1675682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2984238f,
                                                40.0680542f, leg_index, 27.1052933f, 2027,
                                                maneuver_index, 1.07889938f, 64));
  //----------------------------------------------------------------
  // trace point = 329
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2987823f, 40.0680199f, 1491515074, 30.1823502),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2987823f,
                                                40.0680656f, leg_index, 27.0862961f, 2026,
                                                maneuver_index, 1.05990219f, 63));
  //----------------------------------------------------------------
  // trace point = 330
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2991333f, 40.0680313f, 1491515075, 30.2772884),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2991333f,
                                                40.0680771f, leg_index, 27.0677013f, 2025,
                                                maneuver_index, 1.04130745f, 62));
  //----------------------------------------------------------------
  // trace point = 331 | Alert | In 1 mile Keep left to take Pennsylvania 2 83 West.
  maneuver_index = 4;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2994919f, 40.0680542f, 1491515076, 30.6643696),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.2994843f, 40.0680923f, leg_index, 27.0490875f,
                                                2024, maneuver_index, 1.02269363f, 61,
                                                instruction_index,
                                                kInitialShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 332
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2998505f, 40.0680809f, 1491515077, 30.6352673),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2998428f,
                                                40.068119f, leg_index, 27.0300159f, 2022,
                                                maneuver_index, 1.00362206f, 59));
  //----------------------------------------------------------------
  // trace point = 333
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.300209f, 40.0681076f, 1491515078, 30.5958424),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3002014f,
                                                40.0681458f, leg_index, 27.0109444f, 2021,
                                                maneuver_index, 0.984550476f, 58));
  //----------------------------------------------------------------
  // trace point = 334
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.30056f, 40.0681458f, 1491515079, 30.4925823),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.30056f,
                                                40.0681686f, leg_index, 26.9919014f, 2020,
                                                maneuver_index, 0.965507507f, 57));
  //----------------------------------------------------------------
  // trace point = 335
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3009186f, 40.0681877f, 1491515080, 30.7702847),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3009109f,
                                                40.0682068f, leg_index, 26.9731369f, 2019,
                                                maneuver_index, 0.946743011f, 56));
  //----------------------------------------------------------------
  // trace point = 336
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3012772f, 40.0682297f, 1491515081, 30.7387218),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3012772f,
                                                40.0682487f, leg_index, 26.9535351f, 2017,
                                                maneuver_index, 0.92714119f, 54));
  //----------------------------------------------------------------
  // trace point = 337
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3016281f, 40.0682755f, 1491515082, 30.3371143),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3016281f,
                                                40.0682869f, leg_index, 26.9347725f, 2016,
                                                maneuver_index, 0.908378601f, 53));
  //----------------------------------------------------------------
  // trace point = 338
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3019791f, 40.068325f, 1491515083, 30.6078606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3019791f,
                                                40.0683365f, leg_index, 26.9158764f, 2015,
                                                maneuver_index, 0.889482498f, 52));
  //----------------------------------------------------------------
  // trace point = 339
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3023376f, 40.0683708f, 1491515084, 30.5239067),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.30233f,
                                                40.0683899f, leg_index, 26.8969364f, 2014,
                                                maneuver_index, 0.870542526f, 51));
  //----------------------------------------------------------------
  // trace point = 340
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3026886f, 40.0684166f, 1491515085, 30.4024849),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.302681f,
                                                40.0684395f, leg_index, 26.878046f, 2013,
                                                maneuver_index, 0.851652145f, 50));
  //----------------------------------------------------------------
  // trace point = 341
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3030319f, 40.0684624f, 1491515086, 30.04002),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3030243f,
                                                40.0684853f, leg_index, 26.8595982f, 2012,
                                                maneuver_index, 0.833204269f, 49));
  //----------------------------------------------------------------
  // trace point = 342
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3033752f, 40.068512f, 1491515087, 29.7249603),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3033676f,
                                                40.0685349f, leg_index, 26.8411026f, 2011,
                                                maneuver_index, 0.81470871f, 48));
  //----------------------------------------------------------------
  // trace point = 343
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3037109f, 40.0685577f, 1491515088, 28.8071709),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3037033f,
                                                40.0685806f, leg_index, 26.8230534f, 2010,
                                                maneuver_index, 0.79665947f, 47));
  //----------------------------------------------------------------
  // trace point = 344
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3040466f, 40.0685997f, 1491515089, 28.9190865),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.304039f,
                                                40.0686302f, leg_index, 26.8049564f, 2009,
                                                maneuver_index, 0.778562546f, 46));
  //----------------------------------------------------------------
  // trace point = 345
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3043671f, 40.0686417f, 1491515090, 28.0787144),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3043594f,
                                                40.0686798f, leg_index, 26.7876511f, 2008,
                                                maneuver_index, 0.761257172f, 45));
  //----------------------------------------------------------------
  // trace point = 346
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3046951f, 40.0686874f, 1491515091, 28.0524063),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3046875f,
                                                40.0687256f, leg_index, 26.7700005f, 2007,
                                                maneuver_index, 0.743606567f, 44));
  //----------------------------------------------------------------
  // trace point = 347
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3050156f, 40.0687294f, 1491515092, 28.1813774),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3050079f,
                                                40.0687752f, leg_index, 26.7526951f, 2006,
                                                maneuver_index, 0.726301193f, 43));
  //----------------------------------------------------------------
  // trace point = 348
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3053436f, 40.068779f, 1491515093, 28.4696674),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.305336f,
                                                40.068821f, leg_index, 26.7350445f, 2005,
                                                maneuver_index, 0.708650589f, 42));
  //----------------------------------------------------------------
  // trace point = 349
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3056717f, 40.0688248f, 1491515094, 28.3682671),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3056641f,
                                                40.0688705f, leg_index, 26.7173424f, 2004,
                                                maneuver_index, 0.690948486f, 41));
  //----------------------------------------------------------------
  // trace point = 350
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3060074f, 40.0688705f, 1491515095, 28.9104481),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3059921f,
                                                40.0689201f, leg_index, 26.6996422f, 2003,
                                                maneuver_index, 0.673248291f, 40));
  //----------------------------------------------------------------
  // trace point = 351
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3063431f, 40.0689163f, 1491515096, 28.6790371),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3063278f,
                                                40.0689697f, leg_index, 26.6815434f, 2002,
                                                maneuver_index, 0.65514946f, 39));
  //----------------------------------------------------------------
  // trace point = 352
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3066711f, 40.0689621f, 1491515097, 28.679018),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3066559f,
                                                40.0690155f, leg_index, 26.6638927f, 2001,
                                                maneuver_index, 0.637498856f, 38));
  //----------------------------------------------------------------
  // trace point = 353
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3069992f, 40.0690079f, 1491515098, 28.7038937),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3069839f,
                                                40.0690651f, leg_index, 26.6461906f, 2000,
                                                maneuver_index, 0.619796753f, 37));
  //----------------------------------------------------------------
  // trace point = 354
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3073273f, 40.0690498f, 1491515099, 28.2652035),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.307312f,
                                                40.0691109f, leg_index, 26.6285381f, 1998,
                                                maneuver_index, 0.602144241f, 35));
  //----------------------------------------------------------------
  // trace point = 355
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3076553f, 40.0690956f, 1491515100, 28.5114594),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3076401f,
                                                40.0691528f, leg_index, 26.6109314f, 1997,
                                                maneuver_index, 0.584537506f, 34));
  //----------------------------------------------------------------
  // trace point = 356
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.307991f, 40.0691452f, 1491515101, 28.5531349),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3079758f,
                                                40.0691986f, leg_index, 26.5928822f, 1996,
                                                maneuver_index, 0.566488266f, 33));
  //----------------------------------------------------------------
  // trace point = 357
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3083191f, 40.0691948f, 1491515102, 28.9505787),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3083115f,
                                                40.0692406f, leg_index, 26.5748768f, 1995,
                                                maneuver_index, 0.548482895f, 32));
  //----------------------------------------------------------------
  // trace point = 358 | Alert | In a half mile Keep left to take Pennsylvania 2 83 West.
  maneuver_index = 4;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3086548f, 40.0692406f, 1491515103, 28.7426262),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.3086472f, 40.0692825f, leg_index, 26.5568733f,
                                                1994, maneuver_index, 0.530479431f, 31,
                                                instruction_index,
                                                kFinalLongTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 359
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3089828f, 40.0692863f, 1491515104, 28.6788845),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3089752f,
                                                40.0693245f, leg_index, 26.5392666f, 1993,
                                                maneuver_index, 0.512872696f, 30));
  //----------------------------------------------------------------
  // trace point = 360
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3093185f, 40.0693321f, 1491515105, 29.1618195),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3093109f,
                                                40.0693703f, leg_index, 26.5212173f, 1992,
                                                maneuver_index, 0.494823456f, 29));
  //----------------------------------------------------------------
  // trace point = 361
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3096619f, 40.0693779f, 1491515106, 29.1890469),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3096542f,
                                                40.069416f, leg_index, 26.5027714f, 1991,
                                                maneuver_index, 0.476377487f, 28));
  //----------------------------------------------------------------
  // trace point = 362
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3099976f, 40.0694275f, 1491515107, 29.2432556),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3099899f,
                                                40.0694656f, leg_index, 26.4846745f, 1990,
                                                maneuver_index, 0.458280563f, 27));
  //----------------------------------------------------------------
  // trace point = 363
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3103409f, 40.0694771f, 1491515108, 29.6615028),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3103333f,
                                                40.0695152f, leg_index, 26.4661808f, 1989,
                                                maneuver_index, 0.439786911f, 26));
  //----------------------------------------------------------------
  // trace point = 364
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3106842f, 40.0695229f, 1491515109, 29.9168301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3106766f,
                                                40.0695648f, leg_index, 26.4476852f, 1988,
                                                maneuver_index, 0.421291351f, 25));
  //----------------------------------------------------------------
  // trace point = 365
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3110275f, 40.0695724f, 1491515110, 29.8288307),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3110199f,
                                                40.0696106f, leg_index, 26.4292393f, 1987,
                                                maneuver_index, 0.402845383f, 24));
  //----------------------------------------------------------------
  // trace point = 366
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3113785f, 40.069622f, 1491515111, 30.0799255),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3113708f,
                                                40.069664f, leg_index, 26.4103012f, 1986,
                                                maneuver_index, 0.383907318f, 23));
  //----------------------------------------------------------------
  // trace point = 367
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3117218f, 40.0696716f, 1491515112, 30.1007023),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3117142f,
                                                40.0697098f, leg_index, 26.3918533f, 1984,
                                                maneuver_index, 0.365459442f, 21));
  //----------------------------------------------------------------
  // trace point = 368
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3120804f, 40.0697174f, 1491515113, 30.6724091),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3120728f,
                                                40.0697594f, leg_index, 26.3725643f, 1983,
                                                maneuver_index, 0.346170425f, 20));
  //----------------------------------------------------------------
  // trace point = 369
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3124313f, 40.069767f, 1491515114, 30.290987),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3124237f,
                                                40.0698051f, leg_index, 26.3537197f, 1982,
                                                maneuver_index, 0.327325821f, 19));
  //----------------------------------------------------------------
  // trace point = 370
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3127823f, 40.0698166f, 1491515115, 30.6460533),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3127747f,
                                                40.0698547f, leg_index, 26.3348293f, 1981,
                                                maneuver_index, 0.30843544f, 18));
  //----------------------------------------------------------------
  // trace point = 371
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3131409f, 40.0698624f, 1491515116, 30.9429817),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3131332f,
                                                40.0699043f, leg_index, 26.3155403f, 1980,
                                                maneuver_index, 0.289146423f, 17));
  //----------------------------------------------------------------
  // trace point = 372
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3134918f, 40.069912f, 1491515117, 30.7497978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3134842f,
                                                40.0699501f, leg_index, 26.2966976f, 1978,
                                                maneuver_index, 0.270303726f, 15));
  //----------------------------------------------------------------
  // trace point = 373
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3138504f, 40.0699654f, 1491515118, 30.7908535),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3138428f,
                                                40.0699997f, leg_index, 26.2774086f, 1977,
                                                maneuver_index, 0.251014709f, 14));
  //----------------------------------------------------------------
  // trace point = 374
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.314209f, 40.070015f, 1491515119, 30.9813385),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3142014f,
                                                40.0700493f, leg_index, 26.2581196f, 1976,
                                                maneuver_index, 0.231725693f, 13));
  //----------------------------------------------------------------
  // trace point = 375
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3145676f, 40.0700645f, 1491515120, 30.9375439),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3145599f,
                                                40.0701027f, leg_index, 26.238781f, 1975,
                                                maneuver_index, 0.212387085f, 12));
  //----------------------------------------------------------------
  // trace point = 376
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3149185f, 40.0701141f, 1491515121, 30.8537827),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3149033f,
                                                40.0701599f, leg_index, 26.2201843f, 1974,
                                                maneuver_index, 0.193790436f, 11));
  //----------------------------------------------------------------
  // trace point = 377
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3152771f, 40.0701637f, 1491515122, 30.8974171),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3152618f,
                                                40.0702133f, leg_index, 26.2008476f, 1973,
                                                maneuver_index, 0.174453735f, 10));
  //----------------------------------------------------------------
  // trace point = 378 | Pre | Keep left to take Pennsylvania 2 83 West toward Harrisburg.
  maneuver_index = 4;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3156281f, 40.0702133f, 1491515123, 30.163393),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.3156128f, 40.0702667f, leg_index, 26.1819077f,
                                                1972, maneuver_index, 0.155513763f, 9,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 379
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.315979f, 40.0702667f, 1491515124, 30.4351521),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3159637f,
                                                40.0703239f, leg_index, 26.1629143f, 1971,
                                                maneuver_index, 0.136520386f, 8));
  //----------------------------------------------------------------
  // trace point = 380
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3163223f, 40.0703125f, 1491515125, 29.7680092),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3163071f,
                                                40.0703735f, leg_index, 26.1444187f, 1970,
                                                maneuver_index, 0.118024826f, 7));
  //----------------------------------------------------------------
  // trace point = 381
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3166656f, 40.0703583f, 1491515126, 30.1034966),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3166504f,
                                                40.0704231f, leg_index, 26.125927f, 1969,
                                                maneuver_index, 0.0995330811f, 6));
  //----------------------------------------------------------------
  // trace point = 382
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3170166f, 40.0704041f, 1491515127, 30.2153435),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3170013f,
                                                40.0704689f, leg_index, 26.1070824f, 1968,
                                                maneuver_index, 0.0806884766f, 5));
  //----------------------------------------------------------------
  // trace point = 383
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3173676f, 40.0704498f, 1491515128, 29.9815674),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3173523f,
                                                40.0705147f, leg_index, 26.0882378f, 1967,
                                                maneuver_index, 0.0618438721f, 4));
  //----------------------------------------------------------------
  // trace point = 384
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3177109f, 40.0704956f, 1491515129, 29.8715076),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3176956f,
                                                40.0705566f, leg_index, 26.0698357f, 1966,
                                                maneuver_index, 0.0434417725f, 3));
  //----------------------------------------------------------------
  // trace point = 385
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3180542f, 40.0705414f, 1491515130, 29.8135624),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3180389f,
                                                40.0706024f, leg_index, 26.0513878f, 1964,
                                                maneuver_index, 0.0249938965f, 1));
  //----------------------------------------------------------------
  // trace point = 386
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3184052f, 40.0705833f, 1491515131, 30.0116673),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3183899f,
                                                40.0706482f, leg_index, 26.0325432f, 1963,
                                                maneuver_index, 0.00614929199f, 0));
  //----------------------------------------------------------------
  // trace point = 387
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3187485f, 40.0706253f, 1491515132, 29.8261299),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3187408f,
                                                40.0706787f, leg_index, 26.0138321f, 1962,
                                                maneuver_index, 16.8315315f, 955));
  //----------------------------------------------------------------
  // trace point = 388
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3190994f, 40.0706673f, 1491515133, 30.0786896),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3190918f,
                                                40.0706978f, leg_index, 25.9952087f, 1961,
                                                maneuver_index, 16.8129082f, 954));
  //----------------------------------------------------------------
  // trace point = 389 | Post | Continue for 16.8 miles.
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3194504f, 40.0707016f, 1491515134, 30.3369904),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.3194427f, 40.0707283f, leg_index, 25.9765129f,
                                                1960, maneuver_index, 16.7942123f, 953,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 390
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.319809f, 40.0707359f, 1491515135, 30.6464787),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.319809f,
                                                40.070755f, leg_index, 25.957037f, 1959,
                                                maneuver_index, 16.7747364f, 952));
  //----------------------------------------------------------------
  // trace point = 391
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3201599f, 40.0707664f, 1491515136, 30.1713085),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3201599f,
                                                40.0707779f, leg_index, 25.9383945f, 1958,
                                                maneuver_index, 16.756094f, 951));
  //----------------------------------------------------------------
  // trace point = 392
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3205109f, 40.0707932f, 1491515137, 30.2438755),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3205109f,
                                                40.0708046f, leg_index, 25.9197273f, 1957,
                                                maneuver_index, 16.7374268f, 950));
  //----------------------------------------------------------------
  // trace point = 393
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3208694f, 40.0708275f, 1491515138, 30.2813301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3208694f,
                                                40.0708389f, leg_index, 25.9005985f, 1956,
                                                maneuver_index, 16.718298f, 949));
  //----------------------------------------------------------------
  // trace point = 394
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3212128f, 40.0708542f, 1491515139, 29.8203812),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3212128f,
                                                40.0708694f, leg_index, 25.8823032f, 1955,
                                                maneuver_index, 16.7000027f, 948));
  //----------------------------------------------------------------
  // trace point = 395
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3215637f, 40.0708847f, 1491515140, 29.9172783),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3215637f,
                                                40.0709038f, leg_index, 25.863575f, 1954,
                                                maneuver_index, 16.6812744f, 947));
  //----------------------------------------------------------------
  // trace point = 396
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3219147f, 40.0709114f, 1491515141, 29.9665852),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3219147f,
                                                40.0709343f, leg_index, 25.8448792f, 1953,
                                                maneuver_index, 16.6625786f, 946));
  //----------------------------------------------------------------
  // trace point = 397
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3222656f, 40.0709419f, 1491515142, 29.9172535),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3222656f,
                                                40.0709648f, leg_index, 25.8261814f, 1952,
                                                maneuver_index, 16.6438808f, 945));
  //----------------------------------------------------------------
  // trace point = 398
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3226089f, 40.0709763f, 1491515143, 29.8857079),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3226089f,
                                                40.0709953f, leg_index, 25.8078861f, 1951,
                                                maneuver_index, 16.6255856f, 944));
  //----------------------------------------------------------------
  // trace point = 399
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3229599f, 40.0710182f, 1491515144, 30.1131248),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3229599f,
                                                40.0710258f, leg_index, 25.7891922f, 1950,
                                                maneuver_index, 16.6068916f, 943));
  //----------------------------------------------------------------
  // trace point = 400
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3233032f, 40.0710716f, 1491515145, 29.9541206),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3233032f,
                                                40.0710716f, leg_index, 25.7707138f, 1948,
                                                maneuver_index, 16.5884132f, 941));
  //----------------------------------------------------------------
  // trace point = 401
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3236542f, 40.0711365f, 1491515146, 30.5719223),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3236542f,
                                                40.0711365f, leg_index, 25.7516022f, 1947,
                                                maneuver_index, 16.5693016f, 940));
  //----------------------------------------------------------------
  // trace point = 402
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3240051f, 40.071209f, 1491515147, 30.7093658),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3239975f,
                                                40.0712204f, leg_index, 25.7325153f, 1946,
                                                maneuver_index, 16.5502148f, 939));
  //----------------------------------------------------------------
  // trace point = 403
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3243408f, 40.0712852f, 1491515148, 30.3506546),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3243332f,
                                                40.0713005f, leg_index, 25.713892f, 1946,
                                                maneuver_index, 16.5315914f, 939));
  //----------------------------------------------------------------
  // trace point = 404
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3246765f, 40.0713692f, 1491515149, 29.603384),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3246765f,
                                                40.0713692f, leg_index, 25.6951103f, 1945,
                                                maneuver_index, 16.5128098f, 938));
  //----------------------------------------------------------------
  // trace point = 405
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3250198f, 40.0714417f, 1491515150, 30.5219746),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3250198f,
                                                40.0714455f, leg_index, 25.6761856f, 1944,
                                                maneuver_index, 16.493885f, 937));
  //----------------------------------------------------------------
  // trace point = 406
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3253784f, 40.0715256f, 1491515151, 31.7352295),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3253784f,
                                                40.0715332f, leg_index, 25.6562538f, 1942,
                                                maneuver_index, 16.4739532f, 935));
  //----------------------------------------------------------------
  // trace point = 407
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3257294f, 40.0716095f, 1491515152, 31.2953777),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3257217f,
                                                40.0716248f, leg_index, 25.6370106f, 1941,
                                                maneuver_index, 16.45471f, 934));
  //----------------------------------------------------------------
  // trace point = 408
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3260727f, 40.0716896f, 1491515153, 30.9535389),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3260651f,
                                                40.0717125f, leg_index, 25.6178513f, 1940,
                                                maneuver_index, 16.4355507f, 933));
  //----------------------------------------------------------------
  // trace point = 409
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3264236f, 40.0717697f, 1491515154, 31.0850372),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3264084f,
                                                40.0718002f, leg_index, 25.5986919f, 1939,
                                                maneuver_index, 16.4163914f, 932));
  //----------------------------------------------------------------
  // trace point = 410
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3267746f, 40.0718498f, 1491515155, 30.9368572),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3267593f,
                                                40.0718918f, leg_index, 25.5790653f, 1938,
                                                maneuver_index, 16.3967648f, 931));
  //----------------------------------------------------------------
  // trace point = 411
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3271027f, 40.0719299f, 1491515156, 29.9278946),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3270798f,
                                                40.0719719f, leg_index, 25.5612221f, 1936,
                                                maneuver_index, 16.3789215f, 929));
  //----------------------------------------------------------------
  // trace point = 412
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3274307f, 40.0720177f, 1491515157, 29.5280476),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3274078f,
                                                40.0720634f, leg_index, 25.5427361f, 1935,
                                                maneuver_index, 16.3604355f, 928));
  //----------------------------------------------------------------
  // trace point = 413
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3277512f, 40.0721169f, 1491515158, 29.1618214),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3277283f,
                                                40.0721588f, leg_index, 25.524538f, 1934,
                                                maneuver_index, 16.3422375f, 927));
  //----------------------------------------------------------------
  // trace point = 414
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3280716f, 40.0722122f, 1491515159, 29.5195351),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3280487f,
                                                40.0722504f, leg_index, 25.5064335f, 1933,
                                                maneuver_index, 16.3241329f, 926));
  //----------------------------------------------------------------
  // trace point = 415
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3283997f, 40.0723152f, 1491515160, 29.997591),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3283691f,
                                                40.072361f, leg_index, 25.4878254f, 1932,
                                                maneuver_index, 16.3055248f, 925));
  //----------------------------------------------------------------
  // trace point = 416
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3287277f, 40.0724182f, 1491515161, 30.0949688),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3286972f,
                                                40.0724716f, leg_index, 25.4688473f, 1931,
                                                maneuver_index, 16.2865467f, 924));
  //----------------------------------------------------------------
  // trace point = 417
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3290482f, 40.0725327f, 1491515162, 30.3174877),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3290176f,
                                                40.0725822f, leg_index, 25.4502373f, 1930,
                                                maneuver_index, 16.2679367f, 923));
  //----------------------------------------------------------------
  // trace point = 418
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3293762f, 40.0726471f, 1491515163, 30.4894657),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3293457f,
                                                40.0727005f, leg_index, 25.4310436f, 1929,
                                                maneuver_index, 16.2487431f, 922));
  //----------------------------------------------------------------
  // trace point = 419
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.329689f, 40.0727692f, 1491515164, 30.1003628),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3296585f,
                                                40.0728149f, leg_index, 25.4126892f, 1928,
                                                maneuver_index, 16.2303886f, 921));
  //----------------------------------------------------------------
  // trace point = 420
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3300018f, 40.0728951f, 1491515165, 30.2512379),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3299713f,
                                                40.072937f, leg_index, 25.394104f, 1927,
                                                maneuver_index, 16.2118034f, 920));
  //----------------------------------------------------------------
  // trace point = 421
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3303146f, 40.0730171f, 1491515166, 29.5710697),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3302841f,
                                                40.0730629f, leg_index, 25.3753929f, 1927,
                                                maneuver_index, 16.1930923f, 920));
  //----------------------------------------------------------------
  // trace point = 422
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3306198f, 40.0731468f, 1491515167, 30.0811329),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3305817f,
                                                40.0731926f, leg_index, 25.3572598f, 1926,
                                                maneuver_index, 16.1749592f, 919));
  //----------------------------------------------------------------
  // trace point = 423
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3309326f, 40.0732803f, 1491515168, 30.3289604),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3308945f,
                                                40.0733337f, leg_index, 25.3380375f, 1925,
                                                maneuver_index, 16.1557369f, 918));
  //----------------------------------------------------------------
  // trace point = 424
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3312378f, 40.0734177f, 1491515169, 30.3471413),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.331192f,
                                                40.0734711f, leg_index, 25.3196468f, 1923,
                                                maneuver_index, 16.1373463f, 916));
  //----------------------------------------------------------------
  // trace point = 425
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.331543f, 40.0735588f, 1491515170, 30.4119835),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3315048f,
                                                40.0736122f, leg_index, 25.3004227f, 1923,
                                                maneuver_index, 16.1181221f, 916));
  //----------------------------------------------------------------
  // trace point = 426
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3318481f, 40.0737038f, 1491515171, 30.2523403),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.33181f,
                                                40.0737495f, leg_index, 25.2816849f, 1922,
                                                maneuver_index, 16.0993843f, 915));
  //----------------------------------------------------------------
  // trace point = 427
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3321609f, 40.0738487f, 1491515172, 30.9494019),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3321228f,
                                                40.0738945f, leg_index, 25.2623272f, 1921,
                                                maneuver_index, 16.0800266f, 914));
  //----------------------------------------------------------------
  // trace point = 428
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3324738f, 40.0739899f, 1491515173, 30.9798565),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3324356f,
                                                40.0740395f, leg_index, 25.2429714f, 1920,
                                                maneuver_index, 16.0606709f, 913));
  //----------------------------------------------------------------
  // trace point = 429
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3327866f, 40.0741386f, 1491515174, 31.4288406),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3327484f,
                                                40.0741844f, leg_index, 25.2236156f, 1919,
                                                maneuver_index, 16.0413151f, 912));
  //----------------------------------------------------------------
  // trace point = 430
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3330994f, 40.0742798f, 1491515175, 31.2725582),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3330612f,
                                                40.0743256f, leg_index, 25.2043953f, 1918,
                                                maneuver_index, 16.0220947f, 911));
  //----------------------------------------------------------------
  // trace point = 431
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3334198f, 40.0744247f, 1491515176, 31.4025383),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3333817f,
                                                40.0744743f, leg_index, 25.184557f, 1916,
                                                maneuver_index, 16.0022564f, 909));
  //----------------------------------------------------------------
  // trace point = 432
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3337326f, 40.0745697f, 1491515177, 31.589735),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3336945f,
                                                40.0746193f, leg_index, 25.1652012f, 1915,
                                                maneuver_index, 15.9829006f, 908));
  //----------------------------------------------------------------
  // trace point = 433
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.334053f, 40.0747185f, 1491515178, 31.8354702),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3340149f,
                                                40.0747681f, leg_index, 25.1453629f, 1914,
                                                maneuver_index, 15.9630623f, 907));
  //----------------------------------------------------------------
  // trace point = 434
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3343735f, 40.0748634f, 1491515179, 31.4593601),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3343277f,
                                                40.0749168f, leg_index, 25.1258659f, 1913,
                                                maneuver_index, 15.9435654f, 906));
  //----------------------------------------------------------------
  // trace point = 435
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3346863f, 40.0750122f, 1491515180, 31.545042),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3346405f,
                                                40.0750656f, leg_index, 25.1063709f, 1912,
                                                maneuver_index, 15.9240704f, 905));
  //----------------------------------------------------------------
  // trace point = 436
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3349991f, 40.0751572f, 1491515181, 30.9064198),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3349533f,
                                                40.0752144f, leg_index, 25.086874f, 1911,
                                                maneuver_index, 15.9045734f, 904));
  //----------------------------------------------------------------
  // trace point = 437
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3353043f, 40.0752983f, 1491515182, 30.499649),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3352585f,
                                                40.0753555f, leg_index, 25.0680008f, 1910,
                                                maneuver_index, 15.8857002f, 903));
  //----------------------------------------------------------------
  // trace point = 438
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3356094f, 40.0754395f, 1491515183, 30.4265995),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3355713f,
                                                40.0754929f, leg_index, 25.0489063f, 1908,
                                                maneuver_index, 15.8666058f, 901));
  //----------------------------------------------------------------
  // trace point = 439
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.335907f, 40.0755768f, 1491515184, 29.7138939),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3358765f,
                                                40.0756187f, leg_index, 25.0305576f, 1907,
                                                maneuver_index, 15.8482571f, 900));
  //----------------------------------------------------------------
  // trace point = 440
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3362045f, 40.0757179f, 1491515185, 29.7283134),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.336174f,
                                                40.0757523f, leg_index, 25.0122852f, 1907,
                                                maneuver_index, 15.8299847f, 900));
  //----------------------------------------------------------------
  // trace point = 441
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3365021f, 40.0758514f, 1491515186, 29.5558224),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3364716f,
                                                40.0758934f, leg_index, 24.9937572f, 1905,
                                                maneuver_index, 15.8114567f, 898));
  //----------------------------------------------------------------
  // trace point = 442
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3368073f, 40.0759888f, 1491515187, 29.9223595),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3367691f,
                                                40.0760307f, leg_index, 24.9753647f, 1904,
                                                maneuver_index, 15.7930641f, 897));
  //----------------------------------------------------------------
  // trace point = 443
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3371048f, 40.0761261f, 1491515188, 29.7433014),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3370667f,
                                                40.0761719f, leg_index, 24.9568367f, 1903,
                                                maneuver_index, 15.7745361f, 896));
  //----------------------------------------------------------------
  // trace point = 444
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3374023f, 40.0762672f, 1491515189, 29.8009052),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3373642f,
                                                40.0763092f, leg_index, 24.9384441f, 1902,
                                                maneuver_index, 15.7561436f, 895));
  //----------------------------------------------------------------
  // trace point = 445
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3377075f, 40.0764046f, 1491515190, 29.816185),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3376694f,
                                                40.0764503f, leg_index, 24.9195709f, 1901,
                                                maneuver_index, 15.7372704f, 894));
  //----------------------------------------------------------------
  // trace point = 446
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3380127f, 40.0765419f, 1491515191, 30.165617),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3379745f,
                                                40.0765915f, leg_index, 24.9006977f, 1900,
                                                maneuver_index, 15.7183971f, 893));
  //----------------------------------------------------------------
  // trace point = 447
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3383102f, 40.076683f, 1491515192, 30.1655712),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3382721f,
                                                40.0767288f, leg_index, 24.8823051f, 1899,
                                                maneuver_index, 15.7000046f, 892));
  //----------------------------------------------------------------
  // trace point = 448
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.338623f, 40.0768204f, 1491515193, 30.5152092),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3385849f,
                                                40.0768738f, leg_index, 24.8629494f, 1897,
                                                maneuver_index, 15.6806488f, 890));
  //----------------------------------------------------------------
  // trace point = 449
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3389282f, 40.0769653f, 1491515194, 30.4688854),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3388901f,
                                                40.0770149f, leg_index, 24.8440762f, 1896,
                                                maneuver_index, 15.6617756f, 889));
  //----------------------------------------------------------------
  // trace point = 450
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3392334f, 40.0771103f, 1491515195, 30.8904095),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3391953f,
                                                40.0771561f, leg_index, 24.8252029f, 1895,
                                                maneuver_index, 15.6429024f, 888));
  //----------------------------------------------------------------
  // trace point = 451
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3395462f, 40.0772514f, 1491515196, 30.8903618),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3395081f,
                                                40.077301f, leg_index, 24.8058472f, 1894,
                                                maneuver_index, 15.6235466f, 887));
  //----------------------------------------------------------------
  // trace point = 452
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.339859f, 40.0773964f, 1491515197, 31.1090336),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3398209f,
                                                40.0774422f, leg_index, 24.7866268f, 1893,
                                                maneuver_index, 15.6043262f, 886));
  //----------------------------------------------------------------
  // trace point = 453
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3401718f, 40.0775414f, 1491515198, 31.2551079),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3401337f,
                                                40.0775871f, leg_index, 24.767271f, 1892,
                                                maneuver_index, 15.5849705f, 885));
  //----------------------------------------------------------------
  // trace point = 454
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3404922f, 40.0776863f, 1491515199, 31.3122997),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3404541f,
                                                40.0777359f, leg_index, 24.7474346f, 1890,
                                                maneuver_index, 15.565134f, 883));
  //----------------------------------------------------------------
  // trace point = 455
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3408051f, 40.0778351f, 1491515200, 31.7475357),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3407669f,
                                                40.0778809f, leg_index, 24.7280769f, 1890,
                                                maneuver_index, 15.5457764f, 883));
  //----------------------------------------------------------------
  // trace point = 456
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3411255f, 40.07798f, 1491515201, 31.6213264),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3410873f,
                                                40.0780296f, leg_index, 24.7082405f, 1889,
                                                maneuver_index, 15.5259399f, 882));
  //----------------------------------------------------------------
  // trace point = 457
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3414536f, 40.0781288f, 1491515202, 32.1413841),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3414154f,
                                                40.0781784f, leg_index, 24.688055f, 1887,
                                                maneuver_index, 15.5057545f, 880));
  //----------------------------------------------------------------
  // trace point = 458
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.341774f, 40.0782776f, 1491515203, 32.0526428),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3417358f,
                                                40.0783272f, leg_index, 24.6682186f, 1886,
                                                maneuver_index, 15.485918f, 879));
  //----------------------------------------------------------------
  // trace point = 459
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3421021f, 40.0784264f, 1491515204, 32.5322456),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3420639f,
                                                40.078476f, leg_index, 24.648035f, 1885,
                                                maneuver_index, 15.4657345f, 878));
  //----------------------------------------------------------------
  // trace point = 460
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3424225f, 40.0785713f, 1491515205, 31.9713326),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3423843f,
                                                40.0786247f, leg_index, 24.6281967f, 1884,
                                                maneuver_index, 15.4458961f, 877));
  //----------------------------------------------------------------
  // trace point = 461
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3427505f, 40.0787239f, 1491515206, 32.4592476),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3427124f,
                                                40.0787773f, leg_index, 24.6078777f, 1883,
                                                maneuver_index, 15.4255772f, 876));
  //----------------------------------------------------------------
  // trace point = 462
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3430786f, 40.0788765f, 1491515207, 32.5897484),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3430405f,
                                                40.0789261f, leg_index, 24.5876942f, 1882,
                                                maneuver_index, 15.4053936f, 875));
  //----------------------------------------------------------------
  // trace point = 463
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3434067f, 40.0790291f, 1491515208, 32.996479),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3433685f,
                                                40.0790787f, leg_index, 24.5673752f, 1881,
                                                maneuver_index, 15.3850746f, 874));
  //----------------------------------------------------------------
  // trace point = 464
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.34375f, 40.0791779f, 1491515209, 33.3736954),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3437119f,
                                                40.0792313f, leg_index, 24.5463619f, 1879,
                                                maneuver_index, 15.3640614f, 872));
  //----------------------------------------------------------------
  // trace point = 465
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3440781f, 40.0793266f, 1491515210, 32.6902809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3440399f,
                                                40.0793762f, leg_index, 24.5263138f, 1878,
                                                maneuver_index, 15.3440132f, 871));
  //----------------------------------------------------------------
  // trace point = 466
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3444214f, 40.079464f, 1491515211, 32.7594452),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3443832f,
                                                40.0795174f, leg_index, 24.5056877f, 1877,
                                                maneuver_index, 15.3233871f, 870));
  //----------------------------------------------------------------
  // trace point = 467
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3447571f, 40.0795975f, 1491515212, 32.627018),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3447189f,
                                                40.0796509f, leg_index, 24.4851837f, 1876,
                                                maneuver_index, 15.3028831f, 869));
  //----------------------------------------------------------------
  // trace point = 468
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3451004f, 40.079731f, 1491515213, 32.6265068),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3450699f,
                                                40.0797806f, leg_index, 24.4650345f, 1875,
                                                maneuver_index, 15.2827339f, 868));
  //----------------------------------------------------------------
  // trace point = 469
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3454437f, 40.0798531f, 1491515214, 32.1709251),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3454056f,
                                                40.0799065f, leg_index, 24.4452477f, 1874,
                                                maneuver_index, 15.2629471f, 867));
  //----------------------------------------------------------------
  // trace point = 470
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.345787f, 40.0799713f, 1491515215, 32.1101913),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3457565f,
                                                40.0800285f, leg_index, 24.4238873f, 1872,
                                                maneuver_index, 15.2415867f, 865));
  //----------------------------------------------------------------
  // trace point = 471
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3461304f, 40.0800858f, 1491515216, 31.6962585),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3460999f,
                                                40.0801468f, leg_index, 24.404911f, 1871,
                                                maneuver_index, 15.2226105f, 864));
  //----------------------------------------------------------------
  // trace point = 472
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3464737f, 40.0801926f, 1491515217, 31.9075546),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3464432f,
                                                40.0802536f, leg_index, 24.3852959f, 1869,
                                                maneuver_index, 15.2029953f, 862));
  //----------------------------------------------------------------
  // trace point = 473
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3468246f, 40.0802956f, 1491515218, 31.741415),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3467941f,
                                                40.0803604f, leg_index, 24.3653069f, 1868,
                                                maneuver_index, 15.1830063f, 861));
  //----------------------------------------------------------------
  // trace point = 474
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3471756f, 40.0803909f, 1491515219, 31.6689262),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3471451f,
                                                40.0804596f, leg_index, 24.3455067f, 1867,
                                                maneuver_index, 15.1632061f, 860));
  //----------------------------------------------------------------
  // trace point = 475
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3475266f, 40.0804825f, 1491515220, 31.9161243),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3475037f,
                                                40.0805435f, leg_index, 24.3256607f, 1865,
                                                maneuver_index, 15.1433601f, 858));
  //----------------------------------------------------------------
  // trace point = 476
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3478851f, 40.080574f, 1491515221, 32.2238617),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3478622f,
                                                40.0806236f, leg_index, 24.3058891f, 1864,
                                                maneuver_index, 15.1235886f, 857));
  //----------------------------------------------------------------
  // trace point = 477
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3482513f, 40.080658f, 1491515222, 32.3667221),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3482285f,
                                                40.0807076f, leg_index, 24.2856579f, 1863,
                                                maneuver_index, 15.1033573f, 856));
  //----------------------------------------------------------------
  // trace point = 478
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3486176f, 40.0807381f, 1491515223, 32.3391838),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3486023f,
                                                40.0807877f, leg_index, 24.2651043f, 1861,
                                                maneuver_index, 15.0828037f, 854));
  //----------------------------------------------------------------
  // trace point = 479
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3489914f, 40.0808144f, 1491515224, 32.9856834),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3489761f,
                                                40.0808601f, leg_index, 24.2446899f, 1861,
                                                maneuver_index, 15.0623894f, 854));
  //----------------------------------------------------------------
  // trace point = 480
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3493652f, 40.0808868f, 1491515225, 33.0133781),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.34935f,
                                                40.0809288f, leg_index, 24.2243423f, 1859,
                                                maneuver_index, 15.0420418f, 852));
  //----------------------------------------------------------------
  // trace point = 481
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3497391f, 40.0809517f, 1491515226, 32.7497902),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3497314f,
                                                40.0809898f, leg_index, 24.2037125f, 1859,
                                                maneuver_index, 15.0214119f, 852));
  //----------------------------------------------------------------
  // trace point = 482
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3501205f, 40.0810127f, 1491515227, 33.0106888),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3501053f,
                                                40.0810547f, leg_index, 24.1834259f, 1858,
                                                maneuver_index, 15.0011253f, 851));
  //----------------------------------------------------------------
  // trace point = 483
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3504944f, 40.0810661f, 1491515228, 32.6266518),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3504868f,
                                                40.0811119f, leg_index, 24.1628513f, 1856,
                                                maneuver_index, 14.9805508f, 849));
  //----------------------------------------------------------------
  // trace point = 484
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3508682f, 40.0811272f, 1491515229, 32.4876099),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3508606f,
                                                40.0811653f, leg_index, 24.142725f, 1855,
                                                maneuver_index, 14.9604244f, 848));
  //----------------------------------------------------------------
  // trace point = 485
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3512421f, 40.0811844f, 1491515230, 32.292244),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3512344f,
                                                40.0812263f, leg_index, 24.1224957f, 1854,
                                                maneuver_index, 14.9401951f, 847));
  //----------------------------------------------------------------
  // trace point = 486
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3516159f, 40.0812454f, 1491515231, 32.3811226),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3516083f,
                                                40.0812836f, leg_index, 24.1023178f, 1853,
                                                maneuver_index, 14.9200172f, 846));
  //----------------------------------------------------------------
  // trace point = 487
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3519897f, 40.0813065f, 1491515232, 32.6311684),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3519821f,
                                                40.0813408f, leg_index, 24.08214f, 1852,
                                                maneuver_index, 14.8998394f, 845));
  //----------------------------------------------------------------
  // trace point = 488
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3523636f, 40.0813599f, 1491515233, 32.2921638),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.352356f,
                                                40.081398f, leg_index, 24.0619621f, 1851,
                                                maneuver_index, 14.8796616f, 844));
  //----------------------------------------------------------------
  // trace point = 489
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3527374f, 40.0814209f, 1491515234, 32.4416695),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3527298f,
                                                40.0814552f, leg_index, 24.0417862f, 1850,
                                                maneuver_index, 14.8594856f, 843));
  //----------------------------------------------------------------
  // trace point = 490
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3531113f, 40.0814743f, 1491515235, 32.480545),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3531036f,
                                                40.0815125f, leg_index, 24.0216084f, 1848,
                                                maneuver_index, 14.8393078f, 841));
  //----------------------------------------------------------------
  // trace point = 491
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3534851f, 40.0815315f, 1491515236, 32.7144356),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3534775f,
                                                40.0815697f, leg_index, 24.0014343f, 1847,
                                                maneuver_index, 14.8191338f, 840));
  //----------------------------------------------------------------
  // trace point = 492
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3538589f, 40.0815926f, 1491515237, 32.3208313),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3538513f,
                                                40.0816269f, leg_index, 23.9812508f, 1846,
                                                maneuver_index, 14.7989502f, 839));
  //----------------------------------------------------------------
  // trace point = 493
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3542328f, 40.0816498f, 1491515238, 32.3972244),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3542252f,
                                                40.0816841f, leg_index, 23.9610729f, 1844,
                                                maneuver_index, 14.7787724f, 837));
  //----------------------------------------------------------------
  // trace point = 494
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.354599f, 40.081707f, 1491515239, 32.1466675),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3545914f,
                                                40.0817413f, leg_index, 23.9412918f, 1843,
                                                maneuver_index, 14.7589912f, 836));
  //----------------------------------------------------------------
  // trace point = 495
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3549652f, 40.0817604f, 1491515240, 32.0823936),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3549576f,
                                                40.0817947f, leg_index, 23.9215622f, 1842,
                                                maneuver_index, 14.7392616f, 835));
  //----------------------------------------------------------------
  // trace point = 496
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3553391f, 40.0818214f, 1491515241, 32.2975235),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3553314f,
                                                40.081852f, leg_index, 23.9013844f, 1841,
                                                maneuver_index, 14.7190838f, 834));
  //----------------------------------------------------------------
  // trace point = 497
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3557129f, 40.0818825f, 1491515242, 32.4414597),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3557053f,
                                                40.0819054f, leg_index, 23.881258f, 1840,
                                                maneuver_index, 14.6989574f, 833));
  //----------------------------------------------------------------
  // trace point = 498
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3560867f, 40.0819359f, 1491515243, 32.6051636),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3560791f,
                                                40.0819588f, leg_index, 23.8611317f, 1838,
                                                maneuver_index, 14.6788311f, 831));
  //----------------------------------------------------------------
  // trace point = 499
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3564682f, 40.0819931f, 1491515244, 32.7312317),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3564606f,
                                                40.082016f, leg_index, 23.840559f, 1837,
                                                maneuver_index, 14.6582584f, 830));
  //----------------------------------------------------------------
  // trace point = 500
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3568497f, 40.0820503f, 1491515245, 33.3594246),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.356842f,
                                                40.0820732f, leg_index, 23.8199844f, 1836,
                                                maneuver_index, 14.6376839f, 829));
  //----------------------------------------------------------------
  // trace point = 501
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3572311f, 40.0821152f, 1491515246, 33.2600937),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3572311f,
                                                40.0821304f, leg_index, 23.799015f, 1835,
                                                maneuver_index, 14.6167145f, 828));
  //----------------------------------------------------------------
  // trace point = 502
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3576202f, 40.0821724f, 1491515247, 33.7988243),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3576126f,
                                                40.0821877f, leg_index, 23.7784405f, 1834,
                                                maneuver_index, 14.5961399f, 827));
  //----------------------------------------------------------------
  // trace point = 503
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3580093f, 40.0822334f, 1491515248, 33.8822937),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3580017f,
                                                40.0822563f, leg_index, 23.757309f, 1833,
                                                maneuver_index, 14.5750084f, 826));
  //----------------------------------------------------------------
  // trace point = 504
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3583984f, 40.0822945f, 1491515249, 33.6317978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3583908f,
                                                40.0823135f, leg_index, 23.7363396f, 1832,
                                                maneuver_index, 14.554039f, 825));
  //----------------------------------------------------------------
  // trace point = 505
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3587799f, 40.0823593f, 1491515250, 33.4140282),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3587799f,
                                                40.0823708f, leg_index, 23.7153702f, 1831,
                                                maneuver_index, 14.5330696f, 824));
  //----------------------------------------------------------------
  // trace point = 506
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.359169f, 40.0824165f, 1491515251, 33.3787842),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3591614f,
                                                40.0824318f, leg_index, 23.6947441f, 1830,
                                                maneuver_index, 14.5124435f, 823));
  //----------------------------------------------------------------
  // trace point = 507
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3595505f, 40.0824776f, 1491515252, 33.4871712),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3595428f,
                                                40.0824928f, leg_index, 23.6741199f, 1828,
                                                maneuver_index, 14.4918194f, 821));
  //----------------------------------------------------------------
  // trace point = 508
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3599319f, 40.0825386f, 1491515253, 33.3203545),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3599319f,
                                                40.0825539f, leg_index, 23.6530991f, 1827,
                                                maneuver_index, 14.4707985f, 820));
  //----------------------------------------------------------------
  // trace point = 509
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.360321f, 40.0825958f, 1491515254, 33.5261955),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3603134f,
                                                40.0826149f, leg_index, 23.632473f, 1826,
                                                maneuver_index, 14.4501724f, 819));
  //----------------------------------------------------------------
  // trace point = 510
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3607025f, 40.0826569f, 1491515255, 33.4870872),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3606949f,
                                                40.0826759f, leg_index, 23.6118469f, 1825,
                                                maneuver_index, 14.4295464f, 818));
  //----------------------------------------------------------------
  // trace point = 511
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3610916f, 40.0827179f, 1491515256, 33.5704651),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.361084f,
                                                40.082737f, leg_index, 23.590826f, 1824,
                                                maneuver_index, 14.4085255f, 817));
  //----------------------------------------------------------------
  // trace point = 512
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3614731f, 40.0827751f, 1491515257, 33.2537918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3614655f,
                                                40.0827942f, leg_index, 23.5702553f, 1823,
                                                maneuver_index, 14.3879547f, 816));
  //----------------------------------------------------------------
  // trace point = 513
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3618546f, 40.0828362f, 1491515258, 33.2755547),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3618469f,
                                                40.0828552f, leg_index, 23.5496292f, 1821,
                                                maneuver_index, 14.3673286f, 814));
  //----------------------------------------------------------------
  // trace point = 514
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.362236f, 40.0828934f, 1491515259, 33.108551),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3622284f,
                                                40.0829163f, leg_index, 23.5290031f, 1820,
                                                maneuver_index, 14.3467026f, 813));
  //----------------------------------------------------------------
  // trace point = 515
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3626175f, 40.0829544f, 1491515260, 32.9866943),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3626099f,
                                                40.0829735f, leg_index, 23.5084305f, 1819,
                                                maneuver_index, 14.3261299f, 812));
  //----------------------------------------------------------------
  // trace point = 516
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.362999f, 40.0830193f, 1491515261, 33.3662148),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.362999f,
                                                40.0830307f, leg_index, 23.4874611f, 1818,
                                                maneuver_index, 14.3051605f, 811));
  //----------------------------------------------------------------
  // trace point = 517
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3633804f, 40.0830765f, 1491515262, 33.0473099),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3633804f,
                                                40.0830879f, leg_index, 23.4668903f, 1817,
                                                maneuver_index, 14.2845898f, 810));
  //----------------------------------------------------------------
  // trace point = 518
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3637619f, 40.0831375f, 1491515263, 33.3589134),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3637619f,
                                                40.0831451f, leg_index, 23.4463158f, 1816,
                                                maneuver_index, 14.2640152f, 809));
  //----------------------------------------------------------------
  // trace point = 519
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.364151f, 40.0831985f, 1491515264, 33.4034348),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.364151f,
                                                40.0832062f, leg_index, 23.4252949f, 1814,
                                                maneuver_index, 14.2429943f, 807));
  //----------------------------------------------------------------
  // trace point = 520
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3645325f, 40.0832558f, 1491515265, 33.2535629),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3645325f,
                                                40.0832672f, leg_index, 23.4046707f, 1813,
                                                maneuver_index, 14.2223701f, 806));
  //----------------------------------------------------------------
  // trace point = 521
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3649216f, 40.083313f, 1491515266, 33.5878067),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3649139f,
                                                40.0833282f, leg_index, 23.3840446f, 1812,
                                                maneuver_index, 14.2017441f, 805));
  //----------------------------------------------------------------
  // trace point = 522
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.365303f, 40.083374f, 1491515267, 33.214016),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.365303f,
                                                40.0833855f, leg_index, 23.3630733f, 1811,
                                                maneuver_index, 14.1807728f, 804));
  //----------------------------------------------------------------
  // trace point = 523
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3656845f, 40.0834351f, 1491515268, 33.464325),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3656845f,
                                                40.0834427f, leg_index, 23.3425007f, 1809,
                                                maneuver_index, 14.1602001f, 802));
  //----------------------------------------------------------------
  // trace point = 524
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3660736f, 40.0834923f, 1491515269, 33.4642944),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3660736f,
                                                40.0834999f, leg_index, 23.3215313f, 1808,
                                                maneuver_index, 14.1392307f, 801));
  //----------------------------------------------------------------
  // trace point = 525
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3664551f, 40.0835571f, 1491515270, 33.592762),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3664551f,
                                                40.0835571f, leg_index, 23.3009586f, 1807,
                                                maneuver_index, 14.1186581f, 800));
  //----------------------------------------------------------------
  // trace point = 526
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3668442f, 40.0836143f, 1491515271, 33.5664406),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3668442f,
                                                40.0836182f, leg_index, 23.2799377f, 1806,
                                                maneuver_index, 14.0976372f, 799));
  //----------------------------------------------------------------
  // trace point = 527
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3672256f, 40.0836754f, 1491515272, 33.6391525),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3672256f,
                                                40.0836792f, leg_index, 23.2593098f, 1805,
                                                maneuver_index, 14.0770092f, 798));
  //----------------------------------------------------------------
  // trace point = 528
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3676147f, 40.083744f, 1491515273, 33.7114334),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3676147f,
                                                40.0837479f, leg_index, 23.2381783f, 1804,
                                                maneuver_index, 14.0558777f, 797));
  //----------------------------------------------------------------
  // trace point = 529
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3680038f, 40.0838127f, 1491515274, 34.0355301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3680038f,
                                                40.0838203f, leg_index, 23.2169819f, 1803,
                                                maneuver_index, 14.0346813f, 796));
  //----------------------------------------------------------------
  // trace point = 530
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3683853f, 40.0838928f, 1491515275, 33.7320747),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3683853f,
                                                40.0839005f, leg_index, 23.1960468f, 1802,
                                                maneuver_index, 14.0137463f, 795));
  //----------------------------------------------------------------
  // trace point = 531
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3687668f, 40.0839806f, 1491515276, 33.9962044),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3687668f,
                                                40.0839844f, leg_index, 23.1750412f, 1800,
                                                maneuver_index, 13.9927406f, 793));
  //----------------------------------------------------------------
  // trace point = 532
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3691483f, 40.0840721f, 1491515277, 33.8318748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3691483f,
                                                40.0840721f, leg_index, 23.1539612f, 1799,
                                                maneuver_index, 13.9716606f, 792));
  //----------------------------------------------------------------
  // trace point = 533
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3695221f, 40.0841675f, 1491515278, 33.4371834),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3695221f,
                                                40.0841675f, leg_index, 23.1331005f, 1798,
                                                maneuver_index, 13.9507999f, 791));
  //----------------------------------------------------------------
  // trace point = 534
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3698883f, 40.0842667f, 1491515279, 33.0261993),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3698883f,
                                                40.0842705f, leg_index, 23.1124496f, 1797,
                                                maneuver_index, 13.9301491f, 790));
  //----------------------------------------------------------------
  // trace point = 535
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3702469f, 40.0843658f, 1491515280, 32.9399948),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3702469f,
                                                40.0843697f, leg_index, 23.0922756f, 1796,
                                                maneuver_index, 13.9099751f, 789));
  //----------------------------------------------------------------
  // trace point = 536
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3706055f, 40.0844727f, 1491515281, 32.5002518),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3705978f,
                                                40.0844803f, leg_index, 23.0721874f, 1795,
                                                maneuver_index, 13.8898869f, 788));
  //----------------------------------------------------------------
  // trace point = 537
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3709488f, 40.0845833f, 1491515282, 31.9903469),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3709412f,
                                                40.0845909f, leg_index, 23.0524712f, 1794,
                                                maneuver_index, 13.8701706f, 787));
  //----------------------------------------------------------------
  // trace point = 538
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3712921f, 40.0846977f, 1491515283, 31.5277252),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3712845f,
                                                40.0847092f, leg_index, 23.0325394f, 1793,
                                                maneuver_index, 13.8502388f, 786));
  //----------------------------------------------------------------
  // trace point = 539
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3716278f, 40.0848122f, 1491515284, 31.3330631),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3716202f,
                                                40.0848274f, leg_index, 23.0129833f, 1792,
                                                maneuver_index, 13.8306828f, 785));
  //----------------------------------------------------------------
  // trace point = 540
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3719559f, 40.0849342f, 1491515285, 31.2918987),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3719482f,
                                                40.0849495f, leg_index, 22.993679f, 1791,
                                                maneuver_index, 13.8113785f, 784));
  //----------------------------------------------------------------
  // trace point = 541
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3722839f, 40.0850639f, 1491515286, 31.5114784),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3722763f,
                                                40.0850754f, leg_index, 22.9742508f, 1789,
                                                maneuver_index, 13.7919502f, 782));
  //----------------------------------------------------------------
  // trace point = 542
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3726196f, 40.0851974f, 1491515287, 32.0175591),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.372612f,
                                                40.0852127f, leg_index, 22.9541054f, 1788,
                                                maneuver_index, 13.7718048f, 781));
  //----------------------------------------------------------------
  // trace point = 543
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3729477f, 40.0853348f, 1491515288, 32.2120247),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3729401f,
                                                40.08535f, leg_index, 22.9343166f, 1787,
                                                maneuver_index, 13.7520161f, 780));
  //----------------------------------------------------------------
  // trace point = 544
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3732758f, 40.0854759f, 1491515289, 31.8204765),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3732681f,
                                                40.0854912f, leg_index, 22.9143982f, 1786,
                                                maneuver_index, 13.7320976f, 779));
  //----------------------------------------------------------------
  // trace point = 545
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3735962f, 40.0856247f, 1491515290, 31.6590233),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3735809f,
                                                40.08564f, leg_index, 22.8949051f, 1785,
                                                maneuver_index, 13.7126045f, 778));
  //----------------------------------------------------------------
  // trace point = 546
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3739166f, 40.0857697f, 1491515291, 31.7010479),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3739014f,
                                                40.0857887f, leg_index, 22.8750706f, 1784,
                                                maneuver_index, 13.69277f, 777));
  //----------------------------------------------------------------
  // trace point = 547
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3742294f, 40.0859184f, 1491515292, 31.8466473),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3742142f,
                                                40.0859337f, leg_index, 22.8557167f, 1783,
                                                maneuver_index, 13.6734161f, 776));
  //----------------------------------------------------------------
  // trace point = 548
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3745575f, 40.086071f, 1491515293, 32.2965698),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3745422f,
                                                40.0860901f, leg_index, 22.8352604f, 1781,
                                                maneuver_index, 13.6529598f, 774));
  //----------------------------------------------------------------
  // trace point = 549
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3748779f, 40.0862236f, 1491515294, 32.311039),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3748627f,
                                                40.0862389f, leg_index, 22.815424f, 1780,
                                                maneuver_index, 13.6331234f, 773));
  //----------------------------------------------------------------
  // trace point = 550
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3751984f, 40.0863686f, 1491515295, 31.7890453),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3751831f,
                                                40.0863876f, leg_index, 22.7955894f, 1779,
                                                maneuver_index, 13.6132889f, 772));
  //----------------------------------------------------------------
  // trace point = 551
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3755264f, 40.0865173f, 1491515296, 32.1227074),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3755035f,
                                                40.0865402f, leg_index, 22.7756157f, 1778,
                                                maneuver_index, 13.5933151f, 771));
  //----------------------------------------------------------------
  // trace point = 552
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3758392f, 40.0866661f, 1491515297, 31.8313999),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.375824f,
                                                40.086689f, leg_index, 22.7557793f, 1777,
                                                maneuver_index, 13.5734787f, 770));
  //----------------------------------------------------------------
  // trace point = 553
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3761597f, 40.0868149f, 1491515298, 31.3519001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3761368f,
                                                40.0868378f, leg_index, 22.7362881f, 1776,
                                                maneuver_index, 13.5539875f, 769));
  //----------------------------------------------------------------
  // trace point = 554
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3764725f, 40.0869637f, 1491515299, 31.7006435),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.376442f,
                                                40.0869942f, leg_index, 22.7168484f, 1774,
                                                maneuver_index, 13.5345478f, 767));
  //----------------------------------------------------------------
  // trace point = 555
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3767853f, 40.0871086f, 1491515300, 31.0757847),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3767548f,
                                                40.0871429f, leg_index, 22.6973534f, 1774,
                                                maneuver_index, 13.5150528f, 767));
  //----------------------------------------------------------------
  // trace point = 556
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3770981f, 40.0872574f, 1491515301, 31.613409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3770752f,
                                                40.0872917f, leg_index, 22.6775169f, 1772,
                                                maneuver_index, 13.4952164f, 765));
  //----------------------------------------------------------------
  // trace point = 557
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3774109f, 40.0874023f, 1491515302, 31.0756874),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.377388f,
                                                40.0874367f, leg_index, 22.6581631f, 1771,
                                                maneuver_index, 13.4758625f, 764));
  //----------------------------------------------------------------
  // trace point = 558
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3777161f, 40.0875473f, 1491515303, 30.5100594),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3776855f,
                                                40.0875816f, leg_index, 22.6394939f, 1770,
                                                maneuver_index, 13.4571934f, 763));
  //----------------------------------------------------------------
  // trace point = 559
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3780212f, 40.0876961f, 1491515304, 30.7004089),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.378006f,
                                                40.0877342f, leg_index, 22.6195202f, 1769,
                                                maneuver_index, 13.4372196f, 762));
  //----------------------------------------------------------------
  // trace point = 560
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3783264f, 40.0878372f, 1491515305, 30.4801159),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3783035f,
                                                40.0878639f, leg_index, 22.6013985f, 1768,
                                                maneuver_index, 13.4190979f, 761));
  //----------------------------------------------------------------
  // trace point = 561
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3786316f, 40.0879784f, 1491515306, 30.4653015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3786163f,
                                                40.0880013f, leg_index, 22.5823174f, 1767,
                                                maneuver_index, 13.4000168f, 760));
  //----------------------------------------------------------------
  // trace point = 562
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3789368f, 40.0881233f, 1491515307, 30.6545467),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3789215f,
                                                40.0881424f, leg_index, 22.5634308f, 1766,
                                                maneuver_index, 13.3811302f, 759));
  //----------------------------------------------------------------
  // trace point = 563
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3792419f, 40.0882721f, 1491515308, 30.7723141),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3792267f,
                                                40.0882912f, leg_index, 22.5442829f, 1765,
                                                maneuver_index, 13.3619823f, 758));
  //----------------------------------------------------------------
  // trace point = 564
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3795547f, 40.0884209f, 1491515309, 30.9889393),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3795395f,
                                                40.0884361f, leg_index, 22.524929f, 1764,
                                                maneuver_index, 13.3426285f, 757));
  //----------------------------------------------------------------
  // trace point = 565
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3798676f, 40.0885658f, 1491515310, 31.3086262),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3798523f,
                                                40.0885849f, leg_index, 22.505434f, 1763,
                                                maneuver_index, 13.3231335f, 756));
  //----------------------------------------------------------------
  // trace point = 566
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3801804f, 40.0887108f, 1491515311, 31.2646217),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3801651f,
                                                40.0887299f, leg_index, 22.4860802f, 1761,
                                                maneuver_index, 13.3037796f, 754));
  //----------------------------------------------------------------
  // trace point = 567
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3805008f, 40.0888634f, 1491515312, 31.7438164),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3804855f,
                                                40.0888786f, leg_index, 22.4662457f, 1760,
                                                maneuver_index, 13.2839451f, 753));
  //----------------------------------------------------------------
  // trace point = 568
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3808136f, 40.0890121f, 1491515313, 31.513792),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.380806f,
                                                40.0890236f, leg_index, 22.4465466f, 1759,
                                                maneuver_index, 13.264246f, 752));
  //----------------------------------------------------------------
  // trace point = 569
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.381134f, 40.0891647f, 1491515314, 32.4264069),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3811264f,
                                                40.0891724f, leg_index, 22.426712f, 1758,
                                                maneuver_index, 13.2444115f, 751));
  //----------------------------------------------------------------
  // trace point = 570
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3814545f, 40.0893135f, 1491515315, 31.7726326),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3814468f,
                                                40.0893211f, leg_index, 22.4068775f, 1756,
                                                maneuver_index, 13.224577f, 749));
  //----------------------------------------------------------------
  // trace point = 571
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3817749f, 40.0894661f, 1491515316, 32.2372169),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3817673f,
                                                40.0894737f, leg_index, 22.3869038f, 1755,
                                                maneuver_index, 13.2046032f, 748));
  //----------------------------------------------------------------
  // trace point = 572
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.382103f, 40.0896187f, 1491515317, 32.731678),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3820953f,
                                                40.0896301f, leg_index, 22.3664474f, 1754,
                                                maneuver_index, 13.1841469f, 747));
  //----------------------------------------------------------------
  // trace point = 573
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3824387f, 40.0897675f, 1491515318, 32.8203926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3824234f,
                                                40.0897865f, leg_index, 22.345993f, 1753,
                                                maneuver_index, 13.1636925f, 746));
  //----------------------------------------------------------------
  // trace point = 574
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3827667f, 40.08992f, 1491515319, 33.0236435),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3827515f,
                                                40.0899391f, leg_index, 22.3256741f, 1752,
                                                maneuver_index, 13.1433735f, 745));
  //----------------------------------------------------------------
  // trace point = 575
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3830948f, 40.0900764f, 1491515320, 32.9639053),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3830795f,
                                                40.0900955f, leg_index, 22.3052216f, 1751,
                                                maneuver_index, 13.122921f, 744));
  //----------------------------------------------------------------
  // trace point = 576
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3834305f, 40.0902328f, 1491515321, 33.3121529),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3834152f,
                                                40.0902481f, leg_index, 22.2845573f, 1750,
                                                maneuver_index, 13.1022568f, 743));
  //----------------------------------------------------------------
  // trace point = 577
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3837662f, 40.0903893f, 1491515322, 33.4572182),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3837509f,
                                                40.0904045f, leg_index, 22.2637596f, 1749,
                                                maneuver_index, 13.081459f, 742));
  //----------------------------------------------------------------
  // trace point = 578
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3841019f, 40.0905457f, 1491515323, 33.399147),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3840866f,
                                                40.0905647f, leg_index, 22.2428188f, 1747,
                                                maneuver_index, 13.0605183f, 740));
  //----------------------------------------------------------------
  // trace point = 579
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3844376f, 40.0907059f, 1491515324, 33.5879135),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3844223f,
                                                40.0907288f, leg_index, 22.2217312f, 1746,
                                                maneuver_index, 13.0394306f, 739));
  //----------------------------------------------------------------
  // trace point = 580
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3847733f, 40.0908623f, 1491515325, 33.4570503),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.384758f,
                                                40.0908813f, leg_index, 22.201067f, 1745,
                                                maneuver_index, 13.0187664f, 738));
  //----------------------------------------------------------------
  // trace point = 581
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3851089f, 40.0910187f, 1491515326, 33.6174736),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3850937f,
                                                40.0910339f, leg_index, 22.1804047f, 1744,
                                                maneuver_index, 12.9981041f, 737));
  //----------------------------------------------------------------
  // trace point = 582
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3854446f, 40.0911751f, 1491515327, 33.3703575),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3854294f,
                                                40.0911942f, leg_index, 22.1594677f, 1742,
                                                maneuver_index, 12.9771671f, 735));
  //----------------------------------------------------------------
  // trace point = 583
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3857727f, 40.0913315f, 1491515328, 33.1808968),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3857574f,
                                                40.0913506f, leg_index, 22.1390133f, 1742,
                                                maneuver_index, 12.9567127f, 735));
  //----------------------------------------------------------------
  // trace point = 584
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3861084f, 40.0914879f, 1491515329, 33.3260918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3860931f,
                                                40.091507f, leg_index, 22.1182137f, 1741,
                                                maneuver_index, 12.9359131f, 734));
  //----------------------------------------------------------------
  // trace point = 585
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3864365f, 40.0916405f, 1491515330, 32.9773941),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3864212f,
                                                40.0916595f, leg_index, 22.0978966f, 1740,
                                                maneuver_index, 12.915596f, 733));
  //----------------------------------------------------------------
  // trace point = 586
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3867722f, 40.0918007f, 1491515331, 33.5735512),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3867569f,
                                                40.0918159f, leg_index, 22.0770988f, 1738,
                                                maneuver_index, 12.8947983f, 731));
  //----------------------------------------------------------------
  // trace point = 587
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3871155f, 40.0919571f, 1491515332, 33.7475967),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3871002f,
                                                40.0919762f, leg_index, 22.0558186f, 1737,
                                                maneuver_index, 12.873518f, 730));
  //----------------------------------------------------------------
  // trace point = 588
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3874512f, 40.0921173f, 1491515333, 33.7182884),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3874359f,
                                                40.0921326f, leg_index, 22.0350189f, 1736,
                                                maneuver_index, 12.8527184f, 729));
  //----------------------------------------------------------------
  // trace point = 589
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3877869f, 40.0922775f, 1491515334, 33.7767982),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3877792f,
                                                40.092289f, leg_index, 22.013876f, 1735,
                                                maneuver_index, 12.8315754f, 728));
  //----------------------------------------------------------------
  // trace point = 590
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3881226f, 40.0924339f, 1491515335, 33.7325096),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3881149f,
                                                40.0924454f, leg_index, 21.9930763f, 1734,
                                                maneuver_index, 12.8107758f, 727));
  //----------------------------------------------------------------
  // trace point = 591
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3884583f, 40.0925903f, 1491515336, 33.4564285),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3884506f,
                                                40.0926018f, leg_index, 21.9722786f, 1732,
                                                maneuver_index, 12.789978f, 725));
  //----------------------------------------------------------------
  // trace point = 592
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3888016f, 40.0927467f, 1491515337, 33.616848),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3887863f,
                                                40.092762f, leg_index, 21.9513397f, 1731,
                                                maneuver_index, 12.7690392f, 724));
  //----------------------------------------------------------------
  // trace point = 593
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3891373f, 40.092907f, 1491515338, 33.8489609),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3891296f,
                                                40.0929222f, leg_index, 21.9300594f, 1730,
                                                maneuver_index, 12.7477589f, 723));
  //----------------------------------------------------------------
  // trace point = 594
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.389473f, 40.0930634f, 1491515339, 33.8200722),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3894577f,
                                                40.0930786f, leg_index, 21.9096069f, 1729,
                                                maneuver_index, 12.7273064f, 722));
  //----------------------------------------------------------------
  // trace point = 595
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3898163f, 40.0932274f, 1491515340, 34.1971931),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3898087f,
                                                40.0932388f, leg_index, 21.8879814f, 1728,
                                                maneuver_index, 12.7056808f, 721));
  //----------------------------------------------------------------
  // trace point = 596
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3901596f, 40.0933876f, 1491515341, 34.1389236),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.390152f,
                                                40.0934029f, leg_index, 21.8665638f, 1727,
                                                maneuver_index, 12.6842632f, 720));
  //----------------------------------------------------------------
  // trace point = 597
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3905029f, 40.093544f, 1491515342, 34.0232391),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3904877f,
                                                40.0935593f, leg_index, 21.8457642f, 1725,
                                                maneuver_index, 12.6634636f, 718));
  //----------------------------------------------------------------
  // trace point = 598
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3908463f, 40.0937042f, 1491515343, 34.1245193),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.390831f,
                                                40.0937233f, leg_index, 21.8243465f, 1725,
                                                maneuver_index, 12.642046f, 718));
  //----------------------------------------------------------------
  // trace point = 599
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3911819f, 40.0938683f, 1491515344, 34.3010368),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3911743f,
                                                40.0938797f, leg_index, 21.8032055f, 1724,
                                                maneuver_index, 12.6209049f, 717));
  //----------------------------------------------------------------
  // trace point = 600
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3915253f, 40.0940285f, 1491515345, 34.1535454),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3915176f,
                                                40.0940437f, leg_index, 21.781786f, 1722,
                                                maneuver_index, 12.5994854f, 715));
  //----------------------------------------------------------------
  // trace point = 601
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3918686f, 40.0941849f, 1491515346, 34.1117401),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3918533f,
                                                40.0942001f, leg_index, 21.7609901f, 1721,
                                                maneuver_index, 12.5786896f, 714));
  //----------------------------------------------------------------
  // trace point = 602
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3922119f, 40.0943451f, 1491515347, 33.7900009),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3921967f,
                                                40.0943642f, leg_index, 21.7395687f, 1720,
                                                maneuver_index, 12.5572681f, 713));
  //----------------------------------------------------------------
  // trace point = 603
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3925476f, 40.0945053f, 1491515348, 33.776001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3925323f,
                                                40.0945206f, leg_index, 21.718771f, 1719,
                                                maneuver_index, 12.5364704f, 712));
  //----------------------------------------------------------------
  // trace point = 604
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3928833f, 40.0946617f, 1491515349, 33.9350815),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.392868f,
                                                40.094677f, leg_index, 21.6979733f, 1717,
                                                maneuver_index, 12.5156727f, 710));
  //----------------------------------------------------------------
  // trace point = 605
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3932266f, 40.0948219f, 1491515350, 34.0077095),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3932114f,
                                                40.0948372f, leg_index, 21.6766949f, 1716,
                                                maneuver_index, 12.4943943f, 709));
  //----------------------------------------------------------------
  // trace point = 606
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3935699f, 40.0949783f, 1491515351, 33.7048607),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3935623f,
                                                40.0949936f, leg_index, 21.6552029f, 1715,
                                                maneuver_index, 12.4729023f, 708));
  //----------------------------------------------------------------
  // trace point = 607
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3939056f, 40.0951309f, 1491515352, 33.5586128),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.393898f,
                                                40.0951424f, leg_index, 21.634676f, 1713,
                                                maneuver_index, 12.4523754f, 706));
  //----------------------------------------------------------------
  // trace point = 608
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.394249f, 40.0952797f, 1491515353, 33.5156403),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3942413f,
                                                40.0952911f, leg_index, 21.6137905f, 1712,
                                                maneuver_index, 12.4314899f, 705));
  //----------------------------------------------------------------
  // trace point = 609
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3945923f, 40.0954208f, 1491515354, 33.3319588),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3945847f,
                                                40.0954323f, leg_index, 21.5931664f, 1711,
                                                maneuver_index, 12.4108658f, 704));
  //----------------------------------------------------------------
  // trace point = 610
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3949432f, 40.0955544f, 1491515355, 33.0774269),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3949356f,
                                                40.095562f, leg_index, 21.5725422f, 1710,
                                                maneuver_index, 12.3902416f, 703));
  //----------------------------------------------------------------
  // trace point = 611
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3952942f, 40.0956841f, 1491515356, 33.3064461),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3952866f,
                                                40.0956917f, leg_index, 21.5519161f, 1709,
                                                maneuver_index, 12.3696156f, 702));
  //----------------------------------------------------------------
  // trace point = 612
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3956528f, 40.0958138f, 1491515357, 33.6044273),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3956528f,
                                                40.0958176f, leg_index, 21.530674f, 1708,
                                                maneuver_index, 12.3483734f, 701));
  //----------------------------------------------------------------
  // trace point = 613
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3960114f, 40.0959282f, 1491515358, 33.1514359),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3960037f,
                                                40.0959396f, leg_index, 21.5102692f, 1706,
                                                maneuver_index, 12.3279686f, 699));
  //----------------------------------------------------------------
  // trace point = 614
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3963699f, 40.0960464f, 1491515359, 33.5002785),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3963699f,
                                                40.0960503f, leg_index, 21.4894352f, 1705,
                                                maneuver_index, 12.3071346f, 698));
  //----------------------------------------------------------------
  // trace point = 615
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3967285f, 40.0961533f, 1491515360, 33.09021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3967209f,
                                                40.0961609f, leg_index, 21.468504f, 1704,
                                                maneuver_index, 12.2862034f, 697));
  //----------------------------------------------------------------
  // trace point = 616
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3970947f, 40.0962563f, 1491515361, 32.7300186),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3970947f,
                                                40.0962639f, leg_index, 21.448328f, 1703,
                                                maneuver_index, 12.2660275f, 696));
  //----------------------------------------------------------------
  // trace point = 617
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3974457f, 40.0963554f, 1491515362, 32.1007347),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3974457f,
                                                40.0963593f, leg_index, 21.4286175f, 1702,
                                                maneuver_index, 12.2463169f, 695));
  //----------------------------------------------------------------
  // trace point = 618
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3977966f, 40.0964432f, 1491515363, 31.6503906),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3977966f,
                                                40.0964508f, leg_index, 21.4085293f, 1701,
                                                maneuver_index, 12.2262287f, 694));
  //----------------------------------------------------------------
  // trace point = 619
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3981476f, 40.0965233f, 1491515364, 31.1563225),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3981476f,
                                                40.0965271f, leg_index, 21.3896847f, 1700,
                                                maneuver_index, 12.2073841f, 693));
  //----------------------------------------------------------------
  // trace point = 620
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3984985f, 40.0965958f, 1491515365, 30.5935211),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3984985f,
                                                40.0966034f, leg_index, 21.3703804f, 1699,
                                                maneuver_index, 12.1880798f, 692));
  //----------------------------------------------------------------
  // trace point = 621
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3988419f, 40.0966682f, 1491515366, 30.2297096),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3988419f,
                                                40.0966759f, leg_index, 21.351532f, 1698,
                                                maneuver_index, 12.1692314f, 691));
  //----------------------------------------------------------------
  // trace point = 622
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3991776f, 40.0967293f, 1491515367, 29.7638988),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3991699f,
                                                40.0967445f, leg_index, 21.3335381f, 1697,
                                                maneuver_index, 12.1512375f, 690));
  //----------------------------------------------------------------
  // trace point = 623
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3995209f, 40.0967903f, 1491515368, 30.1210365),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3995132f,
                                                40.0968056f, leg_index, 21.3148842f, 1697,
                                                maneuver_index, 12.1325836f, 690));
  //----------------------------------------------------------------
  // trace point = 624
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3998718f, 40.0968513f, 1491515369, 30.3290253),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3998718f,
                                                40.0968628f, leg_index, 21.2955036f, 1696,
                                                maneuver_index, 12.113203f, 689));
  //----------------------------------------------------------------
  // trace point = 625
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4002151f, 40.0969048f, 1491515370, 29.7825871),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4002151f,
                                                40.0969124f, leg_index, 21.2770157f, 1695,
                                                maneuver_index, 12.0947151f, 688));
  //----------------------------------------------------------------
  // trace point = 626
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4005508f, 40.0969543f, 1491515371, 29.2945309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4005508f,
                                                40.096962f, leg_index, 21.2589245f, 1693,
                                                maneuver_index, 12.0766239f, 686));
  //----------------------------------------------------------------
  // trace point = 627
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4008865f, 40.0970001f, 1491515372, 29.0471039),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4008865f,
                                                40.0970116f, leg_index, 21.2408352f, 1692,
                                                maneuver_index, 12.0585346f, 685));
  //----------------------------------------------------------------
  // trace point = 628
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4012222f, 40.0970497f, 1491515373, 29.2739697),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4012222f,
                                                40.0970612f, leg_index, 21.2227421f, 1692,
                                                maneuver_index, 12.0404415f, 685));
  //----------------------------------------------------------------
  // trace point = 629
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4015656f, 40.0970955f, 1491515374, 29.3781414),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4015656f,
                                                40.0971069f, leg_index, 21.2042999f, 1691,
                                                maneuver_index, 12.0219994f, 684));
  //----------------------------------------------------------------
  // trace point = 630
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4019089f, 40.0971451f, 1491515375, 29.629179),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4019089f,
                                                40.0971565f, leg_index, 21.1858139f, 1690,
                                                maneuver_index, 12.0035133f, 683));
  //----------------------------------------------------------------
  // trace point = 631
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4022446f, 40.0971985f, 1491515376, 29.1915016),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4022446f,
                                                40.0972061f, leg_index, 21.1677227f, 1689,
                                                maneuver_index, 11.9854221f, 682));
  //----------------------------------------------------------------
  // trace point = 632
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4025574f, 40.0972404f, 1491515377, 27.4346409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4025574f,
                                                40.0972519f, leg_index, 21.1508713f, 1688,
                                                maneuver_index, 11.9685707f, 681));
  //----------------------------------------------------------------
  // trace point = 633
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4028549f, 40.0972824f, 1491515378, 25.3641644),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4028549f,
                                                40.0972939f, leg_index, 21.1348629f, 1687,
                                                maneuver_index, 11.9525623f, 680));
  //----------------------------------------------------------------
  // trace point = 634
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4031372f, 40.0973206f, 1491515379, 24.2615509),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4031296f,
                                                40.0973358f, leg_index, 21.1200428f, 1686,
                                                maneuver_index, 11.9377422f, 679));
  //----------------------------------------------------------------
  // trace point = 635
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4034042f, 40.0973587f, 1491515380, 23.7744026),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4033966f,
                                                40.097374f, leg_index, 21.1056709f, 1685,
                                                maneuver_index, 11.9233704f, 678));
  //----------------------------------------------------------------
  // trace point = 636
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4036789f, 40.0973969f, 1491515381, 23.5252419),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4036713f,
                                                40.0974159f, leg_index, 21.0908527f, 1684,
                                                maneuver_index, 11.9085522f, 677));
  //----------------------------------------------------------------
  // trace point = 637
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4039536f, 40.0974388f, 1491515382, 23.5237179),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4039459f,
                                                40.0974541f, leg_index, 21.0760803f, 1684,
                                                maneuver_index, 11.8937798f, 677));
  //----------------------------------------------------------------
  // trace point = 638
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4042282f, 40.0974808f, 1491515383, 24.0907974),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4042282f,
                                                40.0974922f, leg_index, 21.0609131f, 1683,
                                                maneuver_index, 11.8786125f, 676));
  //----------------------------------------------------------------
  // trace point = 639
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4045029f, 40.0975227f, 1491515384, 24.0465031),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4045029f,
                                                40.0975342f, leg_index, 21.0460949f, 1682,
                                                maneuver_index, 11.8637943f, 675));
  //----------------------------------------------------------------
  // trace point = 640
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4047852f, 40.0975647f, 1491515385, 24.2759514),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4047852f,
                                                40.0975723f, leg_index, 21.0309277f, 1681,
                                                maneuver_index, 11.8486271f, 674));
  //----------------------------------------------------------------
  // trace point = 641
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4050751f, 40.0976067f, 1491515386, 25.364048),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4050751f,
                                                40.0976143f, leg_index, 21.015316f, 1680,
                                                maneuver_index, 11.8330154f, 673));
  //----------------------------------------------------------------
  // trace point = 642
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4053726f, 40.0976486f, 1491515387, 25.7396336),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4053726f,
                                                40.0976562f, leg_index, 20.9993076f, 1680,
                                                maneuver_index, 11.8170071f, 673));
  //----------------------------------------------------------------
  // trace point = 643
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4056778f, 40.0976944f, 1491515388, 26.5770741),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4056778f,
                                                40.097702f, leg_index, 20.982851f, 1679,
                                                maneuver_index, 11.8005505f, 672));
  //----------------------------------------------------------------
  // trace point = 644
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4059906f, 40.097744f, 1491515389, 27.183382),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4059906f,
                                                40.0977478f, leg_index, 20.9660015f, 1678,
                                                maneuver_index, 11.7837009f, 671));
  //----------------------------------------------------------------
  // trace point = 645
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.406311f, 40.0977898f, 1491515390, 27.6423912),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.406311f,
                                                40.0977974f, leg_index, 20.9487038f, 1677,
                                                maneuver_index, 11.7664032f, 670));
  //----------------------------------------------------------------
  // trace point = 646
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4066315f, 40.0978355f, 1491515391, 27.7057152),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4066315f,
                                                40.0978432f, leg_index, 20.9314556f, 1676,
                                                maneuver_index, 11.749155f, 669));
  //----------------------------------------------------------------
  // trace point = 647
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4069595f, 40.0978813f, 1491515392, 27.9569855),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4069595f,
                                                40.0978928f, leg_index, 20.9137611f, 1675,
                                                maneuver_index, 11.7314606f, 668));
  //----------------------------------------------------------------
  // trace point = 648
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4072876f, 40.0979233f, 1491515393, 28.375946),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.40728f,
                                                40.0979424f, leg_index, 20.8964634f, 1674,
                                                maneuver_index, 11.7141628f, 667));
  //----------------------------------------------------------------
  // trace point = 649
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.407608f, 40.0979729f, 1491515394, 28.3120365),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4076004f,
                                                40.0979881f, leg_index, 20.8792152f, 1673,
                                                maneuver_index, 11.6969147f, 666));
  //----------------------------------------------------------------
  // trace point = 650
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4079437f, 40.0980186f, 1491515395, 28.5414696),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4079361f,
                                                40.0980377f, leg_index, 20.8611259f, 1672,
                                                maneuver_index, 11.6788254f, 665));
  //----------------------------------------------------------------
  // trace point = 651
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4082718f, 40.0980759f, 1491515396, 28.6530304),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4082642f,
                                                40.0980911f, leg_index, 20.8433743f, 1670,
                                                maneuver_index, 11.6610737f, 663));
  //----------------------------------------------------------------
  // trace point = 652
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4086075f, 40.0981293f, 1491515397, 29.3424473),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4085999f,
                                                40.0981483f, leg_index, 20.8251762f, 1669,
                                                maneuver_index, 11.6428757f, 662));
  //----------------------------------------------------------------
  // trace point = 653
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4089355f, 40.0981865f, 1491515398, 29.0236778),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4089279f,
                                                40.0982056f, leg_index, 20.8073711f, 1668,
                                                maneuver_index, 11.6250706f, 661));
  //----------------------------------------------------------------
  // trace point = 654
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4092712f, 40.0982552f, 1491515399, 29.2134686),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4092636f,
                                                40.0982666f, leg_index, 20.789114f, 1667,
                                                maneuver_index, 11.6068134f, 660));
  //----------------------------------------------------------------
  // trace point = 655
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4095993f, 40.0983276f, 1491515400, 28.944952),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4095993f,
                                                40.0983353f, leg_index, 20.7707291f, 1667,
                                                maneuver_index, 11.5884285f, 660));
  //----------------------------------------------------------------
  // trace point = 656
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4099197f, 40.0984039f, 1491515401, 28.9881134),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4099197f,
                                                40.0984116f, leg_index, 20.7529716f, 1666,
                                                maneuver_index, 11.5706711f, 659));
  //----------------------------------------------------------------
  // trace point = 657
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4102402f, 40.0984802f, 1491515402, 28.6799583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4102325f,
                                                40.0984879f, leg_index, 20.7355995f, 1664,
                                                maneuver_index, 11.553299f, 657));
  //----------------------------------------------------------------
  // trace point = 658
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4105682f, 40.0985603f, 1491515403, 28.794899),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4105606f,
                                                40.0985756f, leg_index, 20.7172108f, 1663,
                                                maneuver_index, 11.5349102f, 656));
  //----------------------------------------------------------------
  // trace point = 659
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4108887f, 40.0986404f, 1491515404, 28.817831),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.410881f,
                                                40.0986633f, leg_index, 20.6992035f, 1662,
                                                maneuver_index, 11.5169029f, 655));
  //----------------------------------------------------------------
  // trace point = 660
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4112015f, 40.098732f, 1491515405, 28.754034),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4111938f,
                                                40.0987511f, leg_index, 20.6815777f, 1661,
                                                maneuver_index, 11.4992771f, 654));
  //----------------------------------------------------------------
  // trace point = 661
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4115143f, 40.0988274f, 1491515406, 28.8734303),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4115067f,
                                                40.0988426f, leg_index, 20.6638489f, 1661,
                                                maneuver_index, 11.4815483f, 654));
  //----------------------------------------------------------------
  // trace point = 662
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4118347f, 40.0989265f, 1491515407, 29.1562576),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4118271f,
                                                40.0989418f, leg_index, 20.6455593f, 1660,
                                                maneuver_index, 11.4632587f, 653));
  //----------------------------------------------------------------
  // trace point = 663
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4121475f, 40.0990334f, 1491515408, 29.3815918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4121399f,
                                                40.099041f, leg_index, 20.6276417f, 1659,
                                                maneuver_index, 11.4453411f, 652));
  //----------------------------------------------------------------
  // trace point = 664
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.412468f, 40.0991402f, 1491515409, 29.5949039),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4124603f,
                                                40.0991516f, leg_index, 20.6090393f, 1657,
                                                maneuver_index, 11.4267387f, 650));
  //----------------------------------------------------------------
  // trace point = 665
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4127808f, 40.0992546f, 1491515410, 29.7030354),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4127731f,
                                                40.0992622f, leg_index, 20.5908051f, 1656,
                                                maneuver_index, 11.4085045f, 649));
  //----------------------------------------------------------------
  // trace point = 666
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4131012f, 40.0993729f, 1491515411, 30.2012329),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4130936f,
                                                40.0993843f, leg_index, 20.5718651f, 1656,
                                                maneuver_index, 11.3895645f, 649));
  //----------------------------------------------------------------
  // trace point = 667
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4134293f, 40.0994911f, 1491515412, 30.5978813),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.413414f,
                                                40.0995102f, leg_index, 20.552803f, 1655,
                                                maneuver_index, 11.3705025f, 648));
  //----------------------------------------------------------------
  // trace point = 668
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4137421f, 40.0996208f, 1491515413, 30.673645),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4137344f,
                                                40.0996361f, leg_index, 20.5337448f, 1653,
                                                maneuver_index, 11.3514442f, 646));
  //----------------------------------------------------------------
  // trace point = 669
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4140701f, 40.0997505f, 1491515414, 31.0554562),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4140549f,
                                                40.0997658f, leg_index, 20.5145626f, 1652,
                                                maneuver_index, 11.332262f, 645));
  //----------------------------------------------------------------
  // trace point = 670
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4143906f, 40.099884f, 1491515415, 31.235527),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4143829f,
                                                40.0998993f, leg_index, 20.4949017f, 1651,
                                                maneuver_index, 11.3126011f, 644));
  //----------------------------------------------------------------
  // trace point = 671
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4147186f, 40.1000175f, 1491515416, 31.4318924),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.414711f,
                                                40.1000328f, leg_index, 20.4752388f, 1650,
                                                maneuver_index, 11.2929382f, 643));
  //----------------------------------------------------------------
  // trace point = 672
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4150467f, 40.1001511f, 1491515417, 31.7158508),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4150391f,
                                                40.1001663f, leg_index, 20.4555759f, 1649,
                                                maneuver_index, 11.2732754f, 642));
  //----------------------------------------------------------------
  // trace point = 673
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4153748f, 40.1002846f, 1491515418, 31.5589294),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4153595f,
                                                40.1002998f, leg_index, 20.4362698f, 1647,
                                                maneuver_index, 11.2539692f, 640));
  //----------------------------------------------------------------
  // trace point = 674
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4157028f, 40.1004105f, 1491515419, 31.4299068),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4156799f,
                                                40.1004372f, leg_index, 20.4168358f, 1646,
                                                maneuver_index, 11.2345352f, 639));
  //----------------------------------------------------------------
  // trace point = 675
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4160309f, 40.1005402f, 1491515420, 31.6070938),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.416008f,
                                                40.1005707f, leg_index, 20.397171f, 1645,
                                                maneuver_index, 11.2148705f, 638));
  //----------------------------------------------------------------
  // trace point = 676
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4163589f, 40.1006737f, 1491515421, 31.4316559),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4163361f,
                                                40.1007042f, leg_index, 20.3775101f, 1644,
                                                maneuver_index, 11.1952095f, 637));
  //----------------------------------------------------------------
  // trace point = 677
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.416687f, 40.1008034f, 1491515422, 31.3562012),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4166641f,
                                                40.1008339f, leg_index, 20.3579712f, 1643,
                                                maneuver_index, 11.1756706f, 636));
  //----------------------------------------------------------------
  // trace point = 678
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4170074f, 40.1009331f, 1491515423, 31.3561554),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4169922f,
                                                40.1009598f, leg_index, 20.3385525f, 1641,
                                                maneuver_index, 11.1562519f, 634));
  //----------------------------------------------------------------
  // trace point = 679
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4173355f, 40.1010628f, 1491515424, 31.1264648),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4173203f,
                                                40.1010818f, leg_index, 20.3192501f, 1640,
                                                maneuver_index, 11.1369495f, 633));
  //----------------------------------------------------------------
  // trace point = 680
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4176636f, 40.1011963f, 1491515425, 31.5877934),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4176483f,
                                                40.1012154f, leg_index, 20.2995892f, 1639,
                                                maneuver_index, 11.1172886f, 632));
  //----------------------------------------------------------------
  // trace point = 681
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4179916f, 40.1013336f, 1491515426, 31.6721611),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.417984f,
                                                40.1013489f, leg_index, 20.2795696f, 1638,
                                                maneuver_index, 11.0972691f, 631));
  //----------------------------------------------------------------
  // trace point = 682
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4183121f, 40.1014633f, 1491515427, 31.3326588),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4183044f,
                                                40.1014786f, leg_index, 20.2603893f, 1637,
                                                maneuver_index, 11.0780888f, 630));
  //----------------------------------------------------------------
  // trace point = 683
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4186401f, 40.1016006f, 1491515428, 31.3945923),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4186325f,
                                                40.1016121f, leg_index, 20.2407265f, 1636,
                                                maneuver_index, 11.0584259f, 629));
  //----------------------------------------------------------------
  // trace point = 684
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4189606f, 40.1017342f, 1491515429, 31.341013),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4189529f,
                                                40.1017418f, leg_index, 20.2215462f, 1635,
                                                maneuver_index, 11.0392456f, 628));
  //----------------------------------------------------------------
  // trace point = 685
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.419281f, 40.1018715f, 1491515430, 31.1381493),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.419281f,
                                                40.1018715f, leg_index, 20.2020073f, 1633,
                                                maneuver_index, 11.0197067f, 626));
  //----------------------------------------------------------------
  // trace point = 686
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4196014f, 40.102005f, 1491515431, 31.3944473),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4196014f,
                                                40.1020012f, leg_index, 20.182827f, 1633,
                                                maneuver_index, 11.0005264f, 626));
  //----------------------------------------------------------------
  // trace point = 687
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4199219f, 40.1021423f, 1491515432, 31.0964603),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4199295f,
                                                40.1021309f, leg_index, 20.1632881f, 1631,
                                                maneuver_index, 10.9809875f, 624));
  //----------------------------------------------------------------
  // trace point = 688
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4202423f, 40.102272f, 1491515433, 30.5515594),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4202499f,
                                                40.1022606f, leg_index, 20.1441078f, 1630,
                                                maneuver_index, 10.9618073f, 623));
  //----------------------------------------------------------------
  // trace point = 689
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4205551f, 40.1024017f, 1491515434, 30.77672),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4205627f,
                                                40.1023903f, leg_index, 20.1252823f, 1629,
                                                maneuver_index, 10.9429817f, 622));
  //----------------------------------------------------------------
  // trace point = 690
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4208832f, 40.1025314f, 1491515435, 31.1770515),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4208908f,
                                                40.10252f, leg_index, 20.1057453f, 1628,
                                                maneuver_index, 10.9234447f, 621));
  //----------------------------------------------------------------
  // trace point = 691
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4212036f, 40.1026573f, 1491515436, 30.6725712),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4212112f,
                                                40.1026497f, leg_index, 20.0865631f, 1627,
                                                maneuver_index, 10.9042625f, 620));
  //----------------------------------------------------------------
  // trace point = 692
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.421524f, 40.102787f, 1491515437, 30.6209717),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4215317f,
                                                40.1027756f, leg_index, 20.0675049f, 1626,
                                                maneuver_index, 10.8852043f, 619));
  //----------------------------------------------------------------
  // trace point = 693
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4218445f, 40.1029167f, 1491515438, 30.9269352),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4218521f,
                                                40.1029053f, leg_index, 20.0483246f, 1625,
                                                maneuver_index, 10.866024f, 618));
  //----------------------------------------------------------------
  // trace point = 694
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4221649f, 40.1030464f, 1491515439, 31.1528683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4221725f,
                                                40.103035f, leg_index, 20.0291462f, 1624,
                                                maneuver_index, 10.8468456f, 617));
  //----------------------------------------------------------------
  // trace point = 695
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.422493f, 40.1031761f, 1491515440, 31.7086849),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4225006f,
                                                40.1031647f, leg_index, 20.0096054f, 1623,
                                                maneuver_index, 10.8273048f, 616));
  //----------------------------------------------------------------
  // trace point = 696
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.422821f, 40.1033096f, 1491515441, 31.2799435),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4228287f,
                                                40.103302f, leg_index, 19.9898205f, 1622,
                                                maneuver_index, 10.8075199f, 615));
  //----------------------------------------------------------------
  // trace point = 697
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4231491f, 40.1034431f, 1491515442, 31.7675591),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4231491f,
                                                40.1034393f, leg_index, 19.9703865f, 1621,
                                                maneuver_index, 10.7880859f, 614));
  //----------------------------------------------------------------
  // trace point = 698
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4234772f, 40.1035805f, 1491515443, 31.5647526),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4234772f,
                                                40.1035767f, leg_index, 19.9505997f, 1620,
                                                maneuver_index, 10.7682991f, 613));
  //----------------------------------------------------------------
  // trace point = 699
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4237976f, 40.1037102f, 1491515444, 31.2044983),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4237976f,
                                                40.1037102f, leg_index, 19.9312916f, 1619,
                                                maneuver_index, 10.748991f, 612));
  //----------------------------------------------------------------
  // trace point = 700
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4241257f, 40.1038437f, 1491515445, 31.1292),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4241257f,
                                                40.1038361f, leg_index, 19.9118729f, 1618,
                                                maneuver_index, 10.7295723f, 611));
  //----------------------------------------------------------------
  // trace point = 701
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4244537f, 40.1039696f, 1491515446, 31.4792252),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4244537f,
                                                40.1039658f, leg_index, 19.892334f, 1617,
                                                maneuver_index, 10.7100334f, 610));
  //----------------------------------------------------------------
  // trace point = 702
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4247818f, 40.1040993f, 1491515447, 31.4285526),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4247818f,
                                                40.1040955f, leg_index, 19.8727951f, 1616,
                                                maneuver_index, 10.6904945f, 609));
  //----------------------------------------------------------------
  // trace point = 703
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4251022f, 40.104229f, 1491515448, 31.1008892),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4251099f,
                                                40.1042137f, leg_index, 19.8535995f, 1614,
                                                maneuver_index, 10.671299f, 607));
  //----------------------------------------------------------------
  // trace point = 704
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4254303f, 40.1043587f, 1491515449, 31.1008415),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4254379f,
                                                40.1043434f, leg_index, 19.834053f, 1613,
                                                maneuver_index, 10.6517525f, 606));
  //----------------------------------------------------------------
  // trace point = 705
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4257507f, 40.1044884f, 1491515450, 31.1589432),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4257584f,
                                                40.1044731f, leg_index, 19.8148727f, 1612,
                                                maneuver_index, 10.6325722f, 605));
  //----------------------------------------------------------------
  // trace point = 706
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4260788f, 40.1046181f, 1491515451, 31.1762981),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4260864f,
                                                40.1046066f, leg_index, 19.7952118f, 1611,
                                                maneuver_index, 10.6129112f, 604));
  //----------------------------------------------------------------
  // trace point = 707
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4263992f, 40.1047478f, 1491515452, 31.3524475),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4264069f,
                                                40.1047363f, leg_index, 19.7760315f, 1609,
                                                maneuver_index, 10.5937309f, 602));
  //----------------------------------------------------------------
  // trace point = 708
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4267273f, 40.1048775f, 1491515453, 31.2040806),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4267349f,
                                                40.104866f, leg_index, 19.7564945f, 1608,
                                                maneuver_index, 10.574194f, 601));
  //----------------------------------------------------------------
  // trace point = 709
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4270477f, 40.1050072f, 1491515454, 31.0767899),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4270554f,
                                                40.1049957f, leg_index, 19.7373142f, 1607,
                                                maneuver_index, 10.5550137f, 600));
  //----------------------------------------------------------------
  // trace point = 710
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4273758f, 40.1051445f, 1491515455, 31.8418255),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4273834f,
                                                40.1051292f, leg_index, 19.7176533f, 1606,
                                                maneuver_index, 10.5353527f, 599));
  //----------------------------------------------------------------
  // trace point = 711
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4277115f, 40.105278f, 1491515456, 31.910408),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4277191f,
                                                40.1052666f, leg_index, 19.6975117f, 1605,
                                                maneuver_index, 10.5152111f, 598));
  //----------------------------------------------------------------
  // trace point = 712
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4280472f, 40.1054115f, 1491515457, 32.1647263),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4280548f,
                                                40.1054001f, leg_index, 19.677496f, 1604,
                                                maneuver_index, 10.4951954f, 597));
  //----------------------------------------------------------------
  // trace point = 713
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4283829f, 40.105545f, 1491515458, 32.2880363),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4283905f,
                                                40.1055374f, leg_index, 19.6573544f, 1603,
                                                maneuver_index, 10.4750538f, 596));
  //----------------------------------------------------------------
  // trace point = 714
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4287186f, 40.1056786f, 1491515459, 32.1367531),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4287262f,
                                                40.1056709f, leg_index, 19.6373367f, 1602,
                                                maneuver_index, 10.4550362f, 595));
  //----------------------------------------------------------------
  // trace point = 715
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4290466f, 40.1058159f, 1491515460, 32.300251),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4290543f,
                                                40.1058044f, leg_index, 19.6176758f, 1601,
                                                maneuver_index, 10.4353752f, 594));
  //----------------------------------------------------------------
  // trace point = 716
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4293823f, 40.1059532f, 1491515461, 32.172184),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.42939f,
                                                40.105938f, leg_index, 19.5976582f, 1600,
                                                maneuver_index, 10.4153576f, 593));
  //----------------------------------------------------------------
  // trace point = 717
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.429718f, 40.1060905f, 1491515462, 32.1644745),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4297333f,
                                                40.1060715f, leg_index, 19.577282f, 1598,
                                                maneuver_index, 10.3949814f, 591));
  //----------------------------------------------------------------
  // trace point = 718
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4300537f, 40.1062279f, 1491515463, 32.7201424),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.430069f,
                                                40.1062088f, leg_index, 19.5571423f, 1597,
                                                maneuver_index, 10.3748417f, 590));
  //----------------------------------------------------------------
  // trace point = 719
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.430397f, 40.1063614f, 1491515464, 32.743824),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4304123f,
                                                40.1063461f, leg_index, 19.5366459f, 1596,
                                                maneuver_index, 10.3543453f, 589));
  //----------------------------------------------------------------
  // trace point = 720
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4307327f, 40.1065025f, 1491515465, 32.5466652),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.430748f,
                                                40.1064796f, leg_index, 19.5166283f, 1595,
                                                maneuver_index, 10.3343277f, 588));
  //----------------------------------------------------------------
  // trace point = 721
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4310684f, 40.106636f, 1491515466, 32.4657707),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4310837f,
                                                40.1066132f, leg_index, 19.4966106f, 1594,
                                                maneuver_index, 10.3143101f, 587));
  //----------------------------------------------------------------
  // trace point = 722
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4314041f, 40.1067696f, 1491515467, 32.2395248),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4314194f,
                                                40.1067467f, leg_index, 19.476593f, 1593,
                                                maneuver_index, 10.2942924f, 586));
  //----------------------------------------------------------------
  // trace point = 723
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4317398f, 40.1069069f, 1491515468, 32.3148232),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4317551f,
                                                40.106884f, leg_index, 19.4564533f, 1592,
                                                maneuver_index, 10.2741528f, 585));
  //----------------------------------------------------------------
  // trace point = 724
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4320831f, 40.1070404f, 1491515469, 32.6680984),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4320908f,
                                                40.107029f, leg_index, 19.436058f, 1591,
                                                maneuver_index, 10.2537575f, 584));
  //----------------------------------------------------------------
  // trace point = 725
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4324188f, 40.1071777f, 1491515470, 32.4940186),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4324265f,
                                                40.1071701f, leg_index, 19.4157906f, 1589,
                                                maneuver_index, 10.23349f, 582));
  //----------------------------------------------------------------
  // trace point = 726
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4327545f, 40.1073151f, 1491515471, 32.3900719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4327621f,
                                                40.1073074f, leg_index, 19.3956547f, 1588,
                                                maneuver_index, 10.2133541f, 581));
  //----------------------------------------------------------------
  // trace point = 727
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4330978f, 40.1074486f, 1491515472, 32.4387436),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4331055f,
                                                40.1074409f, leg_index, 19.3752632f, 1587,
                                                maneuver_index, 10.1929626f, 580));
  //----------------------------------------------------------------
  // trace point = 728
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4334335f, 40.1075897f, 1491515473, 32.9295311),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4334488f,
                                                40.1075706f, leg_index, 19.3550072f, 1586,
                                                maneuver_index, 10.1727066f, 579));
  //----------------------------------------------------------------
  // trace point = 729
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4337692f, 40.1077232f, 1491515474, 32.3899193),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4337845f,
                                                40.1077003f, leg_index, 19.3351078f, 1585,
                                                maneuver_index, 10.1528072f, 578));
  //----------------------------------------------------------------
  // trace point = 730
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4341125f, 40.1078606f, 1491515475, 32.8187752),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4341278f,
                                                40.1078377f, leg_index, 19.3146114f, 1584,
                                                maneuver_index, 10.1323109f, 577));
  //----------------------------------------------------------------
  // trace point = 731
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4344559f, 40.1079979f, 1491515476, 32.92202),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4344711f,
                                                40.107975f, leg_index, 19.2941132f, 1583,
                                                maneuver_index, 10.1118126f, 576));
  //----------------------------------------------------------------
  // trace point = 732
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4347992f, 40.108139f, 1491515477, 33.1540871),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4348145f,
                                                40.1081123f, leg_index, 19.2736168f, 1582,
                                                maneuver_index, 10.0913162f, 575));
  //----------------------------------------------------------------
  // trace point = 733
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4351425f, 40.1082764f, 1491515478, 33.148037),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4351578f,
                                                40.1082497f, leg_index, 19.2531185f, 1580,
                                                maneuver_index, 10.0708179f, 573));
  //----------------------------------------------------------------
  // trace point = 734
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4354858f, 40.1084175f, 1491515479, 33.0790138),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4355087f,
                                                40.1083908f, leg_index, 19.2321415f, 1579,
                                                maneuver_index, 10.0498409f, 572));
  //----------------------------------------------------------------
  // trace point = 735
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4358292f, 40.108551f, 1491515480, 32.7673073),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4358444f,
                                                40.1085281f, leg_index, 19.2120018f, 1578,
                                                maneuver_index, 10.0297012f, 571));
  //----------------------------------------------------------------
  // trace point = 736
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4361649f, 40.1086884f, 1491515481, 32.6439133),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4361801f,
                                                40.1086617f, leg_index, 19.1919842f, 1577,
                                                maneuver_index, 10.0096836f, 570));
  //----------------------------------------------------------------
  // trace point = 737
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4365082f, 40.1088257f, 1491515482, 32.5166321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4365234f,
                                                40.1088028f, leg_index, 19.1713638f, 1575,
                                                maneuver_index, 9.98906326f, 568));
  //----------------------------------------------------------------
  // trace point = 738
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4368439f, 40.1089592f, 1491515483, 32.464901),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4368591f,
                                                40.1089363f, leg_index, 19.1513481f, 1574,
                                                maneuver_index, 9.96904755f, 567));
  //----------------------------------------------------------------
  // trace point = 739
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4371872f, 40.1090965f, 1491515484, 32.8427811),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4372025f,
                                                40.1090736f, leg_index, 19.1308517f, 1573,
                                                maneuver_index, 9.94855118f, 566));
  //----------------------------------------------------------------
  // trace point = 740
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4375305f, 40.1092262f, 1491515485, 32.7673454),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4375381f,
                                                40.109211f, leg_index, 19.1107101f, 1572,
                                                maneuver_index, 9.92840958f, 565));
  //----------------------------------------------------------------
  // trace point = 741
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4378738f, 40.1093559f, 1491515486, 32.7718735),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4378815f,
                                                40.1093483f, leg_index, 19.0902138f, 1571,
                                                maneuver_index, 9.90791321f, 564));
  //----------------------------------------------------------------
  // trace point = 742
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4382248f, 40.1094894f, 1491515487, 33.0719872),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4382248f,
                                                40.1094894f, leg_index, 19.0695934f, 1570,
                                                maneuver_index, 9.88729286f, 563));
  //----------------------------------------------------------------
  // trace point = 743
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4385757f, 40.1096306f, 1491515488, 33.4538841),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4385757f,
                                                40.1096306f, leg_index, 19.0486183f, 1568,
                                                maneuver_index, 9.86631775f, 561));
  //----------------------------------------------------------------
  // trace point = 744
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4389191f, 40.1097717f, 1491515489, 33.581295),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4389191f,
                                                40.1097717f, leg_index, 19.0279961f, 1567,
                                                maneuver_index, 9.8456955f, 560));
  //----------------------------------------------------------------
  // trace point = 745
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.43927f, 40.1099129f, 1491515490, 33.5061493),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.43927f,
                                                40.1099129f, leg_index, 19.007019f, 1566,
                                                maneuver_index, 9.82471848f, 559));
  //----------------------------------------------------------------
  // trace point = 746
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4396133f, 40.110054f, 1491515491, 33.2811089),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4396133f,
                                                40.1100502f, leg_index, 18.9865227f, 1565,
                                                maneuver_index, 9.80422211f, 558));
  //----------------------------------------------------------------
  // trace point = 747
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4399567f, 40.110199f, 1491515492, 33.3664284),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4399643f,
                                                40.1101913f, leg_index, 18.9655457f, 1564,
                                                maneuver_index, 9.78324509f, 557));
  //----------------------------------------------------------------
  // trace point = 748
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4402924f, 40.1103439f, 1491515493, 32.7172432),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4403f,
                                                40.1103287f, leg_index, 18.9454021f, 1563,
                                                maneuver_index, 9.76310158f, 556));
  //----------------------------------------------------------------
  // trace point = 749
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4406204f, 40.1104889f, 1491515494, 32.3658485),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4406281f,
                                                40.1104736f, leg_index, 18.9253597f, 1562,
                                                maneuver_index, 9.74305916f, 555));
  //----------------------------------------------------------------
  // trace point = 750
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4409485f, 40.1106339f, 1491515495, 32.2367973),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4409561f,
                                                40.1106186f, leg_index, 18.9053154f, 1561,
                                                maneuver_index, 9.72301483f, 554));
  //----------------------------------------------------------------
  // trace point = 751
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4412766f, 40.1107788f, 1491515496, 32.2930107),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4412842f,
                                                40.1107635f, leg_index, 18.8852711f, 1560,
                                                maneuver_index, 9.7029705f, 553));
  //----------------------------------------------------------------
  // trace point = 752
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.441597f, 40.1109314f, 1491515497, 32.3024826),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4416122f,
                                                40.1109123f, leg_index, 18.8650913f, 1558,
                                                maneuver_index, 9.68279076f, 551));
  //----------------------------------------------------------------
  // trace point = 753
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4419174f, 40.1110802f, 1491515498, 32.0568542),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4419327f,
                                                40.1110649f, leg_index, 18.8451233f, 1557,
                                                maneuver_index, 9.66282272f, 550));
  //----------------------------------------------------------------
  // trace point = 754
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4422379f, 40.1112328f, 1491515499, 31.8539734),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4422455f,
                                                40.1112213f, leg_index, 18.8253555f, 1556,
                                                maneuver_index, 9.64305496f, 549));
  //----------------------------------------------------------------
  // trace point = 755
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4425507f, 40.1113853f, 1491515500, 31.7461777),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4425583f,
                                                40.1113739f, leg_index, 18.805727f, 1555,
                                                maneuver_index, 9.62342644f, 548));
  //----------------------------------------------------------------
  // trace point = 756
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4428635f, 40.1115417f, 1491515501, 31.7461281),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4428711f,
                                                40.1115303f, leg_index, 18.7859573f, 1554,
                                                maneuver_index, 9.60365677f, 547));
  //----------------------------------------------------------------
  // trace point = 757
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4431763f, 40.1116943f, 1491515502, 31.5426464),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4431839f,
                                                40.1116829f, leg_index, 18.7658482f, 1553,
                                                maneuver_index, 9.58354759f, 546));
  //----------------------------------------------------------------
  // trace point = 758
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4434891f, 40.1118507f, 1491515503, 31.6032352),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4434967f,
                                                40.1118431f, leg_index, 18.7464142f, 1552,
                                                maneuver_index, 9.56411362f, 545));
  //----------------------------------------------------------------
  // trace point = 759
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4438019f, 40.1120071f, 1491515504, 31.8675308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4438095f,
                                                40.1120033f, leg_index, 18.7264977f, 1551,
                                                maneuver_index, 9.54419708f, 544));
  //----------------------------------------------------------------
  // trace point = 760
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4441071f, 40.1121597f, 1491515505, 31.4710636),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4441071f,
                                                40.1121559f, leg_index, 18.7075462f, 1550,
                                                maneuver_index, 9.52524567f, 543));
  //----------------------------------------------------------------
  // trace point = 761
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4444199f, 40.1123161f, 1491515506, 31.3996525),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4444199f,
                                                40.1123161f, leg_index, 18.6876297f, 1549,
                                                maneuver_index, 9.50532913f, 542));
  //----------------------------------------------------------------
  // trace point = 762
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4447327f, 40.1124725f, 1491515507, 31.6639481),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4447327f,
                                                40.1124763f, leg_index, 18.6677151f, 1548,
                                                maneuver_index, 9.48541451f, 541));
  //----------------------------------------------------------------
  // trace point = 763
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4450378f, 40.1126289f, 1491515508, 31.7156677),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4450378f,
                                                40.1126328f, leg_index, 18.6482811f, 1546,
                                                maneuver_index, 9.46598053f, 539));
  //----------------------------------------------------------------
  // trace point = 764
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.445343f, 40.1127815f, 1491515509, 31.1960716),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4453354f,
                                                40.1127892f, leg_index, 18.6291828f, 1545,
                                                maneuver_index, 9.44688225f, 538));
  //----------------------------------------------------------------
  // trace point = 765
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4456558f, 40.1129341f, 1491515510, 31.5422401),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4456482f,
                                                40.1129456f, leg_index, 18.6094284f, 1544,
                                                maneuver_index, 9.42712784f, 537));
  //----------------------------------------------------------------
  // trace point = 766
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.445961f, 40.1130867f, 1491515511, 30.8604774),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4459534f,
                                                40.1130943f, leg_index, 18.5902615f, 1543,
                                                maneuver_index, 9.40796089f, 536));
  //----------------------------------------------------------------
  // trace point = 767
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4462738f, 40.1132393f, 1491515512, 31.2783546),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4462662f,
                                                40.1132431f, leg_index, 18.570776f, 1542,
                                                maneuver_index, 9.38847542f, 535));
  //----------------------------------------------------------------
  // trace point = 768
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4465866f, 40.1133919f, 1491515513, 31.8284779),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4465866f,
                                                40.1133919f, leg_index, 18.5509472f, 1541,
                                                maneuver_index, 9.36864662f, 534));
  //----------------------------------------------------------------
  // trace point = 769
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4468918f, 40.1135521f, 1491515514, 31.7153702),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4468994f,
                                                40.1135445f, leg_index, 18.5313187f, 1540,
                                                maneuver_index, 9.3490181f, 533));
  //----------------------------------------------------------------
  // trace point = 770
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4472046f, 40.1137047f, 1491515515, 31.5313396),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4472046f,
                                                40.1137009f, leg_index, 18.5118847f, 1539,
                                                maneuver_index, 9.32958412f, 532));
  //----------------------------------------------------------------
  // trace point = 771
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4475174f, 40.1138649f, 1491515516, 31.7861462),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.447525f,
                                                40.1138573f, leg_index, 18.4917774f, 1538,
                                                maneuver_index, 9.30947685f, 531));
  //----------------------------------------------------------------
  // trace point = 772
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4478302f, 40.1140213f, 1491515517, 31.7346649),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4478378f,
                                                40.1140137f, leg_index, 18.4720058f, 1537,
                                                maneuver_index, 9.28970528f, 530));
  //----------------------------------------------------------------
  // trace point = 773
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.448143f, 40.1141777f, 1491515518, 31.8059044),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4481506f,
                                                40.1141701f, leg_index, 18.4522362f, 1536,
                                                maneuver_index, 9.26993561f, 529));
  //----------------------------------------------------------------
  // trace point = 774
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4484558f, 40.1143379f, 1491515519, 32.2545586),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4484634f,
                                                40.1143265f, leg_index, 18.4324646f, 1535,
                                                maneuver_index, 9.25016403f, 528));
  //----------------------------------------------------------------
  // trace point = 775
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4487762f, 40.1144943f, 1491515520, 32.2735672),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4487839f,
                                                40.1144867f, leg_index, 18.4122143f, 1534,
                                                maneuver_index, 9.22991371f, 527));
  //----------------------------------------------------------------
  // trace point = 776
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4490891f, 40.1146584f, 1491515521, 32.5286865),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4491043f,
                                                40.1146431f, leg_index, 18.3921051f, 1532,
                                                maneuver_index, 9.20980453f, 525));
  //----------------------------------------------------------------
  // trace point = 777
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4494171f, 40.1148186f, 1491515522, 32.8941727),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4494247f,
                                                40.1148071f, leg_index, 18.3717155f, 1531,
                                                maneuver_index, 9.18941498f, 524));
  //----------------------------------------------------------------
  // trace point = 778
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4497375f, 40.114975f, 1491515523, 32.6536446),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4497452f,
                                                40.1149635f, leg_index, 18.3516006f, 1530,
                                                maneuver_index, 9.16930008f, 523));
  //----------------------------------------------------------------
  // trace point = 779
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.450058f, 40.1151314f, 1491515524, 32.2594414),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4500656f,
                                                40.1151237f, leg_index, 18.3313484f, 1529,
                                                maneuver_index, 9.14904785f, 522));
  //----------------------------------------------------------------
  // trace point = 780
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.450386f, 40.1152878f, 1491515525, 32.6661644),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.450386f,
                                                40.115284f, leg_index, 18.3110981f, 1527,
                                                maneuver_index, 9.12879753f, 520));
  //----------------------------------------------------------------
  // trace point = 781
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4506989f, 40.1154442f, 1491515526, 32.1516876),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4507065f,
                                                40.1154404f, leg_index, 18.2909889f, 1526,
                                                maneuver_index, 9.10868835f, 519));
  //----------------------------------------------------------------
  // trace point = 782
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4510193f, 40.1156044f, 1491515527, 32.4766235),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4510269f,
                                                40.1156006f, leg_index, 18.2707386f, 1525,
                                                maneuver_index, 9.08843803f, 518));
  //----------------------------------------------------------------
  // trace point = 783
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4513474f, 40.1157646f, 1491515528, 32.8333321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4513474f,
                                                40.1157608f, leg_index, 18.2504864f, 1524,
                                                maneuver_index, 9.06818581f, 517));
  //----------------------------------------------------------------
  // trace point = 784
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4516678f, 40.1159248f, 1491515529, 33.0156326),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4516678f,
                                                40.115921f, leg_index, 18.2302341f, 1523,
                                                maneuver_index, 9.04793358f, 516));
  //----------------------------------------------------------------
  // trace point = 785
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4519806f, 40.1160889f, 1491515530, 32.1832428),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4519882f,
                                                40.1160812f, leg_index, 18.2099819f, 1522,
                                                maneuver_index, 9.02768135f, 515));
  //----------------------------------------------------------------
  // trace point = 786
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.452301f, 40.1162491f, 1491515531, 32.5990448),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4523087f,
                                                40.1162376f, leg_index, 18.1898746f, 1521,
                                                maneuver_index, 9.00757408f, 514));
  //----------------------------------------------------------------
  // trace point = 787
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4526215f, 40.1164131f, 1491515532, 32.4486618),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4526291f,
                                                40.1164017f, leg_index, 18.1694832f, 1519,
                                                maneuver_index, 8.98718262f, 512));
  //----------------------------------------------------------------
  // trace point = 788
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4529343f, 40.1165695f, 1491515533, 32.3547325),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4529419f,
                                                40.1165581f, leg_index, 18.1497097f, 1518,
                                                maneuver_index, 8.96740913f, 511));
  //----------------------------------------------------------------
  // trace point = 789
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4532547f, 40.1167297f, 1491515534, 32.5474663),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4532623f,
                                                40.1167221f, leg_index, 18.1293106f, 1516,
                                                maneuver_index, 8.94701004f, 509));
  //----------------------------------------------------------------
  // trace point = 790
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4535751f, 40.1168861f, 1491515535, 32.4976654),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4535751f,
                                                40.1168861f, leg_index, 18.1092472f, 1515,
                                                maneuver_index, 8.92694664f, 508));
  //----------------------------------------------------------------
  // trace point = 791
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4538956f, 40.1170464f, 1491515536, 32.5579948),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4538956f,
                                                40.1170502f, leg_index, 18.0888519f, 1514,
                                                maneuver_index, 8.90655136f, 507));
  //----------------------------------------------------------------
  // trace point = 792
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4542236f, 40.1172066f, 1491515537, 32.6408539),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.454216f,
                                                40.1172142f, leg_index, 18.0684528f, 1513,
                                                maneuver_index, 8.88615227f, 506));
  //----------------------------------------------------------------
  // trace point = 793
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4545441f, 40.117363f, 1491515538, 32.4864693),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4545364f,
                                                40.1173706f, leg_index, 18.0483456f, 1512,
                                                maneuver_index, 8.866045f, 505));
  //----------------------------------------------------------------
  // trace point = 794
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4548569f, 40.117527f, 1491515539, 32.5895844),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4548569f,
                                                40.117527f, leg_index, 18.0282364f, 1511,
                                                maneuver_index, 8.84593582f, 504));
  //----------------------------------------------------------------
  // trace point = 795
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4551849f, 40.1176872f, 1491515540, 32.8827667),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4551849f,
                                                40.1176872f, leg_index, 18.0076466f, 1510,
                                                maneuver_index, 8.82534599f, 503));
  //----------------------------------------------------------------
  // trace point = 796
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4555054f, 40.1178513f, 1491515541, 32.7404785),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4555054f,
                                                40.1178474f, leg_index, 17.9873962f, 1509,
                                                maneuver_index, 8.80509567f, 502));
  //----------------------------------------------------------------
  // trace point = 797
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4558258f, 40.1180115f, 1491515542, 32.6694069),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4558258f,
                                                40.1180077f, leg_index, 17.9671459f, 1507,
                                                maneuver_index, 8.78484535f, 500));
  //----------------------------------------------------------------
  // trace point = 798
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4561462f, 40.1181755f, 1491515543, 32.679184),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4561539f,
                                                40.1181641f, leg_index, 17.9466972f, 1506,
                                                maneuver_index, 8.76439667f, 499));
  //----------------------------------------------------------------
  // trace point = 799
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4564667f, 40.1183357f, 1491515544, 32.6791306),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4564819f,
                                                40.1183205f, leg_index, 17.9262486f, 1505,
                                                maneuver_index, 8.74394798f, 498));
  //----------------------------------------------------------------
  // trace point = 800
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4567871f, 40.1184921f, 1491515545, 32.3053474),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4567947f,
                                                40.1184769f, leg_index, 17.9064789f, 1504,
                                                maneuver_index, 8.72417831f, 497));
  //----------------------------------------------------------------
  // trace point = 801
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4571075f, 40.1186485f, 1491515546, 32.4369545),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4571152f,
                                                40.1186371f, leg_index, 17.8862286f, 1503,
                                                maneuver_index, 8.70392799f, 496));
  //----------------------------------------------------------------
  // trace point = 802
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.457428f, 40.1188087f, 1491515547, 32.7501869),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4574356f,
                                                40.1187973f, leg_index, 17.8659763f, 1502,
                                                maneuver_index, 8.68367577f, 495));
  //----------------------------------------------------------------
  // trace point = 803
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4577484f, 40.1189728f, 1491515548, 32.9435158),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.457756f,
                                                40.1189575f, leg_index, 17.845726f, 1501,
                                                maneuver_index, 8.66342545f, 494));
  //----------------------------------------------------------------
  // trace point = 804
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4580688f, 40.1191368f, 1491515549, 32.5183754),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4580841f,
                                                40.1191216f, leg_index, 17.8249931f, 1499,
                                                maneuver_index, 8.64269257f, 492));
  //----------------------------------------------------------------
  // trace point = 805
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4583893f, 40.1192932f, 1491515550, 32.4253044),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4583969f,
                                                40.119278f, leg_index, 17.8052235f, 1498,
                                                maneuver_index, 8.6229229f, 491));
  //----------------------------------------------------------------
  // trace point = 806
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4587097f, 40.1194496f, 1491515551, 32.3537674),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4587173f,
                                                40.1194382f, leg_index, 17.7849731f, 1497,
                                                maneuver_index, 8.60267258f, 490));
  //----------------------------------------------------------------
  // trace point = 807
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4590302f, 40.1196098f, 1491515552, 32.3649483),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4590378f,
                                                40.1195984f, leg_index, 17.7647228f, 1496,
                                                maneuver_index, 8.58242226f, 489));
  //----------------------------------------------------------------
  // trace point = 808
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.459343f, 40.1197662f, 1491515553, 32.2217331),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4593506f,
                                                40.1197586f, leg_index, 17.7448082f, 1495,
                                                maneuver_index, 8.56250763f, 488));
  //----------------------------------------------------------------
  // trace point = 809
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4596634f, 40.1199226f, 1491515554, 32.485611),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.459671f,
                                                40.119915f, leg_index, 17.7247028f, 1494,
                                                maneuver_index, 8.54240227f, 487));
  //----------------------------------------------------------------
  // trace point = 810
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4599838f, 40.1200829f, 1491515555, 32.4142075),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4599915f,
                                                40.120079f, leg_index, 17.7043076f, 1492,
                                                maneuver_index, 8.52200699f, 485));
  //----------------------------------------------------------------
  // trace point = 811
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4603043f, 40.1202431f, 1491515556, 32.4141541),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4603119f,
                                                40.1202354f, leg_index, 17.6842003f, 1491,
                                                maneuver_index, 8.50189972f, 484));
  //----------------------------------------------------------------
  // trace point = 812
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4606247f, 40.1203995f, 1491515557, 32.6998863),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4606323f,
                                                40.120388f, leg_index, 17.6642342f, 1490,
                                                maneuver_index, 8.48193359f, 483));
  //----------------------------------------------------------------
  // trace point = 813
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4609451f, 40.1205597f, 1491515558, 32.474968),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4609528f,
                                                40.1205482f, leg_index, 17.6439819f, 1489,
                                                maneuver_index, 8.46168137f, 482));
  //----------------------------------------------------------------
  // trace point = 814
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4612656f, 40.1207199f, 1491515559, 32.4749146),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4612732f,
                                                40.1207085f, leg_index, 17.6237316f, 1488,
                                                maneuver_index, 8.44143105f, 481));
  //----------------------------------------------------------------
  // trace point = 815
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4615784f, 40.1208801f, 1491515560, 32.1292267),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.461586f,
                                                40.1208649f, leg_index, 17.6039619f, 1487,
                                                maneuver_index, 8.42166138f, 480));
  //----------------------------------------------------------------
  // trace point = 816
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4618988f, 40.1210365f, 1491515561, 32.0070648),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4619064f,
                                                40.1210251f, leg_index, 17.5837116f, 1486,
                                                maneuver_index, 8.40141106f, 479));
  //----------------------------------------------------------------
  // trace point = 817
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4622116f, 40.1211891f, 1491515562, 31.6224861),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4622192f,
                                                40.1211815f, leg_index, 17.5639439f, 1484,
                                                maneuver_index, 8.3816433f, 477));
  //----------------------------------------------------------------
  // trace point = 818
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4625168f, 40.1213455f, 1491515563, 31.5188427),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4625244f,
                                                40.1213341f, leg_index, 17.5446568f, 1483,
                                                maneuver_index, 8.36235619f, 476));
  //----------------------------------------------------------------
  // trace point = 819
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.462822f, 40.1214981f, 1491515564, 30.9796333),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4628296f,
                                                40.1214867f, leg_index, 17.5253716f, 1482,
                                                maneuver_index, 8.34307098f, 475));
  //----------------------------------------------------------------
  // trace point = 820
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4631271f, 40.1216469f, 1491515565, 30.7370319),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4631348f,
                                                40.1216393f, leg_index, 17.5060844f, 1481,
                                                maneuver_index, 8.32378387f, 474));
  //----------------------------------------------------------------
  // trace point = 821
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4634247f, 40.1217995f, 1491515566, 30.7150669),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4634323f,
                                                40.121788f, leg_index, 17.4872799f, 1480,
                                                maneuver_index, 8.30497932f, 473));
  //----------------------------------------------------------------
  // trace point = 822
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4637222f, 40.1219482f, 1491515567, 29.8913784),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4637299f,
                                                40.1219368f, leg_index, 17.4684772f, 1479,
                                                maneuver_index, 8.28617668f, 472));
  //----------------------------------------------------------------
  // trace point = 823
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4640198f, 40.122097f, 1491515568, 30.3081512),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4640274f,
                                                40.1220856f, leg_index, 17.4496727f, 1478,
                                                maneuver_index, 8.26737213f, 471));
  //----------------------------------------------------------------
  // trace point = 824
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4643097f, 40.122242f, 1491515569, 29.9119339),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4643173f,
                                                40.1222305f, leg_index, 17.4313507f, 1477,
                                                maneuver_index, 8.24905014f, 470));
  //----------------------------------------------------------------
  // trace point = 825
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4646072f, 40.1223907f, 1491515570, 29.9438343),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4646149f,
                                                40.1223793f, leg_index, 17.4125462f, 1476,
                                                maneuver_index, 8.23024559f, 469));
  //----------------------------------------------------------------
  // trace point = 826
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4649048f, 40.1225395f, 1491515571, 30.2368031),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4649124f,
                                                40.1225319f, leg_index, 17.3935966f, 1475,
                                                maneuver_index, 8.21129608f, 468));
  //----------------------------------------------------------------
  // trace point = 827
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4651947f, 40.1226883f, 1491515572, 29.687746),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4652023f,
                                                40.1226807f, leg_index, 17.3751278f, 1474,
                                                maneuver_index, 8.19282722f, 467));
  //----------------------------------------------------------------
  // trace point = 828
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4654846f, 40.1228371f, 1491515573, 29.7492046),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4654922f,
                                                40.1228256f, leg_index, 17.3568134f, 1473,
                                                maneuver_index, 8.17451286f, 466));
  //----------------------------------------------------------------
  // trace point = 829
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4657745f, 40.122982f, 1491515574, 29.697649),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4657898f,
                                                40.1229668f, leg_index, 17.3382893f, 1472,
                                                maneuver_index, 8.15598869f, 465));
  //----------------------------------------------------------------
  // trace point = 830
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4660721f, 40.123127f, 1491515575, 29.7195606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4660797f,
                                                40.1231117f, leg_index, 17.3199673f, 1471,
                                                maneuver_index, 8.1376667f, 464));
  //----------------------------------------------------------------
  // trace point = 831
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.466362f, 40.1232719f, 1491515576, 29.6367836),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4663773f,
                                                40.1232567f, leg_index, 17.3013077f, 1470,
                                                maneuver_index, 8.11900711f, 463));
  //----------------------------------------------------------------
  // trace point = 832
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4666519f, 40.1234169f, 1491515577, 29.4747906),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4666672f,
                                                40.1234016f, leg_index, 17.2829857f, 1469,
                                                maneuver_index, 8.10068512f, 462));
  //----------------------------------------------------------------
  // trace point = 833
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4669342f, 40.1235619f, 1491515578, 28.670454),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4669495f,
                                                40.1235466f, leg_index, 17.2649994f, 1468,
                                                maneuver_index, 8.08269882f, 461));
  //----------------------------------------------------------------
  // trace point = 834
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4672165f, 40.1236992f, 1491515579, 28.8459244),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4672241f,
                                                40.1236839f, leg_index, 17.2476425f, 1467,
                                                maneuver_index, 8.06534195f, 460));
  //----------------------------------------------------------------
  // trace point = 835
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4674988f, 40.1238327f, 1491515580, 28.4204845),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4675064f,
                                                40.1238251f, leg_index, 17.2298031f, 1466,
                                                maneuver_index, 8.04750252f, 459));
  //----------------------------------------------------------------
  // trace point = 836
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4677887f, 40.1239738f, 1491515581, 28.8943748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4677963f,
                                                40.12397f, leg_index, 17.211483f, 1465,
                                                maneuver_index, 8.02918243f, 458));
  //----------------------------------------------------------------
  // trace point = 837
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.468071f, 40.1241112f, 1491515582, 28.762434),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.468071f,
                                                40.1241074f, leg_index, 17.1941261f, 1464,
                                                maneuver_index, 8.01182556f, 457));
  //----------------------------------------------------------------
  // trace point = 838
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4683533f, 40.1242561f, 1491515583, 28.793602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4683609f,
                                                40.1242523f, leg_index, 17.1758041f, 1463,
                                                maneuver_index, 7.99350357f, 456));
  //----------------------------------------------------------------
  // trace point = 839
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4686356f, 40.1243973f, 1491515584, 28.8835926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4686432f,
                                                40.1243935f, leg_index, 17.1579647f, 1462,
                                                maneuver_index, 7.97566414f, 455));
  //----------------------------------------------------------------
  // trace point = 840
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4689255f, 40.1245461f, 1491515585, 29.2710571),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4689331f,
                                                40.1245384f, leg_index, 17.1396427f, 1461,
                                                maneuver_index, 7.95734215f, 454));
  //----------------------------------------------------------------
  // trace point = 841
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4692078f, 40.124691f, 1491515586, 29.3418446),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4692154f,
                                                40.1246796f, leg_index, 17.1218052f, 1460,
                                                maneuver_index, 7.93950462f, 453));
  //----------------------------------------------------------------
  // trace point = 842
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4694977f, 40.1248322f, 1491515587, 29.1208382),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4695053f,
                                                40.1248245f, leg_index, 17.1034832f, 1459,
                                                maneuver_index, 7.92118263f, 452));
  //----------------------------------------------------------------
  // trace point = 843
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4697952f, 40.1249733f, 1491515588, 29.7189865),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4697952f,
                                                40.1249695f, leg_index, 17.0851612f, 1458,
                                                maneuver_index, 7.90286064f, 451));
  //----------------------------------------------------------------
  // trace point = 844
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4700928f, 40.1251183f, 1491515589, 30.0190163),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4700928f,
                                                40.1251144f, leg_index, 17.0665016f, 1457,
                                                maneuver_index, 7.88420105f, 450));
  //----------------------------------------------------------------
  // trace point = 845
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4703903f, 40.1252632f, 1491515590, 30.2815247),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4703903f,
                                                40.1252632f, leg_index, 17.047699f, 1456,
                                                maneuver_index, 7.86539841f, 449));
  //----------------------------------------------------------------
  // trace point = 846
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4706879f, 40.1254082f, 1491515591, 29.9939556),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4706879f,
                                                40.1254082f, leg_index, 17.0290394f, 1455,
                                                maneuver_index, 7.84673882f, 448));
  //----------------------------------------------------------------
  // trace point = 847
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.470993f, 40.1255531f, 1491515592, 30.2944336),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4709854f,
                                                40.1255569f, leg_index, 17.0102348f, 1454,
                                                maneuver_index, 7.82793427f, 447));
  //----------------------------------------------------------------
  // trace point = 848
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.471283f, 40.1257019f, 1491515593, 30.0324192),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4712753f,
                                                40.1257057f, leg_index, 16.9917698f, 1453,
                                                maneuver_index, 7.80946922f, 446));
  //----------------------------------------------------------------
  // trace point = 849
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4715881f, 40.1258507f, 1491515594, 30.8073807),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4715805f,
                                                40.1258583f, leg_index, 16.9724846f, 1452,
                                                maneuver_index, 7.79018402f, 445));
  //----------------------------------------------------------------
  // trace point = 850
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4718933f, 40.1259956f, 1491515595, 30.6880379),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4718781f,
                                                40.1260109f, leg_index, 16.9535351f, 1450,
                                                maneuver_index, 7.77123451f, 443));
  //----------------------------------------------------------------
  // trace point = 851
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4722061f, 40.1261444f, 1491515596, 30.9508781),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4721832f,
                                                40.1261711f, leg_index, 16.9339523f, 1449,
                                                maneuver_index, 7.75165176f, 442));
  //----------------------------------------------------------------
  // trace point = 852
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4725113f, 40.126297f, 1491515597, 31.2140179),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4724884f,
                                                40.1263275f, leg_index, 16.9145203f, 1448,
                                                maneuver_index, 7.7322197f, 441));
  //----------------------------------------------------------------
  // trace point = 853
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4728165f, 40.1264496f, 1491515598, 31.3237991),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4727936f,
                                                40.1264763f, leg_index, 16.8953781f, 1447,
                                                maneuver_index, 7.71307755f, 440));
  //----------------------------------------------------------------
  // trace point = 854
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4731293f, 40.1266022f, 1491515599, 31.2629642),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4731064f,
                                                40.1266289f, leg_index, 16.8757553f, 1446,
                                                maneuver_index, 7.69345474f, 439));
  //----------------------------------------------------------------
  // trace point = 855
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4734344f, 40.1267586f, 1491515600, 31.4558964),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4734116f,
                                                40.1267815f, leg_index, 16.8564701f, 1445,
                                                maneuver_index, 7.67416954f, 438));
  //----------------------------------------------------------------
  // trace point = 856
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4737549f, 40.1269188f, 1491515601, 32.1887016),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4737396f,
                                                40.1269379f, leg_index, 16.8360233f, 1444,
                                                maneuver_index, 7.65372276f, 437));
  //----------------------------------------------------------------
  // trace point = 857
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4740677f, 40.127079f, 1491515602, 32.1089935),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4740601f,
                                                40.1270905f, leg_index, 16.8160553f, 1442,
                                                maneuver_index, 7.63375473f, 435));
  //----------------------------------------------------------------
  // trace point = 858
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4743881f, 40.1272392f, 1491515603, 32.554554),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4743805f,
                                                40.1272507f, leg_index, 16.7958069f, 1441,
                                                maneuver_index, 7.61350632f, 434));
  //----------------------------------------------------------------
  // trace point = 859
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4747086f, 40.1273994f, 1491515604, 32.5953331),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4747009f,
                                                40.1274071f, leg_index, 16.7757015f, 1440,
                                                maneuver_index, 7.59340096f, 433));
  //----------------------------------------------------------------
  // trace point = 860
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.475029f, 40.1275597f, 1491515605, 32.8898392),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4750214f,
                                                40.1275673f, leg_index, 16.7554512f, 1439,
                                                maneuver_index, 7.57315063f, 432));
  //----------------------------------------------------------------
  // trace point = 861
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4753494f, 40.1277199f, 1491515606, 32.7689972),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4753418f,
                                                40.1277275f, leg_index, 16.7352028f, 1438,
                                                maneuver_index, 7.55290222f, 431));
  //----------------------------------------------------------------
  // trace point = 862
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4756775f, 40.1278801f, 1491515607, 32.6863327),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4756699f,
                                                40.1278915f, leg_index, 16.7144718f, 1436,
                                                maneuver_index, 7.53217125f, 429));
  //----------------------------------------------------------------
  // trace point = 863
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4759979f, 40.1280441f, 1491515608, 32.7276421),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4759903f,
                                                40.1280518f, leg_index, 16.6942215f, 1435,
                                                maneuver_index, 7.51192093f, 428));
  //----------------------------------------------------------------
  // trace point = 864
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4763184f, 40.1282043f, 1491515609, 32.6660004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4763107f,
                                                40.128212f, leg_index, 16.6739731f, 1434,
                                                maneuver_index, 7.49167252f, 427));
  //----------------------------------------------------------------
  // trace point = 865
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4766388f, 40.1283607f, 1491515610, 32.7209816),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4766312f,
                                                40.1283684f, leg_index, 16.6538677f, 1433,
                                                maneuver_index, 7.47156715f, 426));
  //----------------------------------------------------------------
  // trace point = 866
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4769592f, 40.1285172f, 1491515611, 32.4456291),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4769516f,
                                                40.1285248f, leg_index, 16.6337624f, 1432,
                                                maneuver_index, 7.45146179f, 425));
  //----------------------------------------------------------------
  // trace point = 867
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4772873f, 40.1286659f, 1491515612, 32.4001083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4772797f,
                                                40.1286736f, leg_index, 16.6135826f, 1431,
                                                maneuver_index, 7.43128204f, 424));
  //----------------------------------------------------------------
  // trace point = 868
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4776154f, 40.1288109f, 1491515613, 32.3955154),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4776077f,
                                                40.1288185f, leg_index, 16.5935421f, 1431,
                                                maneuver_index, 7.41124153f, 424));
  //----------------------------------------------------------------
  // trace point = 869
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.477951f, 40.1289558f, 1491515614, 32.6019096),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4779434f,
                                                40.1289635f, leg_index, 16.5731525f, 1429,
                                                maneuver_index, 7.39085197f, 422));
  //----------------------------------------------------------------
  // trace point = 870
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4782944f, 40.1290932f, 1491515615, 32.9435196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4782867f,
                                                40.1291008f, leg_index, 16.5526524f, 1429,
                                                maneuver_index, 7.37035179f, 422));
  //----------------------------------------------------------------
  // trace point = 871
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4786301f, 40.1292267f, 1491515616, 32.3548164),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4786301f,
                                                40.1292305f, leg_index, 16.5323982f, 1427,
                                                maneuver_index, 7.35009766f, 420));
  //----------------------------------------------------------------
  // trace point = 872
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4789734f, 40.1293564f, 1491515617, 32.458416),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4789734f,
                                                40.1293564f, leg_index, 16.5122623f, 1426,
                                                maneuver_index, 7.32996178f, 419));
  //----------------------------------------------------------------
  // trace point = 873
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4793243f, 40.1294785f, 1491515618, 32.6159859),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4793243f,
                                                40.1294823f, leg_index, 16.4917603f, 1425,
                                                maneuver_index, 7.30945969f, 418));
  //----------------------------------------------------------------
  // trace point = 874
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4796677f, 40.1295853f, 1491515619, 31.8151093),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.47966f,
                                                40.1295929f, leg_index, 16.4724274f, 1423,
                                                maneuver_index, 7.2901268f, 416));
  //----------------------------------------------------------------
  // trace point = 875
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4800186f, 40.1296959f, 1491515620, 31.88764),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.480011f,
                                                40.1297073f, leg_index, 16.4522476f, 1423,
                                                maneuver_index, 7.26994705f, 416));
  //----------------------------------------------------------------
  // trace point = 876
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4803619f, 40.1297989f, 1491515621, 31.8809433),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4803543f,
                                                40.1298103f, leg_index, 16.432745f, 1422,
                                                maneuver_index, 7.25044441f, 415));
  //----------------------------------------------------------------
  // trace point = 877
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4807205f, 40.1299019f, 1491515622, 32.2782593),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4807205f,
                                                40.1299095f, leg_index, 16.4121971f, 1421,
                                                maneuver_index, 7.22989655f, 414));
  //----------------------------------------------------------------
  // trace point = 878
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4810715f, 40.1299973f, 1491515623, 31.9320946),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4810715f,
                                                40.1300049f, leg_index, 16.3924999f, 1419,
                                                maneuver_index, 7.21019936f, 412));
  //----------------------------------------------------------------
  // trace point = 879
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4814301f, 40.1300888f, 1491515624, 31.8680515),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4814224f,
                                                40.1301003f, leg_index, 16.3728027f, 1417,
                                                maneuver_index, 7.19050217f, 410));
  //----------------------------------------------------------------
  // trace point = 880
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4817886f, 40.1301765f, 1491515625, 32.006424),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4817886f,
                                                40.1301804f, leg_index, 16.3526573f, 1416,
                                                maneuver_index, 7.17035675f, 409));
  //----------------------------------------------------------------
  // trace point = 881
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4821472f, 40.1302567f, 1491515626, 31.6949444),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4821472f,
                                                40.1302567f, leg_index, 16.3329735f, 1415,
                                                maneuver_index, 7.15067291f, 408));
  //----------------------------------------------------------------
  // trace point = 882
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4825058f, 40.1303291f, 1491515627, 31.5934811),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4825058f,
                                                40.1303329f, leg_index, 16.3132858f, 1414,
                                                maneuver_index, 7.13098526f, 407));
  //----------------------------------------------------------------
  // trace point = 883
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4828644f, 40.1304016f, 1491515628, 31.9459476),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4828644f,
                                                40.1304054f, leg_index, 16.2936687f, 1413,
                                                maneuver_index, 7.11136818f, 406));
  //----------------------------------------------------------------
  // trace point = 884
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4832382f, 40.1304665f, 1491515629, 32.2874832),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4832382f,
                                                40.1304703f, leg_index, 16.2733974f, 1412,
                                                maneuver_index, 7.09109688f, 405));
  //----------------------------------------------------------------
  // trace point = 885
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4836044f, 40.1305237f, 1491515630, 32.0633049),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4836044f,
                                                40.1305351f, leg_index, 16.2535133f, 1411,
                                                maneuver_index, 7.07121277f, 404));
  //----------------------------------------------------------------
  // trace point = 886
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4839783f, 40.1305809f, 1491515631, 32.26968),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4839783f,
                                                40.1305885f, leg_index, 16.2333984f, 1410,
                                                maneuver_index, 7.05109787f, 403));
  //----------------------------------------------------------------
  // trace point = 887
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4843521f, 40.1306305f, 1491515632, 32.3988152),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4843521f,
                                                40.1306381f, leg_index, 16.2133312f, 1409,
                                                maneuver_index, 7.03103065f, 402));
  //----------------------------------------------------------------
  // trace point = 888
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.484726f, 40.1306725f, 1491515633, 31.9374123),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.484726f,
                                                40.1306839f, leg_index, 16.1933079f, 1407,
                                                maneuver_index, 7.01100731f, 400));
  //----------------------------------------------------------------
  // trace point = 889
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4850998f, 40.1307106f, 1491515634, 32.1118202),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4850998f,
                                                40.130722f, leg_index, 16.1733627f, 1406,
                                                maneuver_index, 6.99106216f, 399));
  //----------------------------------------------------------------
  // trace point = 890
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4854813f, 40.1307487f, 1491515635, 32.8014336),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4854813f,
                                                40.1307564f, leg_index, 16.1530457f, 1405,
                                                maneuver_index, 6.97074509f, 398));
  //----------------------------------------------------------------
  // trace point = 891
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4858627f, 40.1307793f, 1491515636, 32.5880241),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4858627f,
                                                40.1307793f, leg_index, 16.1328068f, 1403,
                                                maneuver_index, 6.95050621f, 396));
  //----------------------------------------------------------------
  // trace point = 892
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4862442f, 40.1308022f, 1491515637, 32.5566254),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4862442f,
                                                40.1308022f, leg_index, 16.1125698f, 1402,
                                                maneuver_index, 6.93026924f, 395));
  //----------------------------------------------------------------
  // trace point = 893
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4866257f, 40.1308212f, 1491515638, 32.7315369),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4866257f,
                                                40.1308174f, leg_index, 16.0923672f, 1401,
                                                maneuver_index, 6.9100666f, 394));
  //----------------------------------------------------------------
  // trace point = 894
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4870148f, 40.1308289f, 1491515639, 32.8721123),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4870148f,
                                                40.1308327f, leg_index, 16.0717583f, 1400,
                                                maneuver_index, 6.8894577f, 393));
  //----------------------------------------------------------------
  // trace point = 895
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4873962f, 40.1308365f, 1491515640, 32.7749519),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4873962f,
                                                40.1308403f, leg_index, 16.0515766f, 1399,
                                                maneuver_index, 6.86927605f, 392));
  //----------------------------------------------------------------
  // trace point = 896
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4877777f, 40.1308365f, 1491515641, 32.6837883),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4877777f,
                                                40.1308327f, leg_index, 16.0313969f, 1398,
                                                maneuver_index, 6.8490963f, 391));
  //----------------------------------------------------------------
  // trace point = 897
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4881668f, 40.1308327f, 1491515642, 33.1132698),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4881668f,
                                                40.1308365f, leg_index, 16.0108185f, 1397,
                                                maneuver_index, 6.82851791f, 390));
  //----------------------------------------------------------------
  // trace point = 898
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4885483f, 40.130825f, 1491515643, 32.6133156),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4885483f,
                                                40.1308289f, leg_index, 15.990634f, 1396,
                                                maneuver_index, 6.8083334f, 389));
  //----------------------------------------------------------------
  // trace point = 899
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4889374f, 40.1308136f, 1491515644, 32.9611282),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4889374f,
                                                40.1308098f, leg_index, 15.9700069f, 1395,
                                                maneuver_index, 6.78770638f, 388));
  //----------------------------------------------------------------
  // trace point = 900
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4893265f, 40.1307983f, 1491515645, 32.8051987),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4893265f,
                                                40.1307907f, leg_index, 15.9493885f, 1394,
                                                maneuver_index, 6.76708794f, 387));
  //----------------------------------------------------------------
  // trace point = 901
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4897003f, 40.1307716f, 1491515646, 32.4718513),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4897003f,
                                                40.1307602f, leg_index, 15.9295063f, 1392,
                                                maneuver_index, 6.74720573f, 385));
  //----------------------------------------------------------------
  // trace point = 902
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4900818f, 40.1307411f, 1491515647, 32.2968826),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4900818f,
                                                40.1307335f, leg_index, 15.9092474f, 1391,
                                                maneuver_index, 6.72694683f, 384));
  //----------------------------------------------------------------
  // trace point = 903
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4904556f, 40.1306992f, 1491515648, 32.4261398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4904556f,
                                                40.1306953f, leg_index, 15.8892975f, 1390,
                                                maneuver_index, 6.70699692f, 383));
  //----------------------------------------------------------------
  // trace point = 904
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4908371f, 40.1306572f, 1491515649, 32.3260307),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4908371f,
                                                40.1306534f, leg_index, 15.8689146f, 1388,
                                                maneuver_index, 6.68661404f, 381));
  //----------------------------------------------------------------
  // trace point = 905
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4912033f, 40.1306114f, 1491515650, 32.0731964),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4912033f,
                                                40.1306076f, leg_index, 15.8492899f, 1387,
                                                maneuver_index, 6.66698933f, 380));
  //----------------------------------------------------------------
  // trace point = 906
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4915771f, 40.1305656f, 1491515651, 32.0911789),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4915771f,
                                                40.130558f, leg_index, 15.8292246f, 1386,
                                                maneuver_index, 6.64692402f, 379));
  //----------------------------------------------------------------
  // trace point = 907
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4919434f, 40.1305199f, 1491515652, 31.6850319),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4919434f,
                                                40.1305122f, leg_index, 15.8095999f, 1385,
                                                maneuver_index, 6.62729931f, 378));
  //----------------------------------------------------------------
  // trace point = 908
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4923172f, 40.1304741f, 1491515653, 31.8578072),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4923172f,
                                                40.1304665f, leg_index, 15.7895765f, 1384,
                                                maneuver_index, 6.60727596f, 377));
  //----------------------------------------------------------------
  // trace point = 909
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4926834f, 40.1304245f, 1491515654, 31.5413132),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4926834f,
                                                40.1304207f, leg_index, 15.7699518f, 1383,
                                                maneuver_index, 6.58765125f, 376));
  //----------------------------------------------------------------
  // trace point = 910
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.493042f, 40.1303787f, 1491515655, 31.4196682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.493042f,
                                                40.1303749f, leg_index, 15.7507257f, 1382,
                                                maneuver_index, 6.56842518f, 375));
  //----------------------------------------------------------------
  // trace point = 911
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4934158f, 40.1303291f, 1491515656, 31.9605942),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4934158f,
                                                40.1303291f, leg_index, 15.7307024f, 1381,
                                                maneuver_index, 6.54840183f, 374));
  //----------------------------------------------------------------
  // trace point = 912
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4937897f, 40.1302834f, 1491515657, 32.1226692),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4937897f,
                                                40.1302795f, leg_index, 15.7106342f, 1380,
                                                maneuver_index, 6.52833366f, 373));
  //----------------------------------------------------------------
  // trace point = 913
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4941635f, 40.1302376f, 1491515658, 32.2414207),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4941635f,
                                                40.1302299f, leg_index, 15.690568f, 1379,
                                                maneuver_index, 6.5082674f, 372));
  //----------------------------------------------------------------
  // trace point = 914
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4945374f, 40.130188f, 1491515659, 32.2961655),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4945374f,
                                                40.1301842f, leg_index, 15.6705446f, 1378,
                                                maneuver_index, 6.48824406f, 371));
  //----------------------------------------------------------------
  // trace point = 915
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4949112f, 40.1301422f, 1491515660, 32.4273148),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4949112f,
                                                40.1301384f, leg_index, 15.6505213f, 1377,
                                                maneuver_index, 6.46822071f, 370));
  //----------------------------------------------------------------
  // trace point = 916
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.495285f, 40.1300926f, 1491515661, 32.5479088),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.495285f,
                                                40.1300926f, leg_index, 15.6304979f, 1376,
                                                maneuver_index, 6.44819736f, 369));
  //----------------------------------------------------------------
  // trace point = 917
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4956589f, 40.1300468f, 1491515662, 32.1399384),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4956589f,
                                                40.130043f, leg_index, 15.6104317f, 1375,
                                                maneuver_index, 6.4281311f, 368));
  //----------------------------------------------------------------
  // trace point = 918
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4960327f, 40.1300011f, 1491515663, 32.175415),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4960327f,
                                                40.1299973f, leg_index, 15.5904093f, 1374,
                                                maneuver_index, 6.40810871f, 367));
  //----------------------------------------------------------------
  // trace point = 919
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4964066f, 40.1299553f, 1491515664, 32.1575203),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4964066f,
                                                40.1299515f, leg_index, 15.570384f, 1372,
                                                maneuver_index, 6.38808346f, 365));
  //----------------------------------------------------------------
  // trace point = 920
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.496788f, 40.1299095f, 1491515665, 32.6276398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.496788f,
                                                40.1299057f, leg_index, 15.5491238f, 1371,
                                                maneuver_index, 6.3668232f, 364));
  //----------------------------------------------------------------
  // trace point = 921
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4971619f, 40.1298599f, 1491515666, 32.5668564),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4971619f,
                                                40.1298561f, leg_index, 15.5298977f, 1370,
                                                maneuver_index, 6.34759712f, 363));
  //----------------------------------------------------------------
  // trace point = 922
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4975433f, 40.1298141f, 1491515667, 32.5295677),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4975433f,
                                                40.1298027f, leg_index, 15.509388f, 1368,
                                                maneuver_index, 6.3270874f, 361));
  //----------------------------------------------------------------
  // trace point = 923
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4979172f, 40.1297646f, 1491515668, 32.3434944),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4979172f,
                                                40.1297531f, leg_index, 15.4893217f, 1367,
                                                maneuver_index, 6.30702114f, 360));
  //----------------------------------------------------------------
  // trace point = 924
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.498291f, 40.1297188f, 1491515669, 32.59552),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.498291f,
                                                40.1297073f, leg_index, 15.4692984f, 1366,
                                                maneuver_index, 6.2869978f, 359));
  //----------------------------------------------------------------
  // trace point = 925
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4986725f, 40.129673f, 1491515670, 32.5115356),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4986725f,
                                                40.1296616f, leg_index, 15.4488773f, 1365,
                                                maneuver_index, 6.26657677f, 358));
  //----------------------------------------------------------------
  // trace point = 926
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4990463f, 40.1296234f, 1491515671, 32.3617554),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4990463f,
                                                40.1296158f, leg_index, 15.428854f, 1363,
                                                maneuver_index, 6.24655342f, 356));
  //----------------------------------------------------------------
  // trace point = 927
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4994278f, 40.1295776f, 1491515672, 32.7815781),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4994278f,
                                                40.12957f, leg_index, 15.408432f, 1362,
                                                maneuver_index, 6.22613144f, 355));
  //----------------------------------------------------------------
  // trace point = 928
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.4998016f, 40.129528f, 1491515673, 32.7999306),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.4998016f,
                                                40.1295242f, leg_index, 15.3884087f, 1362,
                                                maneuver_index, 6.20610809f, 355));
  //----------------------------------------------------------------
  // trace point = 929
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5001831f, 40.1294785f, 1491515674, 32.5297241),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5001831f,
                                                40.1294746f, leg_index, 15.3679447f, 1360,
                                                maneuver_index, 6.18564415f, 353));
  //----------------------------------------------------------------
  // trace point = 930
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5005569f, 40.1294289f, 1491515675, 32.5482178),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5005569f,
                                                40.1294289f, leg_index, 15.3479214f, 1359,
                                                maneuver_index, 6.1656208f, 352));
  //----------------------------------------------------------------
  // trace point = 931
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5009308f, 40.1293793f, 1491515676, 32.3804321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5009308f,
                                                40.1293793f, leg_index, 15.3278542f, 1358,
                                                maneuver_index, 6.14555359f, 351));
  //----------------------------------------------------------------
  // trace point = 932
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5013046f, 40.1293335f, 1491515677, 32.2965622),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5013046f,
                                                40.1293335f, leg_index, 15.3078308f, 1357,
                                                maneuver_index, 6.12553024f, 350));
  //----------------------------------------------------------------
  // trace point = 933
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5016861f, 40.1292877f, 1491515678, 32.3258972),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5016861f,
                                                40.1292839f, leg_index, 15.2873669f, 1355,
                                                maneuver_index, 6.1050663f, 348));
  //----------------------------------------------------------------
  // trace point = 934
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5020599f, 40.1292381f, 1491515679, 32.6621208),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5020599f,
                                                40.1292343f, leg_index, 15.267293f, 1354,
                                                maneuver_index, 6.08499241f, 347));
  //----------------------------------------------------------------
  // trace point = 935
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5024414f, 40.1291924f, 1491515680, 32.5298576),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5024414f,
                                                40.1291847f, leg_index, 15.246829f, 1353,
                                                maneuver_index, 6.06452847f, 346));
  //----------------------------------------------------------------
  // trace point = 936
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5028152f, 40.1291428f, 1491515681, 32.5298805),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5028152f,
                                                40.1291389f, leg_index, 15.2268057f, 1352,
                                                maneuver_index, 6.04450512f, 345));
  //----------------------------------------------------------------
  // trace point = 937
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5031891f, 40.1291008f, 1491515682, 32.2914619),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5031891f,
                                                40.1290932f, leg_index, 15.2067823f, 1351,
                                                maneuver_index, 6.02448177f, 344));
  //----------------------------------------------------------------
  // trace point = 938
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5035706f, 40.129055f, 1491515683, 32.392643),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5035706f,
                                                40.1290436f, leg_index, 15.1863184f, 1350,
                                                maneuver_index, 6.00401783f, 343));
  //----------------------------------------------------------------
  // trace point = 939
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5039444f, 40.1290054f, 1491515684, 32.3350601),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5039444f,
                                                40.1289978f, leg_index, 15.1662951f, 1348,
                                                maneuver_index, 5.98399448f, 341));
  //----------------------------------------------------------------
  // trace point = 940
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5043182f, 40.1289558f, 1491515685, 32.6978951),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5043182f,
                                                40.128952f, leg_index, 15.1462717f, 1347,
                                                maneuver_index, 5.96397114f, 340));
  //----------------------------------------------------------------
  // trace point = 941
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5046997f, 40.1289062f, 1491515686, 32.5673027),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5046997f,
                                                40.1289024f, leg_index, 15.1258068f, 1346,
                                                maneuver_index, 5.94350624f, 339));
  //----------------------------------------------------------------
  // trace point = 942
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5050735f, 40.1288567f, 1491515687, 32.71632),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5050735f,
                                                40.1288567f, leg_index, 15.1057835f, 1345,
                                                maneuver_index, 5.92348289f, 338));
  //----------------------------------------------------------------
  // trace point = 943
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5054474f, 40.1288071f, 1491515688, 32.3157921),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5054474f,
                                                40.1288071f, leg_index, 15.0857172f, 1344,
                                                maneuver_index, 5.90341663f, 337));
  //----------------------------------------------------------------
  // trace point = 944
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5058289f, 40.1287575f, 1491515689, 32.8842239),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5058289f,
                                                40.1287613f, leg_index, 15.0652962f, 1343,
                                                maneuver_index, 5.88299561f, 336));
  //----------------------------------------------------------------
  // trace point = 945
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5062103f, 40.1287079f, 1491515690, 32.9681854),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5062103f,
                                                40.1287117f, leg_index, 15.0448313f, 1341,
                                                maneuver_index, 5.86253071f, 334));
  //----------------------------------------------------------------
  // trace point = 946
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5065918f, 40.1286659f, 1491515691, 32.5440788),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5065918f,
                                                40.1286659f, leg_index, 15.0244102f, 1340,
                                                maneuver_index, 5.84210968f, 333));
  //----------------------------------------------------------------
  // trace point = 947
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5069656f, 40.1286163f, 1491515692, 32.5674362),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5069656f,
                                                40.1286163f, leg_index, 15.004344f, 1339,
                                                maneuver_index, 5.82204342f, 332));
  //----------------------------------------------------------------
  // trace point = 948
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5073471f, 40.1285667f, 1491515693, 32.6800728),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5073471f,
                                                40.1285706f, leg_index, 14.983922f, 1338,
                                                maneuver_index, 5.80162144f, 331));
  //----------------------------------------------------------------
  // trace point = 949
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5077209f, 40.128521f, 1491515694, 32.5120697),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5077209f,
                                                40.128521f, leg_index, 14.9638557f, 1337,
                                                maneuver_index, 5.78155518f, 330));
  //----------------------------------------------------------------
  // trace point = 950
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5081024f, 40.1284714f, 1491515695, 32.6981239),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5081024f,
                                                40.1284752f, leg_index, 14.9434338f, 1336,
                                                maneuver_index, 5.76113319f, 329));
  //----------------------------------------------------------------
  // trace point = 951
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5084763f, 40.1284256f, 1491515696, 32.4647789),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5084763f,
                                                40.1284256f, leg_index, 14.9233675f, 1335,
                                                maneuver_index, 5.74106693f, 328));
  //----------------------------------------------------------------
  // trace point = 952
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5088577f, 40.1283722f, 1491515697, 32.8382301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5088577f,
                                                40.1283798f, leg_index, 14.9029465f, 1333,
                                                maneuver_index, 5.7206459f, 326));
  //----------------------------------------------------------------
  // trace point = 953
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5092545f, 40.1283455f, 1491515698, 33.5104179),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5092545f,
                                                40.1283302f, leg_index, 14.8816853f, 1332,
                                                maneuver_index, 5.69938469f, 325));
  //----------------------------------------------------------------
  // trace point = 954
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5096283f, 40.1282921f, 1491515699, 32.542614),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5096283f,
                                                40.1282806f, leg_index, 14.861619f, 1331,
                                                maneuver_index, 5.67931843f, 324));
  //----------------------------------------------------------------
  // trace point = 955
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5100021f, 40.1282387f, 1491515700, 32.2915573),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5100021f,
                                                40.1282349f, leg_index, 14.8415956f, 1330,
                                                maneuver_index, 5.65929508f, 323));
  //----------------------------------------------------------------
  // trace point = 956
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.510376f, 40.1281891f, 1491515701, 32.2516441),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.510376f,
                                                40.1281853f, leg_index, 14.8215294f, 1329,
                                                maneuver_index, 5.63922882f, 322));
  //----------------------------------------------------------------
  // trace point = 957
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5107498f, 40.1281395f, 1491515702, 32.7166557),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5107498f,
                                                40.1281395f, leg_index, 14.8015051f, 1328,
                                                maneuver_index, 5.61920452f, 321));
  //----------------------------------------------------------------
  // trace point = 958
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5111313f, 40.1280937f, 1491515703, 32.5612869),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5111313f,
                                                40.1280937f, leg_index, 14.7810831f, 1327,
                                                maneuver_index, 5.59878254f, 320));
  //----------------------------------------------------------------
  // trace point = 959
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5115051f, 40.1280479f, 1491515704, 32.3264694),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5115051f,
                                                40.1280556f, leg_index, 14.7611361f, 1325,
                                                maneuver_index, 5.57883549f, 318));
  //----------------------------------------------------------------
  // trace point = 960
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.511879f, 40.1280136f, 1491515705, 32.2397499),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.511879f,
                                                40.1280212f, leg_index, 14.7412224f, 1324,
                                                maneuver_index, 5.55892181f, 317));
  //----------------------------------------------------------------
  // trace point = 961
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5122604f, 40.1279869f, 1491515706, 32.4634399),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5122604f,
                                                40.1279945f, leg_index, 14.7209597f, 1323,
                                                maneuver_index, 5.5386591f, 316));
  //----------------------------------------------------------------
  // trace point = 962
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5126419f, 40.1279678f, 1491515707, 32.7527695),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5126419f,
                                                40.1279716f, leg_index, 14.7007179f, 1321,
                                                maneuver_index, 5.51841736f, 314));
  //----------------------------------------------------------------
  // trace point = 963
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5130234f, 40.1279526f, 1491515708, 32.5570297),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5130234f,
                                                40.1279526f, leg_index, 14.6805019f, 1320,
                                                maneuver_index, 5.49820137f, 313));
  //----------------------------------------------------------------
  // trace point = 964
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5134125f, 40.1279373f, 1491515709, 32.5629349),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5134125f,
                                                40.1279411f, leg_index, 14.6599064f, 1319,
                                                maneuver_index, 5.47760582f, 312));
  //----------------------------------------------------------------
  // trace point = 965
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5137863f, 40.1279335f, 1491515710, 32.3487511),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5137863f,
                                                40.1279373f, leg_index, 14.6401329f, 1318,
                                                maneuver_index, 5.45783234f, 311));
  //----------------------------------------------------------------
  // trace point = 966
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5141754f, 40.1279335f, 1491515711, 32.5993118),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5141754f,
                                                40.1279335f, leg_index, 14.6195507f, 1316,
                                                maneuver_index, 5.43725014f, 309));
  //----------------------------------------------------------------
  // trace point = 967
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5145493f, 40.1279373f, 1491515712, 32.4338531),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5145493f,
                                                40.1279335f, leg_index, 14.599782f, 1315,
                                                maneuver_index, 5.41748142f, 308));
  //----------------------------------------------------------------
  // trace point = 968
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5149307f, 40.1279488f, 1491515713, 32.3631096),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5149307f,
                                                40.1279526f, leg_index, 14.5791607f, 1314,
                                                maneuver_index, 5.39686012f, 307));
  //----------------------------------------------------------------
  // trace point = 969
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5153122f, 40.127964f, 1491515714, 32.4720306),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5153122f,
                                                40.1279716f, leg_index, 14.5593443f, 1313,
                                                maneuver_index, 5.37704372f, 306));
  //----------------------------------------------------------------
  // trace point = 970
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5156937f, 40.1279831f, 1491515715, 32.1819191),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5156937f,
                                                40.1279907f, leg_index, 14.5391264f, 1312,
                                                maneuver_index, 5.35682583f, 305));
  //----------------------------------------------------------------
  // trace point = 971
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5160751f, 40.1280136f, 1491515716, 32.5784988),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5160751f,
                                                40.1280136f, leg_index, 14.518878f, 1311,
                                                maneuver_index, 5.33657742f, 304));
  //----------------------------------------------------------------
  // trace point = 972
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.516449f, 40.1280479f, 1491515717, 32.3110123),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.516449f,
                                                40.1280479f, leg_index, 14.4989624f, 1309,
                                                maneuver_index, 5.31666183f, 302));
  //----------------------------------------------------------------
  // trace point = 973
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5168228f, 40.1280823f, 1491515718, 32.3518143),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5168228f,
                                                40.1280823f, leg_index, 14.4790497f, 1308,
                                                maneuver_index, 5.29674911f, 301));
  //----------------------------------------------------------------
  // trace point = 974
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5172043f, 40.1281242f, 1491515719, 32.7643013),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5172043f,
                                                40.1281281f, leg_index, 14.4586267f, 1307,
                                                maneuver_index, 5.27632618f, 300));
  //----------------------------------------------------------------
  // trace point = 975
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5175858f, 40.1281776f, 1491515720, 32.735405),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5175858f,
                                                40.1281738f, leg_index, 14.4382067f, 1306,
                                                maneuver_index, 5.25590611f, 299));
  //----------------------------------------------------------------
  // trace point = 976
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5179596f, 40.128231f, 1491515721, 32.5630341),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5179596f,
                                                40.128231f, leg_index, 14.4180336f, 1305,
                                                maneuver_index, 5.23573303f, 298));
  //----------------------------------------------------------------
  // trace point = 977
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5183334f, 40.1282883f, 1491515722, 32.3596916),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5183334f,
                                                40.1282921f, leg_index, 14.3978138f, 1304,
                                                maneuver_index, 5.21551323f, 297));
  //----------------------------------------------------------------
  // trace point = 978
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5187073f, 40.1283531f, 1491515723, 32.5620956),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5187073f,
                                                40.1283531f, leg_index, 14.3775978f, 1303,
                                                maneuver_index, 5.19529724f, 296));
  //----------------------------------------------------------------
  // trace point = 979
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5190811f, 40.1284218f, 1491515724, 32.5294075),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5190811f,
                                                40.1284218f, leg_index, 14.3572626f, 1301,
                                                maneuver_index, 5.17496204f, 294));
  //----------------------------------------------------------------
  // trace point = 980
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5194473f, 40.1284943f, 1491515725, 32.3593216),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5194473f,
                                                40.1284943f, leg_index, 14.3372564f, 1300,
                                                maneuver_index, 5.15495586f, 293));
  //----------------------------------------------------------------
  // trace point = 981
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5198135f, 40.1285667f, 1491515726, 32.4416847),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5198135f,
                                                40.1285706f, leg_index, 14.3171825f, 1299,
                                                maneuver_index, 5.13488197f, 292));
  //----------------------------------------------------------------
  // trace point = 982
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5201797f, 40.1286392f, 1491515727, 32.2490311),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5201797f,
                                                40.128643f, leg_index, 14.2971764f, 1298,
                                                maneuver_index, 5.11487579f, 291));
  //----------------------------------------------------------------
  // trace point = 983
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5205536f, 40.1287155f, 1491515728, 32.3873596),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5205536f,
                                                40.1287193f, leg_index, 14.2767124f, 1297,
                                                maneuver_index, 5.09441185f, 290));
  //----------------------------------------------------------------
  // trace point = 984
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5209198f, 40.1287842f, 1491515729, 32.3865356),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5209198f,
                                                40.1287918f, leg_index, 14.2567062f, 1296,
                                                maneuver_index, 5.07440567f, 289));
  //----------------------------------------------------------------
  // trace point = 985
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5212936f, 40.1288605f, 1491515730, 32.6625824),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5212936f,
                                                40.1288681f, leg_index, 14.2362423f, 1295,
                                                maneuver_index, 5.05394173f, 288));
  //----------------------------------------------------------------
  // trace point = 986
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5216599f, 40.1289368f, 1491515731, 32.6625481),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5216599f,
                                                40.1289406f, leg_index, 14.2162361f, 1293,
                                                maneuver_index, 5.03393555f, 286));
  //----------------------------------------------------------------
  // trace point = 987
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5220337f, 40.1290092f, 1491515732, 32.4414864),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5220337f,
                                                40.1290131f, leg_index, 14.1958399f, 1292,
                                                maneuver_index, 5.01353931f, 285));
  //----------------------------------------------------------------
  // trace point = 988
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5223999f, 40.1290817f, 1491515733, 32.4979362),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5223999f,
                                                40.1290894f, leg_index, 14.175766f, 1291,
                                                maneuver_index, 4.99346542f, 284));
  //----------------------------------------------------------------
  // trace point = 989
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5227661f, 40.129158f, 1491515734, 32.3623428),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5227661f,
                                                40.1291618f, leg_index, 14.1557598f, 1290,
                                                maneuver_index, 4.97345924f, 283));
  //----------------------------------------------------------------
  // trace point = 990
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5231323f, 40.1292343f, 1491515735, 32.3623085),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5231323f,
                                                40.1292343f, leg_index, 14.1357536f, 1289,
                                                maneuver_index, 4.95345306f, 282));
  //----------------------------------------------------------------
  // trace point = 991
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5235062f, 40.1293106f, 1491515736, 32.7164688),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5235062f,
                                                40.1293106f, leg_index, 14.1152906f, 1288,
                                                maneuver_index, 4.93299007f, 281));
  //----------------------------------------------------------------
  // trace point = 992
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.52388f, 40.1293831f, 1491515737, 32.6061478),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.52388f,
                                                40.1293869f, leg_index, 14.0948267f, 1286,
                                                maneuver_index, 4.91252613f, 279));
  //----------------------------------------------------------------
  // trace point = 993
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5242462f, 40.1294556f, 1491515738, 32.3870277),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5242462f,
                                                40.1294594f, leg_index, 14.0748205f, 1285,
                                                maneuver_index, 4.89251995f, 278));
  //----------------------------------------------------------------
  // trace point = 994
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5246124f, 40.129528f, 1491515739, 32.2764816),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5246124f,
                                                40.1295319f, leg_index, 14.0548143f, 1284,
                                                maneuver_index, 4.87251377f, 277));
  //----------------------------------------------------------------
  // trace point = 995
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5249786f, 40.1296005f, 1491515740, 32.4412193),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5249786f,
                                                40.1296043f, leg_index, 14.0348082f, 1283,
                                                maneuver_index, 4.85250759f, 276));
  //----------------------------------------------------------------
  // trace point = 996
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5253525f, 40.129673f, 1491515741, 32.4134827),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5253525f,
                                                40.1296806f, leg_index, 14.0143452f, 1282,
                                                maneuver_index, 4.8320446f, 275));
  //----------------------------------------------------------------
  // trace point = 997
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5257187f, 40.1297493f, 1491515742, 32.5515556),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5257187f,
                                                40.1297531f, leg_index, 13.994339f, 1281,
                                                maneuver_index, 4.81203842f, 274));
  //----------------------------------------------------------------
  // trace point = 998
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5260925f, 40.1298256f, 1491515743, 32.6907806),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5260925f,
                                                40.1298294f, leg_index, 13.973875f, 1280,
                                                maneuver_index, 4.79157448f, 273));
  //----------------------------------------------------------------
  // trace point = 999
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5264587f, 40.1299019f, 1491515744, 32.6907463),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5264587f,
                                                40.1299019f, leg_index, 13.9538689f, 1278,
                                                maneuver_index, 4.7715683f, 271));
  //----------------------------------------------------------------
  // trace point = 1000
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5268326f, 40.1299744f, 1491515745, 32.690712),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5268326f,
                                                40.1299744f, leg_index, 13.9334726f, 1277,
                                                maneuver_index, 4.75117207f, 270));
  //----------------------------------------------------------------
  // trace point = 1001
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5272064f, 40.1300545f, 1491515746, 32.8018112),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5272064f,
                                                40.1300507f, leg_index, 13.9130096f, 1276,
                                                maneuver_index, 4.73070908f, 269));
  //----------------------------------------------------------------
  // trace point = 1002
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5275726f, 40.1301308f, 1491515747, 32.532753),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5275726f,
                                                40.1301231f, leg_index, 13.8930044f, 1275,
                                                maneuver_index, 4.71070385f, 268));
  //----------------------------------------------------------------
  // trace point = 1003
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5279388f, 40.1302147f, 1491515748, 32.4611855),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5279465f,
                                                40.1301994f, leg_index, 13.8725414f, 1274,
                                                maneuver_index, 4.69024086f, 267));
  //----------------------------------------------------------------
  // trace point = 1004
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5283051f, 40.1302948f, 1491515749, 32.3482742),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5283127f,
                                                40.1302757f, leg_index, 13.8524675f, 1273,
                                                maneuver_index, 4.67016697f, 266));
  //----------------------------------------------------------------
  // trace point = 1005
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5286789f, 40.1303711f, 1491515750, 32.8309326),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5286865f,
                                                40.130352f, leg_index, 13.8320045f, 1272,
                                                maneuver_index, 4.64970398f, 265));
  //----------------------------------------------------------------
  // trace point = 1006
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5290451f, 40.1304474f, 1491515751, 32.3617706),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5290527f,
                                                40.1304283f, leg_index, 13.8119307f, 1270,
                                                maneuver_index, 4.62963009f, 263));
  //----------------------------------------------------------------
  // trace point = 1007
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5294113f, 40.1305237f, 1491515752, 32.7159195),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5294189f,
                                                40.1305046f, leg_index, 13.7918568f, 1269,
                                                maneuver_index, 4.6095562f, 262));
  //----------------------------------------------------------------
  // trace point = 1008
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5297852f, 40.1305962f, 1491515753, 32.497261),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5297928f,
                                                40.1305771f, leg_index, 13.7714596f, 1268,
                                                maneuver_index, 4.58915901f, 261));
  //----------------------------------------------------------------
  // trace point = 1009
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5301514f, 40.1306725f, 1491515754, 32.5260048),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.530159f,
                                                40.1306496f, leg_index, 13.7514534f, 1267,
                                                maneuver_index, 4.56915283f, 260));
  //----------------------------------------------------------------
  // trace point = 1010
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5305252f, 40.1307487f, 1491515755, 32.6879654),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5305328f,
                                                40.130722f, leg_index, 13.7310572f, 1266,
                                                maneuver_index, 4.5487566f, 259));
  //----------------------------------------------------------------
  // trace point = 1011
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5308914f, 40.1308212f, 1491515756, 32.5230827),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.530899f,
                                                40.1307945f, leg_index, 13.711051f, 1265,
                                                maneuver_index, 4.52875042f, 258));
  //----------------------------------------------------------------
  // trace point = 1012
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5312653f, 40.1308937f, 1491515757, 32.4971275),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5312729f,
                                                40.1308708f, leg_index, 13.690588f, 1264,
                                                maneuver_index, 4.50828743f, 257));
  //----------------------------------------------------------------
  // trace point = 1013
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5316315f, 40.13097f, 1491515758, 32.5510063),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5316391f,
                                                40.1309433f, leg_index, 13.6705828f, 1263,
                                                maneuver_index, 4.4882822f, 256));
  //----------------------------------------------------------------
  // trace point = 1014
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5320053f, 40.1310463f, 1491515759, 32.6370888),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5320129f,
                                                40.1310158f, leg_index, 13.6501865f, 1261,
                                                maneuver_index, 4.46788597f, 254));
  //----------------------------------------------------------------
  // trace point = 1015
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5323715f, 40.1311226f, 1491515760, 32.3325462),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5323792f,
                                                40.1310883f, leg_index, 13.6301804f, 1260,
                                                maneuver_index, 4.44787979f, 253));
  //----------------------------------------------------------------
  // trace point = 1016
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5327377f, 40.1311913f, 1491515761, 32.3303413),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5327454f,
                                                40.1311607f, leg_index, 13.6101742f, 1259,
                                                maneuver_index, 4.42787361f, 252));
  //----------------------------------------------------------------
  // trace point = 1017
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5331116f, 40.1312675f, 1491515762, 32.5792236),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5331192f,
                                                40.131237f, leg_index, 13.5893211f, 1258,
                                                maneuver_index, 4.40702057f, 251));
  //----------------------------------------------------------------
  // trace point = 1018
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5334778f, 40.1313438f, 1491515763, 32.6614647),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.533493f,
                                                40.1313095f, leg_index, 13.5693159f, 1257,
                                                maneuver_index, 4.38701534f, 250));
  //----------------------------------------------------------------
  // trace point = 1019
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5338516f, 40.1314163f, 1491515764, 32.6052322),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5338593f,
                                                40.1313858f, leg_index, 13.549243f, 1256,
                                                maneuver_index, 4.36694241f, 249));
  //----------------------------------------------------------------
  // trace point = 1020
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5342178f, 40.1314888f, 1491515765, 32.4403763),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5342255f,
                                                40.1314621f, leg_index, 13.52917f, 1255,
                                                maneuver_index, 4.34686947f, 248));
  //----------------------------------------------------------------
  // trace point = 1021
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5345917f, 40.1315613f, 1491515766, 32.633091),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5345993f,
                                                40.1315384f, leg_index, 13.508707f, 1254,
                                                maneuver_index, 4.32640648f, 247));
  //----------------------------------------------------------------
  // trace point = 1022
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5349579f, 40.1316414f, 1491515767, 32.390522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5349655f,
                                                40.1316109f, leg_index, 13.4887009f, 1253,
                                                maneuver_index, 4.3064003f, 246));
  //----------------------------------------------------------------
  // trace point = 1023
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5353317f, 40.1317139f, 1491515768, 32.79776),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5353394f,
                                                40.1316872f, leg_index, 13.4682379f, 1252,
                                                maneuver_index, 4.28593731f, 245));
  //----------------------------------------------------------------
  // trace point = 1024
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5356979f, 40.1317863f, 1491515769, 32.5506401),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5357056f,
                                                40.1317596f, leg_index, 13.4482327f, 1251,
                                                maneuver_index, 4.26593208f, 244));
  //----------------------------------------------------------------
  // trace point = 1025
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5360718f, 40.1318626f, 1491515770, 32.661232),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5360794f,
                                                40.1318321f, leg_index, 13.4278364f, 1250,
                                                maneuver_index, 4.24553585f, 243));
  //----------------------------------------------------------------
  // trace point = 1026
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.536438f, 40.1319427f, 1491515771, 32.5840607),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5364532f,
                                                40.1319122f, leg_index, 13.4073029f, 1248,
                                                maneuver_index, 4.22500229f, 241));
  //----------------------------------------------------------------
  // trace point = 1027
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5368042f, 40.1320267f, 1491515772, 32.279232),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5368195f,
                                                40.1319923f, leg_index, 13.3871593f, 1247,
                                                maneuver_index, 4.20485878f, 240));
  //----------------------------------------------------------------
  // trace point = 1028
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5371704f, 40.1321106f, 1491515773, 32.6372871),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5371857f,
                                                40.1320801f, leg_index, 13.3668661f, 1246,
                                                maneuver_index, 4.18456554f, 239));
  //----------------------------------------------------------------
  // trace point = 1029
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5375366f, 40.1322021f, 1491515774, 32.6190376),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5375519f,
                                                40.1321754f, leg_index, 13.3464088f, 1244,
                                                maneuver_index, 4.16410828f, 237));
  //----------------------------------------------------------------
  // trace point = 1030
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5378952f, 40.1323013f, 1491515775, 32.5670204),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5379105f,
                                                40.1322746f, leg_index, 13.3262453f, 1243,
                                                maneuver_index, 4.14394474f, 236));
  //----------------------------------------------------------------
  // trace point = 1031
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5382538f, 40.1324005f, 1491515776, 32.7592583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5382614f,
                                                40.1323776f, leg_index, 13.3055153f, 1242,
                                                maneuver_index, 4.12321472f, 235));
  //----------------------------------------------------------------
  // trace point = 1032
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5386124f, 40.1325111f, 1491515777, 32.7589378),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.53862f,
                                                40.1324883f, leg_index, 13.2859135f, 1241,
                                                maneuver_index, 4.1036129f, 234));
  //----------------------------------------------------------------
  // trace point = 1033
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5389633f, 40.1326294f, 1491515778, 32.5113335),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5389786f,
                                                40.1326027f, leg_index, 13.2653637f, 1239,
                                                maneuver_index, 4.08306313f, 232));
  //----------------------------------------------------------------
  // trace point = 1034
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5393066f, 40.1327477f, 1491515779, 32.336174),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5393219f,
                                                40.1327209f, leg_index, 13.2454519f, 1238,
                                                maneuver_index, 4.06315136f, 231));
  //----------------------------------------------------------------
  // trace point = 1035
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5396576f, 40.1328697f, 1491515780, 32.8618774),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5396729f,
                                                40.1328506f, leg_index, 13.2248316f, 1237,
                                                maneuver_index, 4.04253101f, 230));
  //----------------------------------------------------------------
  // trace point = 1036
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5400085f, 40.1330032f, 1491515781, 32.9371529),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5400238f,
                                                40.1329803f, leg_index, 13.2042179f, 1236,
                                                maneuver_index, 4.02191734f, 229));
  //----------------------------------------------------------------
  // trace point = 1037
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5403519f, 40.1331367f, 1491515782, 32.682354),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5403671f,
                                                40.1331177f, leg_index, 13.1837282f, 1235,
                                                maneuver_index, 4.00142765f, 228));
  //----------------------------------------------------------------
  // trace point = 1038
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5406876f, 40.1332741f, 1491515783, 32.5828133),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5407028f,
                                                40.133255f, leg_index, 13.1635914f, 1234,
                                                maneuver_index, 3.98129082f, 227));
  //----------------------------------------------------------------
  // trace point = 1039
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5410233f, 40.133419f, 1491515784, 32.8184929),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5410385f,
                                                40.1334f, leg_index, 13.1432047f, 1233,
                                                maneuver_index, 3.96090412f, 226));
  //----------------------------------------------------------------
  // trace point = 1040
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5413513f, 40.1335678f, 1491515785, 32.5615082),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5413666f,
                                                40.1335526f, leg_index, 13.1228962f, 1231,
                                                maneuver_index, 3.94059563f, 224));
  //----------------------------------------------------------------
  // trace point = 1041
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5416794f, 40.1337166f, 1491515786, 32.7316971),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.541687f,
                                                40.1337051f, leg_index, 13.102931f, 1230,
                                                maneuver_index, 3.92063046f, 223));
  //----------------------------------------------------------------
  // trace point = 1042
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5419998f, 40.1338692f, 1491515787, 31.9056454),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5420074f,
                                                40.1338577f, leg_index, 13.0829687f, 1229,
                                                maneuver_index, 3.90066814f, 222));
  //----------------------------------------------------------------
  // trace point = 1043
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5423126f, 40.1340256f, 1491515788, 31.6673317),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5423203f,
                                                40.1340179f, leg_index, 13.063056f, 1228,
                                                maneuver_index, 3.88075542f, 221));
  //----------------------------------------------------------------
  // trace point = 1044
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5426178f, 40.1341858f, 1491515789, 31.27075),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5426254f,
                                                40.1341782f, leg_index, 13.0434771f, 1227,
                                                maneuver_index, 3.86117649f, 220));
  //----------------------------------------------------------------
  // trace point = 1045
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5429153f, 40.134346f, 1491515790, 31.0556049),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5429153f,
                                                40.1343422f, leg_index, 13.0244045f, 1226,
                                                maneuver_index, 3.84210396f, 219));
  //----------------------------------------------------------------
  // trace point = 1046
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5432053f, 40.1345177f, 1491515791, 31.2365494),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5432129f,
                                                40.13451f, leg_index, 13.0048494f, 1224,
                                                maneuver_index, 3.82254887f, 217));
  //----------------------------------------------------------------
  // trace point = 1047
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5434952f, 40.1346855f, 1491515792, 31.1035042),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5435028f,
                                                40.1346817f, leg_index, 12.9854574f, 1223,
                                                maneuver_index, 3.80315685f, 216));
  //----------------------------------------------------------------
  // trace point = 1048
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5437851f, 40.1348572f, 1491515793, 31.3726711),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5437851f,
                                                40.134861f, leg_index, 12.9660482f, 1222,
                                                maneuver_index, 3.78374767f, 215));
  //----------------------------------------------------------------
  // trace point = 1049
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5440826f, 40.1350327f, 1491515794, 31.509779),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5440674f,
                                                40.1350441f, leg_index, 12.9464722f, 1220,
                                                maneuver_index, 3.7641716f, 213));
  //----------------------------------------------------------------
  // trace point = 1050
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5443649f, 40.1352119f, 1491515795, 31.5325241),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5443497f,
                                                40.1352272f, leg_index, 12.9268961f, 1219,
                                                maneuver_index, 3.74459553f, 212));
  //----------------------------------------------------------------
  // trace point = 1051
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5446472f, 40.1353951f, 1491515796, 31.3505421),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5446243f,
                                                40.1354179f, leg_index, 12.9072742f, 1218,
                                                maneuver_index, 3.72497368f, 211));
  //----------------------------------------------------------------
  // trace point = 1052
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5449142f, 40.1355858f, 1491515797, 31.3689651),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5448914f,
                                                40.1356087f, leg_index, 12.887949f, 1217,
                                                maneuver_index, 3.70564842f, 210));
  //----------------------------------------------------------------
  // trace point = 1053
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5451889f, 40.1357803f, 1491515798, 31.6724205),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5451584f,
                                                40.1358032f, leg_index, 12.8684425f, 1215,
                                                maneuver_index, 3.68614197f, 208));
  //----------------------------------------------------------------
  // trace point = 1054
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5454483f, 40.1359787f, 1491515799, 31.5349941),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5454178f,
                                                40.1360016f, leg_index, 12.8490419f, 1214,
                                                maneuver_index, 3.66674137f, 207));
  //----------------------------------------------------------------
  // trace point = 1055
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5457153f, 40.1361809f, 1491515800, 31.7692642),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5456924f,
                                                40.1362f, leg_index, 12.8290634f, 1213,
                                                maneuver_index, 3.64676285f, 206));
  //----------------------------------------------------------------
  // trace point = 1056
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5459824f, 40.1363792f, 1491515801, 31.5182648),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5459595f,
                                                40.1363983f, leg_index, 12.8093748f, 1212,
                                                maneuver_index, 3.62707424f, 205));
  //----------------------------------------------------------------
  // trace point = 1057
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5462494f, 40.1365776f, 1491515802, 31.8730659),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5462189f,
                                                40.1366005f, leg_index, 12.7897892f, 1211,
                                                maneuver_index, 3.60748863f, 204));
  //----------------------------------------------------------------
  // trace point = 1058
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5465164f, 40.136776f, 1491515803, 31.4133015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5464859f,
                                                40.1367989f, leg_index, 12.7701006f, 1210,
                                                maneuver_index, 3.58780003f, 203));
  //----------------------------------------------------------------
  // trace point = 1059
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5467682f, 40.1369705f, 1491515804, 30.9541645),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5467453f,
                                                40.1369896f, leg_index, 12.7510681f, 1208,
                                                maneuver_index, 3.56876755f, 201));
  //----------------------------------------------------------------
  // trace point = 1060
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5470276f, 40.1371651f, 1491515805, 30.658865),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5470047f,
                                                40.1371841f, leg_index, 12.7318525f, 1207,
                                                maneuver_index, 3.54955196f, 200));
  //----------------------------------------------------------------
  // trace point = 1061
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.547287f, 40.1373558f, 1491515806, 30.5032177),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5472641f,
                                                40.1373749f, leg_index, 12.7128201f, 1206,
                                                maneuver_index, 3.53051949f, 199));
  //----------------------------------------------------------------
  // trace point = 1062
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5475388f, 40.1375427f, 1491515807, 29.9488201),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5475159f,
                                                40.1375618f, leg_index, 12.6942616f, 1205,
                                                maneuver_index, 3.51196098f, 198));
  //----------------------------------------------------------------
  // trace point = 1063
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5477905f, 40.1377296f, 1491515808, 29.7657547),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5477676f,
                                                40.1377487f, leg_index, 12.675703f, 1204,
                                                maneuver_index, 3.49340248f, 197));
  //----------------------------------------------------------------
  // trace point = 1064
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5480423f, 40.1379166f, 1491515809, 30.0713348),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5480194f,
                                                40.1379356f, leg_index, 12.6571445f, 1203,
                                                maneuver_index, 3.47484398f, 196));
  //----------------------------------------------------------------
  // trace point = 1065
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5482941f, 40.1381035f, 1491515810, 30.0261745),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5482635f,
                                                40.1381264f, leg_index, 12.63869f, 1202,
                                                maneuver_index, 3.45638943f, 195));
  //----------------------------------------------------------------
  // trace point = 1066
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5485458f, 40.1382942f, 1491515811, 30.1210804),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5485229f,
                                                40.1383133f, leg_index, 12.6198425f, 1201,
                                                maneuver_index, 3.43754196f, 194));
  //----------------------------------------------------------------
  // trace point = 1067
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5487976f, 40.1384811f, 1491515812, 29.8874817),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5487747f,
                                                40.1385002f, leg_index, 12.601284f, 1200,
                                                maneuver_index, 3.41898346f, 193));
  //----------------------------------------------------------------
  // trace point = 1068
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.549057f, 40.1386757f, 1491515813, 30.519804),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5490341f,
                                                40.1386948f, leg_index, 12.5820675f, 1199,
                                                maneuver_index, 3.39976692f, 192));
  //----------------------------------------------------------------
  // trace point = 1069
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5493088f, 40.1388626f, 1491515814, 30.1816044),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5492859f,
                                                40.1388817f, leg_index, 12.563509f, 1198,
                                                maneuver_index, 3.38120842f, 191));
  //----------------------------------------------------------------
  // trace point = 1070
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5495605f, 40.1390533f, 1491515815, 30.2256222),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5495377f,
                                                40.1390724f, leg_index, 12.5447664f, 1197,
                                                maneuver_index, 3.36246586f, 190));
  //----------------------------------------------------------------
  // trace point = 1071
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5498199f, 40.1392441f, 1491515816, 30.3809738),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5497971f,
                                                40.1392632f, leg_index, 12.5257349f, 1196,
                                                maneuver_index, 3.34343433f, 189));
  //----------------------------------------------------------------
  // trace point = 1072
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5500793f, 40.1394348f, 1491515817, 30.6863117),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5500488f,
                                                40.1394577f, leg_index, 12.5068083f, 1195,
                                                maneuver_index, 3.32450771f, 188));
  //----------------------------------------------------------------
  // trace point = 1073
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5503387f, 40.1396255f, 1491515818, 30.7635021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5503082f,
                                                40.1396484f, leg_index, 12.4877758f, 1194,
                                                maneuver_index, 3.30547523f, 187));
  //----------------------------------------------------------------
  // trace point = 1074
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5505981f, 40.1398201f, 1491515819, 30.9632378),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5505676f,
                                                40.139843f, leg_index, 12.4685612f, 1193,
                                                maneuver_index, 3.2862606f, 186));
  //----------------------------------------------------------------
  // trace point = 1075
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5508652f, 40.1400108f, 1491515820, 30.8860531),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5508347f,
                                                40.1400375f, leg_index, 12.4490557f, 1192,
                                                maneuver_index, 3.2667551f, 185));
  //----------------------------------------------------------------
  // trace point = 1076
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5511246f, 40.1402092f, 1491515821, 31.4302788),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5510941f,
                                                40.1402321f, leg_index, 12.4298401f, 1191,
                                                maneuver_index, 3.24753952f, 184));
  //----------------------------------------------------------------
  // trace point = 1077
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.551384f, 40.1404114f, 1491515822, 31.430233),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5513611f,
                                                40.1404305f, leg_index, 12.4101515f, 1190,
                                                maneuver_index, 3.22785091f, 183));
  //----------------------------------------------------------------
  // trace point = 1078
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.551651f, 40.1406097f, 1491515823, 31.8112583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5516281f,
                                                40.140625f, leg_index, 12.3906469f, 1189,
                                                maneuver_index, 3.20834637f, 182));
  //----------------------------------------------------------------
  // trace point = 1079
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.551918f, 40.1408119f, 1491515824, 32.045826),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5519028f,
                                                40.1408272f, leg_index, 12.3704844f, 1187,
                                                maneuver_index, 3.18818378f, 180));
  //----------------------------------------------------------------
  // trace point = 1080
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5521927f, 40.1410179f, 1491515825, 32.1846237),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5521774f,
                                                40.1410294f, leg_index, 12.3503246f, 1186,
                                                maneuver_index, 3.16802406f, 179));
  //----------------------------------------------------------------
  // trace point = 1081
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5524597f, 40.1412201f, 1491515826, 32.3234291),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5524521f,
                                                40.1412277f, leg_index, 12.3303404f, 1186,
                                                maneuver_index, 3.14803982f, 179));
  //----------------------------------------------------------------
  // trace point = 1082
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5527267f, 40.1414261f, 1491515827, 32.2631683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5527267f,
                                                40.1414261f, leg_index, 12.3103609f, 1184,
                                                maneuver_index, 3.12806034f, 177));
  //----------------------------------------------------------------
  // trace point = 1083
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5529938f, 40.1416321f, 1491515828, 32.1242294),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5530014f,
                                                40.1416245f, leg_index, 12.2903814f, 1183,
                                                maneuver_index, 3.10808086f, 176));
  //----------------------------------------------------------------
  // trace point = 1084
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5532608f, 40.1418343f, 1491515829, 32.2447853),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5532684f,
                                                40.1418266f, leg_index, 12.2705135f, 1182,
                                                maneuver_index, 3.08821297f, 175));
  //----------------------------------------------------------------
  // trace point = 1085
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5535355f, 40.1420364f, 1491515830, 32.2709465),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5535507f,
                                                40.142025f, leg_index, 12.2502375f, 1180,
                                                maneuver_index, 3.0679369f, 173));
  //----------------------------------------------------------------
  // trace point = 1086
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5538025f, 40.1422386f, 1491515831, 32.3485641),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5538177f,
                                                40.1422272f, leg_index, 12.2303648f, 1179,
                                                maneuver_index, 3.04806423f, 172));
  //----------------------------------------------------------------
  // trace point = 1087
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5540771f, 40.1424446f, 1491515832, 32.1842842),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5540924f,
                                                40.1424332f, leg_index, 12.2100191f, 1178,
                                                maneuver_index, 3.02771854f, 171));
  //----------------------------------------------------------------
  // trace point = 1088
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5543442f, 40.1426468f, 1491515833, 32.4871254),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5543594f,
                                                40.1426315f, leg_index, 12.1903305f, 1177,
                                                maneuver_index, 3.00802994f, 170));
  //----------------------------------------------------------------
  // trace point = 1089
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5546188f, 40.1428528f, 1491515834, 32.6430931),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5546341f,
                                                40.1428375f, leg_index, 12.1699858f, 1175,
                                                maneuver_index, 2.9876852f, 168));
  //----------------------------------------------------------------
  // trace point = 1090
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5548935f, 40.1430588f, 1491515835, 32.3049622),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5549164f,
                                                40.1430435f, leg_index, 12.1493511f, 1174,
                                                maneuver_index, 2.96705055f, 167));
  //----------------------------------------------------------------
  // trace point = 1091
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5551605f, 40.143261f, 1491515836, 32.2444458),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5551834f,
                                                40.1432419f, leg_index, 12.1296625f, 1173,
                                                maneuver_index, 2.94736195f, 166));
  //----------------------------------------------------------------
  // trace point = 1092
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5554276f, 40.1434593f, 1491515837, 31.6551533),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5554504f,
                                                40.1434441f, leg_index, 12.1097898f, 1172,
                                                maneuver_index, 2.92748928f, 165));
  //----------------------------------------------------------------
  // trace point = 1093
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5556946f, 40.1436577f, 1491515838, 31.6851177),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5557175f,
                                                40.1436424f, leg_index, 12.0901022f, 1171,
                                                maneuver_index, 2.90780163f, 164));
  //----------------------------------------------------------------
  // trace point = 1094
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5559616f, 40.1438484f, 1491515839, 31.4083767),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5559769f,
                                                40.1438332f, leg_index, 12.0710716f, 1170,
                                                maneuver_index, 2.88877106f, 163));
  //----------------------------------------------------------------
  // trace point = 1095
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5562363f, 40.144043f, 1491515840, 31.5941811),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5562515f,
                                                40.1440315f, leg_index, 12.0510931f, 1169,
                                                maneuver_index, 2.86879253f, 162));
  //----------------------------------------------------------------
  // trace point = 1096
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5565033f, 40.1442413f, 1491515841, 31.4553223),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5565186f,
                                                40.1442299f, leg_index, 12.0314045f, 1168,
                                                maneuver_index, 2.84910393f, 161));
  //----------------------------------------------------------------
  // trace point = 1097
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5567703f, 40.1444397f, 1491515842, 31.7160511),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5567856f,
                                                40.1444283f, leg_index, 12.0117159f, 1167,
                                                maneuver_index, 2.82941532f, 160));
  //----------------------------------------------------------------
  // trace point = 1098
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5570374f, 40.1446381f, 1491515843, 31.7160034),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5570526f,
                                                40.1446266f, leg_index, 11.9920282f, 1166,
                                                maneuver_index, 2.80972767f, 159));
  //----------------------------------------------------------------
  // trace point = 1099
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5573044f, 40.1448326f, 1491515844, 31.5774136),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5573196f,
                                                40.1448212f, leg_index, 11.9725237f, 1165,
                                                maneuver_index, 2.79022312f, 158));
  //----------------------------------------------------------------
  // trace point = 1100
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5575714f, 40.1450348f, 1491515845, 31.9930344),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5575867f,
                                                40.1450233f, leg_index, 11.952652f, 1164,
                                                maneuver_index, 2.77035141f, 157));
  //----------------------------------------------------------------
  // trace point = 1101
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5578384f, 40.145237f, 1491515846, 32.0875587),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5578537f,
                                                40.1452217f, leg_index, 11.9329643f, 1163,
                                                maneuver_index, 2.75066376f, 156));
  //----------------------------------------------------------------
  // trace point = 1102
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5581131f, 40.1454391f, 1491515847, 32.3043861),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5581284f,
                                                40.1454239f, leg_index, 11.9128027f, 1162,
                                                maneuver_index, 2.73050213f, 155));
  //----------------------------------------------------------------
  // trace point = 1103
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5583878f, 40.1456413f, 1491515848, 32.25383),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.558403f,
                                                40.1456299f, leg_index, 11.8924589f, 1161,
                                                maneuver_index, 2.71015835f, 154));
  //----------------------------------------------------------------
  // trace point = 1104
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5586548f, 40.1458435f, 1491515849, 32.3476868),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.55867f,
                                                40.1458282f, leg_index, 11.8727684f, 1160,
                                                maneuver_index, 2.69046783f, 153));
  //----------------------------------------------------------------
  // trace point = 1105
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5589294f, 40.1460419f, 1491515850, 31.8382549),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5589447f,
                                                40.1460304f, leg_index, 11.8526087f, 1159,
                                                maneuver_index, 2.67030811f, 152));
  //----------------------------------------------------------------
  // trace point = 1106
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5591965f, 40.1462402f, 1491515851, 31.5159054),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5592117f,
                                                40.1462326f, leg_index, 11.832736f, 1157,
                                                maneuver_index, 2.65043545f, 150));
  //----------------------------------------------------------------
  // trace point = 1107
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5594559f, 40.1464348f, 1491515852, 31.1943398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5594711f,
                                                40.1464272f, leg_index, 11.8135214f, 1156,
                                                maneuver_index, 2.63122082f, 149));
  //----------------------------------------------------------------
  // trace point = 1108
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5597153f, 40.1466255f, 1491515853, 30.7618637),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5597229f,
                                                40.1466179f, leg_index, 11.7947788f, 1155,
                                                maneuver_index, 2.61247826f, 148));
  //----------------------------------------------------------------
  // trace point = 1109
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5599747f, 40.1468201f, 1491515854, 30.9169216),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5599823f,
                                                40.1468124f, leg_index, 11.7755651f, 1155,
                                                maneuver_index, 2.59326458f, 148));
  //----------------------------------------------------------------
  // trace point = 1110
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5602341f, 40.1470108f, 1491515855, 30.3462086),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5602493f,
                                                40.1470032f, leg_index, 11.7562428f, 1154,
                                                maneuver_index, 2.57394218f, 147));
  //----------------------------------------------------------------
  // trace point = 1111
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5604858f, 40.1471977f, 1491515856, 30.2690907),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5605011f,
                                                40.1471863f, leg_index, 11.7378683f, 1153,
                                                maneuver_index, 2.55556774f, 146));
  //----------------------------------------------------------------
  // trace point = 1112
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5607452f, 40.1473808f, 1491515857, 30.0873432),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5607529f,
                                                40.1473732f, leg_index, 11.7193117f, 1151,
                                                maneuver_index, 2.53701115f, 144));
  //----------------------------------------------------------------
  // trace point = 1113
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5610046f, 40.1475716f, 1491515858, 30.2403927),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5610123f,
                                                40.1475639f, leg_index, 11.7002811f, 1150,
                                                maneuver_index, 2.51798058f, 143));
  //----------------------------------------------------------------
  // trace point = 1114
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5612564f, 40.1477623f, 1491515859, 30.4845409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5612717f,
                                                40.1477509f, leg_index, 11.6814318f, 1149,
                                                maneuver_index, 2.4991312f, 142));
  //----------------------------------------------------------------
  // trace point = 1115
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5615158f, 40.147953f, 1491515860, 30.3011799),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5615311f,
                                                40.1479416f, leg_index, 11.6624012f, 1148,
                                                maneuver_index, 2.48010063f, 141));
  //----------------------------------------------------------------
  // trace point = 1116
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5617752f, 40.1481438f, 1491515861, 30.7779713),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5617905f,
                                                40.1481323f, leg_index, 11.6433706f, 1147,
                                                maneuver_index, 2.46107006f, 140));
  //----------------------------------------------------------------
  // trace point = 1117
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5620346f, 40.1483307f, 1491515862, 30.2391262),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5620422f,
                                                40.1483231f, leg_index, 11.624629f, 1146,
                                                maneuver_index, 2.44232845f, 139));
  //----------------------------------------------------------------
  // trace point = 1118
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.562294f, 40.1485214f, 1491515863, 30.6392536),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5623016f,
                                                40.1485176f, leg_index, 11.6054134f, 1145,
                                                maneuver_index, 2.42311287f, 138));
  //----------------------------------------------------------------
  // trace point = 1119
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5625534f, 40.1487122f, 1491515864, 30.622839),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.562561f,
                                                40.1487083f, leg_index, 11.5863829f, 1144,
                                                maneuver_index, 2.4040823f, 137));
  //----------------------------------------------------------------
  // trace point = 1120
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5628128f, 40.1489067f, 1491515865, 30.8387794),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5628204f,
                                                40.1488991f, leg_index, 11.5673532f, 1142,
                                                maneuver_index, 2.38505268f, 135));
  //----------------------------------------------------------------
  // trace point = 1121
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5630646f, 40.1491013f, 1491515866, 30.7168789),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5630798f,
                                                40.1490898f, leg_index, 11.5483208f, 1141,
                                                maneuver_index, 2.3660202f, 134));
  //----------------------------------------------------------------
  // trace point = 1122
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.563324f, 40.149292f, 1491515867, 30.517374),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5633392f,
                                                40.1492805f, leg_index, 11.527976f, 1140,
                                                maneuver_index, 2.34567547f, 133));
  //----------------------------------------------------------------
  // trace point = 1123
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5635834f, 40.1494904f, 1491515868, 30.9097061),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5635986f,
                                                40.1494789f, leg_index, 11.5098925f, 1139,
                                                maneuver_index, 2.3275919f, 132));
  //----------------------------------------------------------------
  // trace point = 1124
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5638351f, 40.1496811f, 1491515869, 30.5953522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5638504f,
                                                40.1496696f, leg_index, 11.4911499f, 1138,
                                                maneuver_index, 2.30884933f, 131));
  //----------------------------------------------------------------
  // trace point = 1125
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5640869f, 40.1498718f, 1491515870, 30.3178329),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5641022f,
                                                40.1498604f, leg_index, 11.4724083f, 1137,
                                                maneuver_index, 2.29010773f, 130));
  //----------------------------------------------------------------
  // trace point = 1126
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5643463f, 40.1500626f, 1491515871, 30.10116),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5643539f,
                                                40.1500549f, leg_index, 11.4534807f, 1136,
                                                maneuver_index, 2.27118015f, 129));
  //----------------------------------------------------------------
  // trace point = 1127
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5645905f, 40.1502495f, 1491515872, 29.7983418),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5645981f,
                                                40.1502419f, leg_index, 11.4352102f, 1135,
                                                maneuver_index, 2.25290966f, 128));
  //----------------------------------------------------------------
  // trace point = 1128
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5648346f, 40.150444f, 1491515873, 30.0547829),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5648499f,
                                                40.1504326f, leg_index, 11.4164686f, 1133,
                                                maneuver_index, 2.23416805f, 126));
  //----------------------------------------------------------------
  // trace point = 1129
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5650864f, 40.1506386f, 1491515874, 30.2735424),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5651016f,
                                                40.1506271f, leg_index, 11.397541f, 1132,
                                                maneuver_index, 2.21524048f, 125));
  //----------------------------------------------------------------
  // trace point = 1130
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5653305f, 40.1508331f, 1491515875, 30.1343765),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5653534f,
                                                40.1508179f, leg_index, 11.3787994f, 1131,
                                                maneuver_index, 2.19649887f, 124));
  //----------------------------------------------------------------
  // trace point = 1131
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5655823f, 40.1510277f, 1491515876, 30.193737),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5656052f,
                                                40.1510124f, leg_index, 11.35987f, 1130,
                                                maneuver_index, 2.17756939f, 123));
  //----------------------------------------------------------------
  // trace point = 1132
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5658264f, 40.1512222f, 1491515877, 30.2942448),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5658493f,
                                                40.1512032f, leg_index, 11.3414125f, 1129,
                                                maneuver_index, 2.15911198f, 122));
  //----------------------------------------------------------------
  // trace point = 1133
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5660782f, 40.1514168f, 1491515878, 30.4324837),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5661011f,
                                                40.1513977f, leg_index, 11.322484f, 1128,
                                                maneuver_index, 2.14018345f, 121));
  //----------------------------------------------------------------
  // trace point = 1134
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.56633f, 40.1516113f, 1491515879, 30.3535461),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5663528f,
                                                40.1515923f, leg_index, 11.3035555f, 1127,
                                                maneuver_index, 2.12125492f, 120));
  //----------------------------------------------------------------
  // trace point = 1135
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5665817f, 40.1518059f, 1491515880, 30.6108665),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5666046f,
                                                40.1517868f, leg_index, 11.2846279f, 1126,
                                                maneuver_index, 2.10232735f, 119));
  //----------------------------------------------------------------
  // trace point = 1136
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5668335f, 40.1520004f, 1491515881, 30.432354),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5668564f,
                                                40.1519814f, leg_index, 11.2656994f, 1125,
                                                maneuver_index, 2.08339882f, 118));
  //----------------------------------------------------------------
  // trace point = 1137
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5670929f, 40.152195f, 1491515882, 30.5342617),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5671158f,
                                                40.1521797f, leg_index, 11.2462997f, 1123,
                                                maneuver_index, 2.06399918f, 116));
  //----------------------------------------------------------------
  // trace point = 1138
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5673447f, 40.1523895f, 1491515883, 30.6311893),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5673676f,
                                                40.1523743f, leg_index, 11.2273712f, 1123,
                                                maneuver_index, 2.04507065f, 116));
  //----------------------------------------------------------------
  // trace point = 1139 | Alert | In 2 miles Take the Pennsylvania 7 43 exit on the right.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5676041f, 40.1525841f, 1491515884, 30.8117695),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5676193f, 40.1525688f, leg_index, 11.2084436f,
                                                1122, maneuver_index, 2.02614307f, 115,
                                                instruction_index,
                                                kInitialLongTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1140
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5678635f, 40.1527786f, 1491515885, 30.7768955),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5678787f,
                                                40.1527672f, leg_index, 11.189044f, 1121,
                                                maneuver_index, 2.00674343f, 114));
  //----------------------------------------------------------------
  // trace point = 1141
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5681152f, 40.1529655f, 1491515886, 30.3000317),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5681229f,
                                                40.1529579f, leg_index, 11.1705866f, 1120,
                                                maneuver_index, 1.98828602f, 113));
  //----------------------------------------------------------------
  // trace point = 1142
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.568367f, 40.1531639f, 1491515887, 30.8489761),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5683823f,
                                                40.1531563f, leg_index, 11.1511879f, 1118,
                                                maneuver_index, 1.96888733f, 111));
  //----------------------------------------------------------------
  // trace point = 1143
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5686264f, 40.1533623f, 1491515888, 30.7294884),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.568634f,
                                                40.1533546f, leg_index, 11.1320715f, 1117,
                                                maneuver_index, 1.94977093f, 110));
  //----------------------------------------------------------------
  // trace point = 1144
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5688782f, 40.1535568f, 1491515889, 30.8488865),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5688858f,
                                                40.1535492f, leg_index, 11.1131439f, 1116,
                                                maneuver_index, 1.93084335f, 109));
  //----------------------------------------------------------------
  // trace point = 1145
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5691299f, 40.1537514f, 1491515890, 30.612339),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5691376f,
                                                40.1537437f, leg_index, 11.0942154f, 1115,
                                                maneuver_index, 1.91191483f, 108));
  //----------------------------------------------------------------
  // trace point = 1146
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5693893f, 40.1539497f, 1491515891, 30.889986),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.569397f,
                                                40.1539421f, leg_index, 11.0748167f, 1114,
                                                maneuver_index, 1.89251614f, 107));
  //----------------------------------------------------------------
  // trace point = 1147
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5696411f, 40.1541481f, 1491515892, 31.1077003),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5696487f,
                                                40.1541405f, leg_index, 11.0557013f, 1113,
                                                maneuver_index, 1.87340069f, 106));
  //----------------------------------------------------------------
  // trace point = 1148
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5699005f, 40.1543465f, 1491515893, 31.0073757),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5699081f,
                                                40.1543388f, leg_index, 11.0363016f, 1112,
                                                maneuver_index, 1.85400105f, 105));
  //----------------------------------------------------------------
  // trace point = 1149
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5701523f, 40.1545448f, 1491515894, 30.9279003),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5701675f,
                                                40.1545372f, leg_index, 11.0169029f, 1111,
                                                maneuver_index, 1.83460236f, 104));
  //----------------------------------------------------------------
  // trace point = 1150
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5704117f, 40.1547432f, 1491515895, 31.0668793),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5704193f,
                                                40.1547356f, leg_index, 10.9977875f, 1110,
                                                maneuver_index, 1.81548691f, 103));
  //----------------------------------------------------------------
  // trace point = 1151
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5706711f, 40.1549416f, 1491515896, 31.1073971),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5706787f,
                                                40.1549339f, leg_index, 10.9783878f, 1109,
                                                maneuver_index, 1.79608727f, 102));
  //----------------------------------------------------------------
  // trace point = 1152
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5709229f, 40.1551361f, 1491515897, 30.7934532),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5709305f,
                                                40.1551285f, leg_index, 10.9594603f, 1108,
                                                maneuver_index, 1.77715969f, 101));
  //----------------------------------------------------------------
  // trace point = 1153
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5711823f, 40.1553345f, 1491515898, 31.1862831),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5711899f,
                                                40.1553268f, leg_index, 10.9400606f, 1106,
                                                maneuver_index, 1.75776005f, 99));
  //----------------------------------------------------------------
  // trace point = 1154
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.571434f, 40.1555367f, 1491515899, 31.1286659),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5714493f,
                                                40.1555252f, leg_index, 10.9206619f, 1105,
                                                maneuver_index, 1.73836136f, 98));
  //----------------------------------------------------------------
  // trace point = 1155
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.571701f, 40.1557388f, 1491515900, 31.5654869),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5717087f,
                                                40.1557312f, leg_index, 10.9008875f, 1104,
                                                maneuver_index, 1.71858692f, 97));
  //----------------------------------------------------------------
  // trace point = 1156
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5719604f, 40.155941f, 1491515901, 31.5633411),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5719681f,
                                                40.1559334f, leg_index, 10.8813009f, 1103,
                                                maneuver_index, 1.69900036f, 96));
  //----------------------------------------------------------------
  // trace point = 1157
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5722198f, 40.156147f, 1491515902, 32.0994682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5722351f,
                                                40.1561356f, leg_index, 10.8614311f, 1102,
                                                maneuver_index, 1.67913055f, 95));
  //----------------------------------------------------------------
  // trace point = 1158
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5724945f, 40.156353f, 1491515903, 32.3801193),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5725021f,
                                                40.1563454f, leg_index, 10.8411856f, 1101,
                                                maneuver_index, 1.658885f, 94));
  //----------------------------------------------------------------
  // trace point = 1159
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5727615f, 40.1565514f, 1491515904, 31.9461365),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5727692f,
                                                40.1565475f, leg_index, 10.8213158f, 1100,
                                                maneuver_index, 1.6390152f, 93));
  //----------------------------------------------------------------
  // trace point = 1160
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5730209f, 40.1567535f, 1491515905, 31.5049953),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5730286f,
                                                40.1567497f, leg_index, 10.8017292f, 1099,
                                                maneuver_index, 1.61942863f, 92));
  //----------------------------------------------------------------
  // trace point = 1161
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.573288f, 40.1569557f, 1491515906, 31.6626759),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.573288f,
                                                40.1569519f, leg_index, 10.7821426f, 1097,
                                                maneuver_index, 1.59984207f, 90));
  //----------------------------------------------------------------
  // trace point = 1162
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5735474f, 40.1571579f, 1491515907, 31.6626301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.573555f,
                                                40.1571541f, leg_index, 10.7622728f, 1096,
                                                maneuver_index, 1.57997227f, 89));
  //----------------------------------------------------------------
  // trace point = 1163
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5738068f, 40.1573563f, 1491515908, 31.3443642),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5738068f,
                                                40.1573524f, leg_index, 10.7431574f, 1095,
                                                maneuver_index, 1.56085682f, 88));
  //----------------------------------------------------------------
  // trace point = 1164
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5740738f, 40.1575584f, 1491515909, 31.7293892),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5740738f,
                                                40.1575584f, leg_index, 10.7230988f, 1094,
                                                maneuver_index, 1.54079819f, 87));
  //----------------------------------------------------------------
  // trace point = 1165
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5743408f, 40.1577606f, 1491515910, 31.9029884),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5743408f,
                                                40.1577606f, leg_index, 10.7032299f, 1093,
                                                maneuver_index, 1.52092934f, 86));
  //----------------------------------------------------------------
  // trace point = 1166
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5746002f, 40.1579628f, 1491515911, 31.9599915),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5746002f,
                                                40.1579628f, leg_index, 10.6836433f, 1092,
                                                maneuver_index, 1.50134277f, 85));
  //----------------------------------------------------------------
  // trace point = 1167
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5748672f, 40.1581612f, 1491515912, 31.4865017),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5748596f,
                                                40.158165f, leg_index, 10.6640577f, 1091,
                                                maneuver_index, 1.48175716f, 84));
  //----------------------------------------------------------------
  // trace point = 1168
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.575119f, 40.1583633f, 1491515913, 31.2647686),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.575119f,
                                                40.1583633f, leg_index, 10.644659f, 1090,
                                                maneuver_index, 1.46235847f, 83));
  //----------------------------------------------------------------
  // trace point = 1169
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.575386f, 40.1585617f, 1491515914, 31.3657188),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.575386f,
                                                40.1585655f, leg_index, 10.6247883f, 1089,
                                                maneuver_index, 1.44248772f, 82));
  //----------------------------------------------------------------
  // trace point = 1170
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5756378f, 40.1587601f, 1491515915, 31.0659752),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5756378f,
                                                40.1587601f, leg_index, 10.6058617f, 1087,
                                                maneuver_index, 1.4235611f, 80));
  //----------------------------------------------------------------
  // trace point = 1171
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5758972f, 40.1589584f, 1491515916, 30.7686348),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5758972f,
                                                40.1589584f, leg_index, 10.586463f, 1086,
                                                maneuver_index, 1.40416241f, 79));
  //----------------------------------------------------------------
  // trace point = 1172
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.576149f, 40.1591568f, 1491515917, 30.9468346),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.576149f,
                                                40.1591568f, leg_index, 10.5673475f, 1086,
                                                maneuver_index, 1.38504696f, 79));
  //----------------------------------------------------------------
  // trace point = 1173
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5763931f, 40.1593513f, 1491515918, 30.3515797),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5764008f,
                                                40.1593475f, leg_index, 10.5486097f, 1084,
                                                maneuver_index, 1.36630917f, 77));
  //----------------------------------------------------------------
  // trace point = 1174
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5766449f, 40.1595459f, 1491515919, 30.1123619),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5766525f,
                                                40.1595421f, leg_index, 10.5296812f, 1083,
                                                maneuver_index, 1.34738064f, 76));
  //----------------------------------------------------------------
  // trace point = 1175
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5768967f, 40.1597366f, 1491515920, 30.1341991),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5768967f,
                                                40.1597366f, leg_index, 10.5110369f, 1082,
                                                maneuver_index, 1.32873631f, 75));
  //----------------------------------------------------------------
  // trace point = 1176
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5771408f, 40.1599274f, 1491515921, 30.1719494),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5771408f,
                                                40.1599274f, leg_index, 10.4925814f, 1081,
                                                maneuver_index, 1.3102808f, 74));
  //----------------------------------------------------------------
  // trace point = 1177
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5773926f, 40.1601219f, 1491515922, 30.1719074),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5773926f,
                                                40.1601181f, leg_index, 10.4738407f, 1080,
                                                maneuver_index, 1.29154015f, 73));
  //----------------------------------------------------------------
  // trace point = 1178
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5776443f, 40.1603165f, 1491515923, 30.4105721),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5776443f,
                                                40.1603127f, leg_index, 10.4549141f, 1079,
                                                maneuver_index, 1.27261353f, 72));
  //----------------------------------------------------------------
  // trace point = 1179
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5778885f, 40.160511f, 1491515924, 30.0526047),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5778961f,
                                                40.1605034f, leg_index, 10.4361753f, 1078,
                                                maneuver_index, 1.25387478f, 71));
  //----------------------------------------------------------------
  // trace point = 1180
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5781403f, 40.1607018f, 1491515925, 30.0327854),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5781479f,
                                                40.1606979f, leg_index, 10.4172468f, 1077,
                                                maneuver_index, 1.23494625f, 70));
  //----------------------------------------------------------------
  // trace point = 1181
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5783844f, 40.1608925f, 1491515926, 29.8563023),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.578392f,
                                                40.1608887f, leg_index, 10.3987913f, 1076,
                                                maneuver_index, 1.21649075f, 69));
  //----------------------------------------------------------------
  // trace point = 1182
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5786362f, 40.1610794f, 1491515927, 29.8991985),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5786362f,
                                                40.1610794f, leg_index, 10.3803368f, 1075,
                                                maneuver_index, 1.19803619f, 68));
  //----------------------------------------------------------------
  // trace point = 1183
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5788879f, 40.161274f, 1491515928, 30.3704147),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5788956f,
                                                40.1612701f, leg_index, 10.36131f, 1074,
                                                maneuver_index, 1.17900944f, 67));
  //----------------------------------------------------------------
  // trace point = 1184
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5791397f, 40.1614685f, 1491515929, 30.5093498),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5791473f,
                                                40.1614647f, leg_index, 10.3423834f, 1073,
                                                maneuver_index, 1.16008282f, 66));
  //----------------------------------------------------------------
  // trace point = 1185
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5793915f, 40.1616631f, 1491515930, 30.5691471),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5793991f,
                                                40.1616554f, leg_index, 10.3236437f, 1072,
                                                maneuver_index, 1.14134312f, 65));
  //----------------------------------------------------------------
  // trace point = 1186
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5796432f, 40.1618576f, 1491515931, 30.3899441),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5796509f,
                                                40.16185f, leg_index, 10.3047171f, 1071,
                                                maneuver_index, 1.1224165f, 64));
  //----------------------------------------------------------------
  // trace point = 1187
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5798874f, 40.1620522f, 1491515932, 30.0324917),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5799026f,
                                                40.1620407f, leg_index, 10.2859774f, 1070,
                                                maneuver_index, 1.1036768f, 63));
  //----------------------------------------------------------------
  // trace point = 1188
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5801315f, 40.1622429f, 1491515933, 29.7146206),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5801468f,
                                                40.1622276f, leg_index, 10.2677116f, 1069,
                                                maneuver_index, 1.08541107f, 62));
  //----------------------------------------------------------------
  // trace point = 1189
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5803833f, 40.1624336f, 1491515934, 29.6946793),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5803986f,
                                                40.1624222f, leg_index, 10.2487803f, 1068,
                                                maneuver_index, 1.06647968f, 61));
  //----------------------------------------------------------------
  // trace point = 1190
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5806274f, 40.1626244f, 1491515935, 29.8742523),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5806427f,
                                                40.1626129f, leg_index, 10.2303247f, 1067,
                                                maneuver_index, 1.04802418f, 60));
  //----------------------------------------------------------------
  // trace point = 1191
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5808716f, 40.1628151f, 1491515936, 29.8336048),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5808868f,
                                                40.1628075f, leg_index, 10.2116795f, 1066,
                                                maneuver_index, 1.02937889f, 59));
  //----------------------------------------------------------------
  // trace point = 1192
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5811081f, 40.1630096f, 1491515937, 29.6974049),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5811234f,
                                                40.1629982f, leg_index, 10.1935053f, 1065,
                                                maneuver_index, 1.01120472f, 58));
  //----------------------------------------------------------------
  // trace point = 1193
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5813599f, 40.1631966f, 1491515938, 29.6752605),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5813675f,
                                                40.1631889f, leg_index, 10.1750498f, 1064,
                                                maneuver_index, 0.992749214f, 57));
  //----------------------------------------------------------------
  // trace point = 1194
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5815964f, 40.1633873f, 1491515939, 29.2377033),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5816116f,
                                                40.1633759f, leg_index, 10.1567841f, 1062,
                                                maneuver_index, 0.97448349f, 55));
  //----------------------------------------------------------------
  // trace point = 1195
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5818405f, 40.1635742f, 1491515940, 29.2177181),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5818558f,
                                                40.1635666f, leg_index, 10.1383257f, 1061,
                                                maneuver_index, 0.956025124f, 54));
  //----------------------------------------------------------------
  // trace point = 1196
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.582077f, 40.1637573f, 1491515941, 29.1195564),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5820923f,
                                                40.1637459f, leg_index, 10.1207161f, 1060,
                                                maneuver_index, 0.938415527f, 53));
  //----------------------------------------------------------------
  // trace point = 1197
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5823288f, 40.1639404f, 1491515942, 29.2223701),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5823364f,
                                                40.1639366f, leg_index, 10.1022615f, 1059,
                                                maneuver_index, 0.919960976f, 52));
  //----------------------------------------------------------------
  // trace point = 1198
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5825653f, 40.1641312f, 1491515943, 29.5553169),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5825806f,
                                                40.1641235f, leg_index, 10.083993f, 1058,
                                                maneuver_index, 0.90169239f, 51));
  //----------------------------------------------------------------
  // trace point = 1199
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5828171f, 40.1643219f, 1491515944, 29.9943542),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5828247f,
                                                40.1643143f, leg_index, 10.0655375f, 1057,
                                                maneuver_index, 0.883236885f, 50));
  //----------------------------------------------------------------
  // trace point = 1200
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5830688f, 40.1645126f, 1491515945, 30.1112804),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5830765f,
                                                40.1645088f, leg_index, 10.0466108f, 1056,
                                                maneuver_index, 0.864310265f, 49));
  //----------------------------------------------------------------
  // trace point = 1201
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.583313f, 40.1647034f, 1491515946, 29.6150017),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5833206f,
                                                40.1646957f, leg_index, 10.0283432f, 1055,
                                                maneuver_index, 0.846042633f, 48));
  //----------------------------------------------------------------
  // trace point = 1202
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5835571f, 40.1648903f, 1491515947, 29.277195),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5835648f,
                                                40.1648865f, leg_index, 10.0098877f, 1054,
                                                maneuver_index, 0.827587128f, 47));
  //----------------------------------------------------------------
  // trace point = 1203
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5837936f, 40.1650696f, 1491515948, 28.5638866),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5838013f,
                                                40.1650658f, leg_index, 9.99228001f, 1053,
                                                maneuver_index, 0.809979439f, 46));
  //----------------------------------------------------------------
  // trace point = 1204
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5840302f, 40.1652489f, 1491515949, 28.1298351),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5840302f,
                                                40.1652489f, leg_index, 9.97476673f, 1052,
                                                maneuver_index, 0.792466164f, 45));
  //----------------------------------------------------------------
  // trace point = 1205
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5842667f, 40.165432f, 1491515950, 28.8003559),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5842667f,
                                                40.165432f, leg_index, 9.95697021f, 1051,
                                                maneuver_index, 0.774669647f, 44));
  //----------------------------------------------------------------
  // trace point = 1206
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5844955f, 40.1656113f, 1491515951, 28.065237),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5844955f,
                                                40.1656075f, leg_index, 9.93983269f, 1050,
                                                maneuver_index, 0.75753212f, 43));
  //----------------------------------------------------------------
  // trace point = 1207
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5847321f, 40.1657944f, 1491515952, 28.3433056),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5847321f,
                                                40.1657906f, leg_index, 9.92203617f, 1049,
                                                maneuver_index, 0.739735603f, 42));
  //----------------------------------------------------------------
  // trace point = 1208
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5849609f, 40.1659775f, 1491515953, 28.5240154),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5849686f,
                                                40.1659737f, leg_index, 9.90310955f, 1048,
                                                maneuver_index, 0.720808983f, 41));
  //----------------------------------------------------------------
  // trace point = 1209
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5851974f, 40.1661644f, 1491515954, 28.7221031),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5852051f,
                                                40.1661568f, leg_index, 9.88645267f, 1047,
                                                maneuver_index, 0.704152107f, 40));
  //----------------------------------------------------------------
  // trace point = 1210
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.585434f, 40.1663475f, 1491515955, 28.7024441),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5854492f,
                                                40.1663361f, leg_index, 9.86854553f, 1046,
                                                maneuver_index, 0.686244965f, 39));
  //----------------------------------------------------------------
  // trace point = 1211
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5856705f, 40.1665268f, 1491515956, 28.5033455),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5856857f,
                                                40.1665154f, leg_index, 9.85093594f, 1045,
                                                maneuver_index, 0.668635368f, 38));
  //----------------------------------------------------------------
  // trace point = 1212
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.585907f, 40.1667061f, 1491515957, 28.1654263),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5859222f,
                                                40.1666985f, leg_index, 9.83313942f, 1044,
                                                maneuver_index, 0.650838852f, 37));
  //----------------------------------------------------------------
  // trace point = 1213
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5861435f, 40.166893f, 1491515958, 28.7603111),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5861588f,
                                                40.1668816f, leg_index, 9.8153429f, 1043,
                                                maneuver_index, 0.633042336f, 36));
  //----------------------------------------------------------------
  // trace point = 1214
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5863876f, 40.1670799f, 1491515959, 29.2767277),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5864029f,
                                                40.1670685f, leg_index, 9.79707623f, 1042,
                                                maneuver_index, 0.614775658f, 35));
  //----------------------------------------------------------------
  // trace point = 1215
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5866241f, 40.1672668f, 1491515960, 29.2766876),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5866394f,
                                                40.1672516f, leg_index, 9.77927971f, 1041,
                                                maneuver_index, 0.596979141f, 34));
  //----------------------------------------------------------------
  // trace point = 1216
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5868683f, 40.16745f, 1491515961, 28.9617329),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5868835f,
                                                40.1674385f, leg_index, 9.76101208f, 1040,
                                                maneuver_index, 0.57871151f, 33));
  //----------------------------------------------------------------
  // trace point = 1217
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5871048f, 40.1676331f, 1491515962, 28.6011562),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5871201f,
                                                40.1676216f, leg_index, 9.74321651f, 1039,
                                                maneuver_index, 0.560915947f, 32));
  //----------------------------------------------------------------
  // trace point = 1218
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5873337f, 40.1678085f, 1491515963, 28.0263271),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5873489f,
                                                40.1677971f, leg_index, 9.72607994f, 1038,
                                                maneuver_index, 0.543779373f, 31));
  //----------------------------------------------------------------
  // trace point = 1219
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5875626f, 40.1679878f, 1491515964, 27.7866974),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5875778f,
                                                40.1679764f, leg_index, 9.70875359f, 1037,
                                                maneuver_index, 0.526453018f, 30));
  //----------------------------------------------------------------
  // trace point = 1220 | Alert | In a half mile Take the Pennsylvania 7 43 exit on the right.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5877991f, 40.1681671f, 1491515965, 28.1443062),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5878143f, 40.1681595f, leg_index, 9.69095802f,
                                                1036, maneuver_index, 0.508657455f, 29,
                                                instruction_index,
                                                kFinalLongTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1221
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.588028f, 40.1683464f, 1491515966, 27.8873615),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5880432f,
                                                40.168335f, leg_index, 9.6738205f, 1035,
                                                maneuver_index, 0.491519928f, 28));
  //----------------------------------------------------------------
  // trace point = 1222
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5882568f, 40.1685257f, 1491515967, 27.9061241),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5882721f,
                                                40.1685143f, leg_index, 9.65649605f, 1034,
                                                maneuver_index, 0.47419548f, 27));
  //----------------------------------------------------------------
  // trace point = 1223
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5884933f, 40.1687012f, 1491515968, 27.9297409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5885086f,
                                                40.1686897f, leg_index, 9.63907337f, 1033,
                                                maneuver_index, 0.456772804f, 26));
  //----------------------------------------------------------------
  // trace point = 1224
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5887299f, 40.1688766f, 1491515969, 28.0261078),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5887451f,
                                                40.168869f, leg_index, 9.62146473f, 1032,
                                                maneuver_index, 0.439164162f, 25));
  //----------------------------------------------------------------
  // trace point = 1225
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5889587f, 40.1690521f, 1491515970, 27.6882),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.588974f,
                                                40.1690445f, leg_index, 9.6043272f, 1031,
                                                maneuver_index, 0.422026634f, 24));
  //----------------------------------------------------------------
  // trace point = 1226
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5891876f, 40.1692276f, 1491515971, 27.5492916),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5892029f,
                                                40.16922f, leg_index, 9.58718967f, 1030,
                                                maneuver_index, 0.404889107f, 23));
  //----------------------------------------------------------------
  // trace point = 1227
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5894165f, 40.1694069f, 1491515972, 27.5681763),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5894318f,
                                                40.1693954f, leg_index, 9.57005215f, 1029,
                                                maneuver_index, 0.387751579f, 22));
  //----------------------------------------------------------------
  // trace point = 1228
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5896454f, 40.1695786f, 1491515973, 27.4891396),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5896606f,
                                                40.1695709f, leg_index, 9.55291557f, 1028,
                                                maneuver_index, 0.370615005f, 21));
  //----------------------------------------------------------------
  // trace point = 1229
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5898743f, 40.1697578f, 1491515974, 27.6280155),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5898895f,
                                                40.1697464f, leg_index, 9.53577805f, 1027,
                                                maneuver_index, 0.353477478f, 20));
  //----------------------------------------------------------------
  // trace point = 1230
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5900955f, 40.1699295f, 1491515975, 27.0123425),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5901108f,
                                                40.1699181f, leg_index, 9.51911259f, 1026,
                                                maneuver_index, 0.336812019f, 19));
  //----------------------------------------------------------------
  // trace point = 1231
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5903168f, 40.1701012f, 1491515976, 27.0913277),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.590332f,
                                                40.1700897f, leg_index, 9.50244713f, 1025,
                                                maneuver_index, 0.320146561f, 18));
  //----------------------------------------------------------------
  // trace point = 1232
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5905457f, 40.1702728f, 1491515977, 26.7946339),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5905609f,
                                                40.1702614f, leg_index, 9.48549652f, 1024,
                                                maneuver_index, 0.303195953f, 17));
  //----------------------------------------------------------------
  // trace point = 1233
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5907669f, 40.1704407f, 1491515978, 26.813364),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5907822f,
                                                40.1704292f, leg_index, 9.46901798f, 1023,
                                                maneuver_index, 0.286717415f, 16));
  //----------------------------------------------------------------
  // trace point = 1234
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5909805f, 40.1706123f, 1491515979, 26.614521),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5909958f,
                                                40.1705971f, leg_index, 9.45282364f, 1022,
                                                maneuver_index, 0.270523071f, 15));
  //----------------------------------------------------------------
  // trace point = 1235
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5912018f, 40.170784f, 1491515980, 26.6341934),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5912247f,
                                                40.1707687f, leg_index, 9.43587685f, 1021,
                                                maneuver_index, 0.253576279f, 14));
  //----------------------------------------------------------------
  // trace point = 1236
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.591423f, 40.1709518f, 1491515981, 26.4754963),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5914383f,
                                                40.1709404f, leg_index, 9.41949177f, 1020,
                                                maneuver_index, 0.2371912f, 13));
  //----------------------------------------------------------------
  // trace point = 1237
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5916367f, 40.1711159f, 1491515982, 25.9798412),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5916519f,
                                                40.1711044f, leg_index, 9.4034853f, 1019,
                                                maneuver_index, 0.221184731f, 12));
  //----------------------------------------------------------------
  // trace point = 1238
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5918579f, 40.1712875f, 1491515983, 26.3560085),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5918732f,
                                                40.1712761f, leg_index, 9.38681984f, 1018,
                                                maneuver_index, 0.204519272f, 11));
  //----------------------------------------------------------------
  // trace point = 1239
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5920715f, 40.1714516f, 1491515984, 25.9616013),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5920868f,
                                                40.1714439f, leg_index, 9.37062454f, 1018,
                                                maneuver_index, 0.188323975f, 11));
  //----------------------------------------------------------------
  // trace point = 1240
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5922852f, 40.1716156f, 1491515985, 25.7807636),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5922928f,
                                                40.171608f, leg_index, 9.35489941f, 1017,
                                                maneuver_index, 0.172598839f, 10));
  //----------------------------------------------------------------
  // trace point = 1241
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5925064f, 40.1717796f, 1491515986, 26.1974564),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5925064f,
                                                40.1717758f, leg_index, 9.33870506f, 1016,
                                                maneuver_index, 0.156404495f, 9));
  //----------------------------------------------------------------
  // trace point = 1242
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.59272f, 40.1719437f, 1491515987, 25.8227558),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.59272f,
                                                40.1719437f, leg_index, 9.32251167f, 1015,
                                                maneuver_index, 0.140211105f, 8));
  //----------------------------------------------------------------
  // trace point = 1243 | Pre | Take the Pennsylvania 7 43 exit on the right toward Hershey.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5929337f, 40.1721115f, 1491515988, 25.9587898),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5929413f, 40.1721077f, leg_index, 9.3062191f,
                                                1014, maneuver_index, 0.123918533f, 7,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1244
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5931473f, 40.1722755f, 1491515989, 25.9010353),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5931549f,
                                                40.1722717f, leg_index, 9.29021263f, 1013,
                                                maneuver_index, 0.107912064f, 6));
  //----------------------------------------------------------------
  // trace point = 1245
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5933609f, 40.1724396f, 1491515990, 25.9985065),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5933685f,
                                                40.1724319f, leg_index, 9.27439213f, 1012,
                                                maneuver_index, 0.0920915604f, 5));
  //----------------------------------------------------------------
  // trace point = 1246
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5935745f, 40.1726074f, 1491515991, 25.998476),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5935898f,
                                                40.1725998f, leg_index, 9.25791359f, 1011,
                                                maneuver_index, 0.0756130219f, 4));
  //----------------------------------------------------------------
  // trace point = 1247
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5937958f, 40.1727753f, 1491515992, 26.4947624),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.593811f,
                                                40.1727676f, leg_index, 9.24143505f, 1010,
                                                maneuver_index, 0.0591344833f, 3));
  //----------------------------------------------------------------
  // trace point = 1248
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.594017f, 40.1729469f, 1491515993, 26.6337776),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5940323f,
                                                40.1729355f, leg_index, 9.22495556f, 1009,
                                                maneuver_index, 0.0426549911f, 2));
  //----------------------------------------------------------------
  // trace point = 1249
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5942383f, 40.1731186f, 1491515994, 26.5349941),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5942535f,
                                                40.1731071f, leg_index, 9.20829201f, 1008,
                                                maneuver_index, 0.0259914398f, 1));
  //----------------------------------------------------------------
  // trace point = 1250
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5944595f, 40.1732864f, 1491515995, 26.872776),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5944748f,
                                                40.173275f, leg_index, 9.19181442f, 1008,
                                                maneuver_index, 0.00951385498f, 1));
  //----------------------------------------------------------------
  // trace point = 1251
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5946808f, 40.1734657f, 1491515996, 27.2107506),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5946884f,
                                                40.1734619f, leg_index, 9.17456818f, 1006,
                                                maneuver_index, 0.460562706f, 30));
  //----------------------------------------------------------------
  // trace point = 1252
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.594902f, 40.1736412f, 1491515997, 27.2909775),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.594902f,
                                                40.1736412f, leg_index, 9.1577549f, 1005,
                                                maneuver_index, 0.443749428f, 29));
  //----------------------------------------------------------------
  // trace point = 1253
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5951233f, 40.1738205f, 1491515998, 27.5106926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5951233f,
                                                40.1738205f, leg_index, 9.14071083f, 1004,
                                                maneuver_index, 0.42670536f, 28));
  //----------------------------------------------------------------
  // trace point = 1254
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5953445f, 40.1739998f, 1491515999, 27.4301357),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5953522f,
                                                40.173996f, leg_index, 9.12357616f, 1002,
                                                maneuver_index, 0.409570694f, 26));
  //----------------------------------------------------------------
  // trace point = 1255
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5955734f, 40.1741791f, 1491516000, 27.5913601),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5955734f,
                                                40.1741753f, leg_index, 9.1065321f, 1001,
                                                maneuver_index, 0.392526627f, 25));
  //----------------------------------------------------------------
  // trace point = 1256
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5957947f, 40.1743546f, 1491516001, 27.5273438),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5957947f,
                                                40.1743507f, leg_index, 9.08967781f, 1000,
                                                maneuver_index, 0.37567234f, 24));
  //----------------------------------------------------------------
  // trace point = 1257
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5960312f, 40.1745377f, 1491516002, 28.3415203),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5960312f,
                                                40.1745377f, leg_index, 9.07169437f, 999,
                                                maneuver_index, 0.357688904f, 23));
  //----------------------------------------------------------------
  // trace point = 1258
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5962524f, 40.1747093f, 1491516003, 26.891674),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5962524f,
                                                40.1747093f, leg_index, 9.05502892f, 998,
                                                maneuver_index, 0.341023445f, 22));
  //----------------------------------------------------------------
  // trace point = 1259
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5964737f, 40.1748772f, 1491516004, 26.3772697),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5964661f,
                                                40.174881f, leg_index, 9.03864479f, 997,
                                                maneuver_index, 0.32463932f, 21));
  //----------------------------------------------------------------
  // trace point = 1260
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5966873f, 40.1750488f, 1491516005, 26.5739555),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5966873f,
                                                40.1750526f, leg_index, 9.02198029f, 996,
                                                maneuver_index, 0.307974815f, 20));
  //----------------------------------------------------------------
  // trace point = 1261
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5969086f, 40.1752129f, 1491516006, 26.039278),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5969009f,
                                                40.1752205f, leg_index, 9.00578594f, 995,
                                                maneuver_index, 0.291780472f, 19));
  //----------------------------------------------------------------
  // trace point = 1262
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5971146f, 40.1753845f, 1491516007, 26.1269913),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5971069f,
                                                40.1753883f, leg_index, 8.98987198f, 994,
                                                maneuver_index, 0.275866508f, 18));
  //----------------------------------------------------------------
  // trace point = 1263
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5973206f, 40.1755447f, 1491516008, 25.2628555),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5973129f,
                                                40.1755486f, leg_index, 8.97433662f, 993,
                                                maneuver_index, 0.260331154f, 17));
  //----------------------------------------------------------------
  // trace point = 1264
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5975266f, 40.1757126f, 1491516009, 25.3718014),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5975266f,
                                                40.1757126f, leg_index, 8.95833302f, 992,
                                                maneuver_index, 0.244327545f, 16));
  //----------------------------------------------------------------
  // trace point = 1265
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5977173f, 40.1758766f, 1491516010, 24.5777283),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5977173f,
                                                40.1758766f, leg_index, 8.94315434f, 991,
                                                maneuver_index, 0.229148865f, 15));
  //----------------------------------------------------------------
  // trace point = 1266
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.597908f, 40.1760445f, 1491516011, 24.5213261),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5979004f,
                                                40.1760483f, leg_index, 8.92782688f, 989,
                                                maneuver_index, 0.213821411f, 13));
  //----------------------------------------------------------------
  // trace point = 1267
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5980988f, 40.1762123f, 1491516012, 24.5212994),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5980911f,
                                                40.1762161f, leg_index, 8.91245174f, 988,
                                                maneuver_index, 0.198446274f, 12));
  //----------------------------------------------------------------
  // trace point = 1268
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5982742f, 40.1763802f, 1491516013, 24.5030918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5982819f,
                                                40.1763763f, leg_index, 8.89614677f, 987,
                                                maneuver_index, 0.182141304f, 11));
  //----------------------------------------------------------------
  // trace point = 1269
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5984573f, 40.1765442f, 1491516014, 23.9358749),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.598465f,
                                                40.1765404f, leg_index, 8.88256073f, 986,
                                                maneuver_index, 0.16855526f, 10));
  //----------------------------------------------------------------
  // trace point = 1270
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5986328f, 40.1767159f, 1491516015, 23.7963867),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5986481f,
                                                40.1767082f, leg_index, 8.8674469f, 985,
                                                maneuver_index, 0.153441429f, 9));
  //----------------------------------------------------------------
  // trace point = 1271
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5988007f, 40.1768761f, 1491516016, 22.9214668),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5988083f,
                                                40.1768684f, leg_index, 8.85349464f, 984,
                                                maneuver_index, 0.139489174f, 8));
  //----------------------------------------------------------------
  // trace point = 1272 | Pre | Keep right to take Pennsylvania 7 43 North toward Hershey. Then
  // Continue on Pennsylvania 7 43 North.
  maneuver_index = 6;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5989609f, 40.1770325f, 1491516017, 22.3266468),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5989609f, 40.1770325f, leg_index, 8.83957291f,
                                                983, maneuver_index, 0.125567436f, 7,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1273
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5991135f, 40.1772003f, 1491516018, 22.4124947),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5991211f,
                                                40.1771965f, leg_index, 8.82542324f, 982,
                                                maneuver_index, 0.11141777f, 6));
  //----------------------------------------------------------------
  // trace point = 1274
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5992584f, 40.1773643f, 1491516019, 22.3113804),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5992661f,
                                                40.1773605f, leg_index, 8.81173611f, 982,
                                                maneuver_index, 0.0977306366f, 6));
  //----------------------------------------------------------------
  // trace point = 1275
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5993958f, 40.1775284f, 1491516020, 21.472908),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5994034f,
                                                40.1775246f, leg_index, 8.79827118f, 981,
                                                maneuver_index, 0.0842657089f, 5));
  //----------------------------------------------------------------
  // trace point = 1276
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5995178f, 40.1776848f, 1491516021, 20.4511871),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5995255f,
                                                40.177681f, leg_index, 8.78567791f, 980,
                                                maneuver_index, 0.0716724396f, 4));
  //----------------------------------------------------------------
  // trace point = 1277
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5996323f, 40.1778336f, 1491516022, 19.159996),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5996399f,
                                                40.1778336f, leg_index, 8.77351284f, 980,
                                                maneuver_index, 0.05950737f, 4));
  //----------------------------------------------------------------
  // trace point = 1278
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5997391f, 40.1779633f, 1491516023, 16.9781914),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5997391f,
                                                40.1779633f, leg_index, 8.76312256f, 979,
                                                maneuver_index, 0.0491170883f, 3));
  //----------------------------------------------------------------
  // trace point = 1279
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.599823f, 40.1780853f, 1491516024, 15.4928083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.599823f,
                                                40.1780853f, leg_index, 8.75359154f, 979,
                                                maneuver_index, 0.0395860672f, 3));
  //----------------------------------------------------------------
  // trace point = 1280
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5999069f, 40.178196f, 1491516025, 14.0247536),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5999069f,
                                                40.178196f, leg_index, 8.74474621f, 978,
                                                maneuver_index, 0.0307407379f, 2));
  //----------------------------------------------------------------
  // trace point = 1281
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5999756f, 40.1782951f, 1491516026, 12.6912336),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5999832f,
                                                40.1782913f, leg_index, 8.73701859f, 978,
                                                maneuver_index, 0.0230131149f, 2));
  //----------------------------------------------------------------
  // trace point = 1282
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6000519f, 40.1783905f, 1491516027, 12.445715),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6000519f,
                                                40.1783905f, leg_index, 8.7292614f, 977,
                                                maneuver_index, 0.015255928f, 1));
  //----------------------------------------------------------------
  // trace point = 1283
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001129f, 40.1784782f, 1491516028, 11.0851822),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001205f,
                                                40.1784744f, leg_index, 8.7224226f, 977,
                                                maneuver_index, 0.00841712952f, 1));
  //----------------------------------------------------------------
  // trace point = 1284
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001816f, 40.1785622f, 1491516029, 10.7237463),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001816f,
                                                40.1785622f, leg_index, 8.71400547f, 976,
                                                maneuver_index, 0.0673074722f, 3));
  //----------------------------------------------------------------
  // trace point = 1285
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6002426f, 40.1786461f, 1491516030, 10.8323326),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6002426f,
                                                40.1786461f, leg_index, 8.7089138f, 975,
                                                maneuver_index, 0.0622158051f, 2));
  //----------------------------------------------------------------
  // trace point = 1286 | Pre | Continue on Pennsylvania 7 43 North for 4.9 miles.
  maneuver_index = 7;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6003036f, 40.17873f, 1491516031, 10.7237387),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6003113f, 40.17873f, leg_index, 8.7020731f, 975,
                                                maneuver_index, 0.0553750992f, 2, instruction_index));
  //----------------------------------------------------------------
  // trace point = 1287
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6003571f, 40.1788101f, 1491516032, 9.90576553),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6003647f,
                                                40.1788063f, leg_index, 8.69608974f, 974,
                                                maneuver_index, 0.0493917465f, 1));
  //----------------------------------------------------------------
  // trace point = 1288
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.600441f, 40.1789436f, 1491516034, 8.25144005),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.600441f,
                                                40.1789436f, leg_index, 8.68435287f, 974,
                                                maneuver_index, 0.0376548767f, 1));
  //----------------------------------------------------------------
  // trace point = 1289
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004791f, 40.1790543f, 1491516036, 6.43122482),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6004562f,
                                                40.1790543f, leg_index, 8.67800331f, 974,
                                                maneuver_index, 0.0313053131f, 1));
  //----------------------------------------------------------------
  // trace point = 1290
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004791f, 40.1791534f, 1491516038, 5.45482111),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6004639f,
                                                40.1791534f, leg_index, 8.67113972f, 974,
                                                maneuver_index, 0.0244417191f, 1));
  //----------------------------------------------------------------
  // trace point = 1291
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004639f, 40.179287f, 1491516040, 7.48937702),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.600441f,
                                                40.179287f, leg_index, 8.66182423f, 974,
                                                maneuver_index, 0.0151262283f, 1));
  //----------------------------------------------------------------
  // trace point = 1292
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004486f, 40.1793709f, 1491516041, 9.32717896),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6004333f,
                                                40.1793709f, leg_index, 8.65601254f, 974,
                                                maneuver_index, 0.00931453705f, 1));
  //----------------------------------------------------------------
  // trace point = 1293
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004257f, 40.1794624f, 1491516042, 10.2863235),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6004181f,
                                                40.1794624f, leg_index, 8.64962864f, 973,
                                                maneuver_index, 0.00293064117f, 0));
  //----------------------------------------------------------------
  // trace point = 1294
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004028f, 40.1795616f, 1491516043, 11.4270592),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6003952f,
                                                40.1795616f, leg_index, 8.64265919f, 973,
                                                maneuver_index, 4.88130474f, 521));
  //----------------------------------------------------------------
  // trace point = 1295
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6003799f, 40.1796761f, 1491516044, 12.8816442),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6003647f,
                                                40.1796722f, leg_index, 8.63483906f, 972,
                                                maneuver_index, 4.87348461f, 520));
  //----------------------------------------------------------------
  // trace point = 1296
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6003494f, 40.1798058f, 1491516045, 14.4607916),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6003342f,
                                                40.1798019f, leg_index, 8.6257267f, 971,
                                                maneuver_index, 4.86437225f, 519));
  //----------------------------------------------------------------
  // trace point = 1297
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6003113f, 40.1799355f, 1491516046, 15.1017809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.600296f,
                                                40.1799316f, leg_index, 8.61653042f, 971,
                                                maneuver_index, 4.85517597f, 519));
  //----------------------------------------------------------------
  // trace point = 1298
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6002808f, 40.180088f, 1491516047, 17.210701),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6002502f,
                                                40.1800842f, leg_index, 8.60569859f, 969,
                                                maneuver_index, 4.84434414f, 517));
  //----------------------------------------------------------------
  // trace point = 1299
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6002502f, 40.1802521f, 1491516048, 18.3119431),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6002197f,
                                                40.1802521f, leg_index, 8.59396458f, 968,
                                                maneuver_index, 4.83261013f, 516));
  //----------------------------------------------------------------
  // trace point = 1300
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6002197f, 40.1804199f, 1491516049, 18.9425068),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6002045f,
                                                40.1804199f, leg_index, 8.58233261f, 966,
                                                maneuver_index, 4.82097816f, 514));
  //----------------------------------------------------------------
  // trace point = 1301
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001892f, 40.1806068f, 1491516050, 20.8204651),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001892f,
                                                40.1806068f, leg_index, 8.56936741f, 965,
                                                maneuver_index, 4.80801296f, 513));
  //----------------------------------------------------------------
  // trace point = 1302
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.600174f, 40.1807938f, 1491516051, 20.9572506),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.600174f,
                                                40.1807938f, leg_index, 8.55641937f, 963,
                                                maneuver_index, 4.79506493f, 511));
  //----------------------------------------------------------------
  // trace point = 1303
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001663f, 40.1809731f, 1491516052, 20.049057),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.600174f,
                                                40.1809731f, leg_index, 8.54402065f, 962,
                                                maneuver_index, 4.78266621f, 510));
  //----------------------------------------------------------------
  // trace point = 1304
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001587f, 40.18116f, 1491516053, 20.5969162),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001816f,
                                                40.18116f, leg_index, 8.53108311f, 961,
                                                maneuver_index, 4.76972866f, 509));
  //----------------------------------------------------------------
  // trace point = 1305
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001663f, 40.1813393f, 1491516054, 20.150444),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001816f,
                                                40.1813393f, leg_index, 8.51868153f, 960,
                                                maneuver_index, 4.75732708f, 508));
  //----------------------------------------------------------------
  // trace point = 1306
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001663f, 40.1815262f, 1491516055, 20.5948086),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001816f,
                                                40.1815262f, leg_index, 8.50575447f, 959,
                                                maneuver_index, 4.74440002f, 507));
  //----------------------------------------------------------------
  // trace point = 1307
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001663f, 40.1817093f, 1491516056, 20.491436),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001892f,
                                                40.1817093f, leg_index, 8.493083f, 957,
                                                maneuver_index, 4.73172855f, 505));
  //----------------------------------------------------------------
  // trace point = 1308
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.600174f, 40.1818886f, 1491516057, 20.1490078),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001968f,
                                                40.1818886f, leg_index, 8.48067474f, 956,
                                                maneuver_index, 4.7193203f, 504));
  //----------------------------------------------------------------
  // trace point = 1309
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001892f, 40.1820679f, 1491516058, 19.9849072),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6002121f,
                                                40.1820679f, leg_index, 8.46824837f, 955,
                                                maneuver_index, 4.70689392f, 503));
  //----------------------------------------------------------------
  // trace point = 1310
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6002121f, 40.1822433f, 1491516059, 19.3409119),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.600235f,
                                                40.1822433f, leg_index, 8.45605087f, 954,
                                                maneuver_index, 4.69469643f, 502));
  //----------------------------------------------------------------
  // trace point = 1311
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.600235f, 40.1824265f, 1491516060, 20.5101776),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6002579f,
                                                40.1824265f, leg_index, 8.44332981f, 952,
                                                maneuver_index, 4.68197536f, 500));
  //----------------------------------------------------------------
  // trace point = 1312
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6002655f, 40.1826019f, 1491516061, 19.9784584),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6002884f,
                                                40.1826019f, leg_index, 8.43108559f, 951,
                                                maneuver_index, 4.66973114f, 499));
  //----------------------------------------------------------------
  // trace point = 1313
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6003036f, 40.1827812f, 1491516062, 20.2700691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6003113f,
                                                40.1827812f, leg_index, 8.41863251f, 949,
                                                maneuver_index, 4.65727806f, 497));
  //----------------------------------------------------------------
  // trace point = 1314
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6003647f, 40.1829529f, 1491516063, 19.4275227),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6003494f,
                                                40.1829529f, leg_index, 8.40659142f, 948,
                                                maneuver_index, 4.64523697f, 496));
  //----------------------------------------------------------------
  // trace point = 1315
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004105f, 40.1831284f, 1491516064, 20.0483875),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6004028f,
                                                40.1831284f, leg_index, 8.39335823f, 947,
                                                maneuver_index, 4.63200378f, 495));
  //----------------------------------------------------------------
  // trace point = 1316
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004639f, 40.1833038f, 1491516065, 20.034605),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6004562f,
                                                40.1833038f, leg_index, 8.38166809f, 946,
                                                maneuver_index, 4.62031364f, 494));
  //----------------------------------------------------------------
  // trace point = 1317
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6005173f, 40.1834831f, 1491516066, 20.6655827),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6005173f,
                                                40.1834831f, leg_index, 8.36885452f, 945,
                                                maneuver_index, 4.60750008f, 493));
  //----------------------------------------------------------------
  // trace point = 1318
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6005783f, 40.1836624f, 1491516067, 20.6981907),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6005859f,
                                                40.1836624f, leg_index, 8.35593891f, 943,
                                                maneuver_index, 4.59458447f, 491));
  //----------------------------------------------------------------
  // trace point = 1319
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6006546f, 40.1838493f, 1491516068, 21.5679188),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6006546f,
                                                40.1838493f, leg_index, 8.34250355f, 942,
                                                maneuver_index, 4.5811491f, 490));
  //----------------------------------------------------------------
  // trace point = 1320
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6007233f, 40.1840324f, 1491516069, 21.2404747),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6007309f,
                                                40.1840286f, leg_index, 8.32946491f, 941,
                                                maneuver_index, 4.56811047f, 489));
  //----------------------------------------------------------------
  // trace point = 1321
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6008148f, 40.1841965f, 1491516070, 19.6354866),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6008148f,
                                                40.1841965f, leg_index, 8.31703472f, 939,
                                                maneuver_index, 4.55568027f, 487));
  //----------------------------------------------------------------
  // trace point = 1322
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6008987f, 40.1843719f, 1491516071, 20.8086205),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6009064f,
                                                40.1843681f, leg_index, 8.30420971f, 938,
                                                maneuver_index, 4.54285526f, 486));
  //----------------------------------------------------------------
  // trace point = 1323
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6009903f, 40.1845436f, 1491516072, 20.7235451),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6010056f,
                                                40.1845398f, leg_index, 8.29123211f, 936,
                                                maneuver_index, 4.52987766f, 484));
  //----------------------------------------------------------------
  // trace point = 1324
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6010818f, 40.1847153f, 1491516073, 20.9173756),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6011047f,
                                                40.1847076f, leg_index, 8.27849102f, 935,
                                                maneuver_index, 4.51713657f, 483));
  //----------------------------------------------------------------
  // trace point = 1325
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.601181f, 40.1848793f, 1491516074, 19.7655067),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6011963f,
                                                40.1848755f, leg_index, 8.26591682f, 934,
                                                maneuver_index, 4.50456238f, 482));
  //----------------------------------------------------------------
  // trace point = 1326
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6012878f, 40.1850395f, 1491516075, 20.2233124),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6012955f,
                                                40.1850395f, leg_index, 8.25341988f, 932,
                                                maneuver_index, 4.49206543f, 480));
  //----------------------------------------------------------------
  // trace point = 1327
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.601387f, 40.1852074f, 1491516076, 20.2367306),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6013947f,
                                                40.1852074f, leg_index, 8.2406826f, 931,
                                                maneuver_index, 4.47932816f, 479));
  //----------------------------------------------------------------
  // trace point = 1328
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6014786f, 40.1853714f, 1491516077, 19.8280048),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6014862f,
                                                40.1853676f, leg_index, 8.22858906f, 930,
                                                maneuver_index, 4.46723461f, 478));
  //----------------------------------------------------------------
  // trace point = 1329
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6015778f, 40.1855316f, 1491516078, 19.9748421),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6015778f,
                                                40.1855316f, leg_index, 8.2162571f, 928,
                                                maneuver_index, 4.45490265f, 476));
  //----------------------------------------------------------------
  // trace point = 1330
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6016769f, 40.1856956f, 1491516079, 19.9322186),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6016769f,
                                                40.1856956f, leg_index, 8.20375919f, 927,
                                                maneuver_index, 4.44240475f, 475));
  //----------------------------------------------------------------
  // trace point = 1331
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6017761f, 40.1858559f, 1491516080, 19.6286812),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6017609f,
                                                40.1858597f, leg_index, 8.19157887f, 925,
                                                maneuver_index, 4.43022442f, 473));
  //----------------------------------------------------------------
  // trace point = 1332
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6018677f, 40.1860123f, 1491516081, 19.1892014),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6018372f,
                                                40.1860199f, leg_index, 8.17978573f, 924,
                                                maneuver_index, 4.41843128f, 472));
  //----------------------------------------------------------------
  // trace point = 1333
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6019669f, 40.1861687f, 1491516082, 19.355257),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6019135f,
                                                40.1861839f, leg_index, 8.1677475f, 923,
                                                maneuver_index, 4.40639305f, 471));
  //----------------------------------------------------------------
  // trace point = 1334
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6020584f, 40.1863251f, 1491516083, 19.0163784),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6019974f,
                                                40.1863441f, leg_index, 8.15581322f, 922,
                                                maneuver_index, 4.39445877f, 470));
  //----------------------------------------------------------------
  // trace point = 1335
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.60215f, 40.1864777f, 1491516084, 18.8441734),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6020737f,
                                                40.1865005f, leg_index, 8.14426804f, 920,
                                                maneuver_index, 4.38291359f, 468));
  //----------------------------------------------------------------
  // trace point = 1336
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6022415f, 40.1866302f, 1491516085, 18.5362225),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.60215f,
                                                40.186657f, leg_index, 8.13272285f, 919,
                                                maneuver_index, 4.37136841f, 467));
  //----------------------------------------------------------------
  // trace point = 1337
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6023331f, 40.1867828f, 1491516086, 18.8796253),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6022339f,
                                                40.186821f, leg_index, 8.12053967f, 917,
                                                maneuver_index, 4.35918522f, 465));
  //----------------------------------------------------------------
  // trace point = 1338
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6024246f, 40.1869354f, 1491516087, 18.8508587),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6023331f,
                                                40.1869698f, leg_index, 8.10899258f, 916,
                                                maneuver_index, 4.34763813f, 464));
  //----------------------------------------------------------------
  // trace point = 1339
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6025238f, 40.1870918f, 1491516088, 18.8874531),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6024323f,
                                                40.1871262f, leg_index, 8.09697151f, 915,
                                                maneuver_index, 4.33561707f, 463));
  //----------------------------------------------------------------
  // trace point = 1340
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6026154f, 40.1872444f, 1491516089, 18.9878902),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6025314f,
                                                40.1872787f, leg_index, 8.08518791f, 914,
                                                maneuver_index, 4.32383347f, 462));
  //----------------------------------------------------------------
  // trace point = 1341
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6027145f, 40.1874046f, 1491516090, 19.4918747),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6026306f,
                                                40.1874352f, leg_index, 8.07316685f, 912,
                                                maneuver_index, 4.3118124f, 460));
  //----------------------------------------------------------------
  // trace point = 1342
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6028137f, 40.1875648f, 1491516091, 19.8663635),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6027298f,
                                                40.1875954f, leg_index, 8.06090832f, 911,
                                                maneuver_index, 4.29955387f, 459));
  //----------------------------------------------------------------
  // trace point = 1343
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6029129f, 40.1877327f, 1491516092, 20.3419342),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6028366f,
                                                40.1877594f, leg_index, 8.04823589f, 910,
                                                maneuver_index, 4.28688145f, 458));
  //----------------------------------------------------------------
  // trace point = 1344
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6030121f, 40.1879005f, 1491516093, 20.7822609),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6029434f,
                                                40.1879272f, leg_index, 8.03532696f, 908,
                                                maneuver_index, 4.27397251f, 456));
  //----------------------------------------------------------------
  // trace point = 1345
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6031189f, 40.1880722f, 1491516094, 20.9615612),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6030579f,
                                                40.1880951f, leg_index, 8.02224064f, 907,
                                                maneuver_index, 4.26088619f, 455));
  //----------------------------------------------------------------
  // trace point = 1346
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6032257f, 40.1882401f, 1491516095, 20.7607708),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6031647f,
                                                40.1882629f, leg_index, 8.00933075f, 905,
                                                maneuver_index, 4.2479763f, 453));
  //----------------------------------------------------------------
  // trace point = 1347
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6033325f, 40.1884041f, 1491516096, 20.6355648),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6032639f,
                                                40.188427f, leg_index, 7.99683619f, 904,
                                                maneuver_index, 4.23548174f, 452));
  //----------------------------------------------------------------
  // trace point = 1348
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6034317f, 40.1885681f, 1491516097, 20.0030594),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6033707f,
                                                40.188591f, leg_index, 7.98416376f, 903,
                                                maneuver_index, 4.22280931f, 451));
  //----------------------------------------------------------------
  // trace point = 1349
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6035309f, 40.1887321f, 1491516098, 20.065876),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6034775f,
                                                40.1887512f, leg_index, 7.9717288f, 901,
                                                maneuver_index, 4.21037436f, 449));
  //----------------------------------------------------------------
  // trace point = 1350
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6036224f, 40.1888962f, 1491516099, 19.7955494),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6035767f,
                                                40.1889114f, leg_index, 7.95947075f, 900,
                                                maneuver_index, 4.1981163f, 448));
  //----------------------------------------------------------------
  // trace point = 1351
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.603714f, 40.1890526f, 1491516100, 19.1483669),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6036758f,
                                                40.1890678f, leg_index, 7.94744968f, 899,
                                                maneuver_index, 4.18609524f, 447));
  //----------------------------------------------------------------
  // trace point = 1352
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6038055f, 40.1892052f, 1491516101, 18.6061974),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.603775f,
                                                40.1892166f, leg_index, 7.9359026f, 898,
                                                maneuver_index, 4.17454815f, 446));
  //----------------------------------------------------------------
  // trace point = 1353
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6038971f, 40.1893501f, 1491516102, 18.1748581),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6038666f,
                                                40.1893616f, leg_index, 7.92476988f, 896,
                                                maneuver_index, 4.16341543f, 444));
  //----------------------------------------------------------------
  // trace point = 1354
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6039886f, 40.1894989f, 1491516103, 17.8573303),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6039581f,
                                                40.1895103f, leg_index, 7.91186047f, 895,
                                                maneuver_index, 4.15050602f, 443));
  //----------------------------------------------------------------
  // trace point = 1355
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6040726f, 40.18964f, 1491516104, 17.5182495),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6040497f,
                                                40.1896477f, leg_index, 7.90274525f, 894,
                                                maneuver_index, 4.1413908f, 442));
  //----------------------------------------------------------------
  // trace point = 1356
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6041641f, 40.189785f, 1491516105, 17.9746418),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6041336f,
                                                40.1897964f, leg_index, 7.89154196f, 892,
                                                maneuver_index, 4.13018751f, 440));
  //----------------------------------------------------------------
  // trace point = 1357
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6042557f, 40.18993f, 1491516106, 17.7560539),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6042252f,
                                                40.1899414f, leg_index, 7.88041067f, 891,
                                                maneuver_index, 4.11905622f, 439));
  //----------------------------------------------------------------
  // trace point = 1358
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6043396f, 40.1900749f, 1491516107, 17.6512756),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6043091f,
                                                40.1900864f, leg_index, 7.86944818f, 890,
                                                maneuver_index, 4.10809374f, 438));
  //----------------------------------------------------------------
  // trace point = 1359
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6044235f, 40.1902237f, 1491516108, 18.0294037),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6044006f,
                                                40.1902313f, leg_index, 7.8583169f, 889,
                                                maneuver_index, 4.09696245f, 437));
  //----------------------------------------------------------------
  // trace point = 1360
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6045227f, 40.1903725f, 1491516109, 18.5126228),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6044922f,
                                                40.1903839f, leg_index, 7.84670639f, 888,
                                                maneuver_index, 4.08535194f, 436));
  //----------------------------------------------------------------
  // trace point = 1361
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6046143f, 40.1905289f, 1491516110, 19.2184124),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6045837f,
                                                40.1905403f, leg_index, 7.83485651f, 886,
                                                maneuver_index, 4.07350206f, 434));
  //----------------------------------------------------------------
  // trace point = 1362
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6047058f, 40.1906891f, 1491516111, 19.4535809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6046829f,
                                                40.1906967f, leg_index, 7.82283783f, 885,
                                                maneuver_index, 4.06148338f, 433));
  //----------------------------------------------------------------
  // trace point = 1363
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.604805f, 40.1908531f, 1491516112, 20.0388794),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6047821f,
                                                40.1908607f, leg_index, 7.81033945f, 884,
                                                maneuver_index, 4.048985f, 432));
  //----------------------------------------------------------------
  // trace point = 1364
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6049042f, 40.1910172f, 1491516113, 20.2019291),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6048813f,
                                                40.1910248f, leg_index, 7.79784155f, 882,
                                                maneuver_index, 4.0364871f, 430));
  //----------------------------------------------------------------
  // trace point = 1365
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6050034f, 40.1911888f, 1491516114, 20.8173771),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6049805f,
                                                40.1911964f, leg_index, 7.78486395f, 881,
                                                maneuver_index, 4.0235095f, 429));
  //----------------------------------------------------------------
  // trace point = 1366
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6051102f, 40.1913605f, 1491516115, 21.2355881),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6050873f,
                                                40.1913681f, leg_index, 7.77171803f, 879,
                                                maneuver_index, 4.01036358f, 427));
  //----------------------------------------------------------------
  // trace point = 1367
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.605217f, 40.1915321f, 1491516116, 20.9186058),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6051941f,
                                                40.1915398f, leg_index, 7.75857162f, 878,
                                                maneuver_index, 3.99721718f, 426));
  //----------------------------------------------------------------
  // trace point = 1368
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6053238f, 40.1917038f, 1491516117, 21.1625729),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6052933f,
                                                40.1917152f, leg_index, 7.74535465f, 877,
                                                maneuver_index, 3.98400021f, 425));
  //----------------------------------------------------------------
  // trace point = 1369
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6054306f, 40.1918793f, 1491516118, 21.8396931),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6054001f,
                                                40.1918907f, leg_index, 7.73196888f, 875,
                                                maneuver_index, 3.97061443f, 423));
  //----------------------------------------------------------------
  // trace point = 1370
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6055374f, 40.1920586f, 1491516119, 21.940731),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6055069f,
                                                40.19207f, leg_index, 7.71834373f, 874,
                                                maneuver_index, 3.95698929f, 422));
  //----------------------------------------------------------------
  // trace point = 1371
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6056442f, 40.1922455f, 1491516120, 22.5529938),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6056213f,
                                                40.1922531f, leg_index, 7.70430994f, 872,
                                                maneuver_index, 3.94295549f, 420));
  //----------------------------------------------------------------
  // trace point = 1372
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6057587f, 40.1924286f, 1491516121, 22.7616043),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6057358f,
                                                40.1924362f, leg_index, 7.69027615f, 871,
                                                maneuver_index, 3.9289217f, 419));
  //----------------------------------------------------------------
  // trace point = 1373
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6058807f, 40.1926231f, 1491516122, 23.6408501),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6058502f,
                                                40.1926346f, leg_index, 7.67528391f, 869,
                                                maneuver_index, 3.91392946f, 417));
  //----------------------------------------------------------------
  // trace point = 1374
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6059952f, 40.1928177f, 1491516123, 24.0509567),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6059647f,
                                                40.1928291f, leg_index, 7.66053152f, 868,
                                                maneuver_index, 3.89917707f, 416));
  //----------------------------------------------------------------
  // trace point = 1375
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6061172f, 40.1930161f, 1491516124, 24.4614048),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6060867f,
                                                40.1930275f, leg_index, 7.64537048f, 866,
                                                maneuver_index, 3.88401604f, 414));
  //----------------------------------------------------------------
  // trace point = 1376
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6062393f, 40.1932144f, 1491516125, 24.3190899),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6062088f,
                                                40.1932259f, leg_index, 7.63020945f, 864,
                                                maneuver_index, 3.868855f, 412));
  //----------------------------------------------------------------
  // trace point = 1377
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6063538f, 40.1934128f, 1491516126, 24.0461864),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6063232f,
                                                40.1934204f, leg_index, 7.61545658f, 863,
                                                maneuver_index, 3.85410213f, 411));
  //----------------------------------------------------------------
  // trace point = 1378
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6064682f, 40.1936073f, 1491516127, 23.7774563),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6064453f,
                                                40.193615f, leg_index, 7.60053539f, 861,
                                                maneuver_index, 3.83918095f, 409));
  //----------------------------------------------------------------
  // trace point = 1379
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6065903f, 40.1938019f, 1491516128, 24.0508957),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6065598f,
                                                40.1938133f, leg_index, 7.58554316f, 860,
                                                maneuver_index, 3.82418871f, 408));
  //----------------------------------------------------------------
  // trace point = 1380
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6067047f, 40.1940002f, 1491516129, 24.0152302),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6066818f,
                                                40.1940079f, leg_index, 7.57062197f, 858,
                                                maneuver_index, 3.80926752f, 406));
  //----------------------------------------------------------------
  // trace point = 1381
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6068268f, 40.1941948f, 1491516130, 23.77742),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6067963f,
                                                40.1942062f, leg_index, 7.55562973f, 856,
                                                maneuver_index, 3.79427528f, 404));
  //----------------------------------------------------------------
  // trace point = 1382
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6069336f, 40.1943817f, 1491516131, 23.2662907),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6069107f,
                                                40.1943893f, leg_index, 7.54159594f, 855,
                                                maneuver_index, 3.78024149f, 403));
  //----------------------------------------------------------------
  // trace point = 1383
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.607048f, 40.1945686f, 1491516132, 22.78689),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6070251f,
                                                40.1945763f, leg_index, 7.52732277f, 853,
                                                maneuver_index, 3.76596832f, 401));
  //----------------------------------------------------------------
  // trace point = 1384
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6071472f, 40.1947556f, 1491516133, 22.3791218),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.607132f,
                                                40.1947594f, leg_index, 7.51345778f, 852,
                                                maneuver_index, 3.75210333f, 400));
  //----------------------------------------------------------------
  // trace point = 1385
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.607254f, 40.1949348f, 1491516134, 21.8679962),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6072464f,
                                                40.1949387f, leg_index, 7.49966383f, 850,
                                                maneuver_index, 3.73830938f, 398));
  //----------------------------------------------------------------
  // trace point = 1386
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6073608f, 40.1951103f, 1491516135, 21.7104359),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6073456f,
                                                40.1951141f, leg_index, 7.48644686f, 849,
                                                maneuver_index, 3.72509241f, 397));
  //----------------------------------------------------------------
  // trace point = 1387
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6074677f, 40.1952858f, 1491516136, 21.3640156),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6074524f,
                                                40.1952896f, leg_index, 7.47306156f, 848,
                                                maneuver_index, 3.71170712f, 396));
  //----------------------------------------------------------------
  // trace point = 1388
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6075745f, 40.1954613f, 1491516137, 21.6320133),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6075592f,
                                                40.1954651f, leg_index, 7.45967579f, 846,
                                                maneuver_index, 3.69832134f, 394));
  //----------------------------------------------------------------
  // trace point = 1389
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6076813f, 40.1956367f, 1491516138, 21.4649754),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.607666f,
                                                40.1956406f, leg_index, 7.44629002f, 845,
                                                maneuver_index, 3.68493557f, 393));
  //----------------------------------------------------------------
  // trace point = 1390
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6077881f, 40.1958122f, 1491516139, 21.8475056),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6077728f,
                                                40.195816f, leg_index, 7.43290424f, 843,
                                                maneuver_index, 3.6715498f, 391));
  //----------------------------------------------------------------
  // trace point = 1391
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6079025f, 40.1959953f, 1491516140, 22.1491508),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6078873f,
                                                40.196003f, leg_index, 7.41863155f, 842,
                                                maneuver_index, 3.65727711f, 390));
  //----------------------------------------------------------------
  // trace point = 1392
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6080093f, 40.1961708f, 1491516141, 21.9115009),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6079941f,
                                                40.1961784f, leg_index, 7.40524864f, 840,
                                                maneuver_index, 3.6438942f, 388));
  //----------------------------------------------------------------
  // trace point = 1393
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6081161f, 40.1963463f, 1491516142, 21.394083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6081009f,
                                                40.1963539f, leg_index, 7.39185905f, 839,
                                                maneuver_index, 3.63050461f, 387));
  //----------------------------------------------------------------
  // trace point = 1394
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.608223f, 40.1965218f, 1491516143, 21.4293594),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6082077f,
                                                40.1965294f, leg_index, 7.37847376f, 837,
                                                maneuver_index, 3.61711931f, 385));
  //----------------------------------------------------------------
  // trace point = 1395
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6083298f, 40.1967049f, 1491516144, 22.0728779),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6083145f,
                                                40.1967087f, leg_index, 7.36484957f, 836,
                                                maneuver_index, 3.60349512f, 384));
  //----------------------------------------------------------------
  // trace point = 1396
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6084366f, 40.1968803f, 1491516145, 21.9761829),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.608429f,
                                                40.1968842f, leg_index, 7.35129261f, 835,
                                                maneuver_index, 3.58993816f, 383));
  //----------------------------------------------------------------
  // trace point = 1397
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6085434f, 40.1970634f, 1491516146, 22.3456249),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6085358f,
                                                40.1970673f, leg_index, 7.33742952f, 833,
                                                maneuver_index, 3.57607508f, 381));
  //----------------------------------------------------------------
  // trace point = 1398
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6086502f, 40.1972504f, 1491516147, 22.685091),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6086502f,
                                                40.1972504f, leg_index, 7.32339478f, 832,
                                                maneuver_index, 3.56204033f, 380));
  //----------------------------------------------------------------
  // trace point = 1399
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6087646f, 40.1974335f, 1491516148, 22.4875145),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6087646f,
                                                40.1974335f, leg_index, 7.30936003f, 830,
                                                maneuver_index, 3.54800558f, 378));
  //----------------------------------------------------------------
  // trace point = 1400
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6088791f, 40.1976166f, 1491516149, 22.4230118),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6088791f,
                                                40.1976166f, leg_index, 7.29532576f, 829,
                                                maneuver_index, 3.53397131f, 377));
  //----------------------------------------------------------------
  // trace point = 1401
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6089859f, 40.1977921f, 1491516150, 21.8107891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6089859f,
                                                40.1977921f, leg_index, 7.28193998f, 827,
                                                maneuver_index, 3.52058554f, 375));
  //----------------------------------------------------------------
  // trace point = 1402
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6091003f, 40.1979675f, 1491516151, 21.5095615),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6090927f,
                                                40.1979713f, leg_index, 7.26831627f, 826,
                                                maneuver_index, 3.50696182f, 374));
  //----------------------------------------------------------------
  // trace point = 1403
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6092072f, 40.1981468f, 1491516152, 22.2497807),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6091995f,
                                                40.1981468f, leg_index, 7.25493002f, 824,
                                                maneuver_index, 3.49357557f, 372));
  //----------------------------------------------------------------
  // trace point = 1404
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6093216f, 40.1983261f, 1491516153, 22.1490135),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.609314f,
                                                40.1983299f, leg_index, 7.24089575f, 823,
                                                maneuver_index, 3.4795413f, 371));
  //----------------------------------------------------------------
  // trace point = 1405
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6094208f, 40.1985092f, 1491516154, 22.0381985),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6094208f,
                                                40.1985092f, leg_index, 7.22727108f, 821,
                                                maneuver_index, 3.46591663f, 369));
  //----------------------------------------------------------------
  // trace point = 1406
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6095276f, 40.1986961f, 1491516155, 22.8221645),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6095352f,
                                                40.1986923f, leg_index, 7.21323681f, 820,
                                                maneuver_index, 3.45188236f, 368));
  //----------------------------------------------------------------
  // trace point = 1407
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6096268f, 40.1988869f, 1491516156, 22.7245369),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6096497f,
                                                40.1988792f, leg_index, 7.19896364f, 818,
                                                maneuver_index, 3.4376092f, 366));
  //----------------------------------------------------------------
  // trace point = 1408
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6097412f, 40.1990662f, 1491516157, 22.233614),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6097565f,
                                                40.1990585f, leg_index, 7.18533897f, 817,
                                                maneuver_index, 3.42398453f, 365));
  //----------------------------------------------------------------
  // trace point = 1409
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6098557f, 40.1992531f, 1491516158, 22.7549839),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6098709f,
                                                40.1992455f, leg_index, 7.17106581f, 815,
                                                maneuver_index, 3.40971136f, 363));
  //----------------------------------------------------------------
  // trace point = 1410
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6099701f, 40.1994362f, 1491516159, 22.7250996),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6099854f,
                                                40.1994324f, leg_index, 7.15679359f, 814,
                                                maneuver_index, 3.39543915f, 362));
  //----------------------------------------------------------------
  // trace point = 1411
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6100845f, 40.1996231f, 1491516160, 22.927042),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6100922f,
                                                40.1996231f, leg_index, 7.14244413f, 812,
                                                maneuver_index, 3.38108969f, 360));
  //----------------------------------------------------------------
  // trace point = 1412
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6101913f, 40.1998138f, 1491516161, 23.0933514),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.610199f,
                                                40.1998138f, leg_index, 7.12809467f, 811,
                                                maneuver_index, 3.36674023f, 359));
  //----------------------------------------------------------------
  // trace point = 1413
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6102982f, 40.2000046f, 1491516162, 23.0920753),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6103058f,
                                                40.2000008f, leg_index, 7.1139884f, 809,
                                                maneuver_index, 3.35263395f, 357));
  //----------------------------------------------------------------
  // trace point = 1414
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.610405f, 40.2001953f, 1491516163, 23.0920658),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6104126f,
                                                40.2001953f, leg_index, 7.09939575f, 808,
                                                maneuver_index, 3.33804131f, 356));
  //----------------------------------------------------------------
  // trace point = 1415
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6105042f, 40.200386f, 1491516164, 22.930748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6105118f,
                                                40.2003822f, leg_index, 7.0854454f, 806,
                                                maneuver_index, 3.32409096f, 354));
  //----------------------------------------------------------------
  // trace point = 1416
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6106033f, 40.2005806f, 1491516165, 23.1172237),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.610611f,
                                                40.2005768f, leg_index, 7.07098818f, 805,
                                                maneuver_index, 3.30963373f, 353));
  //----------------------------------------------------------------
  // trace point = 1417
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6106949f, 40.2007751f, 1491516166, 23.2548275),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6106949f,
                                                40.2007751f, leg_index, 7.05656862f, 803,
                                                maneuver_index, 3.29521418f, 351));
  //----------------------------------------------------------------
  // trace point = 1418
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6107712f, 40.2009697f, 1491516167, 22.6164417),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6107712f,
                                                40.2009697f, leg_index, 7.04252386f, 802,
                                                maneuver_index, 3.28116941f, 350));
  //----------------------------------------------------------------
  // trace point = 1419
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6108551f, 40.2011642f, 1491516168, 22.7197628),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6108551f,
                                                40.2011642f, leg_index, 7.02835512f, 800,
                                                maneuver_index, 3.26700068f, 348));
  //----------------------------------------------------------------
  // trace point = 1420
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.610939f, 40.2013664f, 1491516169, 23.4682941),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6109314f,
                                                40.2013664f, leg_index, 7.01379919f, 798,
                                                maneuver_index, 3.25244474f, 346));
  //----------------------------------------------------------------
  // trace point = 1421
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6110153f, 40.2015724f, 1491516170, 23.8715191),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6110153f,
                                                40.2015724f, leg_index, 6.99887896f, 797,
                                                maneuver_index, 3.23752451f, 345));
  //----------------------------------------------------------------
  // trace point = 1422
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6110916f, 40.2017822f, 1491516171, 24.3837242),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6110916f,
                                                40.2017822f, leg_index, 6.98381805f, 795,
                                                maneuver_index, 3.22246361f, 343));
  //----------------------------------------------------------------
  // trace point = 1423
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6111679f, 40.2019958f, 1491516172, 24.3610287),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6111679f,
                                                40.2019958f, leg_index, 6.968503f, 793,
                                                maneuver_index, 3.20714855f, 341));
  //----------------------------------------------------------------
  // trace point = 1424
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6112442f, 40.2022018f, 1491516173, 23.7792645),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6112366f,
                                                40.2022018f, leg_index, 6.95379972f, 792,
                                                maneuver_index, 3.19244528f, 340));
  //----------------------------------------------------------------
  // trace point = 1425
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6113129f, 40.2024117f, 1491516174, 24.2306995),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6113052f,
                                                40.2024117f, leg_index, 6.93883991f, 790,
                                                maneuver_index, 3.17748547f, 338));
  //----------------------------------------------------------------
  // trace point = 1426
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6113892f, 40.2026176f, 1491516175, 23.9091606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6113739f,
                                                40.2026215f, leg_index, 6.92388058f, 789,
                                                maneuver_index, 3.16252613f, 337));
  //----------------------------------------------------------------
  // trace point = 1427
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6114578f, 40.202816f, 1491516176, 22.8306351),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6114426f,
                                                40.2028198f, leg_index, 6.90968943f, 787,
                                                maneuver_index, 3.14833498f, 335));
  //----------------------------------------------------------------
  // trace point = 1428
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6115189f, 40.2030106f, 1491516177, 22.2053795),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6115036f,
                                                40.2030144f, leg_index, 6.89585209f, 786,
                                                maneuver_index, 3.13449764f, 334));
  //----------------------------------------------------------------
  // trace point = 1429
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6115875f, 40.2031975f, 1491516178, 21.7347527),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6115646f,
                                                40.2032013f, leg_index, 6.8825264f, 784,
                                                maneuver_index, 3.12117195f, 332));
  //----------------------------------------------------------------
  // trace point = 1430
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6116562f, 40.2033806f, 1491516179, 21.1715584),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6116257f,
                                                40.2033882f, leg_index, 6.86920118f, 783,
                                                maneuver_index, 3.10784674f, 331));
  //----------------------------------------------------------------
  // trace point = 1431
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6117096f, 40.2035599f, 1491516180, 20.2032776),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6116791f,
                                                40.2035637f, leg_index, 6.85674143f, 781,
                                                maneuver_index, 3.09538698f, 329));
  //----------------------------------------------------------------
  // trace point = 1432
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6117706f, 40.2037354f, 1491516181, 20.3746719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6117401f,
                                                40.203743f, leg_index, 6.84392786f, 780,
                                                maneuver_index, 3.08257341f, 328));
  //----------------------------------------------------------------
  // trace point = 1433
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6118317f, 40.2039032f, 1491516182, 19.384922),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6117935f,
                                                40.2039108f, leg_index, 6.83198023f, 779,
                                                maneuver_index, 3.07062578f, 327));
  //----------------------------------------------------------------
  // trace point = 1434
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6118927f, 40.2040749f, 1491516183, 19.6639748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6118546f,
                                                40.2040825f, leg_index, 6.81967878f, 777,
                                                maneuver_index, 3.05832434f, 325));
  //----------------------------------------------------------------
  // trace point = 1435
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6119537f, 40.2042427f, 1491516184, 19.2552185),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.611908f,
                                                40.2042503f, leg_index, 6.80773115f, 776,
                                                maneuver_index, 3.04637671f, 324));
  //----------------------------------------------------------------
  // trace point = 1436
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6119995f, 40.2044029f, 1491516185, 18.5881939),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6119614f,
                                                40.2044106f, leg_index, 6.79629517f, 775,
                                                maneuver_index, 3.03494072f, 323));
  //----------------------------------------------------------------
  // trace point = 1437
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6120605f, 40.2045631f, 1491516186, 18.3512344),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6120148f,
                                                40.2045708f, leg_index, 6.78485966f, 774,
                                                maneuver_index, 3.02350521f, 322));
  //----------------------------------------------------------------
  // trace point = 1438
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.612114f, 40.2047157f, 1491516187, 17.7255116),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6120605f,
                                                40.2047272f, leg_index, 6.77377701f, 772,
                                                maneuver_index, 3.01242256f, 320));
  //----------------------------------------------------------------
  // trace point = 1439
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6121674f, 40.2048759f, 1491516188, 18.4147091),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.612114f,
                                                40.2048874f, leg_index, 6.7623415f, 771,
                                                maneuver_index, 3.00098705f, 319));
  //----------------------------------------------------------------
  // trace point = 1440
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6122284f, 40.2050362f, 1491516189, 18.5990562),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6121674f,
                                                40.2050476f, leg_index, 6.75090599f, 770,
                                                maneuver_index, 2.98955154f, 318));
  //----------------------------------------------------------------
  // trace point = 1441
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6122894f, 40.2052078f, 1491516190, 19.817831),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6122208f,
                                                40.2052193f, leg_index, 6.73870182f, 769,
                                                maneuver_index, 2.97734737f, 317));
  //----------------------------------------------------------------
  // trace point = 1442
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6123428f, 40.2053833f, 1491516191, 19.8586349),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6122818f,
                                                40.2053947f, leg_index, 6.72614479f, 767,
                                                maneuver_index, 2.96479034f, 315));
  //----------------------------------------------------------------
  // trace point = 1443
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6124039f, 40.205555f, 1491516192, 19.8368778),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6123352f,
                                                40.2055664f, leg_index, 6.71394062f, 766,
                                                maneuver_index, 2.95258617f, 314));
  //----------------------------------------------------------------
  // trace point = 1444
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6124573f, 40.2057266f, 1491516193, 19.9666023),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6123886f,
                                                40.2057381f, leg_index, 6.70173693f, 765,
                                                maneuver_index, 2.94038248f, 313));
  //----------------------------------------------------------------
  // trace point = 1445
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6125259f, 40.2059059f, 1491516194, 20.503994),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6124496f,
                                                40.2059212f, leg_index, 6.6886673f, 763,
                                                maneuver_index, 2.92731285f, 311));
  //----------------------------------------------------------------
  // trace point = 1446
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.612587f, 40.2060852f, 1491516195, 20.8485813),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6125107f,
                                                40.2061005f, leg_index, 6.67585421f, 762,
                                                maneuver_index, 2.91449976f, 310));
  //----------------------------------------------------------------
  // trace point = 1447
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6126404f, 40.2062683f, 1491516196, 20.7434387),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6125717f,
                                                40.2062836f, leg_index, 6.66278505f, 761,
                                                maneuver_index, 2.90143061f, 309));
  //----------------------------------------------------------------
  // trace point = 1448
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6127014f, 40.2064476f, 1491516197, 20.590004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6126251f,
                                                40.2064629f, leg_index, 6.65006876f, 759,
                                                maneuver_index, 2.88871431f, 307));
  //----------------------------------------------------------------
  // trace point = 1449
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6127701f, 40.2066231f, 1491516198, 20.4185581),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6126862f,
                                                40.2066383f, leg_index, 6.63751173f, 758,
                                                maneuver_index, 2.87615728f, 306));
  //----------------------------------------------------------------
  // trace point = 1450
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6128311f, 40.2068024f, 1491516199, 20.6557121),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6127396f,
                                                40.2068176f, leg_index, 6.62479591f, 757,
                                                maneuver_index, 2.86344147f, 305));
  //----------------------------------------------------------------
  // trace point = 1451
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6128998f, 40.2069817f, 1491516200, 20.46381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6128006f,
                                                40.2070007f, leg_index, 6.61172628f, 755,
                                                maneuver_index, 2.85037184f, 303));
  //----------------------------------------------------------------
  // trace point = 1452
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6129608f, 40.2071609f, 1491516201, 20.9561615),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6128616f,
                                                40.20718f, leg_index, 6.59891272f, 754,
                                                maneuver_index, 2.83755827f, 302));
  //----------------------------------------------------------------
  // trace point = 1453
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6130219f, 40.2073441f, 1491516202, 21.0002594),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6129227f,
                                                40.2073631f, leg_index, 6.58584452f, 752,
                                                maneuver_index, 2.82449007f, 300));
  //----------------------------------------------------------------
  // trace point = 1454
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6130905f, 40.2075233f, 1491516203, 20.8485527),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.612999f,
                                                40.20755f, leg_index, 6.5722847f, 751, maneuver_index,
                                                2.81093025f, 299));
  //----------------------------------------------------------------
  // trace point = 1455
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6131516f, 40.2077065f, 1491516204, 20.8055153),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6130905f,
                                                40.2077255f, leg_index, 6.55922127f, 750,
                                                maneuver_index, 2.79786682f, 298));
  //----------------------------------------------------------------
  // trace point = 1456
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6132202f, 40.2078896f, 1491516205, 21.1758671),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6131744f,
                                                40.207901f, leg_index, 6.54629946f, 749,
                                                maneuver_index, 2.78494501f, 297));
  //----------------------------------------------------------------
  // trace point = 1457
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6132812f, 40.2080727f, 1491516206, 21.0456257),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6132584f,
                                                40.2080765f, leg_index, 6.53338003f, 747,
                                                maneuver_index, 2.77202559f, 295));
  //----------------------------------------------------------------
  // trace point = 1458
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6133499f, 40.208252f, 1491516207, 20.8705387),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6133347f,
                                                40.2082558f, leg_index, 6.52033138f, 746,
                                                maneuver_index, 2.75897694f, 294));
  //----------------------------------------------------------------
  // trace point = 1459
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6134109f, 40.2084351f, 1491516208, 20.9561367),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6133957f,
                                                40.2084389f, leg_index, 6.50726748f, 745,
                                                maneuver_index, 2.74591303f, 293));
  //----------------------------------------------------------------
  // trace point = 1460
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6134796f, 40.2086182f, 1491516209, 21.1527996),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6134644f,
                                                40.208622f, leg_index, 6.49409294f, 743,
                                                maneuver_index, 2.73273849f, 291));
  //----------------------------------------------------------------
  // trace point = 1461
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6135483f, 40.2088013f, 1491516210, 21.4746056),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.613533f,
                                                40.2088051f, leg_index, 6.48091984f, 742,
                                                maneuver_index, 2.71956539f, 290));
  //----------------------------------------------------------------
  // trace point = 1462
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6136169f, 40.2089882f, 1491516211, 21.3062019),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6136017f,
                                                40.208992f, leg_index, 6.467484f, 740, maneuver_index,
                                                2.70612955f, 288));
  //----------------------------------------------------------------
  // trace point = 1463
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6136703f, 40.2091827f, 1491516212, 22.4052963),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6136627f,
                                                40.2091827f, leg_index, 6.45390272f, 739,
                                                maneuver_index, 2.69254827f, 287));
  //----------------------------------------------------------------
  // trace point = 1464
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6137238f, 40.2093697f, 1491516213, 21.373312),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6137238f,
                                                40.2093697f, leg_index, 6.44057798f, 738,
                                                maneuver_index, 2.67922354f, 286));
  //----------------------------------------------------------------
  // trace point = 1465
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6137848f, 40.2095642f, 1491516214, 22.2985458),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6137848f,
                                                40.2095642f, leg_index, 6.42673969f, 736,
                                                maneuver_index, 2.66538525f, 284));
  //----------------------------------------------------------------
  // trace point = 1466
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6138458f, 40.2097511f, 1491516215, 21.3449593),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6138458f,
                                                40.2097511f, leg_index, 6.41341782f, 735,
                                                maneuver_index, 2.65206337f, 283));
  //----------------------------------------------------------------
  // trace point = 1467
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6139145f, 40.209938f, 1491516216, 21.3225746),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6139145f,
                                                40.209938f, leg_index, 6.39996624f, 733,
                                                maneuver_index, 2.63861179f, 281));
  //----------------------------------------------------------------
  // trace point = 1468
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6139755f, 40.2101173f, 1491516217, 20.9154682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6139832f,
                                                40.2101135f, leg_index, 6.38729954f, 732,
                                                maneuver_index, 2.62594509f, 280));
  //----------------------------------------------------------------
  // trace point = 1469
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6140442f, 40.2103004f, 1491516218, 20.8704967),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6140594f,
                                                40.2102966f, leg_index, 6.37400818f, 730,
                                                maneuver_index, 2.61265373f, 278));
  //----------------------------------------------------------------
  // trace point = 1470
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6141052f, 40.2104836f, 1491516219, 21.0422821),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6141357f,
                                                40.2104759f, leg_index, 6.36096859f, 729,
                                                maneuver_index, 2.59961414f, 277));
  //----------------------------------------------------------------
  // trace point = 1471
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6141663f, 40.2106628f, 1491516220, 20.7010651),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6142044f,
                                                40.2106514f, leg_index, 6.34830189f, 727,
                                                maneuver_index, 2.58694744f, 275));
  //----------------------------------------------------------------
  // trace point = 1472
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6142349f, 40.2108383f, 1491516221, 20.6648712),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6142807f,
                                                40.2108269f, leg_index, 6.33551359f, 726,
                                                maneuver_index, 2.57415915f, 274));
  //----------------------------------------------------------------
  // trace point = 1473
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6143036f, 40.21101f, 1491516222, 20.0253201),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6143494f,
                                                40.2109985f, leg_index, 6.32309866f, 725,
                                                maneuver_index, 2.56174421f, 273));
  //----------------------------------------------------------------
  // trace point = 1474
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6143799f, 40.2111855f, 1491516223, 20.5260811),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6144257f,
                                                40.211174f, leg_index, 6.31031084f, 723,
                                                maneuver_index, 2.54895639f, 271));
  //----------------------------------------------------------------
  // trace point = 1475
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6144485f, 40.2113609f, 1491516224, 20.2382908),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6144943f,
                                                40.2113495f, leg_index, 6.29764414f, 722,
                                                maneuver_index, 2.53628969f, 270));
  //----------------------------------------------------------------
  // trace point = 1476
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6145248f, 40.2115364f, 1491516225, 20.524334),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6145706f,
                                                40.211525f, leg_index, 6.28485632f, 721,
                                                maneuver_index, 2.52350187f, 269));
  //----------------------------------------------------------------
  // trace point = 1477
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6146088f, 40.2117043f, 1491516226, 19.9146652),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6146393f,
                                                40.2116966f, leg_index, 6.27244043f, 719,
                                                maneuver_index, 2.51108599f, 267));
  //----------------------------------------------------------------
  // trace point = 1478
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6146927f, 40.2118645f, 1491516227, 19.4380264),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6147232f,
                                                40.211853f, leg_index, 6.26073408f, 718,
                                                maneuver_index, 2.49937963f, 266));
  //----------------------------------------------------------------
  // trace point = 1479
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6147766f, 40.2120285f, 1491516228, 19.4210682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6148148f,
                                                40.2120132f, leg_index, 6.24864292f, 717,
                                                maneuver_index, 2.48728848f, 265));
  //----------------------------------------------------------------
  // trace point = 1480
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6148758f, 40.2121849f, 1491516229, 19.21735),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6149139f,
                                                40.2121735f, leg_index, 6.23638725f, 716,
                                                maneuver_index, 2.47503281f, 264));
  //----------------------------------------------------------------
  // trace point = 1481
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6149673f, 40.2123451f, 1491516230, 19.4525757),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6150055f,
                                                40.2123337f, leg_index, 6.22429609f, 714,
                                                maneuver_index, 2.46294165f, 262));
  //----------------------------------------------------------------
  // trace point = 1482
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6150589f, 40.2125053f, 1491516231, 19.8958473),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.615097f,
                                                40.2124901f, leg_index, 6.21244574f, 713,
                                                maneuver_index, 2.45109129f, 261));
  //----------------------------------------------------------------
  // trace point = 1483
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6151657f, 40.2126694f, 1491516232, 20.1104374),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6151962f,
                                                40.2126579f, leg_index, 6.19970846f, 712,
                                                maneuver_index, 2.43835402f, 260));
  //----------------------------------------------------------------
  // trace point = 1484
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6152725f, 40.2128334f, 1491516233, 20.3968945),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6152954f,
                                                40.2128258f, leg_index, 6.18697119f, 710,
                                                maneuver_index, 2.42561674f, 258));
  //----------------------------------------------------------------
  // trace point = 1485
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6153793f, 40.2129974f, 1491516234, 20.6887169),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6153946f,
                                                40.2129936f, leg_index, 6.17423439f, 709,
                                                maneuver_index, 2.41287994f, 257));
  //----------------------------------------------------------------
  // trace point = 1486
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6154938f, 40.2131653f, 1491516235, 20.8473778),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6154938f,
                                                40.2131653f, leg_index, 6.16125584f, 708,
                                                maneuver_index, 2.39990139f, 256));
  //----------------------------------------------------------------
  // trace point = 1487
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6156235f, 40.2133293f, 1491516236, 21.0810471),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6156158f,
                                                40.2133331f, leg_index, 6.14792633f, 706,
                                                maneuver_index, 2.38657188f, 254));
  //----------------------------------------------------------------
  // trace point = 1488
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6157455f, 40.2134895f, 1491516237, 20.9669266),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6157532f,
                                                40.2134857f, leg_index, 6.13511944f, 704,
                                                maneuver_index, 2.37376499f, 252));
  //----------------------------------------------------------------
  // trace point = 1489
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6158829f, 40.2136421f, 1491516238, 20.3960342),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6158905f,
                                                40.2136383f, leg_index, 6.12231302f, 703,
                                                maneuver_index, 2.36095858f, 251));
  //----------------------------------------------------------------
  // trace point = 1490
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6160126f, 40.2137947f, 1491516239, 20.2096863),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6160278f,
                                                40.2137871f, leg_index, 6.10972309f, 701,
                                                maneuver_index, 2.34836864f, 249));
  //----------------------------------------------------------------
  // trace point = 1491
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6161423f, 40.2139435f, 1491516240, 19.9303112),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6161652f,
                                                40.213932f, leg_index, 6.09734726f, 700,
                                                maneuver_index, 2.33599281f, 248));
  //----------------------------------------------------------------
  // trace point = 1492
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6162643f, 40.2140884f, 1491516241, 19.6061668),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6162949f,
                                                40.2140732f, leg_index, 6.08542109f, 699,
                                                maneuver_index, 2.32406664f, 247));
  //----------------------------------------------------------------
  // trace point = 1493
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.616394f, 40.214241f, 1491516242, 19.9775772),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6164322f,
                                                40.2142181f, leg_index, 6.07304525f, 698,
                                                maneuver_index, 2.31169081f, 246));
  //----------------------------------------------------------------
  // trace point = 1494
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6165314f, 40.214386f, 1491516243, 19.6650944),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6165695f,
                                                40.2143631f, leg_index, 6.0606699f, 696,
                                                maneuver_index, 2.29931545f, 244));
  //----------------------------------------------------------------
  // trace point = 1495
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6166611f, 40.2145386f, 1491516244, 20.397541),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6167068f,
                                                40.2145119f, leg_index, 6.04807949f, 695,
                                                maneuver_index, 2.28672504f, 243));
  //----------------------------------------------------------------
  // trace point = 1496
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6167984f, 40.2146912f, 1491516245, 20.7223549),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6168518f,
                                                40.2146645f, leg_index, 6.03504086f, 693,
                                                maneuver_index, 2.27368641f, 241));
  //----------------------------------------------------------------
  // trace point = 1497
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6169357f, 40.2148514f, 1491516246, 21.1409092),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6169968f,
                                                40.2148209f, leg_index, 6.02178669f, 692,
                                                maneuver_index, 2.26043224f, 240));
  //----------------------------------------------------------------
  // trace point = 1498
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6170731f, 40.2150078f, 1491516247, 21.0966167),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6171341f,
                                                40.2149849f, leg_index, 6.00827551f, 690,
                                                maneuver_index, 2.24692106f, 238));
  //----------------------------------------------------------------
  // trace point = 1499
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6172104f, 40.2151642f, 1491516248, 21.0966015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6172409f,
                                                40.2151527f, leg_index, 5.99536896f, 689,
                                                maneuver_index, 2.23401451f, 237));
  //----------------------------------------------------------------
  // trace point = 1500
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6173477f, 40.2153282f, 1491516249, 21.6068745),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6173477f,
                                                40.2153282f, leg_index, 5.98198366f, 687,
                                                maneuver_index, 2.22062922f, 235));
  //----------------------------------------------------------------
  // trace point = 1501
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.617485f, 40.2154884f, 1491516250, 21.2396507),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6174469f,
                                                40.2154999f, leg_index, 5.96900892f, 686,
                                                maneuver_index, 2.20765448f, 234));
  //----------------------------------------------------------------
  // trace point = 1502
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6176147f, 40.2156563f, 1491516251, 21.4357719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6175613f,
                                                40.2156754f, leg_index, 5.95545626f, 685,
                                                maneuver_index, 2.19410181f, 233));
  //----------------------------------------------------------------
  // trace point = 1503
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6177444f, 40.2158241f, 1491516252, 21.8718281),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6176682f,
                                                40.2158508f, leg_index, 5.94206667f, 683,
                                                maneuver_index, 2.18071222f, 231));
  //----------------------------------------------------------------
  // trace point = 1504
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6178589f, 40.2159958f, 1491516253, 21.3786182),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.617775f,
                                                40.2160263f, leg_index, 5.9286828f, 681,
                                                maneuver_index, 2.16732836f, 229));
  //----------------------------------------------------------------
  // trace point = 1505
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6179733f, 40.2161636f, 1491516254, 21.083559),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6178818f,
                                                40.216198f, leg_index, 5.9155364f, 680,
                                                maneuver_index, 2.15418196f, 228));
  //----------------------------------------------------------------
  // trace point = 1506
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6180725f, 40.2163353f, 1491516255, 21.1870708),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6179886f,
                                                40.2163658f, leg_index, 5.90262842f, 679,
                                                maneuver_index, 2.14127398f, 227));
  //----------------------------------------------------------------
  // trace point = 1507
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6181793f, 40.2165146f, 1491516256, 21.4589844),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.618103f,
                                                40.2165451f, leg_index, 5.88883257f, 677,
                                                maneuver_index, 2.12747812f, 225));
  //----------------------------------------------------------------
  // trace point = 1508
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6182709f, 40.2166901f, 1491516257, 21.3972359),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6182098f,
                                                40.216713f, leg_index, 5.87592506f, 676,
                                                maneuver_index, 2.11457062f, 224));
  //----------------------------------------------------------------
  // trace point = 1509
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6183548f, 40.2168732f, 1491516258, 21.4051685),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6183167f,
                                                40.2168884f, leg_index, 5.86254025f, 674,
                                                maneuver_index, 2.1011858f, 222));
  //----------------------------------------------------------------
  // trace point = 1510
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6184387f, 40.2170563f, 1491516259, 21.5821457),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6184235f,
                                                40.2170601f, leg_index, 5.84939432f, 673,
                                                maneuver_index, 2.08803988f, 221));
  //----------------------------------------------------------------
  // trace point = 1511
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6185074f, 40.2172432f, 1491516260, 21.6646938),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6185303f,
                                                40.2172356f, leg_index, 5.83601093f, 671,
                                                maneuver_index, 2.07465649f, 219));
  //----------------------------------------------------------------
  // trace point = 1512
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.618576f, 40.2174301f, 1491516261, 21.9040642),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6185913f,
                                                40.2174263f, leg_index, 5.82225752f, 670,
                                                maneuver_index, 2.06090307f, 218));
  //----------------------------------------------------------------
  // trace point = 1513
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186371f, 40.2176285f, 1491516262, 22.2791195),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186295f,
                                                40.2176285f, leg_index, 5.80812836f, 668,
                                                maneuver_index, 2.04677391f, 216));
  //----------------------------------------------------------------
  // trace point = 1514
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186981f, 40.2178192f, 1491516263, 22.1706676),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186676f,
                                                40.217823f, leg_index, 5.79452133f, 667,
                                                maneuver_index, 2.03316689f, 215));
  //----------------------------------------------------------------
  // trace point = 1515
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6187515f, 40.2180138f, 1491516264, 21.9519272),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6187057f,
                                                40.2180176f, leg_index, 5.78091717f, 665,
                                                maneuver_index, 2.01956272f, 213));
  //----------------------------------------------------------------
  // trace point = 1516
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6187897f, 40.2182159f, 1491516265, 22.6856537),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6187057f,
                                                40.2182159f, leg_index, 5.76714706f, 663,
                                                maneuver_index, 2.00579262f, 211));
  //----------------------------------------------------------------
  // trace point = 1517
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6188354f, 40.2184105f, 1491516266, 22.0271492),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186981f,
                                                40.2184067f, leg_index, 5.75394773f, 662,
                                                maneuver_index, 1.99259329f, 210));
  //----------------------------------------------------------------
  // trace point = 1518
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6188736f, 40.2186089f, 1491516267, 22.3403053),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186905f,
                                                40.218605f, leg_index, 5.74022055f, 661,
                                                maneuver_index, 1.9788661f, 209));
  //----------------------------------------------------------------
  // trace point = 1519
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6188965f, 40.2188072f, 1491516268, 22.2625484),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186905f,
                                                40.2188034f, leg_index, 5.72650433f, 659,
                                                maneuver_index, 1.96514988f, 207));
  //----------------------------------------------------------------
  // trace point = 1520
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6189041f, 40.2190132f, 1491516269, 22.8363209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186829f,
                                                40.2190094f, leg_index, 5.71224976f, 658,
                                                maneuver_index, 1.95089531f, 206));
  //----------------------------------------------------------------
  // trace point = 1521
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6189117f, 40.2192192f, 1491516270, 22.9374866),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186752f,
                                                40.2192154f, leg_index, 5.69799566f, 656,
                                                maneuver_index, 1.93664122f, 204));
  //----------------------------------------------------------------
  // trace point = 1522
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6189041f, 40.2194366f, 1491516271, 24.0504189),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186676f,
                                                40.2194328f, leg_index, 5.68294954f, 654,
                                                maneuver_index, 1.9215951f, 202));
  //----------------------------------------------------------------
  // trace point = 1523
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6188889f, 40.2196426f, 1491516272, 23.3341484),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.61866f,
                                                40.2196388f, leg_index, 5.66869545f, 653,
                                                maneuver_index, 1.907341f, 201));
  //----------------------------------------------------------------
  // trace point = 1524
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.618866f, 40.2198639f, 1491516273, 24.339592),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186523f,
                                                40.2198601f, leg_index, 5.65338564f, 651,
                                                maneuver_index, 1.89203119f, 199));
  //----------------------------------------------------------------
  // trace point = 1525
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6188354f, 40.2200661f, 1491516274, 22.7509651),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186447f,
                                                40.2200623f, leg_index, 5.63939476f, 650,
                                                maneuver_index, 1.87804031f, 198));
  //----------------------------------------------------------------
  // trace point = 1526
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6187973f, 40.2202759f, 1491516275, 23.4889145),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186371f,
                                                40.2202721f, leg_index, 5.62487698f, 648,
                                                maneuver_index, 1.86352253f, 196));
  //----------------------------------------------------------------
  // trace point = 1527
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6187515f, 40.2204819f, 1491516276, 23.5380764),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186295f,
                                                40.2204781f, leg_index, 5.61062241f, 647,
                                                maneuver_index, 1.84926796f, 195));
  //----------------------------------------------------------------
  // trace point = 1528
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6187134f, 40.2206955f, 1491516277, 23.8083649),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186218f,
                                                40.2206955f, leg_index, 5.59557676f, 645,
                                                maneuver_index, 1.83422232f, 193));
  //----------------------------------------------------------------
  // trace point = 1529
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186829f, 40.2209091f, 1491516278, 24.0974979),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186142f,
                                                40.2209091f, leg_index, 5.58079481f, 644,
                                                maneuver_index, 1.81944036f, 192));
  //----------------------------------------------------------------
  // trace point = 1530
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186523f, 40.2211151f, 1491516279, 22.8518925),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186066f,
                                                40.2211151f, leg_index, 5.56654072f, 642,
                                                maneuver_index, 1.80518627f, 190));
  //----------------------------------------------------------------
  // trace point = 1531
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186218f, 40.2213211f, 1491516280, 23.2852917),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6185989f,
                                                40.2213211f, leg_index, 5.5522871f, 640,
                                                maneuver_index, 1.79093266f, 188));
  //----------------------------------------------------------------
  // trace point = 1532
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186066f, 40.2215233f, 1491516281, 22.3287048),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186142f,
                                                40.2215233f, leg_index, 5.53826809f, 639,
                                                maneuver_index, 1.77691364f, 187));
  //----------------------------------------------------------------
  // trace point = 1533
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6185913f, 40.2217216f, 1491516282, 22.1723003),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186371f,
                                                40.2217178f, leg_index, 5.52475691f, 637,
                                                maneuver_index, 1.76340246f, 185));
  //----------------------------------------------------------------
  // trace point = 1534
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6185913f, 40.2219124f, 1491516283, 21.1513863),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.61866f,
                                                40.2219086f, leg_index, 5.51150846f, 636,
                                                maneuver_index, 1.75015402f, 184));
  //----------------------------------------------------------------
  // trace point = 1535
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6185989f, 40.2220993f, 1491516284, 20.8340912),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186829f,
                                                40.2220955f, leg_index, 5.49852276f, 635,
                                                maneuver_index, 1.73716831f, 183));
  //----------------------------------------------------------------
  // trace point = 1536
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186142f, 40.2222824f, 1491516285, 20.4226475),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6187057f,
                                                40.2222748f, leg_index, 5.48606443f, 633,
                                                maneuver_index, 1.72470999f, 181));
  //----------------------------------------------------------------
  // trace point = 1537
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186371f, 40.2224579f, 1491516286, 19.8805714),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.618721f,
                                                40.2224426f, leg_index, 5.47443151f, 632,
                                                maneuver_index, 1.71307707f, 180));
  //----------------------------------------------------------------
  // trace point = 1538
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186676f, 40.2226334f, 1491516287, 19.5716267),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6187744f,
                                                40.2226181f, leg_index, 5.46197176f, 631,
                                                maneuver_index, 1.70061731f, 179));
  //----------------------------------------------------------------
  // trace point = 1539
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6187057f, 40.2227974f, 1491516288, 18.7447338),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6188202f,
                                                40.2227783f, leg_index, 5.45062876f, 630,
                                                maneuver_index, 1.68927431f, 178));
  //----------------------------------------------------------------
  // trace point = 1540
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6187515f, 40.22295f, 1491516289, 17.2930431),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6188583f,
                                                40.2229309f, leg_index, 5.43988705f, 629,
                                                maneuver_index, 1.6785326f, 177));
  //----------------------------------------------------------------
  // trace point = 1541
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6187897f, 40.2230835f, 1491516290, 15.2993574),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6188965f,
                                                40.2230644f, leg_index, 5.43043423f, 628,
                                                maneuver_index, 1.66907978f, 176));
  //----------------------------------------------------------------
  // trace point = 1542
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6188278f, 40.2232094f, 1491516291, 14.4324236),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6189346f,
                                                40.2231903f, leg_index, 5.42149782f, 627,
                                                maneuver_index, 1.66014338f, 175));
  //----------------------------------------------------------------
  // trace point = 1543
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.618866f, 40.2233353f, 1491516292, 14.0878401),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6189728f,
                                                40.22332f, leg_index, 5.41230297f, 626,
                                                maneuver_index, 1.65094852f, 174));
  //----------------------------------------------------------------
  // trace point = 1544
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6189041f, 40.2234459f, 1491516293, 12.7933731),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6190033f,
                                                40.2234306f, leg_index, 5.40448332f, 625,
                                                maneuver_index, 1.64312887f, 173));
  //----------------------------------------------------------------
  // trace point = 1545
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6189499f, 40.2235489f, 1491516294, 12.0087767),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6190338f,
                                                40.2235336f, leg_index, 5.39717913f, 624,
                                                maneuver_index, 1.63582468f, 172));
  //----------------------------------------------------------------
  // trace point = 1546
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.618988f, 40.2236443f, 1491516295, 11.4018059),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6190567f,
                                                40.2236328f, leg_index, 5.39021778f, 623,
                                                maneuver_index, 1.62886333f, 171));
  //----------------------------------------------------------------
  // trace point = 1547
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6190186f, 40.2237282f, 1491516296, 9.71497154),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6190796f,
                                                40.2237167f, leg_index, 5.38428879f, 623,
                                                maneuver_index, 1.62293434f, 171));
  //----------------------------------------------------------------
  // trace point = 1548
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6190643f, 40.2238655f, 1491516298, 7.72315216),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6191254f,
                                                40.2238579f, leg_index, 5.37423563f, 622,
                                                maneuver_index, 1.61288118f, 170));
  //----------------------------------------------------------------
  // trace point = 1549
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6191025f, 40.2239494f, 1491516301, 3.27204895),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6191483f,
                                                40.2239418f, leg_index, 5.36830616f, 621,
                                                maneuver_index, 1.60695171f, 169));
  //----------------------------------------------------------------
  // trace point = 1550
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6191406f, 40.224041f, 1491516321, 0.551451981),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6191711f,
                                                40.2240372f, leg_index, 5.36160278f, 620,
                                                maneuver_index, 1.60024834f, 168));
  //----------------------------------------------------------------
  // trace point = 1551
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6191788f, 40.2241402f, 1491516323, 5.83232594),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6192017f,
                                                40.2241364f, leg_index, 5.35455656f, 619,
                                                maneuver_index, 1.59320211f, 167));
  //----------------------------------------------------------------
  // trace point = 1552
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6192322f, 40.2242813f, 1491516325, 8.08816719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6192398f,
                                                40.2242813f, leg_index, 5.34433222f, 618,
                                                maneuver_index, 1.58297777f, 166));
  //----------------------------------------------------------------
  // trace point = 1553
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6192627f, 40.2243652f, 1491516326, 9.7383604),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6192703f,
                                                40.2243652f, leg_index, 5.33830643f, 617,
                                                maneuver_index, 1.57695198f, 165));
  //----------------------------------------------------------------
  // trace point = 1554
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6192932f, 40.2244492f, 1491516327, 9.73835945),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6193008f,
                                                40.2244453f, leg_index, 5.33253574f, 617,
                                                maneuver_index, 1.5711813f, 165));
  //----------------------------------------------------------------
  // trace point = 1555
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6193314f, 40.2245407f, 1491516328, 10.6412964),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.619339f,
                                                40.2245407f, leg_index, 5.32563829f, 616,
                                                maneuver_index, 1.56428385f, 164));
  //----------------------------------------------------------------
  // trace point = 1556
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6193771f, 40.2246361f, 1491516329, 11.1890736),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6193771f,
                                                40.2246361f, leg_index, 5.31874084f, 615,
                                                maneuver_index, 1.5573864f, 163));
  //----------------------------------------------------------------
  // trace point = 1557
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6194153f, 40.2247391f, 1491516330, 11.8773022),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6194153f,
                                                40.2247391f, leg_index, 5.31133795f, 615,
                                                maneuver_index, 1.5499835f, 163));
  //----------------------------------------------------------------
  // trace point = 1558
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6194611f, 40.2248497f, 1491516331, 12.854166),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6194611f,
                                                40.2248497f, leg_index, 5.30331421f, 614,
                                                maneuver_index, 1.54195976f, 162));
  //----------------------------------------------------------------
  // trace point = 1559
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6195068f, 40.2249641f, 1491516332, 13.4436903),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6195068f,
                                                40.2249641f, leg_index, 5.29503727f, 613,
                                                maneuver_index, 1.53368282f, 161));
  //----------------------------------------------------------------
  // trace point = 1560
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6195526f, 40.2250824f, 1491516333, 13.5985451),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6195526f,
                                                40.2250824f, leg_index, 5.28650761f, 612,
                                                maneuver_index, 1.52515316f, 160));
  //----------------------------------------------------------------
  // trace point = 1561
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6195984f, 40.2252045f, 1491516334, 14.1805134),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6195984f,
                                                40.2252045f, leg_index, 5.27772522f, 611,
                                                maneuver_index, 1.51637077f, 159));
  //----------------------------------------------------------------
  // trace point = 1562
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6196518f, 40.2253304f, 1491516335, 14.7624817),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6196442f,
                                                40.2253304f, leg_index, 5.26869011f, 610,
                                                maneuver_index, 1.50733566f, 158));
  //----------------------------------------------------------------
  // trace point = 1563
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6197128f, 40.2254601f, 1491516336, 15.401103),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6196976f,
                                                40.2254639f, leg_index, 5.25903368f, 609,
                                                maneuver_index, 1.49767923f, 157));
  //----------------------------------------------------------------
  // trace point = 1564
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6197662f, 40.2255974f, 1491516337, 15.8447208),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.619751f,
                                                40.2256012f, leg_index, 5.24912453f, 608,
                                                maneuver_index, 1.48777008f, 156));
  //----------------------------------------------------------------
  // trace point = 1565
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6198196f, 40.2257347f, 1491516338, 15.9762316),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6198044f,
                                                40.2257385f, leg_index, 5.23921537f, 607,
                                                maneuver_index, 1.47786093f, 155));
  //----------------------------------------------------------------
  // trace point = 1566
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.619873f, 40.225872f, 1491516339, 15.9021282),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6198578f,
                                                40.2258759f, leg_index, 5.2293067f, 606,
                                                maneuver_index, 1.46795225f, 154));
  //----------------------------------------------------------------
  // trace point = 1567
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6199265f, 40.2260094f, 1491516340, 15.9511108),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6199112f,
                                                40.2260132f, leg_index, 5.21939754f, 605,
                                                maneuver_index, 1.4580431f, 153));
  //----------------------------------------------------------------
  // trace point = 1568
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6199875f, 40.2261505f, 1491516341, 16.674099),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6199722f,
                                                40.2261543f, leg_index, 5.20912075f, 604,
                                                maneuver_index, 1.4477663f, 152));
  //----------------------------------------------------------------
  // trace point = 1569
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6200409f, 40.2263031f, 1491516342, 17.6240196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6200256f,
                                                40.2263069f, leg_index, 5.19819975f, 602,
                                                maneuver_index, 1.4368453f, 150));
  //----------------------------------------------------------------
  // trace point = 1570
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6201019f, 40.2264595f, 1491516343, 18.0412407),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6200867f,
                                                40.2264633f, leg_index, 5.18691158f, 601,
                                                maneuver_index, 1.42555714f, 149));
  //----------------------------------------------------------------
  // trace point = 1571
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6201706f, 40.2266197f, 1491516344, 18.6482143),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6201553f,
                                                40.2266235f, leg_index, 5.17525482f, 600,
                                                maneuver_index, 1.41390038f, 148));
  //----------------------------------------------------------------
  // trace point = 1572
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6202316f, 40.2267838f, 1491516345, 19.0495129),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6202164f,
                                                40.2267876f, leg_index, 5.16346073f, 599,
                                                maneuver_index, 1.40210629f, 147));
  //----------------------------------------------------------------
  // trace point = 1573
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6203003f, 40.2269516f, 1491516346, 19.8374691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.620285f,
                                                40.2269554f, leg_index, 5.15129852f, 597,
                                                maneuver_index, 1.38994408f, 145));
  //----------------------------------------------------------------
  // trace point = 1574
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.620369f, 40.2271233f, 1491516347, 19.9280033),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6203461f,
                                                40.2271271f, leg_index, 5.13899851f, 596,
                                                maneuver_index, 1.37764406f, 144));
  //----------------------------------------------------------------
  // trace point = 1575
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6204376f, 40.2272987f, 1491516348, 20.2877998),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6204147f,
                                                40.2273026f, leg_index, 5.12633085f, 595,
                                                maneuver_index, 1.36497641f, 143));
  //----------------------------------------------------------------
  // trace point = 1576
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6205063f, 40.227478f, 1491516349, 20.7238789),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6204834f,
                                                40.2274818f, leg_index, 5.11341429f, 593,
                                                maneuver_index, 1.35205984f, 141));
  //----------------------------------------------------------------
  // trace point = 1577
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.620575f, 40.2276573f, 1491516350, 20.902071),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6205597f,
                                                40.2276611f, leg_index, 5.10037422f, 592,
                                                maneuver_index, 1.33901978f, 140));
  //----------------------------------------------------------------
  // trace point = 1578
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6206436f, 40.2278404f, 1491516351, 20.9380302),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6206284f,
                                                40.2278442f, leg_index, 5.08719969f, 591,
                                                maneuver_index, 1.32584524f, 139));
  //----------------------------------------------------------------
  // trace point = 1579
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6207123f, 40.2280235f, 1491516352, 21.1987514),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.620697f,
                                                40.2280273f, leg_index, 5.07402611f, 590,
                                                maneuver_index, 1.31267166f, 138));
  //----------------------------------------------------------------
  // trace point = 1580
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6207809f, 40.2282028f, 1491516353, 20.9380207),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6207657f,
                                                40.2282066f, leg_index, 5.06110525f, 588,
                                                maneuver_index, 1.2997508f, 136));
  //----------------------------------------------------------------
  // trace point = 1581
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.620842f, 40.2283783f, 1491516354, 20.4633389),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6208344f,
                                                40.2283783f, leg_index, 5.04869032f, 587,
                                                maneuver_index, 1.28733587f, 135));
  //----------------------------------------------------------------
  // trace point = 1582
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6209106f, 40.2285576f, 1491516355, 20.7953568),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.620903f,
                                                40.2285576f, leg_index, 5.03576946f, 586,
                                                maneuver_index, 1.27441502f, 134));
  //----------------------------------------------------------------
  // trace point = 1583
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6209793f, 40.2287483f, 1491516356, 21.6449051),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6209793f,
                                                40.2287483f, leg_index, 5.02197552f, 584,
                                                maneuver_index, 1.26062107f, 132));
  //----------------------------------------------------------------
  // trace point = 1584
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.621048f, 40.2289352f, 1491516357, 21.6269817),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.621048f,
                                                40.2289352f, leg_index, 5.00854778f, 582,
                                                maneuver_index, 1.24719334f, 130));
  //----------------------------------------------------------------
  // trace point = 1585
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6211166f, 40.2291222f, 1491516358, 21.6040878),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6211166f,
                                                40.2291222f, leg_index, 4.99512005f, 581,
                                                maneuver_index, 1.2337656f, 129));
  //----------------------------------------------------------------
  // trace point = 1586
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6211929f, 40.2293129f, 1491516359, 22.2214794),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6211929f,
                                                40.2293129f, leg_index, 4.98132658f, 579,
                                                maneuver_index, 1.21997213f, 127));
  //----------------------------------------------------------------
  // trace point = 1587
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6212692f, 40.2295113f, 1491516360, 22.9444275),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6212692f,
                                                40.2295113f, leg_index, 4.96702623f, 578,
                                                maneuver_index, 1.20567179f, 126));
  //----------------------------------------------------------------
  // trace point = 1588
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6213455f, 40.2297134f, 1491516361, 23.5867767),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6213455f,
                                                40.2297134f, leg_index, 4.95247269f, 576,
                                                maneuver_index, 1.19111824f, 124));
  //----------------------------------------------------------------
  // trace point = 1589
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6214218f, 40.2299118f, 1491516362, 22.9922161),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6214218f,
                                                40.2299118f, leg_index, 4.93817377f, 575,
                                                maneuver_index, 1.17681932f, 123));
  //----------------------------------------------------------------
  // trace point = 1590
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6214981f, 40.2301102f, 1491516363, 23.0912685),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6214905f,
                                                40.2301102f, leg_index, 4.92398262f, 573,
                                                maneuver_index, 1.16262817f, 121));
  //----------------------------------------------------------------
  // trace point = 1591
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6215744f, 40.2303085f, 1491516364, 23.0989437),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6215668f,
                                                40.2303085f, leg_index, 4.90968227f, 572,
                                                maneuver_index, 1.14832783f, 120));
  //----------------------------------------------------------------
  // trace point = 1592
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6216507f, 40.2305031f, 1491516365, 22.5532722),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6216431f,
                                                40.2305069f, leg_index, 4.89538193f, 571,
                                                maneuver_index, 1.13402748f, 119));
  //----------------------------------------------------------------
  // trace point = 1593
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6217194f, 40.23069f, 1491516366, 21.4127274),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6217117f,
                                                40.23069f, leg_index, 4.88220739f, 569,
                                                maneuver_index, 1.12085295f, 117));
  //----------------------------------------------------------------
  // trace point = 1594
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.621788f, 40.2308693f, 1491516367, 20.9510021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6217804f,
                                                40.2308693f, leg_index, 4.86928654f, 568,
                                                maneuver_index, 1.10793209f, 116));
  //----------------------------------------------------------------
  // trace point = 1595
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6218567f, 40.2310448f, 1491516368, 20.1892014),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6218491f,
                                                40.2310486f, leg_index, 4.85636568f, 567,
                                                maneuver_index, 1.09501123f, 115));
  //----------------------------------------------------------------
  // trace point = 1596
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6219254f, 40.2312164f, 1491516369, 19.9756947),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6219101f,
                                                40.2312202f, leg_index, 4.84406424f, 565,
                                                maneuver_index, 1.08270979f, 113));
  //----------------------------------------------------------------
  // trace point = 1597
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6219864f, 40.2313805f, 1491516370, 19.1924515),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6219711f,
                                                40.2313843f, leg_index, 4.83226967f, 564,
                                                maneuver_index, 1.07091522f, 112));
  //----------------------------------------------------------------
  // trace point = 1598
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6220398f, 40.2315292f, 1491516371, 17.4102707),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6220322f,
                                                40.2315331f, leg_index, 4.82148743f, 563,
                                                maneuver_index, 1.06013298f, 111));
  //----------------------------------------------------------------
  // trace point = 1599
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6221008f, 40.2316666f, 1491516372, 15.738266),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6220856f,
                                                40.2316704f, leg_index, 4.81157875f, 562,
                                                maneuver_index, 1.0502243f, 110));
  //----------------------------------------------------------------
  // trace point = 1600
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6221466f, 40.2317886f, 1491516373, 14.2870483),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6221313f,
                                                40.2317924f, leg_index, 4.80279589f, 561,
                                                maneuver_index, 1.04144144f, 109));
  //----------------------------------------------------------------
  // trace point = 1601
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6221848f, 40.2319069f, 1491516374, 13.3749094),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6221771f,
                                                40.2319107f, leg_index, 4.79401302f, 560,
                                                maneuver_index, 1.03265858f, 108));
  //----------------------------------------------------------------
  // trace point = 1602
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6222305f, 40.2320175f, 1491516375, 12.8616753),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6222229f,
                                                40.2320213f, leg_index, 4.78624821f, 559,
                                                maneuver_index, 1.02489376f, 107));
  //----------------------------------------------------------------
  // trace point = 1603
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6222763f, 40.2321281f, 1491516376, 13.2541437),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6222687f,
                                                40.232132f, leg_index, 4.77821207f, 559,
                                                maneuver_index, 1.01685762f, 107));
  //----------------------------------------------------------------
  // trace point = 1604
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6223145f, 40.2322464f, 1491516377, 13.4435787),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6223145f,
                                                40.2322464f, leg_index, 4.7699275f, 558,
                                                maneuver_index, 1.00857306f, 106));
  //----------------------------------------------------------------
  // trace point = 1605
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6223679f, 40.2323723f, 1491516378, 14.7981529),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6223679f,
                                                40.2323723f, leg_index, 4.76077986f, 557,
                                                maneuver_index, 0.999425411f, 105));
  //----------------------------------------------------------------
  // trace point = 1606
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6224136f, 40.2325096f, 1491516379, 15.5498877),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6224136f,
                                                40.2325096f, leg_index, 4.75098276f, 556,
                                                maneuver_index, 0.989628315f, 104));
  //----------------------------------------------------------------
  // trace point = 1607
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.622467f, 40.2326508f, 1491516380, 16.5580463),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.622467f,
                                                40.2326508f, leg_index, 4.74082041f, 554,
                                                maneuver_index, 0.979465961f, 102));
  //----------------------------------------------------------------
  // trace point = 1608
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6225281f, 40.2327995f, 1491516381, 17.3397865),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6225281f,
                                                40.2327995f, leg_index, 4.7300396f, 553,
                                                maneuver_index, 0.96868515f, 101));
  //----------------------------------------------------------------
  // trace point = 1609
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6225815f, 40.2329559f, 1491516382, 18.0989704),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6225815f,
                                                40.2329559f, leg_index, 4.71886349f, 552,
                                                maneuver_index, 0.957509041f, 100));
  //----------------------------------------------------------------
  // trace point = 1610
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6226425f, 40.2331161f, 1491516383, 18.5266323),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6226425f,
                                                40.2331161f, leg_index, 4.7073226f, 551,
                                                maneuver_index, 0.945968151f, 99));
  //----------------------------------------------------------------
  // trace point = 1611
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6227112f, 40.2332764f, 1491516384, 18.5984135),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6227036f,
                                                40.2332764f, leg_index, 4.69578266f, 549,
                                                maneuver_index, 0.934428215f, 97));
  //----------------------------------------------------------------
  // trace point = 1612
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6227722f, 40.2334442f, 1491516385, 19.4180641),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6227722f,
                                                40.2334442f, leg_index, 4.68361998f, 548,
                                                maneuver_index, 0.92226553f, 96));
  //----------------------------------------------------------------
  // trace point = 1613
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6228409f, 40.2336159f, 1491516386, 19.7869892),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6228409f,
                                                40.2336159f, leg_index, 4.67120647f, 547,
                                                maneuver_index, 0.909852028f, 95));
  //----------------------------------------------------------------
  // trace point = 1614
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6229095f, 40.2337837f, 1491516387, 19.7974033),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6229095f,
                                                40.2337837f, leg_index, 4.65904093f, 546,
                                                maneuver_index, 0.897686481f, 94));
  //----------------------------------------------------------------
  // trace point = 1615
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6229706f, 40.2339554f, 1491516388, 19.6313057),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6229706f,
                                                40.2339554f, leg_index, 4.64673805f, 544,
                                                maneuver_index, 0.885383606f, 92));
  //----------------------------------------------------------------
  // trace point = 1616
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6230392f, 40.2341194f, 1491516389, 19.393671),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6230316f,
                                                40.2341194f, leg_index, 4.63356352f, 543,
                                                maneuver_index, 0.872209072f, 91));
  //----------------------------------------------------------------
  // trace point = 1617
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6231003f, 40.2342949f, 1491516390, 20.0348225),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.623085f,
                                                40.2342987f, leg_index, 4.62110233f, 542,
                                                maneuver_index, 0.859747887f, 90));
  //----------------------------------------------------------------
  // trace point = 1618
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6231613f, 40.2344666f, 1491516391, 19.8362999),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6231537f,
                                                40.2344704f, leg_index, 4.60981369f, 541,
                                                maneuver_index, 0.848459244f, 89));
  //----------------------------------------------------------------
  // trace point = 1619
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.62323f, 40.2346382f, 1491516392, 19.9756126),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6232224f,
                                                40.234642f, leg_index, 4.59739971f, 539,
                                                maneuver_index, 0.836045265f, 87));
  //----------------------------------------------------------------
  // trace point = 1620
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6232986f, 40.2348137f, 1491516393, 20.2624874),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6232834f,
                                                40.2348175f, leg_index, 4.58484602f, 538,
                                                maneuver_index, 0.823491573f, 86));
  //----------------------------------------------------------------
  // trace point = 1621
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6233673f, 40.2349892f, 1491516394, 20.4028053),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6233521f,
                                                40.234993f, leg_index, 4.57217884f, 537,
                                                maneuver_index, 0.810824394f, 85));
  //----------------------------------------------------------------
  // trace point = 1622
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6234283f, 40.2351685f, 1491516395, 20.8922844),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6234207f,
                                                40.2351685f, leg_index, 4.55950975f, 536,
                                                maneuver_index, 0.798155308f, 84));
  //----------------------------------------------------------------
  // trace point = 1623
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6235046f, 40.2353516f, 1491516396, 20.9847145),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6234894f,
                                                40.2353554f, leg_index, 4.54607964f, 534,
                                                maneuver_index, 0.784725189f, 82));
  //----------------------------------------------------------------
  // trace point = 1624
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6235733f, 40.2355347f, 1491516397, 21.1985722),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.623558f,
                                                40.2355385f, leg_index, 4.53290606f, 532,
                                                maneuver_index, 0.771551609f, 80));
  //----------------------------------------------------------------
  // trace point = 1625
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.623642f, 40.2357178f, 1491516398, 21.4264011),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6236267f,
                                                40.2357216f, leg_index, 4.51973248f, 531,
                                                maneuver_index, 0.758378029f, 79));
  //----------------------------------------------------------------
  // trace point = 1626
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6237183f, 40.2359047f, 1491516399, 21.9019699),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6236954f,
                                                40.2359085f, leg_index, 4.50517702f, 530,
                                                maneuver_index, 0.743822575f, 78));
  //----------------------------------------------------------------
  // trace point = 1627
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6237946f, 40.2360916f, 1491516400, 21.7804527),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6237717f,
                                                40.2360954f, leg_index, 4.49276257f, 529,
                                                maneuver_index, 0.731408119f, 77));
  //----------------------------------------------------------------
  // trace point = 1628
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6238556f, 40.2362823f, 1491516401, 21.7961788),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6238403f,
                                                40.2362862f, leg_index, 4.47908211f, 527,
                                                maneuver_index, 0.717727661f, 75));
  //----------------------------------------------------------------
  // trace point = 1629
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6239319f, 40.2364731f, 1491516402, 22.3033867),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6239166f,
                                                40.2364769f, leg_index, 4.46528816f, 526,
                                                maneuver_index, 0.703933716f, 74));
  //----------------------------------------------------------------
  // trace point = 1630
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6240082f, 40.2366676f, 1491516403, 22.303381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6239929f,
                                                40.2366714f, leg_index, 4.45124149f, 524,
                                                maneuver_index, 0.689887047f, 72));
  //----------------------------------------------------------------
  // trace point = 1631
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6240768f, 40.2368584f, 1491516404, 22.4459915),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6240616f,
                                                40.2368622f, leg_index, 4.43756104f, 523,
                                                maneuver_index, 0.676206589f, 71));
  //----------------------------------------------------------------
  // trace point = 1632
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6241531f, 40.2370529f, 1491516405, 22.185688),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6241379f,
                                                40.2370567f, leg_index, 4.42351389f, 521,
                                                maneuver_index, 0.662159443f, 69));
  //----------------------------------------------------------------
  // trace point = 1633
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6242218f, 40.2372475f, 1491516406, 22.4852829),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6242142f,
                                                40.2372475f, leg_index, 4.40971994f, 520,
                                                maneuver_index, 0.648365498f, 68));
  //----------------------------------------------------------------
  // trace point = 1634
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6242905f, 40.2374382f, 1491516407, 22.0784569),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6242828f,
                                                40.2374382f, leg_index, 4.39603996f, 518,
                                                maneuver_index, 0.634685516f, 66));
  //----------------------------------------------------------------
  // trace point = 1635
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6243591f, 40.2376366f, 1491516408, 22.6569786),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6243591f,
                                                40.2376366f, leg_index, 4.38174009f, 517,
                                                maneuver_index, 0.620385647f, 65));
  //----------------------------------------------------------------
  // trace point = 1636
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6244278f, 40.2378311f, 1491516409, 22.7002792),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6244354f,
                                                40.2378273f, leg_index, 4.36794519f, 515,
                                                maneuver_index, 0.606590748f, 63));
  //----------------------------------------------------------------
  // trace point = 1637
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6244965f, 40.2380295f, 1491516410, 22.959507),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6245117f,
                                                40.2380257f, leg_index, 4.35364819f, 513,
                                                maneuver_index, 0.592293739f, 61));
  //----------------------------------------------------------------
  // trace point = 1638
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6245728f, 40.2382317f, 1491516411, 23.0047474),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6245956f,
                                                40.238224f, leg_index, 4.33922911f, 512,
                                                maneuver_index, 0.57787466f, 60));
  //----------------------------------------------------------------
  // trace point = 1639
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6246414f, 40.23843f, 1491516412, 23.1349106),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6246719f,
                                                40.2384224f, leg_index, 4.32492542f, 510,
                                                maneuver_index, 0.563570976f, 58));
  //----------------------------------------------------------------
  // trace point = 1640
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6247177f, 40.2386322f, 1491516413, 23.5637302),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6247482f,
                                                40.2386246f, leg_index, 4.31037235f, 509,
                                                maneuver_index, 0.549017906f, 57));
  //----------------------------------------------------------------
  // trace point = 1641
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.624794f, 40.2388382f, 1491516414, 23.4793854),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6248245f,
                                                40.2388306f, leg_index, 4.29556561f, 507,
                                                maneuver_index, 0.534211159f, 55));
  //----------------------------------------------------------------
  // trace point = 1642
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6248703f, 40.2390404f, 1491516415, 23.5025883),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6248932f,
                                                40.2390366f, leg_index, 4.28086662f, 506,
                                                maneuver_index, 0.519512177f, 54));
  //----------------------------------------------------------------
  // trace point = 1643
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6249466f, 40.2392426f, 1491516416, 23.5164013),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6249695f,
                                                40.2392387f, leg_index, 4.26631355f, 504,
                                                maneuver_index, 0.504959106f, 52));
  //----------------------------------------------------------------
  // trace point = 1644
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6250229f, 40.2394485f, 1491516417, 24.0612049),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6250458f,
                                                40.2394447f, leg_index, 4.25150585f, 502,
                                                maneuver_index, 0.490151405f, 50));
  //----------------------------------------------------------------
  // trace point = 1645
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6251068f, 40.2396584f, 1491516418, 24.0010738),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6251221f,
                                                40.2396545f, leg_index, 4.23644447f, 501,
                                                maneuver_index, 0.475090027f, 49));
  //----------------------------------------------------------------
  // trace point = 1646
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6251831f, 40.2398605f, 1491516419, 23.7293606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6251907f,
                                                40.2398605f, leg_index, 4.22173929f, 500,
                                                maneuver_index, 0.460384846f, 48));
  //----------------------------------------------------------------
  // trace point = 1647
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6252594f, 40.2400703f, 1491516420, 24.0353127),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6252594f,
                                                40.2400703f, leg_index, 4.20678234f, 498,
                                                maneuver_index, 0.445427895f, 46));
  //----------------------------------------------------------------
  // trace point = 1648
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6253204f, 40.2402916f, 1491516421, 25.3413162),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6253281f,
                                                40.2402916f, leg_index, 4.19104958f, 497,
                                                maneuver_index, 0.429695129f, 45));
  //----------------------------------------------------------------
  // trace point = 1649
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6253815f, 40.2405167f, 1491516422, 25.4686222),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6253815f,
                                                40.2405167f, leg_index, 4.17522144f, 496,
                                                maneuver_index, 0.413866997f, 44));
  //----------------------------------------------------------------
  // trace point = 1650
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6254425f, 40.2407455f, 1491516423, 25.8230743),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6254349f,
                                                40.2407455f, leg_index, 4.15914202f, 495,
                                                maneuver_index, 0.397787571f, 43));
  //----------------------------------------------------------------
  // trace point = 1651
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6254807f, 40.2409706f, 1491516424, 25.6425056),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.625473f,
                                                40.2409706f, leg_index, 4.1434412f, 493,
                                                maneuver_index, 0.382086754f, 41));
  //----------------------------------------------------------------
  // trace point = 1652
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255112f, 40.2411919f, 1491516425, 24.3924427),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255035f,
                                                40.2411919f, leg_index, 4.12805367f, 491,
                                                maneuver_index, 0.366699219f, 39));
  //----------------------------------------------------------------
  // trace point = 1653
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255341f, 40.2414093f, 1491516426, 24.5681477),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255188f,
                                                40.2414093f, leg_index, 4.11298037f, 489,
                                                maneuver_index, 0.351625919f, 37));
  //----------------------------------------------------------------
  // trace point = 1654
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255417f, 40.2416306f, 1491516427, 24.7247562),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255341f,
                                                40.2416306f, leg_index, 4.09765482f, 488,
                                                maneuver_index, 0.336300373f, 36));
  //----------------------------------------------------------------
  // trace point = 1655
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255493f, 40.2418556f, 1491516428, 24.7222748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255493f,
                                                40.2418556f, leg_index, 4.08127546f, 486,
                                                maneuver_index, 0.319921017f, 34));
  //----------------------------------------------------------------
  // trace point = 1656
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255569f, 40.2420845f, 1491516429, 25.6057396),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255493f,
                                                40.2420845f, leg_index, 4.06623745f, 484,
                                                maneuver_index, 0.304883003f, 32));
  //----------------------------------------------------------------
  // trace point = 1657
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255569f, 40.2423134f, 1491516430, 25.6040478),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255569f,
                                                40.2423134f, leg_index, 4.05040121f, 482,
                                                maneuver_index, 0.289046764f, 30));
  //----------------------------------------------------------------
  // trace point = 1658 | Alert | In a quarter mile Continue on Fishburn Road.
  maneuver_index = 8;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255646f, 40.2425499f, 1491516431, 26.3894234),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.6255569f, 40.2425499f, leg_index, 4.03404236f,
                                                481, maneuver_index, 0.272687912f, 29,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1659
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255722f, 40.2427826f, 1491516432, 25.6085587),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255646f,
                                                40.2427826f, leg_index, 4.01794338f, 479,
                                                maneuver_index, 0.256588936f, 27));
  //----------------------------------------------------------------
  // trace point = 1660
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255722f, 40.2430153f, 1491516433, 26.0522251),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255646f,
                                                40.2430153f, leg_index, 4.00184965f, 478,
                                                maneuver_index, 0.240495205f, 26));
  //----------------------------------------------------------------
  // trace point = 1661
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255798f, 40.2432442f, 1491516434, 25.6103916),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255722f,
                                                40.2432442f, leg_index, 3.98601341f, 476,
                                                maneuver_index, 0.224658966f, 24));
  //----------------------------------------------------------------
  // trace point = 1662
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255951f, 40.2434769f, 1491516435, 25.8430309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255875f,
                                                40.2434769f, leg_index, 3.96990204f, 475,
                                                maneuver_index, 0.208547592f, 23));
  //----------------------------------------------------------------
  // trace point = 1663
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6255951f, 40.2437057f, 1491516436, 25.6047516),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255951f,
                                                40.2437057f, leg_index, 3.95406508f, 473,
                                                maneuver_index, 0.192710638f, 21));
  //----------------------------------------------------------------
  // trace point = 1664
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256027f, 40.2439308f, 1491516437, 24.7222748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6255951f,
                                                40.2439308f, leg_index, 3.93849993f, 472,
                                                maneuver_index, 0.177145481f, 20));
  //----------------------------------------------------------------
  // trace point = 1665
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256104f, 40.2441483f, 1491516438, 24.6134911),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256027f,
                                                40.2441483f, leg_index, 3.92345428f, 470,
                                                maneuver_index, 0.162099838f, 18));
  //----------------------------------------------------------------
  // trace point = 1666
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256256f, 40.2443771f, 1491516439, 25.1755619),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256104f,
                                                40.2443771f, leg_index, 3.90761757f, 468,
                                                maneuver_index, 0.146263123f, 16));
  //----------------------------------------------------------------
  // trace point = 1667
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256332f, 40.2446022f, 1491516440, 25.3843994),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.625618f,
                                                40.2446022f, leg_index, 3.89204502f, 466,
                                                maneuver_index, 0.130690575f, 14));
  //----------------------------------------------------------------
  // trace point = 1668 | Pre | Continue on Fishburn Road for 2.5 miles.
  maneuver_index = 8;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256409f, 40.2448311f, 1491516441, 25.3923626),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6256332f, 40.2448311f, leg_index, 3.87619257f,
                                                465, maneuver_index, 0.114838123f, 13,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1669
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256409f, 40.2450638f, 1491516442, 25.8296165),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256409f,
                                                40.2450638f, leg_index, 3.86008644f, 463,
                                                maneuver_index, 0.0987319946f, 11));
  //----------------------------------------------------------------
  // trace point = 1670
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256485f, 40.2452927f, 1491516443, 25.2765236),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256485f,
                                                40.2452927f, leg_index, 3.84425139f, 461,
                                                maneuver_index, 0.0828969479f, 9));
  //----------------------------------------------------------------
  // trace point = 1671
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256561f, 40.2455215f, 1491516444, 25.6085587),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256485f,
                                                40.2455215f, leg_index, 3.82842159f, 459,
                                                maneuver_index, 0.0670671463f, 7));
  //----------------------------------------------------------------
  // trace point = 1672
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256638f, 40.2457428f, 1491516445, 24.8387833),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256638f,
                                                40.2457428f, leg_index, 3.81309795f, 458,
                                                maneuver_index, 0.0517435074f, 6));
  //----------------------------------------------------------------
  // trace point = 1673
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256714f, 40.2459641f, 1491516446, 24.6068897),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256714f,
                                                40.2459641f, leg_index, 3.79778934f, 456,
                                                maneuver_index, 0.0364348888f, 4));
  //----------------------------------------------------------------
  // trace point = 1674
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.625679f, 40.2461853f, 1491516447, 24.6052761),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256866f,
                                                40.2461853f, leg_index, 3.78246832f, 454,
                                                maneuver_index, 0.0211138725f, 2));
  //----------------------------------------------------------------
  // trace point = 1675
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256866f, 40.2464027f, 1491516448, 24.271368),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256943f,
                                                40.2464027f, leg_index, 3.76742315f, 453,
                                                maneuver_index, 0.00606870651f, 1));
  //----------------------------------------------------------------
  // trace point = 1676
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256866f, 40.2466164f, 1491516449, 23.6021786),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256943f,
                                                40.2466164f, leg_index, 3.75264692f, 452,
                                                maneuver_index, 2.51649237f, 259));
  //----------------------------------------------------------------
  // trace point = 1677
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256943f, 40.2468262f, 1491516450, 23.3895969),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256943f,
                                                40.2468262f, leg_index, 3.73813438f, 450,
                                                maneuver_index, 2.50197983f, 257));
  //----------------------------------------------------------------
  // trace point = 1678
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6257019f, 40.2470322f, 1491516451, 23.0558205),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6256943f,
                                                40.2470322f, leg_index, 3.72388554f, 449,
                                                maneuver_index, 2.48773098f, 256));
  //----------------------------------------------------------------
  // trace point = 1679
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6257095f, 40.247242f, 1491516452, 23.1546097),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6257019f,
                                                40.247242f, leg_index, 3.70936489f, 447,
                                                maneuver_index, 2.47321033f, 254));
  //----------------------------------------------------------------
  // trace point = 1680
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6257172f, 40.247448f, 1491516453, 23.058794),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6257095f,
                                                40.247448f, leg_index, 3.69511032f, 445,
                                                maneuver_index, 2.45895576f, 252));
  //----------------------------------------------------------------
  // trace point = 1681
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6257248f, 40.247654f, 1491516454, 23.0487728),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6257172f,
                                                40.247654f, leg_index, 3.68085599f, 444,
                                                maneuver_index, 2.44470119f, 251));
  //----------------------------------------------------------------
  // trace point = 1682
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6257401f, 40.2478676f, 1491516455, 23.5277195),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6257248f,
                                                40.2478676f, leg_index, 3.66607499f, 442,
                                                maneuver_index, 2.4299202f, 249));
  //----------------------------------------------------------------
  // trace point = 1683
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6257477f, 40.2480698f, 1491516456, 22.7169628),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6257324f,
                                                40.2480698f, leg_index, 3.65208387f, 441,
                                                maneuver_index, 2.41592932f, 248));
  //----------------------------------------------------------------
  // trace point = 1684
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6257629f, 40.2482796f, 1491516457, 23.4165878),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6257401f,
                                                40.2482796f, leg_index, 3.63756752f, 440,
                                                maneuver_index, 2.40141296f, 247));
  //----------------------------------------------------------------
  // trace point = 1685
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6257858f, 40.2484932f, 1491516458, 23.9408665),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6257706f,
                                                40.248497f, leg_index, 3.62237024f, 438,
                                                maneuver_index, 2.38621569f, 245));
  //----------------------------------------------------------------
  // trace point = 1686
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6258316f, 40.2487068f, 1491516459, 23.8680038),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.625824f,
                                                40.2487068f, leg_index, 3.60758805f, 437,
                                                maneuver_index, 2.37143326f, 244));
  //----------------------------------------------------------------
  // trace point = 1687
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6258926f, 40.2489243f, 1491516460, 24.6709805),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6258698f,
                                                40.2489281f, leg_index, 3.59098721f, 435,
                                                maneuver_index, 2.35483265f, 242));
  //----------------------------------------------------------------
  // trace point = 1688
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6259613f, 40.2491417f, 1491516461, 24.8376389),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6259537f,
                                                40.2491417f, leg_index, 3.57668734f, 434,
                                                maneuver_index, 2.34053278f, 241));
  //----------------------------------------------------------------
  // trace point = 1689
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6260376f, 40.2493515f, 1491516462, 24.405426),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6260376f,
                                                40.2493515f, leg_index, 3.5615139f, 432,
                                                maneuver_index, 2.32535934f, 239));
  //----------------------------------------------------------------
  // trace point = 1690
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6261215f, 40.2495613f, 1491516463, 24.5696621),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6261215f,
                                                40.2495613f, leg_index, 3.54633594f, 431,
                                                maneuver_index, 2.31018114f, 238));
  //----------------------------------------------------------------
  // trace point = 1691
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6262283f, 40.2497711f, 1491516464, 25.2597885),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6262207f,
                                                40.2497749f, leg_index, 3.5306592f, 429,
                                                maneuver_index, 2.29450464f, 236));
  //----------------------------------------------------------------
  // trace point = 1692
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6263428f, 40.2499771f, 1491516465, 24.7578335),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6263351f,
                                                40.2499809f, leg_index, 3.51517439f, 428,
                                                maneuver_index, 2.27901983f, 235));
  //----------------------------------------------------------------
  // trace point = 1693
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6264648f, 40.2501869f, 1491516466, 25.4016647),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6264725f,
                                                40.2501831f, leg_index, 3.49940085f, 426,
                                                maneuver_index, 2.26324606f, 233));
  //----------------------------------------------------------------
  // trace point = 1694
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6265869f, 40.2503891f, 1491516467, 24.8264065),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6266174f,
                                                40.2503777f, leg_index, 3.48392129f, 425,
                                                maneuver_index, 2.24776649f, 232));
  //----------------------------------------------------------------
  // trace point = 1695
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6267014f, 40.2505913f, 1491516468, 24.5885639),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6267395f,
                                                40.250576f, leg_index, 3.46874952f, 423,
                                                maneuver_index, 2.23259497f, 230));
  //----------------------------------------------------------------
  // trace point = 1696
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6268234f, 40.2507973f, 1491516469, 25.1656189),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6268616f,
                                                40.2507858f, leg_index, 3.45287061f, 422,
                                                maneuver_index, 2.21671581f, 229));
  //----------------------------------------------------------------
  // trace point = 1697
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6269455f, 40.2509995f, 1491516470, 24.6549187),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.626976f,
                                                40.250988f, leg_index, 3.43763661f, 420,
                                                maneuver_index, 2.20148182f, 227));
  //----------------------------------------------------------------
  // trace point = 1698
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6270599f, 40.2512016f, 1491516471, 24.4154453),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6270905f,
                                                40.2511902f, leg_index, 3.42240286f, 418,
                                                maneuver_index, 2.1862483f, 225));
  //----------------------------------------------------------------
  // trace point = 1699
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6271744f, 40.2513962f, 1491516472, 24.0776787),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6272049f,
                                                40.2513847f, leg_index, 3.40765214f, 417,
                                                maneuver_index, 2.17149734f, 224));
  //----------------------------------------------------------------
  // trace point = 1700
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6272964f, 40.2515945f, 1491516473, 24.2115517),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6273193f,
                                                40.2515869f, leg_index, 3.39241838f, 415,
                                                maneuver_index, 2.15626383f, 222));
  //----------------------------------------------------------------
  // trace point = 1701
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6274109f, 40.2517967f, 1491516474, 24.5174675),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6274338f,
                                                40.2517891f, leg_index, 3.37718463f, 414,
                                                maneuver_index, 2.14102983f, 221));
  //----------------------------------------------------------------
  // trace point = 1702
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6275253f, 40.2519951f, 1491516475, 24.1128426),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6275482f,
                                                40.2519875f, leg_index, 3.36219239f, 412,
                                                maneuver_index, 2.1260376f, 219));
  //----------------------------------------------------------------
  // trace point = 1703
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6276398f, 40.2521896f, 1491516476, 23.7033157),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6276627f,
                                                40.252182f, leg_index, 3.34744167f, 411,
                                                maneuver_index, 2.11128712f, 218));
  //----------------------------------------------------------------
  // trace point = 1704
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6277618f, 40.2523804f, 1491516477, 23.6439457),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6277771f,
                                                40.2523766f, leg_index, 3.33269119f, 409,
                                                maneuver_index, 2.09653664f, 216));
  //----------------------------------------------------------------
  // trace point = 1705
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6278763f, 40.2525673f, 1491516478, 22.9312382),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6278839f,
                                                40.2525673f, leg_index, 3.31834483f, 407,
                                                maneuver_index, 2.08219004f, 214));
  //----------------------------------------------------------------
  // trace point = 1706
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6279907f, 40.2527542f, 1491516479, 22.8587151),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6279755f,
                                                40.252758f, leg_index, 3.30429888f, 406,
                                                maneuver_index, 2.06814432f, 213));
  //----------------------------------------------------------------
  // trace point = 1707
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6281052f, 40.2529373f, 1491516480, 22.6672554),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6280823f,
                                                40.2529449f, leg_index, 3.29019666f, 404,
                                                maneuver_index, 2.05404186f, 211));
  //----------------------------------------------------------------
  // trace point = 1708
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6282272f, 40.2531128f, 1491516481, 21.9091167),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.628212f,
                                                40.2531204f, leg_index, 3.27621317f, 403,
                                                maneuver_index, 2.04005861f, 210));
  //----------------------------------------------------------------
  // trace point = 1709
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6283417f, 40.2532883f, 1491516482, 22.0469761),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6283417f,
                                                40.2532883f, leg_index, 3.26273465f, 402,
                                                maneuver_index, 2.02657986f, 209));
  //----------------------------------------------------------------
  // trace point = 1710
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6284714f, 40.2534561f, 1491516483, 22.0076885),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6284714f,
                                                40.2534561f, leg_index, 3.24925685f, 400,
                                                maneuver_index, 2.01310205f, 207));
  //----------------------------------------------------------------
  // trace point = 1711
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6286163f, 40.2536278f, 1491516484, 22.5344162),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6286087f,
                                                40.2536278f, leg_index, 3.23534465f, 399,
                                                maneuver_index, 1.99918997f, 206));
  //----------------------------------------------------------------
  // trace point = 1712
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6287689f, 40.2537918f, 1491516485, 22.0820808),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6287766f,
                                                40.2537842f, leg_index, 3.22136092f, 398,
                                                maneuver_index, 1.98520625f, 205));
  //----------------------------------------------------------------
  // trace point = 1713
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6289291f, 40.2539482f, 1491516486, 22.2290573),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6289444f,
                                                40.2539368f, leg_index, 3.20758033f, 397,
                                                maneuver_index, 1.97142565f, 204));
  //----------------------------------------------------------------
  // trace point = 1714
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.629097f, 40.2541046f, 1491516487, 22.7584896),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6291122f,
                                                40.2540932f, leg_index, 3.1935966f, 395,
                                                maneuver_index, 1.95744193f, 202));
  //----------------------------------------------------------------
  // trace point = 1715
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6292725f, 40.2542572f, 1491516488, 22.5805836),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6292877f,
                                                40.2542458f, leg_index, 3.1795373f, 394,
                                                maneuver_index, 1.94338262f, 201));
  //----------------------------------------------------------------
  // trace point = 1716
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6294556f, 40.254406f, 1491516489, 22.7697544),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6294708f,
                                                40.2543945f, leg_index, 3.16542363f, 392,
                                                maneuver_index, 1.92926896f, 199));
  //----------------------------------------------------------------
  // trace point = 1717
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6296539f, 40.2545433f, 1491516490, 22.6068935),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6296616f,
                                                40.2545357f, leg_index, 3.1514008f, 390,
                                                maneuver_index, 1.91524613f, 197));
  //----------------------------------------------------------------
  // trace point = 1718
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6298523f, 40.2546768f, 1491516491, 22.6572647),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6298599f,
                                                40.2546654f, leg_index, 3.13761139f, 389,
                                                maneuver_index, 1.90145671f, 196));
  //----------------------------------------------------------------
  // trace point = 1719
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6300659f, 40.2548065f, 1491516492, 23.104435),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6300735f,
                                                40.2547951f, leg_index, 3.12319255f, 388,
                                                maneuver_index, 1.88703787f, 195));
  //----------------------------------------------------------------
  // trace point = 1720
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6302795f, 40.2549286f, 1491516493, 22.4894581),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6302872f,
                                                40.2549171f, leg_index, 3.10910487f, 386,
                                                maneuver_index, 1.8729502f, 193));
  //----------------------------------------------------------------
  // trace point = 1721
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6304932f, 40.2550468f, 1491516494, 22.3747253),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6305008f,
                                                40.2550354f, leg_index, 3.09517407f, 385,
                                                maneuver_index, 1.8590194f, 192));
  //----------------------------------------------------------------
  // trace point = 1722
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6307068f, 40.2551537f, 1491516495, 21.9285564),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.630722f,
                                                40.2551308f, leg_index, 3.08176064f, 384,
                                                maneuver_index, 1.84560597f, 191));
  //----------------------------------------------------------------
  // trace point = 1723
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.630928f, 40.2552528f, 1491516496, 21.535717),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6309433f,
                                                40.2552299f, leg_index, 3.06821561f, 382,
                                                maneuver_index, 1.83206093f, 189));
  //----------------------------------------------------------------
  // trace point = 1724
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.631134f, 40.2553482f, 1491516497, 20.4801731),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6311569f,
                                                40.2553215f, leg_index, 3.05528259f, 381,
                                                maneuver_index, 1.81912792f, 188));
  //----------------------------------------------------------------
  // trace point = 1725
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.63134f, 40.2554398f, 1491516498, 20.1478004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6313629f,
                                                40.2554131f, leg_index, 3.04269886f, 380,
                                                maneuver_index, 1.80654418f, 187));
  //----------------------------------------------------------------
  // trace point = 1726
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6315384f, 40.2555275f, 1491516499, 19.4330196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6315613f,
                                                40.2555008f, leg_index, 3.03059554f, 379,
                                                maneuver_index, 1.79444087f, 186));
  //----------------------------------------------------------------
  // trace point = 1727
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6317291f, 40.2556152f, 1491516500, 18.9867001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.631752f,
                                                40.2555847f, leg_index, 3.01897335f, 378,
                                                maneuver_index, 1.78281868f, 185));
  //----------------------------------------------------------------
  // trace point = 1728
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6319122f, 40.2556953f, 1491516501, 18.1933746),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6319351f,
                                                40.2556648f, leg_index, 3.00783181f, 377,
                                                maneuver_index, 1.77167714f, 184));
  //----------------------------------------------------------------
  // trace point = 1729
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6320953f, 40.2557716f, 1491516502, 17.4376068),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6321182f,
                                                40.2557449f, leg_index, 2.99669075f, 375,
                                                maneuver_index, 1.76053607f, 182));
  //----------------------------------------------------------------
  // trace point = 1730
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6322708f, 40.2558517f, 1491516503, 17.2170372),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6322937f,
                                                40.255825f, leg_index, 2.98589849f, 374,
                                                maneuver_index, 1.74974382f, 181));
  //----------------------------------------------------------------
  // trace point = 1731
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6324463f, 40.2559357f, 1491516504, 17.7938538),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6324692f,
                                                40.2559013f, leg_index, 2.97523713f, 374,
                                                maneuver_index, 1.73908246f, 181));
  //----------------------------------------------------------------
  // trace point = 1732
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6326141f, 40.2560196f, 1491516505, 16.8502598),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6326447f,
                                                40.2559776f, leg_index, 2.96457672f, 372,
                                                maneuver_index, 1.72842205f, 179));
  //----------------------------------------------------------------
  // trace point = 1733
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.632782f, 40.2560921f, 1491516506, 16.5527573),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6328125f,
                                                40.2560463f, leg_index, 2.95452499f, 371,
                                                maneuver_index, 1.71837032f, 178));
  //----------------------------------------------------------------
  // trace point = 1734
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6329422f, 40.2561646f, 1491516507, 16.2027569),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6329803f,
                                                40.2561188f, leg_index, 2.944345f, 370,
                                                maneuver_index, 1.70819032f, 177));
  //----------------------------------------------------------------
  // trace point = 1735
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.63311f, 40.2562294f, 1491516508, 15.9769764),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6331406f,
                                                40.2561874f, leg_index, 2.93464446f, 369,
                                                maneuver_index, 1.69848979f, 176));
  //----------------------------------------------------------------
  // trace point = 1736
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6332855f, 40.2562981f, 1491516509, 16.735918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.633316f,
                                                40.2562599f, leg_index, 2.92411375f, 368,
                                                maneuver_index, 1.68795907f, 175));
  //----------------------------------------------------------------
  // trace point = 1737
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6334686f, 40.2563667f, 1491516510, 16.8348331),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6334915f,
                                                40.2563362f, leg_index, 2.9134531f, 367,
                                                maneuver_index, 1.67729843f, 174));
  //----------------------------------------------------------------
  // trace point = 1738
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6336365f, 40.256443f, 1491516511, 16.7944012),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6336594f,
                                                40.2564163f, leg_index, 2.90298653f, 366,
                                                maneuver_index, 1.66683185f, 173));
  //----------------------------------------------------------------
  // trace point = 1739
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6338043f, 40.2565231f, 1491516512, 16.7276459),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6338196f,
                                                40.2565041f, leg_index, 2.89257789f, 365,
                                                maneuver_index, 1.65642321f, 172));
  //----------------------------------------------------------------
  // trace point = 1740
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6339645f, 40.2566147f, 1491516513, 17.0918274),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6339874f,
                                                40.2565918f, leg_index, 2.88183832f, 364,
                                                maneuver_index, 1.64568365f, 171));
  //----------------------------------------------------------------
  // trace point = 1741
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6341171f, 40.2567177f, 1491516514, 17.4698601),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.63414f,
                                                40.2566986f, leg_index, 2.8709085f, 363,
                                                maneuver_index, 1.63475382f, 170));
  //----------------------------------------------------------------
  // trace point = 1742
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6342697f, 40.2568283f, 1491516515, 17.857523),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6342926f,
                                                40.256813f, leg_index, 2.85961008f, 361,
                                                maneuver_index, 1.62345541f, 168));
  //----------------------------------------------------------------
  // trace point = 1743
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.634407f, 40.2569504f, 1491516516, 17.912981),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6344299f,
                                                40.2569313f, leg_index, 2.84868145f, 360,
                                                maneuver_index, 1.61252677f, 167));
  //----------------------------------------------------------------
  // trace point = 1744
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6345444f, 40.2570801f, 1491516517, 18.3442974),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6345673f,
                                                40.2570686f, leg_index, 2.8367095f, 359,
                                                maneuver_index, 1.60055482f, 166));
  //----------------------------------------------------------------
  // trace point = 1745
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6346664f, 40.2572136f, 1491516518, 18.4595966),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6346893f,
                                                40.2572021f, leg_index, 2.82544827f, 358,
                                                maneuver_index, 1.5892936f, 165));
  //----------------------------------------------------------------
  // trace point = 1746
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6347961f, 40.2573509f, 1491516519, 18.5352669),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.634819f,
                                                40.2573395f, leg_index, 2.8137393f, 357,
                                                maneuver_index, 1.57758462f, 164));
  //----------------------------------------------------------------
  // trace point = 1747
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6349182f, 40.2574806f, 1491516520, 17.7595844),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6349411f,
                                                40.2574654f, leg_index, 2.80289698f, 355,
                                                maneuver_index, 1.5667423f, 162));
  //----------------------------------------------------------------
  // trace point = 1748
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6350327f, 40.2575989f, 1491516521, 16.627449),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6350632f,
                                                40.257576f, leg_index, 2.79289317f, 354,
                                                maneuver_index, 1.5567385f, 161));
  //----------------------------------------------------------------
  // trace point = 1749
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6351624f, 40.2577095f, 1491516522, 16.3764229),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6351929f,
                                                40.2576904f, leg_index, 2.78243017f, 353,
                                                maneuver_index, 1.5462755f, 160));
  //----------------------------------------------------------------
  // trace point = 1750
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6352997f, 40.2578087f, 1491516523, 16.2598419),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6353226f,
                                                40.2577934f, leg_index, 2.77253366f, 352,
                                                maneuver_index, 1.53637898f, 159));
  //----------------------------------------------------------------
  // trace point = 1751
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6354446f, 40.2579117f, 1491516524, 16.3523998),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6354599f,
                                                40.2579002f, leg_index, 2.7621839f, 351,
                                                maneuver_index, 1.52602923f, 158));
  //----------------------------------------------------------------
  // trace point = 1752
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6355743f, 40.258007f, 1491516525, 15.5674276),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6355896f,
                                                40.2579956f, leg_index, 2.75267768f, 350,
                                                maneuver_index, 1.516523f, 157));
  //----------------------------------------------------------------
  // trace point = 1753
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6357117f, 40.2581062f, 1491516526, 15.8901682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6357193f,
                                                40.2580986f, leg_index, 2.74279952f, 349,
                                                maneuver_index, 1.50664485f, 156));
  //----------------------------------------------------------------
  // trace point = 1754
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.635849f, 40.2582054f, 1491516527, 16.0438251),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6358566f,
                                                40.2581978f, leg_index, 2.7328217f, 348,
                                                maneuver_index, 1.49666703f, 155));
  //----------------------------------------------------------------
  // trace point = 1755
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6359863f, 40.2583084f, 1491516528, 16.6429691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.635994f,
                                                40.2583046f, leg_index, 2.72247148f, 347,
                                                maneuver_index, 1.4863168f, 154));
  //----------------------------------------------------------------
  // trace point = 1756
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6361313f, 40.2584114f, 1491516529, 16.7198429),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6361313f,
                                                40.2584114f, leg_index, 2.71212268f, 346,
                                                maneuver_index, 1.475968f, 153));
  //----------------------------------------------------------------
  // trace point = 1757
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6362762f, 40.2585182f, 1491516530, 17.0427818),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6362762f,
                                                40.2585182f, leg_index, 2.70148611f, 345,
                                                maneuver_index, 1.46533144f, 152));
  //----------------------------------------------------------------
  // trace point = 1758
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6364288f, 40.258625f, 1491516531, 17.4429035),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6364288f,
                                                40.258625f, leg_index, 2.69055629f, 344,
                                                maneuver_index, 1.45440161f, 151));
  //----------------------------------------------------------------
  // trace point = 1759
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6365738f, 40.2587318f, 1491516532, 17.4428883),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6365814f,
                                                40.258728f, leg_index, 2.67980456f, 343,
                                                maneuver_index, 1.44364989f, 150));
  //----------------------------------------------------------------
  // trace point = 1760
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6367264f, 40.2588348f, 1491516533, 17.2165527),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6367264f,
                                                40.2588348f, leg_index, 2.66917181f, 342,
                                                maneuver_index, 1.43301713f, 149));
  //----------------------------------------------------------------
  // trace point = 1761
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6368713f, 40.2589417f, 1491516534, 17.166769),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.636879f,
                                                40.2589378f, leg_index, 2.65841985f, 341,
                                                maneuver_index, 1.42226517f, 148));
  //----------------------------------------------------------------
  // trace point = 1762
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6370087f, 40.2590408f, 1491516535, 16.2433548),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6370163f,
                                                40.2590332f, leg_index, 2.6486187f, 339,
                                                maneuver_index, 1.41246402f, 146));
  //----------------------------------------------------------------
  // trace point = 1763
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6371536f, 40.2591362f, 1491516536, 15.7775183),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6371536f,
                                                40.2591324f, leg_index, 2.63863921f, 338,
                                                maneuver_index, 1.40248454f, 145));
  //----------------------------------------------------------------
  // trace point = 1764
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.637291f, 40.2592316f, 1491516537, 16.1267376),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.637291f,
                                                40.2592278f, leg_index, 2.62883806f, 337,
                                                maneuver_index, 1.39268339f, 144));
  //----------------------------------------------------------------
  // trace point = 1765
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6374283f, 40.2593307f, 1491516538, 16.0009346),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6374359f,
                                                40.2593269f, leg_index, 2.61856151f, 336,
                                                maneuver_index, 1.38240683f, 143));
  //----------------------------------------------------------------
  // trace point = 1766
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6375656f, 40.2594223f, 1491516539, 15.7253742),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6375656f,
                                                40.2594185f, leg_index, 2.60923553f, 335,
                                                maneuver_index, 1.37308085f, 142));
  //----------------------------------------------------------------
  // trace point = 1767
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6376877f, 40.25951f, 1491516540, 14.0611086),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6376953f,
                                                40.2595062f, leg_index, 2.60008764f, 334,
                                                maneuver_index, 1.36393297f, 141));
  //----------------------------------------------------------------
  // trace point = 1768
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6378021f, 40.2595978f, 1491516541, 13.7229967),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6378098f,
                                                40.2595901f, leg_index, 2.59171224f, 334,
                                                maneuver_index, 1.35555756f, 141));
  //----------------------------------------------------------------
  // trace point = 1769
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6379166f, 40.2596817f, 1491516542, 13.2196436),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6379318f,
                                                40.2596703f, leg_index, 2.58321786f, 333,
                                                maneuver_index, 1.34706318f, 140));
  //----------------------------------------------------------------
  // trace point = 1770
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.638031f, 40.2597618f, 1491516543, 13.1701403),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6380463f,
                                                40.2597504f, leg_index, 2.57502055f, 332,
                                                maneuver_index, 1.33886588f, 139));
  //----------------------------------------------------------------
  // trace point = 1771
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6381454f, 40.2598343f, 1491516544, 12.7863836),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6381531f,
                                                40.2598267f, leg_index, 2.56729817f, 331,
                                                maneuver_index, 1.3311435f, 138));
  //----------------------------------------------------------------
  // trace point = 1772
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6382599f, 40.259922f, 1491516545, 13.7846136),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6382751f,
                                                40.2599106f, leg_index, 2.55862641f, 330,
                                                maneuver_index, 1.32247174f, 137));
  //----------------------------------------------------------------
  // trace point = 1773
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6383896f, 40.2600136f, 1491516546, 15.0754395),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6384048f,
                                                40.2600021f, leg_index, 2.54930043f, 329,
                                                maneuver_index, 1.31314576f, 136));
  //----------------------------------------------------------------
  // trace point = 1774
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6385269f, 40.2601089f, 1491516547, 15.9890099),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6385345f,
                                                40.2601013f, leg_index, 2.53961086f, 328,
                                                maneuver_index, 1.30345619f, 135));
  //----------------------------------------------------------------
  // trace point = 1775
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6386719f, 40.2602158f, 1491516548, 17.1348457),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6386795f,
                                                40.2602081f, leg_index, 2.52897453f, 327,
                                                maneuver_index, 1.29281986f, 134));
  //----------------------------------------------------------------
  // trace point = 1776
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6388321f, 40.2603264f, 1491516549, 17.9680138),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6388397f,
                                                40.2603226f, leg_index, 2.51739144f, 326,
                                                maneuver_index, 1.28123677f, 133));
  //----------------------------------------------------------------
  // trace point = 1777
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6389847f, 40.2604408f, 1491516550, 18.2101269),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6389923f,
                                                40.2604332f, leg_index, 2.50628209f, 325,
                                                maneuver_index, 1.27012742f, 132));
  //----------------------------------------------------------------
  // trace point = 1778
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6391373f, 40.2605515f, 1491516551, 18.1951714),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6391449f,
                                                40.2605438f, leg_index, 2.49517012f, 324,
                                                maneuver_index, 1.25901544f, 131));
  //----------------------------------------------------------------
  // trace point = 1779
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6392975f, 40.2606659f, 1491516552, 18.1487656),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6393051f,
                                                40.2606583f, leg_index, 2.48358703f, 323,
                                                maneuver_index, 1.24743235f, 130));
  //----------------------------------------------------------------
  // trace point = 1780
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6394501f, 40.2607841f, 1491516553, 18.8165531),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6394653f,
                                                40.2607765f, leg_index, 2.47181273f, 322,
                                                maneuver_index, 1.23565805f, 129));
  //----------------------------------------------------------------
  // trace point = 1781
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.639595f, 40.2609024f, 1491516554, 18.1241417),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6396027f,
                                                40.2608948f, leg_index, 2.46088433f, 320,
                                                maneuver_index, 1.22472966f, 127));
  //----------------------------------------------------------------
  // trace point = 1782
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6397324f, 40.2610245f, 1491516555, 17.6661358),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.63974f,
                                                40.2610207f, leg_index, 2.44954014f, 319,
                                                maneuver_index, 1.21338546f, 126));
  //----------------------------------------------------------------
  // trace point = 1783
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6398621f, 40.2611504f, 1491516556, 17.9919357),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6398697f,
                                                40.2611465f, leg_index, 2.43846488f, 318,
                                                maneuver_index, 1.2023102f, 125));
  //----------------------------------------------------------------
  // trace point = 1784
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6399918f, 40.2612839f, 1491516557, 18.4704456),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6399994f,
                                                40.2612801f, leg_index, 2.42696953f, 317,
                                                maneuver_index, 1.19081485f, 124));
  //----------------------------------------------------------------
  // trace point = 1785
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6401215f, 40.2614212f, 1491516558, 19.0693913),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6401367f,
                                                40.2614136f, leg_index, 2.41523099f, 316,
                                                maneuver_index, 1.17907631f, 123));
  //----------------------------------------------------------------
  // trace point = 1786
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6402588f, 40.2615623f, 1491516559, 19.4505692),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.640274f,
                                                40.2615547f, leg_index, 2.40307093f, 314,
                                                maneuver_index, 1.16691625f, 121));
  //----------------------------------------------------------------
  // trace point = 1787
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6403961f, 40.2617073f, 1491516560, 19.7454567),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6404114f,
                                                40.2616997f, leg_index, 2.39069748f, 313,
                                                maneuver_index, 1.1545428f, 120));
  //----------------------------------------------------------------
  // trace point = 1788
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6405334f, 40.2618523f, 1491516561, 19.909399),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6405411f,
                                                40.2618484f, leg_index, 2.37723041f, 312,
                                                maneuver_index, 1.14107573f, 119));
  //----------------------------------------------------------------
  // trace point = 1789
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6406784f, 40.2619972f, 1491516562, 20.061573),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.640686f,
                                                40.2619896f, leg_index, 2.36594605f, 311,
                                                maneuver_index, 1.12979138f, 118));
  //----------------------------------------------------------------
  // trace point = 1790
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6408157f, 40.2621346f, 1491516563, 19.4640236),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.640831f,
                                                40.2621269f, leg_index, 2.35375047f, 310,
                                                maneuver_index, 1.11759579f, 117));
  //----------------------------------------------------------------
  // trace point = 1791
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6409454f, 40.2622795f, 1491516564, 19.5625534),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6409683f,
                                                40.2622681f, leg_index, 2.34158874f, 309,
                                                maneuver_index, 1.10543406f, 116));
  //----------------------------------------------------------------
  // trace point = 1792
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6410828f, 40.2624207f, 1491516565, 19.5790787),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.641098f,
                                                40.262413f, leg_index, 2.32944775f, 308,
                                                maneuver_index, 1.09329307f, 115));
  //----------------------------------------------------------------
  // trace point = 1793
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6412201f, 40.2625656f, 1491516566, 19.9990616),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6412277f,
                                                40.262558f, leg_index, 2.31730676f, 306,
                                                maneuver_index, 1.08115208f, 113));
  //----------------------------------------------------------------
  // trace point = 1794
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6413574f, 40.2627144f, 1491516567, 20.0802288),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6413651f,
                                                40.2627106f, leg_index, 2.30450273f, 305,
                                                maneuver_index, 1.06834805f, 112));
  //----------------------------------------------------------------
  // trace point = 1795
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6414948f, 40.2628593f, 1491516568, 19.9822311),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6415024f,
                                                40.2628555f, leg_index, 2.29213357f, 304,
                                                maneuver_index, 1.05597889f, 111));
  //----------------------------------------------------------------
  // trace point = 1796
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6416245f, 40.2630043f, 1491516569, 19.7023945),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6416321f,
                                                40.2630005f, leg_index, 2.27999234f, 302,
                                                maneuver_index, 1.04383767f, 109));
  //----------------------------------------------------------------
  // trace point = 1797
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6417618f, 40.2631454f, 1491516570, 19.4792423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6417618f,
                                                40.2631454f, leg_index, 2.26785254f, 301,
                                                maneuver_index, 1.03169787f, 108));
  //----------------------------------------------------------------
  // trace point = 1798
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6418915f, 40.2632866f, 1491516571, 19.2490158),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6418915f,
                                                40.2632866f, leg_index, 2.25592709f, 300,
                                                maneuver_index, 1.01977241f, 107));
  //----------------------------------------------------------------
  // trace point = 1799
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6420212f, 40.2634277f, 1491516572, 19.0028286),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6420212f,
                                                40.2634277f, leg_index, 2.24400377f, 298,
                                                maneuver_index, 1.0078491f, 105));
  //----------------------------------------------------------------
  // trace point = 1800
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6421509f, 40.2635651f, 1491516573, 18.9193115),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6421432f,
                                                40.2635689f, leg_index, 2.23230839f, 297,
                                                maneuver_index, 0.996153712f, 104));
  //----------------------------------------------------------------
  // trace point = 1801
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6422806f, 40.2637024f, 1491516574, 18.7793293),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6422729f,
                                                40.2637062f, leg_index, 2.22060275f, 296,
                                                maneuver_index, 0.984448075f, 103));
  //----------------------------------------------------------------
  // trace point = 1802
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6424103f, 40.2638359f, 1491516575, 18.4589691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.642395f,
                                                40.2638435f, leg_index, 2.20912433f, 295,
                                                maneuver_index, 0.972969651f, 102));
  //----------------------------------------------------------------
  // trace point = 1803
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.64254f, 40.2639732f, 1491516576, 18.738945),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6425247f,
                                                40.2639847f, leg_index, 2.19719577f, 294,
                                                maneuver_index, 0.961041093f, 101));
  //----------------------------------------------------------------
  // trace point = 1804
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6426697f, 40.2641029f, 1491516577, 18.4471512),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6426544f,
                                                40.2641106f, leg_index, 2.18611979f, 293,
                                                maneuver_index, 0.949965119f, 100));
  //----------------------------------------------------------------
  // trace point = 1805
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6427994f, 40.2642365f, 1491516578, 18.5210686),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6427841f,
                                                40.2642441f, leg_index, 2.17462635f, 291,
                                                maneuver_index, 0.938471675f, 98));
  //----------------------------------------------------------------
  // trace point = 1806
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6429214f, 40.2643661f, 1491516579, 17.561203),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6429138f,
                                                40.26437f, leg_index, 2.16355014f, 290,
                                                maneuver_index, 0.927395463f, 97));
  //----------------------------------------------------------------
  // trace point = 1807
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6430435f, 40.2644958f, 1491516580, 17.8594265),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6430435f,
                                                40.2644958f, leg_index, 2.15247393f, 289,
                                                maneuver_index, 0.916319251f, 96));
  //----------------------------------------------------------------
  // trace point = 1808
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6431656f, 40.2646141f, 1491516581, 17.0319042),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6431656f,
                                                40.2646141f, leg_index, 2.14206219f, 288,
                                                maneuver_index, 0.905907512f, 95));
  //----------------------------------------------------------------
  // trace point = 1809
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.64328f, 40.2647362f, 1491516582, 16.5490322),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6432877f,
                                                40.2647324f, leg_index, 2.13165212f, 287,
                                                maneuver_index, 0.895497441f, 94));
  //----------------------------------------------------------------
  // trace point = 1810
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6433945f, 40.2648544f, 1491516583, 16.2806454),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6434021f,
                                                40.2648506f, leg_index, 2.12148786f, 286,
                                                maneuver_index, 0.88533318f, 93));
  //----------------------------------------------------------------
  // trace point = 1811
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6435089f, 40.2649651f, 1491516584, 15.5946484),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6435242f,
                                                40.2649536f, leg_index, 2.11188293f, 284,
                                                maneuver_index, 0.87572825f, 91));
  //----------------------------------------------------------------
  // trace point = 1812
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6436386f, 40.2650757f, 1491516585, 16.6268368),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6436615f,
                                                40.2650604f, leg_index, 2.10154223f, 283,
                                                maneuver_index, 0.865387559f, 90));
  //----------------------------------------------------------------
  // trace point = 1813
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6437759f, 40.2651901f, 1491516586, 17.0549679),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6437988f,
                                                40.2651749f, leg_index, 2.09080482f, 282,
                                                maneuver_index, 0.85465014f, 89));
  //----------------------------------------------------------------
  // trace point = 1814
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6439285f, 40.2652893f, 1491516587, 17.4431515),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6439438f,
                                                40.2652702f, leg_index, 2.08064032f, 281,
                                                maneuver_index, 0.844485641f, 88));
  //----------------------------------------------------------------
  // trace point = 1815
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6440887f, 40.2653923f, 1491516588, 17.7166309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6441116f,
                                                40.2653732f, leg_index, 2.06925273f, 280,
                                                maneuver_index, 0.833098054f, 87));
  //----------------------------------------------------------------
  // trace point = 1816
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.644249f, 40.2654915f, 1491516589, 17.6997204),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6442719f,
                                                40.2654724f, leg_index, 2.05835152f, 279,
                                                maneuver_index, 0.822196841f, 86));
  //----------------------------------------------------------------
  // trace point = 1817
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6444244f, 40.2655869f, 1491516590, 17.8986397),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6444397f,
                                                40.2655754f, leg_index, 2.046983f, 278,
                                                maneuver_index, 0.810828328f, 85));
  //----------------------------------------------------------------
  // trace point = 1818
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6445847f, 40.2656784f, 1491516591, 17.1535683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6445923f,
                                                40.2656708f, leg_index, 2.03657341f, 277,
                                                maneuver_index, 0.800418735f, 84));
  //----------------------------------------------------------------
  // trace point = 1819
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6447449f, 40.26577f, 1491516592, 17.0873241),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6447525f,
                                                40.2657661f, leg_index, 2.02584887f, 276,
                                                maneuver_index, 0.78969419f, 83));
  //----------------------------------------------------------------
  // trace point = 1820
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6449051f, 40.2658615f, 1491516593, 16.9529533),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6449051f,
                                                40.2658615f, leg_index, 2.01543951f, 275,
                                                maneuver_index, 0.779284835f, 82));
  //----------------------------------------------------------------
  // trace point = 1821
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6450729f, 40.2659531f, 1491516594, 17.1557751),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6450653f,
                                                40.2659569f, leg_index, 2.00471497f, 274,
                                                maneuver_index, 0.76856029f, 81));
  //----------------------------------------------------------------
  // trace point = 1822
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6452408f, 40.2660446f, 1491516595, 17.6957302),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6452332f,
                                                40.2660561f, leg_index, 1.99350667f, 273,
                                                maneuver_index, 0.757351995f, 80));
  //----------------------------------------------------------------
  // trace point = 1823
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6454163f, 40.26614f, 1491516596, 18.2544308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6454086f,
                                                40.2661514f, leg_index, 1.98213673f, 272,
                                                maneuver_index, 0.745982051f, 79));
  //----------------------------------------------------------------
  // trace point = 1824
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6455917f, 40.266243f, 1491516597, 18.7059975),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6455841f,
                                                40.2662506f, leg_index, 1.97061086f, 271,
                                                maneuver_index, 0.734456182f, 78));
  //----------------------------------------------------------------
  // trace point = 1825
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6457672f, 40.2663422f, 1491516598, 18.8441429),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6457596f,
                                                40.2663498f, leg_index, 1.95908523f, 269,
                                                maneuver_index, 0.722930551f, 76));
  //----------------------------------------------------------------
  // trace point = 1826
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6459427f, 40.2664413f, 1491516599, 18.6380043),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6459351f,
                                                40.266449f, leg_index, 1.9475596f, 268,
                                                maneuver_index, 0.71140492f, 75));
  //----------------------------------------------------------------
  // trace point = 1827
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6461258f, 40.2665443f, 1491516600, 18.9127102),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6461182f,
                                                40.266552f, leg_index, 1.93555319f, 267,
                                                maneuver_index, 0.699398518f, 74));
  //----------------------------------------------------------------
  // trace point = 1828
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.646286f, 40.2666359f, 1491516601, 17.4227905),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6462784f,
                                                40.2666397f, leg_index, 1.92514467f, 266,
                                                maneuver_index, 0.688989997f, 73));
  //----------------------------------------------------------------
  // trace point = 1829
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6464462f, 40.2667236f, 1491516602, 16.4210243),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6464386f,
                                                40.2667313f, leg_index, 1.91458082f, 265,
                                                maneuver_index, 0.678426147f, 72));
  //----------------------------------------------------------------
  // trace point = 1830
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6465836f, 40.2667961f, 1491516603, 14.6635361),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6465759f,
                                                40.2668037f, leg_index, 1.90577042f, 264,
                                                maneuver_index, 0.669615746f, 71));
  //----------------------------------------------------------------
  // trace point = 1831
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6467209f, 40.2668648f, 1491516604, 13.679266),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6467056f,
                                                40.26688f, leg_index, 1.89712989f, 263,
                                                maneuver_index, 0.660975218f, 70));
  //----------------------------------------------------------------
  // trace point = 1832
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6468506f, 40.2669334f, 1491516605, 13.1784763),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6468353f,
                                                40.2669487f, leg_index, 1.88880026f, 262,
                                                maneuver_index, 0.652645588f, 69));
  //----------------------------------------------------------------
  // trace point = 1833
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.646965f, 40.2670059f, 1491516606, 12.7780104),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6469574f,
                                                40.2670174f, leg_index, 1.88079596f, 261,
                                                maneuver_index, 0.64464128f, 68));
  //----------------------------------------------------------------
  // trace point = 1834
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6470947f, 40.2670784f, 1491516607, 13.710248),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6470871f,
                                                40.2670898f, leg_index, 1.87231088f, 260,
                                                maneuver_index, 0.636156201f, 67));
  //----------------------------------------------------------------
  // trace point = 1835
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6472244f, 40.2671547f, 1491516608, 14.050313),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6472168f,
                                                40.2671623f, leg_index, 1.8638258f, 260,
                                                maneuver_index, 0.627671123f, 67));
  //----------------------------------------------------------------
  // trace point = 1836
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.647377f, 40.2672424f, 1491516609, 15.8020201),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6473694f,
                                                40.2672501f, leg_index, 1.85374308f, 259,
                                                maneuver_index, 0.617588401f, 66));
  //----------------------------------------------------------------
  // trace point = 1837
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.647522f, 40.2673302f, 1491516610, 16.3440132),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.647522f,
                                                40.2673302f, leg_index, 1.84397125f, 258,
                                                maneuver_index, 0.607816577f, 65));
  //----------------------------------------------------------------
  // trace point = 1838
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6476822f, 40.2674255f, 1491516611, 17.1533985),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6476822f,
                                                40.2674217f, leg_index, 1.83340681f, 256,
                                                maneuver_index, 0.597252131f, 63));
  //----------------------------------------------------------------
  // trace point = 1839
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.64785f, 40.2675247f, 1491516612, 17.5581856),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6478577f,
                                                40.2675209f, leg_index, 1.82188118f, 255,
                                                maneuver_index, 0.5857265f, 62));
  //----------------------------------------------------------------
  // trace point = 1840
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6480179f, 40.2676201f, 1491516613, 17.8949718),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6480179f,
                                                40.2676163f, leg_index, 1.81115687f, 254,
                                                maneuver_index, 0.575002193f, 61));
  //----------------------------------------------------------------
  // trace point = 1841
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6481705f, 40.2677155f, 1491516614, 16.8824844),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6481781f,
                                                40.2677078f, leg_index, 1.80059266f, 253,
                                                maneuver_index, 0.564437985f, 60));
  //----------------------------------------------------------------
  // trace point = 1842
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6483307f, 40.2678032f, 1491516615, 17.0186615),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6483307f,
                                                40.2677994f, leg_index, 1.79034901f, 252,
                                                maneuver_index, 0.554194331f, 59));
  //----------------------------------------------------------------
  // trace point = 1843
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6484985f, 40.2678909f, 1491516616, 16.8268127),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6484985f,
                                                40.2678947f, leg_index, 1.77930415f, 251,
                                                maneuver_index, 0.543149471f, 58));
  //----------------------------------------------------------------
  // trace point = 1844
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6486511f, 40.2679787f, 1491516617, 16.6841812),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6486511f,
                                                40.2679787f, leg_index, 1.76937032f, 249,
                                                maneuver_index, 0.533215642f, 56));
  //----------------------------------------------------------------
  // trace point = 1845
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6488113f, 40.2680702f, 1491516618, 16.8183327),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.648819f,
                                                40.2680626f, leg_index, 1.75878f, 248, maneuver_index,
                                                0.522625327f, 55));
  //----------------------------------------------------------------
  // trace point = 1846
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6489639f, 40.2681503f, 1491516619, 15.8175564),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6489716f,
                                                40.2681427f, leg_index, 1.7490052f, 247,
                                                maneuver_index, 0.512850523f, 54));
  //----------------------------------------------------------------
  // trace point = 1847
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6491089f, 40.2682266f, 1491516620, 14.8721848),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6491165f,
                                                40.268219f, leg_index, 1.73971128f, 246,
                                                maneuver_index, 0.503556609f, 53));
  //----------------------------------------------------------------
  // trace point = 1848
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6492462f, 40.2682991f, 1491516621, 14.1881247),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6492462f,
                                                40.2682953f, leg_index, 1.73107052f, 245,
                                                maneuver_index, 0.494915843f, 52));
  //----------------------------------------------------------------
  // trace point = 1849
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6493835f, 40.2683716f, 1491516622, 13.9207811),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6493835f,
                                                40.2683716f, leg_index, 1.72210479f, 244,
                                                maneuver_index, 0.485950112f, 51));
  //----------------------------------------------------------------
  // trace point = 1850
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6495132f, 40.2684441f, 1491516623, 13.9968491),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6495132f,
                                                40.2684441f, leg_index, 1.71361983f, 243,
                                                maneuver_index, 0.477465153f, 50));
  //----------------------------------------------------------------
  // trace point = 1851
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6496506f, 40.2685165f, 1491516624, 14.2573366),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6496506f,
                                                40.2685204f, leg_index, 1.70465398f, 242,
                                                maneuver_index, 0.468499303f, 49));
  //----------------------------------------------------------------
  // trace point = 1852
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6498032f, 40.2685928f, 1491516625, 15.0121536),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6497955f,
                                                40.2686005f, leg_index, 1.69520748f, 241,
                                                maneuver_index, 0.459052801f, 48));
  //----------------------------------------------------------------
  // trace point = 1853
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6499329f, 40.2686691f, 1491516626, 14.2482805),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6499252f,
                                                40.2686729f, leg_index, 1.68672252f, 240,
                                                maneuver_index, 0.450567842f, 47));
  //----------------------------------------------------------------
  // trace point = 1854
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6500626f, 40.2687492f, 1491516627, 14.1810207),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6500626f,
                                                40.2687492f, leg_index, 1.67775679f, 239,
                                                maneuver_index, 0.441602111f, 46));
  //----------------------------------------------------------------
  // trace point = 1855
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6501999f, 40.2688255f, 1491516628, 14.248456),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6501999f,
                                                40.2688255f, leg_index, 1.66879106f, 238,
                                                maneuver_index, 0.43263638f, 45));
  //----------------------------------------------------------------
  // trace point = 1856
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6503372f, 40.2689018f, 1491516629, 14.1189709),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6503372f,
                                                40.2689018f, leg_index, 1.65982533f, 238,
                                                maneuver_index, 0.42367065f, 45));
  //----------------------------------------------------------------
  // trace point = 1857
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6504669f, 40.2689781f, 1491516630, 14.249423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6504669f,
                                                40.2689743f, leg_index, 1.65134037f, 237,
                                                maneuver_index, 0.41518569f, 44));
  //----------------------------------------------------------------
  // trace point = 1858
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6506042f, 40.2690582f, 1491516631, 14.7208595),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6506119f,
                                                40.2690544f, leg_index, 1.64189386f, 236,
                                                maneuver_index, 0.405739188f, 43));
  //----------------------------------------------------------------
  // trace point = 1859
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6507416f, 40.2691383f, 1491516632, 14.5872383),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6507492f,
                                                40.2691307f, leg_index, 1.63292813f, 235,
                                                maneuver_index, 0.396773458f, 42));
  //----------------------------------------------------------------
  // trace point = 1860
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6508865f, 40.2692146f, 1491516633, 14.7927618),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6508865f,
                                                40.2692108f, leg_index, 1.62380672f, 234,
                                                maneuver_index, 0.38765204f, 41));
  //----------------------------------------------------------------
  // trace point = 1861
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6510239f, 40.2692947f, 1491516634, 14.9956284),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6510315f,
                                                40.2692871f, leg_index, 1.61451602f, 233,
                                                maneuver_index, 0.378361344f, 40));
  //----------------------------------------------------------------
  // trace point = 1862
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6511688f, 40.2693748f, 1491516635, 15.1955433),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6511765f,
                                                40.269371f, leg_index, 1.60491371f, 232,
                                                maneuver_index, 0.368759036f, 39));
  //----------------------------------------------------------------
  // trace point = 1863
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6513138f, 40.269455f, 1491516636, 15.0591326),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6513214f,
                                                40.2694511f, leg_index, 1.59546733f, 231,
                                                maneuver_index, 0.359312654f, 38));
  //----------------------------------------------------------------
  // trace point = 1864
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6514587f, 40.2695389f, 1491516637, 15.3286791),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6514664f,
                                                40.2695312f, leg_index, 1.58602083f, 230,
                                                maneuver_index, 0.349866152f, 37));
  //----------------------------------------------------------------
  // trace point = 1865
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6516037f, 40.2696228f, 1491516638, 15.8724728),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6516113f,
                                                40.2696152f, leg_index, 1.57641864f, 229,
                                                maneuver_index, 0.340263963f, 36));
  //----------------------------------------------------------------
  // trace point = 1866
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6517563f, 40.2697105f, 1491516639, 15.9366474),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6517639f,
                                                40.2696991f, leg_index, 1.56649137f, 228,
                                                maneuver_index, 0.33033669f, 35));
  //----------------------------------------------------------------
  // trace point = 1867
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6519012f, 40.2697868f, 1491516640, 14.9418859),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6519089f,
                                                40.2697792f, leg_index, 1.55704498f, 227,
                                                maneuver_index, 0.320890307f, 34));
  //----------------------------------------------------------------
  // trace point = 1868
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6520615f, 40.2698746f, 1491516641, 16.6794395),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6520691f,
                                                40.2698669f, leg_index, 1.54663694f, 226,
                                                maneuver_index, 0.310482264f, 33));
  //----------------------------------------------------------------
  // trace point = 1869
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6522141f, 40.2699623f, 1491516642, 16.2073193),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6522217f,
                                                40.2699547f, leg_index, 1.53655398f, 225,
                                                maneuver_index, 0.300399303f, 32));
  //----------------------------------------------------------------
  // trace point = 1870
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6523666f, 40.2700462f, 1491516643, 16.0727272),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6523743f,
                                                40.2700386f, leg_index, 1.52662683f, 224,
                                                maneuver_index, 0.29047215f, 31));
  //----------------------------------------------------------------
  // trace point = 1871 | Alert | In a quarter mile Bear left onto Hockersville Road.
  maneuver_index = 9;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6525192f, 40.2701302f, 1491516644, 15.9411068),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.6525269f, 40.2701225f, leg_index, 1.51669967f,
                                                223, maneuver_index, 0.280544996f, 30,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1872
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6526642f, 40.2702179f, 1491516645, 15.8723822),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6526718f,
                                                40.2702065f, leg_index, 1.50709748f, 222,
                                                maneuver_index, 0.270942807f, 29));
  //----------------------------------------------------------------
  // trace point = 1873
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6528168f, 40.2703018f, 1491516646, 15.599122),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6528244f,
                                                40.2702904f, leg_index, 1.49717033f, 221,
                                                maneuver_index, 0.261015654f, 28));
  //----------------------------------------------------------------
  // trace point = 1874
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6529541f, 40.2703781f, 1491516647, 14.9304581),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6529617f,
                                                40.2703667f, leg_index, 1.48820472f, 220,
                                                maneuver_index, 0.252050042f, 27));
  //----------------------------------------------------------------
  // trace point = 1875
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6530991f, 40.2704582f, 1491516648, 15.1259069),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6531067f,
                                                40.2704468f, leg_index, 1.47875834f, 219,
                                                maneuver_index, 0.24260366f, 26));
  //----------------------------------------------------------------
  // trace point = 1876
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.653244f, 40.2705421f, 1491516649, 15.4634523),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6532516f,
                                                40.2705307f, leg_index, 1.46915627f, 218,
                                                maneuver_index, 0.23300159f, 25));
  //----------------------------------------------------------------
  // trace point = 1877
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6533966f, 40.2706337f, 1491516650, 16.3417778),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6534119f,
                                                40.2706184f, leg_index, 1.45874822f, 217,
                                                maneuver_index, 0.222593546f, 24));
  //----------------------------------------------------------------
  // trace point = 1878
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6535416f, 40.2707253f, 1491516651, 15.876235),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6535645f,
                                                40.2707024f, leg_index, 1.44882119f, 216,
                                                maneuver_index, 0.212666512f, 23));
  //----------------------------------------------------------------
  // trace point = 1879
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6536942f, 40.2708015f, 1491516652, 15.6291666),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6537094f,
                                                40.2707863f, leg_index, 1.43921924f, 215,
                                                maneuver_index, 0.203064561f, 22));
  //----------------------------------------------------------------
  // trace point = 1880
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6538467f, 40.2708817f, 1491516653, 15.4748688),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.653862f,
                                                40.2708664f, leg_index, 1.42944789f, 214,
                                                maneuver_index, 0.193293214f, 21));
  //----------------------------------------------------------------
  // trace point = 1881
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6539917f, 40.2709656f, 1491516654, 15.669425),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.654007f,
                                                40.2709503f, leg_index, 1.41984558f, 213,
                                                maneuver_index, 0.183690906f, 20));
  //----------------------------------------------------------------
  // trace point = 1882
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6541443f, 40.2710457f, 1491516655, 15.4110498),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6541595f,
                                                40.2710342f, leg_index, 1.40991843f, 212,
                                                maneuver_index, 0.173763752f, 19));
  //----------------------------------------------------------------
  // trace point = 1883
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6542892f, 40.271122f, 1491516656, 15.1332827),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6542969f,
                                                40.2711105f, leg_index, 1.40095294f, 211,
                                                maneuver_index, 0.16479826f, 18));
  //----------------------------------------------------------------
  // trace point = 1884
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6544266f, 40.2712021f, 1491516657, 14.8561792),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6544342f,
                                                40.2711906f, leg_index, 1.39182997f, 210,
                                                maneuver_index, 0.155675292f, 17));
  //----------------------------------------------------------------
  // trace point = 1885
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6545563f, 40.2712784f, 1491516658, 14.1814356),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6545715f,
                                                40.2712669f, leg_index, 1.38286579f, 209,
                                                maneuver_index, 0.146711111f, 16));
  //----------------------------------------------------------------
  // trace point = 1886
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6546936f, 40.2713547f, 1491516659, 14.0454588),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6547012f,
                                                40.2713432f, leg_index, 1.37422299f, 208,
                                                maneuver_index, 0.138068318f, 15));
  //----------------------------------------------------------------
  // trace point = 1887
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6548233f, 40.271431f, 1491516660, 13.8444185),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6548309f,
                                                40.2714272f, leg_index, 1.365237f, 207,
                                                maneuver_index, 0.129082322f, 14));
  //----------------------------------------------------------------
  // trace point = 1888
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6549301f, 40.2715149f, 1491516661, 13.183979),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6549377f,
                                                40.2715073f, leg_index, 1.35733986f, 206,
                                                maneuver_index, 0.121185184f, 13));
  //----------------------------------------------------------------
  // trace point = 1889
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6550369f, 40.2715988f, 1491516662, 12.9686289),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6550446f,
                                                40.2715912f, leg_index, 1.34921539f, 205,
                                                maneuver_index, 0.113060713f, 12));
  //----------------------------------------------------------------
  // trace point = 1890
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6551208f, 40.2716751f, 1491516663, 11.2058315),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6551361f,
                                                40.2716675f, leg_index, 1.34206617f, 205,
                                                maneuver_index, 0.105911493f, 12));
  //----------------------------------------------------------------
  // trace point = 1891
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6552277f, 40.2717857f, 1491516665, 7.59934616),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6552353f,
                                                40.2717819f, leg_index, 1.33251047f, 203,
                                                maneuver_index, 0.0963557959f, 10));
  //----------------------------------------------------------------
  // trace point = 1892
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6552963f, 40.2718582f, 1491516667, 4.98130798),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6552811f,
                                                40.271862f, leg_index, 1.32646692f, 203,
                                                maneuver_index, 0.0903122425f, 10));
  //----------------------------------------------------------------
  // trace point = 1893
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6553268f, 40.2719421f, 1491516671, 2.36700606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6553268f,
                                                40.2719421f, leg_index, 1.31977499f, 202,
                                                maneuver_index, 0.0836203098f, 9));
  //----------------------------------------------------------------
  // trace point = 1894
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.655365f, 40.2720299f, 1491516673, 5.117805),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6553726f,
                                                40.2720299f, leg_index, 1.31387949f, 201,
                                                maneuver_index, 0.0777248144f, 8));
  //----------------------------------------------------------------
  // trace point = 1895
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6554184f, 40.2721481f, 1491516675, 7.16726589),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.655426f,
                                                40.2721443f, leg_index, 1.30547774f, 200,
                                                maneuver_index, 0.0693230629f, 7));
  //----------------------------------------------------------------
  // trace point = 1896
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6554565f, 40.2722282f, 1491516676, 9.47237492),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6554718f,
                                                40.2722244f, leg_index, 1.29943061f, 200,
                                                maneuver_index, 0.0632759333f, 7));
  //----------------------------------------------------------------
  // trace point = 1897
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6554947f, 40.2723083f, 1491516677, 9.23411846),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6555099f,
                                                40.2723045f, leg_index, 1.2935431f, 199,
                                                maneuver_index, 0.0573884249f, 6));
  //----------------------------------------------------------------
  // trace point = 1898
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6555328f, 40.2723923f, 1491516678, 9.81199074),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6555557f,
                                                40.2723846f, leg_index, 1.28749835f, 198,
                                                maneuver_index, 0.0513436794f, 5));
  //----------------------------------------------------------------
  // trace point = 1899
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6555634f, 40.2724724f, 1491516679, 9.20465374),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6556015f,
                                                40.2724609f, leg_index, 1.28169668f, 198,
                                                maneuver_index, 0.0455420017f, 5));
  //----------------------------------------------------------------
  // trace point = 1900
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6556396f, 40.2726135f, 1491516681, 8.58882618),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6556778f,
                                                40.2726021f, leg_index, 1.2711364f, 197,
                                                maneuver_index, 0.0349817276f, 4));
  //----------------------------------------------------------------
  // trace point = 1901
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6556931f, 40.2727203f, 1491516683, 6.44907808),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6557388f,
                                                40.2727051f, leg_index, 1.26332021f, 196,
                                                maneuver_index, 0.0271655321f, 3));
  //----------------------------------------------------------------
  // trace point = 1902
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6557388f, 40.2728004f, 1491516685, 4.76071215),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6557846f,
                                                40.2727852f, leg_index, 1.25727618f, 195,
                                                maneuver_index, 0.0211215019f, 2));
  //----------------------------------------------------------------
  // trace point = 1903 | Pre | Bear left onto Hockersville Road.
  maneuver_index = 9;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.655777f, 40.2728767f, 1491516695, 0.911692977),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6558228f, 40.2728615f, leg_index, 1.25163054f,
                                                195, maneuver_index, 0.0154758692f, 2,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1904
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6558304f, 40.2729645f, 1491516713, 0.598846018),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6558685f,
                                                40.2729492f, leg_index, 1.24510121f, 194,
                                                maneuver_index, 0.00894653797f, 1));
  //----------------------------------------------------------------
  // trace point = 1905
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6559067f, 40.2730675f, 1491516715, 6.59053183),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6559296f,
                                                40.2730598f, leg_index, 1.23615468f, 193,
                                                maneuver_index, 0.572380662f, 91));
  //----------------------------------------------------------------
  // trace point = 1906
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6560593f, 40.2731667f, 1491516717, 8.29490757),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6560745f,
                                                40.2731514f, leg_index, 1.22686887f, 192,
                                                maneuver_index, 0.563094854f, 90));
  //----------------------------------------------------------------
  // trace point = 1907 | Post | Continue for 6 tenths of a mile.
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6561432f, 40.2732201f, 1491516718, 9.53092003),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.6561584f, 40.2732048f, leg_index, 1.22110307f,
                                                191, maneuver_index, 0.557329059f, 89,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1908
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6562424f, 40.2732735f, 1491516719, 10.4053869),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6562576f,
                                                40.273262f, leg_index, 1.21454263f, 190,
                                                maneuver_index, 0.550768614f, 88));
  //----------------------------------------------------------------
  // trace point = 1909
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6563492f, 40.2733421f, 1491516720, 11.8990698),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6563644f,
                                                40.2733269f, leg_index, 1.20733869f, 189,
                                                maneuver_index, 0.543564677f, 87));
  //----------------------------------------------------------------
  // trace point = 1910
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6564636f, 40.2734146f, 1491516721, 12.640645),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6564789f,
                                                40.2733994f, leg_index, 1.19949114f, 187,
                                                maneuver_index, 0.53571713f, 85));
  //----------------------------------------------------------------
  // trace point = 1911
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6565857f, 40.2734909f, 1491516722, 13.5731001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.656601f,
                                                40.2734718f, leg_index, 1.19132817f, 186,
                                                maneuver_index, 0.527554154f, 84));
  //----------------------------------------------------------------
  // trace point = 1912
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.656723f, 40.273571f, 1491516723, 14.3843985),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6567383f,
                                                40.2735558f, leg_index, 1.18204236f, 185,
                                                maneuver_index, 0.518268347f, 83));
  //----------------------------------------------------------------
  // trace point = 1913
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6568604f, 40.2736588f, 1491516724, 15.471837),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6568756f,
                                                40.2736435f, leg_index, 1.17258954f, 183,
                                                maneuver_index, 0.508815527f, 81));
  //----------------------------------------------------------------
  // trace point = 1914
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6570053f, 40.2737427f, 1491516725, 15.2612305),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6570206f,
                                                40.2737312f, leg_index, 1.16282594f, 182,
                                                maneuver_index, 0.499051929f, 80));
  //----------------------------------------------------------------
  // trace point = 1915
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6571503f, 40.2738266f, 1491516726, 15.5979824),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6571579f,
                                                40.273819f, leg_index, 1.15337336f, 180,
                                                maneuver_index, 0.489599347f, 78));
  //----------------------------------------------------------------
  // trace point = 1916
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6572952f, 40.2739143f, 1491516727, 15.6671743),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6573029f,
                                                40.2739105f, leg_index, 1.14344227f, 178,
                                                maneuver_index, 0.47966826f, 76));
  //----------------------------------------------------------------
  // trace point = 1917
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6574326f, 40.2740097f, 1491516728, 15.703928),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6574402f,
                                                40.2739983f, leg_index, 1.13398969f, 177,
                                                maneuver_index, 0.470215678f, 75));
  //----------------------------------------------------------------
  // trace point = 1918
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6575623f, 40.2740936f, 1491516729, 14.5278034),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6575775f,
                                                40.2740822f, leg_index, 1.1247046f, 175,
                                                maneuver_index, 0.460930586f, 73));
  //----------------------------------------------------------------
  // trace point = 1919
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.657692f, 40.2741814f, 1491516730, 14.750803),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6577072f,
                                                40.2741661f, leg_index, 1.11573076f, 174,
                                                maneuver_index, 0.451956749f, 72));
  //----------------------------------------------------------------
  // trace point = 1920
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6578293f, 40.2742729f, 1491516731, 15.6287508),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6578522f,
                                                40.2742538f, leg_index, 1.10596716f, 172,
                                                maneuver_index, 0.442193151f, 70));
  //----------------------------------------------------------------
  // trace point = 1921
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6579666f, 40.2743607f, 1491516732, 14.7507811),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6579895f,
                                                40.2743416f, leg_index, 1.09651458f, 171,
                                                maneuver_index, 0.432740569f, 69));
  //----------------------------------------------------------------
  // trace point = 1922
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6581116f, 40.2744484f, 1491516733, 15.9384003),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6581345f,
                                                40.2744293f, leg_index, 1.08675098f, 169,
                                                maneuver_index, 0.422976971f, 67));
  //----------------------------------------------------------------
  // trace point = 1923
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6582489f, 40.2745323f, 1491516734, 14.7889423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6582642f,
                                                40.2745171f, leg_index, 1.07760978f, 168,
                                                maneuver_index, 0.413835764f, 66));
  //----------------------------------------------------------------
  // trace point = 1924
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6583786f, 40.2746162f, 1491516735, 14.8005533),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6584015f,
                                                40.2745972f, leg_index, 1.06849217f, 166,
                                                maneuver_index, 0.404718161f, 64));
  //----------------------------------------------------------------
  // trace point = 1925
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6585159f, 40.2747078f, 1491516736, 15.1846056),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6585388f,
                                                40.2746849f, leg_index, 1.05903959f, 165,
                                                maneuver_index, 0.395265579f, 63));
  //----------------------------------------------------------------
  // trace point = 1926
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6586533f, 40.2747993f, 1491516737, 15.894309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6586838f,
                                                40.2747726f, leg_index, 1.04927623f, 163,
                                                maneuver_index, 0.385502219f, 61));
  //----------------------------------------------------------------
  // trace point = 1927
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6587982f, 40.2748909f, 1491516738, 15.8806314),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6588287f,
                                                40.2748642f, leg_index, 1.03934503f, 162,
                                                maneuver_index, 0.375571012f, 60));
  //----------------------------------------------------------------
  // trace point = 1928
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6589508f, 40.2749786f, 1491516739, 15.8003883),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6589737f,
                                                40.2749557f, leg_index, 1.02941394f, 160,
                                                maneuver_index, 0.365639925f, 58));
  //----------------------------------------------------------------
  // trace point = 1929
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6590881f, 40.2750664f, 1491516740, 15.7442207),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.659111f,
                                                40.2750435f, leg_index, 1.01996148f, 159,
                                                maneuver_index, 0.356187463f, 57));
  //----------------------------------------------------------------
  // trace point = 1930
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6592255f, 40.2751465f, 1491516741, 14.3175926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6592484f,
                                                40.2751274f, leg_index, 1.01067662f, 157,
                                                maneuver_index, 0.346902609f, 55));
  //----------------------------------------------------------------
  // trace point = 1931
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6593704f, 40.2752457f, 1491516742, 16.2412891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6593933f,
                                                40.2752228f, leg_index, 1.00057805f, 155,
                                                maneuver_index, 0.336804032f, 53));
  //----------------------------------------------------------------
  // trace point = 1932
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.659523f, 40.275341f, 1491516743, 16.9666786),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6595459f,
                                                40.2753181f, leg_index, 0.990168452f, 154,
                                                maneuver_index, 0.326394439f, 52));
  //----------------------------------------------------------------
  // trace point = 1933
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6596832f, 40.2754364f, 1491516744, 17.28619),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6597061f,
                                                40.2754173f, leg_index, 0.979280055f, 152,
                                                maneuver_index, 0.315506041f, 50));
  //----------------------------------------------------------------
  // trace point = 1934
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6598434f, 40.2755356f, 1491516745, 17.3566971),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6598663f,
                                                40.2755165f, leg_index, 0.968391836f, 150,
                                                maneuver_index, 0.304617822f, 48));
  //----------------------------------------------------------------
  // trace point = 1935
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6600037f, 40.2756424f, 1491516746, 18.2626305),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6600266f,
                                                40.2756195f, leg_index, 0.957336783f, 148,
                                                maneuver_index, 0.29356277f, 46));
  //----------------------------------------------------------------
  // trace point = 1936
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6601562f, 40.2757454f, 1491516747, 17.3297482),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6601868f,
                                                40.2757187f, leg_index, 0.946447432f, 147,
                                                maneuver_index, 0.282673419f, 45));
  //----------------------------------------------------------------
  // trace point = 1937 | Alert | In a quarter mile Turn right onto U.S. 4 22.
  maneuver_index = 10;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6603088f, 40.2758408f, 1491516748, 16.6880398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.6603394f, 40.2758141f, leg_index, 0.936038792f,
                                                145, maneuver_index, 0.272264779f, 43,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1938
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6604691f, 40.2759514f, 1491516749, 18.2549171),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6604996f,
                                                40.2759209f, leg_index, 0.924812376f, 143,
                                                maneuver_index, 0.261038363f, 41));
  //----------------------------------------------------------------
  // trace point = 1939
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.660614f, 40.2760391f, 1491516750, 15.8679295),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6606445f,
                                                40.2760124f, leg_index, 0.914881587f, 142,
                                                maneuver_index, 0.251107574f, 40));
  //----------------------------------------------------------------
  // trace point = 1940
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.660759f, 40.2761307f, 1491516751, 16.0116386),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6607895f,
                                                40.276104f, leg_index, 0.904950857f, 140,
                                                maneuver_index, 0.241176844f, 38));
  //----------------------------------------------------------------
  // trace point = 1941
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6609039f, 40.2762222f, 1491516752, 16.2899704),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6609344f,
                                                40.2761955f, leg_index, 0.895020187f, 139,
                                                maneuver_index, 0.231246173f, 37));
  //----------------------------------------------------------------
  // trace point = 1942
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6610489f, 40.2762947f, 1491516753, 14.3317986),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6610641f,
                                                40.2762794f, leg_index, 0.886045456f, 137,
                                                maneuver_index, 0.222271442f, 35));
  //----------------------------------------------------------------
  // trace point = 1943
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6611862f, 40.2763824f, 1491516754, 15.1292906),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6612015f,
                                                40.2763672f, leg_index, 0.876592755f, 136,
                                                maneuver_index, 0.212818742f, 34));
  //----------------------------------------------------------------
  // trace point = 1944
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6613235f, 40.2764626f, 1491516755, 14.9910421),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6613312f,
                                                40.2764549f, leg_index, 0.867448151f, 134,
                                                maneuver_index, 0.203674138f, 32));
  //----------------------------------------------------------------
  // trace point = 1945
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6614609f, 40.2765465f, 1491516756, 14.8555822),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6614685f,
                                                40.2765388f, leg_index, 0.858165264f, 133,
                                                maneuver_index, 0.194391251f, 31));
  //----------------------------------------------------------------
  // trace point = 1946
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6616058f, 40.2766342f, 1491516757, 15.6001158),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6616058f,
                                                40.2766304f, leg_index, 0.848542571f, 131,
                                                maneuver_index, 0.184768558f, 29));
  //----------------------------------------------------------------
  // trace point = 1947
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6617432f, 40.2767334f, 1491516758, 16.1968384),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6617508f,
                                                40.276722f, leg_index, 0.838611901f, 129,
                                                maneuver_index, 0.174837887f, 27));
  //----------------------------------------------------------------
  // trace point = 1948
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6618881f, 40.2768364f, 1491516759, 16.6881447),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6619034f,
                                                40.2768211f, leg_index, 0.828033686f, 128,
                                                maneuver_index, 0.164259672f, 26));
  //----------------------------------------------------------------
  // trace point = 1949
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6620331f, 40.2769241f, 1491516760, 16.0702991),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6620483f,
                                                40.2769127f, leg_index, 0.818102717f, 126,
                                                maneuver_index, 0.154328704f, 24));
  //----------------------------------------------------------------
  // trace point = 1950
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6621933f, 40.2770271f, 1491516761, 17.3843765),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6622086f,
                                                40.2770157f, leg_index, 0.807046652f, 125,
                                                maneuver_index, 0.143272638f, 23));
  //----------------------------------------------------------------
  // trace point = 1951
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6623459f, 40.2771187f, 1491516762, 16.4079762),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6623611f,
                                                40.2771072f, leg_index, 0.796805084f, 123,
                                                maneuver_index, 0.13303107f, 21));
  //----------------------------------------------------------------
  // trace point = 1952
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6624985f, 40.2772141f, 1491516763, 16.8897915),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6625061f,
                                                40.2772026f, leg_index, 0.786706388f, 121,
                                                maneuver_index, 0.122932374f, 19));
  //----------------------------------------------------------------
  // trace point = 1953
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6626434f, 40.2773018f, 1491516764, 15.6733074),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6626511f,
                                                40.2772942f, leg_index, 0.776776016f, 120,
                                                maneuver_index, 0.113002002f, 18));
  //----------------------------------------------------------------
  // trace point = 1954
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6627808f, 40.2773895f, 1491516765, 15.1990232),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6627884f,
                                                40.2773781f, leg_index, 0.767491698f, 118,
                                                maneuver_index, 0.103717685f, 16));
  //----------------------------------------------------------------
  // trace point = 1955
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6629181f, 40.2774696f, 1491516766, 15.0572805),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6629257f,
                                                40.2774582f, leg_index, 0.758366346f, 117,
                                                maneuver_index, 0.0945923328f, 15));
  //----------------------------------------------------------------
  // trace point = 1956
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6630554f, 40.277523f, 1491516767, 12.6682596),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6630478f,
                                                40.2775307f, leg_index, 0.750204921f, 116,
                                                maneuver_index, 0.0864309072f, 14));
  //----------------------------------------------------------------
  // trace point = 1957
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6631927f, 40.2775917f, 1491516768, 14.2158813),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6631775f,
                                                40.2776031f, leg_index, 0.741721034f, 114,
                                                maneuver_index, 0.0779470205f, 12));
  //----------------------------------------------------------------
  // trace point = 1958 | Pre | Turn right onto U.S. 4 22, West Chocolate Avenue.
  maneuver_index = 10;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6633224f, 40.2776756f, 1491516769, 14.4683704),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6633148f, 40.2776833f, leg_index, 0.732598782f,
                                                113, maneuver_index, 0.0688247681f, 11,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1959
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6634445f, 40.2777519f, 1491516770, 13.4390564),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6634445f,
                                                40.2777557f, leg_index, 0.724115014f, 111,
                                                maneuver_index, 0.0603410006f, 9));
  //----------------------------------------------------------------
  // trace point = 1960
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6635666f, 40.2778206f, 1491516771, 12.6264963),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6635666f,
                                                40.2778244f, leg_index, 0.71611172f, 110,
                                                maneuver_index, 0.0523377061f, 8));
  //----------------------------------------------------------------
  // trace point = 1961
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6636734f, 40.2778854f, 1491516772, 11.6826954),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6636734f,
                                                40.2778854f, leg_index, 0.70906961f, 109,
                                                maneuver_index, 0.0452955961f, 7));
  //----------------------------------------------------------------
  // trace point = 1962
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.663765f, 40.2779388f, 1491516773, 9.65567493),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.663765f,
                                                40.2779388f, leg_index, 0.702987671f, 108,
                                                maneuver_index, 0.0392136574f, 6));
  //----------------------------------------------------------------
  // trace point = 1963
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6638336f, 40.2780075f, 1491516774, 9.55908966),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6638489f,
                                                40.2779884f, leg_index, 0.697386801f, 107,
                                                maneuver_index, 0.0336127877f, 5));
  //----------------------------------------------------------------
  // trace point = 1964
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6639099f, 40.2780647f, 1491516775, 9.09633446),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6639404f,
                                                40.278038f, leg_index, 0.691463232f, 106,
                                                maneuver_index, 0.0276892185f, 4));
  //----------------------------------------------------------------
  // trace point = 1965
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6639862f, 40.2781219f, 1491516776, 9.1294632),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6640244f,
                                                40.2780838f, leg_index, 0.686020792f, 106,
                                                maneuver_index, 0.022246778f, 4));
  //----------------------------------------------------------------
  // trace point = 1966
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6641388f, 40.2782135f, 1491516778, 8.27234554),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6641769f,
                                                40.2781754f, leg_index, 0.675779164f, 104,
                                                maneuver_index, 0.0120051503f, 2));
  //----------------------------------------------------------------
  // trace point = 1967
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6642303f, 40.2783051f, 1491516780, 6.3788209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6643066f,
                                                40.2783432f, leg_index, 0.658479869f, 101,
                                                maneuver_index, 0.658479869f, 101));
  //----------------------------------------------------------------
  // trace point = 1968 | Post | Continue for 7 tenths of a mile.
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6641846f, 40.2784309f, 1491516782, 7.32250881),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.6642151f, 40.2784462f, leg_index, 0.649871826f,
                                                99, maneuver_index, 0.649871826f, 99,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1969
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6641312f, 40.2785034f, 1491516783, 9.13783932),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6641541f,
                                                40.2785149f, leg_index, 0.644133151f, 99,
                                                maneuver_index, 0.644133151f, 99));
  //----------------------------------------------------------------
  // trace point = 1970
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6640778f, 40.2785873f, 1491516784, 10.5149536),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6640854f,
                                                40.2785912f, leg_index, 0.637731671f, 98,
                                                maneuver_index, 0.637731671f, 98));
  //----------------------------------------------------------------
  // trace point = 1971
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6640091f, 40.2786827f, 1491516785, 11.8308477),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6640015f,
                                                40.2786789f, leg_index, 0.630222917f, 96,
                                                maneuver_index, 0.630222917f, 96));
  //----------------------------------------------------------------
  // trace point = 1972
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6639328f, 40.2787781f, 1491516786, 12.6679754),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6639175f,
                                                40.2787704f, leg_index, 0.622496188f, 95,
                                                maneuver_index, 0.622496188f, 95));
  //----------------------------------------------------------------
  // trace point = 1973
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6638412f, 40.2788887f, 1491516787, 14.5244398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.663826f,
                                                40.2788811f, leg_index, 0.613451183f, 94,
                                                maneuver_index, 0.613451183f, 94));
  //----------------------------------------------------------------
  // trace point = 1974
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6637497f, 40.2789993f, 1491516788, 14.6194592),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6637268f,
                                                40.2789879f, leg_index, 0.604398727f, 92,
                                                maneuver_index, 0.604398727f, 92));
  //----------------------------------------------------------------
  // trace point = 1975
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6636505f, 40.2791061f, 1491516789, 14.6184702),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6636353f,
                                                40.2790985f, leg_index, 0.595354617f, 91,
                                                maneuver_index, 0.595354617f, 91));
  //----------------------------------------------------------------
  // trace point = 1976
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.663559f, 40.2792168f, 1491516790, 14.4799757),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6635361f,
                                                40.2792053f, leg_index, 0.586301446f, 90,
                                                maneuver_index, 0.586301446f, 90));
  //----------------------------------------------------------------
  // trace point = 1977
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6634674f, 40.2793274f, 1491516791, 14.8037739),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6634369f,
                                                40.2793121f, leg_index, 0.577249825f, 88,
                                                maneuver_index, 0.577249825f, 88));
  //----------------------------------------------------------------
  // trace point = 1978
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6633682f, 40.2794456f, 1491516792, 15.5485792),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6633377f,
                                                40.2794304f, leg_index, 0.567542374f, 87,
                                                maneuver_index, 0.567542374f, 87));
  //----------------------------------------------------------------
  // trace point = 1979
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6632614f, 40.2795601f, 1491516793, 15.4614258),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6632309f,
                                                40.2795448f, leg_index, 0.557830453f, 85,
                                                maneuver_index, 0.557830453f, 85));
  //----------------------------------------------------------------
  // trace point = 1980
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6631622f, 40.2796822f, 1491516794, 16.3462067),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6631317f,
                                                40.2796669f, leg_index, 0.547897458f, 84,
                                                maneuver_index, 0.547897458f, 84));
  //----------------------------------------------------------------
  // trace point = 1981
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6630554f, 40.2798042f, 1491516795, 16.1542034),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6630249f,
                                                40.279789f, leg_index, 0.537747443f, 82,
                                                maneuver_index, 0.537747443f, 82));
  //----------------------------------------------------------------
  // trace point = 1982
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6629486f, 40.2799377f, 1491516796, 17.3643913),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6629181f,
                                                40.2799225f, leg_index, 0.52693218f, 81,
                                                maneuver_index, 0.52693218f, 81));
  //----------------------------------------------------------------
  // trace point = 1983
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6628342f, 40.2800636f, 1491516797, 16.8524208),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6628036f,
                                                40.2800484f, leg_index, 0.516336322f, 79,
                                                maneuver_index, 0.516336322f, 79));
  //----------------------------------------------------------------
  // trace point = 1984
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6627197f, 40.2801857f, 1491516798, 16.6293678),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6626968f,
                                                40.2801743f, leg_index, 0.50596565f, 77,
                                                maneuver_index, 0.50596565f, 77));
  //----------------------------------------------------------------
  // trace point = 1985
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6626053f, 40.2803116f, 1491516799, 17.2800121),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6625824f,
                                                40.2803001f, leg_index, 0.495369583f, 76,
                                                maneuver_index, 0.495369583f, 76));
  //----------------------------------------------------------------
  // trace point = 1986
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6624908f, 40.2804413f, 1491516800, 17.131855),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.662468f,
                                                40.2804298f, leg_index, 0.484554678f, 74,
                                                maneuver_index, 0.484554678f, 74));
  //----------------------------------------------------------------
  // trace point = 1987
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6623764f, 40.2805634f, 1491516801, 16.9091816),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6623611f,
                                                40.2805557f, leg_index, 0.474189669f, 73,
                                                maneuver_index, 0.474189669f, 73));
  //----------------------------------------------------------------
  // trace point = 1988
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.662262f, 40.2806892f, 1491516802, 16.9583645),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6622314f,
                                                40.2806702f, leg_index, 0.463721544f, 71,
                                                maneuver_index, 0.463721544f, 71));
  //----------------------------------------------------------------
  // trace point = 1989
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6621475f, 40.2808075f, 1491516803, 16.5217361),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6621094f,
                                                40.2807846f, leg_index, 0.453518122f, 69,
                                                maneuver_index, 0.453518122f, 69));
  //----------------------------------------------------------------
  // trace point = 1990
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6620178f, 40.2809181f, 1491516804, 16.5995655),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6619797f,
                                                40.2808952f, leg_index, 0.443252653f, 68,
                                                maneuver_index, 0.443252653f, 68));
  //----------------------------------------------------------------
  // trace point = 1991
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6618805f, 40.2810249f, 1491516805, 16.4321671),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.66185f,
                                                40.281002f, leg_index, 0.433168203f, 66,
                                                maneuver_index, 0.433168203f, 66));
  //----------------------------------------------------------------
  // trace point = 1992
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6617432f, 40.2811241f, 1491516806, 16.1359444),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6617203f,
                                                40.281105f, leg_index, 0.423293322f, 64,
                                                maneuver_index, 0.423293322f, 64));
  //----------------------------------------------------------------
  // trace point = 1993
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6615982f, 40.2812195f, 1491516807, 16.1033001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6615753f,
                                                40.2811966f, leg_index, 0.413360119f, 63,
                                                maneuver_index, 0.413360119f, 63));
  //----------------------------------------------------------------
  // trace point = 1994
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6614532f, 40.2813034f, 1491516808, 15.599453),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6614304f,
                                                40.2812805f, leg_index, 0.40375635f, 61,
                                                maneuver_index, 0.40375635f, 61));
  //----------------------------------------------------------------
  // trace point = 1995
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6613007f, 40.2813835f, 1491516809, 15.6822901),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6612854f,
                                                40.2813644f, leg_index, 0.394150615f, 60,
                                                maneuver_index, 0.394150615f, 60));
  //----------------------------------------------------------------
  // trace point = 1996
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6611481f, 40.281456f, 1491516810, 15.5565939),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6611328f,
                                                40.2814331f, leg_index, 0.38480252f, 59,
                                                maneuver_index, 0.38480252f, 59));
  //----------------------------------------------------------------
  // trace point = 1997
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6609802f, 40.2815285f, 1491516811, 16.2903004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.660965f,
                                                40.2815094f, leg_index, 0.374011368f, 57,
                                                maneuver_index, 0.374011368f, 57));
  //----------------------------------------------------------------
  // trace point = 1998
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6608124f, 40.2815933f, 1491516812, 16.0728321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6607971f,
                                                40.2815742f, leg_index, 0.364561439f, 55,
                                                maneuver_index, 0.364561439f, 55));
  //----------------------------------------------------------------
  // trace point = 1999
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6606445f, 40.281662f, 1491516813, 16.2496719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6606293f,
                                                40.2816391f, leg_index, 0.354633361f, 54,
                                                maneuver_index, 0.354633361f, 54));
  //----------------------------------------------------------------
  // trace point = 2000
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6604691f, 40.2817307f, 1491516814, 16.6782761),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6604538f,
                                                40.2817078f, leg_index, 0.344228595f, 52,
                                                maneuver_index, 0.344228595f, 52));
  //----------------------------------------------------------------
  // trace point = 2001
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6602859f, 40.2817955f, 1491516815, 16.7308254),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6602783f,
                                                40.2817879f, leg_index, 0.333426356f, 51,
                                                maneuver_index, 0.333426356f, 51));
  //----------------------------------------------------------------
  // trace point = 2002
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6601181f, 40.2818604f, 1491516816, 16.4007168),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6601257f,
                                                40.281868f, leg_index, 0.323628336f, 49,
                                                maneuver_index, 0.323628336f, 49));
  //----------------------------------------------------------------
  // trace point = 2003
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6599274f, 40.2819405f, 1491516817, 18.0248203),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6599426f,
                                                40.2819633f, leg_index, 0.311879456f, 47,
                                                maneuver_index, 0.311879456f, 47));
  //----------------------------------------------------------------
  // trace point = 2004
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6597595f, 40.2820015f, 1491516818, 16.0784359),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6597748f,
                                                40.2820244f, leg_index, 0.302068084f, 46,
                                                maneuver_index, 0.302068084f, 46));
  //----------------------------------------------------------------
  // trace point = 2005
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6596069f, 40.2820702f, 1491516819, 15.1291695),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6596146f,
                                                40.2820854f, leg_index, 0.292618513f, 44,
                                                maneuver_index, 0.292618513f, 44));
  //----------------------------------------------------------------
  // trace point = 2006
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6594391f, 40.2821274f, 1491516820, 15.6761379),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6594543f,
                                                40.2821465f, leg_index, 0.28316915f, 43,
                                                maneuver_index, 0.28316915f, 43));
  //----------------------------------------------------------------
  // trace point = 2007 | Alert | In a quarter mile You will arrive at your destination.
  maneuver_index = 11;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6592712f, 40.2821884f, 1491516821, 15.9540434),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.6592865f, 40.2822075f, leg_index, 0.2733576f, 41,
                                                maneuver_index, 0.2733576f, 41, instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 2008
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6590958f, 40.2822495f, 1491516822, 16.1385441),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.659111f,
                                                40.2822723f, leg_index, 0.263068736f, 40,
                                                maneuver_index, 0.263068736f, 40));
  //----------------------------------------------------------------
  // trace point = 2009
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6589203f, 40.2823143f, 1491516823, 16.6056595),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6589355f,
                                                40.2823372f, leg_index, 0.252779871f, 38,
                                                maneuver_index, 0.252779871f, 38));
  //----------------------------------------------------------------
  // trace point = 2010
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6587448f, 40.2823792f, 1491516824, 16.4808254),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6587601f,
                                                40.282402f, leg_index, 0.242491111f, 36,
                                                maneuver_index, 0.242491111f, 36));
  //----------------------------------------------------------------
  // trace point = 2011
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6585617f, 40.2824402f, 1491516825, 17.0498638),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.658577f,
                                                40.2824707f, leg_index, 0.231724709f, 35,
                                                maneuver_index, 0.231724709f, 35));
  //----------------------------------------------------------------
  // trace point = 2012
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6583862f, 40.2825127f, 1491516826, 17.0063858),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6584015f,
                                                40.2825356f, leg_index, 0.221436217f, 34,
                                                maneuver_index, 0.221436217f, 34));
  //----------------------------------------------------------------
  // trace point = 2013
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6582031f, 40.2825737f, 1491516827, 16.9152069),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.658226f,
                                                40.2826042f, leg_index, 0.211030945f, 32,
                                                maneuver_index, 0.211030945f, 32));
  //----------------------------------------------------------------
  // trace point = 2014
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6580276f, 40.2826385f, 1491516828, 16.4528923),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6580505f,
                                                40.2826691f, leg_index, 0.20074214f, 30,
                                                maneuver_index, 0.20074214f, 30));
  //----------------------------------------------------------------
  // trace point = 2015
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6578522f, 40.2827072f, 1491516829, 16.80439),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6578674f,
                                                40.2827377f, leg_index, 0.189975843f, 29,
                                                maneuver_index, 0.189975843f, 29));
  //----------------------------------------------------------------
  // trace point = 2016
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6576614f, 40.2827721f, 1491516830, 17.4975567),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6576843f,
                                                40.2828064f, leg_index, 0.179209545f, 27,
                                                maneuver_index, 0.179209545f, 27));
  //----------------------------------------------------------------
  // trace point = 2017
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.657486f, 40.2828369f, 1491516831, 16.6543636),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6575089f,
                                                40.2828712f, leg_index, 0.168920726f, 26,
                                                maneuver_index, 0.168920726f, 26));
  //----------------------------------------------------------------
  // trace point = 2018
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6573105f, 40.2829094f, 1491516832, 17.1329021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6573257f,
                                                40.2829361f, leg_index, 0.158269912f, 24,
                                                maneuver_index, 0.158269912f, 24));
  //----------------------------------------------------------------
  // trace point = 2019
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6571426f, 40.2829742f, 1491516833, 16.0725708),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6571579f,
                                                40.2830009f, leg_index, 0.148343027f, 23,
                                                maneuver_index, 0.148343027f, 23));
  //----------------------------------------------------------------
  // trace point = 2020
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6569748f, 40.2830276f, 1491516834, 15.4470234),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6569977f,
                                                40.283062f, leg_index, 0.138894081f, 21,
                                                maneuver_index, 0.138894081f, 21));
  //----------------------------------------------------------------
  // trace point = 2021
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6568222f, 40.2830887f, 1491516835, 14.4327259),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6568375f,
                                                40.2831192f, leg_index, 0.129559755f, 20,
                                                maneuver_index, 0.129559755f, 20));
  //----------------------------------------------------------------
  // trace point = 2022
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.656662f, 40.2831421f, 1491516836, 14.564333),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6566849f,
                                                40.2831764f, leg_index, 0.120588101f, 18,
                                                maneuver_index, 0.120588101f, 18));
  //----------------------------------------------------------------
  // trace point = 2023
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.656517f, 40.2832031f, 1491516837, 14.1809378),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6565323f,
                                                40.2832298f, leg_index, 0.111732654f, 17,
                                                maneuver_index, 0.111732654f, 17));
  //----------------------------------------------------------------
  // trace point = 2024
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6563797f, 40.2832642f, 1491516838, 13.7653122),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.656395f,
                                                40.2832832f, leg_index, 0.103599235f, 16,
                                                maneuver_index, 0.103599235f, 16));
  //----------------------------------------------------------------
  // trace point = 2025
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6562347f, 40.2833138f, 1491516839, 13.1469545),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.65625f,
                                                40.2833405f, leg_index, 0.0949874744f, 14,
                                                maneuver_index, 0.0949874744f, 14));
  //----------------------------------------------------------------
  // trace point = 2026
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6560898f, 40.283371f, 1491516840, 13.6756849),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.656105f,
                                                40.2833939f, leg_index, 0.0864938274f, 13,
                                                maneuver_index, 0.0864938274f, 13));
  //----------------------------------------------------------------
  // trace point = 2027
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6559448f, 40.2834244f, 1491516841, 14.209712),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6559601f,
                                                40.2834511f, leg_index, 0.0778833032f, 12,
                                                maneuver_index, 0.0778833032f, 12));
  //----------------------------------------------------------------
  // trace point = 2028
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6557999f, 40.283474f, 1491516842, 13.1272917),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6558151f,
                                                40.2835007f, leg_index, 0.069503665f, 11,
                                                maneuver_index, 0.069503665f, 11));
  //----------------------------------------------------------------
  // trace point = 2029
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6556625f, 40.283535f, 1491516843, 13.48878),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6556778f,
                                                40.2835541f, leg_index, 0.0613717586f, 9,
                                                maneuver_index, 0.0613717586f, 9));
  //----------------------------------------------------------------
  // trace point = 2030
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6555252f, 40.2835846f, 1491516844, 12.7253675),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6555405f,
                                                40.2836037f, leg_index, 0.0533545092f, 8,
                                                maneuver_index, 0.0533545092f, 8));
  //----------------------------------------------------------------
  // trace point = 2031 | Pre | You have arrived at your destination.
  maneuver_index = 11;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6553879f, 40.2836304f, 1491516845, 12.8339186),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6554031f, 40.2836533f, leg_index, 0.0453380719f,
                                                7, maneuver_index, 0.0453380719f, 7,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 2032
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6552505f, 40.2836761f, 1491516846, 12.879694),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6552658f,
                                                40.2837067f, leg_index, 0.0372044854f, 6,
                                                maneuver_index, 0.0372044854f, 6));
  //----------------------------------------------------------------
  // trace point = 2033
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6551285f, 40.2837181f, 1491516847, 11.5214148),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6551514f,
                                                40.2837486f, leg_index, 0.0305051226f, 5,
                                                maneuver_index, 0.0305051226f, 5));
  //----------------------------------------------------------------
  // trace point = 2034
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.655014f, 40.2837677f, 1491516848, 11.1022015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6550293f,
                                                40.2837944f, leg_index, 0.0233288761f, 4,
                                                maneuver_index, 0.0233288761f, 4));
  //----------------------------------------------------------------
  // trace point = 2035
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6548462f, 40.2838287f, 1491516850, 7.82171202),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6548615f,
                                                40.2838593f, leg_index, 0.0134001942f, 2,
                                                maneuver_index, 0.0134001942f, 2));
  //----------------------------------------------------------------
  // trace point = 2036
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6547241f, 40.2838783f, 1491516852, 5.79080677),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6547394f,
                                                40.283905f, leg_index, 0.00622309279f, 1,
                                                maneuver_index, 0.00622309279f, 1));
  //----------------------------------------------------------------
  // trace point = 2037 | Complete
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6546249f, 40.2839241f, 1491516854, 5.14119196),
                            GetNavigationStatus(NavigationStatus_RouteState_kComplete, -76.6546326f,
                                                40.2839432f, leg_index, 0.f, 0, maneuver_index, 0.f,
                                                0));
}

/* AutoMiddletownRoadToLandingsDrive geojson to visualize test route
{"type":"FeatureCollection","features":[
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706558,40.246048]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":0}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706604,40.246197]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706650,40.246349]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706680,40.246498]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":3}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706726,40.246655]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":4}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706764,40.246807]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":5}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706802,40.246967]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":6}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706841,40.247124]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":7}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706902,40.247288]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":8}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706963,40.247440]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":9}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707039,40.247593]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":10}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707138,40.247749]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":11}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707245,40.247894]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":12}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707359,40.248035]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":13}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707474,40.248177]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":14}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707588,40.248322]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":15}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707703,40.248463]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":16}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707817,40.248604]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":17}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707924,40.248737]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":18}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708031,40.248875]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":19}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708138,40.249001]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":20}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708237,40.249134]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":21}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708336,40.249283]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":22}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708420,40.249424]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":23}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708504,40.249569]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":24}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708549,40.249722]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":25}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708603,40.249874]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":26}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708641,40.250034]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":27}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708664,40.250191]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":28}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708664,40.250362]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":29}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708664,40.250534]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":30}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708641,40.250702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":31}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708611,40.250874]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":32}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708595,40.251038]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":33}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708572,40.251221]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":34}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708565,40.251392]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":35}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708565,40.251572]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":36}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708565,40.251751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":37}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708572,40.251926]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":38}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708588,40.252090]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":39}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708595,40.252254]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":40}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708618,40.252422]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":41}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708633,40.252586]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":42}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708656,40.252750]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":43}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708672,40.252918]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":44}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708687,40.253078]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":45}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708710,40.253239]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":46}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708725,40.253403]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":47}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708748,40.253571]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":48}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708771,40.253731]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":49}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708794,40.253899]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":50}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708809,40.254063]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":51}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708839,40.254234]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":52}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708855,40.254406]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":53}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708885,40.254578]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":54}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708908,40.254753]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":55}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708923,40.254929]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":56}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708954,40.255108]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":57}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708977,40.255291]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":58}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708992,40.255474]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":59}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709015,40.255657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":60}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709038,40.255833]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":61}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709068,40.256004]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":62}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709091,40.256176]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":63}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709114,40.256336]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":64}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709129,40.256500]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":65}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709137,40.256653]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":66}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709152,40.256805]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":67}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709160,40.256947]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":68}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709167,40.257072]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":69}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709175,40.257191]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":70}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709183,40.257298]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":71}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709183,40.257397]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":72}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709145,40.257542]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":73}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709023,40.257622]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":74}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708832,40.257641]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":75}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708588,40.257656]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":76}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708435,40.257660]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":77}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708267,40.257668]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":78}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708076,40.257679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":79}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707870,40.257687]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":80}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707649,40.257698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":81}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707428,40.257698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":82}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707184,40.257706]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":83}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706940,40.257721]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":84}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706688,40.257729]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":85}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706436,40.257736]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":86}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706184,40.257748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":87}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.705933,40.257759]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":88}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.705681,40.257767]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":89}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.705429,40.257782]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":90}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.705177,40.257790]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":91}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.704926,40.257805]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":92}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.704681,40.257820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":93}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.704437,40.257835]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":94}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.704185,40.257858]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":95}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.703934,40.257881]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":96}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.703674,40.257900]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":97}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.703430,40.257931]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":98}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.703178,40.257961]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":99}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.702927,40.257999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":100}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.702667,40.258034]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":101}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.702415,40.258068]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.702164,40.258099]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":103}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.701904,40.258129]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":104}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.701653,40.258160]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":105}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.701408,40.258190]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":106}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.701164,40.258224]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700928,40.258251]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700714,40.258278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700516,40.258308]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":110}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700340,40.258331]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":111}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700172,40.258350]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":112}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700035,40.258369]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699829,40.258392]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":114}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699692,40.258423]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":115}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699493,40.258495]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":116}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699287,40.258587]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699150,40.258636]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":118}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699013,40.258698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":119}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698860,40.258766]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":120}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698692,40.258842]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":121}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698517,40.258926]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698334,40.259007]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698135,40.259098]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697929,40.259190]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697716,40.259285]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":126}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697502,40.259380]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":127}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697273,40.259483]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697044,40.259586]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.696808,40.259686]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":130}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.696571,40.259796]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":131}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.696335,40.259907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":132}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.696083,40.260021]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":133}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.695839,40.260139]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":134}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.695595,40.260262]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":135}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.695343,40.260391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":136}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.695091,40.260517]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":137}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.694847,40.260647]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":138}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.694603,40.260780]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":139}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.694359,40.260921]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":140}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.694122,40.261063]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":141}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.693871,40.261204]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":142}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.693626,40.261341]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":143}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.693390,40.261494]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":144}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.693146,40.261646]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":145}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.692902,40.261814]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":146}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.692665,40.261971]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":147}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.692421,40.262135]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":148}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.692184,40.262295]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":149}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691956,40.262466]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":150}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691727,40.262627]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":151}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691505,40.262787]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":152}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691284,40.262939]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":153}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691055,40.263092]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":154}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690842,40.263248]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":155}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690628,40.263401]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":156}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690414,40.263550]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":157}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690208,40.263702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":158}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690002,40.263840]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":159}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.689796,40.263973]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":160}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.689400,40.264233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":161}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.689217,40.264351]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":162}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.689026,40.264462]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":163}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.688828,40.264561]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":164}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.688622,40.264652]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":165}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.688408,40.264729]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":166}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.688187,40.264793]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":167}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687965,40.264847]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":168}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687744,40.264896]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":169}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687531,40.264935]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":170}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687325,40.264984]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":171}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687111,40.265041]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":172}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686913,40.265110]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":173}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686714,40.265190]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":174}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686523,40.265278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":175}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686317,40.265381]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":176}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686134,40.265484]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":177}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.685944,40.265591]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":178}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.685547,40.265808]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":179}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.685150,40.266018]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":180}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684753,40.266232]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":181}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684570,40.266327]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":182}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684387,40.266426]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684196,40.266525]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":184}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.683998,40.266624]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":185}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.683609,40.266819]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.683228,40.266998]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682869,40.267174]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":188}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682686,40.267254]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":189}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682533,40.267323]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":190}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682404,40.267368]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":191}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682182,40.267433]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":192}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682030,40.267456]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681747,40.267509]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681503,40.267540]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":195}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681351,40.267555]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":196}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681198,40.267574]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":197}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681030,40.267593]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":198}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680855,40.267620]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680679,40.267635]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":200}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680496,40.267654]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":201}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680313,40.267666]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":202}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680115,40.267685]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":203}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679901,40.267704]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":204}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679695,40.267731]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679482,40.267754]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679268,40.267773]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679047,40.267796]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.678818,40.267822]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.678596,40.267841]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":210}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.678375,40.267864]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":211}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.678154,40.267891]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":212}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677933,40.267914]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":213}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677727,40.267918]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677505,40.267937]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":215}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677284,40.267956]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":216}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677063,40.267963]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":217}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.676842,40.267979]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.676605,40.267994]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.676376,40.268009]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":220}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.676155,40.268021]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.675934,40.268032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.675484,40.268066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.675270,40.268085]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":224}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.675041,40.268105]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.674820,40.268120]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":226}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.674599,40.268147]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":227}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.674385,40.268173]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":228}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.674164,40.268204]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":229}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673958,40.268227]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":230}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673752,40.268265]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":231}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673538,40.268291]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":232}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673332,40.268318]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":233}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673119,40.268345]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":234}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672897,40.268383]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672684,40.268414]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":236}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672462,40.268444]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":237}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672241,40.268478]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":238}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672020,40.268505]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":239}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.671791,40.268532]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":240}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.671555,40.268566]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":241}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.671318,40.268600]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":242}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.671089,40.268639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":243}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670868,40.268673]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670647,40.268707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670433,40.268738]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":246}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670219,40.268764]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":247}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669998,40.268803]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":248}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669777,40.268841]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":249}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669556,40.268879]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":250}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669327,40.268921]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":251}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669090,40.268967]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":252}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.668854,40.269009]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":253}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.668625,40.269054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":254}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.668411,40.269096]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":255}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.668190,40.269142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":256}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667984,40.269188]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":257}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667778,40.269241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":258}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667580,40.269295]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667374,40.269348]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":260}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667175,40.269402]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666977,40.269463]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":262}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666779,40.269524]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":263}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666580,40.269592]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":264}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666389,40.269657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666214,40.269722]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":266}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666031,40.269794]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665855,40.269863]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":268}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665688,40.269924]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":269}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665504,40.269993]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":270}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665344,40.270058]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665176,40.270119]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665024,40.270191]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":273}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664848,40.270260]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":274}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664688,40.270329]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":275}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664520,40.270401]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":276}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664345,40.270477]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":277}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664177,40.270542]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":278}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664001,40.270607]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":279}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663834,40.270679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":280}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663666,40.270748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":281}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663490,40.270824]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":282}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663315,40.270893]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":283}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663139,40.270966]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":284}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662956,40.271042]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":285}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662766,40.271114]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":286}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662590,40.271183]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":287}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662415,40.271248]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662239,40.271313]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662071,40.271370]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":290}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661919,40.271423]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":291}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661781,40.271473]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661644,40.271519]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":293}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661522,40.271564]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":294}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661400,40.271603]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":295}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661285,40.271645]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":296}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661171,40.271679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":297}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661064,40.271717]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660950,40.271751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660835,40.271786]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":300}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660637,40.271843]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":301}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660507,40.271881]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":302}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660355,40.271923]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":303}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660240,40.271950]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":304}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660133,40.271984]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":305}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659904,40.272026]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":306}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659729,40.272072]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":307}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659515,40.272129]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659401,40.272163]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":309}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659279,40.272198]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659157,40.272228]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":311}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659042,40.272259]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":312}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658913,40.272289]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":313}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658783,40.272331]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":314}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658669,40.272373]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":315}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658546,40.272400]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":316}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658424,40.272434]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":317}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658302,40.272465]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":318}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658173,40.272495]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":319}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658043,40.272530]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":320}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657921,40.272560]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":321}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657784,40.272591]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":322}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657646,40.272629]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":323}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657516,40.272655]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":324}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657394,40.272686]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":325}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657265,40.272717]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":326}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657135,40.272751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":327}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657005,40.272781]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":328}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656883,40.272816]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":329}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656754,40.272846]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":330}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656631,40.272881]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":331}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656517,40.272907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":332}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656342,40.272949]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":333}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656227,40.272980]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":334}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656059,40.273022]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":335}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655884,40.273060]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":336}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655663,40.273106]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":337}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655540,40.273136]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":338}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655396,40.273170]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":339}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655251,40.273205]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":340}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655113,40.273243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":341}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654976,40.273277]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":342}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654846,40.273315]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":343}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654701,40.273350]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":344}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654549,40.273376]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":345}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654381,40.273407]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":346}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654213,40.273438]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":347}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654022,40.273472]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":348}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653847,40.273510]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":349}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653656,40.273548]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":350}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653465,40.273594]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":351}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653275,40.273643]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":352}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653076,40.273685]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":353}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652870,40.273727]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652657,40.273773]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652451,40.273815]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652245,40.273857]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":357}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652039,40.273907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":358}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651840,40.273952]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651642,40.273998]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651443,40.274048]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":361}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651237,40.274094]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651039,40.274136]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":363}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650841,40.274181]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":364}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650642,40.274223]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":365}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650444,40.274269]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650246,40.274315]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":367}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650055,40.274361]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":368}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649857,40.274403]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":369}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649658,40.274445]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":370}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649460,40.274487]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":371}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649269,40.274517]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":372}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649086,40.274551]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":373}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648918,40.274586]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648773,40.274620]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":375}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648643,40.274651]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":376}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648514,40.274673]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":377}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648399,40.274696]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":378}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648270,40.274712]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":379}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648132,40.274727]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":380}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647980,40.274746]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":381}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647827,40.274773]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":382}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647667,40.274796]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":383}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647499,40.274818]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":384}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647316,40.274841]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":385}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647133,40.274864]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":386}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646942,40.274887]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":387}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646751,40.274910]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646545,40.274933]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":389}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646347,40.274956]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":390}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646149,40.274979]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":391}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645943,40.274998]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":392}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645737,40.275017]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":393}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645538,40.275032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":394}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645325,40.275051]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":395}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645111,40.275070]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":396}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644890,40.275093]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":397}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644653,40.275116]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":398}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644417,40.275135]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":399}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644180,40.275154]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":400}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643936,40.275173]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":401}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643700,40.275192]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":402}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643456,40.275204]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":403}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643219,40.275208]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":404}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642982,40.275211]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":405}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642746,40.275219]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":406}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642517,40.275223]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":407}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642296,40.275230]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":408}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642075,40.275234]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":409}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641861,40.275242]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":410}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641655,40.275246]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":411}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641449,40.275249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":412}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641258,40.275257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":413}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641068,40.275257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":414}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640877,40.275261]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":415}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640701,40.275265]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":416}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640533,40.275265]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":417}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640373,40.275269]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":418}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640236,40.275272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":419}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640114,40.275272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639938,40.275272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639809,40.275276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639679,40.275276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":423}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639519,40.275280]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":424}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639275,40.275280]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":425}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639137,40.275276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":426}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638992,40.275280]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":427}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638809,40.275269]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":428}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638634,40.275265]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":429}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638443,40.275257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":430}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638245,40.275257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":431}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638039,40.275249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":432}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637840,40.275242]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":433}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637627,40.275234]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637421,40.275230]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637207,40.275215]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636993,40.275204]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":437}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636772,40.275188]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":438}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636551,40.275177]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":439}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636330,40.275162]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":440}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636108,40.275150]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635887,40.275131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":442}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635674,40.275116]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":443}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635460,40.275101]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":444}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635262,40.275082]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":445}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635063,40.275066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":446}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634865,40.275051]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":447}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634682,40.275040]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":448}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634499,40.275032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":449}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634338,40.275032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":450}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634186,40.275028]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634064,40.275032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":452}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633858,40.275002]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":453}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633705,40.274914]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":454}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633537,40.274868]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":455}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633354,40.274891]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":456}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633133,40.274918]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":457}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633003,40.274925]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":458}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632858,40.274929]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":459}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632706,40.274929]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":460}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632538,40.274925]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632362,40.274918]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":462}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632179,40.274906]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":463}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631989,40.274895]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":464}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631790,40.274883]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":465}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631599,40.274868]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":466}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631401,40.274857]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":467}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631195,40.274841]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":468}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630989,40.274826]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":469}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630783,40.274811]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":470}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630577,40.274796]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630379,40.274780]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630173,40.274761]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629959,40.274746]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":474}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629753,40.274731]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":475}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629539,40.274715]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629318,40.274696]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629089,40.274681]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":478}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628860,40.274658]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":479}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628639,40.274639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":480}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628410,40.274624]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":481}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628181,40.274601]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":482}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627960,40.274582]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627731,40.274567]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627518,40.274548]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627296,40.274532]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":486}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627090,40.274521]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":487}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626892,40.274517]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":488}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626694,40.274525]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":489}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626503,40.274548]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":490}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626328,40.274570]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":491}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626183,40.274593]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":492}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626045,40.274616]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":493}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625824,40.274597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":494}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625656,40.274532]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":495}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625496,40.274509]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":496}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625313,40.274563]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625206,40.274597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":498}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625076,40.274635]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":499}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624931,40.274670]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":500}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624771,40.274700]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":501}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624596,40.274731]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":502}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624413,40.274757]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":503}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624222,40.274792]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":504}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624023,40.274818]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":505}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623825,40.274845]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":506}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623627,40.274872]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":507}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623436,40.274902]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":508}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623238,40.274925]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":509}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623039,40.274952]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":510}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622833,40.274971]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":511}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622635,40.274994]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":512}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622444,40.275017]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":513}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622246,40.275040]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":514}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622040,40.275066]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":515}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621849,40.275085]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":516}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621651,40.275108]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":517}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621460,40.275131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":518}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621269,40.275150]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":519}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621086,40.275177]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":520}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620911,40.275196]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620735,40.275215]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":522}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620560,40.275238]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":523}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620377,40.275257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":524}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620193,40.275276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":525}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620010,40.275299]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":526}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619827,40.275322]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":527}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619637,40.275345]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619438,40.275368]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":529}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619247,40.275391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619049,40.275417]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618851,40.275436]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":532}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618645,40.275459]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":533}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618446,40.275478]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":534}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618240,40.275505]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":535}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618042,40.275536]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":536}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617844,40.275566]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":537}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617645,40.275597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":538}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617455,40.275635]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617256,40.275669]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617065,40.275707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616867,40.275749]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":542}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616676,40.275791]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616493,40.275837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":544}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616302,40.275879]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":545}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616119,40.275921]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":546}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615936,40.275967]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":547}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615746,40.276012]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":548}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615555,40.276058]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615356,40.276104]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":550}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615150,40.276154]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":551}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614944,40.276199]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":552}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614731,40.276245]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":553}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614525,40.276295]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":554}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614311,40.276344]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614098,40.276398]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":556}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613884,40.276451]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":557}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613670,40.276501]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":558}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613457,40.276550]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":559}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613243,40.276600]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":560}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613029,40.276649]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":561}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612823,40.276695]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612617,40.276741]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":563}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612427,40.276783]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612228,40.276829]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":565}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612030,40.276867]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":566}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611832,40.276905]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611626,40.276936]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":568}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611412,40.276958]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":569}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611191,40.276970]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":570}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610970,40.276978]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":571}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610741,40.276989]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":572}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610512,40.276997]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":573}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610283,40.277008]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":574}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610046,40.277020]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":575}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609818,40.277031]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":576}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609596,40.277039]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609367,40.277050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":578}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609138,40.277058]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":579}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608917,40.277065]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":580}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608688,40.277073]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":581}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608467,40.277081]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":582}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608246,40.277092]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":583}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608032,40.277100]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":584}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607811,40.277111]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":585}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607590,40.277115]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":586}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607361,40.277126]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":587}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607124,40.277134]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":588}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606888,40.277142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":589}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606644,40.277153]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":590}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606407,40.277161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":591}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606163,40.277172]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":592}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605911,40.277180]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":593}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605659,40.277187]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":594}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605415,40.277195]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605171,40.277206]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604919,40.277218]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":597}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604683,40.277225]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":598}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604446,40.277229]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":599}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604210,40.277237]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":600}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603981,40.277241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":601}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603760,40.277245]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":602}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603539,40.277248]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":603}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603310,40.277248]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":604}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603096,40.277248]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":605}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602882,40.277248]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":606}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602676,40.277248]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":607}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602470,40.277245]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":608}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602264,40.277241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":609}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602066,40.277233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601860,40.277229]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601662,40.277225]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601448,40.277218]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601250,40.277214]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601044,40.277203]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600838,40.277191]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600624,40.277184]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":617}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600403,40.277172]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":618}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600182,40.277161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":619}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599960,40.277149]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":620}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599731,40.277142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":621}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599503,40.277134]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":622}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599274,40.277126]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":623}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599045,40.277119]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":624}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598808,40.277111]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":625}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598579,40.277103]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":626}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598351,40.277096]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":627}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598129,40.277088]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":628}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597908,40.277077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597694,40.277073]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597481,40.277069]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":631}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597282,40.277061]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":632}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597084,40.277054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":633}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596901,40.277050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":634}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596741,40.277046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596588,40.277046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":636}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596443,40.277042]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":637}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596298,40.277042]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":638}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596161,40.277046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":639}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596016,40.277046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595863,40.277046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":641}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595711,40.277050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":642}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595551,40.277054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595375,40.277054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595200,40.277058]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595016,40.277061]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594826,40.277065]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":647}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594635,40.277069]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":648}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594444,40.277077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594254,40.277081]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":650}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594063,40.277088]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":651}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593880,40.277096]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":652}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593689,40.277100]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":653}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593506,40.277107]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":654}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593315,40.277115]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":655}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593132,40.277119]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":656}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592949,40.277130]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":657}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592766,40.277138]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":658}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592590,40.277142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":659}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592415,40.277149]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592232,40.277149]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":661}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592049,40.277157]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":662}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591866,40.277161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":663}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591675,40.277164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":664}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591492,40.277168]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":665}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591286,40.277168]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":666}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591095,40.277176]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":667}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590897,40.277176]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":668}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590706,40.277180]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":669}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590500,40.277180]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":670}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590294,40.277184]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":671}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590088,40.277187]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":672}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589882,40.277195]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":673}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589684,40.277203]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":674}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589493,40.277203]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":675}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589302,40.277203]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":676}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589127,40.277210]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":677}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588959,40.277210]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":678}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588791,40.277222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":679}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588623,40.277218]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":680}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588470,40.277222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":681}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588303,40.277222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":682}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588142,40.277222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":683}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587967,40.277225]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587791,40.277225]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":685}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587624,40.277222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":686}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587448,40.277218]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587280,40.277218]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":688}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587120,40.277222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":689}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586967,40.277229]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":690}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586823,40.277237]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":691}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586693,40.277245]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":692}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586571,40.277252]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":693}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586365,40.277267]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":694}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586182,40.277279]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":695}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586037,40.277287]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":696}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585915,40.277294]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":697}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585777,40.277309]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":698}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585655,40.277317]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":699}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585472,40.277336]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585266,40.277355]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":701}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585022,40.277378]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":702}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584900,40.277390]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":703}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584770,40.277405]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":704}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584648,40.277420]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":705}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584518,40.277439]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":706}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584396,40.277454]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":707}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584274,40.277470]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":708}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584152,40.277485]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":709}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584030,40.277504]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583801,40.277531]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":711}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583679,40.277546]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":712}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583557,40.277557]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":713}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583427,40.277569]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":714}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583305,40.277580]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":715}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583176,40.277592]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":716}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583054,40.277603]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":717}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582924,40.277615]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":718}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582802,40.277622]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":719}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582672,40.277630]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":720}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582535,40.277637]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":721}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582397,40.277649]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":722}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582260,40.277657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":723}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582123,40.277668]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":724}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581978,40.277679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":725}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581833,40.277691]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":726}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581673,40.277702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":727}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581528,40.277718]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":728}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581367,40.277725]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581200,40.277733]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":730}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581032,40.277740]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580856,40.277748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":732}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580666,40.277760]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":733}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580475,40.277771]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":734}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580276,40.277782]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":735}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580078,40.277794]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":736}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579880,40.277805]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":737}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579674,40.277821]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":738}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579468,40.277836]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":739}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579262,40.277847]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":740}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579056,40.277859]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":741}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578850,40.277870]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":742}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578651,40.277885]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578445,40.277889]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":744}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578239,40.277901]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":745}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578041,40.277912]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":746}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577843,40.277924]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":747}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577660,40.277935]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":748}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577507,40.277946]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":749}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577385,40.277962]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":750}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577255,40.277969]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":751}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577164,40.278042]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":752}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577118,40.278179]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":753}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577049,40.278347]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":754}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577011,40.278431]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":755}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576973,40.278526]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":756}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576935,40.278625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":757}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576897,40.278732]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":758}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576859,40.278843]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":759}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576805,40.278957]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":760}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576759,40.279076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":761}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576714,40.279190]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":762}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576653,40.279316]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":763}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576599,40.279442]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":764}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576546,40.279564]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":765}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576492,40.279686]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":766}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576447,40.279808]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":767}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576393,40.279930]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":768}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576340,40.280056]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576279,40.280178]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576225,40.280304]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576172,40.280437]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":772}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576118,40.280563]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":773}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576065,40.280697]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576012,40.280819]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":775}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575958,40.280933]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":776}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575912,40.281052]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":777}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575851,40.281166]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":778}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575798,40.281281]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575729,40.281391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":780}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575653,40.281494]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":781}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575562,40.281582]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":782}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575447,40.281666]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":783}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575317,40.281734]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":784}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575172,40.281811]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":785}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575035,40.281883]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":786}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574890,40.281967]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":787}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574730,40.282047]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":788}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574570,40.282131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":789}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574409,40.282215]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":790}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574249,40.282303]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":791}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574089,40.282391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":792}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573929,40.282475]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":793}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573776,40.282558]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":794}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573624,40.282639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":795}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573486,40.282707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573372,40.282772]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":797}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573273,40.282822]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":798}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573120,40.282898]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":799}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573013,40.282948]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":800}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572952,40.283051]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":801}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572983,40.283188]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":802}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573029,40.283356]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":803}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573051,40.283447]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":804}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573074,40.283550]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573097,40.283653]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573128,40.283756]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573158,40.283855]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573189,40.283951]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":809}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573212,40.284039]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":810}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573250,40.284184]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":811}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573265,40.284298]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573250,40.284401]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573212,40.284534]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":814}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573227,40.284630]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":815}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573303,40.284721]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":816}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573395,40.284821]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":817}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573463,40.284939]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":818}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573502,40.285069]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":819}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573547,40.285202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":820}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573586,40.285336]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":821}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573631,40.285469]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":822}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573669,40.285606]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":823}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573715,40.285751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":824}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573761,40.285904]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":825}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573807,40.286057]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":826}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573860,40.286217]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":827}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573914,40.286377]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":828}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573959,40.286545]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":829}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573990,40.286633]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":830}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574013,40.286728]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574043,40.286819]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":832}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574081,40.286915]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":833}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574104,40.287010]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":834}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574135,40.287109]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":835}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574165,40.287209]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":836}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574196,40.287312]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":837}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574234,40.287415]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":838}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574265,40.287521]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574295,40.287621]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":840}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574318,40.287727]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574348,40.287830]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":842}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574371,40.287930]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":843}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574402,40.288033]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574425,40.288128]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":845}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574478,40.288292]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":846}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574509,40.288406]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":847}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574509,40.288498]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":848}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574387,40.288609]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":849}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574234,40.288651]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574043,40.288704]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573936,40.288750]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":852}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573837,40.288807]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":853}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573723,40.288864]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":854}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573608,40.288929]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":855}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573494,40.289001]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573372,40.289074]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":857}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573242,40.289154]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":858}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573097,40.289238]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572952,40.289322]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572823,40.289410]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":861}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572678,40.289490]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572533,40.289577]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572388,40.289665]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":864}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572250,40.289749]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":865}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572113,40.289829]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":866}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571976,40.289909]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":867}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571854,40.289982]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":868}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571747,40.290043]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571579,40.290131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":870}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571442,40.290123]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571327,40.290031]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":872}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571198,40.289909]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":873}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571114,40.289837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":874}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571022,40.289768]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":875}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570923,40.289707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":876}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570824,40.289654]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":877}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570717,40.289604]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":878}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570610,40.289551]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":879}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570496,40.289494]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":880}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570374,40.289433]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":881}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570251,40.289368]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":882}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570129,40.289307]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":883}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570000,40.289242]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":884}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569878,40.289177]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":885}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569748,40.289112]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":886}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569618,40.289047]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569496,40.288982]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569366,40.288918]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569244,40.288857]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569122,40.288792]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":891}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569000,40.288727]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":892}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568871,40.288658]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":893}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568748,40.288597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":894}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568626,40.288532]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568504,40.288467]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":896}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568382,40.288406]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":897}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568253,40.288345]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":898}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568138,40.288284]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":899}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568016,40.288223]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":900}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567894,40.288158]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":901}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567772,40.288097]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":902}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567650,40.288033]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":903}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567528,40.287968]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":904}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567406,40.287907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":905}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567299,40.287853]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":906}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567108,40.287766]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":907}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566956,40.287697]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":908}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566849,40.287651]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":909}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566780,40.287552]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":910}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566826,40.287464]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":911}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566902,40.287361]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":912}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567001,40.287228]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":913}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567108,40.287067]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":914}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567184,40.286942]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567245,40.286831]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567307,40.286747]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567390,40.286640]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":918}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567482,40.286514]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":919}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567566,40.286381]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":920}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706596,40.246044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":0,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.181,"distance_from_trace_point":3.247}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706635,40.246193]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.263,"distance_from_trace_point":2.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706673,40.246346]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.346,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706703,40.246494]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":3,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.427,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706749,40.246651]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":4,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.513,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706787,40.246803]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":5,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.597,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706833,40.246964]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":6,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.685,"distance_from_trace_point":2.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706871,40.247120]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":7,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.771,"distance_from_trace_point":2.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706924,40.247284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":8,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.862,"distance_from_trace_point":1.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706993,40.247433]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":9,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.946,"distance_from_trace_point":2.710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707069,40.247581]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":10,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.027,"distance_from_trace_point":2.869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707176,40.247734]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":11,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.104,"distance_from_trace_point":3.635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707283,40.247875]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":12,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.177,"distance_from_trace_point":3.849}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707397,40.248016]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":13,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.251,"distance_from_trace_point":3.849}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707512,40.248161]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":14,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.326,"distance_from_trace_point":3.635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707626,40.248302]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":15,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.400,"distance_from_trace_point":3.849}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707741,40.248444]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":16,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.474,"distance_from_trace_point":3.848}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707855,40.248589]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":17,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.549,"distance_from_trace_point":3.635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707962,40.248722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":18,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.619,"distance_from_trace_point":3.635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708069,40.248856]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":19,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.688,"distance_from_trace_point":3.848}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708176,40.248985]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":20,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.757,"distance_from_trace_point":3.635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708275,40.249119]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":21,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.825,"distance_from_trace_point":3.635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708366,40.249271]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":22,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.900,"distance_from_trace_point":2.869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708458,40.249409]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":23,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.968,"distance_from_trace_point":3.634}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708534,40.249561]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":24,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.085,"distance_from_trace_point":2.710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708595,40.249714]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":25,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.232,"distance_from_trace_point":3.954}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708633,40.249870]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":26,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.379,"distance_from_trace_point":2.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708656,40.250034]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":27,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.531,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708664,40.250191]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":28,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.676,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708664,40.250362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":29,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.834,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708649,40.250534]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":30,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.993,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708633,40.250702]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":31,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.111,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708618,40.250874]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":32,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.230,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708611,40.251038]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":33,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.343,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708595,40.251221]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":34,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.470,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708580,40.251392]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":35,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.589,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708572,40.251572]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":36,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.712,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708572,40.251751]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":37,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.837,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708580,40.251926]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":38,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.958,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708603,40.252090]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":39,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.105,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708618,40.252254]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":40,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.270,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708641,40.252422]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":41,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.440,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708656,40.252586]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":42,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.606,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708679,40.252750]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":43,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.772,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708694,40.252918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":44,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.942,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708717,40.253078]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":45,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.052,"distance_from_trace_point":2.575}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708733,40.253239]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":46,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.133,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708755,40.253399]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":47,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.214,"distance_from_trace_point":2.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708771,40.253571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":48,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.300,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708794,40.253731]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":49,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.381,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708809,40.253899]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":50,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.465,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708832,40.254063]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":51,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.548,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708855,40.254234]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":52,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.635,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708870,40.254406]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":53,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.721,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708893,40.254578]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":54,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.808,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708916,40.254753]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":55,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.896,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708939,40.254929]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":56,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.985,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708954,40.255108]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":57,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.278,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708977,40.255291]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":58,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.621,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709000,40.255474]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":59,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.965,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709023,40.255657]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":60,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.188,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709038,40.255833]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":61,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.666,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709061,40.256004]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":62,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.029,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709084,40.256176]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":63,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.129,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709099,40.256336]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":64,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.223,"distance_from_trace_point":1.288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709122,40.256500]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":65,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.319,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709145,40.256653]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":66,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.408,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709160,40.256805]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":67,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.497,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709183,40.256947]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":68,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.580,"distance_from_trace_point":1.931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709198,40.257069]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":69,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.651,"distance_from_trace_point":2.609}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709213,40.257191]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":70,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.722,"distance_from_trace_point":3.219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709221,40.257298]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":71,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.784,"distance_from_trace_point":3.219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709221,40.257397]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":72,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.842,"distance_from_trace_point":3.219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709221,40.257545]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":73,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.929,"distance_from_trace_point":6.451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.709023,40.257671]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":74,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.019,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708832,40.257679]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":75,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.039,"distance_from_trace_point":4.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708588,40.257687]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":76,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.064,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708435,40.257690]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":77,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.080,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708267,40.257694]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":78,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.097,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.708076,40.257698]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":79,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.117,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707870,40.257706]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":80,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.138,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707649,40.257710]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":81,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.161,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707428,40.257721]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":82,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.184,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.707184,40.257732]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":83,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.209,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706940,40.257744]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":84,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.234,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706688,40.257755]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":85,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.260,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706436,40.257759]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":86,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.286,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.706184,40.257763]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":87,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.312,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.705933,40.257767]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":88,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.338,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.705681,40.257771]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":89,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.364,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.705429,40.257778]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":90,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.390,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.705177,40.257786]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":91,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.416,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.704926,40.257797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":92,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.442,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.704681,40.257812]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":93,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.467,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.704437,40.257828]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":94,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.492,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.704185,40.257858]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":95,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.519,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.703934,40.257885]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":96,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.545,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.703674,40.257912]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":97,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.572,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.703430,40.257938]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":98,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.597,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.703178,40.257969]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":99,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.624,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.702927,40.257996]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":100,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.650,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.702667,40.258026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":101,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.677,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.702415,40.258053]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":102,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.703,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.702164,40.258087]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":103,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.730,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.701904,40.258125]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":104,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.757,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.701653,40.258160]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":105,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.783,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.701408,40.258194]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":106,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.809,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.701164,40.258228]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":107,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.834,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700928,40.258259]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":108,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.859,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700714,40.258289]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":109,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.882,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700516,40.258316]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":110,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.902,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700340,40.258339]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":111,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.921,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700172,40.258366]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":112,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.938,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.700035,40.258385]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":113,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.953,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699837,40.258411]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":114,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.974,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699692,40.258430]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":115,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.989,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699486,40.258488]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":116,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.026,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699280,40.258575]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":117,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.079,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699150,40.258633]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":118,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.113,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.699013,40.258694]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":119,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.149,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698853,40.258759]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":120,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.191,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698685,40.258835]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":121,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.235,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698502,40.258911]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":122,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.282,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698318,40.258991]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":123,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.330,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.698120,40.259083]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":124,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.383,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697922,40.259178]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":125,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.436,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697708,40.259277]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":126,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.493,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697502,40.259377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":127,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.548,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697273,40.259483]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":128,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.609,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.697044,40.259590]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":129,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.670,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.696823,40.259701]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":130,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.730,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.696587,40.259811]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":131,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.793,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.696350,40.259926]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":132,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.857,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.696106,40.260044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":133,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.923,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.695862,40.260170]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":134,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.990,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.695618,40.260292]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":135,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.030,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.695366,40.260422]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":136,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.067,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.695122,40.260551]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":137,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.103,"distance_from_trace_point":4.587}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.694878,40.260681]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":138,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.139,"distance_from_trace_point":4.587}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.694633,40.260811]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":139,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.175,"distance_from_trace_point":4.244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.694382,40.260944]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":140,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.212,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.694145,40.261086]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":141,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.248,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.693893,40.261230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":142,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.287,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.693657,40.261368]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":143,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.323,"distance_from_trace_point":3.918}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.693413,40.261513]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":144,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.360,"distance_from_trace_point":2.860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.693161,40.261658]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":145,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.398,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.692917,40.261826]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":146,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.438,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.692688,40.261990]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":147,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.476,"distance_from_trace_point":2.860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.692444,40.262157]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":148,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.516,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.692215,40.262321]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":149,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.554,"distance_from_trace_point":3.918}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691978,40.262486]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":150,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.593,"distance_from_trace_point":2.860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691750,40.262646]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":151,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.631,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691528,40.262806]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":152,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.668,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691307,40.262959]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":153,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.704,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.691086,40.263119]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":154,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.741,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690872,40.263271]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":155,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.776,"distance_from_trace_point":3.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690651,40.263424]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":156,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.812,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690437,40.263573]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":157,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.847,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690231,40.263721]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":158,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.882,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.690033,40.263863]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":159,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.915,"distance_from_trace_point":3.610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.689835,40.264000]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":160,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.947,"distance_from_trace_point":4.368}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.689430,40.264267]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":161,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.015,"distance_from_trace_point":4.587}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.689232,40.264370]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":162,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.051,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.689041,40.264473]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":163,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.085,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.688835,40.264572]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":164,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.122,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.688629,40.264660]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":165,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.157,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.688408,40.264732]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":166,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.193,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.688187,40.264801]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":167,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.228,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687965,40.264851]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":168,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.263,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687744,40.264904]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":169,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.297,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687538,40.264950]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":170,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.329,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687332,40.264999]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":171,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.361,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.687119,40.265053]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":172,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.395,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686920,40.265118]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":173,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.427,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686722,40.265198]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":174,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.461,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686531,40.265282]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":175,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.493,"distance_from_trace_point":0.770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686325,40.265388]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":176,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.530,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.686134,40.265488]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":177,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.565,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.685944,40.265591]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":178,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.600,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.685547,40.265808]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":179,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.672,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.685150,40.266018]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":180,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.744,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684753,40.266235]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":181,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.817,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684586,40.266338]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":182,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.849,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684410,40.266445]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":183,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.883,"distance_from_trace_point":2.859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684227,40.266556]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":184,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.917,"distance_from_trace_point":4.244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.684044,40.266666]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":185,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.952,"distance_from_trace_point":6.037}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.683662,40.266880]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":186,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.120,"distance_from_trace_point":8.114}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.683281,40.267063]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":187,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.475,"distance_from_trace_point":8.468}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682922,40.267239]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":188,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.810,"distance_from_trace_point":8.468}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682739,40.267323]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":189,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.978,"distance_from_trace_point":8.828}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682579,40.267387]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":190,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.641,"distance_from_trace_point":8.144}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682442,40.267437]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":191,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.264,"distance_from_trace_point":8.246}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682205,40.267506]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":192,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.229,"distance_from_trace_point":8.243}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.682053,40.267536]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":193,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.743,"distance_from_trace_point":9.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681763,40.267578]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":194,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.206,"distance_from_trace_point":7.700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681519,40.267605]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":195,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.441,"distance_from_trace_point":7.285}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681366,40.267624]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":196,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.588,"distance_from_trace_point":7.700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681213,40.267643]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":197,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.736,"distance_from_trace_point":7.700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.681046,40.267662]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":198,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.897,"distance_from_trace_point":7.700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680870,40.267681]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":199,"matched_point_type":"matched","edge_index":19,"distance_along_edge":0.136,"distance_from_trace_point":6.870}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680695,40.267700]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":200,"matched_point_type":"matched","edge_index":19,"distance_along_edge":0.484,"distance_from_trace_point":7.285}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680511,40.267723]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":201,"matched_point_type":"matched","edge_index":19,"distance_along_edge":0.849,"distance_from_trace_point":7.700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680328,40.267742]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":202,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.125,"distance_from_trace_point":8.533}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.680130,40.267765]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":203,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.357,"distance_from_trace_point":8.950}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679916,40.267792]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":204,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.607,"distance_from_trace_point":9.786}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679710,40.267815]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":205,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.848,"distance_from_trace_point":9.368}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679489,40.267838]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":206,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.059,"distance_from_trace_point":9.301}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679276,40.267853]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":207,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.195,"distance_from_trace_point":8.881}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.679054,40.267868]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":208,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.337,"distance_from_trace_point":8.040}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.678825,40.267883]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":209,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.483,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.678604,40.267902]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":210,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.624,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.678383,40.267918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":211,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.766,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.678162,40.267933]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":212,"matched_point_type":"matched","edge_index":21,"distance_along_edge":0.907,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677940,40.267948]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":213,"matched_point_type":"matched","edge_index":22,"distance_along_edge":0.122,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677734,40.267963]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":214,"matched_point_type":"matched","edge_index":22,"distance_along_edge":0.451,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677513,40.267979]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":215,"matched_point_type":"matched","edge_index":22,"distance_along_edge":0.805,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677292,40.267994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":216,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.030,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.677071,40.268009]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":217,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.096,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.676849,40.268024]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":218,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.163,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.676613,40.268044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":219,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.234,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.676384,40.268059]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":220,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.303,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.676163,40.268074]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":221,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.369,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.675941,40.268089]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":222,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.436,"distance_from_trace_point":6.359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.675491,40.268120]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":223,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.571,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.675278,40.268135]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":224,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.636,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.675049,40.268158]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":225,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.705,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.674828,40.268181]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":226,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.772,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.674606,40.268204]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":227,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.838,"distance_from_trace_point":6.359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.674393,40.268227]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":228,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.903,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.674171,40.268250]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":229,"matched_point_type":"matched","edge_index":23,"distance_along_edge":0.970,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673958,40.268272]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":230,"matched_point_type":"matched","edge_index":24,"distance_along_edge":1.000,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673759,40.268299]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":231,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.141,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673546,40.268326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":232,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.293,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673340,40.268356]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":233,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.440,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.673126,40.268383]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":234,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.591,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672905,40.268417]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":235,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.749,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672691,40.268452]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":236,"matched_point_type":"matched","edge_index":25,"distance_along_edge":0.902,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672470,40.268482]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":237,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.091,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672249,40.268517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":238,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.334,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.672028,40.268547]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":239,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.575,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.671806,40.268581]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":240,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.817,"distance_from_trace_point":5.632}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.671570,40.268616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":241,"matched_point_type":"matched","edge_index":27,"distance_along_edge":0.069,"distance_from_trace_point":5.632}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.671326,40.268646]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":242,"matched_point_type":"matched","edge_index":27,"distance_along_edge":0.310,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.671097,40.268681]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":243,"matched_point_type":"matched","edge_index":27,"distance_along_edge":0.538,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670876,40.268711]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":244,"matched_point_type":"matched","edge_index":27,"distance_along_edge":0.758,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670654,40.268742]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":245,"matched_point_type":"matched","edge_index":27,"distance_along_edge":0.977,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670441,40.268772]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":246,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.088,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670227,40.268799]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":247,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.186,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.670006,40.268829]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":248,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.287,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669785,40.268860]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":249,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.389,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669563,40.268894]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":250,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.491,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669327,40.268932]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":251,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.601,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.669090,40.268970]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":252,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.710,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.668854,40.269016]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":253,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.820,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.668633,40.269066]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":254,"matched_point_type":"matched","edge_index":28,"distance_along_edge":0.925,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.668411,40.269104]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":255,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.054,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.668190,40.269146]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":256,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.257,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667984,40.269192]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":257,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.448,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667778,40.269241]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":258,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.639,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667580,40.269295]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":259,"matched_point_type":"matched","edge_index":29,"distance_along_edge":0.826,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667374,40.269352]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":260,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.016,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.667175,40.269405]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":261,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.158,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666977,40.269459]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":262,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.301,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666771,40.269516]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":263,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.450,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666580,40.269592]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":264,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.596,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666397,40.269669]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":265,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.737,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666222,40.269737]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":266,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.871,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.666046,40.269810]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":267,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.007,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665871,40.269882]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":268,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.154,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665703,40.269951]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":269,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.294,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665527,40.270023]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":270,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.441,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665367,40.270092]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":271,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.577,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665207,40.270161]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":272,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.712,"distance_from_trace_point":5.306}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.665047,40.270226]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":273,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.846,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664871,40.270298]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":274,"matched_point_type":"matched","edge_index":31,"distance_along_edge":0.993,"distance_from_trace_point":4.639}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664711,40.270363]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":275,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.128,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664536,40.270428]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":276,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.272,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664360,40.270496]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":277,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.418,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664192,40.270561]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":278,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.557,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.664017,40.270630]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":279,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.703,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663841,40.270695]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":280,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.847,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663673,40.270760]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":281,"matched_point_type":"matched","edge_index":32,"distance_along_edge":0.986,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663490,40.270828]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":282,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.038,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663315,40.270893]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":283,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.079,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.663132,40.270958]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":284,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.120,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662949,40.271027]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":285,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.162,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662758,40.271095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":286,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.206,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662575,40.271160]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":287,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.248,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662399,40.271225]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":288,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.288,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662224,40.271290]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":289,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.328,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.662056,40.271351]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":290,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.366,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661911,40.271404]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":291,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.400,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661766,40.271450]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":292,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.432,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661629,40.271496]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":293,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.463,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661507,40.271542]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":294,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.491,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661385,40.271580]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":295,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.518,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661270,40.271622]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":296,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.544,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661156,40.271660]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":297,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.570,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.661049,40.271694]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":298,"matched_point_type":"interpolated","edge_index":33,"distance_along_edge":0.594,"distance_from_trace_point":2.839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660942,40.271732]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":299,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.618,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660828,40.271767]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":300,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.644,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660629,40.271828]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":301,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.688,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660500,40.271870]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":302,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.717,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660355,40.271919]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":303,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.750,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660240,40.271954]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":304,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.775,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.660133,40.271988]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":305,"matched_point_type":"interpolated","edge_index":33,"distance_along_edge":0.799,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659920,40.272053]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":306,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.847,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659744,40.272099]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":307,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.885,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659531,40.272156]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":308,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.932,"distance_from_trace_point":3.221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659409,40.272186]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":309,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.958,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659286,40.272221]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":310,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.985,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659164,40.272251]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":311,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.016,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.659050,40.272282]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":312,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.051,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658920,40.272312]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":313,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.090,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658791,40.272346]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":314,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.130,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658669,40.272377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":315,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.167,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658546,40.272408]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":316,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.204,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658424,40.272438]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":317,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.242,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658302,40.272469]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":318,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.279,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658173,40.272503]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":319,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.319,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.658043,40.272537]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":320,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.358,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657921,40.272568]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":321,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.395,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657791,40.272602]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":322,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.435,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657646,40.272636]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":323,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.479,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657524,40.272667]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":324,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.516,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657402,40.272697]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":325,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.553,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657272,40.272732]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":326,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.593,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657143,40.272762]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":327,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.632,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.657013,40.272797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":328,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.672,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656891,40.272827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":329,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.709,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656761,40.272861]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":330,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.749,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656639,40.272892]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":331,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.786,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656525,40.272919]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":332,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.821,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656349,40.272964]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":333,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.875,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656235,40.272991]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":334,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.909,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.656067,40.273037]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":335,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.961,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655891,40.273079]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":336,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.060,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655670,40.273129]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":337,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.345,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655548,40.273159]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":338,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.505,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655403,40.273193]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":339,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.693,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655258,40.273228]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":340,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.881,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.655121,40.273262]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":341,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.013,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654984,40.273293]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":342,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.051,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654846,40.273319]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":343,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.088,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654701,40.273350]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":344,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.128,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654549,40.273384]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":345,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.170,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654388,40.273418]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":346,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.214,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654221,40.273457]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":347,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.260,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.654030,40.273495]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":348,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.312,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653854,40.273533]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":349,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.360,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653664,40.273575]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":350,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.413,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653473,40.273617]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":351,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.465,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653282,40.273659]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":352,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.518,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.653084,40.273701]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":353,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.572,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652878,40.273746]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":354,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.629,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652664,40.273792]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":355,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.688,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652458,40.273834]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":356,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.744,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652252,40.273880]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":357,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.801,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.652046,40.273926]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":358,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.858,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651848,40.273968]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":359,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.912,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651649,40.274010]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":360,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.966,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651443,40.274055]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":361,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.032,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651237,40.274101]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":362,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.110,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.651047,40.274147]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":363,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.182,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650848,40.274193]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":364,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.258,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650650,40.274235]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":365,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.332,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650452,40.274281]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":366,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.408,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650253,40.274326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":367,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.483,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.650055,40.274368]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":368,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.558,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649864,40.274414]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":369,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.630,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649666,40.274460]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":370,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.705,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649467,40.274506]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":371,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.781,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649277,40.274548]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":372,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.853,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.649101,40.274586]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":373,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.919,"distance_from_trace_point":4.008}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648933,40.274624]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":374,"matched_point_type":"matched","edge_index":37,"distance_along_edge":0.983,"distance_from_trace_point":4.410}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648788,40.274658]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":375,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.216,"distance_from_trace_point":4.410}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648666,40.274685]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":376,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.482,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648521,40.274704]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":377,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.789,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648407,40.274723]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":378,"matched_point_type":"interpolated","edge_index":39,"distance_along_edge":0.002,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648277,40.274738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":379,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.017,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.648140,40.274754]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":380,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.032,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647987,40.274776]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":381,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.050,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647835,40.274792]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":382,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.067,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647667,40.274815]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":383,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.087,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647499,40.274834]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":384,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.106,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647316,40.274857]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":385,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.127,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.647133,40.274879]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":386,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.148,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646942,40.274902]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":387,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.169,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646751,40.274925]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":388,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.191,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646545,40.274952]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":389,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.215,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646347,40.274971]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":390,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.237,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.646149,40.274994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":391,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.260,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645943,40.275013]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":392,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.284,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645737,40.275032]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":393,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.307,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645538,40.275051]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":394,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.330,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645332,40.275074]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":395,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.353,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.645119,40.275093]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":396,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.377,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644890,40.275112]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":397,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.403,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644653,40.275135]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":398,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.430,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644417,40.275154]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":399,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.457,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.644180,40.275173]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":400,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.484,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643936,40.275192]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":401,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.512,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643700,40.275208]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":402,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.539,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643456,40.275219]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":403,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.566,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.643219,40.275227]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":404,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.593,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642982,40.275238]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":405,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.620,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642746,40.275242]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":406,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.647,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642517,40.275246]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":407,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.672,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642296,40.275249]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":408,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.697,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.642075,40.275249]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":409,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.722,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641861,40.275253]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":410,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.747,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641655,40.275253]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":411,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.770,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641449,40.275257]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":412,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.793,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641258,40.275257]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":413,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.815,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.641068,40.275261]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":414,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.836,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640877,40.275261]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":415,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.858,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640701,40.275265]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":416,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.878,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640533,40.275265]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":417,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.897,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640373,40.275265]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":418,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.915,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640236,40.275269]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":419,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.930,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.640114,40.275269]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":420,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.944,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639938,40.275269]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":421,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.964,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639809,40.275272]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":422,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.978,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639679,40.275272]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":423,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.993,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639519,40.275276]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":424,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.018,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639275,40.275284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":425,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.063,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.639137,40.275288]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":426,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.088,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638992,40.275284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":427,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.115,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638809,40.275280]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":428,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.149,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638634,40.275276]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":429,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.181,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638443,40.275272]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":430,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.216,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638245,40.275269]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":431,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.253,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.638039,40.275261]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":432,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.291,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637840,40.275257]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":433,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.327,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637627,40.275246]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":434,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.366,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637421,40.275230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":435,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.404,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.637207,40.275215]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":436,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.444,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636993,40.275204]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":437,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.483,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636772,40.275188]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":438,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.524,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636551,40.275177]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":439,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.565,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636330,40.275166]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":440,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.606,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.636108,40.275154]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":441,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.646,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635887,40.275143]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":442,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.687,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635674,40.275131]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":443,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.727,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635460,40.275112]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":444,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.766,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635262,40.275097]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":445,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.803,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.635063,40.275082]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":446,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.839,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634865,40.275066]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":447,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.876,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634682,40.275051]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":448,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.910,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634499,40.275040]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":449,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.944,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634338,40.275032]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":450,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.973,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634186,40.275028]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":451,"matched_point_type":"matched","edge_index":41,"distance_along_edge":0.015,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.634071,40.275009]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":452,"matched_point_type":"matched","edge_index":41,"distance_along_edge":0.243,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633881,40.274971]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":453,"matched_point_type":"matched","edge_index":41,"distance_along_edge":0.630,"distance_from_trace_point":3.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633728,40.274887]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":454,"matched_point_type":"matched","edge_index":41,"distance_along_edge":0.000,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633530,40.274826]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":455,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.874,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633347,40.274864]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":456,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.162,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633125,40.274891]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":457,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.521,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.633003,40.274899]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":458,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.717,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632858,40.274910]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":459,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.951,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632706,40.274906]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":460,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.022,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632538,40.274899]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":461,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.052,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632362,40.274891]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":462,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.083,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.632179,40.274883]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":463,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.116,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631989,40.274872]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":464,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.150,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631790,40.274864]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":465,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.186,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631599,40.274853]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":466,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.220,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631401,40.274841]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":467,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.256,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.631195,40.274834]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":468,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.292,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630989,40.274822]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":469,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.329,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630783,40.274803]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":470,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.366,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630577,40.274788]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":471,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.403,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630379,40.274773]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":472,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.439,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.630173,40.274757]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":473,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.476,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629959,40.274742]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":474,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.514,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629753,40.274727]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":475,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.551,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629539,40.274708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":476,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.590,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629318,40.274693]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":477,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.630,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.629089,40.274673]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":478,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.671,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628860,40.274658]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":479,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.712,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628639,40.274639]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":480,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.752,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628410,40.274620]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":481,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.793,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.628181,40.274601]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":482,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.834,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627960,40.274582]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":483,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.874,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627731,40.274563]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":484,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.915,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627518,40.274544]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":485,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.953,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627296,40.274525]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":486,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.993,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.627098,40.274502]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":487,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.245,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626892,40.274487]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":488,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.561,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626694,40.274490]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":489,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.861,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626495,40.274506]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":490,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.128,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626320,40.274529]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":491,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.339,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626167,40.274555]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":492,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.524,"distance_from_trace_point":4.410}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.626038,40.274578]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":493,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.683,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625824,40.274578]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":494,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.936,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625664,40.274483]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":495,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.000,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625481,40.274475]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":496,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.218,"distance_from_trace_point":4.008}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625305,40.274551]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":497,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.044,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625198,40.274586]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":498,"matched_point_type":"interpolated","edge_index":53,"distance_along_edge":0.115,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.625069,40.274616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":499,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.198,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624924,40.274647]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":500,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.289,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624763,40.274681]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":501,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.390,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624588,40.274708]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":502,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.499,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624405,40.274734]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":503,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.612,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624214,40.274761]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":504,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.730,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.624016,40.274788]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":505,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.853,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623825,40.274841]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":506,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.976,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623627,40.274868]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":507,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.021,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623436,40.274895]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":508,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.047,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623238,40.274918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":509,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.073,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.623039,40.274944]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":510,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.100,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622833,40.274971]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":511,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.127,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622635,40.274994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":512,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.154,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622444,40.275021]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":513,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.179,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622246,40.275043]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":514,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.206,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.622040,40.275070]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":515,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.233,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621849,40.275093]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":516,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.259,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621651,40.275116]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":517,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.285,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621460,40.275139]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":518,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.311,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621269,40.275162]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":519,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.336,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.621086,40.275185]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":520,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.360,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620911,40.275208]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":521,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.384,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620735,40.275227]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":522,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.407,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620560,40.275249]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":523,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.431,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620377,40.275269]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":524,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.455,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620193,40.275291]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":525,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.479,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.620010,40.275314]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":526,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.504,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619827,40.275337]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":527,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.528,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619637,40.275360]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":528,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.554,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619438,40.275383]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":529,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.580,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619247,40.275406]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":530,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.606,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.619049,40.275429]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":531,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.632,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618851,40.275452]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":532,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.658,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618652,40.275478]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":533,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.685,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618454,40.275501]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":534,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.711,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618248,40.275528]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":535,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.739,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.618042,40.275551]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":536,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.766,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617851,40.275585]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":537,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.792,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617653,40.275616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":538,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.819,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617462,40.275650]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":539,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.844,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617264,40.275692]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":540,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.872,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.617073,40.275734]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":541,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.898,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616875,40.275776]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":542,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.925,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616684,40.275818]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":543,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.951,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616501,40.275856]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":544,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.976,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616310,40.275898]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":545,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.041,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.616127,40.275940]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":546,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.530,"distance_from_trace_point":2.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615944,40.275982]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":547,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.001,"distance_from_trace_point":1.806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615746,40.276020]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":548,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.023,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615555,40.276058]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":549,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.045,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615356,40.276104]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":550,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.068,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.615150,40.276154]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":551,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.092,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614944,40.276199]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":552,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.116,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614731,40.276249]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":553,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.141,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614525,40.276299]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":554,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.165,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614311,40.276348]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":555,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.189,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.614098,40.276398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":556,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.214,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613884,40.276447]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":557,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.239,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613670,40.276497]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":558,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.263,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613457,40.276546]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":559,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.288,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613243,40.276596]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":560,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.313,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.613029,40.276646]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":561,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.338,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612823,40.276695]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":562,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.361,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612617,40.276741]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":563,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.385,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612427,40.276787]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":564,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.407,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612228,40.276833]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":565,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.430,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.612030,40.276875]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":566,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.453,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611832,40.276909]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":567,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.476,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611626,40.276939]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":568,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.499,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611412,40.276970]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":569,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.523,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.611191,40.276989]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":570,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.548,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610970,40.277000]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":571,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.572,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610741,40.277008]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":572,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.597,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610512,40.277016]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":573,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.623,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610283,40.277027]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":574,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.648,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.610046,40.277035]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":575,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.674,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609818,40.277046]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":576,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.700,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609596,40.277054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":577,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.724,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609367,40.277061]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":578,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.750,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.609138,40.277073]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":579,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.775,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608917,40.277081]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":580,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.800,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608688,40.277088]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":581,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.825,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608467,40.277100]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":582,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.849,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608246,40.277107]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":583,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.874,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.608032,40.277115]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":584,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.898,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607811,40.277126]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":585,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.922,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607590,40.277134]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":586,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.947,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607361,40.277142]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":587,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.972,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.607124,40.277153]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":588,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.998,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606888,40.277157]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":589,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.049,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606644,40.277164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":590,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.103,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606407,40.277172]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":591,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.155,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.606163,40.277176]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":592,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.209,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605911,40.277184]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":593,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.264,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605659,40.277191]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":594,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.320,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605415,40.277195]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":595,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.374,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.605171,40.277203]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":596,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.428,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604919,40.277210]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":597,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.483,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604683,40.277214]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":598,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.535,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604446,40.277222]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":599,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.588,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.604210,40.277229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":600,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.640,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603981,40.277233]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":601,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.690,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603760,40.277241]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":602,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.739,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603539,40.277245]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":603,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.788,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603310,40.277252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":604,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.838,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.603096,40.277256]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":605,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.886,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602882,40.277260]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":606,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.933,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602676,40.277252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":607,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.978,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602470,40.277252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":608,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.073,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602264,40.277248]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":609,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.214,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.602066,40.277245]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":610,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.349,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601860,40.277245]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":611,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.490,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601662,40.277241]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":612,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.625,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601448,40.277237]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":613,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.771,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601250,40.277233]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":614,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.906,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.601044,40.277229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":615,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.040,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600838,40.277225]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":616,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.159,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600624,40.277218]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":617,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.282,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600403,40.277210]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":618,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.410,"distance_from_trace_point":4.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.600182,40.277203]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":619,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.538,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599960,40.277199]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":620,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.665,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599731,40.277191]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":621,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.797,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599503,40.277184]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":622,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.930,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599274,40.277176]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":623,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.030,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.599045,40.277164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":624,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.093,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598808,40.277157]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":625,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.159,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598579,40.277145]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":626,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.222,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598351,40.277138]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":627,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.286,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.598129,40.277126]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":628,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.347,"distance_from_trace_point":4.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597908,40.277119]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":629,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.408,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597694,40.277111]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":630,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.467,"distance_from_trace_point":4.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597481,40.277103]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":631,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.527,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597282,40.277096]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":632,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.582,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.597084,40.277088]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":633,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.637,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596901,40.277084]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":634,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.687,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596741,40.277077]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":635,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.732,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596588,40.277073]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":636,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.774,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596443,40.277077]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":637,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.814,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596298,40.277081]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":638,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.854,"distance_from_trace_point":4.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596161,40.277084]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":639,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.892,"distance_from_trace_point":4.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.596016,40.277088]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":640,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.932,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595863,40.277088]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":641,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.975,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595711,40.277092]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":642,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.007,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595551,40.277096]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":643,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.026,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595375,40.277100]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":644,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.047,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595200,40.277107]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":645,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.068,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.595016,40.277111]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":646,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.090,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594826,40.277115]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":647,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.113,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594635,40.277119]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":648,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.136,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594444,40.277122]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":649,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.159,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594254,40.277126]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":650,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.182,"distance_from_trace_point":5.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.594063,40.277130]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":651,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.204,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593880,40.277130]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":652,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.226,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593689,40.277134]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":653,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.249,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593506,40.277138]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":654,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.271,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593315,40.277138]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":655,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.294,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.593132,40.277142]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":656,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.316,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592949,40.277145]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":657,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.338,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592766,40.277149]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":658,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.360,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592590,40.277153]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":659,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.380,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592415,40.277157]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":660,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.401,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592232,40.277161]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":661,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.423,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.592049,40.277164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":662,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.445,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591866,40.277168]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":663,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.467,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591675,40.277172]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":664,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.490,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591492,40.277172]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":665,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.512,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591286,40.277180]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":666,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.537,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.591095,40.277184]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":667,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.559,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590897,40.277187]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":668,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.583,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590706,40.277191]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":669,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.606,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590500,40.277199]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":670,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.631,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590294,40.277203]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":671,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.655,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.590088,40.277206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":672,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.680,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589882,40.277210]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":673,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.704,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589684,40.277214]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":674,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.728,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589493,40.277218]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":675,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.751,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589302,40.277218]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":676,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.774,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.589127,40.277218]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":677,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.795,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588959,40.277222]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":678,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.815,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588791,40.277222]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":679,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.835,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588623,40.277222]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":680,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.855,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588470,40.277225]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":681,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.873,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588303,40.277225]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":682,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.893,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.588142,40.277225]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":683,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.912,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587967,40.277229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":684,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.933,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587791,40.277229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":685,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.954,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587624,40.277233]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":686,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.974,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587448,40.277233]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":687,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.995,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587280,40.277241]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":688,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.067,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.587120,40.277252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":689,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.150,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586975,40.277264]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":690,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.225,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586830,40.277275]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":691,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.301,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586700,40.277283]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":692,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.368,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586578,40.277290]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":693,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.431,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586372,40.277306]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":694,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.538,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586189,40.277317]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":695,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.632,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.586044,40.277328]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":696,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.708,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585922,40.277336]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":697,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.771,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585785,40.277348]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":698,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.842,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585663,40.277355]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":699,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.905,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585480,40.277367]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":700,"matched_point_type":"matched","edge_index":62,"distance_along_edge":1.000,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585274,40.277390]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":701,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.160,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.585030,40.277420]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":702,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.480,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584908,40.277431]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":703,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.640,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584778,40.277447]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":704,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.810,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584656,40.277462]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":705,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.970,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584526,40.277477]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":706,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.050,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584404,40.277493]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":707,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.108,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584282,40.277504]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":708,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.165,"distance_from_trace_point":3.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584160,40.277515]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":709,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.223,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.584038,40.277531]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":710,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.280,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583809,40.277554]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":711,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.388,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583687,40.277569]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":712,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.445,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583565,40.277580]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":713,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.503,"distance_from_trace_point":2.611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583435,40.277596]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":714,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.564,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583313,40.277607]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":715,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.621,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583176,40.277622]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":716,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.686,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.583054,40.277630]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":717,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.743,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582924,40.277637]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":718,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.804,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582802,40.277645]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":719,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.861,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582672,40.277653]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":720,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.921,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582535,40.277660]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":721,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.986,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582397,40.277672]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":722,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.060,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582260,40.277683]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":723,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.136,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.582123,40.277695]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":724,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.213,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581985,40.277706]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":725,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.290,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581841,40.277718]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":726,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.371,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581680,40.277733]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":727,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.460,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581528,40.277740]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":728,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.545,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581367,40.277752]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":729,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.635,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581200,40.277760]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":730,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.728,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.581032,40.277771]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":731,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.822,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580856,40.277782]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":732,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.919,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580666,40.277794]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":733,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.019,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580475,40.277802]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":734,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.097,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580276,40.277813]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":735,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.179,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.580078,40.277824]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":736,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.261,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579880,40.277836]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":737,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.343,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579674,40.277847]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":738,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.428,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579468,40.277859]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":739,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.513,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579262,40.277870]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":740,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.597,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.579056,40.277885]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":741,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.682,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578850,40.277897]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":742,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.767,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578651,40.277908]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":743,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.849,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578445,40.277920]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":744,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.934,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578239,40.277931]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":745,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.042,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.578041,40.277939]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":746,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.224,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577843,40.277950]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":747,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.406,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577660,40.277958]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":748,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.573,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577507,40.277966]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":749,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.713,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577385,40.277973]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":750,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.825,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577255,40.277977]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":751,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.944,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577171,40.278046]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":752,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.065,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577133,40.278183]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":753,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.200,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577065,40.278351]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":754,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.369,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.577026,40.278435]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":755,"matched_point_type":"interpolated","edge_index":69,"distance_along_edge":0.454,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576988,40.278530]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":756,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.550,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576942,40.278629]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":757,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.651,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576897,40.278732]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":758,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.756,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576851,40.278839]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":759,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.864,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576797,40.278954]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":760,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.981,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576752,40.279072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":761,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.093,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576698,40.279186]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":762,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.202,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576653,40.279316]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":763,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.323,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576599,40.279442]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":764,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.442,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576546,40.279564]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":765,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.558,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576500,40.279686]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":766,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.672,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576447,40.279808]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":767,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.788,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576393,40.279930]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":768,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.904,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576347,40.280056]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":769,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.010,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576309,40.280178]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":770,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.063,"distance_from_trace_point":2.574}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576248,40.280312]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":771,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.122,"distance_from_trace_point":2.107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576195,40.280441]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":772,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.179,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576141,40.280567]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":773,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.235,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576080,40.280701]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":774,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.294,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.576027,40.280823]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":775,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.348,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575981,40.280937]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":776,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.399,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575928,40.281055]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":777,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.451,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575867,40.281170]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":778,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.503,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575806,40.281284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":779,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.555,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575745,40.281395]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":780,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.606,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575668,40.281502]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":781,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.657,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575577,40.281593]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":782,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.706,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575447,40.281670]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":783,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.758,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575325,40.281738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":784,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.807,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575180,40.281818]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":785,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.864,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.575043,40.281891]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":786,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.918,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574890,40.281967]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":787,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.977,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574730,40.282051]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":788,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.053,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574570,40.282135]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":789,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.137,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574409,40.282219]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":790,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.221,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574249,40.282307]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":791,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.307,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574089,40.282391]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":792,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.391,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573929,40.282475]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":793,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.475,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573776,40.282558]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":794,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.557,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573624,40.282639]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":795,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.637,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573486,40.282711]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":796,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.709,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573372,40.282772]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":797,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.770,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573273,40.282825]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":798,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.822,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573128,40.282906]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":799,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.900,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573021,40.282959]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":800,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.955,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572952,40.283051]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":801,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.033,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572990,40.283188]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":802,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.132,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573036,40.283356]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":803,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.252,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573059,40.283447]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":804,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.318,"distance_from_trace_point":0.644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573090,40.283546]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":805,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.389,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573120,40.283649]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":806,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.463,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573143,40.283752]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":807,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.537,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573174,40.283852]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":808,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.608,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573204,40.283947]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":809,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.677,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573227,40.284035]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":810,"matched_point_type":"interpolated","edge_index":73,"distance_along_edge":0.740,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573265,40.284180]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":811,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.844,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573296,40.284294]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":812,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.926,"distance_from_trace_point":2.608}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573265,40.284420]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":813,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.095,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573166,40.284531]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":814,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.346,"distance_from_trace_point":3.884}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573181,40.284649]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":815,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.568,"distance_from_trace_point":4.399}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573296,40.284733]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":816,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.793,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573463,40.284809]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":817,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.021,"distance_from_trace_point":5.928}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573494,40.284935]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":818,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.060,"distance_from_trace_point":2.608}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573532,40.285061]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":819,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.100,"distance_from_trace_point":2.709}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573578,40.285198]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":820,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.143,"distance_from_trace_point":2.608}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573616,40.285332]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":821,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.185,"distance_from_trace_point":2.608}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573654,40.285465]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":822,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.227,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573692,40.285603]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":823,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.271,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573738,40.285748]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":824,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.317,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573784,40.285900]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":825,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.365,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573830,40.286053]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":826,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.413,"distance_from_trace_point":1.976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573875,40.286213]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":827,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.463,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573921,40.286377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":828,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.515,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573975,40.286541]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":829,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.567,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573997,40.286633]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":830,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.596,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574028,40.286724]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":831,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.625,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574051,40.286819]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":832,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.654,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574081,40.286915]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":833,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.685,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574112,40.287010]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":834,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.715,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574142,40.287109]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":835,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.746,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574173,40.287209]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":836,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.778,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574203,40.287312]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":837,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.810,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574234,40.287415]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":838,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.843,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574265,40.287521]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":839,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.876,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574295,40.287621]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":840,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.907,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574326,40.287727]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":841,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.941,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574356,40.287830]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":842,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.974,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574379,40.287930]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":843,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.022,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574409,40.288033]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":844,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.171,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574432,40.288128]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":845,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.308,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574471,40.288292]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":846,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.544,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574501,40.288406]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":847,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.709,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574516,40.288498]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":848,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.840,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574394,40.288635]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":849,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.042,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574234,40.288662]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":850,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.086,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.574036,40.288700]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":851,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.141,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573944,40.288757]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":852,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.173,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573845,40.288815]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":853,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.206,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573738,40.288879]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":854,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.242,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573624,40.288944]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":855,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.281,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573509,40.289013]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":856,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.320,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573387,40.289085]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":857,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.361,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573250,40.289165]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":858,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.407,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.573105,40.289249]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":859,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.456,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572968,40.289333]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":860,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.503,"distance_from_trace_point":1.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572830,40.289413]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":861,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.550,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572685,40.289497]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":862,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.598,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572540,40.289585]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":863,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.648,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572395,40.289673]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":864,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.698,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572258,40.289753]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":865,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.744,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.572121,40.289833]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":866,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.790,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571983,40.289917]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":867,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.837,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571861,40.289989]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":868,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.879,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571754,40.290051]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":869,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.915,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571594,40.290146]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":870,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.969,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571434,40.290131]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":871,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.019,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571327,40.290031]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":872,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.048,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571198,40.289909]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":873,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.083,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571114,40.289841]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":874,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.104,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.571007,40.289783]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":875,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.126,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570900,40.289730]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":876,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.148,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570801,40.289677]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":877,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.169,"distance_from_trace_point":3.183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570702,40.289623]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":878,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.189,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570595,40.289570]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":879,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.211,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570480,40.289513]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":880,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.234,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570358,40.289448]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":881,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.259,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570236,40.289383]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":882,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.285,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.570114,40.289322]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":883,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.310,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569984,40.289257]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":884,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.336,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569862,40.289192]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":885,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.361,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569740,40.289124]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":886,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.387,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569611,40.289059]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":887,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.413,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569489,40.288994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":888,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.439,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569359,40.288929]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":889,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.465,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569237,40.288864]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":890,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.490,"distance_from_trace_point":1.061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.569115,40.288803]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":891,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.515,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568993,40.288738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":892,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.540,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568855,40.288673]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":893,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.568,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568741,40.288609]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":894,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.592,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568611,40.288548]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":895,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.618,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568489,40.288483]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":896,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.643,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568367,40.288422]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":897,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.668,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568245,40.288357]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":898,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.693,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568123,40.288300]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":899,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.718,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.568008,40.288235]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":900,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.742,"distance_from_trace_point":1.420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567879,40.288174]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":901,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.768,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567757,40.288113]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":902,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.792,"distance_from_trace_point":2.122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567635,40.288052]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":903,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.817,"distance_from_trace_point":2.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567505,40.287994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":904,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.843,"distance_from_trace_point":3.528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567383,40.287937]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":905,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.867,"distance_from_trace_point":3.887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567276,40.287888]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":906,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.888,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567078,40.287800]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":907,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.927,"distance_from_trace_point":4.586}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566925,40.287731]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":908,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.957,"distance_from_trace_point":4.586}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566826,40.287682]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":909,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.978,"distance_from_trace_point":3.887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566765,40.287548]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":910,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.055,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566818,40.287460]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":911,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.111,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566887,40.287357]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":912,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.179,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.566978,40.287220]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":913,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.269,"distance_from_trace_point":2.107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567078,40.287056]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":914,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.375,"distance_from_trace_point":2.868}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567169,40.286934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":915,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.457,"distance_from_trace_point":1.539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567238,40.286831]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":916,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.524,"distance_from_trace_point":0.643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567299,40.286743]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":917,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.582,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567383,40.286636]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":918,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.655,"distance_from_trace_point":0.769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567467,40.286510]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":919,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.738,"distance_from_trace_point":1.354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.567566,40.286381]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":920,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.826,"distance_from_trace_point":0.000}}
]}
 */
void TestAutoMiddletownRoadToLandingsDrive() {
  std::string route_json_str =
      R"({"trip":{"language":"en-US","summary":{"max_lon":-76.566711,"max_lat":40.290199,"time":976,"length":15.425,"min_lat":40.246044,"min_lon":-76.709229},"locations":[{"lon":-76.706558,"lat":40.246048,"type":"break"},{"lon":-76.567642,"lat":40.286263,"type":"break"}],"units":"kilometers","legs":[{"shape":"wplwkAhqxhqCi\\bFmYdFkQvCsQbGyLnHeOzKiMlJoXpSmZzTiWhNyMdEsPxBwS\\qLm@us@gCyR_@{Pl@w|@dF_{BvMk`@xB}D\\{UlAglA~HsP\\eKm@{AqdBsAww@]_}@]ya@sA}h@{Lm|BwWyqDca@cnAqa@mhAy[a{@oJmUuN_^g^sy@gm@enAqa@ap@sz@msA{_A_{AeO_^iGePgDwMgEqRgH_h@oDkV}DsQiH}SaHuPgYco@uT_h@cf@k_A_DoIgMq\\{KeZgDkLyCiMqBqQsF{_AqBm^eEmt@eFw`BsAaf@{Aag@mEo|AyF{hAWeFuEaq@uDed@?Mu@_JuEgm@yG}}@cGsz@kFyk@uEo]MuE_EeZuDkUkB_J_Ice@qQyl@_^kiAc\\ijAqf@wkBaRgw@oSk_A{Keo@wr@_lEmJeo@cp@mbFmd@_dDaByLeAgNiBgY}Jg`BiGmsAgE{hAyA_}@_@syA]{rB]k`@\\}h@d@}h@bB{k@nC}qA~DglALuOt@mJ\\wDrAuEpBeDdA{Al@kBLyBMyBe@yB}@{JcAwYlDsmBvIujChCar@jAa\\t@sPjAsQWkUkAqR{AiMMuETmJjB{AjAyBd@wCGgDm@iCe@{@m@}@gCyLqCkU]yCmEyk@qBkLiM_pBq\\auF_Egb@oN_gAmDkVaC}Tei@etDaIkj@mE{j@oIgeGsEoaGLwWd@qzAjBgkBxB_{AzAaqAe@or@}@cnAe@ytAcAglBmAccBe@isCiGswBOgDuDun@qHewA{Ayk@{@iWsA_^kBww@qGsvCcA{j@e@iWuOgDil@aSiaAeZaHyA_w@iWsUmKsFeEoDcG{P{_@aCqG}}@guBavAhWs@iC{AyBqB}@yBMcB\\iBzAmAxBe@vCFfDceEry@kj@nIoDm^}|Ay}CnTkUj~@cyBnm@azAb\\{~@dd@xVjo@ja@","summary":{"max_lon":-76.566711,"max_lat":40.290199,"time":976,"length":15.425,"min_lat":40.246044,"min_lon":-76.709229},"maneuvers":[{"travel_mode":"drive","begin_shape_index":0,"length":1.336,"time":90,"type":1,"end_shape_index":25,"instruction":"Drive north on Middletown Road.","verbal_pre_transition_instruction":"Drive north on Middletown Road for 1.3 kilometers.","travel_type":"car","street_names":["Middletown Road"]},{"travel_type":"car","verbal_pre_transition_instruction":"Turn right.","verbal_transition_alert_instruction":"Turn right.","length":0.824,"instruction":"Turn right.","end_shape_index":32,"type":10,"time":45,"verbal_post_transition_instruction":"Continue for 800 meters.","begin_shape_index":25,"travel_mode":"drive"},{"travel_type":"car","verbal_pre_transition_instruction":"Stay straight to take the U.S. 3 22 East ramp.","verbal_transition_alert_instruction":"Stay straight to take the U.S. 3 22 East ramp.","instruction":"Stay straight to take the US 322 East ramp.","end_shape_index":36,"type":17,"time":22,"begin_shape_index":32,"length":0.374,"sign":{"exit_branch_elements":[{"consecutive_count":1,"text":"US 322 East"}]},"travel_mode":"drive"},{"travel_type":"car","verbal_pre_transition_instruction":"Merge onto U.S. 3 22 East.","verbal_post_transition_instruction":"Continue for 700 meters.","instruction":"Merge onto US 322 East.","end_shape_index":42,"type":25,"time":32,"street_names":["US 322 East"],"length":0.701,"begin_shape_index":36,"travel_mode":"drive"},{"travel_type":"car","verbal_pre_transition_instruction":"Take the U.S. 3 22 East exit on the right toward Ephrata.","verbal_transition_alert_instruction":"Take the U.S. 3 22 East exit on the right.","instruction":"Take the US 322 East exit on the right toward Ephrata.","end_shape_index":54,"type":20,"time":37,"begin_shape_index":42,"length":0.570,"sign":{"exit_toward_elements":[{"text":"Ephrata"}],"exit_branch_elements":[{"text":"US 322 East"}]},"travel_mode":"drive"},{"travel_type":"car","verbal_pre_transition_instruction":"Continue on U.S. 3 22 for 4.4 kilometers.","verbal_transition_alert_instruction":"Continue on U.S. 3 22.","length":4.414,"instruction":"Continue on US 322.","end_shape_index":114,"type":8,"time":250,"street_names":["US 322"],"begin_shape_index":54,"travel_mode":"drive"},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Enter the roundabout and take the 2nd exit.","verbal_transition_alert_instruction":"Enter the roundabout and take the 2nd exit.","length":0.027,"instruction":"Enter the roundabout and take the 2nd exit.","end_shape_index":119,"type":26,"time":2,"street_names":["US 322"],"begin_shape_index":114,"roundabout_exit_count":2},{"travel_type":"car","verbal_pre_transition_instruction":"Exit the roundabout onto East Governor Road, U.S. 3 22 East.","verbal_post_transition_instruction":"Continue for 700 meters.","instruction":"Exit the roundabout onto East Governor Road\/US 322 East.","end_shape_index":132,"type":27,"time":31,"street_names":["East Governor Road","US 322 East"],"length":0.656,"begin_shape_index":119,"travel_mode":"drive"},{"travel_type":"car","verbal_pre_transition_instruction":"Enter the roundabout and take the 2nd exit.","verbal_transition_alert_instruction":"Enter the roundabout and take the 2nd exit.","length":0.042,"instruction":"Enter the roundabout and take the 2nd exit.","end_shape_index":139,"type":26,"time":3,"begin_shape_index":132,"roundabout_exit_count":2,"travel_mode":"drive"},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Exit the roundabout onto East Governor Road, U.S. 3 22 East.","begin_street_names":["East Governor Road","US 322 East"],"verbal_post_transition_instruction":"Continue on U.S. 3 22 East for 4.1 kilometers.","instruction":"Exit the roundabout onto East Governor Road\/US 322 East. Continue on US 322 East.","end_shape_index":177,"type":27,"time":197,"street_names":["US 322 East"],"length":4.131,"begin_shape_index":139},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn left onto Hinkle Road.","verbal_transition_alert_instruction":"Turn left onto Hinkle Road.","length":0.701,"instruction":"Turn left onto Hinkle Road.","end_shape_index":188,"type":15,"time":105,"verbal_post_transition_instruction":"Continue for 700 meters.","street_names":["Hinkle Road"],"begin_shape_index":177},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn left onto South Forge Road, Pennsylvania 1 17.","verbal_transition_alert_instruction":"Turn left onto South Forge Road.","length":0.158,"instruction":"Turn left onto South Forge Road\/PA 117.","end_shape_index":189,"type":15,"time":22,"verbal_post_transition_instruction":"Continue for 200 meters.","street_names":["South Forge Road","PA 117"],"begin_shape_index":188},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Enter the roundabout and take the 1st exit.","verbal_transition_alert_instruction":"Enter the roundabout and take the 1st exit.","length":0.061,"instruction":"Enter the roundabout and take the 1st exit.","end_shape_index":198,"type":26,"time":5,"street_names":["South Forge Road","PA 117"],"begin_shape_index":189,"roundabout_exit_count":1},{"travel_type":"car","verbal_pre_transition_instruction":"Exit the roundabout onto South Forge Road, Pennsylvania 1 17.","verbal_post_transition_instruction":"Continue for 400 meters.","instruction":"Exit the roundabout onto South Forge Road\/PA 117.","end_shape_index":200,"type":27,"time":28,"street_names":["South Forge Road","PA 117"],"length":0.441,"begin_shape_index":198,"travel_mode":"drive"},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn right onto Airport Road.","verbal_transition_alert_instruction":"Turn right onto Airport Road.","length":0.317,"instruction":"Turn right onto Airport Road.","end_shape_index":202,"type":10,"time":41,"verbal_post_transition_instruction":"Continue for 300 meters.","street_names":["Airport Road"],"begin_shape_index":200},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn right onto Taxiway Road.","verbal_transition_alert_instruction":"Turn right onto Taxiway Road.","length":0.500,"instruction":"Turn right onto Taxiway Road.","end_shape_index":206,"type":10,"time":48,"verbal_post_transition_instruction":"Continue for a half kilometer.","street_names":["Taxiway Road"],"begin_shape_index":202},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn right onto Landings Drive.","verbal_transition_alert_instruction":"Turn right onto Landings Drive.","length":0.172,"instruction":"Turn right onto Landings Drive.","end_shape_index":208,"type":10,"time":18,"verbal_post_transition_instruction":"Continue for 200 meters.","street_names":["Landings Drive"],"begin_shape_index":206},{"travel_type":"car","travel_mode":"drive","begin_shape_index":208,"time":0,"type":4,"end_shape_index":208,"instruction":"You have arrived at your destination.","length":0.000,"verbal_transition_alert_instruction":"You will arrive at your destination.","verbal_pre_transition_instruction":"You have arrived at your destination."}]}],"status_message":"Found route between points","status":0}})";
  NavigatorTest nav;
  nav.SetRoute(route_json_str);
  uint32_t leg_index = 0;

  TryRouteLegCount(nav, 1);
  TryRouteManeuverCount(nav, leg_index, 18);
  TryRouteLanguage(nav, "en-US");
  TryRouteUnits(nav, "kilometers", true);

  // Test the final long transition alert methods for metric values
  TryLength("GetFinalLongTransitionAlertLength", nav.GetFinalLongTransitionAlertLength(),
            kFinalLongTransitionAlertMetricLength);
  TryLength("GetFinalLongTransitionAlertLowerLength", nav.GetFinalLongTransitionAlertLowerLength(),
            kFinalLongTransitionAlertLowerMetricLength);
  TryLength("GetFinalLongTransitionAlertUpperLength", nav.GetFinalLongTransitionAlertUpperLength(),
            kFinalLongTransitionAlertUpperMetricLength);
  TryLength("GetFinalLongTransitionAlertMinManeuverLength",
            nav.GetFinalLongTransitionAlertMinManeuverLength(),
            kFinalLongTransitionAlertMinManeuverMetricLength);

  // Test the final medium transition alert methods for metric values
  TryLength("GetFinalMediumTransitionAlertLength", nav.GetFinalMediumTransitionAlertLength(),
            kFinalMediumTransitionAlertMetricLength);
  TryLength("GetFinalMediumTransitionAlertLowerLength",
            nav.GetFinalMediumTransitionAlertLowerLength(),
            kFinalMediumTransitionAlertLowerMetricLength);
  TryLength("GetFinalMediumTransitionAlertUpperLength",
            nav.GetFinalMediumTransitionAlertUpperLength(),
            kFinalMediumTransitionAlertUpperMetricLength);
  TryLength("GetFinalMediumTransitionAlertMinManeuverLength",
            nav.GetFinalMediumTransitionAlertMinManeuverLength(),
            kFinalMediumTransitionAlertMinManeuverMetricLength);

  // Test the final short transition alert methods for metric values
  TryLength("GetFinalShortTransitionAlertLength", nav.GetFinalShortTransitionAlertLength(),
            kFinalShortTransitionAlertMetricLength);
  TryLength("GetFinalShortTransitionAlertLowerLength", nav.GetFinalShortTransitionAlertLowerLength(),
            kFinalShortTransitionAlertLowerMetricLength);
  TryLength("GetFinalShortTransitionAlertUpperLength", nav.GetFinalShortTransitionAlertUpperLength(),
            kFinalShortTransitionAlertUpperMetricLength);

  ////////////////////////////////////////////////////////////////////////////
  uint32_t maneuver_index = 0;
  uint32_t instruction_index = 0;
  ////////////////////////////////////////////////////////////////////////////

  //----------------------------------------------------------------
  // trace point = 0 | Pre | Drive north on Middletown Road for 1.3 kilometers.
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7065582f, 40.246048f, 1489614783, 0),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.7065964f, 40.2460442f, leg_index, 15.4219952f,
                                                973, maneuver_index, 1.33632565f, 88,
                                                instruction_index),
                            true);
  //----------------------------------------------------------------
  // trace point = 1
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.706604f, 40.2461967f, 1489614784, 17.0408878),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7066345f,
                                                40.2461929f, leg_index, 15.4051208f, 971,
                                                maneuver_index, 1.31945133f, 86),
                            true);
  //----------------------------------------------------------------
  // trace point = 2
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7066498f, 40.2463493f, 1489614785, 17.3661098),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7066727f,
                                                40.2463455f, leg_index, 15.3878298f, 970,
                                                maneuver_index, 1.30216026f, 85));
  //----------------------------------------------------------------
  // trace point = 3
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7066803f, 40.2464981f, 1489614786, 16.7813263),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7067032f,
                                                40.2464943f, leg_index, 15.368866f, 969,
                                                maneuver_index, 1.28319645f, 84));
  //----------------------------------------------------------------
  // trace point = 4
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7067261f, 40.2466545f, 1489614787, 17.9088612),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.706749f,
                                                40.2466507f, leg_index, 15.3532391f, 968,
                                                maneuver_index, 1.26756954f, 83));
  //----------------------------------------------------------------
  // trace point = 5
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7067642f, 40.2468071f, 1489614788, 17.2258587),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7067871f,
                                                40.2468033f, leg_index, 15.3359489f, 967,
                                                maneuver_index, 1.25027943f, 82));
  //----------------------------------------------------------------
  // trace point = 6
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7068024f, 40.2469673f, 1489614789, 18.1013985),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7068329f,
                                                40.2469635f, leg_index, 15.3176947f, 966,
                                                maneuver_index, 1.23202515f, 81));
  //----------------------------------------------------------------
  // trace point = 7
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7068405f, 40.2471237f, 1489614790, 17.7885265),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.706871f,
                                                40.2471199f, leg_index, 15.2999868f, 965,
                                                maneuver_index, 1.21431732f, 80));
  //----------------------------------------------------------------
  // trace point = 8
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7069016f, 40.2472878f, 1489614791, 18.9778519),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7069244f,
                                                40.2472839f, leg_index, 15.281147f, 964,
                                                maneuver_index, 1.19547749f, 79));
  //----------------------------------------------------------------
  // trace point = 9
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7069626f, 40.2474403f, 1489614792, 17.696558),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7069931f,
                                                40.2474327f, leg_index, 15.2635937f, 963,
                                                maneuver_index, 1.17792416f, 78));
  //----------------------------------------------------------------
  // trace point = 10
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7070389f, 40.2475929f, 1489614793, 18.2149124),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7070694f,
                                                40.2475815f, leg_index, 15.2457237f, 961,
                                                maneuver_index, 1.16005421f, 76));
  //----------------------------------------------------------------
  // trace point = 11
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7071381f, 40.2477493f, 1489614794, 19.295784),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7071762f,
                                                40.2477341f, leg_index, 15.2246599f, 960,
                                                maneuver_index, 1.1389904f, 75));
  //----------------------------------------------------------------
  // trace point = 12
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7072449f, 40.2478943f, 1489614795, 18.5254974),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.707283f,
                                                40.2478752f, leg_index, 15.2083073f, 959,
                                                maneuver_index, 1.12263775f, 74));
  //----------------------------------------------------------------
  // trace point = 13
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7073593f, 40.2480354f, 1489614796, 18.444109),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7073975f,
                                                40.2480164f, leg_index, 15.1898308f, 958,
                                                maneuver_index, 1.10416126f, 73));
  //----------------------------------------------------------------
  // trace point = 14
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7074738f, 40.2481766f, 1489614797, 18.5834579),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7075119f,
                                                40.2481613f, leg_index, 15.17099f, 957,
                                                maneuver_index, 1.08532047f, 72));
  //----------------------------------------------------------------
  // trace point = 15
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7075882f, 40.2483215f, 1489614798, 18.8244743),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7076263f,
                                                40.2483025f, leg_index, 15.1525126f, 955,
                                                maneuver_index, 1.06684303f, 70));
  //----------------------------------------------------------------
  // trace point = 16
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7077026f, 40.2484627f, 1489614799, 18.4888382),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7077408f,
                                                40.2484436f, leg_index, 15.1340361f, 954,
                                                maneuver_index, 1.04836655f, 69));
  //----------------------------------------------------------------
  // trace point = 17
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7078171f, 40.2486038f, 1489614800, 18.444067),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7078552f,
                                                40.2485886f, leg_index, 15.1151962f, 953,
                                                maneuver_index, 1.02952671f, 68));
  //----------------------------------------------------------------
  // trace point = 18
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7079239f, 40.2487373f, 1489614801, 17.3738937),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.707962f,
                                                40.2487221f, leg_index, 15.0977821f, 951,
                                                maneuver_index, 1.01211262f, 66));
  //----------------------------------------------------------------
  // trace point = 19
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7080307f, 40.2488747f, 1489614802, 17.8505764),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7080688f,
                                                40.2488556f, leg_index, 15.080368f, 950,
                                                maneuver_index, 0.994698524f, 65));
  //----------------------------------------------------------------
  // trace point = 20
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7081375f, 40.2490005f, 1489614803, 16.7148075),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7081757f,
                                                40.2489853f, leg_index, 15.0633173f, 949,
                                                maneuver_index, 0.977647781f, 64));
  //----------------------------------------------------------------
  // trace point = 21
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7082367f, 40.2491341f, 1489614804, 17.0280514),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7082748f,
                                                40.2491188f, leg_index, 15.0462275f, 948,
                                                maneuver_index, 0.960557938f, 63));
  //----------------------------------------------------------------
  // trace point = 22
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7083359f, 40.2492828f, 1489614805, 18.5975018),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7083664f,
                                                40.2492714f, leg_index, 15.0275488f, 947,
                                                maneuver_index, 0.941879272f, 62));
  //----------------------------------------------------------------
  // trace point = 23
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7084198f, 40.249424f, 1489614806, 17.2424393),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7084579f,
                                                40.2494087f, leg_index, 15.0104017f, 946,
                                                maneuver_index, 0.924732208f, 61));
  //----------------------------------------------------------------
  // trace point = 24
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085037f, 40.2495689f, 1489614807, 17.6487389),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7085342f,
                                                40.2495613f, leg_index, 14.9921989f, 944,
                                                maneuver_index, 0.906529427f, 59));
  //----------------------------------------------------------------
  // trace point = 25
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085495f, 40.2497215f, 1489614808, 17.4557419),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7085953f,
                                                40.2497139f, leg_index, 14.9744253f, 943,
                                                maneuver_index, 0.888755798f, 58));
  //----------------------------------------------------------------
  // trace point = 26
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086029f, 40.2498741f, 1489614809, 17.5315475),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086334f,
                                                40.2498703f, leg_index, 14.9567223f, 942,
                                                maneuver_index, 0.871052742f, 57));
  //----------------------------------------------------------------
  // trace point = 27
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086411f, 40.2500343f, 1489614810, 18.1013718),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086563f,
                                                40.2500343f, leg_index, 14.9383268f, 941,
                                                maneuver_index, 0.852657318f, 56));
  //----------------------------------------------------------------
  // trace point = 28
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086639f, 40.2501907f, 1489614811, 17.5860672),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086639f,
                                                40.2501907f, leg_index, 14.9209051f, 940,
                                                maneuver_index, 0.835235596f, 55));
  //----------------------------------------------------------------
  // trace point = 29
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086639f, 40.2503624f, 1489614812, 19.0356331),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086639f,
                                                40.2503624f, leg_index, 14.9017448f, 938,
                                                maneuver_index, 0.816075325f, 53));
  //----------------------------------------------------------------
  // trace point = 30
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086639f, 40.2505341f, 1489614813, 19.1469517),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086487f,
                                                40.2505341f, leg_index, 14.8817444f, 937,
                                                maneuver_index, 0.796074867f, 52));
  //----------------------------------------------------------------
  // trace point = 31
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086411f, 40.2507019f, 1489614814, 18.8034897),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086334f,
                                                40.2507019f, leg_index, 14.863863f, 936,
                                                maneuver_index, 0.778193474f, 51));
  //----------------------------------------------------------------
  // trace point = 32
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086105f, 40.2508736f, 1489614815, 19.3158588),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086182f,
                                                40.2508736f, leg_index, 14.8447104f, 935,
                                                maneuver_index, 0.759040833f, 50));
  //----------------------------------------------------------------
  // trace point = 33
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085953f, 40.2510376f, 1489614816, 18.3069363),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086105f,
                                                40.2510376f, leg_index, 14.8264456f, 934,
                                                maneuver_index, 0.740776062f, 49));
  //----------------------------------------------------------------
  // trace point = 34
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085724f, 40.2512207f, 1489614817, 20.4649754),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7085953f,
                                                40.2512207f, leg_index, 14.8060207f, 932,
                                                maneuver_index, 0.720351219f, 47));
  //----------------------------------------------------------------
  // trace point = 35
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085648f, 40.2513924f, 1489614818, 19.0449219),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.70858f,
                                                40.2513924f, leg_index, 14.7868662f, 931,
                                                maneuver_index, 0.701196671f, 46));
  //----------------------------------------------------------------
  // trace point = 36
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085648f, 40.2515717f, 1489614819, 20.037508),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7085724f,
                                                40.2515717f, leg_index, 14.7668982f, 930,
                                                maneuver_index, 0.681228638f, 45));
  //----------------------------------------------------------------
  // trace point = 37
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085648f, 40.2517509f, 1489614820, 19.9261894),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7085724f,
                                                40.2517509f, leg_index, 14.7468977f, 929,
                                                maneuver_index, 0.66122818f, 44));
  //----------------------------------------------------------------
  // trace point = 38
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085724f, 40.2519264f, 1489614821, 19.4899864),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.70858f,
                                                40.2519264f, leg_index, 14.7273626f, 927,
                                                maneuver_index, 0.641693115f, 42));
  //----------------------------------------------------------------
  // trace point = 39
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085876f, 40.2520905f, 1489614822, 18.3069344),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086029f,
                                                40.2520905f, leg_index, 14.7090025f, 927,
                                                maneuver_index, 0.623332977f, 42));
  //----------------------------------------------------------------
  // trace point = 40
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085953f, 40.2522545f, 1489614823, 18.2660809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086182f,
                                                40.2522545f, leg_index, 14.6906986f, 925,
                                                maneuver_index, 0.605029106f, 40));
  //----------------------------------------------------------------
  // trace point = 41
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086182f, 40.2524223f, 1489614824, 18.803484),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086411f,
                                                40.2524223f, leg_index, 14.671916f, 924,
                                                maneuver_index, 0.58624649f, 39));
  //----------------------------------------------------------------
  // trace point = 42
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086334f, 40.2525864f, 1489614825, 18.3008213),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086563f,
                                                40.2525864f, leg_index, 14.6536121f, 923,
                                                maneuver_index, 0.567942619f, 38));
  //----------------------------------------------------------------
  // trace point = 43
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086563f, 40.2527504f, 1489614826, 18.3606739),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086792f,
                                                40.2527504f, leg_index, 14.6352539f, 922,
                                                maneuver_index, 0.549584389f, 37));
  //----------------------------------------------------------------
  // trace point = 44
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086716f, 40.2529182f, 1489614827, 18.7510128),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7086945f,
                                                40.2529182f, leg_index, 14.6165276f, 920,
                                                maneuver_index, 0.53085804f, 35));
  //----------------------------------------------------------------
  // trace point = 45
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7086868f, 40.2530785f, 1489614828, 17.8566513),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7087173f,
                                                40.2530785f, leg_index, 14.598588f, 919,
                                                maneuver_index, 0.512918472f, 34));
  //----------------------------------------------------------------
  // trace point = 46
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7087097f, 40.2532387f, 1489614829, 18.0286465),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7087326f,
                                                40.2532387f, leg_index, 14.5807085f, 918,
                                                maneuver_index, 0.495038986f, 33));
  //----------------------------------------------------------------
  // trace point = 47
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.708725f, 40.2534027f, 1489614830, 18.3008213),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7087555f,
                                                40.2533989f, leg_index, 14.5627699f, 917,
                                                maneuver_index, 0.477100372f, 32));
  //----------------------------------------------------------------
  // trace point = 48
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7087479f, 40.2535706f, 1489614831, 18.8034801),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7087708f,
                                                40.2535706f, leg_index, 14.543622f, 915,
                                                maneuver_index, 0.457952499f, 30));
  //----------------------------------------------------------------
  // trace point = 49 | Alert 0.4 | Turn right.
  maneuver_index = 0;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7087708f, 40.2537308f, 1489614832, 17.9179859),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.7087936f, 40.2537308f, leg_index, 14.5256834f,
                                                914, maneuver_index, 0.440013885f, 29,
                                                instruction_index, 0.4f));
  //----------------------------------------------------------------
  // trace point = 50
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7087936f, 40.2538986f, 1489614833, 18.8034801),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7088089f,
                                                40.2538986f, leg_index, 14.506959f, 913,
                                                maneuver_index, 0.421289444f, 28));
  //----------------------------------------------------------------
  // trace point = 51
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7088089f, 40.2540627f, 1489614834, 18.3008194),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7088318f,
                                                40.2540627f, leg_index, 14.4885969f, 912,
                                                maneuver_index, 0.402927399f, 27));
  //----------------------------------------------------------------
  // trace point = 52
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7088394f, 40.2542343f, 1489614835, 19.2055016),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7088547f,
                                                40.2542343f, leg_index, 14.4693909f, 910,
                                                maneuver_index, 0.383721352f, 25));
  //----------------------------------------------------------------
  // trace point = 53
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7088547f, 40.254406f, 1489614836, 19.1951427),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7088699f,
                                                40.254406f, leg_index, 14.450243f, 909,
                                                maneuver_index, 0.364573479f, 24));
  //----------------------------------------------------------------
  // trace point = 54
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7088852f, 40.2545776f, 1489614837, 19.3158417),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7088928f,
                                                40.2545776f, leg_index, 14.431035f, 908,
                                                maneuver_index, 0.345365524f, 23));
  //----------------------------------------------------------------
  // trace point = 55
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7089081f, 40.2547531f, 1489614838, 19.5786629),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7089157f,
                                                40.2547531f, leg_index, 14.4114056f, 907,
                                                maneuver_index, 0.325736046f, 22));
  //----------------------------------------------------------------
  // trace point = 56
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7089233f, 40.2549286f, 1489614839, 19.6336308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7089386f,
                                                40.2549286f, leg_index, 14.3917885f, 905,
                                                maneuver_index, 0.306118965f, 20));
  //----------------------------------------------------------------
  // trace point = 57
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7089539f, 40.2551079f, 1489614840, 20.0994797),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7089539f,
                                                40.2551079f, leg_index, 14.3717813f, 904,
                                                maneuver_index, 0.286111832f, 19));
  //----------------------------------------------------------------
  // trace point = 58
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7089767f, 40.255291f, 1489614841, 20.4649639),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7089767f,
                                                40.255291f, leg_index, 14.351306f, 903,
                                                maneuver_index, 0.265636444f, 18));
  //----------------------------------------------------------------
  // trace point = 59
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.708992f, 40.2554741f, 1489614842, 20.4112854),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7089996f,
                                                40.2554741f, leg_index, 14.3287153f, 901,
                                                maneuver_index, 0.243045807f, 16));
  //----------------------------------------------------------------
  // trace point = 60
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7090149f, 40.2556572f, 1489614843, 20.464962),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7090225f,
                                                40.2556572f, leg_index, 14.3103485f, 899,
                                                maneuver_index, 0.224678993f, 14));
  //----------------------------------------------------------------
  // trace point = 61
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7090378f, 40.2558327f, 1489614844, 19.6894264),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7090378f,
                                                40.2558327f, leg_index, 14.2907743f, 898,
                                                maneuver_index, 0.205104828f, 13));
  //----------------------------------------------------------------
  // trace point = 62
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7090683f, 40.2560043f, 1489614845, 19.2054939),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7090607f,
                                                40.2560043f, leg_index, 14.2715664f, 897,
                                                maneuver_index, 0.185896873f, 12));
  //----------------------------------------------------------------
  // trace point = 63
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7090912f, 40.256176f, 1489614846, 19.246397),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7090836f,
                                                40.256176f, leg_index, 14.2523594f, 896,
                                                maneuver_index, 0.166689873f, 11));
  //----------------------------------------------------------------
  // trace point = 64
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091141f, 40.2563362f, 1489614847, 17.9179764),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7090988f,
                                                40.2563362f, leg_index, 14.2344828f, 894,
                                                maneuver_index, 0.148813248f, 9));
  //----------------------------------------------------------------
  // trace point = 65
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091293f, 40.2565002f, 1489614848, 18.3008175),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7091217f,
                                                40.2565002f, leg_index, 14.2161198f, 893,
                                                maneuver_index, 0.130450249f, 8));
  //----------------------------------------------------------------
  // trace point = 66
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.709137f, 40.2566528f, 1489614849, 17.0454369),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7091446f,
                                                40.2566528f, leg_index, 14.1990252f, 892,
                                                maneuver_index, 0.113355637f, 7));
  //----------------------------------------------------------------
  // trace point = 67
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091522f, 40.2568054f, 1489614850, 16.9684792),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7091599f,
                                                40.2568054f, leg_index, 14.1819944f, 891,
                                                maneuver_index, 0.0963249207f, 6));
  //----------------------------------------------------------------
  // trace point = 68 | Pre | Turn right.
  maneuver_index = 0;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091599f, 40.2569466f, 1489614851, 15.8219709),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.7091827f, 40.2569466f, leg_index, 14.1661663f,
                                                890, maneuver_index, 0.080496788f, 5,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 69
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091675f, 40.2570724f, 1489614852, 13.9276381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.709198f,
                                                40.2570686f, leg_index, 14.1525154f, 889,
                                                maneuver_index, 0.0668458939f, 4));
  //----------------------------------------------------------------
  // trace point = 70
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091751f, 40.2571907f, 1489614853, 13.2644405),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7092133f,
                                                40.2571907f, leg_index, 14.1388655f, 888,
                                                maneuver_index, 0.0531959534f, 3));
  //----------------------------------------------------------------
  // trace point = 71
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091827f, 40.2572975f, 1489614854, 11.9305592),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7092209f,
                                                40.2572975f, leg_index, 14.1269588f, 887,
                                                maneuver_index, 0.0412893295f, 2));
  //----------------------------------------------------------------
  // trace point = 72
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091827f, 40.2573967f, 1489614855, 11.0206299),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7092209f,
                                                40.2573967f, leg_index, 14.1159325f, 887,
                                                maneuver_index, 0.0302629471f, 2));
  //----------------------------------------------------------------
  // trace point = 73
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7091446f, 40.2575417f, 1489614857, 8.23048973),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7092209f,
                                                40.2575455f, leg_index, 14.0993195f, 886,
                                                maneuver_index, 0.0136499405f, 1));
  //----------------------------------------------------------------
  // trace point = 74
  maneuver_index = 1;
  instruction_index = maneuver_index;
  std::cout << maneuver_index << " " << instruction_index << std::endl;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7090225f, 40.2576218f, 1489614859, 6.83240986),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7090225f,
                                                40.2576714f, leg_index, 14.0701094f, 884,
                                                maneuver_index, 0.80796814f, 44));
  //----------------------------------------------------------------
  // trace point = 75 | Post | Continue for 800 meters.
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7088318f, 40.2576408f, 1489614861, 8.18165302),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.7088318f, 40.257679f, leg_index, 14.0538855f, 883,
                                                maneuver_index, 0.791744232f, 43));
  //----------------------------------------------------------------
  // trace point = 76
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7085876f, 40.2576561f, 1489614863, 10.3978453),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7085876f,
                                                40.2576866f, leg_index, 14.0331278f, 882,
                                                maneuver_index, 0.770986557f, 42));
  //----------------------------------------------------------------
  // trace point = 77
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7084351f, 40.2576599f, 1489614864, 13.0054398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7084351f,
                                                40.2576904f, leg_index, 14.0201578f, 881,
                                                maneuver_index, 0.758016586f, 41));
  //----------------------------------------------------------------
  // trace point = 78
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7082672f, 40.2576675f, 1489614865, 14.2998667),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7082672f,
                                                40.2576942f, leg_index, 14.0058928f, 880,
                                                maneuver_index, 0.743751526f, 40));
  //----------------------------------------------------------------
  // trace point = 79
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7080765f, 40.257679f, 1489614866, 16.2721653),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7080765f,
                                                40.2576981f, leg_index, 13.9896841f, 879,
                                                maneuver_index, 0.727542877f, 39));
  //----------------------------------------------------------------
  // trace point = 80
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7078705f, 40.2576866f, 1489614867, 17.5229645),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7078705f,
                                                40.2577057f, leg_index, 13.9721651f, 878,
                                                maneuver_index, 0.71002388f, 38));
  //----------------------------------------------------------------
  // trace point = 81
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7076492f, 40.2576981f, 1489614868, 18.8145008),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7076492f,
                                                40.2577095f, leg_index, 13.9533691f, 877,
                                                maneuver_index, 0.691227913f, 37));
  //----------------------------------------------------------------
  // trace point = 82
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.707428f, 40.2576981f, 1489614869, 18.7746105),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.707428f,
                                                40.2577209f, leg_index, 13.9345303f, 877,
                                                maneuver_index, 0.67238903f, 37));
  //----------------------------------------------------------------
  // trace point = 83
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7071838f, 40.2577057f, 1489614870, 20.7476482),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7071838f,
                                                40.2577324f, leg_index, 13.9137506f, 875,
                                                maneuver_index, 0.651609421f, 35));
  //----------------------------------------------------------------
  // trace point = 84
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7069397f, 40.2577209f, 1489614871, 20.7956696),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7069397f,
                                                40.2577438f, leg_index, 13.892971f, 874,
                                                maneuver_index, 0.630829811f, 34));
  //----------------------------------------------------------------
  // trace point = 85
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7066879f, 40.2577286f, 1489614872, 21.4266586),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7066879f,
                                                40.2577553f, leg_index, 13.8702507f, 873,
                                                maneuver_index, 0.608109474f, 33));
  //----------------------------------------------------------------
  // trace point = 86
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7064362f, 40.2577362f, 1489614873, 21.4223194),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7064362f,
                                                40.2577591f, leg_index, 13.8501539f, 872,
                                                maneuver_index, 0.588012695f, 32));
  //----------------------------------------------------------------
  // trace point = 87
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7061844f, 40.2577477f, 1489614874, 21.4497757),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7061844f,
                                                40.2577629f, leg_index, 13.8287611f, 870,
                                                maneuver_index, 0.566619873f, 30));
  //----------------------------------------------------------------
  // trace point = 88
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7059326f, 40.2577591f, 1489614875, 21.3583126),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7059326f,
                                                40.2577667f, leg_index, 13.8073683f, 869,
                                                maneuver_index, 0.545227051f, 29));
  //----------------------------------------------------------------
  // trace point = 89
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7056808f, 40.2577667f, 1489614876, 21.4266472),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7056808f,
                                                40.2577705f, leg_index, 13.7859755f, 868,
                                                maneuver_index, 0.523834229f, 28));
  //----------------------------------------------------------------
  // trace point = 90
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7054291f, 40.257782f, 1489614877, 21.4731503),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7054291f,
                                                40.2577782f, leg_index, 13.7645702f, 866,
                                                maneuver_index, 0.502429008f, 26));
  //----------------------------------------------------------------
  // trace point = 91
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7051773f, 40.2577896f, 1489614878, 21.4266396),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7051773f,
                                                40.2577858f, leg_index, 13.7431669f, 865,
                                                maneuver_index, 0.481025696f, 25));
  //----------------------------------------------------------------
  // trace point = 92
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7049255f, 40.2578049f, 1489614879, 21.3884487),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7049255f,
                                                40.2577972f, leg_index, 13.7217312f, 864,
                                                maneuver_index, 0.459589958f, 24));
  //----------------------------------------------------------------
  // trace point = 93 | Alert 0.4 | Stay straight to take the U.S. 3 22 East ramp.
  maneuver_index = 1;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7046814f, 40.2578201f, 1489614880, 20.8803196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.7046814f, 40.2578125f, leg_index, 13.7009211f,
                                                863, maneuver_index, 0.438779831f, 23,
                                                instruction_index, 0.4f));
  //----------------------------------------------------------------
  // trace point = 94
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7044373f, 40.2578354f, 1489614881, 20.7956352),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7044373f,
                                                40.2578278f, leg_index, 13.6801205f, 862,
                                                maneuver_index, 0.41797924f, 22));
  //----------------------------------------------------------------
  // trace point = 95
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7041855f, 40.2578583f, 1489614882, 21.560667),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7041855f,
                                                40.2578583f, leg_index, 13.6584663f, 861,
                                                maneuver_index, 0.396325111f, 21));
  //----------------------------------------------------------------
  // trace point = 96
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7039337f, 40.2578812f, 1489614883, 21.4763107),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7039337f,
                                                40.257885f, leg_index, 13.6368723f, 860,
                                                maneuver_index, 0.374731064f, 20));
  //----------------------------------------------------------------
  // trace point = 97
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7036743f, 40.2579002f, 1489614884, 22.1887493),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7036743f,
                                                40.2579117f, leg_index, 13.6146364f, 859,
                                                maneuver_index, 0.352495193f, 19));
  //----------------------------------------------------------------
  // trace point = 98
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7034302f, 40.2579308f, 1489614885, 21.0137539),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7034302f,
                                                40.2579384f, leg_index, 13.5936842f, 858,
                                                maneuver_index, 0.331542969f, 18));
  //----------------------------------------------------------------
  // trace point = 99
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7031784f, 40.2579613f, 1489614886, 21.6669903),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7031784f,
                                                40.2579689f, leg_index, 13.5720301f, 856,
                                                maneuver_index, 0.30988884f, 16));
  //----------------------------------------------------------------
  // trace point = 100
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7029266f, 40.2579994f, 1489614887, 21.7386551),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7029266f,
                                                40.2579956f, leg_index, 13.550436f, 855,
                                                maneuver_index, 0.288294792f, 15));
  //----------------------------------------------------------------
  // trace point = 101
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7026672f, 40.2580338f, 1489614888, 22.4286766),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7026672f,
                                                40.2580261f, leg_index, 13.5281401f, 854,
                                                maneuver_index, 0.26599884f, 14));
  //----------------------------------------------------------------
  // trace point = 102
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7024155f, 40.2580681f, 1489614889, 21.7400417),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7024155f,
                                                40.2580528f, leg_index, 13.506546f, 853,
                                                maneuver_index, 0.244404793f, 13));
  //----------------------------------------------------------------
  // trace point = 103
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7021637f, 40.2580986f, 1489614890, 21.6005192),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7021637f,
                                                40.2580872f, leg_index, 13.4848185f, 852,
                                                maneuver_index, 0.222677231f, 12));
  //----------------------------------------------------------------
  // trace point = 104
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7019043f, 40.2581291f, 1489614891, 22.3386822),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7019043f,
                                                40.2581253f, leg_index, 13.4623775f, 851,
                                                maneuver_index, 0.20023632f, 11));
  //----------------------------------------------------------------
  // trace point = 105 | Pre | Stay straight to take the U.S. 3 22 East ramp.
  maneuver_index = 1;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7016525f, 40.2581596f, 1489614892, 21.6005001),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.7016525f, 40.2581596f, leg_index, 13.44065f, 850,
                                                maneuver_index, 0.178508759f, 10, instruction_index));
  //----------------------------------------------------------------
  // trace point = 106
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7014084f, 40.2581902f, 1489614893, 21.0795574),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7014084f,
                                                40.258194f, leg_index, 13.4195604f, 849,
                                                maneuver_index, 0.157419205f, 9));
  //----------------------------------------------------------------
  // trace point = 107
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7011642f, 40.2582245f, 1489614894, 21.071085),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7011642f,
                                                40.2582283f, leg_index, 13.3984709f, 847,
                                                maneuver_index, 0.136329651f, 7));
  //----------------------------------------------------------------
  // trace point = 108
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7009277f, 40.2582512f, 1489614895, 20.272789),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7009277f,
                                                40.2582588f, leg_index, 13.3780956f, 846,
                                                maneuver_index, 0.115954399f, 6));
  //----------------------------------------------------------------
  // trace point = 109
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7007141f, 40.2582779f, 1489614896, 18.426569),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7007141f,
                                                40.2582893f, leg_index, 13.3596325f, 845,
                                                maneuver_index, 0.0974912643f, 5));
  //----------------------------------------------------------------
  // trace point = 110
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7005157f, 40.2583084f, 1489614897, 17.1488628),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7005157f,
                                                40.258316f, leg_index, 13.3425207f, 844,
                                                maneuver_index, 0.0803794861f, 4));
  //----------------------------------------------------------------
  // trace point = 111
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7003403f, 40.2583313f, 1489614898, 15.1692247),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7003403f,
                                                40.2583389f, leg_index, 13.3273993f, 844,
                                                maneuver_index, 0.0652580261f, 4));
  //----------------------------------------------------------------
  // trace point = 112
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7001724f, 40.2583504f, 1489614899, 14.4278393),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7001724f,
                                                40.2583656f, leg_index, 13.3128386f, 843,
                                                maneuver_index, 0.0506973267f, 3));
  //----------------------------------------------------------------
  // trace point = 113
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.7000351f, 40.2583694f, 1489614900, 11.829071),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.7000351f,
                                                40.2583847f, leg_index, 13.3009796f, 842,
                                                maneuver_index, 0.0388383865f, 2));
  //----------------------------------------------------------------
  // trace point = 114
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6998291f, 40.2583923f, 1489614902, 8.84322071),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6998367f,
                                                40.2584114f, leg_index, 13.2838688f, 841,
                                                maneuver_index, 0.021727562f, 1));
  //----------------------------------------------------------------
  // trace point = 115
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6996918f, 40.2584229f, 1489614912, 1.21392703),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6996918f,
                                                40.2584305f, leg_index, 13.2713728f, 841,
                                                maneuver_index, 0.00923156738f, 1));
  //----------------------------------------------------------------
  // trace point = 116
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6994934f, 40.2584953f, 1489614915, 6.23640394),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6994858f,
                                                40.2584877f, leg_index, 13.2525082f, 840,
                                                maneuver_index, 0.364323616f, 22));
  //----------------------------------------------------------------
  // trace point = 117
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6992874f, 40.2585869f, 1489614917, 10.1382875),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6992798f,
                                                40.2585754f, leg_index, 13.2324677f, 839,
                                                maneuver_index, 0.344283104f, 21));
  //----------------------------------------------------------------
  // trace point = 118
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6991501f, 40.2586365f, 1489614918, 12.8532352),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6991501f,
                                                40.2586327f, leg_index, 13.2197409f, 838,
                                                maneuver_index, 0.33155632f, 20));
  //----------------------------------------------------------------
  // trace point = 119
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6990128f, 40.2586975f, 1489614919, 13.5309696),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6990128f,
                                                40.2586937f, leg_index, 13.2062416f, 837,
                                                maneuver_index, 0.31805706f, 19));
  //----------------------------------------------------------------
  // trace point = 120
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6988602f, 40.2587662f, 1489614920, 15.0412283),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6988525f,
                                                40.2587585f, leg_index, 13.1908407f, 836,
                                                maneuver_index, 0.302656174f, 18));
  //----------------------------------------------------------------
  // trace point = 121
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6986923f, 40.2588425f, 1489614921, 16.5910397),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6986847f,
                                                40.2588348f, leg_index, 13.1742487f, 835,
                                                maneuver_index, 0.286064148f, 17));
  //----------------------------------------------------------------
  // trace point = 122
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6985168f, 40.2589264f, 1489614922, 17.5627823),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6985016f,
                                                40.2589111f, leg_index, 13.1565275f, 834,
                                                maneuver_index, 0.268342972f, 16));
  //----------------------------------------------------------------
  // trace point = 123
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6983337f, 40.2590065f, 1489614923, 17.971796),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6983185f,
                                                40.2589912f, leg_index, 13.1385975f, 833,
                                                maneuver_index, 0.250412941f, 15));
  //----------------------------------------------------------------
  // trace point = 124
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6981354f, 40.2590981f, 1489614924, 19.7080631),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6981201f,
                                                40.2590828f, leg_index, 13.1189051f, 832,
                                                maneuver_index, 0.23072052f, 14));
  //----------------------------------------------------------------
  // trace point = 125
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6979294f, 40.2591896f, 1489614925, 20.2764416),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6979218f,
                                                40.2591782f, leg_index, 13.0989895f, 831,
                                                maneuver_index, 0.210804939f, 13));
  //----------------------------------------------------------------
  // trace point = 126
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6977158f, 40.259285f, 1489614926, 20.9583321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6977081f,
                                                40.2592773f, leg_index, 13.0777473f, 829,
                                                maneuver_index, 0.189562798f, 11));
  //----------------------------------------------------------------
  // trace point = 127
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6975021f, 40.2593803f, 1489614927, 21.0316963),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6975021f,
                                                40.2593765f, leg_index, 13.0570574f, 828,
                                                maneuver_index, 0.168872833f, 10));
  //----------------------------------------------------------------
  // trace point = 128 | Pre | Merge onto U.S. 3 22 East.
  maneuver_index = 2;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6972733f, 40.2594833f, 1489614928, 22.581295),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6972733f, 40.2594833f, leg_index, 13.0342655f,
                                                827, maneuver_index, 0.146080971f, 9,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 129
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6970444f, 40.2595863f, 1489614929, 22.5812683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6970444f,
                                                40.2595901f, leg_index, 13.0114756f, 825,
                                                maneuver_index, 0.123291016f, 7));
  //----------------------------------------------------------------
  // trace point = 130
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6968079f, 40.2596855f, 1489614930, 22.9315472),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6968231f,
                                                40.2597008f, leg_index, 12.9890089f, 824,
                                                maneuver_index, 0.100824356f, 6));
  //----------------------------------------------------------------
  // trace point = 131
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6965714f, 40.2597961f, 1489614931, 23.5646191),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6965866f,
                                                40.2598114f, leg_index, 12.9654446f, 822,
                                                maneuver_index, 0.0772600174f, 4));
  //----------------------------------------------------------------
  // trace point = 132
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6963348f, 40.2599068f, 1489614932, 23.5502701),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6963501f,
                                                40.2599258f, leg_index, 12.9416552f, 821,
                                                maneuver_index, 0.0534706116f, 3));
  //----------------------------------------------------------------
  // trace point = 133
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6960831f, 40.2600212f, 1489614933, 24.8862438),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.696106f,
                                                40.2600441f, leg_index, 12.9170885f, 820,
                                                maneuver_index, 0.0289039612f, 2));
  //----------------------------------------------------------------
  // trace point = 134
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6958389f, 40.2601395f, 1489614934, 24.5395241),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6958618f,
                                                40.26017f, leg_index, 12.8920584f, 818,
                                                maneuver_index, 0.00387382507f, 0));
  //----------------------------------------------------------------
  // trace point = 135
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6955948f, 40.2602615f, 1489614935, 24.8418827),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6956177f,
                                                40.2602921f, leg_index, 12.8672647f, 817,
                                                maneuver_index, 0.679763794f, 30));
  //----------------------------------------------------------------
  // trace point = 136
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.695343f, 40.2603912f, 1489614936, 25.7776871),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6953659f,
                                                40.2604218f, leg_index, 12.8414574f, 816,
                                                maneuver_index, 0.653956413f, 29));
  //----------------------------------------------------------------
  // trace point = 137 | Post | Continue for 700 meters.
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6950912f, 40.2605171f, 1489614937, 25.5931149),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.6951218f, 40.2605515f, leg_index, 12.8161879f,
                                                815, maneuver_index, 0.628686905f, 28));
  //----------------------------------------------------------------
  // trace point = 138
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6948471f, 40.2606468f, 1489614938, 25.2796478),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6948776f,
                                                40.2606812f, leg_index, 12.7909174f, 814,
                                                maneuver_index, 0.603416443f, 27));
  //----------------------------------------------------------------
  // trace point = 139
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.694603f, 40.2607803f, 1489614939, 25.4722633),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6946335f,
                                                40.2608109f, leg_index, 12.7656469f, 813,
                                                maneuver_index, 0.578145981f, 26));
  //----------------------------------------------------------------
  // trace point = 140
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6943588f, 40.2609215f, 1489614940, 25.999958),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6943817f,
                                                40.2609444f, leg_index, 12.739604f, 812,
                                                maneuver_index, 0.552103043f, 25));
  //----------------------------------------------------------------
  // trace point = 141
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6941223f, 40.2610626f, 1489614941, 25.5969849),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6941452f,
                                                40.2610855f, leg_index, 12.7140999f, 811,
                                                maneuver_index, 0.52659893f, 24));
  //----------------------------------------------------------------
  // trace point = 142
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6938705f, 40.2612038f, 1489614942, 26.4763775),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6938934f,
                                                40.2612305f, leg_index, 12.6873083f, 810,
                                                maneuver_index, 0.499807358f, 23));
  //----------------------------------------------------------------
  // trace point = 143
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6936264f, 40.2613411f, 1489614943, 25.8019619),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6936569f,
                                                40.2613678f, leg_index, 12.6620617f, 809,
                                                maneuver_index, 0.474560738f, 22));
  //----------------------------------------------------------------
  // trace point = 144
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6933899f, 40.2614937f, 1489614944, 26.3058815),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6934128f,
                                                40.2615128f, leg_index, 12.6357851f, 807,
                                                maneuver_index, 0.448284149f, 20));
  //----------------------------------------------------------------
  // trace point = 145
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6931458f, 40.2616463f, 1489614945, 26.756834),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.693161f,
                                                40.2616577f, leg_index, 12.6089926f, 806,
                                                maneuver_index, 0.421491623f, 19));
  //----------------------------------------------------------------
  // trace point = 146
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6929016f, 40.2618141f, 1489614946, 27.9172592),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6929169f,
                                                40.2618256f, leg_index, 12.581068f, 805,
                                                maneuver_index, 0.393567085f, 18));
  //----------------------------------------------------------------
  // trace point = 147
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6926651f, 40.2619705f, 1489614947, 26.6603279),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.692688f,
                                                40.2619896f, leg_index, 12.5543966f, 804,
                                                maneuver_index, 0.366895676f, 17));
  //----------------------------------------------------------------
  // trace point = 148
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.692421f, 40.2621346f, 1489614948, 27.6208725),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6924438f,
                                                40.2621574f, leg_index, 12.5264835f, 803,
                                                maneuver_index, 0.338982582f, 16));
  //----------------------------------------------------------------
  // trace point = 149
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6921844f, 40.2622948f, 1489614949, 26.8803654),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.692215f,
                                                40.2623215f, leg_index, 12.4998112f, 801,
                                                maneuver_index, 0.312310219f, 14));
  //----------------------------------------------------------------
  // trace point = 150
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6919556f, 40.2624664f, 1489614950, 27.1564426),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6919785f,
                                                40.2624855f, leg_index, 12.4726629f, 800,
                                                maneuver_index, 0.285161972f, 13));
  //----------------------------------------------------------------
  // trace point = 151
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6917267f, 40.2626266f, 1489614951, 26.4504585),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6917496f,
                                                40.2626457f, leg_index, 12.4462795f, 799,
                                                maneuver_index, 0.258778572f, 12));
  //----------------------------------------------------------------
  // trace point = 152 | Pre | Take the U.S. 3 22 East exit on the right toward Ephrata.
  maneuver_index = 3;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6915054f, 40.2627869f, 1489614952, 25.9396687),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6915283f, 40.2628059f, leg_index, 12.4203701f,
                                                798, maneuver_index, 0.232869148f, 11,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 153
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6912842f, 40.2629395f, 1489614953, 25.2732582),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6913071f,
                                                40.2629585f, leg_index, 12.3950396f, 797,
                                                maneuver_index, 0.207538605f, 10));
  //----------------------------------------------------------------
  // trace point = 154
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6910553f, 40.263092f, 1489614954, 25.8552265),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6910858f,
                                                40.2631187f, leg_index, 12.3691311f, 796,
                                                maneuver_index, 0.181630135f, 9));
  //----------------------------------------------------------------
  // trace point = 155
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6908417f, 40.2632484f, 1489614955, 25.0788345),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6908722f,
                                                40.2632713f, leg_index, 12.3442755f, 794,
                                                maneuver_index, 0.156774521f, 7));
  //----------------------------------------------------------------
  // trace point = 156
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6906281f, 40.263401f, 1489614956, 24.9106445),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6906509f,
                                                40.2634239f, leg_index, 12.318944f, 793,
                                                maneuver_index, 0.131443024f, 6));
  //----------------------------------------------------------------
  // trace point = 157
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6904144f, 40.2635498f, 1489614957, 24.6083145),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6904373f,
                                                40.2635727f, leg_index, 12.2943764f, 792,
                                                maneuver_index, 0.10687542f, 5));
  //----------------------------------------------------------------
  // trace point = 158
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6902084f, 40.2637024f, 1489614958, 24.3415852),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6902313f,
                                                40.2637215f, leg_index, 12.2702856f, 791,
                                                maneuver_index, 0.0827846527f, 4));
  //----------------------------------------------------------------
  // trace point = 159
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6900024f, 40.2638397f, 1489614959, 23.2851849),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.690033f,
                                                40.2638626f, leg_index, 12.2472467f, 790,
                                                maneuver_index, 0.0597457886f, 3));
  //----------------------------------------------------------------
  // trace point = 160
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6897964f, 40.2639732f, 1489614960, 22.9217682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6898346f,
                                                40.2639999f, leg_index, 12.2244959f, 789,
                                                maneuver_index, 0.0369949341f, 2));
  //----------------------------------------------------------------
  // trace point = 161
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6893997f, 40.2642326f, 1489614962, 22.1879997),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6894302f,
                                                40.264267f, leg_index, 12.1789808f, 787,
                                                maneuver_index, 0.561860085f, 36));
  //----------------------------------------------------------------
  // trace point = 162
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6892166f, 40.2643509f, 1489614963, 20.3516579),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6892319f,
                                                40.26437f, leg_index, 12.1585999f, 785,
                                                maneuver_index, 0.541479111f, 34));
  //----------------------------------------------------------------
  // trace point = 163
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6890259f, 40.2644615f, 1489614964, 20.3939629),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6890411f,
                                                40.264473f, leg_index, 12.1387529f, 784,
                                                maneuver_index, 0.521632195f, 33));
  //----------------------------------------------------------------
  // trace point = 164
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6888275f, 40.2645607f, 1489614965, 20.1080322),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6888351f,
                                                40.2645721f, leg_index, 12.1180639f, 783,
                                                maneuver_index, 0.500943184f, 32));
  //----------------------------------------------------------------
  // trace point = 165
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6886215f, 40.2646523f, 1489614966, 20.2192211),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6886292f,
                                                40.2646599f, leg_index, 12.0979815f, 782,
                                                maneuver_index, 0.48086071f, 31));
  //----------------------------------------------------------------
  // trace point = 166
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6884079f, 40.2647285f, 1489614967, 20.0976162),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6884079f,
                                                40.2647324f, leg_index, 12.0775261f, 780,
                                                maneuver_index, 0.46040535f, 29));
  //----------------------------------------------------------------
  // trace point = 167
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6881866f, 40.2647934f, 1489614968, 20.0790977),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6881866f,
                                                40.264801f, leg_index, 12.0572147f, 779,
                                                maneuver_index, 0.440093994f, 28));
  //----------------------------------------------------------------
  // trace point = 168
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6879654f, 40.2648468f, 1489614969, 19.7925091),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6879654f,
                                                40.2648506f, leg_index, 12.0376272f, 777,
                                                maneuver_index, 0.420506477f, 26));
  //----------------------------------------------------------------
  // trace point = 169
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6877441f, 40.2648964f, 1489614970, 19.5490284),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6877441f,
                                                40.264904f, leg_index, 12.0179176f, 776,
                                                maneuver_index, 0.40079689f, 25));
  //----------------------------------------------------------------
  // trace point = 170
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6875305f, 40.2649345f, 1489614971, 18.606636),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6875381f,
                                                40.2649498f, leg_index, 11.9996929f, 775,
                                                maneuver_index, 0.382572174f, 24));
  //----------------------------------------------------------------
  // trace point = 171
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6873245f, 40.2649841f, 1489614972, 18.3288994),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6873322f,
                                                40.2649994f, leg_index, 11.9813452f, 774,
                                                maneuver_index, 0.364224434f, 23));
  //----------------------------------------------------------------
  // trace point = 172
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6871109f, 40.2650414f, 1489614973, 19.253582),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6871185f,
                                                40.2650528f, leg_index, 11.9622259f, 773,
                                                maneuver_index, 0.345105171f, 22));
  //----------------------------------------------------------------
  // trace point = 173
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6869125f, 40.26511f, 1489614974, 18.4897919),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6869202f,
                                                40.2651176f, leg_index, 11.9438992f, 771,
                                                maneuver_index, 0.326778412f, 20));
  //----------------------------------------------------------------
  // trace point = 174
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6867142f, 40.2651901f, 1489614975, 19.1061993),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6867218f,
                                                40.2651978f, leg_index, 11.9247808f, 770,
                                                maneuver_index, 0.307660103f, 19));
  //----------------------------------------------------------------
  // trace point = 175
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6865234f, 40.2652779f, 1489614976, 18.952282),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6865311f,
                                                40.2652817f, leg_index, 11.9060783f, 769,
                                                maneuver_index, 0.288957596f, 18));
  //----------------------------------------------------------------
  // trace point = 176
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6863174f, 40.2653809f, 1489614977, 20.9203014),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6863251f,
                                                40.2653885f, leg_index, 11.884923f, 768,
                                                maneuver_index, 0.267802238f, 17));
  //----------------------------------------------------------------
  // trace point = 177
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6861343f, 40.2654839f, 1489614978, 19.3158531),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6861343f,
                                                40.2654877f, leg_index, 11.8653164f, 766,
                                                maneuver_index, 0.248195648f, 15));
  //----------------------------------------------------------------
  // trace point = 178
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6859436f, 40.2655907f, 1489614979, 20.0586643),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6859436f,
                                                40.2655907f, leg_index, 11.8454685f, 765,
                                                maneuver_index, 0.228347778f, 14));
  //----------------------------------------------------------------
  // trace point = 179
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6855469f, 40.2658081f, 1489614981, 20.7408104),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6855469f,
                                                40.2658081f, leg_index, 11.803978f, 762,
                                                maneuver_index, 0.186857224f, 11));
  //----------------------------------------------------------------
  // trace point = 180 | Pre | Continue on U.S. 3 22 for 4.4 kilometers.
  maneuver_index = 4;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6851501f, 40.2660179f, 1489614983, 20.5163231),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6851501f, 40.2660179f, leg_index, 11.7629738f,
                                                760, maneuver_index, 0.145853043f, 9,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 181
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6847534f, 40.2662315f, 1489614985, 20.6439323),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6847534f,
                                                40.2662354f, leg_index, 11.7214708f, 758,
                                                maneuver_index, 0.10435009f, 7));
  //----------------------------------------------------------------
  // trace point = 182
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6845703f, 40.2663269f, 1489614986, 18.800724),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6845856f,
                                                40.2663383f, leg_index, 11.7031755f, 757,
                                                maneuver_index, 0.0860548019f, 6));
  //----------------------------------------------------------------
  // trace point = 183
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6843872f, 40.2664261f, 1489614987, 19.0547295),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6844101f,
                                                40.2664452f, leg_index, 11.6841087f, 755,
                                                maneuver_index, 0.0669879913f, 4));
  //----------------------------------------------------------------
  // trace point = 184
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6841965f, 40.2665253f, 1489614988, 19.612999),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.684227f,
                                                40.2665558f, leg_index, 11.6642694f, 754,
                                                maneuver_index, 0.0471487045f, 3));
  //----------------------------------------------------------------
  // trace point = 185
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6839981f, 40.2666245f, 1489614989, 20.1076031),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6840439f,
                                                40.2666664f, leg_index, 11.6444311f, 753,
                                                maneuver_index, 0.0273103714f, 2));
  //----------------------------------------------------------------
  // trace point = 186
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.683609f, 40.266819f, 1489614991, 19.7674026),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6836624f,
                                                40.26688f, leg_index, 11.6041765f, 750,
                                                maneuver_index, 4.40131712f, 250));
  //----------------------------------------------------------------
  // trace point = 187
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6832275f, 40.2669983f, 1489614993, 19.0026054),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6832809f,
                                                40.2670631f, leg_index, 11.5658998f, 748,
                                                maneuver_index, 4.36304045f, 248));
  //----------------------------------------------------------------
  // trace point = 188
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.682869f, 40.2671738f, 1489614995, 18.1227264),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6829224f,
                                                40.2672386f, leg_index, 11.5297155f, 746,
                                                maneuver_index, 4.32685614f, 246));
  //----------------------------------------------------------------
  // trace point = 189
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6826859f, 40.2672539f, 1489614996, 17.9145603),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6827393f,
                                                40.2673225f, leg_index, 11.5115738f, 745,
                                                maneuver_index, 4.30871439f, 245));
  //----------------------------------------------------------------
  // trace point = 190
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6825333f, 40.2673225f, 1489614997, 15.0961428),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.682579f,
                                                40.2673874f, leg_index, 11.4961576f, 744,
                                                maneuver_index, 4.29329824f, 244));
  //----------------------------------------------------------------
  // trace point = 191
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6824036f, 40.2673683f, 1489614998, 12.0481482),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6824417f,
                                                40.267437f, leg_index, 11.4832401f, 744,
                                                maneuver_index, 4.28038073f, 244));
  //----------------------------------------------------------------
  // trace point = 192
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6821823f, 40.2674332f, 1489615001, 6.7324872),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6822052f,
                                                40.2675056f, leg_index, 11.4616585f, 743,
                                                maneuver_index, 4.25879908f, 243));
  //----------------------------------------------------------------
  // trace point = 193
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6820297f, 40.2674561f, 1489615027, 0.506246984),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6820526f,
                                                40.2675362f, leg_index, 11.4482594f, 742,
                                                maneuver_index, 4.24539995f, 242));
  //----------------------------------------------------------------
  // trace point = 194
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6817474f, 40.2675095f, 1489615030, 8.25055599),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6817627f,
                                                40.2675781f, leg_index, 11.4231739f, 741,
                                                maneuver_index, 4.2203145f, 241));
  //----------------------------------------------------------------
  // trace point = 195
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6815033f, 40.26754f, 1489615032, 10.5054255),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6815186f,
                                                40.2676048f, leg_index, 11.4022255f, 740,
                                                maneuver_index, 4.19936609f, 240));
  //----------------------------------------------------------------
  // trace point = 196
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6813507f, 40.2675552f, 1489615033, 13.0185041),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.681366f,
                                                40.2676239f, leg_index, 11.3890924f, 739,
                                                maneuver_index, 4.18623304f, 239));
  //----------------------------------------------------------------
  // trace point = 197
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6811981f, 40.2675743f, 1489615034, 13.1668987),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6812134f,
                                                40.267643f, leg_index, 11.3759604f, 738,
                                                maneuver_index, 4.17310095f, 238));
  //----------------------------------------------------------------
  // trace point = 198
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6810303f, 40.2675934f, 1489615035, 14.42591),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6810455f,
                                                40.267662f, leg_index, 11.3615465f, 738,
                                                maneuver_index, 4.15868711f, 238));
  //----------------------------------------------------------------
  // trace point = 199
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6808548f, 40.2676201f, 1489615036, 15.1654205),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6808701f,
                                                40.2676811f, leg_index, 11.3464909f, 737,
                                                maneuver_index, 4.14363146f, 237));
  //----------------------------------------------------------------
  // trace point = 200
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6806793f, 40.2676353f, 1489615037, 15.0424957),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6806946f,
                                                40.2677002f, leg_index, 11.3314362f, 736,
                                                maneuver_index, 4.12857676f, 236));
  //----------------------------------------------------------------
  // trace point = 201
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6804962f, 40.2676544f, 1489615038, 15.6873484),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6805115f,
                                                40.2677231f, leg_index, 11.3156776f, 735,
                                                maneuver_index, 4.11281824f, 235));
  //----------------------------------------------------------------
  // trace point = 202
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6803131f, 40.2676659f, 1489615039, 15.6014013),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6803284f,
                                                40.2677422f, leg_index, 11.2999811f, 735,
                                                maneuver_index, 4.09712172f, 235));
  //----------------------------------------------------------------
  // trace point = 203
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6801147f, 40.2676849f, 1489615040, 16.9506855),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.68013f,
                                                40.267765f, leg_index, 11.2829409f, 734,
                                                maneuver_index, 4.08008146f, 234));
  //----------------------------------------------------------------
  // trace point = 204
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6799011f, 40.267704f, 1489615041, 18.2998924),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6799164f,
                                                40.2677917f, leg_index, 11.264555f, 733,
                                                maneuver_index, 4.06169558f, 233));
  //----------------------------------------------------------------
  // trace point = 205
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6796951f, 40.2677307f, 1489615042, 17.7539883),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6797104f,
                                                40.2678146f, leg_index, 11.2468729f, 732,
                                                maneuver_index, 4.0440135f, 232));
  //----------------------------------------------------------------
  // trace point = 206
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6794815f, 40.2677536f, 1489615043, 18.2725658),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6794891f,
                                                40.2678375f, leg_index, 11.2279043f, 730,
                                                maneuver_index, 4.02504492f, 230));
  //----------------------------------------------------------------
  // trace point = 207
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6792679f, 40.2677727f, 1489615044, 18.2998734),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6792755f,
                                                40.2678528f, leg_index, 11.2096806f, 729,
                                                maneuver_index, 4.00682116f, 229));
  //----------------------------------------------------------------
  // trace point = 208
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6790466f, 40.2677956f, 1489615045, 18.9456139),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6790543f,
                                                40.267868f, leg_index, 11.1908102f, 728,
                                                maneuver_index, 3.9879508f, 228));
  //----------------------------------------------------------------
  // trace point = 209
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6788177f, 40.2678223f, 1489615046, 19.6654797),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6788254f,
                                                40.2678833f, leg_index, 11.1712961f, 727,
                                                maneuver_index, 3.96843672f, 227));
  //----------------------------------------------------------------
  // trace point = 210
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6785965f, 40.2678413f, 1489615047, 18.9749851),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6786041f,
                                                40.2679024f, leg_index, 11.1523857f, 726,
                                                maneuver_index, 3.94952631f, 226));
  //----------------------------------------------------------------
  // trace point = 211
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6783752f, 40.2678642f, 1489615048, 18.9455948),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6783829f,
                                                40.2679176f, leg_index, 11.1335163f, 725,
                                                maneuver_index, 3.93065691f, 225));
  //----------------------------------------------------------------
  // trace point = 212
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.678154f, 40.2678909f, 1489615049, 19.0108833),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6781616f,
                                                40.2679329f, leg_index, 11.1146469f, 724,
                                                maneuver_index, 3.91178751f, 224));
  //----------------------------------------------------------------
  // trace point = 213
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6779327f, 40.2679138f, 1489615050, 18.9455814),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6779404f,
                                                40.2679482f, leg_index, 11.0957756f, 723,
                                                maneuver_index, 3.8929162f, 223));
  //----------------------------------------------------------------
  // trace point = 214
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6777267f, 40.2679176f, 1489615051, 17.503336),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6777344f,
                                                40.2679634f, leg_index, 11.0781965f, 722,
                                                maneuver_index, 3.87533712f, 222));
  //----------------------------------------------------------------
  // trace point = 215
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6775055f, 40.2679367f, 1489615052, 18.9749584),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6775131f,
                                                40.2679787f, leg_index, 11.0593271f, 721,
                                                maneuver_index, 3.85646772f, 221));
  //----------------------------------------------------------------
  // trace point = 216
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6772842f, 40.2679558f, 1489615053, 18.8905449),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6772919f,
                                                40.2679939f, leg_index, 11.0404558f, 720,
                                                maneuver_index, 3.83759642f, 220));
  //----------------------------------------------------------------
  // trace point = 217
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.677063f, 40.2679634f, 1489615054, 18.7879295),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6770706f,
                                                40.2680092f, leg_index, 11.0215864f, 719,
                                                maneuver_index, 3.81872702f, 219));
  //----------------------------------------------------------------
  // trace point = 218
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6768417f, 40.2679787f, 1489615055, 18.8560677),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6768494f,
                                                40.2680244f, leg_index, 11.002717f, 717,
                                                maneuver_index, 3.79985762f, 217));
  //----------------------------------------------------------------
  // trace point = 219
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6766052f, 40.2679939f, 1489615056, 20.1999302),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6766129f,
                                                40.2680435f, leg_index, 10.9825172f, 717,
                                                maneuver_index, 3.77965784f, 217));
  //----------------------------------------------------------------
  // trace point = 220
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6763763f, 40.2680092f, 1489615057, 19.52281),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.676384f,
                                                40.2680588f, leg_index, 10.9630022f, 715,
                                                maneuver_index, 3.7601428f, 215));
  //----------------------------------------------------------------
  // trace point = 221
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6761551f, 40.2680206f, 1489615058, 18.8192177),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6761627f,
                                                40.268074f, leg_index, 10.9441319f, 714,
                                                maneuver_index, 3.74127245f, 214));
  //----------------------------------------------------------------
  // trace point = 222
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6759338f, 40.2680321f, 1489615059, 18.8116398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6759415f,
                                                40.2680893f, leg_index, 10.9252625f, 713,
                                                maneuver_index, 3.72240305f, 213));
  //----------------------------------------------------------------
  // trace point = 223
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6754837f, 40.2680664f, 1489615061, 19.2049637),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6754913f,
                                                40.2681198f, leg_index, 10.8868771f, 711,
                                                maneuver_index, 3.68401766f, 211));
  //----------------------------------------------------------------
  // trace point = 224
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6752701f, 40.2680855f, 1489615062, 18.2997894),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6752777f,
                                                40.2681351f, leg_index, 10.86866f, 710,
                                                maneuver_index, 3.66580057f, 210));
  //----------------------------------------------------------------
  // trace point = 225
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6750412f, 40.2681046f, 1489615063, 19.5782452),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6750488f,
                                                40.268158f, leg_index, 10.8490353f, 709,
                                                maneuver_index, 3.64617586f, 209));
  //----------------------------------------------------------------
  // trace point = 226
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6748199f, 40.2681198f, 1489615064, 18.8458405),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6748276f,
                                                40.2681808f, leg_index, 10.8300695f, 708,
                                                maneuver_index, 3.62721014f, 208));
  //----------------------------------------------------------------
  // trace point = 227
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6745987f, 40.2681465f, 1489615065, 19.0108128),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6746063f,
                                                40.2682037f, leg_index, 10.8111048f, 707,
                                                maneuver_index, 3.60824537f, 207));
  //----------------------------------------------------------------
  // trace point = 228
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6743851f, 40.2681732f, 1489615066, 18.4061127),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6743927f,
                                                40.2682266f, leg_index, 10.7927818f, 706,
                                                maneuver_index, 3.58992243f, 206));
  //----------------------------------------------------------------
  // trace point = 229
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6741638f, 40.2682037f, 1489615067, 19.0862617),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6741714f,
                                                40.2682495f, leg_index, 10.7738171f, 704,
                                                maneuver_index, 3.57095766f, 204));
  //----------------------------------------------------------------
  // trace point = 230
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6739578f, 40.2682266f, 1489615068, 17.6839256),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6739578f,
                                                40.2682724f, leg_index, 10.7554932f, 703,
                                                maneuver_index, 3.55263376f, 203));
  //----------------------------------------------------------------
  // trace point = 231
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6737518f, 40.2682648f, 1489615069, 18.0016556),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6737595f,
                                                40.2682991f, leg_index, 10.7383833f, 702,
                                                maneuver_index, 3.53552389f, 202));
  //----------------------------------------------------------------
  // trace point = 232
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6735382f, 40.2682915f, 1489615070, 18.4060822),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6735458f,
                                                40.2683258f, leg_index, 10.7199984f, 701,
                                                maneuver_index, 3.51713896f, 201));
  //----------------------------------------------------------------
  // trace point = 233
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6733322f, 40.2683182f, 1489615071, 17.7538395),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6733398f,
                                                40.2683563f, leg_index, 10.702178f, 700,
                                                maneuver_index, 3.4993186f, 200));
  //----------------------------------------------------------------
  // trace point = 234
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6731186f, 40.2683449f, 1489615072, 18.3401031),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6731262f,
                                                40.268383f, leg_index, 10.6837931f, 699,
                                                maneuver_index, 3.48093367f, 199));
  //----------------------------------------------------------------
  // trace point = 235
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6728973f, 40.268383f, 1489615073, 19.3252449),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.672905f,
                                                40.2684174f, leg_index, 10.6646147f, 698,
                                                maneuver_index, 3.46175528f, 198));
  //----------------------------------------------------------------
  // trace point = 236
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6726837f, 40.2684135f, 1489615074, 18.4182968),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6726913f,
                                                40.2684517f, leg_index, 10.6460724f, 697,
                                                maneuver_index, 3.44321299f, 197));
  //----------------------------------------------------------------
  // trace point = 237
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6724625f, 40.2684441f, 1489615075, 19.1500168),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6724701f,
                                                40.2684822f, leg_index, 10.626956f, 695,
                                                maneuver_index, 3.42409658f, 195));
  //----------------------------------------------------------------
  // trace point = 238
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6722412f, 40.2684784f, 1489615076, 19.1493874),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6722488f,
                                                40.2685165f, leg_index, 10.6077776f, 694,
                                                maneuver_index, 3.40491819f, 194));
  //----------------------------------------------------------------
  // trace point = 239
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.67202f, 40.2685051f, 1489615077, 19.0107155),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6720276f,
                                                40.2685471f, leg_index, 10.5886803f, 693,
                                                maneuver_index, 3.38582087f, 193));
  //----------------------------------------------------------------
  // trace point = 240
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6717911f, 40.2685318f, 1489615078, 19.6819706),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6718063f,
                                                40.2685814f, leg_index, 10.5695047f, 692,
                                                maneuver_index, 3.36664534f, 192));
  //----------------------------------------------------------------
  // trace point = 241
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6715546f, 40.2685661f, 1489615079, 20.3998737),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6715698f,
                                                40.2686157f, leg_index, 10.5490551f, 691,
                                                maneuver_index, 3.3461957f, 191));
  //----------------------------------------------------------------
  // trace point = 242
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6713181f, 40.2686005f, 1489615080, 20.4833336),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6713257f,
                                                40.2686462f, leg_index, 10.5280457f, 689,
                                                maneuver_index, 3.32518625f, 189));
  //----------------------------------------------------------------
  // trace point = 243
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6710892f, 40.2686386f, 1489615081, 19.9297085),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6710968f,
                                                40.2686806f, leg_index, 10.508234f, 688,
                                                maneuver_index, 3.30537462f, 188));
  //----------------------------------------------------------------
  // trace point = 244
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6708679f, 40.2686729f, 1489615082, 19.149334),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6708755f,
                                                40.2687111f, leg_index, 10.4891367f, 687,
                                                maneuver_index, 3.28627729f, 187));
  //----------------------------------------------------------------
  // trace point = 245
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6706467f, 40.2687073f, 1489615083, 19.1493244),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6706543f,
                                                40.2687416f, leg_index, 10.4700394f, 686,
                                                maneuver_index, 3.26717997f, 186));
  //----------------------------------------------------------------
  // trace point = 246
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.670433f, 40.2687378f, 1489615084, 18.5016537),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6704407f,
                                                40.2687721f, leg_index, 10.4515791f, 685,
                                                maneuver_index, 3.24871969f, 185));
  //----------------------------------------------------------------
  // trace point = 247
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6702194f, 40.2687645f, 1489615085, 18.4059563),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6702271f,
                                                40.2687988f, leg_index, 10.4331942f, 684,
                                                maneuver_index, 3.23033476f, 184));
  //----------------------------------------------------------------
  // trace point = 248
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6699982f, 40.2688026f, 1489615086, 19.2670383),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6700058f,
                                                40.2688293f, leg_index, 10.4140968f, 683,
                                                maneuver_index, 3.21123743f, 183));
  //----------------------------------------------------------------
  // trace point = 249
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6697769f, 40.2688408f, 1489615087, 19.2422485),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6697845f,
                                                40.2688599f, leg_index, 10.3949986f, 682,
                                                maneuver_index, 3.19213915f, 182));
  //----------------------------------------------------------------
  // trace point = 250
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6695557f, 40.2688789f, 1489615088, 19.242239),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6695633f,
                                                40.2688942f, leg_index, 10.3758192f, 680,
                                                maneuver_index, 3.1729598f, 180));
  //----------------------------------------------------------------
  // trace point = 251
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6693268f, 40.2689209f, 1489615089, 20.0050354),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6693268f,
                                                40.2689323f, leg_index, 10.3552866f, 679,
                                                maneuver_index, 3.1524272f, 179));
  //----------------------------------------------------------------
  // trace point = 252
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6690903f, 40.2689667f, 1489615090, 20.7715912),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6690903f,
                                                40.2689705f, leg_index, 10.334754f, 678,
                                                maneuver_index, 3.13189459f, 178));
  //----------------------------------------------------------------
  // trace point = 253
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6688538f, 40.2690086f, 1489615091, 20.5835857),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6688538f,
                                                40.2690163f, leg_index, 10.314024f, 677,
                                                maneuver_index, 3.11116457f, 177));
  //----------------------------------------------------------------
  // trace point = 254
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6686249f, 40.2690544f, 1489615092, 20.0856743),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6686325f,
                                                40.2690659f, leg_index, 10.2944374f, 676,
                                                maneuver_index, 3.09157801f, 176));
  //----------------------------------------------------------------
  // trace point = 255
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6684113f, 40.2690964f, 1489615093, 18.7685528),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6684113f,
                                                40.269104f, leg_index, 10.2751007f, 674,
                                                maneuver_index, 3.07224131f, 174));
  //----------------------------------------------------------------
  // trace point = 256
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.66819f, 40.2691422f, 1489615094, 19.4573517),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.66819f,
                                                40.269146f, leg_index, 10.2556915f, 673,
                                                maneuver_index, 3.05283213f, 173));
  //----------------------------------------------------------------
  // trace point = 257
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.667984f, 40.2691879f, 1489615095, 18.2312584),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.667984f,
                                                40.2691917f, leg_index, 10.2374687f, 672,
                                                maneuver_index, 3.03460932f, 172));
  //----------------------------------------------------------------
  // trace point = 258
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.667778f, 40.2692413f, 1489615096, 18.4652634),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.667778f,
                                                40.2692413f, leg_index, 10.2191181f, 671,
                                                maneuver_index, 3.01625872f, 171));
  //----------------------------------------------------------------
  // trace point = 259
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6675797f, 40.2692947f, 1489615097, 17.8598442),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6675797f,
                                                40.2692947f, leg_index, 10.2006388f, 670,
                                                maneuver_index, 2.99777937f, 170));
  //----------------------------------------------------------------
  // trace point = 260
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6673737f, 40.2693481f, 1489615098, 18.4652386),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6673737f,
                                                40.269352f, leg_index, 10.1826153f, 669,
                                                maneuver_index, 2.97975588f, 169));
  //----------------------------------------------------------------
  // trace point = 261
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6671753f, 40.2694016f, 1489615099, 17.9398251),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6671753f,
                                                40.2694054f, leg_index, 10.1647482f, 668,
                                                maneuver_index, 2.96188879f, 168));
  //----------------------------------------------------------------
  // trace point = 262
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6669769f, 40.2694626f, 1489615100, 18.1369305),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6669769f,
                                                40.2694588f, leg_index, 10.1468821f, 667,
                                                maneuver_index, 2.94402266f, 167));
  //----------------------------------------------------------------
  // trace point = 263
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6667786f, 40.2695236f, 1489615101, 18.1369171),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6667709f,
                                                40.269516f, leg_index, 10.1282244f, 666,
                                                maneuver_index, 2.92536497f, 166));
  //----------------------------------------------------------------
  // trace point = 264
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6665802f, 40.2695923f, 1489615102, 18.5202961),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6665802f,
                                                40.2695923f, leg_index, 10.1099319f, 665,
                                                maneuver_index, 2.90707254f, 165));
  //----------------------------------------------------------------
  // trace point = 265
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6663895f, 40.2696571f, 1489615103, 17.763649),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6663971f,
                                                40.2696686f, leg_index, 10.0922136f, 664,
                                                maneuver_index, 2.88935423f, 164));
  //----------------------------------------------------------------
  // trace point = 266
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.666214f, 40.269722f, 1489615104, 16.5317535),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6662216f,
                                                40.2697372f, leg_index, 10.0754642f, 663,
                                                maneuver_index, 2.87260485f, 163));
  //----------------------------------------------------------------
  // trace point = 267
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6660309f, 40.2697945f, 1489615105, 17.4884129),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6660461f,
                                                40.2698097f, leg_index, 10.0585155f, 661,
                                                maneuver_index, 2.85565615f, 161));
  //----------------------------------------------------------------
  // trace point = 268
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6658554f, 40.2698631f, 1489615106, 16.8069153),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6658707f,
                                                40.2698822f, leg_index, 10.0415678f, 661,
                                                maneuver_index, 2.8387084f, 161));
  //----------------------------------------------------------------
  // trace point = 269
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6656876f, 40.2699242f, 1489615107, 15.7262001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6657028f,
                                                40.2699509f, leg_index, 10.0253916f, 660,
                                                maneuver_index, 2.82253218f, 160));
  //----------------------------------------------------------------
  // trace point = 270
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6655045f, 40.2699928f, 1489615108, 17.4140453),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6655273f,
                                                40.2700233f, leg_index, 10.0084438f, 659,
                                                maneuver_index, 2.80558443f, 159));
  //----------------------------------------------------------------
  // trace point = 271
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6653442f, 40.2700577f, 1489615109, 15.3962479),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6653671f,
                                                40.270092f, leg_index, 9.992836f, 658, maneuver_index,
                                                2.7899766f, 158));
  //----------------------------------------------------------------
  // trace point = 272
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6651764f, 40.2701187f, 1489615110, 15.8028164),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6652069f,
                                                40.2701607f, leg_index, 9.97723007f, 657,
                                                maneuver_index, 2.77437067f, 157));
  //----------------------------------------------------------------
  // trace point = 273
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6650238f, 40.2701912f, 1489615111, 15.1960735),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6650467f,
                                                40.2702255f, leg_index, 9.96182537f, 656,
                                                maneuver_index, 2.75896597f, 156));
  //----------------------------------------------------------------
  // trace point = 274
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6648483f, 40.2702599f, 1489615112, 16.8068371),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6648712f,
                                                40.270298f, leg_index, 9.94410515f, 655,
                                                maneuver_index, 2.74124575f, 155));
  //----------------------------------------------------------------
  // trace point = 275
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6646881f, 40.2703285f, 1489615113, 15.6104164),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.664711f,
                                                40.2703629f, leg_index, 9.92947388f, 655,
                                                maneuver_index, 2.72661448f, 155));
  //----------------------------------------------------------------
  // trace point = 276
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6645203f, 40.270401f, 1489615114, 16.3663349),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6645355f,
                                                40.2704277f, leg_index, 9.91291523f, 654,
                                                maneuver_index, 2.71005583f, 154));
  //----------------------------------------------------------------
  // trace point = 277
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6643448f, 40.2704773f, 1489615115, 17.1030426),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.66436f,
                                                40.2704964f, leg_index, 9.89616489f, 653,
                                                maneuver_index, 2.69330549f, 153));
  //----------------------------------------------------------------
  // trace point = 278
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6641769f, 40.2705421f, 1489615116, 15.9991274),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6641922f,
                                                40.2705612f, leg_index, 9.88018513f, 652,
                                                maneuver_index, 2.67732573f, 152));
  //----------------------------------------------------------------
  // trace point = 279
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6640015f, 40.270607f, 1489615117, 16.6079903),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6640167f,
                                                40.2706299f, leg_index, 9.86343575f, 651,
                                                maneuver_index, 2.66057634f, 151));
  //----------------------------------------------------------------
  // trace point = 280
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6638336f, 40.2706795f, 1489615118, 16.2922821),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6638412f,
                                                40.2706947f, leg_index, 9.84687805f, 650,
                                                maneuver_index, 2.64401865f, 150));
  //----------------------------------------------------------------
  // trace point = 281
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6636658f, 40.2707481f, 1489615119, 16.2053375),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6636734f,
                                                40.2707596f, leg_index, 9.82935715f, 649,
                                                maneuver_index, 2.62649775f, 149));
  //----------------------------------------------------------------
  // trace point = 282
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6634903f, 40.2708244f, 1489615120, 17.1768436),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6634903f,
                                                40.2708282f, leg_index, 9.8135643f, 648,
                                                maneuver_index, 2.6107049f, 148));
  //----------------------------------------------------------------
  // trace point = 283
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6633148f, 40.2708931f, 1489615121, 16.7312107),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6633148f,
                                                40.2708931f, leg_index, 9.7970047f, 647,
                                                maneuver_index, 2.5941453f, 147));
  //----------------------------------------------------------------
  // trace point = 284
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6631393f, 40.2709656f, 1489615122, 17.0148106),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6631317f,
                                                40.2709579f, leg_index, 9.77985859f, 646,
                                                maneuver_index, 2.57699919f, 146));
  //----------------------------------------------------------------
  // trace point = 285
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6629562f, 40.2710419f, 1489615123, 17.6966591),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6629486f,
                                                40.2710266f, leg_index, 9.76253128f, 645,
                                                maneuver_index, 2.55967188f, 145));
  //----------------------------------------------------------------
  // trace point = 286
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6627655f, 40.2711143f, 1489615124, 18.0186367),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6627579f,
                                                40.2710953f, leg_index, 9.74461842f, 644,
                                                maneuver_index, 2.54175901f, 144));
  //----------------------------------------------------------------
  // trace point = 287
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.66259f, 40.271183f, 1489615125, 16.8066559),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6625748f,
                                                40.2711601f, leg_index, 9.72747326f, 644,
                                                maneuver_index, 2.52461386f, 144));
  //----------------------------------------------------------------
  // trace point = 288
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6624146f, 40.2712479f, 1489615126, 16.5314522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6623993f,
                                                40.271225f, leg_index, 9.7109127f, 643,
                                                maneuver_index, 2.5080533f, 143));
  //----------------------------------------------------------------
  // trace point = 289
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6622391f, 40.2713127f, 1489615127, 16.6078491),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6622238f,
                                                40.2712898f, leg_index, 9.69435215f, 642,
                                                maneuver_index, 2.49149275f, 142));
  //----------------------------------------------------------------
  // trace point = 290
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6620712f, 40.2713699f, 1489615128, 15.616416),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.662056f,
                                                40.2713509f, leg_index, 9.67856026f, 641,
                                                maneuver_index, 2.47570086f, 141));
  //----------------------------------------------------------------
  // trace point = 291
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6619186f, 40.2714233f, 1489615129, 14.1944876),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.661911f,
                                                40.2714043f, leg_index, 9.66489124f, 640,
                                                maneuver_index, 2.46203184f, 140));
  //----------------------------------------------------------------
  // trace point = 292
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6617813f, 40.2714729f, 1489615130, 12.9755516),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6617661f,
                                                40.27145f, leg_index, 9.65157032f, 639,
                                                maneuver_index, 2.44871092f, 139));
  //----------------------------------------------------------------
  // trace point = 293
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.661644f, 40.2715187f, 1489615131, 12.7130775),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6616287f,
                                                40.2714958f, leg_index, 9.63884258f, 638,
                                                maneuver_index, 2.43598318f, 138));
  //----------------------------------------------------------------
  // trace point = 294
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6615219f, 40.2715645f, 1489615132, 11.509469),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6615067f,
                                                40.2715416f, leg_index, 9.62729836f, 638,
                                                maneuver_index, 2.42443895f, 138));
  //----------------------------------------------------------------
  // trace point = 295
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6613998f, 40.2716026f, 1489615133, 11.2348633),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6613846f,
                                                40.2715797f, leg_index, 9.61609936f, 637,
                                                maneuver_index, 2.41323996f, 137));
  //----------------------------------------------------------------
  // trace point = 296
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6612854f, 40.2716446f, 1489615134, 10.8289061),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6612701f,
                                                40.2716217f, leg_index, 9.60531902f, 636,
                                                maneuver_index, 2.40245962f, 136));
  //----------------------------------------------------------------
  // trace point = 297
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.661171f, 40.2716789f, 1489615135, 10.3960924),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6611557f,
                                                40.2716599f, leg_index, 9.59471226f, 636,
                                                maneuver_index, 2.39185286f, 136));
  //----------------------------------------------------------------
  // trace point = 298
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6610641f, 40.2717171f, 1489615136, 10.0243425),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6610489f,
                                                40.2716942f, leg_index, 9.58487129f, 635,
                                                maneuver_index, 2.38201189f, 135));
  //----------------------------------------------------------------
  // trace point = 299
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6609497f, 40.2717514f, 1489615137, 10.3960829),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6609421f,
                                                40.2717323f, leg_index, 9.57485867f, 635,
                                                maneuver_index, 2.37199926f, 135));
  //----------------------------------------------------------------
  // trace point = 300
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6608353f, 40.2717857f, 1489615138, 10.5159636),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6608276f,
                                                40.2717667f, leg_index, 9.56441593f, 634,
                                                maneuver_index, 2.36155653f, 134));
  //----------------------------------------------------------------
  // trace point = 301
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6606369f, 40.271843f, 1489615140, 8.9872036),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6606293f,
                                                40.2718277f, leg_index, 9.5462513f, 633,
                                                maneuver_index, 2.3433919f, 133));
  //----------------------------------------------------------------
  // trace point = 302
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6605072f, 40.2718811f, 1489615142, 5.91207409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6604996f,
                                                40.2718697f, leg_index, 9.53428555f, 633,
                                                maneuver_index, 2.33142614f, 133));
  //----------------------------------------------------------------
  // trace point = 303
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6603546f, 40.2719231f, 1489615145, 4.57688999),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6603546f,
                                                40.2719193f, leg_index, 9.52079773f, 632,
                                                maneuver_index, 2.31793833f, 132));
  //----------------------------------------------------------------
  // trace point = 304
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6602402f, 40.2719498f, 1489615148, 3.40650606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6602402f,
                                                40.2719536f, leg_index, 9.510355f, 631,
                                                maneuver_index, 2.30749559f, 131));
  //----------------------------------------------------------------
  // trace point = 305
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6601334f, 40.2719841f, 1489615153, 1.96893799),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6601334f,
                                                40.2719879f, leg_index, 9.50051022f, 631,
                                                maneuver_index, 2.29765081f, 131));
  //----------------------------------------------------------------
  // trace point = 306
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6599045f, 40.2720261f, 1489615158, 4.00083399),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6599197f,
                                                40.2720528f, leg_index, 9.48097992f, 630,
                                                maneuver_index, 2.27812052f, 130));
  //----------------------------------------------------------------
  // trace point = 307
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.659729f, 40.2720718f, 1489615160, 7.86048222),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6597443f,
                                                40.2720985f, leg_index, 9.46522903f, 629,
                                                maneuver_index, 2.26236963f, 129));
  //----------------------------------------------------------------
  // trace point = 308
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6595154f, 40.2721291f, 1489615162, 9.62589169),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6595306f,
                                                40.2721558f, leg_index, 9.44599915f, 628,
                                                maneuver_index, 2.24313974f, 128));
  //----------------------------------------------------------------
  // trace point = 309
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6594009f, 40.2721634f, 1489615163, 10.3960276),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6594086f,
                                                40.2721863f, leg_index, 9.43509293f, 627,
                                                maneuver_index, 2.23223352f, 127));
  //----------------------------------------------------------------
  // trace point = 310
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6592789f, 40.2721977f, 1489615164, 11.0703306),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6592865f,
                                                40.2722206f, leg_index, 9.4240427f, 626,
                                                maneuver_index, 2.2211833f, 126));
  //----------------------------------------------------------------
  // trace point = 311
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6591568f, 40.2722282f, 1489615165, 10.8869057),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6591644f,
                                                40.2722511f, leg_index, 9.41312981f, 625,
                                                maneuver_index, 2.2102704f, 125));
  //----------------------------------------------------------------
  // trace point = 312
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6590424f, 40.2722588f, 1489615166, 10.3591824),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.65905f,
                                                40.2722816f, leg_index, 9.40283394f, 625,
                                                maneuver_index, 2.19997454f, 125));
  //----------------------------------------------------------------
  // trace point = 313
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6589127f, 40.2722893f, 1489615167, 11.4542284),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6589203f,
                                                40.2723122f, leg_index, 9.39130878f, 624,
                                                maneuver_index, 2.18844938f, 124));
  //----------------------------------------------------------------
  // trace point = 314
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.658783f, 40.2723312f, 1489615168, 11.990593),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6587906f,
                                                40.2723465f, leg_index, 9.37965012f, 623,
                                                maneuver_index, 2.17679071f, 123));
  //----------------------------------------------------------------
  // trace point = 315
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6586685f, 40.2723732f, 1489615169, 10.7522631),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6586685f,
                                                40.272377f, leg_index, 9.36873913f, 623,
                                                maneuver_index, 2.16587973f, 123));
  //----------------------------------------------------------------
  // trace point = 316
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6585464f, 40.2723999f, 1489615170, 10.870719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6585464f,
                                                40.2724075f, leg_index, 9.35782909f, 622,
                                                maneuver_index, 2.15496969f, 122));
  //----------------------------------------------------------------
  // trace point = 317
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6584244f, 40.2724342f, 1489615171, 11.0316095),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6584244f,
                                                40.272438f, leg_index, 9.34691906f, 622,
                                                maneuver_index, 2.14405966f, 122));
  //----------------------------------------------------------------
  // trace point = 318
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6583023f, 40.2724648f, 1489615172, 10.9215326),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6583023f,
                                                40.2724686f, leg_index, 9.33600807f, 621,
                                                maneuver_index, 2.13314867f, 121));
  //----------------------------------------------------------------
  // trace point = 319
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6581726f, 40.2724953f, 1489615173, 11.454196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6581726f,
                                                40.2725029f, leg_index, 9.3243494f, 620,
                                                maneuver_index, 2.12149f, 120));
  //----------------------------------------------------------------
  // trace point = 320
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6580429f, 40.2725296f, 1489615174, 11.7087269),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6580429f,
                                                40.2725372f, leg_index, 9.31269073f, 620,
                                                maneuver_index, 2.10983133f, 120));
  //----------------------------------------------------------------
  // trace point = 321
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6579208f, 40.2725601f, 1489615175, 10.886857),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6579208f,
                                                40.2725677f, leg_index, 9.30177975f, 619,
                                                maneuver_index, 2.09892035f, 119));
  //----------------------------------------------------------------
  // trace point = 322
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6577835f, 40.2725906f, 1489615176, 12.1369362),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6577911f,
                                                40.2726021f, leg_index, 9.29012108f, 618,
                                                maneuver_index, 2.08726168f, 118));
  //----------------------------------------------------------------
  // trace point = 323
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6576462f, 40.2726288f, 1489615177, 12.4609089),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6576462f,
                                                40.2726364f, leg_index, 9.27723217f, 618,
                                                maneuver_index, 2.07437277f, 118));
  //----------------------------------------------------------------
  // trace point = 324
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6575165f, 40.2726555f, 1489615178, 11.4144907),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6575241f,
                                                40.2726669f, leg_index, 9.26632118f, 617,
                                                maneuver_index, 2.06346178f, 117));
  //----------------------------------------------------------------
  // trace point = 325
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6573944f, 40.272686f, 1489615179, 10.9215002),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.657402f,
                                                40.2726974f, leg_index, 9.25541115f, 616,
                                                maneuver_index, 2.05255175f, 116));
  //----------------------------------------------------------------
  // trace point = 326
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6572647f, 40.2727165f, 1489615180, 11.487112),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6572723f,
                                                40.2727318f, leg_index, 9.24375248f, 616,
                                                maneuver_index, 2.04089308f, 116));
  //----------------------------------------------------------------
  // trace point = 327
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.657135f, 40.2727509f, 1489615181, 11.672122),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6571426f,
                                                40.2727623f, leg_index, 9.23222637f, 615,
                                                maneuver_index, 2.02936697f, 115));
  //----------------------------------------------------------------
  // trace point = 328
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6570053f, 40.2727814f, 1489615182, 11.5354223),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6570129f,
                                                40.2727966f, leg_index, 9.2205677f, 614,
                                                maneuver_index, 2.0177083f, 114));
  //----------------------------------------------------------------
  // trace point = 329
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6568832f, 40.2728157f, 1489615183, 11.0702419),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6568909f,
                                                40.2728271f, leg_index, 9.20965767f, 614,
                                                maneuver_index, 2.00679827f, 114));
  //----------------------------------------------------------------
  // trace point = 330
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6567535f, 40.2728462f, 1489615184, 11.4541416),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6567612f,
                                                40.2728615f, leg_index, 9.197999f, 613,
                                                maneuver_index, 1.9951396f, 113));
  //----------------------------------------------------------------
  // trace point = 331
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6566315f, 40.2728806f, 1489615185, 11.1497726),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6566391f,
                                                40.272892f, leg_index, 9.18708897f, 613,
                                                maneuver_index, 1.98422956f, 113));
  //----------------------------------------------------------------
  // trace point = 332
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.656517f, 40.2729073f, 1489615186, 10.1057978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6565247f,
                                                40.2729187f, leg_index, 9.17692661f, 612,
                                                maneuver_index, 1.97406721f, 112));
  //----------------------------------------------------------------
  // trace point = 333
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6563416f, 40.2729492f, 1489615188, 7.79071522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6563492f,
                                                40.2729645f, leg_index, 9.16117668f, 611,
                                                maneuver_index, 1.95831728f, 111));
  //----------------------------------------------------------------
  // trace point = 334
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6562271f, 40.2729797f, 1489615190, 5.17954206),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6562347f,
                                                40.2729912f, leg_index, 9.15101528f, 611,
                                                maneuver_index, 1.94815588f, 111));
  //----------------------------------------------------------------
  // trace point = 335
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6560593f, 40.2730217f, 1489615239, 0.306434989),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6560669f,
                                                40.273037f, leg_index, 9.13588238f, 610,
                                                maneuver_index, 1.93302298f, 110));
  //----------------------------------------------------------------
  // trace point = 336
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6558838f, 40.2730598f, 1489615241, 7.72681713),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6558914f,
                                                40.2730789f, leg_index, 9.1202631f, 609,
                                                maneuver_index, 1.9174037f, 109));
  //----------------------------------------------------------------
  // trace point = 337
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6556625f, 40.2731056f, 1489615243, 9.72814465),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6556702f,
                                                40.2731285f, leg_index, 9.10068035f, 608,
                                                maneuver_index, 1.89782095f, 108));
  //----------------------------------------------------------------
  // trace point = 338
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6555405f, 40.2731361f, 1489615244, 10.9676428),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6555481f,
                                                40.273159f, leg_index, 9.08977127f, 607,
                                                maneuver_index, 1.88691187f, 107));
  //----------------------------------------------------------------
  // trace point = 339
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6553955f, 40.2731705f, 1489615245, 12.8026848),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6554031f,
                                                40.2731934f, leg_index, 9.07688046f, 606,
                                                maneuver_index, 1.87402105f, 106));
  //----------------------------------------------------------------
  // trace point = 340
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6552505f, 40.2732048f, 1489615246, 12.9169807),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6552582f,
                                                40.2732277f, leg_index, 9.06398869f, 605,
                                                maneuver_index, 1.86112928f, 105));
  //----------------------------------------------------------------
  // trace point = 341
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6551132f, 40.273243f, 1489615247, 12.4608088),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6551208f,
                                                40.273262f, leg_index, 9.05171776f, 605,
                                                maneuver_index, 1.84885836f, 105));
  //----------------------------------------------------------------
  // trace point = 342
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6549759f, 40.2732773f, 1489615248, 12.2359686),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6549835f,
                                                40.2732925f, leg_index, 9.03956985f, 604,
                                                maneuver_index, 1.83671045f, 104));
  //----------------------------------------------------------------
  // trace point = 343
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6548462f, 40.2733154f, 1489615249, 11.8239298),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6548462f,
                                                40.2733192f, leg_index, 9.02753735f, 604,
                                                maneuver_index, 1.82467794f, 104));
  //----------------------------------------------------------------
  // trace point = 344
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6547012f, 40.2733498f, 1489615250, 12.9169559),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6547012f,
                                                40.2733498f, leg_index, 9.01476574f, 603,
                                                maneuver_index, 1.81190634f, 103));
  //----------------------------------------------------------------
  // trace point = 345
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6545486f, 40.2733765f, 1489615251, 13.2303286),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6545486f,
                                                40.2733841f, leg_index, 9.00125504f, 602,
                                                maneuver_index, 1.79839563f, 102));
  //----------------------------------------------------------------
  // trace point = 346
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6543808f, 40.273407f, 1489615252, 14.6801615),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6543884f,
                                                40.2734184f, leg_index, 8.98712063f, 601,
                                                maneuver_index, 1.78426123f, 101));
  //----------------------------------------------------------------
  // trace point = 347
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.654213f, 40.2734375f, 1489615253, 14.6801548),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6542206f,
                                                40.2734566f, leg_index, 8.97224712f, 600,
                                                maneuver_index, 1.76938772f, 100));
  //----------------------------------------------------------------
  // trace point = 348
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6540222f, 40.2734718f, 1489615254, 16.6579189),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6540298f,
                                                40.2734947f, leg_index, 8.95550346f, 600,
                                                maneuver_index, 1.75264406f, 100));
  //----------------------------------------------------------------
  // trace point = 349
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6538467f, 40.27351f, 1489615255, 15.4535379),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6538544f,
                                                40.2735329f, leg_index, 8.94000626f, 599,
                                                maneuver_index, 1.73714685f, 99));
  //----------------------------------------------------------------
  // trace point = 350
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.653656f, 40.2735481f, 1489615256, 16.7646828),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6536636f,
                                                40.2735748f, leg_index, 8.92314625f, 598,
                                                maneuver_index, 1.72028685f, 98));
  //----------------------------------------------------------------
  // trace point = 351
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6534653f, 40.2735939f, 1489615257, 17.0112228),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6534729f,
                                                40.2736168f, leg_index, 8.90628719f, 597,
                                                maneuver_index, 1.70342779f, 97));
  //----------------------------------------------------------------
  // trace point = 352
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6532745f, 40.2736435f, 1489615258, 17.0342216),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6532822f,
                                                40.2736588f, leg_index, 8.88942719f, 596,
                                                maneuver_index, 1.68656778f, 96));
  //----------------------------------------------------------------
  // trace point = 353
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6530762f, 40.2736855f, 1489615259, 17.5364037),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6530838f,
                                                40.2737007f, leg_index, 8.87194443f, 595,
                                                maneuver_index, 1.66908503f, 95));
  //----------------------------------------------------------------
  // trace point = 354
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6528702f, 40.2737274f, 1489615260, 18.1100979),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6528778f,
                                                40.2737465f, leg_index, 8.85372257f, 594,
                                                maneuver_index, 1.65086317f, 94));
  //----------------------------------------------------------------
  // trace point = 355
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6526566f, 40.2737732f, 1489615261, 18.8014545),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6526642f,
                                                40.2737923f, leg_index, 8.83487701f, 593,
                                                maneuver_index, 1.63201761f, 93));
  //----------------------------------------------------------------
  // trace point = 356
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6524506f, 40.2738152f, 1489615262, 18.110075),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6524582f,
                                                40.2738342f, leg_index, 8.8167696f, 592,
                                                maneuver_index, 1.6139102f, 92));
  //----------------------------------------------------------------
  // trace point = 357
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6522446f, 40.2738571f, 1489615263, 18.1100655),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6522522f,
                                                40.27388f, leg_index, 8.79854774f, 591,
                                                maneuver_index, 1.59568834f, 91));
  //----------------------------------------------------------------
  // trace point = 358
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6520386f, 40.2739067f, 1489615264, 18.360136),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6520462f,
                                                40.2739258f, leg_index, 8.78032589f, 590,
                                                maneuver_index, 1.57746649f, 90));
  //----------------------------------------------------------------
  // trace point = 359
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6518402f, 40.2739525f, 1489615265, 17.6283169),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6518478f,
                                                40.2739677f, leg_index, 8.76284313f, 589,
                                                maneuver_index, 1.55998373f, 89));
  //----------------------------------------------------------------
  // trace point = 360
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6516418f, 40.2739983f, 1489615266, 17.5789948),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6516495f,
                                                40.2740097f, leg_index, 8.74535942f, 588,
                                                maneuver_index, 1.54250002f, 88));
  //----------------------------------------------------------------
  // trace point = 361
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6514435f, 40.2740479f, 1489615267, 17.7944641),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6514435f,
                                                40.2740555f, leg_index, 8.72713661f, 587,
                                                maneuver_index, 1.52427721f, 87));
  //----------------------------------------------------------------
  // trace point = 362
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6512375f, 40.2740936f, 1489615268, 18.2300396),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6512375f,
                                                40.2741013f, leg_index, 8.70891476f, 586,
                                                maneuver_index, 1.50605536f, 86));
  //----------------------------------------------------------------
  // trace point = 363
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6510391f, 40.2741356f, 1489615269, 17.4544525),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6510468f,
                                                40.274147f, leg_index, 8.69193459f, 585,
                                                maneuver_index, 1.48907518f, 85));
  //----------------------------------------------------------------
  // trace point = 364
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6508408f, 40.2741814f, 1489615270, 17.5468464),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6508484f,
                                                40.2741928f, leg_index, 8.67433357f, 584,
                                                maneuver_index, 1.47147417f, 84));
  //----------------------------------------------------------------
  // trace point = 365
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6506424f, 40.2742233f, 1489615271, 17.536274),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.65065f,
                                                40.2742348f, leg_index, 8.65685368f, 583,
                                                maneuver_index, 1.45399427f, 83));
  //----------------------------------------------------------------
  // trace point = 366
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.650444f, 40.2742691f, 1489615272, 17.5789299),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6504517f,
                                                40.2742805f, leg_index, 8.63925171f, 582,
                                                maneuver_index, 1.43639231f, 82));
  //----------------------------------------------------------------
  // trace point = 367
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6502457f, 40.2743149f, 1489615273, 17.5789185),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6502533f,
                                                40.2743263f, leg_index, 8.6216507f, 581,
                                                maneuver_index, 1.41879129f, 81));
  //----------------------------------------------------------------
  // trace point = 368
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6500549f, 40.2743607f, 1489615274, 17.0110474),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6500549f,
                                                40.2743683f, leg_index, 8.6041708f, 580,
                                                maneuver_index, 1.4013114f, 80));
  //----------------------------------------------------------------
  // trace point = 369
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6498566f, 40.2744026f, 1489615275, 17.4543877),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6498642f,
                                                40.2744141f, leg_index, 8.58719063f, 579,
                                                maneuver_index, 1.38433123f, 79));
  //----------------------------------------------------------------
  // trace point = 370
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6496582f, 40.2744446f, 1489615276, 17.5362206),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6496658f,
                                                40.2744598f, leg_index, 8.56958961f, 578,
                                                maneuver_index, 1.36673021f, 78));
  //----------------------------------------------------------------
  // trace point = 371
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6494598f, 40.2744865f, 1489615277, 17.4543667),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6494675f,
                                                40.2745056f, leg_index, 8.5519886f, 577,
                                                maneuver_index, 1.3491292f, 77));
  //----------------------------------------------------------------
  // trace point = 372
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6492691f, 40.2745171f, 1489615278, 16.5621796),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6492767f,
                                                40.2745476f, leg_index, 8.53512955f, 576,
                                                maneuver_index, 1.33227015f, 76));
  //----------------------------------------------------------------
  // trace point = 373
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.649086f, 40.2745514f, 1489615279, 15.9967289),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6491013f,
                                                40.2745857f, leg_index, 8.51963425f, 575,
                                                maneuver_index, 1.31677485f, 75));
  //----------------------------------------------------------------
  // trace point = 374
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6489182f, 40.2745857f, 1489615280, 14.7909279),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6489334f,
                                                40.2746239f, leg_index, 8.5047617f, 574,
                                                maneuver_index, 1.30190229f, 74));
  //----------------------------------------------------------------
  // trace point = 375
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6487732f, 40.2746201f, 1489615281, 12.8835936),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6487885f,
                                                40.2746582f, leg_index, 8.49186897f, 574,
                                                maneuver_index, 1.28900957f, 74));
  //----------------------------------------------------------------
  // trace point = 376
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6486435f, 40.2746506f, 1489615282, 11.5678492),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6486664f,
                                                40.2746849f, leg_index, 8.48108387f, 573,
                                                maneuver_index, 1.27822447f, 73));
  //----------------------------------------------------------------
  // trace point = 377
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6485138f, 40.2746735f, 1489615283, 11.2265654),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6485214f,
                                                40.274704f, leg_index, 8.46859169f, 572,
                                                maneuver_index, 1.26573229f, 72));
  //----------------------------------------------------------------
  // trace point = 378
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6483994f, 40.2746964f, 1489615284, 10.0971489),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.648407f,
                                                40.2747231f, leg_index, 8.45865154f, 572,
                                                maneuver_index, 1.25579214f, 72));
  //----------------------------------------------------------------
  // trace point = 379
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6482697f, 40.2747116f, 1489615285, 11.1000109),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6482773f,
                                                40.2747383f, leg_index, 8.44750595f, 571,
                                                maneuver_index, 1.24464655f, 71));
  //----------------------------------------------------------------
  // trace point = 380
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6481323f, 40.2747269f, 1489615286, 11.8389101),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.64814f,
                                                40.2747536f, leg_index, 8.4357233f, 571,
                                                maneuver_index, 1.2328639f, 71));
  //----------------------------------------------------------------
  // trace point = 381
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6479797f, 40.2747459f, 1489615287, 13.0817165),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6479874f,
                                                40.2747765f, leg_index, 8.42252159f, 570,
                                                maneuver_index, 1.21966219f, 70));
  //----------------------------------------------------------------
  // trace point = 382
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6478271f, 40.2747726f, 1489615288, 13.3375998),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6478348f,
                                                40.2747917f, leg_index, 8.40945721f, 569,
                                                maneuver_index, 1.20659781f, 69));
  //----------------------------------------------------------------
  // trace point = 383
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6476669f, 40.2747955f, 1489615289, 13.8281422),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6476669f,
                                                40.2748146f, leg_index, 8.39497757f, 568,
                                                maneuver_index, 1.19211817f, 68));
  //----------------------------------------------------------------
  // trace point = 384
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6474991f, 40.2748184f, 1489615290, 14.4771404),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6474991f,
                                                40.2748337f, leg_index, 8.38056564f, 567,
                                                maneuver_index, 1.17770624f, 67));
  //----------------------------------------------------------------
  // trace point = 385
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.647316f, 40.2748413f, 1489615291, 15.7519341),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.647316f,
                                                40.2748566f, leg_index, 8.36480713f, 566,
                                                maneuver_index, 1.16194773f, 66));
  //----------------------------------------------------------------
  // trace point = 386
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6471329f, 40.2748642f, 1489615292, 15.7519293),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6471329f,
                                                40.2748795f, leg_index, 8.34904861f, 565,
                                                maneuver_index, 1.14618921f, 65));
  //----------------------------------------------------------------
  // trace point = 387
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6469421f, 40.2748871f, 1489615293, 16.4227104),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6469421f,
                                                40.2749023f, leg_index, 8.33264923f, 564,
                                                maneuver_index, 1.12978983f, 64));
  //----------------------------------------------------------------
  // trace point = 388
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6467514f, 40.27491f, 1489615294, 16.4227047),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6467514f,
                                                40.2749252f, leg_index, 8.3162508f, 563,
                                                maneuver_index, 1.1133914f, 63));
  //----------------------------------------------------------------
  // trace point = 389
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6465454f, 40.2749329f, 1489615295, 17.682209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6465454f,
                                                40.2749519f, leg_index, 8.29850578f, 562,
                                                maneuver_index, 1.09564638f, 62));
  //----------------------------------------------------------------
  // trace point = 390
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.646347f, 40.2749557f, 1489615296, 17.0101967),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.646347f,
                                                40.274971f, leg_index, 8.2815237f, 561,
                                                maneuver_index, 1.0786643f, 61));
  //----------------------------------------------------------------
  // trace point = 391
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6461487f, 40.2749786f, 1489615297, 17.010191),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6461487f,
                                                40.2749939f, leg_index, 8.26448822f, 560,
                                                maneuver_index, 1.06162882f, 60));
  //----------------------------------------------------------------
  // trace point = 392
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6459427f, 40.2749977f, 1489615298, 17.6232243),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6459427f,
                                                40.275013f, leg_index, 8.24686527f, 559,
                                                maneuver_index, 1.04400587f, 59));
  //----------------------------------------------------------------
  // trace point = 393
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6457367f, 40.2750168f, 1489615299, 17.6232185),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6457367f,
                                                40.275032f, leg_index, 8.22924232f, 558,
                                                maneuver_index, 1.02638292f, 58));
  //----------------------------------------------------------------
  // trace point = 394
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6455383f, 40.275032f, 1489615300, 16.9835987),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6455383f,
                                                40.2750511f, leg_index, 8.21226215f, 557,
                                                maneuver_index, 1.00940275f, 57));
  //----------------------------------------------------------------
  // trace point = 395
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6453247f, 40.2750511f, 1489615301, 18.2135715),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6453323f,
                                                40.275074f, leg_index, 8.19458389f, 556,
                                                maneuver_index, 0.991724491f, 56));
  //----------------------------------------------------------------
  // trace point = 396
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6451111f, 40.2750702f, 1489615302, 18.2979259),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6451187f,
                                                40.2750931f, leg_index, 8.17632866f, 555,
                                                maneuver_index, 0.973469257f, 55));
  //----------------------------------------------------------------
  // trace point = 397
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6448898f, 40.2750931f, 1489615303, 18.9436073),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6448898f,
                                                40.2751122f, leg_index, 8.15676022f, 554,
                                                maneuver_index, 0.953900814f, 54));
  //----------------------------------------------------------------
  // trace point = 398
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6446533f, 40.275116f, 1489615304, 20.2908611),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6446533f,
                                                40.275135f, leg_index, 8.13651466f, 553,
                                                maneuver_index, 0.933655262f, 53));
  //----------------------------------------------------------------
  // trace point = 399
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6444168f, 40.275135f, 1489615305, 20.1550255),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6444168f,
                                                40.2751541f, leg_index, 8.11631489f, 551,
                                                maneuver_index, 0.913455486f, 51));
  //----------------------------------------------------------------
  // trace point = 400
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6441803f, 40.2751541f, 1489615306, 20.2394829),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6441803f,
                                                40.2751732f, leg_index, 8.09611702f, 550,
                                                maneuver_index, 0.893257618f, 50));
  //----------------------------------------------------------------
  // trace point = 401
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6439362f, 40.2751732f, 1489615307, 20.8308334),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6439362f,
                                                40.2751923f, leg_index, 8.075284f, 549,
                                                maneuver_index, 0.872424603f, 49));
  //----------------------------------------------------------------
  // trace point = 402
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6436996f, 40.2751923f, 1489615308, 20.1550083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6436996f,
                                                40.2752075f, leg_index, 8.05510807f, 548,
                                                maneuver_index, 0.852248669f, 48));
  //----------------------------------------------------------------
  // trace point = 403
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6434555f, 40.2752037f, 1489615309, 20.7661762),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6434555f,
                                                40.275219f, leg_index, 8.03433418f, 547,
                                                maneuver_index, 0.831474781f, 47));
  //----------------------------------------------------------------
  // trace point = 404
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.643219f, 40.2752075f, 1489615310, 20.1335716),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.643219f,
                                                40.2752266f, leg_index, 8.01423168f, 545,
                                                maneuver_index, 0.81137228f, 45));
  //----------------------------------------------------------------
  // trace point = 405
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6429825f, 40.2752113f, 1489615311, 20.1314163),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6429825f,
                                                40.275238f, leg_index, 7.99410439f, 544,
                                                maneuver_index, 0.791244984f, 44));
  //----------------------------------------------------------------
  // trace point = 406
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.642746f, 40.275219f, 1489615312, 20.063488),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.642746f,
                                                40.2752419f, leg_index, 7.97400188f, 543,
                                                maneuver_index, 0.771142483f, 43));
  //----------------------------------------------------------------
  // trace point = 407
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6425171f, 40.2752228f, 1489615313, 19.4542904),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6425171f,
                                                40.2752457f, leg_index, 7.95455837f, 542,
                                                maneuver_index, 0.751698971f, 42));
  //----------------------------------------------------------------
  // trace point = 408
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6422958f, 40.2752304f, 1489615314, 18.7859135),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6422958f,
                                                40.2752495f, leg_index, 7.93576336f, 541,
                                                maneuver_index, 0.732903957f, 41));
  //----------------------------------------------------------------
  // trace point = 409
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6420746f, 40.2752342f, 1489615315, 18.7750244),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6420746f,
                                                40.2752495f, leg_index, 7.91697407f, 540,
                                                maneuver_index, 0.714114666f, 40));
  //----------------------------------------------------------------
  // trace point = 410
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.641861f, 40.2752419f, 1489615316, 18.1970291),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.641861f,
                                                40.2752533f, leg_index, 7.89882612f, 539,
                                                maneuver_index, 0.695966721f, 39));
  //----------------------------------------------------------------
  // trace point = 411
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.641655f, 40.2752457f, 1489615317, 17.501442),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.641655f,
                                                40.2752533f, leg_index, 7.8813343f, 538,
                                                maneuver_index, 0.678474903f, 38));
  //----------------------------------------------------------------
  // trace point = 412
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.641449f, 40.2752495f, 1489615318, 17.4989624),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.641449f,
                                                40.2752571f, leg_index, 7.86383295f, 537,
                                                maneuver_index, 0.660973549f, 37));
  //----------------------------------------------------------------
  // trace point = 413
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6412582f, 40.2752571f, 1489615319, 16.2462368),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6412582f,
                                                40.2752571f, leg_index, 7.84763479f, 536,
                                                maneuver_index, 0.644775391f, 36));
  //----------------------------------------------------------------
  // trace point = 414
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6410675f, 40.2752571f, 1489615320, 16.1368771),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6410675f,
                                                40.2752609f, leg_index, 7.83143187f, 535,
                                                maneuver_index, 0.628572464f, 35));
  //----------------------------------------------------------------
  // trace point = 415
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6408768f, 40.2752609f, 1489615321, 16.2279186),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6408768f,
                                                40.2752609f, leg_index, 7.81523323f, 534,
                                                maneuver_index, 0.612373829f, 34));
  //----------------------------------------------------------------
  // trace point = 416
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6407013f, 40.2752647f, 1489615322, 14.9544725),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6407013f,
                                                40.2752647f, leg_index, 7.80032635f, 533,
                                                maneuver_index, 0.597466946f, 33));
  //----------------------------------------------------------------
  // trace point = 417
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6405334f, 40.2752647f, 1489615323, 14.2683954),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6405334f,
                                                40.2752647f, leg_index, 7.78607178f, 532,
                                                maneuver_index, 0.583212376f, 32));
  //----------------------------------------------------------------
  // trace point = 418
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6403732f, 40.2752686f, 1489615324, 13.596241),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6403732f,
                                                40.2752647f, leg_index, 7.77246523f, 532,
                                                maneuver_index, 0.569605827f, 32));
  //----------------------------------------------------------------
  // trace point = 419
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6402359f, 40.2752724f, 1489615325, 11.6403275),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6402359f,
                                                40.2752686f, leg_index, 7.76079798f, 531,
                                                maneuver_index, 0.557938576f, 31));
  //----------------------------------------------------------------
  // trace point = 420
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6401138f, 40.2752724f, 1489615326, 10.3615723),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6401138f,
                                                40.2752686f, leg_index, 7.75043058f, 530,
                                                maneuver_index, 0.547571182f, 30));
  //----------------------------------------------------------------
  // trace point = 421
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6399384f, 40.2752724f, 1489615328, 7.47392082),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6399384f,
                                                40.2752686f, leg_index, 7.73552847f, 530,
                                                maneuver_index, 0.532669067f, 30));
  //----------------------------------------------------------------
  // trace point = 422
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6398087f, 40.2752762f, 1489615331, 3.65504408),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6398087f,
                                                40.2752724f, leg_index, 7.72450972f, 529,
                                                maneuver_index, 0.521650314f, 29));
  //----------------------------------------------------------------
  // trace point = 423
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.639679f, 40.2752762f, 1489615354, 0.480044007),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.639679f,
                                                40.2752724f, leg_index, 7.71349478f, 528,
                                                maneuver_index, 0.510635376f, 28));
  //----------------------------------------------------------------
  // trace point = 424
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6395187f, 40.27528f, 1489615356, 6.79812002),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6395187f,
                                                40.2752762f, leg_index, 7.69987631f, 527,
                                                maneuver_index, 0.497016907f, 27));
  //----------------------------------------------------------------
  // trace point = 425
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6392746f, 40.27528f, 1489615358, 10.3615704),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6392746f,
                                                40.2752838f, leg_index, 7.67912483f, 526,
                                                maneuver_index, 0.47626543f, 26));
  //----------------------------------------------------------------
  // trace point = 426
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6391373f, 40.2752762f, 1489615359, 11.7289209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6391373f,
                                                40.2752876f, leg_index, 7.66745615f, 525,
                                                maneuver_index, 0.464596748f, 25));
  //----------------------------------------------------------------
  // trace point = 427
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6389923f, 40.27528f, 1489615360, 12.3230305),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6389923f,
                                                40.2752838f, leg_index, 7.65513563f, 525,
                                                maneuver_index, 0.45227623f, 25));
  //----------------------------------------------------------------
  // trace point = 428 | Alert 0.4 | Enter the roundabout and take the 2nd exit.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6388092f, 40.2752686f, 1489615361, 15.5905199),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.6388092f, 40.27528f, leg_index, 7.63957834f, 524,
                                                maneuver_index, 0.436718941f, 24, instruction_index,
                                                0.4f));
  //----------------------------------------------------------------
  // trace point = 429
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6386337f, 40.2752647f, 1489615362, 14.8695803),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6386337f,
                                                40.2752762f, leg_index, 7.62466955f, 523,
                                                maneuver_index, 0.42181015f, 23));
  //----------------------------------------------------------------
  // trace point = 430
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.638443f, 40.2752571f, 1489615363, 16.2462349),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.638443f,
                                                40.2752724f, leg_index, 7.60846567f, 522,
                                                maneuver_index, 0.40560627f, 22));
  //----------------------------------------------------------------
  // trace point = 431
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6382446f, 40.2752571f, 1489615364, 16.8163242),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6382446f,
                                                40.2752686f, leg_index, 7.59161186f, 521,
                                                maneuver_index, 0.38875246f, 21));
  //----------------------------------------------------------------
  // trace point = 432
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6380386f, 40.2752495f, 1489615365, 17.518425),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6380386f,
                                                40.2752609f, leg_index, 7.57409716f, 520,
                                                maneuver_index, 0.371237755f, 20));
  //----------------------------------------------------------------
  // trace point = 433
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6378403f, 40.2752419f, 1489615366, 16.9192123),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6378403f,
                                                40.2752571f, leg_index, 7.55724621f, 519,
                                                maneuver_index, 0.354386806f, 19));
  //----------------------------------------------------------------
  // trace point = 434
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6376266f, 40.2752342f, 1489615367, 18.1122017),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6376266f,
                                                40.2752457f, leg_index, 7.53905535f, 519,
                                                maneuver_index, 0.336195946f, 19));
  //----------------------------------------------------------------
  // trace point = 435
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6374207f, 40.2752304f, 1489615368, 17.5014439),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6374207f,
                                                40.2752304f, leg_index, 7.52147818f, 518,
                                                maneuver_index, 0.318618774f, 18));
  //----------------------------------------------------------------
  // trace point = 436
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.637207f, 40.2752151f, 1489615369, 18.2517719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.637207f,
                                                40.2752151f, leg_index, 7.50325632f, 517,
                                                maneuver_index, 0.300396919f, 17));
  //----------------------------------------------------------------
  // trace point = 437
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6369934f, 40.2752037f, 1489615370, 18.2164364),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6369934f,
                                                40.2752037f, leg_index, 7.48506784f, 516,
                                                maneuver_index, 0.282208443f, 16));
  //----------------------------------------------------------------
  // trace point = 438
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6367722f, 40.2751884f, 1489615371, 18.8540707),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6367722f,
                                                40.2751884f, leg_index, 7.46620321f, 515,
                                                maneuver_index, 0.263343811f, 15));
  //----------------------------------------------------------------
  // trace point = 439
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6365509f, 40.275177f, 1489615372, 18.8096581),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6365509f,
                                                40.275177f, leg_index, 7.4473691f, 514,
                                                maneuver_index, 0.244509697f, 14));
  //----------------------------------------------------------------
  // trace point = 440
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6363297f, 40.2751617f, 1489615373, 18.8438892),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6363297f,
                                                40.2751656f, leg_index, 7.42853451f, 513,
                                                maneuver_index, 0.225675106f, 13));
  //----------------------------------------------------------------
  // trace point = 441
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6361084f, 40.2751503f, 1489615374, 18.9019585),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6361084f,
                                                40.2751541f, leg_index, 7.40970039f, 512,
                                                maneuver_index, 0.206840992f, 12));
  //----------------------------------------------------------------
  // trace point = 442
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6358871f, 40.2751312f, 1489615375, 18.8885612),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6358871f,
                                                40.2751427f, leg_index, 7.39086676f, 511,
                                                maneuver_index, 0.188007355f, 11));
  //----------------------------------------------------------------
  // trace point = 443
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6356735f, 40.275116f, 1489615376, 18.1672249),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6356735f,
                                                40.2751312f, leg_index, 7.37268353f, 510,
                                                maneuver_index, 0.169824123f, 10));
  //----------------------------------------------------------------
  // trace point = 444
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6354599f, 40.2751007f, 1489615377, 18.2518024),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6354599f,
                                                40.2751122f, leg_index, 7.35441828f, 509,
                                                maneuver_index, 0.151558876f, 9));
  //----------------------------------------------------------------
  // trace point = 445
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6352615f, 40.2750816f, 1489615378, 16.9488564),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6352615f,
                                                40.2750969f, leg_index, 7.33748579f, 508,
                                                maneuver_index, 0.134626389f, 8));
  //----------------------------------------------------------------
  // trace point = 446 | Pre | Enter the roundabout and take the 2nd exit.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6350632f, 40.2750664f, 1489615379, 16.9948921),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6350632f, 40.2750816f, leg_index, 7.32055283f,
                                                507, maneuver_index, 0.117693424f, 7,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 447
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6348648f, 40.2750511f, 1489615380, 16.8990726),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6348648f,
                                                40.2750664f, leg_index, 7.30361986f, 506,
                                                maneuver_index, 0.10076046f, 6));
  //----------------------------------------------------------------
  // trace point = 448
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6346817f, 40.2750397f, 1489615381, 15.5905724),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6346817f,
                                                40.2750511f, leg_index, 7.2879777f, 505,
                                                maneuver_index, 0.0851182938f, 5));
  //----------------------------------------------------------------
  // trace point = 449
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6344986f, 40.275032f, 1489615382, 15.5679045),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6344986f,
                                                40.2750397f, leg_index, 7.27237749f, 504,
                                                maneuver_index, 0.0695180893f, 4));
  //----------------------------------------------------------------
  // trace point = 450
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6343384f, 40.275032f, 1489615383, 13.6739264),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6343384f,
                                                40.275032f, leg_index, 7.25873327f, 504,
                                                maneuver_index, 0.0558738708f, 4));
  //----------------------------------------------------------------
  // trace point = 451
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6341858f, 40.2750282f, 1489615384, 12.917223),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6341858f,
                                                40.2750282f, leg_index, 7.24577808f, 503,
                                                maneuver_index, 0.0429186821f, 3));
  //----------------------------------------------------------------
  // trace point = 452
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6340637f, 40.275032f, 1489615385, 10.371172),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6340714f,
                                                40.2750092f, leg_index, 7.23583412f, 502,
                                                maneuver_index, 0.03297472f, 2));
  //----------------------------------------------------------------
  // trace point = 453
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6338577f, 40.2750015f, 1489615387, 8.90585613),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6338806f,
                                                40.274971f, leg_index, 7.2189641f, 501,
                                                maneuver_index, 0.0161046982f, 1));
  //----------------------------------------------------------------
  // trace point = 454 | Pre | Exit the roundabout onto East Governor Road, U.S. 3 22 East.
  maneuver_index = 6;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6337051f, 40.2749138f, 1489615389, 8.13665581),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.633728f, 40.2748871f, leg_index, 7.2028594f, 500,
                                                maneuver_index, 0.0267596245f, 0, instruction_index));
  //----------------------------------------------------------------
  // trace point = 455
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6335373f, 40.274868f, 1489615391, 7.57975817),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6335297f,
                                                40.274826f, leg_index, 7.18368959f, 500,
                                                maneuver_index, 0.00758981705f, 0));
  //----------------------------------------------------------------
  // trace point = 456
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6333542f, 40.2748909f, 1489615393, 7.87596083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6333466f,
                                                40.2748642f, leg_index, 7.16750765f, 499,
                                                maneuver_index, 0.647401333f, 29));
  //----------------------------------------------------------------
  // trace point = 457 | Post | Continue for 700 meters.
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6331329f, 40.2749176f, 1489615395, 9.50447845),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.6331253f, 40.2748909f, leg_index, 7.14845943f,
                                                498, maneuver_index, 0.628353119f, 28,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 458
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6330032f, 40.2749252f, 1489615396, 11.0685396),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6330032f,
                                                40.2748985f, leg_index, 7.13806009f, 498,
                                                maneuver_index, 0.617953777f, 28));
  //----------------------------------------------------------------
  // trace point = 459
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6328583f, 40.274929f, 1489615397, 12.3230934),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6328583f,
                                                40.27491f, leg_index, 7.12568712f, 497,
                                                maneuver_index, 0.605580807f, 27));
  //----------------------------------------------------------------
  // trace point = 460
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6327057f, 40.274929f, 1489615398, 12.909565),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6327057f,
                                                40.2749062f, leg_index, 7.11265945f, 497,
                                                maneuver_index, 0.592553139f, 27));
  //----------------------------------------------------------------
  // trace point = 461
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6325378f, 40.2749252f, 1489615399, 14.2754126),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6325378f,
                                                40.2748985f, leg_index, 7.09837914f, 496,
                                                maneuver_index, 0.57827282f, 26));
  //----------------------------------------------------------------
  // trace point = 462
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6323624f, 40.2749176f, 1489615400, 14.9682159),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6323624f,
                                                40.2748909f, leg_index, 7.08345175f, 495,
                                                maneuver_index, 0.563345432f, 25));
  //----------------------------------------------------------------
  // trace point = 463
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6321793f, 40.2749062f, 1489615401, 15.599741),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6321793f,
                                                40.2748833f, leg_index, 7.06787825f, 495,
                                                maneuver_index, 0.547771931f, 25));
  //----------------------------------------------------------------
  // trace point = 464
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6319885f, 40.2748947f, 1489615402, 16.1833553),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6319885f,
                                                40.2748718f, leg_index, 7.05163002f, 494,
                                                maneuver_index, 0.531523705f, 24));
  //----------------------------------------------------------------
  // trace point = 465
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6317902f, 40.2748833f, 1489615403, 16.9540558),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6317902f,
                                                40.2748642f, leg_index, 7.03476334f, 493,
                                                maneuver_index, 0.514657021f, 23));
  //----------------------------------------------------------------
  // trace point = 466
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6315994f, 40.274868f, 1489615404, 16.3076134),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6315994f,
                                                40.2748528f, leg_index, 7.01851511f, 492,
                                                maneuver_index, 0.498408794f, 22));
  //----------------------------------------------------------------
  // trace point = 467
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6314011f, 40.2748566f, 1489615405, 16.8609467),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6314011f,
                                                40.2748413f, leg_index, 7.00162077f, 491,
                                                maneuver_index, 0.481514454f, 21));
  //----------------------------------------------------------------
  // trace point = 468
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6311951f, 40.2748413f, 1489615406, 17.5863056),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6311951f,
                                                40.2748337f, leg_index, 6.98410702f, 491,
                                                maneuver_index, 0.464000702f, 21));
  //----------------------------------------------------------------
  // trace point = 469
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6309891f, 40.274826f, 1489615407, 17.5753841),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6309891f,
                                                40.2748222f, leg_index, 6.96656799f, 490,
                                                maneuver_index, 0.446461678f, 20));
  //----------------------------------------------------------------
  // trace point = 470
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6307831f, 40.2748108f, 1489615408, 17.575388),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6307831f,
                                                40.2748032f, leg_index, 6.94894791f, 489,
                                                maneuver_index, 0.428841591f, 19));
  //----------------------------------------------------------------
  // trace point = 471
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6305771f, 40.2747955f, 1489615409, 17.5753918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6305771f,
                                                40.2747879f, leg_index, 6.93137026f, 488,
                                                maneuver_index, 0.411263943f, 18));
  //----------------------------------------------------------------
  // trace point = 472
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6303787f, 40.2747803f, 1489615410, 16.9105015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6303787f,
                                                40.2747726f, leg_index, 6.91443729f, 487,
                                                maneuver_index, 0.394330978f, 17));
  //----------------------------------------------------------------
  // trace point = 473
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6301727f, 40.2747612f, 1489615411, 17.6232796),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6301727f,
                                                40.2747574f, leg_index, 6.89685965f, 486,
                                                maneuver_index, 0.37675333f, 16));
  //----------------------------------------------------------------
  // trace point = 474
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6299591f, 40.2747459f, 1489615412, 18.2518978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6299591f,
                                                40.2747421f, leg_index, 6.87863731f, 486,
                                                maneuver_index, 0.358530998f, 16));
  //----------------------------------------------------------------
  // trace point = 475
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6297531f, 40.2747307f, 1489615413, 17.575407),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6297531f,
                                                40.2747269f, leg_index, 6.86105967f, 485,
                                                maneuver_index, 0.34095335f, 15));
  //----------------------------------------------------------------
  // trace point = 476
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6295395f, 40.2747154f, 1489615414, 18.2624264),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6295395f,
                                                40.2747078f, leg_index, 6.84279537f, 484,
                                                maneuver_index, 0.322689056f, 14));
  //----------------------------------------------------------------
  // trace point = 477
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6293182f, 40.2746964f, 1489615415, 18.8886814),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6293182f,
                                                40.2746925f, leg_index, 6.82392788f, 483,
                                                maneuver_index, 0.303821564f, 13));
  //----------------------------------------------------------------
  // trace point = 478
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6290894f, 40.2746811f, 1489615416, 19.520895),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6290894f,
                                                40.2746735f, leg_index, 6.80437422f, 482,
                                                maneuver_index, 0.284267902f, 12));
  //----------------------------------------------------------------
  // trace point = 479
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6288605f, 40.2746582f, 1489615417, 19.6171532),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6288605f,
                                                40.2746582f, leg_index, 6.78486347f, 481,
                                                maneuver_index, 0.264757156f, 11));
  //----------------------------------------------------------------
  // trace point = 480
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6286392f, 40.2746391f, 1489615418, 18.8886967),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6286392f,
                                                40.2746391f, leg_index, 6.76595163f, 481,
                                                maneuver_index, 0.245845318f, 11));
  //----------------------------------------------------------------
  // trace point = 481
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6284103f, 40.2746239f, 1489615419, 19.5209103),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6284103f,
                                                40.2746201f, leg_index, 6.74639654f, 480,
                                                maneuver_index, 0.226290226f, 10));
  //----------------------------------------------------------------
  // trace point = 482
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6281815f, 40.274601f, 1489615420, 19.6171703),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6281815f,
                                                40.274601f, leg_index, 6.72684193f, 479,
                                                maneuver_index, 0.206735611f, 9));
  //----------------------------------------------------------------
  // trace point = 483
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6279602f, 40.2745819f, 1489615421, 18.8887119),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6279602f,
                                                40.2745819f, leg_index, 6.70793056f, 478,
                                                maneuver_index, 0.187824249f, 8));
  //----------------------------------------------------------------
  // trace point = 484
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6277313f, 40.2745667f, 1489615422, 19.5209274),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6277313f,
                                                40.2745628f, leg_index, 6.68837595f, 477,
                                                maneuver_index, 0.168269634f, 7));
  //----------------------------------------------------------------
  // trace point = 485
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6275177f, 40.2745476f, 1489615423, 18.2136993),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6275177f,
                                                40.2745438f, leg_index, 6.67010641f, 476,
                                                maneuver_index, 0.150000095f, 6));
  //----------------------------------------------------------------
  // trace point = 486 | Pre | Enter the roundabout and take the 2nd exit.
  maneuver_index = 7;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6272964f, 40.2745323f, 1489615424, 18.9388065),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6272964f, 40.2745247f, leg_index, 6.651196f, 475,
                                                maneuver_index, 0.131089687f, 5, instruction_index));
  //----------------------------------------------------------------
  // trace point = 487
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6270905f, 40.2745209f, 1489615425, 17.5387611),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6270981f,
                                                40.2745018f, leg_index, 6.63415813f, 475,
                                                maneuver_index, 0.114051819f, 5));
  //----------------------------------------------------------------
  // trace point = 488
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6268921f, 40.2745171f, 1489615426, 16.822403),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6268921f,
                                                40.2744865f, leg_index, 6.61645937f, 474,
                                                maneuver_index, 0.096353054f, 4));
  //----------------------------------------------------------------
  // trace point = 489
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6266937f, 40.2745247f, 1489615427, 16.8400726),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6266937f,
                                                40.2744904f, leg_index, 6.59960842f, 473,
                                                maneuver_index, 0.0795021057f, 3));
  //----------------------------------------------------------------
  // trace point = 490
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.626503f, 40.2745476f, 1489615428, 16.4227905),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6264954f,
                                                40.2745056f, leg_index, 6.58266306f, 473,
                                                maneuver_index, 0.0625567436f, 3));
  //----------------------------------------------------------------
  // trace point = 491
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6263275f, 40.2745705f, 1489615429, 15.0634823),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6263199f,
                                                40.2745285f, leg_index, 6.56754351f, 472,
                                                maneuver_index, 0.047437191f, 2));
  //----------------------------------------------------------------
  // trace point = 492
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6261826f, 40.2745934f, 1489615430, 12.5784454),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6261673f,
                                                40.2745552f, leg_index, 6.55424309f, 471,
                                                maneuver_index, 0.0341367722f, 1));
  //----------------------------------------------------------------
  // trace point = 493
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6260452f, 40.2746162f, 1489615431, 11.9969759),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6260376f,
                                                40.2745781f, leg_index, 6.54288769f, 471,
                                                maneuver_index, 0.0227813721f, 1));
  //----------------------------------------------------------------
  // trace point = 494
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.625824f, 40.2745972f, 1489615433, 9.44435406),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.625824f,
                                                40.2745781f, leg_index, 6.52466249f, 470,
                                                maneuver_index, 0.00455617905f, 0));
  //----------------------------------------------------------------
  // trace point = 495 | Pre | Exit the roundabout onto East Governor Road, U.S. 3 22 East.
  maneuver_index = 8;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6256561f, 40.2745323f, 1489615435, 7.99918222),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.6256943f, 40.274498f, leg_index, 6.50940084f, 469,
                                                maneuver_index, 0.0304069519f, 1, instruction_index));
  //----------------------------------------------------------------
  // trace point = 496
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6254959f, 40.2745094f, 1489615437, 6.91409683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6254807f,
                                                40.2744751f, leg_index, 6.49022341f, 468,
                                                maneuver_index, 0.0112295151f, 0));
  //----------------------------------------------------------------
  // trace point = 497
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6253128f, 40.2745628f, 1489615439, 8.33224583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6253052f,
                                                40.2745514f, leg_index, 6.47279358f, 468,
                                                maneuver_index, 4.12477875f, 198));
  //----------------------------------------------------------------
  // trace point = 498
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.625206f, 40.2745972f, 1489615440, 9.84436321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6251984f,
                                                40.2745857f, leg_index, 6.46295118f, 467,
                                                maneuver_index, 4.11493587f, 197));
  //----------------------------------------------------------------
  // trace point = 499
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6250763f, 40.2746353f, 1489615441, 11.8237286),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6250687f,
                                                40.2746162f, leg_index, 6.45138264f, 466,
                                                maneuver_index, 4.10336781f, 196));
  //----------------------------------------------------------------
  // trace point = 500 | Post | Continue on U.S. 3 22 East for 4.1 kilometers.
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6249313f, 40.2746696f, 1489615442, 12.9167261),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.6249237f, 40.2746468f, leg_index, 6.43861198f,
                                                466, maneuver_index, 4.09059715f, 196,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 501
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6247711f, 40.2747002f, 1489615443, 13.9934092),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6247635f,
                                                40.2746811f, leg_index, 6.42448425f, 465,
                                                maneuver_index, 4.07646942f, 195));
  //----------------------------------------------------------------
  // trace point = 502
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6245956f, 40.2747307f, 1489615444, 15.2583904),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.624588f,
                                                40.2747078f, leg_index, 6.40928555f, 464,
                                                maneuver_index, 4.06127071f, 194));
  //----------------------------------------------------------------
  // trace point = 503
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6244125f, 40.2747574f, 1489615445, 15.8096704),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6244049f,
                                                40.2747345f, leg_index, 6.39345264f, 464,
                                                maneuver_index, 4.04543781f, 194));
  //----------------------------------------------------------------
  // trace point = 504
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6242218f, 40.2747917f, 1489615446, 16.6832561),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6242142f,
                                                40.2747612f, leg_index, 6.37698269f, 463,
                                                maneuver_index, 4.02896786f, 193));
  //----------------------------------------------------------------
  // trace point = 505
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6240234f, 40.2748184f, 1489615447, 17.147398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6240158f,
                                                40.2747879f, leg_index, 6.35852385f, 462,
                                                maneuver_index, 4.01050854f, 192));
  //----------------------------------------------------------------
  // trace point = 506
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6238251f, 40.2748451f, 1489615448, 17.0829201),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6238251f,
                                                40.2748413f, leg_index, 6.34264421f, 461,
                                                maneuver_index, 3.99462914f, 191));
  //----------------------------------------------------------------
  // trace point = 507
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6236267f, 40.2748718f, 1489615449, 17.0829124),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6236267f,
                                                40.274868f, leg_index, 6.32552338f, 461,
                                                maneuver_index, 3.97750831f, 191));
  //----------------------------------------------------------------
  // trace point = 508
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.623436f, 40.2749023f, 1489615450, 16.562088),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.623436f,
                                                40.2748947f, leg_index, 6.30905581f, 460,
                                                maneuver_index, 3.96104074f, 190));
  //----------------------------------------------------------------
  // trace point = 509
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6232376f, 40.2749252f, 1489615451, 17.0102043),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6232376f,
                                                40.2749176f, leg_index, 6.29201889f, 459,
                                                maneuver_index, 3.94400382f, 189));
  //----------------------------------------------------------------
  // trace point = 510
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6230392f, 40.2749519f, 1489615452, 17.1665058),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6230392f,
                                                40.2749443f, leg_index, 6.27491188f, 458,
                                                maneuver_index, 3.92689681f, 188));
  //----------------------------------------------------------------
  // trace point = 511
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6228333f, 40.274971f, 1489615453, 17.62323),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6228333f,
                                                40.274971f, leg_index, 6.25716591f, 458,
                                                maneuver_index, 3.90915084f, 188));
  //----------------------------------------------------------------
  // trace point = 512
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6226349f, 40.2749939f, 1489615454, 17.0101871),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6226349f,
                                                40.2749939f, leg_index, 6.24012899f, 457,
                                                maneuver_index, 3.89211392f, 187));
  //----------------------------------------------------------------
  // trace point = 513
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6224442f, 40.2750168f, 1489615455, 16.4226799),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6224442f,
                                                40.2750206f, leg_index, 6.2236619f, 456,
                                                maneuver_index, 3.87564683f, 186));
  //----------------------------------------------------------------
  // trace point = 514
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6222458f, 40.2750397f, 1489615456, 17.0101757),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6222458f,
                                                40.2750435f, leg_index, 6.20662403f, 455,
                                                maneuver_index, 3.85860896f, 185));
  //----------------------------------------------------------------
  // trace point = 515
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6220398f, 40.2750664f, 1489615457, 17.7336102),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6220398f,
                                                40.2750702f, leg_index, 6.18887901f, 454,
                                                maneuver_index, 3.84086394f, 184));
  //----------------------------------------------------------------
  // trace point = 516
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6218491f, 40.2750854f, 1489615458, 16.3591576),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6218491f,
                                                40.2750931f, leg_index, 6.17247963f, 454,
                                                maneuver_index, 3.82446456f, 184));
  //----------------------------------------------------------------
  // trace point = 517
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6216507f, 40.2751083f, 1489615459, 17.0101585),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6216507f,
                                                40.275116f, leg_index, 6.15544081f, 453,
                                                maneuver_index, 3.80742574f, 183));
  //----------------------------------------------------------------
  // trace point = 518
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.62146f, 40.2751312f, 1489615460, 16.4226532),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.62146f,
                                                40.2751389f, leg_index, 6.13904238f, 452,
                                                maneuver_index, 3.79102731f, 182));
  //----------------------------------------------------------------
  // trace point = 519
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6212692f, 40.2751503f, 1489615461, 16.3591404),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6212692f,
                                                40.2751617f, leg_index, 6.12264395f, 451,
                                                maneuver_index, 3.77462888f, 181));
  //----------------------------------------------------------------
  // trace point = 520
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6210861f, 40.275177f, 1489615462, 15.8303328),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6210861f,
                                                40.2751846f, leg_index, 6.10688543f, 450,
                                                maneuver_index, 3.75887036f, 180));
  //----------------------------------------------------------------
  // trace point = 521
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6209106f, 40.2751961f, 1489615463, 15.0126677),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6209106f,
                                                40.2752075f, leg_index, 6.09176731f, 450,
                                                maneuver_index, 3.74375224f, 180));
  //----------------------------------------------------------------
  // trace point = 522
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6207352f, 40.2752151f, 1489615464, 15.0967522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6207352f,
                                                40.2752266f, leg_index, 6.07671499f, 449,
                                                maneuver_index, 3.72869992f, 179));
  //----------------------------------------------------------------
  // trace point = 523
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6205597f, 40.275238f, 1489615465, 15.0818367),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6205597f,
                                                40.2752495f, leg_index, 6.06159639f, 448,
                                                maneuver_index, 3.71358132f, 178));
  //----------------------------------------------------------------
  // trace point = 524
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6203766f, 40.2752571f, 1489615466, 15.6856174),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6203766f,
                                                40.2752686f, leg_index, 6.04590416f, 448,
                                                maneuver_index, 3.69788909f, 178));
  //----------------------------------------------------------------
  // trace point = 525
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6201935f, 40.2752762f, 1489615467, 15.7697725),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6201935f,
                                                40.2752914f, leg_index, 6.03014565f, 447,
                                                maneuver_index, 3.68213058f, 177));
  //----------------------------------------------------------------
  // trace point = 526
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6200104f, 40.2752991f, 1489615468, 15.7518301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6200104f,
                                                40.2753143f, leg_index, 6.01438713f, 446,
                                                maneuver_index, 3.66637206f, 176));
  //----------------------------------------------------------------
  // trace point = 527
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6198273f, 40.275322f, 1489615469, 15.7518253),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6198273f,
                                                40.2753372f, leg_index, 5.99862909f, 445,
                                                maneuver_index, 3.65061402f, 175));
  //----------------------------------------------------------------
  // trace point = 528
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6196365f, 40.2753448f, 1489615470, 16.3387146),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6196365f,
                                                40.2753601f, leg_index, 5.98223066f, 445,
                                                maneuver_index, 3.63421559f, 175));
  //----------------------------------------------------------------
  // trace point = 529
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6194382f, 40.2753677f, 1489615471, 17.0940628),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6194382f,
                                                40.275383f, leg_index, 5.96519184f, 444,
                                                maneuver_index, 3.61717677f, 174));
  //----------------------------------------------------------------
  // trace point = 530
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6192474f, 40.2753906f, 1489615472, 16.4225922),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6192474f,
                                                40.2754059f, leg_index, 5.94879341f, 443,
                                                maneuver_index, 3.60077834f, 173));
  //----------------------------------------------------------------
  // trace point = 531
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6190491f, 40.2754173f, 1489615473, 17.0635452),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6190491f,
                                                40.2754288f, leg_index, 5.93175507f, 442,
                                                maneuver_index, 3.58374f, 172));
  //----------------------------------------------------------------
  // trace point = 532
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6188507f, 40.2754364f, 1489615474, 16.9487724),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6188507f,
                                                40.2754517f, leg_index, 5.91471672f, 441,
                                                maneuver_index, 3.56670165f, 171));
  //----------------------------------------------------------------
  // trace point = 533
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6186447f, 40.2754593f, 1489615475, 17.6820736),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6186523f,
                                                40.2754784f, leg_index, 5.89761162f, 441,
                                                maneuver_index, 3.54959655f, 171));
  //----------------------------------------------------------------
  // trace point = 534
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6184464f, 40.2754784f, 1489615476, 17.0330334),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.618454f,
                                                40.2755013f, leg_index, 5.88057327f, 440,
                                                maneuver_index, 3.5325582f, 170));
  //----------------------------------------------------------------
  // trace point = 535
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6182404f, 40.2755051f, 1489615477, 17.7520065),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.618248f,
                                                40.275528f, leg_index, 5.86282969f, 439,
                                                maneuver_index, 3.51481462f, 169));
  //----------------------------------------------------------------
  // trace point = 536
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.618042f, 40.2755356f, 1489615478, 17.1666927),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.618042f,
                                                40.2755508f, leg_index, 5.84514999f, 438,
                                                maneuver_index, 3.49713492f, 168));
  //----------------------------------------------------------------
  // trace point = 537
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6178436f, 40.2755661f, 1489615479, 17.1446533),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6178513f,
                                                40.2755852f, leg_index, 5.82850838f, 438,
                                                maneuver_index, 3.48049331f, 168));
  //----------------------------------------------------------------
  // trace point = 538
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6176453f, 40.2755966f, 1489615480, 17.2498837),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6176529f,
                                                40.2756157f, leg_index, 5.8113246f, 437,
                                                maneuver_index, 3.46330953f, 167));
  //----------------------------------------------------------------
  // trace point = 539
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6174545f, 40.2756348f, 1489615481, 16.6820297),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6174622f,
                                                40.27565f, leg_index, 5.79467678f, 436,
                                                maneuver_index, 3.44666171f, 166));
  //----------------------------------------------------------------
  // trace point = 540
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6172562f, 40.2756691f, 1489615482, 17.319767),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6172638f,
                                                40.275692f, leg_index, 5.77719402f, 435,
                                                maneuver_index, 3.42917895f, 165));
  //----------------------------------------------------------------
  // trace point = 541
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6170654f, 40.2757072f, 1489615483, 16.764183),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6170731f,
                                                40.2757339f, leg_index, 5.76033497f, 434,
                                                maneuver_index, 3.4123199f, 164));
  //----------------------------------------------------------------
  // trace point = 542
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6168671f, 40.2757492f, 1489615484, 17.4540653),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6168747f,
                                                40.2757759f, leg_index, 5.74285269f, 433,
                                                maneuver_index, 3.39483762f, 163));
  //----------------------------------------------------------------
  // trace point = 543
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6166763f, 40.2757912f, 1489615485, 16.882021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.616684f,
                                                40.2758179f, leg_index, 5.72599363f, 433,
                                                maneuver_index, 3.37797856f, 163));
  //----------------------------------------------------------------
  // trace point = 544
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6164932f, 40.2758369f, 1489615486, 16.3640633),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6165009f,
                                                40.275856f, leg_index, 5.70987368f, 432,
                                                maneuver_index, 3.36185861f, 162));
  //----------------------------------------------------------------
  // trace point = 545
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6163025f, 40.2758789f, 1489615487, 16.882),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6163101f,
                                                40.275898f, leg_index, 5.69301367f, 431,
                                                maneuver_index, 3.3449986f, 161));
  //----------------------------------------------------------------
  // trace point = 546
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6161194f, 40.2759209f, 1489615488, 16.230217),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.616127f,
                                                40.2759399f, leg_index, 5.67677641f, 430,
                                                maneuver_index, 3.32876134f, 160));
  //----------------------------------------------------------------
  // trace point = 547
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6159363f, 40.2759666f, 1489615489, 16.3640347),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6159439f,
                                                40.2759819f, leg_index, 5.66052866f, 429,
                                                maneuver_index, 3.31251359f, 159));
  //----------------------------------------------------------------
  // trace point = 548
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6157455f, 40.2760124f, 1489615490, 16.8963623),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6157455f,
                                                40.2760201f, leg_index, 5.64315462f, 429,
                                                maneuver_index, 3.29513955f, 159));
  //----------------------------------------------------------------
  // trace point = 549
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6155548f, 40.2760582f, 1489615491, 17.0106602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6155548f,
                                                40.2760582f, leg_index, 5.62639904f, 428,
                                                maneuver_index, 3.27838397f, 158));
  //----------------------------------------------------------------
  // trace point = 550
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6153564f, 40.276104f, 1489615492, 17.6597576),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6153564f,
                                                40.276104f, leg_index, 5.6087985f, 427,
                                                maneuver_index, 3.26078343f, 157));
  //----------------------------------------------------------------
  // trace point = 551
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6151505f, 40.2761536f, 1489615493, 18.359581),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6151505f,
                                                40.2761536f, leg_index, 5.5904541f, 426,
                                                maneuver_index, 3.24243903f, 156));
  //----------------------------------------------------------------
  // trace point = 552
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6149445f, 40.2761993f, 1489615494, 18.1985607),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6149445f,
                                                40.2761993f, leg_index, 5.5722332f, 425,
                                                maneuver_index, 3.22421813f, 155));
  //----------------------------------------------------------------
  // trace point = 553
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6147308f, 40.2762451f, 1489615495, 18.8008175),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6147308f,
                                                40.2762489f, leg_index, 5.55326891f, 425,
                                                maneuver_index, 3.20525384f, 155));
  //----------------------------------------------------------------
  // trace point = 554
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6145248f, 40.2762947f, 1489615496, 18.3595467),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6145248f,
                                                40.2762985f, leg_index, 5.53492451f, 424,
                                                maneuver_index, 3.18690944f, 154));
  //----------------------------------------------------------------
  // trace point = 555
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6143112f, 40.2763443f, 1489615497, 18.9758129),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6143112f,
                                                40.2763481f, leg_index, 5.51596022f, 423,
                                                maneuver_index, 3.16794515f, 153));
  //----------------------------------------------------------------
  // trace point = 556
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6140976f, 40.2763977f, 1489615498, 19.0626049),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6140976f,
                                                40.2763977f, leg_index, 5.49699593f, 422,
                                                maneuver_index, 3.14898086f, 152));
  //----------------------------------------------------------------
  // trace point = 557
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.613884f, 40.2764511f, 1489615499, 19.1085434),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.613884f,
                                                40.2764473f, leg_index, 5.47803164f, 421,
                                                maneuver_index, 3.13001657f, 151));
  //----------------------------------------------------------------
  // trace point = 558
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6136703f, 40.2765007f, 1489615500, 19.0080719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6136703f,
                                                40.2764969f, leg_index, 5.45906734f, 420,
                                                maneuver_index, 3.11105227f, 150));
  //----------------------------------------------------------------
  // trace point = 559
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6134567f, 40.2765503f, 1489615501, 18.8944321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6134567f,
                                                40.2765465f, leg_index, 5.44010305f, 419,
                                                maneuver_index, 3.09208798f, 149));
  //----------------------------------------------------------------
  // trace point = 560
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6132431f, 40.2765999f, 1489615502, 19.0080471),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6132431f,
                                                40.2765961f, leg_index, 5.42113924f, 418,
                                                maneuver_index, 3.07312417f, 148));
  //----------------------------------------------------------------
  // trace point = 561
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6130295f, 40.2766495f, 1489615503, 18.9757347),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6130295f,
                                                40.2766457f, leg_index, 5.40217495f, 417,
                                                maneuver_index, 3.05415988f, 147));
  //----------------------------------------------------------------
  // trace point = 562
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6128235f, 40.2766953f, 1489615504, 18.229393),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6128235f,
                                                40.2766953f, leg_index, 5.3838315f, 417,
                                                maneuver_index, 3.03581643f, 147));
  //----------------------------------------------------------------
  // trace point = 563
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6126175f, 40.276741f, 1489615505, 18.2293816),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6126175f,
                                                40.276741f, leg_index, 5.36560965f, 416,
                                                maneuver_index, 3.01759458f, 146));
  //----------------------------------------------------------------
  // trace point = 564
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6124268f, 40.276783f, 1489615506, 16.8002014),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6124268f,
                                                40.2767868f, leg_index, 5.34862947f, 415,
                                                maneuver_index, 3.0006144f, 145));
  //----------------------------------------------------------------
  // trace point = 565
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6122284f, 40.2768288f, 1489615507, 17.659584),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6122284f,
                                                40.2768326f, leg_index, 5.33102942f, 414,
                                                maneuver_index, 2.98301435f, 144));
  //----------------------------------------------------------------
  // trace point = 566
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.61203f, 40.2768669f, 1489615508, 17.3398266),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.61203f,
                                                40.2768745f, leg_index, 5.31356716f, 413,
                                                maneuver_index, 2.96555209f, 143));
  //----------------------------------------------------------------
  // trace point = 567
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6118317f, 40.2769051f, 1489615509, 17.339817),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6118317f,
                                                40.2769089f, leg_index, 5.29624796f, 412,
                                                maneuver_index, 2.94823289f, 142));
  //----------------------------------------------------------------
  // trace point = 568
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6116257f, 40.2769356f, 1489615510, 17.8324356),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6116257f,
                                                40.2769394f, leg_index, 5.27842665f, 411,
                                                maneuver_index, 2.93041158f, 141));
  //----------------------------------------------------------------
  // trace point = 569
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.611412f, 40.2769585f, 1489615511, 18.3390255),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.611412f,
                                                40.2769699f, leg_index, 5.25996828f, 410,
                                                maneuver_index, 2.91195321f, 140));
  //----------------------------------------------------------------
  // trace point = 570
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6111908f, 40.2769699f, 1489615512, 18.81674),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6111908f,
                                                40.276989f, leg_index, 5.24101162f, 410,
                                                maneuver_index, 2.89299655f, 140));
  //----------------------------------------------------------------
  // trace point = 571
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6109695f, 40.2769775f, 1489615513, 18.7903748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6109695f,
                                                40.2770004f, leg_index, 5.22217989f, 409,
                                                maneuver_index, 2.87416482f, 139));
  //----------------------------------------------------------------
  // trace point = 572
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6107407f, 40.276989f, 1489615514, 19.4871979),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6107407f,
                                                40.2770081f, leg_index, 5.2027235f, 408,
                                                maneuver_index, 2.85470843f, 138));
  //----------------------------------------------------------------
  // trace point = 573
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6105118f, 40.2769966f, 1489615515, 19.4690628),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6105118f,
                                                40.2770157f, leg_index, 5.18326664f, 407,
                                                maneuver_index, 2.83525157f, 137));
  //----------------------------------------------------------------
  // trace point = 574
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6102829f, 40.2770081f, 1489615516, 19.4871922),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6102829f,
                                                40.2770271f, leg_index, 5.16378784f, 406,
                                                maneuver_index, 2.81577277f, 136));
  //----------------------------------------------------------------
  // trace point = 575
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6100464f, 40.2770195f, 1489615517, 20.1723881),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6100464f,
                                                40.2770348f, leg_index, 5.14368439f, 405,
                                                maneuver_index, 2.79566932f, 135));
  //----------------------------------------------------------------
  // trace point = 576
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6098175f, 40.2770309f, 1489615518, 19.4024258),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6098175f,
                                                40.2770462f, leg_index, 5.12420559f, 404,
                                                maneuver_index, 2.77619052f, 134));
  //----------------------------------------------------------------
  // trace point = 577
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6095963f, 40.2770386f, 1489615519, 18.8751926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6095963f,
                                                40.2770538f, leg_index, 5.10539627f, 403,
                                                maneuver_index, 2.7573812f, 133));
  //----------------------------------------------------------------
  // trace point = 578
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6093674f, 40.27705f, 1489615520, 19.4871807),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6093674f,
                                                40.2770615f, leg_index, 5.08593988f, 402,
                                                maneuver_index, 2.73792481f, 132));
  //----------------------------------------------------------------
  // trace point = 579
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6091385f, 40.2770576f, 1489615521, 19.4690456),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6091385f,
                                                40.2770729f, leg_index, 5.06646109f, 401,
                                                maneuver_index, 2.71844602f, 131));
  //----------------------------------------------------------------
  // trace point = 580
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6089172f, 40.2770653f, 1489615522, 18.7854042),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6089172f,
                                                40.2770805f, leg_index, 5.04765177f, 400,
                                                maneuver_index, 2.6996367f, 130));
  //----------------------------------------------------------------
  // trace point = 581
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6086884f, 40.2770729f, 1489615523, 19.4690418),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6086884f,
                                                40.2770882f, leg_index, 5.02819538f, 399,
                                                maneuver_index, 2.68018031f, 129));
  //----------------------------------------------------------------
  // trace point = 582
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6084671f, 40.2770805f, 1489615524, 18.7903461),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6084671f,
                                                40.2770996f, leg_index, 5.00936365f, 399,
                                                maneuver_index, 2.66134858f, 129));
  //----------------------------------------------------------------
  // trace point = 583
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6082458f, 40.277092f, 1489615525, 18.8091297),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6082458f,
                                                40.2771072f, leg_index, 4.99055386f, 398,
                                                maneuver_index, 2.64253879f, 128));
  //----------------------------------------------------------------
  // trace point = 584
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6080322f, 40.2770996f, 1489615526, 18.1965313),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6080322f,
                                                40.2771149f, leg_index, 4.97239161f, 397,
                                                maneuver_index, 2.62437654f, 127));
  //----------------------------------------------------------------
  // trace point = 585
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.607811f, 40.2771111f, 1489615527, 18.8091259),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.607811f,
                                                40.2771263f, leg_index, 4.95355988f, 396,
                                                maneuver_index, 2.60554481f, 126));
  //----------------------------------------------------------------
  // trace point = 586
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6075897f, 40.2771149f, 1489615528, 18.7745018),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6075897f,
                                                40.2771339f, leg_index, 4.93475056f, 395,
                                                maneuver_index, 2.58673549f, 125));
  //----------------------------------------------------------------
  // trace point = 587
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6073608f, 40.2771263f, 1489615529, 19.4871578),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6073608f,
                                                40.2771416f, leg_index, 4.91529465f, 394,
                                                maneuver_index, 2.56727958f, 124));
  //----------------------------------------------------------------
  // trace point = 588
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6071243f, 40.2771339f, 1489615530, 20.1477661),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6071243f,
                                                40.277153f, leg_index, 4.89387417f, 393,
                                                maneuver_index, 2.5458591f, 123));
  //----------------------------------------------------------------
  // trace point = 589
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6068878f, 40.2771416f, 1489615531, 20.0629177),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6068878f,
                                                40.2771568f, leg_index, 4.87507963f, 392,
                                                maneuver_index, 2.52706456f, 122));
  //----------------------------------------------------------------
  // trace point = 590
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6066437f, 40.277153f, 1489615532, 20.7587166),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6066437f,
                                                40.2771645f, leg_index, 4.85432816f, 391,
                                                maneuver_index, 2.50631309f, 121));
  //----------------------------------------------------------------
  // trace point = 591
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6064072f, 40.2771606f, 1489615533, 20.1477585),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6064072f,
                                                40.2771721f, leg_index, 4.83422422f, 390,
                                                maneuver_index, 2.48620915f, 120));
  //----------------------------------------------------------------
  // trace point = 592
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.606163f, 40.2771721f, 1489615534, 20.7587109),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.606163f,
                                                40.2771759f, leg_index, 4.81348753f, 389,
                                                maneuver_index, 2.46547246f, 119));
  //----------------------------------------------------------------
  // trace point = 593
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6059113f, 40.2771797f, 1489615535, 21.4205093),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6059113f,
                                                40.2771835f, leg_index, 4.79208803f, 388,
                                                maneuver_index, 2.44407296f, 118));
  //----------------------------------------------------------------
  // trace point = 594
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6056595f, 40.2771873f, 1489615536, 21.4161663),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6056595f,
                                                40.2771912f, leg_index, 4.77068901f, 387,
                                                maneuver_index, 2.42267394f, 117));
  //----------------------------------------------------------------
  // trace point = 595
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6054153f, 40.277195f, 1489615537, 20.7416821),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6054153f,
                                                40.277195f, leg_index, 4.74995232f, 386,
                                                maneuver_index, 2.40193725f, 116));
  //----------------------------------------------------------------
  // trace point = 596
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6051712f, 40.2772064f, 1489615538, 20.7586994),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6051712f,
                                                40.2772026f, leg_index, 4.72920084f, 385,
                                                maneuver_index, 2.38118577f, 115));
  //----------------------------------------------------------------
  // trace point = 597
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6049194f, 40.2772179f, 1489615539, 21.4436245),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6049194f,
                                                40.2772102f, leg_index, 4.70780182f, 384,
                                                maneuver_index, 2.35978675f, 114));
  //----------------------------------------------------------------
  // trace point = 598
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6046829f, 40.2772255f, 1489615540, 20.05826),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6046829f,
                                                40.2772141f, leg_index, 4.68771219f, 383,
                                                maneuver_index, 2.33969712f, 113));
  //----------------------------------------------------------------
  // trace point = 599
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6044464f, 40.2772293f, 1489615541, 20.1329708),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6044464f,
                                                40.2772217f, leg_index, 4.66760826f, 382,
                                                maneuver_index, 2.31959319f, 112));
  //----------------------------------------------------------------
  // trace point = 600
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6042099f, 40.2772369f, 1489615542, 20.0628891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6042099f,
                                                40.2772293f, leg_index, 4.64750481f, 381,
                                                maneuver_index, 2.29948974f, 111));
  //----------------------------------------------------------------
  // trace point = 601
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.603981f, 40.2772408f, 1489615543, 19.4537106),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.603981f,
                                                40.2772331f, leg_index, 4.62806273f, 380,
                                                maneuver_index, 2.28004766f, 110));
  //----------------------------------------------------------------
  // trace point = 602
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6037598f, 40.2772446f, 1489615544, 18.7744675),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6037598f,
                                                40.2772408f, leg_index, 4.60925436f, 379,
                                                maneuver_index, 2.26123929f, 109));
  //----------------------------------------------------------------
  // trace point = 603
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6035385f, 40.2772484f, 1489615545, 18.7721558),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6035385f,
                                                40.2772446f, leg_index, 4.59045982f, 378,
                                                maneuver_index, 2.24244475f, 108));
  //----------------------------------------------------------------
  // trace point = 604
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6033096f, 40.2772484f, 1489615546, 19.4486122),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6033096f,
                                                40.2772522f, leg_index, 4.57100391f, 377,
                                                maneuver_index, 2.22298884f, 107));
  //----------------------------------------------------------------
  // trace point = 605
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.603096f, 40.2772484f, 1489615547, 18.1746845),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.603096f,
                                                40.277256f, leg_index, 4.55285692f, 376,
                                                maneuver_index, 2.20484185f, 106));
  //----------------------------------------------------------------
  // trace point = 606
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6028824f, 40.2772484f, 1489615548, 18.1746845),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6028824f,
                                                40.2772598f, leg_index, 4.53470898f, 375,
                                                maneuver_index, 2.18669391f, 105));
  //----------------------------------------------------------------
  // trace point = 607
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6026764f, 40.2772484f, 1489615549, 17.4952583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6026764f,
                                                40.2772522f, leg_index, 4.51720047f, 374,
                                                maneuver_index, 2.1691854f, 104));
  //----------------------------------------------------------------
  // trace point = 608
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6024704f, 40.2772446f, 1489615550, 17.4984455),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6024704f,
                                                40.2772522f, leg_index, 4.49970675f, 374,
                                                maneuver_index, 2.15169168f, 104));
  //----------------------------------------------------------------
  // trace point = 609
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6022644f, 40.2772408f, 1489615551, 17.5009251),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6022644f,
                                                40.2772484f, leg_index, 4.4822073f, 373,
                                                maneuver_index, 2.13419223f, 103));
  //----------------------------------------------------------------
  // trace point = 610
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.602066f, 40.2772331f, 1489615552, 16.8393974),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.602066f,
                                                40.2772446f, leg_index, 4.46535587f, 372,
                                                maneuver_index, 2.1173408f, 102));
  //----------------------------------------------------------------
  // trace point = 611
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.60186f, 40.2772293f, 1489615553, 17.500927),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.60186f,
                                                40.2772446f, leg_index, 4.44786358f, 371,
                                                maneuver_index, 2.09984851f, 101));
  //----------------------------------------------------------------
  // trace point = 612
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6016617f, 40.2772255f, 1489615554, 16.8217297),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6016617f,
                                                40.2772408f, leg_index, 4.43101168f, 370,
                                                maneuver_index, 2.08299661f, 100));
  //----------------------------------------------------------------
  // trace point = 613
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6014481f, 40.2772179f, 1489615555, 18.1913891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6014481f,
                                                40.2772369f, leg_index, 4.41286469f, 369,
                                                maneuver_index, 2.06484962f, 99));
  //----------------------------------------------------------------
  // trace point = 614
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6012497f, 40.2772141f, 1489615556, 16.8217316),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6012497f,
                                                40.2772331f, leg_index, 4.39601469f, 369,
                                                maneuver_index, 2.04799962f, 99));
  //----------------------------------------------------------------
  // trace point = 615
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6010437f, 40.2772026f, 1489615557, 17.5380688),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6010437f,
                                                40.2772293f, leg_index, 4.37850761f, 368,
                                                maneuver_index, 2.03049254f, 98));
  //----------------------------------------------------------------
  // trace point = 616
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6008377f, 40.2771912f, 1489615558, 17.5461941),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6008377f,
                                                40.2772255f, leg_index, 4.3610096f, 367,
                                                maneuver_index, 2.01299453f, 97));
  //----------------------------------------------------------------
  // trace point = 617
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6006241f, 40.2771835f, 1489615559, 18.1913986),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6006241f,
                                                40.2772179f, leg_index, 4.34284735f, 366,
                                                maneuver_index, 1.99483228f, 96));
  //----------------------------------------------------------------
  // trace point = 618
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6004028f, 40.2771721f, 1489615560, 18.8166809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6004028f,
                                                40.2772102f, leg_index, 4.32403803f, 365,
                                                maneuver_index, 1.97602296f, 95));
  //----------------------------------------------------------------
  // trace point = 619
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.6001816f, 40.2771606f, 1489615561, 18.8091087),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.6001816f,
                                                40.2772026f, leg_index, 4.30522823f, 364,
                                                maneuver_index, 1.95721316f, 94));
  //----------------------------------------------------------------
  // trace point = 620
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5999603f, 40.2771492f, 1489615562, 18.9014015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5999603f,
                                                40.2771988f, leg_index, 4.2864356f, 363,
                                                maneuver_index, 1.93842053f, 93));
  //----------------------------------------------------------------
  // trace point = 621
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5997314f, 40.2771416f, 1489615563, 19.4642448),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5997314f,
                                                40.2771912f, leg_index, 4.26697874f, 362,
                                                maneuver_index, 1.91896367f, 92));
  //----------------------------------------------------------------
  // trace point = 622
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5995026f, 40.2771339f, 1489615564, 19.384182),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5995026f,
                                                40.2771835f, leg_index, 4.2475214f, 361,
                                                maneuver_index, 1.89950633f, 91));
  //----------------------------------------------------------------
  // trace point = 623
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5992737f, 40.2771263f, 1489615565, 19.4690247),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5992737f,
                                                40.2771759f, leg_index, 4.22806454f, 361,
                                                maneuver_index, 1.88004947f, 91));
  //----------------------------------------------------------------
  // trace point = 624
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5990448f, 40.2771187f, 1489615566, 19.4642525),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5990448f,
                                                40.2771645f, leg_index, 4.20858526f, 360,
                                                maneuver_index, 1.86057019f, 90));
  //----------------------------------------------------------------
  // trace point = 625
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5988083f, 40.2771111f, 1489615567, 20.1477699),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5988083f,
                                                40.2771568f, leg_index, 4.18848228f, 359,
                                                maneuver_index, 1.84046721f, 89));
  //----------------------------------------------------------------
  // trace point = 626
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5985794f, 40.2771034f, 1489615568, 19.4690304),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5985794f,
                                                40.2771454f, leg_index, 4.16900301f, 358,
                                                maneuver_index, 1.82098794f, 88));
  //----------------------------------------------------------------
  // trace point = 627
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5983505f, 40.2770958f, 1489615569, 19.3793983),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5983505f,
                                                40.2771378f, leg_index, 4.1495471f, 357,
                                                maneuver_index, 1.80153203f, 87));
  //----------------------------------------------------------------
  // trace point = 628
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5981293f, 40.2770882f, 1489615570, 18.8751774),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5981293f,
                                                40.2771263f, leg_index, 4.13071585f, 356,
                                                maneuver_index, 1.78270078f, 86));
  //----------------------------------------------------------------
  // trace point = 629
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.597908f, 40.2770767f, 1489615571, 18.8091316),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.597908f,
                                                40.2771187f, leg_index, 4.11190557f, 355,
                                                maneuver_index, 1.7638905f, 85));
  //----------------------------------------------------------------
  // trace point = 630
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5976944f, 40.2770729f, 1489615572, 18.1801853),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5976944f,
                                                40.2771111f, leg_index, 4.09374285f, 354,
                                                maneuver_index, 1.74572778f, 84));
  //----------------------------------------------------------------
  // trace point = 631
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5974808f, 40.2770691f, 1489615573, 18.0952835),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5974808f,
                                                40.2771034f, leg_index, 4.07558012f, 354,
                                                maneuver_index, 1.72756505f, 84));
  //----------------------------------------------------------------
  // trace point = 632
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5972824f, 40.2770615f, 1489615574, 16.9242516),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5972824f,
                                                40.2770958f, leg_index, 4.05871201f, 353,
                                                maneuver_index, 1.71069694f, 83));
  //----------------------------------------------------------------
  // trace point = 633
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.597084f, 40.2770538f, 1489615575, 16.8339214),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.597084f,
                                                40.2770882f, leg_index, 4.04184437f, 352,
                                                maneuver_index, 1.6938293f, 82));
  //----------------------------------------------------------------
  // trace point = 634
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5969009f, 40.27705f, 1489615576, 15.5483255),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5969009f,
                                                40.2770844f, leg_index, 4.02629042f, 351,
                                                maneuver_index, 1.67827535f, 81));
  //----------------------------------------------------------------
  // trace point = 635
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5967407f, 40.2770462f, 1489615577, 13.5958834),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5967407f,
                                                40.2770767f, leg_index, 4.0126586f, 351,
                                                maneuver_index, 1.66464353f, 81));
  //----------------------------------------------------------------
  // trace point = 636
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5965881f, 40.2770462f, 1489615578, 12.9940891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5965881f,
                                                40.2770729f, leg_index, 3.99969268f, 350,
                                                maneuver_index, 1.65167761f, 80));
  //----------------------------------------------------------------
  // trace point = 637
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5964432f, 40.2770424f, 1489615579, 12.3227081),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5964432f,
                                                40.2770767f, leg_index, 3.98737431f, 350,
                                                maneuver_index, 1.63935924f, 80));
  //----------------------------------------------------------------
  // trace point = 638
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5962982f, 40.2770424f, 1489615580, 12.3146601),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5962982f,
                                                40.2770805f, leg_index, 3.97505593f, 349,
                                                maneuver_index, 1.62704086f, 79));
  //----------------------------------------------------------------
  // trace point = 639
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5961609f, 40.2770462f, 1489615581, 11.6437483),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5961609f,
                                                40.2770844f, leg_index, 3.96338534f, 349,
                                                maneuver_index, 1.61537027f, 79));
  //----------------------------------------------------------------
  // trace point = 640
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5960159f, 40.2770462f, 1489615582, 12.3146601),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5960159f,
                                                40.2770882f, leg_index, 3.95106792f, 348,
                                                maneuver_index, 1.60305285f, 78));
  //----------------------------------------------------------------
  // trace point = 641
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5958633f, 40.2770462f, 1489615583, 12.9940891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5958633f,
                                                40.2770882f, leg_index, 3.93811584f, 347,
                                                maneuver_index, 1.59010077f, 77));
  //----------------------------------------------------------------
  // trace point = 642
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5957108f, 40.27705f, 1489615584, 12.9168377),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5957108f,
                                                40.277092f, leg_index, 3.92514777f, 347,
                                                maneuver_index, 1.5771327f, 77));
  //----------------------------------------------------------------
  // trace point = 643
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5955505f, 40.2770538f, 1489615585, 13.5958824),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5955505f,
                                                40.2770958f, leg_index, 3.91153407f, 346,
                                                maneuver_index, 1.563519f, 76));
  //----------------------------------------------------------------
  // trace point = 644
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5953751f, 40.2770538f, 1489615586, 14.9474478),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5953751f,
                                                40.2770996f, leg_index, 3.89662528f, 346,
                                                maneuver_index, 1.54861021f, 76));
  //----------------------------------------------------------------
  // trace point = 645
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5951996f, 40.2770576f, 1489615587, 14.8691874),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5951996f,
                                                40.2771072f, leg_index, 3.8817029f, 345,
                                                maneuver_index, 1.53368783f, 75));
  //----------------------------------------------------------------
  // trace point = 646
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5950165f, 40.2770615f, 1489615588, 15.6304436),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5950165f,
                                                40.2771111f, leg_index, 3.86614609f, 344,
                                                maneuver_index, 1.51813102f, 74));
  //----------------------------------------------------------------
  // trace point = 647
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5948257f, 40.2770653f, 1489615589, 16.1425896),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5948257f,
                                                40.2771149f, leg_index, 3.84994173f, 343,
                                                maneuver_index, 1.50192666f, 73));
  //----------------------------------------------------------------
  // trace point = 648
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.594635f, 40.2770691f, 1489615590, 16.2274857),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.594635f,
                                                40.2771187f, leg_index, 3.83373833f, 343,
                                                maneuver_index, 1.48572326f, 73));
  //----------------------------------------------------------------
  // trace point = 649
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5944443f, 40.2770767f, 1489615591, 16.2458),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5944443f,
                                                40.2771225f, leg_index, 3.81753159f, 341,
                                                maneuver_index, 1.46951652f, 71));
  //----------------------------------------------------------------
  // trace point = 650
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5942535f, 40.2770805f, 1489615592, 16.1425858),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5942535f,
                                                40.2771263f, leg_index, 3.80132771f, 341,
                                                maneuver_index, 1.45331264f, 71));
  //----------------------------------------------------------------
  // trace point = 651
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5940628f, 40.2770882f, 1489615593, 16.2400761),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5940628f,
                                                40.2771301f, leg_index, 3.78512383f, 340,
                                                maneuver_index, 1.43710876f, 70));
  //----------------------------------------------------------------
  // trace point = 652
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5938797f, 40.2770958f, 1489615594, 15.5674334),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5938797f,
                                                40.2771301f, leg_index, 3.76957536f, 339,
                                                maneuver_index, 1.42156029f, 69));
  //----------------------------------------------------------------
  // trace point = 653
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.593689f, 40.2770996f, 1489615595, 16.227478),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.593689f,
                                                40.2771339f, leg_index, 3.75337148f, 338,
                                                maneuver_index, 1.40535641f, 68));
  //----------------------------------------------------------------
  // trace point = 654
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5935059f, 40.2771072f, 1489615596, 15.5614586),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5935059f,
                                                40.2771378f, leg_index, 3.73781562f, 338,
                                                maneuver_index, 1.38980055f, 68));
  //----------------------------------------------------------------
  // trace point = 655
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5933151f, 40.2771149f, 1489615597, 16.2457905),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5933151f,
                                                40.2771378f, leg_index, 3.72161984f, 337,
                                                maneuver_index, 1.37360477f, 67));
  //----------------------------------------------------------------
  // trace point = 656
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.593132f, 40.2771187f, 1489615598, 15.5483112),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.593132f,
                                                40.2771416f, leg_index, 3.70606303f, 336,
                                                maneuver_index, 1.35804796f, 66));
  //----------------------------------------------------------------
  // trace point = 657
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5929489f, 40.2771301f, 1489615599, 15.5900965),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5929489f,
                                                40.2771454f, leg_index, 3.69050622f, 335,
                                                maneuver_index, 1.34249115f, 65));
  //----------------------------------------------------------------
  // trace point = 658
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5927658f, 40.2771378f, 1489615600, 15.5674229),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5927658f,
                                                40.2771492f, leg_index, 3.67494965f, 335,
                                                maneuver_index, 1.32693458f, 65));
  //----------------------------------------------------------------
  // trace point = 659
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5925903f, 40.2771416f, 1489615601, 14.9540596),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5925903f,
                                                40.277153f, leg_index, 3.66004062f, 334,
                                                maneuver_index, 1.31202555f, 64));
  //----------------------------------------------------------------
  // trace point = 660
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5924149f, 40.2771492f, 1489615602, 14.8829117),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5924149f,
                                                40.2771568f, leg_index, 3.64513183f, 333,
                                                maneuver_index, 1.29711676f, 63));
  //----------------------------------------------------------------
  // trace point = 661
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5922318f, 40.2771492f, 1489615603, 15.5419264),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5922318f,
                                                40.2771606f, leg_index, 3.62957501f, 332,
                                                maneuver_index, 1.28155994f, 62));
  //----------------------------------------------------------------
  // trace point = 662
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5920486f, 40.2771568f, 1489615604, 15.5674191),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5920486f,
                                                40.2771645f, leg_index, 3.61401844f, 332,
                                                maneuver_index, 1.26600337f, 62));
  //----------------------------------------------------------------
  // trace point = 663
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5918655f, 40.2771606f, 1489615605, 15.5483017),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5918655f,
                                                40.2771683f, leg_index, 3.5984621f, 331,
                                                maneuver_index, 1.25044703f, 61));
  //----------------------------------------------------------------
  // trace point = 664
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5916748f, 40.2771645f, 1489615606, 16.2247887),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5916748f,
                                                40.2771721f, leg_index, 3.58225822f, 330,
                                                maneuver_index, 1.23424315f, 60));
  //----------------------------------------------------------------
  // trace point = 665
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5914917f, 40.2771683f, 1489615607, 15.5482998),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5914917f,
                                                40.2771721f, leg_index, 3.56671381f, 329,
                                                maneuver_index, 1.21869874f, 59));
  //----------------------------------------------------------------
  // trace point = 666
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5912857f, 40.2771683f, 1489615608, 17.4952774),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5912857f,
                                                40.2771797f, leg_index, 3.54919934f, 329,
                                                maneuver_index, 1.20118427f, 59));
  //----------------------------------------------------------------
  // trace point = 667
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.591095f, 40.2771759f, 1489615609, 16.2457771),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.591095f,
                                                40.2771835f, leg_index, 3.53299499f, 328,
                                                maneuver_index, 1.18497992f, 58));
  //----------------------------------------------------------------
  // trace point = 668
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5908966f, 40.2771759f, 1489615610, 16.8158474),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5908966f,
                                                40.2771873f, leg_index, 3.51614308f, 328,
                                                maneuver_index, 1.16812801f, 58));
  //----------------------------------------------------------------
  // trace point = 669
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5907059f, 40.2771797f, 1489615611, 16.227459),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5907059f,
                                                40.2771912f, leg_index, 3.49993896f, 327,
                                                maneuver_index, 1.15192389f, 57));
  //----------------------------------------------------------------
  // trace point = 670
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5904999f, 40.2771797f, 1489615612, 17.4952755),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5904999f,
                                                40.2771988f, leg_index, 3.4824264f, 326,
                                                maneuver_index, 1.13441133f, 56));
  //----------------------------------------------------------------
  // trace point = 671
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5902939f, 40.2771835f, 1489615613, 17.5009403),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5902939f,
                                                40.2772026f, leg_index, 3.46492648f, 325,
                                                maneuver_index, 1.11691141f, 55));
  //----------------------------------------------------------------
  // trace point = 672
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5900879f, 40.2771873f, 1489615614, 17.4984608),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5900879f,
                                                40.2772064f, leg_index, 3.44742703f, 324,
                                                maneuver_index, 1.09941196f, 54));
  //----------------------------------------------------------------
  // trace point = 673
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5898819f, 40.277195f, 1489615615, 17.5179234),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5898819f,
                                                40.2772102f, leg_index, 3.42992854f, 323,
                                                maneuver_index, 1.08191347f, 53));
  //----------------------------------------------------------------
  // trace point = 674
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5896835f, 40.2772026f, 1489615616, 16.839407),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5896835f,
                                                40.2772141f, leg_index, 3.41307211f, 322,
                                                maneuver_index, 1.06505704f, 52));
  //----------------------------------------------------------------
  // trace point = 675
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5894928f, 40.2772026f, 1489615617, 16.2213421),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5894928f,
                                                40.2772179f, leg_index, 3.39686966f, 321,
                                                maneuver_index, 1.04885459f, 51));
  //----------------------------------------------------------------
  // trace point = 676
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5893021f, 40.2772026f, 1489615618, 16.2213421),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5893021f,
                                                40.2772179f, leg_index, 3.38067174f, 321,
                                                maneuver_index, 1.03265667f, 51));
  //----------------------------------------------------------------
  // trace point = 677
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5891266f, 40.2772102f, 1489615619, 14.8828983),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5891266f,
                                                40.2772179f, leg_index, 3.36576962f, 320,
                                                maneuver_index, 1.01775455f, 50));
  //----------------------------------------------------------------
  // trace point = 678
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5889587f, 40.2772102f, 1489615620, 14.2679853),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5889587f,
                                                40.2772217f, leg_index, 3.35151076f, 319,
                                                maneuver_index, 1.00349569f, 49));
  //----------------------------------------------------------------
  // trace point = 679
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5887909f, 40.2772217f, 1489615621, 14.3303804),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5887909f,
                                                40.2772217f, leg_index, 3.33725667f, 319,
                                                maneuver_index, 0.9892416f, 49));
  //----------------------------------------------------------------
  // trace point = 680
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.588623f, 40.2772179f, 1489615622, 14.274929),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.588623f,
                                                40.2772217f, leg_index, 3.32300258f, 318,
                                                maneuver_index, 0.974987507f, 48));
  //----------------------------------------------------------------
  // trace point = 681
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5884705f, 40.2772217f, 1489615623, 13.0016832),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5884705f,
                                                40.2772255f, leg_index, 3.31003952f, 317,
                                                maneuver_index, 0.96202445f, 47));
  //----------------------------------------------------------------
  // trace point = 682
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5883026f, 40.2772217f, 1489615624, 14.183054),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5883026f,
                                                40.2772255f, leg_index, 3.29578519f, 317,
                                                maneuver_index, 0.947770119f, 47));
  //----------------------------------------------------------------
  // trace point = 683
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5881424f, 40.2772217f, 1489615625, 13.6734829),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5881424f,
                                                40.2772255f, leg_index, 3.28217912f, 316,
                                                maneuver_index, 0.934164047f, 46));
  //----------------------------------------------------------------
  // trace point = 684
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5879669f, 40.2772255f, 1489615626, 14.8662329),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5879669f,
                                                40.2772293f, leg_index, 3.26727247f, 315,
                                                maneuver_index, 0.919257402f, 45));
  //----------------------------------------------------------------
  // trace point = 685
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5877914f, 40.2772255f, 1489615627, 14.9474096),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5877914f,
                                                40.2772293f, leg_index, 3.25237036f, 315,
                                                maneuver_index, 0.904355288f, 45));
  //----------------------------------------------------------------
  // trace point = 686
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5876236f, 40.2772217f, 1489615628, 14.186985),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5876236f,
                                                40.2772331f, leg_index, 3.23811245f, 314,
                                                maneuver_index, 0.89009738f, 44));
  //----------------------------------------------------------------
  // trace point = 687
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5874481f, 40.2772179f, 1489615629, 14.9540405),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5874481f,
                                                40.2772331f, leg_index, 3.22320962f, 313,
                                                maneuver_index, 0.87519455f, 43));
  //----------------------------------------------------------------
  // trace point = 688
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5872803f, 40.2772179f, 1489615630, 14.2679834),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5872803f,
                                                40.2772408f, leg_index, 3.20892286f, 312,
                                                maneuver_index, 0.860907793f, 42));
  //----------------------------------------------------------------
  // trace point = 689
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5871201f, 40.2772217f, 1489615631, 13.595849),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5871201f,
                                                40.2772522f, leg_index, 3.19525623f, 312,
                                                maneuver_index, 0.847241163f, 42));
  //----------------------------------------------------------------
  // trace point = 690
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5869675f, 40.2772293f, 1489615632, 13.0173988),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5869751f,
                                                40.2772636f, leg_index, 3.1828804f, 311,
                                                maneuver_index, 0.834865332f, 41));
  //----------------------------------------------------------------
  // trace point = 691
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5868225f, 40.2772369f, 1489615633, 12.2620792),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5868301f,
                                                40.2772751f, leg_index, 3.17050457f, 310,
                                                maneuver_index, 0.8224895f, 40));
  //----------------------------------------------------------------
  // trace point = 692
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5866928f, 40.2772446f, 1489615634, 11.0765562),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5867004f,
                                                40.2772827f, leg_index, 3.15945768f, 310,
                                                maneuver_index, 0.811442614f, 40));
  //----------------------------------------------------------------
  // trace point = 693
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5865707f, 40.2772522f, 1489615635, 10.3905296),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5865784f,
                                                40.2772903f, leg_index, 3.14905596f, 309,
                                                maneuver_index, 0.801040888f, 39));
  //----------------------------------------------------------------
  // trace point = 694
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5863647f, 40.2772675f, 1489615637, 8.78737926),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5863724f,
                                                40.2773056f, leg_index, 3.13147926f, 309,
                                                maneuver_index, 0.783464193f, 39));
  //----------------------------------------------------------------
  // trace point = 695
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5861816f, 40.2772789f, 1489615639, 7.79960012),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5861893f,
                                                40.277317f, leg_index, 3.11587691f, 308,
                                                maneuver_index, 0.767861843f, 38));
  //----------------------------------------------------------------
  // trace point = 696
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5860367f, 40.2772865f, 1489615641, 6.173388),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5860443f,
                                                40.2773285f, leg_index, 3.10350108f, 307,
                                                maneuver_index, 0.755486012f, 37));
  //----------------------------------------------------------------
  // trace point = 697
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5859146f, 40.2772942f, 1489615643, 5.195261),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5859222f,
                                                40.2773361f, leg_index, 3.09309936f, 307,
                                                maneuver_index, 0.745084286f, 37));
  //----------------------------------------------------------------
  // trace point = 698
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5857773f, 40.2773094f, 1489615646, 3.94615602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5857849f,
                                                40.2773476f, leg_index, 3.08136988f, 306,
                                                maneuver_index, 0.733354807f, 36));
  //----------------------------------------------------------------
  // trace point = 699
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5856552f, 40.277317f, 1489615648, 5.19972992),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5856628f,
                                                40.2773552f, leg_index, 3.07096767f, 306,
                                                maneuver_index, 0.722952604f, 36));
  //----------------------------------------------------------------
  // trace point = 700
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5854721f, 40.2773361f, 1489615650, 7.84257221),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5854797f,
                                                40.2773666f, leg_index, 3.05536485f, 305,
                                                maneuver_index, 0.707349777f, 35));
  //----------------------------------------------------------------
  // trace point = 701
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5852661f, 40.2773552f, 1489615652, 8.81130886),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5852737f,
                                                40.2773895f, leg_index, 3.03768158f, 305,
                                                maneuver_index, 0.68966651f, 35));
  //----------------------------------------------------------------
  // trace point = 702
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.585022f, 40.2773781f, 1489615654, 10.4400368),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5850296f,
                                                40.27742f, leg_index, 3.01667166f, 304,
                                                maneuver_index, 0.668656588f, 34));
  //----------------------------------------------------------------
  // trace point = 703
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5848999f, 40.2773895f, 1489615655, 10.4470053),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5849075f,
                                                40.2774315f, leg_index, 3.00623226f, 303,
                                                maneuver_index, 0.658217192f, 33));
  //----------------------------------------------------------------
  // trace point = 704
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5847702f, 40.2774048f, 1489615656, 11.1662283),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5847778f,
                                                40.2774467f, leg_index, 2.99508786f, 303,
                                                maneuver_index, 0.647072792f, 33));
  //----------------------------------------------------------------
  // trace point = 705
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5846481f, 40.27742f, 1489615657, 10.4949312),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5846558f,
                                                40.277462f, leg_index, 2.98458409f, 302,
                                                maneuver_index, 0.636569023f, 32));
  //----------------------------------------------------------------
  // trace point = 706
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5845184f, 40.2774391f, 1489615658, 11.2414341),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5845261f,
                                                40.2774773f, leg_index, 2.97343445f, 301,
                                                maneuver_index, 0.625419378f, 31));
  //----------------------------------------------------------------
  // trace point = 707
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5843964f, 40.2774544f, 1489615659, 10.4949265),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.584404f,
                                                40.2774925f, leg_index, 2.96293163f, 301,
                                                maneuver_index, 0.614916563f, 31));
  //----------------------------------------------------------------
  // trace point = 708
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5842743f, 40.2774696f, 1489615660, 10.5132093),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5842819f,
                                                40.277504f, leg_index, 2.95248818f, 300,
                                                maneuver_index, 0.604473114f, 30));
  //----------------------------------------------------------------
  // trace point = 709
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5841522f, 40.2774849f, 1489615661, 10.4949217),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5841599f,
                                                40.2775154f, leg_index, 2.94204473f, 300,
                                                maneuver_index, 0.594029665f, 30));
  //----------------------------------------------------------------
  // trace point = 710
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5840302f, 40.277504f, 1489615662, 10.5749063),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5840378f,
                                                40.2775307f, leg_index, 2.93154168f, 299,
                                                maneuver_index, 0.583526611f, 29));
  //----------------------------------------------------------------
  // trace point = 711
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5838013f, 40.2775307f, 1489615664, 9.83970642),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5838089f,
                                                40.2775536f, leg_index, 2.91193819f, 298,
                                                maneuver_index, 0.56392312f, 28));
  //----------------------------------------------------------------
  // trace point = 712
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5836792f, 40.2775459f, 1489615665, 10.4949121),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5836868f,
                                                40.2775688f, leg_index, 2.9014349f, 298,
                                                maneuver_index, 0.553419828f, 28));
  //----------------------------------------------------------------
  // trace point = 713
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5835571f, 40.2775574f, 1489615666, 10.4333296),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5835648f,
                                                40.2775803f, leg_index, 2.89099193f, 297,
                                                maneuver_index, 0.542976856f, 27));
  //----------------------------------------------------------------
  // trace point = 714
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5834274f, 40.2775688f, 1489615667, 11.1211643),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5834351f,
                                                40.2775955f, leg_index, 2.87984705f, 297,
                                                maneuver_index, 0.53183198f, 27));
  //----------------------------------------------------------------
  // trace point = 715
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5833054f, 40.2775803f, 1489615668, 10.4333258),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.583313f,
                                                40.277607f, leg_index, 2.86940503f, 296,
                                                maneuver_index, 0.521389961f, 26));
  //----------------------------------------------------------------
  // trace point = 716
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5831757f, 40.2775917f, 1489615669, 11.0368519),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5831757f,
                                                40.2776222f, leg_index, 2.8576169f, 296,
                                                maneuver_index, 0.509601831f, 26));
  //----------------------------------------------------------------
  // trace point = 717
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5830536f, 40.2776031f, 1489615670, 10.4333229),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5830536f,
                                                40.2776299f, leg_index, 2.8472147f, 295,
                                                maneuver_index, 0.499199629f, 25));
  //----------------------------------------------------------------
  // trace point = 718
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5829239f, 40.2776146f, 1489615671, 11.1211567),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5829239f,
                                                40.2776375f, leg_index, 2.8361671f, 295,
                                                maneuver_index, 0.488152027f, 25));
  //----------------------------------------------------------------
  // trace point = 719
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5828018f, 40.2776222f, 1489615672, 10.3904734),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5828018f,
                                                40.2776451f, leg_index, 2.82576537f, 294,
                                                maneuver_index, 0.477750301f, 24));
  //----------------------------------------------------------------
  // trace point = 720
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5826721f, 40.2776299f, 1489615673, 11.0764933),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5826721f,
                                                40.2776527f, leg_index, 2.81471801f, 294,
                                                maneuver_index, 0.466702938f, 24));
  //----------------------------------------------------------------
  // trace point = 721
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5825348f, 40.2776375f, 1489615674, 11.6611938),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5825348f,
                                                40.2776604f, leg_index, 2.80303288f, 293,
                                                maneuver_index, 0.455017805f, 23));
  //----------------------------------------------------------------
  // trace point = 722 | Alert 0.4 | Turn left onto Hinkle Road.
  maneuver_index = 9;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5823975f, 40.2776489f, 1489615675, 11.7959375),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5823975f, 40.2776718f, leg_index, 2.79129553f,
                                                292, maneuver_index, 0.443280458f, 22,
                                                instruction_index, 0.4f));
  //----------------------------------------------------------------
  // trace point = 723
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5822601f, 40.2776566f, 1489615676, 11.669158),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5822601f,
                                                40.2776833f, leg_index, 2.77956295f, 292,
                                                maneuver_index, 0.43154788f, 22));
  //----------------------------------------------------------------
  // trace point = 724
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5821228f, 40.277668f, 1489615677, 11.6993828),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5821228f,
                                                40.2776947f, leg_index, 2.7671864f, 291,
                                                maneuver_index, 0.419171333f, 21));
  //----------------------------------------------------------------
  // trace point = 725
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5819778f, 40.2776794f, 1489615678, 12.3752766),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5819855f,
                                                40.2777061f, leg_index, 2.75609803f, 290,
                                                maneuver_index, 0.408082962f, 20));
  //----------------------------------------------------------------
  // trace point = 726
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5818329f, 40.2776909f, 1489615679, 12.3867846),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5818405f,
                                                40.2777176f, leg_index, 2.74372149f, 290,
                                                maneuver_index, 0.395706415f, 20));
  //----------------------------------------------------------------
  // trace point = 727
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5816727f, 40.2777023f, 1489615680, 13.6435213),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5816803f,
                                                40.2777328f, leg_index, 2.73001194f, 289,
                                                maneuver_index, 0.38199687f, 19));
  //----------------------------------------------------------------
  // trace point = 728
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5815277f, 40.2777176f, 1489615681, 12.4426765),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5815277f,
                                                40.2777405f, leg_index, 2.71702671f, 288,
                                                maneuver_index, 0.369011641f, 18));
  //----------------------------------------------------------------
  // trace point = 729
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5813675f, 40.2777252f, 1489615682, 13.6955681),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5813675f,
                                                40.2777519f, leg_index, 2.70336175f, 288,
                                                maneuver_index, 0.35534668f, 18));
  //----------------------------------------------------------------
  // trace point = 730
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5811996f, 40.2777328f, 1489615683, 14.2108793),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5811996f,
                                                40.2777596f, leg_index, 2.68908429f, 287,
                                                maneuver_index, 0.341069221f, 17));
  //----------------------------------------------------------------
  // trace point = 731
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5810318f, 40.2777405f, 1489615684, 14.2891369),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5810318f,
                                                40.277771f, leg_index, 2.67477345f, 286,
                                                maneuver_index, 0.326758385f, 16));
  //----------------------------------------------------------------
  // trace point = 732
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5808563f, 40.2777481f, 1489615685, 14.9738007),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5808563f,
                                                40.2777824f, leg_index, 2.65981722f, 286,
                                                maneuver_index, 0.311802149f, 16));
  //----------------------------------------------------------------
  // trace point = 733
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5806656f, 40.2777596f, 1489615686, 16.1914806),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5806656f,
                                                40.2777939f, leg_index, 2.64356804f, 285,
                                                maneuver_index, 0.295552969f, 15));
  //----------------------------------------------------------------
  // trace point = 734
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5804749f, 40.277771f, 1489615687, 16.2673588),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5804749f,
                                                40.2778015f, leg_index, 2.62735105f, 284,
                                                maneuver_index, 0.279335976f, 14));
  //----------------------------------------------------------------
  // trace point = 735
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5802765f, 40.2777824f, 1489615688, 16.9449291),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5802765f,
                                                40.277813f, leg_index, 2.61045623f, 283,
                                                maneuver_index, 0.262441158f, 13));
  //----------------------------------------------------------------
  // trace point = 736
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5800781f, 40.2777939f, 1489615689, 16.8686714),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5800781f,
                                                40.2778244f, leg_index, 2.59356141f, 282,
                                                maneuver_index, 0.245546341f, 12));
  //----------------------------------------------------------------
  // trace point = 737
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5798798f, 40.2778053f, 1489615690, 16.860218),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5798798f,
                                                40.2778358f, leg_index, 2.57666659f, 281,
                                                maneuver_index, 0.228651524f, 11));
  //----------------------------------------------------------------
  // trace point = 738
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5796738f, 40.2778206f, 1489615691, 17.5855427),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5796738f,
                                                40.2778473f, leg_index, 2.5591259f, 281,
                                                maneuver_index, 0.21111083f, 11));
  //----------------------------------------------------------------
  // trace point = 739
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5794678f, 40.2778358f, 1489615692, 17.5746117),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5794678f,
                                                40.2778587f, leg_index, 2.54158497f, 280,
                                                maneuver_index, 0.193569899f, 10));
  //----------------------------------------------------------------
  // trace point = 740
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5792618f, 40.2778473f, 1489615693, 17.5379047),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5792618f,
                                                40.2778702f, leg_index, 2.52404404f, 279,
                                                maneuver_index, 0.176028967f, 9));
  //----------------------------------------------------------------
  // trace point = 741
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5790558f, 40.2778587f, 1489615694, 17.5460262),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5790558f,
                                                40.2778854f, leg_index, 2.50647163f, 278,
                                                maneuver_index, 0.158456564f, 8));
  //----------------------------------------------------------------
  // trace point = 742
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5788498f, 40.2778702f, 1489615695, 17.537899),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5788498f,
                                                40.2778969f, leg_index, 2.48893094f, 277,
                                                maneuver_index, 0.140915871f, 7));
  //----------------------------------------------------------------
  // trace point = 743
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5786514f, 40.2778854f, 1489615696, 16.9828892),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5786514f,
                                                40.2779083f, leg_index, 2.47203636f, 276,
                                                maneuver_index, 0.124021292f, 6));
  //----------------------------------------------------------------
  // trace point = 744 | Pre | Turn left onto Hinkle Road.
  maneuver_index = 9;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5784454f, 40.2778893f, 1489615697, 17.5007572),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5784454f, 40.2779198f, leg_index, 2.45449615f,
                                                276, maneuver_index, 0.106481075f, 6,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 745
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5782394f, 40.2779007f, 1489615698, 17.5460148),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5782394f,
                                                40.2779312f, leg_index, 2.43695569f, 275,
                                                maneuver_index, 0.0889406204f, 5));
  //----------------------------------------------------------------
  // trace point = 746
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5780411f, 40.2779121f, 1489615699, 16.8601913),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5780411f,
                                                40.2779388f, leg_index, 2.42008901f, 274,
                                                maneuver_index, 0.0720739365f, 4));
  //----------------------------------------------------------------
  // trace point = 747
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5778427f, 40.2779236f, 1489615700, 16.868639),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5778427f,
                                                40.2779503f, leg_index, 2.40319562f, 273,
                                                maneuver_index, 0.0551805496f, 3));
  //----------------------------------------------------------------
  // trace point = 748
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5776596f, 40.277935f, 1489615701, 15.5899115),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5776596f,
                                                40.2779579f, leg_index, 2.38762164f, 272,
                                                maneuver_index, 0.0396065712f, 2));
  //----------------------------------------------------------------
  // trace point = 749
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.577507f, 40.2779465f, 1489615702, 13.0514879),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.577507f,
                                                40.2779655f, leg_index, 2.37463522f, 271,
                                                maneuver_index, 0.0266201496f, 1));
  //----------------------------------------------------------------
  // trace point = 750
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5773849f, 40.2779617f, 1489615703, 10.513135),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5773849f,
                                                40.2779732f, leg_index, 2.36423612f, 271,
                                                maneuver_index, 0.0162210464f, 1));
  //----------------------------------------------------------------
  // trace point = 751
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5772552f, 40.2779694f, 1489615705, 5.53402185),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5772552f,
                                                40.277977f, leg_index, 2.35321736f, 270,
                                                maneuver_index, 0.0052022934f, 0));
  //----------------------------------------------------------------
  // trace point = 752 | Post | Continue for 700 meters.
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5771637f, 40.2780418f, 1489615710, 2.24290299),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.5771713f, 40.2780457f, leg_index, 2.34054255f,
                                                268, maneuver_index, 0.692618966f, 103,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 753
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5771179f, 40.2781792f, 1489615712, 7.87159395),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5771332f,
                                                40.278183f, leg_index, 2.32492304f, 266,
                                                maneuver_index, 0.67699945f, 101));
  //----------------------------------------------------------------
  // trace point = 754
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5770493f, 40.278347f, 1489615714, 9.79913139),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5770645f,
                                                40.2783508f, leg_index, 2.30533981f, 263,
                                                maneuver_index, 0.657416224f, 98));
  //----------------------------------------------------------------
  // trace point = 755
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5770111f, 40.2784309f, 1489615715, 9.89207458),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5770264f,
                                                40.2784348f, leg_index, 2.29545212f, 261,
                                                maneuver_index, 0.647528529f, 96));
  //----------------------------------------------------------------
  // trace point = 756
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.576973f, 40.2785263f, 1489615716, 11.0568085),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5769882f,
                                                40.2785301f, leg_index, 2.28435755f, 260,
                                                maneuver_index, 0.636433959f, 95));
  //----------------------------------------------------------------
  // trace point = 757
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5769348f, 40.2786255f, 1489615717, 11.4834318),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5769424f,
                                                40.2786293f, leg_index, 2.2726531f, 258,
                                                maneuver_index, 0.624729514f, 93));
  //----------------------------------------------------------------
  // trace point = 758
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5768967f, 40.2787323f, 1489615718, 12.3406334),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5768967f,
                                                40.2787323f, leg_index, 2.26054597f, 256,
                                                maneuver_index, 0.61262238f, 91));
  //----------------------------------------------------------------
  // trace point = 759
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5768585f, 40.2788429f, 1489615719, 12.7709455),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5768509f,
                                                40.2788391f, leg_index, 2.24803805f, 254,
                                                maneuver_index, 0.600114465f, 89));
  //----------------------------------------------------------------
  // trace point = 760
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5768051f, 40.2789574f, 1489615720, 13.4936438),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5767975f,
                                                40.2789536f, leg_index, 2.23229694f, 252,
                                                maneuver_index, 0.584373355f, 87));
  //----------------------------------------------------------------
  // trace point = 761
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5767593f, 40.2790756f, 1489615721, 13.8110495),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5767517f,
                                                40.2790718f, leg_index, 2.22079253f, 250,
                                                maneuver_index, 0.572868943f, 85));
  //----------------------------------------------------------------
  // trace point = 762
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5767136f, 40.2791901f, 1489615722, 13.253377),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5766983f,
                                                40.2791862f, leg_index, 2.20728016f, 248,
                                                maneuver_index, 0.55935657f, 83));
  //----------------------------------------------------------------
  // trace point = 763
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5766525f, 40.2793159f, 1489615723, 14.9523659),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5766525f,
                                                40.2793159f, leg_index, 2.19233966f, 246,
                                                maneuver_index, 0.54441607f, 81));
  //----------------------------------------------------------------
  // trace point = 764
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5765991f, 40.2794418f, 1489615724, 14.7569361),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5765991f,
                                                40.2794418f, leg_index, 2.17761087f, 243,
                                                maneuver_index, 0.529687285f, 78));
  //----------------------------------------------------------------
  // trace point = 765
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5765457f, 40.2795639f, 1489615725, 14.3074274),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5765457f,
                                                40.2795639f, leg_index, 2.16328788f, 241,
                                                maneuver_index, 0.515364289f, 76));
  //----------------------------------------------------------------
  // trace point = 766
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5764923f, 40.279686f, 1489615726, 14.3343687),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5764999f,
                                                40.279686f, leg_index, 2.14915848f, 239,
                                                maneuver_index, 0.501234889f, 74));
  //----------------------------------------------------------------
  // trace point = 767
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5764465f, 40.279808f, 1489615727, 14.1084328),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5764465f,
                                                40.279808f, leg_index, 2.13483477f, 237,
                                                maneuver_index, 0.486911178f, 72));
  //----------------------------------------------------------------
  // trace point = 768
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5763931f, 40.2799301f, 1489615728, 14.334363),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5763931f,
                                                40.2799301f, leg_index, 2.12051606f, 235,
                                                maneuver_index, 0.472592473f, 70));
  //----------------------------------------------------------------
  // trace point = 769
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5763397f, 40.280056f, 1489615729, 14.7307529),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5763474f,
                                                40.280056f, leg_index, 2.10597253f, 233,
                                                maneuver_index, 0.45804894f, 68));
  //----------------------------------------------------------------
  // trace point = 770
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5762787f, 40.2801781f, 1489615730, 14.5354624),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5763092f,
                                                40.2801781f, leg_index, 2.09200287f, 231,
                                                maneuver_index, 0.44407928f, 66));
  //----------------------------------------------------------------
  // trace point = 771
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5762253f, 40.280304f, 1489615731, 14.756918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5762482f,
                                                40.2803116f, leg_index, 2.07626462f, 228,
                                                maneuver_index, 0.428341031f, 63));
  //----------------------------------------------------------------
  // trace point = 772
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5761719f, 40.2804375f, 1489615732, 15.4745464),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5761948f,
                                                40.2804413f, leg_index, 2.06113219f, 226,
                                                maneuver_index, 0.413208604f, 61));
  //----------------------------------------------------------------
  // trace point = 773
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5761185f, 40.2805634f, 1489615733, 14.7569122),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5761414f,
                                                40.2805672f, leg_index, 2.04640341f, 224,
                                                maneuver_index, 0.398479819f, 59));
  //----------------------------------------------------------------
  // trace point = 774
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5760651f, 40.2806969f, 1489615734, 15.5810814),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5760803f,
                                                40.2807007f, leg_index, 2.03066492f, 222,
                                                maneuver_index, 0.382741332f, 57));
  //----------------------------------------------------------------
  // trace point = 775
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5760117f, 40.2808189f, 1489615735, 14.3074007),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5760269f,
                                                40.2808228f, leg_index, 2.01633954f, 219,
                                                maneuver_index, 0.368415952f, 54));
  //----------------------------------------------------------------
  // trace point = 776
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5759583f, 40.2809334f, 1489615736, 13.493598),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5759811f,
                                                40.2809372f, leg_index, 2.00302386f, 217,
                                                maneuver_index, 0.355100274f, 52));
  //----------------------------------------------------------------
  // trace point = 777
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5759125f, 40.2810516f, 1489615737, 13.811017),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5759277f,
                                                40.2810555f, leg_index, 1.98688126f, 215,
                                                maneuver_index, 0.338957667f, 50));
  //----------------------------------------------------------------
  // trace point = 778
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5758514f, 40.2811661f, 1489615738, 13.7070312),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5758667f,
                                                40.2811699f, leg_index, 1.97533727f, 213,
                                                maneuver_index, 0.327413678f, 48));
  //----------------------------------------------------------------
  // trace point = 779
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.575798f, 40.2812805f, 1489615739, 13.569932),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5758057f,
                                                40.2812843f, leg_index, 1.96158385f, 211,
                                                maneuver_index, 0.313660264f, 46));
  //----------------------------------------------------------------
  // trace point = 780
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5757294f, 40.2813911f, 1489615740, 13.5749664),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5757446f,
                                                40.281395f, leg_index, 1.94822299f, 209,
                                                maneuver_index, 0.300299406f, 44));
  //----------------------------------------------------------------
  // trace point = 781
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5756531f, 40.2814941f, 1489615741, 13.15763),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5756683f,
                                                40.2815018f, leg_index, 1.9346385f, 207,
                                                maneuver_index, 0.286714911f, 42));
  //----------------------------------------------------------------
  // trace point = 782
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5755615f, 40.2815819f, 1489615742, 12.4774122),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5755768f,
                                                40.2815933f, leg_index, 1.92173564f, 205,
                                                maneuver_index, 0.273812056f, 40));
  //----------------------------------------------------------------
  // trace point = 783
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5754471f, 40.2816658f, 1489615743, 13.5209322),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5754471f,
                                                40.2816696f, leg_index, 1.90781033f, 204,
                                                maneuver_index, 0.259886742f, 39));
  //----------------------------------------------------------------
  // trace point = 784
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5753174f, 40.2817345f, 1489615744, 13.3858757),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.575325f,
                                                40.2817383f, leg_index, 1.89493191f, 202,
                                                maneuver_index, 0.247008324f, 37));
  //----------------------------------------------------------------
  // trace point = 785
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5751724f, 40.2818108f, 1489615745, 15.0033989),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5751801f,
                                                40.2818184f, leg_index, 1.87973189f, 199,
                                                maneuver_index, 0.231808305f, 34));
  //----------------------------------------------------------------
  // trace point = 786
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5750351f, 40.2818832f, 1489615746, 14.1279736),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5750427f,
                                                40.2818909f, leg_index, 1.86555099f, 197,
                                                maneuver_index, 0.217627406f, 32));
  //----------------------------------------------------------------
  // trace point = 787
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5748901f, 40.2819672f, 1489615747, 15.4617958),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5748901f,
                                                40.2819672f, leg_index, 1.85004544f, 195,
                                                maneuver_index, 0.202121854f, 30));
  //----------------------------------------------------------------
  // trace point = 788
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5747299f, 40.2820473f, 1489615748, 16.2459736),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5747299f,
                                                40.2820511f, leg_index, 1.83353853f, 193,
                                                maneuver_index, 0.185614944f, 28));
  //----------------------------------------------------------------
  // trace point = 789
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5745697f, 40.2821312f, 1489615749, 16.4942532),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5745697f,
                                                40.282135f, leg_index, 1.81703436f, 190,
                                                maneuver_index, 0.169110775f, 25));
  //----------------------------------------------------------------
  // trace point = 790
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5744095f, 40.2822151f, 1489615750, 16.5642662),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5744095f,
                                                40.2822189f, leg_index, 1.8005302f, 188,
                                                maneuver_index, 0.152606606f, 23));
  //----------------------------------------------------------------
  // trace point = 791 | Pre | Turn left onto South Forge Road, Pennsylvania 1 17.
  maneuver_index = 10;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5742493f, 40.2823029f, 1489615751, 16.7506771),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5742493f, 40.2823067f, leg_index, 1.78378296f,
                                                185, maneuver_index, 0.13585937f, 20,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 792
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5740891f, 40.2823906f, 1489615752, 16.7506618),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5740891f,
                                                40.2823906f, leg_index, 1.76727879f, 183,
                                                maneuver_index, 0.119355202f, 18));
  //----------------------------------------------------------------
  // trace point = 793
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5739288f, 40.2824745f, 1489615753, 16.4941959),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5739288f,
                                                40.2824745f, leg_index, 1.75077486f, 180,
                                                maneuver_index, 0.102851272f, 15));
  //----------------------------------------------------------------
  // trace point = 794
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5737762f, 40.2825584f, 1489615754, 15.9432659),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5737762f,
                                                40.2825584f, leg_index, 1.73480177f, 178,
                                                maneuver_index, 0.0868781805f, 13));
  //----------------------------------------------------------------
  // trace point = 795
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5736237f, 40.2826385f, 1489615755, 15.7455759),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5736237f,
                                                40.2826385f, leg_index, 1.71907198f, 176,
                                                maneuver_index, 0.0711483955f, 11));
  //----------------------------------------------------------------
  // trace point = 796
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5734863f, 40.2827072f, 1489615756, 13.9513435),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5734863f,
                                                40.282711f, leg_index, 1.70489097f, 174,
                                                maneuver_index, 0.0569673777f, 9));
  //----------------------------------------------------------------
  // trace point = 797
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5733719f, 40.2827721f, 1489615757, 12.0863209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5733719f,
                                                40.2827721f, leg_index, 1.69303286f, 172,
                                                maneuver_index, 0.045109272f, 7));
  //----------------------------------------------------------------
  // trace point = 798
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5732727f, 40.2828217f, 1489615758, 10.0827332),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5732727f,
                                                40.2828255f, leg_index, 1.68272352f, 170,
                                                maneuver_index, 0.0347999334f, 5));
  //----------------------------------------------------------------
  // trace point = 799
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5731201f, 40.2828979f, 1489615760, 7.75231695),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5731277f,
                                                40.2829056f, leg_index, 1.66752625f, 168,
                                                maneuver_index, 0.0196026564f, 3));
  //----------------------------------------------------------------
  // trace point = 800
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5730133f, 40.2829475f, 1489615763, 3.5519321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5730209f,
                                                40.282959f, leg_index, 1.65668499f, 166,
                                                maneuver_index, 0.00876140594f, 1));
  //----------------------------------------------------------------
  // trace point = 801 | Alert In 150 meters | Enter the roundabout and take the 1st exit.
  maneuver_index = 11;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5729523f, 40.2830505f, 1489615771, 1.57272398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5729523f, 40.2830505f, leg_index, 1.6426698f, 164,
                                                maneuver_index, 0.153226972f, 21, instruction_index,
                                                kFinalShortTransitionAlertMetricLength));
  //----------------------------------------------------------------
  // trace point = 802
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5729828f, 40.2831879f, 1489615773, 7.73815823),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5729904f,
                                                40.2831879f, leg_index, 1.62704277f, 162,
                                                maneuver_index, 0.137599945f, 19));
  //----------------------------------------------------------------
  // trace point = 803
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5730286f, 40.2833557f, 1489615775, 9.55264759),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5730362f,
                                                40.2833557f, leg_index, 1.6079582f, 160,
                                                maneuver_index, 0.118515372f, 17));
  //----------------------------------------------------------------
  // trace point = 804
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5730515f, 40.2834473f, 1489615776, 10.3009109),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5730591f,
                                                40.2834473f, leg_index, 1.59758532f, 158,
                                                maneuver_index, 0.108142495f, 15));
  //----------------------------------------------------------------
  // trace point = 805
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5730743f, 40.2835503f, 1489615777, 11.6310759),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5730896f,
                                                40.2835464f, leg_index, 1.58624661f, 156,
                                                maneuver_index, 0.0968037844f, 13));
  //----------------------------------------------------------------
  // trace point = 806 | Pre | Enter the roundabout and take the 1st exit.
  maneuver_index = 11;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5730972f, 40.2836533f, 1489615778, 11.6310749),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5731201f, 40.2836494f, leg_index, 1.57449293f,
                                                155, maneuver_index, 0.085050106f, 12,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 807
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5731277f, 40.2837563f, 1489615779, 11.7642365),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.573143f,
                                                40.2837524f, leg_index, 1.5628742f, 153,
                                                maneuver_index, 0.0734313726f, 10));
  //----------------------------------------------------------------
  // trace point = 808
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5731583f, 40.2838554f, 1489615780, 11.3112583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5731735f,
                                                40.2838516f, leg_index, 1.55153513f, 152,
                                                maneuver_index, 0.0620923042f, 9));
  //----------------------------------------------------------------
  // trace point = 809
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5731888f, 40.2839508f, 1489615781, 11.0061398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.573204f,
                                                40.283947f, leg_index, 1.54061282f, 150,
                                                maneuver_index, 0.0511699915f, 7));
  //----------------------------------------------------------------
  // trace point = 810
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5732117f, 40.2840385f, 1489615782, 9.98892879),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5732269f,
                                                40.2840347f, leg_index, 1.53065455f, 149,
                                                maneuver_index, 0.0412117243f, 6));
  //----------------------------------------------------------------
  // trace point = 811
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5732498f, 40.2841835f, 1489615784, 8.2303648),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5732651f,
                                                40.2841797f, leg_index, 1.51419663f, 146,
                                                maneuver_index, 0.024753809f, 3));
  //----------------------------------------------------------------
  // trace point = 812
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5732651f, 40.2842979f, 1489615786, 6.37709522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5732956f,
                                                40.2842941f, leg_index, 1.50119805f, 145,
                                                maneuver_index, 0.011755228f, 2));
  //----------------------------------------------------------------
  // trace point = 813
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5732498f, 40.2844009f, 1489615788, 5.76822281),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5732651f,
                                                40.28442f, leg_index, 1.482898f, 142, maneuver_index,
                                                0.0544025898f, 7));
  //----------------------------------------------------------------
  // trace point = 814 | Pre | Exit the roundabout onto South Forge Road, Pennsylvania 1 17.
  maneuver_index = 12;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5732117f, 40.2845345f, 1489615791, 5.05102491),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5731659f, 40.2845306f, leg_index, 1.46832931f,
                                                140, maneuver_index, 0.0398339033f, 5,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 815
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5732269f, 40.2846298f, 1489615793, 5.38115883),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5731812f,
                                                40.2846489f, leg_index, 1.45483768f, 138,
                                                maneuver_index, 0.0263422728f, 3));
  //----------------------------------------------------------------
  // trace point = 816
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5733032f, 40.2847214f, 1489615795, 6.00563478),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5732956f,
                                                40.2847328f, leg_index, 1.44108915f, 136,
                                                maneuver_index, 0.0125937462f, 1));
  //----------------------------------------------------------------
  // trace point = 817
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5733948f, 40.2848206f, 1489615797, 6.7999301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5734634f,
                                                40.2848091f, leg_index, 1.42102563f, 134,
                                                maneuver_index, 0.432891071f, 27));
  //----------------------------------------------------------------
  // trace point = 818 | Post | Continue for 400 meters.
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5734634f, 40.2849388f, 1489615799, 7.17445612),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.573494f, 40.284935f, leg_index, 1.40678549f, 134,
                                                maneuver_index, 0.418650925f, 27, instruction_index));
  //----------------------------------------------------------------
  // trace point = 819
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5735016f, 40.2850685f, 1489615801, 7.4228282),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5735321f,
                                                40.2850609f, leg_index, 1.39240253f, 133,
                                                maneuver_index, 0.404267967f, 26));
  //----------------------------------------------------------------
  // trace point = 820
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5735474f, 40.285202f, 1489615803, 7.64534521),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5735779f,
                                                40.2851982f, leg_index, 1.37663436f, 132,
                                                maneuver_index, 0.388499796f, 25));
  //----------------------------------------------------------------
  // trace point = 821
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5735855f, 40.2853355f, 1489615805, 7.64001703),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.573616f,
                                                40.2853317f, leg_index, 1.36142302f, 131,
                                                maneuver_index, 0.373288453f, 24));
  //----------------------------------------------------------------
  // trace point = 822
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5736313f, 40.2854691f, 1489615807, 7.64534283),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5736542f,
                                                40.2854652f, leg_index, 1.34621167f, 130,
                                                maneuver_index, 0.358077109f, 23));
  //----------------------------------------------------------------
  // trace point = 823
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5736694f, 40.2856064f, 1489615809, 7.794209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5736923f,
                                                40.2856026f, leg_index, 1.33058619f, 129,
                                                maneuver_index, 0.342451632f, 22));
  //----------------------------------------------------------------
  // trace point = 824
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5737152f, 40.2857513f, 1489615811, 8.30362797),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5737381f,
                                                40.2857475f, leg_index, 1.31398952f, 128,
                                                maneuver_index, 0.325854957f, 21));
  //----------------------------------------------------------------
  // trace point = 825
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.573761f, 40.2859039f, 1489615813, 8.73704147),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5737839f,
                                                40.2859001f, leg_index, 1.29656446f, 127,
                                                maneuver_index, 0.308429897f, 20));
  //----------------------------------------------------------------
  // trace point = 826
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5738068f, 40.2860565f, 1489615815, 8.73703957),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5738297f,
                                                40.2860527f, leg_index, 1.27913928f, 125,
                                                maneuver_index, 0.291004717f, 18));
  //----------------------------------------------------------------
  // trace point = 827
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5738602f, 40.2862167f, 1489615817, 9.18547058),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5738754f,
                                                40.2862129f, leg_index, 1.26088572f, 124,
                                                maneuver_index, 0.272751153f, 17));
  //----------------------------------------------------------------
  // trace point = 828
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5739136f, 40.286377f, 1489615819, 9.19596291),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5739212f,
                                                40.286377f, leg_index, 1.24221802f, 123,
                                                maneuver_index, 0.254083455f, 16));
  //----------------------------------------------------------------
  // trace point = 829
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5739594f, 40.2865448f, 1489615821, 9.54403782),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5739746f,
                                                40.286541f, leg_index, 1.22340763f, 122,
                                                maneuver_index, 0.235273063f, 15));
  //----------------------------------------------------------------
  // trace point = 830
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5739899f, 40.2866325f, 1489615822, 10.1436434),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5739975f,
                                                40.2866325f, leg_index, 1.21303809f, 121,
                                                maneuver_index, 0.224903524f, 14));
  //----------------------------------------------------------------
  // trace point = 831
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5740128f, 40.2867279f, 1489615823, 10.7541885),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.574028f,
                                                40.2867241f, leg_index, 1.20252585f, 121,
                                                maneuver_index, 0.214391291f, 14));
  //----------------------------------------------------------------
  // trace point = 832
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5740433f, 40.2868195f, 1489615824, 10.4454832),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5740509f,
                                                40.2868195f, leg_index, 1.19174266f, 120,
                                                maneuver_index, 0.203608096f, 13));
  //----------------------------------------------------------------
  // trace point = 833
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5740814f, 40.2869148f, 1489615825, 11.1632128),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5740814f,
                                                40.2869148f, leg_index, 1.18081594f, 119,
                                                maneuver_index, 0.192681372f, 12));
  //----------------------------------------------------------------
  // trace point = 834 | Alert 0.15 | Turn right onto Airport Road.
  maneuver_index = 13;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5741043f, 40.2870102f, 1489615826, 10.7541876),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5741119f, 40.2870102f, leg_index, 1.16988945f,
                                                119, maneuver_index, 0.181754887f, 12,
                                                instruction_index, 0.15f));
  //----------------------------------------------------------------
  // trace point = 835
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5741348f, 40.2871094f, 1489615827, 11.330658),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5741425f,
                                                40.2871094f, leg_index, 1.15854895f, 118,
                                                maneuver_index, 0.170414388f, 11));
  //----------------------------------------------------------------
  // trace point = 836
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5741653f, 40.2872086f, 1489615828, 11.4197168),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.574173f,
                                                40.2872086f, leg_index, 1.14720869f, 117,
                                                maneuver_index, 0.159074128f, 10));
  //----------------------------------------------------------------
  // trace point = 837
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5741959f, 40.2873116f, 1489615829, 11.764205),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5742035f,
                                                40.2873116f, leg_index, 1.13545394f, 116,
                                                maneuver_index, 0.147319376f, 9));
  //----------------------------------------------------------------
  // trace point = 838
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.574234f, 40.2874146f, 1489615830, 11.9113083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.574234f,
                                                40.2874146f, leg_index, 1.12369931f, 116,
                                                maneuver_index, 0.135564744f, 9));
  //----------------------------------------------------------------
  // trace point = 839
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5742645f, 40.2875214f, 1489615831, 12.0899248),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5742645f,
                                                40.2875214f, leg_index, 1.11153007f, 115,
                                                maneuver_index, 0.123395503f, 8));
  //----------------------------------------------------------------
  // trace point = 840
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.574295f, 40.2876205f, 1489615832, 11.419714),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.574295f,
                                                40.2876205f, leg_index, 1.10018992f, 114,
                                                maneuver_index, 0.112055361f, 7));
  //----------------------------------------------------------------
  // trace point = 841
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5743179f, 40.2877274f, 1489615833, 11.9604034),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5743256f,
                                                40.2877274f, leg_index, 1.08802092f, 113,
                                                maneuver_index, 0.0998863578f, 6));
  //----------------------------------------------------------------
  // trace point = 842
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5743484f, 40.2878304f, 1489615834, 11.745491),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5743561f,
                                                40.2878304f, leg_index, 1.07626641f, 113,
                                                maneuver_index, 0.088131845f, 6));
  //----------------------------------------------------------------
  // trace point = 843
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5743713f, 40.2879295f, 1489615835, 11.3019772),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.574379f,
                                                40.2879295f, leg_index, 1.06505454f, 112,
                                                maneuver_index, 0.0769199729f, 5));
  //----------------------------------------------------------------
  // trace point = 844 | Pre | Turn right onto Airport Road.
  maneuver_index = 13;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5744019f, 40.2880325f, 1489615836, 11.7641993),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5744095f, 40.2880325f, leg_index, 1.05330873f,
                                                111, maneuver_index, 0.0651741624f, 4,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 845
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5744247f, 40.2881279f, 1489615837, 10.7541819),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5744324f,
                                                40.2881279f, leg_index, 1.04251599f, 110,
                                                maneuver_index, 0.0543814301f, 3));
  //----------------------------------------------------------------
  // trace point = 846
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5744781f, 40.2882919f, 1489615839, 9.40146637),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5744705f,
                                                40.2882919f, leg_index, 1.02397108f, 109,
                                                maneuver_index, 0.0358365178f, 2));
  //----------------------------------------------------------------
  // trace point = 847
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5745087f, 40.2884064f, 1489615841, 6.48027897),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.574501f,
                                                40.2884064f, leg_index, 1.01097429f, 108,
                                                maneuver_index, 0.022839725f, 1));
  //----------------------------------------------------------------
  // trace point = 848
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5745087f, 40.2884979f, 1489615843, 5.12069702),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5745163f,
                                                40.2884979f, leg_index, 1.00071967f, 108,
                                                maneuver_index, 0.0125851035f, 1));
  //----------------------------------------------------------------
  // trace point = 849
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5743866f, 40.2886086f, 1489615846, 5.37486076),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5743942f,
                                                40.2886353f, leg_index, 0.974841118f, 105,
                                                maneuver_index, 0.303573132f, 39));
  //----------------------------------------------------------------
  // trace point = 850 | Post | Continue for 300 meters.
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.574234f, 40.2886505f, 1489615848, 6.90376806),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.574234f, 40.288662f, leg_index, 0.9609164f, 103,
                                                maneuver_index, 0.289648414f, 37, instruction_index));
  //----------------------------------------------------------------
  // trace point = 851
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5740433f, 40.2887039f, 1489615850, 8.62919235),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5740356f,
                                                40.2887001f, leg_index, 0.943503141f, 101,
                                                maneuver_index, 0.272235155f, 35));
  //----------------------------------------------------------------
  // trace point = 852
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5739365f, 40.2887497f, 1489615851, 10.4294434),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5739441f,
                                                40.2887573f, leg_index, 0.933456421f, 100,
                                                maneuver_index, 0.262188435f, 34));
  //----------------------------------------------------------------
  // trace point = 853
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5738373f, 40.2888069f, 1489615852, 10.532361),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5738449f,
                                                40.2888145f, leg_index, 0.922897577f, 99,
                                                maneuver_index, 0.251629591f, 33));
  //----------------------------------------------------------------
  // trace point = 854
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5737228f, 40.2888641f, 1489615853, 11.574439),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5737381f,
                                                40.2888794f, leg_index, 0.9113065f, 97,
                                                maneuver_index, 0.240038514f, 31));
  //----------------------------------------------------------------
  // trace point = 855
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5736084f, 40.288929f, 1489615854, 12.1537342),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5736237f,
                                                40.2889442f, leg_index, 0.8992033f, 95,
                                                maneuver_index, 0.227935314f, 29));
  //----------------------------------------------------------------
  // trace point = 856
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.573494f, 40.2890015f, 1489615855, 12.5676508),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5735092f,
                                                40.2890129f, leg_index, 0.886839986f, 94,
                                                maneuver_index, 0.215572f, 28));
  //----------------------------------------------------------------
  // trace point = 857
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5733719f, 40.2890739f, 1489615856, 13.1664562),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5733871f,
                                                40.2890854f, leg_index, 0.873704493f, 92,
                                                maneuver_index, 0.202436507f, 26));
  //----------------------------------------------------------------
  // trace point = 858
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5732422f, 40.2891541f, 1489615857, 14.1832008),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5732498f,
                                                40.2891655f, leg_index, 0.859024644f, 90,
                                                maneuver_index, 0.187756658f, 24));
  //----------------------------------------------------------------
  // trace point = 859 | Alert 0.15 | Turn right onto Taxiway Road.
  maneuver_index = 14;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5730972f, 40.289238f, 1489615858, 15.460741),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5731049f, 40.2892494f, leg_index, 0.843572736f,
                                                88, maneuver_index, 0.172304749f, 22,
                                                instruction_index, 0.15f));
  //----------------------------------------------------------------
  // trace point = 860
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5729523f, 40.2893219f, 1489615859, 15.4607286),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5729675f,
                                                40.2893333f, leg_index, 0.828632593f, 86,
                                                maneuver_index, 0.157364607f, 20));
  //----------------------------------------------------------------
  // trace point = 861
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5728226f, 40.2894096f, 1489615860, 14.6952276),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5728302f,
                                                40.2894135f, leg_index, 0.813952804f, 84,
                                                maneuver_index, 0.142684817f, 18));
  //----------------------------------------------------------------
  // trace point = 862
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5726776f, 40.2894897f, 1489615861, 15.1955318),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5726852f,
                                                40.2894974f, leg_index, 0.798501074f, 82,
                                                maneuver_index, 0.127233088f, 16));
  //----------------------------------------------------------------
  // trace point = 863
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5725327f, 40.2895775f, 1489615862, 15.6649323),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5725403f,
                                                40.2895851f, leg_index, 0.782788754f, 80,
                                                maneuver_index, 0.111520767f, 14));
  //----------------------------------------------------------------
  // trace point = 864 | Pre | Turn right onto Taxiway Road.
  maneuver_index = 14;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5723877f, 40.2896652f, 1489615863, 15.7339859),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5723953f, 40.2896729f, leg_index, 0.767076612f,
                                                78, maneuver_index, 0.0958086252f, 12,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 865
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5722504f, 40.2897491f, 1489615864, 14.9916258),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.572258f,
                                                40.289753f, leg_index, 0.752396762f, 76,
                                                maneuver_index, 0.0811287761f, 10));
  //----------------------------------------------------------------
  // trace point = 866
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.572113f, 40.2898293f, 1489615865, 14.6504793),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5721207f,
                                                40.2898331f, leg_index, 0.737717032f, 75,
                                                maneuver_index, 0.0664490461f, 9));
  //----------------------------------------------------------------
  // trace point = 867
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5719757f, 40.2899094f, 1489615866, 14.6504679),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5719833f,
                                                40.289917f, leg_index, 0.722777426f, 73,
                                                maneuver_index, 0.0515094399f, 7));
  //----------------------------------------------------------------
  // trace point = 868
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5718536f, 40.2899818f, 1489615867, 13.1663485),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5718613f,
                                                40.2899895f, leg_index, 0.709642231f, 71,
                                                maneuver_index, 0.0383742452f, 5));
  //----------------------------------------------------------------
  // trace point = 869
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5717468f, 40.2900429f, 1489615868, 11.3428106),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5717545f,
                                                40.2900505f, leg_index, 0.698310733f, 69,
                                                maneuver_index, 0.0270427465f, 3));
  //----------------------------------------------------------------
  // trace point = 870
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.571579f, 40.2901306f, 1489615870, 8.65248108),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5715942f,
                                                40.2901459f, leg_index, 0.681054056f, 67,
                                                maneuver_index, 0.00978606939f, 1));
  //----------------------------------------------------------------
  // trace point = 871
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5714417f, 40.290123f, 1489615872, 5.83350801),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.571434f,
                                                40.2901306f, leg_index, 0.661657214f, 65,
                                                maneuver_index, 0.489900321f, 47));
  //----------------------------------------------------------------
  // trace point = 872 | Post | Continue for a half kilometer.
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5713272f, 40.2900314f, 1489615874, 7.07530499),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.5713272f, 40.2900314f, leg_index, 0.647369802f,
                                                63, maneuver_index, 0.475612909f, 45,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 873
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5711975f, 40.2899094f, 1489615876, 8.72391319),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5711975f,
                                                40.2899094f, leg_index, 0.629880965f, 62,
                                                maneuver_index, 0.458124071f, 44));
  //----------------------------------------------------------------
  // trace point = 874
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5711136f, 40.2898369f, 1489615877, 10.7291689),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5711136f,
                                                40.2898407f, leg_index, 0.619305372f, 61,
                                                maneuver_index, 0.447548479f, 43));
  //----------------------------------------------------------------
  // trace point = 875
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.571022f, 40.2897682f, 1489615878, 10.9556036),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5710068f,
                                                40.2897835f, leg_index, 0.608223557f, 60,
                                                maneuver_index, 0.436466664f, 42));
  //----------------------------------------------------------------
  // trace point = 876
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5709229f, 40.2897072f, 1489615879, 10.8063707),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5709f,
                                                40.2897301f, leg_index, 0.597380698f, 59,
                                                maneuver_index, 0.425623804f, 41));
  //----------------------------------------------------------------
  // trace point = 877
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5708237f, 40.2896538f, 1489615880, 10.2701626),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5708008f,
                                                40.2896767f, leg_index, 0.587073267f, 58,
                                                maneuver_index, 0.415316373f, 40));
  //----------------------------------------------------------------
  // trace point = 878
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5707169f, 40.2896042f, 1489615881, 10.6550283),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5707016f,
                                                40.2896233f, leg_index, 0.576766133f, 57,
                                                maneuver_index, 0.40500924f, 39));
  //----------------------------------------------------------------
  // trace point = 879
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.57061f, 40.2895508f, 1489615882, 10.833231),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5705948f,
                                                40.2895699f, leg_index, 0.565923035f, 56,
                                                maneuver_index, 0.394166142f, 38));
  //----------------------------------------------------------------
  // trace point = 880
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5704956f, 40.2894936f, 1489615883, 11.5743561),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5704803f,
                                                40.2895126f, leg_index, 0.554305732f, 55,
                                                maneuver_index, 0.382548839f, 37));
  //----------------------------------------------------------------
  // trace point = 881
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5703735f, 40.2894325f, 1489615884, 12.3865995),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5703583f,
                                                40.2894478f, leg_index, 0.541674852f, 54,
                                                maneuver_index, 0.369917959f, 36));
  //----------------------------------------------------------------
  // trace point = 882
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5702515f, 40.2893677f, 1489615885, 12.7059097),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5702362f,
                                                40.2893829f, leg_index, 0.529044032f, 52,
                                                maneuver_index, 0.357287139f, 34));
  //----------------------------------------------------------------
  // trace point = 883
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5701294f, 40.2893066f, 1489615886, 12.3866158),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5701141f,
                                                40.2893219f, leg_index, 0.516652107f, 51,
                                                maneuver_index, 0.344895214f, 33));
  //----------------------------------------------------------------
  // trace point = 884
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5699997f, 40.2892418f, 1489615887, 13.1279278),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5699844f,
                                                40.289257f, leg_index, 0.503486335f, 50,
                                                maneuver_index, 0.331729442f, 32));
  //----------------------------------------------------------------
  // trace point = 885
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5698776f, 40.2891769f, 1489615888, 12.6362267),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5698624f,
                                                40.2891922f, leg_index, 0.490855098f, 49,
                                                maneuver_index, 0.319098204f, 31));
  //----------------------------------------------------------------
  // trace point = 886
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5697479f, 40.2891121f, 1489615889, 13.1988792),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5697403f,
                                                40.2891235f, leg_index, 0.477985442f, 47,
                                                maneuver_index, 0.306228548f, 29));
  //----------------------------------------------------------------
  // trace point = 887
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5696182f, 40.2890472f, 1489615890, 13.1988878),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5696106f,
                                                40.2890587f, leg_index, 0.464818478f, 46,
                                                maneuver_index, 0.293061584f, 28));
  //----------------------------------------------------------------
  // trace point = 888
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5694962f, 40.2889824f, 1489615891, 12.6362514),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5694885f,
                                                40.2889938f, leg_index, 0.452187777f, 45,
                                                maneuver_index, 0.280430883f, 27));
  //----------------------------------------------------------------
  // trace point = 889
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5693665f, 40.2889175f, 1489615892, 13.1382093),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5693588f,
                                                40.288929f, leg_index, 0.4390212f, 44, maneuver_index,
                                                0.267264307f, 26));
  //----------------------------------------------------------------
  // trace point = 890
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5692444f, 40.2888565f, 1489615893, 12.386673),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5692368f,
                                                40.2888641f, leg_index, 0.426391482f, 42,
                                                maneuver_index, 0.254634589f, 24));
  //----------------------------------------------------------------
  // trace point = 891
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5691223f, 40.2887917f, 1489615894, 12.6362753),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5691147f,
                                                40.2888031f, leg_index, 0.413997412f, 42,
                                                maneuver_index, 0.242240518f, 24));
  //----------------------------------------------------------------
  // trace point = 892
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5690002f, 40.2887268f, 1489615895, 12.6362839),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5689926f,
                                                40.2887383f, leg_index, 0.401368022f, 40,
                                                maneuver_index, 0.229611129f, 22));
  //----------------------------------------------------------------
  // trace point = 893
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5688705f, 40.2886581f, 1489615896, 13.3785896),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5688553f,
                                                40.2886734f, leg_index, 0.387662053f, 39,
                                                maneuver_index, 0.21590516f, 21));
  //----------------------------------------------------------------
  // trace point = 894
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5687485f, 40.2885971f, 1489615897, 12.4578104),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5687408f,
                                                40.2886086f, leg_index, 0.375570834f, 38,
                                                maneuver_index, 0.20381394f, 20));
  //----------------------------------------------------------------
  // trace point = 895
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5686264f, 40.2885323f, 1489615898, 12.6363077),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5686111f,
                                                40.2885475f, leg_index, 0.36263895f, 37,
                                                maneuver_index, 0.190882057f, 19));
  //----------------------------------------------------------------
  // trace point = 896 | Alert 0.15 | Turn right onto Landings Drive.
  maneuver_index = 15;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5685043f, 40.2884674f, 1489615899, 12.6363163),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5684891f, 40.2884827f, leg_index, 0.350009203f,
                                                35, maneuver_index, 0.17825231f, 17,
                                                instruction_index, 0.15f));
  //----------------------------------------------------------------
  // trace point = 897
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5683823f, 40.2884064f, 1489615900, 12.3867311),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.568367f,
                                                40.2884216f, leg_index, 0.337615967f, 34,
                                                maneuver_index, 0.165859073f, 16));
  //----------------------------------------------------------------
  // trace point = 898
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5682526f, 40.2883453f, 1489615901, 12.8879824),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5682449f,
                                                40.2883568f, leg_index, 0.324986517f, 33,
                                                maneuver_index, 0.153229624f, 15));
  //----------------------------------------------------------------
  // trace point = 899
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5681381f, 40.2882843f, 1489615902, 11.8940868),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5681229f,
                                                40.2882996f, leg_index, 0.312830448f, 32,
                                                maneuver_index, 0.141073555f, 14));
  //----------------------------------------------------------------
  // trace point = 900
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5680161f, 40.2882233f, 1489615903, 12.386754),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5680084f,
                                                40.2882347f, leg_index, 0.300739259f, 31,
                                                maneuver_index, 0.128982365f, 13));
  //----------------------------------------------------------------
  // trace point = 901
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.567894f, 40.2881584f, 1489615904, 12.6363554),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5678787f,
                                                40.2881737f, leg_index, 0.287807435f, 29,
                                                maneuver_index, 0.116050541f, 11));
  //----------------------------------------------------------------
  // trace point = 902
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5677719f, 40.2880974f, 1489615905, 12.3867702),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5677567f,
                                                40.2881126f, leg_index, 0.275434822f, 28,
                                                maneuver_index, 0.103677928f, 10));
  //----------------------------------------------------------------
  // trace point = 903
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5676498f, 40.2880325f, 1489615906, 12.5729589),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5676346f,
                                                40.2880516f, leg_index, 0.26300934f, 27,
                                                maneuver_index, 0.0912524462f, 9));
  //----------------------------------------------------------------
  // trace point = 904 | Pre | Turn right onto Landings Drive.
  maneuver_index = 15;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5675278f, 40.2879677f, 1489615907, 12.6363792),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5675049f, 40.2879944f, leg_index, 0.250287801f,
                                                26, maneuver_index, 0.0785309076f, 8,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 905
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5674057f, 40.2879066f, 1489615908, 12.386795),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5673828f,
                                                40.2879372f, leg_index, 0.238122344f, 24,
                                                maneuver_index, 0.0663654506f, 6));
  //----------------------------------------------------------------
  // trace point = 906
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5672989f, 40.2878532f, 1489615909, 10.8944473),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.567276f,
                                                40.2878876f, leg_index, 0.227504998f, 23,
                                                maneuver_index, 0.055748105f, 5));
  //----------------------------------------------------------------
  // trace point = 907
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5671082f, 40.2877655f, 1489615911, 9.44516563),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5670776f,
                                                40.2877998f, leg_index, 0.208035558f, 21,
                                                maneuver_index, 0.0362786651f, 3));
  //----------------------------------------------------------------
  // trace point = 908
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5669556f, 40.2876968f, 1489615913, 7.5098629),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.566925f,
                                                40.2877312f, leg_index, 0.192992344f, 20,
                                                maneuver_index, 0.0212354511f, 2));
  //----------------------------------------------------------------
  // trace point = 909
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5668488f, 40.2876511f, 1489615915, 5.21478319),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5668259f,
                                                40.2876816f, leg_index, 0.182933927f, 19,
                                                maneuver_index, 0.0111770332f, 1));
  //----------------------------------------------------------------
  // trace point = 910 | Alert In 150 meters | You will arrive at your destination.
  maneuver_index = 16;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5667801f, 40.2875519f, 1489615920, 2.49626994),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.5667648f, 40.2875481f, leg_index, 0.161372095f,
                                                17, maneuver_index, 0.161372095f, 17,
                                                instruction_index,
                                                kFinalShortTransitionAlertMetricLength));
  //----------------------------------------------------------------
  // trace point = 911
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5668259f, 40.2874641f, 1489615922, 5.27308321),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5668182f,
                                                40.2874603f, leg_index, 0.150606364f, 16,
                                                maneuver_index, 0.150606364f, 16));
  //----------------------------------------------------------------
  // trace point = 912
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5669022f, 40.2873611f, 1489615924, 6.57867479),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5668869f,
                                                40.2873573f, leg_index, 0.137744263f, 14,
                                                maneuver_index, 0.137744263f, 14));
  //----------------------------------------------------------------
  // trace point = 913
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5670013f, 40.2872276f, 1489615926, 8.51285267),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5669785f,
                                                40.28722f, leg_index, 0.120596804f, 12,
                                                maneuver_index, 0.120596804f, 12));
  //----------------------------------------------------------------
  // trace point = 914
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5671082f, 40.2870674f, 1489615928, 10.0470066),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5670776f,
                                                40.287056f, leg_index, 0.0980155766f, 10,
                                                maneuver_index, 0.0980155766f, 10));
  //----------------------------------------------------------------
  // trace point = 915
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5671844f, 40.2869415f, 1489615930, 7.66933489),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5671692f,
                                                40.2869339f, leg_index, 0.0848487169f, 9,
                                                maneuver_index, 0.0848487169f, 9));
  //----------------------------------------------------------------
  // trace point = 916
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5672455f, 40.2868309f, 1489615935, 2.67965293),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5672379f,
                                                40.2868271f, leg_index, 0.0716166496f, 7,
                                                maneuver_index, 0.0716166496f, 7));
  //----------------------------------------------------------------
  // trace point = 917
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5673065f, 40.286747f, 1489615937, 5.36555004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5672989f,
                                                40.2867432f, leg_index, 0.0609332621f, 6,
                                                maneuver_index, 0.0609332621f, 6));
  //----------------------------------------------------------------
  // trace point = 918
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5673904f, 40.2866402f, 1489615939, 6.92011023),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5673752f,
                                                40.2866364f, leg_index, 0.0473925173f, 5,
                                                maneuver_index, 0.0473925173f, 5));
  //----------------------------------------------------------------
  // trace point = 919 | Pre | You have arrived at your destination.
  maneuver_index = 16;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.567482f, 40.2865143f, 1489615941, 8.02758503),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.5674667f, 40.2865067f, leg_index, 0.0309944768f,
                                                3, maneuver_index, 0.0309944768f, 3,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 920
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.5675659f, 40.2863808f, 1489615943, 8.21709442),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.5675583f,
                                                40.286377f, leg_index, 0.0145969158f, 2,
                                                maneuver_index, 0.0145969158f, 2));
}

/* AutoLancasterVerbalMultiCue geojson to visualize test route
{"type":"FeatureCollection","features":[
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300018,40.042339]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":0}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299995,40.042233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299934,40.042110]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299919,40.042000]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":3}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299919,40.041908]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":4}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299835,40.041851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":5}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299690,40.041878]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":6}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299492,40.041885]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":7}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299377,40.041893]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":8}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299271,40.041904]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":9}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299156,40.041920]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":10}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299034,40.041931]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":11}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298912,40.041946]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":12}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298798,40.041950]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":13}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298607,40.041969]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":14}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298454,40.041981]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":15}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298355,40.042007]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":16}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298187,40.042027]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":17}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298035,40.042034]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":18}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297852,40.042057]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":19}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297638,40.042076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":20}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297523,40.042076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":21}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297401,40.042091]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":22}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297264,40.042103]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":23}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297127,40.042110]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":24}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296989,40.042130]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":25}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296844,40.042145]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":26}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296707,40.042156]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":27}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296570,40.042168]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":28}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296432,40.042179]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":29}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296295,40.042202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":30}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296173,40.042210]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":31}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296043,40.042217]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":32}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295906,40.042236]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":33}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295761,40.042252]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":34}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295624,40.042267]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":35}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295479,40.042274]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":36}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295334,40.042294]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":37}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295189,40.042313]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":38}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295044,40.042332]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":39}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294891,40.042343]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":40}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294739,40.042362]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":41}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294594,40.042374]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":42}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294449,40.042393]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":43}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294304,40.042408]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":44}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294167,40.042419]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":45}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294037,40.042435]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":46}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293854,40.042461]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":47}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293777,40.042534]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":48}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293793,40.042667]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":49}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293823,40.042820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":50}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293846,40.042908]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":51}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293877,40.043076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":52}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293739,40.043129]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":53}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293564,40.043140]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":54}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293449,40.043159]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":55}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293320,40.043171]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":56}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293182,40.043194]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":57}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293060,40.043205]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":58}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292938,40.043221]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":59}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292816,40.043236]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":60}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292702,40.043247]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":61}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292580,40.043255]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":62}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292465,40.043266]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":63}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292351,40.043282]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":64}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292183,40.043297]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":65}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292076,40.043293]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":66}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291992,40.043388]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":67}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292007,40.043537]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":68}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292030,40.043655]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":69}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292061,40.043797]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":70}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292076,40.043911]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":71}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292091,40.044014]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":72}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292114,40.044121]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":73}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292221,40.044182]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":74}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292389,40.044170]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":75}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292496,40.044159]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":76}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292625,40.044144]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":77}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292763,40.044132]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":78}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292900,40.044121]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":79}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293053,40.044109]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":80}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293205,40.044086]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":81}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293365,40.044067]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":82}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293541,40.044044]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":83}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293716,40.044029]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":84}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293892,40.044014]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":85}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294075,40.044003]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":86}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294243,40.043983]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":87}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294426,40.043961]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":88}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294609,40.043949]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":89}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294769,40.043930]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":90}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294930,40.043900]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":91}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295090,40.043880]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":92}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295250,40.043861]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":93}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295410,40.043846]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":94}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295570,40.043819]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":95}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295731,40.043793]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":96}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295898,40.043781]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":97}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296059,40.043762]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":98}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296211,40.043751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":99}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296379,40.043739]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":100}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296539,40.043720]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":101}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296700,40.043701]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296852,40.043682]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":103}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296997,40.043655]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":104}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297134,40.043633]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":105}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297256,40.043610]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":106}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297363,40.043591]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297531,40.043552]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297646,40.043522]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297775,40.043518]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":110}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297890,40.043480]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":111}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297981,40.043404]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":112}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297989,40.043274]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297958,40.043121]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":114}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297935,40.043037]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":115}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297920,40.042946]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":116}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297890,40.042793]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297890,40.042690]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":118}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298027,40.042648]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":119}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298164,40.042641]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":120}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298164,40.042641]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":121}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300003,40.042339]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":0,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.141,"distance_from_trace_point":1.292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299980,40.042233]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.306,"distance_from_trace_point":1.292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299957,40.042107]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.500,"distance_from_trace_point":1.983}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299934,40.041996]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":3,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.671,"distance_from_trace_point":1.359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299919,40.041908]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":4,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.806,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299904,40.041843]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":5,"matched_point_type":"interpolated","edge_index":0,"distance_along_edge":0.907,"distance_from_trace_point":5.873}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299675,40.041805]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":6,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.100,"distance_from_trace_point":8.117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299484,40.041824]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":7,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.187,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299370,40.041840]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":8,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.239,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299263,40.041851]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":9,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.287,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299149,40.041862]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":10,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.339,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299026,40.041874]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":11,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.394,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298904,40.041889]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":12,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.450,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298790,40.041901]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":13,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.502,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298599,40.041920]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":14,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.588,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298447,40.041935]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":15,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.657,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298347,40.041946]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":16,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.702,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298180,40.041965]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":17,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.779,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298027,40.041981]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":18,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.848,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297844,40.042000]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":19,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.931,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297630,40.042023]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":20,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.072,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297516,40.042034]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":21,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.206,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297394,40.042046]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":22,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.348,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297256,40.042057]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":23,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.509,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297119,40.042072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":24,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.670,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296982,40.042084]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":25,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.830,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296837,40.042099]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":26,"matched_point_type":"matched","edge_index":2,"distance_along_edge":1.000,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296700,40.042114]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":27,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.115,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296562,40.042130]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":28,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.231,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296425,40.042141]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":29,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.346,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296288,40.042156]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":30,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.461,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296165,40.042171]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":31,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.564,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296036,40.042183]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":32,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.673,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295898,40.042198]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":33,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.788,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295753,40.042213]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":34,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.910,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295616,40.042229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":35,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.016,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295471,40.042244]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":36,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.094,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295326,40.042263]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":37,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.172,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295181,40.042278]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":38,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.250,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295036,40.042294]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":39,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.328,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294884,40.042313]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":40,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.410,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294731,40.042332]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":41,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.492,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294586,40.042347]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":42,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.570,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294441,40.042362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":43,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.647,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294296,40.042381]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":44,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.726,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294159,40.042397]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":45,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.799,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294029,40.042412]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":46,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.869,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293846,40.042431]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":47,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.967,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293800,40.042530]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":48,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.142,"distance_from_trace_point":1.983}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293823,40.042664]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":49,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.349,"distance_from_trace_point":2.617}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293854,40.042816]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":50,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.586,"distance_from_trace_point":2.617}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293869,40.042904]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":51,"matched_point_type":"interpolated","edge_index":5,"distance_along_edge":0.722,"distance_from_trace_point":1.983}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293900,40.043083]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":52,"matched_point_type":"matched","edge_index":5,"distance_along_edge":1.000,"distance_from_trace_point":2.113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293732,40.043102]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":53,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.087,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293556,40.043118]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":54,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.177,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293442,40.043133]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":55,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.236,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293312,40.043144]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":56,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.303,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293175,40.043159]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":57,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.374,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293053,40.043175]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":58,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.437,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292931,40.043186]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":59,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.500,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292809,40.043198]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":60,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.563,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292694,40.043213]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":61,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.622,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292572,40.043224]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":62,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.685,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292458,40.043236]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":63,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.744,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292343,40.043247]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":64,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.803,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292175,40.043266]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":65,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.890,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292076,40.043278]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":66,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.941,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.291977,40.043388]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":67,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.111,"distance_from_trace_point":1.292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292007,40.043537]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":68,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.278,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292030,40.043655]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":69,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.411,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292053,40.043797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":70,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.568,"distance_from_trace_point":0.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292076,40.043911]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":71,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.697,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292091,40.044014]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":72,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.812,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292114,40.044121]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":73,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.932,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292221,40.044170]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":74,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.051,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292389,40.044155]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":75,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.138,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292496,40.044144]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":76,"matched_point_type":"interpolated","edge_index":8,"distance_along_edge":0.193,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292625,40.044128]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":77,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.260,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292763,40.044117]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":78,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.331,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.292900,40.044102]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":79,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.401,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293045,40.044086]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":80,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.476,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293205,40.044067]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":81,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.559,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293365,40.044052]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":82,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.642,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293541,40.044033]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":83,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.732,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293716,40.044014]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":84,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.823,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.293892,40.043999]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":85,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.913,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294067,40.043980]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":86,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.005,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294235,40.043961]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":87,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.125,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294426,40.043941]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":88,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.261,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294601,40.043922]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":89,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.386,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294762,40.043907]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":90,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.500,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.294930,40.043892]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":91,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.619,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295090,40.043873]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":92,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.734,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295250,40.043858]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":93,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.848,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295410,40.043839]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":94,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.962,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295570,40.043823]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":95,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.043,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295731,40.043808]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":96,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.106,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.295898,40.043789]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":97,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.173,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296059,40.043770]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":98,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.237,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296211,40.043755]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":99,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.298,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296379,40.043732]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":100,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.365,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296539,40.043716]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":101,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.428,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296700,40.043697]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":102,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.492,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296852,40.043671]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":103,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.554,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.296997,40.043648]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":104,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.612,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297134,40.043621]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":105,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.668,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297256,40.043598]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":106,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.718,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297356,40.043579]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":107,"matched_point_type":"interpolated","edge_index":10,"distance_along_edge":0.758,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297523,40.043541]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":108,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.827,"distance_from_trace_point":1.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297646,40.043518]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":109,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.877,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297768,40.043495]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":110,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.926,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297890,40.043476]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":111,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.975,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297943,40.043407]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":112,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.072,"distance_from_trace_point":3.256}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297920,40.043282]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":113,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.230,"distance_from_trace_point":5.873}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297890,40.043129]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":114,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.423,"distance_from_trace_point":5.873}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297874,40.043045]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":115,"matched_point_type":"interpolated","edge_index":11,"distance_along_edge":0.529,"distance_from_trace_point":5.235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297859,40.042953]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":116,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.644,"distance_from_trace_point":5.235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297829,40.042801]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":117,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.837,"distance_from_trace_point":5.235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.297882,40.042664]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":118,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.073,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298027,40.042645]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":119,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.212,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298164,40.042629]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":120,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.343,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.298164,40.042629]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":121,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.343,"distance_from_trace_point":1.265}}
]}
 */
void TestAutoLancasterVerbalMultiCue() {
  std::string route_json_str =
      R"({"trip":{"language":"en-US","summary":{"max_lon":-76.291962,"max_lat":40.044182,"time":242,"length":0.963,"min_lat":40.041782,"min_lon":-76.300003},"locations":[{"lon":-76.300018,"lat":40.042339,"type":"break"},{"lon":-76.298164,"lat":40.042641,"type":"break"}],"units":"miles","legs":[{"shape":"eu~jkAfm~opCxa@uEqMyhCeDmt@{FkiAkLisBig@bF{KcxBwv@~HrKbxB`HvvAhBn^nDdn@rAfNTvCxBpRxBrPjBpRpp@aHrAlU","summary":{"max_lon":-76.291962,"max_lat":40.044182,"time":242,"length":0.963,"min_lat":40.041782,"min_lon":-76.300003},"maneuvers":[{"travel_type":"car","street_names":["North Shippen Street"],"verbal_pre_transition_instruction":"Drive south on North Shippen Street for 200 feet. Then Turn left onto East Chestnut Street.","instruction":"Drive south on North Shippen Street.","end_shape_index":1,"type":1,"time":6,"verbal_multi_cue":true,"length":0.039,"begin_shape_index":0,"travel_mode":"drive"},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn left onto East Chestnut Street, Pennsylvania 23 East.","verbal_transition_alert_instruction":"Turn left onto East Chestnut Street.","length":0.326,"instruction":"Turn left onto East Chestnut Street\/PA 23 East.","end_shape_index":5,"type":15,"time":87,"verbal_post_transition_instruction":"Continue for 3 tenths of a mile.","street_names":["East Chestnut Street","PA 23 East"],"begin_shape_index":1},{"travel_type":"car","travel_mode":"drive","verbal_multi_cue":true,"verbal_pre_transition_instruction":"Turn left onto North Marshall Street. Then Turn right onto East Fulton Street.","verbal_transition_alert_instruction":"Turn left onto North Marshall Street.","length":0.045,"instruction":"Turn left onto North Marshall Street.","end_shape_index":6,"type":15,"time":8,"verbal_post_transition_instruction":"Continue for 200 feet.","street_names":["North Marshall Street"],"begin_shape_index":5},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn right onto East Fulton Street.","verbal_transition_alert_instruction":"Turn right onto East Fulton Street.","length":0.104,"instruction":"Turn right onto East Fulton Street.","end_shape_index":7,"type":10,"time":21,"verbal_post_transition_instruction":"Continue for 1 tenth of a mile.","street_names":["East Fulton Street"],"begin_shape_index":6},{"travel_type":"car","travel_mode":"drive","verbal_multi_cue":true,"verbal_pre_transition_instruction":"Turn left onto North Franklin Street. Then Turn left onto East Walnut Street.","verbal_transition_alert_instruction":"Turn left onto North Franklin Street.","length":0.062,"instruction":"Turn left onto North Franklin Street.","end_shape_index":8,"type":15,"time":25,"verbal_post_transition_instruction":"Continue for 300 feet.","street_names":["North Franklin Street"],"begin_shape_index":7},{"travel_type":"car","travel_mode":"drive","verbal_pre_transition_instruction":"Turn left onto East Walnut Street, Pennsylvania 23 West.","verbal_transition_alert_instruction":"Turn left onto East Walnut Street.","length":0.313,"instruction":"Turn left onto East Walnut Street\/PA 23 West\/PA 462 West.","end_shape_index":17,"type":15,"time":66,"verbal_post_transition_instruction":"Continue for 3 tenths of a mile.","street_names":["East Walnut Street","PA 23 West","PA 462 West"],"begin_shape_index":8},{"travel_type":"car","travel_mode":"drive","verbal_multi_cue":true,"verbal_pre_transition_instruction":"Turn left onto North Plum Street. Then Turn right onto East Fulton Street.","verbal_transition_alert_instruction":"Turn left onto North Plum Street.","length":0.055,"instruction":"Turn left onto North Plum Street.","end_shape_index":18,"type":15,"time":20,"verbal_post_transition_instruction":"Continue for 300 feet.","street_names":["North Plum Street"],"begin_shape_index":17},{"travel_type":"car","travel_mode":"drive","verbal_multi_cue":true,"verbal_pre_transition_instruction":"Turn right onto East Fulton Street. Then You will arrive at your destination.","verbal_transition_alert_instruction":"Turn right onto East Fulton Street.","length":0.019,"instruction":"Turn right onto East Fulton Street.","end_shape_index":19,"type":10,"time":9,"verbal_post_transition_instruction":"Continue for 100 feet.","street_names":["East Fulton Street"],"begin_shape_index":18},{"travel_type":"car","travel_mode":"drive","begin_shape_index":19,"time":0,"type":4,"end_shape_index":19,"instruction":"You have arrived at your destination.","length":0.000,"verbal_transition_alert_instruction":"You will arrive at your destination.","verbal_pre_transition_instruction":"You have arrived at your destination."}]}],"status_message":"Found route between points","status":0}})";
  NavigatorTest nav;
  nav.SetRoute(route_json_str);
  uint32_t leg_index = 0;

  TryRouteLegCount(nav, 1);
  TryRouteManeuverCount(nav, leg_index, 9);
  TryRouteLanguage(nav, "en-US");
  TryRouteUnits(nav, "miles", false);

  ////////////////////////////////////////////////////////////////////////////
  uint32_t maneuver_index = 0;
  uint32_t instruction_index = 0;
  ////////////////////////////////////////////////////////////////////////////

  //----------------------------------------------------------------
  // trace point = 0 | Pre | Drive south on North Shippen Street for 200 feet. Then Turn left onto
  // East Chestnut Street.
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.3000183f, 40.0423393f, 1510692420, 0),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.3000031f, 40.0423393f, leg_index, 0.964204311f,
                                                244, maneuver_index, 0.0389375091f, 6,
                                                instruction_index),
                            true);
  //----------------------------------------------------------------
  // trace point = 1
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2999954f, 40.0422325f, 1510692424, 2.93912911),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2999802f,
                                                40.0422325f, leg_index, 0.956718624f, 243,
                                                maneuver_index, 0.0314518213f, 5),
                            true);
  //----------------------------------------------------------------
  // trace point = 2 | Pre | Turn left onto East Chestnut Street, Pennsylvania 23 East.
  maneuver_index = 0;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2999344f, 40.0421104f, 1510692426, 7.18354702),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2999573f, 40.0421066f, leg_index, 0.947927177f,
                                                241, maneuver_index, 0.0226603746f, 3,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 3
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2999191f, 40.0419998f, 1510692429, 4.27199888),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2999344f,
                                                40.041996f, leg_index, 0.940180719f, 240,
                                                maneuver_index, 0.0149139166f, 2));
  //----------------------------------------------------------------
  // trace point = 4
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2999191f, 40.0419083f, 1510692439, 1.00187504),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2999191f,
                                                40.0419083f, leg_index, 0.934058487f, 239,
                                                maneuver_index, 0.0087916851f, 1));
  //----------------------------------------------------------------
  // trace point = 5
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2998352f, 40.041851f, 1510692449, 0.926688015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2999115f,
                                                40.0418434f, leg_index, 0.92956537f, 239,
                                                maneuver_index, 0.00429856777f, 1));
  //----------------------------------------------------------------
  // trace point = 6
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2996902f, 40.0418777f, 1510692450, 12.8004837),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.299675f,
                                                40.0418053f, leg_index, 0.913443804f, 235,
                                                maneuver_index, 0.314969718f, 84));
  //----------------------------------------------------------------
  // trace point = 7 | Post | Continue for 3 tenths of a mile.
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2994919f, 40.0418854f, 1510692452, 8.4519701),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.2994843f, 40.0418243f, leg_index, 0.903257847f,
                                                233, maneuver_index, 0.304783762f, 82,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 8
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2993774f, 40.041893f, 1510692453, 9.83161354),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2993698f,
                                                40.0418396f, leg_index, 0.897110224f, 231,
                                                maneuver_index, 0.298636138f, 80));
  //----------------------------------------------------------------
  // trace point = 9
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2992706f, 40.0419044f, 1510692454, 9.56984138),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.299263f,
                                                40.041851f, leg_index, 0.891398907f, 229,
                                                maneuver_index, 0.292924821f, 78));
  //----------------------------------------------------------------
  // trace point = 10
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2991562f, 40.0419197f, 1510692455, 9.97574806),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2991486f,
                                                40.0418625f, leg_index, 0.885287225f, 228,
                                                maneuver_index, 0.28681314f, 77));
  //----------------------------------------------------------------
  // trace point = 11
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2990341f, 40.0419312f, 1510692456, 10.0079918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2990265f,
                                                40.0418739f, leg_index, 0.878775358f, 226,
                                                maneuver_index, 0.280301273f, 75));
  //----------------------------------------------------------------
  // trace point = 12
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.298912f, 40.0419464f, 1510692457, 10.6327038),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2989044f,
                                                40.0418892f, leg_index, 0.872227609f, 224,
                                                maneuver_index, 0.273753524f, 73));
  //----------------------------------------------------------------
  // trace point = 13
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2987976f, 40.0419502f, 1510692458, 9.81078529),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.29879f,
                                                40.0419006f, leg_index, 0.866115928f, 223,
                                                maneuver_index, 0.267641842f, 72));
  //----------------------------------------------------------------
  // trace point = 14
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2986069f, 40.0419693f, 1510692460, 8.36296844),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2985992f,
                                                40.0419197f, leg_index, 0.855929852f, 220,
                                                maneuver_index, 0.257455766f, 69));
  //----------------------------------------------------------------
  // trace point = 15
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2984543f, 40.0419807f, 1510692462, 6.30556107),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2984467f,
                                                40.041935f, leg_index, 0.847781062f, 218,
                                                maneuver_index, 0.249306977f, 67));
  //----------------------------------------------------------------
  // trace point = 16
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2983551f, 40.0420074f, 1510692466, 2.31070995),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2983475f,
                                                40.0419464f, leg_index, 0.842469931f, 216,
                                                maneuver_index, 0.243995845f, 65));
  //----------------------------------------------------------------
  // trace point = 17
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2981873f, 40.0420265f, 1510692475, 1.61746597),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2981796f,
                                                40.0419655f, leg_index, 0.833484948f, 214,
                                                maneuver_index, 0.235010862f, 63));
  //----------------------------------------------------------------
  // trace point = 18
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2980347f, 40.0420341f, 1510692477, 6.57727289),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.298027f,
                                                40.0419807f, leg_index, 0.825336158f, 212,
                                                maneuver_index, 0.226862073f, 61));
  //----------------------------------------------------------------
  // trace point = 19
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2978516f, 40.042057f, 1510692479, 7.88503313),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2978439f,
                                                40.0419998f, leg_index, 0.815550208f, 209,
                                                maneuver_index, 0.217076123f, 58));
  //----------------------------------------------------------------
  // trace point = 20
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2976379f, 40.0420761f, 1510692481, 9.03182125),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2976303f,
                                                40.0420227f, leg_index, 0.804128468f, 206,
                                                maneuver_index, 0.205654383f, 55));
  //----------------------------------------------------------------
  // trace point = 21
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2975235f, 40.0420761f, 1510692482, 9.88838673),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2975159f,
                                                40.0420341f, leg_index, 0.798016846f, 205,
                                                maneuver_index, 0.199542761f, 54));
  //----------------------------------------------------------------
  // trace point = 22
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2974014f, 40.0420914f, 1510692483, 10.6326818),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2973938f,
                                                40.0420456f, leg_index, 0.791504204f, 203,
                                                maneuver_index, 0.193030119f, 52));
  //----------------------------------------------------------------
  // trace point = 23
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2972641f, 40.0421028f, 1510692484, 11.4006214),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2972565f,
                                                40.042057f, leg_index, 0.784190238f, 201,
                                                maneuver_index, 0.185716152f, 50));
  //----------------------------------------------------------------
  // trace point = 24
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2971268f, 40.0421104f, 1510692485, 11.80338),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2971191f,
                                                40.0420723f, leg_index, 0.776843011f, 199,
                                                maneuver_index, 0.178368926f, 48));
  //----------------------------------------------------------------
  // trace point = 25
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2969894f, 40.0421295f, 1510692486, 12.2320423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2969818f,
                                                40.0420837f, leg_index, 0.769529283f, 197,
                                                maneuver_index, 0.171055198f, 46));
  //----------------------------------------------------------------
  // trace point = 26
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2968445f, 40.0421448f, 1510692487, 12.3163605),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2968369f,
                                                40.042099f, leg_index, 0.7617805f, 195,
                                                maneuver_index, 0.163306415f, 44));
  //----------------------------------------------------------------
  // trace point = 27
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2967072f, 40.0421562f, 1510692488, 11.6803265),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2966995f,
                                                40.0421143f, leg_index, 0.754432142f, 193,
                                                maneuver_index, 0.155958056f, 42));
  //----------------------------------------------------------------
  // trace point = 28
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2965698f, 40.0421677f, 1510692489, 11.8363953),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2965622f,
                                                40.0421295f, leg_index, 0.747083962f, 191,
                                                maneuver_index, 0.148609877f, 40));
  //----------------------------------------------------------------
  // trace point = 29
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2964325f, 40.0421791f, 1510692490, 11.4131021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2964249f,
                                                40.042141f, leg_index, 0.739771783f, 189,
                                                maneuver_index, 0.141297698f, 38));
  //----------------------------------------------------------------
  // trace point = 30
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2962952f, 40.042202f, 1510692491, 11.9696493),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2962875f,
                                                40.0421562f, leg_index, 0.732423425f, 187,
                                                maneuver_index, 0.133949339f, 36));
  //----------------------------------------------------------------
  // trace point = 31
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2961731f, 40.0422096f, 1510692492, 10.6900272),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2961655f,
                                                40.0421715f, leg_index, 0.725875616f, 185,
                                                maneuver_index, 0.127401531f, 34));
  //----------------------------------------------------------------
  // trace point = 32
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2960434f, 40.0422173f, 1510692493, 10.8804092),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2960358f,
                                                40.0421829f, leg_index, 0.718963504f, 184,
                                                maneuver_index, 0.120489419f, 33));
  //----------------------------------------------------------------
  // trace point = 33 | Alert 500 feet | Turn left onto North Marshall Street.
  maneuver_index = 1;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2959061f, 40.0422363f, 1510692494, 11.9961939),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.2958984f, 40.0421982f, leg_index, 0.711615145f,
                                                182, maneuver_index, 0.11314106f, 31,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 34
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2957611f, 40.0422516f, 1510692495, 12.3163414),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2957535f,
                                                40.0422134f, leg_index, 0.703866303f, 180,
                                                maneuver_index, 0.105392218f, 29));
  //----------------------------------------------------------------
  // trace point = 35
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2956238f, 40.0422668f, 1510692496, 12.1878691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2956161f,
                                                40.0422287f, leg_index, 0.696517944f, 177,
                                                maneuver_index, 0.098043859f, 26));
  //----------------------------------------------------------------
  // trace point = 36
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2954788f, 40.0422745f, 1510692497, 12.5026474),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2954712f,
                                                40.042244f, leg_index, 0.688769996f, 175,
                                                maneuver_index, 0.0902959108f, 24));
  //----------------------------------------------------------------
  // trace point = 37
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2953339f, 40.0422935f, 1510692498, 12.2850847),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2953262f,
                                                40.042263f, leg_index, 0.680982947f, 173,
                                                maneuver_index, 0.082508862f, 22));
  //----------------------------------------------------------------
  // trace point = 38
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2951889f, 40.0423126f, 1510692499, 12.5013752),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2951813f,
                                                40.0422783f, leg_index, 0.67323494f, 171,
                                                maneuver_index, 0.0747608542f, 20));
  //----------------------------------------------------------------
  // trace point = 39 | Pre | Turn left onto North Marshall Street. Then Turn right onto East Fulton
  // Street.
  maneuver_index = 1;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2950439f, 40.0423317f, 1510692500, 12.7050419),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2950363f, 40.0422935f, leg_index, 0.665487051f,
                                                169, maneuver_index, 0.0670129657f, 18,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 40
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2948914f, 40.0423431f, 1510692501, 12.9626675),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2948837f,
                                                40.0423126f, leg_index, 0.657300353f, 167,
                                                maneuver_index, 0.0588262677f, 16));
  //----------------------------------------------------------------
  // trace point = 41
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2947388f, 40.0423622f, 1510692502, 12.8384333),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2947311f,
                                                40.0423317f, leg_index, 0.649113894f, 165,
                                                maneuver_index, 0.0506398082f, 14));
  //----------------------------------------------------------------
  // trace point = 42
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2945938f, 40.0423737f, 1510692503, 12.5987616),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2945862f,
                                                40.042347f, leg_index, 0.641365826f, 162,
                                                maneuver_index, 0.0428917408f, 11));
  //----------------------------------------------------------------
  // trace point = 43
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2944489f, 40.0423927f, 1510692504, 12.7541418),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2944412f,
                                                40.0423622f, leg_index, 0.633617878f, 160,
                                                maneuver_index, 0.0351437926f, 9));
  //----------------------------------------------------------------
  // trace point = 44
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2943039f, 40.042408f, 1510692505, 12.518693),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2942963f,
                                                40.0423813f, leg_index, 0.625831008f, 158,
                                                maneuver_index, 0.0273569226f, 7));
  //----------------------------------------------------------------
  // trace point = 45
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2941666f, 40.0424194f, 1510692506, 11.5587912),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2941589f,
                                                40.0423965f, leg_index, 0.618482411f, 156,
                                                maneuver_index, 0.0200083256f, 5));
  //----------------------------------------------------------------
  // trace point = 46
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2940369f, 40.0424347f, 1510692507, 11.1880255),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2940292f,
                                                40.0424118f, leg_index, 0.61153394f, 154,
                                                maneuver_index, 0.0130598545f, 3));
  //----------------------------------------------------------------
  // trace point = 47
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2938538f, 40.0424614f, 1510692509, 7.82676888),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2938461f,
                                                40.0424309f, leg_index, 0.60174942f, 152,
                                                maneuver_index, 0.00327533484f, 1));
  //----------------------------------------------------------------
  // trace point = 48
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2937775f, 40.0425339f, 1510692511, 5.13638687),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2938004f,
                                                40.0425301f, leg_index, 0.59209013f, 150,
                                                maneuver_index, 0.0386193395f, 7));
  //----------------------------------------------------------------
  // trace point = 49 | Pre | Turn right onto East Fulton Street.
  maneuver_index = 2;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2937927f, 40.0426674f, 1510692513, 7.485744),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2938232f, 40.0426636f, leg_index, 0.58277595f,
                                                148, maneuver_index, 0.02930516f, 5,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 50
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2938232f, 40.04282f, 1510692515, 8.72780132),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2938538f,
                                                40.0428162f, leg_index, 0.572099686f, 146,
                                                maneuver_index, 0.0186288953f, 3));
  //----------------------------------------------------------------
  // trace point = 51
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2938461f, 40.0429077f, 1510692516, 9.709445),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.293869f,
                                                40.0429039f, leg_index, 0.565977335f, 145,
                                                maneuver_index, 0.0125065446f, 2));
  //----------------------------------------------------------------
  // trace point = 52
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2938766f, 40.0430756f, 1510692518, 9.46240425),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2938766f,
                                                40.043087f, leg_index, 0.552234054f, 143,
                                                maneuver_index, 0.102366507f, 21));
  //----------------------------------------------------------------
  // trace point = 53 | Alert In 500 feet | Turn left onto North Franklin Street.
  maneuver_index = 3;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2937393f, 40.043129f, 1510692521, 4.54608583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.2937317f, 40.0431023f, leg_index, 0.544485569f,
                                                141, maneuver_index, 0.0946180224f, 19,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 54
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2935638f, 40.0431404f, 1510692523, 7.48675919),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2935562f,
                                                40.0431175f, leg_index, 0.53513658f, 139,
                                                maneuver_index, 0.0852690339f, 17));
  //----------------------------------------------------------------
  // trace point = 55
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2934494f, 40.0431595f, 1510692524, 9.66950607),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2934418f,
                                                40.0431328f, leg_index, 0.528988719f, 138,
                                                maneuver_index, 0.0791211724f, 16));
  //----------------------------------------------------------------
  // trace point = 56
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2933197f, 40.0431709f, 1510692525, 11.134613),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2933121f,
                                                40.0431442f, leg_index, 0.522076964f, 137,
                                                maneuver_index, 0.0722094178f, 15));
  //----------------------------------------------------------------
  // trace point = 57
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2931824f, 40.0431938f, 1510692526, 11.7539282),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2931747f,
                                                40.0431595f, leg_index, 0.514728665f, 135,
                                                maneuver_index, 0.0648611188f, 13));
  //----------------------------------------------------------------
  // trace point = 58
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2930603f, 40.0432053f, 1510692527, 10.4689083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2930527f,
                                                40.0431747f, leg_index, 0.508180857f, 134,
                                                maneuver_index, 0.0583133101f, 12));
  //----------------------------------------------------------------
  // trace point = 59 | Pre | Turn left onto North Franklin Street. Then Turn left onto East Walnut
  // Street.
  maneuver_index = 3;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2929382f, 40.0432205f, 1510692528, 10.8669863),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2929306f, 40.0431862f, leg_index, 0.501669168f,
                                                132, maneuver_index, 0.0518016219f, 10,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 60
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2928162f, 40.0432358f, 1510692529, 10.2441111),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2928085f,
                                                40.0431976f, leg_index, 0.495157629f, 131,
                                                maneuver_index, 0.0452900827f, 9));
  //----------------------------------------------------------------
  // trace point = 61
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2927017f, 40.0432472f, 1510692530, 10.0600176),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2926941f,
                                                40.0432129f, leg_index, 0.489009917f, 130,
                                                maneuver_index, 0.0391423702f, 8));
  //----------------------------------------------------------------
  // trace point = 62
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2925797f, 40.0432549f, 1510692531, 10.2562418),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.292572f,
                                                40.0432243f, leg_index, 0.482498139f, 129,
                                                maneuver_index, 0.0326305926f, 7));
  //----------------------------------------------------------------
  // trace point = 63
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2924652f, 40.0432663f, 1510692532, 9.97555447),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2924576f,
                                                40.0432358f, leg_index, 0.476386786f, 127,
                                                maneuver_index, 0.0265192389f, 5));
  //----------------------------------------------------------------
  // trace point = 64
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2923508f, 40.0432816f, 1510692533, 9.73061657),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2923431f,
                                                40.0432472f, leg_index, 0.470275581f, 126,
                                                maneuver_index, 0.0204080343f, 4));
  //----------------------------------------------------------------
  // trace point = 65
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2921829f, 40.0432968f, 1510692535, 7.28562689),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2921753f,
                                                40.0432663f, leg_index, 0.46129033f, 124,
                                                maneuver_index, 0.0114227831f, 2));
  //----------------------------------------------------------------
  // trace point = 66
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2920761f, 40.043293f, 1510692538, 3.0416081),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2920761f,
                                                40.0432777f, leg_index, 0.455979228f, 123,
                                                maneuver_index, 0.00611168146f, 1));
  //----------------------------------------------------------------
  // trace point = 67
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2919922f, 40.0433884f, 1510692542, 3.27460003),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2919769f,
                                                40.0433884f, leg_index, 0.442961097f, 119,
                                                maneuver_index, 0.0554183722f, 22));
  //----------------------------------------------------------------
  // trace point = 68
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2920074f, 40.0435371f, 1510692544, 8.33697319),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2920074f,
                                                40.0435371f, leg_index, 0.43254596f, 115,
                                                maneuver_index, 0.0450032353f, 18));
  //----------------------------------------------------------------
  // trace point = 69 | Pre | Turn left onto East Walnut Street, Pennsylvania 23 West.
  maneuver_index = 4;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2920303f, 40.0436554f, 1510692545, 13.1586323),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2920303f, 40.0436554f, leg_index, 0.424277574f,
                                                112, maneuver_index, 0.0367348492f, 15,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 70
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2920609f, 40.0437965f, 1510692546, 15.8070602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2920532f,
                                                40.0437965f, leg_index, 0.414440155f, 108,
                                                maneuver_index, 0.0268974304f, 11));
  //----------------------------------------------------------------
  // trace point = 71
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2920761f, 40.043911f, 1510692547, 12.9761791),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2920761f,
                                                40.043911f, leg_index, 0.406433374f, 105,
                                                maneuver_index, 0.0188906491f, 8));
  //----------------------------------------------------------------
  // trace point = 72
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2920914f, 40.044014f, 1510692548, 11.5466986),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2920914f,
                                                40.044014f, leg_index, 0.399264783f, 102,
                                                maneuver_index, 0.0117220581f, 5));
  //----------------------------------------------------------------
  // trace point = 73
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2921143f, 40.0441208f, 1510692549, 11.8001652),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2921143f,
                                                40.0441208f, leg_index, 0.391784102f, 99,
                                                maneuver_index, 0.00424137712f, 2));
  //----------------------------------------------------------------
  // trace point = 74
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2922211f, 40.0441818f, 1510692551, 5.718081),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2922211f,
                                                40.0441704f, leg_index, 0.382231891f, 96,
                                                maneuver_index, 0.307604313f, 67));
  //----------------------------------------------------------------
  // trace point = 75 | Post  | Continue for 3 tenths of a mile.
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2923889f, 40.0441704f, 1510692553, 7.35449123),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.2923889f, 40.0441551f, leg_index, 0.373282671f,
                                                94, maneuver_index, 0.298655093f, 65,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 76
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2924957f, 40.0441589f, 1510692554, 9.31688976),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2924957f,
                                                40.0441437f, leg_index, 0.367571473f, 93,
                                                maneuver_index, 0.292943895f, 64));
  //----------------------------------------------------------------
  // trace point = 77
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2926254f, 40.0441437f, 1510692555, 10.8845015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2926254f,
                                                40.0441284f, leg_index, 0.360623777f, 91,
                                                maneuver_index, 0.285996199f, 62));
  //----------------------------------------------------------------
  // trace point = 78
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2927628f, 40.0441322f, 1510692556, 11.5483122),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2927628f,
                                                40.044117f, leg_index, 0.353311509f, 90,
                                                maneuver_index, 0.278683931f, 61));
  //----------------------------------------------------------------
  // trace point = 79
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2929001f, 40.0441208f, 1510692557, 12.1748238),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2929001f,
                                                40.0441017f, leg_index, 0.345963418f, 88,
                                                maneuver_index, 0.27133584f, 59));
  //----------------------------------------------------------------
  // trace point = 80
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2930527f, 40.0441093f, 1510692558, 12.8777351),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.293045f,
                                                40.0440865f, leg_index, 0.338215113f, 86,
                                                maneuver_index, 0.263587534f, 57));
  //----------------------------------------------------------------
  // trace point = 81
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2932053f, 40.0440865f, 1510692559, 13.2274628),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2932053f,
                                                40.0440674f, leg_index, 0.329630554f, 85,
                                                maneuver_index, 0.255002975f, 56));
  //----------------------------------------------------------------
  // trace point = 82
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2933655f, 40.0440674f, 1510692560, 14.189127),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2933655f,
                                                40.0440521f, leg_index, 0.321081549f, 83,
                                                maneuver_index, 0.246453971f, 54));
  //----------------------------------------------------------------
  // trace point = 83
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.293541f, 40.0440445f, 1510692561, 14.674902),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.293541f,
                                                40.0440331f, leg_index, 0.31169641f, 81,
                                                maneuver_index, 0.237068832f, 52));
  //----------------------------------------------------------------
  // trace point = 84
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2937164f, 40.0440292f, 1510692562, 15.1042175),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2937164f,
                                                40.044014f, leg_index, 0.30231145f, 79,
                                                maneuver_index, 0.227683872f, 50));
  //----------------------------------------------------------------
  // trace point = 85
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2938919f, 40.044014f, 1510692563, 15.2609062),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2938919f,
                                                40.0439987f, leg_index, 0.292962253f, 77,
                                                maneuver_index, 0.218334675f, 48));
  //----------------------------------------------------------------
  // trace point = 86
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.294075f, 40.0440025f, 1510692564, 15.5584278),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2940674f,
                                                40.0439796f, leg_index, 0.283576429f, 75,
                                                maneuver_index, 0.208948851f, 46));
  //----------------------------------------------------------------
  // trace point = 87
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2942429f, 40.0439835f, 1510692565, 14.6575947),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2942352f,
                                                40.0439606f, leg_index, 0.274591684f, 73,
                                                maneuver_index, 0.199964106f, 44));
  //----------------------------------------------------------------
  // trace point = 88
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.294426f, 40.0439606f, 1510692566, 15.8707104),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.294426f,
                                                40.0439415f, leg_index, 0.264405817f, 71,
                                                maneuver_index, 0.189778239f, 42));
  //----------------------------------------------------------------
  // trace point = 89
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2946091f, 40.0439491f, 1510692567, 15.1250944),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2946014f,
                                                40.0439224f, leg_index, 0.255020797f, 69,
                                                maneuver_index, 0.180393219f, 40));
  //----------------------------------------------------------------
  // trace point = 90
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2947693f, 40.0439301f, 1510692568, 14.1703653),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2947617f,
                                                40.0439072f, leg_index, 0.246471643f, 67,
                                                maneuver_index, 0.171844065f, 38));
  //----------------------------------------------------------------
  // trace point = 91
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2949295f, 40.0438995f, 1510692569, 14.12113),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2949295f,
                                                40.0438919f, leg_index, 0.237522453f, 65,
                                                maneuver_index, 0.162894875f, 36));
  //----------------------------------------------------------------
  // trace point = 92
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2950897f, 40.0438805f, 1510692570, 13.4144936),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2950897f,
                                                40.0438728f, leg_index, 0.228937835f, 63,
                                                maneuver_index, 0.154310256f, 34));
  //----------------------------------------------------------------
  // trace point = 93
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2952499f, 40.0438614f, 1510692571, 14.0892553),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2952499f,
                                                40.0438576f, leg_index, 0.220388889f, 61,
                                                maneuver_index, 0.145761311f, 32));
  //----------------------------------------------------------------
  // trace point = 94
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2954102f, 40.0438461f, 1510692572, 13.6131754),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2954102f,
                                                40.0438385f, leg_index, 0.211806029f, 60,
                                                maneuver_index, 0.137178451f, 31));
  //----------------------------------------------------------------
  // trace point = 95
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2955704f, 40.0438194f, 1510692573, 14.3130074),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2955704f,
                                                40.0438232f, leg_index, 0.203255221f, 57,
                                                maneuver_index, 0.128627628f, 28));
  //----------------------------------------------------------------
  // trace point = 96
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2957306f, 40.0437927f, 1510692574, 13.9626856),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2957306f,
                                                40.043808f, leg_index, 0.19470717f, 56,
                                                maneuver_index, 0.120079584f, 27));
  //----------------------------------------------------------------
  // trace point = 97 | Alert 500 feet | Turn left onto North Plum Street.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2958984f, 40.0437813f, 1510692575, 14.0628624),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.2958984f, 40.0437889f, leg_index, 0.185721666f,
                                                54, maneuver_index, 0.11109408f, 25,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 98
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2960587f, 40.0437622f, 1510692576, 13.6131916),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2960587f,
                                                40.0437698f, leg_index, 0.177135438f, 51,
                                                maneuver_index, 0.102507852f, 22));
  //----------------------------------------------------------------
  // trace point = 99
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2962112f, 40.0437508f, 1510692577, 13.4259272),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2962112f,
                                                40.0437546f, leg_index, 0.168988556f, 50,
                                                maneuver_index, 0.09436097f, 21));
  //----------------------------------------------------------------
  // trace point = 100
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2963791f, 40.0437393f, 1510692578, 14.1475697),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2963791f,
                                                40.0437317f, leg_index, 0.159963638f, 48,
                                                maneuver_index, 0.0853360519f, 19));
  //----------------------------------------------------------------
  // trace point = 101
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2965393f, 40.0437202f, 1510692579, 14.2196522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2965393f,
                                                40.0437164f, leg_index, 0.151417524f, 46,
                                                maneuver_index, 0.0767899379f, 17));
  //----------------------------------------------------------------
  // trace point = 102 | Pre | Turn left onto North Plum Street. Then Turn right onto East Fulton
  // Street.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2966995f, 40.0437012f, 1510692580, 13.4977379),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2966995f, 40.0436974f, leg_index, 0.142833024f,
                                                44, maneuver_index, 0.0682054386f, 15,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 103
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2968521f, 40.0436821f, 1510692581, 13.3306227),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2968521f,
                                                40.0436707f, leg_index, 0.134544641f, 42,
                                                maneuver_index, 0.0599170551f, 13));
  //----------------------------------------------------------------
  // trace point = 104
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2969971f, 40.0436554f, 1510692582, 12.5834446),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2969971f,
                                                40.0436478f, leg_index, 0.126706213f, 41,
                                                maneuver_index, 0.052078627f, 12));
  //----------------------------------------------------------------
  // trace point = 105
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2971344f, 40.0436325f, 1510692583, 12.1432838),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2971344f,
                                                40.0436211f, leg_index, 0.119202264f, 39,
                                                maneuver_index, 0.0445746779f, 10));
  //----------------------------------------------------------------
  // trace point = 106
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2972565f, 40.0436096f, 1510692584, 10.5156965),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2972565f,
                                                40.0435982f, leg_index, 0.1125471f, 37,
                                                maneuver_index, 0.0379195139f, 8));
  //----------------------------------------------------------------
  // trace point = 107
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2973633f, 40.0435905f, 1510692585, 9.63508797),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2973557f,
                                                40.0435791f, leg_index, 0.107131921f, 36,
                                                maneuver_index, 0.032504335f, 7));
  //----------------------------------------------------------------
  // trace point = 108
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2975311f, 40.0435524f, 1510692587, 7.45613098),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2975235f,
                                                40.043541f, leg_index, 0.0978620723f, 34,
                                                maneuver_index, 0.0232344866f, 5));
  //----------------------------------------------------------------
  // trace point = 109
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2976456f, 40.0435219f, 1510692589, 4.89905787),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2976456f,
                                                40.0435181f, leg_index, 0.091204755f, 32,
                                                maneuver_index, 0.0165771693f, 3));
  //----------------------------------------------------------------
  // trace point = 110
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2977753f, 40.0435181f, 1510692608, 0.588307977),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2977676f,
                                                40.0434952f, leg_index, 0.0845508054f, 31,
                                                maneuver_index, 0.00992321968f, 2));
  //----------------------------------------------------------------
  // trace point = 111
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2978897f, 40.0434799f, 1510692610, 5.25906801),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2978897f,
                                                40.0434761f, leg_index, 0.077954717f, 30,
                                                maneuver_index, 0.00332713127f, 1));
  //----------------------------------------------------------------
  // trace point = 112
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2979813f, 40.0434036f, 1510692612, 5.84461498),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2979431f,
                                                40.0434074f, leg_index, 0.070652023f, 28,
                                                maneuver_index, 0.0514428206f, 19));
  //----------------------------------------------------------------
  // trace point = 113
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2979889f, 40.0432739f, 1510692614, 7.29752493),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2979202f,
                                                40.0432816f, leg_index, 0.061860472f, 25,
                                                maneuver_index, 0.0426512696f, 16));
  //----------------------------------------------------------------
  // trace point = 114 | Pre | Turn right onto East Fulton Street. Then You will arrive at your
  // destination.
  maneuver_index = 6;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2979584f, 40.0431213f, 1510692616, 8.69317341),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2978897f, 40.043129f, leg_index, 0.0511835031f,
                                                21, maneuver_index, 0.0319743007f, 12,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 115
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2979355f, 40.0430374f, 1510692617, 9.1927681),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2978745f,
                                                40.043045f, leg_index, 0.0453225225f, 18,
                                                maneuver_index, 0.0261133201f, 9));
  //----------------------------------------------------------------
  // trace point = 116
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2979202f, 40.0429459f, 1510692618, 10.3106565),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2978592f,
                                                40.0429535f, leg_index, 0.0389390588f, 16,
                                                maneuver_index, 0.0197298564f, 7));
  //----------------------------------------------------------------
  // trace point = 117
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2978897f, 40.0427933f, 1510692620, 8.74806118),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.2978287f,
                                                40.0428009f, leg_index, 0.0282621756f, 12,
                                                maneuver_index, 0.00905297324f, 3));
  //----------------------------------------------------------------
  // trace point = 118 |  Pre | You have arrived at your destination.
  maneuver_index = 7;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2978897f, 40.0426903f, 1510692622, 5.73865223),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.2978821f, 40.0426636f, leg_index, 0.0151359448f,
                                                7, maneuver_index, 0.0151359448f, 7,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 119
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.298027f, 40.0426483f, 1510692625, 4.01934481),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.298027f,
                                                40.0426445f, leg_index, 0.00734865898f, 3,
                                                maneuver_index, 0.00734865898f, 3));
  //----------------------------------------------------------------
  // trace point = 120 | Complete
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.2981644f, 40.0426407f, 1510692627, 5.86975288),
                            GetNavigationStatus(NavigationStatus_RouteState_kComplete, -76.2981644f,
                                                40.0426292f, leg_index, 0.f, 0, maneuver_index, 0.f,
                                                0));
}

/* PedestrianCatalinasToOffice geojson to visualize test route
{"type":"FeatureCollection","features":[
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307518,40.039223]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":0}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307411,40.039257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307327,40.039318]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307205,40.039345]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":3}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307098,40.039371]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":4}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306999,40.039413]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":5}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306892,40.039452]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":6}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306801,40.039509]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":7}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306709,40.039562]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":8}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306610,40.039600]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":9}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306503,40.039597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":10}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306396,40.039623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":11}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306290,40.039665]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":12}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306168,40.039639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":13}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306068,40.039619]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":14}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306000,40.039684]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":15}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305992,40.039783]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":16}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306015,40.039864]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":17}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306091,40.039940]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":18}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306152,40.040009]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":19}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306152,40.040100]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":20}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306183,40.040180]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":21}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306213,40.040264]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":22}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306221,40.040363]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":23}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306221,40.040443]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":24}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306221,40.040531]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":25}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306267,40.040611]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":26}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306297,40.040699]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":27}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306313,40.040783]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":28}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306252,40.040859]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":29}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306213,40.040943]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":30}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306259,40.041019]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":31}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306175,40.041077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":32}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306122,40.041172]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":33}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306007,40.041218]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":34}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305908,40.041271]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":35}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305817,40.041233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":36}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305695,40.041241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":37}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305656,40.041317]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":38}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305550,40.041084]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":39}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305466,40.041138]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":40}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305428,40.041229]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":41}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305367,40.041298]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":42}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305260,40.041283]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":43}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305161,40.041328]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":44}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305054,40.041344]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":45}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304955,40.041393]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":46}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304840,40.041405]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":47}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304733,40.041370]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":48}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304626,40.041367]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":49}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304504,40.041382]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":50}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304390,40.041370]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":51}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304306,40.041431]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":52}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304184,40.041428]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":53}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304077,40.041439]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":54}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303970,40.041420]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":55}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303848,40.041424]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":56}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303734,40.041420]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":57}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303619,40.041420]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":58}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303513,40.041439]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":59}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303368,40.041435]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":60}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303253,40.041389]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":61}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303154,40.041443]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":62}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303047,40.041485]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":63}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302948,40.041451]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":64}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302818,40.041466]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":65}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302704,40.041485]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":66}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302582,40.041477]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":67}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302483,40.041454]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":68}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302361,40.041458]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":69}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302261,40.041489]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":70}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302147,40.041496]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":71}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302063,40.041557]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":72}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301964,40.041592]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":73}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301857,40.041569]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":74}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301743,40.041588]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":75}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301643,40.041618]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":76}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301514,40.041607]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":77}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301392,40.041622]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":78}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301285,40.041668]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":79}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301186,40.041698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":80}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301064,40.041695]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":81}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300957,40.041683]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":82}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300835,40.041687]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":83}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300720,40.041679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":84}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300598,40.041710]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":85}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300491,40.041733]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":86}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300385,40.041679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":87}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300262,40.041683]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":88}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300148,40.041706]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":89}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300034,40.041710]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":90}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299927,40.041733]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":91}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299858,40.041798]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":92}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299881,40.041878]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":93}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299934,40.041965]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":94}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299988,40.042042]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":95}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299950,40.042130]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":96}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299988,40.042213]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":97}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300011,40.042297]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":98}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299973,40.042385]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":99}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299866,40.042374]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":100}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299782,40.042442]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":101}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299675,40.042461]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299561,40.042488]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":103}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299454,40.042503]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":104}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299454,40.042503]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":105}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307541,40.039364]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":0,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.527,"distance_from_trace_point":15.726}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307434,40.039375]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1,"matched_point_type":"interpolated","edge_index":0,"distance_along_edge":0.634,"distance_from_trace_point":13.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307343,40.039387]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.725,"distance_from_trace_point":7.701}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307213,40.039398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":3,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.855,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.307106,40.039410]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":4,"matched_point_type":"interpolated","edge_index":0,"distance_along_edge":0.962,"distance_from_trace_point":4.267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306999,40.039421]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":5,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.063,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306892,40.039436]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":6,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.162,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306786,40.039448]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":7,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.261,"distance_from_trace_point":6.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306686,40.039459]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":8,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.352,"distance_from_trace_point":11.552}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306587,40.039471]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":9,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.444,"distance_from_trace_point":14.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306480,40.039482]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":10,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.542,"distance_from_trace_point":12.801}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306374,40.039497]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":11,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.641,"distance_from_trace_point":14.053}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306259,40.039509]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":12,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.746,"distance_from_trace_point":17.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306145,40.039524]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":13,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.852,"distance_from_trace_point":12.801}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306053,40.039536]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":14,"matched_point_type":"interpolated","edge_index":1,"distance_along_edge":0.937,"distance_from_trace_point":9.369}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306015,40.039684]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":15,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.090,"distance_from_trace_point":1.292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306030,40.039780]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":16,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.150,"distance_from_trace_point":3.257}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306046,40.039860]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":17,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.200,"distance_from_trace_point":2.618}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306061,40.039944]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":18,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.254,"distance_from_trace_point":2.618}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306076,40.040016]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":19,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.300,"distance_from_trace_point":6.513}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306091,40.040108]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":20,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.357,"distance_from_trace_point":5.235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306107,40.040188]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":21,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.408,"distance_from_trace_point":6.513}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306129,40.040276]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":22,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.464,"distance_from_trace_point":7.216}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306145,40.040371]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":23,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.524,"distance_from_trace_point":6.513}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306160,40.040451]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":24,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.575,"distance_from_trace_point":5.235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306175,40.040535]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":25,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.628,"distance_from_trace_point":3.898}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306190,40.040619]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":26,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.681,"distance_from_trace_point":6.513}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306213,40.040710]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":27,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.739,"distance_from_trace_point":7.216}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306229,40.040794]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":28,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.792,"distance_from_trace_point":7.216}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306236,40.040859]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":29,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.833,"distance_from_trace_point":1.292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306252,40.040939]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":30,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.884,"distance_from_trace_point":3.257}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306267,40.041019]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":31,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.935,"distance_from_trace_point":0.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306282,40.041065]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":32,"matched_point_type":"interpolated","edge_index":2,"distance_along_edge":0.964,"distance_from_trace_point":9.130}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306114,40.041142]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":33,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.174,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.306000,40.041153]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":34,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.288,"distance_from_trace_point":7.199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305893,40.041164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":35,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.394,"distance_from_trace_point":11.880}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305809,40.041172]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":36,"matched_point_type":"interpolated","edge_index":3,"distance_along_edge":0.477,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305687,40.041183]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":37,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.598,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305634,40.041191]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":38,"matched_point_type":"interpolated","edge_index":3,"distance_along_edge":0.652,"distance_from_trace_point":14.053}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305573,40.041195]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":39,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.712,"distance_from_trace_point":12.384}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305481,40.041206]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":40,"matched_point_type":"interpolated","edge_index":3,"distance_along_edge":0.803,"distance_from_trace_point":7.701}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305428,40.041210]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":41,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.856,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305351,40.041218]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":42,"matched_point_type":"interpolated","edge_index":3,"distance_along_edge":0.932,"distance_from_trace_point":8.951}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305252,40.041229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":43,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.030,"distance_from_trace_point":5.940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305145,40.041241]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":44,"matched_point_type":"interpolated","edge_index":4,"distance_along_edge":0.134,"distance_from_trace_point":9.787}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.305038,40.041252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":45,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.237,"distance_from_trace_point":10.205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304932,40.041264]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":46,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.341,"distance_from_trace_point":14.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304817,40.041275]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":47,"matched_point_type":"interpolated","edge_index":4,"distance_along_edge":0.452,"distance_from_trace_point":14.471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304718,40.041283]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":48,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.548,"distance_from_trace_point":9.787}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304611,40.041294]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":49,"matched_point_type":"interpolated","edge_index":4,"distance_along_edge":0.652,"distance_from_trace_point":8.117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304489,40.041309]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":50,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.771,"distance_from_trace_point":8.117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304382,40.041321]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":51,"matched_point_type":"interpolated","edge_index":4,"distance_along_edge":0.874,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304291,40.041328]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":52,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.963,"distance_from_trace_point":11.461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304169,40.041340]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":53,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.084,"distance_from_trace_point":9.787}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.304062,40.041351]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":54,"matched_point_type":"interpolated","edge_index":5,"distance_along_edge":0.192,"distance_from_trace_point":9.787}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303963,40.041363]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":55,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.292,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303841,40.041374]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":56,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.415,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303726,40.041386]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":57,"matched_point_type":"interpolated","edge_index":5,"distance_along_edge":0.531,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303612,40.041397]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":58,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.646,"distance_from_trace_point":2.612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303505,40.041409]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":59,"matched_point_type":"interpolated","edge_index":5,"distance_along_edge":0.754,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303368,40.041424]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":60,"matched_point_type":"matched","edge_index":5,"distance_along_edge":0.892,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303261,40.041435]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":61,"matched_point_type":"matched","edge_index":5,"distance_along_edge":1.000,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303154,40.041447]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":62,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.032,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.303040,40.041458]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":63,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.066,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302948,40.041466]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":64,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.093,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302818,40.041477]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":65,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.131,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302704,40.041489]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":66,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.165,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302589,40.041504]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":67,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.200,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302490,40.041512]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":68,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.229,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302368,40.041523]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":69,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.265,"distance_from_trace_point":7.199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302269,40.041534]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":70,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.295,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302155,40.041546]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":71,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.329,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.302063,40.041553]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":72,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.356,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301956,40.041565]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":73,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.388,"distance_from_trace_point":3.022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301857,40.041576]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":74,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.417,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301743,40.041588]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":75,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.451,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301643,40.041599]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":76,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.481,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301514,40.041611]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":77,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.519,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301392,40.041626]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":78,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.555,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301277,40.041637]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":79,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.589,"distance_from_trace_point":3.435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301178,40.041649]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":80,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.619,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.301056,40.041660]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":81,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.655,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300957,40.041672]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":82,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.685,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300835,40.041683]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":83,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.721,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300720,40.041695]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":84,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.755,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300598,40.041710]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":85,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.791,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300491,40.041721]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":86,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.823,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300392,40.041729]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":87,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.852,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300270,40.041744]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":88,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.889,"distance_from_trace_point":6.779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300156,40.041756]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":89,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.923,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300041,40.041767]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":90,"matched_point_type":"interpolated","edge_index":6,"distance_along_edge":0.957,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299934,40.041779]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":91,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.989,"distance_from_trace_point":5.102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299896,40.041794]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":92,"matched_point_type":"interpolated","edge_index":7,"distance_along_edge":0.017,"distance_from_trace_point":3.256}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299911,40.041874]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":93,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.141,"distance_from_trace_point":2.617}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299934,40.041965]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":94,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.283,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299950,40.042046]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":95,"matched_point_type":"interpolated","edge_index":7,"distance_along_edge":0.407,"distance_from_trace_point":3.256}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299965,40.042130]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":96,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.536,"distance_from_trace_point":1.292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299980,40.042213]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":97,"matched_point_type":"interpolated","edge_index":7,"distance_along_edge":0.665,"distance_from_trace_point":0.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299995,40.042297]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":98,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.794,"distance_from_trace_point":1.292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.300011,40.042381]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":99,"matched_point_type":"matched","edge_index":7,"distance_along_edge":0.924,"distance_from_trace_point":3.256}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299881,40.042446]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":100,"matched_point_type":"interpolated","edge_index":8,"distance_along_edge":0.118,"distance_from_trace_point":8.117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299782,40.042454]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":101,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.202,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299675,40.042465]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":102,"matched_point_type":"interpolated","edge_index":8,"distance_along_edge":0.294,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299561,40.042477]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":103,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.392,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299454,40.042488]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":104,"matched_point_type":"interpolated","edge_index":8,"distance_along_edge":0.484,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.299454,40.042488]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":105,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.484,"distance_from_trace_point":1.687}}
]}
 */

void TestPedestrianCatalinasToOffice() {
  std::string route_json_str =
      R"({"trip":{"language":"en-US","summary":{"max_lon":-76.299454,"max_lat":40.042488,"time":694,"length":0.611,"min_lat":40.039364,"min_lon":-76.307541},"locations":[{"lon":-76.307518,"lat":40.039223,"type":"break"},{"lon":-76.299454,"lat":40.042503,"type":"break"}],"units":"miles","legs":[{"shape":"e{xjkAhdmppCcBq\\cGubAuaB`RmE}}@uEk_AmE_}@aGopAsLw_Cog@rFsBib@","summary":{"max_lon":-76.299454,"max_lat":40.042488,"time":694,"length":0.611,"min_lat":40.039364,"min_lon":-76.307541},"maneuvers":[{"travel_mode":"pedestrian","begin_shape_index":0,"length":0.083,"time":94,"type":1,"end_shape_index":2,"instruction":"Walk east on West Orange Street.","verbal_pre_transition_instruction":"Walk east on West Orange Street for 400 feet.","travel_type":"foot","street_names":["West Orange Street"]},{"travel_type":"foot","travel_mode":"pedestrian","verbal_pre_transition_instruction":"Turn left onto North Queen Street, Pennsylvania 72 North.","verbal_transition_alert_instruction":"Turn left onto North Queen Street.","length":0.111,"instruction":"Turn left onto North Queen Street\/PA 72 North.","end_shape_index":3,"type":15,"time":126,"verbal_post_transition_instruction":"Continue for 1 tenth of a mile.","street_names":["North Queen Street","PA 72 North"],"begin_shape_index":2},{"travel_type":"foot","travel_mode":"pedestrian","verbal_pre_transition_instruction":"Turn right onto East Chestnut Street, Pennsylvania 23 East.","verbal_transition_alert_instruction":"Turn right onto East Chestnut Street.","length":0.342,"instruction":"Turn right onto East Chestnut Street\/PA 23 East.","end_shape_index":8,"type":10,"time":389,"verbal_post_transition_instruction":"Continue for 3 tenths of a mile.","street_names":["East Chestnut Street","PA 23 East"],"begin_shape_index":3},{"travel_type":"foot","travel_mode":"pedestrian","verbal_pre_transition_instruction":"Turn left onto North Shippen Street.","verbal_transition_alert_instruction":"Turn left onto North Shippen Street.","length":0.045,"instruction":"Turn left onto North Shippen Street.","end_shape_index":9,"type":15,"time":51,"verbal_post_transition_instruction":"Continue for 200 feet.","street_names":["North Shippen Street"],"begin_shape_index":8},{"travel_type":"foot","travel_mode":"pedestrian","verbal_pre_transition_instruction":"Turn right onto East Fulton Street.","verbal_transition_alert_instruction":"Turn right onto East Fulton Street.","length":0.030,"instruction":"Turn right onto East Fulton Street.","end_shape_index":10,"type":10,"time":34,"verbal_post_transition_instruction":"Continue for 200 feet.","street_names":["East Fulton Street"],"begin_shape_index":9},{"travel_type":"foot","travel_mode":"pedestrian","begin_shape_index":10,"time":0,"type":4,"end_shape_index":10,"instruction":"You have arrived at your destination.","length":0.000,"verbal_transition_alert_instruction":"You will arrive at your destination.","verbal_pre_transition_instruction":"You have arrived at your destination."}]}],"status_message":"Found route between points","status":0}})";
  NavigatorTest nav;
  nav.SetRoute(route_json_str);
  uint32_t leg_index = 0;

  TryRouteLegCount(nav, 1);
  TryRouteManeuverCount(nav, leg_index, 6);
  TryRouteLanguage(nav, "en-US");
  TryRouteUnits(nav, "miles", false);

  ////////////////////////////////////////////////////////////////////////////
  uint32_t maneuver_index = 0;
  uint32_t instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////
  bool expected_on_route_location_close_to_origin = true;

  // reset the route
  nav.SetRoute(route_json_str);

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // Start maneuver at beginning of route
  // trace_pt[0] | begin of maneuver index 0
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.307518f, 40.039223f, 0),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.3075409f, 40.0393639f, leg_index, 0.610896409f,
                                                693, maneuver_index, 0.0833536983f, 95,
                                                instruction_index),
                            expected_on_route_location_close_to_origin);

  // kPreTransition instruction should be used after OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  // reset the route
  nav.SetRoute(route_json_str);

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // Start maneuver near beginning of route
  // trace_pt[1] | near begin of maneuver index 0
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.307411f, 40.039257f, 7),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.3074341f, 40.0393753f, leg_index, 0.605184913f,
                                                686, maneuver_index, 0.0776422024f, 88,
                                                instruction_index),
                            expected_on_route_location_close_to_origin);

  // kPreTransition instruction should be used after OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  // reset the route
  nav.SetRoute(route_json_str);

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // No kPreTransition because not within time threshold for pre transition
  // trace_pt[3] | near begin of maneuver index 0 | maneuver speed ~ 3.1787231 MPH
  expected_on_route_location_close_to_origin = false;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.307205f, 40.039345f, 20),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3072128f,
                                                40.0393982f, leg_index, 0.593361795f, 673,
                                                maneuver_index, 0.0658190846f, 75));

  // kPreTransition instruction should be not used after OnLocationChanged
  // because not within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // trace_pt[12] | near end of maneuver index 0 | fix speed ~ 5.5 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.30629f, 40.0396659f, 78, 2.5f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3062592f,
                                                40.0395088f, leg_index, 0.542280436f, 615,
                                                maneuver_index, 0.0147377253f, 17));

  // kPreTransition instruction should not be used after OnLocationChanged
  // because it is not within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // trace_pt[13] | near end of maneuver index 0 | fix speed ~ 5.5 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.306168f, 40.039639f, 85, 2.5f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.3061447f, 40.0395241f, leg_index, 0.536129236f,
                                                608, maneuver_index, 0.00858652592f, 10,
                                                instruction_index));

  // kPreTransition instruction should be used after OnLocationChanged
  // because it was within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 1;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // trace_pt[31] | end of maneuver index 1 | fix speed ~ 2.2 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.306259f, 40.041019f, 212, 1.5f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.3062668f, 40.0410194f, leg_index, 0.424339741f,
                                                481, maneuver_index, 0.00722661614f, 8,
                                                instruction_index));

  // kPreTransition instruction should be used after OnLocationChanged
  // because it was within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 2;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // kPostTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPostTransition>(nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[33] | near begin of maneuver index 2 |  maneuver speed ~ 3.16503859 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.306122f, 40.041172f, 230),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.3061142f,
                                                40.0411415f, leg_index, 0.407727778f, 463,
                                                maneuver_index, 0.332249224f, 378));

  // kPostTransition instruction should not be used after OnLocationChanged
  // because it was less than the time threshold for post-transition
  TryUsedInstructions(std::get<kPostTransition>(nav.used_instructions().at(instruction_index)),
                      false);

  // TODO: Had to insert this point. Post does not show up on this maneuver other wise.
  // May need to increase threshold based on frequency of trace points along the path
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.30621f, 40.041148f, 230),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.3062134f, 40.0411301f, leg_index, 0.413038582f,
                                                469, maneuver_index, 0.337560028f, 384,
                                                instruction_index));

  // kPostTransition instruction should be used after OnLocationChanged
  // because it was within the time threshold for post-transition
  TryUsedInstructions(std::get<kPostTransition>(nav.used_instructions().at(instruction_index)), true);

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // kFinalTransitionAlert instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[49]  | 500 feet before end of maneuver index 3 | maneuver speed ~ 3.16503859 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.301743f, 40.041588f, 496),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.3017426f, 40.0415878f, leg_index, 0.174173057f,
                                                197, maneuver_index, 0.0986945033f, 112,
                                                instruction_index, 0.095f));

  // kFinalTransitionAlert instruction should be used after OnLocationChanged
  // because it was within length bounds for final transition alert
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      true);
}

/* RoadBikeLunchRideLoop geojson to visualize test route
{"type":"FeatureCollection","features":[
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122696,39.651386]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":0}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122551,39.651695]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122498,39.651802]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":2}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122437,39.651917]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":3}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122375,39.652031]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":4}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122314,39.652149]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":5}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122253,39.652271]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":6}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122185,39.652393]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":7}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122116,39.652519]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":8}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121979,39.652775]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":9}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121902,39.652905]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":10}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121826,39.653034]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":11}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121674,39.653290]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":12}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121597,39.653419]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":13}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121529,39.653545]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":14}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121468,39.653667]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":15}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121353,39.653919]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":16}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121307,39.654041]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":17}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121254,39.654167]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":18}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121208,39.654289]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":19}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121162,39.654415]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":20}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121078,39.654671]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":21}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121033,39.654800]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":22}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120926,39.655056]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":23}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120827,39.655304]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":24}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120773,39.655422]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":25}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120728,39.655537]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":26}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120621,39.655762]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":27}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120476,39.656075]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":28}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120323,39.656345]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":29}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120071,39.656738]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":30}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120010,39.656837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":31}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119957,39.656933]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":32}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119942,39.656979]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":33}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119919,39.657024]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":34}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119888,39.657108]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":35}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119843,39.657238]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":36}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119827,39.657322]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":37}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119804,39.657410]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":38}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119766,39.657585]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":39}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119720,39.657810]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":40}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119652,39.658215]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":41}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119614,39.658398]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":42}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119484,39.658978]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":43}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119354,39.659538]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":44}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119316,39.659634]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":45}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119072,39.660168]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":46}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118797,39.660698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":47}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118584,39.661118]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":48}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118454,39.661369]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":49}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118401,39.661469]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":50}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118370,39.661533]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":51}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118279,39.661709]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":52}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118248,39.661766]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":53}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118217,39.661819]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":54}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118172,39.661900]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":55}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118111,39.661995]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":56}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118042,39.662086]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":57}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117966,39.662205]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":58}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117912,39.662273]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":59}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117828,39.662380]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":60}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117737,39.662514]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":61}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117668,39.662624]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":62}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117615,39.662712]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":63}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117569,39.662796]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":64}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117516,39.662907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":65}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117439,39.663082]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":66}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117416,39.663147]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":67}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117393,39.663212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":68}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117348,39.663380]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":69}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117317,39.663486]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":70}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117294,39.663570]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":71}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117271,39.663700]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":72}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117249,39.663868]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":73}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117241,39.663990]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":74}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117226,39.664146]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":75}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117218,39.664326]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":76}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117210,39.664494]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":77}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117149,39.665047]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":78}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117065,39.665352]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":79}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116997,39.665539]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":80}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116936,39.665676]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":81}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116852,39.665836]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":82}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116783,39.665966]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":83}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116669,39.666161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":84}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116364,39.666702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":85}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116241,39.666920]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":86}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115936,39.667461]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":87}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115898,39.667538]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":88}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115807,39.667690]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":89}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115646,39.667988]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":90}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115349,39.668537]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":91}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115067,39.669067]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":92}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114967,39.669239]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":93}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114883,39.669392]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":94}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114853,39.669460]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":95}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114815,39.669525]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":96}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114761,39.669624]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":97}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114708,39.669720]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":98}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114639,39.669849]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":99}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114571,39.669975]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":100}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114479,39.670155]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":101}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114433,39.670235]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114388,39.670315]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":103}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114281,39.670521]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":104}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114174,39.670689]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":105}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114052,39.670826]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":106}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113831,39.670994]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113525,39.671116]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113426,39.671143]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112900,39.671215]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":110}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112350,39.671276]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":111}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112228,39.671288]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":112}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111488,39.671364]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110725,39.671444]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":114}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110481,39.671467]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":115}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110092,39.671497]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":116}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109802,39.671520]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109512,39.671543]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":118}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109200,39.671570]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":119}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109039,39.671581]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":120}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108879,39.671597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":121}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108704,39.671612]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108536,39.671623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108360,39.671638]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108192,39.671654]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108017,39.671673]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":126}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107666,39.671700]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":127}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107483,39.671715]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107307,39.671730]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107124,39.671741]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":130}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106934,39.671757]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":131}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106750,39.671776]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":132}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106567,39.671787]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":133}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106194,39.671818]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":134}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105820,39.671848]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":135}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105637,39.671864]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":136}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105453,39.671883]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":137}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105103,39.671913]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":138}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104927,39.671928]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":139}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104752,39.671940]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":140}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104584,39.671951]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":141}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104248,39.671974]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":142}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.103523,39.672035]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":143}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.102730,39.672096]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":144}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101921,39.672157]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":145}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101379,39.672207]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":146}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101112,39.672230]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":147}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.100708,39.672260]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":148}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.100189,39.672302]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":149}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.099480,39.672359]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":150}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098839,39.672379]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":151}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098724,39.672359]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":152}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098625,39.672337]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":153}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098549,39.672314]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":154}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098465,39.672291]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":155}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098389,39.672268]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":156}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098328,39.672249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":157}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098244,39.672222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":158}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098160,39.672203]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":159}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098061,39.672188]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":160}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097946,39.672192]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":161}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097839,39.672199]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":162}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097740,39.672211]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":163}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097633,39.672226]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":164}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097504,39.672241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":165}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097389,39.672256]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":166}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097290,39.672272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":167}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097160,39.672298]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":168}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097046,39.672314]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":169}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096924,39.672333]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":170}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096802,39.672356]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":171}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096565,39.672394]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":172}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096367,39.672424]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":173}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096031,39.672466]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":174}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.095276,39.672554]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":175}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.094528,39.672634]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":176}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093834,39.672707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":177}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093285,39.672768]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":178}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.092705,39.672829]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":179}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091957,39.672909]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":180}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091171,39.673000]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":181}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090408,39.673080]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":182}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089699,39.673161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089287,39.673203]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":184}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088516,39.673271]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":185}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088272,39.673290]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087898,39.673313]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087524,39.673328]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":188}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087151,39.673355]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":189}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.086792,39.673393]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":190}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.086037,39.673473]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":191}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085884,39.673485]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":192}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085686,39.673504]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085556,39.673512]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085426,39.673523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":195}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085220,39.673542]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":196}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.084625,39.673595]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":197}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083923,39.673649]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":198}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083313,39.673698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083069,39.673717]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":200}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.082634,39.673748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":201}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081909,39.673801]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":202}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081207,39.673843]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":203}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080605,39.673885]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":204}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080193,39.673916]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079468,39.673965]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078758,39.674019]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078690,39.674023]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078194,39.674057]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.077499,39.674114]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":210}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.077057,39.674152]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":211}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.076302,39.674221]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":212}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.075592,39.674282]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":213}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074883,39.674343]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074150,39.674377]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":215}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073395,39.674431]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":216}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073296,39.674438]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":217}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073189,39.674446]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072983,39.674461]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072556,39.674488]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":220}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071815,39.674553]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071358,39.674599]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.070602,39.674664]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069832,39.674740]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":224}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069725,39.674751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069008,39.674824]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":226}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068504,39.674881]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":227}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068344,39.674896]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":228}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068199,39.674911]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":229}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068047,39.674923]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":230}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067856,39.674942]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":231}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067574,39.674973]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":232}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067352,39.674995]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":233}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067093,39.675018]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":234}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.066383,39.675083]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.065628,39.675167]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":236}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.065102,39.675220]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":237}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.064705,39.675255]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":238}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.064278,39.675297]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":239}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063591,39.675365]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":240}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063240,39.675407]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":241}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062889,39.675449]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":242}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062660,39.675476]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":243}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062347,39.675495]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.061745,39.675537]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060966,39.675610]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":246}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060272,39.675716]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":247}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060089,39.675770]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":248}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.059486,39.676094]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":249}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.059357,39.676182]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":250}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.058701,39.676487]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":251}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.057953,39.676598]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":252}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.057259,39.676670]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":253}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.056725,39.676716]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":254}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.055923,39.676781]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":255}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.055550,39.676815]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":256}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.054916,39.676884]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":257}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.054672,39.676910]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":258}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053978,39.676975]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053246,39.677048]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":260}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.052467,39.677128]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":261}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051727,39.677185]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":262}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050926,39.677254]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":263}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050209,39.677334]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":264}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049942,39.677368]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049461,39.677444]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":266}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048958,39.677528]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":267}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048576,39.677605]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":268}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048157,39.677673]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":269}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048103,39.677696]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":270}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047913,39.677876]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047882,39.677917]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047531,39.678391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":273}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047165,39.678875]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":274}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046860,39.679405]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":275}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046593,39.679939]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":276}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046333,39.680473]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":277}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046059,39.681023]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":278}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045944,39.681236]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":279}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045677,39.681767]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":280}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045486,39.682259]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":281}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045532,39.682526]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":282}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045555,39.682590]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":283}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045807,39.683117]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":284}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046074,39.683647]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":285}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046135,39.683765]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":286}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046188,39.683887]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":287}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046219,39.683949]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":288}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046280,39.684071]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":289}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046356,39.684231]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":290}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046417,39.684364]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":291}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046532,39.684620]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":292}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046768,39.685154]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":293}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046890,39.685421]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":294}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047104,39.685905]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":295}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047180,39.686073]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":296}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047333,39.686451]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":297}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047340,39.687000]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047142,39.687515]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046928,39.687912]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":300}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046631,39.688461]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":301}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046341,39.688999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":302}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046120,39.689411]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":303}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045883,39.689846]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":304}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045692,39.690193]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":305}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045647,39.690266]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":306}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045387,39.690315]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":307}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044662,39.690212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":308}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043961,39.690109]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":309}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043808,39.690090]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":310}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043114,39.689995]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":311}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.042427,39.689857]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":312}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041763,39.689632]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":313}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041626,39.689571]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":314}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041008,39.689228]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":315}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040421,39.688854]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":316}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039841,39.688492]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":317}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039291,39.688145]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":318}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039124,39.688026]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":319}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038841,39.687984]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":320}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038696,39.688004]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":321}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038551,39.688023]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":322}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037918,39.688099]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":323}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037529,39.688145]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":324}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036835,39.688259]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":325}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036133,39.688305]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":326}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035751,39.688290]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":327}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035278,39.688259]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":328}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035080,39.688248]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":329}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.034317,39.688198]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":330}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.033760,39.688164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":331}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.033112,39.688122]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":332}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.032440,39.688072]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":333}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.032135,39.688061]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":334}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031822,39.688065]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":335}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031494,39.688091]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":336}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031158,39.688122]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":337}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030807,39.688160]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":338}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030434,39.688194]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":339}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030182,39.688217]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":340}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029533,39.688286]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":341}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029282,39.688309]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":342}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029015,39.688332]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":343}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028488,39.688377]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":344}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027977,39.688427]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":345}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027725,39.688450]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":346}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027237,39.688496]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":347}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.026489,39.688564]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":348}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.026367,39.688576]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":349}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.025604,39.688648]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":350}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.025085,39.688698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":351}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.024300,39.688763]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":352}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023895,39.688789]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":353}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023521,39.688828]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":354}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023293,39.688862]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023178,39.688889]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":356}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022781,39.689041]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":357}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022209,39.689404]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":358}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022087,39.689484]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":359}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.021790,39.689617]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.021538,39.689682]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":361}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.021286,39.689739]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":362}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.020981,39.689808]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":363}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.020523,39.689907]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":364}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.020134,39.689999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":365}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.019676,39.690109]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018936,39.690311]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":367}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018539,39.690422]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":368}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018211,39.690514]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":369}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017990,39.690575]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":370}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017647,39.690678]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":371}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017426,39.690762]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":372}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017326,39.690815]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":373}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017067,39.691017]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016930,39.691174]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":375}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016754,39.691410]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":376}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016533,39.691715]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":377}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016220,39.692249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":378}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016205,39.692333]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":379}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016167,39.692657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":380}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016174,39.692894]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":381}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016212,39.693462]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":382}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016220,39.693699]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":383}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016235,39.693897]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":384}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016243,39.693951]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":385}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016243,39.694111]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":386}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016243,39.694683]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":387}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016197,39.695061]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015953,39.695522]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":389}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015747,39.695728]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":390}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015129,39.696087]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":391}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015038,39.696125]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":392}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014664,39.696262]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":393}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014046,39.696621]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":394}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013664,39.697113]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":395}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013603,39.697205]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":396}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013527,39.697330]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":397}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013451,39.697449]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":398}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013336,39.697636]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":399}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013268,39.697735]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":400}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013161,39.697899]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":401}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013031,39.698105]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":402}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012772,39.698502]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":403}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012505,39.699032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":404}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012489,39.699070]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":405}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012352,39.699516]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":406}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012222,39.700001]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":407}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012093,39.700531]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":408}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011978,39.700977]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":409}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011833,39.701546]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":410}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011681,39.702114]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":411}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011536,39.702675]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":412}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011421,39.703091]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":413}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011353,39.703354]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":414}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011292,39.703651]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":415}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011215,39.703819]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":416}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011139,39.703964]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":417}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011177,39.704533]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":418}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011200,39.704693]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":419}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011276,39.705257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":420}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011330,39.705856]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011391,39.706398]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011391,39.706459]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":423}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011459,39.707024]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":424}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011505,39.707603]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":425}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011536,39.707848]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":426}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011566,39.708115]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":427}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011581,39.708305]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":428}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011650,39.708855]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":429}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011696,39.709259]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":430}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011749,39.709660]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":431}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011772,39.709923]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":432}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011826,39.710487]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":433}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011879,39.711056]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011940,39.711639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":435}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011955,39.711781]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011986,39.712002]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":437}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012009,39.712154]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":438}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012032,39.712315]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":439}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012062,39.712570]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":440}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012108,39.712917]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012131,39.713291]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":442}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012138,39.713390]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":443}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012146,39.713493]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":444}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012169,39.713783]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":445}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012184,39.713970]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":446}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012207,39.714142]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":447}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012283,39.714684]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":448}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012360,39.715256]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":449}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012375,39.715370]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":450}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012390,39.715527]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012436,39.715809]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":452}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012459,39.716030]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":453}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012474,39.716164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":454}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012505,39.716373]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":455}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012520,39.716534]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":456}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012550,39.716713]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":457}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012573,39.716888]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":458}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012589,39.716980]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":459}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012596,39.717068]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":460}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012611,39.717186]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012619,39.717270]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":462}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012619,39.717392]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":463}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012672,39.717743]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":464}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012695,39.717972]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":465}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012718,39.718140]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":466}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012741,39.718483]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":467}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012794,39.718998]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":468}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012848,39.719551]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":469}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012909,39.720112]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":470}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012970,39.720654]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":471}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013046,39.721252]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":472}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013077,39.721516]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":473}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013123,39.721855]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":474}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013168,39.722164]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":475}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013199,39.722412]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013222,39.722580]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013237,39.722744]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":478}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013260,39.722900]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":479}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013290,39.723198]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":480}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013344,39.723660]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":481}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013412,39.724243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":482}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013435,39.724487]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013489,39.724884]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013557,39.725483]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013618,39.726032]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":486}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013626,39.726162]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":487}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013695,39.726707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":488}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013725,39.727013]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":489}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013756,39.727272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":490}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013779,39.727417]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":491}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013786,39.727516]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":492}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013802,39.727650]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":493}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013824,39.727783]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":494}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013847,39.728008]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":495}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013908,39.728561]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":496}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013916,39.728607]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013947,39.729012]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":498}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013969,39.729214]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":499}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014030,39.729794]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":500}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014084,39.730270]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":501}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014137,39.730820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":502}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014214,39.731415]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":503}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014259,39.731869]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":504}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014282,39.732059]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":505}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014313,39.732304]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":506}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014343,39.732609]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":507}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014381,39.733173]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":508}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014496,39.733723]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":509}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014702,39.734058]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":510}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014687,39.734371]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":511}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014542,39.734837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":512}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014557,39.735233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":513}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014595,39.735428]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":514}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014618,39.735523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":515}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014626,39.735622]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":516}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014641,39.735714]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":517}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014648,39.735809]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":518}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014633,39.736076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":519}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014526,39.736622]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":520}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014465,39.736870]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014442,39.736958]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":522}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014427,39.737041]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":523}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014389,39.737160]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":524}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014336,39.737309]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":525}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014244,39.737572]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":526}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014153,39.737831]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":527}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014030,39.738140]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":528}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013962,39.738335]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":529}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013885,39.738541]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":530}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013702,39.739079]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013626,39.739586]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":532}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013969,39.740063]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":533}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014374,39.740528]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":534}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014771,39.740990]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":535}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015144,39.741409]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":536}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015259,39.741543]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":537}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015236,39.741943]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":538}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015068,39.742214]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":539}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014908,39.742474]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":540}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014771,39.742699]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":541}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014671,39.742855]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":542}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014572,39.743015]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014496,39.743141]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":544}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014366,39.743343]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":545}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014267,39.743511]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":546}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014076,39.743820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":547}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013947,39.744022]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":548}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013802,39.744259]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013680,39.744461]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":550}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013573,39.744640]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":551}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013489,39.744774]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":552}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013435,39.744850]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":553}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013390,39.744923]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":554}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013321,39.745022]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013245,39.745152]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":556}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013168,39.745270]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":557}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013023,39.745468]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":558}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012665,39.745930]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":559}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012444,39.746212]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":560}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012337,39.746361]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":561}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012245,39.746490]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012085,39.746586]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":563}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011490,39.746307]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011238,39.746185]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":565}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010872,39.746014]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":566}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010475,39.745823]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009941,39.745575]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":568}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009727,39.745472]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":569}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009521,39.745377]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":570}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009201,39.745228]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":571}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.008591,39.744938]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":572}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.007896,39.744606]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":573}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.007256,39.744293]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":574}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.006615,39.743992]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":575}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005943,39.743671]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":576}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005295,39.743359]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004951,39.743195]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":578}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004333,39.742905]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":579}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004150,39.742821]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":580}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.003510,39.742523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":581}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.002869,39.742233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":582}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.002373,39.742027]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":583}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001732,39.741745]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":584}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001518,39.741650]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":585}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001297,39.741558]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":586}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000664,39.741283]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":587}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999992,39.740963]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":588}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999557,39.740681]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":589}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999336,39.740536]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":590}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999123,39.740391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":591}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998909,39.740250]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":592}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998589,39.740036]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":593}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997963,39.739609]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":594}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997536,39.739326]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997124,39.739052]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996918,39.738922]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":597}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996719,39.738792]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":598}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996155,39.738426]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":599}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995544,39.738033]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":600}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994965,39.737648]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":601}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994415,39.737286]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":602}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994324,39.737221]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":603}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994133,39.737091]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":604}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993759,39.736828]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":605}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993492,39.736629]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":606}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992928,39.736206]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":607}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992455,39.735889]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":608}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992393,39.735851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":609}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992210,39.735760]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":610}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992050,39.735680]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":611}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991943,39.735634]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":612}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991798,39.735569]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991653,39.735512]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":614}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991432,39.735420]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991264,39.735355]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991135,39.735302]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":617}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990997,39.735249]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":618}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990448,39.735031]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":619}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990402,39.735012]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":620}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990097,39.734894]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":621}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989853,39.734802]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":622}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989471,39.734688]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":623}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989426,39.734669]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":624}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989357,39.734516]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":625}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989326,39.734341]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":626}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989296,39.734268]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":627}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989037,39.733791]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":628}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989006,39.733704]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988983,39.733521]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988968,39.733337]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":631}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988945,39.733063]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":632}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988914,39.732517]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":633}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988899,39.732330]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":634}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988853,39.731781]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":635}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988823,39.731506]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":636}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988815,39.731220]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":637}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989105,39.730793]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":638}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989204,39.730732]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":639}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989410,39.730614]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989616,39.730499]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":641}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989815,39.730385]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":642}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990433,39.730083]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":643}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991165,39.729851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":644}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991852,39.729660]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992531,39.729469]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992981,39.729340]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":647}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993454,39.729202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":648}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993790,39.729111]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994461,39.728920]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":650}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995110,39.728703]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":651}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995636,39.728333]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":652}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996231,39.727943]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":653}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996445,39.727837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":654}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996552,39.727779]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":655}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996643,39.727703]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":656}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996742,39.727505]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":657}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996758,39.727283]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":658}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996742,39.727173]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":659}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996727,39.727062]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996712,39.726952]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":661}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996689,39.726837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":662}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996666,39.726612]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":663}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996651,39.726383]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":664}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996643,39.726269]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":665}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996613,39.726028]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":666}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996590,39.725796]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":667}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996567,39.725567]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":668}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996536,39.725342]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":669}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996490,39.724915]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":670}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996407,39.724304]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":671}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996353,39.723721]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":672}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996346,39.723625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":673}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996284,39.723072]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":674}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996223,39.722523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":675}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996208,39.722389]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":676}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996185,39.722179]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":677}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996124,39.721577]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":678}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996056,39.721035]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":679}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996010,39.720627]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":680}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995956,39.720097]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":681}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995941,39.719940]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":682}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995941,39.719891]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":683}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995949,39.719791]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995949,39.719639]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":685}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995949,39.719490]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":686}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995941,39.719307]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995918,39.719120]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":688}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995888,39.718784]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":689}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995850,39.718437]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":690}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995811,39.717899]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":691}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995811,39.717850]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":692}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995789,39.717613]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":693}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995758,39.717033]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":694}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995720,39.716457]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":695}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995674,39.715958]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":696}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995651,39.715660]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":697}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995621,39.715343]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":698}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995583,39.714912]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":699}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995544,39.714420]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995522,39.714020]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":701}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995506,39.713810]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":702}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995499,39.713707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":703}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995491,39.713402]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":704}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995476,39.712898]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":705}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995445,39.712326]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":706}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995422,39.712147]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":707}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995377,39.711563]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":708}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995361,39.711227]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":709}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995354,39.711052]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":710}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995338,39.710880]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":711}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995331,39.710766]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":712}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995300,39.710159]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":713}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995270,39.709564]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":714}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995247,39.709309]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":715}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995224,39.708977]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":716}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995163,39.708431]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":717}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995132,39.707825]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":718}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995171,39.707218]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":719}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995178,39.707108]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":720}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995216,39.706886]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":721}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995247,39.706673]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":722}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995270,39.706570]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":723}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995308,39.706375]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":724}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995338,39.706181]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":725}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995361,39.705803]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":726}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995392,39.705395]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":727}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995430,39.705109]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":728}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995445,39.704967]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":729}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995453,39.704830]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":730}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995506,39.704288]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995522,39.704163]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":732}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995544,39.703739]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":733}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995483,39.703590]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":734}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995461,39.703552]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":735}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995430,39.703243]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":736}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995399,39.702682]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":737}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995338,39.702091]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":738}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995300,39.701591]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":739}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995255,39.701054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":740}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995239,39.700878]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":741}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995209,39.700668]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":742}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995193,39.700527]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995178,39.700401]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":744}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995155,39.700272]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":745}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995094,39.699711]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":746}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995033,39.699089]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":747}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995018,39.698986]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":748}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995003,39.698780]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":749}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994987,39.698570]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":750}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994972,39.698360]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":751}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994965,39.698154]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":752}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994949,39.697948]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":753}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994926,39.697651]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":754}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994865,39.697102]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":755}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994797,39.696503]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":756}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994766,39.696278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":757}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994751,39.696133]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":758}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994743,39.696045]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":759}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994736,39.695965]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":760}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994736,39.695850]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":761}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994705,39.695679]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":762}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994675,39.695362]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":763}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994598,39.694820]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":764}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994560,39.694550]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":765}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994553,39.694511]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":766}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994598,39.694244]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":767}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995285,39.693985]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":768}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995728,39.693840]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":769}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996399,39.693615]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":770}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997108,39.693386]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":771}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997185,39.693359]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":772}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997841,39.693157]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":773}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998077,39.693077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998146,39.693039]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":775}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998245,39.692730]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":776}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998230,39.692120]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":777}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998215,39.691597]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":778}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998207,39.691055]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":779}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998177,39.690426]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":780}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998169,39.690323]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":781}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998161,39.690025]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":782}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998154,39.689827]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":783}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998161,39.689728]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":784}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998161,39.689625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":785}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998177,39.689419]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":786}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998222,39.688992]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":787}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998260,39.688663]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":788}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998283,39.688450]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":789}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998314,39.688122]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":790}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998329,39.687904]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":791}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998329,39.687580]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":792}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998299,39.686985]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":793}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998299,39.686531]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":794}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998299,39.686047]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":795}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998283,39.685844]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998253,39.685665]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":797}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998238,39.685608]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":798}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998215,39.685551]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":799}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998100,39.685398]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":800}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998047,39.685352]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":801}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997894,39.685211]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":802}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997421,39.684811]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":803}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997063,39.684502]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":804}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996605,39.684040]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996185,39.683571]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":806}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995857,39.683186]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":807}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995720,39.683025]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995529,39.682789]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":809}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995399,39.682636]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":810}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994904,39.682255]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":811}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994720,39.682159]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994118,39.681862]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993683,39.681637]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":814}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993423,39.681488]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":815}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993187,39.681332]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":816}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992577,39.680939]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":817}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991997,39.680576]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":818}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991737,39.680420]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":819}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991173,39.680077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":820}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990562,39.679703]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":821}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989967,39.679340]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":822}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989609,39.679119]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":823}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989334,39.678951]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":824}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988716,39.678558]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":825}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988129,39.678192]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":826}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.987579,39.677860]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":827}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.987030,39.677490]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":828}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986732,39.677204]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":829}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986595,39.677067]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":830}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986122,39.676594]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986076,39.676533]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":832}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986168,39.676178]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":833}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986588,39.675659]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":834}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986908,39.675514]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":835}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.987602,39.675339]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":836}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988342,39.675159]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":837}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988541,39.675114]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":838}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989029,39.674984]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":839}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989517,39.674706]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":840}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990128,39.674313]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":841}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990700,39.674000]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":842}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991310,39.673717]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":843}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991974,39.673405]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992615,39.673168]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":845}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992874,39.673096]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":846}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993042,39.673050]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":847}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993149,39.673023]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":848}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993279,39.672989]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":849}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993446,39.672943]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993523,39.672924]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993668,39.672882]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":852}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993774,39.672844]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":853}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993965,39.672771]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":854}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994232,39.672661]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":855}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994888,39.672405]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995041,39.672363]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":857}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995682,39.672169]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":858}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996124,39.672039]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":859}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996445,39.671940]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":860}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996765,39.671848]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":861}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997452,39.671650]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":862}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998161,39.671387]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":863}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998505,39.671242]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":864}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998672,39.671173]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":865}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999298,39.670925]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":866}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999779,39.670723]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":867}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000023,39.670616]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":868}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000175,39.670555]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":869}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000336,39.670486]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":870}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000534,39.670410]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000755,39.670322]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":872}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001274,39.670105]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":873}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001808,39.669884]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":874}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.002480,39.669609]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":875}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.003128,39.669346]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":876}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.003792,39.669083]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":877}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004456,39.668812]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":878}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004791,39.668671]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":879}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004929,39.668610]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":880}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005066,39.668552]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":881}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005295,39.668461]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":882}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005577,39.668343]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":883}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.006233,39.668076]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":884}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.006798,39.667843]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":885}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.007156,39.667702]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":886}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.007507,39.667557]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":887}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.008163,39.667286]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.008835,39.667015]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009430,39.666725]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010017,39.666416]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":891}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010376,39.666229]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":892}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010963,39.665932]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":893}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011551,39.665627]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":894}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012131,39.665333]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012733,39.665024]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":896}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012955,39.664909]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":897}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013107,39.664837]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":898}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013351,39.664707]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":899}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013802,39.664482]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":900}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014244,39.664257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":901}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014481,39.664146]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":902}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014763,39.663990]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":903}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014946,39.663910]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":904}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015289,39.663841]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":905}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015602,39.663788]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":906}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015938,39.663723]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":907}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016457,39.663616]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":908}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016884,39.663528]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":909}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017532,39.663391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":910}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018250,39.663216]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":911}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018875,39.662937]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":912}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.019318,39.662743]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":913}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.019989,39.662479]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":914}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.020699,39.662296]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.021393,39.662155]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022125,39.662003]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022575,39.661892]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":918}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022667,39.661873]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":919}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023132,39.661751]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":920}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023514,39.661655]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":921}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.024078,39.661514]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":922}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.024727,39.661366]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":923}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.025475,39.661179]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":924}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.025993,39.661003]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":925}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.026550,39.660675]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":926}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027031,39.660255]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":927}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027184,39.660103]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":928}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027328,39.659943]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":929}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027466,39.659786]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":930}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027710,39.659458]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":931}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028069,39.658962]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":932}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028198,39.658798]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028473,39.658470]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":934}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028801,39.658161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":935}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028984,39.658016]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":936}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029266,39.657803]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":937}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029556,39.657585]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":938}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029739,39.657440]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":939}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029922,39.657303]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":940}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030098,39.657162]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":941}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030586,39.656773]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":942}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031105,39.656322]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":943}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031631,39.655865]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":944}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031853,39.655663]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":945}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.032372,39.655209]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.032867,39.654785]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.033333,39.654377]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":948}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.033844,39.653934]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":949}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.034317,39.653522]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":950}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.034859,39.653118]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":951}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035271,39.652817]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035774,39.652435]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":953}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035934,39.652321]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":954}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036255,39.652077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":955}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036491,39.651897]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":956}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036644,39.651787]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":957}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036957,39.651558]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":958}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037354,39.651257]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":959}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037598,39.651073]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":960}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037819,39.650898]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":961}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037964,39.650791]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":962}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038345,39.650864]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":963}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038429,39.650925]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":964}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038597,39.651054]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":965}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038765,39.651180]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":966}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038841,39.651241]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":967}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038910,39.651306]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":968}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039047,39.651432]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":969}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039330,39.651779]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":970}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039505,39.652046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":971}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039696,39.652332]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":972}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039795,39.652477]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":973}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039879,39.652603]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":974}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039970,39.652729]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":975}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040092,39.652843]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":976}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040268,39.652992]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040558,39.653160]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040848,39.653267]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":979}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040947,39.653301]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":980}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041092,39.653355]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":981}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041237,39.653404]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":982}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041695,39.653561]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":983}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041885,39.653625]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":984}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.042084,39.653694]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":985}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.042389,39.653797]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":986}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.042656,39.653889]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":987}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043091,39.654041]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":988}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043762,39.654278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":989}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043854,39.654312]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":990}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044159,39.654430]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":991}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044319,39.654488]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":992}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044441,39.654530]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044624,39.654594]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":994}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045181,39.654812]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":995}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045448,39.654911]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":996}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045631,39.654980]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":997}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045898,39.655087]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":998}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046074,39.655155]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":999}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046257,39.655197]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046455,39.655167]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1001}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046646,39.655052]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1002}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047234,39.654675]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1003}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047806,39.654320]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1004}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047897,39.654259]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1005}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047997,39.654202]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1006}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048256,39.654037]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1007}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048470,39.653904]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1008}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048553,39.653851]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1009}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048637,39.653801]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1010}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048721,39.653748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1011}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048805,39.653698]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1012}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048904,39.653637]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1013}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049011,39.653576]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1014}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049210,39.653454]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1015}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049767,39.653118]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1016}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050087,39.652927]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1017}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050346,39.652782]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1018}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050453,39.652737]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1019}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050545,39.652737]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1020}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050606,39.652748]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1021}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050629,39.652737]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1022}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050705,39.652657]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051178,39.652233]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1024}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051437,39.652016]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1025}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051651,39.651840]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1026}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051941,39.651608]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1027}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.052162,39.651428]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1028}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.052391,39.651245]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1029}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.052689,39.651001]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1030}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053215,39.650574]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1031}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053444,39.650391]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053940,39.649986]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1033}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.054489,39.649544]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1034}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.054893,39.649220]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1035}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.055397,39.648811]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1036}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.055931,39.648373]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1037}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.056107,39.648235]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1038}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.056610,39.647831]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1039}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.056725,39.647739]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1040}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.057213,39.647343]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1041}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.057701,39.646950]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1042}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.058144,39.646587]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1043}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.058510,39.646278]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1044}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.058945,39.645905]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1045}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.059433,39.645485]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1046}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.059906,39.645077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1047}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060425,39.644638]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1048}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060936,39.644199]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1049}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.061447,39.643761]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1050}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.061951,39.643333]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1051}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062454,39.642933]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1052}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062706,39.642754]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1053}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063049,39.642502]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1054}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063499,39.642197]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1055}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063835,39.641983]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1056}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.064407,39.641632]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1057}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.064842,39.641388]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1058}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.065475,39.641048]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1059}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.066093,39.640739]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1060}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.066658,39.640446]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1061}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.066971,39.640263]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1062}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067177,39.640148]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1063}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067436,39.639999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067741,39.639816]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068336,39.639458]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1066}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068550,39.639332]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068886,39.639137]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1068}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069145,39.638981]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1069}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069321,39.638874]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1070}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069527,39.638744]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1071}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069710,39.638641]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1072}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069954,39.638489]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1073}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.070145,39.638374]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1074}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.070404,39.638222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1075}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.070694,39.638046]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1076}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071098,39.637798]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1077}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071266,39.637730]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1078}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071800,39.637768]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1079}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072159,39.637871]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1080}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072403,39.637913]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1081}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072456,39.637901]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1082}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072617,39.637726]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1083}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072929,39.637222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1084}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073128,39.636951]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1085}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073212,39.636848]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1086}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073380,39.636646]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1087}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073608,39.636410]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1088}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073853,39.636169]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1089}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074402,39.635727]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1090}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074471,39.635670]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1091}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074699,39.635498]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1092}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074989,39.635281]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1093}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.075340,39.635021]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1094}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.075874,39.634609]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1095}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.076088,39.634445]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1096}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.076195,39.634361]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1097}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.076591,39.634068]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1098}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.077156,39.633652]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1099}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.077522,39.633377]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1100}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078079,39.632965]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1101}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078629,39.632561]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1102}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078888,39.632366]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1103}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079170,39.632156]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1104}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079269,39.632084]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1105}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079391,39.631969]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1106}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079529,39.631882]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1107}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079910,39.631599]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1108}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080139,39.631435]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080284,39.631325]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1110}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080505,39.631161]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1111}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080780,39.630955]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1112}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081223,39.630623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1113}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081566,39.630371]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1114}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081886,39.630131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1115}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.082153,39.629929]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1116}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.082649,39.629574]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1117}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083183,39.629173]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1118}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083229,39.629131]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1119}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083710,39.628780]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1120}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.084251,39.628368]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1121}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.084480,39.628204]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1122}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.084991,39.627827]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085526,39.627434]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1124}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085823,39.627205]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1125}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.086365,39.626804]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1126}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.086906,39.626415]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1127}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087418,39.626026]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087692,39.625816]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088211,39.625412]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1130}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088356,39.625298]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1131}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088654,39.625488]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1132}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088928,39.625999]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1133}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089195,39.626549]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1134}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089264,39.626850]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1135}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089310,39.627293]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1136}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089333,39.627483]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1137}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089355,39.627674]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1138}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089394,39.627964]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1139}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089432,39.628155]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1140}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089485,39.628338]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1141}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089531,39.628437]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1142}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089584,39.628529]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1143}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089691,39.628696]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1144}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089897,39.628941]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1145}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090210,39.629250]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1146}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090446,39.629498]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1147}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090691,39.629745]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1148}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090843,39.629906]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1149}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090973,39.630077]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1150}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091171,39.630672]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1151}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091309,39.631271]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1152}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091377,39.631592]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1153}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091400,39.631718]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1154}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091408,39.631760]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1155}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091431,39.631886]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1156}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091446,39.631969]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1157}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091454,39.632092]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1158}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091484,39.632339]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1159}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091461,39.632824]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1160}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091492,39.633038]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1161}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091530,39.633064]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1162}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091827,39.633110]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1163}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.092628,39.633133]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1164}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.092758,39.633137]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1165}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093018,39.633144]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1166}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093269,39.633156]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1167}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093513,39.633167]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1168}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093765,39.633175]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1169}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.094261,39.633194]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1170}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.094948,39.633224]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1171}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.095528,39.633244]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1172}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.095863,39.633251]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1173}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.095978,39.633251]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1174}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096436,39.633255]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1175}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096992,39.633270]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1176}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097778,39.633320]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1177}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097885,39.633327]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1178}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098312,39.633392]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1179}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.099007,39.633595]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1180}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.099632,39.633854]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1181}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.099724,39.633888]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1182}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.100456,39.634106]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1183}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101128,39.634281]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1184}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101868,39.634491]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1185}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.102547,39.634674]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1186}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.103203,39.634865]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1187}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.103630,39.634991]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1188}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104370,39.635216]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1189}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105087,39.635418]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1190}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105209,39.635452]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1191}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105484,39.635525]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1192}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105766,39.635601]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106049,39.635674]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106331,39.635746]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1195}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106468,39.635784]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1196}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106606,39.635818]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1197}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106743,39.635857]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1198}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106880,39.635891]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1199}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107147,39.635963]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1200}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107666,39.636105]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1201}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108185,39.636250]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1202}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108543,39.636349]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1203}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108772,39.636414]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1204}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109474,39.636623]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1205}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109726,39.636700]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109932,39.636765]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110062,39.636806]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110207,39.636848]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110352,39.636890]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1210}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111031,39.637081]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1211}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111671,39.637310]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1212}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111778,39.637360]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1213}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111969,39.637466]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112152,39.637577]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1215}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112747,39.637909]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1216}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113266,39.638138]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1217}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113411,39.638191]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113495,39.638222]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1219}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113625,39.638252]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1220}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113869,39.638294]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1221}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114563,39.638386]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1222}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114723,39.638409]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1223}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115440,39.638523]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1224}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116127,39.638672]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116547,39.638783]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1226}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116859,39.638882]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1227}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117058,39.638947]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1228}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117256,39.639011]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1229}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117821,39.639198]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1230}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118507,39.639427]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1231}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119209,39.639648]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1232}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119286,39.639675]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1233}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119926,39.639900]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1234}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120377,39.640095]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1235}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120667,39.640221]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1236}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120766,39.640263]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1237}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120949,39.640343]},"properties":{"marker-color":"#abd9e9","marker-size":"small","trace_point_index":1238}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122726,39.651394]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":0,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.435,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122574,39.651703]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.728,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122513,39.651806]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":2,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.828,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122459,39.651924]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":3,"matched_point_type":"matched","edge_index":0,"distance_along_edge":0.939,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122398,39.652039]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":4,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.087,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122337,39.652157]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":5,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.290,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122276,39.652279]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":6,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.498,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122208,39.652401]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":7,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.710,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122139,39.652527]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":8,"matched_point_type":"matched","edge_index":1,"distance_along_edge":0.928,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.122002,39.652782]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":9,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.458,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121925,39.652912]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":10,"matched_point_type":"matched","edge_index":2,"distance_along_edge":0.739,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121841,39.653042]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":11,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.006,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121704,39.653297]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":12,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.134,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121635,39.653431]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":13,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.200,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121567,39.653557]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":14,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.263,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121498,39.653679]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":15,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.324,"distance_from_trace_point":2.890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121376,39.653923]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":16,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.445,"distance_from_trace_point":1.994}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121338,39.654049]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":17,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.504,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121292,39.654175]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":18,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.565,"distance_from_trace_point":3.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121239,39.654297]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":19,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.624,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121185,39.654423]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":20,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.685,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121094,39.654675]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":21,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.806,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.121040,39.654804]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":22,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.869,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120949,39.655060]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":23,"matched_point_type":"matched","edge_index":3,"distance_along_edge":0.991,"distance_from_trace_point":1.994}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120850,39.655308]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":24,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.428,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120804,39.655430]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":25,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.656,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120758,39.655544]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":26,"matched_point_type":"matched","edge_index":4,"distance_along_edge":0.871,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120667,39.655773]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":27,"matched_point_type":"matched","edge_index":6,"distance_along_edge":0.703,"distance_from_trace_point":4.097}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120506,39.656086]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":28,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.259,"distance_from_trace_point":2.890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120331,39.656349]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":29,"matched_point_type":"matched","edge_index":8,"distance_along_edge":0.771,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120087,39.656742]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":30,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.335,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120026,39.656841]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":31,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.456,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119980,39.656940]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":32,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.572,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119957,39.656982]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":33,"matched_point_type":"interpolated","edge_index":9,"distance_along_edge":0.623,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119942,39.657028]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":34,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.675,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119911,39.657112]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":35,"matched_point_type":"interpolated","edge_index":9,"distance_along_edge":0.771,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119873,39.657242]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":36,"matched_point_type":"matched","edge_index":9,"distance_along_edge":0.918,"distance_from_trace_point":2.632}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119850,39.657326]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":37,"matched_point_type":"interpolated","edge_index":10,"distance_along_edge":0.044,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119827,39.657413]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":38,"matched_point_type":"matched","edge_index":10,"distance_along_edge":0.386,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119781,39.657585]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":39,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.017,"distance_from_trace_point":1.299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119736,39.657810]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":40,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.263,"distance_from_trace_point":1.299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119652,39.658215]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":41,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.707,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119614,39.658398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":42,"matched_point_type":"matched","edge_index":11,"distance_along_edge":0.908,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119507,39.658981]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":43,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.173,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119370,39.659542]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":44,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.369,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119331,39.659637]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":45,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.403,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119080,39.660172]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":46,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.598,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118805,39.660702]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":47,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.794,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118607,39.661125]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":48,"matched_point_type":"matched","edge_index":12,"distance_along_edge":0.948,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118484,39.661377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":49,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.063,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118439,39.661480]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":50,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.120,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118408,39.661545]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":51,"matched_point_type":"interpolated","edge_index":13,"distance_along_edge":0.157,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118317,39.661720]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":52,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.257,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118286,39.661777]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":53,"matched_point_type":"interpolated","edge_index":13,"distance_along_edge":0.290,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118256,39.661831]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":54,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.321,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118210,39.661911]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":55,"matched_point_type":"interpolated","edge_index":13,"distance_along_edge":0.368,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118149,39.662010]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":56,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.426,"distance_from_trace_point":3.659}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118080,39.662102]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":57,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.482,"distance_from_trace_point":3.659}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118004,39.662220]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":58,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.552,"distance_from_trace_point":3.659}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117950,39.662292]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":59,"matched_point_type":"interpolated","edge_index":13,"distance_along_edge":0.596,"distance_from_trace_point":3.872}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117874,39.662399]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":60,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.661,"distance_from_trace_point":4.430}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117783,39.662533]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":61,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.741,"distance_from_trace_point":4.430}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117706,39.662640]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":62,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.806,"distance_from_trace_point":3.659}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117653,39.662724]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":63,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.855,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117607,39.662807]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":64,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.903,"distance_from_trace_point":3.485}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117554,39.662914]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":65,"matched_point_type":"matched","edge_index":13,"distance_along_edge":0.964,"distance_from_trace_point":3.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117485,39.663094]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":66,"matched_point_type":"matched","edge_index":14,"distance_along_edge":0.904,"distance_from_trace_point":4.097}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117462,39.663155]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":67,"matched_point_type":"interpolated","edge_index":15,"distance_along_edge":0.029,"distance_from_trace_point":3.987}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117439,39.663219]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":68,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.068,"distance_from_trace_point":3.987}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117386,39.663387]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":69,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.166,"distance_from_trace_point":3.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117348,39.663494]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":70,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.230,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117325,39.663574]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":71,"matched_point_type":"interpolated","edge_index":15,"distance_along_edge":0.277,"distance_from_trace_point":2.632}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117287,39.663700]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":72,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.351,"distance_from_trace_point":1.299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117264,39.663868]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":73,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.447,"distance_from_trace_point":1.299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117256,39.663990]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":74,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.517,"distance_from_trace_point":1.299}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117249,39.664146]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":75,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.607,"distance_from_trace_point":1.948}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117241,39.664326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":76,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.709,"distance_from_trace_point":1.948}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117233,39.664494]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":77,"matched_point_type":"matched","edge_index":15,"distance_along_edge":0.805,"distance_from_trace_point":1.948}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117172,39.665051]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":78,"matched_point_type":"matched","edge_index":16,"distance_along_edge":0.585,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117088,39.665356]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":79,"matched_point_type":"matched","edge_index":17,"distance_along_edge":0.516,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117012,39.665543]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":80,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.114,"distance_from_trace_point":1.365}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116959,39.665680]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":81,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.447,"distance_from_trace_point":1.993}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116875,39.665844]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":82,"matched_point_type":"matched","edge_index":18,"distance_along_edge":0.856,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116806,39.665974]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":83,"matched_point_type":"matched","edge_index":19,"distance_along_edge":0.207,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116699,39.666168]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":84,"matched_point_type":"matched","edge_index":19,"distance_along_edge":0.768,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116394,39.666714]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":85,"matched_point_type":"matched","edge_index":20,"distance_along_edge":0.946,"distance_from_trace_point":2.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116272,39.666927]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":86,"matched_point_type":"matched","edge_index":22,"distance_along_edge":0.550,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115967,39.667473]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":87,"matched_point_type":"matched","edge_index":24,"distance_along_edge":0.555,"distance_from_trace_point":2.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115929,39.667545]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":88,"matched_point_type":"interpolated","edge_index":24,"distance_along_edge":0.742,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115837,39.667702]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":89,"matched_point_type":"matched","edge_index":26,"distance_along_edge":0.136,"distance_from_trace_point":2.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115677,39.667999]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":90,"matched_point_type":"matched","edge_index":27,"distance_along_edge":0.336,"distance_from_trace_point":2.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115364,39.668545]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":91,"matched_point_type":"matched","edge_index":30,"distance_along_edge":0.905,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115089,39.669075]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":92,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.249,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114998,39.669247]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":93,"matched_point_type":"matched","edge_index":33,"distance_along_edge":0.665,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114914,39.669399]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":94,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.045,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114883,39.669468]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":95,"matched_point_type":"interpolated","edge_index":34,"distance_along_edge":0.243,"distance_from_trace_point":2.731}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114845,39.669537]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":96,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.448,"distance_from_trace_point":2.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114792,39.669636]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":97,"matched_point_type":"matched","edge_index":34,"distance_along_edge":0.741,"distance_from_trace_point":2.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114746,39.669724]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":98,"matched_point_type":"matched","edge_index":34,"distance_along_edge":1.000,"distance_from_trace_point":3.274}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114677,39.669861]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":99,"matched_point_type":"matched","edge_index":35,"distance_along_edge":0.852,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114609,39.669987]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":100,"matched_point_type":"matched","edge_index":36,"distance_along_edge":0.743,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114517,39.670166]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":101,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.169,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114471,39.670246]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":102,"matched_point_type":"interpolated","edge_index":38,"distance_along_edge":0.576,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114426,39.670326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":103,"matched_point_type":"matched","edge_index":38,"distance_along_edge":0.984,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114311,39.670532]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":104,"matched_point_type":"matched","edge_index":39,"distance_along_edge":0.680,"distance_from_trace_point":2.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114204,39.670700]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":105,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.192,"distance_from_trace_point":2.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114067,39.670841]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":106,"matched_point_type":"matched","edge_index":40,"distance_along_edge":0.598,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113846,39.671009]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":107,"matched_point_type":"matched","edge_index":41,"distance_along_edge":0.202,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113533,39.671139]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":108,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.024,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113426,39.671154]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":109,"matched_point_type":"interpolated","edge_index":42,"distance_along_edge":0.134,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112900,39.671223]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":110,"matched_point_type":"matched","edge_index":42,"distance_along_edge":0.678,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112350,39.671284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":111,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.349,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112228,39.671295]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":112,"matched_point_type":"matched","edge_index":43,"distance_along_edge":0.528,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111488,39.671379]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":113,"matched_point_type":"matched","edge_index":44,"distance_along_edge":0.574,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110725,39.671444]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":114,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.081,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110481,39.671463]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":115,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.125,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110092,39.671497]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":116,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.196,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109802,39.671520]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":117,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.248,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109512,39.671543]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":118,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.301,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109200,39.671566]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":119,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.357,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109039,39.671581]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":120,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.386,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108879,39.671593]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":121,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.415,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108704,39.671608]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":122,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.447,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108536,39.671623]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":123,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.477,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108360,39.671635]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":124,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.509,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108192,39.671650]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":125,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.539,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108017,39.671665]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":126,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.571,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107666,39.671696]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":127,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.634,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107483,39.671711]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":128,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.668,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107307,39.671722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":129,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.699,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107124,39.671738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":130,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.732,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106934,39.671757]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":131,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.767,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106750,39.671772]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":132,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.800,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106567,39.671787]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":133,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.833,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106194,39.671818]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":134,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.901,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105820,39.671848]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":135,"matched_point_type":"matched","edge_index":45,"distance_along_edge":0.968,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105637,39.671864]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":136,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.001,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105453,39.671879]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":137,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.028,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105103,39.671913]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":138,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.080,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104927,39.671928]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":139,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.105,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104752,39.671944]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":140,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.131,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104584,39.671959]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":141,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.156,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104248,39.671989]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":142,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.205,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.103523,39.672054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":143,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.311,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.102730,39.672115]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":144,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.428,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101921,39.672173]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":145,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.546,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101379,39.672215]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":146,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.626,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101112,39.672234]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":147,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.665,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.100708,39.672264]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":148,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.724,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.100189,39.672306]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":149,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.800,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.099480,39.672359]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":150,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.904,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098839,39.672390]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":151,"matched_point_type":"matched","edge_index":46,"distance_along_edge":0.998,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098724,39.672363]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":152,"matched_point_type":"interpolated","edge_index":47,"distance_along_edge":0.097,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098625,39.672337]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":153,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.193,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098549,39.672318]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":154,"matched_point_type":"interpolated","edge_index":47,"distance_along_edge":0.267,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098465,39.672295]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":155,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.349,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098389,39.672276]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":156,"matched_point_type":"interpolated","edge_index":47,"distance_along_edge":0.423,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098320,39.672260]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":157,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.488,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098236,39.672241]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":158,"matched_point_type":"interpolated","edge_index":47,"distance_along_edge":0.569,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098152,39.672230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":159,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.647,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098053,39.672218]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":160,"matched_point_type":"interpolated","edge_index":47,"distance_along_edge":0.739,"distance_from_trace_point":3.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097946,39.672218]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":161,"matched_point_type":"matched","edge_index":47,"distance_along_edge":0.838,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097839,39.672226]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":162,"matched_point_type":"interpolated","edge_index":47,"distance_along_edge":0.937,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097748,39.672234]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":163,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.013,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097641,39.672249]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":164,"matched_point_type":"interpolated","edge_index":48,"distance_along_edge":0.071,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097511,39.672268]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":165,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.142,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097397,39.672283]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":166,"matched_point_type":"interpolated","edge_index":48,"distance_along_edge":0.204,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097298,39.672298]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":167,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.259,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097168,39.672318]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":168,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.330,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097054,39.672337]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":169,"matched_point_type":"interpolated","edge_index":48,"distance_along_edge":0.393,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096931,39.672356]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":170,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.459,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096802,39.672371]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":171,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.530,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096565,39.672405]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":172,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.659,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096367,39.672432]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":173,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.767,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096031,39.672474]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":174,"matched_point_type":"matched","edge_index":48,"distance_along_edge":0.950,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.095276,39.672554]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":175,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.261,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.094528,39.672634]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":176,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.555,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093834,39.672707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":177,"matched_point_type":"matched","edge_index":49,"distance_along_edge":0.828,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093285,39.672771]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":178,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.051,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.092705,39.672836]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":179,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.307,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091957,39.672920]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":180,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.638,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091171,39.673012]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":181,"matched_point_type":"matched","edge_index":50,"distance_along_edge":0.986,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090408,39.673088]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":182,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.281,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089699,39.673164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":183,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.554,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089287,39.673206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":184,"matched_point_type":"matched","edge_index":51,"distance_along_edge":0.713,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088516,39.673283]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":185,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.004,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088272,39.673294]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":186,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.045,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087898,39.673309]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":187,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.108,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087524,39.673328]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":188,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.171,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087151,39.673359]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":189,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.235,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.086792,39.673405]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":190,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.296,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.086037,39.673481]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":191,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.425,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085884,39.673496]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":192,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.451,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085686,39.673515]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":193,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.484,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085556,39.673527]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":194,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.506,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085426,39.673538]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":195,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.528,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085220,39.673561]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":196,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.564,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.084625,39.673615]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":197,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.665,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083923,39.673672]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":198,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.784,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083313,39.673721]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":199,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.887,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083076,39.673748]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":200,"matched_point_type":"matched","edge_index":52,"distance_along_edge":0.928,"distance_from_trace_point":3.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.082642,39.673782]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":201,"matched_point_type":"matched","edge_index":53,"distance_along_edge":0.012,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081909,39.673832]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":202,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.027,"distance_from_trace_point":3.374}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081207,39.673870]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":203,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.206,"distance_from_trace_point":2.952}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080605,39.673908]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":204,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.360,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080193,39.673939]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":205,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.466,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079468,39.673985]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":206,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.651,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078758,39.674034]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":207,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.832,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078690,39.674038]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":208,"matched_point_type":"interpolated","edge_index":54,"distance_along_edge":0.850,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078194,39.674072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":209,"matched_point_type":"matched","edge_index":54,"distance_along_edge":0.977,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.077499,39.674126]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":210,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.272,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.077057,39.674164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":211,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.472,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.076302,39.674229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":212,"matched_point_type":"matched","edge_index":55,"distance_along_edge":0.814,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.075592,39.674290]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":213,"matched_point_type":"matched","edge_index":56,"distance_along_edge":0.330,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074883,39.674355]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":214,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.087,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074150,39.674393]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":215,"matched_point_type":"matched","edge_index":57,"distance_along_edge":0.688,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073395,39.674438]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":216,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.241,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073296,39.674446]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":217,"matched_point_type":"interpolated","edge_index":58,"distance_along_edge":0.305,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073189,39.674458]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":218,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.375,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072983,39.674473]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":219,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.507,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072556,39.674503]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":220,"matched_point_type":"matched","edge_index":58,"distance_along_edge":0.783,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071815,39.674572]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":221,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.268,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071365,39.674618]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":222,"matched_point_type":"matched","edge_index":59,"distance_along_edge":0.566,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.070602,39.674686]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":223,"matched_point_type":"matched","edge_index":60,"distance_along_edge":0.291,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069832,39.674751]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":224,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.198,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069725,39.674759]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":225,"matched_point_type":"interpolated","edge_index":61,"distance_along_edge":0.239,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069008,39.674835]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":226,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.517,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068504,39.674889]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":227,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.713,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068344,39.674904]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":228,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.775,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068199,39.674919]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":229,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.831,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068047,39.674934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":230,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.890,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067856,39.674953]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":231,"matched_point_type":"matched","edge_index":61,"distance_along_edge":0.964,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067574,39.674980]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":232,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.100,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067352,39.674995]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":233,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.216,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067093,39.675018]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":234,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.352,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.066383,39.675072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":235,"matched_point_type":"matched","edge_index":62,"distance_along_edge":0.724,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.065620,39.675144]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":236,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.013,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.065102,39.675213]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":237,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.042,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.064705,39.675255]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":238,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.064,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.064278,39.675297]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":239,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.087,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063591,39.675362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":240,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.125,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063240,39.675400]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":241,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.145,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062889,39.675438]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":242,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.164,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062660,39.675461]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":243,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.176,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062347,39.675484]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":244,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.194,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.061745,39.675549]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":245,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.227,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060974,39.675636]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":246,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.269,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060280,39.675747]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":247,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.308,"distance_from_trace_point":3.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060104,39.675797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":248,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.319,"distance_from_trace_point":3.225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.059509,39.676121]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":249,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.359,"distance_from_trace_point":3.537}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.059380,39.676201]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":250,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.368,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.058708,39.676506]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":251,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.411,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.057953,39.676613]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":252,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.453,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.057259,39.676685]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":253,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.491,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.056725,39.676731]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":254,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.520,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.055923,39.676785]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":255,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.564,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.055550,39.676823]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":256,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.585,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.054916,39.676891]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":257,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.620,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.054672,39.676914]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":258,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.633,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053978,39.676983]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":259,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.672,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053246,39.677055]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":260,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.712,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.052467,39.677124]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":261,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.755,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051727,39.677185]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":262,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.795,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050926,39.677261]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":263,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.839,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050209,39.677338]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":264,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.879,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049942,39.677376]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":265,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.894,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049461,39.677448]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":266,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.921,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048958,39.677536]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":267,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.949,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048576,39.677608]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":268,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.970,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048157,39.677670]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":269,"matched_point_type":"matched","edge_index":63,"distance_along_edge":0.994,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048103,39.677677]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":270,"matched_point_type":"interpolated","edge_index":63,"distance_along_edge":0.997,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047997,39.677883]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":271,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.020,"distance_from_trace_point":7.191}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047958,39.677944]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":272,"matched_point_type":"interpolated","edge_index":64,"distance_along_edge":0.027,"distance_from_trace_point":7.132}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047577,39.678410]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":273,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.081,"distance_from_trace_point":4.430}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047203,39.678894]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":274,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.137,"distance_from_trace_point":3.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046898,39.679417]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":275,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.194,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046631,39.679951]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":276,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.251,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046371,39.680485]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":277,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.307,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046097,39.681034]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":278,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.366,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045998,39.681248]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":279,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.388,"distance_from_trace_point":4.717}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045715,39.681778]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":280,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.445,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045540,39.682262]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":281,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.495,"distance_from_trace_point":4.564}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045593,39.682510]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":282,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.520,"distance_from_trace_point":5.461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045616,39.682571]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":283,"matched_point_type":"interpolated","edge_index":64,"distance_along_edge":0.527,"distance_from_trace_point":5.605}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045868,39.683098]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":284,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.582,"distance_from_trace_point":5.605}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046127,39.683632]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":285,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.639,"distance_from_trace_point":4.847}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046181,39.683750]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":286,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.651,"distance_from_trace_point":4.245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046242,39.683868]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":287,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.664,"distance_from_trace_point":5.010}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046280,39.683929]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":288,"matched_point_type":"interpolated","edge_index":64,"distance_along_edge":0.671,"distance_from_trace_point":5.605}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046341,39.684052]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":289,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.684,"distance_from_trace_point":5.605}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046417,39.684200]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":290,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.699,"distance_from_trace_point":6.193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046486,39.684345]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":291,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.715,"distance_from_trace_point":6.212}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046600,39.684601]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":292,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.742,"distance_from_trace_point":6.212}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046829,39.685139]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":293,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.798,"distance_from_trace_point":5.461}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046944,39.685394]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":294,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.825,"distance_from_trace_point":5.419}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047157,39.685890]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":295,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.876,"distance_from_trace_point":4.847}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047234,39.686062]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":296,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.894,"distance_from_trace_point":4.717}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047379,39.686443]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":297,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.934,"distance_from_trace_point":3.985}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047379,39.687004]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":298,"matched_point_type":"matched","edge_index":64,"distance_along_edge":0.990,"distance_from_trace_point":3.273}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047173,39.687523]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":299,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.126,"distance_from_trace_point":2.730}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046974,39.687927]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":300,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.249,"distance_from_trace_point":4.245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046677,39.688477]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":301,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.419,"distance_from_trace_point":4.245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046379,39.689011]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":302,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.584,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046158,39.689423]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":303,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.711,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045921,39.689857]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":304,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.845,"distance_from_trace_point":3.484}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045738,39.690205]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":305,"matched_point_type":"matched","edge_index":65,"distance_along_edge":0.951,"distance_from_trace_point":4.095}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045708,39.690281]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":306,"matched_point_type":"interpolated","edge_index":65,"distance_along_edge":0.974,"distance_from_trace_point":5.460}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045387,39.690323]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":307,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.075,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044662,39.690224]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":308,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.270,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043953,39.690125]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":309,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.462,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043808,39.690105]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":310,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.501,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043114,39.690006]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":311,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.689,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.042419,39.689877]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":312,"matched_point_type":"matched","edge_index":66,"distance_along_edge":0.880,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041756,39.689640]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":313,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.075,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041626,39.689575]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":314,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.116,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041000,39.689232]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":315,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.322,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040405,39.688866]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":316,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.526,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039825,39.688507]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":317,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.725,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039299,39.688137]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":318,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.916,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039154,39.688007]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":319,"matched_point_type":"matched","edge_index":67,"distance_along_edge":0.975,"distance_from_trace_point":3.345}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038849,39.688011]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":320,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.022,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038704,39.688034]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":321,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.035,"distance_from_trace_point":3.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038559,39.688053]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":322,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.048,"distance_from_trace_point":3.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037926,39.688133]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":323,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.104,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037537,39.688179]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":324,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.139,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036835,39.688282]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":325,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.202,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036133,39.688324]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":326,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.263,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035751,39.688305]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":327,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.297,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035278,39.688274]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":328,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.338,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035080,39.688259]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":329,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.356,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.034317,39.688198]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":330,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.423,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.033760,39.688168]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":331,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.472,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.033112,39.688129]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":332,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.529,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.032440,39.688080]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":333,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.588,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.032135,39.688072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":334,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.615,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031822,39.688080]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":335,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.642,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031502,39.688118]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":336,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.671,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031166,39.688152]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":337,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.700,"distance_from_trace_point":3.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030815,39.688187]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":338,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.731,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030441,39.688221]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":339,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.764,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030190,39.688244]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":340,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.787,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029541,39.688309]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":341,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.844,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029282,39.688332]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":342,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.867,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029015,39.688354]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":343,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.890,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028488,39.688400]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":344,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.937,"distance_from_trace_point":2.531}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027977,39.688446]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":345,"matched_point_type":"matched","edge_index":68,"distance_along_edge":0.982,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027725,39.688469]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":346,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.002,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027237,39.688515]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":347,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.023,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.026489,39.688583]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":348,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.055,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.026367,39.688595]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":349,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.061,"distance_from_trace_point":2.109}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.025604,39.688663]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":350,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.094,"distance_from_trace_point":1.687}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.025085,39.688709]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":351,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.116,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.024300,39.688774]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":352,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.150,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023895,39.688801]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":353,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.168,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023529,39.688850]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":354,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.184,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023300,39.688889]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":355,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.194,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023193,39.688919]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":356,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.199,"distance_from_trace_point":3.615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022797,39.689060]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":357,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.217,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022240,39.689430]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":358,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.249,"distance_from_trace_point":3.932}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022110,39.689507]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":359,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.256,"distance_from_trace_point":3.193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.021805,39.689640]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":360,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.271,"distance_from_trace_point":2.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.021545,39.689705]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":361,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.283,"distance_from_trace_point":2.613}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.021294,39.689758]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":362,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.294,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.020988,39.689823]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":363,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.308,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.020523,39.689915]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":364,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.328,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.020134,39.689999]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":365,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.346,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.019676,39.690113]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":366,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.366,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018936,39.690315]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":367,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.400,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018539,39.690430]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":368,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.418,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018219,39.690525]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":369,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.433,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017998,39.690594]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":370,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.443,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017654,39.690697]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":371,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.459,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017441,39.690781]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":372,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.470,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017349,39.690838]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":373,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.475,"distance_from_trace_point":3.193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017128,39.691048]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":374,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.490,"distance_from_trace_point":6.193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016991,39.691204]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":375,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.500,"distance_from_trace_point":6.193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016800,39.691433]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":376,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.515,"distance_from_trace_point":4.645}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016563,39.691730]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":377,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.535,"distance_from_trace_point":3.096}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016289,39.692284]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":378,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.568,"distance_from_trace_point":6.967}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016258,39.692348]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":379,"matched_point_type":"interpolated","edge_index":69,"distance_along_edge":0.572,"distance_from_trace_point":4.847}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016197,39.692657]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":380,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.590,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016205,39.692890]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":381,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.603,"distance_from_trace_point":2.630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016266,39.693459]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":382,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.634,"distance_from_trace_point":4.563}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016273,39.693699]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":383,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.648,"distance_from_trace_point":4.544}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016273,39.693897]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":384,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.659,"distance_from_trace_point":3.245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016273,39.693951]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":385,"matched_point_type":"interpolated","edge_index":69,"distance_along_edge":0.662,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016273,39.694111]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":386,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.671,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016273,39.694683]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":387,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.703,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016273,39.695061]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":388,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.724,"distance_from_trace_point":6.491}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016006,39.695557]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":389,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.754,"distance_from_trace_point":5.921}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015785,39.695755]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":390,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.769,"distance_from_trace_point":4.387}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015137,39.696102]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":391,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.803,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015045,39.696140]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":392,"matched_point_type":"interpolated","edge_index":69,"distance_along_edge":0.808,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014679,39.696281]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":393,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.825,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014099,39.696655]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":394,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.858,"distance_from_trace_point":5.921}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013718,39.697132]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":395,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.890,"distance_from_trace_point":5.009}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013657,39.697224]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":396,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.895,"distance_from_trace_point":5.009}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013573,39.697350]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":397,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.903,"distance_from_trace_point":4.429}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013496,39.697464]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":398,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.911,"distance_from_trace_point":4.244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013374,39.697651]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":399,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.922,"distance_from_trace_point":3.658}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013306,39.697750]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":400,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.929,"distance_from_trace_point":3.658}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013199,39.697914]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":401,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.939,"distance_from_trace_point":3.658}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013062,39.698116]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":402,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.952,"distance_from_trace_point":2.888}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012810,39.698517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":403,"matched_point_type":"matched","edge_index":69,"distance_along_edge":0.976,"distance_from_trace_point":3.658}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012543,39.699039]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":404,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.028,"distance_from_trace_point":3.353}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012527,39.699078]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":405,"matched_point_type":"interpolated","edge_index":70,"distance_along_edge":0.036,"distance_from_trace_point":3.353}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012398,39.699524]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":406,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.126,"distance_from_trace_point":3.985}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012260,39.700008]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":407,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.224,"distance_from_trace_point":3.353}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012131,39.700535]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":408,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.330,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012016,39.700981]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":409,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.420,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011871,39.701550]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":410,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.535,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011726,39.702122]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":411,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.650,"distance_from_trace_point":3.984}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011581,39.702682]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":412,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.763,"distance_from_trace_point":3.984}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011482,39.703098]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":413,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.847,"distance_from_trace_point":5.260}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011406,39.703365]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":414,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.901,"distance_from_trace_point":4.716}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011307,39.703655]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":415,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.960,"distance_from_trace_point":1.365}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011253,39.703827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":416,"matched_point_type":"matched","edge_index":70,"distance_along_edge":0.995,"distance_from_trace_point":3.353}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011192,39.703987]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":417,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.034,"distance_from_trace_point":5.200}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011223,39.704529]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":418,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.163,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011246,39.704689]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":419,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.201,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011322,39.705254]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":420,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.336,"distance_from_trace_point":3.917}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011391,39.705853]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":421,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.479,"distance_from_trace_point":5.209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011452,39.706394]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":422,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.608,"distance_from_trace_point":5.209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011459,39.706455]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":423,"matched_point_type":"interpolated","edge_index":71,"distance_along_edge":0.622,"distance_from_trace_point":5.856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011520,39.707020]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":424,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.757,"distance_from_trace_point":5.209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011574,39.707600]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":425,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.895,"distance_from_trace_point":5.856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011597,39.707844]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":426,"matched_point_type":"matched","edge_index":71,"distance_along_edge":0.953,"distance_from_trace_point":5.209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011620,39.708115]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":427,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.009,"distance_from_trace_point":4.543}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011635,39.708302]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":428,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.031,"distance_from_trace_point":4.562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011696,39.708851]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":429,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.097,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011742,39.709255]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":430,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.145,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011795,39.709656]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":431,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.193,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011826,39.709919]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":432,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.224,"distance_from_trace_point":4.562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011887,39.710484]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":433,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.292,"distance_from_trace_point":5.209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011948,39.711052]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":434,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.360,"distance_from_trace_point":5.856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012009,39.711636]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":435,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.429,"distance_from_trace_point":5.856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012024,39.711777]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":436,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.446,"distance_from_trace_point":5.856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012047,39.711998]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":437,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.473,"distance_from_trace_point":5.208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012070,39.712147]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":438,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.491,"distance_from_trace_point":5.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012093,39.712307]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":439,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.510,"distance_from_trace_point":5.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012131,39.712566]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":440,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.541,"distance_from_trace_point":5.855}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012169,39.712914]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":441,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.582,"distance_from_trace_point":5.208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012199,39.713287]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":442,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.627,"distance_from_trace_point":5.855}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012207,39.713387]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":443,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.639,"distance_from_trace_point":5.855}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012222,39.713490]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":444,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.651,"distance_from_trace_point":6.503}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012253,39.713779]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":445,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.686,"distance_from_trace_point":7.150}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012268,39.713966]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":446,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.708,"distance_from_trace_point":7.150}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012283,39.714138]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":447,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.729,"distance_from_trace_point":6.503}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012337,39.714680]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":448,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.793,"distance_from_trace_point":4.562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012405,39.715252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":449,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.862,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012421,39.715366]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":450,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.875,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012444,39.715523]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":451,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.894,"distance_from_trace_point":4.562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012474,39.715805]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":452,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.928,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012505,39.716026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":453,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.954,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012527,39.716160]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":454,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.970,"distance_from_trace_point":4.562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012550,39.716370]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":455,"matched_point_type":"matched","edge_index":72,"distance_along_edge":0.995,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012573,39.716530]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":456,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.023,"distance_from_trace_point":4.562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012596,39.716709]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":457,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.056,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012611,39.716885]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":458,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.088,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012627,39.716976]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":459,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.105,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012634,39.717064]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":460,"matched_point_type":"interpolated","edge_index":73,"distance_along_edge":0.121,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012650,39.717182]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":461,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.143,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012657,39.717266]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":462,"matched_point_type":"interpolated","edge_index":73,"distance_along_edge":0.159,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012672,39.717388]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":463,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.181,"distance_from_trace_point":4.562}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012711,39.717739]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":464,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.246,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012741,39.717968]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":465,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.289,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012756,39.718136]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":466,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.320,"distance_from_trace_point":3.272}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012802,39.718479]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":467,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.383,"distance_from_trace_point":5.208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012856,39.718994]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":468,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.479,"distance_from_trace_point":5.208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012909,39.719547]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":469,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.581,"distance_from_trace_point":5.208}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012962,39.720108]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":470,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.685,"distance_from_trace_point":4.561}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013016,39.720650]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":471,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.785,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013077,39.721249]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":472,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.895,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013107,39.721512]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":473,"matched_point_type":"matched","edge_index":73,"distance_along_edge":0.944,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013153,39.721851]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":474,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.003,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013191,39.722164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":475,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.028,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013222,39.722412]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":476,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.048,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013245,39.722580]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":477,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.062,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013260,39.722744]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":478,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.075,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013283,39.722900]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":479,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.088,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013321,39.723194]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":480,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.112,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013374,39.723656]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":481,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.149,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013443,39.724239]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":482,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.196,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013474,39.724483]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":483,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.216,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013519,39.724884]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":484,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.248,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013580,39.725483]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":485,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.297,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013641,39.726032]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":486,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.341,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013657,39.726158]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":487,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.351,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013718,39.726707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":488,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.396,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013756,39.727009]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":489,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.420,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013786,39.727268]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":490,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.441,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013802,39.727417]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":491,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.453,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013809,39.727516]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":492,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.461,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013824,39.727650]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":493,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.472,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013840,39.727783]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":494,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.483,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013870,39.728008]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":495,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.501,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013931,39.728561]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":496,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.546,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013931,39.728607]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":497,"matched_point_type":"interpolated","edge_index":74,"distance_along_edge":0.550,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013977,39.729012]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":498,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.582,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014000,39.729214]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":499,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.599,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014061,39.729790]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":500,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.645,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014114,39.730267]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":501,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.684,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014175,39.730816]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":502,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.728,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014244,39.731411]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":503,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.776,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014297,39.731865]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":504,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.813,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014320,39.732056]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":505,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.828,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014351,39.732300]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":506,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.848,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014381,39.732609]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":507,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.873,"distance_from_trace_point":3.244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014412,39.733173]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":508,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.919,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014549,39.733704]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":509,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.963,"distance_from_trace_point":5.007}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014763,39.734035]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":510,"matched_point_type":"matched","edge_index":74,"distance_along_edge":0.993,"distance_from_trace_point":5.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014709,39.734379]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":511,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.020,"distance_from_trace_point":2.121}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014572,39.734837]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":512,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.055,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014603,39.735229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":513,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.084,"distance_from_trace_point":3.915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014641,39.735424]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":514,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.099,"distance_from_trace_point":3.915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014656,39.735519]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":515,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.106,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014671,39.735619]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":516,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.113,"distance_from_trace_point":3.915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014687,39.735710]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":517,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.120,"distance_from_trace_point":3.915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014694,39.735809]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":518,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.127,"distance_from_trace_point":3.892}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014664,39.736080]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":519,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.147,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014549,39.736626]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":520,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.188,"distance_from_trace_point":1.991}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014488,39.736874]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":521,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.206,"distance_from_trace_point":1.991}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014465,39.736961]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":522,"matched_point_type":"interpolated","edge_index":75,"distance_along_edge":0.213,"distance_from_trace_point":1.991}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014442,39.737045]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":523,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.219,"distance_from_trace_point":1.364}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014404,39.737164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":524,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.228,"distance_from_trace_point":1.364}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014351,39.737312]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":525,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.240,"distance_from_trace_point":1.364}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014252,39.737576]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":526,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.260,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014153,39.737831]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":527,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.280,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014030,39.738140]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":528,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.303,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013969,39.738335]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":529,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.318,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013901,39.738544]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":530,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.334,"distance_from_trace_point":1.364}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013725,39.739082]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":531,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.375,"distance_from_trace_point":1.991}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013664,39.739574]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":532,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.412,"distance_from_trace_point":3.481}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014015,39.740040]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":533,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.451,"distance_from_trace_point":4.642}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014420,39.740505]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":534,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.493,"distance_from_trace_point":4.642}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014816,39.740967]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":535,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.533,"distance_from_trace_point":4.642}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015175,39.741386]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":536,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.570,"distance_from_trace_point":3.624}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015297,39.741531]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":537,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.583,"distance_from_trace_point":3.481}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015228,39.741940]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":538,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.615,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015068,39.742214]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":539,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.637,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014908,39.742474]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":540,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.658,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014771,39.742699]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":541,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.677,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014671,39.742855]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":542,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.689,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014572,39.743015]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":543,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.702,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014496,39.743141]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":544,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.713,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014366,39.743343]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":545,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.729,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014267,39.743511]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":546,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.743,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014076,39.743820]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":547,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.768,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013947,39.744022]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":548,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.785,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013802,39.744259]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":549,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.804,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013680,39.744461]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":550,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.820,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013573,39.744640]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":551,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.835,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013489,39.744774]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":552,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.846,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013435,39.744850]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":553,"matched_point_type":"interpolated","edge_index":75,"distance_along_edge":0.852,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013390,39.744923]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":554,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.858,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013329,39.745026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":555,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.866,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013245,39.745152]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":556,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.877,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013168,39.745270]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":557,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.887,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013023,39.745468]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":558,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.903,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012672,39.745934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":559,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.943,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012451,39.746216]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":560,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.967,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012337,39.746361]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":561,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.980,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012238,39.746487]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":562,"matched_point_type":"matched","edge_index":75,"distance_along_edge":0.991,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012100,39.746571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":563,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.013,"distance_from_trace_point":2.128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011505,39.746292]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":564,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.160,"distance_from_trace_point":2.128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011246,39.746174]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":565,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.224,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010880,39.746002]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":566,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.314,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010483,39.745815]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":567,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.412,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009949,39.745567]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":568,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.544,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009735,39.745464]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":569,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.597,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009529,39.745369]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":570,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.647,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009209,39.745220]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":571,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.726,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.008598,39.744930]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":572,"matched_point_type":"matched","edge_index":76,"distance_along_edge":0.878,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.007904,39.744602]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":573,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.095,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.007256,39.744293]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":574,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.407,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.006615,39.743988]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":575,"matched_point_type":"matched","edge_index":77,"distance_along_edge":0.715,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005943,39.743671]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":576,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.045,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005287,39.743366]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":577,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.432,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004944,39.743206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":578,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.635,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004326,39.742920]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":579,"matched_point_type":"matched","edge_index":78,"distance_along_edge":0.000,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004143,39.742836]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":580,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.011,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.003494,39.742542]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":581,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.051,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.002853,39.742252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":582,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.091,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.002365,39.742039]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":583,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.120,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001724,39.741760]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":584,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.159,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001503,39.741669]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":585,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.173,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001289,39.741573]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":586,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.186,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000656,39.741299]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":587,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.224,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999977,39.740974]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":588,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.267,"distance_from_trace_point":1.812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999550,39.740688]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":589,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.297,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999329,39.740540]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":590,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.312,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999115,39.740398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":591,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.327,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998901,39.740257]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":592,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.342,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998581,39.740044]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":593,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.365,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997948,39.739624]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":594,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.409,"distance_from_trace_point":2.128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997520,39.739342]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":595,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.439,"distance_from_trace_point":2.128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997108,39.739067]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":596,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.468,"distance_from_trace_point":2.128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996902,39.738934]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":597,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.482,"distance_from_trace_point":1.812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996704,39.738804]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":598,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.496,"distance_from_trace_point":1.812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996140,39.738438]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":599,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.535,"distance_from_trace_point":1.812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995529,39.738045]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":600,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.577,"distance_from_trace_point":1.812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994949,39.737663]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":601,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.617,"distance_from_trace_point":2.128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994400,39.737297]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":602,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.656,"distance_from_trace_point":1.812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994308,39.737232]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":603,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.663,"distance_from_trace_point":1.812}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994125,39.737099]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":604,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.676,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993759,39.736832]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":605,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.703,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993484,39.736633]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":606,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.723,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992897,39.736229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":607,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.764,"distance_from_trace_point":3.625}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992432,39.735912]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":608,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.797,"distance_from_trace_point":3.192}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992371,39.735874]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":609,"matched_point_type":"interpolated","edge_index":79,"distance_along_edge":0.801,"distance_from_trace_point":3.192}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992195,39.735779]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":610,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.813,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992035,39.735699]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":611,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.823,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991928,39.735657]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":612,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.829,"distance_from_trace_point":2.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991783,39.735592]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":613,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.838,"distance_from_trace_point":2.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991638,39.735535]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":614,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.847,"distance_from_trace_point":2.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991417,39.735443]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":615,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.860,"distance_from_trace_point":2.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991249,39.735374]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":616,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.870,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991119,39.735321]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":617,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.878,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990982,39.735268]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":618,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.886,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990440,39.735046]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":619,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.919,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990387,39.735027]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":620,"matched_point_type":"interpolated","edge_index":79,"distance_along_edge":0.922,"distance_from_trace_point":2.128}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990089,39.734909]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":621,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.940,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989845,39.734821]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":622,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.954,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989464,39.734707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":623,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.976,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989418,39.734695]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":624,"matched_point_type":"interpolated","edge_index":79,"distance_along_edge":0.978,"distance_from_trace_point":3.023}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989296,39.734669]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":625,"matched_point_type":"matched","edge_index":79,"distance_along_edge":0.985,"distance_from_trace_point":17.651}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989044,39.734344]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":626,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.025,"distance_from_trace_point":24.006}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989037,39.734272]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":627,"matched_point_type":"interpolated","edge_index":80,"distance_along_edge":0.032,"distance_from_trace_point":22.060}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989006,39.733791]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":628,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.077,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988998,39.733704]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":629,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.086,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988983,39.733521]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":630,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.103,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988968,39.733337]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":631,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.120,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988945,39.733063]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":632,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.146,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988907,39.732517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":633,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.198,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988892,39.732330]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":634,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.215,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988846,39.731781]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":635,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.267,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988831,39.731506]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":636,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.293,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988815,39.731220]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":637,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.320,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989113,39.730801]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":638,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.367,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989212,39.730740]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":639,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.376,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989418,39.730621]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":640,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.395,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989616,39.730503]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":641,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.413,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989815,39.730389]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":642,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.431,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990425,39.730076]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":643,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.485,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991165,39.729843]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":644,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.543,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991844,39.729645]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":645,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.595,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992523,39.729450]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":646,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.648,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992973,39.729321]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":647,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.683,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993446,39.729183]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":648,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.720,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993782,39.729092]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":649,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.745,"distance_from_trace_point":2.206}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994446,39.728889]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":650,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.797,"distance_from_trace_point":3.615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995087,39.728672]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":651,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.848,"distance_from_trace_point":3.895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995613,39.728317]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":652,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.899,"distance_from_trace_point":2.576}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996216,39.727924]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":653,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.957,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996429,39.727818]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":654,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.975,"distance_from_trace_point":2.476}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996544,39.727764]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":655,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.985,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996635,39.727699]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":656,"matched_point_type":"matched","edge_index":80,"distance_along_edge":0.994,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996735,39.727505]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":657,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.067,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996712,39.727287]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":658,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.164,"distance_from_trace_point":3.915}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996704,39.727177]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":659,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.213,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996689,39.727066]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":660,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.262,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996674,39.726955]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":661,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.312,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996666,39.726837]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":662,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.364,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996635,39.726616]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":663,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.463,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996613,39.726387]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":664,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.565,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996597,39.726273]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":665,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.616,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996574,39.726028]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":666,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.725,"distance_from_trace_point":3.244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996559,39.725800]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":667,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.827,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996529,39.725571]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":668,"matched_point_type":"matched","edge_index":81,"distance_along_edge":0.929,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996513,39.725342]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":669,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.019,"distance_from_trace_point":1.946}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996460,39.724918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":670,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.135,"distance_from_trace_point":2.629}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996391,39.724304]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":671,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.303,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996315,39.723724]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":672,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.462,"distance_from_trace_point":3.271}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996307,39.723625]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":673,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.489,"distance_from_trace_point":3.244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996269,39.723072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":674,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.641,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996223,39.722523]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":675,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.791,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996201,39.722389]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":676,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.828,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996170,39.722179]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":677,"matched_point_type":"matched","edge_index":82,"distance_along_edge":0.885,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996117,39.721577]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":678,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.078,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996063,39.721035]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":679,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.310,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996017,39.720627]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":680,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.484,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995964,39.720097]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":681,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.711,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995949,39.719940]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":682,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.778,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995941,39.719891]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":683,"matched_point_type":"interpolated","edge_index":83,"distance_along_edge":0.799,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995934,39.719791]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":684,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.842,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995918,39.719639]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":685,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.907,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995903,39.719494]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":686,"matched_point_type":"matched","edge_index":83,"distance_along_edge":0.969,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995888,39.719307]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":687,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.029,"distance_from_trace_point":4.542}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995872,39.719124]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":688,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.076,"distance_from_trace_point":3.916}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995850,39.718784]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":689,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.163,"distance_from_trace_point":3.244}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995827,39.718437]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":690,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.252,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995789,39.717899]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":691,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.389,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995781,39.717850]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":692,"matched_point_type":"interpolated","edge_index":84,"distance_along_edge":0.402,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995766,39.717613]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":693,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.462,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995728,39.717033]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":694,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.611,"distance_from_trace_point":2.595}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995689,39.716457]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":695,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.758,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995651,39.715958]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":696,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.886,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995628,39.715660]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":697,"matched_point_type":"matched","edge_index":84,"distance_along_edge":0.962,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995605,39.715343]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":698,"matched_point_type":"matched","edge_index":85,"distance_along_edge":0.530,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995567,39.714912]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":699,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.046,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995529,39.714420]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":700,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.126,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995506,39.714020]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":701,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.191,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995499,39.713810]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":702,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.226,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995491,39.713707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":703,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.242,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995476,39.713402]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":704,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.292,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995445,39.712898]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":705,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.374,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995407,39.712326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":706,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.467,"distance_from_trace_point":3.245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995399,39.712147]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":707,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.496,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995361,39.711563]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":708,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.591,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995338,39.711227]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":709,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.646,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995331,39.711052]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":710,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.675,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995316,39.710880]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":711,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.702,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995308,39.710766]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":712,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.721,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995270,39.710159]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":713,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.820,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995232,39.709564]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":714,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.917,"distance_from_trace_point":3.245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995209,39.709309]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":715,"matched_point_type":"matched","edge_index":86,"distance_along_edge":0.958,"distance_from_trace_point":3.245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995193,39.708977]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":716,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.014,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995163,39.708431]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":717,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.116,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995117,39.707825]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":718,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.230,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995171,39.707218]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":719,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.344,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995186,39.707108]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":720,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.364,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995216,39.706886]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":721,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.406,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995247,39.706673]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":722,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.446,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995255,39.706570]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":723,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.466,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995277,39.706371]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":724,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.503,"distance_from_trace_point":2.630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995308,39.706177]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":725,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.539,"distance_from_trace_point":2.630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995346,39.705803]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":726,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.610,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995384,39.705395]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":727,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.686,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995415,39.705109]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":728,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.740,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995430,39.704967]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":729,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.766,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995445,39.704830]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":730,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.792,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995499,39.704288]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":731,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.894,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995506,39.704163]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":732,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.917,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995506,39.703739]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":733,"matched_point_type":"matched","edge_index":87,"distance_along_edge":0.996,"distance_from_trace_point":3.245}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995453,39.703594]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":734,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.014,"distance_from_trace_point":2.630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995445,39.703552]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":735,"matched_point_type":"interpolated","edge_index":88,"distance_along_edge":0.018,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995415,39.703243]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":736,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.051,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995384,39.702682]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":737,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.110,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995338,39.702091]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":738,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.173,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995293,39.701591]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":739,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.225,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995232,39.701054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":740,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.282,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995209,39.700882]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":741,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.300,"distance_from_trace_point":2.630}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995186,39.700668]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":742,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.323,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995171,39.700527]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":743,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.338,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995155,39.700401]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":744,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.351,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995140,39.700272]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":745,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.365,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995087,39.699711]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":746,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.424,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995041,39.699089]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":747,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.490,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995041,39.698986]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":748,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.501,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995026,39.698780]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":749,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.522,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995003,39.698570]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":750,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.545,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994980,39.698360]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":751,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.567,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994957,39.698154]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":752,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.588,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994934,39.697948]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":753,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.610,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994904,39.697651]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":754,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.642,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994858,39.697102]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":755,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.700,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994804,39.696503]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":756,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.763,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994781,39.696278]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":757,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.787,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994774,39.696133]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":758,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.802,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994766,39.696045]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":759,"matched_point_type":"interpolated","edge_index":88,"distance_along_edge":0.811,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994759,39.695965]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":760,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.820,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994743,39.695850]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":761,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.832,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994720,39.695679]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":762,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.850,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994667,39.695362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":763,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.884,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994576,39.694820]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":764,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.941,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994545,39.694550]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":765,"matched_point_type":"matched","edge_index":88,"distance_along_edge":0.970,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994537,39.694511]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":766,"matched_point_type":"interpolated","edge_index":88,"distance_along_edge":0.974,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994591,39.694229]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":767,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.023,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995285,39.693977]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":768,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.210,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995720,39.693832]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":769,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.326,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996391,39.693607]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":770,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.504,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997101,39.693378]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":771,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.692,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997177,39.693352]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":772,"matched_point_type":"interpolated","edge_index":89,"distance_along_edge":0.712,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997833,39.693138]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":773,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.886,"distance_from_trace_point":2.207}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998070,39.693066]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":774,"matched_point_type":"matched","edge_index":89,"distance_along_edge":0.948,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998146,39.693043]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":775,"matched_point_type":"interpolated","edge_index":89,"distance_along_edge":0.968,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998230,39.692730]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":776,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.022,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998184,39.692120]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":777,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.071,"distance_from_trace_point":3.895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998169,39.691597]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":778,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.112,"distance_from_trace_point":3.895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998161,39.691055]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":779,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.156,"distance_from_trace_point":3.895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998146,39.690426]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":780,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.206,"distance_from_trace_point":2.596}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998146,39.690323]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":781,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.214,"distance_from_trace_point":1.947}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998154,39.690025]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":782,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.238,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998161,39.689827]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":783,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.253,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998169,39.689728]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":784,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.261,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998169,39.689625]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":785,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.269,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998192,39.689419]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":786,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.286,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998238,39.688992]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":787,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.320,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998245,39.688663]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":788,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.346,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998253,39.688450]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":789,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.363,"distance_from_trace_point":2.597}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998253,39.688122]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":790,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.389,"distance_from_trace_point":5.193}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998260,39.687904]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":791,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.407,"distance_from_trace_point":5.842}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998260,39.687580]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":792,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.432,"distance_from_trace_point":5.842}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998268,39.686985]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":793,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.480,"distance_from_trace_point":2.597}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998283,39.686531]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":794,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.516,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998283,39.686047]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":795,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.554,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998276,39.685844]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":796,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.571,"distance_from_trace_point":0.649}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998253,39.685665]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":797,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.585,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998230,39.685612]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":798,"matched_point_type":"interpolated","edge_index":90,"distance_along_edge":0.589,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998199,39.685551]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":799,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.595,"distance_from_trace_point":1.298}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998077,39.685410]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":800,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.608,"distance_from_trace_point":2.322}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998032,39.685364]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":801,"matched_point_type":"interpolated","edge_index":90,"distance_along_edge":0.613,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997871,39.685226]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":802,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.627,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997398,39.684826]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":803,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.670,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997040,39.684517]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":804,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.703,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996582,39.684055]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":805,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.750,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996178,39.683575]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":806,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.795,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995850,39.683193]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":807,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.832,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995705,39.683033]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":808,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.847,"distance_from_trace_point":1.548}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995514,39.682796]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":809,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.869,"distance_from_trace_point":1.548}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995384,39.682648]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":810,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.884,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994896,39.682266]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":811,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.927,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994705,39.682171]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":812,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.941,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994110,39.681870]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":813,"matched_point_type":"matched","edge_index":90,"distance_along_edge":0.984,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993675,39.681644]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":814,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.025,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993423,39.681484]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":815,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.056,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993187,39.681335]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":816,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.085,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992561,39.680954]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":817,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.161,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991974,39.680595]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":818,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.232,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991714,39.680439]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":819,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.264,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991150,39.680096]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":820,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.332,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990547,39.679722]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":821,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.405,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989952,39.679356]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":822,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.478,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989594,39.679131]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":823,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.522,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989326,39.678963]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":824,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.554,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988701,39.678574]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":825,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.631,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988113,39.678207]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":826,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.703,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.987564,39.677876]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":827,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.769,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.987030,39.677490]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":828,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.838,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986717,39.677216]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":829,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.884,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986572,39.677082]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":830,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.905,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986099,39.676609]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":831,"matched_point_type":"matched","edge_index":91,"distance_along_edge":0.979,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986038,39.676559]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":832,"matched_point_type":"interpolated","edge_index":91,"distance_along_edge":0.987,"distance_from_trace_point":4.388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986160,39.676174]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":833,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.021,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986580,39.675652]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":834,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.058,"distance_from_trace_point":1.064}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.986908,39.675518]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":835,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.076,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.987602,39.675343]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":836,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.110,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988342,39.675159]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":837,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.146,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.988541,39.675110]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":838,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.155,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989029,39.674984]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":839,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.179,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.989517,39.674706]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":840,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.207,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990128,39.674313]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":841,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.244,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.990700,39.674004]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":842,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.277,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991302,39.673706]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":843,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.310,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.991974,39.673401]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":844,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.346,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992615,39.673164]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":845,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.379,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.992874,39.673092]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":846,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.392,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993042,39.673050]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":847,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.400,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993149,39.673019]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":848,"matched_point_type":"interpolated","edge_index":92,"distance_along_edge":0.405,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993279,39.672985]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":849,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.412,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993446,39.672939]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":850,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.420,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993523,39.672916]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":851,"matched_point_type":"interpolated","edge_index":92,"distance_along_edge":0.424,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993668,39.672874]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":852,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.431,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993774,39.672844]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":853,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.436,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.993965,39.672771]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":854,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.446,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994225,39.672653]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":855,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.460,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.994888,39.672398]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":856,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.494,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995033,39.672352]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":857,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.502,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.995682,39.672161]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":858,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.534,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996117,39.672031]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":859,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.555,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996445,39.671932]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":860,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.572,"distance_from_trace_point":0.844}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.996758,39.671837]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":861,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.587,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.997444,39.671635]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":862,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.621,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998146,39.671360]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":863,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.658,"distance_from_trace_point":3.225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998489,39.671223]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":864,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.676,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.998665,39.671158]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":865,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.685,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999290,39.670918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":866,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.717,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-75.999786,39.670731]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":867,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.743,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000038,39.670635]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":868,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.756,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000191,39.670570]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":869,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.764,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000351,39.670502]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":870,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.772,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000542,39.670422]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":871,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.782,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.000763,39.670326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":872,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.794,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001274,39.670109]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":873,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.821,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.001808,39.669888]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":874,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.849,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.002480,39.669609]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":875,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.885,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.003128,39.669342]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":876,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.919,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.003784,39.669075]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":877,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.953,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004448,39.668797]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":878,"matched_point_type":"matched","edge_index":92,"distance_along_edge":0.988,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004784,39.668655]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":879,"matched_point_type":"matched","edge_index":93,"distance_along_edge":0.157,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.004921,39.668602]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":880,"matched_point_type":"matched","edge_index":93,"distance_along_edge":0.343,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005058,39.668545]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":881,"matched_point_type":"matched","edge_index":93,"distance_along_edge":0.533,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005287,39.668449]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":882,"matched_point_type":"matched","edge_index":93,"distance_along_edge":0.849,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.005569,39.668327]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":883,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.093,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.006226,39.668060]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":884,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.441,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.006790,39.667831]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":885,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.740,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.007141,39.667683]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":886,"matched_point_type":"matched","edge_index":94,"distance_along_edge":0.928,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.007500,39.667542]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":887,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.097,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.008163,39.667282]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":888,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.387,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.008827,39.667004]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":889,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.682,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.009422,39.666714]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":890,"matched_point_type":"matched","edge_index":95,"distance_along_edge":0.956,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010010,39.666409]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":891,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.098,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010376,39.666225]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":892,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.170,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.010956,39.665924]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":893,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.286,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.011543,39.665619]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":894,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.402,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012123,39.665321]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":895,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.517,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012726,39.665009]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":896,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.637,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.012947,39.664898]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":897,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.681,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013092,39.664822]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":898,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.709,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013344,39.664696]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":899,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.759,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.013786,39.664467]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":900,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.847,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014229,39.664238]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":901,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.935,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014450,39.664120]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":902,"matched_point_type":"matched","edge_index":96,"distance_along_edge":0.979,"distance_from_trace_point":3.932}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014748,39.663971]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":903,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.472,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.014931,39.663891]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":904,"matched_point_type":"matched","edge_index":97,"distance_along_edge":0.906,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015282,39.663807]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":905,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.097,"distance_from_trace_point":3.851}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015587,39.663746]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":906,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.191,"distance_from_trace_point":4.818}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.015923,39.663677]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":907,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.295,"distance_from_trace_point":5.225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016441,39.663570]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":908,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.456,"distance_from_trace_point":5.225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.016869,39.663483]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":909,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.588,"distance_from_trace_point":5.225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.017517,39.663353]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":910,"matched_point_type":"matched","edge_index":98,"distance_along_edge":0.788,"distance_from_trace_point":4.413}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018242,39.663204]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":911,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.007,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.018867,39.662926]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":912,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.129,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.019302,39.662727]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":913,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.214,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.019966,39.662437]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":914,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.343,"distance_from_trace_point":5.032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.020676,39.662243]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":915,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.470,"distance_from_trace_point":6.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.021370,39.662094]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":916,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.591,"distance_from_trace_point":7.024}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022102,39.661938]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":917,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.719,"distance_from_trace_point":7.430}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022552,39.661835]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":918,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.799,"distance_from_trace_point":6.620}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.022644,39.661812]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":919,"matched_point_type":"interpolated","edge_index":99,"distance_along_edge":0.815,"distance_from_trace_point":7.024}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023117,39.661705]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":920,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.898,"distance_from_trace_point":5.225}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.023499,39.661613]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":921,"matched_point_type":"matched","edge_index":99,"distance_along_edge":0.965,"distance_from_trace_point":4.818}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.024071,39.661484]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":922,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.019,"distance_from_trace_point":3.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.024712,39.661335]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":923,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.052,"distance_from_trace_point":3.616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.025459,39.661148]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":924,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.090,"distance_from_trace_point":3.616}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.025970,39.660976]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":925,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.117,"distance_from_trace_point":3.537}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.026527,39.660652]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":926,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.151,"distance_from_trace_point":3.194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027023,39.660252]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":927,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.186,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027168,39.660095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":928,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.199,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027313,39.659935]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":929,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.211,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027443,39.659775]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":930,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.223,"distance_from_trace_point":2.323}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.027687,39.659451]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":931,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.247,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028053,39.658958]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":932,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.283,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028175,39.658791]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":933,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.295,"distance_from_trace_point":2.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028442,39.658455]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":934,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.320,"distance_from_trace_point":3.097}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028786,39.658150]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":935,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.345,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.028961,39.658001]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":936,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.358,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029236,39.657776]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":937,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.378,"distance_from_trace_point":3.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029518,39.657558]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":938,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.397,"distance_from_trace_point":4.389}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029716,39.657421]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":939,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.410,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.029907,39.657288]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":940,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.423,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030098,39.657162]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":941,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.435,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.030586,39.656773]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":942,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.469,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031113,39.656326]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":943,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.508,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031631,39.655865]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":944,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.546,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.031860,39.655666]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":945,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.563,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.032379,39.655212]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":946,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.601,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.032867,39.654785]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":947,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.638,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.033333,39.654377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":948,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.672,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.033836,39.653931]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":949,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.709,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.034317,39.653522]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":950,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.744,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.034843,39.653107]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":951,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.781,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035255,39.652805]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":952,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.809,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035767,39.652431]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":953,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.843,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.035927,39.652313]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":954,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.854,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036247,39.652073]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":955,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.876,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036484,39.651894]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":956,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.892,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036636,39.651779]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":957,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.902,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.036942,39.651546]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":958,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.923,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037331,39.651237]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":959,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.951,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037575,39.651058]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":960,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.967,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037811,39.650890]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":961,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.983,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.037964,39.650791]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":962,"matched_point_type":"matched","edge_index":100,"distance_along_edge":0.993,"distance_from_trace_point":0.000}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038330,39.650879]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":963,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.032,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038414,39.650936]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":964,"matched_point_type":"interpolated","edge_index":101,"distance_along_edge":0.042,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038597,39.651058]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":965,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.065,"distance_from_trace_point":0.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038773,39.651176]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":966,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.088,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038849,39.651234]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":967,"matched_point_type":"interpolated","edge_index":101,"distance_along_edge":0.098,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.038933,39.651295]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":968,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.109,"distance_from_trace_point":2.323}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039062,39.651424]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":969,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.129,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039345,39.651772]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":970,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.180,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039536,39.652031]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":971,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.217,"distance_from_trace_point":3.098}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039734,39.652317]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":972,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.258,"distance_from_trace_point":3.660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039833,39.652462]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":973,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.278,"distance_from_trace_point":3.660}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.039925,39.652580]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":974,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.295,"distance_from_trace_point":4.647}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040024,39.652702]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":975,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.313,"distance_from_trace_point":5.421}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040138,39.652813]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":976,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.331,"distance_from_trace_point":5.155}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040314,39.652958]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":977,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.355,"distance_from_trace_point":5.440}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040588,39.653114]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":978,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.388,"distance_from_trace_point":5.689}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040878,39.653221]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":979,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.419,"distance_from_trace_point":5.689}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.040970,39.653259]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":980,"matched_point_type":"interpolated","edge_index":101,"distance_along_edge":0.429,"distance_from_trace_point":5.032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041115,39.653313]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":981,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.444,"distance_from_trace_point":5.032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041260,39.653358]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":982,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.459,"distance_from_trace_point":5.423}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041718,39.653515]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":983,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.507,"distance_from_trace_point":5.423}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.041908,39.653580]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":984,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.527,"distance_from_trace_point":5.423}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.042107,39.653648]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":985,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.548,"distance_from_trace_point":5.423}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.042412,39.653755]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":986,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.580,"distance_from_trace_point":5.032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.042679,39.653847]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":987,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.608,"distance_from_trace_point":5.032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043114,39.653999]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":988,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.654,"distance_from_trace_point":5.032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043785,39.654240]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":989,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.725,"distance_from_trace_point":4.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.043877,39.654270]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":990,"matched_point_type":"interpolated","edge_index":101,"distance_along_edge":0.734,"distance_from_trace_point":5.032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044189,39.654381]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":991,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.767,"distance_from_trace_point":6.067}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044350,39.654442]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":992,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.784,"distance_from_trace_point":5.689}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044472,39.654484]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":993,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.797,"distance_from_trace_point":5.689}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.044647,39.654552]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":994,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.816,"distance_from_trace_point":5.032}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045212,39.654758]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":995,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.876,"distance_from_trace_point":6.451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045479,39.654858]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":996,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.904,"distance_from_trace_point":6.451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045662,39.654926]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":997,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.924,"distance_from_trace_point":6.451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.045937,39.655029]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":998,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.953,"distance_from_trace_point":7.111}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046112,39.655098]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":999,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.972,"distance_from_trace_point":7.111}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046280,39.655167]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1000,"matched_point_type":"matched","edge_index":101,"distance_along_edge":0.990,"distance_from_trace_point":3.896}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046448,39.655159]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1001,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.067,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.046631,39.655041]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1002,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.234,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047218,39.654659]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1003,"matched_point_type":"matched","edge_index":102,"distance_along_edge":0.770,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047783,39.654301]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1004,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.216,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047882,39.654243]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1005,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.282,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.047974,39.654182]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1006,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.347,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048233,39.654018]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1007,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.528,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048447,39.653885]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1008,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.675,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048538,39.653831]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1009,"matched_point_type":"interpolated","edge_index":103,"distance_along_edge":0.737,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048615,39.653782]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1010,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.791,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048706,39.653728]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1011,"matched_point_type":"interpolated","edge_index":103,"distance_along_edge":0.852,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048790,39.653679]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1012,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.909,"distance_from_trace_point":2.477}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048889,39.653622]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1013,"matched_point_type":"matched","edge_index":103,"distance_along_edge":0.976,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.048996,39.653561]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1014,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.041,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049202,39.653442]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1015,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.162,"distance_from_trace_point":1.422}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.049759,39.653111]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1016,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.494,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050072,39.652916]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1017,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.683,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050331,39.652767]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1018,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.835,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050430,39.652714]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1019,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.892,"distance_from_trace_point":3.194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050545,39.652687]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1020,"matched_point_type":"interpolated","edge_index":104,"distance_along_edge":0.950,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050606,39.652687]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1021,"matched_point_type":"matched","edge_index":104,"distance_along_edge":0.978,"distance_from_trace_point":6.748}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050629,39.652687]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1022,"matched_point_type":"interpolated","edge_index":104,"distance_along_edge":0.989,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.050667,39.652649]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1023,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.008,"distance_from_trace_point":3.355}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051170,39.652218]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1024,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.133,"distance_from_trace_point":1.808}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051430,39.652008]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1025,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.194,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051636,39.651833]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1026,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.244,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.051918,39.651592]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1027,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.313,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.052139,39.651413]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1028,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.365,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.052368,39.651230]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1029,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.419,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.052673,39.650990]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1030,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.490,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053200,39.650566]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1031,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.615,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053429,39.650379]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1032,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.669,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.053925,39.649979]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1033,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.786,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.054482,39.649536]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1034,"matched_point_type":"matched","edge_index":105,"distance_along_edge":0.917,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.054886,39.649212]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1035,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.034,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.055389,39.648804]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1036,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.364,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.055923,39.648369]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1037,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.714,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.056099,39.648228]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1038,"matched_point_type":"matched","edge_index":106,"distance_along_edge":0.829,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.056602,39.647823]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1039,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.128,"distance_from_trace_point":1.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.056709,39.647732]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1040,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.187,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.057198,39.647335]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1041,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.450,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.057678,39.646935]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1042,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.711,"distance_from_trace_point":2.577}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.058128,39.646576]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1043,"matched_point_type":"matched","edge_index":107,"distance_along_edge":0.950,"distance_from_trace_point":1.813}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.058495,39.646271]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1044,"matched_point_type":"matched","edge_index":108,"distance_along_edge":0.367,"distance_from_trace_point":1.549}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.058937,39.645901]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1045,"matched_point_type":"matched","edge_index":108,"distance_along_edge":0.958,"distance_from_trace_point":0.774}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.059387,39.645458]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1046,"matched_point_type":"matched","edge_index":109,"distance_along_edge":0.350,"distance_from_trace_point":4.889}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.059875,39.645054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1047,"matched_point_type":"matched","edge_index":109,"distance_along_edge":0.716,"distance_from_trace_point":3.627}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060394,39.644615]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1048,"matched_point_type":"matched","edge_index":110,"distance_along_edge":0.344,"distance_from_trace_point":3.627}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.060905,39.644180]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1049,"matched_point_type":"matched","edge_index":111,"distance_along_edge":0.394,"distance_from_trace_point":3.346}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.061417,39.643738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1050,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.074,"distance_from_trace_point":3.627}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.061928,39.643314]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1051,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.318,"distance_from_trace_point":2.871}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062439,39.642918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1052,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.554,"distance_from_trace_point":2.129}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.062683,39.642738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1053,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.663,"distance_from_trace_point":2.578}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063026,39.642487]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1054,"matched_point_type":"matched","edge_index":112,"distance_along_edge":0.817,"distance_from_trace_point":2.578}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063484,39.642185]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1055,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.012,"distance_from_trace_point":1.814}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.063812,39.641960]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1056,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.152,"distance_from_trace_point":3.194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.064384,39.641609]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1057,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.385,"distance_from_trace_point":3.194}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.064812,39.641357]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1058,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.556,"distance_from_trace_point":4.259}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.065453,39.641022]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1059,"matched_point_type":"matched","edge_index":113,"distance_along_edge":0.803,"distance_from_trace_point":3.538}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.066055,39.640701]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1060,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.014,"distance_from_trace_point":5.323}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.066605,39.640388]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1061,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.101,"distance_from_trace_point":7.791}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.066917,39.640213]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1062,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.150,"distance_from_trace_point":7.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067123,39.640095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1063,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.182,"distance_from_trace_point":7.453}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067383,39.639942]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1064,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.223,"distance_from_trace_point":7.791}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.067688,39.639763]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1065,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.272,"distance_from_trace_point":7.453}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068298,39.639416]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1066,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.367,"distance_from_trace_point":5.664}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068504,39.639290]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1067,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.400,"distance_from_trace_point":6.059}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.068840,39.639091]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1068,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.454,"distance_from_trace_point":6.388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069099,39.638935]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1069,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.495,"distance_from_trace_point":6.388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069275,39.638828]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1070,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.524,"distance_from_trace_point":6.388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069481,39.638699]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1071,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.557,"distance_from_trace_point":6.388}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069656,39.638592]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1072,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.585,"distance_from_trace_point":7.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.069901,39.638439]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1073,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.625,"distance_from_trace_point":7.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.070091,39.638325]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1074,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.655,"distance_from_trace_point":7.123}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.070343,39.638168]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1075,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.696,"distance_from_trace_point":7.866}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.070625,39.637985]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1076,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.743,"distance_from_trace_point":8.929}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071022,39.637730]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1077,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.808,"distance_from_trace_point":9.992}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071182,39.637638]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1078,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.833,"distance_from_trace_point":12.391}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.071838,39.637676]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1079,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.920,"distance_from_trace_point":10.631}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072205,39.637779]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1080,"matched_point_type":"matched","edge_index":114,"distance_along_edge":0.969,"distance_from_trace_point":10.847}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072433,39.637848]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1081,"matched_point_type":"matched","edge_index":114,"distance_along_edge":1.000,"distance_from_trace_point":7.627}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072433,39.637848]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1082,"matched_point_type":"interpolated","edge_index":115,"distance_along_edge":0.000,"distance_from_trace_point":6.218}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072510,39.637695]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1083,"matched_point_type":"matched","edge_index":115,"distance_along_edge":0.070,"distance_from_trace_point":9.700}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.072861,39.637192]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1084,"matched_point_type":"matched","edge_index":115,"distance_along_edge":0.315,"distance_from_trace_point":6.750}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073067,39.636925]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1085,"matched_point_type":"matched","edge_index":115,"distance_along_edge":0.448,"distance_from_trace_point":5.977}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073143,39.636826]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1086,"matched_point_type":"matched","edge_index":115,"distance_along_edge":0.498,"distance_from_trace_point":6.371}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073311,39.636612]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1087,"matched_point_type":"matched","edge_index":115,"distance_along_edge":0.605,"distance_from_trace_point":6.971}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073540,39.636372]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1088,"matched_point_type":"matched","edge_index":115,"distance_along_edge":0.733,"distance_from_trace_point":7.209}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.073792,39.636127]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1089,"matched_point_type":"matched","edge_index":115,"distance_along_edge":0.866,"distance_from_trace_point":6.967}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074348,39.635685]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1090,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.094,"distance_from_trace_point":6.497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074425,39.635632]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1091,"matched_point_type":"interpolated","edge_index":116,"distance_along_edge":0.118,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074646,39.635460]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1092,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.192,"distance_from_trace_point":6.202}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.074936,39.635239]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1093,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.289,"distance_from_trace_point":6.497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.075279,39.634975]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1094,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.404,"distance_from_trace_point":7.255}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.075821,39.634563]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1095,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.584,"distance_from_trace_point":6.804}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.076042,39.634407]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1096,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.655,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.076149,39.634327]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1097,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.690,"distance_from_trace_point":5.441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.076553,39.634033]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1098,"matched_point_type":"matched","edge_index":116,"distance_along_edge":0.822,"distance_from_trace_point":4.996}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.077110,39.633617]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1099,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.004,"distance_from_trace_point":5.441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.077477,39.633343]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1100,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.094,"distance_from_trace_point":5.441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078033,39.632931]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1101,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.230,"distance_from_trace_point":5.441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078583,39.632523]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1102,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.364,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.078842,39.632328]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1103,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.428,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079124,39.632118]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1104,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.497,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079224,39.632046]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1105,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.521,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079361,39.631943]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1106,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.555,"distance_from_trace_point":3.933}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079483,39.631847]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1107,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.585,"distance_from_trace_point":5.441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.079865,39.631565]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1108,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.679,"distance_from_trace_point":5.441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080093,39.631397]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1109,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.734,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080238,39.631287]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1110,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.770,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080460,39.631123]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1111,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.824,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.080734,39.630917]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1112,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.892,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081177,39.630589]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1113,"matched_point_type":"matched","edge_index":117,"distance_along_edge":0.000,"distance_from_trace_point":5.441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081520,39.630337]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1114,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.485,"distance_from_trace_point":5.441}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.081848,39.630100]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1115,"matched_point_type":"matched","edge_index":118,"distance_along_edge":0.944,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.082115,39.629898]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1116,"matched_point_type":"matched","edge_index":119,"distance_along_edge":0.352,"distance_from_trace_point":4.684}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.082596,39.629532]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1117,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.023,"distance_from_trace_point":6.497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083122,39.629128]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1118,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.215,"distance_from_trace_point":7.255}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083176,39.629089]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1119,"matched_point_type":"interpolated","edge_index":120,"distance_along_edge":0.233,"distance_from_trace_point":6.497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.083649,39.628731]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1120,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.405,"distance_from_trace_point":7.555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.084198,39.628323]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1121,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.601,"distance_from_trace_point":6.804}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.084419,39.628155]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1122,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.681,"distance_from_trace_point":7.555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.084930,39.627777]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1123,"matched_point_type":"matched","edge_index":120,"distance_along_edge":0.864,"distance_from_trace_point":7.555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085457,39.627377]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1124,"matched_point_type":"matched","edge_index":121,"distance_along_edge":0.438,"distance_from_trace_point":8.615}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.085762,39.627155]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1125,"matched_point_type":"matched","edge_index":122,"distance_along_edge":0.111,"distance_from_trace_point":7.555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.086304,39.626755]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1126,"matched_point_type":"matched","edge_index":122,"distance_along_edge":0.686,"distance_from_trace_point":7.555}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.086845,39.626362]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1127,"matched_point_type":"matched","edge_index":123,"distance_along_edge":0.536,"distance_from_trace_point":7.867}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087364,39.625980]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1128,"matched_point_type":"matched","edge_index":124,"distance_along_edge":0.226,"distance_from_trace_point":6.805}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.087639,39.625774]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1129,"matched_point_type":"matched","edge_index":124,"distance_along_edge":0.428,"distance_from_trace_point":6.497}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088165,39.625374]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1130,"matched_point_type":"matched","edge_index":124,"distance_along_edge":0.818,"distance_from_trace_point":5.743}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088303,39.625259]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1131,"matched_point_type":"matched","edge_index":124,"distance_along_edge":0.924,"distance_from_trace_point":6.203}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088669,39.625484]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1132,"matched_point_type":"matched","edge_index":125,"distance_along_edge":0.406,"distance_from_trace_point":1.366}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.088989,39.625969]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1133,"matched_point_type":"matched","edge_index":125,"distance_along_edge":1.000,"distance_from_trace_point":6.197}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089249,39.626537]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1134,"matched_point_type":"matched","edge_index":126,"distance_along_edge":0.500,"distance_from_trace_point":4.721}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089317,39.626846]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1135,"matched_point_type":"matched","edge_index":126,"distance_along_edge":0.760,"distance_from_trace_point":4.568}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089371,39.627285]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1136,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.038,"distance_from_trace_point":5.266}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089401,39.627476]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1137,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.088,"distance_from_trace_point":5.908}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089439,39.627666]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1138,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.137,"distance_from_trace_point":7.196}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089485,39.627953]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1139,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.210,"distance_from_trace_point":7.899}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089523,39.628143]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1140,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.260,"distance_from_trace_point":7.899}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089554,39.628330]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1141,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.308,"distance_from_trace_point":5.908}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089584,39.628426]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1142,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.333,"distance_from_trace_point":4.721}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089630,39.628502]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1143,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.354,"distance_from_trace_point":4.890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089752,39.628670]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1144,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.404,"distance_from_trace_point":5.978}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.089943,39.628918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1145,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.477,"distance_from_trace_point":4.648}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090256,39.629223]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1146,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.577,"distance_from_trace_point":4.890}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090508,39.629463]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1147,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.656,"distance_from_trace_point":6.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090759,39.629707]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1148,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.735,"distance_from_trace_point":7.210}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.090904,39.629871]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1149,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.786,"distance_from_trace_point":6.436}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091042,39.630054]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1150,"matched_point_type":"matched","edge_index":127,"distance_along_edge":0.840,"distance_from_trace_point":6.371}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091232,39.630661]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1151,"matched_point_type":"matched","edge_index":127,"distance_along_edge":1.000,"distance_from_trace_point":5.349}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091362,39.631264]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1152,"matched_point_type":"matched","edge_index":128,"distance_along_edge":0.255,"distance_from_trace_point":4.625}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091431,39.631584]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1153,"matched_point_type":"matched","edge_index":128,"distance_along_edge":0.391,"distance_from_trace_point":4.625}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091461,39.631714]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1154,"matched_point_type":"matched","edge_index":128,"distance_along_edge":0.446,"distance_from_trace_point":5.214}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091461,39.631756]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1155,"matched_point_type":"interpolated","edge_index":128,"distance_along_edge":0.463,"distance_from_trace_point":4.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091484,39.631882]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1156,"matched_point_type":"matched","edge_index":128,"distance_along_edge":0.516,"distance_from_trace_point":4.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091492,39.631966]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1157,"matched_point_type":"interpolated","edge_index":128,"distance_along_edge":0.552,"distance_from_trace_point":3.921}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091507,39.632088]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1158,"matched_point_type":"matched","edge_index":128,"distance_along_edge":0.603,"distance_from_trace_point":4.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091530,39.632336]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1159,"matched_point_type":"matched","edge_index":128,"distance_along_edge":0.707,"distance_from_trace_point":3.921}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091515,39.632828]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1160,"matched_point_type":"matched","edge_index":128,"distance_along_edge":0.913,"distance_from_trace_point":4.567}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091492,39.633026]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1161,"matched_point_type":"matched","edge_index":129,"distance_along_edge":0.028,"distance_from_trace_point":1.265}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091530,39.633030]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1162,"matched_point_type":"interpolated","edge_index":129,"distance_along_edge":0.051,"distance_from_trace_point":3.796}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.091835,39.633041]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1163,"matched_point_type":"matched","edge_index":129,"distance_along_edge":0.238,"distance_from_trace_point":7.620}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.092636,39.633072]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1164,"matched_point_type":"matched","edge_index":129,"distance_along_edge":0.729,"distance_from_trace_point":6.780}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.092766,39.633080]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1165,"matched_point_type":"matched","edge_index":129,"distance_along_edge":0.808,"distance_from_trace_point":6.360}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093018,39.633087]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1166,"matched_point_type":"matched","edge_index":129,"distance_along_edge":0.963,"distance_from_trace_point":6.327}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093269,39.633095]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1167,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.048,"distance_from_trace_point":6.748}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093513,39.633102]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1168,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.110,"distance_from_trace_point":7.170}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.093765,39.633110]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1169,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.174,"distance_from_trace_point":7.170}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.094269,39.633129]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1170,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.302,"distance_from_trace_point":7.200}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.094955,39.633156]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1171,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.476,"distance_from_trace_point":7.620}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.095528,39.633186]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1172,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.621,"distance_from_trace_point":6.327}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.095863,39.633198]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1173,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.706,"distance_from_trace_point":5.905}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.095978,39.633202]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1174,"matched_point_type":"interpolated","edge_index":130,"distance_along_edge":0.735,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096436,39.633213]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1175,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.851,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.096992,39.633228]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1176,"matched_point_type":"matched","edge_index":130,"distance_along_edge":0.992,"distance_from_trace_point":4.640}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097786,39.633270]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1177,"matched_point_type":"matched","edge_index":131,"distance_along_edge":0.244,"distance_from_trace_point":5.521}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.097885,39.633278]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1178,"matched_point_type":"interpolated","edge_index":131,"distance_along_edge":0.276,"distance_from_trace_point":5.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.098328,39.633335]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1179,"matched_point_type":"matched","edge_index":131,"distance_along_edge":0.419,"distance_from_trace_point":6.459}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.099037,39.633556]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1180,"matched_point_type":"matched","edge_index":131,"distance_along_edge":0.664,"distance_from_trace_point":4.954}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.099655,39.633816]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1181,"matched_point_type":"matched","edge_index":131,"distance_along_edge":0.888,"distance_from_trace_point":4.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.099747,39.633850]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1182,"matched_point_type":"interpolated","edge_index":131,"distance_along_edge":0.920,"distance_from_trace_point":4.646}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.100479,39.634060]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1183,"matched_point_type":"matched","edge_index":132,"distance_along_edge":0.202,"distance_from_trace_point":5.424}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101143,39.634239]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1184,"matched_point_type":"matched","edge_index":132,"distance_along_edge":0.469,"distance_from_trace_point":4.818}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.101891,39.634441]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1185,"matched_point_type":"matched","edge_index":132,"distance_along_edge":0.770,"distance_from_trace_point":5.819}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.102570,39.634624]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1186,"matched_point_type":"matched","edge_index":133,"distance_along_edge":0.078,"distance_from_trace_point":5.819}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.103226,39.634808]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1187,"matched_point_type":"matched","edge_index":133,"distance_along_edge":0.558,"distance_from_trace_point":6.620}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.103661,39.634918]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1188,"matched_point_type":"matched","edge_index":133,"distance_along_edge":0.873,"distance_from_trace_point":8.425}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.104408,39.635136]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1189,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.096,"distance_from_trace_point":9.434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105125,39.635342]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1190,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.215,"distance_from_trace_point":9.039}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105247,39.635372]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1191,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.235,"distance_from_trace_point":9.434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105522,39.635448]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1192,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.281,"distance_from_trace_point":9.039}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.105804,39.635525]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1193,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.327,"distance_from_trace_point":9.039}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106079,39.635601]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1194,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.373,"distance_from_trace_point":8.425}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106361,39.635670]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1195,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.419,"distance_from_trace_point":8.827}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106499,39.635704]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1196,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.442,"distance_from_trace_point":9.231}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106636,39.635738]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1197,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.464,"distance_from_trace_point":9.231}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106781,39.635773]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1198,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.488,"distance_from_trace_point":9.831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.106918,39.635807]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1199,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.510,"distance_from_trace_point":9.831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107185,39.635876]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1200,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.554,"distance_from_trace_point":10.230}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.107704,39.636021]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1201,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.640,"distance_from_trace_point":9.831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108223,39.636150]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1202,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.725,"distance_from_trace_point":11.437}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108597,39.636246]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1203,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.787,"distance_from_trace_point":12.262}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.108826,39.636318]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1204,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.826,"distance_from_trace_point":11.483}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109520,39.636539]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1205,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.943,"distance_from_trace_point":10.065}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109772,39.636623]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1206,"matched_point_type":"matched","edge_index":134,"distance_along_edge":0.986,"distance_from_trace_point":9.293}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.109970,39.636684]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1207,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.023,"distance_from_trace_point":9.434}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110100,39.636723]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1208,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.049,"distance_from_trace_point":9.831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110245,39.636765]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1209,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.079,"distance_from_trace_point":9.831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.110390,39.636806]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1210,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.108,"distance_from_trace_point":9.831}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111069,39.637005]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1211,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.246,"distance_from_trace_point":9.039}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111725,39.637241]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1212,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.384,"distance_from_trace_point":8.850}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.111839,39.637295]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1213,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.410,"distance_from_trace_point":8.856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112030,39.637402]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1214,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.455,"distance_from_trace_point":8.856}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112221,39.637508]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1215,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.499,"distance_from_trace_point":9.582}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.112801,39.637844]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1216,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.637,"distance_from_trace_point":8.491}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113304,39.638050]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1217,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.746,"distance_from_trace_point":10.230}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113472,39.638100]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1218,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.780,"distance_from_trace_point":11.379}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113541,39.638126]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1219,"matched_point_type":"interpolated","edge_index":135,"distance_along_edge":0.795,"distance_from_trace_point":11.242}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113663,39.638161]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1220,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.820,"distance_from_trace_point":10.631}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.113899,39.638206]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1221,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.866,"distance_from_trace_point":10.043}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114586,39.638298]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1222,"matched_point_type":"matched","edge_index":135,"distance_along_edge":0.999,"distance_from_trace_point":9.895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.114746,39.638321]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1223,"matched_point_type":"matched","edge_index":136,"distance_along_edge":0.057,"distance_from_trace_point":9.895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.115463,39.638435]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1224,"matched_point_type":"matched","edge_index":136,"distance_along_edge":0.328,"distance_from_trace_point":9.895}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116173,39.638584]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1225,"matched_point_type":"matched","edge_index":136,"distance_along_edge":0.601,"distance_from_trace_point":10.455}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116585,39.638710]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1226,"matched_point_type":"matched","edge_index":136,"distance_along_edge":0.764,"distance_from_trace_point":8.647}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.116898,39.638809]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1227,"matched_point_type":"matched","edge_index":136,"distance_along_edge":0.889,"distance_from_trace_point":8.647}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117096,39.638874]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1228,"matched_point_type":"matched","edge_index":136,"distance_along_edge":0.969,"distance_from_trace_point":8.647}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117294,39.638947]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1229,"matched_point_type":"matched","edge_index":137,"distance_along_edge":0.022,"distance_from_trace_point":7.872}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.117851,39.639149]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1230,"matched_point_type":"matched","edge_index":137,"distance_along_edge":0.120,"distance_from_trace_point":6.068}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.118530,39.639378]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1231,"matched_point_type":"matched","edge_index":137,"distance_along_edge":0.239,"distance_from_trace_point":5.819}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119232,39.639599]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1232,"matched_point_type":"matched","edge_index":137,"distance_along_edge":0.361,"distance_from_trace_point":5.819}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119316,39.639622]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1233,"matched_point_type":"interpolated","edge_index":137,"distance_along_edge":0.375,"distance_from_trace_point":6.451}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.119957,39.639858]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1234,"matched_point_type":"matched","edge_index":137,"distance_along_edge":0.489,"distance_from_trace_point":5.318}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120407,39.640057]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1235,"matched_point_type":"matched","edge_index":137,"distance_along_edge":0.572,"distance_from_trace_point":4.954}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120697,39.640179]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1236,"matched_point_type":"matched","edge_index":137,"distance_along_edge":0.625,"distance_from_trace_point":5.318}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120796,39.640221]},"properties":{"marker-color":"#ffffbf","marker-size":"small","matched_point_index":1237,"matched_point_type":"interpolated","edge_index":137,"distance_along_edge":0.643,"distance_from_trace_point":5.318}},
{"type":"Feature","geometry":{"type":"Point","coordinates":[-76.120979,39.640301]},"properties":{"marker-color":"#2c7bb6","marker-size":"medium","matched_point_index":1238,"matched_point_type":"matched","edge_index":137,"distance_along_edge":0.677,"distance_from_trace_point":5.318}}
]}
 */
void TestRoadBikeLunchRideLoop() {
  std::string route_json_str =
      R"({"trip":{"language":"en-US","summary":{"max_lon":-75.985947,"max_lat":39.746597,"time":6389,"length":27.065,"min_lat":39.625179,"min_lon":-76.122726},"locations":[{"lon":-76.122696,"lat":39.651386,"type":"break"},{"lon":-76.120949,"lat":39.640343,"type":"break"}],"units":"miles","legs":[{"shape":"accsjAledepCuKgDsJcF{KgEgc@oR{KuE_O_I}EiCqV}JmKcF{JwDgNgDiSqGsd@yLqWqHsAm@eEkAcB]wHiCcGyByB{@uYeP}@m@}EiCsFwCoHuDkG{BqGiB}JiB}NiCqC]ms@oIkFm@gJ}@mYeD{KkBwH{AiNeEwR_Iwc@aRqa@gOuT_IwS}JcQ{JcFwDkVwNaHgD}EkA{FiBqLiCsKgCgIkBwIm@sj@{@yGOkGm@}DM{Fm@iHkAwH{AqGkBqMeEwIuD_D{AuOaHiLcFmKeF_D{Am@]qQoImKcFaW{K}@m@kKuEkV{KmD{AmA{@{JuE_JeEqMeFwXyL{EiBiMsF_IgDsGyC{EiBmKuEwMqGyB}@e@]qCiBuDgDwCwDgDeEaDqGqAgEsAsFkBoSiCag@oCmi@cAcP{As[_D}}@yCq{@{K_yCuDecAsKelCoIamCqBen@cBgc@cA}^]yM?{K~BgNxCsQTgD\\cFNuE]{KsGey@gDaf@cAePyG{sA}Em~@m@mIoH{tAcAcPiCee@}@qRyLqcC]mI}@gn@cAk`@aC}^}Dq{@aC{i@aCil@u@aSyBco@kAoTMwCOuEGuDm@sQMmJWqHkAmi@]uOGgC_DogAcBwm@iBso@]iL{Awc@sGqdBqBse@{@sPyBikAmFy_ByAc[}@aR{@eO}@aSU}I}@{UiBag@uEm~@iBk`@sAiWeFazA{@e[gEyk@iB_^{A_]m@oSyB__@aCse@aBaf@}Dyl@sBed@cAwMwCmTwDiNcFmJ{LkUkFoIiBuEkBsFaBqHyBgN{A}SwCmi@_@aHiBya@sAil@sAq\\sAkVgDqq@oCso@m@kKaC}i@cBud@aBsd@OeFsAwXyBed@{@eP{A_SmE_h@_DuZe@uDcBsPsAsQUeE_Dm@aC]eAOsAOkA{@aCyBsEwDgDgCgJsG{PiMgCkAcLoIyBkA{FgCeEyBgNcGqMsF}n@{UaMeFyLcFqMeFkKgDyLaG}PaHsKwDiGkAeF]sFz@yGjBkKdEsFhBab@dPkQpGk[xMgCz@ab@vNib@fNuJfDgh@rPcFzAkGjAmEl@kF\\{EM{GkAyGm@wMeEwXmKiXiL_X{LiXyL}XiM}OaHmU{JkKuEmE{AqB}@|Daf@pCwb@tDah@jA_SxBkVjB_ShBuOjAoIdFmUz@wCfDmJvC_I~IaSzEkKlKaSpRm^`R_^hHiM`ByB|EaH|JiLeAaHs@uF}@{KyBq\\cAsPmA_SoCm_@}@}Jk@oIGwCOuE]mJNiLDyBN_Jd@gNl@gY\\_It@qQbAq]t@a\\l@_^zAib@j@kUFwN?cFU{Lu@{KaBo]gEk_AyBse@yGscBG{AeOkpDiB_h@WcQkAcPcAsQsA}IgDuOqC_JmDaHmAyBiG{KiCgDgCeEaCcFcBuEqBsGsAqGu@sF}I_s@kBsPcA_I{AkLaB{KiR_fAaNsp@{E{U}@gDqBuEsAiCsAiBiCiCkKmJkVoRiHeFgDyBs_@wNkKgDaDM}N?uJl@k`@hBc`@?u_@?c`@?yC]sEkBmKaH_ImImEuFoDsEqBwD_D_IyBqGoDiM}DgNwCmKkBqGyBsFqByC}EsE{FsFgIaHuJsG{ZqQg]aSaXeOqWyMuEyB{~@qQ_{AkV_rA_T_^sFif@{KyC{@yBm@qB]uE?{Jj@{`@xC{`@fCy`@xBe`@xBqa@jBc`@hBuT|@eFl@qGOkKl@ud@hCiq@dE}n@fDyp@fDso@fDaSxBw\\xA}Ol@kiBnI}}@bGm^fCouBzLmJl@_lArFqpA`GwDNw\\hB{L|@}`CdOqp@vDwqA`HsqCtOcdAbF{zDnT}YjAm_@\\}Dl@eFhBo]`S_YoIuJyBwC]qG?}J\\_c@dEmENyHO{P{AaMyAwg@_JmPeEwX_I}ScGiw@sQyQwCqGOqCN_DjAmEhCoeBdwAkAzA_@\\qBzAgDjA}Dl@yBMiBm@_YyMueD_fB}YsPqRiMo|@un@p_BmmEdUwm@nm@g`BnN}_@bp@{hBpk@eaBfw@eaCfD{KvDkLlD_IfIuOzVya@jyAkgCbVwb@flAwuBnIgNdi@ux@rAkBrj@y_AhGwNnNuc@b`@oqA|EsQlEaR~CmTbBiM~Hl@fNOtjDkLzJMdFL`C^|DhBhCxBnDtE~CrFz_@fx@pC`HlEfN~BzJnJre@v\\nfB~CdO`Ifc@~M`q@jFzUxCnIpBdE`G|JtOlUlKpQlElKhH`RrAhBxBjBfDhBdE?xdAcGvYkA~H]tD]`I_@|D?piByKdj@kBh\\kArUiBnYmAjj@gCveBoIbGOhk@iBn}@iCl`CaHxRm@r_@iB|wAwCxHOrnAgDpB?fhDmJnIO`HMr`@}@rV{@`MLhMl@x\\hClm@tD~N|@vb@hBzVjA|^zBdZz@bP]dFkBxGm@pGMfx@kBx{@uDv{@eFb{@eEtm@kAh|@uEh{@uDjy@yCvw@oHbk@wDzAdFxB~HpGxVlKvc@hL`g@rL|h@pLnh@nIz`@hBm@jB?tTkAje@}@be@Ode@k@dd@?ld@z@rd@hCxkA\\zd@?dd@l@dd@?zL}@vC{@zAm@nDiCdEeEbe@yk@jLwNrPcQrPuO~SuOre@y`@fSwOjL{KtDsEdF{K~Rse@|JiXxBcFnDqGtTya@~b@uy@`b@ey@`eA_pBhb@gw@jQa]zOc[tJsPpRiWzKiMdKkLrK{J`BmAxHmItDeFbB|@~WrPnJbGtDvCbBhBjA|@l@z@rAxBjAfEzA~HxBhM`Hfc@nHbe@rGvb@`B~HzAtEbAxBjAhC~S|^|JrQtEzJ|DnIvCpHdFxLbLtYnN|_@bKb[xB`H`CfNtJzj@zFdZfDhMpGpQzK|_@bGbZlJre@fNpq@pLxl@|E~RpMxa@x[xjAbLha@vXp{@vNtd@vb@huAtTjt@tT`r@xGjU`IzUbo@xtBzQtn@fNbe@dExLz@hCzGrPfHbQbf@xkAxVvl@jLfXtY~r@|Ylt@tZzs@fDrGnIdZt^fsCpGli@t@nI~ClJ|EvNb`@|gA|EbQxBzL`CtOvC|TpBrP~Hdn@`XvjBdEtZnCpQrGde@~C~RrFzV~CxLhHrQjK`Q`MbQrKhM|T`RjL~IlTfNdZnSbLpGhHbGzKjLlZl_@jUp\\tZng@jZz_@|_DrvDlP`R`a@li@l~@huAb[dd@z[tc@tOlUlE`HzKpRiXbe@_IhMaMxLyGdF}J`HwMnIcRzKgIrF_I`GkFrFgDvDoDrFwCrFwDxLoHfYeKvb@gI`]}Nvl@q]ztAmJ|]cQbp@_Nbf@wClJdJrPjPdZfOjV|I`RvX~g@fItOjGxMfSha@pHfO\\j@pBfEjFlJpHdOvC`Hd@xB?hC?vCvDjAbGpHnNnS|TtYvRjVvSvXnSfYjp@n{@|s@haAllAj~A`MrPlZz`@p\\fb@rVp\\|IjLpHlJpQ|ThHnIxCvChLlKrLtO~\\vb@zPlT`CfDfDdE`ChCbVfYdZ|^fIzK`RhWpHhLvWz`@bLnSpMlTxBvD`Wpf@~NdZbAhCpH`RbFhMjLfXfIdPpVli@`NvXlOb[bVpg@j[tm@z[fn@bUvb@dUx`@|EzKLtO{EtZyGp\\bKdElKrFvRfNpRhNvChBnI`H`RrPvNrPzF`IfNnSnaAvuA~Xfc@ba@xl@~|D`~Fl_@jk@n]ng@|OnTdx@fkArVn^jQxWzAxBtJfNdO|Tjj@bz@~IfNvHzKpCdEhWj`@bFpHdP|TpBfCjF`HiG`Ia]pQ}IbGcMpGiVpGmF|@{PjAgIN}h@rFia@dEiCl@wCzAoTfNgDvC}IlJyMfNmEtEaH`HmEdFeJ~HuDfD}ExBmEhBoD|@yWdE_NhBuJjAsPxBqRhCoNjAgI\\sUlAgOm@uI_@yB]sA]eA}@_C`eBm@`q@mA`|@{@vb@e@vb@OlTGtEEbQWtPe@tYUhLe@zLm@xL{AbQyBxLiCxL{EdPqHzU{AdEaBbGsAtEcB~H}I~g@w]lqB{Kfm@gIrf@wTjhAqQddAyHpf@gCrP}O`|@mEp\\gIl^sFxWgDfOwChL{G~]eD`R}Jbf@qB|IyB`IiC`H}ExKoMhXuE|JeFzJ}D|JwClJ{ArFcA`HcBbGkA`He@bFkA~I_@`H]~Hm@pHu@|I_Dz`@mDj_@_DhXuE~S}EzUqGvXmOhl@u@fCqHr[oSh`AeJp\\oDzKwHnT_@jA{Jb[","summary":{"max_lon":-75.985947,"max_lat":39.746597,"time":6389,"length":27.065,"min_lat":39.625179,"min_lon":-76.122726},"maneuvers":[{"travel_mode":"bicycle","begin_shape_index":0,"length":3.602,"time":906,"type":1,"end_shape_index":157,"instruction":"Bike north on Liberty Grove Road.","verbal_pre_transition_instruction":"Bike north on Liberty Grove Road for 3.6 miles.","travel_type":"road","street_names":["Liberty Grove Road"]},{"travel_type":"road","verbal_pre_transition_instruction":"Continue on Barnes Corner Road for 1.4 miles.","verbal_transition_alert_instruction":"Continue on Barnes Corner Road.","length":1.399,"instruction":"Continue on Barnes Corner Road.","end_shape_index":213,"type":8,"time":341,"street_names":["Barnes Corner Road"],"begin_shape_index":157,"travel_mode":"bicycle"},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Montgomery Road.","verbal_transition_alert_instruction":"Turn left onto Montgomery Road.","length":0.941,"instruction":"Turn left onto Montgomery Road.","end_shape_index":270,"type":15,"time":228,"verbal_post_transition_instruction":"Continue for 9 tenths of a mile.","street_names":["Montgomery Road"],"begin_shape_index":213},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Biggs Highway, Maryland 2 74.","verbal_transition_alert_instruction":"Turn right onto Biggs Highway.","length":0.399,"instruction":"Turn right onto Biggs Highway\/MD 274.","end_shape_index":291,"type":10,"time":94,"verbal_post_transition_instruction":"Continue for 4 tenths of a mile.","street_names":["Biggs Highway","MD 274"],"begin_shape_index":270},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Wilson Road.","verbal_transition_alert_instruction":"Turn left onto Wilson Road.","length":3.444,"instruction":"Turn left onto Wilson Road.","end_shape_index":430,"type":15,"time":793,"verbal_post_transition_instruction":"Continue for 3.4 miles.","street_names":["Wilson Road"],"begin_shape_index":291},{"travel_type":"road","verbal_pre_transition_instruction":"Continue on Stoney Lane for 1.8 miles.","verbal_transition_alert_instruction":"Continue on Stoney Lane.","length":1.798,"instruction":"Continue on Stoney Lane.","end_shape_index":474,"type":8,"time":459,"street_names":["Stoney Lane"],"begin_shape_index":430,"travel_mode":"bicycle"},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto East Christine Road, Pennsylvania 2 72.","verbal_transition_alert_instruction":"Turn right onto East Christine Road.","length":1.492,"instruction":"Turn right onto East Christine Road\/PA 272.","end_shape_index":500,"type":10,"time":334,"verbal_post_transition_instruction":"Continue for 1.5 miles.","street_names":["East Christine Road","PA 272"],"begin_shape_index":474},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto McCoury Road.","verbal_transition_alert_instruction":"Turn right onto McCoury Road.","length":1.142,"instruction":"Turn right onto McCoury Road.","end_shape_index":542,"type":10,"time":229,"verbal_post_transition_instruction":"Continue for 1.1 miles.","street_names":["McCoury Road"],"begin_shape_index":500},{"travel_type":"road","verbal_pre_transition_instruction":"Continue on Chandlee Road for 1.3 miles.","verbal_transition_alert_instruction":"Continue on Chandlee Road.","length":1.250,"instruction":"Continue on Chandlee Road.","end_shape_index":569,"type":8,"time":297,"street_names":["Chandlee Road"],"begin_shape_index":542,"travel_mode":"bicycle"},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Bear left onto Thankless Lane.","verbal_transition_alert_instruction":"Bear left onto Thankless Lane.","length":0.657,"instruction":"Bear left onto Thankless Lane.","end_shape_index":582,"type":16,"time":163,"verbal_post_transition_instruction":"Continue for 7 tenths of a mile.","street_names":["Thankless Lane"],"begin_shape_index":569},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Calvert Road.","verbal_transition_alert_instruction":"Turn right onto Calvert Road.","length":0.218,"instruction":"Turn right onto Calvert Road.","end_shape_index":590,"type":10,"time":52,"verbal_post_transition_instruction":"Continue for 2 tenths of a mile.","street_names":["Calvert Road"],"begin_shape_index":582},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto England Creamery Road.","verbal_transition_alert_instruction":"Turn left onto England Creamery Road.","length":1.430,"instruction":"Turn left onto England Creamery Road.","end_shape_index":637,"type":15,"time":335,"verbal_post_transition_instruction":"Continue for 1.4 miles.","street_names":["England Creamery Road"],"begin_shape_index":590},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Crothers Road.","verbal_transition_alert_instruction":"Turn right onto Crothers Road.","length":1.767,"instruction":"Turn right onto Crothers Road.","end_shape_index":700,"type":10,"time":424,"verbal_post_transition_instruction":"Continue for 1.8 miles.","street_names":["Crothers Road"],"begin_shape_index":637},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Bear right onto Principio Road.","verbal_transition_alert_instruction":"Bear right onto Principio Road.","length":1.612,"instruction":"Bear right onto Principio Road.","end_shape_index":744,"type":9,"time":338,"verbal_post_transition_instruction":"Continue for 1.6 miles.","street_names":["Principio Road"],"begin_shape_index":700},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Red Toad Road.","verbal_transition_alert_instruction":"Turn right onto Red Toad Road.","length":0.556,"instruction":"Turn right onto Red Toad Road.","end_shape_index":767,"type":10,"time":154,"verbal_post_transition_instruction":"Continue for 6 tenths of a mile.","street_names":["Red Toad Road"],"begin_shape_index":744},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Post Road.","verbal_transition_alert_instruction":"Turn left onto Post Road.","length":0.289,"instruction":"Turn left onto Post Road.","end_shape_index":784,"type":15,"time":90,"verbal_post_transition_instruction":"Continue for 3 tenths of a mile.","street_names":["Post Road"],"begin_shape_index":767},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Hopewell Road.","verbal_transition_alert_instruction":"Turn left onto Hopewell Road.","length":1.587,"instruction":"Turn left onto Hopewell Road.","end_shape_index":838,"type":15,"time":386,"verbal_post_transition_instruction":"Continue for 1.6 miles.","street_names":["Hopewell Road"],"begin_shape_index":784},{"travel_type":"road","travel_mode":"bicycle","end_shape_index":870,"verbal_pre_transition_instruction":"Turn left onto Jacob Tome Highway, Maryland 2 76.","begin_street_names":["Jacob Tome Highway","MD 276","MD 222 Truck"],"verbal_transition_alert_instruction":"Turn left onto Jacob Tome Highway.","length":1.226,"instruction":"Turn left onto Jacob Tome Highway\/MD 276\/MD 222 Truck. Continue on Jacob Tome Highway\/MD 276.","type":15,"time":290,"verbal_post_transition_instruction":"Continue on Jacob Tome Highway, Maryland 2 76 for 1.2 miles.","street_names":["Jacob Tome Highway","MD 276"],"begin_shape_index":838},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Waibel Road.","verbal_transition_alert_instruction":"Turn right onto Waibel Road.","length":0.583,"instruction":"Turn right onto Waibel Road.","end_shape_index":907,"type":10,"time":121,"verbal_post_transition_instruction":"Continue for 6 tenths of a mile.","street_names":["Waibel Road"],"begin_shape_index":870},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Doctor Jack Road.","verbal_transition_alert_instruction":"Turn left onto Doctor Jack Road.","length":1.674,"instruction":"Turn left onto Doctor Jack Road.","end_shape_index":980,"type":15,"time":355,"verbal_post_transition_instruction":"Continue for 1.7 miles.","street_names":["Doctor Jack Road"],"begin_shape_index":907},{"travel_type":"road","travel_mode":"bicycle","begin_shape_index":980,"time":0,"type":4,"end_shape_index":980,"instruction":"You have arrived at your destination.","length":0.000,"verbal_transition_alert_instruction":"You will arrive at your destination.","verbal_pre_transition_instruction":"You have arrived at your destination."}]}],"status_message":"Found route between points","status":0}})";
  NavigatorTest nav;
  nav.SetRoute(route_json_str);
  uint32_t leg_index = 0;

  TryRouteLegCount(nav, 1);
  TryRouteManeuverCount(nav, leg_index, 21);
  TryRouteLanguage(nav, "en-US");
  TryRouteUnits(nav, "miles", false);

  ////////////////////////////////////////////////////////////////////////////
  uint32_t maneuver_index = 0;
  uint32_t instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////
  bool expected_on_route_location_close_to_origin = true;

  // reset the route
  nav.SetRoute(route_json_str);

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // Start maneuver at beginning of route
  // trace_pt[0] | begin of maneuver index 0
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.122696f, 39.651386f, 0),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.1227264f, 39.6513939f, leg_index, 27.0611706f,
                                                6390, maneuver_index, 3.60108376f, 905,
                                                instruction_index),
                            expected_on_route_location_close_to_origin);

  // kPreTransition instruction should be used after OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  // reset the route
  nav.SetRoute(route_json_str);

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // kPreTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // No kPreTransition because not within time threshold for pre transition
  // trace_pt[1] | near begin of maneuver index 0 | maneuver speed ~ 14.312582 MPH
  expected_on_route_location_close_to_origin = false;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.122551f, 39.651695f, 6),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1225739f,
                                                39.6517029f, leg_index, 27.0382481f, 6384,
                                                maneuver_index, 3.57816124f, 899));

  // kPreTransition instruction should be not used after OnLocationChanged
  // because not within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // kInitialTransitionAlert instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kInitialTransitionAlert>(
                          nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[179]  | 1 mile before end of maneuver index 0 | maneuver speed ~ 14.312582 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.092705f, 39.672829f, 652),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0927048f,
                                                39.6728363f, leg_index, 24.474535f, 5738,
                                                maneuver_index, 1.01444817f, 253));

  // kInitialTransitionAlert instruction should not be used after OnLocationChanged
  // because the speed was not high enough for an initial transition alert.
  TryUsedInstructions(std::get<kInitialTransitionAlert>(
                          nav.used_instructions().at(instruction_index)),
                      false);

  // kFinalTransitionAlert instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      false);

  // TODO: Reconsider speed bounds for transition alerts. Must be going over 62 MPH for long final
  // transition
  //       but only 40 MPH for initial short when a route manuever like this one qualifies for both.
  // trace_pt[200]  | 1/2 mile before end of maneuver index 0 | maneuver speed ~ 14.312582 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.083069f, 39.673717f, 781),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0830765f,
                                                39.673748f, leg_index, 23.9578571f, 5609,
                                                maneuver_index, 0.497770309f, 124));

  // kFinalTransitionAlert instruction should not be used after OnLocationChanged
  // because the speed was not high enough for a final transition alert.
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      false);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 2;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // Increment instruction_index for testing instructions prior to next maneuver
  ++instruction_index;

  // kFinalTransitionAlert instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kFinalTransitionAlert>(nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[304] | near end of maneuver index 2 | fix speed >~ 15 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.045883f, 39.689846f, 1458, 6.75f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0459213f,
                                                39.6898575f, leg_index, 21.1585541f, 4932,
                                                maneuver_index, 0.0377235413f, 10));

  // kPreTransition instruction should not be used after OnLocationChanged
  // because it is not within the time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), false);

  // trace_pt[305] | near end of maneuver index 2 | fix speed >~ 15 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.045692f, 39.690193f, 1465, 6.75f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0457382f, 39.6902046f, leg_index, 21.1326389f,
                                                4925, maneuver_index, 0.0118083954f, 3,
                                                instruction_index));

  // kPreTransition instruction should be used after OnLocationChanged
  // because it was within time threshold for pre transition
  TryUsedInstructions(std::get<kPreTransition>(nav.used_instructions().at(instruction_index)), true);

  ////////////////////////////////////////////////////////////////////////////
  maneuver_index = 3;
  instruction_index = maneuver_index;
  ////////////////////////////////////////////////////////////////////////////

  // kPostTransition instruction should not be used prior to OnLocationChanged
  TryUsedInstructions(std::get<kPostTransition>(nav.used_instructions().at(instruction_index)),
                      false);

  // trace_pt[307] | near end of maneuver index 0 | manuever speed ~ 15.2808495 MPH
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.045387f, 39.690315f, 1472),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.0453873f, 39.6903229f, leg_index, 21.1059246f,
                                                4918, maneuver_index, 0.384138107f, 89,
                                                instruction_index));

  // kPostTransition instruction should be used after OnLocationChanged
  // because it was within the time threshold for post-transition
  TryUsedInstructions(std::get<kPostTransition>(nav.used_instructions().at(instruction_index)), true);

  //----------------------------------------------------------------
  // reset the route
  nav.SetRoute(route_json_str);

  //----------------------------------------------------------------
  // trace point = 0
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1226959f, 39.6513863f, 1483287460, 0),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.1227264f, 39.6513939f, leg_index, 27.0608215f,
                                                6388, maneuver_index, 3.60108376f, 905,
                                                instruction_index),
                            true);
  //----------------------------------------------------------------
  // trace point = 1
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.122551f, 39.6516953f, 1483287463, 12.1411963),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1225739f,
                                                39.6517029f, leg_index, 27.037899f, 6382,
                                                maneuver_index, 3.57816124f, 899));
  //----------------------------------------------------------------
  // trace point = 2
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1224976f, 39.6518021f, 1483287464, 12.9772072),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1225128f,
                                                39.6518059f, leg_index, 27.0300808f, 6379,
                                                maneuver_index, 3.57034302f, 896));
  //----------------------------------------------------------------
  // trace point = 3
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1224365f, 39.6519165f, 1483287465, 13.4944811),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1224594f,
                                                39.6519241f, leg_index, 27.0214214f, 6377,
                                                maneuver_index, 3.56168365f, 894));
  //----------------------------------------------------------------
  // trace point = 4
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1223755f, 39.6520309f, 1483287466, 13.8678951),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1223984f,
                                                39.6520386f, leg_index, 27.0128574f, 6375,
                                                maneuver_index, 3.55311966f, 892));
  //----------------------------------------------------------------
  // trace point = 5
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1223145f, 39.6521492f, 1483287467, 14.0667114),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1223373f,
                                                39.6521568f, leg_index, 27.0040569f, 6373,
                                                maneuver_index, 3.54431915f, 890));
  //----------------------------------------------------------------
  // trace point = 6
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1222534f, 39.6522713f, 1483287468, 14.5437498),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1222763f,
                                                39.6522789f, leg_index, 26.9950123f, 6370,
                                                maneuver_index, 3.53527451f, 887));
  //----------------------------------------------------------------
  // trace point = 7
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1221848f, 39.6523933f, 1483287469, 14.9177284),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1222076f,
                                                39.652401f, leg_index, 26.9858112f, 6368,
                                                maneuver_index, 3.52607346f, 885));
  //----------------------------------------------------------------
  // trace point = 8
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1221161f, 39.6525192f, 1483287470, 15.1562376),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.122139f,
                                                39.6525269f, leg_index, 26.976368f, 6366,
                                                maneuver_index, 3.51663017f, 883));
  //----------------------------------------------------------------
  // trace point = 9
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1219788f, 39.6527748f, 1483287472, 15.4438753),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1220016f,
                                                39.6527824f, leg_index, 26.9572239f, 6361,
                                                maneuver_index, 3.49748611f, 878));
  //----------------------------------------------------------------
  // trace point = 10
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1219025f, 39.6529045f, 1483287473, 15.6671753),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1219254f,
                                                39.6529121f, leg_index, 26.9473782f, 6358,
                                                maneuver_index, 3.48764038f, 875));
  //----------------------------------------------------------------
  // trace point = 11
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1218262f, 39.6530342f, 1483287474, 15.7684803),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1218414f,
                                                39.6530418f, leg_index, 26.937355f, 6356,
                                                maneuver_index, 3.47761726f, 873));
  //----------------------------------------------------------------
  // trace point = 12
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1216736f, 39.6532898f, 1483287476, 15.8220129),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1217041f,
                                                39.6532974f, leg_index, 26.9182091f, 6351,
                                                maneuver_index, 3.4584713f, 868));
  //----------------------------------------------------------------
  // trace point = 13
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1215973f, 39.6534195f, 1483287477, 15.6640024),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1216354f,
                                                39.6534309f, leg_index, 26.9082756f, 6349,
                                                maneuver_index, 3.44853783f, 866));
  //----------------------------------------------------------------
  // trace point = 14
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1215286f, 39.6535454f, 1483287478, 15.2219763),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1215668f,
                                                39.6535568f, leg_index, 26.8988285f, 6346,
                                                maneuver_index, 3.43909073f, 863));
  //----------------------------------------------------------------
  // trace point = 15
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1214676f, 39.6536674f, 1483287479, 14.7822304),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1214981f,
                                                39.6536789f, leg_index, 26.8896275f, 6344,
                                                maneuver_index, 3.42988968f, 861));
  //----------------------------------------------------------------
  // trace point = 16
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1213531f, 39.6539192f, 1483287481, 14.7055006),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.121376f,
                                                39.653923f, leg_index, 26.8715324f, 6339,
                                                maneuver_index, 3.41179466f, 856));
  //----------------------------------------------------------------
  // trace point = 17
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1213074f, 39.6540413f, 1483287482, 14.4283237),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1213379f,
                                                39.6540489f, leg_index, 26.8625946f, 6337,
                                                maneuver_index, 3.40285683f, 854));
  //----------------------------------------------------------------
  // trace point = 18
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.121254f, 39.6541672f, 1483287483, 14.4535007),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1212921f,
                                                39.6541748f, leg_index, 26.8535557f, 6335,
                                                maneuver_index, 3.3938179f, 852));
  //----------------------------------------------------------------
  // trace point = 19
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1212082f, 39.6542892f, 1483287484, 14.2969599),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1212387f,
                                                39.6542969f, leg_index, 26.8446465f, 6332,
                                                maneuver_index, 3.38490868f, 849));
  //----------------------------------------------------------------
  // trace point = 20
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1211624f, 39.6544151f, 1483287485, 14.3793602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1211853f,
                                                39.6544228f, leg_index, 26.8354874f, 6330,
                                                maneuver_index, 3.37574959f, 847));
  //----------------------------------------------------------------
  // trace point = 21
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1210785f, 39.6546707f, 1483287487, 14.7298326),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1210938f,
                                                39.6546745f, leg_index, 26.8174f, 6325,
                                                maneuver_index, 3.3576622f, 842));
  //----------------------------------------------------------------
  // trace point = 22
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1210327f, 39.6548004f, 1483287488, 14.9989386),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1210403f,
                                                39.6548042f, leg_index, 26.807991f, 6323,
                                                maneuver_index, 3.34825325f, 840));
  //----------------------------------------------------------------
  // trace point = 23
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1209259f, 39.655056f, 1483287490, 14.9424448),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1209488f,
                                                39.6550598f, leg_index, 26.7882729f, 6318,
                                                maneuver_index, 3.32853508f, 835));
  //----------------------------------------------------------------
  // trace point = 24
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1208267f, 39.655304f, 1483287492, 14.492116),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1208496f,
                                                39.6553078f, leg_index, 26.7717075f, 6314,
                                                maneuver_index, 3.31196976f, 831));
  //----------------------------------------------------------------
  // trace point = 25
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1207733f, 39.6554222f, 1483287493, 13.7112236),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1208038f,
                                                39.6554298f, leg_index, 26.7629185f, 6312,
                                                maneuver_index, 3.30318069f, 829));
  //----------------------------------------------------------------
  // trace point = 26
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1207275f, 39.6555367f, 1483287494, 13.6327591),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1207581f,
                                                39.6555443f, leg_index, 26.7546215f, 6309,
                                                maneuver_index, 3.29488373f, 826));
  //----------------------------------------------------------------
  // trace point = 27
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1206207f, 39.6557617f, 1483287496, 13.2025013),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1206665f,
                                                39.6557732f, leg_index, 26.7380505f, 6305,
                                                maneuver_index, 3.27831268f, 822));
  //----------------------------------------------------------------
  // trace point = 28
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1204758f, 39.6560745f, 1483287499, 12.2673788),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1205063f,
                                                39.656086f, leg_index, 26.7147808f, 6300,
                                                maneuver_index, 3.25504303f, 817));
  //----------------------------------------------------------------
  // trace point = 29
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1203232f, 39.6563454f, 1483287502, 11.0330915),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1203308f,
                                                39.6563492f, leg_index, 26.6943188f, 6295,
                                                maneuver_index, 3.23458099f, 812));
  //----------------------------------------------------------------
  // trace point = 30
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1200714f, 39.6567383f, 1483287508, 8.11488247),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1200867f,
                                                39.6567421f, leg_index, 26.6641827f, 6287,
                                                maneuver_index, 3.20444489f, 804));
  //----------------------------------------------------------------
  // trace point = 31
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1200104f, 39.6568375f, 1483287510, 6.13111305),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1200256f,
                                                39.6568413f, leg_index, 26.6557007f, 6285,
                                                maneuver_index, 3.19596291f, 802));
  //----------------------------------------------------------------
  // trace point = 32
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.119957f, 39.6569328f, 1483287512, 5.75689411),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1199799f,
                                                39.6569405f, leg_index, 26.6493034f, 6284,
                                                maneuver_index, 3.18956566f, 801));
  //----------------------------------------------------------------
  // trace point = 33
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1199417f, 39.6569786f, 1483287513, 5.39995384),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.119957f,
                                                39.6569824f, leg_index, 26.6459103f, 6283,
                                                maneuver_index, 3.18617249f, 800));
  //----------------------------------------------------------------
  // trace point = 34
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1199188f, 39.6570244f, 1483287514, 5.11015177),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1199417f,
                                                39.6570282f, leg_index, 26.6428967f, 6283,
                                                maneuver_index, 3.18315887f, 800));
  //----------------------------------------------------------------
  // trace point = 35
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1198883f, 39.6571083f, 1483287516, 5.09535217),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1199112f,
                                                39.6571121f, leg_index, 26.635994f, 6281,
                                                maneuver_index, 3.17625618f, 798));
  //----------------------------------------------------------------
  // trace point = 36
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1198425f, 39.657238f, 1483287519, 4.90591097),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.119873f,
                                                39.6572418f, leg_index, 26.6276569f, 6279,
                                                maneuver_index, 3.16791916f, 796));
  //----------------------------------------------------------------
  // trace point = 37
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1198273f, 39.6573219f, 1483287521, 4.79354095),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1198502f,
                                                39.6573257f, leg_index, 26.6217232f, 6277,
                                                maneuver_index, 3.1619854f, 794));
  //----------------------------------------------------------------
  // trace point = 38
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1198044f, 39.6574097f, 1483287523, 4.862854),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1198273f,
                                                39.6574135f, leg_index, 26.6155338f, 6276,
                                                maneuver_index, 3.15579605f, 793));
  //----------------------------------------------------------------
  // trace point = 39
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1197662f, 39.6575851f, 1483287527, 5.01340723),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1197815f,
                                                39.6575851f, leg_index, 26.603405f, 6273,
                                                maneuver_index, 3.14366722f, 790));
  //----------------------------------------------------------------
  // trace point = 40
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1197205f, 39.6578102f, 1483287532, 5.02706623),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1197357f,
                                                39.6578102f, leg_index, 26.5876427f, 6269,
                                                maneuver_index, 3.12790489f, 786));
  //----------------------------------------------------------------
  // trace point = 41
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1196518f, 39.6582146f, 1483287541, 5.07094383),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1196518f,
                                                39.6582146f, leg_index, 26.5593185f, 6262,
                                                maneuver_index, 3.09958076f, 779));
  //----------------------------------------------------------------
  // trace point = 42
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1196136f, 39.6583977f, 1483287545, 5.15094185),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1196136f,
                                                39.6583977f, leg_index, 26.5464916f, 6258,
                                                maneuver_index, 3.08675385f, 775));
  //----------------------------------------------------------------
  // trace point = 43
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1194839f, 39.6589775f, 1483287557, 5.44484711),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1195068f,
                                                39.6589813f, leg_index, 26.5057201f, 6249,
                                                maneuver_index, 3.04598236f, 766));
  //----------------------------------------------------------------
  // trace point = 44
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1193542f, 39.6595383f, 1483287568, 5.75257778),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1193695f,
                                                39.6595421f, leg_index, 26.466238f, 6238,
                                                maneuver_index, 3.00650024f, 755));
  //----------------------------------------------------------------
  // trace point = 45
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1193161f, 39.6596336f, 1483287570, 5.49624681),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1193314f,
                                                39.6596375f, leg_index, 26.4593334f, 6237,
                                                maneuver_index, 2.99959564f, 754));
  //----------------------------------------------------------------
  // trace point = 46
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.119072f, 39.6601677f, 1483287581, 5.75987816),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1190796f,
                                                39.6601715f, leg_index, 26.4200134f, 6227,
                                                maneuver_index, 2.96027565f, 744));
  //----------------------------------------------------------------
  // trace point = 47
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1187973f, 39.6606979f, 1483287592, 5.7655921),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1188049f,
                                                39.6607018f, leg_index, 26.3796406f, 6217,
                                                maneuver_index, 2.9199028f, 734));
  //----------------------------------------------------------------
  // trace point = 48
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1185837f, 39.6611176f, 1483287602, 5.01714277),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1186066f,
                                                39.6611252f, leg_index, 26.3493881f, 6210,
                                                maneuver_index, 2.88965034f, 727));
  //----------------------------------------------------------------
  // trace point = 49
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.118454f, 39.6613693f, 1483287609, 4.29718494),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1184845f,
                                                39.661377f, leg_index, 26.330801f, 6205,
                                                maneuver_index, 2.87106323f, 722));
  //----------------------------------------------------------------
  // trace point = 50
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1184006f, 39.6614685f, 1483287612, 4.04200602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1184387f,
                                                39.6614799f, leg_index, 26.3232708f, 6203,
                                                maneuver_index, 2.86353302f, 720));
  //----------------------------------------------------------------
  // trace point = 51
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1183701f, 39.6615334f, 1483287614, 3.749717),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1184082f,
                                                39.6615448f, leg_index, 26.3185005f, 6202,
                                                maneuver_index, 2.85876274f, 719));
  //----------------------------------------------------------------
  // trace point = 52
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1182785f, 39.6617088f, 1483287620, 3.5026381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1183167f,
                                                39.6617203f, leg_index, 26.3054085f, 6199,
                                                maneuver_index, 2.8456707f, 716));
  //----------------------------------------------------------------
  // trace point = 53
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.118248f, 39.6617661f, 1483287622, 3.45622206),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1182861f,
                                                39.6617775f, leg_index, 26.3011322f, 6198,
                                                maneuver_index, 2.84139442f, 715));
  //----------------------------------------------------------------
  // trace point = 54
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1182175f, 39.6618195f, 1483287624, 3.37206888),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1182556f,
                                                39.6618309f, leg_index, 26.2970982f, 6197,
                                                maneuver_index, 2.83736038f, 714));
  //----------------------------------------------------------------
  // trace point = 55
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1181717f, 39.6618996f, 1483287627, 3.19053698),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1182098f,
                                                39.661911f, leg_index, 26.291048f, 6196,
                                                maneuver_index, 2.83131027f, 713));
  //----------------------------------------------------------------
  // trace point = 56
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1181107f, 39.6619949f, 1483287631, 2.99919891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1181488f,
                                                39.6620102f, leg_index, 26.2834587f, 6193,
                                                maneuver_index, 2.82372093f, 710));
  //----------------------------------------------------------------
  // trace point = 57
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.118042f, 39.6620865f, 1483287635, 2.91455293),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1180801f,
                                                39.6621017f, leg_index, 26.2761574f, 6192,
                                                maneuver_index, 2.8164196f, 709));
  //----------------------------------------------------------------
  // trace point = 58
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1179657f, 39.6622047f, 1483287640, 2.91602397),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1180038f,
                                                39.66222f, leg_index, 26.2658978f, 6189,
                                                maneuver_index, 2.80615997f, 706));
  //----------------------------------------------------------------
  // trace point = 59
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1179123f, 39.6622734f, 1483287643, 2.99218607),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1179504f,
                                                39.6622925f, leg_index, 26.2612495f, 6188,
                                                maneuver_index, 2.80151176f, 705));
  //----------------------------------------------------------------
  // trace point = 60
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1178284f, 39.6623802f, 1483287648, 2.81140208),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1178741f,
                                                39.6623993f, leg_index, 26.2528095f, 6186,
                                                maneuver_index, 2.79307175f, 703));
  //----------------------------------------------------------------
  // trace point = 61
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1177368f, 39.6625137f, 1483287654, 2.75926995),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1177826f,
                                                39.6625328f, leg_index, 26.2423668f, 6184,
                                                maneuver_index, 2.78262901f, 701));
  //----------------------------------------------------------------
  // trace point = 62
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1176682f, 39.6626244f, 1483287659, 2.73813701),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1177063f,
                                                39.6626396f, leg_index, 26.2339382f, 6182,
                                                maneuver_index, 2.77420044f, 699));
  //----------------------------------------------------------------
  // trace point = 63
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1176147f, 39.6627121f, 1483287663, 2.67422605),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1176529f,
                                                39.6627235f, leg_index, 26.2274742f, 6180,
                                                maneuver_index, 2.76773643f, 697));
  //----------------------------------------------------------------
  // trace point = 64
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.117569f, 39.662796f, 1483287667, 2.57939291),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1176071f,
                                                39.6628075f, leg_index, 26.2211819f, 6178,
                                                maneuver_index, 2.76144409f, 695));
  //----------------------------------------------------------------
  // trace point = 65
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1175156f, 39.6629066f, 1483287672, 2.61206388),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1175537f,
                                                39.6629143f, leg_index, 26.2132282f, 6176,
                                                maneuver_index, 2.75349045f, 693));
  //----------------------------------------------------------------
  // trace point = 66
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1174393f, 39.6630821f, 1483287680, 2.55783606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.117485f,
                                                39.6630936f, leg_index, 26.199419f, 6173,
                                                maneuver_index, 2.73968124f, 690));
  //----------------------------------------------------------------
  // trace point = 67
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1174164f, 39.663147f, 1483287683, 2.50746107),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1174622f,
                                                39.6631546f, leg_index, 26.1958942f, 6172,
                                                maneuver_index, 2.73615646f, 689));
  //----------------------------------------------------------------
  // trace point = 68
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1173935f, 39.6632118f, 1483287686, 2.43604398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1174393f,
                                                39.6632195f, leg_index, 26.1912479f, 6171,
                                                maneuver_index, 2.73151016f, 688));
  //----------------------------------------------------------------
  // trace point = 69
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1173477f, 39.6633797f, 1483287694, 2.42780209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1173859f,
                                                39.6633873f, leg_index, 26.1792927f, 6167,
                                                maneuver_index, 2.7195549f, 684));
  //----------------------------------------------------------------
  // trace point = 70
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1173172f, 39.6634865f, 1483287699, 2.43709302),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1173477f,
                                                39.6634941f, leg_index, 26.1716347f, 6166,
                                                maneuver_index, 2.7118969f, 683));
  //----------------------------------------------------------------
  // trace point = 71
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1172943f, 39.6635704f, 1483287703, 2.36930203),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1173248f,
                                                39.6635742f, leg_index, 26.1659603f, 6164,
                                                maneuver_index, 2.70622253f, 681));
  //----------------------------------------------------------------
  // trace point = 72
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1172714f, 39.6637001f, 1483287709, 2.43418908),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1172867f,
                                                39.6637001f, leg_index, 26.1569996f, 6162,
                                                maneuver_index, 2.69726181f, 679));
  //----------------------------------------------------------------
  // trace point = 73
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1172485f, 39.663868f, 1483287716, 2.67625499),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1172638f,
                                                39.663868f, leg_index, 26.1453228f, 6159,
                                                maneuver_index, 2.68558502f, 676));
  //----------------------------------------------------------------
  // trace point = 74
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1172409f, 39.66399f, 1483287721, 2.72159696),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1172562f,
                                                39.66399f, leg_index, 26.1368713f, 6157,
                                                maneuver_index, 2.67713356f, 674));
  //----------------------------------------------------------------
  // trace point = 75
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1172256f, 39.6641464f, 1483287727, 2.90025592),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1172485f,
                                                39.6641464f, leg_index, 26.1260452f, 6154,
                                                maneuver_index, 2.66630745f, 671));
  //----------------------------------------------------------------
  // trace point = 76
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.117218f, 39.6643257f, 1483287733, 3.3410809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1172409f,
                                                39.6643257f, leg_index, 26.113636f, 6151,
                                                maneuver_index, 2.65389824f, 668));
  //----------------------------------------------------------------
  // trace point = 77
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1172104f, 39.6644936f, 1483287738, 3.698349),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1172333f,
                                                39.6644936f, leg_index, 26.1020222f, 6148,
                                                maneuver_index, 2.64228439f, 665));
  //----------------------------------------------------------------
  // trace point = 78
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1171494f, 39.6650467f, 1483287751, 4.76837301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1171722f,
                                                39.6650505f, leg_index, 26.0630474f, 6139,
                                                maneuver_index, 2.60330963f, 656));
  //----------------------------------------------------------------
  // trace point = 79
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1170654f, 39.6653519f, 1483287758, 4.92704582),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1170883f,
                                                39.6653557f, leg_index, 26.0414696f, 6133,
                                                maneuver_index, 2.5817318f, 650));
  //----------------------------------------------------------------
  // trace point = 80
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1169968f, 39.6655388f, 1483287763, 4.36371708),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.117012f,
                                                39.6655426f, leg_index, 26.0281658f, 6130,
                                                maneuver_index, 2.56842804f, 647));
  //----------------------------------------------------------------
  // trace point = 81
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1169357f, 39.6656761f, 1483287767, 4.07076883),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1169586f,
                                                39.6656799f, leg_index, 26.0182552f, 6128,
                                                maneuver_index, 2.55851746f, 645));
  //----------------------------------------------------------------
  // trace point = 82
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1168518f, 39.6658363f, 1483287772, 3.86277795),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1168747f,
                                                39.665844f, leg_index, 26.0060558f, 6125,
                                                maneuver_index, 2.54631805f, 642));
  //----------------------------------------------------------------
  // trace point = 83
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1167831f, 39.665966f, 1483287776, 3.840523),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.116806f,
                                                39.6659737f, leg_index, 25.9957237f, 6122,
                                                maneuver_index, 2.53598595f, 639));
  //----------------------------------------------------------------
  // trace point = 84
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1166687f, 39.6661606f, 1483287782, 3.99013805),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1166992f,
                                                39.6661682f, leg_index, 25.98176f, 6119,
                                                maneuver_index, 2.52202225f, 636));
  //----------------------------------------------------------------
  // trace point = 85
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1163635f, 39.6667023f, 1483287796, 4.68446016),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.116394f,
                                                39.6667137f, leg_index, 25.9406738f, 6108,
                                                maneuver_index, 2.48093605f, 625));
  //----------------------------------------------------------------
  // trace point = 86
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1162415f, 39.6669197f, 1483287800, 6.57203293),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.116272f,
                                                39.6669273f, leg_index, 25.9245281f, 6104,
                                                maneuver_index, 2.46479034f, 621));
  //----------------------------------------------------------------
  // trace point = 87
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1159363f, 39.6674614f, 1483287808, 8.21492386),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1159668f,
                                                39.6674728f, leg_index, 25.8834419f, 6093,
                                                maneuver_index, 2.42370415f, 610));
  //----------------------------------------------------------------
  // trace point = 88
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1158981f, 39.6675377f, 1483287809, 9.22783756),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1159286f,
                                                39.6675453f, leg_index, 25.8780327f, 6092,
                                                maneuver_index, 2.41829491f, 609));
  //----------------------------------------------------------------
  // trace point = 89
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1158066f, 39.6676903f, 1483287811, 9.31316376),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1158371f,
                                                39.6677017f, leg_index, 25.8661499f, 6088,
                                                maneuver_index, 2.40641212f, 605));
  //----------------------------------------------------------------
  // trace point = 90
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1156464f, 39.6679878f, 1483287815, 8.9562006),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1156769f,
                                                39.6679993f, leg_index, 25.8438702f, 6083,
                                                maneuver_index, 2.38413239f, 600));
  //----------------------------------------------------------------
  // trace point = 91
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1153488f, 39.6685371f, 1483287823, 8.28760147),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1153641f,
                                                39.6685448f, leg_index, 25.8012218f, 6071,
                                                maneuver_index, 2.34148407f, 588));
  //----------------------------------------------------------------
  // trace point = 92
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1150665f, 39.6690674f, 1483287832, 7.07979918),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1150894f,
                                                39.669075f, leg_index, 25.7631054f, 6062,
                                                maneuver_index, 2.30336761f, 579));
  //----------------------------------------------------------------
  // trace point = 93
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1149673f, 39.669239f, 1483287836, 5.24383593),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1149979f,
                                                39.6692467f, leg_index, 25.7502689f, 6059,
                                                maneuver_index, 2.29053116f, 576));
  //----------------------------------------------------------------
  // trace point = 94
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1148834f, 39.6693916f, 1483287840, 4.57218695),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1149139f,
                                                39.6693993f, leg_index, 25.7388077f, 6056,
                                                maneuver_index, 2.2790699f, 573));
  //----------------------------------------------------------------
  // trace point = 95
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1148529f, 39.6694603f, 1483287842, 4.08706284),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1148834f,
                                                39.6694679f, leg_index, 25.7337933f, 6055,
                                                maneuver_index, 2.27405548f, 572));
  //----------------------------------------------------------------
  // trace point = 96
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1148148f, 39.6695251f, 1483287844, 3.96732092),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1148453f,
                                                39.6695366f, leg_index, 25.7286263f, 6054,
                                                maneuver_index, 2.26888847f, 571));
  //----------------------------------------------------------------
  // trace point = 97
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1147614f, 39.6696243f, 1483287847, 3.98419499),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1147919f,
                                                39.6696358f, leg_index, 25.7212009f, 6052,
                                                maneuver_index, 2.26146317f, 569));
  //----------------------------------------------------------------
  // trace point = 98
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1147079f, 39.6697197f, 1483287850, 3.90471101),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1147461f,
                                                39.6697235f, leg_index, 25.7146606f, 6050,
                                                maneuver_index, 2.25492287f, 567));
  //----------------------------------------------------------------
  // trace point = 99
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1146393f, 39.6698494f, 1483287854, 3.84855008),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1146774f,
                                                39.6698608f, leg_index, 25.7044907f, 6047,
                                                maneuver_index, 2.24475288f, 564));
  //----------------------------------------------------------------
  // trace point = 100
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1145706f, 39.6699753f, 1483287858, 3.77290106),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1146088f,
                                                39.6699867f, leg_index, 25.6950493f, 6045,
                                                maneuver_index, 2.23531151f, 562));
  //----------------------------------------------------------------
  // trace point = 101
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1144791f, 39.6701546f, 1483287864, 3.57675791),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1145172f,
                                                39.670166f, leg_index, 25.6817131f, 6041,
                                                maneuver_index, 2.22197533f, 558));
  //----------------------------------------------------------------
  // trace point = 102
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1144333f, 39.6702347f, 1483287867, 3.3257041),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1144714f,
                                                39.6702461f, leg_index, 25.6756611f, 6040,
                                                maneuver_index, 2.21592331f, 557));
  //----------------------------------------------------------------
  // trace point = 103
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1143875f, 39.6703148f, 1483287870, 3.20174289),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1144257f,
                                                39.6703262f, leg_index, 25.6693649f, 6038,
                                                maneuver_index, 2.20962715f, 555));
  //----------------------------------------------------------------
  // trace point = 104
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1142807f, 39.6705208f, 1483287878, 3.08222198),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1143112f,
                                                39.6705322f, leg_index, 25.6541157f, 6034,
                                                maneuver_index, 2.1943779f, 551));
  //----------------------------------------------------------------
  // trace point = 105
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1141739f, 39.6706886f, 1483287884, 3.52999401),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1142044f,
                                                39.6707001f, leg_index, 25.6396084f, 6031,
                                                maneuver_index, 2.17987061f, 548));
  //----------------------------------------------------------------
  // trace point = 106
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1140518f, 39.670826f, 1483287889, 3.661273),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1140671f,
                                                39.6708412f, leg_index, 25.6289291f, 6028,
                                                maneuver_index, 2.16919136f, 545));
  //----------------------------------------------------------------
  // trace point = 107
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1138306f, 39.6709938f, 1483287896, 3.810817),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1138458f,
                                                39.6710091f, leg_index, 25.6123199f, 6024,
                                                maneuver_index, 2.15258217f, 541));
  //----------------------------------------------------------------
  // trace point = 108
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1135254f, 39.6711159f, 1483287903, 4.21138382),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.113533f,
                                                39.6711388f, leg_index, 25.5932636f, 6019,
                                                maneuver_index, 2.13352585f, 536));
  //----------------------------------------------------------------
  // trace point = 109
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1134262f, 39.6711426f, 1483287905, 4.44679499),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1134262f,
                                                39.671154f, leg_index, 25.5874805f, 6018,
                                                maneuver_index, 2.12774277f, 535));
  //----------------------------------------------------------------
  // trace point = 110
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1128998f, 39.6712151f, 1483287915, 4.54597521),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1128998f,
                                                39.6712227f, leg_index, 25.5590286f, 6010,
                                                maneuver_index, 2.09929085f, 527));
  //----------------------------------------------------------------
  // trace point = 111
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1123505f, 39.6712761f, 1483287925, 4.80694294),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1123505f,
                                                39.6712837f, leg_index, 25.5294781f, 6003,
                                                maneuver_index, 2.0697403f, 520));
  //----------------------------------------------------------------
  // trace point = 112
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1122284f, 39.6712875f, 1483287927, 5.21996117),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1122284f,
                                                39.6712952f, leg_index, 25.5229321f, 6001,
                                                maneuver_index, 2.06319427f, 518));
  //----------------------------------------------------------------
  // trace point = 113
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1114883f, 39.6713638f, 1483287938, 5.78580189),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1114883f,
                                                39.6713791f, leg_index, 25.4831028f, 5991,
                                                maneuver_index, 2.02336502f, 508));
  //----------------------------------------------------------------
  // trace point = 114
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1107254f, 39.6714439f, 1483287946, 8.28823853),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1107254f,
                                                39.6714439f, leg_index, 25.4422302f, 5981,
                                                maneuver_index, 1.98249245f, 498));
  //----------------------------------------------------------------
  // trace point = 115
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1104813f, 39.6714668f, 1483287948, 10.3483734),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1104813f,
                                                39.671463f, leg_index, 25.4291649f, 5977,
                                                maneuver_index, 1.96942711f, 494));
  //----------------------------------------------------------------
  // trace point = 116
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1100922f, 39.6714973f, 1483287951, 11.2953548),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1100922f,
                                                39.6714973f, leg_index, 25.4083138f, 5972,
                                                maneuver_index, 1.94857597f, 489));
  //----------------------------------------------------------------
  // trace point = 117
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1098022f, 39.6715202f, 1483287953, 12.2286634),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1098022f,
                                                39.6715202f, leg_index, 25.3927975f, 5968,
                                                maneuver_index, 1.93305969f, 485));
  //----------------------------------------------------------------
  // trace point = 118
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1095123f, 39.6715431f, 1483287955, 12.7457571),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1095123f,
                                                39.6715431f, leg_index, 25.3772812f, 5964,
                                                maneuver_index, 1.91754341f, 481));
  //----------------------------------------------------------------
  // trace point = 119
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1091995f, 39.6715698f, 1483287957, 13.317112),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1091995f,
                                                39.671566f, leg_index, 25.3605518f, 5960,
                                                maneuver_index, 1.90081406f, 477));
  //----------------------------------------------------------------
  // trace point = 120
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1090393f, 39.6715813f, 1483287958, 13.785677),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1090393f,
                                                39.6715813f, leg_index, 25.3519573f, 5958,
                                                maneuver_index, 1.89221954f, 475));
  //----------------------------------------------------------------
  // trace point = 121
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1088791f, 39.6715965f, 1483287959, 14.2118006),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1088791f,
                                                39.6715927f, leg_index, 25.3433914f, 5956,
                                                maneuver_index, 1.88365364f, 473));
  //----------------------------------------------------------------
  // trace point = 122
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1087036f, 39.6716118f, 1483287960, 14.5766191),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1087036f,
                                                39.671608f, leg_index, 25.3339901f, 5954,
                                                maneuver_index, 1.87425232f, 471));
  //----------------------------------------------------------------
  // trace point = 123
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1085358f, 39.6716232f, 1483287961, 14.6494789),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1085358f,
                                                39.6716232f, leg_index, 25.3249912f, 5951,
                                                maneuver_index, 1.86525345f, 468));
  //----------------------------------------------------------------
  // trace point = 124
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1083603f, 39.6716385f, 1483287962, 14.9171457),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1083603f,
                                                39.6716347f, leg_index, 25.3156185f, 5949,
                                                maneuver_index, 1.85588074f, 466));
  //----------------------------------------------------------------
  // trace point = 125
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1081924f, 39.6716537f, 1483287963, 15.0022917),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1081924f,
                                                39.6716499f, leg_index, 25.3066196f, 5947,
                                                maneuver_index, 1.84688187f, 464));
  //----------------------------------------------------------------
  // trace point = 126
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.108017f, 39.6716728f, 1483287964, 15.0150862),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.108017f,
                                                39.6716652f, leg_index, 25.2972183f, 5944,
                                                maneuver_index, 1.83748055f, 461));
  //----------------------------------------------------------------
  // trace point = 127
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.107666f, 39.6716995f, 1483287966, 15.2033672),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.107666f,
                                                39.6716957f, leg_index, 25.2784138f, 5940,
                                                maneuver_index, 1.81867599f, 457));
  //----------------------------------------------------------------
  // trace point = 128
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1074829f, 39.6717148f, 1483287967, 15.3429337),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1074829f,
                                                39.671711f, leg_index, 25.2686081f, 5937,
                                                maneuver_index, 1.80887032f, 454));
  //----------------------------------------------------------------
  // trace point = 129
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1073074f, 39.67173f, 1483287968, 15.5984812),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1073074f,
                                                39.6717224f, leg_index, 25.2592354f, 5935,
                                                maneuver_index, 1.7994976f, 452));
  //----------------------------------------------------------------
  // trace point = 130
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1071243f, 39.6717415f, 1483287969, 15.7574739),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1071243f,
                                                39.6717377f, leg_index, 25.2494278f, 5932,
                                                maneuver_index, 1.78969002f, 449));
  //----------------------------------------------------------------
  // trace point = 131
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1069336f, 39.6717567f, 1483287970, 15.9392843),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1069336f,
                                                39.6717567f, leg_index, 25.2391911f, 5930,
                                                maneuver_index, 1.77945328f, 447));
  //----------------------------------------------------------------
  // trace point = 132
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1067505f, 39.6717758f, 1483287971, 15.9513273),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1067505f,
                                                39.671772f, leg_index, 25.2285767f, 5927,
                                                maneuver_index, 1.76883888f, 444));
  //----------------------------------------------------------------
  // trace point = 133
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1065674f, 39.6717873f, 1483287972, 16.1097126),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1065674f,
                                                39.6717873f, leg_index, 25.2195778f, 5924,
                                                maneuver_index, 1.75984001f, 441));
  //----------------------------------------------------------------
  // trace point = 134
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1061935f, 39.6718178f, 1483287974, 16.1097069),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1061935f,
                                                39.6718178f, leg_index, 25.1995621f, 5919,
                                                maneuver_index, 1.7398243f, 436));
  //----------------------------------------------------------------
  // trace point = 135
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1058197f, 39.6718483f, 1483287976, 15.9392653),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1058197f,
                                                39.6718483f, leg_index, 25.1795464f, 5914,
                                                maneuver_index, 1.71980858f, 431));
  //----------------------------------------------------------------
  // trace point = 136
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1056366f, 39.6718636f, 1483287977, 15.7810211),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1056366f,
                                                39.6718636f, leg_index, 25.1697426f, 5912,
                                                maneuver_index, 1.71000481f, 429));
  //----------------------------------------------------------------
  // trace point = 137
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1054535f, 39.6718826f, 1483287978, 15.5387993),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1054535f,
                                                39.6718788f, leg_index, 25.1599369f, 5910,
                                                maneuver_index, 1.70019913f, 427));
  //----------------------------------------------------------------
  // trace point = 138
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1051025f, 39.6719131f, 1483287980, 15.3428917),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1051025f,
                                                39.6719131f, leg_index, 25.1411018f, 5905,
                                                maneuver_index, 1.68136406f, 422));
  //----------------------------------------------------------------
  // trace point = 139
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1049271f, 39.6719284f, 1483287981, 15.087388),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1049271f,
                                                39.6719284f, leg_index, 25.1317005f, 5903,
                                                maneuver_index, 1.67196274f, 420));
  //----------------------------------------------------------------
  // trace point = 140
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1047516f, 39.6719398f, 1483287982, 14.8938026),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1047516f,
                                                39.6719437f, leg_index, 25.1222973f, 5900,
                                                maneuver_index, 1.66255951f, 417));
  //----------------------------------------------------------------
  // trace point = 141
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1045837f, 39.6719513f, 1483287983, 14.8937998),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1045837f,
                                                39.6719589f, leg_index, 25.1132984f, 5898,
                                                maneuver_index, 1.65356064f, 415));
  //----------------------------------------------------------------
  // trace point = 142
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.104248f, 39.6719742f, 1483287985, 14.1530256),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.104248f,
                                                39.6719894f, leg_index, 25.0953026f, 5894,
                                                maneuver_index, 1.6355648f, 411));
  //----------------------------------------------------------------
  // trace point = 143
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1035233f, 39.6720352f, 1483287990, 12.5592728),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1035233f,
                                                39.6720543f, leg_index, 25.0564537f, 5884,
                                                maneuver_index, 1.59671593f, 401));
  //----------------------------------------------------------------
  // trace point = 144
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1027298f, 39.6720963f, 1483287996, 11.4093771),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1027298f,
                                                39.6721153f, leg_index, 25.013998f, 5873,
                                                maneuver_index, 1.55426025f, 390));
  //----------------------------------------------------------------
  // trace point = 145
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1019211f, 39.6721573f, 1483288002, 11.5267582),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1019211f,
                                                39.6721725f, leg_index, 24.9707603f, 5862,
                                                maneuver_index, 1.51102257f, 379));
  //----------------------------------------------------------------
  // trace point = 146
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1013794f, 39.6722069f, 1483288006, 11.7080154),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1013794f,
                                                39.6722145f, leg_index, 24.9417744f, 5855,
                                                maneuver_index, 1.48203659f, 372));
  //----------------------------------------------------------------
  // trace point = 147
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1011124f, 39.6722298f, 1483288008, 11.6804981),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1011124f,
                                                39.6722336f, leg_index, 24.927496f, 5851,
                                                maneuver_index, 1.46775818f, 368));
  //----------------------------------------------------------------
  // trace point = 148
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.100708f, 39.6722603f, 1483288011, 11.5428391),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.100708f,
                                                39.6722641f, leg_index, 24.9058628f, 5846,
                                                maneuver_index, 1.44612503f, 363));
  //----------------------------------------------------------------
  // trace point = 149
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1001892f, 39.6723022f, 1483288015, 11.114809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1001892f,
                                                39.6723061f, leg_index, 24.878088f, 5839,
                                                maneuver_index, 1.41835022f, 356));
  //----------------------------------------------------------------
  // trace point = 150
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0994797f, 39.6723595f, 1483288021, 10.2529621),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0994797f,
                                                39.6723595f, leg_index, 24.8401299f, 5829,
                                                maneuver_index, 1.38039207f, 346));
  //----------------------------------------------------------------
  // trace point = 151
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0988388f, 39.6723785f, 1483288028, 7.8391428),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0988388f,
                                                39.67239f, leg_index, 24.8051033f, 5820,
                                                maneuver_index, 1.34536552f, 337));
  //----------------------------------------------------------------
  // trace point = 152
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0987244f, 39.6723595f, 1483288030, 5.14695978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0987244f,
                                                39.6723633f, leg_index, 24.799509f, 5819,
                                                maneuver_index, 1.33977127f, 336));
  //----------------------------------------------------------------
  // trace point = 153
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0986252f, 39.6723366f, 1483288032, 4.3074379),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0986252f,
                                                39.6723366f, leg_index, 24.7939148f, 5818,
                                                maneuver_index, 1.33417702f, 335));
  //----------------------------------------------------------------
  // trace point = 154
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0985489f, 39.6723137f, 1483288034, 3.59920311),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0985489f,
                                                39.6723175f, leg_index, 24.7896461f, 5817,
                                                maneuver_index, 1.32990837f, 334));
  //----------------------------------------------------------------
  // trace point = 155
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.098465f, 39.6722908f, 1483288036, 3.59920406),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.098465f,
                                                39.6722946f, leg_index, 24.7849064f, 5816,
                                                maneuver_index, 1.32516861f, 333));
  //----------------------------------------------------------------
  // trace point = 156
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0983887f, 39.6722679f, 1483288038, 3.438555),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0983887f,
                                                39.6722755f, leg_index, 24.7806358f, 5815,
                                                maneuver_index, 1.32089806f, 332));
  //----------------------------------------------------------------
  // trace point = 157
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0983276f, 39.6722488f, 1483288040, 3.02007699),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.09832f,
                                                39.6722603f, leg_index, 24.7768383f, 5814,
                                                maneuver_index, 1.31710052f, 331));
  //----------------------------------------------------------------
  // trace point = 158
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0982437f, 39.6722221f, 1483288043, 2.5065639),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0982361f,
                                                39.6722412f, leg_index, 24.7721672f, 5813,
                                                maneuver_index, 1.31242943f, 330));
  //----------------------------------------------------------------
  // trace point = 159
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0981598f, 39.6722031f, 1483288046, 2.51130199),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0981522f,
                                                39.6722298f, leg_index, 24.7676296f, 5811,
                                                maneuver_index, 1.30789185f, 328));
  //----------------------------------------------------------------
  // trace point = 160
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0980606f, 39.6721878f, 1483288049, 2.89657211),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.098053f,
                                                39.6722183f, leg_index, 24.7622948f, 5810,
                                                maneuver_index, 1.30255699f, 327));
  //----------------------------------------------------------------
  // trace point = 161
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0979462f, 39.6721916f, 1483288052, 3.22763205),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0979462f,
                                                39.6722183f, leg_index, 24.7565823f, 5808,
                                                maneuver_index, 1.29684448f, 325));
  //----------------------------------------------------------------
  // trace point = 162
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0978394f, 39.6721992f, 1483288055, 3.1310339),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0978394f,
                                                39.672226f, leg_index, 24.7508717f, 5807,
                                                maneuver_index, 1.29113388f, 324));
  //----------------------------------------------------------------
  // trace point = 163
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0977402f, 39.6722107f, 1483288058, 2.86240602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0977478f,
                                                39.6722336f, leg_index, 24.7459583f, 5805,
                                                maneuver_index, 1.28622055f, 322));
  //----------------------------------------------------------------
  // trace point = 164
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0976334f, 39.672226f, 1483288061, 3.21878409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.097641f,
                                                39.6722488f, leg_index, 24.7401752f, 5804,
                                                maneuver_index, 1.28043747f, 321));
  //----------------------------------------------------------------
  // trace point = 165
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0975037f, 39.6722412f, 1483288065, 2.80350494),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0975113f,
                                                39.6722679f, leg_index, 24.7331448f, 5802,
                                                maneuver_index, 1.27340698f, 319));
  //----------------------------------------------------------------
  // trace point = 166
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0973892f, 39.6722565f, 1483288069, 2.47740602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0973969f,
                                                39.6722832f, leg_index, 24.726963f, 5801,
                                                maneuver_index, 1.26722527f, 318));
  //----------------------------------------------------------------
  // trace point = 167
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.09729f, 39.6722717f, 1483288072, 2.74184108),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0972977f,
                                                39.6722984f, leg_index, 24.7215767f, 5799,
                                                maneuver_index, 1.26183891f, 316));
  //----------------------------------------------------------------
  // trace point = 168
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0971603f, 39.6722984f, 1483288076, 2.98882699),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.097168f,
                                                39.6723175f, leg_index, 24.7145462f, 5797,
                                                maneuver_index, 1.25480843f, 314));
  //----------------------------------------------------------------
  // trace point = 169
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0970459f, 39.6723137f, 1483288080, 2.44010997),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0970535f,
                                                39.6723366f, leg_index, 24.7083149f, 5796,
                                                maneuver_index, 1.24857712f, 313));
  //----------------------------------------------------------------
  // trace point = 170
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0969238f, 39.6723328f, 1483288084, 2.62432694),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0969315f,
                                                39.6723557f, leg_index, 24.701685f, 5794,
                                                maneuver_index, 1.24194717f, 311));
  //----------------------------------------------------------------
  // trace point = 171
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0968018f, 39.6723557f, 1483288088, 2.75305295),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0968018f,
                                                39.6723709f, leg_index, 24.6947021f, 5792,
                                                maneuver_index, 1.23496437f, 309));
  //----------------------------------------------------------------
  // trace point = 172
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0965652f, 39.6723938f, 1483288095, 2.90337992),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0965652f,
                                                39.6724052f, leg_index, 24.6818886f, 5789,
                                                maneuver_index, 1.2221508f, 306));
  //----------------------------------------------------------------
  // trace point = 173
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0963669f, 39.6724243f, 1483288100, 3.4961381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0963669f,
                                                39.6724319f, leg_index, 24.6711674f, 5786,
                                                maneuver_index, 1.2114296f, 303));
  //----------------------------------------------------------------
  // trace point = 174
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0960312f, 39.6724663f, 1483288107, 4.20810509),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0960312f,
                                                39.6724739f, leg_index, 24.6530609f, 5781,
                                                maneuver_index, 1.19332314f, 298));
  //----------------------------------------------------------------
  // trace point = 175
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0952759f, 39.672554f, 1483288119, 5.44392204),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0952759f,
                                                39.672554f, leg_index, 24.6124649f, 5771,
                                                maneuver_index, 1.15272713f, 288));
  //----------------------------------------------------------------
  // trace point = 176
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0945282f, 39.6726341f, 1483288129, 6.45066977),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0945282f,
                                                39.6726341f, leg_index, 24.5722733f, 5761,
                                                maneuver_index, 1.11253548f, 278));
  //----------------------------------------------------------------
  // trace point = 177
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0938339f, 39.6727066f, 1483288138, 6.66852903),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0938339f,
                                                39.6727066f, leg_index, 24.5349712f, 5752,
                                                maneuver_index, 1.07523346f, 269));
  //----------------------------------------------------------------
  // trace point = 178
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0932846f, 39.6727676f, 1483288145, 6.84799719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0932846f,
                                                39.6727715f, leg_index, 24.5053806f, 5744,
                                                maneuver_index, 1.04564285f, 261));
  //----------------------------------------------------------------
  // trace point = 179
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0927048f, 39.6728287f, 1483288152, 7.10696602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0927048f,
                                                39.6728363f, leg_index, 24.4741859f, 5736,
                                                maneuver_index, 1.01444817f, 253));
  //----------------------------------------------------------------
  // trace point = 180
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0919571f, 39.6729088f, 1483288161, 7.230762),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0919571f,
                                                39.6729202f, leg_index, 24.4339581f, 5726,
                                                maneuver_index, 0.974220276f, 243));
  //----------------------------------------------------------------
  // trace point = 181
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0911713f, 39.6730003f, 1483288170, 7.52582312),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0911713f,
                                                39.6730118f, leg_index, 24.3916416f, 5715,
                                                maneuver_index, 0.931903839f, 232));
  //----------------------------------------------------------------
  // trace point = 182
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0904083f, 39.6730804f, 1483288178, 8.28307724),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0904083f,
                                                39.6730881f, leg_index, 24.3506813f, 5705,
                                                maneuver_index, 0.890943527f, 222));
  //----------------------------------------------------------------
  // trace point = 183
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0896988f, 39.6731606f, 1483288185, 8.77873135),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0896988f,
                                                39.6731644f, leg_index, 24.3125381f, 5695,
                                                maneuver_index, 0.852800369f, 212));
  //----------------------------------------------------------------
  // trace point = 184
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0892868f, 39.6732025f, 1483288189, 8.82115746),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0892868f,
                                                39.6732063f, leg_index, 24.2904129f, 5690,
                                                maneuver_index, 0.830675125f, 207));
  //----------------------------------------------------------------
  // trace point = 185
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0885162f, 39.6732712f, 1483288196, 9.51488495),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0885162f,
                                                39.6732826f, leg_index, 24.2490387f, 5680,
                                                maneuver_index, 0.789300919f, 197));
  //----------------------------------------------------------------
  // trace point = 186
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0882721f, 39.6732903f, 1483288198, 10.3787384),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0882721f,
                                                39.6732941f, leg_index, 24.2360172f, 5677,
                                                maneuver_index, 0.776279449f, 194));
  //----------------------------------------------------------------
  // trace point = 187
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0878983f, 39.6733131f, 1483288201, 10.6816263),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0878983f,
                                                39.6733093f, leg_index, 24.2160854f, 5672,
                                                maneuver_index, 0.756347656f, 189));
  //----------------------------------------------------------------
  // trace point = 188
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0875244f, 39.6733284f, 1483288204, 10.8143797),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0875244f,
                                                39.6733284f, leg_index, 24.1961384f, 5667,
                                                maneuver_index, 0.736400604f, 184));
  //----------------------------------------------------------------
  // trace point = 189
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0871506f, 39.6733551f, 1483288207, 10.665019),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0871506f,
                                                39.6733589f, leg_index, 24.1761169f, 5662,
                                                maneuver_index, 0.716379166f, 179));
  //----------------------------------------------------------------
  // trace point = 190
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.086792f, 39.6733932f, 1483288210, 10.303648),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.086792f,
                                                39.6734047f, leg_index, 24.156765f, 5657,
                                                maneuver_index, 0.697027206f, 174));
  //----------------------------------------------------------------
  // trace point = 191
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0860367f, 39.6734734f, 1483288218, 8.23492432),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0860367f,
                                                39.673481f, leg_index, 24.1162033f, 5647,
                                                maneuver_index, 0.65646553f, 164));
  //----------------------------------------------------------------
  // trace point = 192
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0858841f, 39.6734848f, 1483288220, 6.33906412),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0858841f,
                                                39.6734962f, leg_index, 24.1080112f, 5645,
                                                maneuver_index, 0.648273468f, 162));
  //----------------------------------------------------------------
  // trace point = 193
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0856857f, 39.6735039f, 1483288223, 5.77525091),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0856857f,
                                                39.6735153f, leg_index, 24.0973682f, 5642,
                                                maneuver_index, 0.637630463f, 159));
  //----------------------------------------------------------------
  // trace point = 194
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.085556f, 39.6735115f, 1483288225, 5.55444908),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.085556f,
                                                39.6735268f, leg_index, 24.0904198f, 5640,
                                                maneuver_index, 0.630681992f, 157));
  //----------------------------------------------------------------
  // trace point = 195
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0854263f, 39.6735229f, 1483288227, 5.6454711),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0854263f,
                                                39.6735382f, leg_index, 24.0834694f, 5639,
                                                maneuver_index, 0.623731613f, 156));
  //----------------------------------------------------------------
  // trace point = 196
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0852203f, 39.673542f, 1483288230, 5.88882113),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0852203f,
                                                39.6735611f, leg_index, 24.0723915f, 5636,
                                                maneuver_index, 0.612653732f, 153));
  //----------------------------------------------------------------
  // trace point = 197
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0846252f, 39.6735954f, 1483288238, 6.38321209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0846252f,
                                                39.6736145f, leg_index, 24.040493f, 5628,
                                                maneuver_index, 0.580755234f, 145));
  //----------------------------------------------------------------
  // trace point = 198
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0839233f, 39.6736488f, 1483288247, 6.7354269),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0839233f,
                                                39.6736717f, leg_index, 24.0029125f, 5619,
                                                maneuver_index, 0.543174744f, 136));
  //----------------------------------------------------------------
  // trace point = 199
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.083313f, 39.6736984f, 1483288256, 5.83201885),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.083313f,
                                                39.6737213f, leg_index, 23.9702358f, 5611,
                                                maneuver_index, 0.510498047f, 128));
  //----------------------------------------------------------------
  // trace point = 200
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0830688f, 39.6737175f, 1483288260, 5.33852196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0830765f,
                                                39.673748f, leg_index, 23.9575081f, 5607,
                                                maneuver_index, 0.497770309f, 124));
  //----------------------------------------------------------------
  // trace point = 201
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.082634f, 39.673748f, 1483288267, 5.35663891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0826416f,
                                                39.6737823f, leg_index, 23.9342232f, 5602,
                                                maneuver_index, 0.474485397f, 119));
  //----------------------------------------------------------------
  // trace point = 202
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0819092f, 39.6738014f, 1483288277, 6.22281885),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0819092f,
                                                39.6738319f, leg_index, 23.8950729f, 5593,
                                                maneuver_index, 0.435335159f, 110));
  //----------------------------------------------------------------
  // trace point = 203
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0812073f, 39.6738434f, 1483288285, 7.51257515),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0812073f,
                                                39.6738701f, leg_index, 23.8576107f, 5583,
                                                maneuver_index, 0.397872925f, 100));
  //----------------------------------------------------------------
  // trace point = 204
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0806046f, 39.6738853f, 1483288291, 8.71406078),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0806046f,
                                                39.6739082f, leg_index, 23.8254128f, 5574,
                                                maneuver_index, 0.365674973f, 91));
  //----------------------------------------------------------------
  // trace point = 205
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0801926f, 39.6739159f, 1483288295, 8.73404598),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0801926f,
                                                39.6739388f, leg_index, 23.8033772f, 5569,
                                                maneuver_index, 0.343639374f, 86));
  //----------------------------------------------------------------
  // trace point = 206
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0794678f, 39.6739655f, 1483288303, 7.79982996),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0794678f,
                                                39.6739845f, leg_index, 23.7646618f, 5559,
                                                maneuver_index, 0.304924011f, 76));
  //----------------------------------------------------------------
  // trace point = 207
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0787582f, 39.6740189f, 1483288312, 6.76858521),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0787582f,
                                                39.6740341f, leg_index, 23.7267303f, 5550,
                                                maneuver_index, 0.266992569f, 67));
  //----------------------------------------------------------------
  // trace point = 208
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0786896f, 39.6740227f, 1483288313, 6.37550688),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0786896f,
                                                39.6740379f, leg_index, 23.7230663f, 5549,
                                                maneuver_index, 0.263328552f, 66));
  //----------------------------------------------------------------
  // trace point = 209
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0781937f, 39.674057f, 1483288320, 6.07221985),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0781937f,
                                                39.6740723f, leg_index, 23.6965561f, 5542,
                                                maneuver_index, 0.236818314f, 59));
  //----------------------------------------------------------------
  // trace point = 210
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0774994f, 39.6741142f, 1483288330, 6.00329113),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0774994f,
                                                39.6741257f, leg_index, 23.6594105f, 5533,
                                                maneuver_index, 0.199672699f, 50));
  //----------------------------------------------------------------
  // trace point = 211
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0770569f, 39.6741524f, 1483288336, 6.34114981),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0770569f,
                                                39.6741638f, leg_index, 23.635704f, 5527,
                                                maneuver_index, 0.175966263f, 44));
  //----------------------------------------------------------------
  // trace point = 212
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0763016f, 39.674221f, 1483288346, 6.50327301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0763016f,
                                                39.6742287f, leg_index, 23.5952415f, 5517,
                                                maneuver_index, 0.135503769f, 34));
  //----------------------------------------------------------------
  // trace point = 213 | Alert | In 500 feet Continue on Barnes Corner Road.
  maneuver_index = 0;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.075592f, 39.6742821f, 1483288356, 6.12365103),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.075592f, 39.6742897f, leg_index, 23.55723f, 5507,
                                                maneuver_index, 0.097492218f, 24, instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 214
  maneuver_index = 0;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0748825f, 39.6743431f, 1483288367, 5.57241011),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0748825f,
                                                39.6743546f, leg_index, 23.5191727f, 5498,
                                                maneuver_index, 0.0594348907f, 15));
  //----------------------------------------------------------------
  // trace point = 215 | Pre | Continue on Barnes Corner Road for 1.4 miles.
  maneuver_index = 0;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0741501f, 39.6743774f, 1483288378, 5.6887269),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0741501f, 39.6743927f, leg_index, 23.4800892f,
                                                5488, maneuver_index, 0.0203514099f, 5,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 216
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0733948f, 39.6744308f, 1483288385, 9.28901482),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0733948f,
                                                39.6744385f, leg_index, 23.4397488f, 5478,
                                                maneuver_index, 1.37852859f, 334));
  //----------------------------------------------------------------
  // trace point = 217
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0732956f, 39.6744385f, 1483288386, 8.81006241),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0732956f,
                                                39.6744461f, leg_index, 23.4344425f, 5477,
                                                maneuver_index, 1.37322235f, 333));
  //----------------------------------------------------------------
  // trace point = 218
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0731888f, 39.6744461f, 1483288387, 8.86995029),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0731888f,
                                                39.6744576f, leg_index, 23.4287052f, 5476,
                                                maneuver_index, 1.36748505f, 332));
  //----------------------------------------------------------------
  // trace point = 219
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0729828f, 39.6744614f, 1483288389, 8.94013977),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0729828f,
                                                39.6744728f, leg_index, 23.4176884f, 5473,
                                                maneuver_index, 1.3564682f, 329));
  //----------------------------------------------------------------
  // trace point = 220
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0725555f, 39.6744881f, 1483288393, 9.11795712),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0725555f,
                                                39.6745033f, leg_index, 23.3948441f, 5467,
                                                maneuver_index, 1.33362389f, 323));
  //----------------------------------------------------------------
  // trace point = 221
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0718155f, 39.6745529f, 1483288401, 7.97380781),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0718155f,
                                                39.674572f, leg_index, 23.355154f, 5458,
                                                maneuver_index, 1.29393387f, 314));
  //----------------------------------------------------------------
  // trace point = 222
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0713577f, 39.6745987f, 1483288407, 6.63822603),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0713654f,
                                                39.6746178f, leg_index, 23.3309784f, 5452,
                                                maneuver_index, 1.26975822f, 308));
  //----------------------------------------------------------------
  // trace point = 223
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0706024f, 39.6746635f, 1483288416, 7.22581196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0706024f,
                                                39.6746864f, leg_index, 23.2900715f, 5442,
                                                maneuver_index, 1.22885132f, 298));
  //----------------------------------------------------------------
  // trace point = 224
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0698318f, 39.6747398f, 1483288424, 8.36565495),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0698318f,
                                                39.6747513f, leg_index, 23.2488022f, 5433,
                                                maneuver_index, 1.18758202f, 289));
  //----------------------------------------------------------------
  // trace point = 225
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.069725f, 39.6747513f, 1483288425, 8.73993683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.069725f,
                                                39.6747589f, leg_index, 23.2430992f, 5431,
                                                maneuver_index, 1.18187904f, 287));
  //----------------------------------------------------------------
  // trace point = 226
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0690079f, 39.6748238f, 1483288433, 7.77968597),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0690079f,
                                                39.6748352f, leg_index, 23.2045517f, 5422,
                                                maneuver_index, 1.14333153f, 278));
  //----------------------------------------------------------------
  // trace point = 227
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0685043f, 39.674881f, 1483288441, 5.43064117),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0685043f,
                                                39.6748886f, leg_index, 23.1774902f, 5416,
                                                maneuver_index, 1.11627007f, 272));
  //----------------------------------------------------------------
  // trace point = 228
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0683441f, 39.6748962f, 1483288444, 4.57504082),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0683441f,
                                                39.6749039f, leg_index, 23.1688957f, 5413,
                                                maneuver_index, 1.10767555f, 269));
  //----------------------------------------------------------------
  // trace point = 229
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0681992f, 39.6749115f, 1483288447, 4.3483448),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0681992f,
                                                39.6749191f, leg_index, 23.1611061f, 5412,
                                                maneuver_index, 1.09988594f, 268));
  //----------------------------------------------------------------
  // trace point = 230
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0680466f, 39.6749229f, 1483288450, 4.278687),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0680466f,
                                                39.6749344f, leg_index, 23.152914f, 5410,
                                                maneuver_index, 1.09169388f, 266));
  //----------------------------------------------------------------
  // trace point = 231
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0678558f, 39.674942f, 1483288454, 4.16775179),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0678558f,
                                                39.6749535f, leg_index, 23.1426754f, 5407,
                                                maneuver_index, 1.08145523f, 263));
  //----------------------------------------------------------------
  // trace point = 232
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0675735f, 39.6749725f, 1483288460, 4.01129913),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0675735f,
                                                39.6749802f, leg_index, 23.1275272f, 5404,
                                                maneuver_index, 1.06630707f, 260));
  //----------------------------------------------------------------
  // trace point = 233
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0673523f, 39.6749954f, 1483288465, 3.90645194),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0673523f,
                                                39.6749954f, leg_index, 23.1157017f, 5401,
                                                maneuver_index, 1.05448151f, 257));
  //----------------------------------------------------------------
  // trace point = 234
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0670929f, 39.6750183f, 1483288470, 4.38002586),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0670929f,
                                                39.6750183f, leg_index, 23.1018028f, 5397,
                                                maneuver_index, 1.04058266f, 253));
  //----------------------------------------------------------------
  // trace point = 235
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0663834f, 39.6750832f, 1483288483, 4.70900202),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0663834f,
                                                39.6750717f, leg_index, 23.0638485f, 5388,
                                                maneuver_index, 1.00262833f, 244));
  //----------------------------------------------------------------
  // trace point = 236
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0656281f, 39.6751671f, 1483288496, 5.04372501),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0656204f,
                                                39.6751442f, leg_index, 23.0228825f, 5378,
                                                maneuver_index, 0.961662292f, 234));
  //----------------------------------------------------------------
  // trace point = 237
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0651016f, 39.6752205f, 1483288502, 7.56864691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0651016f,
                                                39.6752129f, leg_index, 22.994854f, 5372,
                                                maneuver_index, 0.933633804f, 228));
  //----------------------------------------------------------------
  // trace point = 238
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0647049f, 39.6752548f, 1483288506, 8.59577084),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0647049f,
                                                39.6752548f, leg_index, 22.9735355f, 5367,
                                                maneuver_index, 0.912315369f, 223));
  //----------------------------------------------------------------
  // trace point = 239
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0642776f, 39.6752968f, 1483288510, 9.28445244),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0642776f,
                                                39.6752968f, leg_index, 22.9506016f, 5361,
                                                maneuver_index, 0.889381409f, 217));
  //----------------------------------------------------------------
  // trace point = 240
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.063591f, 39.6753654f, 1483288516, 9.83687115),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.063591f,
                                                39.6753616f, leg_index, 22.9137516f, 5353,
                                                maneuver_index, 0.852531433f, 209));
  //----------------------------------------------------------------
  // trace point = 241
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0632401f, 39.6754074f, 1483288519, 10.1675501),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0632401f,
                                                39.6753998f, leg_index, 22.8948822f, 5348,
                                                maneuver_index, 0.833662033f, 204));
  //----------------------------------------------------------------
  // trace point = 242
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0628891f, 39.6754494f, 1483288522, 10.0828381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0628891f,
                                                39.6754379f, leg_index, 22.8760128f, 5343,
                                                maneuver_index, 0.814792633f, 199));
  //----------------------------------------------------------------
  // trace point = 243
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0626602f, 39.6754761f, 1483288524, 9.89102745),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0626602f,
                                                39.6754608f, leg_index, 22.8637238f, 5341,
                                                maneuver_index, 0.802503586f, 197));
  //----------------------------------------------------------------
  // trace point = 244
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0623474f, 39.6754951f, 1483288527, 9.11230659),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0623474f,
                                                39.6754837f, leg_index, 22.8469963f, 5336,
                                                maneuver_index, 0.785776138f, 192));
  //----------------------------------------------------------------
  // trace point = 245
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0617447f, 39.6755371f, 1483288533, 8.55898476),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0617447f,
                                                39.6755486f, leg_index, 22.8145866f, 5328,
                                                maneuver_index, 0.75336647f, 184));
  //----------------------------------------------------------------
  // trace point = 246
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0609665f, 39.6756096f, 1483288541, 8.44387531),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0609741f,
                                                39.6756363f, leg_index, 22.7730999f, 5318,
                                                maneuver_index, 0.71187973f, 174));
  //----------------------------------------------------------------
  // trace point = 247
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0602722f, 39.6757164f, 1483288548, 8.66638947),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0602798f,
                                                39.6757469f, leg_index, 22.735281f, 5308,
                                                maneuver_index, 0.674060822f, 164));
  //----------------------------------------------------------------
  // trace point = 248
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0600891f, 39.6757698f, 1483288550, 8.43608189),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0601044f,
                                                39.6757965f, leg_index, 22.7252827f, 5306,
                                                maneuver_index, 0.6640625f, 162));
  //----------------------------------------------------------------
  // trace point = 249
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0594864f, 39.6760941f, 1483288558, 7.82422304),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0595093f,
                                                39.6761208f, leg_index, 22.6862659f, 5296,
                                                maneuver_index, 0.625045776f, 152));
  //----------------------------------------------------------------
  // trace point = 250
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0593567f, 39.6761818f, 1483288560, 7.45348406),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0593796f,
                                                39.6762009f, leg_index, 22.6774178f, 5294,
                                                maneuver_index, 0.616197586f, 150));
  //----------------------------------------------------------------
  // trace point = 251
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0587006f, 39.676487f, 1483288569, 7.28787184),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0587082f,
                                                39.676506f, leg_index, 22.6354084f, 5284,
                                                maneuver_index, 0.574188232f, 140));
  //----------------------------------------------------------------
  // trace point = 252
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0579529f, 39.6765976f, 1483288577, 8.15583706),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0579529f,
                                                39.6766129f, leg_index, 22.594471f, 5274,
                                                maneuver_index, 0.533250809f, 130));
  //----------------------------------------------------------------
  // trace point = 253
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0572586f, 39.6766701f, 1483288584, 8.63185406),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0572586f,
                                                39.6766853f, leg_index, 22.5571671f, 5265,
                                                maneuver_index, 0.495946884f, 121));
  //----------------------------------------------------------------
  // trace point = 254
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0567245f, 39.6767159f, 1483288589, 9.1419611),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0567245f,
                                                39.6767311f, leg_index, 22.5285473f, 5258,
                                                maneuver_index, 0.467327118f, 114));
  //----------------------------------------------------------------
  // trace point = 255
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0559235f, 39.6767807f, 1483288596, 9.88437557),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0559235f,
                                                39.6767845f, leg_index, 22.4857368f, 5248,
                                                maneuver_index, 0.424516678f, 104));
  //----------------------------------------------------------------
  // trace point = 256
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0555496f, 39.676815f, 1483288599, 10.7270737),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0555496f,
                                                39.6768227f, leg_index, 22.4656582f, 5243,
                                                maneuver_index, 0.404438019f, 99));
  //----------------------------------------------------------------
  // trace point = 257
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0549164f, 39.6768837f, 1483288604, 10.9489107),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0549164f,
                                                39.6768913f, leg_index, 22.4316139f, 5234,
                                                maneuver_index, 0.370393753f, 90));
  //----------------------------------------------------------------
  // trace point = 258
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0546722f, 39.6769104f, 1483288606, 10.5601749),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0546722f,
                                                39.6769142f, leg_index, 22.41852f, 5231,
                                                maneuver_index, 0.357299805f, 87));
  //----------------------------------------------------------------
  // trace point = 259
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.053978f, 39.6769753f, 1483288612, 10.013773),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.053978f,
                                                39.6769829f, leg_index, 22.3812542f, 5222,
                                                maneuver_index, 0.320034027f, 78));
  //----------------------------------------------------------------
  // trace point = 260
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0532455f, 39.6770477f, 1483288619, 9.06476974),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0532455f,
                                                39.6770554f, leg_index, 22.3419361f, 5212,
                                                maneuver_index, 0.280715942f, 68));
  //----------------------------------------------------------------
  // trace point = 261
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0524673f, 39.6771278f, 1483288626, 9.62131119),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0524673f,
                                                39.677124f, leg_index, 22.3002357f, 5202,
                                                maneuver_index, 0.239015579f, 58));
  //----------------------------------------------------------------
  // trace point = 262
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0517273f, 39.6771851f, 1483288632, 10.5344324),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0517273f,
                                                39.6771851f, leg_index, 22.2606106f, 5192,
                                                maneuver_index, 0.199390411f, 48));
  //----------------------------------------------------------------
  // trace point = 263
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0509262f, 39.6772537f, 1483288639, 9.86874962),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0509262f,
                                                39.6772614f, leg_index, 22.2176304f, 5182,
                                                maneuver_index, 0.156410217f, 38));
  //----------------------------------------------------------------
  // trace point = 264
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.050209f, 39.6773338f, 1483288646, 8.8751564),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.050209f,
                                                39.6773376f, leg_index, 22.179081f, 5173,
                                                maneuver_index, 0.117860794f, 29));
  //----------------------------------------------------------------
  // trace point = 265 | Alert | In 500 feet Turn left onto Montgomery Road.
  maneuver_index = 1;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.049942f, 39.6773682f, 1483288649, 7.71978521),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.049942f, 39.6773758f, leg_index, 22.1646252f,
                                                5169, maneuver_index, 0.103404999f, 25,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 266
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0494614f, 39.6774445f, 1483288655, 7.02568722),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0494614f,
                                                39.6774483f, leg_index, 22.1385479f, 5163,
                                                maneuver_index, 0.0773277283f, 19));
  //----------------------------------------------------------------
  // trace point = 267
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0489578f, 39.6775284f, 1483288662, 6.2844491),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0489578f,
                                                39.677536f, leg_index, 22.1110611f, 5156,
                                                maneuver_index, 0.0498409271f, 12));
  //----------------------------------------------------------------
  // trace point = 268
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0485764f, 39.6776047f, 1483288668, 5.66172409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0485764f,
                                                39.6776085f, leg_index, 22.0901432f, 5151,
                                                maneuver_index, 0.0289230347f, 7));
  //----------------------------------------------------------------
  // trace point = 269 | Pre | Turn left onto Montgomery Road.
  maneuver_index = 1;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0481567f, 39.6776733f, 1483288675, 5.22388315),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0481567f, 39.6776695f, leg_index, 22.0665588f,
                                                5145, maneuver_index, 0.00533866882f, 1,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 270
  maneuver_index = 1;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0481033f, 39.6776962f, 1483288676, 5.10728979),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0481033f,
                                                39.6776772f, leg_index, 22.0645123f, 5145,
                                                maneuver_index, 0.00329208374f, 1));
  //----------------------------------------------------------------
  // trace point = 271 | Post | Continue for 9 tenths of a mile.
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0479126f, 39.6778755f, 1483288681, 5.17872286),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.0479965f, 39.6778831f, leg_index, 22.0472832f,
                                                5140, maneuver_index, 0.926801682f, 220,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 272
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0478821f, 39.6779175f, 1483288682, 5.28064394),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0479584f,
                                                39.6779442f, leg_index, 22.0425491f, 5139,
                                                maneuver_index, 0.922067642f, 219));
  //----------------------------------------------------------------
  // trace point = 273
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0475311f, 39.6783905f, 1483288692, 6.08300781),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0475769f,
                                                39.6784096f, leg_index, 22.0044632f, 5130,
                                                maneuver_index, 0.883981705f, 210));
  //----------------------------------------------------------------
  // trace point = 274
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0471649f, 39.678875f, 1483288701, 6.94686317),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0472031f,
                                                39.678894f, leg_index, 21.9654388f, 5121,
                                                maneuver_index, 0.844957352f, 201));
  //----------------------------------------------------------------
  // trace point = 275
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0468597f, 39.6794052f, 1483288710, 7.16582584),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0468979f,
                                                39.6794167f, leg_index, 21.9257679f, 5112,
                                                maneuver_index, 0.805286407f, 192));
  //----------------------------------------------------------------
  // trace point = 276
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0465927f, 39.6799393f, 1483288719, 7.06690884),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0466309f,
                                                39.6799507f, leg_index, 21.886179f, 5102,
                                                maneuver_index, 0.765697479f, 182));
  //----------------------------------------------------------------
  // trace point = 277
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0463333f, 39.6804733f, 1483288728, 7.05011606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0463715f,
                                                39.6804848f, leg_index, 21.8467388f, 5093,
                                                maneuver_index, 0.726257324f, 173));
  //----------------------------------------------------------------
  // trace point = 278
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0460587f, 39.6810226f, 1483288738, 6.57573891),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0460968f,
                                                39.6810341f, leg_index, 21.8060265f, 5083,
                                                maneuver_index, 0.685544968f, 163));
  //----------------------------------------------------------------
  // trace point = 279
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0459442f, 39.6812363f, 1483288742, 6.35131502),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0459976f,
                                                39.6812477f, leg_index, 21.7903233f, 5080,
                                                maneuver_index, 0.669841766f, 160));
  //----------------------------------------------------------------
  // trace point = 280
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0456772f, 39.6817665f, 1483288752, 6.36517715),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0457153f,
                                                39.681778f, leg_index, 21.7506618f, 5070,
                                                maneuver_index, 0.630180359f, 150));
  //----------------------------------------------------------------
  // trace point = 281
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0454865f, 39.6822586f, 1483288760, 7.11543798),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0455399f,
                                                39.6822624f, leg_index, 21.7157288f, 5062,
                                                maneuver_index, 0.595247269f, 142));
  //----------------------------------------------------------------
  // trace point = 282
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0455322f, 39.6825256f, 1483288764, 7.49009609),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0455933f,
                                                39.6825104f, leg_index, 21.698164f, 5058,
                                                maneuver_index, 0.577682495f, 138));
  //----------------------------------------------------------------
  // trace point = 283
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0455551f, 39.6825905f, 1483288765, 7.78391409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0456161f,
                                                39.6825714f, leg_index, 21.6937695f, 5057,
                                                maneuver_index, 0.573287964f, 137));
  //----------------------------------------------------------------
  // trace point = 284
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0458069f, 39.6831169f, 1483288773, 7.78225613),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0458679f,
                                                39.6830978f, leg_index, 21.6549625f, 5047,
                                                maneuver_index, 0.534481049f, 127));
  //----------------------------------------------------------------
  // trace point = 285
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0460739f, 39.6836472f, 1483288784, 5.76481819),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0461273f,
                                                39.6836319f, leg_index, 21.6155243f, 5038,
                                                maneuver_index, 0.495042801f, 118));
  //----------------------------------------------------------------
  // trace point = 286
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0461349f, 39.6837654f, 1483288788, 3.54842496),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0461807f,
                                                39.6837502f, leg_index, 21.6068668f, 5036,
                                                maneuver_index, 0.486385345f, 116));
  //----------------------------------------------------------------
  // trace point = 287
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0461884f, 39.6838875f, 1483288792, 3.58187103),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0462418f,
                                                39.6838684f, leg_index, 21.5980663f, 5034,
                                                maneuver_index, 0.477584839f, 114));
  //----------------------------------------------------------------
  // trace point = 288
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0462189f, 39.6839485f, 1483288794, 3.52639794),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0462799f,
                                                39.6839294f, leg_index, 21.5933933f, 5033,
                                                maneuver_index, 0.472911835f, 113));
  //----------------------------------------------------------------
  // trace point = 289
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0462799f, 39.6840706f, 1483288798, 3.63028908),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0463409f,
                                                39.6840515f, leg_index, 21.5843449f, 5031,
                                                maneuver_index, 0.463863373f, 111));
  //----------------------------------------------------------------
  // trace point = 290
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0463562f, 39.6842308f, 1483288803, 3.82311702),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0464172f,
                                                39.6842003f, leg_index, 21.5732822f, 5028,
                                                maneuver_index, 0.452800751f, 108));
  //----------------------------------------------------------------
  // trace point = 291
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0464172f, 39.6843643f, 1483288807, 3.98811102),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0464859f,
                                                39.6843452f, leg_index, 21.5626106f, 5026,
                                                maneuver_index, 0.442129135f, 106));
  //----------------------------------------------------------------
  // trace point = 292
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0465317f, 39.6846199f, 1483288814, 4.3075242),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0466003f,
                                                39.6846008f, leg_index, 21.5439129f, 5021,
                                                maneuver_index, 0.423431396f, 101));
  //----------------------------------------------------------------
  // trace point = 293
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0467682f, 39.685154f, 1483288827, 4.81755495),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0468292f,
                                                39.6851387f, leg_index, 21.5047646f, 5012,
                                                maneuver_index, 0.384283066f, 92));
  //----------------------------------------------------------------
  // trace point = 294
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0468903f, 39.685421f, 1483288833, 5.2510519),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0469437f,
                                                39.6853943f, leg_index, 21.4860649f, 5007,
                                                maneuver_index, 0.36558342f, 87));
  //----------------------------------------------------------------
  // trace point = 295
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0471039f, 39.6859055f, 1483288844, 5.15676308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0471573f,
                                                39.6858902f, leg_index, 21.4499264f, 4998,
                                                maneuver_index, 0.329444885f, 78));
  //----------------------------------------------------------------
  // trace point = 296
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0471802f, 39.6860733f, 1483288848, 5.03653383),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0472336f,
                                                39.6860619f, leg_index, 21.4373779f, 4995,
                                                maneuver_index, 0.316896439f, 75));
  //----------------------------------------------------------------
  // trace point = 297
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0473328f, 39.686451f, 1483288857, 4.88197803),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0473785f,
                                                39.6864433f, leg_index, 21.4098587f, 4989,
                                                maneuver_index, 0.289377213f, 69));
  //----------------------------------------------------------------
  // trace point = 298
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0473404f, 39.6870003f, 1483288868, 5.54617691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0473785f,
                                                39.6870041f, leg_index, 21.3705769f, 4980,
                                                maneuver_index, 0.250095367f, 60));
  //----------------------------------------------------------------
  // trace point = 299
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.047142f, 39.6875153f, 1483288877, 6.68576193),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0471725f,
                                                39.6875229f, leg_index, 21.3329067f, 4971,
                                                maneuver_index, 0.212425232f, 51));
  //----------------------------------------------------------------
  // trace point = 300
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0469284f, 39.687912f, 1483288884, 6.80169821),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0469742f,
                                                39.6879272f, leg_index, 21.3030052f, 4964,
                                                maneuver_index, 0.182523727f, 44));
  //----------------------------------------------------------------
  // trace point = 301
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0466309f, 39.6884613f, 1483288894, 6.60630608),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0466766f,
                                                39.6884766f, leg_index, 21.2618408f, 4954,
                                                maneuver_index, 0.141359329f, 34));
  //----------------------------------------------------------------
  // trace point = 302 | Alert | In 500 feet Turn right onto Biggs Highway.
  maneuver_index = 2;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0463409f, 39.6889992f, 1483288905, 5.90033007),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0463791f, 39.6890106f, leg_index, 21.2216473f,
                                                4945, maneuver_index, 0.101165771f, 25,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 303
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0461197f, 39.6894112f, 1483288914, 5.55096102),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0461578f,
                                                39.6894226f, leg_index, 21.1908131f, 4937,
                                                maneuver_index, 0.0703315735f, 17));
  //----------------------------------------------------------------
  // trace point = 304
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0458832f, 39.689846f, 1483288924, 5.24382114),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0459213f,
                                                39.6898575f, leg_index, 21.158205f, 4930,
                                                maneuver_index, 0.0377235413f, 10));
  //----------------------------------------------------------------
  // trace point = 305 | Pre | Turn right onto Biggs Highway, Maryland 2 74.
  maneuver_index = 2;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0456924f, 39.6901932f, 1483288933, 4.6688118),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0457382f, 39.6902046f, leg_index, 21.1322899f,
                                                4923, maneuver_index, 0.0118083954f, 3,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 306
  maneuver_index = 2;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0456467f, 39.6902657f, 1483288935, 4.24094391),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0457077f,
                                                39.6902809f, leg_index, 21.1267776f, 4921,
                                                maneuver_index, 0.00629615784f, 1));
  //----------------------------------------------------------------
  // trace point = 307 | Post | Continue for 4 tenths of a mile.
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0453873f, 39.6903152f, 1483288940, 4.67987919),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.0453873f, 39.6903229f, leg_index, 21.1055756f,
                                                4916, maneuver_index, 0.384138107f, 89,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 308
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0446625f, 39.6902122f, 1483288950, 6.31539679),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0446625f,
                                                39.6902237f, leg_index, 21.0663872f, 4908,
                                                maneuver_index, 0.344949722f, 81));
  //----------------------------------------------------------------
  // trace point = 309
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0439606f, 39.6901093f, 1483288959, 6.75995111),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0439529f,
                                                39.6901245f, leg_index, 21.0280018f, 4899,
                                                maneuver_index, 0.306564331f, 72));
  //----------------------------------------------------------------
  // trace point = 310
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.043808f, 39.6900902f, 1483288961, 6.76476717),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.043808f,
                                                39.6901054f, leg_index, 21.020174f, 4897,
                                                maneuver_index, 0.298736572f, 70));
  //----------------------------------------------------------------
  // trace point = 311
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0431137f, 39.6899948f, 1483288970, 6.69480324),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0431137f,
                                                39.6900063f, leg_index, 20.9825726f, 4888,
                                                maneuver_index, 0.261135101f, 61));
  //----------------------------------------------------------------
  // trace point = 312
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0424271f, 39.6898575f, 1483288979, 6.75174809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0424194f,
                                                39.6898766f, leg_index, 20.9445343f, 4880,
                                                maneuver_index, 0.223096848f, 53));
  //----------------------------------------------------------------
  // trace point = 313
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0417633f, 39.6896324f, 1483288988, 6.87072992),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0417557f,
                                                39.68964f, leg_index, 20.9045353f, 4870,
                                                maneuver_index, 0.183097839f, 43));
  //----------------------------------------------------------------
  // trace point = 314
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.041626f, 39.6895714f, 1483288990, 6.98338318),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.041626f,
                                                39.6895752f, leg_index, 20.8972778f, 4869,
                                                maneuver_index, 0.175840378f, 42));
  //----------------------------------------------------------------
  // trace point = 315
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.041008f, 39.6892281f, 1483288999, 7.2674408),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0410004f,
                                                39.6892319f, leg_index, 20.8563461f, 4859,
                                                maneuver_index, 0.134908676f, 32));
  //----------------------------------------------------------------
  // trace point = 316 | Alert | In 500 feet Turn left onto Wilson Road.
  maneuver_index = 3;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0404205f, 39.6888542f, 1483289008, 7.23788404),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0404053f, 39.6888657f, leg_index, 20.8157864f,
                                                4850, maneuver_index, 0.0943489075f, 23,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 317
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0398407f, 39.6884918f, 1483289017, 7.10065413),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0398254f,
                                                39.6885071f, leg_index, 20.7761898f, 4840,
                                                maneuver_index, 0.0547523499f, 13));
  //----------------------------------------------------------------
  // trace point = 318 | Pre | Turn left onto Wilson Road.
  maneuver_index = 3;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0392914f, 39.6881447f, 1483289026, 6.74745989),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.039299f, 39.6881371f, leg_index, 20.7381897f,
                                                4831, maneuver_index, 0.016752243f, 4,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 319
  maneuver_index = 3;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0391235f, 39.6880264f, 1483289029, 6.54109287),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0391541f,
                                                39.6880074f, leg_index, 20.7263565f, 4828,
                                                maneuver_index, 0.00491905212f, 1));
  //----------------------------------------------------------------
  // trace point = 320 | Post | Continue for 3.4 miles.
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0388412f, 39.6879845f, 1483289033, 6.23551083),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.0388489f, 39.6880112f, leg_index, 20.7078552f,
                                                4823, maneuver_index, 3.43022156f, 794,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 321
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0386963f, 39.6880035f, 1483289035, 6.23997116),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0387039f,
                                                39.6880341f, leg_index, 20.6999798f, 4821,
                                                maneuver_index, 3.42234612f, 792));
  //----------------------------------------------------------------
  // trace point = 322
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0385513f, 39.6880226f, 1483289037, 6.30961895),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.038559f,
                                                39.6880531f, leg_index, 20.692152f, 4819,
                                                maneuver_index, 3.41451836f, 790));
  //----------------------------------------------------------------
  // trace point = 323
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0379181f, 39.6880989f, 1483289046, 6.11118603),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0379257f,
                                                39.6881332f, leg_index, 20.6579914f, 4811,
                                                maneuver_index, 3.38035774f, 782));
  //----------------------------------------------------------------
  // trace point = 324
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.037529f, 39.6881447f, 1483289052, 5.59084606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0375366f,
                                                39.688179f, leg_index, 20.6370411f, 4806,
                                                maneuver_index, 3.35940742f, 777));
  //----------------------------------------------------------------
  // trace point = 325
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0368347f, 39.6882591f, 1483289062, 6.09576082),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0368347f,
                                                39.688282f, leg_index, 20.5990028f, 4798,
                                                maneuver_index, 3.32136917f, 769));
  //----------------------------------------------------------------
  // trace point = 326
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0361328f, 39.6883049f, 1483289070, 7.53120995),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0361328f,
                                                39.688324f, leg_index, 20.5613995f, 4790,
                                                maneuver_index, 3.28376579f, 761));
  //----------------------------------------------------------------
  // trace point = 327
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0357513f, 39.6882896f, 1483289074, 8.06180763),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0357513f,
                                                39.6883049f, leg_index, 20.5410423f, 4785,
                                                maneuver_index, 3.26340866f, 756));
  //----------------------------------------------------------------
  // trace point = 328
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0352783f, 39.6882591f, 1483289079, 8.21486378),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0352783f,
                                                39.6882744f, leg_index, 20.5157738f, 4779,
                                                maneuver_index, 3.23814011f, 750));
  //----------------------------------------------------------------
  // trace point = 329
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.03508f, 39.6882477f, 1483289081, 8.34082794),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.03508f,
                                                39.6882591f, leg_index, 20.5051537f, 4777,
                                                maneuver_index, 3.22751999f, 748));
  //----------------------------------------------------------------
  // trace point = 330
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.034317f, 39.6881981f, 1483289089, 8.22228146),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.034317f,
                                                39.6881981f, leg_index, 20.4643269f, 4767,
                                                maneuver_index, 3.18669319f, 738));
  //----------------------------------------------------------------
  // trace point = 331
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0337601f, 39.6881638f, 1483289095, 8.00738716),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0337601f,
                                                39.6881676f, leg_index, 20.4346046f, 4760,
                                                maneuver_index, 3.15697098f, 731));
  //----------------------------------------------------------------
  // trace point = 332
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0331116f, 39.6881218f, 1483289102, 7.90670681),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0331116f,
                                                39.6881294f, leg_index, 20.3999767f, 4752,
                                                maneuver_index, 3.12234306f, 723));
  //----------------------------------------------------------------
  // trace point = 333
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0324402f, 39.6880722f, 1483289109, 8.28345966),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0324402f,
                                                39.6880798f, leg_index, 20.3640747f, 4744,
                                                maneuver_index, 3.08644104f, 715));
  //----------------------------------------------------------------
  // trace point = 334
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.032135f, 39.6880608f, 1483289112, 8.72693825),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.032135f,
                                                39.6880722f, leg_index, 20.3478203f, 4740,
                                                maneuver_index, 3.07018661f, 711));
  //----------------------------------------------------------------
  // trace point = 335
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0318222f, 39.6880646f, 1483289115, 8.88429737),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0318222f,
                                                39.6880798f, leg_index, 20.3311539f, 4737,
                                                maneuver_index, 3.0535202f, 708));
  //----------------------------------------------------------------
  // trace point = 336
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0314941f, 39.6880913f, 1483289118, 9.44017124),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0315018f,
                                                39.688118f, leg_index, 20.3138885f, 4732,
                                                maneuver_index, 3.03625488f, 703));
  //----------------------------------------------------------------
  // trace point = 337
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0311584f, 39.6881218f, 1483289121, 9.78092194),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0311661f,
                                                39.6881523f, leg_index, 20.2958641f, 4728,
                                                maneuver_index, 3.01823044f, 699));
  //----------------------------------------------------------------
  // trace point = 338
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0308075f, 39.6881599f, 1483289124, 10.0879822),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0308151f,
                                                39.6881866f, leg_index, 20.2770329f, 4724,
                                                maneuver_index, 2.99939919f, 695));
  //----------------------------------------------------------------
  // trace point = 339
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0304337f, 39.6881943f, 1483289127, 10.635931),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0304413f,
                                                39.688221f, leg_index, 20.2569923f, 4719,
                                                maneuver_index, 2.97935867f, 690));
  //----------------------------------------------------------------
  // trace point = 340
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0301819f, 39.6882172f, 1483289129, 11.0175896),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0301895f,
                                                39.6882439f, leg_index, 20.2434978f, 4716,
                                                maneuver_index, 2.96586418f, 687));
  //----------------------------------------------------------------
  // trace point = 341
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0295334f, 39.6882858f, 1483289134, 11.1309071),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.029541f,
                                                39.6883087f, leg_index, 20.2086887f, 4707,
                                                maneuver_index, 2.93105507f, 678));
  //----------------------------------------------------------------
  // trace point = 342
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0292816f, 39.6883087f, 1483289136, 11.1735868),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0292816f,
                                                39.6883316f, leg_index, 20.1947899f, 4704,
                                                maneuver_index, 2.91715622f, 675));
  //----------------------------------------------------------------
  // trace point = 343
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0290146f, 39.6883316f, 1483289138, 11.2586365),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0290146f,
                                                39.6883545f, leg_index, 20.1804886f, 4701,
                                                maneuver_index, 2.90285492f, 672));
  //----------------------------------------------------------------
  // trace point = 344
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0284882f, 39.6883774f, 1483289142, 11.3341656),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0284882f,
                                                39.6884003f, leg_index, 20.1522884f, 4694,
                                                maneuver_index, 2.87465477f, 665));
  //----------------------------------------------------------------
  // trace point = 345
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.027977f, 39.688427f, 1483289146, 11.0919123),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.027977f,
                                                39.688446f, leg_index, 20.1248951f, 4688,
                                                maneuver_index, 2.84726143f, 659));
  //----------------------------------------------------------------
  // trace point = 346
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0277252f, 39.6884499f, 1483289148, 10.8267221),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0277252f,
                                                39.6884689f, leg_index, 20.1114006f, 4684,
                                                maneuver_index, 2.83376694f, 655));
  //----------------------------------------------------------------
  // trace point = 347
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0272369f, 39.6884956f, 1483289152, 10.5469751),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0272369f,
                                                39.6885147f, leg_index, 20.0852184f, 4678,
                                                maneuver_index, 2.80758476f, 649));
  //----------------------------------------------------------------
  // trace point = 348
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0264893f, 39.6885643f, 1483289158, 10.7110996),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0264893f,
                                                39.6885834f, leg_index, 20.0451374f, 4669,
                                                maneuver_index, 2.76750374f, 640));
  //----------------------------------------------------------------
  // trace point = 349
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0263672f, 39.6885757f, 1483289159, 10.8759308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0263672f,
                                                39.6885948f, leg_index, 20.0385933f, 4668,
                                                maneuver_index, 2.76095963f, 639));
  //----------------------------------------------------------------
  // trace point = 350
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0256042f, 39.6886482f, 1483289165, 10.9892864),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0256042f,
                                                39.6886635f, leg_index, 19.9977055f, 4658,
                                                maneuver_index, 2.72007179f, 629));
  //----------------------------------------------------------------
  // trace point = 351
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0250854f, 39.6886978f, 1483289169, 11.1625729),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0250854f,
                                                39.6887093f, leg_index, 19.9699097f, 4652,
                                                maneuver_index, 2.692276f, 623));
  //----------------------------------------------------------------
  // trace point = 352
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0242996f, 39.6887627f, 1483289175, 11.3129148),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0242996f,
                                                39.6887741f, leg_index, 19.9278469f, 4642,
                                                maneuver_index, 2.65021324f, 613));
  //----------------------------------------------------------------
  // trace point = 353
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0238953f, 39.6887894f, 1483289178, 11.4723253),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0238953f,
                                                39.6888008f, leg_index, 19.9062119f, 4637,
                                                maneuver_index, 2.62857819f, 608));
  //----------------------------------------------------------------
  // trace point = 354
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0235214f, 39.6888275f, 1483289181, 10.8192196),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0235291f,
                                                39.6888504f, leg_index, 19.8864212f, 4632,
                                                maneuver_index, 2.60878754f, 603));
  //----------------------------------------------------------------
  // trace point = 355
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0232925f, 39.6888618f, 1483289183, 10.1260662),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0233002f,
                                                39.6888885f, leg_index, 19.8739128f, 4630,
                                                maneuver_index, 2.59627914f, 601));
  //----------------------------------------------------------------
  // trace point = 356
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0231781f, 39.6888885f, 1483289184, 9.90748692),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0231934f,
                                                39.6889191f, leg_index, 19.8678417f, 4628,
                                                maneuver_index, 2.59020805f, 599));
  //----------------------------------------------------------------
  // trace point = 357
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0227814f, 39.6890411f, 1483289188, 9.50823116),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0227966f,
                                                39.6890602f, leg_index, 19.8445415f, 4623,
                                                maneuver_index, 2.56690788f, 594));
  //----------------------------------------------------------------
  // trace point = 358
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0222092f, 39.6894035f, 1483289196, 7.93016386),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0222397f,
                                                39.6894302f, leg_index, 19.8053265f, 4614,
                                                maneuver_index, 2.52769279f, 585));
  //----------------------------------------------------------------
  // trace point = 359
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0220871f, 39.6894836f, 1483289198, 6.82660294),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.02211f,
                                                39.6895065f, leg_index, 19.7966309f, 4611,
                                                maneuver_index, 2.51899719f, 582));
  //----------------------------------------------------------------
  // trace point = 360
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0217896f, 39.6896172f, 1483289203, 5.97249079),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0218048f,
                                                39.68964f, leg_index, 19.7778873f, 4606,
                                                maneuver_index, 2.50025368f, 577));
  //----------------------------------------------------------------
  // trace point = 361
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0215378f, 39.689682f, 1483289207, 5.59945583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0215454f,
                                                39.6897049f, leg_index, 19.7633476f, 4602,
                                                maneuver_index, 2.48571396f, 573));
  //----------------------------------------------------------------
  // trace point = 362
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.021286f, 39.6897392f, 1483289211, 5.67397118),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0212936f,
                                                39.6897583f, leg_index, 19.7494469f, 4599,
                                                maneuver_index, 2.4718132f, 570));
  //----------------------------------------------------------------
  // trace point = 363
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0209808f, 39.6898079f, 1483289216, 5.38905001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0209885f,
                                                39.6898232f, leg_index, 19.7325954f, 4595,
                                                maneuver_index, 2.45496178f, 566));
  //----------------------------------------------------------------
  // trace point = 364
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0205231f, 39.6899071f, 1483289224, 5.10539198),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0205231f,
                                                39.6899147f, leg_index, 19.7070255f, 4589,
                                                maneuver_index, 2.42939186f, 560));
  //----------------------------------------------------------------
  // trace point = 365
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.020134f, 39.6899986f, 1483289231, 4.99428892),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.020134f,
                                                39.6899986f, leg_index, 19.6855183f, 4583,
                                                maneuver_index, 2.4078846f, 554));
  //----------------------------------------------------------------
  // trace point = 366
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0196762f, 39.6901093f, 1483289239, 5.13559484),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0196762f,
                                                39.6901131f, leg_index, 19.6598911f, 4577,
                                                maneuver_index, 2.38225746f, 548));
  //----------------------------------------------------------------
  // trace point = 367
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0189362f, 39.6903114f, 1483289249, 6.72663498),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0189362f,
                                                39.6903152f, leg_index, 19.6180916f, 4567,
                                                maneuver_index, 2.34045792f, 538));
  //----------------------------------------------------------------
  // trace point = 368
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0185394f, 39.6904221f, 1483289253, 9.0753336),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0185394f,
                                                39.6904297f, leg_index, 19.5955372f, 4563,
                                                maneuver_index, 2.31790352f, 534));
  //----------------------------------------------------------------
  // trace point = 369
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0182114f, 39.6905136f, 1483289256, 9.83570004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.018219f,
                                                39.6905251f, leg_index, 19.5772495f, 4558,
                                                maneuver_index, 2.29961586f, 529));
  //----------------------------------------------------------------
  // trace point = 370
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0179901f, 39.6905746f, 1483289258, 10.2147551),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0179977f,
                                                39.6905937f, leg_index, 19.5645504f, 4556,
                                                maneuver_index, 2.28691673f, 527));
  //----------------------------------------------------------------
  // trace point = 371
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0176468f, 39.6906776f, 1483289261, 10.5398159),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0176544f,
                                                39.6906967f, leg_index, 19.5449371f, 4551,
                                                maneuver_index, 2.26730347f, 522));
  //----------------------------------------------------------------
  // trace point = 372
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0174255f, 39.6907616f, 1483289263, 10.5437145),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0174408f,
                                                39.6907806f, leg_index, 19.5321388f, 4548,
                                                maneuver_index, 2.25450516f, 519));
  //----------------------------------------------------------------
  // trace point = 373
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0173264f, 39.690815f, 1483289264, 10.5350571),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0173492f,
                                                39.6908379f, leg_index, 19.5248966f, 4546,
                                                maneuver_index, 2.24726295f, 517));
  //----------------------------------------------------------------
  // trace point = 374
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.017067f, 39.6910172f, 1483289267, 10.4896393),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.017128f,
                                                39.6910477f, leg_index, 19.5070915f, 4542,
                                                maneuver_index, 2.22945786f, 513));
  //----------------------------------------------------------------
  // trace point = 375
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0169296f, 39.6911736f, 1483289269, 10.479661),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0169907f,
                                                39.6912041f, leg_index, 19.4940262f, 4539,
                                                maneuver_index, 2.21639252f, 510));
  //----------------------------------------------------------------
  // trace point = 376
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0167542f, 39.6914101f, 1483289272, 10.1251717),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0167999f,
                                                39.691433f, leg_index, 19.4752216f, 4535,
                                                maneuver_index, 2.19758797f, 506));
  //----------------------------------------------------------------
  // trace point = 377
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0165329f, 39.6917152f, 1483289276, 9.72193813),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0165634f,
                                                39.6917305f, leg_index, 19.4510803f, 4529,
                                                maneuver_index, 2.17344666f, 500));
  //----------------------------------------------------------------
  // trace point = 378
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0162201f, 39.6922493f, 1483289283, 9.31040573),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162888f,
                                                39.6922836f, leg_index, 19.4100895f, 4519,
                                                maneuver_index, 2.13245583f, 490));
  //----------------------------------------------------------------
  // trace point = 379
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0162048f, 39.6923332f, 1483289284, 9.63162518),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162582f,
                                                39.6923485f, leg_index, 19.405323f, 4518,
                                                maneuver_index, 2.12768936f, 489));
  //----------------------------------------------------------------
  // trace point = 380
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0161667f, 39.6926575f, 1483289288, 8.99809074),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0161972f,
                                                39.6926575f, leg_index, 19.3835068f, 4514,
                                                maneuver_index, 2.10587311f, 485));
  //----------------------------------------------------------------
  // trace point = 381
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0161743f, 39.692894f, 1483289291, 8.79984665),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162048f,
                                                39.6928902f, leg_index, 19.3673954f, 4510,
                                                maneuver_index, 2.08976173f, 481));
  //----------------------------------------------------------------
  // trace point = 382
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0162125f, 39.6934624f, 1483289299, 7.93986416),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162659f,
                                                39.6934586f, leg_index, 19.3279438f, 4500,
                                                maneuver_index, 2.05031013f, 471));
  //----------------------------------------------------------------
  // trace point = 383
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0162201f, 39.6936989f, 1483289303, 6.48944998),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162735f,
                                                39.6936989f, leg_index, 19.3113041f, 4497,
                                                maneuver_index, 2.03367043f, 468));
  //----------------------------------------------------------------
  // trace point = 384
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0162354f, 39.6938972f, 1483289307, 5.60563803),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162735f,
                                                39.6938972f, leg_index, 19.2975826f, 4494,
                                                maneuver_index, 2.01994896f, 465));
  //----------------------------------------------------------------
  // trace point = 385
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.016243f, 39.6939507f, 1483289308, 5.90242004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162735f,
                                                39.6939507f, leg_index, 19.29389f, 4493,
                                                maneuver_index, 2.01625633f, 464));
  //----------------------------------------------------------------
  // trace point = 386
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.016243f, 39.6941109f, 1483289311, 5.93710804),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162735f,
                                                39.6941109f, leg_index, 19.2828064f, 4491,
                                                maneuver_index, 2.00517273f, 462));
  //----------------------------------------------------------------
  // trace point = 387
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.016243f, 39.6946831f, 1483289320, 7.062603),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162735f,
                                                39.6946831f, leg_index, 19.2432251f, 4482,
                                                maneuver_index, 1.96559143f, 453));
  //----------------------------------------------------------------
  // trace point = 388
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0161972f, 39.6950607f, 1483289325, 8.44360542),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0162735f,
                                                39.6950607f, leg_index, 19.217104f, 4476,
                                                maneuver_index, 1.93947029f, 447));
  //----------------------------------------------------------------
  // trace point = 389
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0159531f, 39.6955223f, 1483289331, 9.32466698),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0160065f,
                                                39.6955566f, leg_index, 19.1792259f, 4467,
                                                maneuver_index, 1.90159225f, 438));
  //----------------------------------------------------------------
  // trace point = 390
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0157471f, 39.6957283f, 1483289334, 9.53932381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0157852f,
                                                39.695755f, leg_index, 19.1600094f, 4463,
                                                maneuver_index, 1.88237572f, 434));
  //----------------------------------------------------------------
  // trace point = 391
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0151291f, 39.6960869f, 1483289341, 9.4748621),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0151367f,
                                                39.6961021f, leg_index, 19.1187344f, 4454,
                                                maneuver_index, 1.84110069f, 425));
  //----------------------------------------------------------------
  // trace point = 392
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0150375f, 39.696125f, 1483289342, 9.07044411),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0150452f,
                                                39.6961403f, leg_index, 19.1131935f, 4453,
                                                maneuver_index, 1.83555984f, 424));
  //----------------------------------------------------------------
  // trace point = 393
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0146637f, 39.6962624f, 1483289346, 8.82400608),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.014679f,
                                                39.6962814f, leg_index, 19.0913944f, 4448,
                                                maneuver_index, 1.81376076f, 419));
  //----------------------------------------------------------------
  // trace point = 394
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0140457f, 39.6966209f, 1483289354, 8.2736311),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0140991f,
                                                39.6966553f, leg_index, 19.0503521f, 4438,
                                                maneuver_index, 1.77271843f, 409));
  //----------------------------------------------------------------
  // trace point = 395
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0136642f, 39.697113f, 1483289364, 6.39378309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0137177f,
                                                39.6971321f, leg_index, 19.0115566f, 4430,
                                                maneuver_index, 1.73392296f, 401));
  //----------------------------------------------------------------
  // trace point = 396
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0136032f, 39.6972046f, 1483289366, 5.59140015),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0136566f,
                                                39.6972237f, leg_index, 19.0044403f, 4428,
                                                maneuver_index, 1.72680664f, 399));
  //----------------------------------------------------------------
  // trace point = 397
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0135269f, 39.6973305f, 1483289369, 5.19104004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0135727f,
                                                39.6973495f, leg_index, 18.9946537f, 4426,
                                                maneuver_index, 1.71702003f, 397));
  //----------------------------------------------------------------
  // trace point = 398
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0134506f, 39.6974487f, 1483289372, 4.95333719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0134964f,
                                                39.697464f, leg_index, 18.9857578f, 4424,
                                                maneuver_index, 1.70812416f, 395));
  //----------------------------------------------------------------
  // trace point = 399
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0133362f, 39.6976357f, 1483289377, 4.59317684),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0133743f,
                                                39.6976509f, leg_index, 18.9712868f, 4420,
                                                maneuver_index, 1.69365311f, 391));
  //----------------------------------------------------------------
  // trace point = 400
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0132675f, 39.6977348f, 1483289380, 4.15501308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0133057f,
                                                39.6977501f, leg_index, 18.9635143f, 4419,
                                                maneuver_index, 1.68588066f, 390));
  //----------------------------------------------------------------
  // trace point = 401
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0131607f, 39.6978989f, 1483289385, 4.0778718),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0131989f,
                                                39.6979141f, leg_index, 18.9508247f, 4416,
                                                maneuver_index, 1.67319107f, 387));
  //----------------------------------------------------------------
  // trace point = 402
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.013031f, 39.6981049f, 1483289391, 4.21956921),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0130615f,
                                                39.6981163f, leg_index, 18.9350452f, 4412,
                                                maneuver_index, 1.65741158f, 383));
  //----------------------------------------------------------------
  // trace point = 403
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0127716f, 39.6985016f, 1483289402, 4.50441694),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0128098f,
                                                39.6985168f, leg_index, 18.9042683f, 4405,
                                                maneuver_index, 1.6266346f, 376));
  //----------------------------------------------------------------
  // trace point = 404
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0125046f, 39.6990318f, 1483289416, 4.53313684),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0125427f,
                                                39.6990395f, leg_index, 18.8652496f, 4395,
                                                maneuver_index, 1.58761597f, 366));
  //----------------------------------------------------------------
  // trace point = 405
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0124893f, 39.69907f, 1483289417, 4.50401306),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0125275f,
                                                39.6990776f, leg_index, 18.8624973f, 4395,
                                                maneuver_index, 1.58486366f, 366));
  //----------------------------------------------------------------
  // trace point = 406
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.012352f, 39.6995163f, 1483289428, 4.63252687),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0123978f,
                                                39.6995239f, leg_index, 18.830864f, 4388,
                                                maneuver_index, 1.55323029f, 359));
  //----------------------------------------------------------------
  // trace point = 407
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0122223f, 39.7000008f, 1483289439, 4.97859621),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0122604f,
                                                39.7000084f, leg_index, 18.796566f, 4380,
                                                maneuver_index, 1.51893234f, 351));
  //----------------------------------------------------------------
  // trace point = 408
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0120926f, 39.700531f, 1483289450, 5.47404814),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0121307f,
                                                39.7005348f, leg_index, 18.7595043f, 4371,
                                                maneuver_index, 1.48187065f, 342));
  //----------------------------------------------------------------
  // trace point = 409
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0119781f, 39.7009773f, 1483289459, 5.62709522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0120163f,
                                                39.7009811f, leg_index, 18.7280369f, 4364,
                                                maneuver_index, 1.45040321f, 335));
  //----------------------------------------------------------------
  // trace point = 410
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0118332f, 39.7015457f, 1483289470, 5.86097813),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0118713f,
                                                39.7015495f, leg_index, 18.6879711f, 4354,
                                                maneuver_index, 1.41033745f, 325));
  //----------------------------------------------------------------
  // trace point = 411
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0116806f, 39.7021141f, 1483289481, 5.86870289),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0117264f,
                                                39.7021217f, leg_index, 18.647646f, 4345,
                                                maneuver_index, 1.37001228f, 316));
  //----------------------------------------------------------------
  // trace point = 412
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0115356f, 39.7026749f, 1483289492, 5.79617691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0115814f,
                                                39.7026825f, leg_index, 18.6081009f, 4336,
                                                maneuver_index, 1.33046722f, 307));
  //----------------------------------------------------------------
  // trace point = 413
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0114212f, 39.7030907f, 1483289501, 5.26106882),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0114822f,
                                                39.7030983f, leg_index, 18.5788574f, 4329,
                                                maneuver_index, 1.30122375f, 300));
  //----------------------------------------------------------------
  // trace point = 414
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0113525f, 39.7033539f, 1483289508, 4.24308014),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0114059f,
                                                39.7033653f, leg_index, 18.5599403f, 4325,
                                                maneuver_index, 1.28230667f, 296));
  //----------------------------------------------------------------
  // trace point = 415
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0112915f, 39.7036514f, 1483289517, 3.74655795),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0113068f,
                                                39.7036552f, leg_index, 18.5392056f, 4320,
                                                maneuver_index, 1.26157188f, 291));
  //----------------------------------------------------------------
  // trace point = 416
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0112152f, 39.7038193f, 1483289524, 2.80982399),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0112534f,
                                                39.7038269f, leg_index, 18.5269966f, 4317,
                                                maneuver_index, 1.24936295f, 288));
  //----------------------------------------------------------------
  // trace point = 417
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0111389f, 39.7039642f, 1483289529, 3.4808569),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0111923f,
                                                39.7039871f, leg_index, 18.5154457f, 4315,
                                                maneuver_index, 1.23781204f, 286));
  //----------------------------------------------------------------
  // trace point = 418
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0111771f, 39.7045326f, 1483289544, 4.24466801),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0112228f,
                                                39.7045288f, leg_index, 18.4777813f, 4306,
                                                maneuver_index, 1.20014763f, 277));
  //----------------------------------------------------------------
  // trace point = 419
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0112f, 39.7046928f, 1483289548, 4.44773483),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0112457f,
                                                39.704689f, leg_index, 18.4666328f, 4303,
                                                maneuver_index, 1.18899918f, 274));
  //----------------------------------------------------------------
  // trace point = 420
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0112762f, 39.7052574f, 1483289561, 4.86131191),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.011322f,
                                                39.7052536f, leg_index, 18.4273663f, 4294,
                                                maneuver_index, 1.14973259f, 265));
  //----------------------------------------------------------------
  // trace point = 421
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0113297f, 39.7058563f, 1483289572, 6.05905008),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0113907f,
                                                39.7058525f, leg_index, 18.3857784f, 4284,
                                                maneuver_index, 1.10814476f, 255));
  //----------------------------------------------------------------
  // trace point = 422
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0113907f, 39.706398f, 1483289581, 6.72505283),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0114517f,
                                                39.7063942f, leg_index, 18.3481693f, 4276,
                                                maneuver_index, 1.07053566f, 247));
  //----------------------------------------------------------------
  // trace point = 423
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0113907f, 39.706459f, 1483289582, 6.80990314),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0114594f,
                                                39.7064552f, leg_index, 18.3439293f, 4275,
                                                maneuver_index, 1.06629562f, 246));
  //----------------------------------------------------------------
  // trace point = 424
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0114594f, 39.7070236f, 1483289591, 7.01655293),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0115204f,
                                                39.7070198f, leg_index, 18.3047447f, 4265,
                                                maneuver_index, 1.02711105f, 236));
  //----------------------------------------------------------------
  // trace point = 425
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0115051f, 39.7076035f, 1483289601, 6.49182177),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0115738f,
                                                39.7075996f, leg_index, 18.2645302f, 4256,
                                                maneuver_index, 0.986896515f, 227));
  //----------------------------------------------------------------
  // trace point = 426
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0115356f, 39.7078476f, 1483289606, 5.40623617),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0115967f,
                                                39.7078438f, leg_index, 18.2475986f, 4252,
                                                maneuver_index, 0.969964981f, 223));
  //----------------------------------------------------------------
  // trace point = 427
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0115662f, 39.7081146f, 1483289612, 5.00795889),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0116196f,
                                                39.7081146f, leg_index, 18.2287483f, 4248,
                                                maneuver_index, 0.951114655f, 219));
  //----------------------------------------------------------------
  // trace point = 428
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0115814f, 39.7083054f, 1483289616, 5.25129795),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0116348f,
                                                39.7083015f, leg_index, 18.2157803f, 4245,
                                                maneuver_index, 0.938146591f, 216));
  //----------------------------------------------------------------
  // trace point = 429
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0116501f, 39.7088547f, 1483289627, 5.59831095),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0116959f,
                                                39.7088509f, leg_index, 18.1776447f, 4236,
                                                maneuver_index, 0.900011063f, 207));
  //----------------------------------------------------------------
  // trace point = 430
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0116959f, 39.709259f, 1483289635, 5.64605188),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0117416f,
                                                39.7092552f, leg_index, 18.1495705f, 4229,
                                                maneuver_index, 0.871936798f, 200));
  //----------------------------------------------------------------
  // trace point = 431
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0117493f, 39.7096596f, 1483289644, 4.99589491),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.011795f,
                                                39.7096558f, leg_index, 18.1217194f, 4223,
                                                maneuver_index, 0.844085693f, 194));
  //----------------------------------------------------------------
  // trace point = 432
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0117722f, 39.7099228f, 1483289650, 4.83702421),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0118256f,
                                                39.709919f, leg_index, 18.1034412f, 4219,
                                                maneuver_index, 0.825807571f, 190));
  //----------------------------------------------------------------
  // trace point = 433
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0118256f, 39.7104874f, 1483289663, 4.85117579),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0118866f,
                                                39.7104836f, leg_index, 18.0642529f, 4210,
                                                maneuver_index, 0.786619186f, 181));
  //----------------------------------------------------------------
  // trace point = 434
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.011879f, 39.7110558f, 1483289674, 5.76182985),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0119476f,
                                                39.7110519f, leg_index, 18.0248032f, 4201,
                                                maneuver_index, 0.747169495f, 172));
  //----------------------------------------------------------------
  // trace point = 435
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.01194f, 39.7116394f, 1483289683, 7.27210283),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0120087f,
                                                39.7116356f, leg_index, 17.9843006f, 4192,
                                                maneuver_index, 0.706666946f, 163));
  //----------------------------------------------------------------
  // trace point = 436
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0119553f, 39.7117805f, 1483289685, 7.94117212),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0120239f,
                                                39.7117767f, leg_index, 17.9745045f, 4190,
                                                maneuver_index, 0.696870804f, 161));
  //----------------------------------------------------------------
  // trace point = 437
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0119858f, 39.7120018f, 1483289688, 8.2052927),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0120468f,
                                                39.711998f, leg_index, 17.9591522f, 4186,
                                                maneuver_index, 0.681518555f, 157));
  //----------------------------------------------------------------
  // trace point = 438
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0120087f, 39.7121544f, 1483289690, 8.55471039),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0120697f,
                                                39.7121468f, leg_index, 17.9487801f, 4184,
                                                maneuver_index, 0.671146393f, 155));
  //----------------------------------------------------------------
  // trace point = 439
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0120316f, 39.7123146f, 1483289692, 8.96465111),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0120926f,
                                                39.712307f, leg_index, 17.9376392f, 4181,
                                                maneuver_index, 0.660005569f, 152));
  //----------------------------------------------------------------
  // trace point = 440
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0120621f, 39.7125702f, 1483289695, 9.47785282),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0121307f,
                                                39.7125664f, leg_index, 17.9195557f, 4177,
                                                maneuver_index, 0.641921997f, 148));
  //----------------------------------------------------------------
  // trace point = 441
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0121078f, 39.7129173f, 1483289699, 9.78385925),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0121689f,
                                                39.7129135f, leg_index, 17.8946686f, 4171,
                                                maneuver_index, 0.617034912f, 142));
  //----------------------------------------------------------------
  // trace point = 442
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0121307f, 39.7132912f, 1483289703, 10.3954544),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0121994f,
                                                39.7132874f, leg_index, 17.8695488f, 4166,
                                                maneuver_index, 0.591915131f, 137));
  //----------------------------------------------------------------
  // trace point = 443
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0121384f, 39.7133904f, 1483289704, 11.0326004),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.012207f,
                                                39.7133865f, leg_index, 17.8626785f, 4164,
                                                maneuver_index, 0.585044861f, 135));
  //----------------------------------------------------------------
  // trace point = 444
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.012146f, 39.7134933f, 1483289705, 11.2592363),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0122223f,
                                                39.7134895f, leg_index, 17.8555126f, 4162,
                                                maneuver_index, 0.577878952f, 133));
  //----------------------------------------------------------------
  // trace point = 445
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0121689f, 39.7137833f, 1483289708, 10.8887157),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0122528f,
                                                39.7137794f, leg_index, 17.8353939f, 4158,
                                                maneuver_index, 0.557760239f, 129));
  //----------------------------------------------------------------
  // trace point = 446
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0121841f, 39.7139702f, 1483289710, 10.3291378),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0122681f,
                                                39.7139664f, leg_index, 17.8224392f, 4155,
                                                maneuver_index, 0.544805527f, 126));
  //----------------------------------------------------------------
  // trace point = 447
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.012207f, 39.7141418f, 1483289712, 9.61970425),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0122833f,
                                                39.714138f, leg_index, 17.8105373f, 4152,
                                                maneuver_index, 0.532903671f, 123));
  //----------------------------------------------------------------
  // trace point = 448
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0122833f, 39.7146835f, 1483289719, 8.63637543),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0123367f,
                                                39.7146797f, leg_index, 17.7729607f, 4143,
                                                maneuver_index, 0.495326996f, 114));
  //----------------------------------------------------------------
  // trace point = 449
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0123596f, 39.7152557f, 1483289728, 7.10898304),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0124054f,
                                                39.7152519f, leg_index, 17.7332115f, 4134,
                                                maneuver_index, 0.45557785f, 105));
  //----------------------------------------------------------------
  // trace point = 450
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0123749f, 39.7153702f, 1483289730, 6.44212103),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0124207f,
                                                39.7153664f, leg_index, 17.7252541f, 4133,
                                                maneuver_index, 0.447620392f, 104));
  //----------------------------------------------------------------
  // trace point = 451
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0123901f, 39.7155266f, 1483289733, 5.81396198),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0124435f,
                                                39.7155228f, leg_index, 17.7143688f, 4130,
                                                maneuver_index, 0.436735153f, 101));
  //----------------------------------------------------------------
  // trace point = 452
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0124359f, 39.7158089f, 1483289739, 5.30310678),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0124741f,
                                                39.7158051f, leg_index, 17.6947784f, 4126,
                                                maneuver_index, 0.417144775f, 97));
  //----------------------------------------------------------------
  // trace point = 453
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0124588f, 39.7160301f, 1483289744, 4.96260214),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0125046f,
                                                39.7160263f, leg_index, 17.679388f, 4122,
                                                maneuver_index, 0.401754379f, 93));
  //----------------------------------------------------------------
  // trace point = 454
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0124741f, 39.7161636f, 1483289747, 4.9906702),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0125275f,
                                                39.7161598f, leg_index, 17.6700764f, 4120,
                                                maneuver_index, 0.392442703f, 91));
  //----------------------------------------------------------------
  // trace point = 455
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0125046f, 39.7163734f, 1483289752, 4.69657421),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0125504f,
                                                39.7163696f, leg_index, 17.6555195f, 4117,
                                                maneuver_index, 0.377885818f, 88));
  //----------------------------------------------------------------
  // trace point = 456
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0125198f, 39.7165337f, 1483289756, 4.39021683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0125732f,
                                                39.7165298f, leg_index, 17.6443672f, 4114,
                                                maneuver_index, 0.366733551f, 85));
  //----------------------------------------------------------------
  // trace point = 457
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0125504f, 39.716713f, 1483289761, 4.08030415),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0125961f,
                                                39.7167091f, leg_index, 17.6319065f, 4111,
                                                maneuver_index, 0.354272842f, 82));
  //----------------------------------------------------------------
  // trace point = 458
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0125732f, 39.7168884f, 1483289766, 3.87174296),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0126114f,
                                                39.7168846f, leg_index, 17.6197453f, 4109,
                                                maneuver_index, 0.342111588f, 80));
  //----------------------------------------------------------------
  // trace point = 459
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0125885f, 39.71698f, 1483289769, 3.47079611),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0126266f,
                                                39.7169762f, leg_index, 17.6133652f, 4107,
                                                maneuver_index, 0.335731506f, 78));
  //----------------------------------------------------------------
  // trace point = 460
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0125961f, 39.7170677f, 1483289772, 3.27147889),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0126343f,
                                                39.7170639f, leg_index, 17.6072845f, 4106,
                                                maneuver_index, 0.329650879f, 77));
  //----------------------------------------------------------------
  // trace point = 461
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0126114f, 39.717186f, 1483289776, 3.28412199),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0126495f,
                                                39.7171822f, leg_index, 17.5990639f, 4104,
                                                maneuver_index, 0.321430206f, 75));
  //----------------------------------------------------------------
  // trace point = 462
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.012619f, 39.7172699f, 1483289779, 3.15727901),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0126572f,
                                                39.7172661f, leg_index, 17.5932465f, 4103,
                                                maneuver_index, 0.315612793f, 74));
  //----------------------------------------------------------------
  // trace point = 463
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.012619f, 39.717392f, 1483289783, 3.36850309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0126724f,
                                                39.7173882f, leg_index, 17.5847626f, 4101,
                                                maneuver_index, 0.307128906f, 72));
  //----------------------------------------------------------------
  // trace point = 464
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0126724f, 39.7177429f, 1483289794, 3.58176589),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0127106f,
                                                39.7177391f, leg_index, 17.5604038f, 4095,
                                                maneuver_index, 0.282770157f, 66));
  //----------------------------------------------------------------
  // trace point = 465
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0126953f, 39.7179718f, 1483289801, 3.67252207),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0127411f,
                                                39.717968f, leg_index, 17.5444889f, 4091,
                                                maneuver_index, 0.26685524f, 62));
  //----------------------------------------------------------------
  // trace point = 466
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0127182f, 39.7181396f, 1483289806, 3.733814),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0127563f,
                                                39.7181358f, leg_index, 17.5328541f, 4089,
                                                maneuver_index, 0.255220413f, 60));
  //----------------------------------------------------------------
  // trace point = 467
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0127411f, 39.718483f, 1483289816, 3.83636308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0128021f,
                                                39.7184792f, leg_index, 17.5084572f, 4083,
                                                maneuver_index, 0.230823517f, 54));
  //----------------------------------------------------------------
  // trace point = 468
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0127945f, 39.718998f, 1483289829, 4.41370487),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0128555f,
                                                39.7189941f, leg_index, 17.4732494f, 4075,
                                                maneuver_index, 0.195615768f, 46));
  //----------------------------------------------------------------
  // trace point = 469
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0128479f, 39.7195511f, 1483289841, 5.154212),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0129089f,
                                                39.7195473f, leg_index, 17.4348831f, 4066,
                                                maneuver_index, 0.157249451f, 37));
  //----------------------------------------------------------------
  // trace point = 470
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0129089f, 39.7201118f, 1483289852, 5.69583893),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0129623f,
                                                39.720108f, leg_index, 17.3959904f, 4057,
                                                maneuver_index, 0.118356705f, 28));
  //----------------------------------------------------------------
  // trace point = 471 | Alert | In 500 feet Continue on Stoney Lane.
  maneuver_index = 4;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.01297f, 39.7206535f, 1483289862, 6.05835104),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0130157f, 39.7206497f, leg_index, 17.3584137f,
                                                4048, maneuver_index, 0.0807800293f, 19,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 472
  maneuver_index = 4;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0130463f, 39.7212524f, 1483289872, 6.68699217),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0130768f,
                                                39.7212486f, leg_index, 17.3168583f, 4038,
                                                maneuver_index, 0.0392246246f, 9));
  //----------------------------------------------------------------
  // trace point = 473 | Pre | Continue on Stoney Lane for 1.8 miles.
  maneuver_index = 4;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0130768f, 39.7215157f, 1483289876, 7.35326385),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0131073f, 39.7215118f, leg_index, 17.2985802f,
                                                4034, maneuver_index, 0.0209465027f, 5,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 474
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0131226f, 39.7218552f, 1483289881, 7.60884809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0131531f,
                                                39.7218513f, leg_index, 17.2749672f, 4028,
                                                maneuver_index, 1.79487991f, 457));
  //----------------------------------------------------------------
  // trace point = 475
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0131683f, 39.7221642f, 1483289885, 8.64854908),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0131912f,
                                                39.7221642f, leg_index, 17.2532368f, 4023,
                                                maneuver_index, 1.77314949f, 452));
  //----------------------------------------------------------------
  // trace point = 476
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0131989f, 39.7224121f, 1483289888, 9.25048256),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0132217f,
                                                39.7224121f, leg_index, 17.2360077f, 4018,
                                                maneuver_index, 1.75592041f, 447));
  //----------------------------------------------------------------
  // trace point = 477
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0132217f, 39.72258f, 1483289890, 9.33069897),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0132446f,
                                                39.72258f, leg_index, 17.2243347f, 4015,
                                                maneuver_index, 1.74424744f, 444));
  //----------------------------------------------------------------
  // trace point = 478
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.013237f, 39.722744f, 1483289892, 9.11285114),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0132599f,
                                                39.722744f, leg_index, 17.2129631f, 4012,
                                                maneuver_index, 1.73287582f, 441));
  //----------------------------------------------------------------
  // trace point = 479
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0132599f, 39.7229004f, 1483289894, 8.83177567),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0132828f,
                                                39.7229004f, leg_index, 17.2020779f, 4010,
                                                maneuver_index, 1.72199059f, 439));
  //----------------------------------------------------------------
  // trace point = 480
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0132904f, 39.7231979f, 1483289898, 8.32902527),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0133209f,
                                                39.7231941f, leg_index, 17.1816597f, 4004,
                                                maneuver_index, 1.70157242f, 433));
  //----------------------------------------------------------------
  // trace point = 481
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0133438f, 39.7236595f, 1483289904, 8.58158207),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0133743f,
                                                39.7236557f, leg_index, 17.1496048f, 3996,
                                                maneuver_index, 1.66951752f, 425));
  //----------------------------------------------------------------
  // trace point = 482
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0134125f, 39.7242432f, 1483289911, 9.29381943),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.013443f,
                                                39.7242393f, leg_index, 17.1090679f, 3986,
                                                maneuver_index, 1.62898064f, 415));
  //----------------------------------------------------------------
  // trace point = 483
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0134354f, 39.7244873f, 1483289914, 9.19764042),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0134735f,
                                                39.7244835f, leg_index, 17.092104f, 3982,
                                                maneuver_index, 1.61201668f, 411));
  //----------------------------------------------------------------
  // trace point = 484
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0134888f, 39.724884f, 1483289919, 8.83098984),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0135193f,
                                                39.724884f, leg_index, 17.064291f, 3975,
                                                maneuver_index, 1.58420372f, 404));
  //----------------------------------------------------------------
  // trace point = 485
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0135574f, 39.7254829f, 1483289927, 8.36769485),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0135803f,
                                                39.7254829f, leg_index, 17.0227375f, 3964,
                                                maneuver_index, 1.54265022f, 393));
  //----------------------------------------------------------------
  // trace point = 486
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0136185f, 39.7260323f, 1483289935, 7.66624022),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0136414f,
                                                39.7260323f, leg_index, 16.984602f, 3954,
                                                maneuver_index, 1.50451469f, 383));
  //----------------------------------------------------------------
  // trace point = 487
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0136261f, 39.726162f, 1483289937, 7.26054382),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0136566f,
                                                39.7261581f, leg_index, 16.9758568f, 3952,
                                                maneuver_index, 1.4957695f, 381));
  //----------------------------------------------------------------
  // trace point = 488
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0136948f, 39.7267075f, 1483289946, 6.79482079),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0137177f,
                                                39.7267075f, leg_index, 16.9377213f, 3942,
                                                maneuver_index, 1.45763397f, 371));
  //----------------------------------------------------------------
  // trace point = 489
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0137253f, 39.7270126f, 1483289952, 5.62404585),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0137558f,
                                                39.7270088f, leg_index, 16.9167786f, 3937,
                                                maneuver_index, 1.43669128f, 366));
  //----------------------------------------------------------------
  // trace point = 490
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0137558f, 39.727272f, 1483289958, 4.84280205),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0137863f,
                                                39.7272682f, leg_index, 16.8987617f, 3932,
                                                maneuver_index, 1.41867447f, 361));
  //----------------------------------------------------------------
  // trace point = 491
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0137787f, 39.727417f, 1483289962, 4.07756805),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0138016f,
                                                39.727417f, leg_index, 16.8884392f, 3930,
                                                maneuver_index, 1.4083519f, 359));
  //----------------------------------------------------------------
  // trace point = 492
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0137863f, 39.7275162f, 1483289965, 3.76088095),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0138092f,
                                                39.7275162f, leg_index, 16.8815708f, 3928,
                                                maneuver_index, 1.40148354f, 357));
  //----------------------------------------------------------------
  // trace point = 493
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0138016f, 39.7276497f, 1483289969, 3.69598389),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0138245f,
                                                39.7276497f, leg_index, 16.8722992f, 3926,
                                                maneuver_index, 1.39221191f, 355));
  //----------------------------------------------------------------
  // trace point = 494
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0138245f, 39.7277832f, 1483289973, 3.78134108),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0138397f,
                                                39.7277832f, leg_index, 16.8630295f, 3923,
                                                maneuver_index, 1.3829422f, 352));
  //----------------------------------------------------------------
  // trace point = 495
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0138474f, 39.7280083f, 1483289979, 4.18850422),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0138702f,
                                                39.7280083f, leg_index, 16.8473778f, 3919,
                                                maneuver_index, 1.3672905f, 348));
  //----------------------------------------------------------------
  // trace point = 496
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0139084f, 39.7285614f, 1483289992, 4.75181818),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0139313f,
                                                39.7285614f, leg_index, 16.808979f, 3909,
                                                maneuver_index, 1.32889175f, 338));
  //----------------------------------------------------------------
  // trace point = 497
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.013916f, 39.7286072f, 1483289993, 4.79897308),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0139313f,
                                                39.7286072f, leg_index, 16.8058243f, 3908,
                                                maneuver_index, 1.325737f, 337));
  //----------------------------------------------------------------
  // trace point = 498
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0139465f, 39.7290115f, 1483290003, 4.50845003),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0139771f,
                                                39.7290115f, leg_index, 16.7777481f, 3901,
                                                maneuver_index, 1.29766083f, 330));
  //----------------------------------------------------------------
  // trace point = 499
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0139694f, 39.7292137f, 1483290008, 4.51451683),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0139999f,
                                                39.7292137f, leg_index, 16.7637119f, 3897,
                                                maneuver_index, 1.28362465f, 326));
  //----------------------------------------------------------------
  // trace point = 500
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0140305f, 39.7297935f, 1483290022, 4.62542582),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.014061f,
                                                39.7297897f, leg_index, 16.7237339f, 3887,
                                                maneuver_index, 1.24364662f, 316));
  //----------------------------------------------------------------
  // trace point = 501
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0140839f, 39.7302704f, 1483290033, 4.84547901),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0141144f,
                                                39.7302666f, leg_index, 16.69063f, 3879,
                                                maneuver_index, 1.21054268f, 308));
  //----------------------------------------------------------------
  // trace point = 502
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0141373f, 39.7308197f, 1483290046, 4.73064518),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0141754f,
                                                39.7308159f, leg_index, 16.6524944f, 3869,
                                                maneuver_index, 1.17240715f, 298));
  //----------------------------------------------------------------
  // trace point = 503
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0142136f, 39.7314148f, 1483290057, 6.05967093),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0142441f,
                                                39.731411f, leg_index, 16.6111698f, 3859,
                                                maneuver_index, 1.13108253f, 288));
  //----------------------------------------------------------------
  // trace point = 504
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0142593f, 39.7318687f, 1483290064, 7.24571419),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0142975f,
                                                39.7318649f, leg_index, 16.5796413f, 3851,
                                                maneuver_index, 1.09955406f, 280));
  //----------------------------------------------------------------
  // trace point = 505
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0142822f, 39.7320595f, 1483290067, 7.00171423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0143204f,
                                                39.7320557f, leg_index, 16.566391f, 3847,
                                                maneuver_index, 1.08630371f, 276));
  //----------------------------------------------------------------
  // trace point = 506
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0143127f, 39.7323036f, 1483290071, 6.90005112),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0143509f,
                                                39.7322998f, leg_index, 16.5494289f, 3843,
                                                maneuver_index, 1.06934166f, 272));
  //----------------------------------------------------------------
  // trace point = 507
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0143433f, 39.7326088f, 1483290076, 6.81539583),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0143814f,
                                                39.7326088f, leg_index, 16.5279942f, 3837,
                                                maneuver_index, 1.04790688f, 266));
  //----------------------------------------------------------------
  // trace point = 508
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0143814f, 39.7331734f, 1483290085, 6.98587799),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0144119f,
                                                39.7331734f, leg_index, 16.4889011f, 3827,
                                                maneuver_index, 1.00881386f, 256));
  //----------------------------------------------------------------
  // trace point = 509
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0144958f, 39.7337227f, 1483290094, 6.87802982),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0145493f,
                                                39.7337036f, leg_index, 16.4508991f, 3817,
                                                maneuver_index, 0.970811844f, 246));
  //----------------------------------------------------------------
  // trace point = 510
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0147018f, 39.7340584f, 1483290101, 5.92184782),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0147629f,
                                                39.7340355f, leg_index, 16.4252853f, 3811,
                                                maneuver_index, 0.945198059f, 240));
  //----------------------------------------------------------------
  // trace point = 511
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0146866f, 39.7343712f, 1483290106, 6.90724277),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0147095f,
                                                39.7343788f, leg_index, 16.3999863f, 3804,
                                                maneuver_index, 0.919898987f, 233));
  //----------------------------------------------------------------
  // trace point = 512
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0145416f, 39.7348366f, 1483290112, 8.90462971),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0145721f,
                                                39.7348366f, leg_index, 16.3673725f, 3797,
                                                maneuver_index, 0.887285233f, 226));
  //----------------------------------------------------------------
  // trace point = 513
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0145569f, 39.7352333f, 1483290116, 11.0247021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0146027f,
                                                39.7352295f, leg_index, 16.340126f, 3790,
                                                maneuver_index, 0.860038757f, 219));
  //----------------------------------------------------------------
  // trace point = 514
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.014595f, 39.7354279f, 1483290118, 10.8584423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0146408f,
                                                39.735424f, leg_index, 16.326519f, 3787,
                                                maneuver_index, 0.846431732f, 216));
  //----------------------------------------------------------------
  // trace point = 515
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0146179f, 39.7355232f, 1483290119, 11.0298929),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0146561f,
                                                39.7355194f, leg_index, 16.3198738f, 3785,
                                                maneuver_index, 0.83978653f, 214));
  //----------------------------------------------------------------
  // trace point = 516
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0146255f, 39.7356224f, 1483290120, 10.864296),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0146713f,
                                                39.7356186f, leg_index, 16.3129654f, 3783,
                                                maneuver_index, 0.832878113f, 212));
  //----------------------------------------------------------------
  // trace point = 517
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0146408f, 39.735714f, 1483290121, 10.4123545),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0146866f,
                                                39.7357101f, leg_index, 16.3065758f, 3782,
                                                maneuver_index, 0.826488495f, 211));
  //----------------------------------------------------------------
  // trace point = 518
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0146484f, 39.7358093f, 1483290122, 10.8254414),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0146942f,
                                                39.7358093f, leg_index, 16.2997017f, 3780,
                                                maneuver_index, 0.81961441f, 209));
  //----------------------------------------------------------------
  // trace point = 519
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0146332f, 39.7360764f, 1483290125, 9.80674839),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0146637f,
                                                39.7360802f, leg_index, 16.2808762f, 3775,
                                                maneuver_index, 0.800788879f, 204));
  //----------------------------------------------------------------
  // trace point = 520
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0145264f, 39.7366219f, 1483290133, 7.6914978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0145493f,
                                                39.7366257f, leg_index, 16.2426224f, 3765,
                                                maneuver_index, 0.762535095f, 194));
  //----------------------------------------------------------------
  // trace point = 521
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0144653f, 39.7368698f, 1483290138, 5.61002922),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0144882f,
                                                39.7368736f, leg_index, 16.2251682f, 3760,
                                                maneuver_index, 0.745080948f, 189));
  //----------------------------------------------------------------
  // trace point = 522
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0144424f, 39.7369576f, 1483290140, 5.02002621),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0144653f,
                                                39.7369614f, leg_index, 16.2189789f, 3759,
                                                maneuver_index, 0.738891602f, 188));
  //----------------------------------------------------------------
  // trace point = 523
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0144272f, 39.7370415f, 1483290142, 4.66022301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0144424f,
                                                39.7370453f, leg_index, 16.2130489f, 3757,
                                                maneuver_index, 0.732961655f, 186));
  //----------------------------------------------------------------
  // trace point = 524
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.014389f, 39.7371597f, 1483290145, 4.48475981),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0144043f,
                                                39.7371635f, leg_index, 16.2046185f, 3755,
                                                maneuver_index, 0.724531174f, 184));
  //----------------------------------------------------------------
  // trace point = 525
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0143356f, 39.7373085f, 1483290149, 4.35269022),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0143509f,
                                                39.7373123f, leg_index, 16.193943f, 3753,
                                                maneuver_index, 0.713855743f, 182));
  //----------------------------------------------------------------
  // trace point = 526
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0142441f, 39.7375717f, 1483290156, 4.35610914),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0142517f,
                                                39.7375755f, leg_index, 16.174984f, 3747,
                                                maneuver_index, 0.694896698f, 176));
  //----------------------------------------------------------------
  // trace point = 527
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0141525f, 39.7378311f, 1483290163, 4.26971292),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0141525f,
                                                39.7378311f, leg_index, 16.1565342f, 3743,
                                                maneuver_index, 0.676446915f, 172));
  //----------------------------------------------------------------
  // trace point = 528
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0140305f, 39.7381401f, 1483290171, 4.47528791),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0140305f,
                                                39.7381401f, leg_index, 16.1336918f, 3737,
                                                maneuver_index, 0.653604507f, 166));
  //----------------------------------------------------------------
  // trace point = 529
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0139618f, 39.7383347f, 1483290176, 4.45845699),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0139694f,
                                                39.7383347f, leg_index, 16.1203594f, 3734,
                                                maneuver_index, 0.640272141f, 163));
  //----------------------------------------------------------------
  // trace point = 530
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0138855f, 39.7385406f, 1483290181, 4.74912405),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0139008f,
                                                39.7385445f, leg_index, 16.1053944f, 3730,
                                                maneuver_index, 0.625307083f, 159));
  //----------------------------------------------------------------
  // trace point = 531
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0137024f, 39.7390785f, 1483290193, 5.18394423),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0137253f,
                                                39.7390823f, leg_index, 16.06703f, 3721,
                                                maneuver_index, 0.586942673f, 150));
  //----------------------------------------------------------------
  // trace point = 532
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0136261f, 39.7395859f, 1483290204, 5.15125799),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0136642f,
                                                39.7395744f, leg_index, 16.0325336f, 3712,
                                                maneuver_index, 0.552446365f, 141));
  //----------------------------------------------------------------
  // trace point = 533
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0139694f, 39.7400627f, 1483290216, 5.05208588),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0140152f,
                                                39.7400398f, leg_index, 15.9952049f, 3702,
                                                maneuver_index, 0.515117645f, 131));
  //----------------------------------------------------------------
  // trace point = 534
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0143738f, 39.7405281f, 1483290227, 5.66393518),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0144196f,
                                                39.7405052f, leg_index, 15.9564905f, 3693,
                                                maneuver_index, 0.476403236f, 122));
  //----------------------------------------------------------------
  // trace point = 535
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0147705f, 39.7409897f, 1483290237, 6.15026379),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0148163f,
                                                39.7409668f, leg_index, 15.9182196f, 3683,
                                                maneuver_index, 0.438132286f, 112));
  //----------------------------------------------------------------
  // trace point = 536
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0151443f, 39.7414093f, 1483290246, 6.30139685),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0151749f,
                                                39.7413864f, leg_index, 15.8835192f, 3674,
                                                maneuver_index, 0.403431892f, 103));
  //----------------------------------------------------------------
  // trace point = 537
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0152588f, 39.7415428f, 1483290249, 5.91066599),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0152969f,
                                                39.7415314f, leg_index, 15.8713827f, 3671,
                                                maneuver_index, 0.391295433f, 100));
  //----------------------------------------------------------------
  // trace point = 538
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0152359f, 39.7419434f, 1483290257, 5.58359098),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0152283f,
                                                39.7419395f, leg_index, 15.8416929f, 3663,
                                                maneuver_index, 0.361605644f, 92));
  //----------------------------------------------------------------
  // trace point = 539
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0150681f, 39.7422142f, 1483290263, 5.6114192),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0150681f,
                                                39.7422142f, leg_index, 15.8208694f, 3658,
                                                maneuver_index, 0.340782166f, 87));
  //----------------------------------------------------------------
  // trace point = 540
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0149078f, 39.7424736f, 1483290269, 5.27411604),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0149078f,
                                                39.7424736f, leg_index, 15.8010063f, 3653,
                                                maneuver_index, 0.320919037f, 82));
  //----------------------------------------------------------------
  // trace point = 541
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0147705f, 39.7426987f, 1483290275, 4.63220978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0147705f,
                                                39.7426987f, leg_index, 15.7838097f, 3648,
                                                maneuver_index, 0.303722382f, 77));
  //----------------------------------------------------------------
  // trace point = 542
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0146713f, 39.7428551f, 1483290280, 3.88266611),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0146713f,
                                                39.7428551f, leg_index, 15.7717743f, 3645,
                                                maneuver_index, 0.291687012f, 74));
  //----------------------------------------------------------------
  // trace point = 543
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0145721f, 39.7430153f, 1483290285, 3.93965507),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0145721f,
                                                39.7430153f, leg_index, 15.7595005f, 3642,
                                                maneuver_index, 0.279413223f, 71));
  //----------------------------------------------------------------
  // trace point = 544
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0144958f, 39.7431412f, 1483290289, 3.95222497),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0144958f,
                                                39.7431412f, leg_index, 15.7498941f, 3640,
                                                maneuver_index, 0.269806862f, 69));
  //----------------------------------------------------------------
  // trace point = 545
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0143661f, 39.7433434f, 1483290295, 4.11689901),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0143661f,
                                                39.7433434f, leg_index, 15.7343025f, 3636,
                                                maneuver_index, 0.25421524f, 65));
  //----------------------------------------------------------------
  // trace point = 546
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.014267f, 39.7435112f, 1483290300, 4.12784719),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.014267f,
                                                39.7435112f, leg_index, 15.7215519f, 3633,
                                                maneuver_index, 0.241464615f, 62));
  //----------------------------------------------------------------
  // trace point = 547
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0140762f, 39.7438202f, 1483290309, 4.223557),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0140762f,
                                                39.7438202f, leg_index, 15.6978931f, 3627,
                                                maneuver_index, 0.217805862f, 56));
  //----------------------------------------------------------------
  // trace point = 548
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0139465f, 39.7440224f, 1483290315, 4.18562222),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0139465f,
                                                39.7440224f, leg_index, 15.6823015f, 3623,
                                                maneuver_index, 0.202214241f, 52));
  //----------------------------------------------------------------
  // trace point = 549
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0138016f, 39.7442589f, 1483290322, 4.17564297),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0138016f,
                                                39.7442589f, leg_index, 15.664216f, 3618,
                                                maneuver_index, 0.184128761f, 47));
  //----------------------------------------------------------------
  // trace point = 550
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0136795f, 39.7444611f, 1483290328, 4.14600611),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0136795f,
                                                39.7444611f, leg_index, 15.648798f, 3614,
                                                maneuver_index, 0.168710709f, 43));
  //----------------------------------------------------------------
  // trace point = 551
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0135727f, 39.7446404f, 1483290334, 3.67705989),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0135727f,
                                                39.7446404f, leg_index, 15.6351595f, 3610,
                                                maneuver_index, 0.155072212f, 39));
  //----------------------------------------------------------------
  // trace point = 552
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0134888f, 39.7447739f, 1483290339, 3.26707602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0134888f,
                                                39.7447739f, leg_index, 15.6249027f, 3608,
                                                maneuver_index, 0.144815445f, 37));
  //----------------------------------------------------------------
  // trace point = 553
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0134354f, 39.7448502f, 1483290342, 3.110533),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0134354f,
                                                39.7448502f, leg_index, 15.6175499f, 3606,
                                                maneuver_index, 0.137462616f, 35));
  //----------------------------------------------------------------
  // trace point = 554
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0133896f, 39.7449226f, 1483290345, 3.09433794),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0133896f,
                                                39.7449226f, leg_index, 15.6133385f, 3604,
                                                maneuver_index, 0.13325119f, 33));
  //----------------------------------------------------------------
  // trace point = 555
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0133209f, 39.7450218f, 1483290349, 3.1112051),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0133286f,
                                                39.7450256f, leg_index, 15.6055155f, 3602,
                                                maneuver_index, 0.1254282f, 31));
  //----------------------------------------------------------------
  // trace point = 556 | Alert | In 500 feet Turn right onto East Christine Road.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0132446f, 39.7451515f, 1483290354, 3.216748),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0132446f, 39.7451515f, leg_index, 15.5957308f,
                                                3600, maneuver_index, 0.115643501f, 29,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 557
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0131683f, 39.7452698f, 1483290358, 3.61475301),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0131683f,
                                                39.7452698f, leg_index, 15.5866032f, 3598,
                                                maneuver_index, 0.106515884f, 27));
  //----------------------------------------------------------------
  // trace point = 558
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0130234f, 39.7454681f, 1483290364, 4.23442602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0130234f,
                                                39.7454681f, leg_index, 15.5708647f, 3594,
                                                maneuver_index, 0.0907773972f, 23));
  //----------------------------------------------------------------
  // trace point = 559
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0126648f, 39.7459297f, 1483290377, 4.60507917),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0126724f,
                                                39.7459335f, leg_index, 15.5336533f, 3585,
                                                maneuver_index, 0.053565979f, 14));
  //----------------------------------------------------------------
  // trace point = 560
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0124435f, 39.746212f, 1483290386, 4.08214712),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0124512f,
                                                39.7462158f, leg_index, 15.5108557f, 3579,
                                                maneuver_index, 0.0307683945f, 8));
  //----------------------------------------------------------------
  // trace point = 561 | Pre | Turn right onto East Christine Road, Pennsylvania 2 72.
  maneuver_index = 5;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0123367f, 39.7463608f, 1483290391, 3.77297711),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0123367f, 39.7463608f, leg_index, 15.4991264f,
                                                3576, maneuver_index, 0.0190391541f, 5,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 562
  maneuver_index = 5;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0122452f, 39.7464905f, 1483290396, 3.26182389),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0122375f,
                                                39.7464867f, leg_index, 15.488946f, 3573,
                                                maneuver_index, 0.00885868073f, 2));
  //----------------------------------------------------------------
  // trace point = 563
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.012085f, 39.7465858f, 1483290401, 3.43521404),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0121002f,
                                                39.7465706f, leg_index, 15.4767008f, 3570,
                                                maneuver_index, 1.48844814f, 333));
  //----------------------------------------------------------------
  // trace point = 564
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0114899f, 39.7463074f, 1483290410, 6.6488328),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0115051f,
                                                39.7462921f, leg_index, 15.4396505f, 3562,
                                                maneuver_index, 1.4513979f, 325));
  //----------------------------------------------------------------
  // trace point = 565
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0112381f, 39.7461853f, 1483290413, 8.50070763),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0112457f,
                                                39.7461739f, leg_index, 15.4236126f, 3558,
                                                maneuver_index, 1.43535995f, 321));
  //----------------------------------------------------------------
  // trace point = 566
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0108719f, 39.7460136f, 1483290417, 9.21143913),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0108795f,
                                                39.7460022f, leg_index, 15.4008026f, 3553,
                                                maneuver_index, 1.41254997f, 316));
  //----------------------------------------------------------------
  // trace point = 567
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0104752f, 39.7458229f, 1483290421, 10.027976),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0104828f,
                                                39.7458153f, leg_index, 15.3760557f, 3548,
                                                maneuver_index, 1.38780308f, 311));
  //----------------------------------------------------------------
  // trace point = 568
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0099411f, 39.745575f, 1483290426, 10.6178169),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0099487f,
                                                39.7455673f, leg_index, 15.3428755f, 3540,
                                                maneuver_index, 1.35462284f, 303));
  //----------------------------------------------------------------
  // trace point = 569
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0097275f, 39.745472f, 1483290428, 10.6732769),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0097351f,
                                                39.7454643f, leg_index, 15.3294668f, 3537,
                                                maneuver_index, 1.34121418f, 300));
  //----------------------------------------------------------------
  // trace point = 570
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0095215f, 39.7453766f, 1483290430, 10.5862913),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0095291f,
                                                39.745369f, leg_index, 15.316678f, 3534,
                                                maneuver_index, 1.32842541f, 297));
  //----------------------------------------------------------------
  // trace point = 571
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.009201f, 39.7452278f, 1483290433, 10.5618629),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0092087f,
                                                39.7452202f, leg_index, 15.2967701f, 3530,
                                                maneuver_index, 1.30851746f, 293));
  //----------------------------------------------------------------
  // trace point = 572
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0085907f, 39.7449379f, 1483290439, 10.2650318),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0085983f,
                                                39.7449303f, leg_index, 15.2586136f, 3522,
                                                maneuver_index, 1.27036095f, 285));
  //----------------------------------------------------------------
  // trace point = 573
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0078964f, 39.744606f, 1483290446, 9.98598385),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0079041f,
                                                39.7446022f, leg_index, 15.2152719f, 3512,
                                                maneuver_index, 1.22701931f, 275));
  //----------------------------------------------------------------
  // trace point = 574
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0072556f, 39.7442932f, 1483290453, 9.27460766),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0072556f,
                                                39.7442932f, leg_index, 15.174696f, 3503,
                                                maneuver_index, 1.18644333f, 266));
  //----------------------------------------------------------------
  // trace point = 575
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0066147f, 39.7439919f, 1483290460, 9.19026947),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0066147f,
                                                39.743988f, leg_index, 15.1346045f, 3494,
                                                maneuver_index, 1.14635181f, 257));
  //----------------------------------------------------------------
  // trace point = 576
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0059433f, 39.7436714f, 1483290468, 8.42427635),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0059433f,
                                                39.7436714f, leg_index, 15.0927134f, 3485,
                                                maneuver_index, 1.10446072f, 248));
  //----------------------------------------------------------------
  // trace point = 577
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0052948f, 39.7433586f, 1483290476, 8.23875713),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0052872f,
                                                39.7433662f, leg_index, 15.0519285f, 3475,
                                                maneuver_index, 1.06367588f, 238));
  //----------------------------------------------------------------
  // trace point = 578
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0049515f, 39.7431946f, 1483290480, 8.58868599),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0049438f,
                                                39.743206f, leg_index, 15.0305681f, 3471,
                                                maneuver_index, 1.04231548f, 234));
  //----------------------------------------------------------------
  // trace point = 579
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0043335f, 39.7429047f, 1483290487, 8.88879204),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0043259f,
                                                39.7429199f, leg_index, 14.9922018f, 3462,
                                                maneuver_index, 1.00394917f, 225));
  //----------------------------------------------------------------
  // trace point = 580
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0041504f, 39.7428207f, 1483290489, 9.04041481),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0041428f,
                                                39.742836f, leg_index, 14.9808645f, 3459,
                                                maneuver_index, 0.992611885f, 222));
  //----------------------------------------------------------------
  // trace point = 581
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0035095f, 39.7425232f, 1483290496, 9.10338402),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0034943f,
                                                39.7425423f, leg_index, 14.9408331f, 3450,
                                                maneuver_index, 0.952580452f, 213));
  //----------------------------------------------------------------
  // trace point = 582
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0028687f, 39.7422333f, 1483290503, 9.18420696),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0028534f,
                                                39.7422523f, leg_index, 14.9012861f, 3441,
                                                maneuver_index, 0.913033485f, 204));
  //----------------------------------------------------------------
  // trace point = 583
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0023727f, 39.7420273f, 1483290508, 9.64622307),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0023651f,
                                                39.7420387f, leg_index, 14.8714037f, 3435,
                                                maneuver_index, 0.883151054f, 198));
  //----------------------------------------------------------------
  // trace point = 584
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0017319f, 39.741745f, 1483290514, 10.5355711),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0017242f,
                                                39.7417603f, leg_index, 14.8322515f, 3426,
                                                maneuver_index, 0.843998909f, 189));
  //----------------------------------------------------------------
  // trace point = 585
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0015182f, 39.7416496f, 1483290516, 10.5479774),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.001503f,
                                                39.7416687f, leg_index, 14.8188896f, 3423,
                                                maneuver_index, 0.830636978f, 186));
  //----------------------------------------------------------------
  // trace point = 586
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.001297f, 39.7415581f, 1483290518, 10.5305939),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0012894f,
                                                39.7415733f, leg_index, 14.8057528f, 3420,
                                                maneuver_index, 0.817500114f, 183));
  //----------------------------------------------------------------
  // trace point = 587
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0006638f, 39.7412834f, 1483290524, 10.45928),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0006561f,
                                                39.7412987f, leg_index, 14.767086f, 3412,
                                                maneuver_index, 0.778833389f, 175));
  //----------------------------------------------------------------
  // trace point = 588
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9999924f, 39.740963f, 1483290530, 11.262023),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9999771f,
                                                39.7409744f, leg_index, 14.7244616f, 3402,
                                                maneuver_index, 0.736208916f, 165));
  //----------------------------------------------------------------
  // trace point = 589
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9995575f, 39.7406807f, 1483290534, 12.1919956),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9995499f,
                                                39.7406883f, leg_index, 14.6943188f, 3395,
                                                maneuver_index, 0.706066132f, 158));
  //----------------------------------------------------------------
  // trace point = 590
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9993362f, 39.7405357f, 1483290536, 12.3407507),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9993286f,
                                                39.7405396f, leg_index, 14.6786871f, 3392,
                                                maneuver_index, 0.690434456f, 155));
  //----------------------------------------------------------------
  // trace point = 591
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9991226f, 39.7403908f, 1483290538, 12.2030716),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.999115f,
                                                39.7403984f, leg_index, 14.6637049f, 3389,
                                                maneuver_index, 0.675452232f, 152));
  //----------------------------------------------------------------
  // trace point = 592
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.998909f, 39.7402496f, 1483290540, 12.097827),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9989014f,
                                                39.7402573f, leg_index, 14.6487246f, 3386,
                                                maneuver_index, 0.660471916f, 149));
  //----------------------------------------------------------------
  // trace point = 593
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9985886f, 39.740036f, 1483290543, 12.0331688),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9985809f,
                                                39.7400436f, leg_index, 14.6261673f, 3381,
                                                maneuver_index, 0.637914658f, 144));
  //----------------------------------------------------------------
  // trace point = 594
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.997963f, 39.7396088f, 1483290549, 11.9153118),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9979477f,
                                                39.739624f, leg_index, 14.5817051f, 3371,
                                                maneuver_index, 0.593452454f, 134));
  //----------------------------------------------------------------
  // trace point = 595
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9975357f, 39.7393265f, 1483290553, 12.0110159),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9975204f,
                                                39.7393417f, leg_index, 14.5517435f, 3364,
                                                maneuver_index, 0.563490868f, 127));
  //----------------------------------------------------------------
  // trace point = 596
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9971237f, 39.7390518f, 1483290557, 11.7886629),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9971085f,
                                                39.7390671f, leg_index, 14.5227404f, 3357,
                                                maneuver_index, 0.534487724f, 120));
  //----------------------------------------------------------------
  // trace point = 597
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9969177f, 39.7389221f, 1483290559, 11.3398209),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9969025f,
                                                39.7389336f, leg_index, 14.5084105f, 3354,
                                                maneuver_index, 0.520157814f, 117));
  //----------------------------------------------------------------
  // trace point = 598
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9967194f, 39.7387924f, 1483290561, 11.1758928),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9967041f,
                                                39.7388039f, leg_index, 14.4945612f, 3351,
                                                maneuver_index, 0.506308556f, 114));
  //----------------------------------------------------------------
  // trace point = 599
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9961548f, 39.7384262f, 1483290567, 10.4624386),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9961395f,
                                                39.7384377f, leg_index, 14.4552746f, 3342,
                                                maneuver_index, 0.467021942f, 105));
  //----------------------------------------------------------------
  // trace point = 600
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955444f, 39.7380333f, 1483290574, 9.7623682),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9955292f,
                                                39.7380447f, leg_index, 14.4129362f, 3332,
                                                maneuver_index, 0.424683571f, 95));
  //----------------------------------------------------------------
  // trace point = 601
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9949646f, 39.737648f, 1483290581, 9.41414928),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9949493f,
                                                39.7376633f, leg_index, 14.3723469f, 3323,
                                                maneuver_index, 0.384094238f, 86));
  //----------------------------------------------------------------
  // trace point = 602
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9944153f, 39.7372856f, 1483290587, 10.2759819),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9944f,
                                                39.7372971f, leg_index, 14.3336735f, 3314,
                                                maneuver_index, 0.345420837f, 77));
  //----------------------------------------------------------------
  // trace point = 603
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9943237f, 39.7372208f, 1483290588, 10.8116465),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9943085f,
                                                39.7372322f, leg_index, 14.3270473f, 3313,
                                                maneuver_index, 0.338794708f, 76));
  //----------------------------------------------------------------
  // trace point = 604
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.994133f, 39.7370911f, 1483290590, 10.8215265),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9941254f,
                                                39.7370987f, leg_index, 14.3136253f, 3310,
                                                maneuver_index, 0.325372696f, 73));
  //----------------------------------------------------------------
  // trace point = 605
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9937592f, 39.7368279f, 1483290594, 10.8867378),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9937592f,
                                                39.7368317f, leg_index, 14.2867813f, 3304,
                                                maneuver_index, 0.298528671f, 67));
  //----------------------------------------------------------------
  // trace point = 606 | Alert | In a quarter mile Turn right onto McCoury Road.
  maneuver_index = 6;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9934921f, 39.7366295f, 1483290597, 10.6873226),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -75.9934845f, 39.7366333f, leg_index, 14.2667418f,
                                                3300, maneuver_index, 0.278489113f, 63,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 607
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9929276f, 39.7362061f, 1483290604, 9.63212585),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.992897f,
                                                39.7362289f, leg_index, 14.2247849f, 3291,
                                                maneuver_index, 0.236532211f, 54));
  //----------------------------------------------------------------
  // trace point = 608
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9924545f, 39.7358894f, 1483290611, 7.61637783),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9924316f,
                                                39.7359123f, leg_index, 14.1917334f, 3283,
                                                maneuver_index, 0.203480721f, 46));
  //----------------------------------------------------------------
  // trace point = 609
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9923935f, 39.7358513f, 1483290612, 6.65077019),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9923706f,
                                                39.7358742f, leg_index, 14.1864195f, 3282,
                                                maneuver_index, 0.198166847f, 45));
  //----------------------------------------------------------------
  // trace point = 610
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9922104f, 39.7357597f, 1483290615, 6.25314283),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9921951f,
                                                39.7357788f, leg_index, 14.1761084f, 3279,
                                                maneuver_index, 0.187855721f, 42));
  //----------------------------------------------------------------
  // trace point = 611
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9920502f, 39.7356796f, 1483290618, 5.55625916),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9920349f,
                                                39.7356987f, leg_index, 14.1659336f, 3277,
                                                maneuver_index, 0.177680969f, 40));
  //----------------------------------------------------------------
  // trace point = 612
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9919434f, 39.7356339f, 1483290620, 5.19165277),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9919281f,
                                                39.7356567f, leg_index, 14.1595564f, 3276,
                                                maneuver_index, 0.171303749f, 39));
  //----------------------------------------------------------------
  // trace point = 613
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9917984f, 39.735569f, 1483290623, 4.82024717),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9917831f,
                                                39.7355919f, leg_index, 14.1506395f, 3274,
                                                maneuver_index, 0.162386894f, 37));
  //----------------------------------------------------------------
  // trace point = 614
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9916534f, 39.7355118f, 1483290626, 4.56288385),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9916382f,
                                                39.7355347f, leg_index, 14.1419754f, 3272,
                                                maneuver_index, 0.153722763f, 35));
  //----------------------------------------------------------------
  // trace point = 615
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9914322f, 39.7354202f, 1483290631, 4.30154991),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9914169f,
                                                39.7354431f, leg_index, 14.1286077f, 3268,
                                                maneuver_index, 0.14035511f, 31));
  //----------------------------------------------------------------
  // trace point = 616
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9912643f, 39.7353554f, 1483290635, 4.10096979),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9912491f,
                                                39.7353745f, leg_index, 14.118495f, 3266,
                                                maneuver_index, 0.130242348f, 29));
  //----------------------------------------------------------------
  // trace point = 617
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9911346f, 39.735302f, 1483290638, 4.11363602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9911194f,
                                                39.735321f, leg_index, 14.1106701f, 3264,
                                                maneuver_index, 0.12241745f, 27));
  //----------------------------------------------------------------
  // trace point = 618
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9909973f, 39.7352486f, 1483290641, 4.31691504),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9909821f,
                                                39.7352676f, leg_index, 14.1024847f, 3263,
                                                maneuver_index, 0.114232063f, 26));
  //----------------------------------------------------------------
  // trace point = 619
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.990448f, 39.7350311f, 1483290652, 4.82000589),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9904404f,
                                                39.7350464f, leg_index, 14.0698605f, 3255,
                                                maneuver_index, 0.0816078186f, 18));
  //----------------------------------------------------------------
  // trace point = 620
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9904022f, 39.7350121f, 1483290653, 4.80210495),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.990387f,
                                                39.7350273f, leg_index, 14.0667305f, 3255,
                                                maneuver_index, 0.0784778595f, 18));
  //----------------------------------------------------------------
  // trace point = 621
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.990097f, 39.7348938f, 1483290659, 4.82469511),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9900894f,
                                                39.7349091f, leg_index, 14.0489168f, 3251,
                                                maneuver_index, 0.0606641769f, 14));
  //----------------------------------------------------------------
  // trace point = 622
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9898529f, 39.7348022f, 1483290664, 4.66415691),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9898453f,
                                                39.7348213f, leg_index, 14.0345831f, 3247,
                                                maneuver_index, 0.046330452f, 10));
  //----------------------------------------------------------------
  // trace point = 623
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9894714f, 39.7346878f, 1483290672, 4.36982584),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9894638f,
                                                39.7347069f, leg_index, 14.0127707f, 3243,
                                                maneuver_index, 0.024518013f, 6));
  //----------------------------------------------------------------
  // trace point = 624
  maneuver_index = 6;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9894257f, 39.7346687f, 1483290673, 4.3754549),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.989418f,
                                                39.7346954f, leg_index, 14.010211f, 3242,
                                                maneuver_index, 0.0219583511f, 5));
  //----------------------------------------------------------------
  // trace point = 625 | Pre | Turn right onto McCoury Road.
  maneuver_index = 6;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.989357f, 39.7345161f, 1483290677, 4.49842405),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -75.989296f, 39.7346687f, leg_index, 14.0034618f,
                                                3241, maneuver_index, 0.015209198f, 4,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 626 | Post | Continue for 1.1 miles.
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9893265f, 39.7343407f, 1483290680, 6.62340498),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -75.9890442f, 39.7343445f, leg_index, 13.9699821f,
                                                3234, maneuver_index, 1.1235733f, 226,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 627
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.989296f, 39.7342682f, 1483290681, 8.17827606),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9890366f,
                                                39.734272f, leg_index, 13.9649601f, 3233,
                                                maneuver_index, 1.11855125f, 225));
  //----------------------------------------------------------------
  // trace point = 628
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9890366f, 39.7337914f, 1483290687, 9.58436871),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.989006f,
                                                39.7337914f, leg_index, 13.9316673f, 3226,
                                                maneuver_index, 1.08525848f, 218));
  //----------------------------------------------------------------
  // trace point = 629
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.989006f, 39.7337036f, 1483290688, 10.1734858),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9889984f,
                                                39.7337036f, leg_index, 13.9255848f, 3225,
                                                maneuver_index, 1.07917595f, 217));
  //----------------------------------------------------------------
  // trace point = 630
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9889832f, 39.7335205f, 1483290690, 10.2417898),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9889832f,
                                                39.7335205f, leg_index, 13.9128933f, 3223,
                                                maneuver_index, 1.06648445f, 215));
  //----------------------------------------------------------------
  // trace point = 631
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9889679f, 39.7333374f, 1483290692, 10.3170509),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9889679f,
                                                39.7333374f, leg_index, 13.9002028f, 3220,
                                                maneuver_index, 1.05379391f, 212));
  //----------------------------------------------------------------
  // trace point = 632
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.988945f, 39.7330627f, 1483290695, 10.2202339),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.988945f,
                                                39.7330627f, leg_index, 13.8811655f, 3216,
                                                maneuver_index, 1.03475666f, 208));
  //----------------------------------------------------------------
  // trace point = 633
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9889145f, 39.7325172f, 1483290701, 10.1224766),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9889069f,
                                                39.7325172f, leg_index, 13.8433781f, 3209,
                                                maneuver_index, 0.996969223f, 201));
  //----------------------------------------------------------------
  // trace point = 634
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9888992f, 39.7323303f, 1483290703, 10.2589111),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9888916f,
                                                39.7323303f, leg_index, 13.8304234f, 3206,
                                                maneuver_index, 0.984014511f, 198));
  //----------------------------------------------------------------
  // trace point = 635
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9888535f, 39.731781f, 1483290709, 10.2457561),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9888458f,
                                                39.731781f, leg_index, 13.7923498f, 3199,
                                                maneuver_index, 0.945940971f, 191));
  //----------------------------------------------------------------
  // trace point = 636
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9888229f, 39.7315063f, 1483290712, 10.277112),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9888306f,
                                                39.7315063f, leg_index, 13.7733421f, 3195,
                                                maneuver_index, 0.926933289f, 187));
  //----------------------------------------------------------------
  // trace point = 637
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9888153f, 39.7312202f, 1483290715, 10.5042791),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9888153f,
                                                39.7312202f, leg_index, 13.7535286f, 3190,
                                                maneuver_index, 0.907119751f, 182));
  //----------------------------------------------------------------
  // trace point = 638
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9891052f, 39.730793f, 1483290720, 10.7648058),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9891129f,
                                                39.7308006f, leg_index, 13.7192516f, 3183,
                                                maneuver_index, 0.872842789f, 175));
  //----------------------------------------------------------------
  // trace point = 639
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9892044f, 39.730732f, 1483290721, 11.1305151),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.989212f,
                                                39.7307396f, leg_index, 13.7124863f, 3182,
                                                maneuver_index, 0.866077423f, 174));
  //----------------------------------------------------------------
  // trace point = 640
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9894104f, 39.7306137f, 1483290723, 11.0294838),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.989418f,
                                                39.7306213f, leg_index, 13.6988115f, 3179,
                                                maneuver_index, 0.852402687f, 171));
  //----------------------------------------------------------------
  // trace point = 641
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9896164f, 39.7304993f, 1483290725, 10.8962154),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9896164f,
                                                39.7305031f, leg_index, 13.685462f, 3176,
                                                maneuver_index, 0.839053154f, 168));
  //----------------------------------------------------------------
  // trace point = 642
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9898148f, 39.7303848f, 1483290727, 10.656208),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9898148f,
                                                39.7303886f, leg_index, 13.6722708f, 3174,
                                                maneuver_index, 0.825861931f, 166));
  //----------------------------------------------------------------
  // trace point = 643
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9904327f, 39.7300835f, 1483290733, 10.3634024),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9904251f,
                                                39.7300758f, leg_index, 13.6332083f, 3166,
                                                maneuver_index, 0.786799431f, 158));
  //----------------------------------------------------------------
  // trace point = 644
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9911652f, 39.7298508f, 1483290740, 9.70561695),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9911652f,
                                                39.7298431f, leg_index, 13.5906591f, 3157,
                                                maneuver_index, 0.744250298f, 149));
  //----------------------------------------------------------------
  // trace point = 645
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9918518f, 39.72966f, 1483290747, 8.90060616),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9918442f,
                                                39.7296448f, leg_index, 13.5520201f, 3150,
                                                maneuver_index, 0.705611229f, 142));
  //----------------------------------------------------------------
  // trace point = 646
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9925308f, 39.7294693f, 1483290755, 7.81230021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9925232f,
                                                39.7294502f, leg_index, 13.5134726f, 3142,
                                                maneuver_index, 0.667063713f, 134));
  //----------------------------------------------------------------
  // trace point = 647
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.992981f, 39.7293396f, 1483290761, 6.79254389),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9929733f,
                                                39.7293205f, leg_index, 13.4878998f, 3137,
                                                maneuver_index, 0.641490936f, 129));
  //----------------------------------------------------------------
  // trace point = 648
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.993454f, 39.7292023f, 1483290768, 6.23302698),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9934464f,
                                                39.7291832f, leg_index, 13.4609985f, 3131,
                                                maneuver_index, 0.614589691f, 123));
  //----------------------------------------------------------------
  // trace point = 649
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9937897f, 39.7291107f, 1483290773, 5.99415493),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.993782f,
                                                39.7290916f, leg_index, 13.4415798f, 3127,
                                                maneuver_index, 0.595170975f, 119));
  //----------------------------------------------------------------
  // trace point = 650
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9944611f, 39.72892f, 1483290783, 6.14541483),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9944458f,
                                                39.7288895f, leg_index, 13.4040756f, 3120,
                                                maneuver_index, 0.557666779f, 112));
  //----------------------------------------------------------------
  // trace point = 651
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9951096f, 39.7287025f, 1483290793, 6.0935812),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9950867f,
                                                39.728672f, leg_index, 13.3667545f, 3112,
                                                maneuver_index, 0.520345688f, 104));
  //----------------------------------------------------------------
  // trace point = 652
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.995636f, 39.7283325f, 1483290801, 7.64731884),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9956131f,
                                                39.7283173f, leg_index, 13.3294544f, 3105,
                                                maneuver_index, 0.483045578f, 97));
  //----------------------------------------------------------------
  // trace point = 653
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9962311f, 39.7279434f, 1483290808, 9.51404285),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9962158f,
                                                39.7279243f, leg_index, 13.2873497f, 3097,
                                                maneuver_index, 0.440940857f, 89));
  //----------------------------------------------------------------
  // trace point = 654
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9964447f, 39.7278366f, 1483290810, 10.9747143),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9964294f,
                                                39.7278175f, leg_index, 13.2737961f, 3094,
                                                maneuver_index, 0.427387238f, 86));
  //----------------------------------------------------------------
  // trace point = 655
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9965515f, 39.7277794f, 1483290811, 11.2719259),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9965439f,
                                                39.7277641f, leg_index, 13.2666759f, 3092,
                                                maneuver_index, 0.420267105f, 84));
  //----------------------------------------------------------------
  // trace point = 656
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9966431f, 39.7277031f, 1483290812, 11.3973188),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9966354f,
                                                39.7276993f, leg_index, 13.2600136f, 3091,
                                                maneuver_index, 0.413604736f, 83));
  //----------------------------------------------------------------
  // trace point = 657
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9967422f, 39.7275047f, 1483290814, 11.8228188),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9967346f,
                                                39.7275047f, leg_index, 13.2451677f, 3089,
                                                maneuver_index, 0.398758888f, 81));
  //----------------------------------------------------------------
  // trace point = 658
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9967575f, 39.7272835f, 1483290816, 12.3068361),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9967117f,
                                                39.7272873f, leg_index, 13.2300663f, 3085,
                                                maneuver_index, 0.383657455f, 77));
  //----------------------------------------------------------------
  // trace point = 659
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9967422f, 39.7271729f, 1483290817, 12.3923006),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9967041f,
                                                39.7271767f, leg_index, 13.2224083f, 3084,
                                                maneuver_index, 0.375999451f, 76));
  //----------------------------------------------------------------
  // trace point = 660
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.996727f, 39.7270622f, 1483290818, 12.4321632),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9966888f,
                                                39.727066f, leg_index, 13.2147141f, 3082,
                                                maneuver_index, 0.368305206f, 74));
  //----------------------------------------------------------------
  // trace point = 661
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9967117f, 39.7269516f, 1483290819, 12.5734549),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9966736f,
                                                39.7269554f, leg_index, 13.2070208f, 3080,
                                                maneuver_index, 0.360611916f, 72));
  //----------------------------------------------------------------
  // trace point = 662
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9966888f, 39.7268372f, 1483290820, 12.562665),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.996666f,
                                                39.7268372f, leg_index, 13.1988373f, 3079,
                                                maneuver_index, 0.352428436f, 71));
  //----------------------------------------------------------------
  // trace point = 663
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.996666f, 39.7266121f, 1483290822, 12.6764641),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9966354f,
                                                39.7266159f, leg_index, 13.1834488f, 3076,
                                                maneuver_index, 0.337039948f, 68));
  //----------------------------------------------------------------
  // trace point = 664
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9966507f, 39.7263832f, 1483290824, 12.7622461),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9966125f,
                                                39.726387f, leg_index, 13.1675749f, 3073,
                                                maneuver_index, 0.321166039f, 65));
  //----------------------------------------------------------------
  // trace point = 665
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9966431f, 39.7262688f, 1483290825, 12.8303394),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9965973f,
                                                39.7262726f, leg_index, 13.1596146f, 3071,
                                                maneuver_index, 0.313205719f, 63));
  //----------------------------------------------------------------
  // trace point = 666
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9966125f, 39.7260284f, 1483290827, 13.3491602),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9965744f,
                                                39.7260284f, leg_index, 13.142684f, 3068,
                                                maneuver_index, 0.296275139f, 60));
  //----------------------------------------------------------------
  // trace point = 667 | Alert | In a quarter mile Continue on Chandlee Road.
  maneuver_index = 7;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9965897f, 39.7257957f, 1483290829, 13.1170454),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -75.9965591f, 39.7257996f, leg_index, 13.1268396f,
                                                3065, maneuver_index, 0.280430794f, 57,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 668
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9965668f, 39.7255669f, 1483290831, 12.7983809),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9965286f,
                                                39.7255707f, leg_index, 13.1109161f, 3062,
                                                maneuver_index, 0.264507294f, 54));
  //----------------------------------------------------------------
  // trace point = 669
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9965363f, 39.7253418f, 1483290833, 12.4186716),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9965134f,
                                                39.7253418f, leg_index, 13.0950489f, 3059,
                                                maneuver_index, 0.24864006f, 51));
  //----------------------------------------------------------------
  // trace point = 670
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9964905f, 39.7249146f, 1483290837, 11.986887),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.99646f,
                                                39.7249184f, leg_index, 13.0656223f, 3053,
                                                maneuver_index, 0.219213486f, 45));
  //----------------------------------------------------------------
  // trace point = 671
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9964066f, 39.7243042f, 1483290843, 11.3720818),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9963913f,
                                                39.7243042f, leg_index, 13.0229845f, 3044,
                                                maneuver_index, 0.176575661f, 36));
  //----------------------------------------------------------------
  // trace point = 672
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9963531f, 39.7237206f, 1483290849, 10.8285828),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.996315f,
                                                39.7237244f, leg_index, 12.9813595f, 3036,
                                                maneuver_index, 0.134950638f, 28));
  //----------------------------------------------------------------
  // trace point = 673
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9963455f, 39.7236252f, 1483290850, 10.6099548),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9963074f,
                                                39.7236252f, leg_index, 12.9758043f, 3034,
                                                maneuver_index, 0.129395485f, 26));
  //----------------------------------------------------------------
  // trace point = 674
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9962845f, 39.7230721f, 1483290856, 10.3465128),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9962692f,
                                                39.7230721f, leg_index, 12.9374943f, 3027,
                                                maneuver_index, 0.091085434f, 19));
  //----------------------------------------------------------------
  // trace point = 675
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9962234f, 39.7225227f, 1483290863, 8.74455547),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9962234f,
                                                39.7225227f, leg_index, 12.8994236f, 3019,
                                                maneuver_index, 0.0530147552f, 11));
  //----------------------------------------------------------------
  // trace point = 676
  maneuver_index = 7;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9962082f, 39.7223892f, 1483290865, 7.56268978),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9962006f,
                                                39.7223892f, leg_index, 12.89011f, 3017,
                                                maneuver_index, 0.0437011719f, 9));
  //----------------------------------------------------------------
  // trace point = 677 | Pre | Continue on Chandlee Road for 1.3 miles.
  maneuver_index = 7;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9961853f, 39.7221794f, 1483290868, 7.77838278),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -75.99617f, 39.7221794f, leg_index, 12.8755054f, 3014,
                                                maneuver_index, 0.0290966034f, 6, instruction_index));
  //----------------------------------------------------------------
  // trace point = 678
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9961243f, 39.7215767f, 1483290875, 9.58858871),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9961166f,
                                                39.7215767f, leg_index, 12.8337183f, 3005,
                                                maneuver_index, 1.23766994f, 293));
  //----------------------------------------------------------------
  // trace point = 679
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9960556f, 39.721035f, 1483290881, 10.0985508),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9960632f,
                                                39.721035f, leg_index, 12.7961416f, 2997,
                                                maneuver_index, 1.20009327f, 285));
  //----------------------------------------------------------------
  // trace point = 680
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9960098f, 39.7206268f, 1483290886, 9.18579102),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9960175f,
                                                39.7206268f, leg_index, 12.7678041f, 2990,
                                                maneuver_index, 1.17175579f, 278));
  //----------------------------------------------------------------
  // trace point = 681
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9959564f, 39.7200966f, 1483290894, 7.37326479),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9959641f,
                                                39.7200966f, leg_index, 12.7310171f, 2981,
                                                maneuver_index, 1.13496876f, 269));
  //----------------------------------------------------------------
  // trace point = 682
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9959412f, 39.7199402f, 1483290897, 5.79564619),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9959488f,
                                                39.7199402f, leg_index, 12.7201681f, 2979,
                                                maneuver_index, 1.12411976f, 267));
  //----------------------------------------------------------------
  // trace point = 683
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9959412f, 39.7198906f, 1483290898, 5.56663322),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9959412f,
                                                39.7198906f, leg_index, 12.7167168f, 2978,
                                                maneuver_index, 1.12066841f, 266));
  //----------------------------------------------------------------
  // trace point = 684
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9959488f, 39.7197914f, 1483290900, 5.51297522),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9959335f,
                                                39.7197914f, leg_index, 12.7098455f, 2976,
                                                maneuver_index, 1.11379719f, 264));
  //----------------------------------------------------------------
  // trace point = 685
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9959488f, 39.7196388f, 1483290903, 5.71618223),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9959183f,
                                                39.7196388f, leg_index, 12.6992598f, 2974,
                                                maneuver_index, 1.1032114f, 262));
  //----------------------------------------------------------------
  // trace point = 686
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9959488f, 39.7194901f, 1483290906, 5.45473003),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.995903f,
                                                39.7194939f, leg_index, 12.6891994f, 2971,
                                                maneuver_index, 1.09315109f, 259));
  //----------------------------------------------------------------
  // trace point = 687
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9959412f, 39.7193069f, 1483290910, 5.09934092),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9958878f,
                                                39.7193069f, leg_index, 12.675189f, 2968,
                                                maneuver_index, 1.07914066f, 256));
  //----------------------------------------------------------------
  // trace point = 688
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9959183f, 39.71912f, 1483290914, 5.221766),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9958725f,
                                                39.7191238f, leg_index, 12.6635523f, 2965,
                                                maneuver_index, 1.06750393f, 253));
  //----------------------------------------------------------------
  // trace point = 689
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9958878f, 39.7187843f, 1483290921, 5.37631178),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9958496f,
                                                39.7187843f, leg_index, 12.6400366f, 2959,
                                                maneuver_index, 1.04398823f, 247));
  //----------------------------------------------------------------
  // trace point = 690
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9958496f, 39.7184372f, 1483290928, 5.533916),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9958267f,
                                                39.7184372f, leg_index, 12.6159945f, 2954,
                                                maneuver_index, 1.0199461f, 242));
  //----------------------------------------------------------------
  // trace point = 691
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9958115f, 39.7178993f, 1483290939, 5.46353483),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9957886f,
                                                39.7178993f, leg_index, 12.5787334f, 2945,
                                                maneuver_index, 0.982685089f, 233));
  //----------------------------------------------------------------
  // trace point = 692
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9958115f, 39.7178497f, 1483290940, 5.34950686),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9957809f,
                                                39.7178497f, leg_index, 12.5752878f, 2944,
                                                maneuver_index, 0.979239464f, 232));
  //----------------------------------------------------------------
  // trace point = 693
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9957886f, 39.7176132f, 1483290945, 5.26543093),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9957657f,
                                                39.7176132f, leg_index, 12.5589104f, 2941,
                                                maneuver_index, 0.962862015f, 229));
  //----------------------------------------------------------------
  // trace point = 694
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9957581f, 39.7170334f, 1483290956, 5.89507818),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9957275f,
                                                39.7170334f, leg_index, 12.5187483f, 2931,
                                                maneuver_index, 0.922699928f, 219));
  //----------------------------------------------------------------
  // trace point = 695
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9957199f, 39.7164574f, 1483290965, 7.12175083),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9956894f,
                                                39.7164574f, leg_index, 12.4788523f, 2922,
                                                maneuver_index, 0.882803917f, 210));
  //----------------------------------------------------------------
  // trace point = 696
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9956741f, 39.7159576f, 1483290972, 7.95210409),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9956512f,
                                                39.7159576f, leg_index, 12.4442272f, 2913,
                                                maneuver_index, 0.848178864f, 201));
  //----------------------------------------------------------------
  // trace point = 697
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9956512f, 39.7156601f, 1483290976, 8.34118176),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9956284f,
                                                39.7156601f, leg_index, 12.4236107f, 2908,
                                                maneuver_index, 0.827562332f, 196));
  //----------------------------------------------------------------
  // trace point = 698
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9956207f, 39.7153435f, 1483290980, 8.84388638),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9956055f,
                                                39.7153435f, leg_index, 12.4016762f, 2903,
                                                maneuver_index, 0.805627823f, 191));
  //----------------------------------------------------------------
  // trace point = 699
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955826f, 39.7149124f, 1483290985, 9.62016201),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9955673f,
                                                39.7149124f, leg_index, 12.3717842f, 2896,
                                                maneuver_index, 0.775735855f, 184));
  //----------------------------------------------------------------
  // trace point = 700
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955444f, 39.7144203f, 1483290990, 10.945303),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9955292f,
                                                39.7144203f, leg_index, 12.3376818f, 2888,
                                                maneuver_index, 0.741633415f, 176));
  //----------------------------------------------------------------
  // trace point = 701
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955215f, 39.7140198f, 1483290994, 11.1736507),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9955063f,
                                                39.7140198f, leg_index, 12.3099499f, 2882,
                                                maneuver_index, 0.71390152f, 170));
  //----------------------------------------------------------------
  // trace point = 702
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955063f, 39.71381f, 1483290996, 11.5927343),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954987f,
                                                39.71381f, leg_index, 12.2954321f, 2878,
                                                maneuver_index, 0.699383736f, 166));
  //----------------------------------------------------------------
  // trace point = 703
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9954987f, 39.713707f, 1483290997, 11.4774132),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.995491f,
                                                39.713707f, leg_index, 12.2882977f, 2877,
                                                maneuver_index, 0.692249298f, 165));
  //----------------------------------------------------------------
  // trace point = 704
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.995491f, 39.7134018f, 1483291000, 11.356884),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954758f,
                                                39.7134018f, leg_index, 12.2671719f, 2872,
                                                maneuver_index, 0.671123505f, 160));
  //----------------------------------------------------------------
  // trace point = 705
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9954758f, 39.7128983f, 1483291005, 11.2013073),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954453f,
                                                39.7128983f, leg_index, 12.2323027f, 2863,
                                                maneuver_index, 0.636254311f, 151));
  //----------------------------------------------------------------
  // trace point = 706
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9954453f, 39.712326f, 1483291011, 10.6434202),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954071f,
                                                39.712326f, leg_index, 12.1926708f, 2853,
                                                maneuver_index, 0.596622467f, 141));
  //----------------------------------------------------------------
  // trace point = 707
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9954224f, 39.7121468f, 1483291013, 10.1107445),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953995f,
                                                39.7121468f, leg_index, 12.1802645f, 2850,
                                                maneuver_index, 0.584216118f, 138));
  //----------------------------------------------------------------
  // trace point = 708
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953766f, 39.7115631f, 1483291020, 9.27561092),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953613f,
                                                39.7115631f, leg_index, 12.1398373f, 2840,
                                                maneuver_index, 0.54378891f, 128));
  //----------------------------------------------------------------
  // trace point = 709
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953613f, 39.7112274f, 1483291025, 7.50733089),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953384f,
                                                39.7112274f, leg_index, 12.1165848f, 2835,
                                                maneuver_index, 0.520536423f, 123));
  //----------------------------------------------------------------
  // trace point = 710
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953537f, 39.7110519f, 1483291028, 6.4976511),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953308f,
                                                39.7110519f, leg_index, 12.1044426f, 2832,
                                                maneuver_index, 0.508394241f, 120));
  //----------------------------------------------------------------
  // trace point = 711
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953384f, 39.7108803f, 1483291031, 6.27885389),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953156f,
                                                39.7108803f, leg_index, 12.0925417f, 2829,
                                                maneuver_index, 0.49649334f, 117));
  //----------------------------------------------------------------
  // trace point = 712
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953308f, 39.7107658f, 1483291033, 6.51908398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953079f,
                                                39.7107658f, leg_index, 12.0846157f, 2827,
                                                maneuver_index, 0.488567352f, 115));
  //----------------------------------------------------------------
  // trace point = 713
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953003f, 39.7101593f, 1483291042, 7.5029068),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952698f,
                                                39.7101593f, leg_index, 12.0426121f, 2817,
                                                maneuver_index, 0.446563721f, 105));
  //----------------------------------------------------------------
  // trace point = 714
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9952698f, 39.7095642f, 1483291049, 9.43797398),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952316f,
                                                39.7095642f, leg_index, 12.0014f, 2807,
                                                maneuver_index, 0.405351639f, 95));
  //----------------------------------------------------------------
  // trace point = 715
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9952469f, 39.7093086f, 1483291052, 9.54870892),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952087f,
                                                39.7093086f, leg_index, 11.9836807f, 2803,
                                                maneuver_index, 0.38763237f, 91));
  //----------------------------------------------------------------
  // trace point = 716
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.995224f, 39.7089767f, 1483291056, 9.2298069),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9951935f,
                                                39.7089767f, leg_index, 11.9607124f, 2798,
                                                maneuver_index, 0.364664078f, 86));
  //----------------------------------------------------------------
  // trace point = 717
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.995163f, 39.7084312f, 1483291063, 8.72973919),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.995163f,
                                                39.7084312f, leg_index, 11.9229431f, 2789,
                                                maneuver_index, 0.32689476f, 77));
  //----------------------------------------------------------------
  // trace point = 718
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9951324f, 39.7078247f, 1483291071, 8.45375061),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9951172f,
                                                39.7078247f, leg_index, 11.8809166f, 2779,
                                                maneuver_index, 0.28486824f, 67));
  //----------------------------------------------------------------
  // trace point = 719 | Alert | In a quarter mile Bear left onto Thankless Lane.
  maneuver_index = 8;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9951706f, 39.7072182f, 1483291077, 11.2770166),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -75.9951706f, 39.7072182f, leg_index, 11.8388395f,
                                                2768, maneuver_index, 0.242791176f, 56,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 720
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9951782f, 39.7071075f, 1483291078, 12.1772671),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9951859f,
                                                39.7071075f, leg_index, 11.8311443f, 2767,
                                                maneuver_index, 0.235095978f, 55));
  //----------------------------------------------------------------
  // trace point = 721
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9952164f, 39.7068863f, 1483291080, 12.3215714),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952164f,
                                                39.7068863f, leg_index, 11.8157539f, 2763,
                                                maneuver_index, 0.219705582f, 51));
  //----------------------------------------------------------------
  // trace point = 722
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9952469f, 39.7066727f, 1483291082, 12.0051022),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952469f,
                                                39.7066727f, leg_index, 11.80089f, 2760,
                                                maneuver_index, 0.204841614f, 48));
  //----------------------------------------------------------------
  // trace point = 723
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9952698f, 39.7065697f, 1483291083, 11.6684113),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952545f,
                                                39.7065697f, leg_index, 11.7937593f, 2758,
                                                maneuver_index, 0.197710991f, 46));
  //----------------------------------------------------------------
  // trace point = 724
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953079f, 39.7063751f, 1483291085, 10.9947262),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952774f,
                                                39.7063713f, leg_index, 11.7799854f, 2755,
                                                maneuver_index, 0.183937073f, 43));
  //----------------------------------------------------------------
  // trace point = 725
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953384f, 39.7061806f, 1483291087, 10.9196739),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953079f,
                                                39.7061768f, leg_index, 11.7664423f, 2751,
                                                maneuver_index, 0.170393944f, 39));
  //----------------------------------------------------------------
  // trace point = 726
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953613f, 39.7058029f, 1483291091, 10.5380001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953461f,
                                                39.7058029f, leg_index, 11.7404966f, 2746,
                                                maneuver_index, 0.14444828f, 34));
  //----------------------------------------------------------------
  // trace point = 727
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953918f, 39.7053947f, 1483291096, 9.09917641),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953842f,
                                                39.7053947f, leg_index, 11.7121897f, 2739,
                                                maneuver_index, 0.116141319f, 27));
  //----------------------------------------------------------------
  // trace point = 728
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.99543f, 39.7051086f, 1483291100, 7.9925642),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954147f,
                                                39.7051086f, leg_index, 11.6923342f, 2735,
                                                maneuver_index, 0.09628582f, 23));
  //----------------------------------------------------------------
  // trace point = 729
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9954453f, 39.7049675f, 1483291102, 7.87088823),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.99543f,
                                                39.7049675f, leg_index, 11.6825428f, 2732,
                                                maneuver_index, 0.0864944458f, 20));
  //----------------------------------------------------------------
  // trace point = 730
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9954529f, 39.7048302f, 1483291104, 7.63992023),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954453f,
                                                39.7048302f, leg_index, 11.6730013f, 2731,
                                                maneuver_index, 0.0769529343f, 19));
  //----------------------------------------------------------------
  // trace point = 731
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955063f, 39.7042885f, 1483291112, 7.5656848),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954987f,
                                                39.7042885f, leg_index, 11.6354179f, 2722,
                                                maneuver_index, 0.0393695831f, 10));
  //----------------------------------------------------------------
  // trace point = 732 | TODO Missing Pre | Bear left onto Thankless Lane.
  maneuver_index = 8;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955215f, 39.7041626f, 1483291114, 6.91508007),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9955063f,
                                                39.7041626f, leg_index, 11.6267023f, 2720,
                                                maneuver_index, 0.0306539536f, 8));
  //----------------------------------------------------------------
  // trace point = 733
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955444f, 39.7037392f, 1483291125, 4.30574179),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9955063f,
                                                39.7037392f, leg_index, 11.5960484f, 2712,
                                                maneuver_index, 0.656776428f, 164));
  //----------------------------------------------------------------
  // trace point = 734 | Post | Continue for 7 tenths of a mile.
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9954834f, 39.7035904f, 1483291130, 3.48316598),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -75.9954529f, 39.7035942f, leg_index, 11.586853f,
                                                2710, maneuver_index, 0.6475811f, 162,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 735
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9954605f, 39.7035522f, 1483291131, 4.6992569),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954453f,
                                                39.7035522f, leg_index, 11.5839224f, 2710,
                                                maneuver_index, 0.644650459f, 162));
  //----------------------------------------------------------------
  // trace point = 736
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.99543f, 39.7032433f, 1483291137, 5.71529198),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9954147f,
                                                39.7032433f, leg_index, 11.5624733f, 2704,
                                                maneuver_index, 0.62320137f, 156));
  //----------------------------------------------------------------
  // trace point = 737
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953995f, 39.7026825f, 1483291147, 6.26292801),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953842f,
                                                39.7026825f, leg_index, 11.5236511f, 2695,
                                                maneuver_index, 0.584379196f, 147));
  //----------------------------------------------------------------
  // trace point = 738
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953384f, 39.7020912f, 1483291156, 7.31635618),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953384f,
                                                39.7020912f, leg_index, 11.4826746f, 2684,
                                                maneuver_index, 0.543402672f, 136));
  //----------------------------------------------------------------
  // trace point = 739
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953003f, 39.7015915f, 1483291163, 7.96569681),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952927f,
                                                39.7015915f, leg_index, 11.4480228f, 2676,
                                                maneuver_index, 0.508750916f, 128));
  //----------------------------------------------------------------
  // trace point = 740
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9952545f, 39.7010536f, 1483291171, 7.52949381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952316f,
                                                39.7010536f, leg_index, 11.410675f, 2666,
                                                maneuver_index, 0.471403122f, 118));
  //----------------------------------------------------------------
  // trace point = 741
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9952393f, 39.7008781f, 1483291174, 6.47694921),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9952087f,
                                                39.700882f, leg_index, 11.3987408f, 2663,
                                                maneuver_index, 0.459468842f, 115));
  //----------------------------------------------------------------
  // trace point = 742
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9952087f, 39.7006683f, 1483291178, 5.81957817),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9951859f,
                                                39.7006683f, leg_index, 11.383914f, 2659,
                                                maneuver_index, 0.444642067f, 111));
  //----------------------------------------------------------------
  // trace point = 743
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9951935f, 39.7005272f, 1483291181, 5.25719309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9951706f,
                                                39.7005272f, leg_index, 11.3741169f, 2657,
                                                maneuver_index, 0.434844971f, 109));
  //----------------------------------------------------------------
  // trace point = 744
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9951782f, 39.7004013f, 1483291184, 4.81424379),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9951553f,
                                                39.7004013f, leg_index, 11.3653717f, 2654,
                                                maneuver_index, 0.426099777f, 106));
  //----------------------------------------------------------------
  // trace point = 745
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9951553f, 39.7002716f, 1483291187, 4.84542418),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9951401f,
                                                39.7002716f, leg_index, 11.3563643f, 2652,
                                                maneuver_index, 0.417092323f, 104));
  //----------------------------------------------------------------
  // trace point = 746
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9950943f, 39.6997108f, 1483291197, 6.24838305),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9950867f,
                                                39.6997108f, leg_index, 11.3174734f, 2643,
                                                maneuver_index, 0.378201485f, 95));
  //----------------------------------------------------------------
  // trace point = 747
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9950333f, 39.6990891f, 1483291204, 9.92152309),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9950409f,
                                                39.6990891f, leg_index, 11.2743855f, 2632,
                                                maneuver_index, 0.335113525f, 84));
  //----------------------------------------------------------------
  // trace point = 748
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.995018f, 39.6989861f, 1483291205, 11.380724),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9950409f,
                                                39.6989861f, leg_index, 11.2672672f, 2630,
                                                maneuver_index, 0.3279953f, 82));
  //----------------------------------------------------------------
  // trace point = 749
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9950027f, 39.6987801f, 1483291207, 11.5502634),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9950256f,
                                                39.6987801f, leg_index, 11.2529993f, 2626,
                                                maneuver_index, 0.313727379f, 78));
  //----------------------------------------------------------------
  // trace point = 750
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9949875f, 39.6985703f, 1483291209, 11.7641783),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9950027f,
                                                39.6985703f, leg_index, 11.2384291f, 2623,
                                                maneuver_index, 0.299157143f, 75));
  //----------------------------------------------------------------
  // trace point = 751
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9949722f, 39.6983604f, 1483291211, 11.6442327),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9949799f,
                                                39.6983604f, leg_index, 11.2238646f, 2619,
                                                maneuver_index, 0.284592628f, 71));
  //----------------------------------------------------------------
  // trace point = 752 | Alert | In a quarter mile Turn right onto Calvert Road.
  maneuver_index = 9;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9949646f, 39.6981544f, 1483291213, 11.5311937),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -75.994957f, 39.6981544f, leg_index, 11.2095642f,
                                                2616, maneuver_index, 0.270292282f, 68,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 753
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9949493f, 39.6979485f, 1483291215, 11.4182816),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9949341f,
                                                39.6979485f, leg_index, 11.1952648f, 2612,
                                                maneuver_index, 0.255992889f, 64));
  //----------------------------------------------------------------
  // trace point = 754
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9949265f, 39.6976509f, 1483291218, 11.0826244),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9949036f,
                                                39.6976509f, leg_index, 11.1746197f, 2606,
                                                maneuver_index, 0.235347748f, 58));
  //----------------------------------------------------------------
  // trace point = 755
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9948654f, 39.6971016f, 1483291224, 10.2008381),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9948578f,
                                                39.6971016f, leg_index, 11.1365471f, 2597,
                                                maneuver_index, 0.197275162f, 49));
  //----------------------------------------------------------------
  // trace point = 756
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9947968f, 39.6965027f, 1483291232, 8.38647079),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9948044f,
                                                39.6965027f, leg_index, 11.0950203f, 2587,
                                                maneuver_index, 0.155748367f, 39));
  //----------------------------------------------------------------
  // trace point = 757
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9947662f, 39.6962776f, 1483291236, 6.31414509),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9947815f,
                                                39.6962776f, leg_index, 11.0794067f, 2583,
                                                maneuver_index, 0.140134811f, 35));
  //----------------------------------------------------------------
  // trace point = 758
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.994751f, 39.6961327f, 1483291239, 5.36533499),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9947739f,
                                                39.6961327f, leg_index, 11.0693741f, 2580,
                                                maneuver_index, 0.130102158f, 32));
  //----------------------------------------------------------------
  // trace point = 759
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9947433f, 39.6960449f, 1483291241, 4.85166883),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9947662f,
                                                39.6960449f, leg_index, 11.0632915f, 2579,
                                                maneuver_index, 0.124019623f, 31));
  //----------------------------------------------------------------
  // trace point = 760
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9947357f, 39.6959648f, 1483291243, 4.57393408),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9947586f,
                                                39.6959648f, leg_index, 11.0577354f, 2578,
                                                maneuver_index, 0.118463516f, 30));
  //----------------------------------------------------------------
  // trace point = 761
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9947357f, 39.6958504f, 1483291246, 4.15592813),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9947433f,
                                                39.6958504f, leg_index, 11.0497694f, 2575,
                                                maneuver_index, 0.110497475f, 27));
  //----------------------------------------------------------------
  // trace point = 762
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9947052f, 39.6956787f, 1483291250, 4.90337515),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9947205f,
                                                39.6956787f, leg_index, 11.037837f, 2572,
                                                maneuver_index, 0.0985651016f, 24));
  //----------------------------------------------------------------
  // trace point = 763
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9946747f, 39.6953621f, 1483291256, 5.85994387),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9946671f,
                                                39.6953621f, leg_index, 11.0157528f, 2567,
                                                maneuver_index, 0.0764808655f, 19));
  //----------------------------------------------------------------
  // trace point = 764
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9945984f, 39.6948204f, 1483291266, 6.04639816),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9945755f,
                                                39.6948204f, leg_index, 10.9779692f, 2558,
                                                maneuver_index, 0.0386972427f, 10));
  //----------------------------------------------------------------
  // trace point = 765 | Pre | Turn right onto Calvert Road.
  maneuver_index = 9;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9945602f, 39.6945496f, 1483291272, 5.05130482),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -75.994545f, 39.6945496f, leg_index, 10.9591656f,
                                                2553, maneuver_index, 0.0198936462f, 5,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 766
  maneuver_index = 9;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9945526f, 39.6945114f, 1483291273, 4.64378023),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9945374f,
                                                39.6945114f, leg_index, 10.9565001f, 2552,
                                                maneuver_index, 0.0172281265f, 4));
  //----------------------------------------------------------------
  // trace point = 767
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9945984f, 39.6942444f, 1483291281, 3.74779201),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9945908f,
                                                39.6942291f, leg_index, 10.9342146f, 2546,
                                                maneuver_index, 0.213305473f, 49));
  //----------------------------------------------------------------
  // trace point = 768
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.995285f, 39.693985f, 1483291292, 5.97378778),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.995285f,
                                                39.6939774f, leg_index, 10.8933611f, 2537,
                                                maneuver_index, 0.172451973f, 40));
  //----------------------------------------------------------------
  // trace point = 769
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9957275f, 39.69384f, 1483291298, 6.85233402),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9957199f,
                                                39.6938324f, leg_index, 10.8681393f, 2531,
                                                maneuver_index, 0.147230148f, 34));
  //----------------------------------------------------------------
  // trace point = 770 | Alert | In 500 feet Turn left onto England Creamery Road.
  maneuver_index = 10;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9963989f, 39.693615f, 1483291307, 6.96624613),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -75.9963913f, 39.6936073f, leg_index, 10.8291578f,
                                                2522, maneuver_index, 0.108248711f, 25,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 771
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9971085f, 39.6933861f, 1483291316, 7.28406477),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9971008f,
                                                39.6933784f, leg_index, 10.788209f, 2513,
                                                maneuver_index, 0.0672998428f, 16));
  //----------------------------------------------------------------
  // trace point = 772
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9971848f, 39.6933594f, 1483291317, 7.55514908),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9971771f,
                                                39.6933517f, leg_index, 10.7837496f, 2512,
                                                maneuver_index, 0.0628404617f, 15));
  //----------------------------------------------------------------
  // trace point = 773 | Pre | Turn left onto England Creamery Road.
  maneuver_index = 10;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9978409f, 39.6931572f, 1483291325, 7.5659132),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -75.9978333f, 39.6931381f, leg_index, 10.7458296f,
                                                2503, maneuver_index, 0.0249204636f, 6,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 774
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9980774f, 39.6930771f, 1483291328, 7.38944101),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9980698f,
                                                39.6930656f, leg_index, 10.7322798f, 2500,
                                                maneuver_index, 0.0113706589f, 3));
  //----------------------------------------------------------------
  // trace point = 775
  maneuver_index = 10;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981461f, 39.6930389f, 1483291329, 7.26820183),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981461f,
                                                39.6930428f, leg_index, 10.7279215f, 2499,
                                                maneuver_index, 0.00701236725f, 2));
  //----------------------------------------------------------------
  // trace point = 776 | Post | Continue for 1.4 miles.
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982452f, 39.6927299f, 1483291334, 7.07221413),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -75.99823f, 39.6927299f, leg_index, 10.7016888f, 2492,
                                                maneuver_index, 1.41049385f, 332, instruction_index));
  //----------------------------------------------------------------
  // trace point = 777
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.99823f, 39.6921196f, 1483291343, 7.53527021),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981842f,
                                                39.6921196f, leg_index, 10.6593857f, 2482,
                                                maneuver_index, 1.36819077f, 322));
  //----------------------------------------------------------------
  // trace point = 778
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982147f, 39.691597f, 1483291350, 8.33395767),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981689f,
                                                39.691597f, leg_index, 10.6232281f, 2473,
                                                maneuver_index, 1.33203316f, 313));
  //----------------------------------------------------------------
  // trace point = 779
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982071f, 39.6910553f, 1483291357, 8.60511208),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981613f,
                                                39.6910553f, leg_index, 10.5857553f, 2465,
                                                maneuver_index, 1.29456043f, 305));
  //----------------------------------------------------------------
  // trace point = 780
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981766f, 39.6904259f, 1483291364, 10.0405054),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981461f,
                                                39.6904259f, leg_index, 10.5422029f, 2454,
                                                maneuver_index, 1.25100803f, 294));
  //----------------------------------------------------------------
  // trace point = 781
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981689f, 39.6903229f, 1483291365, 11.2484884),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981461f,
                                                39.6903229f, leg_index, 10.535078f, 2452,
                                                maneuver_index, 1.24388313f, 292));
  //----------------------------------------------------------------
  // trace point = 782
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981613f, 39.6900253f, 1483291368, 11.1020422),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981537f,
                                                39.6900253f, leg_index, 10.5144863f, 2447,
                                                maneuver_index, 1.2232914f, 287));
  //----------------------------------------------------------------
  // trace point = 783
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981537f, 39.689827f, 1483291370, 11.077035),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981613f,
                                                39.689827f, leg_index, 10.5007591f, 2444,
                                                maneuver_index, 1.20956421f, 284));
  //----------------------------------------------------------------
  // trace point = 784
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981613f, 39.6897278f, 1483291371, 11.1349154),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981689f,
                                                39.6897278f, leg_index, 10.4938889f, 2442,
                                                maneuver_index, 1.20269394f, 282));
  //----------------------------------------------------------------
  // trace point = 785
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981613f, 39.6896248f, 1483291372, 11.3626633),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981689f,
                                                39.6896248f, leg_index, 10.4867697f, 2441,
                                                maneuver_index, 1.19557476f, 281));
  //----------------------------------------------------------------
  // trace point = 786
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981766f, 39.6894188f, 1483291374, 11.4815769),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981918f,
                                                39.6894188f, leg_index, 10.4724665f, 2437,
                                                maneuver_index, 1.18127155f, 277));
  //----------------------------------------------------------------
  // trace point = 787
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982224f, 39.6889915f, 1483291378, 11.9018764),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982376f,
                                                39.6889915f, leg_index, 10.4428129f, 2430,
                                                maneuver_index, 1.151618f, 270));
  //----------------------------------------------------------------
  // trace point = 788
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982605f, 39.6886635f, 1483291381, 12.1822433),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982452f,
                                                39.6886635f, leg_index, 10.4201059f, 2425,
                                                maneuver_index, 1.12891102f, 265));
  //----------------------------------------------------------------
  // trace point = 789
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982834f, 39.6884499f, 1483291383, 12.0593758),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982529f,
                                                39.6884499f, leg_index, 10.4053259f, 2421,
                                                maneuver_index, 1.11413097f, 261));
  //----------------------------------------------------------------
  // trace point = 790
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9983139f, 39.6881218f, 1483291386, 12.199069),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982529f,
                                                39.6881218f, leg_index, 10.3826342f, 2416,
                                                maneuver_index, 1.09143925f, 256));
  //----------------------------------------------------------------
  // trace point = 791
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9983292f, 39.6879044f, 1483291388, 12.0873508),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982605f,
                                                39.6879044f, leg_index, 10.3675909f, 2412,
                                                maneuver_index, 1.07639599f, 252));
  //----------------------------------------------------------------
  // trace point = 792
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9983292f, 39.6875801f, 1483291391, 11.9484282),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982605f,
                                                39.6875801f, leg_index, 10.3451624f, 2407,
                                                maneuver_index, 1.05396748f, 247));
  //----------------------------------------------------------------
  // trace point = 793
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982986f, 39.686985f, 1483291397, 11.0835009),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982681f,
                                                39.686985f, leg_index, 10.3039904f, 2397,
                                                maneuver_index, 1.01279545f, 237));
  //----------------------------------------------------------------
  // trace point = 794
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982986f, 39.6865311f, 1483291402, 10.1302042),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982834f,
                                                39.6865311f, leg_index, 10.2725811f, 2390,
                                                maneuver_index, 0.981386185f, 230));
  //----------------------------------------------------------------
  // trace point = 795
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982986f, 39.6860466f, 1483291408, 8.99832535),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982834f,
                                                39.6860466f, leg_index, 10.239069f, 2381,
                                                maneuver_index, 0.947874069f, 221));
  //----------------------------------------------------------------
  // trace point = 796
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982834f, 39.6858444f, 1483291411, 7.46501684),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982758f,
                                                39.6858444f, leg_index, 10.2250605f, 2378,
                                                maneuver_index, 0.933865547f, 218));
  //----------------------------------------------------------------
  // trace point = 797
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982529f, 39.6856651f, 1483291414, 6.70079279),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9982529f,
                                                39.6856651f, leg_index, 10.2126217f, 2375,
                                                maneuver_index, 0.921426773f, 215));
  //----------------------------------------------------------------
  // trace point = 798
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982376f, 39.6856079f, 1483291415, 6.49156713),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.99823f,
                                                39.6856117f, leg_index, 10.2076025f, 2374,
                                                maneuver_index, 0.916407585f, 214));
  //----------------------------------------------------------------
  // trace point = 799
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9982147f, 39.6855507f, 1483291416, 6.56418896),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981995f,
                                                39.6855507f, leg_index, 10.2042103f, 2373,
                                                maneuver_index, 0.913015366f, 213));
  //----------------------------------------------------------------
  // trace point = 800
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981003f, 39.6853981f, 1483291419, 6.63388395),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9980774f,
                                                39.6854095f, leg_index, 10.1924648f, 2370,
                                                maneuver_index, 0.901269913f, 210));
  //----------------------------------------------------------------
  // trace point = 801
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9980469f, 39.6853523f, 1483291420, 6.654531),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9980316f,
                                                39.6853638f, leg_index, 10.1884689f, 2369,
                                                maneuver_index, 0.897274017f, 209));
  //----------------------------------------------------------------
  // trace point = 802
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9978943f, 39.6852112f, 1483291423, 6.82480001),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9978714f,
                                                39.6852264f, leg_index, 10.175703f, 2366,
                                                maneuver_index, 0.884508133f, 206));
  //----------------------------------------------------------------
  // trace point = 803
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9974213f, 39.6848106f, 1483291432, 6.705019),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9973984f,
                                                39.6848259f, leg_index, 10.1382647f, 2358,
                                                maneuver_index, 0.84706974f, 198));
  //----------------------------------------------------------------
  // trace point = 804
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9970627f, 39.6845016f, 1483291439, 6.53913879),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9970398f,
                                                39.6845169f, leg_index, 10.1096029f, 2350,
                                                maneuver_index, 0.818408012f, 190));
  //----------------------------------------------------------------
  // trace point = 805
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9966049f, 39.6840401f, 1483291449, 6.49483299),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.996582f,
                                                39.6840553f, leg_index, 10.0694304f, 2341,
                                                maneuver_index, 0.778235435f, 181));
  //----------------------------------------------------------------
  // trace point = 806
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9961853f, 39.6835709f, 1483291457, 7.90707397),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9961777f,
                                                39.6835747f, leg_index, 10.0297985f, 2332,
                                                maneuver_index, 0.738603592f, 172));
  //----------------------------------------------------------------
  // trace point = 807
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9958572f, 39.6831856f, 1483291462, 10.2408533),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9958496f,
                                                39.6831932f, leg_index, 9.9981575f, 2324,
                                                maneuver_index, 0.706962585f, 164));
  //----------------------------------------------------------------
  // trace point = 808
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9957199f, 39.6830254f, 1483291464, 10.688344),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9957047f,
                                                39.683033f, leg_index, 9.98377132f, 2321,
                                                maneuver_index, 0.692576408f, 161));
  //----------------------------------------------------------------
  // trace point = 809
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9955292f, 39.6827888f, 1483291467, 10.410037),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9955139f,
                                                39.6827965f, leg_index, 9.96540451f, 2317,
                                                maneuver_index, 0.674209595f, 157));
  //----------------------------------------------------------------
  // trace point = 810
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9953995f, 39.6826363f, 1483291469, 10.1295767),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9953842f,
                                                39.6826477f, leg_index, 9.95301056f, 2314,
                                                maneuver_index, 0.661815643f, 154));
  //----------------------------------------------------------------
  // trace point = 811
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9949036f, 39.6822548f, 1483291475, 9.9621582),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9948959f,
                                                39.6822662f, leg_index, 9.91566563f, 2306,
                                                maneuver_index, 0.624470711f, 146));
  //----------------------------------------------------------------
  // trace point = 812
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9947205f, 39.6821594f, 1483291477, 9.6645546),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9947052f,
                                                39.6821709f, leg_index, 9.90355873f, 2303,
                                                maneuver_index, 0.612363815f, 143));
  //----------------------------------------------------------------
  // trace point = 813
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9941177f, 39.6818619f, 1483291484, 8.72940063),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9941101f,
                                                39.6818695f, leg_index, 9.8656311f, 2294,
                                                maneuver_index, 0.574436188f, 134));
  //----------------------------------------------------------------
  // trace point = 814
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9936829f, 39.6816368f, 1483291490, 7.47334099),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9936752f,
                                                39.6816444f, leg_index, 9.83769703f, 2288,
                                                maneuver_index, 0.546502113f, 128));
  //----------------------------------------------------------------
  // trace point = 815
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9934235f, 39.681488f, 1483291494, 6.96016788),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9934235f,
                                                39.6814842f, leg_index, 9.82030201f, 2284,
                                                maneuver_index, 0.529107094f, 124));
  //----------------------------------------------------------------
  // trace point = 816
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.993187f, 39.6813316f, 1483291498, 6.62879705),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.993187f,
                                                39.6813354f, leg_index, 9.80404186f, 2280,
                                                maneuver_index, 0.512846947f, 120));
  //----------------------------------------------------------------
  // trace point = 817
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9925766f, 39.6809387f, 1483291507, 7.572752),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9925613f,
                                                39.680954f, leg_index, 9.76155186f, 2270,
                                                maneuver_index, 0.470356941f, 110));
  //----------------------------------------------------------------
  // trace point = 818
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9919968f, 39.6805763f, 1483291514, 9.19760513),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9919739f,
                                                39.6805954f, leg_index, 9.72163486f, 2260,
                                                maneuver_index, 0.430439949f, 100));
  //----------------------------------------------------------------
  // trace point = 819
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9917374f, 39.6804199f, 1483291517, 9.28029633),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9917145f,
                                                39.680439f, leg_index, 9.70409203f, 2256,
                                                maneuver_index, 0.41289711f, 96));
  //----------------------------------------------------------------
  // trace point = 820
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9911728f, 39.6800766f, 1483291524, 8.81709385),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9911499f,
                                                39.6800957f, leg_index, 9.66578674f, 2247,
                                                maneuver_index, 0.374591827f, 87));
  //----------------------------------------------------------------
  // trace point = 821
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9905624f, 39.6797028f, 1483291532, 8.3687067),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9905472f,
                                                39.6797218f, leg_index, 9.62457752f, 2237,
                                                maneuver_index, 0.333382607f, 77));
  //----------------------------------------------------------------
  // trace point = 822
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9899673f, 39.6793404f, 1483291539, 9.24429607),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9899521f,
                                                39.6793556f, leg_index, 9.58401585f, 2228,
                                                maneuver_index, 0.29282093f, 68));
  //----------------------------------------------------------------
  // trace point = 823
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9896088f, 39.6791191f, 1483291543, 9.89731789),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9895935f,
                                                39.6791306f, leg_index, 9.55938244f, 2222,
                                                maneuver_index, 0.268187523f, 62));
  //----------------------------------------------------------------
  // trace point = 824 | Alert | In a querter mile Turn right onto Crothers Road.
  maneuver_index = 11;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9893341f, 39.6789513f, 1483291546, 10.0500641),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -75.9893265f, 39.6789627f, leg_index, 9.54102802f,
                                                2218, maneuver_index, 0.249833107f, 58,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 825
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9887161f, 39.6785583f, 1483291553, 9.76331043),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9887009f,
                                                39.6785736f, leg_index, 9.49820805f, 2208,
                                                maneuver_index, 0.20701313f, 48));
  //----------------------------------------------------------------
  // trace point = 826
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9881287f, 39.6781921f, 1483291561, 8.09932137),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9881134f,
                                                39.6782074f, leg_index, 9.45796299f, 2198,
                                                maneuver_index, 0.166768074f, 38));
  //----------------------------------------------------------------
  // trace point = 827
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9875793f, 39.6778603f, 1483291569, 7.55356216),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9875641f,
                                                39.6778755f, leg_index, 9.42077732f, 2189,
                                                maneuver_index, 0.129582405f, 29));
  //----------------------------------------------------------------
  // trace point = 828
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.98703f, 39.6774902f, 1483291576, 8.93161201),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.98703f,
                                                39.6774902f, leg_index, 9.38176346f, 2180,
                                                maneuver_index, 0.0905685425f, 20));
  //----------------------------------------------------------------
  // trace point = 829
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9867325f, 39.6772041f, 1483291580, 10.1141157),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9867172f,
                                                39.6772156f, leg_index, 9.35649395f, 2175,
                                                maneuver_index, 0.0652990341f, 15));
  //----------------------------------------------------------------
  // trace point = 830
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9865952f, 39.6770668f, 1483291582, 9.71882629),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9865723f,
                                                39.6770821f, leg_index, 9.34445858f, 2172,
                                                maneuver_index, 0.0532636642f, 12));
  //----------------------------------------------------------------
  // trace point = 831 | Pre | Turn right onto Crothers Road.
  maneuver_index = 11;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9861221f, 39.6765938f, 1483291590, 8.30313206),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -75.9860992f, 39.676609f, leg_index, 9.30310726f,
                                                2163, maneuver_index, 0.0119123459f, 3,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 832
  maneuver_index = 11;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9860764f, 39.6765327f, 1483291591, 8.14191628f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9860382f,
                                                39.6765594f, leg_index, 9.29837894f, 2162,
                                                maneuver_index, 0.00718402863f, 2));
  //----------------------------------------------------------------
  // trace point = 833 | Post | Continue for 1.8 miles.
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9861679f, 39.676178f, 1483291596, 8.0294733f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -75.9861603f, 39.6761742f, leg_index, 9.26697922f,
                                                2154, maneuver_index, 1.74191141f, 414,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 834
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9865875f, 39.6756592f, 1483291604, 8.4675436f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9865799f,
                                                39.6756516f, leg_index, 9.22410679f, 2143,
                                                maneuver_index, 1.69903898f, 403));
  //----------------------------------------------------------------
  // trace point = 835
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.986908f, 39.6755142f, 1483291608, 8.03762722f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.986908f,
                                                39.675518f, leg_index, 9.2042799f, 2139,
                                                maneuver_index, 1.67921209f, 399));
  //----------------------------------------------------------------
  // trace point = 836
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9876022f, 39.6753387f, 1483291616, 7.78657818f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9876022f,
                                                39.6753426f, leg_index, 9.16537476f, 2129,
                                                maneuver_index, 1.64030695f, 389));
  //----------------------------------------------------------------
  // trace point = 837
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9883423f, 39.6751595f, 1483291624, 8.33218002f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9883423f,
                                                39.6751595f, leg_index, 9.12398911f, 2119,
                                                maneuver_index, 1.5989213f, 379));
  //----------------------------------------------------------------
  // trace point = 838
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9885406f, 39.6751137f, 1483291626, 8.87655735f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9885406f,
                                                39.6751099f, leg_index, 9.11288548f, 2117,
                                                maneuver_index, 1.58781767f, 377));
  //----------------------------------------------------------------
  // trace point = 839
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9890289f, 39.674984f, 1483291631, 8.85804272f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9890289f,
                                                39.674984f, leg_index, 9.08545494f, 2110,
                                                maneuver_index, 1.56038713f, 370));
  //----------------------------------------------------------------
  // trace point = 840
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9895172f, 39.6747055f, 1483291637, 8.715271f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9895172f,
                                                39.6747055f, leg_index, 9.05296421f, 2102,
                                                maneuver_index, 1.5278964f, 362));
  //----------------------------------------------------------------
  // trace point = 841
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9901276f, 39.6743126f, 1483291645, 8.49599266f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9901276f,
                                                39.6743126f, leg_index, 9.01060486f, 2092,
                                                maneuver_index, 1.48553705f, 352));
  //----------------------------------------------------------------
  // trace point = 842
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9906998f, 39.6739998f, 1483291653, 7.51663017f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9906998f,
                                                39.6740036f, leg_index, 8.97338295f, 2083,
                                                maneuver_index, 1.44831514f, 343));
  //----------------------------------------------------------------
  // trace point = 843
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9913101f, 39.6737175f, 1483291660, 8.68296146f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9913025f,
                                                39.6737061f, leg_index, 8.93526173f, 2074,
                                                maneuver_index, 1.41019392f, 334));
  //----------------------------------------------------------------
  // trace point = 844
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9919739f, 39.6734047f, 1483291668, 8.37398529f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9919739f,
                                                39.6734009f, leg_index, 8.89374161f, 2064,
                                                maneuver_index, 1.3686738f, 324));
  //----------------------------------------------------------------
  // trace point = 845
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9926147f, 39.6731682f, 1483291676, 7.59096718f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9926147f,
                                                39.6731644f, leg_index, 8.85527515f, 2055,
                                                maneuver_index, 1.33020735f, 315));
  //----------------------------------------------------------------
  // trace point = 846
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9928741f, 39.6730957f, 1483291680, 5.99954605f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9928741f,
                                                39.6730919f, leg_index, 8.84105492f, 2052,
                                                maneuver_index, 1.31598711f, 312));
  //----------------------------------------------------------------
  // trace point = 847
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.993042f, 39.6730499f, 1483291683, 5.08046913f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.993042f,
                                                39.6730499f, leg_index, 8.83166122f, 2050,
                                                maneuver_index, 1.30659342f, 310));
  //----------------------------------------------------------------
  // trace point = 848
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9931488f, 39.6730232f, 1483291685, 4.56255913f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9931488f,
                                                39.6730194f, leg_index, 8.82559681f, 2048,
                                                maneuver_index, 1.300529f, 308));
  //----------------------------------------------------------------
  // trace point = 849
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9932785f, 39.6729889f, 1483291688, 4.13229609f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9932785f,
                                                39.6729851f, leg_index, 8.81829357f, 2047,
                                                maneuver_index, 1.29322577f, 307));
  //----------------------------------------------------------------
  // trace point = 850
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9934464f, 39.6729431f, 1483291692, 3.67069411f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9934464f,
                                                39.6729393f, leg_index, 8.8088131f, 2044,
                                                maneuver_index, 1.28374529f, 304));
  //----------------------------------------------------------------
  // trace point = 851
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9935226f, 39.672924f, 1483291694, 3.49974394f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9935226f,
                                                39.6729164f, leg_index, 8.80445385f, 2043,
                                                maneuver_index, 1.27938604f, 303));
  //----------------------------------------------------------------
  // trace point = 852
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9936676f, 39.6728821f, 1483291698, 3.38951111f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9936676f,
                                                39.6728745f, leg_index, 8.79620838f, 2041,
                                                maneuver_index, 1.27114058f, 301));
  //----------------------------------------------------------------
  // trace point = 853
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9937744f, 39.6728439f, 1483291701, 3.37633109f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9937744f,
                                                39.6728439f, leg_index, 8.79014874f, 2040,
                                                maneuver_index, 1.26508093f, 300));
  //----------------------------------------------------------------
  // trace point = 854
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9939651f, 39.6727715f, 1483291706, 3.613796f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9939651f,
                                                39.6727715f, leg_index, 8.77882576f, 2037,
                                                maneuver_index, 1.25375795f, 297));
  //----------------------------------------------------------------
  // trace point = 855
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9942322f, 39.6726608f, 1483291712, 4.32095289f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9942245f,
                                                39.6726532f, leg_index, 8.76276398f, 2034,
                                                maneuver_index, 1.23769617f, 294));
  //----------------------------------------------------------------
  // trace point = 856
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9948883f, 39.6724052f, 1483291725, 4.82028723f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9948883f,
                                                39.6723976f, leg_index, 8.72322464f, 2024,
                                                maneuver_index, 1.19815683f, 284));
  //----------------------------------------------------------------
  // trace point = 857
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9950409f, 39.6723633f, 1483291728, 4.7720871f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9950333f,
                                                39.6723518f, leg_index, 8.71488571f, 2022,
                                                maneuver_index, 1.18981791f, 282));
  //----------------------------------------------------------------
  // trace point = 858
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9956818f, 39.6721687f, 1483291740, 4.92795181f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9956818f,
                                                39.6721611f, leg_index, 8.6779232f, 2014,
                                                maneuver_index, 1.1528554f, 274));
  //----------------------------------------------------------------
  // trace point = 859
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9961243f, 39.672039f, 1483291748, 5.02291393f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9961166f,
                                                39.6720314f, leg_index, 8.65309334f, 2008,
                                                maneuver_index, 1.12802553f, 268));
  //----------------------------------------------------------------
  // trace point = 860
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9964447f, 39.6719398f, 1483291754, 4.984653f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9964447f,
                                                39.6719322f, leg_index, 8.63432693f, 2003,
                                                maneuver_index, 1.10925913f, 263));
  //----------------------------------------------------------------
  // trace point = 861
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9967651f, 39.6718483f, 1483291760, 4.85129309f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9967575f,
                                                39.6718369f, leg_index, 8.61641502f, 1999,
                                                maneuver_index, 1.09134722f, 259));
  //----------------------------------------------------------------
  // trace point = 862
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9974518f, 39.6716499f, 1483291772, 5.24471092f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9974442f,
                                                39.6716347f, leg_index, 8.57727623f, 1990,
                                                maneuver_index, 1.05220842f, 250));
  //----------------------------------------------------------------
  // trace point = 863
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9981613f, 39.6713867f, 1483291781, 7.48123884f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9981461f,
                                                39.67136f, leg_index, 8.53532887f, 1980,
                                                maneuver_index, 1.01026106f, 240));
  //----------------------------------------------------------------
  // trace point = 864
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9985046f, 39.6712418f, 1483291785, 8.42578411f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9984894f,
                                                39.6712227f, leg_index, 8.51472759f, 1975,
                                                maneuver_index, 0.989659786f, 235));
  //----------------------------------------------------------------
  // trace point = 865
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9986725f, 39.6711731f, 1483291787, 8.09411812f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9986649f,
                                                39.6711578f, leg_index, 8.50436497f, 1973,
                                                maneuver_index, 0.979297161f, 233));
  //----------------------------------------------------------------
  // trace point = 866
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9992981f, 39.6709251f, 1483291795, 7.56939697f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9992905f,
                                                39.6709175f, leg_index, 8.46713924f, 1964,
                                                maneuver_index, 0.942071438f, 224));
  //----------------------------------------------------------------
  // trace point = 867
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-75.9997787f, 39.670723f, 1483291803, 5.86602402f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -75.9997864f,
                                                39.6707306f, leg_index, 8.43773842f, 1957,
                                                maneuver_index, 0.912670612f, 217));
  //----------------------------------------------------------------
  // trace point = 868
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0000229f, 39.6706161f, 1483291808, 4.79473305f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0000381f,
                                                39.6706352f, leg_index, 8.4223175f, 1953,
                                                maneuver_index, 0.897249699f, 213));
  //----------------------------------------------------------------
  // trace point = 869
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0001755f, 39.6705551f, 1483291811, 4.89845181f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0001907f,
                                                39.6705704f, leg_index, 8.41352081f, 1951,
                                                maneuver_index, 0.888453007f, 211));
  //----------------------------------------------------------------
  // trace point = 870
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0003357f, 39.6704865f, 1483291814, 5.1344862f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.000351f,
                                                39.6705017f, leg_index, 8.4037571f, 1949,
                                                maneuver_index, 0.878689289f, 209));
  //----------------------------------------------------------------
  // trace point = 871
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0005341f, 39.6704102f, 1483291818, 4.77740002f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0005417f,
                                                39.6704216f, leg_index, 8.39218903f, 1946,
                                                maneuver_index, 0.86712122f, 206));
  //----------------------------------------------------------------
  // trace point = 872
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0007553f, 39.6703224f, 1483291822, 5.38725424f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0007629f,
                                                39.6703262f, leg_index, 8.37868786f, 1943,
                                                maneuver_index, 0.853620052f, 203));
  //----------------------------------------------------------------
  // trace point = 873
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0012741f, 39.670105f, 1483291831, 5.60642719f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0012741f,
                                                39.6701088f, leg_index, 8.34759331f, 1935,
                                                maneuver_index, 0.822525501f, 195));
  //----------------------------------------------------------------
  // trace point = 874
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0018082f, 39.6698837f, 1483291841, 5.18754816f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0018082f,
                                                39.6698875f, leg_index, 8.31530094f, 1927,
                                                maneuver_index, 0.790233135f, 187));
  //----------------------------------------------------------------
  // trace point = 875
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0024796f, 39.6696091f, 1483291851, 6.49575615f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0024796f,
                                                39.6696091f, leg_index, 8.27469444f, 1918,
                                                maneuver_index, 0.749626637f, 178));
  //----------------------------------------------------------------
  // trace point = 876
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0031281f, 39.6693459f, 1483291860, 6.99453115f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0031281f,
                                                39.669342f, leg_index, 8.23553658f, 1908,
                                                maneuver_index, 0.710468769f, 168));
  //----------------------------------------------------------------
  // trace point = 877
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0037918f, 39.6690826f, 1483291869, 7.10967493f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0037842f,
                                                39.669075f, leg_index, 8.19602013f, 1899,
                                                maneuver_index, 0.67095232f, 159));
  //----------------------------------------------------------------
  // trace point = 878
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0044556f, 39.6688118f, 1483291880, 5.89584398f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0044479f,
                                                39.6687965f, leg_index, 8.15576935f, 1889,
                                                maneuver_index, 0.630701542f, 149));
  //----------------------------------------------------------------
  // trace point = 879
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0047913f, 39.6686707f, 1483291887, 4.65542316f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0047836f,
                                                39.6686554f, leg_index, 8.13539982f, 1884,
                                                maneuver_index, 0.610332012f, 144));
  //----------------------------------------------------------------
  // trace point = 880
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0049286f, 39.6686096f, 1483291890, 4.514678f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.004921f,
                                                39.668602f, leg_index, 8.12720776f, 1883,
                                                maneuver_index, 0.60213995f, 143));
  //----------------------------------------------------------------
  // trace point = 881
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0050659f, 39.6685524f, 1483291893, 4.4909358f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0050583f,
                                                39.6685448f, leg_index, 8.11888885f, 1881,
                                                maneuver_index, 0.593821049f, 141));
  //----------------------------------------------------------------
  // trace point = 882
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0052948f, 39.6684608f, 1483291898, 4.33031797f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0052872f,
                                                39.6684494f, leg_index, 8.10503578f, 1878,
                                                maneuver_index, 0.579967976f, 138));
  //----------------------------------------------------------------
  // trace point = 883
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0055771f, 39.6683426f, 1483291904, 4.6555748f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0055695f,
                                                39.6683273f, leg_index, 8.08779049f, 1874,
                                                maneuver_index, 0.562722683f, 134));
  //----------------------------------------------------------------
  // trace point = 884
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0062332f, 39.6680756f, 1483291914, 6.33588219f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0062256f,
                                                39.6680603f, leg_index, 8.04827309f, 1864,
                                                maneuver_index, 0.52320528f, 124));
  //----------------------------------------------------------------
  // trace point = 885
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0067978f, 39.6678429f, 1483291921, 7.82056379f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0067902f,
                                                39.6678314f, leg_index, 8.01429939f, 1856,
                                                maneuver_index, 0.489231586f, 116));
  //----------------------------------------------------------------
  // trace point = 886
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0071564f, 39.6677017f, 1483291925, 8.6911087f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0071411f,
                                                39.6676826f, leg_index, 7.99297047f, 1851,
                                                maneuver_index, 0.46790266f, 111));
  //----------------------------------------------------------------
  // trace point = 887
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0075073f, 39.6675568f, 1483291929, 8.55833912f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0074997f,
                                                39.6675415f, leg_index, 7.97152615f, 1846,
                                                maneuver_index, 0.44645834f, 106));
  //----------------------------------------------------------------
  // trace point = 888
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0081635f, 39.6672859f, 1483291937, 7.98027515f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0081635f,
                                                39.6672821f, leg_index, 7.9318924f, 1836,
                                                maneuver_index, 0.406824589f, 96));
  //----------------------------------------------------------------
  // trace point = 889
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0088348f, 39.6670151f, 1483291946, 7.22036314f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0088272f,
                                                39.6670036f, leg_index, 7.89163399f, 1827,
                                                maneuver_index, 0.366566181f, 87));
  //----------------------------------------------------------------
  // trace point = 890
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0094299f, 39.6667252f, 1483291954, 7.53270483f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0094223f,
                                                39.6667137f, leg_index, 7.8541255f, 1818,
                                                maneuver_index, 0.329057693f, 78));
  //----------------------------------------------------------------
  // trace point = 891
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0100174f, 39.6664162f, 1483291963, 6.733253f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0100098f,
                                                39.6664085f, leg_index, 7.81638813f, 1809,
                                                maneuver_index, 0.291320324f, 69));
  //----------------------------------------------------------------
  // trace point = 892
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.010376f, 39.6662292f, 1483291969, 6.23661184f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.010376f,
                                                39.6662254f, leg_index, 7.79313707f, 1804,
                                                maneuver_index, 0.268069267f, 64));
  //----------------------------------------------------------------
  // trace point = 893
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0109634f, 39.6659317f, 1483291978, 6.72562313f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0109558f,
                                                39.6659241f, leg_index, 7.75588417f, 1795,
                                                maneuver_index, 0.230816364f, 55));
  //----------------------------------------------------------------
  // trace point = 894
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0115509f, 39.6656265f, 1483291986, 7.55930901f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0115433f,
                                                39.6656189f, leg_index, 7.7181468f, 1786,
                                                maneuver_index, 0.193078995f, 46));
  //----------------------------------------------------------------
  // trace point = 895
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0121307f, 39.6653328f, 1483291994, 7.4668622f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0121231f,
                                                39.6653214f, leg_index, 7.68104076f, 1777,
                                                maneuver_index, 0.155972958f, 37));
  //----------------------------------------------------------------
  // trace point = 896
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0127335f, 39.6650238f, 1483292004, 6.18515587f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0127258f,
                                                39.6650085f, leg_index, 7.64233494f, 1768,
                                                maneuver_index, 0.117267132f, 28));
  //----------------------------------------------------------------
  // trace point = 897 | Alert | In 500 feet Bear right onto Principio Road.
  maneuver_index = 12;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0129547f, 39.6649094f, 1483292009, 4.60427904f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0129471f, 39.6648979f, leg_index, 7.62828684f,
                                                1764, maneuver_index, 0.103219032f, 24,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 898
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0131073f, 39.6648369f, 1483292012, 4.99685812f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.013092f,
                                                39.6648216f, leg_index, 7.61893892f, 1762,
                                                maneuver_index, 0.0938711166f, 22));
  //----------------------------------------------------------------
  // trace point = 899
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0133514f, 39.6647072f, 1483292017, 5.11730909f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0133438f,
                                                39.6646957f, leg_index, 7.60295296f, 1758,
                                                maneuver_index, 0.0778851509f, 18));
  //----------------------------------------------------------------
  // trace point = 900
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0138016f, 39.6644821f, 1483292026, 5.07844687f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0137863f,
                                                39.6644669f, leg_index, 7.5745635f, 1752,
                                                maneuver_index, 0.049495697f, 12));
  //----------------------------------------------------------------
  // trace point = 901
  maneuver_index = 12;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0142441f, 39.664257f, 1483292035, 5.09768486f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0142288f,
                                                39.664238f, leg_index, 7.54617643f, 1745,
                                                maneuver_index, 0.0211086273f, 5));
  //----------------------------------------------------------------
  // trace point = 902 | Pre | Bear right onto Principio Road.
  maneuver_index = 12;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0144806f, 39.6641464f, 1483292044, 2.61874104f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0144501f, 39.6641197f, leg_index, 7.53182936f,
                                                1742, maneuver_index, 0.0067615509f, 2,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 903
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0147629f, 39.66399f, 1483292063, 1.56280899f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0147552f,
                                                39.6639786f, leg_index, 7.51277924f, 1738,
                                                maneuver_index, 1.59890079f, 336));
  //----------------------------------------------------------------
  // trace point = 904 | TODO missing Post | Continue for 1.6 miles.
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.014946f, 39.6639099f, 1483292069, 3.03655291f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.014946f,
                                                39.6639061f, leg_index, 7.50145292f, 1735,
                                                maneuver_index, 1.58757448f, 333));
  //----------------------------------------------------------------
  // trace point = 905
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0152893f, 39.6638412f, 1483292076, 4.2886138f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0152817f,
                                                39.663826f, leg_index, 7.48269796f, 1732,
                                                maneuver_index, 1.56881952f, 330));
  //----------------------------------------------------------------
  // trace point = 906
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0156021f, 39.6637878f, 1483292081, 5.54770184f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0155945f,
                                                39.6637611f, leg_index, 7.46544886f, 1728,
                                                maneuver_index, 1.55157042f, 326));
  //----------------------------------------------------------------
  // trace point = 907
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0159378f, 39.663723f, 1483292086, 5.904459f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0159225f,
                                                39.6636887f, leg_index, 7.44727564f, 1725,
                                                maneuver_index, 1.5333972f, 323));
  //----------------------------------------------------------------
  // trace point = 908
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0164566f, 39.6636162f, 1483292094, 5.78281879f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0164413f,
                                                39.663578f, leg_index, 7.4186101f, 1719,
                                                maneuver_index, 1.50473166f, 317));
  //----------------------------------------------------------------
  // trace point = 909
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0168839f, 39.6635284f, 1483292101, 5.40375996f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0168686f,
                                                39.6634865f, leg_index, 7.39499521f, 1714,
                                                maneuver_index, 1.48111677f, 312));
  //----------------------------------------------------------------
  // trace point = 910
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0175323f, 39.6633911f, 1483292112, 5.22978497f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0175171f,
                                                39.6633492f, leg_index, 7.35918236f, 1706,
                                                maneuver_index, 1.44530392f, 304));
  //----------------------------------------------------------------
  // trace point = 911
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0182495f, 39.6632156f, 1483292122, 6.44095182f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0182419f,
                                                39.6632042f, leg_index, 7.31923914f, 1698,
                                                maneuver_index, 1.4053607f, 296));
  //----------------------------------------------------------------
  // trace point = 912
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0188751f, 39.6629372f, 1483292131, 6.88951492f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0188675f,
                                                39.6629257f, leg_index, 7.2807579f, 1690,
                                                maneuver_index, 1.36687946f, 288));
  //----------------------------------------------------------------
  // trace point = 913
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0193176f, 39.6627426f, 1483292137, 7.31935406f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0193024f,
                                                39.6627274f, leg_index, 7.2538414f, 1685,
                                                maneuver_index, 1.33996296f, 283));
  //----------------------------------------------------------------
  // trace point = 914
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.019989f, 39.6624794f, 1483292146, 7.1357789f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0199661f,
                                                39.6624374f, leg_index, 7.21318483f, 1676,
                                                maneuver_index, 1.29930639f, 274));
  //----------------------------------------------------------------
  // trace point = 915
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0206985f, 39.6622963f, 1483292156, 6.44429302f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0206757f,
                                                39.6622429f, leg_index, 7.17299843f, 1667,
                                                maneuver_index, 1.25911999f, 265));
  //----------------------------------------------------------------
  // trace point = 916
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0213928f, 39.6621552f, 1483292166, 6.11528015f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0213699f,
                                                39.6620941f, leg_index, 7.1346221f, 1659,
                                                maneuver_index, 1.22074366f, 257));
  //----------------------------------------------------------------
  // trace point = 917
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0221252f, 39.6620026f, 1483292175, 7.219069f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0221024f,
                                                39.6619377f, leg_index, 7.09414577f, 1651,
                                                maneuver_index, 1.18026733f, 249));
  //----------------------------------------------------------------
  // trace point = 918
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0225754f, 39.6618919f, 1483292180, 8.11815453f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0225525f,
                                                39.6618347f, leg_index, 7.06914043f, 1645,
                                                maneuver_index, 1.15526199f, 243));
  //----------------------------------------------------------------
  // trace point = 919
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0226669f, 39.6618729f, 1483292181, 8.41951847f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.022644f,
                                                39.6618118f, leg_index, 7.06401539f, 1644,
                                                maneuver_index, 1.15013695f, 242));
  //----------------------------------------------------------------
  // trace point = 920
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0231323f, 39.6617508f, 1483292186, 8.42191601f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0231171f,
                                                39.661705f, leg_index, 7.03776741f, 1639,
                                                maneuver_index, 1.12388897f, 237));
  //----------------------------------------------------------------
  // trace point = 921
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0235138f, 39.6616554f, 1483292190, 8.56835365f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0234985f,
                                                39.6616135f, leg_index, 7.01649046f, 1634,
                                                maneuver_index, 1.10261202f, 232));
  //----------------------------------------------------------------
  // trace point = 922
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0240784f, 39.6615143f, 1483292196, 8.50470734f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0240707f,
                                                39.6614838f, leg_index, 6.98472643f, 1628,
                                                maneuver_index, 1.07084799f, 226));
  //----------------------------------------------------------------
  // trace point = 923
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0247269f, 39.6613655f, 1483292203, 8.27684879f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0247116f,
                                                39.661335f, leg_index, 6.94908047f, 1621,
                                                maneuver_index, 1.03520203f, 219));
  //----------------------------------------------------------------
  // trace point = 924
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0254745f, 39.6611786f, 1483292211, 8.37956047f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0254593f,
                                                39.6611481f, leg_index, 6.90718937f, 1611,
                                                maneuver_index, 0.993310928f, 209));
  //----------------------------------------------------------------
  // trace point = 925
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0259933f, 39.6610031f, 1483292217, 8.0647831f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0259705f,
                                                39.6609764f, leg_index, 6.87748337f, 1605,
                                                maneuver_index, 0.963604927f, 203));
  //----------------------------------------------------------------
  // trace point = 926
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0265503f, 39.660675f, 1483292224, 8.64354897f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0265274f,
                                                39.6606522f, leg_index, 6.84020376f, 1597,
                                                maneuver_index, 0.926325321f, 195));
  //----------------------------------------------------------------
  // trace point = 927
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0270309f, 39.6602554f, 1483292230, 10.3511734f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0270233f,
                                                39.6602516f, leg_index, 6.80191612f, 1589,
                                                maneuver_index, 0.888037682f, 187));
  //----------------------------------------------------------------
  // trace point = 928
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0271835f, 39.6601028f, 1483292232, 10.9058304f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0271683f,
                                                39.6600952f, leg_index, 6.78860903f, 1586,
                                                maneuver_index, 0.874730587f, 184));
  //----------------------------------------------------------------
  // trace point = 929
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0273285f, 39.6599426f, 1483292234, 10.8173141f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0273132f,
                                                39.659935f, leg_index, 6.77510834f, 1583,
                                                maneuver_index, 0.861229897f, 181));
  //----------------------------------------------------------------
  // trace point = 930
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0274658f, 39.6597862f, 1483292236, 10.5273037f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0274429f,
                                                39.6597748f, leg_index, 6.76205015f, 1580,
                                                maneuver_index, 0.848171711f, 178));
  //----------------------------------------------------------------
  // trace point = 931
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.02771f, 39.6594582f, 1483292240, 10.4389391f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0276871f,
                                                39.6594505f, leg_index, 6.73610735f, 1575,
                                                maneuver_index, 0.822228909f, 173));
  //----------------------------------------------------------------
  // trace point = 932
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0280685f, 39.6589622f, 1483292246, 10.5485296f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0280533f,
                                                39.6589584f, leg_index, 6.69687796f, 1567,
                                                maneuver_index, 0.782999516f, 165));
  //----------------------------------------------------------------
  // trace point = 933
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0281982f, 39.6587982f, 1483292248, 10.7230415f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0281754f,
                                                39.6587906f, leg_index, 6.68356609f, 1564,
                                                maneuver_index, 0.769687653f, 162));
  //----------------------------------------------------------------
  // trace point = 934
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0284729f, 39.6584702f, 1483292252, 10.8763695f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0284424f,
                                                39.6584549f, leg_index, 6.65626478f, 1558,
                                                maneuver_index, 0.742386341f, 156));
  //----------------------------------------------------------------
  // trace point = 935
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.028801f, 39.6581612f, 1483292256, 11.0705481f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0287857f,
                                                39.6581497f, leg_index, 6.62832308f, 1552,
                                                maneuver_index, 0.714444637f, 150));
  //----------------------------------------------------------------
  // trace point = 936
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0289841f, 39.6580162f, 1483292258, 11.3032951f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0289612f,
                                                39.6580009f, leg_index, 6.6144228f, 1549,
                                                maneuver_index, 0.700544357f, 147));
  //----------------------------------------------------------------
  // trace point = 937
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0292664f, 39.6578026f, 1483292261, 11.358078f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0292358f,
                                                39.6577759f, leg_index, 6.59304953f, 1545,
                                                maneuver_index, 0.679171085f, 143));
  //----------------------------------------------------------------
  // trace point = 938
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0295563f, 39.6575851f, 1483292264, 11.4712257f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0295181f,
                                                39.6575584f, leg_index, 6.57178688f, 1540,
                                                maneuver_index, 0.65790844f, 138));
  //----------------------------------------------------------------
  // trace point = 939
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0297394f, 39.6574402f, 1483292266, 11.3247471f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0297165f,
                                                39.6574211f, leg_index, 6.55757761f, 1537,
                                                maneuver_index, 0.643699169f, 135));
  //----------------------------------------------------------------
  // trace point = 940
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0299225f, 39.6573029f, 1483292268, 11.003849f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0299072f,
                                                39.6572876f, leg_index, 6.5438509f, 1534,
                                                maneuver_index, 0.629972458f, 132));
  //----------------------------------------------------------------
  // trace point = 941
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.030098f, 39.6571617f, 1483292270, 10.7646236f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.030098f,
                                                39.6571617f, leg_index, 6.53047419f, 1531,
                                                maneuver_index, 0.616595745f, 129));
  //----------------------------------------------------------------
  // trace point = 942
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0305862f, 39.6567726f, 1483292276, 10.0936251f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0305862f,
                                                39.6567726f, leg_index, 6.49301958f, 1523,
                                                maneuver_index, 0.57914114f, 121));
  //----------------------------------------------------------------
  // trace point = 943
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.031105f, 39.6563225f, 1483292283, 9.54643917f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0311127f,
                                                39.6563263f, leg_index, 6.4513154f, 1514,
                                                maneuver_index, 0.537436962f, 112));
  //----------------------------------------------------------------
  // trace point = 944
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0316315f, 39.6558647f, 1483292290, 9.72997189f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0316315f,
                                                39.6558647f, leg_index, 6.40909433f, 1505,
                                                maneuver_index, 0.495215893f, 103));
  //----------------------------------------------------------------
  // trace point = 945
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0318527f, 39.6556625f, 1483292293, 9.79928017f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0318604f,
                                                39.6556664f, leg_index, 6.39074183f, 1501,
                                                maneuver_index, 0.476863384f, 99));
  //----------------------------------------------------------------
  // trace point = 946
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0323715f, 39.6552086f, 1483292300, 9.60176468f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0323792f,
                                                39.6552124f, leg_index, 6.34891701f, 1493,
                                                maneuver_index, 0.435038567f, 91));
  //----------------------------------------------------------------
  // trace point = 947
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0328674f, 39.6547852f, 1483292307, 9.04616547f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0328674f,
                                                39.6547852f, leg_index, 6.30955267f, 1484,
                                                maneuver_index, 0.395674229f, 82));
  //----------------------------------------------------------------
  // trace point = 948
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0333328f, 39.654377f, 1483292314, 8.6725378f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0333328f,
                                                39.654377f, leg_index, 6.2719841f, 1476,
                                                maneuver_index, 0.358105659f, 74));
  //----------------------------------------------------------------
  // trace point = 949
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.033844f, 39.6539345f, 1483292322, 8.2334404f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0338364f,
                                                39.6539307f, leg_index, 6.23109055f, 1468,
                                                maneuver_index, 0.317212105f, 66));
  //----------------------------------------------------------------
  // trace point = 950
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.034317f, 39.6535225f, 1483292330, 7.64106989f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.034317f,
                                                39.6535225f, leg_index, 6.19296265f, 1460,
                                                maneuver_index, 0.279084206f, 58));
  //----------------------------------------------------------------
  // trace point = 951
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0348587f, 39.6531181f, 1483292338, 8.04931831f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0348434f,
                                                39.6531067f, leg_index, 6.1527977f, 1452,
                                                maneuver_index, 0.238919258f, 50));
  //----------------------------------------------------------------
  // trace point = 952
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0352707f, 39.6528168f, 1483292344, 8.19534874f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0352554f,
                                                39.6528053f, leg_index, 6.12253332f, 1446,
                                                maneuver_index, 0.208654881f, 44));
  //----------------------------------------------------------------
  // trace point = 953
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0357742f, 39.6524353f, 1483292351, 8.59734917f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0357666f,
                                                39.6524315f, leg_index, 6.08498621f, 1438,
                                                maneuver_index, 0.171107769f, 36));
  //----------------------------------------------------------------
  // trace point = 954
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0359344f, 39.6523209f, 1483292353, 9.52575111f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0359268f,
                                                39.6523132f, leg_index, 6.07316589f, 1435,
                                                maneuver_index, 0.159287453f, 33));
  //----------------------------------------------------------------
  // trace point = 955
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0362549f, 39.6520767f, 1483292357, 9.64165211f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0362473f,
                                                39.6520729f, leg_index, 6.04934025f, 1431,
                                                maneuver_index, 0.135461807f, 29));
  //----------------------------------------------------------------
  // trace point = 956
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0364914f, 39.6518974f, 1483292360, 9.43852043f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0364838f,
                                                39.6518936f, leg_index, 6.03166437f, 1427,
                                                maneuver_index, 0.117785931f, 25));
  //----------------------------------------------------------------
  // trace point = 957 | Alert | In 500 feet Turn right onto Red Toad Road
  maneuver_index = 13;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.036644f, 39.6517868f, 1483292362, 9.18606091f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0366364f, 39.6517792f, leg_index, 6.02032042f,
                                                1425, maneuver_index, 0.106441975f, 23,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 958
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0369568f, 39.6515579f, 1483292366, 9.11664391f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0369415f,
                                                39.6515465f, leg_index, 5.99744558f, 1420,
                                                maneuver_index, 0.0835671425f, 18));
  //----------------------------------------------------------------
  // trace point = 959
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0373535f, 39.6512566f, 1483292371, 9.57546425f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0373306f,
                                                39.6512375f, leg_index, 5.96767712f, 1413,
                                                maneuver_index, 0.0537986755f, 11));
  //----------------------------------------------------------------
  // trace point = 960 | Pre | Turn right onto Red Toad Road
  maneuver_index = 13;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0375977f, 39.6510735f, 1483292374, 9.70255375f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0375748f, 39.6510582f, leg_index, 5.94970894f,
                                                1410, maneuver_index, 0.0358304977f, 8,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 961
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0378189f, 39.650898f, 1483292377, 9.1037569f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0378113f,
                                                39.6508904f, leg_index, 5.93257523f, 1406,
                                                maneuver_index, 0.018696785f, 4));
  //----------------------------------------------------------------
  // trace point = 962
  maneuver_index = 13;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0379639f, 39.6507912f, 1483292379, 8.58696747f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0379639f,
                                                39.6507912f, leg_index, 5.92194033f, 1404,
                                                maneuver_index, 0.00806188583f, 2));
  //----------------------------------------------------------------
  // trace point = 963 Post | Continue for 6 tenths of a mile.
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0383453f, 39.6508636f, 1483292383, 8.3241272f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.0383301f, 39.6508789f, leg_index, 5.89631462f,
                                                1397, maneuver_index, 0.538780212f, 149,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 964
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0384293f, 39.6509247f, 1483292384, 10.1123877f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.038414f,
                                                39.6509361f, leg_index, 5.89034462f, 1395,
                                                maneuver_index, 0.532810211f, 147));
  //----------------------------------------------------------------
  // trace point = 965
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0385971f, 39.6510544f, 1483292386, 10.1898975f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0385971f,
                                                39.6510582f, leg_index, 5.87744522f, 1392,
                                                maneuver_index, 0.519910812f, 144));
  //----------------------------------------------------------------
  // trace point = 966
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.038765f, 39.6511803f, 1483292388, 9.98958111f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0387726f,
                                                39.6511765f, leg_index, 5.86502647f, 1388,
                                                maneuver_index, 0.507492065f, 140));
  //----------------------------------------------------------------
  // trace point = 967
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0388412f, 39.6512413f, 1483292389, 9.57158947f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0388489f,
                                                39.6512337f, leg_index, 5.85935831f, 1386,
                                                maneuver_index, 0.501823902f, 138));
  //----------------------------------------------------------------
  // trace point = 968
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0389099f, 39.6513062f, 1483292390, 9.34164524f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0389328f,
                                                39.6512947f, leg_index, 5.85318947f, 1385,
                                                maneuver_index, 0.49565506f, 137));
  //----------------------------------------------------------------
  // trace point = 969
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0390472f, 39.651432f, 1483292392, 9.20149708f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0390625f,
                                                39.6514244f, leg_index, 5.84186792f, 1382,
                                                maneuver_index, 0.484333515f, 134));
  //----------------------------------------------------------------
  // trace point = 970
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0393295f, 39.6517792f, 1483292397, 9.14202213f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0393448f,
                                                39.6517715f, leg_index, 5.81351376f, 1374,
                                                maneuver_index, 0.455979347f, 126));
  //----------------------------------------------------------------
  // trace point = 971
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.039505f, 39.6520462f, 1483292401, 8.30797863f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0395355f,
                                                39.6520309f, leg_index, 5.79288673f, 1369,
                                                maneuver_index, 0.435352325f, 121));
  //----------------------------------------------------------------
  // trace point = 972
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0396957f, 39.6523323f, 1483292406, 7.17191076f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0397339f,
                                                39.652317f, leg_index, 5.77045488f, 1362,
                                                maneuver_index, 0.412920475f, 114));
  //----------------------------------------------------------------
  // trace point = 973
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0397949f, 39.6524773f, 1483292409, 6.03242302f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0398331f,
                                                39.652462f, leg_index, 5.75911713f, 1359,
                                                maneuver_index, 0.401582718f, 111));
  //----------------------------------------------------------------
  // trace point = 974
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0398788f, 39.6526031f, 1483292412, 5.30141401f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0399246f,
                                                39.6525803f, leg_index, 5.74959373f, 1356,
                                                maneuver_index, 0.392059326f, 108));
  //----------------------------------------------------------------
  // trace point = 975
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0399704f, 39.652729f, 1483292415, 5.29900503f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0400238f,
                                                39.6527023f, leg_index, 5.73963404f, 1353,
                                                maneuver_index, 0.382099628f, 105));
  //----------------------------------------------------------------
  // trace point = 976
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0400925f, 39.6528435f, 1483292418, 5.48504305f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0401382f,
                                                39.652813f, leg_index, 5.72918558f, 1350,
                                                maneuver_index, 0.371651173f, 102));
  //----------------------------------------------------------------
  // trace point = 977
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0402679f, 39.6529922f, 1483292422, 5.55560493f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0403137f,
                                                39.6529579f, leg_index, 5.716115f, 1347,
                                                maneuver_index, 0.358580589f, 99));
  //----------------------------------------------------------------
  // trace point = 978
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0405579f, 39.6531601f, 1483292428, 5.25293112f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0405884f,
                                                39.6531143f, leg_index, 5.69782829f, 1342,
                                                maneuver_index, 0.340293884f, 94));
  //----------------------------------------------------------------
  // trace point = 979
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0408478f, 39.6532669f, 1483292434, 4.56313896f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0408783f,
                                                39.6532211f, leg_index, 5.68070793f, 1337,
                                                maneuver_index, 0.323173523f, 89));
  //----------------------------------------------------------------
  // trace point = 980
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.040947f, 39.6533012f, 1483292436, 4.50535679f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0409698f,
                                                39.6532593f, leg_index, 5.67517233f, 1335,
                                                maneuver_index, 0.31763792f, 87));
  //----------------------------------------------------------------
  // trace point = 981
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0410919f, 39.6533546f, 1483292439, 4.58563709f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0411148f,
                                                39.6533127f, leg_index, 5.66624594f, 1333,
                                                maneuver_index, 0.308711529f, 85));
  //----------------------------------------------------------------
  // trace point = 982
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0412369f, 39.6534042f, 1483292442, 4.6959691f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0412598f,
                                                39.6533585f, leg_index, 5.65827656f, 1331,
                                                maneuver_index, 0.300742149f, 83));
  //----------------------------------------------------------------
  // trace point = 983
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0416946f, 39.6535606f, 1483292451, 4.73083878f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0417175f,
                                                39.6535149f, leg_index, 5.63160563f, 1323,
                                                maneuver_index, 0.274071217f, 75));
  //----------------------------------------------------------------
  // trace point = 984
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0418854f, 39.6536255f, 1483292455, 4.52491379f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0419083f,
                                                39.6535797f, leg_index, 5.62050152f, 1320,
                                                maneuver_index, 0.26296711f, 72));
  //----------------------------------------------------------------
  // trace point = 985
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0420837f, 39.6536942f, 1483292459, 4.70419693f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0421066f,
                                                39.6536484f, leg_index, 5.60891962f, 1317,
                                                maneuver_index, 0.251385212f, 69));
  //----------------------------------------------------------------
  // trace point = 986
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0423889f, 39.6537971f, 1483292465, 4.69969606f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0424118f,
                                                39.6537552f, leg_index, 5.59106541f, 1312,
                                                maneuver_index, 0.233530998f, 64));
  //----------------------------------------------------------------
  // trace point = 987
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0426559f, 39.6538887f, 1483292470, 5.02078915f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0426788f,
                                                39.6538467f, leg_index, 5.57549858f, 1308,
                                                maneuver_index, 0.217964172f, 60));
  //----------------------------------------------------------------
  // trace point = 988
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0430908f, 39.6540413f, 1483292478, 5.13158989f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0431137f,
                                                39.6539993f, leg_index, 5.5500474f, 1301,
                                                maneuver_index, 0.192512989f, 53));
  //----------------------------------------------------------------
  // trace point = 989
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0437622f, 39.6542778f, 1483292491, 4.87062788f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0437851f,
                                                39.6542397f, leg_index, 5.51061726f, 1290,
                                                maneuver_index, 0.153082848f, 42));
  //----------------------------------------------------------------
  // trace point = 990
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0438538f, 39.6543121f, 1483292493, 4.4745121f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0438766f,
                                                39.6542702f, leg_index, 5.50530577f, 1289,
                                                maneuver_index, 0.147771358f, 41));
  //----------------------------------------------------------------
  // trace point = 991
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0441589f, 39.6544304f, 1483292500, 4.13611698f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0441895f,
                                                39.6543808f, leg_index, 5.48697472f, 1284,
                                                maneuver_index, 0.129440308f, 36));
  //----------------------------------------------------------------
  // trace point = 992
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0443192f, 39.6544876f, 1483292504, 3.73859692f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0443497f,
                                                39.6544418f, leg_index, 5.47745609f, 1281,
                                                maneuver_index, 0.119921684f, 33));
  //----------------------------------------------------------------
  // trace point = 993 | Alert | In 500 feet Turn left onto Post Road.
  maneuver_index = 14;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0444412f, 39.6545296f, 1483292507, 3.96355104f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0444717f, 39.6544838f, leg_index, 5.4703393f,
                                                1279, maneuver_index, 0.11280489f, 31,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 994
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0446243f, 39.6545944f, 1483292511, 4.32245684f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0446472f,
                                                39.6545525f, leg_index, 5.45986032f, 1276,
                                                maneuver_index, 0.102325916f, 28));
  //----------------------------------------------------------------
  // trace point = 995
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0451813f, 39.6548119f, 1483292523, 4.45237207f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0452118f,
                                                39.6547585f, leg_index, 5.42658758f, 1267,
                                                maneuver_index, 0.0690531731f, 19));
  //----------------------------------------------------------------
  // trace point = 996
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0454483f, 39.654911f, 1483292529, 4.19372082f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0454788f,
                                                39.6548576f, leg_index, 5.41079903f, 1262,
                                                maneuver_index, 0.0532646179f, 14));
  //----------------------------------------------------------------
  // trace point = 997
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0456314f, 39.6549797f, 1483292533, 4.35380077f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0456619f,
                                                39.6549263f, leg_index, 5.39995146f, 1260,
                                                maneuver_index, 0.0424170494f, 12));
  //----------------------------------------------------------------
  // trace point = 998
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0458984f, 39.6550865f, 1483292538, 5.21032f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0459366f,
                                                39.6550293f, leg_index, 5.38368368f, 1255,
                                                maneuver_index, 0.0261492729f, 7));
  //----------------------------------------------------------------
  // trace point = 999 | Pre | Turn left onto Post Road.
  maneuver_index = 14;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0460739f, 39.6551552f, 1483292541, 5.56613302f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0461121f, 39.655098f, leg_index, 5.37320137f,
                                                1252, maneuver_index, 0.0156669617f, 4,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1000
  maneuver_index = 14;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.046257f, 39.6551971f, 1483292544, 5.46605492f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0462799f,
                                                39.6551666f, leg_index, 5.36307812f, 1250,
                                                maneuver_index, 0.0055437088f, 2));
  //----------------------------------------------------------------
  // trace point = 1001
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0464554f, 39.6551666f, 1483292547, 5.79304504f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0464478f,
                                                39.655159f, leg_index, 5.352386f, 1246,
                                                maneuver_index, 0.283603668f, 86));
  //----------------------------------------------------------------
  // trace point = 1002 | Post | Continue for 3 tenths of a mile.
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0466461f, 39.6550522f, 1483292550, 6.83734083f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.0466309f, 39.6550407f, leg_index, 5.33965921f,
                                                1243, maneuver_index, 0.270876884f, 83,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1003
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0472336f, 39.6546745f, 1483292559, 7.3544488f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0472183f,
                                                39.6546593f, leg_index, 5.2987299f, 1230,
                                                maneuver_index, 0.229947567f, 70));
  //----------------------------------------------------------------
  // trace point = 1004
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0478058f, 39.6543198f, 1483292569, 6.24284983f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0477829f,
                                                39.6543007f, leg_index, 5.25970745f, 1219,
                                                maneuver_index, 0.190925121f, 59));
  //----------------------------------------------------------------
  // trace point = 1005
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0478973f, 39.6542587f, 1483292571, 5.36713409f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0478821f,
                                                39.6542435f, leg_index, 5.25311327f, 1217,
                                                maneuver_index, 0.18433094f, 57));
  //----------------------------------------------------------------
  // trace point = 1006
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0479965f, 39.6542015f, 1483292573, 5.19567108f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0479736f,
                                                39.6541824f, leg_index, 5.24666595f, 1215,
                                                maneuver_index, 0.177883625f, 55));
  //----------------------------------------------------------------
  // trace point = 1007
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0482559f, 39.6540375f, 1483292579, 4.83505583f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.048233f,
                                                39.6540184f, leg_index, 5.22878981f, 1209,
                                                maneuver_index, 0.160007477f, 49));
  //----------------------------------------------------------------
  // trace point = 1008
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0484695f, 39.653904f, 1483292585, 3.89527702f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0484467f,
                                                39.6538849f, leg_index, 5.21413708f, 1204,
                                                maneuver_index, 0.145354748f, 44));
  //----------------------------------------------------------------
  // trace point = 1009
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0485535f, 39.6538506f, 1483292588, 3.16943598f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0485382f,
                                                39.6538315f, leg_index, 5.20802212f, 1202,
                                                maneuver_index, 0.139239788f, 42));
  //----------------------------------------------------------------
  // trace point = 1010
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0486374f, 39.653801f, 1483292591, 3.03334403f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0486145f,
                                                39.6537819f, leg_index, 5.20270634f, 1201,
                                                maneuver_index, 0.133924007f, 41));
  //----------------------------------------------------------------
  // trace point = 1011
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0487213f, 39.6537476f, 1483292594, 3.03380108f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0487061f,
                                                39.6537285f, leg_index, 5.19658804f, 1199,
                                                maneuver_index, 0.12780571f, 39));
  //----------------------------------------------------------------
  // trace point = 1012
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0488052f, 39.653698f, 1483292597, 3.14654994f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.04879f,
                                                39.6536789f, leg_index, 5.19095564f, 1197,
                                                maneuver_index, 0.122173309f, 37));
  //----------------------------------------------------------------
  // trace point = 1013 | Alert | In 500 feet Turn left onto Hopewell Road.
  maneuver_index = 15;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0489044f, 39.6536369f, 1483292600, 3.4647851f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0488892f, 39.6536217f, leg_index, 5.1843586f,
                                                1195, maneuver_index, 0.115576267f, 35,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1014
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0490112f, 39.6535759f, 1483292603, 3.82563901f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.048996f,
                                                39.6535606f, leg_index, 5.17726851f, 1193,
                                                maneuver_index, 0.108486176f, 33));
  //----------------------------------------------------------------
  // trace point = 1015
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0492096f, 39.6534538f, 1483292608, 4.41419792f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.049202f,
                                                39.6534424f, leg_index, 5.16358566f, 1189,
                                                maneuver_index, 0.0948033333f, 29));
  //----------------------------------------------------------------
  // trace point = 1016
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0497665f, 39.6531181f, 1483292620, 5.04264593f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0497589f,
                                                39.6531105f, leg_index, 5.12607527f, 1177,
                                                maneuver_index, 0.0572929382f, 17));
  //----------------------------------------------------------------
  // trace point = 1017
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.050087f, 39.6529274f, 1483292627, 4.93837214f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0500717f,
                                                39.652916f, leg_index, 5.10465431f, 1171,
                                                maneuver_index, 0.0358719826f, 11));
  //----------------------------------------------------------------
  // trace point = 1018
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0503464f, 39.6527824f, 1483292633, 4.5628581f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0503311f,
                                                39.6527672f, leg_index, 5.08742428f, 1165,
                                                maneuver_index, 0.0186419487f, 5));
  //----------------------------------------------------------------
  // trace point = 1019 | Pre | Turn left onto Hopewell Road.
  maneuver_index = 15;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0504532f, 39.6527367f, 1483292636, 3.60924506f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0504303f, 39.6527138f, leg_index, 5.08000994f,
                                                1163, maneuver_index, 0.0112276077f, 3,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1020
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0505447f, 39.6527367f, 1483292640, 1.99290001f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0505447f,
                                                39.6526871f, leg_index, 5.074471f, 1162,
                                                maneuver_index, 0.0056886673f, 2));
  //----------------------------------------------------------------
  // trace point = 1021
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0506058f, 39.6527481f, 1483292643, 1.61452901f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0506058f,
                                                39.6526871f, leg_index, 5.07121992f, 1161,
                                                maneuver_index, 0.00243759155f, 1));
  //----------------------------------------------------------------
  // trace point = 1022
  maneuver_index = 15;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0506287f, 39.6527367f, 1483292644, 2.56793904f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0506287f,
                                                39.6526871f, leg_index, 5.07000113f, 1160,
                                                maneuver_index, 0.00121879578f, 0));
  //----------------------------------------------------------------
  // trace point = 1023
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.050705f, 39.6526566f, 1483292647, 3.70785403f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0506668f,
                                                39.6526489f, leg_index, 5.06602144f, 1159,
                                                maneuver_index, 1.58341217f, 386));
  //----------------------------------------------------------------
  // trace point = 1024 | TODO Missing Post | Continue for 1.6 miles.
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.051178f, 39.6522331f, 1483292657, 6.19967604f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0511703f,
                                                39.6522179f, leg_index, 5.02554417f, 1149,
                                                maneuver_index, 1.54293489f, 376));
  //----------------------------------------------------------------
  // trace point = 1025
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0514374f, 39.6520157f, 1483292661, 8.19584179f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0514297f,
                                                39.6520081f, leg_index, 5.0055089f, 1144,
                                                maneuver_index, 1.52289963f, 371));
  //----------------------------------------------------------------
  // trace point = 1026
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.051651f, 39.6518402f, 1483292664, 8.87231445f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0516357f,
                                                39.6518326f, leg_index, 4.98914814f, 1140,
                                                maneuver_index, 1.50653887f, 367));
  //----------------------------------------------------------------
  // trace point = 1027
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0519409f, 39.6516075f, 1483292668, 9.0562706f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.051918f,
                                                39.6515923f, leg_index, 4.96673775f, 1135,
                                                maneuver_index, 1.48412848f, 362));
  //----------------------------------------------------------------
  // trace point = 1028
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0521622f, 39.6514282f, 1483292671, 9.24330711f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0521393f,
                                                39.651413f, leg_index, 4.94963074f, 1131,
                                                maneuver_index, 1.46702147f, 358));
  //----------------------------------------------------------------
  // trace point = 1029
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0523911f, 39.6512451f, 1483292674, 9.3309269f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0523682f,
                                                39.6512299f, leg_index, 4.93091822f, 1126,
                                                maneuver_index, 1.44830894f, 353));
  //----------------------------------------------------------------
  // trace point = 1030
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0526886f, 39.651001f, 1483292678, 9.36530209f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0526733f,
                                                39.6509895f, leg_index, 4.90880442f, 1120,
                                                maneuver_index, 1.42619514f, 347));
  //----------------------------------------------------------------
  // trace point = 1031
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.053215f, 39.6505737f, 1483292685, 9.3559103f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0531998f,
                                                39.6505661f, leg_index, 4.86825562f, 1111,
                                                maneuver_index, 1.38564634f, 338));
  //----------------------------------------------------------------
  // trace point = 1032
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0534439f, 39.6503906f, 1483292688, 9.34340763f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0534286f,
                                                39.6503792f, leg_index, 4.8504858f, 1106,
                                                maneuver_index, 1.36787653f, 333));
  //----------------------------------------------------------------
  // trace point = 1033
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0539398f, 39.6499863f, 1483292695, 8.90608406f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0539246f,
                                                39.6499786f, leg_index, 4.8122077f, 1097,
                                                maneuver_index, 1.32959843f, 324));
  //----------------------------------------------------------------
  // trace point = 1034
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0544891f, 39.6495438f, 1483292703, 8.53040981f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0544815f,
                                                39.6495361f, leg_index, 4.76958513f, 1087,
                                                maneuver_index, 1.28697586f, 314));
  //----------------------------------------------------------------
  // trace point = 1035
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0548935f, 39.6492195f, 1483292709, 8.32334328f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0548859f,
                                                39.6492119f, leg_index, 4.73849106f, 1079,
                                                maneuver_index, 1.25588179f, 306));
  //----------------------------------------------------------------
  // trace point = 1036
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.055397f, 39.6488113f, 1483292717, 7.83520222f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0553894f,
                                                39.6488037f, leg_index, 4.69955063f, 1069,
                                                maneuver_index, 1.21694136f, 296));
  //----------------------------------------------------------------
  // trace point = 1037
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0559311f, 39.6483727f, 1483292726, 7.41256189f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0559235f,
                                                39.6483688f, leg_index, 4.65815163f, 1059,
                                                maneuver_index, 1.17554235f, 286));
  //----------------------------------------------------------------
  // trace point = 1038
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0561066f, 39.6482353f, 1483292729, 7.11707783f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0560989f,
                                                39.6482277f, leg_index, 4.64463663f, 1056,
                                                maneuver_index, 1.16202736f, 283));
  //----------------------------------------------------------------
  // trace point = 1039
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0566101f, 39.647831f, 1483292738, 6.95994186f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0566025f,
                                                39.6478233f, leg_index, 4.60589027f, 1047,
                                                maneuver_index, 1.123281f, 274));
  //----------------------------------------------------------------
  // trace point = 1040
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0567245f, 39.6477394f, 1483292740, 6.90826178f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0567093f,
                                                39.6477318f, leg_index, 4.59737682f, 1045,
                                                maneuver_index, 1.11476755f, 272));
  //----------------------------------------------------------------
  // trace point = 1041
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0572128f, 39.6473427f, 1483292748, 7.620996f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0571976f,
                                                39.6473351f, leg_index, 4.55956745f, 1035,
                                                maneuver_index, 1.07695818f, 262));
  //----------------------------------------------------------------
  // trace point = 1042
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0577011f, 39.6469498f, 1483292755, 8.61837578f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0576782f,
                                                39.6469345f, leg_index, 4.52184534f, 1026,
                                                maneuver_index, 1.03923607f, 253));
  //----------------------------------------------------------------
  // trace point = 1043
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0581436f, 39.6465874f, 1483292761, 9.23195362f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0581284f,
                                                39.6465759f, leg_index, 4.48734951f, 1018,
                                                maneuver_index, 1.00474024f, 245));
  //----------------------------------------------------------------
  // trace point = 1044
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0585098f, 39.6462784f, 1483292766, 9.35062885f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0584946f,
                                                39.6462708f, leg_index, 4.45860863f, 1011,
                                                maneuver_index, 0.975999355f, 238));
  //----------------------------------------------------------------
  // trace point = 1045
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0589447f, 39.6459045f, 1483292772, 9.28202724f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0589371f,
                                                39.6459007f, leg_index, 4.4238143f, 1003,
                                                maneuver_index, 0.941205025f, 230));
  //----------------------------------------------------------------
  // trace point = 1046
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.059433f, 39.6454849f, 1483292779, 9.0066843f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0593872f,
                                                39.6454582f, leg_index, 4.38489389f, 993,
                                                maneuver_index, 0.902284622f, 220));
  //----------------------------------------------------------------
  // trace point = 1047
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.059906f, 39.6450768f, 1483292786, 8.64114094f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0598755f,
                                                39.6450539f, leg_index, 4.34669304f, 984,
                                                maneuver_index, 0.864083767f, 211));
  //----------------------------------------------------------------
  // trace point = 1048
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0604248f, 39.6446381f, 1483292794, 8.25351715f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0603943f,
                                                39.6446152f, leg_index, 4.30565262f, 974,
                                                maneuver_index, 0.823043346f, 201));
  //----------------------------------------------------------------
  // trace point = 1049
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.060936f, 39.6441994f, 1483292802, 8.24946785f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0609055f,
                                                39.6441803f, leg_index, 4.26506042f, 964,
                                                maneuver_index, 0.782451153f, 191));
  //----------------------------------------------------------------
  // trace point = 1050
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0614471f, 39.6437607f, 1483292810, 8.16828823f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0614166f,
                                                39.6437378f, leg_index, 4.22409439f, 954,
                                                maneuver_index, 0.741485119f, 181));
  //----------------------------------------------------------------
  // trace point = 1051
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0619507f, 39.6433334f, 1483292818, 8.0497427f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0619278f,
                                                39.6433144f, leg_index, 4.18410015f, 944,
                                                maneuver_index, 0.701490879f, 171));
  //----------------------------------------------------------------
  // trace point = 1052
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0624542f, 39.6429329f, 1483292826, 7.79510117f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.062439f,
                                                39.6429176f, leg_index, 4.1454277f, 935,
                                                maneuver_index, 0.662818432f, 162));
  //----------------------------------------------------------------
  // trace point = 1053
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.062706f, 39.6427536f, 1483292830, 7.23754311f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0626831f,
                                                39.6427383f, leg_index, 4.12745857f, 930,
                                                maneuver_index, 0.6448493f, 157));
  //----------------------------------------------------------------
  // trace point = 1054
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0630493f, 39.6425018f, 1483292836, 6.789011f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0630264f,
                                                39.6424866f, leg_index, 4.10220528f, 924,
                                                maneuver_index, 0.619596004f, 151));
  //----------------------------------------------------------------
  // trace point = 1055
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0634995f, 39.6421967f, 1483292844, 6.41791916f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0634842f,
                                                39.6421852f, leg_index, 4.07011604f, 915,
                                                maneuver_index, 0.587506771f, 142));
  //----------------------------------------------------------------
  // trace point = 1056
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0638351f, 39.641983f, 1483292850, 6.24059391f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0638123f,
                                                39.6419601f, leg_index, 4.04671431f, 909,
                                                maneuver_index, 0.564105034f, 136));
  //----------------------------------------------------------------
  // trace point = 1057
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0644073f, 39.6416321f, 1483292859, 6.98832703f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0643845f,
                                                39.6416092f, leg_index, 4.00774765f, 900,
                                                maneuver_index, 0.525138378f, 127));
  //----------------------------------------------------------------
  // trace point = 1058
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0648422f, 39.6413879f, 1483292865, 7.65368891f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0648117f,
                                                39.6413574f, leg_index, 3.97909069f, 893,
                                                maneuver_index, 0.496481419f, 120));
  //----------------------------------------------------------------
  // trace point = 1059
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0654755f, 39.6410484f, 1483292874, 7.33677292f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0654526f,
                                                39.6410217f, leg_index, 3.93777561f, 883,
                                                maneuver_index, 0.45516634f, 110));
  //----------------------------------------------------------------
  // trace point = 1060
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0660934f, 39.6407394f, 1483292883, 7.03174305f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0660553f,
                                                39.6407013f, leg_index, 3.89875507f, 873,
                                                maneuver_index, 0.416145802f, 100));
  //----------------------------------------------------------------
  // trace point = 1061
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.066658f, 39.6404457f, 1483292892, 6.51072216f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0666046f,
                                                39.6403885f, leg_index, 3.8623569f, 864,
                                                maneuver_index, 0.379747629f, 91));
  //----------------------------------------------------------------
  // trace point = 1062
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0669708f, 39.6402626f, 1483292898, 5.54680395f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0669174f,
                                                39.640213f, leg_index, 3.84174252f, 859,
                                                maneuver_index, 0.359133244f, 86));
  //----------------------------------------------------------------
  // trace point = 1063
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0671768f, 39.6401482f, 1483292902, 5.538589f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0671234f,
                                                39.6400948f, leg_index, 3.82805753f, 855,
                                                maneuver_index, 0.345448256f, 82));
  //----------------------------------------------------------------
  // trace point = 1064
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0674362f, 39.6399994f, 1483292907, 5.51535177f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0673828f,
                                                39.6399422f, leg_index, 3.8106699f, 851,
                                                maneuver_index, 0.328060627f, 78));
  //----------------------------------------------------------------
  // trace point = 1065
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0677414f, 39.6398163f, 1483292913, 5.52417898f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.067688f,
                                                39.6397629f, leg_index, 3.79022384f, 846,
                                                maneuver_index, 0.307614565f, 73));
  //----------------------------------------------------------------
  // trace point = 1066
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0683365f, 39.6394577f, 1483292924, 5.90286684f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0682983f,
                                                39.6394157f, leg_index, 3.74980736f, 837,
                                                maneuver_index, 0.267198086f, 64));
  //----------------------------------------------------------------
  // trace point = 1067
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0685501f, 39.6393318f, 1483292928, 5.705235f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0685043f,
                                                39.6392899f, leg_index, 3.73579884f, 834,
                                                maneuver_index, 0.253189564f, 61));
  //----------------------------------------------------------------
  // trace point = 1068
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0688858f, 39.6391373f, 1483292935, 5.14248419f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.06884f,
                                                39.6390915f, leg_index, 3.71326137f, 829,
                                                maneuver_index, 0.230652094f, 56));
  //----------------------------------------------------------------
  // trace point = 1069
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0691452f, 39.6389809f, 1483292941, 4.73188496f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0690994f,
                                                39.6389351f, leg_index, 3.69571567f, 824,
                                                maneuver_index, 0.213106394f, 51));
  //----------------------------------------------------------------
  // trace point = 1070
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0693207f, 39.6388741f, 1483292945, 4.78951788f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0692749f,
                                                39.6388283f, leg_index, 3.68379879f, 822,
                                                maneuver_index, 0.201189518f, 49));
  //----------------------------------------------------------------
  // trace point = 1071
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0695267f, 39.6387444f, 1483292950, 4.55222511f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0694809f,
                                                39.6386986f, leg_index, 3.66962647f, 818,
                                                maneuver_index, 0.187017202f, 45));
  //----------------------------------------------------------------
  // trace point = 1072
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0697098f, 39.6386414f, 1483292954, 4.83944511f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0696564f,
                                                39.6385918f, leg_index, 3.65771198f, 816,
                                                maneuver_index, 0.175102711f, 43));
  //----------------------------------------------------------------
  // trace point = 1073
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0699539f, 39.6384888f, 1483292960, 4.49433613f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0699005f,
                                                39.6384392f, leg_index, 3.64097285f, 811,
                                                maneuver_index, 0.158363581f, 38));
  //----------------------------------------------------------------
  // trace point = 1074
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0701447f, 39.6383743f, 1483292965, 4.17098808f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0700912f,
                                                39.6383247f, leg_index, 3.62808466f, 808,
                                                maneuver_index, 0.145475388f, 35));
  //----------------------------------------------------------------
  // trace point = 1075
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0704041f, 39.6382217f, 1483292972, 4.01747799f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.070343f,
                                                39.6381683f, leg_index, 3.61085391f, 804,
                                                maneuver_index, 0.128244638f, 31));
  //----------------------------------------------------------------
  // trace point = 1076 | Alert | In 500 feet Turn left onto Jacob Tome Highway.
  maneuver_index = 16;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.070694f, 39.6380463f, 1483292979, 4.47354794f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0706253f, 39.6379852f, leg_index, 3.59119034f,
                                                799, maneuver_index, 0.108581066f, 26,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1077
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0710983f, 39.6377983f, 1483292988, 4.93660116f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.071022f,
                                                39.6377296f, leg_index, 3.56363726f, 792,
                                                maneuver_index, 0.0810279846f, 19));
  //----------------------------------------------------------------
  // trace point = 1078
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0712662f, 39.6377296f, 1483292991, 5.44870806f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0711823f,
                                                39.6376381f, leg_index, 3.55300927f, 790,
                                                maneuver_index, 0.0703999996f, 17));
  //----------------------------------------------------------------
  // trace point = 1079
  maneuver_index = 16;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0718002f, 39.6377678f, 1483292999, 5.74534988f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0718384f,
                                                39.6376762f, leg_index, 3.51646852f, 782,
                                                maneuver_index, 0.0338592529f, 9));
  //----------------------------------------------------------------
  // trace point = 1080 | Pre | Turn left onto Jacob Tome Highway, Maryland 2 76.
  maneuver_index = 16;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0721588f, 39.6378708f, 1483293006, 4.7314949f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0722046f, 39.6377792f, leg_index, 3.49569392f,
                                                776, maneuver_index, 0.01308465f, 3,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1081
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.072403f, 39.6379128f, 1483293013, 3.05847692f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0724335f,
                                                39.6378479f, leg_index, 3.48260927f, 773,
                                                maneuver_index, 1.22591972f, 288));
  //----------------------------------------------------------------
  // trace point = 1082
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0724564f, 39.6379013f, 1483293014, 4.850142f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0724335f,
                                                39.6378479f, leg_index, 3.48260927f, 773,
                                                maneuver_index, 1.22591972f, 288));
  //----------------------------------------------------------------
  // trace point = 1083 | Post | Continue on Jacob Tome Highway, Maryland 2 76 for 1.2 miles.
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0726166f, 39.6377258f, 1483293018, 5.99139404f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.0725098f, 39.6376953f, leg_index, 3.47130036f,
                                                771, maneuver_index, 1.21461082f, 286,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1084
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0729294f, 39.6372223f, 1483293027, 6.87043381f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0728607f,
                                                39.6371918f, leg_index, 3.43171263f, 761,
                                                maneuver_index, 1.17502308f, 276));
  //----------------------------------------------------------------
  // trace point = 1085
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0731277f, 39.6369514f, 1483293032, 6.91830683f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0730667f,
                                                39.6369247f, leg_index, 3.41022635f, 756,
                                                maneuver_index, 1.1535368f, 271));
  //----------------------------------------------------------------
  // trace point = 1086
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0732117f, 39.6368484f, 1483293034, 6.81978893f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.073143f,
                                                39.6368256f, leg_index, 3.4022522f, 754,
                                                maneuver_index, 1.14556265f, 269));
  //----------------------------------------------------------------
  // trace point = 1087
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0733795f, 39.6366463f, 1483293038, 6.73423815f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0733109f,
                                                39.6366119f, leg_index, 3.38497806f, 751,
                                                maneuver_index, 1.12828851f, 266));
  //----------------------------------------------------------------
  // trace point = 1088
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0736084f, 39.6364098f, 1483293043, 6.57708502f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0735397f,
                                                39.6363716f, leg_index, 3.36436462f, 746,
                                                maneuver_index, 1.10767508f, 261));
  //----------------------------------------------------------------
  // trace point = 1089
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0738525f, 39.6361694f, 1483293050, 4.80984783f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0737915f,
                                                39.6361275f, leg_index, 3.34276342f, 741,
                                                maneuver_index, 1.08607388f, 256));
  //----------------------------------------------------------------
  // trace point = 1090
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0744019f, 39.6357269f, 1483293059, 7.55054188f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0743484f,
                                                39.635685f, leg_index, 3.30011797f, 731,
                                                maneuver_index, 1.04342842f, 246));
  //----------------------------------------------------------------
  // trace point = 1091
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0744705f, 39.6356697f, 1483293060, 8.97258472f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0744247f,
                                                39.6356316f, leg_index, 3.2946229f, 729,
                                                maneuver_index, 1.03793335f, 244));
  //----------------------------------------------------------------
  // trace point = 1092
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0746994f, 39.635498f, 1483293063, 9.07981491f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.074646f,
                                                39.6354599f, leg_index, 3.27789307f, 725,
                                                maneuver_index, 1.02120352f, 240));
  //----------------------------------------------------------------
  // trace point = 1093
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0749893f, 39.6352806f, 1483293067, 8.68552208f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0749359f,
                                                39.6352386f, leg_index, 3.25615072f, 720,
                                                maneuver_index, 0.999461174f, 235));
  //----------------------------------------------------------------
  // trace point = 1094
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0753403f, 39.6350212f, 1483293072, 8.29778576f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0752792f,
                                                39.6349754f, leg_index, 3.2303443f, 714,
                                                maneuver_index, 0.973654747f, 229));
  //----------------------------------------------------------------
  // trace point = 1095
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0758743f, 39.6346092f, 1483293081, 7.23198318f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0758209f,
                                                39.6345634f, leg_index, 3.1897862f, 704,
                                                maneuver_index, 0.933096647f, 219));
  //----------------------------------------------------------------
  // trace point = 1096
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.076088f, 39.6344452f, 1483293085, 6.46476316f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0760422f,
                                                39.634407f, leg_index, 3.17378855f, 701,
                                                maneuver_index, 0.917098999f, 216));
  //----------------------------------------------------------------
  // trace point = 1097
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0761948f, 39.6343613f, 1483293087, 6.40088987f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.076149f,
                                                39.6343269f, leg_index, 3.16584754f, 699,
                                                maneuver_index, 0.909157991f, 214));
  //----------------------------------------------------------------
  // trace point = 1098
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0765915f, 39.6340675f, 1483293094, 6.71894217f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0765533f,
                                                39.6340332f, leg_index, 3.13623571f, 692,
                                                maneuver_index, 0.879546165f, 207));
  //----------------------------------------------------------------
  // trace point = 1099
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0771561f, 39.6336517f, 1483293102, 8.38817883f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0771103f,
                                                39.6336174f, leg_index, 3.0949142f, 681,
                                                maneuver_index, 0.838224649f, 196));
  //----------------------------------------------------------------
  // trace point = 1100
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0775223f, 39.6333771f, 1483293107, 8.84139633f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0774765f,
                                                39.6333427f, leg_index, 3.0676837f, 675,
                                                maneuver_index, 0.810994148f, 190));
  //----------------------------------------------------------------
  // trace point = 1101
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0780792f, 39.6329651f, 1483293115, 8.26664925f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0780334f,
                                                39.6329308f, leg_index, 3.02654529f, 665,
                                                maneuver_index, 0.769855738f, 180));
  //----------------------------------------------------------------
  // trace point = 1102
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0786285f, 39.6325607f, 1483293124, 7.18343306f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0785828f,
                                                39.6325226f, leg_index, 2.98588228f, 655,
                                                maneuver_index, 0.729192734f, 170));
  //----------------------------------------------------------------
  // trace point = 1103
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0788879f, 39.6323662f, 1483293129, 6.22278404f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0788422f,
                                                39.632328f, leg_index, 2.96659374f, 651,
                                                maneuver_index, 0.709904194f, 166));
  //----------------------------------------------------------------
  // trace point = 1104
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0791702f, 39.6321564f, 1483293137, 4.27719498f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0791245f,
                                                39.6321182f, leg_index, 2.94569492f, 646,
                                                maneuver_index, 0.689005375f, 161));
  //----------------------------------------------------------------
  // trace point = 1105
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0792694f, 39.6320839f, 1483293142, 2.29764891f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0792236f,
                                                39.6320457f, leg_index, 2.93841171f, 644,
                                                maneuver_index, 0.681722164f, 159));
  //----------------------------------------------------------------
  // trace point = 1106
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0793915f, 39.6319695f, 1483293150, 2.0178051f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.079361f,
                                                39.6319427f, leg_index, 2.92820001f, 642,
                                                maneuver_index, 0.671510458f, 157));
  //----------------------------------------------------------------
  // trace point = 1107
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0795288f, 39.6318817f, 1483293153, 5.25553513f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.079483f,
                                                39.6318474f, leg_index, 2.91893959f, 640,
                                                maneuver_index, 0.662250042f, 155));
  //----------------------------------------------------------------
  // trace point = 1108
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0799103f, 39.6315994f, 1483293159, 7.54567909f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0798645f,
                                                39.6315651f, leg_index, 2.89075708f, 633,
                                                maneuver_index, 0.634067535f, 148));
  //----------------------------------------------------------------
  // trace point = 1109
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0801392f, 39.6314354f, 1483293162, 8.8787117f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0800934f,
                                                39.6313972f, leg_index, 2.87392116f, 629,
                                                maneuver_index, 0.617231607f, 144));
  //----------------------------------------------------------------
  // trace point = 1110
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0802841f, 39.6313248f, 1483293164, 8.83354378f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0802383f,
                                                39.6312866f, leg_index, 2.86305046f, 626,
                                                maneuver_index, 0.606360912f, 141));
  //----------------------------------------------------------------
  // trace point = 1111
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0805054f, 39.6311607f, 1483293167, 8.77050972f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0804596f,
                                                39.6311226f, leg_index, 2.84668994f, 623,
                                                maneuver_index, 0.590000391f, 138));
  //----------------------------------------------------------------
  // trace point = 1112
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.08078f, 39.6309547f, 1483293171, 8.1837616f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0807343f,
                                                39.6309166f, leg_index, 2.82626653f, 618,
                                                maneuver_index, 0.569576979f, 133));
  //----------------------------------------------------------------
  // trace point = 1113
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0812225f, 39.6306229f, 1483293178, 7.56665611f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0811768f,
                                                39.6305885f, leg_index, 2.79354548f, 610,
                                                maneuver_index, 0.536855936f, 125));
  //----------------------------------------------------------------
  // trace point = 1114
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0815659f, 39.6303711f, 1483293184, 6.78964615f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0815201f,
                                                39.6303368f, leg_index, 2.76829052f, 604,
                                                maneuver_index, 0.511600971f, 119));
  //----------------------------------------------------------------
  // trace point = 1115
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0818863f, 39.6301308f, 1483293190, 6.43904781f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0818481f,
                                                39.6301003f, leg_index, 2.74435496f, 599,
                                                maneuver_index, 0.487665415f, 114));
  //----------------------------------------------------------------
  // trace point = 1116
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0821533f, 39.6299286f, 1483293195, 6.37105083f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0821152f,
                                                39.6298981f, leg_index, 2.72440481f, 594,
                                                maneuver_index, 0.467715263f, 109));
  //----------------------------------------------------------------
  // trace point = 1117
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0826492f, 39.6295738f, 1483293204, 6.42394495f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0825958f,
                                                39.6295319f, leg_index, 2.68838382f, 586,
                                                maneuver_index, 0.431694269f, 101));
  //----------------------------------------------------------------
  // trace point = 1118
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0831833f, 39.6291733f, 1483293214, 6.39599895f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0831223f,
                                                39.6291275f, leg_index, 2.64877272f, 577,
                                                maneuver_index, 0.392083168f, 92));
  //----------------------------------------------------------------
  // trace point = 1119
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0832291f, 39.6291313f, 1483293215, 6.16180182f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0831757f,
                                                39.6290894f, leg_index, 2.64489365f, 576,
                                                maneuver_index, 0.388204098f, 91));
  //----------------------------------------------------------------
  // trace point = 1120
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0837097f, 39.6287804f, 1483293224, 6.31387281f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0836487f,
                                                39.6287308f, leg_index, 2.60953498f, 568,
                                                maneuver_index, 0.35284543f, 83));
  //----------------------------------------------------------------
  // trace point = 1121
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0842514f, 39.6283684f, 1483293234, 6.52740097f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.084198f,
                                                39.6283226f, leg_index, 2.5688715f, 558,
                                                maneuver_index, 0.31218195f, 73));
  //----------------------------------------------------------------
  // trace point = 1122
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0844803f, 39.6282043f, 1483293238, 6.69391298f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0844193f,
                                                39.6281548f, leg_index, 2.55232668f, 554,
                                                maneuver_index, 0.295637131f, 69));
  //----------------------------------------------------------------
  // trace point = 1123
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0849915f, 39.6278267f, 1483293247, 6.77025795f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0849304f,
                                                39.6277771f, leg_index, 2.51459169f, 545,
                                                maneuver_index, 0.257902145f, 60));
  //----------------------------------------------------------------
  // trace point = 1124
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0855255f, 39.6274338f, 1483293256, 7.02414703f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0854568f,
                                                39.6273766f, leg_index, 2.47516823f, 536,
                                                maneuver_index, 0.21847868f, 51));
  //----------------------------------------------------------------
  // trace point = 1125
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0858231f, 39.6272049f, 1483293261, 7.19932222f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.085762f,
                                                39.6271553f, leg_index, 2.45284009f, 532,
                                                maneuver_index, 0.196150541f, 47));
  //----------------------------------------------------------------
  // trace point = 1126
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0863647f, 39.6268044f, 1483293270, 7.13055515f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0863037f,
                                                39.6267548f, leg_index, 2.41283441f, 522,
                                                maneuver_index, 0.156144857f, 37));
  //----------------------------------------------------------------
  // trace point = 1127
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0869064f, 39.6264153f, 1483293279, 7.05379295f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0868454f,
                                                39.6263618f, leg_index, 2.37319207f, 513,
                                                maneuver_index, 0.116502523f, 28));
  //----------------------------------------------------------------
  // trace point = 1128 | Alert | In 500 feet Turn right onto Waibel Road.
  maneuver_index = 17;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0874176f, 39.6260262f, 1483293288, 6.84588194f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0873642f, 39.6259804f, leg_index, 2.33497715f,
                                                504, maneuver_index, 0.0782876015f, 19,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1129
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0876923f, 39.6258163f, 1483293293, 6.6599412f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0876389f,
                                                39.6257744f, leg_index, 2.31455755f, 499,
                                                maneuver_index, 0.0578680038f, 14));
  //----------------------------------------------------------------
  // trace point = 1130 | Pre | Turn right onto Waibel Road.
  maneuver_index = 17;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0882111f, 39.625412f, 1483293303, 6.34072018f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0881653f, 39.6253738f, leg_index, 2.27512622f,
                                                489, maneuver_index, 0.0184366703f, 4,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1131
  maneuver_index = 17;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.088356f, 39.6252975f, 1483293306, 5.94188976f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0883026f,
                                                39.6252594f, leg_index, 2.26434636f, 487,
                                                maneuver_index, 0.00765681267f, 2));
  //----------------------------------------------------------------
  // trace point = 1132 | TODO missing Post | Continue for 6 tenths of a mile.
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0886536f, 39.6254883f, 1483293312, 5.49734879f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0886688f,
                                                39.6254845f, leg_index, 2.23094654f, 479,
                                                maneuver_index, 0.557644606f, 119));
  //----------------------------------------------------------------
  // trace point = 1133
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0889282f, 39.6259995f, 1483293321, 6.85661983f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0889893f,
                                                39.6259689f, leg_index, 2.1933136f, 471,
                                                maneuver_index, 0.520011663f, 111));
  //----------------------------------------------------------------
  // trace point = 1134
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0891953f, 39.6265488f, 1483293330, 7.25619698f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0892487f,
                                                39.6265373f, leg_index, 2.15149927f, 461,
                                                maneuver_index, 0.478197336f, 101));
  //----------------------------------------------------------------
  // trace point = 1135
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0892639f, 39.6268501f, 1483293334, 8.44053555f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0893173f,
                                                39.6268463f, leg_index, 2.12974954f, 457,
                                                maneuver_index, 0.456447601f, 97));
  //----------------------------------------------------------------
  // trace point = 1136
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0893097f, 39.6272926f, 1483293339, 9.94595814f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0893707f,
                                                39.627285f, leg_index, 2.09923458f, 451,
                                                maneuver_index, 0.425932646f, 91));
  //----------------------------------------------------------------
  // trace point = 1137
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0893326f, 39.6274834f, 1483293341, 10.6252832f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0894012f,
                                                39.6274757f, leg_index, 2.08594275f, 448,
                                                maneuver_index, 0.41264081f, 88));
  //----------------------------------------------------------------
  // trace point = 1138
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0893555f, 39.6276741f, 1483293343, 10.6621695f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0894394f,
                                                39.6276665f, leg_index, 2.07259607f, 445,
                                                maneuver_index, 0.399294138f, 85));
  //----------------------------------------------------------------
  // trace point = 1139
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0893936f, 39.627964f, 1483293346, 10.8214369f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0894852f,
                                                39.6279526f, leg_index, 2.05265856f, 441,
                                                maneuver_index, 0.379356623f, 81));
  //----------------------------------------------------------------
  // trace point = 1140
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0894318f, 39.6281548f, 1483293348, 10.6261559f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0895233f,
                                                39.6281433f, leg_index, 2.03931046f, 438,
                                                maneuver_index, 0.36600852f, 78));
  //----------------------------------------------------------------
  // trace point = 1141
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0894852f, 39.6283379f, 1483293350, 10.4848166f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0895538f,
                                                39.6283302f, leg_index, 2.02628565f, 435,
                                                maneuver_index, 0.352983713f, 75));
  //----------------------------------------------------------------
  // trace point = 1142
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0895309f, 39.628437f, 1483293351, 11.6764746f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0895844f,
                                                39.6284256f, leg_index, 2.01949072f, 434,
                                                maneuver_index, 0.346188784f, 74));
  //----------------------------------------------------------------
  // trace point = 1143
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0895844f, 39.6285286f, 1483293352, 10.9666805f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0896301f,
                                                39.6285019f, leg_index, 2.01367712f, 433,
                                                maneuver_index, 0.340375185f, 73));
  //----------------------------------------------------------------
  // trace point = 1144
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0896912f, 39.6286964f, 1483293354, 10.434248f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0897522f,
                                                39.6286697f, leg_index, 2.00037098f, 430,
                                                maneuver_index, 0.327069044f, 70));
  //----------------------------------------------------------------
  // trace point = 1145
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0898972f, 39.6289406f, 1483293357, 10.7999334f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0899429f,
                                                39.6289177f, leg_index, 1.97951412f, 426,
                                                maneuver_index, 0.306212187f, 66));
  //----------------------------------------------------------------
  // trace point = 1146 | Alert | In a quarter mile Turn left onto Doctor Jack Road.
  maneuver_index = 18;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.09021f, 39.6292496f, 1483293361, 10.9508133f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.0902557f, 39.6292229f, leg_index, 1.95349228f,
                                                421, maneuver_index, 0.280190349f, 61,
                                                instruction_index,
                                                kFinalMediumTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1147
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0904465f, 39.6294975f, 1483293364, 11.469636f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0905075f,
                                                39.6294632f, leg_index, 1.93213594f, 416,
                                                maneuver_index, 0.258834004f, 56));
  //----------------------------------------------------------------
  // trace point = 1148
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0906906f, 39.6297455f, 1483293367, 11.5756054f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0907593f,
                                                39.6297073f, leg_index, 1.91056049f, 412,
                                                maneuver_index, 0.237258554f, 52));
  //----------------------------------------------------------------
  // trace point = 1149
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0908432f, 39.6299057f, 1483293369, 11.0797586f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0909042f,
                                                39.6298714f, leg_index, 1.89683557f, 409,
                                                maneuver_index, 0.22353363f, 49));
  //----------------------------------------------------------------
  // trace point = 1150
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0909729f, 39.6300774f, 1483293371, 10.9225616f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0910416f,
                                                39.6300545f, leg_index, 1.88076973f, 405,
                                                maneuver_index, 0.207467794f, 45));
  //----------------------------------------------------------------
  // trace point = 1151
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0911713f, 39.6306725f, 1483293378, 9.79564667f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0912323f,
                                                39.6306648f, leg_index, 1.83860886f, 395,
                                                maneuver_index, 0.165306926f, 35));
  //----------------------------------------------------------------
  // trace point = 1152
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0913086f, 39.6312714f, 1483293386, 8.41855145f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.091362f,
                                                39.6312637f, leg_index, 1.79660845f, 387,
                                                maneuver_index, 0.123306513f, 27));
  //----------------------------------------------------------------
  // trace point = 1153
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0913773f, 39.6315918f, 1483293392, 6.01373291f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0914307f,
                                                39.6315842f, leg_index, 1.77414405f, 381,
                                                maneuver_index, 0.100842118f, 21));
  //----------------------------------------------------------------
  // trace point = 1154
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0914001f, 39.6317177f, 1483293395, 4.79489708f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0914612f,
                                                39.6317139f, leg_index, 1.76503325f, 379,
                                                maneuver_index, 0.0917313099f, 19));
  //----------------------------------------------------------------
  // trace point = 1155
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0914078f, 39.6317596f, 1483293396, 4.7254591f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0914612f,
                                                39.6317558f, leg_index, 1.7621516f, 379,
                                                maneuver_index, 0.0888496637f, 19));
  //----------------------------------------------------------------
  // trace point = 1156
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0914307f, 39.6318855f, 1483293399, 4.72545815f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0914841f,
                                                39.6318817f, leg_index, 1.75335932f, 377,
                                                maneuver_index, 0.0800573826f, 17));
  //----------------------------------------------------------------
  // trace point = 1157
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0914459f, 39.6319695f, 1483293401, 4.71377802f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0914917f,
                                                39.6319656f, leg_index, 1.74753797f, 376,
                                                maneuver_index, 0.0742360353f, 16));
  //----------------------------------------------------------------
  // trace point = 1158
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0914536f, 39.6320915f, 1483293404, 4.46825123f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.091507f,
                                                39.6320877f, leg_index, 1.7385298f, 374,
                                                maneuver_index, 0.0652278662f, 14));
  //----------------------------------------------------------------
  // trace point = 1159
  maneuver_index = 18;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0914841f, 39.6323395f, 1483293410, 4.65553522f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0915298f,
                                                39.6323357f, leg_index, 1.72186303f, 371,
                                                maneuver_index, 0.0485610962f, 11));
  //----------------------------------------------------------------
  // trace point = 1160 | Pre | Turn left onto Doctor Jack Road.
  maneuver_index = 18;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0914612f, 39.6328239f, 1483293422, 4.49372721f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.0915146f, 39.6328278f, leg_index, 1.6877352f, 364,
                                                maneuver_index, 0.0144332647f, 4, instruction_index));
  //----------------------------------------------------------------
  // trace point = 1161
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0914917f, 39.6330376f, 1483293428, 3.96495509f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0914917f,
                                                39.6330261f, leg_index, 1.67086661f, 360,
                                                maneuver_index, 1.67086661f, 360));
  //----------------------------------------------------------------
  // trace point = 1162
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0915298f, 39.6330643f, 1483293429, 4.36989212f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0915298f,
                                                39.6330299f, leg_index, 1.66882348f, 360,
                                                maneuver_index, 1.66882348f, 360));
  //----------------------------------------------------------------
  // trace point = 1163 | Post | Continue for 1.7 miles.
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0918274f, 39.63311f, 1483293433, 6.43003798f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPostTransition,
                                                -76.091835f, 39.6330414f, leg_index, 1.65254712f, 356,
                                                maneuver_index, 1.65254712f, 356, instruction_index));
  //----------------------------------------------------------------
  // trace point = 1164
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0926285f, 39.6331329f, 1483293440, 9.8421011f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0926361f,
                                                39.6330719f, leg_index, 1.60982013f, 347,
                                                maneuver_index, 1.60982013f, 347));
  //----------------------------------------------------------------
  // trace point = 1165
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0927582f, 39.6331367f, 1483293441, 11.1540632f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0927658f,
                                                39.6330795f, leg_index, 1.60289264f, 346,
                                                maneuver_index, 1.60289264f, 346));
  //----------------------------------------------------------------
  // trace point = 1166
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0930176f, 39.6331444f, 1483293443, 11.1136007f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0930176f,
                                                39.6330872f, leg_index, 1.58947158f, 343,
                                                maneuver_index, 1.58947158f, 343));
  //----------------------------------------------------------------
  // trace point = 1167
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0932693f, 39.6331558f, 1483293445, 10.733984f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0932693f,
                                                39.6330948f, leg_index, 1.57604671f, 340,
                                                maneuver_index, 1.57604671f, 340));
  //----------------------------------------------------------------
  // trace point = 1168
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0935135f, 39.6331673f, 1483293447, 10.7282085f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0935135f,
                                                39.6331024f, leg_index, 1.56303024f, 337,
                                                maneuver_index, 1.56303024f, 337));
  //----------------------------------------------------------------
  // trace point = 1169
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0937653f, 39.6331749f, 1483293449, 10.7235861f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0937653f,
                                                39.63311f, leg_index, 1.54960811f, 334,
                                                maneuver_index, 1.54960811f, 334));
  //----------------------------------------------------------------
  // trace point = 1170
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0942612f, 39.633194f, 1483293453, 10.5597191f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0942688f,
                                                39.6331291f, leg_index, 1.52275074f, 329,
                                                maneuver_index, 1.52275074f, 329));
  //----------------------------------------------------------------
  // trace point = 1171
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0949478f, 39.6332245f, 1483293459, 9.89020634f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0949554f,
                                                39.6331558f, leg_index, 1.48612654f, 321,
                                                maneuver_index, 1.48612654f, 321));
  //----------------------------------------------------------------
  // trace point = 1172
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0955276f, 39.6332436f, 1483293464, 9.87025642f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0955276f,
                                                39.6331863f, leg_index, 1.45557034f, 315,
                                                maneuver_index, 1.45557034f, 315));
  //----------------------------------------------------------------
  // trace point = 1173
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0958633f, 39.6332512f, 1483293467, 9.77689362f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0958633f,
                                                39.6331978f, leg_index, 1.43767023f, 312,
                                                maneuver_index, 1.43767023f, 312));
  //----------------------------------------------------------------
  // trace point = 1174
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0959778f, 39.6332512f, 1483293468, 9.68834972f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0959778f,
                                                39.6332016f, leg_index, 1.4315685f, 310,
                                                maneuver_index, 1.4315685f, 310));
  //----------------------------------------------------------------
  // trace point = 1175
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0964355f, 39.633255f, 1483293472, 9.68774986f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0964355f,
                                                39.633213f, leg_index, 1.40635622f, 305,
                                                maneuver_index, 1.40635622f, 305));
  //----------------------------------------------------------------
  // trace point = 1176
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0969925f, 39.6332703f, 1483293477, 9.57431316f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0969925f,
                                                39.6332283f, leg_index, 1.37747681f, 299,
                                                maneuver_index, 1.37747681f, 299));
  //----------------------------------------------------------------
  // trace point = 1177
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0977783f, 39.6333199f, 1483293484, 9.65668392f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0977859f,
                                                39.6332703f, leg_index, 1.33509517f, 290,
                                                maneuver_index, 1.33509517f, 290));
  //----------------------------------------------------------------
  // trace point = 1178
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0978851f, 39.6333275f, 1483293485, 9.33966064f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0978851f,
                                                39.6332779f, leg_index, 1.32978535f, 289,
                                                maneuver_index, 1.32978535f, 289));
  //----------------------------------------------------------------
  // trace point = 1179
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0983124f, 39.6333923f, 1483293489, 9.31310558f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0983276f,
                                                39.6333351f, leg_index, 1.30586994f, 284,
                                                maneuver_index, 1.30586994f, 284));
  //----------------------------------------------------------------
  // trace point = 1180
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0990067f, 39.6335945f, 1483293496, 9.11487675f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0990372f,
                                                39.6335564f, leg_index, 1.26496816f, 274,
                                                maneuver_index, 1.26496816f, 274));
  //----------------------------------------------------------------
  // trace point = 1181
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0996323f, 39.6338539f, 1483293503, 8.73887062f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0996552f,
                                                39.6338158f, leg_index, 1.22747064f, 266,
                                                maneuver_index, 1.22747064f, 266));
  //----------------------------------------------------------------
  // trace point = 1182
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.0997238f, 39.6338882f, 1483293504, 8.54567623f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.0997467f,
                                                39.6338501f, leg_index, 1.22204554f, 265,
                                                maneuver_index, 1.22204554f, 265));
  //----------------------------------------------------------------
  // trace point = 1183
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1004562f, 39.6341057f, 1483293512, 8.4254837f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1004791f,
                                                39.6340599f, leg_index, 1.18037975f, 256,
                                                maneuver_index, 1.18037975f, 256));
  //----------------------------------------------------------------
  // trace point = 1184
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1011276f, 39.6342812f, 1483293519, 8.67381954f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1011429f,
                                                39.6342392f, leg_index, 1.14290893f, 248,
                                                maneuver_index, 1.14290893f, 248));
  //----------------------------------------------------------------
  // trace point = 1185
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1018677f, 39.634491f, 1483293527, 8.40677643f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1018906f,
                                                39.6344414f, leg_index, 1.10069573f, 239,
                                                maneuver_index, 1.10069573f, 239));
  //----------------------------------------------------------------
  // trace point = 1186
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1025467f, 39.6346741f, 1483293535, 7.70356703f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1025696f,
                                                39.6346245f, leg_index, 1.06236994f, 231,
                                                maneuver_index, 1.06236994f, 231));
  //----------------------------------------------------------------
  // trace point = 1187
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1032028f, 39.6348648f, 1483293544, 6.70743179f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1032257f,
                                                39.6348076f, leg_index, 1.02519608f, 222,
                                                maneuver_index, 1.02519608f, 222));
  //----------------------------------------------------------------
  // trace point = 1188
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1036301f, 39.6349907f, 1483293551, 5.57739496f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1036606f,
                                                39.6349182f, leg_index, 1.00079918f, 217,
                                                maneuver_index, 1.00079918f, 217));
  //----------------------------------------------------------------
  // trace point = 1189
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1043701f, 39.6352158f, 1483293560, 7.56065798f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1044083f,
                                                39.6351357f, leg_index, 0.958218217f, 208,
                                                maneuver_index, 0.958218217f, 208));
  //----------------------------------------------------------------
  // trace point = 1190
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1050873f, 39.6354179f, 1483293566, 10.9279032f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1051254f,
                                                39.6353416f, leg_index, 0.917442083f, 200,
                                                maneuver_index, 0.917442083f, 200));
  //----------------------------------------------------------------
  // trace point = 1191
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1052094f, 39.6354523f, 1483293567, 11.7253265f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1052475f,
                                                39.6353722f, leg_index, 0.910607398f, 198,
                                                maneuver_index, 0.910607398f, 198));
  //----------------------------------------------------------------
  // trace point = 1192
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.105484f, 39.6355247f, 1483293569, 12.3286762f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1055222f,
                                                39.6354485f, leg_index, 0.895054221f, 195,
                                                maneuver_index, 0.895054221f, 195));
  //----------------------------------------------------------------
  // trace point = 1193
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1057663f, 39.635601f, 1483293571, 12.8514404f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1058044f,
                                                39.6355247f, leg_index, 0.879117668f, 192,
                                                maneuver_index, 0.879117668f, 192));
  //----------------------------------------------------------------
  // trace point = 1194
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1060486f, 39.6356735f, 1483293573, 12.7524071f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1060791f,
                                                39.635601f, leg_index, 0.863571167f, 188,
                                                maneuver_index, 0.863571167f, 188));
  //----------------------------------------------------------------
  // trace point = 1195
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1063309f, 39.635746f, 1483293575, 12.6891212f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1063614f,
                                                39.6356697f, leg_index, 0.847799122f, 185,
                                                maneuver_index, 0.847799122f, 185));
  //----------------------------------------------------------------
  // trace point = 1196
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1064682f, 39.6357841f, 1483293576, 12.5721302f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1064987f,
                                                39.635704f, leg_index, 0.840108156f, 183,
                                                maneuver_index, 0.840108156f, 183));
  //----------------------------------------------------------------
  // trace point = 1197
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1066055f, 39.6358185f, 1483293577, 12.4908972f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.106636f,
                                                39.6357384f, leg_index, 0.832417428f, 181,
                                                maneuver_index, 0.832417428f, 181));
  //----------------------------------------------------------------
  // trace point = 1198
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1067429f, 39.6358566f, 1483293578, 12.4908915f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.106781f,
                                                39.6357727f, leg_index, 0.824338436f, 180,
                                                maneuver_index, 0.824338436f, 180));
  //----------------------------------------------------------------
  // trace point = 1199
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1068802f, 39.635891f, 1483293579, 12.4097195f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1069183f,
                                                39.635807f, leg_index, 0.816647708f, 178,
                                                maneuver_index, 0.816647708f, 178));
  //----------------------------------------------------------------
  // trace point = 1200
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1071472f, 39.6359634f, 1483293581, 12.0452452f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1071854f,
                                                39.6358757f, leg_index, 0.801646888f, 175,
                                                maneuver_index, 0.801646888f, 175));
  //----------------------------------------------------------------
  // trace point = 1201
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.107666f, 39.6361046f, 1483293585, 11.7454538f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1077042f,
                                                39.6360207f, leg_index, 0.772248089f, 169,
                                                maneuver_index, 0.772248089f, 169));
  //----------------------------------------------------------------
  // trace point = 1202
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1081848f, 39.6362495f, 1483293589, 11.8421268f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.108223f,
                                                39.6361504f, leg_index, 0.743174791f, 162,
                                                maneuver_index, 0.743174791f, 162));
  //----------------------------------------------------------------
  // trace point = 1203
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1085434f, 39.6363487f, 1483293592, 10.9487839f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1085968f,
                                                39.6362457f, leg_index, 0.722158432f, 158,
                                                maneuver_index, 0.722158432f, 158));
  //----------------------------------------------------------------
  // trace point = 1204
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1087723f, 39.6364136f, 1483293594, 10.3617077f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1088257f,
                                                39.6363182f, leg_index, 0.708976448f, 155,
                                                maneuver_index, 0.708976448f, 155));
  //----------------------------------------------------------------
  // trace point = 1205
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1094742f, 39.6366234f, 1483293601, 9.26880932f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.10952f,
                                                39.6365395f, leg_index, 0.66894865f, 146,
                                                maneuver_index, 0.66894865f, 146));
  //----------------------------------------------------------------
  // trace point = 1206
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.109726f, 39.6366997f, 1483293604, 7.64028502f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1097717f,
                                                39.6366234f, leg_index, 0.654330969f, 143,
                                                maneuver_index, 0.654330969f, 143));
  //----------------------------------------------------------------
  // trace point = 1207
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1099319f, 39.6367645f, 1483293607, 6.30874014f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1099701f,
                                                39.6366844f, leg_index, 0.642938018f, 141,
                                                maneuver_index, 0.642938018f, 141));
  //----------------------------------------------------------------
  // trace point = 1208
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1100616f, 39.6368065f, 1483293609, 6.20125103f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1100998f,
                                                39.6367226f, leg_index, 0.635542512f, 139,
                                                maneuver_index, 0.635542512f, 139));
  //----------------------------------------------------------------
  // trace point = 1209
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1102066f, 39.6368484f, 1483293611, 6.42080402f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1102448f,
                                                39.6367645f, leg_index, 0.627293289f, 137,
                                                maneuver_index, 0.627293289f, 137));
  //----------------------------------------------------------------
  // trace point = 1210
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1103516f, 39.6368904f, 1483293613, 6.6403389f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1103897f,
                                                39.6368065f, leg_index, 0.619044721f, 135,
                                                maneuver_index, 0.619044721f, 135));
  //----------------------------------------------------------------
  // trace point = 1211
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1110306f, 39.6370811f, 1483293621, 7.77713394f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1110687f,
                                                39.6370049f, leg_index, 0.580352366f, 127,
                                                maneuver_index, 0.580352366f, 127));
  //----------------------------------------------------------------
  // trace point = 1212
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1116714f, 39.63731f, 1483293627, 10.1089849f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1117249f,
                                                39.6372414f, leg_index, 0.541673005f, 118,
                                                maneuver_index, 0.541673005f, 118));
  //----------------------------------------------------------------
  // trace point = 1213
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1117783f, 39.6373596f, 1483293628, 10.4377518f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1118393f,
                                                39.6372948f, leg_index, 0.534541368f, 117,
                                                maneuver_index, 0.534541368f, 117));
  //----------------------------------------------------------------
  // trace point = 1214
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.111969f, 39.6374664f, 1483293630, 10.2260284f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.11203f,
                                                39.6374016f, leg_index, 0.521979213f, 114,
                                                maneuver_index, 0.521979213f, 114));
  //----------------------------------------------------------------
  // trace point = 1215
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1121521f, 39.6375771f, 1483293632, 10.0182972f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1122208f,
                                                39.6375084f, leg_index, 0.509419978f, 111,
                                                maneuver_index, 0.509419978f, 111));
  //----------------------------------------------------------------
  // trace point = 1216
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1127472f, 39.6379089f, 1483293639, 8.95882797f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1128006f,
                                                39.6378441f, leg_index, 0.470766544f, 102,
                                                maneuver_index, 0.470766544f, 102));
  //----------------------------------------------------------------
  // trace point = 1217
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.113266f, 39.6381378f, 1483293647, 6.40037918f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1133041f,
                                                39.6380501f, leg_index, 0.44028607f, 95,
                                                maneuver_index, 0.44028607f, 95));
  //----------------------------------------------------------------
  // trace point = 1218
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1134109f, 39.6381912f, 1483293650, 4.64441681f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.113472f,
                                                39.6380997f, leg_index, 0.430701047f, 93,
                                                maneuver_index, 0.430701047f, 93));
  //----------------------------------------------------------------
  // trace point = 1219
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1134949f, 39.6382217f, 1483293652, 3.98077798f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1135406f,
                                                39.6381264f, leg_index, 0.426590979f, 92,
                                                maneuver_index, 0.426590979f, 92));
  //----------------------------------------------------------------
  // trace point = 1220
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1136246f, 39.6382523f, 1483293655, 3.92717099f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1136627f,
                                                39.6381607f, leg_index, 0.419285625f, 90,
                                                maneuver_index, 0.419285625f, 90));
  //----------------------------------------------------------------
  // trace point = 1221
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1138687f, 39.6382942f, 1483293659, 5.36444902f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1138992f,
                                                39.6382065f, leg_index, 0.406681895f, 88,
                                                maneuver_index, 0.406681895f, 88));
  //----------------------------------------------------------------
  // trace point = 1222
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.114563f, 39.6383858f, 1483293668, 6.71286917f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1145859f,
                                                39.638298f, leg_index, 0.369134039f, 79,
                                                maneuver_index, 0.369134039f, 79));
  //----------------------------------------------------------------
  // trace point = 1223
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1147232f, 39.6384087f, 1483293670, 6.76594496f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1147461f,
                                                39.6383209f, leg_index, 0.360853225f, 77,
                                                maneuver_index, 0.360853225f, 77));
  //----------------------------------------------------------------
  // trace point = 1224
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1154404f, 39.6385231f, 1483293679, 7.02770281f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1154633f,
                                                39.6384354f, leg_index, 0.321835756f, 69,
                                                maneuver_index, 0.321835756f, 69));
  //----------------------------------------------------------------
  // trace point = 1225
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.116127f, 39.6386719f, 1483293687, 7.61248684f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1161728f,
                                                39.6385841f, leg_index, 0.282576919f, 61,
                                                maneuver_index, 0.282576919f, 61));
  //----------------------------------------------------------------
  // trace point = 1226
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1165466f, 39.6387825f, 1483293691, 9.56411266f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1165848f,
                                                39.63871f, leg_index, 0.258966684f, 56,
                                                maneuver_index, 0.258966684f, 56));
  //----------------------------------------------------------------
  // trace point = 1227
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1168594f, 39.6388817f, 1483293694, 9.6285038f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1168976f,
                                                39.6388092f, leg_index, 0.240944773f, 52,
                                                maneuver_index, 0.240944773f, 52));
  //----------------------------------------------------------------
  // trace point = 1228
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1170578f, 39.6389465f, 1483293696, 9.3046608f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1170959f,
                                                39.6388741f, leg_index, 0.229466975f, 50,
                                                maneuver_index, 0.229466975f, 50));
  //----------------------------------------------------------------
  // trace point = 1229
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1172562f, 39.6390114f, 1483293698, 9.16454315f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1172943f,
                                                39.6389465f, leg_index, 0.217771247f, 48,
                                                maneuver_index, 0.217771247f, 48));
  //----------------------------------------------------------------
  // trace point = 1230
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1178207f, 39.6391983f, 1483293704, 8.7528286f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1178513f,
                                                39.6391487f, leg_index, 0.184974447f, 41,
                                                maneuver_index, 0.184974447f, 41));
  //----------------------------------------------------------------
  // trace point = 1231
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1185074f, 39.6394272f, 1483293712, 8.0425663f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1185303f,
                                                39.6393776f, leg_index, 0.145485118f, 31,
                                                maneuver_index, 0.145485118f, 31));
  //----------------------------------------------------------------
  // trace point = 1232 | Alert | In 500 feet You will arrive at your destination.
  maneuver_index = 19;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1192093f, 39.6396484f, 1483293721, 7.21953201f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTransitionAlert,
                                                -76.1192322f, 39.6395988f, leg_index, 0.105086394f,
                                                23, maneuver_index, 0.105086394f, 23,
                                                instruction_index,
                                                kFinalShortTransitionAlertImperialLength));
  //----------------------------------------------------------------
  // trace point = 1233
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1192856f, 39.6396751f, 1483293722, 7.32172489f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1193161f,
                                                39.6396217f, leg_index, 0.100349098f, 22,
                                                maneuver_index, 0.100349098f, 22));
  //----------------------------------------------------------------
  // trace point = 1234
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1199265f, 39.6399002f, 1483293730, 7.5503788f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.119957f,
                                                39.6398582f, leg_index, 0.0624777824f, 13,
                                                maneuver_index, 0.0624777824f, 13));
  //----------------------------------------------------------------
  // trace point = 1235 | Pre | You have arrived at your destination.
  maneuver_index = 19;
  instruction_index = maneuver_index + 1;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1203766f, 39.6400948f, 1483293735, 8.86098289f),
                            GetNavigationStatus(NavigationStatus_RouteState_kPreTransition,
                                                -76.1204071f, 39.6400566f, leg_index, 0.0348483212f,
                                                7, maneuver_index, 0.0348483212f, 7,
                                                instruction_index));
  //----------------------------------------------------------------
  // trace point = 1236
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1206665f, 39.6402206f, 1483293738, 9.49639797f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.120697f,
                                                39.6401787f, leg_index, 0.0172450561f, 4,
                                                maneuver_index, 0.0172450561f, 4));
  //----------------------------------------------------------------
  // trace point = 1237
  maneuver_index = 19;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1207657f, 39.6402626f, 1483293739, 9.33547211f),
                            GetNavigationStatus(NavigationStatus_RouteState_kTracking, -76.1207962f,
                                                39.6402206f, leg_index, 0.0112175941f, 2,
                                                maneuver_index, 0.0112175941f, 2));
  //----------------------------------------------------------------
  // trace point = 1238 | Complete
  maneuver_index = 20;
  instruction_index = maneuver_index;
  TryRouteOnLocationChanged(nav, GetFixLocation(-76.1209488f, 39.6403427f, 1483293741, 8.98219967f),
                            GetNavigationStatus(NavigationStatus_RouteState_kComplete, -76.1209793f,
                                                39.6403008f, leg_index, 0.f, 0, maneuver_index, 0.f,
                                                0));
}

void TryIsTimeWithinBounds(bool found, bool expected) {

  if (found != expected)
    throw std::runtime_error(
        "Incorrect IsTimeWithinBounds value - found: " + std::string(found ? "true" : "false") +
        " | expected: " + std::string(expected ? "true" : "false"));
}

void TestIsTimeWithinBounds() {
  NavigatorTest nav;
  TryIsTimeWithinBounds(nav.IsTimeWithinBounds(1, 2, 6), false);
  TryIsTimeWithinBounds(nav.IsTimeWithinBounds(2, 2, 6), false);
  TryIsTimeWithinBounds(nav.IsTimeWithinBounds(3, 2, 6), true);
  TryIsTimeWithinBounds(nav.IsTimeWithinBounds(4, 2, 6), true);
  TryIsTimeWithinBounds(nav.IsTimeWithinBounds(5, 2, 6), true);
  TryIsTimeWithinBounds(nav.IsTimeWithinBounds(6, 2, 6), false);
  TryIsTimeWithinBounds(nav.IsTimeWithinBounds(20, 2, 6), false);
}

void CreateTestSeed() {
  std::string route_json_str =
      R"({"trip":{"language":"en-US","summary":{"max_lon":-75.985947,"max_lat":39.746597,"time":6389,"length":27.065,"min_lat":39.625179,"min_lon":-76.122726},"locations":[{"lon":-76.122696,"lat":39.651386,"type":"break"},{"lon":-76.120949,"lat":39.640343,"type":"break"}],"units":"miles","legs":[{"shape":"accsjAledepCuKgDsJcF{KgEgc@oR{KuE_O_I}EiCqV}JmKcF{JwDgNgDiSqGsd@yLqWqHsAm@eEkAcB]wHiCcGyByB{@uYeP}@m@}EiCsFwCoHuDkG{BqGiB}JiB}NiCqC]ms@oIkFm@gJ}@mYeD{KkBwH{AiNeEwR_Iwc@aRqa@gOuT_IwS}JcQ{JcFwDkVwNaHgD}EkA{FiBqLiCsKgCgIkBwIm@sj@{@yGOkGm@}DM{Fm@iHkAwH{AqGkBqMeEwIuD_D{AuOaHiLcFmKeF_D{Am@]qQoImKcFaW{K}@m@kKuEkV{KmD{AmA{@{JuE_JeEqMeFwXyL{EiBiMsF_IgDsGyC{EiBmKuEwMqGyB}@e@]qCiBuDgDwCwDgDeEaDqGqAgEsAsFkBoSiCag@oCmi@cAcP{As[_D}}@yCq{@{K_yCuDecAsKelCoIamCqBen@cBgc@cA}^]yM?{K~BgNxCsQTgD\\cFNuE]{KsGey@gDaf@cAePyG{sA}Em~@m@mIoH{tAcAcPiCee@}@qRyLqcC]mI}@gn@cAk`@aC}^}Dq{@aC{i@aCil@u@aSyBco@kAoTMwCOuEGuDm@sQMmJWqHkAmi@]uOGgC_DogAcBwm@iBso@]iL{Awc@sGqdBqBse@{@sPyBikAmFy_ByAc[}@aR{@eO}@aSU}I}@{UiBag@uEm~@iBk`@sAiWeFazA{@e[gEyk@iB_^{A_]m@oSyB__@aCse@aBaf@}Dyl@sBed@cAwMwCmTwDiNcFmJ{LkUkFoIiBuEkBsFaBqHyBgN{A}SwCmi@_@aHiBya@sAil@sAq\\sAkVgDqq@oCso@m@kKaC}i@cBud@aBsd@OeFsAwXyBed@{@eP{A_SmE_h@_DuZe@uDcBsPsAsQUeE_Dm@aC]eAOsAOkA{@aCyBsEwDgDgCgJsG{PiMgCkAcLoIyBkA{FgCeEyBgNcGqMsF}n@{UaMeFyLcFqMeFkKgDyLaG}PaHsKwDiGkAeF]sFz@yGjBkKdEsFhBab@dPkQpGk[xMgCz@ab@vNib@fNuJfDgh@rPcFzAkGjAmEl@kF\\{EM{GkAyGm@wMeEwXmKiXiL_X{LiXyL}XiM}OaHmU{JkKuEmE{AqB}@|Daf@pCwb@tDah@jA_SxBkVjB_ShBuOjAoIdFmUz@wCfDmJvC_I~IaSzEkKlKaSpRm^`R_^hHiM`ByB|EaH|JiLeAaHs@uF}@{KyBq\\cAsPmA_SoCm_@}@}Jk@oIGwCOuE]mJNiLDyBN_Jd@gNl@gY\\_It@qQbAq]t@a\\l@_^zAib@j@kUFwN?cFU{Lu@{KaBo]gEk_AyBse@yGscBG{AeOkpDiB_h@WcQkAcPcAsQsA}IgDuOqC_JmDaHmAyBiG{KiCgDgCeEaCcFcBuEqBsGsAqGu@sF}I_s@kBsPcA_I{AkLaB{KiR_fAaNsp@{E{U}@gDqBuEsAiCsAiBiCiCkKmJkVoRiHeFgDyBs_@wNkKgDaDM}N?uJl@k`@hBc`@?u_@?c`@?yC]sEkBmKaH_ImImEuFoDsEqBwD_D_IyBqGoDiM}DgNwCmKkBqGyBsFqByC}EsE{FsFgIaHuJsG{ZqQg]aSaXeOqWyMuEyB{~@qQ_{AkV_rA_T_^sFif@{KyC{@yBm@qB]uE?{Jj@{`@xC{`@fCy`@xBe`@xBqa@jBc`@hBuT|@eFl@qGOkKl@ud@hCiq@dE}n@fDyp@fDso@fDaSxBw\\xA}Ol@kiBnI}}@bGm^fCouBzLmJl@_lArFqpA`GwDNw\\hB{L|@}`CdOqp@vDwqA`HsqCtOcdAbF{zDnT}YjAm_@\\}Dl@eFhBo]`S_YoIuJyBwC]qG?}J\\_c@dEmENyHO{P{AaMyAwg@_JmPeEwX_I}ScGiw@sQyQwCqGOqCN_DjAmEhCoeBdwAkAzA_@\\qBzAgDjA}Dl@yBMiBm@_YyMueD_fB}YsPqRiMo|@un@p_BmmEdUwm@nm@g`BnN}_@bp@{hBpk@eaBfw@eaCfD{KvDkLlD_IfIuOzVya@jyAkgCbVwb@flAwuBnIgNdi@ux@rAkBrj@y_AhGwNnNuc@b`@oqA|EsQlEaR~CmTbBiM~Hl@fNOtjDkLzJMdFL`C^|DhBhCxBnDtE~CrFz_@fx@pC`HlEfN~BzJnJre@v\\nfB~CdO`Ifc@~M`q@jFzUxCnIpBdE`G|JtOlUlKpQlElKhH`RrAhBxBjBfDhBdE?xdAcGvYkA~H]tD]`I_@|D?piByKdj@kBh\\kArUiBnYmAjj@gCveBoIbGOhk@iBn}@iCl`CaHxRm@r_@iB|wAwCxHOrnAgDpB?fhDmJnIO`HMr`@}@rV{@`MLhMl@x\\hClm@tD~N|@vb@hBzVjA|^zBdZz@bP]dFkBxGm@pGMfx@kBx{@uDv{@eFb{@eEtm@kAh|@uEh{@uDjy@yCvw@oHbk@wDzAdFxB~HpGxVlKvc@hL`g@rL|h@pLnh@nIz`@hBm@jB?tTkAje@}@be@Ode@k@dd@?ld@z@rd@hCxkA\\zd@?dd@l@dd@?zL}@vC{@zAm@nDiCdEeEbe@yk@jLwNrPcQrPuO~SuOre@y`@fSwOjL{KtDsEdF{K~Rse@|JiXxBcFnDqGtTya@~b@uy@`b@ey@`eA_pBhb@gw@jQa]zOc[tJsPpRiWzKiMdKkLrK{J`BmAxHmItDeFbB|@~WrPnJbGtDvCbBhBjA|@l@z@rAxBjAfEzA~HxBhM`Hfc@nHbe@rGvb@`B~HzAtEbAxBjAhC~S|^|JrQtEzJ|DnIvCpHdFxLbLtYnN|_@bKb[xB`H`CfNtJzj@zFdZfDhMpGpQzK|_@bGbZlJre@fNpq@pLxl@|E~RpMxa@x[xjAbLha@vXp{@vNtd@vb@huAtTjt@tT`r@xGjU`IzUbo@xtBzQtn@fNbe@dExLz@hCzGrPfHbQbf@xkAxVvl@jLfXtY~r@|Ylt@tZzs@fDrGnIdZt^fsCpGli@t@nI~ClJ|EvNb`@|gA|EbQxBzL`CtOvC|TpBrP~Hdn@`XvjBdEtZnCpQrGde@~C~RrFzV~CxLhHrQjK`Q`MbQrKhM|T`RjL~IlTfNdZnSbLpGhHbGzKjLlZl_@jUp\\tZng@jZz_@|_DrvDlP`R`a@li@l~@huAb[dd@z[tc@tOlUlE`HzKpRiXbe@_IhMaMxLyGdF}J`HwMnIcRzKgIrF_I`GkFrFgDvDoDrFwCrFwDxLoHfYeKvb@gI`]}Nvl@q]ztAmJ|]cQbp@_Nbf@wClJdJrPjPdZfOjV|I`RvX~g@fItOjGxMfSha@pHfO\\j@pBfEjFlJpHdOvC`Hd@xB?hC?vCvDjAbGpHnNnS|TtYvRjVvSvXnSfYjp@n{@|s@haAllAj~A`MrPlZz`@p\\fb@rVp\\|IjLpHlJpQ|ThHnIxCvChLlKrLtO~\\vb@zPlT`CfDfDdE`ChCbVfYdZ|^fIzK`RhWpHhLvWz`@bLnSpMlTxBvD`Wpf@~NdZbAhCpH`RbFhMjLfXfIdPpVli@`NvXlOb[bVpg@j[tm@z[fn@bUvb@dUx`@|EzKLtO{EtZyGp\\bKdElKrFvRfNpRhNvChBnI`H`RrPvNrPzF`IfNnSnaAvuA~Xfc@ba@xl@~|D`~Fl_@jk@n]ng@|OnTdx@fkArVn^jQxWzAxBtJfNdO|Tjj@bz@~IfNvHzKpCdEhWj`@bFpHdP|TpBfCjF`HiG`Ia]pQ}IbGcMpGiVpGmF|@{PjAgIN}h@rFia@dEiCl@wCzAoTfNgDvC}IlJyMfNmEtEaH`HmEdFeJ~HuDfD}ExBmEhBoD|@yWdE_NhBuJjAsPxBqRhCoNjAgI\\sUlAgOm@uI_@yB]sA]eA}@_C`eBm@`q@mA`|@{@vb@e@vb@OlTGtEEbQWtPe@tYUhLe@zLm@xL{AbQyBxLiCxL{EdPqHzU{AdEaBbGsAtEcB~H}I~g@w]lqB{Kfm@gIrf@wTjhAqQddAyHpf@gCrP}O`|@mEp\\gIl^sFxWgDfOwChL{G~]eD`R}Jbf@qB|IyB`IiC`H}ExKoMhXuE|JeFzJ}D|JwClJ{ArFcA`HcBbGkA`He@bFkA~I_@`H]~Hm@pHu@|I_Dz`@mDj_@_DhXuE~S}EzUqGvXmOhl@u@fCqHr[oSh`AeJp\\oDzKwHnT_@jA{Jb[","summary":{"max_lon":-75.985947,"max_lat":39.746597,"time":6389,"length":27.065,"min_lat":39.625179,"min_lon":-76.122726},"maneuvers":[{"travel_mode":"bicycle","begin_shape_index":0,"length":3.602,"time":906,"type":1,"end_shape_index":157,"instruction":"Bike north on Liberty Grove Road.","verbal_pre_transition_instruction":"Bike north on Liberty Grove Road for 3.6 miles.","travel_type":"road","street_names":["Liberty Grove Road"]},{"travel_type":"road","verbal_pre_transition_instruction":"Continue on Barnes Corner Road for 1.4 miles.","verbal_transition_alert_instruction":"Continue on Barnes Corner Road.","length":1.399,"instruction":"Continue on Barnes Corner Road.","end_shape_index":213,"type":8,"time":341,"street_names":["Barnes Corner Road"],"begin_shape_index":157,"travel_mode":"bicycle"},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Montgomery Road.","verbal_transition_alert_instruction":"Turn left onto Montgomery Road.","length":0.941,"instruction":"Turn left onto Montgomery Road.","end_shape_index":270,"type":15,"time":228,"verbal_post_transition_instruction":"Continue for 9 tenths of a mile.","street_names":["Montgomery Road"],"begin_shape_index":213},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Biggs Highway, Maryland 2 74.","verbal_transition_alert_instruction":"Turn right onto Biggs Highway.","length":0.399,"instruction":"Turn right onto Biggs Highway\/MD 274.","end_shape_index":291,"type":10,"time":94,"verbal_post_transition_instruction":"Continue for 4 tenths of a mile.","street_names":["Biggs Highway","MD 274"],"begin_shape_index":270},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Wilson Road.","verbal_transition_alert_instruction":"Turn left onto Wilson Road.","length":3.444,"instruction":"Turn left onto Wilson Road.","end_shape_index":430,"type":15,"time":793,"verbal_post_transition_instruction":"Continue for 3.4 miles.","street_names":["Wilson Road"],"begin_shape_index":291},{"travel_type":"road","verbal_pre_transition_instruction":"Continue on Stoney Lane for 1.8 miles.","verbal_transition_alert_instruction":"Continue on Stoney Lane.","length":1.798,"instruction":"Continue on Stoney Lane.","end_shape_index":474,"type":8,"time":459,"street_names":["Stoney Lane"],"begin_shape_index":430,"travel_mode":"bicycle"},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto East Christine Road, Pennsylvania 2 72.","verbal_transition_alert_instruction":"Turn right onto East Christine Road.","length":1.492,"instruction":"Turn right onto East Christine Road\/PA 272.","end_shape_index":500,"type":10,"time":334,"verbal_post_transition_instruction":"Continue for 1.5 miles.","street_names":["East Christine Road","PA 272"],"begin_shape_index":474},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto McCoury Road.","verbal_transition_alert_instruction":"Turn right onto McCoury Road.","length":1.142,"instruction":"Turn right onto McCoury Road.","end_shape_index":542,"type":10,"time":229,"verbal_post_transition_instruction":"Continue for 1.1 miles.","street_names":["McCoury Road"],"begin_shape_index":500},{"travel_type":"road","verbal_pre_transition_instruction":"Continue on Chandlee Road for 1.3 miles.","verbal_transition_alert_instruction":"Continue on Chandlee Road.","length":1.250,"instruction":"Continue on Chandlee Road.","end_shape_index":569,"type":8,"time":297,"street_names":["Chandlee Road"],"begin_shape_index":542,"travel_mode":"bicycle"},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Bear left onto Thankless Lane.","verbal_transition_alert_instruction":"Bear left onto Thankless Lane.","length":0.657,"instruction":"Bear left onto Thankless Lane.","end_shape_index":582,"type":16,"time":163,"verbal_post_transition_instruction":"Continue for 7 tenths of a mile.","street_names":["Thankless Lane"],"begin_shape_index":569},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Calvert Road.","verbal_transition_alert_instruction":"Turn right onto Calvert Road.","length":0.218,"instruction":"Turn right onto Calvert Road.","end_shape_index":590,"type":10,"time":52,"verbal_post_transition_instruction":"Continue for 2 tenths of a mile.","street_names":["Calvert Road"],"begin_shape_index":582},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto England Creamery Road.","verbal_transition_alert_instruction":"Turn left onto England Creamery Road.","length":1.430,"instruction":"Turn left onto England Creamery Road.","end_shape_index":637,"type":15,"time":335,"verbal_post_transition_instruction":"Continue for 1.4 miles.","street_names":["England Creamery Road"],"begin_shape_index":590},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Crothers Road.","verbal_transition_alert_instruction":"Turn right onto Crothers Road.","length":1.767,"instruction":"Turn right onto Crothers Road.","end_shape_index":700,"type":10,"time":424,"verbal_post_transition_instruction":"Continue for 1.8 miles.","street_names":["Crothers Road"],"begin_shape_index":637},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Bear right onto Principio Road.","verbal_transition_alert_instruction":"Bear right onto Principio Road.","length":1.612,"instruction":"Bear right onto Principio Road.","end_shape_index":744,"type":9,"time":338,"verbal_post_transition_instruction":"Continue for 1.6 miles.","street_names":["Principio Road"],"begin_shape_index":700},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Red Toad Road.","verbal_transition_alert_instruction":"Turn right onto Red Toad Road.","length":0.556,"instruction":"Turn right onto Red Toad Road.","end_shape_index":767,"type":10,"time":154,"verbal_post_transition_instruction":"Continue for 6 tenths of a mile.","street_names":["Red Toad Road"],"begin_shape_index":744},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Post Road.","verbal_transition_alert_instruction":"Turn left onto Post Road.","length":0.289,"instruction":"Turn left onto Post Road.","end_shape_index":784,"type":15,"time":90,"verbal_post_transition_instruction":"Continue for 3 tenths of a mile.","street_names":["Post Road"],"begin_shape_index":767},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Hopewell Road.","verbal_transition_alert_instruction":"Turn left onto Hopewell Road.","length":1.587,"instruction":"Turn left onto Hopewell Road.","end_shape_index":838,"type":15,"time":386,"verbal_post_transition_instruction":"Continue for 1.6 miles.","street_names":["Hopewell Road"],"begin_shape_index":784},{"travel_type":"road","travel_mode":"bicycle","end_shape_index":870,"verbal_pre_transition_instruction":"Turn left onto Jacob Tome Highway, Maryland 2 76.","begin_street_names":["Jacob Tome Highway","MD 276","MD 222 Truck"],"verbal_transition_alert_instruction":"Turn left onto Jacob Tome Highway.","length":1.226,"instruction":"Turn left onto Jacob Tome Highway\/MD 276\/MD 222 Truck. Continue on Jacob Tome Highway\/MD 276.","type":15,"time":290,"verbal_post_transition_instruction":"Continue on Jacob Tome Highway, Maryland 2 76 for 1.2 miles.","street_names":["Jacob Tome Highway","MD 276"],"begin_shape_index":838},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn right onto Waibel Road.","verbal_transition_alert_instruction":"Turn right onto Waibel Road.","length":0.583,"instruction":"Turn right onto Waibel Road.","end_shape_index":907,"type":10,"time":121,"verbal_post_transition_instruction":"Continue for 6 tenths of a mile.","street_names":["Waibel Road"],"begin_shape_index":870},{"travel_type":"road","travel_mode":"bicycle","verbal_pre_transition_instruction":"Turn left onto Doctor Jack Road.","verbal_transition_alert_instruction":"Turn left onto Doctor Jack Road.","length":1.674,"instruction":"Turn left onto Doctor Jack Road.","end_shape_index":980,"type":15,"time":355,"verbal_post_transition_instruction":"Continue for 1.7 miles.","street_names":["Doctor Jack Road"],"begin_shape_index":907},{"travel_type":"road","travel_mode":"bicycle","begin_shape_index":980,"time":0,"type":4,"end_shape_index":980,"instruction":"You have arrived at your destination.","length":0.000,"verbal_transition_alert_instruction":"You will arrive at your destination.","verbal_pre_transition_instruction":"You have arrived at your destination."}]}],"status_message":"Found route between points","status":0}})";
  NavigatorTest nav;
  nav.SetRoute(route_json_str);
  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 0" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122698000f, 39.651388000f, 1483287460, 0.000000f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122551000f, 39.651695000f, 1483287463, 12.141196f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 2" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122494000f, 39.651803000f, 1483287464, 12.977207f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 3" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122437000f, 39.651916000f, 1483287465, 13.494481f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 4" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122378000f, 39.652032000f, 1483287466, 13.867895f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 5" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122316000f, 39.652149000f, 1483287467, 14.066711f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 6" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122252000f, 39.652270000f, 1483287468, 14.543750f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 7" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122186000f, 39.652394000f, 1483287469, 14.917728f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 8" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.122119000f, 39.652520000f, 1483287470, 15.156238f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 9" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121980000f, 39.652776000f, 1483287472, 15.443875f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 10" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121904000f, 39.652904000f, 1483287473, 15.667175f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 11" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121828000f, 39.653033000f, 1483287474, 15.768480f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 12" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121673000f, 39.653291000f, 1483287476, 15.822013f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 13" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121600000f, 39.653420000f, 1483287477, 15.664002f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 14" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121531000f, 39.653546000f, 1483287478, 15.221976f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 15" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121466000f, 39.653669000f, 1483287479, 14.782230f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 16" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121355000f, 39.653919000f, 1483287481, 14.705501f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 17" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121306000f, 39.654043000f, 1483287482, 14.428324f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 18" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121256000f, 39.654167000f, 1483287483, 14.453501f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 19" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121208000f, 39.654290000f, 1483287484, 14.296960f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 20" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121161000f, 39.654414000f, 1483287485, 14.379360f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 21" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121079000f, 39.654671000f, 1483287487, 14.729833f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 22" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.121033000f, 39.654801000f, 1483287488, 14.998939f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 23" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120928000f, 39.655057000f, 1483287490, 14.942445f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 24" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120825000f, 39.655305000f, 1483287492, 14.492116f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 25" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120775000f, 39.655422000f, 1483287493, 13.711224f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 26" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120724000f, 39.655538000f, 1483287494, 13.632759f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 27" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120619000f, 39.655761000f, 1483287496, 13.202501f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 28" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120477000f, 39.656073000f, 1483287499, 12.267379f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 29" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120321000f, 39.656345000f, 1483287502, 11.033092f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 30" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120069000f, 39.656737000f, 1483287508, 8.114882f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 31" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120009000f, 39.656837000f, 1483287510, 6.131113f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 32" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119959000f, 39.656933000f, 1483287512, 5.756894f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 33" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119939000f, 39.656979000f, 1483287513, 5.399954f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 34" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119922000f, 39.657023000f, 1483287514, 5.110152f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 35" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119885000f, 39.657110000f, 1483287516, 5.095352f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 36" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119842000f, 39.657238000f, 1483287519, 4.905911f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 37" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119824000f, 39.657323000f, 1483287521, 4.793541f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 38" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119804000f, 39.657409000f, 1483287523, 4.862854f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 39" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119768000f, 39.657587000f, 1483287527, 5.013407f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 40" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119722000f, 39.657810000f, 1483287532, 5.027066f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 41" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119648000f, 39.658216000f, 1483287541, 5.070944f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 42" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119612000f, 39.658399000f, 1483287545, 5.150942f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 43" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119487000f, 39.658978000f, 1483287557, 5.444847f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 44" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119353000f, 39.659537000f, 1483287568, 5.752578f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 45" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119318000f, 39.659632000f, 1483287570, 5.496247f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 46" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119073000f, 39.660169000f, 1483287581, 5.759878f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 47" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118795000f, 39.660697000f, 1483287592, 5.765592f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 48" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118586000f, 39.661118000f, 1483287602, 5.017143f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 49" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118456000f, 39.661369000f, 1483287609, 4.297185f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 50" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118403000f, 39.661470000f, 1483287612, 4.042006f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 51" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118372000f, 39.661533000f, 1483287614, 3.749717f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 52" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118280000f, 39.661708000f, 1483287620, 3.502638f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 53" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118248000f, 39.661765000f, 1483287622, 3.456222f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 54" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118215000f, 39.661820000f, 1483287624, 3.372069f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 55" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118168000f, 39.661898000f, 1483287627, 3.190537f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 56" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118107000f, 39.661995000f, 1483287631, 2.999199f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 57" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118042000f, 39.662087000f, 1483287635, 2.914553f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 58" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117963000f, 39.662203000f, 1483287640, 2.916024f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 59" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117911000f, 39.662273000f, 1483287643, 2.992186f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 60" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117826000f, 39.662381000f, 1483287648, 2.811402f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 61" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117737000f, 39.662513000f, 1483287654, 2.759270f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 62" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117671000f, 39.662625000f, 1483287659, 2.738137f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 63" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117618000f, 39.662712000f, 1483287663, 2.674226f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 64" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117570000f, 39.662797000f, 1483287667, 2.579393f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 65" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117517000f, 39.662907000f, 1483287672, 2.612064f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 66" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117440000f, 39.663081000f, 1483287680, 2.557836f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 67" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117416000f, 39.663146000f, 1483287683, 2.507461f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 68" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117397000f, 39.663210000f, 1483287686, 2.436044f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 69" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117346000f, 39.663380000f, 1483287694, 2.427802f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 70" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117316000f, 39.663487000f, 1483287699, 2.437093f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 71" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117298000f, 39.663571000f, 1483287703, 2.369302f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 72" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117275000f, 39.663701000f, 1483287709, 2.434189f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 73" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117248000f, 39.663868000f, 1483287716, 2.676255f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 74" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117238000f, 39.663990000f, 1483287721, 2.721597f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 75" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117225000f, 39.664146000f, 1483287727, 2.900256f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 76" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117218000f, 39.664326000f, 1483287733, 3.341081f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 77" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117210000f, 39.664492000f, 1483287738, 3.698349f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 78" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117151000f, 39.665047000f, 1483287751, 4.768373f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 79" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117067000f, 39.665350000f, 1483287758, 4.927046f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 80" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116995000f, 39.665538000f, 1483287763, 4.363717f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 81" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116932000f, 39.665676000f, 1483287767, 4.070769f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 82" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116848000f, 39.665837000f, 1483287772, 3.862778f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 83" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116781000f, 39.665965000f, 1483287776, 3.840523f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 84" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116666000f, 39.666161000f, 1483287782, 3.990138f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 85" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116363000f, 39.666702000f, 1483287796, 4.684460f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 86" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116239000f, 39.666918000f, 1483287800, 6.572033f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 87" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.115938000f, 39.667461000f, 1483287808, 8.214924f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 88" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.115895000f, 39.667537000f, 1483287809, 9.227838f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 89" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.115810000f, 39.667691000f, 1483287811, 9.313164f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 90" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.115649000f, 39.667988000f, 1483287815, 8.956201f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 91" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.115349000f, 39.668537000f, 1483287823, 8.287601f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 92" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.115065000f, 39.669066000f, 1483287832, 7.079799f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 93" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114968000f, 39.669239000f, 1483287836, 5.243836f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 94" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114887000f, 39.669391000f, 1483287840, 4.572187f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 95" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114851000f, 39.669459000f, 1483287842, 4.087063f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 96" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114813000f, 39.669524000f, 1483287844, 3.967321f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 97" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114759000f, 39.669623000f, 1483287847, 3.984195f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 98" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114706000f, 39.669720000f, 1483287850, 3.904711f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 99" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114638000f, 39.669848000f, 1483287854, 3.848550f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 100" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114573000f, 39.669974000f, 1483287858, 3.772901f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 101" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114480000f, 39.670153000f, 1483287864, 3.576758f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 102" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114433000f, 39.670235000f, 1483287867, 3.325704f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 103" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114391000f, 39.670315000f, 1483287870, 3.201743f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 104" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114282000f, 39.670520000f, 1483287878, 3.082222f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 105" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114171000f, 39.670690000f, 1483287884, 3.529994f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 106" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114049000f, 39.670825000f, 1483287889, 3.661273f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 107" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.113827000f, 39.670993000f, 1483287896, 3.810817f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 108" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.113523000f, 39.671117000f, 1483287903, 4.211384f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 109" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.113424000f, 39.671141000f, 1483287905, 4.446795f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 110" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.112902000f, 39.671214000f, 1483287915, 4.545975f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 111" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.112347000f, 39.671277000f, 1483287925, 4.806943f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 112" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.112226000f, 39.671288000f, 1483287927, 5.219961f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 113" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.111490000f, 39.671365000f, 1483287938, 5.785802f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 114" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.110723000f, 39.671444000f, 1483287946, 8.288239f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 115" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.110483000f, 39.671465000f, 1483287948, 10.348373f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 116" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.110090000f, 39.671499000f, 1483287951, 11.295355f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 117" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.109806000f, 39.671521000f, 1483287953, 12.228663f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 118" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.109510000f, 39.671544000f, 1483287955, 12.745757f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 119" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.109201000f, 39.671570000f, 1483287957, 13.317112f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 120" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.109041000f, 39.671583000f, 1483287958, 13.785677f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 121" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108876000f, 39.671596000f, 1483287959, 14.211801f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 122" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108707000f, 39.671611000f, 1483287960, 14.576619f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 123" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108537000f, 39.671625000f, 1483287961, 14.649479f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 124" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108364000f, 39.671640000f, 1483287962, 14.917146f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 125" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108190000f, 39.671655000f, 1483287963, 15.002292f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 126" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108016000f, 39.671671000f, 1483287964, 15.015086f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 127" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.107663000f, 39.671699000f, 1483287966, 15.203367f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 128" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.107485000f, 39.671714000f, 1483287967, 15.342934f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 129" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.107304000f, 39.671729000f, 1483287968, 15.598481f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 130" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.107121000f, 39.671743000f, 1483287969, 15.757474f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 131" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106936000f, 39.671758000f, 1483287970, 15.939284f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 132" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106751000f, 39.671774000f, 1483287971, 15.951327f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 133" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106564000f, 39.671789000f, 1483287972, 16.109713f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 134" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106190000f, 39.671819000f, 1483287974, 16.109707f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 135" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.105820000f, 39.671849000f, 1483287976, 15.939265f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 136" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.105637000f, 39.671865000f, 1483287977, 15.781021f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 137" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.105457000f, 39.671882000f, 1483287978, 15.538799f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 138" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.105101000f, 39.671912000f, 1483287980, 15.342892f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 139" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.104926000f, 39.671927000f, 1483287981, 15.087388f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 140" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.104753000f, 39.671940000f, 1483287982, 14.893803f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 141" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.104580000f, 39.671953000f, 1483287983, 14.893800f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 142" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.104251000f, 39.671976000f, 1483287985, 14.153026f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 143" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.103522000f, 39.672034000f, 1483287990, 12.559273f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 144" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.102727000f, 39.672095000f, 1483287996, 11.409377f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 145" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.101924000f, 39.672158000f, 1483288002, 11.526758f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 146" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.101381000f, 39.672206000f, 1483288006, 11.708015f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 147" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.101110000f, 39.672229000f, 1483288008, 11.680498f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 148" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.100708000f, 39.672261000f, 1483288011, 11.542839f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 149" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.100192000f, 39.672303000f, 1483288015, 11.114809f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 150" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.099478000f, 39.672361000f, 1483288021, 10.252962f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 151" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098838000f, 39.672379000f, 1483288028, 7.839143f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 152" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098721000f, 39.672358000f, 1483288030, 5.146960f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 153" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098625000f, 39.672335000f, 1483288032, 4.307438f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 154" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098546000f, 39.672313000f, 1483288034, 3.599203f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 155" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098467000f, 39.672291000f, 1483288036, 3.599204f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 156" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098392000f, 39.672269000f, 1483288038, 3.438555f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 157" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098327000f, 39.672248000f, 1483288040, 3.020077f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 158" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098246000f, 39.672222000f, 1483288043, 2.506564f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 159" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098162000f, 39.672202000f, 1483288046, 2.511302f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 160" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098062000f, 39.672189000f, 1483288049, 2.896572f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 161" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097949000f, 39.672190000f, 1483288052, 3.227632f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 162" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097840000f, 39.672199000f, 1483288055, 3.131034f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 163" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097741000f, 39.672211000f, 1483288058, 2.862406f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 164" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097630000f, 39.672226000f, 1483288061, 3.218784f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 165" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097501000f, 39.672243000f, 1483288065, 2.803505f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 166" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097387000f, 39.672258000f, 1483288069, 2.477406f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 167" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097293000f, 39.672273000f, 1483288072, 2.741841f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 168" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097157000f, 39.672297000f, 1483288076, 2.988827f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 169" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097045000f, 39.672313000f, 1483288080, 2.440110f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 170" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.096925000f, 39.672332000f, 1483288084, 2.624327f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 171" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.096800000f, 39.672355000f, 1483288088, 2.753053f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 172" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.096568000f, 39.672393000f, 1483288095, 2.903380f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 173" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.096368000f, 39.672424000f, 1483288100, 3.496138f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 174" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.096029000f, 39.672468000f, 1483288107, 4.208105f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 175" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.095275000f, 39.672555000f, 1483288119, 5.443922f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 176" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.094529000f, 39.672633000f, 1483288129, 6.450670f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 177" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.093835000f, 39.672706000f, 1483288138, 6.668529f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 178" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.093281000f, 39.672766000f, 1483288145, 6.847997f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 179" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.092706000f, 39.672828000f, 1483288152, 7.106966f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 180" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091954000f, 39.672910000f, 1483288161, 7.230762f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 181" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091172000f, 39.672999000f, 1483288170, 7.525823f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 182" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.090406000f, 39.673081000f, 1483288178, 8.283077f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 183" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089696000f, 39.673159000f, 1483288185, 8.778731f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 184" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089288000f, 39.673202000f, 1483288189, 8.821157f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 185" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.088516000f, 39.673272000f, 1483288196, 9.514885f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 186" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.088275000f, 39.673291000f, 1483288198, 10.378738f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 187" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.087902000f, 39.673312000f, 1483288201, 10.681626f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 188" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.087524000f, 39.673329000f, 1483288204, 10.814380f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 189" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.087152000f, 39.673354000f, 1483288207, 10.665019f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 190" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.086795000f, 39.673394000f, 1483288210, 10.303648f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 191" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.086033000f, 39.673473000f, 1483288218, 8.234924f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 192" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.085886000f, 39.673486000f, 1483288220, 6.339064f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 193" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.085685000f, 39.673503000f, 1483288223, 5.775251f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 194" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.085556000f, 39.673513000f, 1483288225, 5.554449f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 195" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.085425000f, 39.673524000f, 1483288227, 5.645471f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 196" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.085220000f, 39.673541000f, 1483288230, 5.888821f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 197" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.084628000f, 39.673594000f, 1483288238, 6.383212f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 198" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.083924000f, 39.673648000f, 1483288247, 6.735427f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 199" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.083315000f, 39.673699000f, 1483288256, 5.832019f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 200" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.083067000f, 39.673718000f, 1483288260, 5.338522f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 201" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.082631000f, 39.673747000f, 1483288267, 5.356639f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 202" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.081908000f, 39.673800000f, 1483288277, 6.222819f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 203" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.081209000f, 39.673845000f, 1483288285, 7.512575f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 204" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.080601000f, 39.673885000f, 1483288291, 8.714061f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 205" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.080195000f, 39.673914000f, 1483288295, 8.734046f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 206" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.079470000f, 39.673967000f, 1483288303, 7.799830f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 207" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.078762000f, 39.674017000f, 1483288312, 6.768585f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 208" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.078688000f, 39.674023000f, 1483288313, 6.375507f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 209" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.078194000f, 39.674058000f, 1483288320, 6.072220f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 210" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.077497000f, 39.674113000f, 1483288330, 6.003291f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 211" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.077056000f, 39.674153000f, 1483288336, 6.341150f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 212" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.076302000f, 39.674220000f, 1483288346, 6.503273f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 213" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.075592000f, 39.674283000f, 1483288356, 6.123651f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 214" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.074881000f, 39.674344000f, 1483288367, 5.572410f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 215" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.074152000f, 39.674378000f, 1483288378, 5.688727f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 216" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.073396000f, 39.674429000f, 1483288385, 9.289015f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 217" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.073294000f, 39.674439000f, 1483288386, 8.810062f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 218" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.073191000f, 39.674447000f, 1483288387, 8.869950f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 219" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.072983000f, 39.674460000f, 1483288389, 8.940140f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 220" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.072559000f, 39.674489000f, 1483288393, 9.117957f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 221" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.071819000f, 39.674552000f, 1483288401, 7.973808f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 222" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.071358000f, 39.674598000f, 1483288407, 6.638226f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 223" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.070604000f, 39.674665000f, 1483288416, 7.225812f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 224" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.069829000f, 39.674740000f, 1483288424, 8.365655f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 225" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.069728000f, 39.674751000f, 1483288425, 8.739937f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 226" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.069008000f, 39.674825000f, 1483288433, 7.779686f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 227" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.068506000f, 39.674880000f, 1483288441, 5.430641f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 228" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.068347000f, 39.674895000f, 1483288444, 4.575041f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 229" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.068196000f, 39.674910000f, 1483288447, 4.348345f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 230" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.068047000f, 39.674922000f, 1483288450, 4.278687f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 231" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.067854000f, 39.674941000f, 1483288454, 4.167752f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 232" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.067576000f, 39.674972000f, 1483288460, 4.011299f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 233" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.067350000f, 39.674995000f, 1483288465, 3.906452f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 234" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.067096000f, 39.675017000f, 1483288470, 4.380026f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 235" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.066387000f, 39.675085000f, 1483288483, 4.709002f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 236" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.065629000f, 39.675166000f, 1483288496, 5.043725f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 237" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.065104000f, 39.675222000f, 1483288502, 7.568647f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 238" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.064705000f, 39.675255000f, 1483288506, 8.595771f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 239" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.064275000f, 39.675297000f, 1483288510, 9.284452f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 240" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.063592000f, 39.675366000f, 1483288516, 9.836871f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 241" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.063240000f, 39.675407000f, 1483288519, 10.167550f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 242" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.062891000f, 39.675448000f, 1483288522, 10.082838f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 243" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.062663000f, 39.675476000f, 1483288524, 9.891027f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 244" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.062345000f, 39.675496000f, 1483288527, 9.112307f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 245" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.061748000f, 39.675537000f, 1483288533, 8.558985f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 246" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.060965000f, 39.675608000f, 1483288541, 8.443875f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 247" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.060271000f, 39.675716000f, 1483288548, 8.666389f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 248" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.060087000f, 39.675770000f, 1483288550, 8.436082f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 249" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.059489000f, 39.676093000f, 1483288558, 7.824223f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 250" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.059359000f, 39.676182000f, 1483288560, 7.453484f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 251" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.058704000f, 39.676487000f, 1483288569, 7.287872f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 252" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.057956000f, 39.676597000f, 1483288577, 8.155837f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 253" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.057257000f, 39.676669000f, 1483288584, 8.631854f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 254" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.056727000f, 39.676716000f, 1483288589, 9.141961f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 255" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.055924000f, 39.676782000f, 1483288596, 9.884376f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 256" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.055551000f, 39.676816000f, 1483288599, 10.727074f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 257" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.054918000f, 39.676883000f, 1483288604, 10.948911f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 258" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.054674000f, 39.676910000f, 1483288606, 10.560175f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 259" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.053978000f, 39.676976000f, 1483288612, 10.013773f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 260" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.053243000f, 39.677046000f, 1483288619, 9.064770f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 261" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.052464000f, 39.677127000f, 1483288626, 9.621311f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 262" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.051730000f, 39.677184000f, 1483288632, 10.534432f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 263" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050929000f, 39.677255000f, 1483288639, 9.868750f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 264" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050211000f, 39.677333000f, 1483288646, 8.875156f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 265" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.049945000f, 39.677370000f, 1483288649, 7.719785f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 266" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.049463000f, 39.677446000f, 1483288655, 7.025687f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 267" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048961000f, 39.677529000f, 1483288662, 6.284449f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 268" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048577000f, 39.677605000f, 1483288668, 5.661724f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 269" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048160000f, 39.677675000f, 1483288675, 5.223883f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 270" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048107000f, 39.677696000f, 1483288676, 5.107290f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 271" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047914000f, 39.677875000f, 1483288681, 5.178723f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 272" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047883000f, 39.677916000f, 1483288682, 5.280644f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 273" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047532000f, 39.678391000f, 1483288692, 6.083008f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 274" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047164000f, 39.678876000f, 1483288701, 6.946863f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 275" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046860000f, 39.679406000f, 1483288710, 7.165826f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 276" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046596000f, 39.679940000f, 1483288719, 7.066909f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 277" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046337000f, 39.680474000f, 1483288728, 7.050116f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 278" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046057000f, 39.681024000f, 1483288738, 6.575739f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 279" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045944000f, 39.681235000f, 1483288742, 6.351315f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 280" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045675000f, 39.681768000f, 1483288752, 6.365177f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 281" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045485000f, 39.682258000f, 1483288760, 7.115438f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 282" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045529000f, 39.682525000f, 1483288764, 7.490096f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 283" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045555000f, 39.682592000f, 1483288765, 7.783914f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 284" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045809000f, 39.683116000f, 1483288773, 7.782256f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 285" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046077000f, 39.683647000f, 1483288784, 5.764818f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 286" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046133000f, 39.683767000f, 1483288788, 3.548425f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 287" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046190000f, 39.683888000f, 1483288792, 3.581871f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 288" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046220000f, 39.683947000f, 1483288794, 3.526398f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 289" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046280000f, 39.684069000f, 1483288798, 3.630289f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 290" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046354000f, 39.684231000f, 1483288803, 3.823117f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 291" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046420000f, 39.684365000f, 1483288807, 3.988111f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 292" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046535000f, 39.684621000f, 1483288814, 4.307524f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 293" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046769000f, 39.685154000f, 1483288827, 4.817555f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 294" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046891000f, 39.685421000f, 1483288833, 5.251052f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 295" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047102000f, 39.685904000f, 1483288844, 5.156763f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 296" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047179000f, 39.686075000f, 1483288848, 5.036534f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 297" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047335000f, 39.686451000f, 1483288857, 4.881978f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 298" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047344000f, 39.686999000f, 1483288868, 5.546177f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 299" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047139000f, 39.687516000f, 1483288877, 6.685762f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 300" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046929000f, 39.687912000f, 1483288884, 6.801698f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 301" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046633000f, 39.688460000f, 1483288894, 6.606306f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 302" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046341000f, 39.688998000f, 1483288905, 5.900330f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 303" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046119000f, 39.689413000f, 1483288914, 5.550961f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 304" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045881000f, 39.689847000f, 1483288924, 5.243821f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 305" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045691000f, 39.690195000f, 1483288933, 4.668812f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 306" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045649000f, 39.690264000f, 1483288935, 4.240944f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 307" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045384000f, 39.690315000f, 1483288940, 4.679879f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 308" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.044659000f, 39.690212000f, 1483288950, 6.315397f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 309" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.043961000f, 39.690111000f, 1483288959, 6.759951f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 310" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.043805000f, 39.690092000f, 1483288961, 6.764767f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 311" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.043113000f, 39.689995000f, 1483288970, 6.694803f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 312" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.042426000f, 39.689859000f, 1483288979, 6.751748f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 313" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.041766000f, 39.689634000f, 1483288988, 6.870730f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 314" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.041625000f, 39.689571000f, 1483288990, 6.983383f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 315" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.041006000f, 39.689227000f, 1483288999, 7.267441f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 316" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.040419000f, 39.688855000f, 1483289008, 7.237884f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 317" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039840000f, 39.688493000f, 1483289017, 7.100654f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 318" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039293000f, 39.688146000f, 1483289026, 6.747460f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 319" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039124000f, 39.688027000f, 1483289029, 6.541093f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 320" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038838000f, 39.687985000f, 1483289033, 6.235511f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 321" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038694000f, 39.688002000f, 1483289035, 6.239971f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 322" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038549000f, 39.688022000f, 1483289037, 6.309619f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 323" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.037915000f, 39.688100000f, 1483289046, 6.111186f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 324" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.037528000f, 39.688146000f, 1483289052, 5.590846f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 325" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.036832000f, 39.688260000f, 1483289062, 6.095761f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 326" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.036131000f, 39.688304000f, 1483289070, 7.531210f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 327" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.035755000f, 39.688290000f, 1483289074, 8.061808f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 328" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.035277000f, 39.688261000f, 1483289079, 8.214864f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 329" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.035083000f, 39.688248000f, 1483289081, 8.340828f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 330" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.034318000f, 39.688197000f, 1483289089, 8.222281f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 331" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.033759000f, 39.688162000f, 1483289095, 8.007387f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 332" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.033115000f, 39.688122000f, 1483289102, 7.906707f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 333" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.032441000f, 39.688074000f, 1483289109, 8.283460f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 334" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.032136000f, 39.688059000f, 1483289112, 8.726938f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 335" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.031825000f, 39.688066000f, 1483289115, 8.884297f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 336" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.031496000f, 39.688091000f, 1483289118, 9.440171f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 337" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.031156000f, 39.688123000f, 1483289121, 9.780922f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 338" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.030806000f, 39.688160000f, 1483289124, 10.087982f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 339" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.030436000f, 39.688193000f, 1483289127, 10.635931f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 340" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.030181000f, 39.688219000f, 1483289129, 11.017590f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 341" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.029537000f, 39.688285000f, 1483289134, 11.130907f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 342" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.029278000f, 39.688309000f, 1483289136, 11.173587f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 343" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.029017000f, 39.688333000f, 1483289138, 11.258636f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 344" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.028491000f, 39.688378000f, 1483289142, 11.334166f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 345" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.027977000f, 39.688427000f, 1483289146, 11.091912f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 346" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.027726000f, 39.688450000f, 1483289148, 10.826722f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 347" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.027237000f, 39.688495000f, 1483289152, 10.546975f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 348" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.026492000f, 39.688563000f, 1483289158, 10.711100f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 349" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.026366000f, 39.688575000f, 1483289159, 10.875931f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 350" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.025602000f, 39.688647000f, 1483289165, 10.989286f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 351" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.025085000f, 39.688698000f, 1483289169, 11.162573f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 352" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.024297000f, 39.688762000f, 1483289175, 11.312915f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 353" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.023897000f, 39.688791000f, 1483289178, 11.472325f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 354" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.023521000f, 39.688827000f, 1483289181, 10.819220f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 355" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.023289000f, 39.688862000f, 1483289183, 10.126066f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 356" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.023178000f, 39.688887000f, 1483289184, 9.907487f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 357" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.022783000f, 39.689043000f, 1483289188, 9.508231f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 358" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.022211000f, 39.689405000f, 1483289196, 7.930164f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 359" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.022088000f, 39.689483000f, 1483289198, 6.826603f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 360" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.021786000f, 39.689617000f, 1483289203, 5.972491f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 361" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.021539000f, 39.689683000f, 1483289207, 5.599456f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 362" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.021285000f, 39.689741000f, 1483289211, 5.673971f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 363" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.020982000f, 39.689806000f, 1483289216, 5.389050f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 364" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.020524000f, 39.689908000f, 1483289224, 5.105392f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 365" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.020133000f, 39.689998000f, 1483289231, 4.994289f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 366" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.019676000f, 39.690110000f, 1483289239, 5.135595f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 367" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.018935000f, 39.690310000f, 1483289249, 6.726635f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 368" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.018537000f, 39.690422000f, 1483289253, 9.075334f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 369" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.018213000f, 39.690512000f, 1483289256, 9.835700f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 370" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.017989000f, 39.690575000f, 1483289258, 10.214755f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 371" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.017644000f, 39.690676000f, 1483289261, 10.539816f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 372" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.017424000f, 39.690761000f, 1483289263, 10.543715f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 373" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.017323000f, 39.690815000f, 1483289264, 10.535057f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 374" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.017066000f, 39.691017000f, 1483289267, 10.489639f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 375" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016929000f, 39.691173000f, 1483289269, 10.479661f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 376" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016751000f, 39.691409000f, 1483289272, 10.125172f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 377" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016532000f, 39.691715000f, 1483289276, 9.721938f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 378" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016223000f, 39.692250000f, 1483289283, 9.310406f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 379" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016202000f, 39.692335000f, 1483289284, 9.631625f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 380" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016164000f, 39.692657000f, 1483289288, 8.998091f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 381" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016175000f, 39.692894000f, 1483289291, 8.799847f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 382" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016209000f, 39.693464000f, 1483289299, 7.939864f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 383" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016221000f, 39.693697000f, 1483289303, 6.489450f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 384" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016238000f, 39.693898000f, 1483289307, 5.605638f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 385" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016240000f, 39.693951000f, 1483289308, 5.902420f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 386" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016241000f, 39.694111000f, 1483289311, 5.937108f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 387" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016241000f, 39.694682000f, 1483289320, 7.062603f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 388" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016201000f, 39.695060000f, 1483289325, 8.443605f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 389" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015950000f, 39.695524000f, 1483289331, 9.324667f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 390" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015745000f, 39.695727000f, 1483289334, 9.539324f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 391" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015129000f, 39.696088000f, 1483289341, 9.474862f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 392" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015034000f, 39.696124000f, 1483289342, 9.070444f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 393" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014663000f, 39.696262000f, 1483289346, 8.824006f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 394" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014047000f, 39.696621000f, 1483289354, 8.273631f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 395" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013664000f, 39.697114000f, 1483289364, 6.393783f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 396" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013606000f, 39.697204000f, 1483289366, 5.591400f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 397" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013527000f, 39.697330000f, 1483289369, 5.191040f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 398" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013451000f, 39.697450000f, 1483289372, 4.953337f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 399" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013335000f, 39.697636000f, 1483289377, 4.593177f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 400" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013267000f, 39.697735000f, 1483289380, 4.155013f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 401" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013161000f, 39.697899000f, 1483289385, 4.077872f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 402" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013033000f, 39.698104000f, 1483289391, 4.219569f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 403" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012774000f, 39.698502000f, 1483289402, 4.504417f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 404" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012501000f, 39.699032000f, 1483289416, 4.533137f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 405" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012487000f, 39.699071000f, 1483289417, 4.504013f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 406" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012353000f, 39.699517000f, 1483289428, 4.632527f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 407" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012225000f, 39.699999000f, 1483289439, 4.978596f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 408" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012091000f, 39.700530000f, 1483289450, 5.474048f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 409" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011981000f, 39.700977000f, 1483289459, 5.627095f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 410" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011834000f, 39.701545000f, 1483289470, 5.860978f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 411" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011682000f, 39.702113000f, 1483289481, 5.868703f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 412" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011532000f, 39.702674000f, 1483289492, 5.796177f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 413" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011423000f, 39.703091000f, 1483289501, 5.261069f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 414" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011351000f, 39.703352000f, 1483289508, 4.243080f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 415" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011288000f, 39.703651000f, 1483289517, 3.746558f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 416" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011213000f, 39.703818000f, 1483289524, 2.809824f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 417" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011137000f, 39.703963000f, 1483289529, 3.480857f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 418" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011180000f, 39.704534000f, 1483289544, 4.244668f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 419" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011201000f, 39.704693000f, 1483289548, 4.447735f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 420" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011273000f, 39.705258000f, 1483289561, 4.861312f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 421" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011332000f, 39.705855000f, 1483289572, 6.059050f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 422" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011388000f, 39.706397000f, 1483289581, 6.725053f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 423" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011394000f, 39.706458000f, 1483289582, 6.809903f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 424" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011460000f, 39.707023000f, 1483289591, 7.016553f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 425" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011508000f, 39.707605000f, 1483289601, 6.491822f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 426" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011534000f, 39.707847000f, 1483289606, 5.406236f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 427" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011563000f, 39.708116000f, 1483289612, 5.007959f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 428" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011584000f, 39.708304000f, 1483289616, 5.251298f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 429" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011648000f, 39.708855000f, 1483289627, 5.598311f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 430" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011697000f, 39.709259000f, 1483289635, 5.646052f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 431" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011748000f, 39.709661000f, 1483289644, 4.995895f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 432" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011773000f, 39.709921000f, 1483289650, 4.837024f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 433" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011827000f, 39.710486000f, 1483289663, 4.851176f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 434" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011878000f, 39.711054000f, 1483289674, 5.761830f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 435" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011940000f, 39.711640000f, 1483289683, 7.272103f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 436" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011958000f, 39.711782000f, 1483289685, 7.941172f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 437" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011987000f, 39.712002000f, 1483289688, 8.205293f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 438" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012006000f, 39.712155000f, 1483289690, 8.554710f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 439" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012030000f, 39.712315000f, 1483289692, 8.964651f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 440" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012065000f, 39.712569000f, 1483289695, 9.477853f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 441" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012108000f, 39.712919000f, 1483289699, 9.783859f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 442" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012134000f, 39.713292000f, 1483289703, 10.395454f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 443" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012140000f, 39.713391000f, 1483289704, 11.032600f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 444" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012147000f, 39.713492000f, 1483289705, 11.259236f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 445" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012168000f, 39.713785000f, 1483289708, 10.888716f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 446" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012187000f, 39.713970000f, 1483289710, 10.329138f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 447" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012209000f, 39.714142000f, 1483289712, 9.619704f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 448" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012284000f, 39.714682000f, 1483289719, 8.636375f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 449" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012357000f, 39.715254000f, 1483289728, 7.108983f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 450" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012374000f, 39.715369000f, 1483289730, 6.442121f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 451" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012393000f, 39.715525000f, 1483289733, 5.813962f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 452" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012435000f, 39.715809000f, 1483289739, 5.303107f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 453" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012461000f, 39.716031000f, 1483289744, 4.962602f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 454" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012476000f, 39.716165000f, 1483289747, 4.990670f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 455" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012502000f, 39.716375000f, 1483289752, 4.696574f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 456" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012522000f, 39.716532000f, 1483289756, 4.390217f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 457" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012550000f, 39.716714000f, 1483289761, 4.080304f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 458" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012573000f, 39.716887000f, 1483289766, 3.871743f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 459" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012586000f, 39.716980000f, 1483289769, 3.470796f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 460" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012593000f, 39.717068000f, 1483289772, 3.271479f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 461" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012613000f, 39.717185000f, 1483289776, 3.284122f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 462" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012618000f, 39.717270000f, 1483289779, 3.157279f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 463" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012622000f, 39.717391000f, 1483289783, 3.368503f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 464" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012670000f, 39.717743000f, 1483289794, 3.581766f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 465" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012697000f, 39.717973000f, 1483289801, 3.672522f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 466" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012717000f, 39.718140000f, 1483289806, 3.733814f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 467" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012744000f, 39.718484000f, 1483289816, 3.836363f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 468" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012794000f, 39.718998000f, 1483289829, 4.413705f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 469" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012849000f, 39.719552000f, 1483289841, 5.154212f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 470" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012908000f, 39.720113000f, 1483289852, 5.695839f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 471" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012972000f, 39.720655000f, 1483289862, 6.058351f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 472" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013046000f, 39.721253000f, 1483289872, 6.686992f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 473" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013079000f, 39.721516000f, 1483289876, 7.353264f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 474" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013124000f, 39.721856000f, 1483289881, 7.608848f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 475" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013167000f, 39.722165000f, 1483289885, 8.648549f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 476" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013200000f, 39.722413000f, 1483289888, 9.250483f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 477" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013219000f, 39.722580000f, 1483289890, 9.330699f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 478" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013239000f, 39.722743000f, 1483289892, 9.112851f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 479" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013258000f, 39.722901000f, 1483289894, 8.831776f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 480" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013294000f, 39.723199000f, 1483289898, 8.329025f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 481" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013343000f, 39.723660000f, 1483289904, 8.581582f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 482" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013412000f, 39.724242000f, 1483289911, 9.293819f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 483" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013439000f, 39.724489000f, 1483289914, 9.197640f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 484" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013486000f, 39.724884000f, 1483289919, 8.830990f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 485" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013555000f, 39.725483000f, 1483289927, 8.367695f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 486" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013615000f, 39.726032000f, 1483289935, 7.666240f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 487" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013629000f, 39.726162000f, 1483289937, 7.260544f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 488" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013695000f, 39.726709000f, 1483289946, 6.794821f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 489" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013729000f, 39.727011000f, 1483289952, 5.624046f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 490" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013759000f, 39.727271000f, 1483289958, 4.842802f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 491" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013775000f, 39.727417000f, 1483289962, 4.077568f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 492" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013786000f, 39.727518000f, 1483289965, 3.760881f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 493" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013805000f, 39.727650000f, 1483289969, 3.695984f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 494" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013825000f, 39.727785000f, 1483289973, 3.781341f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 495" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013849000f, 39.728010000f, 1483289979, 4.188504f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 496" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013909000f, 39.728563000f, 1483289992, 4.751818f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 497" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013913000f, 39.728606000f, 1483289993, 4.798973f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 498" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013950000f, 39.729010000f, 1483290003, 4.508450f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 499" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013973000f, 39.729212000f, 1483290008, 4.514517f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 500" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014031000f, 39.729792000f, 1483290022, 4.625426f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 501" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014085000f, 39.730269000f, 1483290033, 4.845479f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 502" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014137000f, 39.730820000f, 1483290046, 4.730645f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 503" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014212000f, 39.731416000f, 1483290057, 6.059671f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 504" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014262000f, 39.731870000f, 1483290064, 7.245714f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 505" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014283000f, 39.732058000f, 1483290067, 7.001714f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 506" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014311000f, 39.732305000f, 1483290071, 6.900051f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 507" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014345000f, 39.732610000f, 1483290076, 6.815396f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 508" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014384000f, 39.733174000f, 1483290085, 6.985878f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 509" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014499000f, 39.733723000f, 1483290094, 6.878030f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 510" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014705000f, 39.734060000f, 1483290101, 5.921848f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 511" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014689000f, 39.734370000f, 1483290106, 6.907243f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 512" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014545000f, 39.734837000f, 1483290112, 8.904630f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 513" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014559000f, 39.735233000f, 1483290116, 11.024702f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 514" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014596000f, 39.735426000f, 1483290118, 10.858442f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 515" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014615000f, 39.735524000f, 1483290119, 11.029893f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 516" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014629000f, 39.735621000f, 1483290120, 10.864296f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 517" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014642000f, 39.735714000f, 1483290121, 10.412354f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 518" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014651000f, 39.735811000f, 1483290122, 10.825441f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 519" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014635000f, 39.736075000f, 1483290125, 9.806748f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 520" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014523000f, 39.736621000f, 1483290133, 7.691498f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 521" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014465000f, 39.736869000f, 1483290138, 5.610029f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 522" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014446000f, 39.736958000f, 1483290140, 5.020026f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 523" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014424000f, 39.737040000f, 1483290142, 4.660223f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 524" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014390000f, 39.737158000f, 1483290145, 4.484760f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 525" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014337000f, 39.737309000f, 1483290149, 4.352690f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 526" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014242000f, 39.737573000f, 1483290156, 4.356109f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 527" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014150000f, 39.737832000f, 1483290163, 4.269713f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 528" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014034000f, 39.738141000f, 1483290171, 4.475288f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 529" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013960000f, 39.738333000f, 1483290176, 4.458457f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 530" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013888000f, 39.738539000f, 1483290181, 4.749124f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 531" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013701000f, 39.739079000f, 1483290193, 5.183944f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 532" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013629000f, 39.739585000f, 1483290204, 5.151258f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 533" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013966000f, 39.740064000f, 1483290216, 5.052086f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 534" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014373000f, 39.740528000f, 1483290227, 5.663935f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 535" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014769000f, 39.740989000f, 1483290237, 6.150264f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 536" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015144000f, 39.741409000f, 1483290246, 6.301397f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 537" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015258000f, 39.741542000f, 1483290249, 5.910666f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 538" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015239000f, 39.741943000f, 1483290257, 5.583591f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 539" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015067000f, 39.742215000f, 1483290263, 5.611419f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 540" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014909000f, 39.742472000f, 1483290269, 5.274116f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 541" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014771000f, 39.742698000f, 1483290275, 4.632210f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 542" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014675000f, 39.742856000f, 1483290280, 3.882666f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 543" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014574000f, 39.743015000f, 1483290285, 3.939655f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 544" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014494000f, 39.743143000f, 1483290289, 3.952225f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 545" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014369000f, 39.743343000f, 1483290295, 4.116899f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 546" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014267000f, 39.743511000f, 1483290300, 4.127847f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 547" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014078000f, 39.743820000f, 1483290309, 4.223557f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 548" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013950000f, 39.744023000f, 1483290315, 4.185622f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 549" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013803000f, 39.744260000f, 1483290322, 4.175643f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 550" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013676000f, 39.744461000f, 1483290328, 4.146006f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 551" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013571000f, 39.744642000f, 1483290334, 3.677060f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 552" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013485000f, 39.744773000f, 1483290339, 3.267076f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 553" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013439000f, 39.744849000f, 1483290342, 3.110533f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 554" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013389000f, 39.744923000f, 1483290345, 3.094338f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 555" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013324000f, 39.745023000f, 1483290349, 3.111205f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 556" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013242000f, 39.745153000f, 1483290354, 3.216748f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 557" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013166000f, 39.745269000f, 1483290358, 3.614753f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 558" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013023000f, 39.745469000f, 1483290364, 4.234426f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 559" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012665000f, 39.745931000f, 1483290377, 4.605079f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 560" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012442000f, 39.746213000f, 1483290386, 4.082147f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 561" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012337000f, 39.746362000f, 1483290391, 3.772977f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 562" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012242000f, 39.746489000f, 1483290396, 3.261824f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 563" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012087000f, 39.746587000f, 1483290401, 3.435214f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 564" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011491000f, 39.746306000f, 1483290410, 6.648833f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 565" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011238000f, 39.746185000f, 1483290413, 8.500708f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 566" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.010871000f, 39.746012000f, 1483290417, 9.211439f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 567" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.010472000f, 39.745823000f, 1483290421, 10.027976f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 568" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.009943000f, 39.745574000f, 1483290426, 10.617817f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 569" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.009731000f, 39.745473000f, 1483290428, 10.673277f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 570" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.009519000f, 39.745375000f, 1483290430, 10.586291f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 571" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.009202000f, 39.745228000f, 1483290433, 10.561863f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 572" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.008590000f, 39.744937000f, 1483290439, 10.265032f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 573" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.007896000f, 39.744606000f, 1483290446, 9.985984f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 574" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.007256000f, 39.744293000f, 1483290453, 9.274608f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 575" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.006616000f, 39.743990000f, 1483290460, 9.190269f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 576" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.005946000f, 39.743672000f, 1483290468, 8.424276f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 577" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.005294000f, 39.743357000f, 1483290476, 8.238757f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 578" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.004954000f, 39.743193000f, 1483290480, 8.588686f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 579" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.004331000f, 39.742905000f, 1483290487, 8.888792f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 580" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.004151000f, 39.742820000f, 1483290489, 9.040415f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 581" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.003513000f, 39.742525000f, 1483290496, 9.103384f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 582" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.002865000f, 39.742233000f, 1483290503, 9.184207f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 583" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.002370000f, 39.742026000f, 1483290508, 9.646223f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 584" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.001729000f, 39.741744000f, 1483290514, 10.535571f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 585" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.001515000f, 39.741650000f, 1483290516, 10.547977f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 586" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.001300000f, 39.741558000f, 1483290518, 10.530594f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 587" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.000660000f, 39.741283000f, 1483290524, 10.459280f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 588" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.999990000f, 39.740962000f, 1483290530, 11.262023f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 589" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.999554000f, 39.740680000f, 1483290534, 12.191996f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 590" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.999337000f, 39.740534000f, 1483290536, 12.340751f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 591" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.999122000f, 39.740390000f, 1483290538, 12.203072f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 592" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998908000f, 39.740248000f, 1483290540, 12.097827f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 593" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998590000f, 39.740035000f, 1483290543, 12.033169f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 594" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997965000f, 39.739609000f, 1483290549, 11.915312f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 595" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997539000f, 39.739328000f, 1483290553, 12.011016f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 596" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997120000f, 39.739053000f, 1483290557, 11.788663f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 597" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996916000f, 39.738923000f, 1483290559, 11.339821f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 598" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996717000f, 39.738793000f, 1483290561, 11.175893f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 599" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996158000f, 39.738428000f, 1483290567, 10.462439f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 600" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995548000f, 39.738032000f, 1483290574, 9.762368f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 601" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994961000f, 39.737649000f, 1483290581, 9.414149f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 602" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994417000f, 39.737286000f, 1483290587, 10.275982f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 603" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994322000f, 39.737222000f, 1483290588, 10.811646f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 604" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994134000f, 39.737092000f, 1483290590, 10.821527f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 605" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993761000f, 39.736826000f, 1483290594, 10.886738f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 606" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993489000f, 39.736628000f, 1483290597, 10.687323f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 607" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992924000f, 39.736206000f, 1483290604, 9.632126f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 608" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992456000f, 39.735890000f, 1483290611, 7.616378f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 609" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992395000f, 39.735853000f, 1483290612, 6.650770f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 610" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992214000f, 39.735758000f, 1483290615, 6.253143f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 611" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992047000f, 39.735681000f, 1483290618, 5.556259f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 612" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991943000f, 39.735633000f, 1483290620, 5.191653f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 613" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991796000f, 39.735569000f, 1483290623, 4.820247f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 614" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991655000f, 39.735511000f, 1483290626, 4.562884f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 615" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991432000f, 39.735422000f, 1483290631, 4.301550f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 616" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991262000f, 39.735354000f, 1483290635, 4.100970f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 617" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991134000f, 39.735303000f, 1483290638, 4.113636f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 618" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991000000f, 39.735249000f, 1483290641, 4.316915f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 619" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.990448000f, 39.735033000f, 1483290652, 4.820006f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 620" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.990399000f, 39.735012000f, 1483290653, 4.802105f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 621" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.990099000f, 39.734892000f, 1483290659, 4.824695f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 622" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989853000f, 39.734802000f, 1483290664, 4.664157f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 623" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989473000f, 39.734687000f, 1483290672, 4.369826f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 624" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989429000f, 39.734667000f, 1483290673, 4.375455f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 625" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989354000f, 39.734516000f, 1483290677, 4.498424f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 626" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989324000f, 39.734339000f, 1483290680, 6.623405f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 627" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989295000f, 39.734269000f, 1483290681, 8.178276f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 628" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989034000f, 39.733793000f, 1483290687, 9.584369f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 629" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989007000f, 39.733704000f, 1483290688, 10.173486f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 630" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988982000f, 39.733521000f, 1483290690, 10.241790f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 631" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988967000f, 39.733336000f, 1483290692, 10.317051f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 632" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988947000f, 39.733061000f, 1483290695, 10.220234f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 633" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988914000f, 39.732516000f, 1483290701, 10.122477f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 634" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988900000f, 39.732332000f, 1483290703, 10.258911f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 635" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988852000f, 39.731781000f, 1483290709, 10.245756f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 636" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988822000f, 39.731505000f, 1483290712, 10.277112f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 637" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988813000f, 39.731222000f, 1483290715, 10.504279f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 638" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989103000f, 39.730793000f, 1483290720, 10.764806f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 639" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989205000f, 39.730731000f, 1483290721, 11.130515f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 640" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989412000f, 39.730613000f, 1483290723, 11.029484f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 641" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989618000f, 39.730498000f, 1483290725, 10.896215f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 642" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989818000f, 39.730384000f, 1483290727, 10.656208f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 643" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.990429000f, 39.730082000f, 1483290733, 10.363402f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 644" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991163000f, 39.729850000f, 1483290740, 9.705617f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 645" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991848000f, 39.729661000f, 1483290747, 8.900606f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 646" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992534000f, 39.729469000f, 1483290755, 7.812300f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 647" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992980000f, 39.729341000f, 1483290761, 6.792544f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 648" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993457000f, 39.729203000f, 1483290768, 6.233027f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 649" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993786000f, 39.729111000f, 1483290773, 5.994155f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 650" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994459000f, 39.728919000f, 1483290783, 6.145415f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 651" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995113000f, 39.728703000f, 1483290793, 6.093581f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 652" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995639000f, 39.728331000f, 1483290801, 7.647319f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 653" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996230000f, 39.727942000f, 1483290808, 9.514043f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 654" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996447000f, 39.727837000f, 1483290810, 10.974714f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 655" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996554000f, 39.727778000f, 1483290811, 11.271926f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 656" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996646000f, 39.727704000f, 1483290812, 11.397319f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 657" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996746000f, 39.727506000f, 1483290814, 11.822819f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 658" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996755000f, 39.727285000f, 1483290816, 12.306836f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 659" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996744000f, 39.727174000f, 1483290817, 12.392301f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 660" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996728000f, 39.727063000f, 1483290818, 12.432163f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 661" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996709000f, 39.726951000f, 1483290819, 12.573455f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 662" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996691000f, 39.726839000f, 1483290820, 12.562665f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 663" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996667000f, 39.726612000f, 1483290822, 12.676464f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 664" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996652000f, 39.726383000f, 1483290824, 12.762246f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 665" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996642000f, 39.726268000f, 1483290825, 12.830339f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 666" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996616000f, 39.726029000f, 1483290827, 13.349160f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 667" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996593000f, 39.725794000f, 1483290829, 13.117045f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 668" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996566000f, 39.725565000f, 1483290831, 12.798381f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 669" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996537000f, 39.725343000f, 1483290833, 12.418672f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 670" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996487000f, 39.724914000f, 1483290837, 11.986887f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 671" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996409000f, 39.724304000f, 1483290843, 11.372082f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 672" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996352000f, 39.723722000f, 1483290849, 10.828583f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 673" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996342000f, 39.723627000f, 1483290850, 10.609955f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 674" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996286000f, 39.723071000f, 1483290856, 10.346513f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 675" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996227000f, 39.722523000f, 1483290863, 8.744555f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 676" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996207000f, 39.722388000f, 1483290865, 7.562690f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 677" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996186000f, 39.722179000f, 1483290868, 7.778383f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 678" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996123000f, 39.721578000f, 1483290875, 9.588589f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 679" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996058000f, 39.721036000f, 1483290881, 10.098551f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 680" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996011000f, 39.720625000f, 1483290886, 9.185791f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 681" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995953000f, 39.720097000f, 1483290894, 7.373265f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 682" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995943000f, 39.719941000f, 1483290897, 5.795646f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 683" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995942000f, 39.719891000f, 1483290898, 5.566633f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 684" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995946000f, 39.719792000f, 1483290900, 5.512975f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 685" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995951000f, 39.719638000f, 1483290903, 5.716182f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 686" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995952000f, 39.719491000f, 1483290906, 5.454730f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 687" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995940000f, 39.719308000f, 1483290910, 5.099341f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 688" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995920000f, 39.719121000f, 1483290914, 5.221766f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 689" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995885000f, 39.718784000f, 1483290921, 5.376312f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 690" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995851000f, 39.718437000f, 1483290928, 5.533916f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 691" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995811000f, 39.717898000f, 1483290939, 5.463535f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 692" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995808000f, 39.717850000f, 1483290940, 5.349507f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 693" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995788000f, 39.717614000f, 1483290945, 5.265431f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 694" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995756000f, 39.717032000f, 1483290956, 5.895078f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 695" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995717000f, 39.716457000f, 1483290965, 7.121751f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 696" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995675000f, 39.715958000f, 1483290972, 7.952104f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 697" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995648000f, 39.715659000f, 1483290976, 8.341182f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 698" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995619000f, 39.715342000f, 1483290980, 8.843886f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 699" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995579000f, 39.714911000f, 1483290985, 9.620162f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 700" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995547000f, 39.714420000f, 1483290990, 10.945303f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 701" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995521000f, 39.714019000f, 1483290994, 11.173651f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 702" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995507000f, 39.713811000f, 1483290996, 11.592734f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 703" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995501000f, 39.713708000f, 1483290997, 11.477413f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 704" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995493000f, 39.713402000f, 1483291000, 11.356884f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 705" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995479000f, 39.712899000f, 1483291005, 11.201307f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 706" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995443000f, 39.712326000f, 1483291011, 10.643420f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 707" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995423000f, 39.712145000f, 1483291013, 10.110744f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 708" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995373000f, 39.711563000f, 1483291020, 9.275611f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 709" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995358000f, 39.711226000f, 1483291025, 7.507331f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 710" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995350000f, 39.711051000f, 1483291028, 6.497651f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 711" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995339000f, 39.710882000f, 1483291031, 6.278854f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 712" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995332000f, 39.710765000f, 1483291033, 6.519084f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 713" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995297000f, 39.710159000f, 1483291042, 7.502907f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 714" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995266000f, 39.709566000f, 1483291049, 9.437974f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 715" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995249000f, 39.709309000f, 1483291052, 9.548709f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 716" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995222000f, 39.708978000f, 1483291056, 9.229807f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 717" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995162000f, 39.708431000f, 1483291063, 8.729739f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 718" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995129000f, 39.707824000f, 1483291071, 8.453751f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 719" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995170000f, 39.707217000f, 1483291077, 11.277017f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 720" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995182000f, 39.707108000f, 1483291078, 12.177267f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 721" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995214000f, 39.706888000f, 1483291080, 12.321571f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 722" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995249000f, 39.706674000f, 1483291082, 12.005102f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 723" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995266000f, 39.706570000f, 1483291083, 11.668411f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 724" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995307000f, 39.706375000f, 1483291085, 10.994726f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 725" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995335000f, 39.706180000f, 1483291087, 10.919674f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 726" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995364000f, 39.705802000f, 1483291091, 10.538000f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 727" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995395000f, 39.705394000f, 1483291096, 9.099176f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 728" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995429000f, 39.705108000f, 1483291100, 7.992564f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 729" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995443000f, 39.704967000f, 1483291102, 7.870888f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 730" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995454000f, 39.704830000f, 1483291104, 7.639920f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 731" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995510000f, 39.704288000f, 1483291112, 7.565685f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 732" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995520000f, 39.704164000f, 1483291114, 6.915080f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 733" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995546000f, 39.703739000f, 1483291125, 4.305742f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 734" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995484000f, 39.703590000f, 1483291130, 3.483166f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 735" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995463000f, 39.703551000f, 1483291131, 4.699257f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 736" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995430000f, 39.703244000f, 1483291137, 5.715292f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 737" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995396000f, 39.702682000f, 1483291147, 6.262928f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 738" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995341000f, 39.702092000f, 1483291156, 7.316356f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 739" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995302000f, 39.701592000f, 1483291163, 7.965697f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 740" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995257000f, 39.701052000f, 1483291171, 7.529494f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 741" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995239000f, 39.700878000f, 1483291174, 6.476949f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 742" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995211000f, 39.700670000f, 1483291178, 5.819578f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 743" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995193000f, 39.700529000f, 1483291181, 5.257193f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 744" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995175000f, 39.700400000f, 1483291184, 4.814244f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 745" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995159000f, 39.700270000f, 1483291187, 4.845424f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 746" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995093000f, 39.699711000f, 1483291197, 6.248383f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 747" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995030000f, 39.699089000f, 1483291204, 9.921523f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 748" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995021000f, 39.698987000f, 1483291205, 11.380724f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 749" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995002000f, 39.698780000f, 1483291207, 11.550263f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 750" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994986000f, 39.698569000f, 1483291209, 11.764178f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 751" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994974000f, 39.698360000f, 1483291211, 11.644233f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 752" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994963000f, 39.698153000f, 1483291213, 11.531194f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 753" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994953000f, 39.697948000f, 1483291215, 11.418282f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 754" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994927000f, 39.697650000f, 1483291218, 11.082624f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 755" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994869000f, 39.697102000f, 1483291224, 10.200838f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 756" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994795000f, 39.696502000f, 1483291232, 8.386471f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 757" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994769000f, 39.696276000f, 1483291236, 6.314145f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 758" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994752000f, 39.696132000f, 1483291239, 5.365335f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 759" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994745000f, 39.696045000f, 1483291241, 4.851669f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 760" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994738000f, 39.695963000f, 1483291243, 4.573934f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 761" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994738000f, 39.695851000f, 1483291246, 4.155928f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 762" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994702000f, 39.695677000f, 1483291250, 4.903375f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 763" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994672000f, 39.695362000f, 1483291256, 5.859944f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 764" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994596000f, 39.694822000f, 1483291266, 6.046398f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 765" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994562000f, 39.694551000f, 1483291272, 5.051305f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 766" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994552000f, 39.694510000f, 1483291273, 4.643780f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 767" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994598000f, 39.694243000f, 1483291281, 3.747792f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 768" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995288000f, 39.693985000f, 1483291292, 5.973788f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 769" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995730000f, 39.693841000f, 1483291298, 6.852334f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 770" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996401000f, 39.693616000f, 1483291307, 6.966246f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 771" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997105000f, 39.693385000f, 1483291316, 7.284065f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 772" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997187000f, 39.693360000f, 1483291317, 7.555149f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 773" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997842000f, 39.693156000f, 1483291325, 7.565913f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 774" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998079000f, 39.693076000f, 1483291328, 7.389441f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 775" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998148000f, 39.693038000f, 1483291329, 7.268202f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 776" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998249000f, 39.692730000f, 1483291334, 7.072214f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 777" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998228000f, 39.692121000f, 1483291343, 7.535270f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 778" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998218000f, 39.691597000f, 1483291350, 8.333958f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 779" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998204000f, 39.691056000f, 1483291357, 8.605112f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 780" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998176000f, 39.690425000f, 1483291364, 10.040505f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 781" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998172000f, 39.690324000f, 1483291365, 11.248488f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 782" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998158000f, 39.690025000f, 1483291368, 11.102042f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 783" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998155000f, 39.689826000f, 1483291370, 11.077035f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 784" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998158000f, 39.689726000f, 1483291371, 11.134915f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 785" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998163000f, 39.689624000f, 1483291372, 11.362663f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 786" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998177000f, 39.689418000f, 1483291374, 11.481577f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 787" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998226000f, 39.688992000f, 1483291378, 11.901876f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 788" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998264000f, 39.688665000f, 1483291381, 12.182243f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 789" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998286000f, 39.688449000f, 1483291383, 12.059376f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 790" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998315000f, 39.688121000f, 1483291386, 12.199069f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 791" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998326000f, 39.687904000f, 1483291388, 12.087351f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 792" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998328000f, 39.687582000f, 1483291391, 11.948428f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 793" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998300000f, 39.686985000f, 1483291397, 11.083501f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 794" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998297000f, 39.686530000f, 1483291402, 10.130204f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 795" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998297000f, 39.686045000f, 1483291408, 8.998325f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 796" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998286000f, 39.685844000f, 1483291411, 7.465017f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 797" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998255000f, 39.685665000f, 1483291414, 6.700793f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 798" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998239000f, 39.685608000f, 1483291415, 6.491567f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 799" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998215000f, 39.685552000f, 1483291416, 6.564189f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 800" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998097000f, 39.685398000f, 1483291419, 6.633884f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 801" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998049000f, 39.685351000f, 1483291420, 6.654531f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 802" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997894000f, 39.685211000f, 1483291423, 6.824800f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 803" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997420000f, 39.684810000f, 1483291432, 6.705019f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 804" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997066000f, 39.684502000f, 1483291439, 6.539139f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 805" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996603000f, 39.684040000f, 1483291449, 6.494833f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 806" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996188000f, 39.683570000f, 1483291457, 7.907074f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 807" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995857000f, 39.683187000f, 1483291462, 10.240853f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 808" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995721000f, 39.683026000f, 1483291464, 10.688344f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 809" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995528000f, 39.682788000f, 1483291467, 10.410037f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 810" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995396000f, 39.682637000f, 1483291469, 10.129577f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 811" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994907000f, 39.682254000f, 1483291475, 9.962158f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 812" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994719000f, 39.682158000f, 1483291477, 9.664555f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 813" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994120000f, 39.681860000f, 1483291484, 8.729401f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 814" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993685000f, 39.681636000f, 1483291490, 7.473341f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 815" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993424000f, 39.681487000f, 1483291494, 6.960168f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 816" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993189000f, 39.681332000f, 1483291498, 6.628797f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 817" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992579000f, 39.680939000f, 1483291507, 7.572752f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 818" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991995000f, 39.680575000f, 1483291514, 9.197605f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 819" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991741000f, 39.680419000f, 1483291517, 9.280296f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 820" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991175000f, 39.680076000f, 1483291524, 8.817094f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 821" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.990562000f, 39.679703000f, 1483291532, 8.368707f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 822" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989970000f, 39.679342000f, 1483291539, 9.244296f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 823" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989609000f, 39.679120000f, 1483291543, 9.897318f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 824" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989334000f, 39.678951000f, 1483291546, 10.050064f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 825" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988719000f, 39.678560000f, 1483291553, 9.763310f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 826" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988132000f, 39.678193000f, 1483291561, 8.099321f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 827" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.987576000f, 39.677859000f, 1483291569, 7.553562f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 828" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.987027000f, 39.677489000f, 1483291576, 8.931612f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 829" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.986734000f, 39.677204000f, 1483291580, 10.114116f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 830" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.986595000f, 39.677066000f, 1483291582, 9.718826f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 831" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.986119000f, 39.676595000f, 1483291590, 8.303132f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 832" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.986073000f, 39.676531000f, 1483291591, 8.141916f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 833" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.986169000f, 39.676178000f, 1483291596, 8.029473f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 834" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.986586000f, 39.675661000f, 1483291604, 8.467544f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 835" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.986909000f, 39.675514000f, 1483291608, 8.037627f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 836" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.987600000f, 39.675340000f, 1483291616, 7.786578f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 837" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988342000f, 39.675160000f, 1483291624, 8.332180f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 838" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.988540000f, 39.675113000f, 1483291626, 8.876557f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 839" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989029000f, 39.674984000f, 1483291631, 8.858043f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 840" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.989520000f, 39.674705000f, 1483291637, 8.715271f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 841" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.990126000f, 39.674311000f, 1483291645, 8.495993f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 842" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.990698000f, 39.673998000f, 1483291653, 7.516630f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 843" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991307000f, 39.673718000f, 1483291660, 8.682961f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 844" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.991974000f, 39.673404000f, 1483291668, 8.373985f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 845" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992613000f, 39.673168000f, 1483291676, 7.590967f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 846" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.992877000f, 39.673096000f, 1483291680, 5.999546f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 847" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993045000f, 39.673051000f, 1483291683, 5.080469f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 848" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993146000f, 39.673025000f, 1483291685, 4.562559f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 849" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993282000f, 39.672987000f, 1483291688, 4.132296f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 850" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993444000f, 39.672944000f, 1483291692, 3.670694f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 851" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993521000f, 39.672923000f, 1483291694, 3.499744f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 852" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993670000f, 39.672882000f, 1483291698, 3.389511f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 853" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993778000f, 39.672845000f, 1483291701, 3.376331f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 854" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.993967000f, 39.672773000f, 1483291706, 3.613796f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 855" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994233000f, 39.672662000f, 1483291712, 4.320953f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 856" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.994885000f, 39.672407000f, 1483291725, 4.820287f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 857" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995042000f, 39.672363000f, 1483291728, 4.772087f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 858" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.995685000f, 39.672170000f, 1483291740, 4.927952f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 859" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996122000f, 39.672039000f, 1483291748, 5.022914f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 860" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996447000f, 39.671941000f, 1483291754, 4.984653f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 861" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.996764000f, 39.671847000f, 1483291760, 4.851293f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 862" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.997452000f, 39.671649000f, 1483291772, 5.244711f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 863" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998159000f, 39.671385000f, 1483291781, 7.481239f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 864" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998505000f, 39.671241000f, 1483291785, 8.425784f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 865" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.998672000f, 39.671173000f, 1483291787, 8.094118f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 866" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.999301000f, 39.670925000f, 1483291795, 7.569397f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 867" << std::endl;

  nav.SnapToRoute(GetFixLocation(-75.999781000f, 39.670722000f, 1483291803, 5.866024f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 868" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.000026000f, 39.670618000f, 1483291808, 4.794733f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 869" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.000176000f, 39.670554000f, 1483291811, 4.898452f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 870" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.000334000f, 39.670488000f, 1483291814, 5.134486f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 871" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.000532000f, 39.670409000f, 1483291818, 4.777400f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 872" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.000756000f, 39.670321000f, 1483291822, 5.387254f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 873" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.001273000f, 39.670104000f, 1483291831, 5.606427f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 874" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.001806000f, 39.669883000f, 1483291841, 5.187548f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 875" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.002476000f, 39.669610000f, 1483291851, 6.495756f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 876" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.003125000f, 39.669345000f, 1483291860, 6.994531f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 877" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.003789000f, 39.669082000f, 1483291869, 7.109675f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 878" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.004459000f, 39.668811000f, 1483291880, 5.895844f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 879" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.004793000f, 39.668671000f, 1483291887, 4.655423f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 880" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.004929000f, 39.668609000f, 1483291890, 4.514678f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 881" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.005067000f, 39.668551000f, 1483291893, 4.490936f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 882" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.005291000f, 39.668461000f, 1483291898, 4.330318f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 883" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.005578000f, 39.668342000f, 1483291904, 4.655575f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 884" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.006231000f, 39.668075000f, 1483291914, 6.335882f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 885" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.006795000f, 39.667844000f, 1483291921, 7.820564f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 886" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.007155000f, 39.667700000f, 1483291925, 8.691109f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 887" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.007508000f, 39.667556000f, 1483291929, 8.558339f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 888" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.008166000f, 39.667287000f, 1483291937, 7.980275f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 889" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.008837000f, 39.667015000f, 1483291946, 7.220363f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 890" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.009430000f, 39.666724000f, 1483291954, 7.532705f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 891" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.010014000f, 39.666417000f, 1483291963, 6.733253f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 892" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.010376000f, 39.666229000f, 1483291969, 6.236612f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 893" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.010966000f, 39.665930000f, 1483291978, 6.725623f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 894" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.011550000f, 39.665625000f, 1483291986, 7.559309f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 895" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012134000f, 39.665332000f, 1483291994, 7.466862f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 896" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012733000f, 39.665022000f, 1483292004, 6.185156f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 897" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.012958000f, 39.664909000f, 1483292009, 4.604279f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 898" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013105000f, 39.664836000f, 1483292012, 4.996858f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 899" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013353000f, 39.664708000f, 1483292017, 5.117309f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 900" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.013800000f, 39.664484000f, 1483292026, 5.078447f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 901" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014246000f, 39.664256000f, 1483292035, 5.097685f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 902" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014481000f, 39.664146000f, 1483292044, 2.618741f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 903" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014763000f, 39.663991000f, 1483292063, 1.562809f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 904" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.014947000f, 39.663909000f, 1483292069, 3.036553f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 905" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015286000f, 39.663841000f, 1483292076, 4.288614f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 906" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015602000f, 39.663787000f, 1483292081, 5.547702f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 907" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.015936000f, 39.663722000f, 1483292086, 5.904459f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 908" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016458000f, 39.663616000f, 1483292094, 5.782819f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 909" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.016884000f, 39.663527000f, 1483292101, 5.403760f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 910" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.017532000f, 39.663392000f, 1483292112, 5.229785f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 911" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.018248000f, 39.663216000f, 1483292122, 6.440952f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 912" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.018875000f, 39.662938000f, 1483292131, 6.889515f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 913" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.019319000f, 39.662741000f, 1483292137, 7.319354f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 914" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.019988000f, 39.662481000f, 1483292146, 7.135779f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 915" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.020701000f, 39.662297000f, 1483292156, 6.444293f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 916" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.021390000f, 39.662154000f, 1483292166, 6.115280f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 917" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.022122000f, 39.662002000f, 1483292175, 7.219069f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 918" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.022574000f, 39.661893000f, 1483292180, 8.118155f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 919" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.022668000f, 39.661871000f, 1483292181, 8.419518f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 920" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.023134000f, 39.661751000f, 1483292186, 8.421916f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 921" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.023514000f, 39.661655000f, 1483292190, 8.568354f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 922" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.024081000f, 39.661515000f, 1483292196, 8.504707f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 923" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.024728000f, 39.661364000f, 1483292203, 8.276849f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 924" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.025472000f, 39.661178000f, 1483292211, 8.379560f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 925" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.025990000f, 39.661005000f, 1483292217, 8.064783f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 926" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.026551000f, 39.660675000f, 1483292224, 8.643549f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 927" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.027031000f, 39.660257000f, 1483292230, 10.351173f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 928" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.027185000f, 39.660101000f, 1483292232, 10.905830f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 929" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.027332000f, 39.659943000f, 1483292234, 10.817314f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 930" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.027469000f, 39.659786000f, 1483292236, 10.527304f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 931" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.027710000f, 39.659460000f, 1483292240, 10.438939f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 932" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.028071000f, 39.658964000f, 1483292246, 10.548530f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 933" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.028198000f, 39.658798000f, 1483292248, 10.723042f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 934" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.028472000f, 39.658469000f, 1483292252, 10.876369f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 935" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.028799000f, 39.658161000f, 1483292256, 11.070548f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 936" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.028985000f, 39.658017000f, 1483292258, 11.303295f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 937" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.029268000f, 39.657802000f, 1483292261, 11.358078f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 938" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.029554000f, 39.657585000f, 1483292264, 11.471226f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 939" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.029742000f, 39.657442000f, 1483292266, 11.324747f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 940" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.029922000f, 39.657301000f, 1483292268, 11.003849f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 941" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.030098000f, 39.657163000f, 1483292270, 10.764624f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 942" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.030588000f, 39.656771000f, 1483292276, 10.093625f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 943" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.031107000f, 39.656323000f, 1483292283, 9.546439f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 944" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.031631000f, 39.655863000f, 1483292290, 9.729972f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 945" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.031855000f, 39.655663000f, 1483292293, 9.799280f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 946" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.032372000f, 39.655209000f, 1483292300, 9.601765f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 947" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.032866000f, 39.654786000f, 1483292307, 9.046165f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 948" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.033336000f, 39.654378000f, 1483292314, 8.672538f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 949" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.033844000f, 39.653934000f, 1483292322, 8.233440f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 950" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.034317000f, 39.653523000f, 1483292330, 7.641070f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 951" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.034856000f, 39.653120000f, 1483292338, 8.049318f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 952" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.035271000f, 39.652815000f, 1483292344, 8.195349f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 953" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.035773000f, 39.652437000f, 1483292351, 8.597349f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 954" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.035934000f, 39.652319000f, 1483292353, 9.525751f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 955" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.036256000f, 39.652077000f, 1483292357, 9.641652f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 956" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.036492000f, 39.651899000f, 1483292360, 9.438520f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 957" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.036647000f, 39.651785000f, 1483292362, 9.186061f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 958" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.036955000f, 39.651559000f, 1483292366, 9.116644f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 959" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.037354000f, 39.651258000f, 1483292371, 9.575464f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 960" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.037594000f, 39.651073000f, 1483292374, 9.702554f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 961" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.037816000f, 39.650897000f, 1483292377, 9.103757f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 962" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.037964000f, 39.650793000f, 1483292379, 8.586967f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 963" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038342000f, 39.650862000f, 1483292383, 8.324127f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 964" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038427000f, 39.650925000f, 1483292384, 10.112388f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 965" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038597000f, 39.651053000f, 1483292386, 10.189898f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 966" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038763000f, 39.651179000f, 1483292388, 9.989581f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 967" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038839000f, 39.651242000f, 1483292389, 9.571589f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 968" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.038911000f, 39.651305000f, 1483292390, 9.341645f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 969" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039050000f, 39.651431000f, 1483292392, 9.201497f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 970" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039331000f, 39.651780000f, 1483292397, 9.142022f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 971" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039507000f, 39.652046000f, 1483292401, 8.307979f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 972" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039697000f, 39.652333000f, 1483292406, 7.171911f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 973" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039795000f, 39.652477000f, 1483292409, 6.032423f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 974" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039880000f, 39.652604000f, 1483292412, 5.301414f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 975" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.039972000f, 39.652728000f, 1483292415, 5.299005f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 976" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.040091000f, 39.652844000f, 1483292418, 5.485043f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 977" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.040265000f, 39.652992000f, 1483292422, 5.555605f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 978" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.040561000f, 39.653160000f, 1483292428, 5.252931f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 979" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.040848000f, 39.653268000f, 1483292434, 4.563139f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 980" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.040944000f, 39.653301000f, 1483292436, 4.505357f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 981" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.041089000f, 39.653354000f, 1483292439, 4.585637f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 982" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.041240000f, 39.653404000f, 1483292442, 4.695969f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 983" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.041693000f, 39.653561000f, 1483292451, 4.730839f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 984" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.041886000f, 39.653627000f, 1483292455, 4.524914f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 985" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.042087000f, 39.653695000f, 1483292459, 4.704197f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 986" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.042387000f, 39.653799000f, 1483292465, 4.699696f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 987" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.042655000f, 39.653890000f, 1483292470, 5.020789f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 988" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.043092000f, 39.654041000f, 1483292478, 5.131590f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 989" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.043763000f, 39.654279000f, 1483292491, 4.870628f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 990" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.043857000f, 39.654314000f, 1483292493, 4.474512f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 991" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.044160000f, 39.654429000f, 1483292500, 4.136117f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 992" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.044318000f, 39.654486000f, 1483292504, 3.738597f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 993" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.044445000f, 39.654529000f, 1483292507, 3.963551f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 994" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.044627000f, 39.654596000f, 1483292511, 4.322457f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 995" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045185000f, 39.654810000f, 1483292523, 4.452372f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 996" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045447000f, 39.654912000f, 1483292529, 4.193721f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 997" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045630000f, 39.654980000f, 1483292533, 4.353801f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 998" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.045901000f, 39.655086000f, 1483292538, 5.210320f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 999" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046074000f, 39.655155000f, 1483292541, 5.566133f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1000" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046257000f, 39.655198000f, 1483292544, 5.466055f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1001" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046456000f, 39.655168000f, 1483292547, 5.793045f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1002" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.046643000f, 39.655053000f, 1483292550, 6.837341f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1003" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047237000f, 39.654673000f, 1483292559, 7.354449f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1004" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047803000f, 39.654320000f, 1483292569, 6.242850f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1005" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047900000f, 39.654259000f, 1483292571, 5.367134f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1006" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.047995000f, 39.654201000f, 1483292573, 5.195671f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1007" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048257000f, 39.654036000f, 1483292579, 4.835056f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1008" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048468000f, 39.653903000f, 1483292585, 3.895277f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1009" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048555000f, 39.653850000f, 1483292588, 3.169436f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1010" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048639000f, 39.653800000f, 1483292591, 3.033344f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1011" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048722000f, 39.653749000f, 1483292594, 3.033801f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1012" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048809000f, 39.653697000f, 1483292597, 3.146550f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1013" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.048903000f, 39.653638000f, 1483292600, 3.464785f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1014" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.049009000f, 39.653575000f, 1483292603, 3.825639f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1015" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.049213000f, 39.653454000f, 1483292608, 4.414198f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1016" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.049768000f, 39.653118000f, 1483292620, 5.042646f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1017" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050085000f, 39.652926000f, 1483292627, 4.938372f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1018" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050343000f, 39.652781000f, 1483292633, 4.562858f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1019" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050455000f, 39.652736000f, 1483292636, 3.609245f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1020" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050548000f, 39.652737000f, 1483292640, 1.992900f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1021" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050603000f, 39.652747000f, 1483292643, 1.614529f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1022" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050630000f, 39.652737000f, 1483292644, 2.567939f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1023" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.050706000f, 39.652656000f, 1483292647, 3.707854f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1024" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.051175000f, 39.652232000f, 1483292657, 6.199676f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1025" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.051435000f, 39.652016000f, 1483292661, 8.195842f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1026" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.051648000f, 39.651842000f, 1483292664, 8.872314f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1027" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.051939000f, 39.651606000f, 1483292668, 9.056271f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1028" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.052164000f, 39.651427000f, 1483292671, 9.243307f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1029" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.052388000f, 39.651244000f, 1483292674, 9.330927f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1030" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.052689000f, 39.651000000f, 1483292678, 9.365302f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1031" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.053216000f, 39.650574000f, 1483292685, 9.355910f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1032" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.053442000f, 39.650392000f, 1483292688, 9.343408f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1033" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.053943000f, 39.649986000f, 1483292695, 8.906084f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1034" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.054492000f, 39.649542000f, 1483292703, 8.530410f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1035" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.054895000f, 39.649218000f, 1483292709, 8.323343f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1036" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.055399000f, 39.648810000f, 1483292717, 7.835202f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1037" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.055933000f, 39.648374000f, 1483292726, 7.412562f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1038" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.056106000f, 39.648236000f, 1483292729, 7.117078f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1039" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.056612000f, 39.647830000f, 1483292738, 6.959942f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1040" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.056723000f, 39.647740000f, 1483292740, 6.908262f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1041" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.057213000f, 39.647343000f, 1483292748, 7.620996f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1042" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.057699000f, 39.646951000f, 1483292755, 8.618376f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1043" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.058141000f, 39.646588000f, 1483292761, 9.231954f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1044" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.058509000f, 39.646278000f, 1483292766, 9.350629f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1045" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.058942000f, 39.645905000f, 1483292772, 9.282027f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1046" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.059434000f, 39.645484000f, 1483292779, 9.006684f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1047" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.059903000f, 39.645078000f, 1483292786, 8.641141f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1048" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.060421000f, 39.644639000f, 1483292794, 8.253517f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1049" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.060937000f, 39.644199000f, 1483292802, 8.249468f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1050" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.061446000f, 39.643762000f, 1483292810, 8.168288f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1051" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.061950000f, 39.643333000f, 1483292818, 8.049743f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1052" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.062458000f, 39.642932000f, 1483292826, 7.795101f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1053" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.062703000f, 39.642753000f, 1483292830, 7.237543f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1054" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.063050000f, 39.642503000f, 1483292836, 6.789011f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1055" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.063497000f, 39.642196000f, 1483292844, 6.417919f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1056" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.063834000f, 39.641982000f, 1483292850, 6.240594f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1057" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.064411000f, 39.641633000f, 1483292859, 6.988327f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1058" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.064842000f, 39.641388000f, 1483292865, 7.653689f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1059" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.065475000f, 39.641050000f, 1483292874, 7.336773f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1060" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.066093000f, 39.640739000f, 1483292883, 7.031743f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1061" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.066660000f, 39.640445000f, 1483292892, 6.510722f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1062" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.066969000f, 39.640264000f, 1483292898, 5.546804f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1063" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.067179000f, 39.640148000f, 1483292902, 5.538589f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1064" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.067436000f, 39.639999000f, 1483292907, 5.515352f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1065" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.067741000f, 39.639816000f, 1483292913, 5.524179f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1066" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.068339000f, 39.639458000f, 1483292924, 5.902867f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1067" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.068550000f, 39.639333000f, 1483292928, 5.705235f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1068" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.068883000f, 39.639136000f, 1483292935, 5.142484f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1069" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.069147000f, 39.638982000f, 1483292941, 4.731885f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1070" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.069321000f, 39.638874000f, 1483292945, 4.789518f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1071" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.069527000f, 39.638745000f, 1483292950, 4.552225f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1072" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.069707000f, 39.638640000f, 1483292954, 4.839445f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1073" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.069954000f, 39.638490000f, 1483292960, 4.494336f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1074" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.070144000f, 39.638373000f, 1483292965, 4.170988f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1075" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.070407000f, 39.638222000f, 1483292972, 4.017478f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1076" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.070693000f, 39.638047000f, 1483292979, 4.473548f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1077" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.071097000f, 39.637797000f, 1483292988, 4.936601f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1078" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.071266000f, 39.637729000f, 1483292991, 5.448708f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1079" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.071800000f, 39.637766000f, 1483292999, 5.745350f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1080" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.072161000f, 39.637872000f, 1483293006, 4.731495f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1081" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.072405000f, 39.637913000f, 1483293013, 3.058477f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1082" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.072459000f, 39.637900000f, 1483293014, 4.850142f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1083" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.072620000f, 39.637724000f, 1483293018, 5.991394f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1084" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.072926000f, 39.637221000f, 1483293027, 6.870434f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1085" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.073128000f, 39.636952000f, 1483293032, 6.918307f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1086" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.073210000f, 39.636847000f, 1483293034, 6.819789f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1087" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.073383000f, 39.636645000f, 1483293038, 6.734238f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1088" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.073612000f, 39.636408000f, 1483293043, 6.577085f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1089" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.073856000f, 39.636171000f, 1483293050, 4.809848f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1090" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.074400000f, 39.635727000f, 1483293059, 7.550542f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1091" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.074474000f, 39.635670000f, 1483293060, 8.972585f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1092" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.074700000f, 39.635498000f, 1483293063, 9.079815f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1093" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.074990000f, 39.635280000f, 1483293067, 8.685522f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1094" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.075338000f, 39.635021000f, 1483293072, 8.297786f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1095" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.075878000f, 39.634610000f, 1483293081, 7.231983f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1096" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.076089000f, 39.634444000f, 1483293085, 6.464763f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1097" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.076195000f, 39.634363000f, 1483293087, 6.400890f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1098" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.076589000f, 39.634069000f, 1483293094, 6.718942f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1099" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.077153000f, 39.633651000f, 1483293102, 8.388179f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1100" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.077525000f, 39.633376000f, 1483293107, 8.841396f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1101" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.078082000f, 39.632965000f, 1483293115, 8.266649f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1102" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.078625000f, 39.632562000f, 1483293124, 7.183433f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1103" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.078885000f, 39.632367000f, 1483293129, 6.222784f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1104" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.079174000f, 39.632155000f, 1483293137, 4.277195f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1105" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.079270000f, 39.632083000f, 1483293142, 2.297649f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1106" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.079388000f, 39.631970000f, 1483293150, 2.017805f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1107" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.079530000f, 39.631880000f, 1483293153, 5.255535f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1108" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.079913000f, 39.631600000f, 1483293159, 7.545679f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1109" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.080138000f, 39.631435000f, 1483293162, 8.878712f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1110" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.080284000f, 39.631323000f, 1483293164, 8.833544f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1111" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.080505000f, 39.631159000f, 1483293167, 8.770510f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1112" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.080780000f, 39.630955000f, 1483293171, 8.183762f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1113" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.081220000f, 39.630621000f, 1483293178, 7.566656f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1114" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.081567000f, 39.630371000f, 1483293184, 6.789646f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1115" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.081890000f, 39.630129000f, 1483293190, 6.439048f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1116" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.082157000f, 39.629930000f, 1483293195, 6.371051f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1117" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.082648000f, 39.629574000f, 1483293204, 6.423945f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1118" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.083181000f, 39.629172000f, 1483293214, 6.395999f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1119" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.083232000f, 39.629133000f, 1483293215, 6.161802f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1120" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.083712000f, 39.628781000f, 1483293224, 6.313873f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1121" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.084255000f, 39.628370000f, 1483293234, 6.527401f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1122" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.084481000f, 39.628204000f, 1483293238, 6.693913f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1123" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.084995000f, 39.627826000f, 1483293247, 6.770258f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1124" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.085526000f, 39.627432000f, 1483293256, 7.024147f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1125" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.085825000f, 39.627205000f, 1483293261, 7.199322f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1126" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.086364000f, 39.626805000f, 1483293270, 7.130555f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1127" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.086903000f, 39.626414000f, 1483293279, 7.053793f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1128" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.087418000f, 39.626028000f, 1483293288, 6.845882f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1129" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.087692000f, 39.625816000f, 1483293293, 6.659941f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1130" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.088212000f, 39.625411000f, 1483293303, 6.340720f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1131" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.088358000f, 39.625297000f, 1483293306, 5.941890f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1132" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.088651000f, 39.625489000f, 1483293312, 5.497349f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1133" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.088930000f, 39.626000000f, 1483293321, 6.856620f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1134" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089195000f, 39.626550000f, 1483293330, 7.256197f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1135" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089261000f, 39.626849000f, 1483293334, 8.440536f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1136" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089312000f, 39.627294000f, 1483293339, 9.945958f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1137" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089336000f, 39.627484000f, 1483293341, 10.625283f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1138" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089355000f, 39.627675000f, 1483293343, 10.662169f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1139" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089395000f, 39.627965000f, 1483293346, 10.821437f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1140" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089430000f, 39.628154000f, 1483293348, 10.626156f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1141" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089488000f, 39.628337000f, 1483293350, 10.484817f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1142" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089533000f, 39.628436000f, 1483293351, 11.676475f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1143" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089582000f, 39.628527000f, 1483293352, 10.966681f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1144" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089690000f, 39.628695000f, 1483293354, 10.434248f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1145" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.089894000f, 39.628940000f, 1483293357, 10.799933f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1146" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.090207000f, 39.629251000f, 1483293361, 10.950813f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1147" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.090450000f, 39.629497000f, 1483293364, 11.469636f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1148" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.090694000f, 39.629746000f, 1483293367, 11.575605f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1149" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.090846000f, 39.629907000f, 1483293369, 11.079759f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1150" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.090971000f, 39.630078000f, 1483293371, 10.922562f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1151" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091173000f, 39.630674000f, 1483293378, 9.795647f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1152" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091308000f, 39.631270000f, 1483293386, 8.418551f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1153" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091375000f, 39.631590000f, 1483293392, 6.013733f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1154" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091398000f, 39.631718000f, 1483293395, 4.794897f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1155" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091406000f, 39.631760000f, 1483293396, 4.725459f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1156" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091430000f, 39.631886000f, 1483293399, 4.725458f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1157" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091444000f, 39.631970000f, 1483293401, 4.713778f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1158" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091457000f, 39.632090000f, 1483293404, 4.468251f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1159" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091485000f, 39.632340000f, 1483293410, 4.655535f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1160" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091459000f, 39.632824000f, 1483293422, 4.493727f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1161" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091494000f, 39.633036000f, 1483293428, 3.964955f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1162" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091531000f, 39.633063000f, 1483293429, 4.369892f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1163" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.091825000f, 39.633109000f, 1483293433, 6.430038f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1164" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.092628000f, 39.633133000f, 1483293440, 9.842101f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1165" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.092758000f, 39.633137000f, 1483293441, 11.154063f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1166" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.093017000f, 39.633146000f, 1483293443, 11.113601f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1167" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.093267000f, 39.633157000f, 1483293445, 10.733984f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1168" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.093517000f, 39.633166000f, 1483293447, 10.728209f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1169" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.093767000f, 39.633173000f, 1483293449, 10.723586f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1170" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.094259000f, 39.633193000f, 1483293453, 10.559719f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1171" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.094950000f, 39.633224000f, 1483293459, 9.890206f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1172" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.095525000f, 39.633245000f, 1483293464, 9.870256f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1173" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.095867000f, 39.633252000f, 1483293467, 9.776894f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1174" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.095980000f, 39.633253000f, 1483293468, 9.688350f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1175" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.096432000f, 39.633254000f, 1483293472, 9.687750f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1176" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.096990000f, 39.633270000f, 1483293477, 9.574313f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1177" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097776000f, 39.633318000f, 1483293484, 9.656684f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1178" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.097884000f, 39.633329000f, 1483293485, 9.339661f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1179" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.098311000f, 39.633391000f, 1483293489, 9.313106f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1180" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.099006000f, 39.633596000f, 1483293496, 9.114877f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1181" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.099636000f, 39.633854000f, 1483293503, 8.738871f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1182" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.099726000f, 39.633887000f, 1483293504, 8.545676f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1183" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.100460000f, 39.634104000f, 1483293512, 8.425484f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1184" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.101129000f, 39.634283000f, 1483293519, 8.673820f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1185" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.101866000f, 39.634490000f, 1483293527, 8.406776f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1186" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.102544000f, 39.634674000f, 1483293535, 7.703567f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1187" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.103204000f, 39.634863000f, 1483293544, 6.707432f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1188" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.103629000f, 39.634989000f, 1483293551, 5.577395f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1189" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.104367000f, 39.635214000f, 1483293560, 7.560658f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1190" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.105084000f, 39.635419000f, 1483293566, 10.927903f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1191" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.105213000f, 39.635454000f, 1483293567, 11.725327f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1192" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.105485000f, 39.635526000f, 1483293569, 12.328676f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1193" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.105769000f, 39.635600000f, 1483293571, 12.851440f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1194" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106051000f, 39.635673000f, 1483293573, 12.752407f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1195" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106331000f, 39.635747000f, 1483293575, 12.689121f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1196" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106470000f, 39.635783000f, 1483293576, 12.572130f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1197" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106608000f, 39.635819000f, 1483293577, 12.490897f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1198" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106746000f, 39.635855000f, 1483293578, 12.490891f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1199" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.106883000f, 39.635891000f, 1483293579, 12.409719f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1200" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.107148000f, 39.635963000f, 1483293581, 12.045245f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1201" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.107665000f, 39.636103000f, 1483293585, 11.745454f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1202" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108184000f, 39.636249000f, 1483293589, 11.842127f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1203" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108544000f, 39.636350000f, 1483293592, 10.948784f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1204" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.108771000f, 39.636414000f, 1483293594, 10.361708f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1205" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.109477000f, 39.636624000f, 1483293601, 9.268809f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1206" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.109726000f, 39.636699000f, 1483293604, 7.640285f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1207" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.109930000f, 39.636764000f, 1483293607, 6.308740f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1208" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.110064000f, 39.636806000f, 1483293609, 6.201251f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1209" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.110203000f, 39.636849000f, 1483293611, 6.420804f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1210" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.110348000f, 39.636891000f, 1483293613, 6.640339f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1211" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.111031000f, 39.637080000f, 1483293621, 7.777134f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1212" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.111673000f, 39.637309000f, 1483293627, 10.108985f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1213" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.111776000f, 39.637359000f, 1483293628, 10.437752f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1214" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.111969000f, 39.637467000f, 1483293630, 10.226028f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1215" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.112154000f, 39.637577000f, 1483293632, 10.018297f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1216" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.112745000f, 39.637909000f, 1483293639, 8.958828f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1217" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.113263000f, 39.638138000f, 1483293647, 6.400379f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1218" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.113409000f, 39.638193000f, 1483293650, 4.644417f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1219" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.113495000f, 39.638220000f, 1483293652, 3.980778f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1220" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.113626000f, 39.638252000f, 1483293655, 3.927171f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1221" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.113870000f, 39.638295000f, 1483293659, 5.364449f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1222" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114565000f, 39.638385000f, 1483293668, 6.712869f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1223" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.114720000f, 39.638408000f, 1483293670, 6.765945f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1224" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.115442000f, 39.638525000f, 1483293679, 7.027703f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1225" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116127000f, 39.638670000f, 1483293687, 7.612487f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1226" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116548000f, 39.638784000f, 1483293691, 9.564113f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1227" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.116860000f, 39.638882000f, 1483293694, 9.628504f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1228" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117060000f, 39.638947000f, 1483293696, 9.304661f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1229" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117257000f, 39.639011000f, 1483293698, 9.164543f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1230" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.117820000f, 39.639197000f, 1483293704, 8.752829f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1231" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.118508000f, 39.639428000f, 1483293712, 8.042566f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1232" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119209000f, 39.639650000f, 1483293721, 7.219532f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1233" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119288000f, 39.639675000f, 1483293722, 7.321725f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1234" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.119928000f, 39.639902000f, 1483293730, 7.550379f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1235" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120380000f, 39.640095000f, 1483293735, 8.860983f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1236" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120670000f, 39.640220000f, 1483293738, 9.496398f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1237" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120765000f, 39.640261000f, 1483293739, 9.335472f));

  std::cout << "  //----------------------------------------------------------------" << std::endl;
  std::cout << "  // trace point = 1238" << std::endl;

  nav.SnapToRoute(GetFixLocation(-76.120947000f, 39.640341000f, 1483293741, 8.982200f));
}

} // namespace

int main() {
  test::suite suite("navigator");

  // TopLevelLegSetRoute
  suite.test(TEST_CASE(TestTopLevelLegSetRoute));

  // TestAutoLancasterToHershey
  suite.test(TEST_CASE(TestAutoLancasterToHershey));

  // TestAutoMiddletownRoadToLandingsDrive
  suite.test(TEST_CASE(TestAutoMiddletownRoadToLandingsDrive));

  // TestAutoLancasterVerbalMultiCue
  suite.test(TEST_CASE(TestAutoLancasterVerbalMultiCue));

  // TestPedestrianCatalinasToOffice
  suite.test(TEST_CASE(TestPedestrianCatalinasToOffice));

  // TestRoadBikeLunchRideLoop
  suite.test(TEST_CASE(TestRoadBikeLunchRideLoop));

  // TestIsTimeWithinBounds
  suite.test(TEST_CASE(TestIsTimeWithinBounds));

  // CreateTestSeed
  // suite.test(TEST_CASE(CreateTestSeed));

  return suite.tear_down();
}
